/*
 * Khoros: $Id: kread.h,v 1.5 1996/04/04 22:39:07 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Transport Machine Independent Reading Utilities
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Aug 25, 1995
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_kread_h_
#define _kutils_kread_h_


	/*-------------------------------------*
	|       #include 
	---------------------------------------*/


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/


	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/


	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/************************************************************
*
*    Routine Name: kread_bit - read an array of bits
*         Purpose: This module is used to read an array of bits.
*                  The data will need to contain enough memory to
*                  store "num" bits.
*
*                  (note: bits are machine independent and therefore
*                   require no conversion.  This routine is included to
*                   complete the set of read utilities)
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of bits to be read into the data array.
*          Output: data - the char array in which the data will be stored.
*         Returns: the number of bits read
*      Written By: Mark Young
*            Date: Jun 14, 1992
*
*   Declaration: ssize_t kread_bit(
*		 !  int            id,
*		 !  unsigned char *data,
*		 !  size_t         num)
*************************************************************/
#define kread_bit(file, data, num) kread_generic(file, data, num, KBIT)


/************************************************************
*
*    Routine Name: kread_byte - read an array of signed bytes
*         Purpose: This module is used to read an array of signed
*                  bytes.  The data will need to contain enough
*                  memory to store "num" signed bytes.
*
*                  (note: bytes are machine independent and therefore
*                   require no conversion.  This routine is included to
*                   complete the set of read utilities)
*
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of signed bytes to be read into the
*                         data array.
*          Output: data - the char array in which the data will be stored.
*         Returns: the number of signed bytes read
*      Written By: Mark Young
*            Date: Jun 14, 1992
*
*   Declaration: ssize_t kread_byte(
*		 !  int     id,
*		 !  char   *data,
*		 !  size_t  num)
*************************************************************/
#define kread_byte(file, data, num) kread_generic(file, data, num, KBYTE)


/************************************************************
*
*    Routine Name: kread_short - read an array of signed shorts
*         Purpose: This module is used to read an array of signed
*                  shorts.  The data will need to contain enough
*                  memory to store "num" signed shorts.
*           Input: id   - the file id to be read which was opened earlier
*                       with kopen().
*                  num  - the number of signed shorts to be read into the
*                       data array.
*          Output: data - the short array in which the data will be stored.
*         Returns: the number of short integers read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_short(
*		 !  int     id,
*		 !  short  *data,
*		 !  size_t  num)
*************************************************************/
#define kread_short(file, data, num) kread_generic(file, data, num, KSHORT)


/************************************************************
*
*    Routine Name: kread_int - read an array of signed ints
*         Purpose: This module is used to read an array of signed
*                  ints.  The data will need to contain enough
*                  memory to store "num" signed ints.
*           Input: id   - the file id to be read which was opened earlier
*                       with kopen().
*                  num  - the number of signed integers to be read into the
*                         data array.
*          Output: data - the integer array in which the data will be stored.
*         Returns: the number of integers read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_int(
*		 !  int     id,
*		 !  int    *data,
*		 !  size_t  num)
*************************************************************/
#define kread_int(file, data, num) kread_generic(file, data, num, KINT)


/************************************************************
*
*    Routine Name: kread_long - read an array of signed longs
*         Purpose: This module is used to read an array of signed
*                  longs.  The data will need to contain enough
*                  memory to store "num" signed longs.
*           Input: id   - the file id to be read which was opened earlier
*                       with kopen().
*                  num  - the number of signed longs to be read into the
*                       data array.
*          Output: data - the long array in which the data will be stored.
*         Returns: the number of longs read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_long(
*		 !  int     id,
*		 !  long   *data,
*		 !  size_t  num)
*************************************************************/
#define kread_long(file, data, num) kread_generic(file, data, num, KLONG)


/************************************************************
*
*    Routine Name: kread_float - read an array of floats
*         Purpose: This module is used to read an array of floats.
*                  The data will need to contain enough memory to store
*                  "num" floats.
*           Input: id   - the file id to be read which was opened earlier
*                       with kopen().
*                  num  - the number of floats to be read into the
*                       data array.
*          Output: data - the float array in which the data will be stored.
*         Returns: the number of floats read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_float(
*		 !  int    id,
*		 !  float *data,
*		 !  size_t num)
*************************************************************/
#define kread_float(file, data, num) kread_generic(file, data, num, KFLOAT)


/************************************************************
*
*    Routine Name: kread_double - read an array of doubles
*         Purpose: This module is used to read an array of doubles.
*                  The data will need to contain enough memory to store
*                  "num" doubles.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of doubles to be read into the data array.
*          Output: data - the double array in which the data will be stored.
*         Returns: the number of doubles read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_double(
*		 !  int     id,
*		 !  double *data,
*		 !  size_t  num)
*************************************************************/
#define kread_double(file, data, num) kread_generic(file, data, num, KDOUBLE)


/************************************************************
*
*    Routine Name: kread_ubyte - read an array of unsigned bytes
*
*         Purpose: This module is used to read an array of unsigned
*                  bytes.  The data will need to contain enough
*                  memory to store "num" unsigned bytes.
*
*                  (note: bytes are machine independent and therefore
*                   require no conversion.  This routine is included to
*                   complete the set of read utilities)
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of unsigned bytes to be read into the
*                         data array.
*          Output: data - the unsigned char array in which the data will be
*                         stored.
*         Returns: the number of bytes read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_ubyte(
*		 !  int id,
*		 !  unsigned char *data,
*		 !  size_t num)
*************************************************************/
#define kread_ubyte(file, data, num) kread_generic(file, data, num, KUBYTE)


/************************************************************
*
*    Routine Name: kread_ushort - read an array of unsigned shorts
*         Purpose: This module is used to read an array of unsigned
*                  shorts.  The data will need to contain enough
*                  memory to store "num" unsigned shorts.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of unsigned shorts to be read into the
*                         data array.
*          Output: data - the unsigned short array in which the data will be
*                         stored.
*         Returns: the number of shorts read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_ushort(
*		 !  int id,
*		 !  unsigned short *data,
*		 !  size_t num)
*************************************************************/
#define kread_ushort(file, data, num) kread_generic(file, data, num, KUSHORT)


/************************************************************
*
*    Routine Name: kread_uint - read an array of unsigned ints
*         Purpose: This module is used to read an array of unsigned
*                  ints.  The data will need to contain enough
*                  memory to store "num" unsigned ints.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of unsigned ints to be read into the
*                         data array.
*          Output: data - the unsigned integer array in which the data will be
*                         stored.
*         Returns: the number of integers read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_uint(
*		 !  int id,
*		 !  unsigned int *data,
*		 !  size_t num)
*************************************************************/
#define kread_uint(file, data, num) kread_generic(file, data, num, KUINT)


/************************************************************
*
*    Routine Name: kread_ulong - read an array of unsigned longs
*         Purpose: This module is used to read an array of unsigned
*                  longs.  The data will need to contain enough
*                  memory to store "num" unsigned longs.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of unsigned longs to be read into the
*                         data array.
*          Output: data - the unsigned long array in which the data will be
*                         stored.
*         Returns: the number of longs read
*      Written By: Mark Young
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kread_ulong(
*		 !  int id,
*		 !  unsigned long *data,
*		 !  size_t num)
*************************************************************/
#define kread_ulong(file, data, num) kread_generic(file, data, num, KULONG)


/************************************************************
*
*    Routine Name: kread_complex - read an array of complex
*         Purpose: This module is used to read an array of complex
*                  numbers.  The data will need to contain enough
*                  memory to store "num" complex structures.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of complex to be read into the data array.
*          Output: data - the kcomplex array in which the data will be stored.
*         Returns: the number of complex read
*      Written By: Mark Young
*            Date: Jul 20, 1992 16:32
*
*   Declaration: ssize_t kread_complex(
*		 !  int id,
*		 !  kcomplex *data,
*		 !  size_t num)
*************************************************************/
#define kread_complex(file, data, num) kread_generic(file, data, num, KCOMPLEX)


/************************************************************
*
*    Routine Name: kread_dcomplex - read an array of double complex
*         Purpose: This module is used to read an array of double complex
*                  numbers.  The data will need to contain enough
*                  memory to store "num" double complex structures.
*           Input: id   - the file id to be read which was opened earlier
*                         with kopen().
*                  num  - the number of double complex to be read into the
*                         data array.
*          Output: data - the kdcomplex array in which the data will be stored.
*         Returns: the number of double complex read
*      Written By: Mark Young
*            Date: Jul 20, 1992 16:32
*
*   Declaration: ssize_t kread_dcomplex(
*		 !  int id,
*		 !  kdcomplex *data,
*		 !  size_t num)
*************************************************************/
#define kread_dcomplex(file, data, num) kread_generic(file, data, num,KDCOMPLEX)


/************************************************************
*
*    Routine Name: kread_string - read an array of strings
*         Purpose: This module is used to read an array of strings.  The
*                  data will need to contain enough memory to store "num"
*                  string pointers.  The  memory that the strings
*                  are stored in will be allocated by kread_string.
*
*                  Where "kstring" is really a typedef for a character
*                  pointer "char *".  So an array of kstring is really
*                  an array of character pointers.
*           Input: id   - the file id to be read which was opened earlier
*                        with kopen().
*                  num  - the number of string to be read into the
*                         data array.
*          Output: data - the kstring array in which the data will be stored.
*         Returns: the number of strings read
*      Written By: Mark Young
*            Date: Dec 13, 1993
*    Side Effects: the strings that are returned should be freed by the
*                  programmer
*
*   Declaration: ssize_t kread_string(
*		 !  int id,
*		 !  kstring *data,
*		 !  size_t num)
*************************************************************/
#define kread_string(file, data, num) kread_generic(file, data, num, KSTRING)



	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Declaration for public routines.  These routines should used instead of
 *  their system equivalents.  For the most part they have the same parameters
 *  as their system counterparts, but a few are different so you should consult
 *  the appropriate kutils documentation.
 */

/* kread.c */
ssize_t kread_generic PROTO((int, kaddr, size_t, int));
ssize_t kread_array   PROTO((int, kaddr *, size_t, int));
int     kread_pointer PROTO((int, kaddr *, int));
ssize_t kread_struct  PROTO((int, kaddr *, int));

ssize_t kfread_generic   PROTO((kfile *, kaddr, size_t, int));

#endif /* _kutils_kread_h_ */
/* don`t add after the endif */
