/*
 * Khoros: $Id: kutils.h,v 1.13 1996/08/14 00:59:56 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose:  Toplevel include for the kutils library.
   >>>>			This include is what includes all the
   >>>>			other kutils library function and declaration
   >>>>			includes.
   >>>>
   >>>>    Written By:  Mark Young
   >>>>          Date:  May 25, 1993
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_kutils_h_
#define _kutils_kutils_h_

        /*-------------------------------------*
        |       #include
        ---------------------------------------*/

#include <kutils/kvarargs.h>
#include <kutils/knotify.h>
#include <kutils/kfile.h>
#include <kutils/kread.h>
#include <kutils/kwrite.h>
#include <kutils/kparse.h>
#include <kutils/klist.h>
#include <kutils/karray.h>
#include <kutils/khash.h>
#include <kutils/kstrings.h>
#include <kutils/kmemory.h>
#include <kutils/kobject.h>
#include <kutils/kstatus.h>
#include <kutils/kmach.h>
#include <kutils/ksignal.h>
#include <kutils/ktoken.h>


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

#define KMODE_INVALID    (mode_t)0xffff0000

#define KFILE_QUERY      1
#define KFILE_UPDATE     2
#define KFILE_OVERWRITE  3

#define KEDITOR_ICON		"icon"
#define KEDITOR_TITLE		"title"
#define KEDITOR_GEOMETRY	"geometry"
#define KEDITOR_FOREGROUND	"foreground"
#define KEDITOR_BACKGROUND	"background"
#define KEDITOR_PID		"pid"
#define KEDITOR_CHDIR		"chdir"

        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/


        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/


        /*-------------------------------------*
        |       macros
        ---------------------------------------*/

/************************************************************
*
*  Routine Name: knumber - the number of items in an array
*
*       Purpose: This function returns the number of items in an array
*
*         Input: array - the array of items
*        Output:
*       Returns: returns the number of items
*
*  Restrictions: 
*    Written By: Mark Young
*          Date: May 17, 1995
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: size_t knumber(char **array)
*
*************************************************************/
#define knumber(array) (sizeof(array)/sizeof(array[0]))

        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/* dir.c */
char *kdirname	 PROTO((const char *, char *));
char *kbasename  PROTO((const char *, char *));
int  kremove_dir PROTO((const char *));
int  kmake_dir	 PROTO((const char *, mode_t));
int  krmdir	 PROTO((const char *));
int  kmkdir	 PROTO((const char *, mode_t));
int  kchdir	 PROTO((const char *));
char *kgetcwd	 PROTO((char *, size_t));

/* env.c */
char *kgetenv PROTO((const char *));
int  kputenv  PROTO((const char *));
int  kremenv  PROTO((const char *));

/* file.c */
kaddr   kreadfile    PROTO((const char *, size_t *));
int     keditfile    PROTO((const char *, int, kvalist));
ssize_t kwritefile   PROTO((const char *, kaddr, size_t));
ssize_t kcopyfile    PROTO((const char *, const char *));
int     ksedfile     PROTO((const char *, const char *, int, int *, kvalist));
int     kcomparefile PROTO((const char *, const char *, size_t, size_t *));

/* regex.c */
char *kre_comp	  PROTO((char *));
int kre_exec	  PROTO((char *));
int kre_subs	  PROTO((char *, char *));
int kre_pos	  PROTO((int, kaddr *, kaddr *));
void kre_modw     PROTO((char *));
void kre_debug    PROTO((char *, kfile *));
int kre_status    PROTO((void));
char *kre_icomp   PROTO((char *));
char *kregex_replace   PROTO((char *, char *, char *, char *));

/* path.c */
char *kfullpath PROTO((const char *, const char *, char *));
char *kexpandpath PROTO((const char *, const char *, char *));
char *ktbpath PROTO((const char *, char *));

/* program.c */
int  kprog_set_argc	PROTO((int));
int  kprog_set_argv	PROTO((char **));
int  kprog_set_envp	PROTO((char **));
int  kprog_set_program	PROTO((const char *));
int  kprog_set_toolbox	PROTO((const char *));
int  kprog_get_argc	PROTO((void));
char **kprog_get_argv	PROTO((void));
char **kprog_get_envp	PROTO((void));
char *kprog_get_program PROTO((void));
char *kprog_get_toolbox PROTO((void));
char *kprog_get_command PROTO((void));

/* strftime.c */
size_t kstrftime PROTO((char *, size_t, const char *, const struct tm *));

/* aliases.c */
char **kalias_list        PROTO((size_t *));

/* toolboxes.c */
char **ktoolbox_list       PROTO((size_t *));
int  ktoolbox_check        PROTO((const char *));
int  ktoolbox_initialize   PROTO((void));
int  ktoolbox_add          PROTO((const char *, const char *));
int  ktoolbox_delete       PROTO((const char *));
char *ktoolbox_path        PROTO((const char *));
char *ktoolbox_libpath     PROTO((const char *, char *));
char *ktoolbox_siteinclude PROTO((const char *, char *));

/* khoros.c */
int khoros_imprint PROTO((void));
void khoros_initialize PROTO((int, char **, char *));
int  verify_registration PROTO((void));
int  verify_licensed PROTO((void));
int  show_license PROTO((void));
char *get_reg_ver PROTO((void));
char *get_lic_ver PROTO((void));
int  create_khoros_dotfile PROTO((int, int));
int  verify_khoros_dotfile PROTO((void));
int  update_khoros_dotfile PROTO((int, int));
int  inform_user PROTO((int));
int  display_license PROTO((int));

#endif /* _kutils_kutils_h_ */
/* Don't add after this point. */
