/*
 * Khoros: $Id: kwrite.h,v 1.5 1996/04/04 22:39:14 fas Exp $
 */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Transport Machine Independent Writing Utilities
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Aug 25, 1995
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kutils_kwrite_h_
#define _kutils_kwrite_h_


	/*-------------------------------------*
	|       #include 
	---------------------------------------*/


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/


	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/


	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/


	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/************************************************************
*  
*    Routine Name: kwrite_bit - write an array of bits
*         Purpose: This module is used to write an array of bits.
*  
*		   (note: bits are machine independent and therefore
*		   require no conversion.  This routine is included to
*		   complete the set of write utilities)
*           Input: id   - the file id to be written which was opened earlier
*		          with kopen().
*		   data - the unsigned char array which holds the data.
*		   num  - the number of bits to be written.
*         Returns: the number of bits written
*      Written By: Mark Young  
*            Date: Jun 14, 1992
*
*   Declaration: ssize_t kwrite_bit(
*		 !  int id,
*		 !  unsigned char *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_bit(id, data, num) kwrite_generic(id, data, num, KBIT)


/************************************************************
*  
*    Routine Name: kwrite_byte - write an array of signed bytes
*         Purpose: This module is used to write an array of signed
*		   bytes.
*  
*		   (note: bytes are machine independent and therefore
*		    require no conversion.  This routine is included
*		    to complete the set of write utilities)
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the char array which holds the data.
*		   num  - the number of signed bytes to be written.
*         Returns: the number of bytes written
*      Written By: Mark Young  
*            Date: Jun 14, 1992
*
*   Declaration: ssize_t kwrite_byte(
*		 !  int id,
*		 !  char *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_byte(id, data, num) kwrite_generic(id, data, num, KBYTE)


/************************************************************
*  
*    Routine Name: kwrite_short - write an array of signed shorts
*         Purpose: This module is used to write an array of signed
*		   shorts.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the short array which holds the data.
*		   num  - the number of shorts to be written.
*         Returns: the number of shorts written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_short(
*		 !  int id,
*		 !  short *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_short(id, data, num) kwrite_generic(id, data, num, KSHORT)


/************************************************************
*  
*    Routine Name: kwrite_int - write an array of signed ints
*         Purpose: This module is used to write an array of signed
*		   ints.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the integer array which holds the data.
*		   num  - the number of integers to be written.
*         Returns: the number of ints written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_int(
*		 !  int id,
*		 !  int *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_int(id, data, num) kwrite_generic(id, data, num, KINT)


/************************************************************
*  
*    Routine Name: kwrite_long - write an array of signed longs
*         Purpose: This module is used to write an array of signed
*		   longs.  The data will need to contain enough
*		   memory to store "num" signed longs.
*           Input: id   - the file id to be written to which was opened earlier
*		        with kopen().
*		   data - the long array in which holds the data.
*		   num  - the number of longs to be written.
*         Returns: the number of longs written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_long(
*		 !  int id,
*		 !  long *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_long(id, data, num) kwrite_generic(id, data, num, KLONG)


/************************************************************
*  
*    Routine Name: kwrite_float - write an array of floats
*         Purpose: This module is used to write an array of floats.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the float array which holds the data.
*		   num  - the number of floats to be written.
*         Returns: the number of floats written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_float(
*		 !  int id,
*		 !  float *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_float(id, data, num) kwrite_generic(id, data, num, KFLOAT)


/************************************************************
*  
*    Routine Name: kwrite_double - write an array of doubles
*         Purpose: This module is used to write an array of doubles.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the double array which holds the data.
*		   num  - the number of doubles to be written.
*         Returns: the number of doubles written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_double(
*		 !  int id,
*		 !  double *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_double(id, data, num) kwrite_generic(id, data, num, KDOUBLE)


/************************************************************
*  
*    Routine Name: kwrite_ubyte - write an array of unsigned bytes
*         Purpose: This module is used to write an array of unsigned bytes.
*  
*		   (note: bytes are machine independent and therefore
*		    require no conversion.  This routine is included to
*		    complete the set of write utilities)
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the unsigned char array which holds the data.
*		   num  - the number of unsigned bytes to be written.
*         Returns: the number of bytes written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_ubyte(
*		 !  int id,
*		 !  unsigned char *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_ubyte(id, data, num) kwrite_generic(id, data, num, KUBYTE)


/************************************************************
*  
*    Routine Name: kwrite_ushort - write an array of unsigned shorts
*  
*         Purpose: This module is used to write an array of unsigned shorts.
*  
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the unsigned short array which holds the data.
*		   num  - the number of unsigned shorts to be written.
*         Returns: the number of shorts written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_ushort(
*		 !  int id,
*		 !  unsigned short *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_ushort(id, data, num) kwrite_generic(id, data, num, KUSHORT)


/************************************************************
*  
*    Routine Name: kwrite_uint - write an array of unsigned ints
*         Purpose: This module is used to write an array of unsigned ints.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the unsigned integer array which holds the data.
*		   num  - the number of unsigned integers to be written.
*         Returns: the number of ints written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_uint(
*		 !  int id,
*		 !  unsigned int *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_uint(id, data, num) kwrite_generic(id, data, num, KUINT)


/************************************************************
*  
*    Routine Name: kwrite_ulong - write an array of unsigned longs
*         Purpose: This module is used to write an array of unsigned longs.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the unsigned long array which holds the data.
*		   num  - the number of unsigned longs to be written.
*         Returns: the number of longs written
*      Written By: Mark Young  
*            Date: Jun 11, 1992
*
*   Declaration: ssize_t kwrite_ulong(
*		 !  int id,
*		 !  unsigned long *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_ulong(id, data, num) kwrite_generic(id, data, num, KULONG)


/************************************************************
*  
*    Routine Name: kwrite_complex - write an array of complex
*         Purpose: This module is used to write an array of complex numbers.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the kcomplex array which holds the data.
*		   num  - the number of complex pairs to be written.
*         Returns: the number of complexs written
*      Written By: Mark Young  
*            Date: Jul 20, 1992
*
*   Declaration: ssize_t kwrite_complex(
*		 !  int id,
*		 !  kcomplex *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_complex(id, data, num) kwrite_generic(id, data, num, KCOMPLEX)


/************************************************************
*  
*    Routine Name: kwrite_dcomplex - write an array of double complex
*         Purpose: This module is used to write an array of double complex
*		   numbers.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the kdcomplex array which holds the data.
*		   num  - the number of double complex pairs to be written.
*         Returns: the number of double complexs written
*      Written By: Mark Young  
*            Date: Jul 20, 1992
*
*   Declaration: ssize_t kwrite_dcomplex(
*		 !  int id,
*		 !  kdcomplex *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_dcomplex(id, data, num) kwrite_generic(id, data, num,KDCOMPLEX)


/************************************************************
*  
*    Routine Name: kwrite_string - write an array of strings
*         Purpose: This module is used to write an array of strings.
*           Input: id   - the file id to be written to which was opened earlier
*		          with kopen().
*		   data - the string array in which the data will be stored.
*		   num  - the number of strings to be written.
*         Returns: the number of strings written
*      Written By: Mark Young
*            Date: Jul 20, 1992
*
*   Declaration: ssize_t kwrite_string(
*		 !  int id,
*		 !  kstring *data,
*		 !  size_t num)
*************************************************************/
#define kwrite_string(id, data, num) kwrite_generic(id, data, num, KSTRING)


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Declaration for public routines.  These routines should used instead of
 *  their system equivalents.  For the most part they have the same parameters
 *  as their system counterparts, but a few are different so you should consult
 *  the appropriate kutils documentation.
 */

/* kwrite.c */
ssize_t kwrite_generic  PROTO((int, kaddr, size_t, int));
ssize_t kwrite_array    PROTO((int, kaddr, size_t, int));
int     kwrite_pointer  PROTO((int, kaddr *, int));
ssize_t kwrite_struct   PROTO((int, kaddr, int));

ssize_t kfwrite_generic   PROTO((kfile *, kaddr, size_t, int));

#endif /* _kutils_kwrite_h_ */
/* don`t add after the endif */
