/*
  * Khoros: $Id: rpc_main.c,v 1.7 1996/04/04 22:41:42 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: rpc_main.c,v 1.7 1996/04/04 22:41:42 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>      		  RPC Main Code
   >>>>
   >>>>	  This was khoros-ized from the file rpc_main.c 
   >>>>   in the program rpcgen by Sun Microsystems, Inc. 
   >>>>
   >>>>   Static:
   >>>>			extendfile()
   >>>>			open_output()
   >>>>			open_input()
   >>>>  Private:
   >>>>			c_output()
   >>>>			h_output()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

/* @(#)rpc_main.c	2.2 88/08/01 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

/*
 * rpc_main.c, Top level of the RPC protocol compiler. 
 * Copyright (C) 1987, Sun Microsystems, Inc. 
 */
#include "kgenstruct.h"
#include "cpp.h"

static char *extendfile  PROTO((char *, char *, char *, kobject, int));
static void  open_output PROTO((char *, char *));
static void  open_input  PROTO((char *));

static char CPP[] = DEFAULT_CPP;
static char CPPFLAGS[] = "-C";

/*
 * add extension to filename 
 */
static char *
extendfile(char *file, char *toolbox_name, char *object_name, 
           kobject obj, int ext)
{
   char tbname[KLENGTH];
   char path[KLENGTH];
   char *opath = NULL;
   char *base_name = NULL;
   int   otype;
   char *p;

   base_name = kstring_cat("io_", file, NULL);
   p = kstrrchr(base_name, '.');
   if (p != NULL)  /*-- zap out .x --*/
      *p = '\0';

   if (obj != NULL && toolbox_name != NULL)
   {
      /*-- generate the $TOOLBOX name --*/
      tbname[0] = '$';
      kstring_upper(toolbox_name, tbname+1);
      kcms_get_attributes(obj, 
			  KCMS_CMOBJ_TOPSRC, &opath, 
			  KCMS_CMOBJ_TYPE, &otype,
			  KCMS_END);

      if (ext == 'h' && otype == KCMS_LIBRARY)
	 ksprintf(path, "%s/include/%s/%s.h", tbname, object_name, base_name);
      else
	 ksprintf(path, "%s/%s.%c", opath, base_name, (char) ext);
   }
   else  /*-- generate it in ./ --*/
      ksprintf(path, "%s.%c", base_name, (char) ext);
   

   return kstrdup(path);
}


/*
 * Open output file with given extension 
 */
static void
open_output(char *infile, char *outfile)
{
   if (outfile == NULL) 
   {
      fout = kstdout;
      return;
   }
   if (infile != NULL && streq(outfile, infile)) 
   {
      kfprintf(kstderr, "kgenstruct: output would overwrite %s\n", infile);
      kexit(KEXIT_FAILURE);
   }

   fout = kfopen(outfile, "w");
   if (fout == NULL) 
   {
      kfprintf(kstderr, "kgenstruct: unable to open ");
      perror(outfile);
      kexit(KEXIT_FAILURE);
   }
   return;
}

/*
 * Open input file with given define for C-preprocessor 
 */
static void
open_input(char *infile)
{
   char command[KLENGTH];

   ksprintf(command, "%s %s %s", CPP, CPPFLAGS, infile);

   fin = kpopen(command, "r"); 
   if (fin == NULL) 
   {
      kfprintf(kstderr, "kgenstruct: ");
      perror(infile);
      kexit(KEXIT_FAILURE);
   }
   return;
}

/*
 * Compile into an XDR routine output file
 */
void
c_output(char    *infile, 
	 kobject  cmobj)
{
   kobject tb = NULL;
   kobject obj = NULL;
   char *toolbox_name = NULL;
   char *object_name  = NULL;
   int   object_type;
   definition *def;
   long tell;
   int status = TRUE, create = FALSE;
   kstring rcsid_info = NULL;
   kstring rcslog_info = NULL;
   
   if (cmobj)
   { 
      kcms_get_attributes(cmobj, KCMS_PARENT, &tb,
		                 KCMS_NAME, &object_name,
			         KCMS_CMOBJ_TYPE, &object_type,
			         KCMS_END);
      kcms_get_attribute(tb, KCMS_NAME, &toolbox_name);
      obj = cmobj;
   }

   /*-- this tells rpc_scan.c to not propogate directives --*/
   generate_directives(FALSE);

   infilename  = infile;
   outfilename = extendfile(infile, toolbox_name, object_name, obj, 'c');

   create = (kaccess(outfilename, F_OK) == -1);

   if (create == FALSE &&
       kgen_rcs_save(outfilename, &rcsid_info, &rcslog_info) == FALSE)
   {
      status = FALSE;
      return;
   }
   open_input(infile);
   open_output(infile, outfilename);

   pkhoros_chdr(infile, tb, rcsid_info, rcslog_info);
   kfree_and_NULL(rcsid_info);
   kfree_and_NULL(rcslog_info);

   if (infile) 
   {
      if (object_name)
      {
         if (object_type == KCMS_LIBRARY)
	 {
	    kfprintf(fout, "#include \"internals.h\"\n");
	    kfprintf(fout, "#include <%s/%s>\n", object_name, 
		     extendfile(infile, NULL, NULL, NULL, 'h'));
	 }
         else 
	 {
	    kfprintf(fout, "#include \"%s.h\"\n", object_name);
	    kfprintf(fout, "#include \"%s\"\n", 
		     extendfile(infile, NULL, NULL, NULL, 'h'));
	 }
      }
      else
      {
         kfprintf(fout, "#include <bootstrap.h>\n");
         kfprintf(fout, "#include \"%s\"\n", 
	          extendfile(infile, NULL, NULL, NULL, 'h'));
      }
	 
   }

   tell = kftell(fout);
   while ((def = get_definition())) 
      emit_definition(def);

   if (tell == kftell(fout)) 
   {
      (void) kunlink(outfilename);
      status = FALSE;
   }

   kfclose(fin);
   kfclose(fout);

   if (status && obj != NULL && create)
      kcms_create_fileobj(obj, outfilename, NULL, KCMS_FOBJ_TYPE_SRC, 
	  	          KCMS_FOBJ_SUBTYPE_C, KCMS_FOBJ_GEN_STRUCT, 
		          KCMS_FOBJ_ACCESS_RDONLY);
}

/*
 * Compile into an XDR header file
 */
void
h_output(char *infile, 
	 kobject cmobj)
{
   kobject tb = NULL;
   kobject obj = NULL;
   char *toolbox_name;
   char *object_name;
   int   object_type;
   definition *def;
   long  tell;
   char  basename[KLENGTH];
   char *p = NULL;
   int status = TRUE, create = FALSE;
   kstring rcsid_info = NULL;
   kstring rcslog_info = NULL;

   if (cmobj)
   {
      kcms_get_attributes(cmobj, KCMS_PARENT, &tb,
  		                 KCMS_NAME, &object_name,
			         KCMS_CMOBJ_TYPE, &object_type,
			         KCMS_END);
      kcms_get_attribute(tb, KCMS_NAME, &toolbox_name);
      obj = cmobj;
   }

   /*-- this tells rpc_scan.c to propogate directives --*/
   generate_directives(TRUE);

   infilename  = infile;
   outfilename = extendfile(infile, toolbox_name, object_name, obj, 'h');
   
   create = (kaccess(outfilename, F_OK) == -1);

   if (create == FALSE &&
       kgen_rcs_save(outfilename, &rcsid_info, &rcslog_info) == FALSE)
   {
      status = FALSE;
      return;
   }
   open_input(infile);
   open_output(infile, outfilename);
   tell = kftell(fout);

   pkhoros_hhdr(infile, tb, rcsid_info, rcslog_info);
   kfree_and_NULL(rcsid_info);
   kfree_and_NULL(rcslog_info);

   /*-- generate a ifdef so this file isn't included again --*/
   kstring_copy(infile, basename);
   p = kstrrchr(basename, '.');
   if (p != NULL) *p = '\0';

   kfprintf(fout, "#ifndef _io_%s_h_\n", basename);
   kfprintf(fout, "#define _io_%s_h_\n\n", basename);
   
   while ((def = get_definition())) 
      print_datadef(def);

   kfprintf(fout, "#endif /* _io_%s_h_ */\n", basename);

   if (tell == kftell(fout)) 
   {
      (void) kunlink(outfilename);
      status = FALSE;
   }

   kfclose(fin);
   kfclose(fout);

   if (status && obj != NULL && create)
      kcms_create_fileobj(obj, outfilename, NULL, KCMS_FOBJ_TYPE_SRC, 
	  	          KCMS_FOBJ_SUBTYPE_INCL, KCMS_FOBJ_GEN_STRUCT, 
		          KCMS_FOBJ_ACCESS_RDONLY);

}
