/*
  * Khoros: $Id: rpc_util.c,v 1.9 1996/04/04 22:41:45 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: rpc_util.c,v 1.9 1996/04/04 22:41:45 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>      		  RPC Utility Code
   >>>>
   >>>>	  This was khoros-ized from the file rpc_util.c 
   >>>>   in the program rpcgen by Sun Microsystems, Inc. 
   >>>>
   >>>>   Static:
   >>>>			findit()
   >>>>			fixit()
   >>>>			typedefed()
   >>>>			locase()
   >>>>			toktostr()
   >>>>			printbuf()
   >>>>			printwhere()
   >>>>  Private:
   >>>>			reinitialize()
   >>>>			streq()
   >>>>			findval()
   >>>>			storeval()
   >>>>			fixtype()
   >>>>			stringfix()
   >>>>			ptype()
   >>>>			pvname()
   >>>>			error()
   >>>>			record_open()
   >>>>			expected1()
   >>>>			expected2()
   >>>>			expected3()
   >>>>			tabify()
   >>>>			type_name()
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

/* @(#)rpc_util.c	2.1 88/08/01 4.0 RPCSRC */
/*
 * Sun RPC is a product of Sun Microsystems, Inc. and is provided for
 * unrestricted use provided that this legend is included on all tape
 * media and as a part of the software program in whole or part.  Users
 * may copy or modify Sun RPC without charge, but are not authorized
 * to license or distribute it to anyone else except as part of a product or
 * program developed by the user.
 * 
 * SUN RPC IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND INCLUDING THE
 * WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE PRACTICE.
 * 
 * Sun RPC is provided with no support and without any obligation on the
 * part of Sun Microsystems, Inc. to assist in its use, correction,
 * modification or enhancement.
 * 
 * SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
 * INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY SUN RPC
 * OR ANY PART THEREOF.
 * 
 * In no event will Sun Microsystems, Inc. be liable for any lost revenue
 * or profits or other special, indirect and consequential damages, even if
 * Sun has been advised of the possibility of such damages.
 * 
 * Sun Microsystems, Inc.
 * 2550 Garcia Avenue
 * Mountain View, California  94043
 */

/*
 * rpc_util.c, Utility routines for the RPC protocol compiler 
 * Copyright (C) 1987, Sun Microsystems, Inc. 
 */
#include "kgenstruct.h"

char  curline[MAXLINESIZE];	/* current read line */
char *where = curline;	        /* current point in line */
int   linenum = 0;	        /* current line number */

char *infilename;
char *outfilename;

kfile *fout;	/* file pointer of current output */
kfile *fin;	/* file pointer of current input */

list *defined;	/* list of defined things */

static void printwhere PROTO((void));

/*
 * Reinitialize the world 
 */
void
reinitialize(void)
{
   kmemset(curline, 0, MAXLINESIZE);
   where = curline;
   linenum = 0;
   defined = NULL;
   return;
}

/*
 * string equality 
 */
int
streq(char *a, char *b)
{
   return (kstrcmp(a, b) == 0);
}

/*
 * find a value in a list 
 */
char *
findval(list *lst, kaddr val, int (*cmp)(definition *, char *))
{
   for (; lst != NULL; lst = lst->next) 
   {
      if ((*cmp) (lst->val, val)) 
	 return lst->val;
   }
   return NULL;
}

/*
 * store a value in a list 
 */
void
storeval(list **lstp, kaddr val)
{
   list **l;
   list *lst;

   for (l = lstp; *l != NULL; l = (list **) & (*l)->next);
   lst = kmalloc(sizeof(list));
   lst->val = val;
   lst->next = NULL;
   *l = lst;
   return;
}

static int
findit(definition *def, char *type)
{
   return streq(def->def_name, type);
}

static char *
fixit(char *type, char *orig)
{
   definition *def;
   
   def = (definition *) findval(defined, (kaddr) type, findit);
   if (def == NULL || def->def_kind != DEF_TYPEDEF) 
      return orig;

   switch (def->def.ty.rel) 
   {
      case REL_VECTOR:
	 return def->def.ty.old_type;
      case REL_POINTER:      
      case REL_ALIAS:
	 return fixit(def->def.ty.old_type, orig);
      default:
	 return orig;
   }
}

char *
fixtype(char *type)
{
   return fixit(type, type);
}


definition *
resolve_type(char *type, char *orig)
{
   definition *def;
   
   def = (definition *) findval(defined, (kaddr) type, findit);

   if (def == NULL || def->def_kind != DEF_TYPEDEF)
      return (definition *) findval(defined, (kaddr) orig, findit);

   /*-- it is a typedef --*/
   switch (def->def.ty.rel) 
   {
      case REL_POINTER:      
      case REL_VECTOR:
      case REL_ALIAS:
	 return resolve_type(def->def.ty.old_type, orig);
      default:
	 return (definition *) findval(defined, (kaddr) orig, findit);
   }
}

declaration *
resolve_typedef(declaration *orig)
{
   definition  *def;
   declaration *dec = (declaration *) kdupalloc(orig, sizeof(declaration));
   int a_old = 1, a_new = 1;
   static char amax[KLENGTH];

   /*-- find a corresponding typedef - if this is one --*/
   def = resolve_type(dec->type, dec->type);

   /*-- if we found a typedef ... --*/
   if (def && def->def_kind == DEF_TYPEDEF)
   {
      /*-- ... then use its values. --*/
      dec->prefix    = def->def.ty.old_prefix;
      dec->type      = def->def.ty.old_type;

      /*-- if we are supposed to be a pointer, we better make it clear --*/
      if (dec->rel == REL_POINTER || def->def.ty.rel == REL_POINTER)
         dec->rel = REL_POINTER;

      /* array max will accumulate */
      if (dec->array_max != NULL)
         a_old = atoi(dec->array_max);	

      if (def->def.ty.array_max != NULL)
         a_new = atoi(def->def.ty.array_max);

      ksprintf(amax, "%d", a_old * a_new);

      if (dec->array_max != NULL || def->def.ty.array_max != NULL)
        dec->array_max = &(amax[0]); /*-- amax is static --*/ 
      else
        dec->array_max = NULL; 
   }
   return dec;
}

char *
stringfix(char *type)
{
   if (streq(type, "string")) 
      return "wrapstring";
   else 
      return type;
}

void
ptype(char *prefix, char *type, int follow)
{
   if (prefix != NULL) 
   {
      if (streq(prefix, "enum")) 
	 kfprintf(fout, "enum ");
      else 
	 kfprintf(fout, "struct ");

      if (streq(type, "bool")) 
	 kfprintf(fout, "int");
      else if (streq(type, "string"))
	 kfprintf(fout, "char *");
      else
	 kfprintf(fout, "%s ", follow ? fixtype(type) : type);
   }
   return;
}

static int
typedefed(definition *def, char *type)
{
   if (def->def_kind != DEF_TYPEDEF || def->def.ty.old_prefix != NULL)
      return FALSE;
   else
      return streq(def->def_name, type);
}

int
is_structure(char *type)
{
   /*-- kdatatype_to_define doesn't handle this case --*/
   if (kstrcmp(type, "kstring") == 0)
      return FALSE;

   if (kstrcmp(type, "char") == 0)
      return FALSE;

   if (kstrcmp(type, "unsigned char") == 0)
      return FALSE;

   if (kstrcmp(type, "u_char") == 0)
      return FALSE;
	
   if (kdatatype_to_define(type) != KUNDEFINED)
      return FALSE;
   
   return TRUE;
}

int
is_c_type(char *type)
{
   /*-- kdatatype_to_define doesn't handle this case --*/
   if (kstrcmp(type, "kstring") == 0)
      return TRUE;

   if (kstrcmp(type, "char") == 0)
      return TRUE;

   if (kstrcmp(type, "unsigned char") == 0)
      return TRUE;
	
   if (kdatatype_to_define(type) != KUNDEFINED)
      return TRUE;
   
   return FALSE;
}

int
is_typedef_c_type(definition *def)
{
   char    *type = def->def.ty.old_type;
   relation rel  = def->def.ty.rel;

   definition *adef;

   /*-- only consider typedefs --*/
   if (def->def_kind != DEF_TYPEDEF)
      return FALSE;

   for (;;)
   {
      switch (rel)
      {
         case REL_VECTOR:
            return is_c_type(type);
         case REL_ARRAY:
            return FALSE;
         case REL_POINTER:
            return TRUE;
         case REL_ALIAS:
	    if (is_c_type(type))
	       return TRUE;

            adef = (definition *) findval(defined, (kaddr) type, typedefed);
            if (adef == NULL)
               return FALSE;

            type = adef->def.ty.old_type;
            rel = adef->def.ty.rel;
      }
   }
}

int
is_typedef_pointer(definition *def)
{
   char    *type = def->def.ty.old_type;
   relation rel  = def->def.ty.rel;

   definition *adef;

   /*-- only consider typedefs --*/
   if (def->def_kind != DEF_TYPEDEF)
      return FALSE;

   for (;;)
   {
      switch (rel)
      {
         case REL_VECTOR:
            return FALSE;
         case REL_ARRAY:
            return FALSE;
         case REL_POINTER:
            return TRUE;
         case REL_ALIAS:
            adef = (definition *) findval(defined, (kaddr) type, typedefed);
            if (adef == NULL)
               return FALSE;

            type = adef->def.ty.old_type;
            rel = adef->def.ty.rel;
      }
   }
}

int
is_typedef_array(definition *def)
{
   char    *type = def->def.ty.old_type;
   relation rel  = def->def.ty.rel;

   definition *adef;

   /*-- only typedefs can be vectors --*/
   if (def->def_kind != DEF_TYPEDEF)
      return FALSE;

   for (;;)
   {
      switch (rel)
      {
         case REL_VECTOR:
            return !streq(type, "string");
         case REL_ARRAY:
            return FALSE;
         case REL_POINTER:
            return FALSE;
         case REL_ALIAS:
            adef = (definition *) findval(defined, (kaddr) type, typedefed);
            if (adef == NULL)
               return FALSE;

            type = adef->def.ty.old_type;
            rel = adef->def.ty.rel;
      }
   }
}

static char *
locase(char *str)
{
   char c;
   static char buf[100];
   char *p = buf;

   while ((c = *str++)) 
      *p++ = (c >= 'A' && c <= 'Z') ? (c - 'A' + 'a') : (c);

   *p = 0;
   return buf;
}


void
pvname(char *pname, char *vnum)
{
   kfprintf(fout, "%s_%s", locase(pname), vnum);
   return;
}


/*
 * print a useful (?) error message, and then die 
 */
void
error(char *msg)
{
   printwhere();
   kfprintf(kstderr, "%s, line %d: ", infilename, linenum);
   kfprintf(kstderr, "%s\n", msg);
   kexit(KEXIT_FAILURE);
}

static char expectbuf[100];
static char *toktostr(tok_kind kind);

/*
 * error, token encountered was not the expected one 
 */
void
expected1(tok_kind exp1)
{
   s_print(expectbuf, "expected '%s'",
	   toktostr(exp1));
   error(expectbuf);
}

/*
 * error, token encountered was not one of two expected ones 
 */
void
expected2(tok_kind exp1, tok_kind exp2)
{
   s_print(expectbuf, "expected '%s' or '%s'",
	   toktostr(exp1),
	   toktostr(exp2));
   error(expectbuf);
}

/*
 * error, token encountered was not one of 3 expected ones 
 */
void
expected3(tok_kind exp1, tok_kind exp2, tok_kind exp3)
{
   s_print(expectbuf, "expected '%s', '%s' or '%s'",
	   toktostr(exp1),
	   toktostr(exp2),
	   toktostr(exp3));
   error(expectbuf);
}

void
tabify(kfile *f, int tab)
{
   while (tab--)
   {
      (void) kfputc(' ', f);
      (void) kfputc(' ', f);
      (void) kfputc(' ', f);
   }
   return;
}

static token tokstrings[] = {
			     {TOK_IDENT, "identifier"},
			     {TOK_CONST, "const"},
			     {TOK_RPAREN, ")"},
			     {TOK_LPAREN, "("},
			     {TOK_RBRACE, "}"},
			     {TOK_LBRACE, "{"},
			     {TOK_LBRACKET, "["},
			     {TOK_RBRACKET, "]"},
			     {TOK_STAR, "*"},
			     {TOK_COMMA, ","},
			     {TOK_EQUAL, "="},
			     {TOK_COLON, ":"},
			     {TOK_SEMICOLON, ";"},
			     {TOK_UNION, "union"},
			     {TOK_STRUCT, "struct"},
			     {TOK_SWITCH, "switch"},
			     {TOK_CASE, "case"},
			     {TOK_DEFAULT, "default"},
			     {TOK_ENUM, "enum"},
			     {TOK_TYPEDEF, "typedef"},
			     {TOK_INT, "int"},
			     {TOK_SHORT, "short"},
			     {TOK_LONG, "long"},
			     {TOK_UNSIGNED, "unsigned"},
			     {TOK_DOUBLE, "double"},
			     {TOK_FLOAT, "float"},
			     {TOK_CHAR, "char"},
			     {TOK_STRING, "kstring"},
			     {TOK_OPAQUE, "opaque"},
			     {TOK_BOOL, "bool"},
			     {TOK_VOID, "void"},
			     {TOK_PROGRAM, "program"},
			     {TOK_VERSION, "version"},
			     {TOK_EOF, "??????"}
};

static char *
toktostr(tok_kind kind)
{
   token *sp;

   for (sp = tokstrings; sp->kind != TOK_EOF && sp->kind != kind; sp++);
   return sp->str;
}


static void
printbuf(void)
{
   char c;
   int i;
   int cnt;
   
#  define TABSIZE 4

   for (i = 0; (c = curline[i]); i++) 
   {
      if (c == '\t') 
      {
	 cnt = 8 - (i % TABSIZE);
	 c = ' ';
      } 
      else 
   	 cnt = 1;
   
      while (cnt--) 
   	 (void) fputc(c, stderr);
   }
   return;
}

static void
printwhere(void)
{
   int i;
   char c;
   int cnt;
   
   printbuf();
   for (i = 0; i < where - curline; i++) 
   {
      c = curline[i];
      if (c == '\t')
	 cnt = 8 - (i % TABSIZE);
      else 
	 cnt = 1;

      while (cnt--) 
	 (void) kfputc('^', kstderr);
   }
   (void) kfputc('\n', kstderr);
   return;
}

static void
pcopyright(kobject toolbox)
{
   char    *copyright = NULL;
   
   if (toolbox == NULL)
     toolbox = kcms_open_toolbox("bootstrap");

   if (toolbox != NULL)
      copyright = (char *) kcms_get_copyright(toolbox,
					      KCMS_TB_COPYRIGHT_LONG, 
					      KCMS_LANG_C);
   if (copyright)
      kfprintf(fout, "%s\n", copyright);
   else
      kfprintf(fout, "\n", copyright);

   return;
}

static char *
the_time(void)
{
   char           *date;
   struct tm       *localclock;
   time_t          clck;

   date = (char *) kmalloc(sizeof(char)*KLENGTH);
   
   clck = (time_t) time(NULL);
   localclock = (struct tm *) localtime((const time_t *) &clck);
   kstrftime(date, (size_t) KLENGTH, "%h %d, %Y (%H:%M)", localclock);
   return date;
}

static void
pwarning(char *name)
{
   char *date = the_time();

   kfprintf(fout, "/****\n");
   kfprintf(fout, " ****  DO NOT EDIT THIS FILE!\n");
   kfprintf(fout, " ****  generated by kgenstruct from %s on %s\n",
	    name, date);
   kfprintf(fout, " ****/\n\n");
   kfree_and_NULL(date);
   return;
}

void
pkhoros_hhdr(char *name,
	     kobject toolbox,
	     kstring rcsid_info,
	     kstring rcslog_info)
{
   char    author[KLENGTH];

   kgen_rcs_hhdr(fout, rcsid_info, rcslog_info);
   pcopyright(toolbox);
   pwarning(name);

   ksprintf(author, "automatically generated from %s", name);
   kgen_inclfile_hdr(fout, "Structure Header File", author, 
		     "DO NOT EDIT THIS FILE");
   return;
}

void
pkhoros_chdr(char *name,
	     kobject toolbox,
	     kstring rcsid_info,
	     kstring rcslog_info)
{
   kgen_rcs_chdr(fout, rcsid_info, rcslog_info);
   pcopyright(toolbox);
   pwarning(name);

   kgen_srcfile_hdr(fout, "Structure Readers and Writers", 
		    NULL, (size_t) 0, NULL, (size_t) 0, NULL, (size_t) 0);
   return;
}

char *
type_name(char *name)
{
   static char struct_name[KLENGTH];
   char cap_name[KLENGTH];

   if (kstrcmp(name, "char") == 0)
       return "KBYTE";
   else if (kstrcmp(name, "unsigned char") == 0)
       return "KUBYTE";
   else if (kstrcmp(name, "short") == 0)
       return "KSHORT";
   else if (kstrcmp(name, "unsigned short") == 0)
       return "KUSHORT";
   else if (kstrcmp(name, "int") == 0)
       return "KINT";
   else if (kstrcmp(name, "unsigned int") == 0)
       return "KUINT";
   else if (kstrcmp(name, "long") == 0)
       return "KLONG";
   else if (kstrcmp(name, "unsigned long") == 0)
       return "KULONG";
   else if (kstrcmp(name, "float") == 0)
       return "KFLOAT";
   else if (kstrcmp(name, "double") == 0)
       return "KDOUBLE";
   else if (kstrcmp(name, "complex") == 0)
       return "KCOMPLEX";
   else if (kstrcmp(name, "double complex") == 0)
       return "KDCOMPLEX";
   else if (kstrcmp(name, "kstring") == 0)
       return "KSTRING";
   else
   {
      kstring_upper(name, cap_name);
      kstring_cat("KTYPE_", cap_name, struct_name);
      return struct_name;
   }
}
