/*
 * Khoros: $Id: sedutils.c,v 1.5 1996/04/04 22:44:00 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: sedutils.c,v 1.5 1996/04/04 22:44:00 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Library Routine for kcms
   >>>>		
   >>>>  Private:
   >>>>                 _kcms_init_regex()
   >>>>                 kcms_copyfile()
   >>>>                 kcms_checkfile()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static char oldstring[12][KLENGTH];
static char newstring[12][KLENGTH];
static ktoken old_tbno = 0;
static ktoken new_tbno = 0;

/*-----------------------------------------------------------
|
|  Routine Name: _kcms_init_regex - initialize regular expression for
|				    rest of routines
|
|       Purpose: This routine initializes the static variables
|
|         Input: otb - old toolbox object
|		 ntb - new toolbox object
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Steven Jorgensen
|          Date: May 14, 1995
| Modifications:
|
------------------------------------------------------------*/
int _kcms_init_regex(
   kobject otb,
   kobject ntb)
{
   char *old_tbname = NULL;
   char *new_tbname = NULL;
   char old_tbupper[KLENGTH], old_tblower[KLENGTH];
   char new_tbupper[KLENGTH], new_tblower[KLENGTH];
   ktoken old_tok, new_tok;

   if (kcms_get_attribute(otb, KCMS_NAME, &old_tbname) == FALSE ||
       kcms_get_attribute(ntb, KCMS_NAME, &new_tbname) == FALSE)
      return FALSE;

   old_tok = kstring_to_token(old_tbname);
   new_tok = kstring_to_token(new_tbname);

   if (old_tok == old_tbno && new_tok == new_tbno)
      return TRUE;

   if (kstring_upper(old_tbname, old_tbupper) == NULL ||
       kstring_upper(new_tbname, new_tbupper) == NULL ||
       kstring_lower(old_tbname, old_tblower) == NULL ||
       kstring_lower(new_tbname, new_tblower) == NULL)
      return FALSE;

   old_tbno = old_tok;
   new_tbno = new_tok;

   ksprintf(oldstring[0], "\"%s\"", old_tbupper);
   ksprintf(newstring[0], "\"%s\"", new_tbupper);

   ksprintf(oldstring[1], "\"%s\"", old_tblower);
   ksprintf(newstring[1], "\"%s\"", new_tblower);

   ksprintf(oldstring[2], "$%s/", old_tbupper);
   ksprintf(newstring[2], "$%s/", new_tbupper);

   ksprintf(oldstring[3], "$%sBIN/", old_tbupper);
   ksprintf(newstring[3], "$%sBIN/", new_tbupper);

   ksprintf(oldstring[4], ".syntax %s", old_tbupper);
   ksprintf(newstring[4], ".syntax %s", new_tbupper);

   ksprintf(oldstring[5], ".syntax %s", old_tblower);
   ksprintf(newstring[5], ".syntax %s", new_tblower);

   ksprintf(oldstring[6], "<%s.h>", old_tblower);
   ksprintf(newstring[6], "<%s.h>", new_tblower);

   /*-- we could check, and only do this for pane objects -------------*/
   ksprintf(oldstring[7], "kexec -tb %s", old_tbupper);
   ksprintf(newstring[7], "kexec -tb %s", new_tbupper);

   /*-- similarly, this need only be done for info files --------------*/
   ksprintf(oldstring[8], "^Toolbox:(\\s+)$%s$", old_tbupper);
   ksprintf(newstring[8], "Toolbox:\\1$%s\n", new_tbupper);

   /*-- only needs to be done for Imakefile ---------------------------*/
   ksprintf(oldstring[9], "#include %s_INCLUDE", old_tbupper);
   ksprintf(newstring[9], "#include %s_INCLUDE\n", new_tbupper);

   /*-- only needs to be done for Imakefile ---------------------------*/
   ksprintf(oldstring[10], "^TOOLBOX_NAME\\s+=\\s+%s$", old_tblower);
   ksprintf(newstring[10], "TOOLBOX_NAME = %s\n", new_tblower);

   ksprintf(oldstring[11], "($\\w*)/repos/copyright/Copyright");
   ksprintf(newstring[11], "$%s/repos/copyright/Copyright", new_tbupper);

   return TRUE;
}

/*---------------------------------------------------------------
| 
|  Routine Name: kcms_copyfile - copy a file
| 
|       Purpose: copy an file, changing the old toolbox references to
|		 the new.
|
|         Input: otb - old toolbox object
|		 ntb - new toolbox object
|		 ofile - old file name
|		 nfile - new file name
|
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Steven Jorgensen
|          Date: May 14, 1995
---------------------------------------------------------------*/
int kcms_copyfile(
   kobject otb,
   kobject ntb,
   kstring ofile,
   kstring nfile)
{
   if (otb == NULL || ntb == NULL || ofile == NULL || nfile == NULL)
      return FALSE;

   if (_kcms_init_regex(otb, ntb) == FALSE)
      return FALSE;

   if (ksedfile(ofile, nfile, KFILE_UPDATE, NULL,
	        oldstring[0],  newstring[0],  FALSE,
	        oldstring[1],  newstring[1],  FALSE,
	        oldstring[2],  newstring[2],  FALSE,
	        oldstring[3],  newstring[3],  FALSE,
	        oldstring[4],  newstring[4],  FALSE,
	        oldstring[5],  newstring[5],  FALSE,
	        oldstring[6],  newstring[6],  FALSE,
	        oldstring[7],  newstring[7],  FALSE,
	        oldstring[8],  newstring[8],  TRUE,
	        oldstring[9],  newstring[9],  FALSE,
	        oldstring[10], newstring[10], TRUE,
	        oldstring[11], newstring[11], TRUE,
	        NULL) == FALSE)
   {
      errno = KCMS_PERMISSION;
      return FALSE;
   }
   return TRUE;
}

/*---------------------------------------------------------------
| 
|  Routine Name: kcms_checkfile - check for file changes
| 
|       Purpose: Using ksedfile's KFILE_QUERY option check to see
|		 if copying a file from an old toolbox to a new one
|		 will make any difference.
|
|         Input: otb - old toolbox object
|		 ntb - new toolbox object
|		 ofile - old file name
|		 nfile - new file name
|
|       Returns: the status given by ksedfile
|    Written By: Steven Jorgensen
|          Date: May 14, 1995
---------------------------------------------------------------*/
int kcms_checkfile(
   kobject otb,
   kobject ntb,
   kstring ofile,
   kstring nfile)
{
   int status;
   if (otb == NULL || ntb == NULL || ofile == NULL || nfile == NULL)
      return -1;

   if (_kcms_init_regex(otb, ntb) == FALSE)
      return -1;

   if (ksedfile(ofile, nfile, KFILE_QUERY, &status,
	        oldstring[0],  newstring[0],  FALSE,
	        oldstring[1],  newstring[1],  FALSE,
	        oldstring[2],  newstring[2],  FALSE,
	        oldstring[3],  newstring[3],  FALSE,
	        oldstring[4],  newstring[4],  FALSE,
	        oldstring[5],  newstring[5],  FALSE,
	        oldstring[6],  newstring[6],  FALSE,
	        oldstring[7],  newstring[7],  FALSE,
	        oldstring[8],  newstring[8],  TRUE,
	        oldstring[9],  newstring[9],  FALSE,
	        oldstring[10], newstring[10], TRUE,
	        oldstring[11], newstring[11], TRUE,
	        NULL) == FALSE)
   {
      errno = KCMS_PERMISSION;
      return -1;
   }
   return status;
}
