/*
 * Khoros: $Id: gui_generate.c,v 1.10 1996/05/06 21:26:54 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: gui_generate.c,v 1.10 1996/05/06 21:26:54 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Driving routine for code generation
   >>>>		for xvroutines
   >>>>
   >>>>   Public:
   >>>>                 kgen_generate_gui()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


/************************************************************
*  Routine Name: kgen_generate_gui -  generate code for GUI
*       Purpose: Generates the code for a GUI, including GUI Information
*                structure, code to initialize the GUI Information structure,
*                and code to extract user input from the GUI and store it
*                in the GUI Information structure.
*
*         Input: object	- software object (xvroutine) for which to 
*                         generate code 
*
*       Returns: TRUE on success, FALSE otherwise
*    Written By: Danielle Argiro and Neil Bowers
*          Date: 31-jan-1995
*************************************************************/
int
kgen_generate_gui(
   kobject object)
{
	kstring  routine       = "kgen_generate_gui()";
	int      result        = FALSE;
        kform   *form;
        char    *uis_filename;
	kobject  file_object;         /* file object for UIS form file */
	kstring  topsrc;
	char    *tbname;
	char    *oname;
	kstring  datestamp;


	/*
	 * make sure that program is an xvroutine before going on
	 */
	if (!kcms_get_attributes(object,
				 KCMS_NAME,            &oname,
				 KCMS_CMOBJ_UIS_FORM,  &file_object,
				 KCMS_CMOBJ_TOPSRC,    &topsrc,
				 KCMS_END)
	    || !kcms_get_indirect_attribute(object, KCMS_PARENT,
					    KCMS_NAME, &tbname))
	   return(FALSE);

	if (file_object == NULL)
	{
            kwarn(KCODEGEN, routine,
                   "object %s::%s has no form UIS file,\n"
		   "so cannot generate GUI code for the object!",
		   tbname, oname);
	    /*-- return TRUE, so no further messages are generated --*/
	    return(TRUE);
	}
	if (!kcms_get_attribute(file_object, KCMS_PATH, &uis_filename)
	    || uis_filename == NULL)
            return(FALSE);

	/*
	 *  create a temporary directory in TMPDIR in which we will keep
         *  the several temp files from which we create form_drv.c & do_*.c
	 */
	kgen_tmpdirname = ktempnam(NULL, "conductor");
	kunlink(kgen_tmpdirname);

	/*
	 *  figure out which directory generated files will be going to
	 */
	if (kfullpath(topsrc, NULL, kgen_fulltopsrc) == NULL)
	    return(FALSE);

	form = kvf_create_form(uis_filename, KNONE, NULL, NULL);
        if (form == NULL)
        {
           kremove_dir(kgen_tmpdirname);
           return(FALSE);
        }

        /*
         *  call the functions to generate each of the form_*.c files
         */
        result = (/*-- form_info.h --*/
	          kgen_gui_include_file(form, object, oname)

		  /*-- form_init.c --*/
		  && kgen_gui_init_forminfo(form, object, oname)

		  /*-- form_info.c --*/
		  && kgen_gui_get_forminfo(form, object, oname)

		  /*-- form_drv.c --*/
		  && kgen_gui_driver(form, object, oname));

	kvf_destroy_form(form);
	kremove_dir(kgen_tmpdirname);
	kfree_and_NULL(kgen_tmpdirname);

	datestamp = kcms_get_date();
	kcms_set_attribute(object, KCMS_DATE, NULL, NULL, datestamp);
	kfree_and_NULL(datestamp);

	return(result);
}
