/*
  * Khoros: $Id: function.c,v 1.7.2.1 1996/12/06 20:27:18 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: function.c,v 1.7.2.1 1996/12/06 20:27:18 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Various Private Print Routines For the kexpr Library
   >>>>		
   >>>>  Private:
   >>>>                 kexpr_add_function()
   >>>>                 kexpr_check_function()
   >>>>                 kexpr_get_function()
   >>>>   Static:
   >>>>                 init_function()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include "y.tab.h"

kfunc_double f1  = ksinc;
kfunc_double f2  = kfact;
kfunc_double f3  = kimpulse;
kfunc_double f4  = kstep;
kfunc_double f5  = ksign;
kfunc_double f6  = kurng;

/*
 * Function declaration list
 */
static struct _function functions[] =
{
    { "sin",	  (kfunc_double *) &ksin,	 1 },
    { "cos",	  (kfunc_double *) &kcos,	 1 },
    { "tan",	  (kfunc_double *) &ktan,	 1 },

    { "sinh",	  (kfunc_double *) &ksinh,       1 },
    { "cosh",	  (kfunc_double *) &kcosh,       1 },
    { "tanh",	  (kfunc_double *) &ktanh,       1 },

    { "asin",	  (kfunc_double *) &kasin,       1 },
    { "acos",	  (kfunc_double *) &kacos,       1 },
    { "atan",	  (kfunc_double *) &katan,       1 },
    { "atan2",	  (kfunc_double *) &katan2,      2 },
    { "sinc",	  (kfunc_double *) &f1,          1 },

    { "asinh",	  (kfunc_double *) &kasinh,      1 },
    { "acosh",	  (kfunc_double *) &kacosh,      1 },
    { "atanh",	  (kfunc_double *) &katanh,      1 },

    { "ln",	  (kfunc_double *) &klog,	 1 },
    { "log2",	  (kfunc_double *) &klog2,       1 },
    { "log10",	  (kfunc_double *) &klog10,      1 },
    { "log1p",	  (kfunc_double *) &klog1p,      1 },

    { "exp",	  (kfunc_double *) &kexp,	 1 },
    { "exp2",	  (kfunc_double *) &kexp2,       1 },
    { "exp10",	  (kfunc_double *) &kexp10,      1 },
    { "expm1",	  (kfunc_double *) &kexpm1,      1 },

    { "pow",	  (kfunc_double *) &kpow,        2 },
    { "sqrt",	  (kfunc_double *) &ksqrt,       1 },
    { "cbrt",	  (kfunc_double *) &kcbrt,       1 },

    { "abs",	  (kfunc_double *) &kfabs,       1 },
    { "trunc",	  (kfunc_double *) &ktrunc,      1 },
    { "fmod",	  (kfunc_double *) &kfmod,       2 },
    { "ldexp",	  (kfunc_double *) &kldexp,      2 },

    { "floor",	  (kfunc_double *) &kfloor,      1 },
    { "ceil",	  (kfunc_double *) &kceil,       1 },
    { "fact",	  (kfunc_double *) &f2,          1 },

    { "impulse",  (kfunc_double *) &f3,          1 },
    { "step",	  (kfunc_double *) &f4,          1 },
    { "sign",	  (kfunc_double *) &f5,          1 },
    { "erf",	  (kfunc_double *) &kerf,        1 },
    { "erfc",	  (kfunc_double *) &kerfc,       1 },
    { "gamma",	  (kfunc_double *) &kgamma,      1 },

    { "j0",	  (kfunc_double *) &kj0,	 1 },
    { "j1",	  (kfunc_double *) &kj1,	 1 },
    { "y0",	  (kfunc_double *) &ky0,         1 },
    { "y1",	  (kfunc_double *) &ky1,	 1 },

    { "hypot",	  (kfunc_double *) &khypot,      2 },
    { "frexp",	  (kfunc_double *) &kfrexp,      2 },
    { "urng",	  (kfunc_double *) &f6,          0 },

    { NULL,	  0,	       0 },
};

/*-----------------------------------------------------------
|
 | Routine Name: init_function - initialize the kexpr parser
|
|       Purpose: This routine initializes the kexpr parser.  This call
|		 used to initialize the constants, functions, and
|		 global variable lists.  It MUST be called prior to ANY
|		 other calls to the kexpr library.
|
|         Input:
|        Output:
|	Returns:
|
|  Restrictions: 
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
|      Verified:
|  Side Effects:
| Modifications:
|
*-----------------------------------------------------------*/

static void init_function(void)
{
	int	i;
	Symbol  *symbol;
	IdList *idlist;
	static int initialized = FALSE;
	long    id = KEXPR_FUNCTION_ID;


	if (initialized == TRUE)
	   return;

	/*
	 * Install the idlist for Constants, Functions, and Global
	 * variables.
	 */
	initialized = TRUE;
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   for (i = 0; functions[i].name != NULL; i++)
	   {
	       symbol = kexpr_add_symbol(idlist, functions[i].name, FUNCTION,
				0.0);
	       symbol->num      = functions[i].num_args;
	       symbol->Function = *functions[i].routine;
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_add_function()
|
|       Purpose: kexpr_add_function() is called when we want to add a
|		 function to the symbol list.  The function symbol
|		 does not need to be copied, since we want to share
|		 functions within the system.  The current symbol list
|		 is passed in and the number is then tacked onto the
|		 end of the symbol list.  If the symbol list is currently
|		 NULL then we will create a new list and add the variable
|		 to it.
|
|        Input:  symlist    - the current symbol list
|		 function   - the symbol to be added to the list
|		 num_args   - the number of arguments specified with
|			      the function.
|
|       Output:  returns the new symbol list.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
|----------------------------------------------------------*/

SymbolList *kexpr_add_function(
   SymbolList *symlist,
   Symbol     *function,
   int	      num_args)
{
	char	   error[KLENGTH];
	SymbolList *list, *entry;


	/*
	 *  Allocate the entry for our new function
	 */
	if (function->num != num_args && function->type != UNDEFINED)
	{
	   (void) ksprintf(error, "Error! Incorrect number of arguments \
specified.  Function '%s' takes %d arguments, but you specified %d.",
			function->name, (int) function->num, num_args);
	   kexpr_error(error);
	}

	/*
	 *  Allocate the entry for our new function
	 */
	if ((entry = (SymbolList *) kcalloc((size_t) 1, sizeof(SymbolList))) == NULL)
	{
	   (void) ksprintf(error, "Error! Ran out of memory trying to allocate \
reference list for function '%s'", function->name);
	   kexpr_error(error);
	}
	entry->symbol = function;
	entry->next   = NULL;

	/*
	 *  If the current symlist is NULL then return entry as our symlist.
	 */
	if (symlist == NULL)
	{
	   return(entry);
	}

	/*
	 *  The current symbol list is not empty so add the entry to the
	 *  end of the current list.
	 */
	list = symlist;
	while (list->next != NULL)
	   list = list->next;

	list->next = entry;
	return(symlist);
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_check_function()
|
|       Purpose: kexpr_check_function() is called when we want to
|		 create a user defined function.  The main
|		 purpose of this routine is make the function's
|		 argument list be local to only the function and
|		 not global to the rest of the system.
|
|        Input:  symbol   - the current variable symbol
|
|       Output:  None.  Make the functions symlist have only local
|		 variables and then update the function expression
|		 to use these local variables.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
|----------------------------------------------------------*/

void kexpr_check_function(
    Symbol     *function,
    SymbolList *arglist,
    SymbolList *symlist)
{
	SymbolList *list;
	Symbol	   *symbol, *temp;
	int	   num_args = 0;
	char	   error[KLENGTH];


	/*
	 *  If the current function is NULL then just return
	 */
	if (function == NULL)
	   return;

	/*
	 *  Change all function arguments from global to local variables.
	 *  At the same time change the expression to use these local
	 *  variables.
	 */
	while (arglist != NULL)
	{
	   symbol = arglist->symbol;
	   if (symbol->type == VARIABLE || symbol->type == EXPRESSION ||
	       symbol->type == UNDEFINED)
	   {
	      /*
	       *  Allocate the symbol to store our local variable
	       */
	      if ((temp = (Symbol *)kcalloc((size_t)1,sizeof(Symbol))) == NULL)
	      {
	         (void) ksprintf(error, "Error! Ran out of memory trying to \
allocate local argument '%s' for function '%s' ", symbol->name, function->name);
		 kexpr_error(error);
	      }
	      temp->Value = 0.0;
	      temp->type  = EXPRESSION;
	      temp->name  = kstrdup(symbol->name);
	      arglist->symbol = temp;

	      list = symlist;
	      while (list != NULL)
	      {
		 if (list->symbol == symbol)
		    list->symbol = temp;

	         list = list->next;
	      }
	   }
	   arglist = arglist->next; num_args++;
	}
	function->num = num_args;
}

/*-----------------------------------------------------------
|
|  Routine Name: kexpr_get_function - get function symbol
|
|       Purpose: kexpr_get_function() - pulls function name from
|		 the expression string.
|
|         Input: string - the string which contains the desired function
|
|        Output: None, except that the resulting function is stored
|		 in the global kexpr_lval.symbol.
|
|    Written By: Mark Young  
|          Date: Thu Jun 25 1992
| Modifications:
|
------------------------------------------------------------*/

Symbol *kexpr_get_function(
  char	*string)
{
	IdList *idlist;
	Symbol  *symbol = NULL;
	long    id = KEXPR_FUNCTION_ID;


	/*
	 * Check check to see if it is a defined function
	 */
	init_function();
	if ((idlist = kexpr_get_idlist(id)) != NULL)
	{
	   if ((symbol = kexpr_get_symbol(idlist, string, TRUE)) != NULL)
	   {
	      return(symbol);
	   }
	}

	/*
	 * Check to see if it is in the current idlist list.
	 */
	if ((idlist = kexpr_get_idlist(current_id)) != NULL)
	{
	   if ((symbol = kexpr_get_symbol(idlist, string, TRUE)) == NULL)
	   {
	      symbol = kexpr_add_symbol(idlist, string, UNDEFINED, 0.0);
	      return(symbol);
	   }
	}
	return(symbol);
}
