/*
 * Khoros: $Id: copy.c,v 1.5 1996/04/04 22:46:56 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: copy.c,v 1.5 1996/04/04 22:46:56 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		Form Tree Copy Routines                     <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 kvf_copy_sel_list()
   >>>>                 kvf_copy_selection()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/*-----------------------------------------------------------------
|
|  Routine Name: kvf_copy_sel_list
|
|       Purpose: Copies the selection list given into a new selection list
|
|         Input: old_sel_list - pointer to the old selection list
|        Output: Returns pointer to sel_list on success, NULL on failure
|    Written By: Danielle Argiro
|          Date: May 04, 1995
| Modifications:
|
------------------------------------------------------------------*/

kselection *kvf_copy_sel_list(
   kcontrol   *pane,
   kselection *old_sel_list,
   kselection *toggle_start,
   kselection *group_start)
{
	kselection *sel_list = NULL;
	kselection *selection;
	kselection *old_selection;
	int        count = 0;

	if (old_sel_list == NULL) return(NULL);
	old_selection = old_sel_list;
	while (old_selection != NULL)
	{
	    selection = kvf_copy_selection(pane, old_selection,
				           toggle_start, group_start);
	    old_selection = old_selection->next;
	    if (count == 0) sel_list = selection;
	    count++;
	}
	return(sel_list);
}

/*-----------------------------------------------------------------
|
|  Routine Name: kvf_copy_selection
|
|       Purpose: Copies the selection given into a new selection 
|
|         Input: old_selection - pointer to the old selection
|        Output: Returns pointer to selection on success, NULL on failure
|    Written By: Danielle Argiro
|          Date: May 04, 1995
| Modifications:
|
------------------------------------------------------------------*/

kselection *kvf_copy_selection(
   kcontrol   *pane,
   kselection *old_selection,
   kselection *toggle_start,
   kselection *group_start)
{
	kselection *selection;

        selection = (kselection *) kcalloc((size_t)1, sizeof(kselection));
	
	selection->line         = kstrdup(old_selection->line);
	selection->comment      = kstrdup(old_selection->comment);
        selection->type         = old_selection->type;
        selection->modified     = old_selection->modified;
        selection->selected     = old_selection->selected;
        selection->opt_selected = old_selection->opt_selected;
        selection->toggle_num   = old_selection->toggle_num;
        selection->prev_selected= old_selection->prev_selected;
        selection->name_token   = old_selection->name_token;
        selection->var_token    = old_selection->var_token;
	selection->back_group   = group_start;
	selection->back_toggle  = toggle_start;
	selection->back_control = pane;
	selection->back_guide   = pane->back_guide;
	selection->back_subform = pane->back_subform;
	selection->back_form    = pane->back_form;
	selection->back_kformstruct = kvf_create_struct((kaddr) selection,
                                               selection->type, KSELECTION);
	if (group_start != NULL)
	    kvf_link_group(group_start, selection);
	else if (toggle_start != NULL)
	    kvf_link_toggle(toggle_start, selection);
	else kvf_link_selection(pane, selection);

	if (old_selection->group_next != NULL)
	    kvf_copy_sel_list(pane, old_selection->group_next, 
			      NULL, selection);
	
	if (old_selection->toggle_next != NULL)
	    kvf_copy_sel_list(pane, old_selection->toggle_next, 
                              selection, NULL);
	return(selection);
}
