/*
 * Khoros: $Id: destroy.c,v 1.10 1996/08/10 03:14:31 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: destroy.c,v 1.10 1996/08/10 03:14:31 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		Form Deletion/Cleanup Routines             <<<<
   >>>>		<<<<
   >>>>  Private:
   >>>>                 kvf_destroy_subform()
   >>>>                 kvf_destroy_control()
   >>>>                 kvf_destroy_guide()
   >>>>                 kvf_destroy_sel_list()
   >>>>                 kvf_destroy_selection()
   >>>>                 kvf_free_line_info_strings()
   >>>>   Static:
   >>>>                 kfree_kformstruct()
   >>>>                 kfree_callback_list()
   >>>>   Public:
   >>>>                 kvf_destroy_form()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"


static void kfree_kformstruct   PROTO((kform_struct *));
static void kfree_callback_list PROTO((klist *));

/************************************************************
*
*  Routine Name: kvf_destroy_form - destroy and free form tree
*
*       Purpose: Destroys the form tree and frees all memory associated 
*                with the pointer returned by kvf_create_form().  
*                To be used only when the caller is totally finished 
*                with the form tree.  
*
*         Input: form - pointer to the form tree being destroyed
*        Output: none
*       Returns: none
*  Restrictions: none
*    Written By: Danielle Argiro
*          Date: Jul 16, 1992 
*      Verified:
*  Side Effects: none
* Modifications: 
*
*************************************************************/

void kvf_destroy_form(
   kform *form)
{
	ksubform *subform; 
	ksubform *old_subform;

	if (form == NULL) return;
	
	if (form->master != NULL)
	{
	    subform = form->master->subform_list;
	    while (subform != NULL)
	    {
		old_subform = subform;
		subform = subform->next;
		kvf_destroy_subform(old_subform);
	    }
	    kvf_destroy_control(form->master);
	}
	else kvf_destroy_subform(form->subform);

	kfree_and_NULL(form->control_line);
	kfree_and_NULL(form->control_comment);
	kfree_and_NULL(form->uis_location);
	kfree_kformstruct(form->back_kformstruct);

	kvf_delete_entry((char *) form);
	kfree_and_NULL(form);
}




/*------------------------------------------------------------
|
|  Routine Name: kvf_destroy_subform
|
|       Purpose: Destroys the subform tree associated with the 
|                subform pointer.  
|
|         Input: subform - the subform pointer
|        Output: none
|       Returns: nothing
|    Written By: Danielle Argiro and Mark Young
|          Date: May 6, 1992
| Modifications: Converted from Khoros 1.0 (DA)
|
-------------------------------------------------------------*/

void kvf_destroy_subform(
   ksubform *subform)
{
	kguide *guide, *old_guide;

	if (subform == NULL) return;

	if (subform->guidepane != NULL)
	{
	    guide = subform->guidepane->guide_list;
	    while (guide != NULL)
	    {
		old_guide = guide;
		guide = guide->next;
	        kvf_destroy_guide(old_guide);
	    }
	    kvf_destroy_control(subform->guidepane);
	}
	else if (subform->guide != NULL)
	    kvf_destroy_guide(subform->guide);

	kfree_and_NULL(subform->line);
	kfree_and_NULL(subform->comment);
	kfree_and_NULL(subform->control_line);
	kfree_and_NULL(subform->control_comment);
	kfree_kformstruct(subform->back_kformstruct);

	kvf_delete_entry((char *) subform);
	kfree_and_NULL(subform);
}

/*-----------------------------------------------------------
|
|  Routine Name: kvf_destroy_control
|
|       Purpose: destroys the structure associated with a master,
|		 guidepane, or pane.  
|         Input: control - the pointer to control panel structure
|        Output: none
|       Returns: nothing
|          Date: May 7, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/

void kvf_destroy_control(
   kcontrol *control)
{
	kfree_and_NULL(control->control_line);
	kfree_and_NULL(control->control_comment);
	kfree_kformstruct(control->back_kformstruct);

	if (control->sel_list != NULL)
	   kvf_destroy_sel_list(control->sel_list);

	karray_free(control->sel_names, KSTRING, control->sel_count, NULL);
	kfree_and_NULL(control);
}

/*-----------------------------------------------------------
|
|  Routine Name: kvf_destroy_guide
|
|       Purpose: destroys the structure associated with a guide
|         Input: guide - the pointer to guide structure
|        Output: none
|       Returns: nothing
|          Date: May 7, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/
void kvf_destroy_guide(
   kguide *guide)
{
	kfree_and_NULL(guide->line);
	kfree_and_NULL(guide->control_line);
	kfree_and_NULL(guide->comment);
        kfree_kformstruct(guide->back_kformstruct);

	if (guide->pane != NULL) 
	    kvf_destroy_control(guide->pane);

	kfree_and_NULL(guide);
}

/*-----------------------------------------------------------
|
|  Routine Name: kvf_destroy_sel_list
|
|       Purpose: destroys a selection list
|         Input: selection - the header of the selection list
|        Output: none
|       Returns: nothing
|          Date: May 7, 1992
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/

void kvf_destroy_sel_list(
   kselection *sel_list)
{
	kselection *old_sel;
	kselection *selection = sel_list;

	while (selection != NULL)
	{
	    old_sel = selection;
	    selection = selection->next;
	    kvf_destroy_selection(old_sel);
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: kvf_destroy_selection
|
|       Purpose: destroys a selection
|         Input: selection - the selection to be destroyed
|        Output: none
|       Returns: nothing
|          Date: May 7, 1992
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
void kvf_destroy_selection(
   kselection *selection)
{
	int          i;
	kselection   *extension;
	kselection   *toggle,  *old_toggle;
	kselection   *group,   *old_group;
	kform_struct *submenu_ptr, *old_submenu;

	/* sever backpointers from any extensions */
	for (i = 0; i < selection->extension_num; i++)
	{
	    extension = selection->extensions[i];
	    if (karray_locate(extension->extensions, KSTRUCT,
			      selection,(size_t)extension->extension_num)!=-1)
	    {
		extension->extensions = (kselection **)
		     karray_delete(extension->extensions, KSTRUCT,
				   selection,(size_t)extension->extension_num);
		extension->extension_num--;
	    }
	}

	/* free memory associated with toggle */
	toggle = selection->toggle_next;
	while (toggle != NULL)
	{
	    old_toggle = toggle;
	    toggle = toggle->next;
	    kvf_destroy_selection(old_toggle);
	}

	/* free memory associated with group */
	group = selection->group_next;
	while (group != NULL)
	{
	    old_group = group;
	    group = group->next;
	    kvf_destroy_selection(old_group);
	}

	/* free memory associated with selection itself */
	kfree_and_NULL(selection->extensions);
       	kfree_and_NULL(selection->line);
       	kfree_and_NULL(selection->comment);
	kfree_and_NULL(selection->submenu_buttons);

	if (selection->submenu_next != NULL)
	{
	    submenu_ptr = selection->submenu_next;
	    while (submenu_ptr != NULL)
	    {
		old_submenu = submenu_ptr;
		submenu_ptr = submenu_ptr->next;
	        kfree_and_NULL(old_submenu);
	    }
	}
	kfree_kformstruct(selection->back_kformstruct);
        kfree_and_NULL(selection);
	
}

/*-----------------------------------------------------------
|
|  Routine Name: kvf_free_line_info_strings
|
|       Purpose: Frees all strings contained in a Line_Info struct
| 		
|         Input: line_info - pointer to the Line_Info struct
|        Output: none
|       Returns: nothing
|          Date: Jul 6, 1992 
|    Written By: Danielle Argiro 
| Modifications:
|
------------------------------------------------------------*/

void kvf_free_line_info_strings(
   Line_Info *line_info)
{
	kfree_and_NULL(line_info->title);
	kfree_and_NULL(line_info->description);
	kfree_and_NULL(line_info->variable);
	kfree_and_NULL(line_info->routine);
	kfree_and_NULL(line_info->help_file);
	kfree_and_NULL(line_info->filename);
	kfree_and_NULL(line_info->string_val);
	kfree_and_NULL(line_info->string_def);
	kfree_and_NULL(line_info->file_def);
	kfree_and_NULL(line_info->literal);
	kfree_and_NULL(line_info->lib_call);

	karray_free(line_info->val_labels, KSTRING, 
		    line_info->val_labelnum, NULL);
	line_info->val_labelnum = 0;
	line_info->val_labels   = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: kfree_kformstruct
|
|       Purpose: Frees the back_kformstruct structure which is
|                the back kformstruct of the form, subform, guide, control,
|                or selection.
|
|         Input: back_kformstruct - pointer to the kformstruct
|        Output: TRUE
|       Returns: nothing
|          Date: Oct 6, 1993
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/
 
static void kfree_kformstruct(
   kform_struct *back_kformstruct)
{
	if (back_kformstruct == NULL)
	   return;
 
	klist_free(back_kformstruct->callback_list, kfree_callback_list);
	kfree_and_NULL(back_kformstruct);
}

/*-----------------------------------------------------------
|
|  Routine Name: kfree_callback_list
|
|       Purpose: Frees the callback_info structure which is 
|                the client_data of the kformstruct->callback_list.
|                kformstruct->callback list is used to keep a list
|                of the callbacks when xvf_add_gui_callback() has been used
|                to add a callback to the GUI item, which will be
|                fired when an attribute of that GUI item changes.
|
|         Input: list - pointer to the kformstruct->callback_list node
|        Output: TRUE
|       Returns: nothing
|          Date: Oct 6, 1993
|    Written By: Danielle Argiro
| Modifications:
|
------------------------------------------------------------*/

static void kfree_callback_list(
   klist *list)
{
	kaddr call_info;

	call_info = klist_clientdata(list);
	kfree_and_NULL(call_info);
}
