/*
 * Khoros: $Id: flin.c,v 1.9 1996/05/01 17:39:14 mirage Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: flin.c,v 1.9 1996/05/01 17:39:14 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */
 
 
/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Single Precision General Matrix and Vector Processing
   >>>>		
   >>>>		- adapted from linpack and blas -
   >>>>		
   >>>>   Static:
   >>>>                 _kblas_is_amax()
   >>>>   Public:
   >>>>                 kblas_sscal()
   >>>>                 kblas_saxpy()
   >>>>                 kblas_sswap()
   >>>>                 klin_sgefa()
   >>>>                 klin_sgedi()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kblas_is_amax
|
|       Purpose: This routine finds the index of sx which
|	 	 has the maximum absolute value. 
|
|		 The functioning of this code for 
|		 incx != 1 appears to be questionable.
|
|         Input: sx   - linear array to search
|		 n    - number of bins in array to check
|		 incx - increment to use when searching
|        Output: none
|
|       Returns: the index into the array where the maximum absolute 
|		 value is stored on success, -1 on failure
|		 (This actually appears to be the bin number).
|    Written By: Jack Dongarra (adapted from blas by Steve Kubica)
|          Date: Aug 04, 1994 11:44
| Modifications:
|
------------------------------------------------------------*/
static int
_kblas_is_amax(
   float *sx,
   int    n,
   int    incx)
{
   float smax;
   int   i;
   int   ix = 0;
   int   return_value = 0;
   

   if (n < 1)
      return -1;

   if (n == 1)
      return 0;

   if (incx != 1)
   {
      /* -- code for increment not equal to 1 -- */
      smax = kabs(sx[0]);
      ix += incx;
      for (i = 1; i < n; i++, ix += incx)
	 if (kabs(sx[ix]) > smax)
	 {
	    return_value = i; 
	    smax = kabs(sx[ix]);
	 }
   }
   else
   {
      /* -- code for both increments equal to 1 -- */
      smax = kabs(sx[0]);
      for (i = 1; i < n; i++)
	 if (kabs(sx[i]) > smax)
	 {
	     return_value = i;
	     smax = kabs(sx[i]);
	 }
   }

   return return_value;
}

/************************************************************
*
*  Routine Name: kblas_sscal - scale a float vector
*
*       Purpose: This routine multiplies a vector by a constant.
*	 	 It uses unrolled loops to improve performance 
*		 for increments == 1.
*
*         Input: sx   - linear array to scale
*		 sa   - scalar multiplier for sx array
*		 n    - number of bins in array to scale
*		 incx - increment to use on sx array
*
*        Output: sx   - computation is done in place
*       Returns: none
*  Restrictions:
*    Written By: Jack Dongarra (adapted from blas by Steve Kubica)
*          Date: Aug 04, 1994 17:28
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: void kblas_sscal(
*	       	!    float *sx,
*	       	!    float  sa,
*	       	!    int    n,
*	       	!    int    incx)
*
*************************************************************/
void
kblas_sscal(
   float *sx,
   float  sa,
   int    n,
   int    incx)
{
   int nincx;
   int i;
   int m;
   
   
   if (n < 0) 
      return;
 
   if (incx != 1)
   {
      /* -- code for increment not equal to 1 -- */
      nincx = n * incx;
      for (i = 0; i < nincx; i += incx)
      {
	 sx[i] = sa * sx[i];
      }
   }
   else
   {
      /* -- code for increment equal to 1 -- */
      m = n % 5;
      if (m != 0)
      {
	 for (i = 0; i < m; i++)
	    sx[i] = sa*sx[i];
	 if (n < 5)
	    return;
      }
      for (i = m; i < n; i += 5)
      {
	 sx[i]   = sa*sx[i];
	 sx[i+1] = sa*sx[i+1];
	 sx[i+2] = sa*sx[i+2];
	 sx[i+3] = sa*sx[i+3];
	 sx[i+4] = sa*sx[i+4];
      }
   }
   
   return;
}

/************************************************************
*
*  Routine Name: kblas_saxpy - add two float vectors while scaling one
*
*       Purpose: This routine multiplies a vector by a constant
*		 and then adds it to another vector. The calculation
*		 will be : SY = SY + sa*SX
*	 	 It uses unrolled loops to improve performance 
*		 for increments == 1.  The computation is done in place
*		 and returned in sy.
*
*         Input: sx   - linear array to process
*		 sy   - linear array to add to add to sx*sa
*		 sa   - scalar multiplier for sx array
*		 n    - number of bins in array to operate on
*		 incx - increment to use on sx array
*		 incy - increment to use on sy array
*        Output: 
*
*      Returns: none
*  Restrictions:
*    Written By: Jack Dongarra (adapted from blas by Steve Kubica)
*          Date: Aug 04, 1994 17:30
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: void kblas_sscal(
*	       	!    float *sx,
*	       	!    float *sy,
*	       	!    float  sa,
*	       	!    int    n,
*	       	!    int    incx,
*	       	!    int    incy)
*
*************************************************************/
void
kblas_saxpy(
   float *sx,
   float *sy,
   float  sa,
   int    n,
   int    incx,
   int    incy)
{
   int i;
   int ix = 0;
   int iy = 0;
   int m;
   

   if (n < 0 || sa == 0.0)
      return;

   if (incx != 1 || incy != 1)
   {
      /* -- code for unequal increments or equal increments != 1 -- */
   
      if (incx < 0) 
	 ix = (-n+1)*incx;
      if (incy < 0) 
	 iy = (-n+1)*incy;
      
      for (i = 0; i < n; i++, ix += incx, iy += incy)
	 sy[iy] = sy[iy] + sa*sx[ix];
   }
   else
   {
      /* -- code for increments == 1 -- */

      /* -- clean-up loop -- */

      m = n % 4;
      if (m != 0)           
      {
	 for (i = 0; i < m; i++)
	    sy[i] = sy[i] + sa*sx[i];
	 if (n < 4)
	    return;
      }
      for (i = m; i < n; i+=4) 
      {
	 sy[i]   = sy[i]   + sa*sx[i];
	 sy[i+1] = sy[i+1] + sa*sx[i+1];
	 sy[i+2] = sy[i+2] + sa*sx[i+2];
	 sy[i+3] = sy[i+3] + sa*sx[i+3];
      }
   }
   
   return;
}

/************************************************************
*
*  Routine Name: kblas_sswap - swap two float vectors
*
*       Purpose: This routine interchanges two vectors.
*	 	 It uses unrolled loops for increments == 1.
*
*         Input: sx   - linear array to swap
*		 sy   - linear array to swap
*                n    - number of bins in array to swap
*		 incx - increment to use on sx
*		 incy - increment to use on sy
*        Output: sx and sy are swapped
*
*       Returns: none
*  Restrictions:
*    Written By: Jack Dongarra (adapted from blas by Steve Kubica)
*          Date: Aug 04, 1994 17:33
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: void kblas_sswap(
*	       	!    float *sx,
*	       	!    float *sy,
*	       	!    int    n,
*	       	!    int    incx,
*	       	!    int    incy)
*
*************************************************************/
void
kblas_sswap(
   float *sx,
   float *sy,
   int    n,
   int    incx,
   int    incy)
{
   float stemp;
   int i;
   int ix = 0;
   int iy = 0;
   int m;

   
   if (n < 0)
      return;
   
   if (incx != 1 || incy != 1)
   {
      /* -- code for unequal increments of equal increments != 1 -- */
      
      if (incx < 0) 
	 ix = (-n+1)*incx;
      if (incy < 0) 
	 iy = (-n+1)*incy;

      for (i = 0; i < n; i++, ix += incx, iy += incy)
      {
	 stemp = sx[ix];    sx[ix] = sy[iy];    sy[iy] = stemp;
      }
   }
   else
   {
      /* -- code for both increments equal to 1 -- */
	 
      m = n % 3;
      if (m != 0)           
      {
	 for (i = 0; i < m; i++)
	 {
	    stemp = sx[i];    sx[i] = sy[i];    sy[i] = stemp;
	 }
	 if (n < 3)
	    return;
      }
      for (i = m; i < n; i+=3) 
      {
	 stemp = sx[i];    sx[i]   = sy[i];      sy[i]   = stemp;
	 stemp = sx[i+1];  sx[i+1] = sy[i+1];    sy[i+1] = stemp;
	 stemp = sx[i+2];  sx[i+2] = sy[i+2];    sy[i+2] = stemp;
      }
   }
   
   return;
}

/************************************************************
*
*  Routine Name: klin_sgefa - factors a float matrix by gaussian elimination.
*
*       Purpose: This routine is usually called by klin_sgeco, but
*		 it can be called directly with a saving in time if
*		 the rcond is not needed.  
*
*         Input: matrix - input matrix stored in a 1D array
*                         floats in row-major order such that
*               !              m[i][j] = a[i * cols+j]
*
*		 rows   - the number of rows in matrix.
*		 cols   - the number of cols in matrix.
*
*        Output: pivot  - the pivot vector 
*
*       Returns: 0 implies a normal value; a non-zero value of
*		 k means that U(k,k) == 0. 
*		 This is not an error condition for this routine,
*		 but it does indicate that klin_sgesl or 
*		 klin_sgedi will divide by zero if called.  
*		 Use the argument rcond in klin_segco for a reliable 
*		 indication of singularity.
*
*  Restrictions: 
*    Written By: Cleve Moler (adapted from linpack by Steve Kubica)
*
*          Date: Aug 04, 1994 11:00
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: int klin_sgefa(
*		!    float *matrix,
*		!    int    rows,
*		!    int    cols,
*		!    int   *pivot)
*
*************************************************************/
int klin_sgefa(
   float *matrix,
   int    rows,
   int    cols,
   int   *pivot)
{
   int j;
   int k;
   int info = 0;
   int l;
   float t;
   
   /* -- gaussian elimination with partial pivoting -- */
   if (cols > 1) 
   {
      for (k = 0; k < cols-1; k++)
      {
	 /* -- find l = pivot index -- */
	 l = _kblas_is_amax(&(matrix[k*cols+k]),rows-k,cols) + k;
	 pivot[k] = l;
      
	 /* -- 0 pivot implies that this column is already triangularized -- */
	 if (matrix[l*cols+k] != 0.0)
	 {
	    /* -- interchange if necessary -- */
	    if (l != k)
	    {
	       t = matrix[l*cols+k];
	       matrix[l*cols+k] = matrix[k*cols+k];
	       matrix[k*cols+k] = t;
	    }
	    
	    /* -- compute multipliers -- */
	    t = -1.0/matrix[k*cols+k];
	    kblas_sscal(&(matrix[(k+1)*cols+k]), t, rows-k-1, cols);
	    
	    /* 	-- row elimination with column indexing -- */
	    for (j = k+1; j < rows; j++)
	    {
	       t = matrix[l*cols+j];
	       if (l != k)
	       {
		  matrix[l*cols+j] = matrix[k*cols+j];
		  matrix[k*cols+j] = t;
	       }
	       kblas_saxpy(&(matrix[(k+1)*cols+k]),
			   &(matrix[(k+1)*cols+j]),t,rows-k-1,cols,cols);
	    }
	 }
	 else
	    info = k;

      }
   }   
   pivot[cols-1] = cols - 1;
   
   if (matrix[(rows-1)*cols+(rows-1)] == 0.0)
      info = rows;

   return info;
}

/************************************************************
*
*  Routine Name: klin_sgedi - computes the determinate and 
*			      inverse of a matrix
*
*       Purpose: This routine will compute the determinant 
*		 and inverse of an NxN matrix using the factors 
*		 computed by klin_sgeco or klin_sgefa.
*
*		 The determinant functionality is not well tested.
*
*         Input: matrix- input matrix stored in a 1D array of
*                        doubles in row-major order such that
*               !             m[i][j] = a[i * cols+j]
*
*                rows  - the number of rows in matrix.
*		 cols  - the number of columns in matrix.
*
*		 pivot - the pivot vector from klin_sgefa
*
*                job  - defines what to compute:  the inverse, the
*                       determinate, or both of the matrix.
*                       KLIN_INVERSE to compute inverse
*                       KLIN_DETERMINATE to compute determinate
*                       KLIN_INVERSE | KLIN_DETERMINATE to compute both
*
*        Output: matrix - inverse of the original matrix if requested,
*			  othewise unchanged.
*		 det    - determinant of original matrix if requested,
*			  otherwise not referenced.
*			    determinant = det(1)*10.0**det(2)
*			    with 1.0 < kabs(det(1)) < 10.0 or det(1) == 0.0
*
*       Returns: TRUE (1) all the time
*
*  Restrictions: A division by zero will occur if the input factor contains
*	         a zero on the diagonal and the inverse is requested.
*	         It will not occur if the subroutines are called correctly
*	         and if klin_sgeco has set rcond > 0.0 or klin_sgefa has set
*	         info < 0.
*
*    Written By: Cleve Moler (adapted from linpack by Steve Kubica)
*
*          Date: Aug 04, 1994 11:00
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration:
*
*************************************************************/
int klin_sgedi(
   float *matrix,
   int    rows,
   int    cols,
   int   *pivot,
   int    job,
   float *work,
   float *det)
{
   float t;
   float ten;
   int   i;
   int   j;
   int   k;
   int   kb;
   int   kp1;
   int   l;


   /* -- compute determinate -- */
   if (job & KLIN_DETERMINATE)
   {
      det[0] = 1.0;
      det[1] = 0.0;
      ten = 10.0; 
      
      for (i = 0; i < cols; i++)
      {
         if (pivot[i] != i)
            det[0] = -det[0];

         det[0] = matrix[i*cols+i]*det[0];
         
         if (det[0] != 0.0)
	 {
	    while (kabs(det[0]) < 1) 
	    {
	       det[0] = det[0]/ten;
	       det[1] = det[1] + 1.0;
	    }
	    
	    while (kabs(det[0]) >= ten) 
	    {
	       det[0] = det[0]/ten;
	       det[1] = det[1] + 1.0;
	    }
	 }
      }
   }
   
   /* -- compute inverse(u) -- */
   if (job & KLIN_INVERSE)
   {
      for (k = 0; k < rows; k++)
      {
         matrix[k*cols+k] = 1.0/matrix[k*cols+k];
         t = -matrix[k*cols+k];
         kblas_sscal(&(matrix[k]), t,k,cols);

         if (k < rows)
         {
            for (j = k+1; j < rows; j++)
            {
               t = matrix[k*cols+j];
               matrix[k*cols+j] = 0.0;
               kblas_saxpy(&(matrix[k]),&(matrix[j]),t,k+1,cols,cols);
            }
         }
      }

      /* -- form inverse(u)*inverse(1) -- */
      if (rows > 1)
      {
	 for (kb = 0; kb < rows; kb++)
	 {
	    k = cols - kb - 1;
	    kp1 = k + 1;
	    for (i = kp1; i < rows; i++)
	    {
	       work[i] = matrix[i*cols+k];
	       matrix[i*cols+k] = 0.0;
	    }
	    for (j = kp1; j < rows; j++)
	    {
	       t = work[j];
	       kblas_saxpy(&(matrix[j]),&(matrix[k]),t,cols,cols,cols);
	    }
	    l = pivot[k];
	    if (l != k)
	       kblas_sswap(&(matrix[k]),&(matrix[l]),cols,cols,cols);
	 }
      }
   }

   return TRUE;
}
