/*
 * Khoros: $Id: klibm.c,v 1.4 1996/04/04 22:51:16 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: klibm.c,v 1.4 1996/04/04 22:51:16 fas Exp $";
#endif


/*
 * Copyright (C) 1994
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		libm compatibility functions
   >>>>		
   >>>>   Public:
   >>>>                 kacos()
   >>>>                 kasin()
   >>>>                 katan2()
   >>>>                 katan()
   >>>>                 ksinh()
   >>>>                 kcosh()
   >>>>                 ktanh()
   >>>>                 ksin()
   >>>>                 kcos()
   >>>>                 ktan()
   >>>>                 kceil()
   >>>>                 kfloor()
   >>>>                 kexp()
   >>>>                 klog()
   >>>>                 klog10()
   >>>>                 kpow()
   >>>>                 ksqrt()
   >>>>                 kfabs()
   >>>>                 kfmod()
   >>>>                 kfrexp()
   >>>>                 kldexp()
   >>>>                 kmodf()
   >>>>                 khypot()
   >>>>                 ktrunc()
   >>>>                 kgamma()
   >>>>                 kcbrt()
   >>>>                 kj0()
   >>>>                 kj1()
   >>>>                 kjn()
   >>>>                 ky0()
   >>>>                 ky1()
   >>>>                 kyn()
   >>>>                 kerf()
   >>>>                 kerfc()
   >>>>                 kasinh()
   >>>>                 kacosh()
   >>>>                 katanh()
   >>>>                 kexpm1()
   >>>>                 kexp2()
   >>>>                 kexp10()
   >>>>                 klog1p()
   >>>>                 klog2()
   >>>>                 krandom()
   >>>>                 ksrandom()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*
 * interfaces for the posix libm.a
 */

/************************************************************
*
*  Routine Name: kacos - compute the arc cosine of the argument.
*
*       Purpose: The kacos function computes the arc cosine
*		 of the double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kacos is simply a macro to acos because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to take arc cosine of.
*        Output:
*	Returns: the double precision arc cosine of the
*		 double precision input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kacos(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kacos) PROTO((double)) = acos;

/************************************************************
*
*  Routine Name: kasin - compute the arc sine of the 
*			 argument.
*
*       Purpose: The kasin function computes arc sine of
*		 the double precision input argument, whose
*		 value lies between -1 and 1.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kasin is simply a macro to asin because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to take arc sine of.
*        Output:
*	Returns: the double precision arc sine of the
*		 input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kasin(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kasin) PROTO((double)) = asin;

/************************************************************
*
*  Routine Name: katan2 - compute the arc tangent of y/x.
*
*       Purpose: The katan2 function converts the rectangular
*		 coordinates (x,y) into polar coordinates 
*		 and returns the phase component in the range
*		 (-pi/2,pi/2)
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 katan2 is simply a macro to atan2 because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: y - numerator component of the tangent given
*		     y/x.
*		 x - the non-zero denominator component of the tangent given
*		     by y/x.  
*        Output: 
*	Returns: the double precision arc tangent of the
*		 input arguments.
*  Restrictions: The second argument x must be non-zero.
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double katan2(
*		   double y,
*		   double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*katan2) PROTO((double, double)) = atan2;

/************************************************************
*
*  Routine Name: katan - compute the arc tangent of the argument.
*
*       Purpose: The katan function computes arc tangent of
*		 the double precison input argument.  The result
*		 is an angle with the value between -pi/2 and pi/2
*		 radians.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 katan is simply a macro to atan because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to take arc tangent of.
*        Output:
*	Returns: the double precision arc tangent of the
*		 input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double katan(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*katan) PROTO((double)) = atan;

/************************************************************
*
*  Routine Name: ksinh - compute the hyperbolic sine of the argument.
*
*       Purpose: The ksinh function computes the hyperbolic
*		 sine of the input argument
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 ksinh is simply a macro to sinh because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to compute the hyperbolic
*		     sine on.
*        Output:
*	Returns: the double precision hyperbolic sine of the
*		 input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ksinh(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ksinh) PROTO((double)) = sinh;

/************************************************************
*
*  Routine Name: kcosh - compute the hyperbolic cosine of the argument.
*
*       Purpose: The kcosh function computes the hyperbolic
*		 cosine of the double precison input argument.
*
*		 This is provided in all POSIX and X/OPEN compliant C
*		 math libraries.  Currently kcosh is simply a macro to
*		 cosh because Khoros does not run on any machine whose
*		 math library does not include this function.  If
*		 Khoros is ported to a machine without this function
*		 then a replacement will be written.
*
*         Input: x - input argument to compute the double precision hyperbolic
*		     cosine on.
*        Output:
*	Returns: the double precision hyperbolic cosine of the
*		 input argument or KMAXFLOAT if the
*		 input argument is less than one.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kcosh(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kcosh) PROTO((double)) = cosh;

/************************************************************
*
*  Routine Name: ktanh - compute the hyperbolic tangent of the argument.
*
*       Purpose: The ktanh function computes the hyperbolic
*		 tangent of the double precision input argument
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 ktanh is simply a macro to tanh because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to compute the hyperbolic
*		     tangent on.
*        Output:
*	Returns: the double precision hyperbolic tangent of the
*		 input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ktanh(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ktanh) PROTO((double)) = tanh;

/************************************************************
*
*  Routine Name: ksin - compute the double precision sine of the argument.
*
*       Purpose: The ksin function computes the sine of the
*		 double precision input argument, which represents
*		 an angle in radians.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 ksin is simply a macro to sin because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - radian input argument to compute the sine
*		     on.
*        Output:
*	Returns: the double precision sine of the input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ksin(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ksin) PROTO((double)) = sin;

/************************************************************
*
*  Routine Name: kcos - compute the double precision cosine of the argument.
*
*       Purpose: The kcos function computes the cosine of the
*		 double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kcos is simply a macro to cos because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - radian input argument to compute the cosine
*		     on.
*        Output:
*	Returns: the double precision cosine of the input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kcos(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kcos) PROTO((double)) = cos;

/************************************************************
*
*  Routine Name: ktan - compute the double precision tangent of the argument.
*
*       Purpose: The ktan function computes the tangent of the
*		 input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 ktan is simply a macro to tan because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - radian input argument to compute the tangent
*		     on.
*        Output:
*	Returns: the double precision tangent of the input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ktan(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ktan) PROTO((double)) = tan;

/************************************************************
*
*  Routine Name: kceil - compute the ceiling of the argument.
*
*       Purpose: The kceil function computes the least integral
*		 value (smallest integer value) greater than or equal 
*		 to the input argument and returns it as a double.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kceil is simply a macro to ceil because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - number to calculate ceiling of.
*        Output:
*	Returns: the double precision ceiling of the input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kceil(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kceil) PROTO((double)) = ceil;

/************************************************************
*
*  Routine Name: kfloor - compute the floor of the argument.
*
*       Purpose: The kfloor function computes the largest integral
*		 value (largest integer) less than or equal to the 
*		 input argument and returns it as a double.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kfloor is simply a macro to floor because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - number to calculate floor of.
*        Output:
*	Returns: the double precision floor of the input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kfloor(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kfloor) PROTO((double)) = floor;

/************************************************************
*
*  Routine Name: kexp - compute the exponential of the argument.
*
*       Purpose: The kexp function computes the value of
*		 the exponential of the double precision input
*		 argument (e**x).  
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kexp is simply a macro to exp because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - variable whose exponential is to be computed.
*        Output:
*	Returns: the double precision result of the exponential of the 
*		 input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kexp(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kexp) PROTO((double)) = exp;

/************************************************************
*
*  Routine Name: klog - compute the natural logarithm of the argument.
*
*       Purpose: The klog function computes natural logarithm
*		 of the double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 klog is simply a macro to log because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - argument to the natural log function.
*        Output:
*	Returns: the double precision logarithm of the input argument
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double klog(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*klog) PROTO((double)) = log;

/************************************************************
*
*  Routine Name: klog10 - compute the base 10 logarithm of the argument.
*
*       Purpose: The klog10 function computes the base 10 logarithm
*		 of the double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 klog10 is simply a macro to log10 because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - argument to the base 10 log function.
*        Output:
*	Returns: the double precision base 10 logarithm of the input
*		 argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double klog10(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*klog10) PROTO((double)) = log10;

/************************************************************
*
*  Routine Name: kpow - compute x to the y power.
*
*       Purpose: The kpow function computes x**y, which is the value of
*		 one double precision input argument x raised to the
*		 power of another double precision input argument y.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kpow is simply a macro to pow because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - base of the exponentiation operation.
*		 y - exponent to raise the base to.
*        Output:
*	Returns: the double precision result of the input argument x
*		 raised to the power of the input argument y.
*  Restrictions: Neither argument can be 0.
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kpow(
*		   double x,
*		   double y)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kpow) PROTO((double, double)) = pow;

/************************************************************
*
*  Routine Name: ksqrt - compute the square root of the argument.
*
*       Purpose: The ksqrt function computes the square root
*		 of the double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 ksqrt is simply a macro to sqrt because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - number to take square root of.
*        Output:
*	Returns: the double precision square root of the input
*		 argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ksqrt(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ksqrt) PROTO((double)) = sqrt;

/************************************************************
*
*  Routine Name: kfabs - compute the absolute value of the argument.
*
*       Purpose: The kfabs function computes the absolute
*		 value of the double precision input argument.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kfabs is simply a macro to fabs because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - number to take the absolute value of.
*        Output:
*	Returns: the double precision absolute value of the
*		 input argument.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kfabs(double x)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kfabs) PROTO((double)) = fabs;

/************************************************************
*
*  Routine Name: kfmod - compute the floating point modulo of the arguments.
*
*       Purpose: The kfmod function computes the floating
*		 point remainder of x/y.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  On most machines,
*		 kfmod is a macro to fmod.  On certain 
*		 machines that are not fully POSIX compliant,
*		 this is a macro to an internal implementation
*		 of fmod.
*
*         Input: x - numerator of division operation.
*                y - denominator of division operation.
*        Output:
*
*	Returns: the double precision modulo remainder of x/y. 
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Aug 01, 1991 15:30
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kfmod(
*		    double x,
*		    double y)
*
*************************************************************/

#if defined(KFMOD_DEF)
   double (*kfmod) PROTO((double, double)) = __kfmod;
#else
   extern double  fmod PROTO((double, double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kfmod) PROTO((double, double)) = fmod;
#endif

/************************************************************
*
*  Routine Name: kfrexp - compute significand and exponent of argument 
*
*       Purpose: The kfrexp function computes the significand  (or mantissa)
*		 of v as a double quantity between 0.5 and 1.0 and stores the 
*		 exponent in the output argument e.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kfrexp is simply a macro to frexp because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: v - value to compute on.
*        Output: e - exponent of the input argument.
*	Returns: the significand of the input argument, or zero if the input
*		 argument is zero.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young 
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kfrexp(
*		    double v,
*		    int *e)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kfrexp) PROTO((double, int *)) = frexp;

/************************************************************
*
*  Routine Name: kldexp - computes x * 2**n
*
*       Purpose: The kldexp function computes x * 2**n
*		 by performing exponent manipulation rather
*		 than multiplication.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kldexp is simply a macro to ldexp because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - double argument to x * 2**n
*		 n - integer argument to x * 2**n
*        Output: 
*	Returns: the double precision value x * 2**n
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kldexp(
*		    double x,
*		    int n)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kldexp) PROTO((double, int)) = ldexp;

/************************************************************
*
*  Routine Name: kmodf - compute the fractional component of the argument.
*
*       Purpose: The kmodf function computes the fractional
*		 and integral part of the double precision input argument x.
*		 After decomposing the input argument it returns the 
*		 fractional part and places
*		 the integral part in the argument i.
*
*		 This is provided in all POSIX and X/OPEN 
*		 compliant C math libraries.  Currently
*		 kmodf is simply a macro to modf because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - input argument to separate into
*		     integral and fractional components.
*        Output: i - the integral part of the input argument 
*	Returns: the fractional component of the input argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kmodf(
*		    double x,
*		    double *i)
*
*************************************************************/
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kmodf) PROTO((double, double *)) = modf;


/*
 * non posix functions
 */

/************************************************************
*
*  Routine Name: khypot - compute the euclidean distance from the origin of the arguments.
*
*       Purpose: The khypot function computes the Euclidean
*		 distance from (0,0) to (x,y).  This function 
*		 computes the square root of the sum of squares of the double
*		 precision input arguments x and y, giving the return value
*`		 sqrt(x*x + y*y)
*
*		 khypot is not a POSIX function, but is
*		 implemented in BSD math libraries.
*		 khypot is simply a macro to hypot because
*		 Khoros does not run on any machine whose
*		 math library does not include this function.
*		 If Khoros is ported to a machine without this
*		 function then a replacement will be written.
*
*         Input: x - value on x axis of vector to determine 
*		     length of.
*		 y - value on y axis of vector to determine 
*		     length of.
*        Output: 
*	Returns: returns the double precision euclidean distance from (0,0) to
*		 (x,y).
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double khypot(
*		    double x,
*		    double y)
*
*************************************************************/
extern double hypot PROTO((double, double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*khypot) PROTO((double, double)) = hypot;

/************************************************************
*
*  Routine Name: ktrunc - truncate a number
*
*       Purpose: The ktrunc function truncates a double
*		 precision value by removing its fractional
*		 component.
*
*		 ktrunc is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to trunc, 
*		 otherwise ktrunc is a macro to the Khoros
*		 portable version of trunc.
*
*         Input: x - value to truncate
*        Output: 
*	Returns: returns the integral component of the iput argument
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ktrunc(double x)
*
*************************************************************/
#if defined(KTRUNC_DEF)
   double (*ktrunc) PROTO((double)) = __ktrunc;
#else
   extern double  trunc PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*ktrunc) PROTO((double)) = trunc;
#endif

/************************************************************
*
*  Routine Name: kgamma - compute the gamma function of the argument.
*
*       Purpose: The kgamma function computes the log gamma of the
*		 double precision input argument.
*
*         Input:  x - value to compute the log gamma of.
*        Output: 
*	Returns:  the double precision log gamma of the input argument.
*  Restrictions: If gamma does not exist on you system in libm,
*		 then a Khoros version of the program will be 
*		 transparently installed in its place.  This
*		 version of the function is not very good.  The
*		 Khoros Group has plans to improve this function
*		 soon after the main release of Khoros 2.0.
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kgamma(double x)
*
*************************************************************/
#if defined(KGAMMA_DEF)
   double (*kgamma) PROTO((double)) = __kgamma;
#else
   extern double  gamma PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kgamma) PROTO((double)) = gamma;
#endif

/************************************************************
*
*  Routine Name: kcbrt - compute the cube root of the argument.
*
*       Purpose: The kcbrt function computes the cube root of the
*		 double precision input argument.
*
*         Input: x - value perform the cube root function on
*        Output: 
*	Returns: the double precision cube root of the input argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kcbrt(double x)
*
*************************************************************/
#if defined(KCBRT_DEF)
   double (*kcbrt) PROTO((double)) = __kcbrt;
#else
   extern double  cbrt PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kcbrt) PROTO((double)) = cbrt;
#endif

/************************************************************
*
*  Routine Name: kj0 - compute the Bessel function j0 of the argument.
*
*       Purpose: The kj0 function computes an approximation
*		 of the j0 Bessel function.  This function evaluates
*		 the Bessel function of the first kind of integer
*		 order at an input argument x.
*
*		 kj0 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to j0, 
*		 otherwise kj0 is a macro to the Khoros
*		 portable version of j0.
*
*         Input: x - argument to j0
*        Output: approximation of the j0 Bessal function for x
*	Returns: the double precision value of the j0 Bessel function at x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kj0(double x)
*
*************************************************************/
extern double  j0 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kj0) PROTO((double)) = j0;

/************************************************************
*
*  Routine Name: kj1 - compute the Bessel function j1 of the argument.
*
*       Purpose: The kj1 function computes an approximation
*		 of the j1 Bessel function.  This function evaluates
*		 the Bessel function of the first kind integer order
*		 of the input argument x.
*
*		 kj1 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to j1, 
*		 otherwise kj1 is a macro to the Khoros
*		 portable version of j1.
*
*         Input: x - argument to j1
*        Output: 
*	Returns: the double precision value of the j1 Bessel function at x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kj1(double x)
*
*************************************************************/
extern double  j1 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kj1) PROTO((double)) = j1;

/************************************************************
*
*  Routine Name: kjn - compute the general Bessel function jn of the argument.
*
*       Purpose: The kjn function computes an approximation
*		 of the jn Bessel function.  This function evaluates
*		 the Bessel function of the first kind of integer n 
*		 order of the input argument x.
*
*		 kjn is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to jn, 
*		 otherwise kjn is a macro to the Khoros
*		 portable version of jn.
*
*         Input: x - argument to jn
*		 n - integer order of the Bessel function.
*        Output: 
*	Returns: the double precision value of the n-order Bessel function
*		 at x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kjn(int n, double x)
*
*************************************************************/
extern double  jn PROTO((int, double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kjn) PROTO((int, double)) = jn;

/************************************************************
*
*  Routine Name: ky0 - compute the Bessel function y0 of the argument.
*
*       Purpose: The ky0 function computes an approximation
*		 of the y0 Bessel function.  This function evaluates
*		 the Bessel function of the second kind of integer order
*		 at input argument x.  The input argument x must be positive.
*
*		 ky0 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to y0, 
*		 otherwise ky0 is a macro to the Khoros
*		 portable version of y0.
*
*         Input: x - argument to y0
*        Output: 
*	Returns: the double precision value of the y0 Bessel function at x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ky0(double x)
*
*************************************************************/
extern double  y0 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ky0) PROTO((double)) = y0;

/************************************************************
*
*  Routine Name: ky1 - compute the Bessel function y1 of the argument.
*
*       Purpose: The ky1 function computes an approximation
*		 of the y1 Bessel function.  This function evaluates
*		 the Bessel function of the second kind of integer order
*	 	 at input argument x.  The input argument must be positive.
*
*		 ky1 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to y1, 
*		 otherwise ky1 is a macro to the Khoros
*		 portable version of y1.
*
*         Input: x - argument to y1
*        Output: 
*	Returns: the double precision value of the y1 Bessel function at x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double ky1(double x)
*
*************************************************************/
extern double  y1 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*ky1) PROTO((double)) = y1;

/************************************************************
*
*  Routine Name: kyn - compute the general Bessel function yn of the argument.
*
*       Purpose: The kyn function computes an approximation
*		 of the yn Bessel function.  This function evaluates the
*		 Bessel function of the second kind of integer order n at the 
*		 input argument x.  The input argument x must be positive.
*
*		 kyn is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to yn, 
*		 otherwise kyn is a macro to the Khoros
*		 portable version of yn.
*
*         Input: x - argument to yn
*		 n - integer order of the Bessel function
*        Output: 
*	Returns: the double precision value of the n-order Bessel function at
*		 x.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kyn(int n, double x)
*
*************************************************************/
extern double  yn PROTO((int, double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kyn) PROTO((int, double)) = yn;

/************************************************************
*
*  Routine Name: kerf - compute the error function of the argument.
*
*       Purpose: The kerf function computes the error function,
*		 defined by:
*
*		! kerf(x) = 2.0/sqrt(pi)*{integral from 0 to x of exp(-t*t) dt}
*
*		 kerf is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to erf, 
*		 otherwise kerf is a macro to the Khoros
*		 portable version of erf.
*
*         Input: x - argument to erf
*        Output: 
*	Returns: the error function of the input argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kerf(double x)
*
*************************************************************/
extern double  erf PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kerf) PROTO((double)) = erf;

/************************************************************
*
*  Routine Name: kerfc - compute the complement error function of the argument.
*
*       Purpose: The kerfc function computes the complement 
*		 error function, defined by:
*
*		! kerfc(x) = 1.0 - kerf(x)
*
*		 kerfc is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to erfc, 
*		 otherwise kerfc is a macro to the Khoros
*		 portable version of erfc.
*
*         Input: x - argument to kerfc
*        Output: 
*	Returns: One minus the error function of the input argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kerfc(double x)
*
*************************************************************/
extern double  erfc PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
double (*kerfc) PROTO((double)) = erfc;

/************************************************************
*
*  Routine Name: kasinh - compute the arc hyperbolic sine of the argument.
*
*       Purpose: The kasinh function computes the arc hyperbolic
*		 sine function
*
*		 kasinh is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to asinh, 
*		 otherwise kasinh is a macro to the Khoros
*		 portable version of asinh.
*
*         Input: x - argument to kasinh
*        Output: 
*	Returns: the evaluation of asinh(x)
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kasinh(double x)
*
*************************************************************/
#if defined(KASINH_DEF)
   double (*kasinh) PROTO((double)) = __kasinh;
#else
   extern double  asinh PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kasinh) PROTO((double)) = asinh;
#endif

/************************************************************
*
*  Routine Name: kacosh - compute the arc hyperbolic cosine of the argument.
*
*       Purpose: The kacosh function computes the arc hyperbolic
*		 cosine of the double precision argument.
*
*		 kacosh is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to acosh, 
*		 otherwise kasinh is a macro to the Khoros
*		 portable version of acosh.
*
*         Input: x - argument to kacosh
*        Output: 
*	Returns: the double precision are hyperbolic cosine of the
*		 input argument or KMAXFLOAT if the input argument is
*		 less than one.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kacosh(double x)
*
*************************************************************/
#if defined(KACOSH_DEF)
   double (*kacosh) PROTO((double)) = __kacosh;
#else
   extern double  acosh PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kacosh) PROTO((double)) = acosh;
#endif

/************************************************************
*
*  Routine Name: katanh - compute the arc hyperbolic tangent of the argument.
*
*       Purpose: The kasinh function computes the arc hyperbolic
*		 tangent of the double precision input argument whose
*		 value lies between -1 and 1.
*
*		 katanh is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to atanh, 
*		 otherwise katanh is a macro to the Khoros
*		 portable version of atanh.
*
*         Input: x - argument to katanh
*        Output: 
*	Returns: the double precision arc hyperbolic tangent of the input
*		 argument or KMAXFLOAT if the input parameter is less 
*		 than -1 or greater than 1.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double katanh(double x)
*
*************************************************************/
#if defined(KATANH_DEF)
   double (*katanh) PROTO((double)) = __katanh;
#else
   extern double  atanh PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*katanh) PROTO((double)) = atanh;
#endif


/************************************************************
*
*  Routine Name: kexpm1 - compute the exponential function minus 1 
*		 of the argument.
*
*       Purpose: The kexpm1 function computes the exponential
*		 function of the double precision input argument,
*		 then subtracts 1.
*
*		 kexpm1 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to expm1, 
*		 otherwise kexpm1 is a macro to the Khoros
*		 portable version of expm1.
*
*         Input: x - variable whose exponential minus 1 is to be computed.
*        Output: 
*	Returns: the double precision exponential minus 1 of the input
*		 argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kexpm1(double x)
*
*************************************************************/
#if defined(KEXPM1_DEF)
   double (*kexpm1) PROTO((double)) = __kexpm1;
#else
   extern double  expm1 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kexpm1) PROTO((double)) = expm1;
#endif


/************************************************************
*
*  Routine Name: kexp2 - compute the base 2 exponential function of the argument.
*
*       Purpose: The kexp2 function computes the base 2 exponential
*		 of the double precision input argument.
*
*		 kexp2 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to exp2, 
*		 otherwise kexp2 is a macro to the Khoros
*		 portable version of exp2.
*
*         Input: x - variable whose base 2 exponential is to be computed.
*
*        Output: 
*
*	Returns: the double precision base 2 exponential of the input argument.
*               
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kexp2(double x)
*
*************************************************************/
#if defined(KEXP2_DEF)
   double (*kexp2) PROTO((double)) = __kexp2;
#else
   extern double  exp2 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kexp2) PROTO((double)) = exp2;
#endif


/************************************************************
*
*  Routine Name: kexp10 - compute the base 10 exponential function of the argument.
*
*       Purpose: The kexp10 function computes the base 10 exponential
*		 function.
*
*		 kexp10 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to exp10, 
*		 otherwise kexp10 is a macro to the Khoros
*		 portable version of exp10.
*
*         Input: x - variable whose base 10 exponential is to be computed.
*        Output: 
*	Returns: the double precision base 10 exponential of the input 
*		 argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double kexp10(double x)
*
*************************************************************/
#if defined(KEXP10_DEF)
   double (*kexp10) PROTO((double)) = __kexp10;
#else
   extern double  exp10 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*kexp10) PROTO((double)) = exp10;
#endif


/************************************************************
*
*  Routine Name: klog1p - compute the logarithm of x+1 of the argument.
*
*       Purpose: The klog1p function computes the natural
*		 logarithm of the double precision argument plus 1.
*
*		 klog1p is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to log1p, 
*		 otherwise klog1p is a macro to the Khoros
*		 portable version of log1p.
*
*         Input: x - argument to klog1p
*        Output: 
*	Returns: the double precision logarithm plus one of the input
*		 argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double klog1p(double x)
*
*************************************************************/
#if defined(KLOG1P_DEF)
   double (*klog1p) PROTO((double)) = __klog1p;
#else
   extern double  log1p PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*klog1p) PROTO((double)) = log1p;
#endif

/************************************************************
*
*  Routine Name: klog2 - compute the base 2 logarithm of the argument.
*
*       Purpose: The klog2 function computes the base 2
*		 logarithm of the double precision input argument.
*
*		 klog2 is not a POSIX function, but is
*		 implemented in a few BSD math libraries.
*		 If it is implemented for the current architecture
*		 in libm.a, then this is simply a macro to log2, 
*		 otherwise klog2 is a macro to the Khoros
*		 portable version of log2.
*
*         Input: x - argument to klog2
*        Output: 
*	Returns: the double precision base 2 logarithm of the double
*		 precision input argument.
*  Restrictions:
*    Written By: Jeremy Worley and Mark Young
*          Date: Mar 31, 1994
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: double klog2(double x)
*
*************************************************************/
#if defined(KLOG2_DEF)
   double (*klog2) PROTO((double)) = __klog2;
#else
   extern double  log2 PROTO((double));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   double (*klog2) PROTO((double)) = log2;
#endif

/************************************************************
*
*  Routine Name: krandom - generate random number in range [0,2**31-1]
*
*       Purpose: The krandom function generates a pseudo random
*		 number with a value in the range [0,2**31-1].
*		 The starting point of the pseudo random number sequence
*		 is set by calling ksrandom.
*
*		 krandom is not a POSIX function, but is
*		 implemented in most BSD libraries.
*		 If it is implemented for the current architecture
*		 in libc.a, then this is simply a macro to random, 
*		 otherwise krandom is a macro to the Khoros
*		 portable version of random.
*
*         Input: 
*        Output: 
*       Returns: the pseudo random number generated.
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young
*          Date: Jul 11, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: long krandom(void)
*
*************************************************************/
#if defined(KRANDOM_DEF)
   long (*krandom) PROTO((void)) = __krandom;
#else
   extern long random PROTO((void));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   long (*krandom) PROTO((void)) = random;
#endif


/************************************************************
*
*  Routine Name: ksrandom - seed the krandom random number generator.
*
*       Purpose: The ksrandom function sets the seed (starting value)
*		 of the random number generation algorithm used by
*		 krandom.
*
*		 ksrandom is not a POSIX function, but is
*		 implemented in most BSD libraries.
*		 If it is implemented for the current architecture
*		 in libc.a, then this is simply a macro to srandom, 
*		 otherwise ksrandom is a macro to the Khoros
*		 portable version of srandom.
*
*         Input: 
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young
*          Date: Jul 11, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: void ksrandom(int seed)
*
*************************************************************/
#if defined(KSRANDOM_DEF)
   void (*ksrandom) PROTO((int)) = __ksrandom;
#else
   extern void srandom PROTO((int));
/* this comment is a kludge for the header parser -- don't remove
{
*/
   void (*ksrandom) PROTO((int)) = srandom;
#endif
