/*
 * Khoros: $Id: hash.c,v 1.17 1996/08/10 03:14:56 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: hash.c,v 1.17 1996/08/10 03:14:56 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Hash Utility
   >>>>		
   >>>>   Static:
   >>>>                 kvarg_entry()
   >>>>                 kadd_entry()
   >>>>                 kdelete_entry()
   >>>>                 klocate_entry()
   >>>>   Public:
   >>>>                 khash_value()
   >>>>                 khash_init()
   >>>>                 khash_add()
   >>>>                 khash_clientdata()
   >>>>                 khash_check()
   >>>>                 khash_delete()
   >>>>                 khash_free()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-----------------------------------------------------------
|
|  Routine Name: kvarg_entry - gets the entry off the arg list
|
|       Purpose: This routine gets the entry off the arg list.
|         Input: list - the variable arg list
|       Returns: returns a pointer to the argument
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static kaddr kvarg_entry(int type, kaddr entry, kva_list *list, ssize_t *esize)
{

#define doit(dtype, ltype, entry, list) \
{								\
	dtype *data = (dtype *) entry;				\
	*data  = (dtype) kva_arg(*list, ltype);			\
	*esize = (ssize_t)sizeof(dtype);			\
}
	switch (type)
	{
	   case KBYTE:   doit(signed char, signed int, entry, list); break;
	   case KUBYTE:  doit(unsigned char, unsigned int, entry, list); break;
	   case KSHORT:  doit(signed short, signed int, entry, list); break;
	   case KUSHORT: doit(unsigned short, unsigned int, entry, list); break;
	   case KINT:    doit(signed int, signed int, entry, list); break;
	   case KUINT:   doit(unsigned int, unsigned int, entry, list); break;
	   case KLONG:   doit(signed long, signed long, entry, list); break;
	   case KULONG:  doit(unsigned long, unsigned long, entry, list); break;
	   case KFLOAT:  doit(float, double, entry, list); break;
	   case KDOUBLE: doit(double, double, entry, list); break;
	   case KSTRUCT: doit(kaddr, kaddr, entry, list); break;
	   case KLOGICAL: doit(kbool, signed int, entry, list); break;
	   case KSTRING:  entry = kva_arg(*list, kstring), *esize = -1; break;
	   default:
		return(NULL);
	}
#undef doit
	return(entry);
}

/*-----------------------------------------------------------
|
|  Routine Name: kadd_entry - adds the entry into the array
|
|       Purpose: This routine adds the entry in to the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static int kadd_entry(khash *hash, unsigned long indx, kaddr entry, kaddr client_data)
{
	int     type  = hash->type;
	size_t  num   = (size_t) hash->num[indx];
	kaddr  *table = hash->table[indx];
	kaddr  *cdata = hash->data[indx];


#define doit(dtype, table, type, entry, num) \
{									      \
	dtype *data = (dtype *) entry;					      \
	table = karray_insert(table, type, *data, num, KLIST_TAIL, TRUE);     \
}
	switch (type)
	{
	   case KBYTE:    doit(signed char, table, type, entry, num); break;
	   case KUBYTE:   doit(unsigned char, table, type, entry, num); break;
	   case KSHORT:   doit(signed short, table, type, entry, num); break;
	   case KUSHORT:  doit(unsigned short, table, type, entry, num); break;
	   case KINT:     doit(signed int, table, type, entry, num); break;
	   case KUINT:    doit(unsigned int, table, type, entry, num); break;
	   case KLONG:    doit(signed long, table, type, entry, num); break;
	   case KULONG:   doit(unsigned long, table, type, entry, num); break;
	   case KFLOAT:   doit(float, table, type, entry, num); break;
	   case KDOUBLE:  doit(double, table, type, entry, num); break;
	   case KSTRUCT:  doit(kaddr, table, type, entry, num); break;
	   case KLOGICAL: doit(kbool, table, type, entry, num); break;
	   case KSTRING:  doit(kstring, table, type, &entry, num); break;
	   default: return(FALSE);
	}
#undef doit
	cdata = karray_insert(cdata,KSTRUCT,client_data,num++,KLIST_TAIL,TRUE);
	hash->num[indx]   = (int) num;
	hash->table[indx] = table;
	hash->data[indx]  = cdata;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: kdelete_entry - delete the entry into the array
|
|       Purpose: This routine delete the entry from the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static int kdelete_entry(khash *hash, unsigned long indx, kaddr entry)
{
	int    i, type = hash->type;
	size_t  num    = (size_t) hash->num[indx];
	kaddr  *table  = hash->table[indx];
	kaddr  *cdata  = hash->data[indx];


#define doit(dtype, table, type, entry, num) \
{									      \
	dtype *data = (dtype *) entry;					      \
	if ((i = karray_locate(table, type, *data, num)) == -1)               \
	   return(FALSE);						      \
									      \
	table = karray_delete(table, type, *data, num);   		      \
}
	switch (type)
	{
	   case KBYTE:    doit(signed char, table, type, entry, num); break;
	   case KUBYTE:   doit(unsigned char, table, type, entry, num); break;
	   case KSHORT:   doit(signed short, table, type, entry, num); break;
	   case KUSHORT:  doit(unsigned short, table, type, entry, num); break;
	   case KINT:     doit(signed int, table, type, entry, num); break;
	   case KUINT:    doit(unsigned int, table, type, entry, num); break;
	   case KLONG:    doit(signed long, table, type, entry, num); break;
	   case KULONG:   doit(unsigned long, table, type, entry, num); break;
	   case KFLOAT:   doit(float, table, type, entry, num); break;
	   case KDOUBLE:  doit(double, table, type, entry, num); break;
	   case KSTRUCT:  doit(kaddr, table, type, entry, num); break;
	   case KLOGICAL: doit(kbool, table, type, entry, num); break;
	   case KSTRING:  doit(kstring, table, type, &entry, num); break;
	   default: return(FALSE);
	}
#undef doit
	cdata = karray_delete(cdata, KSTRUCT, cdata[i], num--);
	hash->num[indx]   = (int) num;
	hash->table[indx] = table;
	hash->data[indx]  = cdata;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: klocate_entry - gets the location of the entry 
|
|       Purpose: This routine gets the location of the entry in the array
|         Input: 
|       Returns: 
|
|    Written By: Mark Young
|          Date: Sep 13, 1995
|
------------------------------------------------------------*/
static ssize_t klocate_entry(khash *hash, ssize_t indx, kaddr entry)
{
	int     type  = hash->type;
	size_t  num   = (size_t) hash->num[indx];
	kaddr  *table = hash->table[indx];


#define doit(dtype, table, type, entry, num) \
{								\
	dtype *data = (dtype *) entry;				\
	indx = karray_locate(table, type, *data, num);		\
}
	switch (type)
	{
	   case KBYTE:   doit(signed char, table, type, entry, num); break;
	   case KUBYTE:  doit(unsigned char, table, type, entry, num); break;
	   case KSHORT:  doit(signed short, table, type, entry, num); break;
	   case KUSHORT: doit(unsigned short, table, type, entry, num); break;
	   case KINT:    doit(signed int, table, type, entry, num); break;
	   case KUINT:   doit(unsigned int, table, type, entry, num); break;
	   case KLONG:   doit(signed long, table, type, entry, num); break;
	   case KULONG:  doit(unsigned long, table, type, entry, num); break;
	   case KFLOAT:  doit(float, table, type, entry, num); break;
	   case KDOUBLE: doit(double, table, type, entry, num); break;
	   case KSTRUCT: doit(kaddr, table, type, entry, num); break;
	   case KLOGICAL: doit(kbool, table, type, entry, num); break;
	   case KSTRING: doit(kstring, table, type, &entry, num); break;
	   default: return(-1);
	}
#undef doit
	return(indx);
}


/************************************************************
*
*  Routine Name: khash_value - polynomial conversion 
*
*       Purpose: polynomial conversion ignoring overflows
*		 [this seems to work remarkably well, in fact better
*		 then the ndbm hash function. Replace at your own risk]
*
*		 ! 	use:  65599	nice.
*		 ! 	      65587   even better. 
*
*         Input: data - the data in which to create the has for
*		 length - the length of the hash to be used.  If -1,
*			  then the length is computed using kstrlen().
*        Output:
*       Returns: returns the associated hash for the given data.
*
*  Restrictions:
*    Written By: Ozan S. Yigit (oz@nexus.yorku.ca)
*          Date: 1991 
*      Verified:
*  Side Effects:
* Modifications: Integrated into Khoros 2.0 (MY) 4/93
*   Declaration: unsigned long int khash_value(
*		!   char *data,
*		!   ssize_t     length)
*
*************************************************************/

unsigned long khash_value(
   register const char *data,
   register ssize_t  length)
{
	register unsigned long n = 0;


#define HASHC	n = *data++ + 65599 * n
	if (length <= -1)
	   length = (ssize_t) kstrlen(data);

	switch (length % 8)
	{
	   case 7: HASHC;
	   case 6: HASHC;
	   case 5: HASHC;
	   case 4: HASHC;
	   case 3: HASHC;
	   case 2: HASHC;
	   case 1: HASHC;
	}

	length >>= 3;
	while (length-- > 0)
	{
	    HASHC; HASHC; HASHC; HASHC;
	    HASHC; HASHC; HASHC; HASHC;
	}
	return n;
}

/************************************************************
*
*  Routine Name: khash_init - initializes a hash array 
*
*       Purpose: Initializes a hash array, which can then be used
*		 to add, delete, check, and retrieve data associated
*		 with hash entries.
*
*		 The hash utilities, including khash_add(), support 
*		 hash arrays of a variety of data types.  The data type of the 
*		 array being used is specified with the 'type' parameter; 
*		 supported data types include:
*
*		 !	KBYTE    - hash array of characters
*		 !	KUBYTE   - hash array of unsigned characters
*		 !	KSHORT   - hash array of short integers
*		 !	KUSHORT  - hash array of unsigned short integers
*		 !	KINT     - hash array of integers
*		 !	KUINT    - hash array of unsigned integers
*		 !	KLONG    - hash array of long integers
*		 !	KULONG   - hash array of unsigned long integers
*		 !	KFLOAT   - hash array of floating point numbers
*		 !	KDOUBLE  - hash array of double precision numbers
*		 !	KSTRING  - hash array of strings
*		 !	KSTRUCT  - hash array of pointers to structures
*		 !	KLOGICAL - hash array of TRUE/FALSE values
*
*	  Input: type - The hash data type
*		 size - The hash size, if -1 then dynamically grows.
*        Output: 
*       Returns: the newly initialized hash structure, or NULL on failure
*    Written By: Mark Young
*          Date: Sep 13, 1995
*************************************************************/
khash *khash_init(
   int    type,
   ssize_t size)
{
	khash *hash;

	if ((hash = (khash *) kcalloc((size_t) 1, sizeof(khash))) == NULL)
	   return(NULL);

	hash->type = type;
	hash->size = size;
	hash->num   = (int *)   kcalloc((size_t) size, sizeof(int));
	hash->table = (kaddr **) kcalloc((size_t) size, sizeof(kaddr *));
	hash->data  = (kaddr **) kcalloc((size_t) size, sizeof(kaddr *));
	return(hash);
}

/************************************************************
*
*  Routine Name: khash_add - adds an entry into the hash array
*
*       Purpose: Inserts an entry into the hash array, and an
*		 associated piece of data.
*
*		 The hash utilities, including khash_add(), support 
*		 hash arrays of a variety of data types.  The data type of the 
*		 array being used is specified with the 'type' parameter; 
*		 supported data types include:
*
*		 !	KBYTE    - hash array of characters
*		 !	KUBYTE   - hash array of unsigned characters
*		 !	KSHORT   - hash array of short integers
*		 !	KUSHORT  - hash array of unsigned short integers
*		 !	KINT     - hash array of integers
*		 !	KUINT    - hash array of unsigned integers
*		 !	KLONG    - hash array of long integers
*		 !	KULONG   - hash array of unsigned long integers
*		 !	KFLOAT   - hash array of floating point numbers
*		 !	KDOUBLE  - hash array of double precision numbers
*		 !	KSTRING  - hash array of strings
*		 !	KSTRUCT  - hash array of pointers to structures
*		 !	KLOGICAL - hash array of TRUE/FALSE values
*
*		 Note that the hash array and the entry passed in MUST
*		 both be of the type specified by the type parameter,
*		 originally used in creating the hash array or the results
*		 of this routine are unpredictable.
*
*	  Input: hash        - The hash array to which to add the entry
*		 entry       - The entry to be added to the hash array
*		 client_data - The piece of data to be associated with the
*			       hash.
*        Output: 
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_add(
*		!  khash *hash,
*		!  <type>  entry,
*		!  kaddr client_data)
*************************************************************/
int khash_add(
   khash *hash,
   kvalist)
{
	kva_list list;
	unsigned long indx;
	ssize_t  esize;
	char     space[KLENGTH];
	kaddr    entry, client_data;


	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list, &esize);

	/*
	 *  Get the rest of the arguments
	 */
	client_data = kva_arg(list, kaddr);

	indx = khash_value(entry, esize) % hash->size;
	kadd_entry(hash, indx, entry, client_data);
	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_clientdata - return the client data associated with
*				 the hash entry
*
*       Purpose: This function returns the client data that is associated with
*		 the hash entry.  The idea is that a unique hash entry
*		 is given for each entry sent to the function
*		 khash_add().  This routine is used to retrieve a
*		 pointer to the hash's associated client data.
*
*	  Input: hash - the hash entry
*       Returns: The token's corresponding string or NULL, if the token is
*		 not valid.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_clientdata(
*		!  khash *hash,
*		!  <type>  entry,
*		!  kaddr *client_data)
*************************************************************/
int khash_clientdata(
   khash *hash,
   kvalist)
{
	kva_list list;
	ssize_t  indx;
        ssize_t  esize;
	char     space[KLENGTH];
	kaddr    entry, *client_data, *data;

	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list, &esize);

	/*
	 *  Get the rest of the arguments
	 */
	client_data = kva_arg(list, kaddr *);

	indx = khash_value(entry, esize) % hash->size;
	data = hash->data[indx];
	if ((indx = klocate_entry(hash, indx, entry)) == -1)
	   return(FALSE);

	*client_data = data[indx];
	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_check - check to see if a hash entry exists
*
*       Purpose: This function checks to see if a hash entry has already
*		 added using khash_add().
*
*	  Input: hash - the hash array
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_check(
*		!  khash *hash,
*		!  <type>  entry)
*************************************************************/
int khash_check(
   khash *hash,
   kvalist)
{
	kva_list list;
	kaddr    entry;
	ssize_t  esize;
	unsigned long indx;
	char     space[KLENGTH];


	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list, &esize);

	indx = khash_value(entry, esize) % hash->size;
        if (klocate_entry(hash, (ssize_t)indx, entry) == -1)
           return(FALSE);

	return(TRUE);
}

/************************************************************
*
*  Routine Name: khash_delete - delete an entry from the hash array
*
*       Purpose: This function deletes an entry from the hash array.
*
*	  Input: hash - the hash array to delete the entry from
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_delete(
*		!  khash *hash,
*		!  <type>  entry)
*************************************************************/
int khash_delete(
   khash *hash,
   kvalist)
{
	kva_list list;
	ssize_t  esize;
	kaddr    entry;
	int	 status;
	unsigned long indx;
	char     space[KLENGTH];


	/*
	 *  Get the entry off the list
	 */
	kva_start(list, hash);
	entry = kvarg_entry(hash->type, space, &list, &esize);
        indx = khash_value(entry, esize) % hash->size;
	status = kdelete_entry(hash, indx, entry);
	return(status);
}

/************************************************************
*
*  Routine Name: khash_free - frees the hash array and all memory
*
*       Purpose: This function frees the hash array and all associated
*		 memory.
*
*	  Input: hash    - the hash array to be freed
*		 routine - A routine to free the clientdata pointer for each
*			   hash array member.  If routine is NULL, then it
*			   simply calls kfree_and_NULL() to free that
*			   data.
*       Returns: TRUE (1) on success, FALSE (0) otherwise.
*    Written By: Mark Young
*          Date: Sep 13, 1995
*
*   Declaration: int khash_free(
*		!  khash  *hash,
*		!  void  (*routine) PROTO((kaddr)) )
*************************************************************/
int khash_free(
   khash *hash,
   void (*routine) PROTO((kaddr)) )
{
	int i, tmin, tmax, total;
        size_t num;

	tmin = tmax = hash->num[0];
	for (i = total = 0; i < hash->size; i++)
	{
	   if ((num = (size_t) hash->num[i]) == 0)
	      continue;

	   karray_free(hash->data[i],  KSTRUCT, num, routine);
	   karray_free(hash->table[i], hash->type, num, NULL);
#if 0
	   if (hash->num[i] < tmin)
	      tmin = hash->num[i];
	   else if (hash->num[i] > tmax)
	      tmax = hash->num[i];

	   total += hash->num[i];
#endif
	}
#if 0
	kfprintf(kstderr, "Largest  bucket size  was %d\n", tmax);
	kfprintf(kstderr, "Smallest bucket size  was %d\n", tmin);
	kfprintf(kstderr, "Total    bucket elems was %d\n", total);
	kfprintf(kstderr, "Average  bucket size  was %d\n", total/hash->size);
#endif
	kfree_and_NULL(hash->num);
	kfree_and_NULL(hash->data);
	kfree_and_NULL(hash->table);
	kfree_and_NULL(hash);
	return(TRUE);
}
