/*
 * Khoros: $Id: toolboxes.c,v 1.13.2.1 1996/12/06 20:28:07 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: toolboxes.c,v 1.13.2.1 1996/12/06 20:28:07 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Toolbox Utilities
   >>>>		
   >>>>  Private:
   >>>>                 ktoolbox_siteinclude()
   >>>>                 ktoolbox_initialize()
   >>>>                 ktoolbox_list()
   >>>>                 ktoolbox_check()
   >>>>                 ktoolbox_path()
   >>>>                 ktoolbox_libpath()
   >>>>                 ktoolbox_add()
   >>>>                 ktoolbox_delete()
   >>>>   Static:
   >>>>                 ktoolbox_dirinclude()
   >>>>                 ktoolbox_binpath()
   >>>>                 ktoolbox_filepaths()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	


static size_t   numtb = 0;
static char   **toolboxes = NULL;
static char   **toolbox_path = NULL;
static kbool   *toolbox_machdef = NULL;
static kbool   *toolbox_globaldir = NULL;
static ssize_t  bs_indx = -1;
static char    *bs_binpath = NULL;
static char    *bs_libpath = NULL;

#ifndef SITE_INCLUDE_FILE
#define SITE_INCLUDE_FILE NULL
#endif

#ifndef KHOROS_TOOLBOX_PATH
#define KHOROS_TOOLBOX_PATH NULL
#endif

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_siteinclude - 
|
|       Purpose: The path to the site include file
|
|         Input: 
|        Output: the path to the site include, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: John MA Salas
|          Date: Nov 13, 1995
| Modifications:
|
------------------------------------------------------------*/

char *ktoolbox_siteinclude(
   const char *tb_name,
   char *path)
{
        char *karch, *kopsys, *kwidset, filename[KLENGTH];
	int  location;
 
	location = karray_locate(toolboxes, KSTRING, tb_name, numtb);

	if (location == -1)
	   return(NULL);

        if ((toolbox_machdef[location] == TRUE) &&
	    ((karch = kgetenv("KARCH")) != NULL))
        {
           kopsys  = kgetenv("KOPSYS");
           kwidset = kgetenv("KWIDSET");
           if ((kwidset == NULL && kopsys == NULL) ||
               (kwidset != NULL && kopsys == NULL))
           {
              ksprintf(filename, "$%s/config/Site.%s", tb_name, karch);
           }
           else if (kopsys != NULL && kwidset == NULL)
              ksprintf(filename, "$%s/config/Site.%s-%s", tb_name,karch,kopsys);
           else if (kopsys != NULL && kwidset != NULL)
              ksprintf(filename, "$%s/config/Site.%s-%s-%s", tb_name,
			karch, kopsys, kwidset);
        }
	else
           ksprintf(filename, "$%s/config/%s", tb_name, SITE_INCLUDE_FILE);

	path = (path == NULL) ? kstrdup(filename) : kstrcpy(path, filename);
	return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_dirinclude - 
|
|       Purpose: The path to the dir include file
|
|         Input: 
|        Output: the path to the dir include, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: Mark Young
|          Date: Aug 20, 1995
| Modifications:
|
------------------------------------------------------------*/

static char *ktoolbox_dirinclude(
   char *path,
   int   machdef)
{
        char  *karch, *kopsys, *kwidset, filename[KLENGTH];
 
        if ((machdef == TRUE) && ((karch = kgetenv("KARCH")) != NULL))
        {
           kopsys  = kgetenv("KOPSYS");
           kwidset = kgetenv("KWIDSET");
           if ((kwidset == NULL && kopsys == NULL) ||
               (kwidset != NULL && kopsys == NULL))
           {
              ksprintf(filename, "Dir.%s", karch);
           }
           else if (kopsys != NULL && kwidset == NULL)
              ksprintf(filename, "Dir.%s-%s", karch, kopsys);
           else if (kopsys != NULL && kwidset != NULL)
              ksprintf(filename, "Dir.%s-%s-%s", karch, kopsys, kwidset);
        }
	else
	   kstrcpy(filename, "Dir.def");

	path = (path == NULL) ? kstrdup(filename) : kstrcpy(path, filename);
	return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_binpath - 
|
|       Purpose: The toolbox bin path.
|
|         Input: tbname - the toolbox name
|        Output: the path to the toolbox bin, if NULL malloc
|       Returns: return the toolbox bin path or NULL
|
|    Written By: Mark Young & John Salas
|          Date: Jul 17, 1994
| Modifications:
|
------------------------------------------------------------*/

static char *ktoolbox_binpath(
   const char *tbname,
   char  *path,
   int   globaldir,
   int   machdef)
{
	kfile  *file;
	ssize_t count;
	int    status;
        char   temp[KLENGTH], filename[KLENGTH], location[KLENGTH],
	       *binpath, **scan;
 
	if (!ktoolbox_dirinclude(temp, machdef))
	   return(NULL);

	binpath = NULL;
	ksprintf(filename, "$%s/config/%s", tbname, temp);
	if (machdef == TRUE && kaccess(filename, R_OK) != 0)
	   ksprintf(filename, "$%s/config/Dir.def", tbname);
	if ((file = kfinput(filename)) != NULL)
	{
	   scan = kparse_file_scan_delimit(file, "^\\s*BinDir", "$",
			KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

	   if (count > 1)
	      binpath = kstrdup(scan[1]);

	   karray_free(scan, KSTRING, (size_t) count, NULL);
	   kfclose(file);
	}
	else if (globaldir)
	{
	   if (bs_binpath == NULL)
	   {
	      if (!ktoolbox_dirinclude(temp, toolbox_machdef[bs_indx]))
	         return(NULL);

	      kremote_location(kgetenv(tbname), location);
	      ksprintf(filename, "$BOOT""STRAP/config/%s", temp);
	      kremote_file(location, filename, temp);
	      if (toolbox_machdef[bs_indx] == TRUE && kaccess(temp, R_OK) != 0) 
	      {
	         ksprintf(filename, "$BOOT""STRAP/config/Dir.def");
	         kremote_file(location, filename, temp);
	      }

	      if ((file = kfinput(temp)) != NULL)
	      {
	         scan = kparse_file_scan_delimit(file, "^\\s*BinDir", "$",
			KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

	         if (count > 1)
		    bs_binpath = kremote_exec(location, scan[1], NULL);

	         karray_free(scan, KSTRING, (size_t) count, NULL);
	         kfclose(file);
	      }

	      if (bs_binpath == NULL)
	         bs_binpath = kfullpath("$BOOTSTRAP/bin", NULL, NULL);
	   }
	   binpath = kstrdup(bs_binpath);
	}

	/* if no binpath was found then return NULL */
	if (binpath == NULL)
	   return(NULL);

	path = (path == NULL) ? kstrdup(binpath) : kstrcpy(path, binpath);
	kfree(binpath);
	return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_filepaths - 
|
|       Purpose: returns a color seperated list of paths to toolbox config
|		 files
|
|         Input: 
|        Output: the path to the toolbox config files, if NULL malloc
|       Returns: return the site include path or NULL
|
|    Written By: Mark Young
|          Date: Aug 20, 1995
| Modifications:
|
------------------------------------------------------------*/

static char *ktoolbox_filepaths(
   char *paths)
{
	char *names, temp[KLENGTH];

	if ((names = (char *) kgetenv("KHOROS_TOOLBOX")) == NULL)
	{
	   ksprintf(temp, "$HOME/.Toolboxes:%s", KHOROS_TOOLBOX_PATH);
	   names = temp;
	}
	paths = (paths == NULL) ? kstrdup(names) : kstrcpy(paths, names);
	return(paths);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_initialize - initialize the toolbox routines
|
|       Purpose: This subroutine initializes the global list of
|		 toolboxes used to represent the different toolboxes
|		 currently to be used by the Khoros programs.
|
|         Input:
|        Output:
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young  
|          Date: Sep 23, 1993
| Modifications:
|
------------------------------------------------------------*/

int ktoolbox_initialize(void)
{
	kfile  *file;
	char   *filenames, *tmp, *tmp1, filename[KLENGTH],
		temp[KLENGTH], tbname[KLENGTH], tbpath[KLENGTH],
		name[KLENGTH], def[KLENGTH], options[KLENGTH],
		paths[KLENGTH], binpath[KLENGTH];
	int	i, globaldir = FALSE, machdef = FALSE;

	if (toolboxes != NULL)
	   return(TRUE);

	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	while (filenames != NULL)
	{
	   if ((tmp = kstrchr(filenames, ':')) != NULL)
	   {
	      kstring_ncopy(filenames, (size_t) (tmp-filenames), filename);
	      tmp++;
	   }
	   else
	      kstrcpy(filename, filenames);

	   /* KHOROS_TOOLBOX variable cannot be opened */
	   if ((file = kfinput(filename)) == NULL)
	   {
	      kinfo(KSYSLIB, "ktoolbox_initialize: Unable to open toolbox "
			     "file '%s' for reading.", filename);
	      filenames = tmp;
	      continue;
	   }

	   while (kfgets(temp, KLENGTH, file) != NULL)
	   {
	      *options = '\0';
	      if (ksscanf(temp,"%[^#:]:%[^;];%[^\n]",tbname,tbpath,options) < 2)
		 continue;

	      kstring_cleanup(tbname, tbname);
	      kstring_cleanup(tbpath, tbpath);
	      if (kgetenv(tbname) == NULL)
	      {
	         ksprintf(binpath, "%s=%s", tbname, tbpath);
	         kputenv(binpath);
	      }

	      /* make sure the does not already exist */
	      if (karray_locate(toolboxes, KSTRING, tbname, numtb) != -1)
	         continue;

	      if (kstrcmp(tbname, "BOOT""STRAP") == 0)
		 bs_indx = numtb;

	      /*
	       *  Check to see if this toolbox might be supporting multiple
	       *  toolbox bins.  We know this if there is an options part
	       *  of the toolbox definition (from the khoros toolbox file)
	       *  and "machdefs=TRUE" is present.
	       */
	      tmp1 = options;
	      globaldir = machdef = FALSE;
	      while (ksscanf(tmp1, "%[^=]=%[^,\n]", name, def) == 2)
	      {
	         kstring_cleanup(def, def); kstring_lower(def, def);
		 if (kstrcmp(def, "yes") == 0  || kstrcmp(def, "true") == 0 || 
		     kstrcmp(def, "on") == 0 || kstrcmp(def, "1") == 0)
		 {
		    kstring_cleanup(name, name); kstring_lower(name, name);
		    if (kstrcmp(name, "globaldir") == 0)
		       globaldir = TRUE;
		    else
		       machdef = TRUE;
		 }

		 if ((tmp1 = kstrchr(tmp1, ',')) != NULL)
		    tmp1++;
		 else
		    break;
	      }
	      toolboxes = karray_insert(toolboxes, KSTRING, kstrdup(tbname),
				   numtb, KLIST_TAIL, TRUE);
	      toolbox_path = karray_insert(toolbox_path, KSTRING,
				   kstrdup(tbpath), numtb, KLIST_TAIL, TRUE);
	      toolbox_machdef = karray_insert(toolbox_machdef, KLOGICAL,
				   machdef, numtb, KLIST_TAIL, TRUE);
	      toolbox_globaldir = karray_insert(toolbox_globaldir, KLOGICAL,
				   globaldir, numtb++, KLIST_TAIL, TRUE);
	   }
	   kfclose(file);
	   filenames = tmp;
	}

	for (i = 0; i < numtb; i++)
	{
           ksprintf(binpath, "%sBIN", toolboxes[i]);
           if (kgetenv(binpath) != NULL)
              continue;

	   ksprintf(temp, "%s/bin", toolbox_path[i]);
	   if (ktoolbox_binpath(toolboxes[i], temp, toolbox_globaldir[i],
				toolbox_machdef[i]) == NULL)
	      ksprintf(binpath, "%sBIN=%s/bin", toolboxes[i], toolbox_path[i]);
	   else
	      ksprintf(binpath, "%sBIN=%s", toolboxes[i], temp);
	   
	   kinfo(KDEBUG, "defining binpath: '%s'\n", binpath);
	   kputenv(binpath);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_list - create a list of toolbox names
|
|       Purpose: This routine takes the global toolbox list, and
|		 returns an array of toolbox names to the calling
|		 routine.  It also takes a count parameter that
|		 is set to the number of entries in the toolbox array.
|
|         Input: none
|
|        Output: num - The number of toolbox names in the array; 0 on error.
|       Returns: The list of toolbox names if successful, or
|			  NULL on error.
|
|    Written By: Mark Young
|          Date: Sep 23, 1993
| Modifications:
|
------------------------------------------------------------*/

char **ktoolbox_list(
   size_t *num)
{
	ktoolbox_initialize();

	if (num) *num = numtb;
	return(toolboxes);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_check - check to see if the toolbox exists
|				in the toolbox list.
|
|       Purpose: This routine checks to see if a toolbox exists in the
|		 toolbox list.
|         Input: tbname - the toolbox name to check for
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Jul 29, 1994
| Modifications:
|
------------------------------------------------------------*/

int ktoolbox_check(
   const char *tbname)
{
	int  i;

	ktoolbox_initialize();
	for (i = 0; i < numtb; i++)
	{
	   if (kstrcasecmp(tbname, toolboxes[i]) == 0)
	      return(TRUE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_libpath - get library path for toolbox
|
|       Purpose: This routine returns the library path assoicated
|                with a specific toolbox.  Called by kecho.
|         Input: tbname - the name of the toolbox
|        Output:
|       Returns: corresponding toolbox path, or NULL if not found or on error
|
|    Written By: Kevin Sullivan
|          Date: August 29, 1996
| Modifications:
|
------------------------------------------------------------*/

char *ktoolbox_libpath(
   const char *tbname,
   char       *path)
{
        int  i;
        int   globaldir;
        int   machdef;
        kfile  *file;
        ssize_t count;
        int    status;
        char   temp[KLENGTH], filename[KLENGTH], location[KLENGTH],
               *libpath, **scan;
        char   *tbpath;

        ktoolbox_initialize();
        for (i = 0; i < numtb; i++)
        {
           if (kstrcasecmp(tbname, toolboxes[i]) == 0)
           {
              globaldir = toolbox_globaldir[i];
              machdef   = toolbox_machdef[i];
              tbpath    = kstrdup(toolbox_path[i]);
           }
        }

        if (!ktoolbox_dirinclude(temp, machdef))
           return(NULL);

        libpath = NULL;
        ksprintf(filename, "$%s/config/%s", tbname, temp);
        if (machdef == TRUE && kaccess(filename, R_OK) != 0)
           ksprintf(filename, "$%s/config/Dir.def", tbname);
        if ((file = kfinput(filename)) != NULL)
        {
           scan = kparse_file_scan_delimit(file, "^\\s*LibDir", "$",
                        KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

           if (count > 1)
              libpath = kstrdup(scan[1]);

           karray_free(scan, KSTRING, (size_t) count, NULL);
           kfclose(file);
        }
        else if (globaldir)
        {
           if (bs_libpath == NULL)
           {
              if (!ktoolbox_dirinclude(temp, toolbox_machdef[bs_indx]))
                 return(NULL);

              kremote_location(kgetenv(tbname), location);
              ksprintf(filename, "$BOOT""STRAP/config/%s", temp);
              kremote_file(location, filename, temp);
              if (toolbox_machdef[bs_indx] == TRUE && kaccess(temp, R_OK) != 0)
              {
                 ksprintf(filename, "$BOOT""STRAP/config/Dir.def");
                 kremote_file(location, filename, temp);
              }

              if ((file = kfinput(temp)) != NULL)
              {
                 scan = kparse_file_scan_delimit(file, "^\\s*LibDir", "$",
                        KIGNORE_CASE, "[ \t]", NULL, NULL, &count, &status);

                 if (count > 1)
                    bs_libpath = kremote_exec(location, scan[1], NULL);

                 karray_free(scan, KSTRING, (size_t) count, NULL);
                 kfclose(file);
              }

              if (bs_libpath == NULL)
                 bs_libpath = kfullpath("$BOOTSTRAP/lib", NULL, NULL);
           }
           libpath = kstrdup(bs_libpath);
        }

        /* if no libpath was found then return default */
        if (libpath == NULL)
        {
           ksprintf(temp, "%s/lib", tbpath);
           libpath = kstrdup(temp);
        }

        path = (path == NULL) ? kstrdup(libpath) : kstrcpy(path, libpath);
        kfree(libpath);
        return(path);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_path - get a path entry from a specific toolbox
|
|       Purpose: This subroutine returns the path assoicated with
|                a specific toolbox.  The path is part of the global
|                structure used internally by the khoros routines
|                and should be treated as a read-only resource.
|
|         Input: variable - the name of the toolbox
|       Returns: corresponding toolbox path, or NULL if not found or on error
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
char *ktoolbox_path(
   const char *variable)
{
	char  cleanup[KLENGTH];

	ktoolbox_initialize();
	if (!kstring_cleanup(variable, cleanup))
	   return(NULL);

	kstring_upper(cleanup, cleanup);
	return(kgetenv(cleanup));
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_add - add a new toolbox to the list
|
|       Purpose: This subroutine adds a new toolbox to the global
|                toolbox list.  This routine takes all the information
|                required to store a Khoros Toolbox entry.
|
|         Input: variable - the name of the toolbox
|		 path     - the path for the top directory of the toolbox
|       Returns: return TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
int ktoolbox_add(
   const char *variable,
   const char *path)
{
	kfile   *file;
	char    *filenames, *tmp, **list;
	kstring routine = "ktoolbox_add()";
	ssize_t  i, num;
        int     status, written, first = TRUE;
	char    temp[KLENGTH], filename[KLENGTH];
	char    tbname[KLENGTH], tbvar[KLENGTH], tbpath[KLENGTH],
		paths[KLENGTH];


	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	kstring_cleanup(path, tbpath);
	kstring_cleanup(variable, tbvar);
	kstring_upper(tbvar, tbvar);

	while (filenames != NULL)
	{
	  written = FALSE;
	  if ((tmp = kstrchr(filenames, ':')) != NULL)
	  {
	     kstring_ncopy(filenames, (size_t) (tmp - filenames), filename);
	     tmp++;
	  }
	  else kstring_copy(filenames, filename);


	   /* KHOROS_TOOLBOX variable cannot be opened */
	   filenames = tmp;
	   if ((file = kfinput(filename)) != NULL)
	   {
	      list = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
				 KLITERAL, "\n", NULL, NULL, &num, &status);
	      kfclose(file);
	   }
	   else
	   {
	      num = 0;
	      list = NULL;
	   }

	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) == 1 &&
	          kstrcasecmp(tbname, tbvar) == 0)
	      {
	         written = TRUE;
	         ksprintf(temp, "%s:%s\n", tbvar, tbpath);
		 kfree(list[i]);
		 list[i] = kstrdup(temp);
	      }
	   }

	   /*--------------------------------------------------------------
	    * Only want to change the file if it's the first file and the
	    * toolbox entry is not in it, or if the entry is in the
	    *--------------------------------------------------------------*/
	   if (first || written)
	   {
	      /* KHOROS_TOOLBOX variable cannot be opened */
	      if ((file = kfoutput(filename)) == NULL)
	      {
	         if (!written)
	            kwarn(KUTILS, routine, "Unable to open toolbox file '%s' "
			  "for writing, so could not add reference for toolbox"
			  " `%s'.", filename, tbvar);
	         continue;
	      }
	   
	      for (i = 0; i < num; i++)
	         kfprintf(file, "%s\n", list[i]);
	      if (!written)
	      {
	  	 first = FALSE;
	         kfprintf(file, "%s:%s\n", tbvar, tbpath);
	      }
	      kfclose(file);
	   }
	   karray_free(list, KSTRING, (size_t) num, NULL);
	}

	if (!first)
	{
	   ksprintf(temp, "%s=%s", tbvar, tbpath);
	   kputenv(temp);
	   toolboxes = karray_insert(toolboxes, KSTRING, kstrdup(tbvar),
				numtb++, KLIST_TAIL, TRUE);
	}
	return(!first);
}

/*-----------------------------------------------------------
|
|  Routine Name: ktoolbox_delete - delete a toolbox entry from the list
|
|       Purpose: This routine removes a toolbox entry from the
|                global toolbox list.  The toolbox name is taken as
|                an input parameter.
|         Input: variable - the name of the toolbox to delete
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jul 03, 1992
|
------------------------------------------------------------*/
int ktoolbox_delete(
   const char *variable)
{
	kfile   *file;
	char    *filenames, *tmp, **list;
	ssize_t  i, num;
        int     status, mentioned;
	kstring routine = "ktoolbox_delete()";
	char    tbname[KLENGTH], tbvar[KLENGTH], filename[KLENGTH],
		paths[KLENGTH];


	if ((filenames = ktoolbox_filepaths(paths)) == NULL)
	   return(FALSE);

	kstring_cleanup(variable, tbvar);
	kstring_upper(tbvar, tbvar);
	while (filenames != NULL)
	{
	   if ((tmp = kstrchr(filenames, ':')) != NULL)
	   {
	      kstring_ncopy(filenames, (size_t) (tmp - filenames), filename);
	      tmp++;
	   }
	   else kstring_copy(filenames, filename);

	   /* KHOROS_TOOLBOX variable cannot be opened */
	   filenames = tmp;
	   if ((file = kfinput(filename)) == NULL)
	   {
	      kerror(KUTILS, routine, "Unable to open toolbox file '%s' "
				      "for reading.", filename);
	      continue;
	   }

	   list = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
			KLITERAL, "\n", NULL, NULL, &num, &status);
	   if (list == NULL)
	   {
	      kerror(KUTILS,routine,"Cannot read contents of toolbox file '%s'",
		     filename);
	      continue;
	   }
	   kfclose(file);

	   /*-- is the toolbox mentioned? ----------------------------------*/
	   mentioned = FALSE;
	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) != 1 ||
	          kstrcasecmp(tbname, tbvar) == 0)
	      {
	         mentioned = TRUE;
	      }
	   }

	   if (!mentioned)
	      continue;

	   /* KHOROS_TOOLBOX variable cannot be opened */
	   if ((file = kfoutput(filename)) == NULL)
	   {
	      kwarn(KUTILS, routine, "Unable to write to toolbox file `%s' "
			    "-- reference to toolbox `%s' will remain.",
			    filename, tbvar);
	     continue;
	   }

	   for (i = 0; i < num; i++)
	   {
	      if (ksscanf(list[i], "%[^#:]:", tbname) != 1 ||
	          kstrcasecmp(tbname, tbvar) != 0)
	      {
	         kfprintf(file, "%s\n", list[i]);
	      }
	   }
	   kfclose(file);
	   karray_free(list, KSTRING, (size_t) num, NULL);
	}

	kremenv(tbvar);
	for (i = 0; i < numtb; i++)
	{
	   if (kstrcmp(toolboxes[i], tbvar) == 0)
	   {
	      toolboxes = karray_delete(toolboxes,KSTRING,toolboxes[i],numtb--);
	      break;
	   }
	}
	return(TRUE);
}
