#!/bin/sh

#
# Khoros: $Id: kbugreport.sh,v 1.7 1996/08/10 03:15:10 mirage Exp $
#

# Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.


# # ========================================================================
# #
# # define various unix utilities we're gonna use
# #
# # ========================================================================

# ------------------------------------------------------------------------
# set up some other standard commands.
# eventually we should do something like the above code for echo,
# check for each of them on the path.  Oh for perl!

echo ""
echo " Working - just a few seconds while I gather some info"
echo ""

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
 cat="cat"
grep="grep"
strings="strings"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}
KHOROSVER="`kversion -version`"
KHOROSNAME="`kversion -name`"
email="khoros-bugs@khoral.com"

clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi
 
# clean up behind us, for example if this script gets interrupted
# ------------------------------------------------------------------------
cleanup() {
   echo ""
   echo "I'll just tidy up any temp files which might be lying around..."
   echo "Ok, bye..."
   $rm $TMPDIR/kbugreport.$$
   $rm $TMPDIR/bugreport.$$
   $rm $TMPDIR/bugheader.$$
   $rm $TMPDIR/check1.$$
   echo bye
   echo ""
   exit 0
}

# if user biffs ctrl-C, call our cleanup() function
trap cleanup 2

# prompt the user, do they want to continue
# be polite if they don't
# ------------------------------------------------------------------------
ConfirmContinue() {
   YesNoPrompt "Do you wish to continue?"
   if [ "$?" -eq "1" ]; then
      cleanup
   fi
}

# Simple continue prompt to allow the printing out of messages and before
# the next prompt and allowing the user to read the message.
# ------------------------------------------------------------------------
ContinuePrompt() {
   echon [Press \<Return\> to continue]
   read resp
}

# ------------------------------------------------------------------------
# let's see if we can use echo -n for prompting ...

# build a $echoN variable and $echoEnd combination that will give
# will allow the echon procedure to work well.
FindEchoN()
{
   # build a list of potential echos, starting with sh built-in
   echoEnd=""
   elist="echo"
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/echo" ] && elist="$elist $dir/echo"
   done
 
   # work through list, see if one of them groks -n
   for echo in $elist; do
      result=`$echo -n hello`
      if [ "$result" != "-n hello" ]; then
         echoN="$echo -n"
         break
      fi
   done
   if [ "$echoN" = "" ]; then
      echoN=echo
      echoEnd="\c"
      result=`$echoN "testing"$echoEnd`
      if [ "$result" != "testing" ]; then
         unset echoEnd
      fi
   fi
}

# echo without without return
echon()
{
   $echoN "$* $echoEnd"
}

# this is called every time we change `topic'
newtopic() {
   $clr
}

# prompt the user, to see if we should go on
YesNoPrompt()
{
   if [ "$#" -eq 0 ]; then
           echon "Do you wish to continue? (y/n) [y]:"
   else
      if [ "$#" -eq 2 ]; then
              echon "$1 (y/n) [$2]:"
      else
              echon "$1 (y/n) [y]:"
      fi
   fi
   read resp
   if [ "$#" -eq 2 -a "$resp" = "" ]; then
      resp=$2
   fi
   if [ "$resp" = "n" -o "$resp" = "N" ]; then
           return 1
   fi
   return 0
}
 
# Try to figure out the correct editor to use
FindEditor()
{
        if [ "$EDITOR" != "" ]; then
                return
        fi
 
        dlist=`echo $PATH | $sed -e 's/:/ /g'`
 
        for dir in $dlist; do
                if [ -x "$dir/vi" ]; then
                        EDITOR="$dir/vi"
                        return
                fi
        done
 
        for dir in $dlist; do
                if [ -x "$dir/emacs" ]; then
                        EDITOR="$dir/emacs"
                        return
                fi
        done
        for dir in $dlist; do
                if [ -x "$dir/pico" ]; then
                        EDITOR="$dir/pico"
                        return
                fi
        done
        EDITOR="none"
}
 
ReportBug()
{
        $clr
        FindEditor
        val=1
        until [ "$val" -eq 0 ]; do
                if [ "$EDITOR" != "none" ]; then
                        $EDITOR $TMPDIR/bugreport.$$
                else
                        echo "Cannot find an editor to call, Please edit the fil
e 'bugreport_info' in"
                        echo "the current directory and email it to 'khoros-bugs@khoral.com'"
                        $cp $TMPDIR/bugheader.$$ ./bugreport_info
                        cleanup
                fi
 
                $clr
                YesNoPrompt "Are you done editing the Bug Report file?"
                val=$?
        done
}
 
InitiateReport()
{
  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \
  > $TMPDIR/bugheader.$$
  echo "" >> $TMPDIR/bugheader.$$
  echo "" >> $TMPDIR/bugheader.$$
}

FindMailer()
{
        mailer="/usr/ucb/mail"
        if [ -x "$mailer" ]; then
                return
        fi
 
        dlist=`echo $PATH | $sed -e 's/:/ /g'`
 
        for dir in $dlist; do
                if [ -x "$dir/mailx" ]; then
                        mailer="$dir/mailx"
                        return
                fi
        done
 
        for dir in $dlist; do
                if [ -x "$dir/mail" ]; then
                        mailer="$dir/mail"
                        return
                fi
        done
        mailer="none"
}
 
MailIt()
{
        FindMailer
        echo ""
        echo "About to mail the information to '$email'"
        YesNoPrompt "Ready to send it? "
        if [ "$?" -eq 0 ] ; then
                if [ "$mailer" != "none" ]; then
                        $mailer $email <$TMPDIR/kbugreport.$$ >/dev/null 2>&1
                else
                        $cp $TMPDIR/kbugreport.$$ ./kbugreport_info
                        echo ""
                        echo "I cannot find a mailer program that I understand, please"
                        echo "send the file 'kbugreport_info' in the current directory"
                        echo "to 'khoros-bugs@khoral.com'."
                fi
        else
                $cp $TMPDIR/kbugreport.$$ ./kbugreport_info
                echo ""
                echo "Ok exiting kbugreport, but I have left a file in the current"
                echo "directory called kbugreport_info with the information you typed"
                echo "into the bug report.  When you have finalized the bugreport"
                echo "please send the file to 'khoros-bugs@khoral.com' or FAX"
                echo "it to 505-881-3842. Thank You for reporting this bug"
        fi
}

FindEchoN
 
PATH=./bin:$PATH:/usr/local/bin:/usr/lang:/usr/bsd:/usr/ucb:/usr/bin:/bin:/usr/etc:/etc:/usr/local/gnu/bin:/usr/bin/X11
export PATH
 
if [ "$KHOROS_TOOLBOX" = "" ]; then
   $cat <<"EndError"
        The environment variable KHOROS_TOOLBOX has not been set.
        Please use the 'kconfigure' program to correctly set up
        your environment to use Khoros, and rerun kregister.
 
EndError
fi
tblist=""
for tbfile in `echo $KHOROS_TOOLBOX | $sed -e 's/:/ /g'`; do
   [ -f "$tbfile" ] && tblist="$tblist $tbfile"
done

for tbfile in $tblist; do
   bootdir=`$grep "^BOOTSTRAP" $tbfile | $awk -F\; '{ print $1 }' | \
      $awk -F: '{ print $NF }'` 
    if [ "$bootdir" != "" ]; then
      break
    fi
done
 
        if [ "$bootdir" = "" ]; then
                echo ""
                echo "Cannot find the top directory for the BOOTSTRAP Toolbox"
                echo "in the file(s) listed in your KHOROS_TOOLBOX"
                echo "variable.  Please set this variable correctly, or"
                echo "use 'kconfigure' to correctly set up your environment"
                cleanup
        fi
newtopic
 
  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndIntro"
 
   This script is for reporting bugs you have found in the Khoros System.
   You will be prompted for required information first, and then will be
   able to enter your bug description and a workaround if you have devel-
   oped one. You will then be able to E-mail it to Khoral Research or save
   it to a file to be faxed or dealt with later. 

   Remember to be descriptive as possible including all steps that lead to
   the bug. Include any details that would make your Khoros installation
   different from what might be considered standard.

   Khoral Research appreciates your participation in making a better
   Khoros for the future. All bugs are evaluated and recorded into our
   Bug Tracking system.

   This form is also available at our Web Site. The URL for the form is:

   http://www.khoral.com/forms/khoros-bugs.html

EndIntro

ConfirmContinue

if [ "$?" -eq 1 ]; then
   echo "Ok, bye..."
   echo ""
   cleanup
fi

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndExplan"

    You will be prompted for several answers now. We have done our best
    to calculate the correct default responses (they appear in the [ ]
    brackets). If they appear to be correct just hit return, if not, please
    enter the appropriate information. Hints have been included to assist
    getting the correct information. 

EndExplan

sysinfo=`$bootdir/objects/script/kbuild/src/config.guess`
echon "Your Name: "
read name
echo "Name: $name" >> $TMPDIR/bugheader.$$
echo ""
echon "Your E-mail address: "
read umail
echo "E-mail: $umail" >> $TMPDIR/bugheader.$$
echo ""
kdate=`date | awk '{printf "%s-%s-%s\n",$3,$2,$NF}'`
echon "Today's date in this format [$kdate]: "
read udate
if ["$udate" = ""]; then
  udate=$kdate
fi
echo "Date: $udate" >> $TMPDIR/bugheader.$$
echo ""
echon "The version of Khoros (Hint: use kversion) [$KHOROSVER]: "
read ver
if ["$ver" = ""]; then
  ver=$KHOROSVER
fi
echo "Khoros Version: $ver">> $TMPDIR/bugheader.$$
echo ""
hware=`echo $sysinfo | awk -F- '{print $1}'`
echon "Hardware the bug occured on (Hint: use uname -m) [$hware]: "
read hw
if ["$hw" = ""]; then
  hw=$hware
fi
echo "Hardware: $hw" >> $TMPDIR/bugheader.$$
echo ""
osware=`echo $sysinfo | awk -F- '{print $3}'`
echon "Operating System and version (Hint: use uname -r) [$osware]: "
read os
if ["$os" = ""]; then
  os=$osware
fi
echo "Operating System: $os" >> $TMPDIR/bugheader.$$
echo ""
echon "C compiler used: "
read compiler
echo "C Compiler: $compiler" >> $TMPDIR/bugheader.$$
echo ""
echon "Widget set used in compiling Khoros (Motif or Athena): "
read widget
echo "Widget Set: $widget" >> $TMPDIR/bugheader.$$

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndNR"

   The following items are not required but would be appreciated. Even if
   the item does not affect this bug, we can use the data to understand
   what Khoros users are developing with and plan our future releases with
   this data in mind. Thank You!

EndNR
echo ""
echon "Fortran compiler used: "
read fortran
echo "Fortran Compiler: $fortran" >> $TMPDIR/bugheader.$$
echo ""
echon "Version of Perl used (calculated with perl -v): "
read uperl
echo "Perl Version: $uperl" >> $TMPDIR/bugheader.$$
echo ""
echon "Version of X and patches: "
read xver
echo "X Version: $xver" >> $TMPDIR/bugheader.$$
echo ""
echon "Type of display (8 or 24 bit and any other neccessary info): "
read dis
echo "Display Info: $dis" >> $TMPDIR/bugheader.$$
echo "" >> $TMPDIR/bugheader.$$

newtopic

  echo "             $KHOROSNAME $KHOROSVER Bug Reporting Form" \

$cat <<"EndEdit"
 
    Now you will enter the description of the bug and any suggestions or
    workarounds that you have discovered. After typing your report, end your
    editing session and you will be shown the results of your Bug Report.
    You then can have it mailed to us or save it to a file for more editing
    or to FAX to us at 505-881-3842. Thank You!

EndEdit

echo "######################################################################" \
   >> $TMPDIR/bugheader.$$
echo "                      Bug Report" >> $TMPDIR/bugheader.$$
echo "" >> $TMPDIR/bugheader.$$
ConfirmContinue

GETREPORT=1
while [ $GETREPORT = 1 ]; do
   ReportBug
   if [ ! -f $TMPDIR/bugreport.$$ -o -z $TMPDIR/bugreport.$$ ]; then
      echo ""
      echo " You did not enter the description of the bug."
      ConfirmContinue
   else
      GETREPORT=0
   fi
done

$cat $TMPDIR/bugheader.$$ $TMPDIR/bugreport.$$ > $TMPDIR/kbugreport.$$
$cat $TMPDIR/kbugreport.$$ | $KPAGER
MailIt

