#!/bin/sh

#
#  Khoros: $Id: kconfigure.sh,v 1.10 1996/08/13 21:53:17 fas Exp $
#

# Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
# All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.



#************************************************************
#  Routine Name: kconfigure
#       Purpose: 
#         Input: 
#    Written By: Neil Bowers & Tom Sauer
#          Date: July 22, 1993
# Modifications: Lots for Khoros 2.1 by JMAS 
#************************************************************/

  tr="tr"
  cp="cp"
  mv="mv -f"
  rm="rm -f"
 awk="awk"
 sed="sed"
 cat="cat"
grep="grep"
KPAGER=${PAGER:-more}
TMPDIR=${TMPDIR:-/tmp}

clr=""
for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
    [ -x "$dir/clear" ] && clr="$dir/clear"
done

#
#  Assume a vt100 variant
#
if [ "$clr" = "" ]; then
  clr="echo -c [H[J"
fi

# clean up behind us, for example if this script gets interrupted
# ------------------------------------------------------------------------
cleanup() {
   echo ""
   echo "I'll just tidy up any temp files which might be lying around..."
   echo "Ok, bye..."

   $rm $HOME/.Toolboxes.$$ $HOME/.line.$$ $HOME/.khoros_env.$$
   exit 0
}

# if user biffs ctrl-C, call our cleanup() function
trap cleanup 2

# prompt the user, do they want to continue
# be polite if they don't
# ------------------------------------------------------------------------
ConfirmContinue() {
   echon "Do you wish to continue? (y/n) [y]: "
   read resp
   if [ "$resp" = "n" -o "$resp" = "N" ]; then
      cleanup
   fi
}

# Simple continue prompt to allow the printing out of messages and before
# the next prompt and allowing the user to read the message.
# ------------------------------------------------------------------------
ContinuePrompt() {
   echon [Press \<Return\> to continue]
   read resp
}

# build a $echoN variable and $echoEnd combination that will give
# will allow the echon procedure to work well.
FindEchoN()
{
   # build a list of potential echos, starting with sh built-in
   echoEnd=""
   elist="echo"
   for dir in `echo $PATH | $sed -e 's/:/ /g'`; do
      [ -x "$dir/echo" ] && elist="$elist $dir/echo"
   done
 
   # work through list, see if one of them groks -n
   for echo in $elist; do
      result=`$echo -n hello`
      if [ "$result" != "-n hello" ]; then
         echoN="$echo -n"
         break
      fi
   done
   if [ "$echoN" = "" ]; then
      echoN=echo
      echoEnd="\c"
      result=`$echoN "testing"$echoEnd`
      if [ "$result" != "testing" ]; then
         unset echoEnd
      fi
   fi
}

# this is called every time we change `topic'
newtopic() {
   $clr
}

# echo without without return
echon()
{
   $echoN "$* $echoEnd"
}

#
# Create and check the $HOME/.Toolboxes file
#
CreateDotToolboxes()
{
   newtopic

#
# Create $HOME/.Toolboxes if it does not exist.
#
   if [ ! -f $HOME/.Toolboxes ]; then
      echo ""
      echo "Creating an empty .Toolboxes file in your home directory."
      echo ""
      echo "# Personal Khoros 2 Toolboxes Will be Listed Below" > $HOME/.Toolboxes
   else
#
#  Check for and remove duplicate entries if a $HOME/.Toolboxes file does exist
#
      echo ""
      echo "We want to check your \$HOME/.Toolboxes for entries contained in the"
      echo "Khoros 2 Toolboxes file.  We recommend that they be deleted since"
      echo "the ones in your \$HOME/.Toolboxes will override the ones in the"
      echo "Khoros 2 Toolboxes file.  This is most likely not what you want to"
      echo "do, but then again maybe it is."
      echo ""
      echon "Do you want to delete these entries? (y/n) [y]: "
      read ans

      if [ "$ans" != "n" -a "$ans" != "N" ]; then
         if [ -f $TOPDIR/Toolboxes ]; then
	    echo "Checking for:"
	    for TB in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }'`; do
	       echo "     $TB"
	       $grep -v $TB $HOME/.Toolboxes > $HOME/.Toolboxes.$$
	       $mv $HOME/.Toolboxes.$$ $HOME/.Toolboxes
	    done
	    echo "done."
          fi
      fi
   fi

   echo ""
   echo "Your Khoros Toolbox file has been updated for Khoros 2"
   echo ""
   ContinuePrompt
}

ListPossibleKarchs() {

   if [ -d $TOPDIR/$toolbox/config -o -d $TOPDIR/bootstrap/config ]; then
      if [ -d $TOPDIR/$toolbox/config ]; then
         temptb="$toolbox"
      else
         temptb="bootstrap"
      fi

      havedef=0
      havesettings=0
      settings=""
      for dirsetting in `ls $TOPDIR/$temptb/config/Dir.*`; do
         case "$dirsetting" in
            *.def) 
               havedef=1
               ;;
            *)
               settings="$settings $dirsetting"
               havesettings=1
               ;;
         esac
      done

      if [ $havesettings = 1 ]; then
         echo ""
         echo "Here are some possible setting for KARCH I found in"
         echo "$TOPDIR/$temptb/config."
         echo "Talk to the person that installed Khoros to find out which is"
         echo "correct."
         echo ""
         ContinuePrompt

         for dirsetting in $settings; do
            echo `echo $dirsetting | $awk -F/ '{print $NF}' | $sed -e s/Dir.//`
         done
      fi
   fi

}
#
# Get the global library directory:
#	1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#	   do,
#	   a. check for TOOLBOX/config/Dir.KARCH-KOPSYS-KWIDSET
#	   b. check for TOOLBOX/config/Dir.def
#	   c. check for bootstrap/config/Dir.KARCH-KOPSYS-KWIDSET
#	   d. check for bootstrap/config/Dir.def
#	   e. default to bootstrap/lib
#	2. use TOOLBOX/lib
#
GetLibDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`

#
# Determine the name of the file that specifies the LibDir
#
   if [ $nokarch = 1 ]; then
      dirfile=Dir.def
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $2 }'`

      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the library directory in the variable checkdir
#
   checkdir=""
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`

   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $TOPDIR/$toolbox/config ]; then
         if [ -f $TOPDIR/$toolbox/config/$dirfile ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/$toolbox/config/$dirfile | $awk '{ print $2 }'`
            nodirfile=0
         elif [ -f $TOPDIR/$toolbox/config/Dir.def ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/$toolbox/config/Dir.def | $awk '{ print $2 }'`
            nodirfile=1
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The library directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file,"
	       echo ""
	       if [ $nodirfile = 1 ]; then
	          echo "    $TOPDIR/$toolbox/config/Dir.def."
	       else
	          echo "    $TOPDIR/$toolbox/config/$dirfile."
	       fi
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

#
# Check the bootstrap toolbox
#
      if [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
            nodirfile=0
         elif [ -f $TOPDIR/bootstrap/config/Dir.def ]; then
            checkdir=`$grep "^LibDir" $TOPDIR/bootstrap/config/Dir.def | $awk '{ print $2 }'`
            nodirfile=1
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global library directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file,"
	       echo ""
	       if [ $nodirfile = 1 ]; then
	          echo "    $TOPDIR/bootstrap/config/Dir.def."
	       else
	          echo "    $TOPDIR/bootstrap/config/$dirfile."
	       fi
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "   4. Create a $dirfile or Dir.def file in the directory,"
	       echo "         $TOPDIR/$toolbox/config"
               echo "      to point to the correct location"
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/lib
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The library directory,"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. Khoros 2 not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$TOPDIR/$toolbox/lib

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The library directory,"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file,"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo "The library directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to"
      echo "      false, off, no or 0."
      if [ "$dirfile" = Dir.def ]; then
         echo "   2. The Dir.def was deleted from the"
      else
         echo "   2. The Dir.def or $dirfile was deleted from the"
      fi
      echo "      bootstrap/config directory and/or the TOOLBOX/config directory."
      echo "   3. Your current setting for KARCH is wrong or is not set."
      echo ""
      echo "Please fix and re-run this script."

      ListPossibleKarchs
      cleanup
   fi
}

#
# Get the global binary directory:
#       1. If the globaldir in the Toolboxes file is true, yes, on or 1 any case
#          do,
#          a. check for TOOLBOX/config/Dir.KARCH-KOPSYS-KWIDSET
#          b. check for TOOLBOX/config/Dir.def
#          c. check for bootstrap/config/Dir.KARCH-KOPSYS-KWIDSET
#          d. check for bootstrap/config/Dir.def
#          e. default to bootstrap/bin
#       2. use TOOLBOX/bin
#
GetBinDir()
{
   toolbox="$*"
   toolboxtocheck=`echo $toolbox | $tr "[a-z]" "[A-Z]"`

#
# Determine the name of the file that specifies the BinDir
#
   if [ $nokarch = 1 ]; then
      dirfile=Dir.def
   else
      dirfile=Dir.$KARCH
      if [ "$KOPSYS" != "" ]; then
         dirfile=Dir.$KARCH-$KOPSYS
         if [ "$KWIDSET" != "" ]; then
            dirfile=Dir.$KARCH-$KOPSYS-$KWIDSET
         fi
      fi
   fi
#
# Parse out the setting for globaldir from the $TOPDIR/Toolboxes file
#
   if [ -f $TOPDIR/Toolboxes ]; then
      ans=`$grep "^$toolboxtocheck" $TOPDIR/Toolboxes | $awk -F\; '{ print $2 }'`

      for entry in `echo $ans | $sed s/,/\ /g`; do

         case $entry in

            globaldir*) 
                useglobaldir=`echo $entry | $awk -F= '{ print $2 }'`
                break
                ;;
            *)
                useglobaldir=false
                ;;

         esac 
      done
   else
      echo ""
      echo "No $TOPDIR/Toolboxes to use."
      echo "This should not have happened, please mail khoros-bugs@khoral.com."
      echo ""
      cleanup
   fi

#
# Set the binary directory in the variable checkdir
#
   checkdir=""
   useglobaldir=`echo $useglobaldir | $tr "[A-Z]" "[a-z]"`

   if [ "$useglobaldir" = "true" -o "$useglobaldir" = "yes" -o "$useglobaldir" = "on" -o "$useglobaldir" = "1" ]; then

      if [ -d $TOPDIR/$toolbox/config ]; then
         if [ -f $TOPDIR/$toolbox/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/$toolbox/config/$dirfile | $awk '{ print $2 }'`
            nodirfile=0
         elif [ -f $TOPDIR/$toolbox/config/Dir.def ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/$toolbox/config/Dir.def | $awk '{ print $2 }'`
            nodirfile=1
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The binary directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "for $toolbox does not exist as specified in the file,"
	       echo ""
	       if [ $nodirfile = 1 ]; then
	          echo "    $TOPDIR/$toolbox/config/Dir.def."
	       else
	          echo "    $TOPDIR/$toolbox/config/$dirfile."
	       fi
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

#
# Check the bootstrap toolbox
#
      if [ -d $TOPDIR/bootstrap/config ]; then
         if [ -f $TOPDIR/bootstrap/config/$dirfile ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/bootstrap/config/$dirfile | $awk ' { print $2 }'`
            nodirfile=0
         elif [ -f $TOPDIR/bootstrap/config/Dir.def ]; then
            checkdir=`$grep "^BinDir" $TOPDIR/bootstrap/config/Dir.def | $awk '{ print $2 }'`
            nodirfile=1
         fi

	 if [ "$checkdir" != "" ]; then
            if [ -d $checkdir ]; then
               return 0
	    else
	       echo ""
	       echo "The global binary directory,"
	       echo ""
               echo "    $checkdir"
	       echo ""
               echo "does not exist as specified in the file,"
	       echo ""
	       if [ $nodirfile = 1 ]; then
	          echo "    $TOPDIR/bootstrap/config/Dir.def."
	       else
	          echo "    $TOPDIR/bootstrap/config/$dirfile."
	       fi
	       echo ""
	       echo "These could be the problems:"
	       echo "   1. The file entry is wrong and needs updating"
	       echo "   2. The file should not exist for this toolbox"
	       echo "   3. Your current setting for KARCH is wrong or is not"
	       echo "      set."
	       echo "   4. Create a $dirfile or Dir.def file in the directory,"
	       echo "         $TOPDIR/$toolbox/config"
               echo "      to point to the correct location"
	       echo "Please fix and re-run this script."
               ListPossibleKarchs
	       cleanup
            fi
         fi
      fi

      if [ "$checkdir" != "" ]; then
         checkdir=$TOPDIR/bootstrap/bin
         if [ -d $checkdir ]; then
	    return 0
         else
	    echo ""
	    echo "The binary directory,"
	    echo ""
            echo "    $checkdir"
	    echo ""
            echo "does not exist.  These could be the problems:"
	    echo "   1. The directory was deleted accidently."
	    echo "   2. Khoros 2 not really installed on your system."
	    echo ""
	    echo "Please fix and re-run this script."
	    cleanup
         fi
      fi
   else
      checkdir=$TOPDIR/$toolbox/bin

      if [ -d $checkdir ]; then
	 return 0
      else
	 echo ""
	 echo "The binary directory,"
	 echo ""
         echo "    $checkdir"
	 echo ""
         echo "for $toolbox does not exist.  These could be the problems:"
	 echo "   1. The directory was deleted accidently."
	 echo "   2. The toolbox is not really installed on the system"
         echo "      and the entry should be deleted from the file,"
	 echo ""
	 echo "    $TOPDIR/Toolboxes"
	 echo ""
	 echo "Please fix and re-run this script."
	 cleanup
      fi
   fi

   if [ "$checkdir" = "" ]; then
      echo "The binary directory for $toolbox is undefined.  This should not"
      echo "have happened.  These could be the problems:"
      echo "   1. In the $TOPDIR/Toolboxes file"
      echo "      globaldir is set to true, on, yes or 1 and should be set to."
      echo "      false, off, no or 0."
      if [ "$dirfile" = Dir.def ]; then
         echo "   2. The Dir.def was deleted from the"
      else
         echo "   2. The Dir.def or $dirfile was deleted from the"
      fi
      echo "      bootstrap/config directory and/or the TOOLBOX/config directory."
      echo "   3. Your current setting for KARCH is wrong or is not set."
      echo ""
      echo "Please fix and re-run this script."
      ListPossibleKarchs
      cleanup
   fi
}

UpdateLibList()
{
#
# Initialize the list of library directories
#
   if [ "$liblist" = "" ]; then
      liblist=$*
   fi

#
# Search to make sure the current library directory is not already in the list.
#
   doaddtoliblist=0

   for libdirectory in `echo $liblist`; do
      case $libdirectory in
         ${*}*)
		doaddtoliblist=1
	        ;;
             *)
	        ;;
      esac
   done

#
# If not already in the list add it
#
   if [ $doaddtoliblist = 0 ]; then
      liblist=`echo $liblist $*`
   fi

#    echo "$tb libaries located in $*"
}

UpdateBinList()
{
#
# Initialize the list of binary directories
#
   if [ "$binlist" = "" ]; then
      binlist=$*
   fi

#
# Search to make sure the current binary directory is not already in the list.
#
   doaddtobinlist=0

   for bindirectory in `echo $binlist`; do
      case $bindirectory in
         ${*}*)
		doaddtobinlist=1
	        ;;
             *)
	        ;;
      esac
   done

#
# If not already in the list add it
#
   if [ $doaddtobinlist = 0 ]; then
      binlist=`echo $binlist $*`
   fi

   echo "$tb binaries located in $*"
}

#
# Find out the shell the user is using.
#
DetermineShell()
{

   CSHVARIENT=0
   SHVARIENT=0
   DOTFILE=""

#
# See what the SHELL variable is set to
#
   case `echo $SHELL | $awk -F/ '{ print $NF }'` in

       csh) usershell=csh
	    CSHVARIENT=1
	    if [ -f $HOME/.cshrc ]; then
	       DOTFILE=.cshrc
            fi
            ;;
      tcsh) usershell=tcsh
	    CSHVARIENT=1
	    if [ -f $HOME/.tcshrc ]; then
	       DOTFILE=.tcshrc
            else
	       if [ -f $HOME/.cshrc ]; then
	          DOTFILE=.cshrc
               fi
            fi
            ;;
      bash) usershell=bash
	    SHVARIENT=1
	    dotfile=.bashrc
	    if [ -f $HOME/.bash_profile ]; then
	       DOTFILE=.bash_profile
            else
	       if [ -f $HOME/.bash_login ]; then
	          DOTFILE=.bash_login
               else
	          if [ -f $HOME/.profile ]; then
	                DOTFILE=.profile
                  fi
               fi
            fi
            ;;
       zsh) usershell=zsh
	    SHVARIENT=1
	    if [ -f $HOME/.zshrc ]; then
	       DOTFILE=.zshrc
            else
	       if [ -f $HOME/.zprofile ]; then
	          DOTFILE=.zprofile
               else
	          if [ -f $HOME/.zlogin ]; then
	             DOTFILE=.zlogin
                  fi
               fi
            fi
            ;;
       ksh) usershell=ksh
	    SHVARIENT=1
	    DOTFILE=.profile
            ;;
        sh) usershell=sh
	    SHVARIENT=1
	    DOTFILE=.profile
            ;;
#
# If not a known shell ask the user what type of shell variant it is and what
# dot file it uses so that it can be modified.
#
         *) echo "Only understand the following shells:"
	    echo ""
	    echo "         csh, tcsh, bash, zsh, ksh and sh"
	    echo ""

            echon "Do not know this shell $SHELL is it a csh variant? (y/n) [y]: "
            read bot

            if [ "$bot" = "n" -o "$bot" = "N" ]; then
               echon "Do not know this shell $SHELL is it a sh variant? (y/n) [y]: "
               read bot
               if [ "$bot" = "n" -o "$bot" = "N" ]; then
		  echo ""
                  echo "Ok, I stumpt.  You will need to set up under this shell,"
                  echo "the environment variable KHOROS_TOOLBOX to point to the"
		  echo "file in {top level directory of Khoros 2}/Toolboxes and"
		  echo "in your execution path the directories where the Khoros"
		  echo "binaries are located."
		  echo ""
		  echo "All we can do is create a $HOME/.Toolboxes."
		  echo ""
		  CreateDotToolboxes
		  cleanup
	       else
                  needdotfile=1
	          while [ $needdotfile = 1 ]; do
                      echo "Name of dotfile to used by $SHELL to set up"
                      echon "environment? [no default]: "
                      read bot

                      if [ "$bot" != "" ]; then
		         echo ""
		         echo "Will use $bot as the dotfile to modify."
		         echo ""
                         needdotfile=0
		      else
		         echo ""
		         echo "Must enter some file, no default, please re-enter."
		         echo ""
                      fi
                  done
		  DOTFILE=$bot
		  SHVARIENT=1
               fi
            else
               needdotfile=1
	       while [ $needdotfile = 1 ]; do
                   echo "Name of dotfile to used by $SHELL to set up"
                   echon "environment? [no default]: "
                   read bot

                   if [ "$bot" != "" ]; then
		      echo ""
		      echo "Will use $bot as the dotfile to modify."
		      echo ""
                      needdotfile=0
		   else
		      echo ""
		      echo "Must enter some file, no default, please re-enter."
		      echo ""
                   fi
               done
	       DOTFILE=$bot
               CSHVARIENT=1
            fi
            ;;
   esac
}

#
# Find out if we need to define a LD_LIBRARY_PATH
#
CheckForLoadLibrary()
{
   os=`uname`
   release=`uname -r`
   if [ $os = "SunOS" -o $os = "IRIX" -o $os = "OSF1" ]; then
      needldpath=0
   else
      needldpath=1
   fi

   for tb in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }' | $tr "[A-Z]" "[a-z]"`; do
      GetLibDir $tb
      UpdateLibList $checkdir
   done
   ldlist=`echo $liblist | $sed -e 's/ /:/g'`
}

#
# In case dirname not on system
#
dirname () {
   expr ${1-.}'/' : '\(/\)[^/]*//*$' \
        \| ${1-.}'/' : '\(.*[^/]\)//*[^/][^/]*//*$' \
        \| .
}

FindTopDir() {

   command=$1

   case "$command" in
      /*)
         foundtopdir=`dirname $command`
         foundtopdir=`dirname $foundtopdir`
         ;;
      ../*)
         foundtopdir=`pwd`/`dirname $command`
         foundtopdir=`(cd ${cdmountpoint} ; pwd )`
         foundtopdir=`dirname $foundtopdir`
         ;;
      ./*)
         foundtopdir=`dirname $command`
         foundtopdir=`pwd`/` expr ${cdmountpoint} : '\.\/\(.*\)' `
         foundtopdir=`(cd ${cdmountpoint} ; pwd )`
         foundtopdir=`dirname $foundtopdir`
         ;;
      *)
         foundtopdir="no default"
         ;;
   esac

   if [ ! -f "$foundtopdir"/Toolboxes ]; then
      foundtopdir=`dirname $foundtopdir`
      if [ ! -f "$foundtopdir"/Toolboxes ]; then
         foundtopdir="no default"
      fi
   fi
}

#
# Start of Script
#
if [ "$#" = 1 ]; then
   TOPDIR=$1
else
   FindTopDir $0
   TOPDIR=$foundtopdir
fi

DetermineShell

if [ "$KARCH" = "" ]; then
   nokarch=1
else
   nokarch=0
fi

FindEchoN
newtopic

$cat <<EofIntro
 
                                   Khoros 2 
                             User Configuration Script

        This script is used to configure a user's account for the Khoros 2
        software system. This involves the following steps:
                1. Setup and check the KHOROS_TOOLBOX environment variable
                2. Creating a personal Khoros Toolbox file in \$HOME/.Toolboxes
                3. Set up the Khoros environment variables in the appropriate
                   dotfile for your user shell so you can execute Khoros
                   binaries.

        (If you experience any problems with this script, please report
        them to khoros-bugs@khoral.com)

EofIntro

ConfirmContinue

newtopic

ans="n"
echo ""
echo "			Location of Khoros 2 Software"
echo ""
while [ "$ans" = "n" ]; do
   echo ""
   echo "Please enter the full path to the toplevel of Khoros 2"
   echon "[$TOPDIR]: "
   read temp
   if [ "$temp" = "" ]; then
      temp=$TOPDIR
   fi

   if [ -d "$temp/bootstrap" ]; then
      TOPDIR=$temp
      ans="y"
   elif [ "$temp" = "no default" ]; then
      echo ""
      echo there is no default, try again\!
      echo ""
   elif [ ! -d "$temp" ]; then
      echo ""
      echo not a valid directory $temp\!
      echo ""
   else
      echo ""
      echo bootstrap directory does not exist in $temp\!
      echo $temp can not be the toplevel of Khoros 2\!
      echo ""
   fi
done

newtopic
$cat << EofTbFile

				Toolbox File

	Ok, next you need to have a toolbox file.  This contains
	a list of the toolboxes you want to use.  You let Khoros
	know where this file lives using the KHOROS_TOOLBOX environment
	variable.

	Typically, your personal toolbox file will be called .Toolboxes, and
	will live in your home directory.  You will refer to the two
	distribution toolboxes by appending

			$TOPDIR/Toolboxes

	to the KHOROS_TOOLBOX environment variable.  If \$HOME/.Toolboxes file
	does not exist an empty one will be created for you.  There should be
	no duplicate entries in the toolbox files.

EofTbFile

ContinuePrompt

CreateDotToolboxes

newtopic

ans="n"
echo ""
echo "          Determining Location of Khoros Binaries"
echo ""

binlist=""

for tb in `$cat $TOPDIR/Toolboxes | $grep -v "^#" | $awk -F: '{ print $1 }' | $tr "[A-Z]" "[a-z]"`; do
   GetBinDir $tb
   UpdateBinList $checkdir
done

ContinuePrompt

# ========================================================================
# 
# Give the user a .khoros_env file, setup their environment
# This file will be sourced from their .cshrc
#
# ========================================================================

newtopic
echo ""
echo "		Setting up Khoros Environment Variables"
echo ""
echo "You need to ensure certain environment variables are set every"
echo "time you log in."

if [ $CSHVARIENT = 1 ]; then
   echo ""
   echo "Since, you have a csh variant shell, in this step we will give you a,"
   echo ""
   echo "               $HOME/.khoros_env"
   echo ""
   echo "file, which can be source'd from a specific dotfile.  We will set up the"
   echo ".khoros_env to reflect your local configuration."
   echo ""
else
   echo ""
   echo "Since, you have a sh variant shell, in this step we can add for you the"
   echo "the lines to the specific dotfile that will setup you Khoros environment."
   echo ""
fi

ContinuePrompt

CheckForLoadLibrary

if [ $CSHVARIENT = 1 ]; then
   $cat $TOPDIR/bootstrap/repos/dotfiles/khoros_env |  $sed -e s:KHOROSTMPDIR:$TMPDIR:g > $HOME/.khoros_env.tmp.$$
   $cat $HOME/.khoros_env.tmp.$$ |  $sed -e s:TOPDIR:$TOPDIR:g > $HOME/.khoros_env.$$
   $cat $HOME/.khoros_env.$$ | $sed -e s:BINDIR:"$binlist":g > $HOME/.khoros_env
   $rm $HOME/.khoros_env.tmp.$$ $HOME/.khoros_env.$$

   if [ $needldpath = 1 ]; then
$cat <<EofLdLibraryPath >> $HOME/.khoros_env
# Set platform-specific run-time library environment variables for Khoros
if ( \$?LD_LIBRARY_PATH ) then
   setenv LD_LIBRARY_PATH $ldlist:\${LD_LIBRARY_PATH}
else
   setenv LD_LIBRARY_PATH $ldlist
endif
EofLdLibraryPath
   else
$cat <<EofLdLibraryPathComm >> $HOME/.khoros_env
# Set platform-specific run-time library environment variables for Khoros
# if ( \$?LD_LIBRARY_PATH ) then
#    setenv LD_LIBRARY_PATH $ldlist:\${LD_LIBRARY_PATH}
# else
#    setenv LD_LIBRARY_PATH $ldlist
# endif
EofLdLibraryPathComm
   fi

   if [ $nokarch = 0 ]; then
$cat <<EofKarch >> $HOME/.khoros_env
setenv KARCH $KARCH
EofKarch
   fi

   echo "Your .khoros_env file now looks like:"
   echo ""
   $KPAGER $HOME/.khoros_env
   echo ""
   ContinuePrompt

   if [ "$DOTFILE" = "" ]; then
      echo ""
      echo "Can not find a file to add the line source \$HOME/.khoros_env.  To use"
      echo "Khoros you will need to source the $HOME/.khoros_env"
      echo "file by hand.  Or you can talk to you local system manager on which"
      echo "specific dotfile to add the line source \$HOME/.khoros_env."
      echo ""
   elif [ -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Do you want me to add the line source \$HOME/.khoros_env to the"
      echon "bottom of your $DOTFILE? (y/n) [y]: "
      read resp

      if [ "$resp" != "n" -a "$resp" != "N" ]; then

         donotadd=0
	 for entry in `$grep source $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env*)
		  echo ""
                  echo "Line already exists, not adding."
		  echo ""
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
$cat <<EofSource >> $HOME/$DOTFILE
# The following line was added automatically, by Khoros kconfigure script
source \$HOME/.khoros_env
EofSource
         fi

      else
         donotadd=0
	 for entry in `$grep source $HOME/$DOTFILE`; do
            case $entry in
               *HOME/.khoros_env*)
		  donotadd=1
                  break
		  ;;
               *)
                  ;;
            esac
         done

	 if [ $donotadd = 0 ]; then
            echo ""
            echo "This line does not exist in your $DOTFILE.  To use Khoros,"
            echo "you will need to source the $HOME/.khoros_env file by hand"
            echo "or added by hand to your $DOTFILE."
            echo ""
         fi
      fi
   elif [ ! -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Can not write to the file $HOME/$DOTFILE.  To use"
      echo "Khoros you will need to source the $HOME/.khoros_env"
      echo "file by hand or add it to one of your environment setup files"
      echo "by hand.  You can talk to you local system manager on how to"
      echo "do this."
      echo ""
   fi
else

   newtopic
   binlistsh=`echo $binlist | sed s/\ /:/g`
   echo "#"
   echo "# These are default values for environment"
   echo "# variables used by Khoros."
   echo "#"
   echo "KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes"
   echo "KHOROS_LOG=\$HOME/khoros.cmdlog"
   echo "KHOROS_MAIL=\$USER"
   echo "KHOROS_NAME=\$USER"
   echo "KHOROS_NOTIFY=STANDARD"
   echo "KHOROS_ANSFILE=\$HOME/khoros.ans"
   echo "TMPDIR=$TMPDIR"
   echo "export KHOROS_TOOLBOX KHOROS_LOG KHOROS_MAIL KHOROS_NAME KHOROS_NOTIFY KHOROS_ANSFILE TMPDIR"
   echo "#"
   echo "#   Uncomment this if you would like data objects to accumulate "
   echo "#   history when operated on by datamanip operators."
   echo "#"
   echo "#KHOROS_HISTORY=1"
   echo "#export KHOROS_HISTORY"
   echo "# New PATH setting to execute Khoros binaries"
   echo "PATH=\$PATH:$binlistsh"
   echo "export PATH"

   if [ $needldpath = 1 ]; then
      echo "# Set platform-specific run-time library environment variables for Khoros"
      echo "if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
      echo "   LD_LIBRARY_PATH=$ldlist"
      echo "else"
      echo "   LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
      echo "fi"
      echo "export LD_LIBRARY_PATH"
   else
      echo "# Set platform-specific run-time library environment variables for Khoros"
      echo "# if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
      echo "#    LD_LIBRARY_PATH=$ldlist"
      echo "# else"
      echo "#    LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
      echo "# fi"
      echo "# export LD_LIBRARY_PATH"
   fi

   if [ $nokarch = 0 ]; then
      echo "KARCH=$KARCH"
      echo "export KARCH"
   fi

   ContinuePrompt

   if [ "$DOTFILE" = "" ]; then
      echo ""
      echo "Can not find a file to add the above environment variables.  To use"
      echo "Khoros you will need to setup the above environment variables by hand"
      echo "Or you can talk to you local system manager on which specific dotfile"
      echo "to add the above environment variables."
      echo ""
   elif [ -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Do you want me to add these lines to the bottom of your"
      echon "$DOTFILE ? (y/n) [y]: "
      read resp

      if [ "$resp" != "n" -a "$resp" != "N" ]; then

         toolboxesdefined=0
         infile=0
	 for entry in `$grep -v "^#" $HOME/$DOTFILE | $grep -v export | $grep KHOROS_`; do
            case $entry in
               KHOROS_TOOLBOX*)
                  toolboxesdefined=1
		  infile=1
		  ;;
               KHOROS_LOG*)
		  infile=1
		  ;;
               KHOROS_MAIL*)
		  infile=1
		  ;;
	       KHOROS_NAME*)
		  infile=1
		  ;;
               KHOROS_NOTIFY*)
		  infile=1
		  ;;
               KHOROS_ANSFILE*)
		  infile=1
		  ;;
               *)
                  ;;
            esac
         done

         foundpath=0
	 for entry in `$grep PATH $HOME/$DOTFILE`; do
            case $entry in
	       *$binlistsh*)
                  foundpath=1
                  ;;
               *)
                  ;;
            esac
         done

         if [ $infile = 1 ]; then
            if [ $foundpath = 0 ]; then
               echo ""
               echo "The path for Khoros is not set correctly.  Adding a"
               echo "new PATH variable."
               echo ""
            fi

            if [ $toolboxesdefined = 0 ]; then
	       echo ""
               echo "Looks like some of the lines to add are in $DOTFILE."
               echo "However, KHOROS_TOOLBOX is not.  This is the one variable"
	       echo "required by Khoros."
	       echo ""
               echon "Do you want me to just add that line? (y/n) [y]: "
               read resp
               if [ "$resp" = "n" -o "$resp" = "N" ]; then
	          echo ""
                  echo "Please add the following line to the $DOTFILE or set it by hand"
                  echo "to use Khoros 2."
		  echo ""
		  echo "KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes"
		  echo ""
               else
$cat <<EofKT >> $HOME/$DOTFILE
KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes
export KHOROS_TOOLBOX
EofKT
	       fi
	    else
	       echo ""
               echo "Looks the lines to add are in $DOTFILE.  Please check there"
	       echo "setting with the these to make sure your environment is set up"
	       echo "correctly."
	       echo ""
               ContinuePrompt
	       echo "#"
	       echo "# These are default values for environment"
	       echo "# variables used by Khoros."
	       echo "#"
	       echo "KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes"
	       echo "KHOROS_LOG=\$HOME/khoros.cmdlog"
	       echo "KHOROS_MAIL=\$USER"
	       echo "KHOROS_NAME=\$USER"
	       echo "KHOROS_NOTIFY=STANDARD"
	       echo "KHOROS_ANSFILE=\$HOME/khoros.ans"
	       echo "TMPDIR=$TMPDIR"
	       echo "export KHOROS_TOOLBOX KHOROS_LOG KHOROS_MAIL KHOROS_NAME KHOROS_NOTIFY KHOROS_ANSFILE TMPDIR"
	       echo "#"
	       echo "#   Uncomment this if you would like data objects to accumulate "
	       echo "#   history when operated on by datamanip operators."
	       echo "#"
	       echo "#KHOROS_HISTORY=1"
	       echo "#export KHOROS_HISTORY"
	       echo "# New PATH setting to execute Khoros binaries"
	       echo "PATH=\$PATH:$binlistsh"
	       echo "export PATH"

               if [ $needldpath = 1 ]; then
                  echo "# Set platform-specific run-time library environment variables for Khoros"
                  echo "if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
                  echo "   LD_LIBRARY_PATH=$ldlist"
                  echo "else"
                  echo "   LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
                  echo "fi"
                  echo "export LD_LIBRARY_PATH"
               else
                  echo "# Set platform-specific run-time library environment variables for Khoros"
                  echo "# if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
                  echo "#    LD_LIBRARY_PATH=$ldlist"
                  echo "# else"
                  echo "#    LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
                  echo "# fi"
                  echo "# export LD_LIBRARY_PATH"
	       fi

               if [ $nokarch = 0 ]; then
                  echo "KARCH=$KARCH"
                  echo "export KARCH"
               fi
	    fi

            if [ $foundpath = 0 ]; then
$cat <<EofPath >> $HOME/$DOTFILE
# New PATH setting to execute Khoros binaries
PATH=\$PATH:$binlistsh
export PATH
EofPath
            fi
         else
            if [ $foundpath = 1 ]; then
               echo ""
               echo "The path for Khoros is already set correctly.  Not adding a"
               echo "new PATH variable."
               echo ""
            fi

$cat <<EofShdot >> $HOME/$DOTFILE
# The following lines were added automatically, by Khoros kconfigure script
#
# These are default values for environment
# variables used by Khoros.

KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes
KHOROS_LOG=\$HOME/khoros.cmdlog
KHOROS_MAIL=\$USER
KHOROS_NAME=\$USER
KHOROS_NOTIFY=STANDARD
KHOROS_ANSFILE=\$HOME/khoros.ans
TMPDIR=$TMPDIR
export KHOROS_TOOLBOX KHOROS_LOG KHOROS_MAIL KHOROS_NAME KHOROS_NOTIFY KHOROS_ANSFILE TMPDIR
#
# Uncomment this if you would like data objects to accumulate
# history when operated on by datamanip operators.
#
#KHOROS_HISTORY=1
#export KHOROS_HISTORY
EofShdot

            if [ $foundpath = 0 ]; then
$cat <<EofShPath >> $HOME/$DOTFILE
# New PATH setting to execute Khoros binaries
PATH=\$PATH:$binlistsh
export PATH
EofShPath
            fi

            if [ $needldpath = 1 ]; then
$cat <<EofShLdLibPath >> $HOME/$DOTFILE
# Set platform-specific run-time library environment variables for Khoros
if [ "\${LD_LIBRARY_PATH}" = "" ]; then
   LD_LIBRARY_PATH=$ldlist
else
   LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}
fi
export LD_LIBRARY_PATH
EofShLdLibPath
            else
$cat <<EofShLdLibPathComm >> $HOME/$DOTFILE
# Set platform-specific run-time library environment variables for Khoros
# if [ "\${LD_LIBRARY_PATH}" = "" ]; then
#    LD_LIBRARY_PATH=$ldlist
# else
#    LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}
# fi
# export LD_LIBRARY_PATH
EofShLdLibPathComm
            fi

            if [ $nokarch = 0 ]; then
$cat <<EofShKarch >> $HOME/$DOTFILE
KARCH=$KARCH
export KARCH
EofShKarch
            fi
         fi
      fi

   elif [ ! -w "$HOME/$DOTFILE" ]; then
      echo ""
      echo "Can not write to the file $HOME/$DOTFILE.  To use"
      echo "Khoros you will need to setup the above environment variables"
      echo "by hand or add it to one of your environment setup files"
      echo "by hand.  You can talk to you local system manager on how to"
      echo "do this."
      echo ""
      DOTFILE=""
   fi
fi

ContinuePrompt

newtopic
if [ $CSHVARIENT = 1 ]; then
   echo ""
   echo "Now you should source the .khoros_env to have the your environment"
   echo "setup for you to use Khoros 2 routines.  At your prompt type:"
   echo ""
   echo "      source $HOME/.khoros_env"
   echo ""
else
   if [ "$DOTFILE" = "" ]; then
      echo ""
      echo "You now need to set up the environment variables by hand to use"
      echo "Khoros 2 routines.  At your prompt type the following lines:"
      echo ""
      echo "PATH=\$PATH:$binlistsh"
      echo "KHOROS_TOOLBOX=\$HOME/.Toolboxes:$TOPDIR/Toolboxes"
      echo "KHOROS_LOG=\$HOME/khoros.cmdlog"
      echo "KHOROS_MAIL=\$USER"
      echo "KHOROS_NAME=\$USER"
      echo "KHOROS_NOTIFY=STANDARD"
      echo "KHOROS_ANSFILE=\$HOME/khoros.ans"
      echo "TMPDIR=$TMPDIR"
      echo "export PATH KHOROS_TOOLBOX KHOROS_LOG KHOROS_MAIL KHOROS_NAME KHOROS_NOTIFY KHOROS_ANSFILE TMPDIR"
      echo ""

      if [ $needldpath = 1 ]; then
         echo "# Set platform-specific run-time library environment variables for Khoros"
         echo "if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
         echo "   LD_LIBRARY_PATH=$ldlist"
         echo "else"
         echo "   LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
         echo "fi"
         echo "export LD_LIBRARY_PATH"
      else
         echo "# Set platform-specific run-time library environment variables for Khoros"
         echo "# if [ \"\${LD_LIBRARY_PATH}\" = \"\" ]; then"
         echo "#    LD_LIBRARY_PATH=$ldlist"
         echo "# else"
         echo "#    LD_LIBRARY_PATH=$ldlist:\${LD_LIBRARY_PATH}"
         echo "# fi"
         echo "# export LD_LIBRARY_PATH"
      fi

      if [ $nokarch = 0 ]; then
         echo "KARCH=$KARCH"
         echo "export KARCH"
      fi
      echo ""
   else
      echo ""
      echo "You now need to login in again so that your changes"
      echo ""
      echo "   $HOME/$DOTFILE"
      echo ""
      echo "will take effect."
      echo ""
   fi
fi

exit 0
