/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	03.kcms
== Testing:	kcms_open_toolbox(kstring tbname)
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kobject   toolbox;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 3)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	toolbox name is NULL
      ----------------------------------------------------------------*/
      case 1:
	 toolbox = kcms_open_toolbox(NULL);
	 if (toolbox != NULL)
	 {
	    kprintf("Error: kcms_open_toolbox(NULL) returned non-NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
         if (errno != KCMS_ENOTFOUND)
	 {
	    kprintf("Error: unexpected errno (%d)\n", errno);
	    kexit(KEXIT_FAILURE);
	 }
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      | toolbox name is empty string: ""
      ----------------------------------------------------------------*/
      case 2:
	 toolbox = kcms_open_toolbox("");
	 if (toolbox != NULL)
	 {
	    kprintf("Error: kcms_open_toolbox(NULL) returned non-NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
         if (errno != KCMS_ENOTFOUND)
	 {
	    kprintf("Error: unexpected errno (%d)\n", errno);
	    kexit(KEXIT_FAILURE);
	 }
	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      | try to open a non-existent toolbox
      | name of toolbox is passed on the command-line
      ----------------------------------------------------------------*/
      case 3:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (toolbox != NULL)
	 {
	    kprintf("Error: kcms_open_toolbox(NULL) returned non-NULL!\n");
	    kexit(KEXIT_FAILURE);
	 }
         if (errno != KCMS_ENOTFOUND)
	 {
	    kprintf("Error: unexpected errno (%d)\n", errno);
	    kexit(KEXIT_FAILURE);
	 }
	 break;


      /*----------------------------------------------------------------
      | CASE 4
      |
      | toolbox database is missing
      | name of toolbox is passed on the command-line
      ----------------------------------------------------------------*/
      case 4:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (toolbox != NULL)
	 {
	    kprintf("Error: kcms_open_toolbox(%s) returned non-NULL!\n",
		    argv[2]);
	    kexit(KEXIT_FAILURE);
	 }
         if (errno != KCMS_ENODB)
	 {
	    kprintf("Error: unexpected errno (%d)\n", errno);
	    kexit(KEXIT_FAILURE);
	 }
	 break;


      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 printf("incorrect test id (%s) -- should be in the range 1 .. 4\n",
		argv[1]);
	 kexit(KEXIT_FAILURE);
   }

   kexit(KEXIT_SUCCESS);
}

