/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	05.create_manual
== Testing:	kcms_create_manual(
==			kobject  toolbox,
==			kstring  manual_name)
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kobject   toolbox;
   char      manual_name[KLENGTH];


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 3)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);
   kprog_set_toolbox("BOOTSTRAP");
   kprog_set_program("ktest");

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	toolbox and manual name are NULL
      ----------------------------------------------------------------*/
      case 1:
	 if (kcms_create_manual(NULL, NULL))
	 {
	    kprintf("Error: kcms_create_manual(NULL, NULL) returned TRUE!\n");
	    kexit(KEXIT_FAILURE);
	 }
         if (errno != KCMS_ENULLTB)
	 {
	    kprintf("Error: unexpected errno (%d)\n", errno);
	    kexit(KEXIT_FAILURE);
	 }
	 kprintf("kcms_create_manual() case 1 OK\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      |	valid toolbox, name is NULL
      ----------------------------------------------------------------*/
      case 2:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (toolbox == NULL)
	 {
	    kprintf("Error: failed to open toolbox `%s'\n", argv[2]);
	    kexit(KEXIT_FAILURE);
	 }
	 if (kcms_create_manual(toolbox, NULL))
	 {
	    kprintf("Error: kcms_create_manual() "
		    "with valid toolbox but NULL name returned TRUE!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 if (!kcms_close(toolbox))
	 {
	    kprintf("Error: kcms_close() returned FALSE after "
		    "call to kcms_create_manual(toolbox, NULL)\n");
	    kexit(KEXIT_FAILURE);
	 }
	 kprintf("kcms_create_manual() case 2 OK\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      |	valid toolbox, valid manual name
      |		should really check that the manual created ok
      |		and go and run make, make clean, etc :-)
      ----------------------------------------------------------------*/
      case 3:
	 toolbox = kcms_open_toolbox(argv[2]);
	 if (kstring_capitalize(argv[2], manual_name) == NULL)
	 {
	    kprintf("Error: failed to capitalize toolbox `%s'\n", argv[2]);
	    kexit(KEXIT_FAILURE);
	 }
	 if (toolbox == NULL)
	 {
	    kprintf("Error: failed to open toolbox `%s'\n", argv[2]);
	    kexit(KEXIT_FAILURE);
	 }
	 if (!kcms_create_manual(toolbox, manual_name))
	 {
	    kprintf("Error: kcms_create_manual() with "
		    "valid toolbox object and manual name returned FALSE!\n");
	    kexit(KEXIT_FAILURE);
	 }
	 if (!kcms_close(toolbox))
	 {
	    kprintf("Error: kcms_close() returned FALSE after "
		    "successful call to kcms_create_manual()\n");
	    kexit(KEXIT_FAILURE);
	 }
	 kprintf("kcms_create_manual() case 3 OK\n");
	 break;


      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 printf("incorrect test id (%s) -- should be in the range 1 .. 4\n",
		argv[1]);
	 kexit(KEXIT_FAILURE);
   }

   kexit(KEXIT_SUCCESS);
}

