/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	09.kcms
== Testing:	kcms_open_cmobj()
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

static void print_toolbox_list(kstring *, size_t);
char  sprint_buffer[25 * KLENGTH];
static int compare_object_attributes(kobject, kobject);


void
ktest_fatal(
   kstring  format,
   kvalist)
{
   kva_list  arguments;


   kva_start(arguments, format);
   if (kvsprintf(sprint_buffer, format, arguments) == 0)
   {
      /*-- we failed to format the message into the buffer -----------*/
      kfprintf(kstderr, "INTERNAL ERROR - FAILED TO FORMAT MESSAGE!\n");
   }
   else
   {
      kfprintf(kstderr, sprint_buffer);
      kfprintf(kstderr, "\n");
   }
   kva_end(arguments);
   kexit(KEXIT_FAILURE);
}

main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kobject   toolbox;
   kobject   object;
   int       i;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 5)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	both toolbox and object name are NULL
      ----------------------------------------------------------------*/
      case 1:
	 object = kcms_open_cmobj(NULL, NULL);
	 if (object != NULL)
	    ktest_fatal("Error: kcms_open_cmobj(NULL, NULL) returned non-NULL\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      |	toolbox is NULL, object name is a valid identifier
      ----------------------------------------------------------------*/
      case 2:
	 object = kcms_open_cmobj(NULL, "kecho");
	 if (object != NULL && errno != KCMS_ENULLTB)
	    ktest_fatal("Error: kcms_open_cmobj(NULL, \"kecho\") "
			"returned non-NULL\n");
	 else if (errno != KCMS_ENULLTB)
	    ktest_fatal("Error: kcms_open_cmobj(NULL, \"kecho\") "
			"returned errno != KCMS_ENULLTB\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      |	toolbox is valid toolbox object, object name is NULL
      ----------------------------------------------------------------*/
      case 3:
	 kexit(KEXIT_FAILURE);

	 break;


      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 printf("incorrect test id (%s) -- should be in the range 1 .. 4\n",
		argv[1]);
	 kexit(KEXIT_FAILURE);
   }

   kexit(KEXIT_SUCCESS);
}
