/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	16.duplicate
== Testing:	kcms_duplicate(kobject source, kobject destination)
== Description:
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

static void print_toolbox_list(kstring *, size_t);
static int compare_object_attributes(kobject, kobject);


main(
   int    argc,
   char  *argv[])
{
   int       test_id;
   kstring  *toolboxes;
   size_t    countToolboxes;
   kobject   toolbox;
   kobject   object;
   kstring   sourceToolboxName;
   kstring   sourceObjectName;
   kstring   destinationToolboxName;
   kobject   sourceToolbox;
   kobject   sourceObject;
   kobject   destinationToolbox;
   kobject   newObject;
   int       i;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 2 && argc != 3 && argc != 5)
   {
      kprintf("incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }
   test_id = atoi(argv[1]);
   kprog_set_toolbox("BOOTSTRAP");
   kprog_set_program("ktest");

   switch (test_id)
   {
      /*----------------------------------------------------------------
      | CASE 1
      |
      |	both source and destination are NULL
      ----------------------------------------------------------------*/
      case 1:
	 object = kcms_duplicate(NULL, NULL);
	 if (object != NULL)
	    ktest_fatal("Error: kcms_duplicate(NULL, NULL) "
			"returned non-NULL\n");
	 break;


      /*----------------------------------------------------------------
      | CASE 2
      |
      |	source is a valid toolbox, destination is NULL
      | you can't duplicate a toolbox!
      ----------------------------------------------------------------*/
      case 2:
	 toolbox = kcms_open_toolbox(argv[2]);
         if (toolbox == NULL)
            ktest_fatal("Error: failed to open toolbox `%s'\n", argv[2]);
	 if (kcms_duplicate(toolbox, NULL) != NULL)
	    ktest_fatal("kcms_duplicate(toolbox, NULL) returned non-NULL!\n");

	 kprintf("kcms_duplicate() case 2 ok\n");

	 break;


      /*----------------------------------------------------------------
      | CASE 3
      |
      |	source is NULL, destination is a valid toolbox
      ----------------------------------------------------------------*/
      case 3:
	 toolbox = kcms_open_toolbox(argv[2]);
         if (toolbox == NULL)
            ktest_fatal("Error: failed to open toolbox `%s'\n", argv[2]);
	 if (kcms_duplicate(NULL, toolbox) != NULL)
	    ktest_fatal("kcms_duplicate(NULL, toolbox) returned non-NULL!\n");

	 kprintf("kcms_duplicate() case 3 ok\n");

	 break;


      /*----------------------------------------------------------------
      | CASE 4
      |
      |	source is a KROUTINE which was just created,
      | destination is a valid toolbox.
      |	check that object went over ok.
      ----------------------------------------------------------------*/
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
	 sourceToolboxName      = argv[2];
	 sourceObjectName       = argv[3];
	 destinationToolboxName = argv[4];

	 /*-- open the SOURCE TOOLBOX ------------------------*/
	 sourceToolbox = kcms_open_toolbox(sourceToolboxName);
	 if (sourceToolbox == NULL)
	    ktest_fatal("failed to open source toolbox (%s)",
			sourceToolboxName);

	 /*-- open the SOURCE KROUTINE OBJECT ----------------*/
	 sourceObject = kcms_open_cmobj(sourceToolbox, sourceObjectName);
	 if (sourceObject == NULL)
	    ktest_fatal("failed to open source object (%s)", sourceObjectName);

	 /*-- open the DESTINATION TOOLBOX -------------------*/
	 destinationToolbox = kcms_open_toolbox(destinationToolboxName);
	 if (destinationToolbox == NULL)
	    ktest_fatal("failed to open destination toolbox (%s)",
			destinationToolboxName);

	 /*-- duplicate the object ---------------------------*/
	 newObject = kcms_duplicate(sourceObject, destinationToolbox);
	 if (newObject == NULL)
	    ktest_fatal("failed to duplicated object");

	 /*-- check the copy against the original ------------*/
	 if (!compare_object_attributes(sourceObject, newObject))
	    ktest_fatal("object attributes were not the same!");

	 break;
	    

      /*----------------------------------------------------------------
      | bomb out with an error message otherwise
      ----------------------------------------------------------------*/
      default:
	 ktest_fatal("incorrect test id (%s) -- "
		     "should be in the range 1 .. 8\n",
		     argv[1]);
	 break;
   }

   kexit(KEXIT_SUCCESS);
}

typedef struct
{
   kcms_attr		attribute;
   kstring              name;
   unsigned long	object_types;
} attr_info;

attr_info integer_attributes[] =
{
   {KCMS_TYPE,"KCMS_TYPE",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_GENERATE_EXECUTABLE, "KCMS_CMOBJ_GENERATE_EXECUTABLE",
    KCMS_PANE},

   {KCMS_CMOBJ_INCANTATA, "KCMS_CMOBJ_INCANTATA",
    (KCMS_KROUTINE|KCMS_XVROUTINE|KCMS_PANE|KCMS_SCRIPT)},
};

attr_info string_attributes[] =
{
   {KCMS_NAME, "KCMS_NAME",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_AUTHOR,"KCMS_CMOBJ_AUTHOR",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_AUTHOR_EMAIL, "KCMS_CMOBJ_AUTHOR_EMAIL",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_BNAME, "KCMS_CMOBJ_BNAME",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_CATEGORY, "KCMS_CMOBJ_CATEGORY",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_SUBCATEGORY, "KCMS_CMOBJ_SUBCATEGORY",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_GEN_LIBNAME, "KCMS_CMOBJ_GEN_LIBNAME",
    KCMS_KROUTINE},

   {KCMS_CMOBJ_ICON_NAME, "KCMS_CMOBJ_ICON_NAME",
    (KCMS_LIBRARY|KCMS_KROUTINE|KCMS_XVROUTINE|KCMS_PANE|KCMS_SCRIPT)},

   {KCMS_CMOBJ_SHORT_DESC, "KCMS_CMOBJ_SHORT_DESC",
    KCMS_ALL_OBJECT_TYPES},

   {KCMS_CMOBJ_VERSION, "KCMS_CMOBJ_VERSION",
    KCMS_ALL_OBJECT_TYPES}
};

attr_info ulong_attributes[] =
{
   {KCMS_CMOBJ_LANGTYPE, "KCMS_CMOBJ_LANGTYPE",
    (KCMS_LIBRARY|KCMS_KROUTINE|KCMS_XVROUTINE|KCMS_SCRIPT)},

   {KCMS_CMOBJ_TYPE, "KCMS_CMOBJ_TYPE",
    KCMS_ALL_OBJECT_TYPES}
};

static int
compare_object_attributes(
   kobject  objectA,
   kobject  objectB)
{
   unsigned long  ulong_A_value;
   unsigned long  ulong_B_value;
   kstring        string_A_value;
   kstring        string_B_value;
   int            int_A_value;
   int            int_B_value;
   unsigned long  object_type_A;
   unsigned long  object_type_B;
   int            i;
   int            retval          = TRUE;


   if (!kcms_get_attribute(objectA, KCMS_CMOBJ_TYPE, &object_type_A)
       || !kcms_get_attribute(objectA, KCMS_CMOBJ_TYPE, &object_type_B))
   {
      kprintf("Could not determine object types\n");
      return FALSE;
   }

   if (object_type_A != object_type_B)
   {
      kprintf("Object types are not the same!\n");
      return FALSE;
   }

   /*-- compare INTEGER attributes -----------------------------------*/
   for (i=0; i<knumber(integer_attributes); i++)
   {
      if ((object_type_A & integer_attributes[i].object_types)
	  != object_type_A)
	 continue;

      if (!kcms_get_attribute(objectA, integer_attributes[i].attribute,
			      &int_A_value))
      {
	 kprintf("Failed to get attribute %s from the source object\n",
		 integer_attributes[i].name);
	 retval = FALSE;
	 continue;
      }
      if (!kcms_get_attribute(objectB, integer_attributes[i].attribute,
			      &int_B_value))
      {
	 kprintf("Failed to get attribute %s from the destination object\n",
		 integer_attributes[i].name);
	 retval = FALSE;
	 continue;
      }

      /*-- are the values the same? ----*/
      if (int_A_value != int_B_value)
      {
	 kprintf("Different values for the %s attribute\n",
		integer_attributes[i].name );
	 retval = FALSE;
	 continue;
      }
      else
         kprintf("Attribute %s ok\n", integer_attributes[i].name);
   }

   /*-- compare UNSIGNED LONG attributes -------------------------------*/
   for (i=0; i<knumber(ulong_attributes); i++)
   {
      if ((object_type_A & ulong_attributes[i].object_types)
	  != object_type_A)
	 continue;

      if (!kcms_get_attribute(objectA, ulong_attributes[i].attribute,
			      &ulong_A_value))
      {
	 kprintf("Failed to get attribute %s from the source object\n",
		 ulong_attributes[i].name);
	 retval = FALSE;
	 continue;
      }
      if (!kcms_get_attribute(objectB, ulong_attributes[i].attribute,
			      &ulong_B_value))
      {
	 kprintf("Failed to get attribute %s from the destination object\n",
		 ulong_attributes[i].name);
	 retval = FALSE;
	 continue;
      }

      /*-- are the values the same? ----*/
      if (ulong_A_value != ulong_B_value)
      {
	 kprintf("Different values for the %s attribute\n",
		ulong_attributes[i].name );
	 retval = FALSE;
	 continue;
      }
      else
         kprintf("Attribute %s ok\n", ulong_attributes[i].name);
   }

   /*-- compare STRING attributes --------------------------------------*/
   for (i=0; i<knumber(string_attributes); i++)
   {
      if ((object_type_A & string_attributes[i].object_types)
	  != object_type_A)
	 continue;

      if (!kcms_get_attribute(objectA, string_attributes[i].attribute,
			      &string_A_value))
      {
	 kprintf("Failed to get attribute %s from the source object\n",
		 string_attributes[i].name);
	 retval = FALSE;
	 continue;
      }
      if (!kcms_get_attribute(objectB, string_attributes[i].attribute,
			      &string_B_value))
      {
	 kprintf("Failed to get attribute %s from the destination object\n",
		 string_attributes[i].name);
	 retval = FALSE;
	 continue;
      }

      /*-- are the values the same? ----*/
      if (kstrcmp(string_A_value, string_B_value) != 0)
      {
	 kprintf("Different values for the %s attribute\n",
		string_attributes[i].name );
	 retval = FALSE;
      }
      else
         kprintf("Attribute %s ok\n", string_attributes[i].name);
   }

   return retval;
}
