/*
== Toolbox:	$BOOTSTRAP
== Object:	kcms
== Testsuite:	35.kcms
== Testing:	kcms_legal_identifier(
==			kstring  string,
==			int      kotype)
== Description:	string can be any string, kotype must be one of:
==			KOBJ_CMSTB	toolbox object
==			KOBJ_CMSOBJ	software object
*/

#include <bootstrap.h>
#include <ktestutils/ktestutils.h>

typedef struct
{
   kstring  identifier;
   kstring  comment;
   int      object_type;
   int      isvalid;
} TestInfo;

static TestInfo test_cases[] =
{
   {NULL,      "NULL string (toolbox)",                   KOBJ_CMSTB, FALSE},
   {"",        "empty string (toolbox)",                  KOBJ_CMSTB, FALSE},
   {"_fred",   "string starts with underscore (toolbox)", KOBJ_CMSTB, FALSE},
   {"toolbox", "can't have a toolbox called `toolbox'",   KOBJ_CMSTB, FALSE},

   {NULL,    "NULL string (s/w object)",                   KOBJ_CMSOBJ, FALSE},
   {"",      "empty string (s/w object)",                  KOBJ_CMSOBJ, FALSE},
   {"_fred", "string starts with underscore (s/w object)", KOBJ_CMSOBJ, FALSE},

};

main(
   int    argc,
   char  *argv[])
{
   int    result;
   int    i;


   /*-- working directory is passed as first argument ----------------*/
   if (argc != 1)
   {
      kprintf("testsuite FAILED\n"
	      "incorrect number of command-line arguments (%d)\n", argc);
      kexit(1);
   }

   ktest_begin("35.kcms", "kcms_legal_identifier()");

   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/
   for (i = 0; i < knumber(test_cases); i++)
   {
      ktest_case(test_cases[i].comment);
      result = kcms_legal_identifier(test_cases[i].identifier,
				     test_cases[i].object_type);
      ktest_result((result == test_cases[i].isvalid)
		   ? KTEST_PASS
		   : KTEST_FAIL);
   }
   /*-----------------------------------------------------------------*/
   /*-----------------------------------------------------------------*/

   ktest_end();
}
