#include "bootstrap.h"
 
int read_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kread_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);
	entry->next = NULL;
	entry->prev = NULL;

	return(TRUE);
}

int write_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kwrite_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);

	return(TRUE);
}

void free_data(
   klist *list)
{
   kfree(list->identifier);
   kfree(list->client_data);
}

void main(
   int  argc,
   char *argv[])
{
	kfile *file;
	klist *list = NULL, *tmp, *lstore;
	int     i, tmpfid, type;
	ssize_t num;
	int     status;
	char    *word1, *word2, **words, *path = "Makefile";


	if (argc > 1)
	   path = argv[1];

	if ((file = kfopen(path, "r")) == NULL)
	{
	   kprintf("Unable to open file '%s'", path);
	   kexit(KEXIT_FAILURE);
	}

	if ((words = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
                  KLITERAL, " \t\n", NULL, NULL, &num, &status)) == NULL)
        {
           kprintf("Cannot read contents of file '%s'\n", path);
           kexit(KEXIT_FAILURE);
        }
	kfclose(file);

	kfprintf(kstderr, "%s has %ld words\n", path, (long) num);
	for (i = 0; i < num; i++)
	   list = klist_insert(list, kstrdup(words[i]), NULL, KLIST_HEAD, TRUE);

	karray_free(words, KSTRING, (size_t) num, NULL);
	type = kstruct_define("klist", sizeof(klist), read_list,
		   write_list, NULL, NULL);

	tmpfid = ktmpfid();
	tmp = list;
	while (tmp != NULL)
	{
	   kwrite_struct(tmpfid, tmp, type);
	   tmp = klist_next(tmp);
	}
	klseek(tmpfid, (off_t) 0, KSEEK_SET);

	lstore = list;
	while (list != NULL)
	{
	   kread_struct(tmpfid, (kaddr *) &tmp, type);
	   word1 = (char *) klist_identifier(list);
	   word2 = (char *) klist_identifier(tmp);
	   if (kstrcmp(word1, word2) != 0 && word1 != word2)
	   {
	      kfprintf(kstderr, "Error! lists are not identical\n");
	      kexit(KEXIT_FAILURE);
	   }
	   list = klist_next(list);
	   klist_free(tmp, free_data);
	}
	kclose(tmpfid);
	klist_free(lstore, free_data);
	kfprintf(kstderr, "Success! lists are identical\n");
	kexit(KEXIT_SUCCESS);
}
