#include "bootstrap.h"
 
typedef struct
{
	int *i1;
	int ival;
	int *i2;

	float *f1;
	float fval;
	float *f2;

	double *d1;
	double dval;
	double *d2;
} ktest;

int read_test(
   int id,
   kaddr buf,
   int type)
{
	ktest *entry = (ktest *) buf;

	if (kread_pointer(id, (kaddr *) &entry->i1, KINT) != 1)
	   return(FALSE);
	if (kread_int(id, &entry->ival, (size_t)1) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->i2, KINT) != 1)
	   return(FALSE);

	if (kread_pointer(id, (kaddr *) &entry->f1, KFLOAT) != 1)
	   return(FALSE);
	if (kread_float(id, &entry->fval, (size_t)1) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->f2, KFLOAT) != 1)
	   return(FALSE);

	if (kread_pointer(id, (kaddr *) &entry->d1, KDOUBLE) != 1)
	   return(FALSE);
	if (kread_double(id, &entry->dval, (size_t)1) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->d2, KDOUBLE) != 1)
	   return(FALSE);

	return(TRUE);
}

int write_test(
   int id,
   kaddr buf,
   int type)
{
	ktest *entry = (ktest *) buf;

	if (kwrite_pointer(id, (kaddr *) &entry->i1, KINT) != 1)
	   return(FALSE);
	if (kwrite_int(id, &entry->ival, (size_t)1) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->i2, KINT) != 1)
	   return(FALSE);

	if (kwrite_pointer(id, (kaddr *) &entry->f1, KFLOAT) != 1)
	   return(FALSE);
	if (kwrite_float(id, &entry->fval, (size_t)1) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->f2, KFLOAT) != 1)
	   return(FALSE);

	if (kwrite_pointer(id, (kaddr *) &entry->d1, KDOUBLE) != 1)
	   return(FALSE);
	if (kwrite_double(id, &entry->dval, (size_t)1) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->d2, KDOUBLE) != 1)
	   return(FALSE);

	return(TRUE);
}

void main(
   int  argc,
   char *argv[])
{
	ktest *test1, *test2;
	int   i, tmpfid, type, iterations = 1;

	type = kstruct_define("ktest", sizeof(ktest),
		   read_test, write_test, NULL, NULL);

	if (argc > 1)
	   iterations = atoi(argv[1]);

	test1 = kcalloc((size_t) 1, sizeof(ktest));
	test1->ival = 100;
	test1->i1 = &test1->ival;
	test1->i2 = &test1->ival;

	test1->fval = 0.1;
	test1->f1 = &test1->fval;
	test1->f2 = &test1->fval;

	test1->dval = 0.000001;
	test1->d1 = &test1->dval;
	test1->d2 = &test1->dval;

	tmpfid = ktmpfid();
	kwrite_struct(tmpfid, test1, type);
	for (i = 0; i < iterations; i++)
	{
	   klseek(tmpfid, (off_t)0, SEEK_SET);
	   kread_struct(tmpfid, (kaddr *) &test2, type);

	   if (test1->ival != test2->ival)
	   {
	      kfprintf(kstderr, "int vals not the same. should be '%d'\n",
			test1->ival);
	      kexit(KEXIT_FAILURE);
	   }

	   if (test2->i1 != &test2->ival || test2->i2 != &test2->ival)
	   {
	      kfprintf(kstderr, "int pointers not pointing at ival.\n");
	      kexit(KEXIT_FAILURE);
	   }

	   if (test1->fval != test2->fval)
	   {
	      kfprintf(kstderr, "int vals not the same. should be '%d'\n",
			test1->fval);
	      kexit(KEXIT_FAILURE);
	   }

	   if (test2->f1 != &test2->fval || test2->f2 != &test2->fval)
	   {
	      kfprintf(kstderr, "float pointers not pointing at fval.\n");
	      kexit(KEXIT_FAILURE);
	   }

	   if (test1->dval != test2->dval)
	   {
	      kfprintf(kstderr, "double vals not the same. should be '%d'\n",
			test1->dval);
	      kexit(KEXIT_FAILURE);
	   }

	   if (test2->d1 != &test2->dval || test2->d2 != &test2->dval)
	   {
	      kfprintf(kstderr, "double pointers not pointing at dval.\n");
	      kexit(KEXIT_FAILURE);
	   }
	   kfree(test2);
	}
	kclose(tmpfid);
	kfree(test1);

	kfprintf(kstderr, "Success! structures are identical\n");
	kexit(KEXIT_SUCCESS);
}
