#include "bootstrap.h"
 
int read_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kread_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->next, type) != 1)
	   return(FALSE);
	if (kread_pointer(id, (kaddr *) &entry->prev, type) != 1)
	   return(FALSE);

	return(TRUE);
}

int write_list(
   int id,
   kaddr buf,
   int type)
{
	klist *entry = (klist *) buf;

	if (kwrite_string(id, &entry->identifier, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_string(id, &entry->client_data, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_ubyte(id, &entry->head, (size_t) 1) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->next, type) != 1)
	   return(FALSE);
	if (kwrite_pointer(id, (kaddr *) &entry->prev, type) != 1)
	   return(FALSE);

	return(TRUE);
}

int compare_list(
   kaddr buf1,
   kaddr buf2,
   int type)
{
	klist *entry1 = (klist *) buf1;
	klist *entry2 = (klist *) buf2;

	if (kstrcmp(entry1->identifier, entry2->identifier) != 0)
	   return(FALSE);
	if (kstrcmp(entry1->client_data, entry2->client_data) != 0)
	   return(FALSE);
	if (kstruct_compare(entry1->next, entry2->next, type) != TRUE)
	   return(FALSE);
	if (kstruct_compare(entry1->prev, entry2->prev, type) != TRUE)
	   return(FALSE);

	return(TRUE);
}

void free_list(
   kaddr buf,
   int type,
   int freeself)
{
	klist *entry = (klist *) buf;

	kfree(entry->identifier);
	kfree(entry->client_data);
	kstruct_free(entry->next, type, TRUE);
	kstruct_free(entry->prev, type, TRUE);
	kfree(entry);
}

void
main(
   int  argc,
   char *argv[])
{
	kfile *file;
	int   i, type, status;
	ssize_t num;
	char  **words, *path = "Makefile";
	klist *list1 = NULL, *list2 = NULL;


	if (argc > 1)
	   path = argv[1];

	if ((file = kfopen(path, "r")) == NULL)
	{
	   kprintf("Unable to open file '%s'", path);
	   kexit(KEXIT_FAILURE);
	}

	if ((words = kparse_file_scan_delimit(file, KPARSE_BOF, KPARSE_EOF,
                  KLITERAL, " \t\n", NULL, NULL, &num, &status)) == NULL)
        {
           kprintf("Cannot read contents of file '%s'\n", path);
           kexit(KEXIT_FAILURE);
        }
	kfclose(file);

	kfprintf(kstderr, "%s has %d words\n", path, (int) num);
	for (i = 0; i < num; i++)
	   list1 = klist_insert(list1, kstrdup(words[i]), NULL,KLIST_HEAD,TRUE);

	karray_free(words, KSTRING, (size_t) num, NULL);
	type = kstruct_define("klist", sizeof(klist), read_list,
		   write_list, compare_list, free_list);

	list2 = kstruct_duplicate(list1, type, NULL);
	if (kstruct_compare(list1, list2, type) == FALSE)
	{
	   kfprintf(kstderr, "Error! lists are not identical\n");
	   kexit(KEXIT_FAILURE);
	}
	kstruct_free(list1, type, TRUE);
	kstruct_free(list2, type, TRUE);
	kfprintf(kstderr, "Success! lists are identical\n");
	kexit(KEXIT_SUCCESS);
}
