#include <bootstrap.h>
#include "io_test.h"

/*-- write out a strucure to a file --*/
void write_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_WRONLY|O_CREAT|O_TRUNC, 0644);
   kwrite_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- read in a strucure from a file --*/
void read_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_RDONLY, 0644);
   kread_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- simple routines --*/

simple_p create_simple(int z)
{
   simple_p s;
   
   s = (simple_p) kmalloc(sizeof(simple_t));
   s->a = z;
   s->b = (float) z;

   return s;
}

int compare_simple(struct simple *s1, struct simple *s2)
{
   if (kmemcmp(s1, s2, sizeof(struct simple)) == 0)
      return TRUE;
   else
      return FALSE;
}

/*-- intfarm routines --*/

intfarm_p create_intfarm(int z)
{
   intfarm_p i;
   int j;
   
   i = (intfarm_p) kmalloc(sizeof(struct intfarm));
   i->i  = z;
   i->it = z+1;

   i->is = (int *) kdupalloc(&(i->i), sizeof(int));
   i->ip = (int_p) kdupalloc(&(i->i), sizeof(int));

   i->ib[0] = z+2;
   i->ib[1] = z+3;
   i->ib[2] = z+4;
   
   for (j = 0; j < 10; j++)
      i->ia[j] = z+j;
   
   return i;
}

int compare_intfarm(struct intfarm *i1, struct intfarm *i2)
{
   int j;

   if (i1 == NULL && i2 == NULL)
      return TRUE;

   if (i1 == NULL || i2 == NULL)
      return FALSE;
   
   if (i1->i != i2->i)
      return FALSE;

   if (i1->it != i2->it)
      return FALSE;

   if (i1->is != NULL && i2->is != NULL)
   {
      if (*i1->is != *i2->is)
	 return FALSE;
   }
   else if (!(i1->is == NULL && i2->is == NULL))
      return FALSE;

   if (i1->ip != NULL && i2->ip != NULL)
   {
      if (*i1->ip != *i2->ip)
	 return FALSE;
   }
   else if (!(i1->is == NULL && i2->is == NULL))
      return FALSE;

   for (j = 0; j < 10; j++)
      if (i1->ib[j] != i2->ib[j])
	 return FALSE;

   for (j = 0; j < 10; j++)
      if (i1->ia[j] != i2->ia[j])
	 return FALSE;

   return TRUE;
}

/*-- collection routines --*/

collection_t *create_collection(int z)
{
   collection_t *c;
   simple_p      s;
   int j;
   
   c = (collection_t *) kmalloc(sizeof(collection_t));

   c->i1 = create_intfarm(z);
   c->i2 = NULL;
   
   s = create_simple(z+1);
   kmemcpy(&(c->s),  s, sizeof(struct simple));
   kmemcpy(&(c->st), s, sizeof(simple_t));

   c->ss = create_simple(z+2);
   c->sp = create_simple(z+3);

   for (j = 0; j < 3; j++)
      kmemcpy(&(c->sb[j]), s, sizeof(simple_t));

   for (j = 0; j < 3; j++)
      kmemcpy(&(c->sa[j]), s, sizeof(simple_t));

   kfree(s);
   return c;
}

int compare_col(collection_t *c1, collection_t *c2)
{
   int j;

   /*-- compare all variations of int --*/
   if (!compare_intfarm(c1->i1, c2->i1))
      return FALSE;

   if (!compare_intfarm(c1->i2, c2->i2))
      return FALSE;

   /*-- compare all variations of simple --*/
   if (!compare_simple(&c1->s, &c2->s))
      return FALSE;
   if (!compare_simple(&c1->st, &c2->st))
      return FALSE;
   if (!compare_simple(c1->ss, c2->ss))
      return FALSE;
   if (!compare_simple(c1->sp, c2->sp))
      return FALSE;
   
   for (j = 0; j < 3; j++)
      if (!compare_simple(&(c1->sb[j]), &(c2->sb[j])))
	 return FALSE;

   for (j = 0; j < 3; j++)
      if (!compare_simple(&(c1->sa[j]), &(c2->sa[j])))
	 return FALSE;

   return TRUE;
}

void
main()
{
   collection_t *o_col;
   collection_t i_col;

   int i1 = 32;
   int i2 = 29;

   int *o_p;
   int *i_p;
   
   int ix;

   /*-- this should recursively define everything we need --*/
   kdefine_collection_t();

   /*-- simple integer type --*/
   write_structure("./t.data", (kaddr) &i1, KTYPE_INT_T);
   read_structure("./t.data", (kaddr) &ix, KTYPE_INT_T);
   
   if (i1 == ix)
      kprintf("int_t is identical!\n");
   else
      kprintf("int_t is different!\n");

   if (!kstruct_compare((kaddr)&i1, (kaddr)&ix, KTYPE_INT_T))
      kprintf("flaky compare for int_t!\n");
      

   /*-- simple integer pointer --*/
   o_p = &i2;
   write_structure("./t.data", (kaddr) &o_p, KTYPE_INT_P);
   read_structure("./t.data", (kaddr) &i_p, KTYPE_INT_P);

   if (*o_p == *i_p)
      kprintf("int_p is identical!\n");
   else
      kprintf("int_p is different!\n");

   if (!kstruct_compare(&i_p, &o_p, KTYPE_INT_P))
      kprintf("flaky compare for int_p!\n");

   /*-- initialize collection --*/
   o_col = (collection_t *) create_collection(7);

   write_structure("./t.data", (kaddr) o_col, KTYPE_COLLECTION_T);
   read_structure("./t.data", (kaddr) &i_col, KTYPE_COLLECTION_T);

   if (compare_col(o_col, &i_col))
      kprintf("collection is identical!\n");
   else
      kprintf("collection is different!\n");

   if (!kstruct_compare(o_col, &i_col, KTYPE_COLLECTION_T))
      kprintf("flaky compare for collection_t!\n");

   /*-- tidy up --*/
   kstruct_free(o_col, KTYPE_COLLECTION, TRUE);

   kstruct_free(i_col.i1, KTYPE_INTFARM, TRUE);
   kstruct_free(i_col.i2, KTYPE_INTFARM, TRUE);

   kexit(KEXIT_SUCCESS);
}
