#include <bootstrap.h>
#include "io_test.h"

/*-- write out a strucure to a file --*/
void write_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_WRONLY|O_CREAT|O_TRUNC, 0644);
   kwrite_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

/*-- read in a strucure from a file --*/
void read_structure(char *name, kaddr s, int type)
{
   int fid = kopen(name, O_RDONLY, 0644);
   kread_generic(fid, s, (size_t) 1, type);
   kclose(fid);
}

void fill_out_float_vec(FloatVec_p p, int num)
{
   int        i;
   
   p->f.f_val = (float *) kmalloc(sizeof(float) * num);
   p->f.f_len = num;

   for (i = 0; i < num; i++)
      p->f.f_val[i] = i;

   return;
}



/*-- float_vec routines --*/
FloatVec_p create_float_vec(int num)
{
   FloatVec_p p;

   p = (FloatVec_p) kmalloc(sizeof(FloatVec_t));
   
   fill_out_float_vec(p, num);
   
   return p;
}

int compare_float_vec(FloatVec_p p1, FloatVec_p p2)
{
   int i;
   
   if (p1 == NULL && p2 == NULL)
      return TRUE;

   if (p1 == NULL || p2 == NULL)
      return FALSE;

   if (p1->f.f_len != p2->f.f_len)
      return FALSE;

   for (i = 0; i < p1->f.f_len; i++)
      if (p1->f.f_val[i] != p2->f.f_val[i])
	 return FALSE;
   
   return TRUE;
}

void
main()
{
   Container_t *oc = NULL;
   Container_t *ic = NULL;

   /*-- this should recursively define everything we need --*/
   kdefine_Container_p();

   /*-- initialize pixel --*/
   oc = (Container_t *) kmalloc(sizeof(Container_t));
   
   fill_out_float_vec(&oc->a, 10);
   fill_out_float_vec(&oc->b, 5);
   
   oc->c = create_float_vec(15);
   oc->d = create_float_vec(20);

   write_structure("./t.data", (kaddr) &oc, KTYPE_CONTAINER_P);
   read_structure("./t.data", (kaddr) &ic, KTYPE_CONTAINER_P);

   if (oc == NULL || ic == NULL)
      kprintf("failed to read container!\n");

   if (compare_float_vec(&oc->a, &ic->a))
      kprintf("struct FloatVec field is identical!\n");
   else
      kprintf("struct FloatVec field is different!\n");

   if (compare_float_vec(&oc->b, &ic->b))
      kprintf("FloatVec_t field is identical!\n");
   else
      kprintf("FloatVec_t field is different!\n");

   if (compare_float_vec(oc->c, ic->c))
      kprintf("FloatVec_t * field is identical!\n");
   else
      kprintf("FloatVec_t * field is different!\n");

   if (compare_float_vec(oc->d, ic->d))
      kprintf("FloatVec_p field is identical!\n");
   else
      kprintf("FloatVec_p field is different!\n");

   if (!kstruct_compare(&oc, &ic, KTYPE_CONTAINER_P))
      kprintf("flaky compare for Container_t!\n");

   /*-- tidy up --*/
   kstruct_free(oc, KTYPE_CONTAINER_T, TRUE);
   kstruct_free(ic, KTYPE_CONTAINER_T, TRUE);

   kexit(KEXIT_SUCCESS);
}
