#include <bootstrap.h>

/*
 * test write and read the numbers 1-10 in every data type to every possible
 * destination architecture from our current source architecture.
 */

#define FAIL(error)               \
        {                         \
          kinfo(KFORCE,error);    \
          kexit(KEXIT_FAILURE);   \
        }

int machs[5] = { KMACH_LITTLE_ENDIAN_IEEE, 
		 KMACH_LITTLE_ENDIAN_VAX,
		 KMACH_LITTLE_ENDIAN_ALPHA,
		 KMACH_BIG_ENDIAN_IEEE,
		 KMACH_BIG_ENDIAN_CRAY };

#define FIXED_ARRAY {1,   2,   3,   4,   5,   6,   7,   8,   9,   10}
#define FLOAT_ARRAY {1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0}
#define NUM ((size_t) 10)

/*-- handy macro for comparing two arrays --*/
#define compare_array            \
      for (j = 0; j < NUM; j++)  \
	 if (a[j] != b[j])       \
	    FAIL("conversion failed!");

/*-- handy macros for printing arrays --*/
#define print_array               \
      kprintf("\n");              \
      for (j = 0; j < NUM; j++)   \
	 kprintf("%d ", b[j]);    \
      kprintf("\n"); 

#define print_float_array         \
      kprintf("\n");              \
      for (j = 0; j < NUM; j++)   \
	 kprintf("%g ", b[j]);    \
      kprintf("\n"); 

/*-- write out and read in some number of points --*/
void write_n_read(int mach, int datatype, size_t number, kaddr src, kaddr dst,
	          size_t num)
{
   int f;
   
   f = kopen("test.data", O_WRONLY | O_CREAT, 0666);
   kfile_setmachtype(f, mach); 
   kwrite_generic(f, src, number, datatype);
   kclose(f);

   f = kopen("test.data", O_RDONLY, 0666);
   kfile_setmachtype(f, mach); 
   kread_generic(f, dst, number, datatype);
   kclose(f);

   kunlink("test.data");

   return;
}

int main(
   int  argc,
   char *argv[])
{
/*   int f; */
   size_t i, j;

   kprintf("local mach %d\n", KMACH_LOCAL);

   /*-- byte test --*/
   kprintf("byte test... ");
   for (i = 0; i < 5; i++)
   {
      char a[NUM] = FIXED_ARRAY, b[NUM];

      write_n_read(machs[i], KBYTE, NUM, a, b, i);
      compare_array; /* print_array; */
   }
   kprintf("successful\n");

   /*-- short test --*/
   kprintf("short test... ");
   for (i = 0; i < 5; i++)
   {
      short a[NUM] = FIXED_ARRAY, b[NUM];

      write_n_read(machs[i], KSHORT, NUM, a, b, i);
      compare_array;  /* print_array; */
   }
   kprintf("successful\n");

   /*-- integer test --*/
   kprintf("integer test... ");
   for (i = 0; i < 5; i++)
   {
      int a[NUM] = FIXED_ARRAY, b[NUM];

      write_n_read(machs[i], KINT, NUM, a, b, i);
      compare_array; /* print_array; */
   }
   kprintf("successful\n");
   
   /*-- long test --*/
   kprintf("long test... ");
   for (i = 0; i < 5; i++)
   {
      long a[NUM] = FIXED_ARRAY, b[NUM];

      write_n_read(machs[i], KLONG, NUM, a, b, i);
      compare_array; /* print_array; */
   }
   kprintf("successful\n");

   /*-- floating point test --*/
   kprintf("float test... ");
   for (i = 0; i < 5; i++)
   {
      float a[NUM] = FLOAT_ARRAY, b[NUM];
      
      write_n_read(machs[i], KFLOAT, NUM, a, b, i);
      compare_array; /* print_float_array; */
   }
   kprintf("successful\n");

   /*-- double test --*/
   kprintf("double test... ");
   for (i = 0; i < 5; i++)
   {
      double a[NUM] = FLOAT_ARRAY, b[NUM];

      write_n_read(machs[i], KDOUBLE, NUM, a, b, i);
      compare_array; /* print_float_array; */
   }
   kprintf("successful\n");
   
   kexit(KEXIT_SUCCESS);
}
