 /*
  * Khoros: $Id: kimportasc.c,v 1.13 1996/07/05 02:02:04 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: kimportasc.c,v 1.13 1996/07/05 02:02:04 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for kimportasc
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "kimportasc.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Import ASCII Data File to Data Segment
|
|       Purpose: main program for kimportasc
|
|         Input:
|		char *clui_info->i1_file; {ASCII input data file containing new data segment}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->o_file; {output data object}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->i2_file; {data object input to insert the new segment into}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		int clui_info->so_int; {starting point in ASCII input file to start importing at}
|		int clui_info->so_flag; {TRUE if -so specified}
|
|		int clui_info->sf_int; {number of points in ASCII file to skip between reads}
|		int clui_info->sf_flag; {TRUE if -sf specified}
|
|		int clui_info->nr_int; {number of points in ASCII file to read between skips}
|		int clui_info->nr_flag; {TRUE if -nr specified}
|
|		char *clui_info->segment_string; {The name of the segment to insert the ASCII data into}
|		int   clui_info->segment_flag; {TRUE if -segment specified}
|
|		int clui_info->dim_int; {dimension of the segment}
|		int clui_info->dim_flag; {TRUE if -dim specified}
|
|		int   clui_info->type_list; {data type to store ascii data as}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|		M.I. group:
|		    int clui_info->wsize_int; {size of width dimension in index specified}
|		    int clui_info->wsize_flag; {TRUE if -wsize specified}
|
|		    int   clui_info->windex_list; {position in the index order for the width dimension}
|		    char *clui_info->windex_label;
|		    int   clui_info->windex_flag; {TRUE if -windex specified}
|
|		M.I. group:
|		    int clui_info->hsize_int; {size of height dimension in index specified}
|		    int clui_info->hsize_flag; {TRUE if -hsize specified}
|
|		    int   clui_info->hindex_list; {position in the index order for the height dimension}
|		    char *clui_info->hindex_label;
|		    int   clui_info->hindex_flag; {TRUE if -hindex specified}
|
|		M.I. group:
|		    int clui_info->dsize_int; {size of depth dimension in index specified}
|		    int clui_info->dsize_flag; {TRUE if -dsize specified}
|
|		    int   clui_info->dindex_list; {position in the index order for the depth dimension}
|		    char *clui_info->dindex_label;
|		    int   clui_info->dindex_flag; {TRUE if -dindex specified}
|
|		M.I. group:
|		    int clui_info->tsize_int; {size of time dimension in index specified}
|		    int clui_info->tsize_flag; {TRUE if -tsize specified}
|
|		    int   clui_info->tindex_list; {position in the index order for the time dimension}
|		    char *clui_info->tindex_label;
|		    int   clui_info->tindex_flag; {TRUE if -tindex specified}
|
|		M.I. group:
|		    int clui_info->esize_int; {size of elements dimension in index specified}
|		    int clui_info->esize_flag; {TRUE if -esize specified}
|
|		    int   clui_info->eindex_list; {position in the index order for the elements dimension}
|		    char *clui_info->eindex_label;
|		    int   clui_info->eindex_flag; {TRUE if -eindex specified}
|
|        Output:
|       Returns:
|
|    Written By: John M. Salas, Donna Koechner, Jeremy Worley & Becky Bishop
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	int     *size_vector = NULL;        /* size of output object */
        kobject source_viff = NULL; 	    /* Object to insert data into */
	kobject destination_object = NULL;  /* Output object */
	char    *prog = "main";		    /* String for error reporting */
	char	*rtn = "kimportasc";	    /* String for error reporting */
	char	**tmp_index_string;	    /* Temporary for index order */
	char	index_string[50];	    /* Index order */
        int	i;			    /* Loop variable */
        int	dimension;		    /* Appropriate dimension */
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "DATAMANIP");
	kexit_handler(kimportasc_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "DATAMANIP", "kimportasc",
		kimportasc_usage_additions, kimportasc_get_args,
		kimportasc_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	kpds_init();

        /* Instantiate objects.  */
        if (clui_info->i2_flag &&
	     (source_viff = kdms_open(clui_info->i2_file, KOBJ_READ)) == NULL) 
	{
           kerror(rtn, prog, "Failed to open VIFF input object");
           kexit(KEXIT_FAILURE);
        }

        if ((destination_object = kdms_open(clui_info->o_file,
                                            KOBJ_WRITE)) == NULL) 
        {
           kerror(rtn, prog, "Failed to open output object");
           kexit(KEXIT_FAILURE);
        }

        if (kstring_lower(clui_info->segment_string, 
			  clui_info->segment_string) == NULL)
        {
	   kerror(rtn, prog, "Unable to allocate memory ");
	   kexit(KEXIT_FAILURE);
        }

	if (kstrcmp(clui_info->segment_string, "time") == 0)
        {
           if (clui_info->wsize_flag || clui_info->hsize_flag ||
	       clui_info->dsize_flag || clui_info->esize_flag)
	   {
	      kerror(rtn, prog, "When importing ASCII data to the time " 
				"segment, the data must be 1 dimensional and "
			        "'tsize' is the only size variable that may " 
				"be specified.");
	      kexit(KEXIT_FAILURE);
	   }
	   if (clui_info->tindex_list != 1)
	   {
	      kerror(rtn, prog, "When importing ASCII data to the time " 
			        "segment, the index order along time must be "
				"first.");
	      kexit(KEXIT_FAILURE);
	   }
	   /* Convert index order dimensions into strings */
           tmp_index_string = (char **) kmalloc(1* sizeof(char *));
           tmp_index_string[0] = kstrdup("t");
	   ksprintf(index_string, "%s", tmp_index_string[0]);

	   size_vector = (int *) kmalloc(1 * sizeof(int));
	   size_vector[0] = clui_info->tsize_int;
           dimension = 1;
        }

	else if (kstrcmp(clui_info->segment_string, "location") == 0)
        {
           if (clui_info->tsize_flag)
           {
	      kerror(rtn, prog, "When importing ASCII data to the location "
		           "segment, the data must be 4 dimensional and "
			   "'wsize', 'hsize', 'dsize', 'esize' are " 
			   "the only size variables that may be specified.");
	      kexit(KEXIT_FAILURE);
           }
           if (clui_info->windex_list == 5 || clui_info->hindex_list == 5 ||
	       clui_info->dindex_list == 5 || clui_info->eindex_list == 5)
           {
	      kerror(rtn, prog, "When importing ASCII data to the location "
		           "segment, the index order variables 'windex', " 
			   "'hindex', 'dindex', and 'eindex' "  
			   "must range from 1 to 4 and be unique");
	      kexit(KEXIT_FAILURE);
           }

	   /* Check to make sure that the index order dimensions are unique */
	   if (clui_info->windex_list == clui_info->hindex_list ||
	       clui_info->windex_list == clui_info->dindex_list ||
	       clui_info->windex_list == clui_info->eindex_list ||
	       clui_info->hindex_list == clui_info->dindex_list ||
	       clui_info->hindex_list == clui_info->eindex_list ||
	       clui_info->dindex_list == clui_info->eindex_list)
	   {
	      kerror(rtn, prog, "The index order for a dimension must be " 
			  "unique. One of the specified values is the same.");
	      kexit(KEXIT_FAILURE);
	   }

	   /* Convert index order dimensions into strings */
           tmp_index_string = (char **) kmalloc(4 * sizeof(char *));
	   tmp_index_string[clui_info->windex_list-1] = kstrdup("w");
	   tmp_index_string[clui_info->hindex_list-1] = kstrdup("h");
	   tmp_index_string[clui_info->dindex_list-1] = kstrdup("d");
	   tmp_index_string[clui_info->eindex_list-1] = kstrdup("n");
           ksprintf(index_string, "%s %s %s %s", 
	            tmp_index_string[0], tmp_index_string[1], 
		    tmp_index_string[2], tmp_index_string[3]);

	   size_vector = (int *) kmalloc(4 * sizeof(int));
	   size_vector[clui_info->windex_list-1] = clui_info->wsize_int;
	   size_vector[clui_info->hindex_list-1] = clui_info->hsize_int;
	   size_vector[clui_info->dindex_list-1] = clui_info->dsize_int;
	   size_vector[clui_info->eindex_list-1] = clui_info->esize_int;
           dimension = 4;
        }
        else
        {
	   /* Convert index order dimensions into strings */
	   tmp_index_string = (char **) kmalloc(5 * sizeof(char *));
           if (kstrcmp(clui_info->segment_string, "map") == 0)
           {
	      tmp_index_string[clui_info->windex_list-1] = kstrdup("mw");
	      tmp_index_string[clui_info->hindex_list-1] = kstrdup("mh");
	      tmp_index_string[clui_info->dindex_list-1] = kstrdup("md");
	      tmp_index_string[clui_info->tindex_list-1] = kstrdup("mt");
	      tmp_index_string[clui_info->eindex_list-1] = kstrdup("me");
           }
           else
           {
	      tmp_index_string[clui_info->windex_list-1] = kstrdup("w");
	      tmp_index_string[clui_info->hindex_list-1] = kstrdup("h");
	      tmp_index_string[clui_info->dindex_list-1] = kstrdup("d");
	      tmp_index_string[clui_info->tindex_list-1] = kstrdup("t");
	      tmp_index_string[clui_info->eindex_list-1] = kstrdup("e");
	   }
	   ksprintf(index_string, "%s %s %s %s %s", 
		 tmp_index_string[0], tmp_index_string[1], tmp_index_string[2],
		 tmp_index_string[3], tmp_index_string[4]);

	   /* Check to make sure that the index order dimensions are unique */
	   if (clui_info->windex_list == clui_info->hindex_list ||
               clui_info->windex_list == clui_info->dindex_list ||
               clui_info->windex_list == clui_info->tindex_list ||
               clui_info->windex_list == clui_info->eindex_list ||
               clui_info->hindex_list == clui_info->dindex_list ||
               clui_info->hindex_list == clui_info->tindex_list ||
               clui_info->hindex_list == clui_info->eindex_list ||
               clui_info->dindex_list == clui_info->tindex_list ||
               clui_info->dindex_list == clui_info->eindex_list ||
               clui_info->tindex_list == clui_info->eindex_list)
           {
              kerror(rtn, prog, "The index order for a dimension must be "
			  "unique.  One of the specified values is the same.");
              kexit(KEXIT_FAILURE);
           }

	   size_vector = (int *) kmalloc(5 * sizeof(int));
	   size_vector[clui_info->windex_list-1] = clui_info->wsize_int;
	   size_vector[clui_info->hindex_list-1] = clui_info->hsize_int;
	   size_vector[clui_info->dindex_list-1] = clui_info->dsize_int;
	   size_vector[clui_info->tindex_list-1] = clui_info->tsize_int;
	   size_vector[clui_info->eindex_list-1] = clui_info->esize_int;
           dimension = 5;
        }

        if (!clui_info->wsize_flag && !clui_info->hsize_flag && 
	    !clui_info->dsize_flag && !clui_info->tsize_flag)
        {
           kinfo(KSTANDARD, "You did not specify any size variables, "
			    "therefore the size of '%s' will be size '1'",
			    clui_info->o_file);
        }

        for (i = 0; i < dimension; i++)
           kfree(tmp_index_string[i]);

	kfree(tmp_index_string);
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!lkimportasc(clui_info->i1_file, source_viff,
			     clui_info->segment_string,
			     clui_info->dim_int,
			     index_string,
			     size_vector,
			     clui_info->type_label,
			     clui_info->so_int,
			     clui_info->sf_int, 
			     clui_info->nr_int,
		             destination_object)) 
	{
	   kfree_and_NULL(size_vector);
	   kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	kfree_and_NULL(size_vector);

        /* add history to the output object */
        if (!kpds_set_attribute(destination_object, 
                                KPDS_HISTORY, kpds_history_string()))
        {
           kerror(rtn, prog, 
                  "Unable to set history on the destination object");
           kexit(KEXIT_FAILURE);
        }
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: kimportasc_usage_additions
| 
|       Purpose: Prints usage additions in kimportasc_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kimportasc
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void kimportasc_usage_additions(void)
{
	kfprintf(kstderr, "\tImport ASCII Data File to Data Segment\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: kimportasc_free_args
| 
|       Purpose: Frees CLUI struct allocated in kimportasc_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname kimportasc
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
kimportasc_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->segment_string);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
