 /*
  * Khoros: $Id: lkgate.c,v 1.13 1996/07/05 02:10:52 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkgate.c,v 1.13 1996/07/05 02:10:52 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for kgate
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkgate
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/*-- disable common subexpression elimination optimization --*/
/*-- for centerline cc (clcc version: 1.1.0_940707) ---------*/
#ifdef __CLCC__
#pragma Off(Global_CSE)
#endif

void kgate_uchar    PROTO((long, double, int, unsigned char *, 
                           unsigned char *, unsigned char *, unsigned char *, 
			   unsigned char *));
void kgate_ulong    PROTO((long, unsigned long, int, unsigned long *, 
                           unsigned long *, unsigned char *, unsigned char *, 
			   unsigned long *));
void kgate_long     PROTO((long, long, int, long *, long *, 
			   unsigned char *, unsigned char *, long *));
void kgate_double   PROTO((long, double, int, double *, double *, 
			   unsigned char *, unsigned char *, double *));
void kgate_dcomplex PROTO((long, double, int, kdcomplex *, kdcomplex *, 
			   unsigned char *, unsigned char *, double *));
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkgate - based on gate data, destination value will be from source 1 or source 2
* 
*       Purpose: For each point in the source objects (src1_obj and 
*		 src2_obj), \fIlkgate\fP assigns to the destination 
*		 object (dest_obj) either the value from src1_obj or
*		 src2_obj, depending on the corresponding value in 
*		 the gating object (gate_obj), the specified gate 
*		 value (gate_value), and the gating logic (gate_logic).
*	         .IP "gate_value"
*		 \fIkgate\fP will compare the elements of gate_obj 
*		 against gate_value, and will gate based on whether the 
*		 values in gate_obj are equal/unequal to gate_value.  
*		 How the gating is applied depends on the gate_logic
*		 parameter, described below.
*		 .IP "gate_logic TRUE"
*		 When gate_logic is TRUE, the following rule applies:
*		 If the value in gate_obj equals gate_value, the result
*		 copied into dest_obj will be from src2_obj, otherwise 
*		 the result will be copied from src1_obj. 
*		 .IP "gate_logic FALSE"
*		 When gate_logic is FALSE, the following rule applies:
*		 If the value in gate_obj equals gate_value, the result
*		 copied into dest_obj will be from src1_obj, otherwise 
*		 the result will be copied from src2_obj. 
*		 .IP "use_subpos" 5
*		 If use_subpos is TRUE, then the subobject position of 
*		 each object will be applied.  This includes its use in 
*		 calculating the destination object size.  The subobject 
*		 position functionality is disabled if use_subpos is FALSE.
*		 .IP "Data Type" 5
*		 If the gate object is complex, only the real component 
*		 will be used.
*		 .IP
*		 .cI $DATAMANIP/repos/shared/man/sections/value_type_2input
*		 .IP "Map Data" 5
*		 .cI $DATAMANIP/repos/shared/man/sections/map_2input
*		 .IP "Validity Mask" 5
*		 If there is a validity mask associated with either 
*		 source object, the destination object mask value will 
*		 be assigned according to the same rules as apply to 
*		 gating the value data.
*		 .IP
*		 Important: The Gating object may contain mask data 
*		 \fIor\fP value data, but \fInot\fP both.  If mask data 
*		 is supplied in the gating object, it will be used only 
*		 as gate values, and will not be propagated as a mask
*		 to the output object.
*		 .IP "Source Objects of Different Sizes - Resizing & Padding" 5
*		 .cI $DATAMANIP/repos/shared/man/sections/resize_2input
*		 .IP
*		 \fBPadding\fP: If src1_obj and/or src2_obj are enlarged, 
*		 the value and/or map data will be zero padded; masks will 
*		 be padded with ones.  If the gating object is enlarged, 
*		 it will be padded with a value \fIunequal\fP to the gating 
*		 value.  
*		 .IP "Explicit Location and Time Data" 5
*		 .cI $DATAMANIP/repos/shared/man/sections/loc_and_time_2input
*		 .IP "Failure Modes" 5
*		 .cI $DATAMANIP/repos/shared/man/sections/fail_2input
*
*         Input: src1_obj   - First source object
*                src2_obj   - Second source object
*                gate_obj   - Gating object which determines the output
*		 gate_value - Value defining when to perform gating operation
*		 gate_logic - Gating logic: if gating element = gate_value, 
*			     (0) use src1_obj, (1) use src2_obj
*
*        Output: dest_obj   - Destination object which contains results
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Donna Koechner & Ashish Malhotra
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lkgate(
    kobject src1_obj,
    kobject src2_obj,
    kobject gate_obj,
    double  gate_value,
    int     gate_logic,
    int	    use_subpos,
    kobject dest_obj)
/* -library_def_end */

/* -library_code */
{
   klist         *objlist = NULL;
   char          *lib = "kdatamanip"; 
   char          *rtn = "lkgate";
   char          *region=NULL, *opt_region=NULL, *datatype=NULL; 
   char          *region_size=NULL, *interptype=NULL, *position=NULL; 
   char          *gate_region=NULL, *gate_datatype=NULL, *gate_interptype=NULL; 
   char          *gate_region_size=NULL, *gate_position=NULL; 
   char          *gate_padvalue=NULL; 
   kaddr         *data1 = NULL, *data2 = NULL, *gate_data = NULL;
   unsigned char *mask1 = NULL, *mask2 = NULL;
   int            w2 = 1, d2 = 1, h2 = 1, t2 = 1, e2 = 1;
   int            ptype;
   int            masktype, maptype, valuetype;
   int            upgrade_mapping, upgrade_type, upgrade_size;
   int            num, i, num_rgns;
   long           num_pts;
   int		  grid;
   int            src1_mask_exists = FALSE; 
   int            src2_mask_exists = FALSE; 
   int            gate_mask_exists = FALSE;
   int            process_map      = FALSE; 
   int            process_value    = FALSE; 
   int            process_mask     = FALSE; 
   int            transfer_loc     = FALSE; 
   int            transfer_time    = FALSE; 
   double 	  gate_pad;

        
   /*-------------------------------------------------------------------
   | INITIAL ERROR CHECKING
   -------------------------------------------------------------------*/
   /*-------------------------------------------------------------------
   | Check that the objects passed into lkgate are valid data objects.
   -------------------------------------------------------------------*/
   if ((src1_obj == KOBJECT_INVALID) || (src2_obj == KOBJECT_INVALID) ||
       (gate_obj == KOBJECT_INVALID) || (dest_obj == KOBJECT_INVALID))
   {
      kerror(lib, rtn, "An invalid object was passed into lkgate.");
      return FALSE;
   }

   if (kpds_query_mask(gate_obj) && kpds_query_value(gate_obj))
   {
      kerror(lib, rtn, "The gate object cannot have both value and mask data.");
      return FALSE;
   }

   /*-------------------------------------------------------------------
   | Check if sources have location and/or time. If source 1 does, 
   | transfer it to the destination.  Set up destination here, and copy 
   | the data later on (after setting up all of the other attributes).
   -------------------------------------------------------------------*/
   transfer_loc = kapu_multi_input_loc(&grid, 2, src1_obj, src2_obj);
   if (transfer_loc)
   {
      if (!kpds_query_location(dest_obj)) 
         KCALL(kpds_create_location(dest_obj));
      KCALL(kpds_copy_location_attr(src1_obj, dest_obj));
   }

   transfer_time = kapu_multi_input_time(2, src1_obj, src2_obj);
   if (transfer_time)
   {
      if (!kpds_query_time(dest_obj)) 
         KCALL(kpds_create_time(dest_obj));
      KCALL(kpds_copy_time_attr(src1_obj, dest_obj));
   }

   /*-------------------------------------------------------------------
   | Check if sources have mask data.
   -------------------------------------------------------------------*/
   src1_mask_exists = kpds_query_mask(src1_obj);
   src2_mask_exists = kpds_query_mask(src2_obj);
   gate_mask_exists = kpds_query_mask(gate_obj);

   /*-------------------------------------------------------------------
   | Reference the source and gating objects.
   -------------------------------------------------------------------*/
   KCALL((src1_obj = kpds_reference_object(src1_obj)) != KOBJECT_INVALID);
   objlist = klist_add(objlist, src1_obj, "KOBJECT");
 
   KCALL((src2_obj = kpds_reference_object(src2_obj)) != KOBJECT_INVALID);
   objlist = klist_add(objlist, src2_obj, "KOBJECT");
 
   KCALL((gate_obj = kpds_reference_object(gate_obj)) != KOBJECT_INVALID);
   objlist = klist_add(objlist, gate_obj, "KOBJECT");

   /*-------------------------------------------------------------------
   | Check that source objects can work together polymorphically.  Set 
   | them up have the same size, mapping, and data type.  
   | Gate object will currently affect size, data type, an whether map 
   | or value is processed.
   -------------------------------------------------------------------*/
   upgrade_mapping = TRUE;
   upgrade_type    = TRUE;
   upgrade_size    = TRUE;

   if (!kapu_upgrade_objects(&process_map, &process_value, &process_mask,
			     &maptype, &valuetype, &masktype, 
		             upgrade_mapping, upgrade_type, upgrade_size, 
			     use_subpos, 
			     3, src1_obj, src2_obj, gate_obj))
   {
      (void)klist_free(objlist, (kfunc_void)lkcall_free);
      return FALSE;
   }

   /*-------------------------------------------------------------------
   | If only the gate object has a mask, do not do normal mask processing.
   -------------------------------------------------------------------*/
   if (process_mask && ((!src1_mask_exists) && (!src2_mask_exists)))
      process_mask = FALSE;

   /*-------------------------------------------------------------------
   | Source objects must have value and/or map data
   -------------------------------------------------------------------*/
   if (!process_map && !process_value)
   {
      kerror(lib, rtn, "No Map or Value data exist for source objects");
      (void)klist_free(objlist, (kfunc_void)lkcall_free);
      return FALSE;
   }

   /*-------------------------------------------------------------------
   | Check and create segments in destination object if they do not
   | exist.  Otherwise, if data does not exist in source objects, but 
   | does exist in the destination, destroy it in the destination.
   -------------------------------------------------------------------*/
   if (process_map)
   {
      if (!kpds_query_map(dest_obj))  KCALL(kpds_create_map(dest_obj));
      KCALL(kpds_copy_attribute(src1_obj, dest_obj, KPDS_MAP_SIZE));
      KCALL(kpds_set_attribute(dest_obj, KPDS_MAP_DATA_TYPE, maptype));
   }
   else 
   {
      if (kpds_query_map(dest_obj))  KCALL(kpds_destroy_map(dest_obj));
      KCALL(kpds_copy_attribute(src1_obj, dest_obj, KPDS_SUBOBJECT_POSITION));
   }

   if (process_mask && (src1_mask_exists || src2_mask_exists))
   {
      if (!kpds_query_mask(dest_obj))  KCALL(kpds_create_mask(dest_obj));
      KCALL(kpds_set_attribute(dest_obj, KPDS_MASK_DATA_TYPE, masktype));
   }
   else 
      if (kpds_query_mask(dest_obj))  KCALL(kpds_destroy_mask(dest_obj));

   if (process_value)
   {
      if (!kpds_query_value(dest_obj))  KCALL(kpds_create_value(dest_obj));
      KCALL(kpds_copy_attribute(src1_obj, dest_obj, KPDS_VALUE_SIZE));
      KCALL(kpds_copy_attribute(src1_obj, dest_obj, KPDS_VALUE_DATA_TYPE));
      KCALL(kpds_set_attribute(dest_obj, KPDS_VALUE_DATA_TYPE, valuetype));
   }
   else 
      if (kpds_query_value(dest_obj))  KCALL(kpds_destroy_value(dest_obj));

/*
kpds_print_attribute(dest_obj, KPDS_MASK_SIZE, kstdout);
kinfo(KSTANDARD, " <-- dest_obj, KPDS_MASK_SIZE\n", kstdout);
*/

   /*-------------------------------------------------------------------
   | Set up the attribute names and values to pass into kpds_set_attribute 
   | calls.  If the source objects contain only map data, set the 
   | attribute names to be the map attribute names.  Otherwise, the 
   | attribute names will be those of value segment.  
   -------------------------------------------------------------------*/
   if (process_map)
   {
      region      = KPDS_MAP_REGION;
      region_size = KPDS_MAP_REGION_SIZE;
      opt_region  = KPDS_MAP_OPTIMAL_REGION_SIZE;
      datatype    = KPDS_MAP_DATA_TYPE;
      interptype  = KPDS_MAP_INTERPOLATE;
      position    = KPDS_MAP_POSITION;
      gate_padvalue    = KPDS_MAP_PAD_VALUE;
      gate_region      = KPDS_MAP_REGION;
      gate_region_size = KPDS_MAP_REGION_SIZE;
      gate_datatype    = KPDS_MAP_DATA_TYPE;
      gate_interptype  = KPDS_MAP_INTERPOLATE;
      gate_position    = KPDS_MAP_POSITION;
      ptype = maptype;
   }
   else 
   {
      region      = KPDS_VALUE_REGION;
      region_size = KPDS_VALUE_REGION_SIZE;
      opt_region  = KPDS_VALUE_OPTIMAL_REGION_SIZE;
      datatype    = KPDS_VALUE_DATA_TYPE;
      interptype  = KPDS_VALUE_INTERPOLATE;
      position    = KPDS_VALUE_POSITION;
      ptype = valuetype;
      if (gate_mask_exists)
      {
         gate_padvalue    = KPDS_MASK_PAD_VALUE;
         gate_region      = KPDS_MASK_REGION;
         gate_region_size = KPDS_MASK_REGION_SIZE;
         gate_datatype    = KPDS_MASK_DATA_TYPE;
         gate_interptype  = KPDS_MASK_INTERPOLATE;
         gate_position    = KPDS_MASK_POSITION;
      }
      else
      {
         gate_padvalue    = KPDS_VALUE_PAD_VALUE;
         gate_region      = KPDS_VALUE_REGION;
         gate_region_size = KPDS_VALUE_REGION_SIZE;
         gate_datatype    = KPDS_VALUE_DATA_TYPE;
         gate_interptype  = KPDS_VALUE_INTERPOLATE;
         gate_position    = KPDS_VALUE_POSITION;
      }
   }
 
   KCALL(kpds_set_attribute(src1_obj, position, 0, 0, 0, 0, 0));
   KCALL(kpds_set_attribute(src1_obj, interptype, KPAD));
 
   KCALL(kpds_set_attribute(src2_obj, position, 0, 0, 0, 0, 0));
   KCALL(kpds_set_attribute(src2_obj, interptype, KPAD));

   /*-------------------------------------------------------------------
   | Set the gate object pad value to a value not equal to the gating.
   | This will insure that unexpected gating does not occur.  (Need to
   | set the attribute explicitly if mask is being used because it 
   | takes only one argument.
   -------------------------------------------------------------------*/
   gate_pad = 1;
   if (gate_value != 0.0)
      gate_pad = 0;

   KCALL(kpds_set_attribute(gate_obj, gate_position, 0, 0, 0, 0, 0));
   KCALL(kpds_set_attribute(gate_obj, gate_interptype, KPAD));
   if (process_value && gate_mask_exists)
   {
      KCALL(kpds_set_attribute(gate_obj, gate_padvalue, gate_pad));
   }
   else
   {
      KCALL(kpds_set_attribute(gate_obj, gate_padvalue, gate_pad, gate_pad));
   }
   
   if (process_mask)
   {
      if (process_value && src1_mask_exists)
      {
         KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_DATA_TYPE, KUBYTE));
	 KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_INTERPOLATE, KPAD));
	 KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_PAD_VALUE, 1.0));
	 KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_POSITION, 0, 0, 0, 0, 0));
      }
      if (process_value && src2_mask_exists)
      {
         KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_DATA_TYPE, KUBYTE));
	 KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_INTERPOLATE, KPAD));
	 KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_PAD_VALUE, 1.0));
	 KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_POSITION, 0, 0, 0, 0, 0));
      }
   }

   /*-------------------------------------------------------------------
   | After setting all of the attributes that should be reflected in the 
   | pysically storage of the data, create a reference on the destination 
   | object.
   -------------------------------------------------------------------*/
   KCALL((dest_obj = kpds_reference_object(dest_obj))!= KOBJECT_INVALID);
   objlist = klist_add(objlist,dest_obj,"KOBJECT");
 
   /*-------------------------------------------------------------------
   | Decide which data type is best to process the data, and set all
   | datatype attributes to that (except src1 and sr2 masks, which are 
   | always UBYTE).  If the type comes back as double complex, set the 
   | gate data type to double.
   -------------------------------------------------------------------*/
   ptype = kdatatype_cast_process(ptype, ptype, 
			(KUBYTE | KLONG | KULONG | KDOUBLE | KDCOMPLEX));
   if (ptype == KDCOMPLEX)
   {
      KCALL(kpds_set_attribute(gate_obj, gate_datatype, KDOUBLE));
   }
   else
   {
      KCALL(kpds_set_attribute(gate_obj, gate_datatype, ptype));
   }

   KCALL(kpds_set_attribute(src1_obj, datatype, ptype));
   KCALL(kpds_set_attribute(src2_obj, datatype, ptype));
   KCALL(kpds_set_attribute(dest_obj, datatype, ptype));

   /*-------------------------------------------------------------------
   | Get the optimal region size for processing (using the first source
   | object as the master), and use this info to set the region size
   | attribute on all objects.
   -------------------------------------------------------------------*/
   KCALL(kpds_get_attribute(src1_obj, opt_region, &w2, &h2, &d2, &t2, &e2, 
		            &num_rgns));
   num_pts = w2*h2*d2*t2*e2;
   KCALL(kpds_set_attribute(src1_obj, region_size, w2,h2,d2,t2,e2));
   KCALL(kpds_set_attribute(src2_obj, region_size, w2,h2,d2,t2,e2));
   KCALL(kpds_set_attribute(gate_obj, gate_region_size, w2,h2,d2,t2,e2));
   KCALL(kpds_set_attribute(dest_obj, region_size, w2,h2,d2,t2,e2));
        
   if (process_value && src1_mask_exists)
   {
      KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_REGION_SIZE, 
			       w2,h2,d2,t2,e2));
      KCALL(kpds_set_attribute(src1_obj, KPDS_MASK_PAD_VALUE, 1.0));
   }
   if (process_value && src2_mask_exists)
   {
      KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_REGION_SIZE, 
			       w2,h2,d2,t2,e2));
      KCALL(kpds_set_attribute(src2_obj, KPDS_MASK_PAD_VALUE, 1.0));
   }
   if (process_value && (src1_mask_exists || src2_mask_exists))
   {
      KCALL(kpds_set_attribute(dest_obj, KPDS_MASK_REGION_SIZE, 
			       w2,h2,d2,t2,e2));
      KCALL(kpds_set_attribute(dest_obj, KPDS_MASK_DATA_TYPE, KUBYTE));
   }
 
   if (process_mask)
   {
      if ((mask1 = kmalloc((size_t)(num_pts * sizeof(unsigned char)))) 
		== NULL)
      {
         kerror(lib, rtn, "Mask 1 memory allocation failed.");
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }
      objlist = klist_add(objlist,mask1,"KMALLOC");

      if ((mask2 = kmalloc((size_t)(num_pts * sizeof(unsigned char)))) 
		== NULL)
      {
	 kerror(lib, rtn, "Mask 2 memory allocation failed.");
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }
      objlist = klist_add(objlist, mask2, "KMALLOC");
   }

   for (num=0; num < num_rgns; num++)
   {
      if ((data1 = kpds_get_data(src1_obj, region, data1)) == NULL)
      {
         kerror(lib, rtn, "Failed to get %s from src1_obj", region);
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }
      if (num == 0)
         objlist = klist_add(objlist, data1, "KMALLOC");

      if ((data2 = kpds_get_data(src2_obj, region, data2)) == NULL)
      {
         kerror(lib, rtn, "Failed to get %s from src2_obj", region);
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }
      if (num == 0)
         objlist = klist_add(objlist, data2, "KMALLOC");

      if ((gate_data = kpds_get_data(gate_obj, gate_region, gate_data)) == NULL)
      {
         kerror(lib, rtn, "Failed to get %s from gate_obj", region);
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }
      if (num == 0)
         objlist = klist_add(objlist, gate_data, "KMALLOC");
           
      /*----------------------------------------------------------------
      | Get mask data from the two source objects if masks exist.
      ----------------------------------------------------------------*/
      if (process_value && process_mask)
      {
         if (src1_mask_exists)
	 {
            if ((mask1 = (unsigned char *) kpds_get_data(src1_obj,
                             KPDS_MASK_REGION, (kaddr)mask1)) == NULL)
            {
               kerror(lib, rtn, "Failed to get mask region from src1_obj");
               (void)klist_free(objlist, (kfunc_void)lkcall_free);
               return FALSE;
            }
	 }
	 else
	 {
	    for (i=0; i< num_pts; i++)
	       mask1[i] = 1;
	 }

	 if (src2_mask_exists)
	 {
            if ((mask2 = (unsigned char *) kpds_get_data(src2_obj,
                             KPDS_MASK_REGION, (kaddr)mask2)) == NULL)
            {
               kerror(lib, rtn, "Failed to get mask region from src2_obj");
               (void)klist_free(objlist, (kfunc_void)lkcall_free);
               return FALSE;
            }
         }
	 else
	 {
	    for (i=0; i< num_pts; i++)
	       mask2[i] = 1;
         }
      }

      /*----------------------------------------------------------------
      | The gating functions put the results in data1 and mask1 (if 
      | any mask exists) 
      ----------------------------------------------------------------*/
      if (ptype == KUBYTE)
      {
         kgate_uchar(num_pts, gate_value, gate_logic,
		     (unsigned char *) data1, (unsigned char *) data2, 
		     mask1, mask2, (unsigned char *) gate_data);
      }

      else if (ptype == KULONG)
      {
	 kgate_ulong(num_pts, (unsigned long) gate_value, gate_logic,
		     (unsigned long *) data1, (unsigned long *) data2, 
		     mask1, mask2, (unsigned long *) gate_data);
      }

      else if (ptype == KLONG)
      {
	 kgate_long(num_pts, (long) gate_value, gate_logic, (long *) data1, 
                    (long *) data2, mask1, mask2, (long *) gate_data);
      }

      else if (ptype == KDOUBLE)
      {
         kgate_double(num_pts, gate_value, gate_logic, (double *) data1, 
                      (double *) data2, mask1, mask2, (double *) gate_data);
      }

      else if (ptype == KDCOMPLEX)
      {
	 kgate_dcomplex(num_pts, gate_value, gate_logic, 
                        (kdcomplex *) data1, (kdcomplex *) data2, 
			mask1, mask2, (double *) gate_data);
      }

      else 
      {
         /*-------------------------------------------------------------
	 | unrecognized processing type - print error and get out 
         -------------------------------------------------------------*/
         kerror(lib, rtn, "unrecognized processing type (%d)",ptype);
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }

      if (! kpds_put_data(dest_obj, region, (kaddr) data1) )
      {
         kerror(lib, rtn, "Failed to put %s data in output object.", region);
         (void)klist_free(objlist, (kfunc_void)lkcall_free);
         return FALSE;
      }

      if (process_mask)
      {
         if (! kpds_put_data(dest_obj, KPDS_MASK_REGION, (kaddr) mask1) )
         {
            kerror(lib, rtn, "Failed to put mask data in output object.");
            (void)klist_free(objlist, (kfunc_void)lkcall_free);
            return FALSE;
         }
      }
   }

   /*-------------------------------------------------------------------
   | If source 1 has location or time data, transfer it to destination
   -------------------------------------------------------------------*/
   if (transfer_loc)
      KCALL(kpds_copy_location_data(src1_obj, dest_obj, TRUE));
 
   if (transfer_time)
      KCALL(kpds_copy_time_data(src1_obj, dest_obj, TRUE));
 
   (void)klist_free(objlist, (kfunc_void)lkcall_free);
   return TRUE;

}

/*-----------------------------------------------------------
|
|  Routine Name: kgate_uchar - gating for unsigned char data
|
|       Purpose: This function replaces values in the data1 and 
|		 mask1 arrays with those in the data2 and mask2
|		 arrays when the value in gate_data is equal to 
|		 gate_val, and gate_logic is true.  If gate_locic 
|		 is false, then data1 and mask1 values are replaced 
|		 when the gate_data is not equal to gate_val.
|                It is assumed that if either mask1 or mask2 is 
|                passed in, both masks are passed in.  
|
|         Input: numpts - number of points in data, gate, and mask arrays
|                gate_val - value defining when to perform gating operation
|		 gate_logic - gating logic: if gating element = gate_val, 
|		     	      (0) use src1_obj, (1) use src2_obj
|		 data1 - first data array
|		 data2 - second data array
|		 mask1 - mask associated with first data array
|		 mask2 - mask associated with second data array
|                gate_data - gating data array
|
|        Output: data1 - data results are stored in data1
|		 mask1 - if a mask exists, results are stored in mask1
|       Returns: nothing
|
|    Written By: Donna Koechner
|          Date: Thu Apr 28 1994
| Modifications:
|
------------------------------------------------------------*/

void kgate_uchar(
   long           numpts, 
   double         gate_val, 
   int            gate_logic,
   unsigned char *data1, 
   unsigned char *data2, 
   unsigned char *mask1, 
   unsigned char *mask2, 
   unsigned char *gate_data )
{
   int j;
   
   /* gate_logic = 1:  if gating element = gate_val, use src2_obj */
   if (gate_logic == 1) 
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] == gate_val)
	 {
	    data1[j] = data2[j];
	    if (mask1)
	       mask1[j] = mask2[j];
	 }
      }
   }

   /* gate_logic = 0:  if gating element != gate_val, use src2_obj */
   else
   {
      for (j = 0; j < numpts; j++)
      {
	 if (gate_data[j] != gate_val)
	 {
	    data1[j] = data2[j];
	    if (mask1)
	       mask1[j] = mask2[j];
         }
      }
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kgate_ulong - gating for unsigned long data
|
|       Purpose: This function replaces values in the data1 and 
|		 mask1 arrays with those in the data2 and mask2
|		 arrays when the value in gate_data is equal to 
|		 gate_val, and gate_logic is true.  If gate_locic 
|		 is false, then data1 and mask1 values are replaced 
|		 when the gate_data is not equal to gate_val.
|                It is assumed that if either mask1 or mask2 is 
|                passed in, both masks are passed in.  
|
|         Input: numpts - number of points in data, gate, and mask arrays
|                gate_val - value defining when to perform gating operation
|		 gate_logic - gating logic: if gating element = gate_val, 
|		     	      (0) use src1_obj, (1) use src2_obj
|		 data1 - first data array
|		 data2 - second data array
|		 mask1 - mask associated with first data array
|		 mask2 - mask associated with second data array
|                gate_data - gating data array
|
|        Output: data1 - data results are stored in data1
|		 mask1 - if a mask exists, results are stored in mask1
|       Returns: nothing
|
|    Written By: Donna Koechner
|          Date: Thu Apr 28 1994
| Modifications:
|
------------------------------------------------------------*/

void kgate_ulong(
   long           numpts, 
   unsigned long  gate_val, 
   int            gate_logic,
   unsigned long *data1, 
   unsigned long *data2, 
   unsigned char *mask1, 
   unsigned char *mask2, 
   unsigned long *gate_data)
{
   int j;

   /* gate_logic = 1:  if gating element = gate_val, use src2_obj */
   if (gate_logic == 1) 
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] == gate_val)
	 {
	    data1[j] = data2[j];
	    if (mask1)
	       mask1[j] = mask2[j];
	 }
      }
   }

   /* gate_logic = 0:  if gating element != gate_val, use src2_obj */
   else
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] != gate_val)
         {
	    data1[j] = data2[j];
	    if (mask1)
	       mask1[j] = mask2[j];
         }
      }
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kgate_long - gating for long data
|
|       Purpose: This function replaces values in the data1 and 
|		 mask1 arrays with those in the data2 and mask2
|		 arrays when the value in gate_data is equal to 
|		 gate_val, and gate_logic is true.  If gate_locic 
|		 is false, then data1 and mask1 values are replaced 
|		 when the gate_data is not equal to gate_val.
|                It is assumed that if either mask1 or mask2 is 
|                passed in, both masks are passed in.  
|
|         Input: numpts - number of points in data, gate, and mask arrays
|                gate_val - value defining when to perform gating operation
|		 gate_logic - gating logic: if gating element = gate_val, 
|		     	      (0) use src1_obj, (1) use src2_obj
|		 data1 - first data array
|		 data2 - second data array
|		 mask1 - mask associated with first data array
|		 mask2 - mask associated with second data array
|                gate_data - gating data array
|
|        Output: data1 - data results are stored in data1
|		 mask1 - if a mask exists, results are stored in mask1
|       Returns: nothing
|
|    Written By: Donna Koechner
|          Date: Thu Apr 28 1994
| Modifications:
|
------------------------------------------------------------*/

void kgate_long(
   long           numpts, 
   long           gate_val, 
   int            gate_logic,
   long          *data1, 
   long          *data2, 
   unsigned char *mask1, 
   unsigned char *mask2, 
   long          *gate_data)
{
   int j;

   /* gate_logic = 1:  if gating element = gate_val, use src2_obj */
   if (gate_logic == 1) 
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] == gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }

   /* gate_logic = 0:  if gating element != gate_val, use src2_obj */
   else
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] != gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kgate_double - gating for double data
|
|       Purpose: This function replaces values in the data1 and 
|		 mask1 arrays with those in the data2 and mask2
|		 arrays when the value in gate_data is equal to 
|		 gate_val, and gate_logic is true.  If gate_locic 
|		 is false, then data1 and mask1 values are replaced 
|		 when the gate_data is not equal to gate_val.
|                It is assumed that if either mask1 or mask2 is 
|                passed in, both masks are passed in.  
|
|         Input: numpts - number of points in data, gate, and mask arrays
|                gate_val - value defining when to perform gating operation
|		 gate_logic - gating logic: if gating element = gate_val, 
|		     	      (0) use src1_obj, (1) use src2_obj
|		 data1 - first data array
|		 data2 - second data array
|		 mask1 - mask associated with first data array
|		 mask2 - mask associated with second data array
|                gate_data - gating data array
|
|        Output: data1 - data results are stored in data1
|		 mask1 - if a mask exists, results are stored in mask1
|       Returns: nothing
|
|    Written By: Donna Koechner
|          Date: Thu Apr 28 1994
| Modifications:
|
------------------------------------------------------------*/

void kgate_double(
   long           numpts, 
   double         gate_val, 
   int            gate_logic,
   double        *data1, 
   double        *data2, 
   unsigned char *mask1, 
   unsigned char *mask2, 
   double        *gate_data)
{
   int j;

   /* gate_logic = 1:  if gating element = gate_val, use src2_obj */
   if (gate_logic == 1) 
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] == gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }

   /* gate_logic = 0:  if gating element != gate_val, use src2_obj */
   else
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] != gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }
}


/*-----------------------------------------------------------
|
|  Routine Name: kgate_dcomplex - gating for double complex data
|
|       Purpose: This function replaces values in the data1 and 
|		 mask1 arrays with those in the data2 and mask2
|		 arrays when the value in gate_data is equal to 
|		 gate_val, and gate_logic is true.  If gate_locic 
|		 is false, then data1 and mask1 values are replaced 
|		 when the gate_data is not equal to gate_val.
|                It is assumed that if either mask1 or mask2 is 
|                passed in, both masks are passed in.  
|
|         Input: numpts - number of points in data, gate, and mask arrays
|                gate_val - value defining when to perform gating operation
|		 gate_logic - gating logic: if gating element = gate_val, 
|		     	      (0) use src1_obj, (1) use src2_obj
|		 data1 - first data array
|		 data2 - second data array
|		 mask1 - mask associated with first data array
|		 mask2 - mask associated with second data array
|                gate_data - gating data array
|
|        Output: data1 - data results are stored in data1
|		 mask1 - if a mask exists, results are stored in mask1
|       Returns: nothing
|
|    Written By: Donna Koechner
|          Date: Thu Apr 28 1994
| Modifications:
|
------------------------------------------------------------*/

void kgate_dcomplex(
   long           numpts, 
   double         gate_val, 
   int            gate_logic,
   kdcomplex     *data1, 
   kdcomplex     *data2, 
   unsigned char *mask1, 
   unsigned char *mask2, 
   double        *gate_data)
{
   int j;

   /* gate_logic = 1:  if gating element = gate_val, use src2_obj */
   if (gate_logic == 1) 
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] == gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }

   /* gate_logic = 0:  if gating element != gate_val, use src2_obj */
   else
   {
      for (j = 0; j < numpts; j++)
      {
         if (gate_data[j] != gate_val)
         {
            data1[j] = data2[j];
            if (mask1)
               mask1[j] = mask2[j];
         }
      }
   }
}
/* -library_code_end */
