 /*
  * Khoros: $Id: lkhistops.c,v 1.15 1996/07/05 02:11:03 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkhistops.c,v 1.15 1996/07/05 02:11:03 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for khistops
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkhistops
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#include "internals.h"

#define KHISTOPS_EQUALIZE 1
#define KHISTOPS_STRETCH 2
#define KHISTOPS_INVERT 3
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkhistops - perform histogram equalization and stretching
* 
*       Purpose: Perform histogram equalization and stretching with
*		 optional inversion. Equalization occurs using the
*                standard algorithm. 
*
*                Stretching is done between the
*                most negative data value (mapping to the most negative
*                value possible for the input data storage type) and the most
*                positive data value (mapping to the most positive value
*                possible for the input data storage type).
*
*                No span limits are applied, as was the case in the K1.5
*                vhstr routine.
*
*         Input: in_obj - object to be processed
*		 function - a string containing either "equalize" or "stretch"
*		 invertflag - if non-zero, invert greylevels after processing
*		 w h d t e - flags that, if set, indicate that the processing 
*			unit for independent histograms should include the 
*			specified axis
*
*        Output: out_obj - output object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: KBYTE (8-bit), KUBYTE(8-bit), KSHORT(16-bit), KUSHORT(16-bit) data only
*    Written By: Scott Wilson
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 21-July-94 Scott Wilson - Fixed incorrect behavior
*		 caused by MAPPING_MODE being set after getting the
*		 size attributes of the input object.
****************************************************************/
/* -library_def */
int lkhistops(
  kobject in_obj,
  char *function,
  int invertflag,
  int w, int h, int d, int t, int e,
  kobject out_obj)
/* -library_def_end */

/* -library_code */
{
        char *lib = "kdatamanip", *rtn = "lkhistops";
        klist *objlist=NULL;
	long i,k;
	int mostneg,khistopfunc,type;
	long hist[65536],map[65536];
	unsigned int wc,hc,dc,tc,ec; /* Dimensions of data set */
        unsigned int suw,suh,sud,sut,sue,junk; /* Dimensions of subunit */
        unsigned int uw,uh,ud,ut,ue;           /* Dimensions of unit */
        unsigned int nuw,nuh,nud,nut,nue; /* Number of units in
                                             each direction */
        unsigned int nsw,nsh,nsd,nst,nse; /* Number of subunits per unit in
                                             each direction*/
        long maxcnt,lower,upper,n,map1[65536];
        long *b=NULL,*bmask=NULL;
        unsigned int numptsunit,numptssub;
        unsigned int numu_w,numu_h,numu_d,numu_t,numu_e; /* Unit counters */
        unsigned int nums_w,nums_h,nums_d,nums_t,nums_e; /* Subunit counters */
        unsigned int upos_w,upos_h,upos_d,upos_t,upos_e; /* Unit pos'n */
        unsigned int spos_w,spos_h,spos_d,spos_t,spos_e; /* Subunit pos'n */
        unsigned int wcon,hcon,dcon,tcon,econ; /* Optimal region constraints */
        unsigned int wflag,hflag,dflag,tflag,eflag; /* Optimal region flags */
	double scale,x;
        unsigned map_exists,mask_exists;

        /* Make sure we have a valid object */
        if (in_obj == KOBJECT_INVALID)
        {
           kerror(lib, rtn, "Bogus input object");
           return(FALSE);
        }

        /* Make sure the operation is known */
	if (kstrcmp(function,"equalize")==0) khistopfunc=KHISTOPS_EQUALIZE;
	else if (kstrcmp(function,"stretch")==0) khistopfunc=KHISTOPS_STRETCH;
	else
	  {
            kerror(lib, rtn, "Unsupported operation requested.");
            return(FALSE);
          }

        /* Reference the input object to avoid side effects */
        KCALL((in_obj = kpds_reference_object(in_obj)) != KOBJECT_INVALID);
        objlist = klist_add(objlist,in_obj,"KOBJECT");

        mask_exists=kpds_query_mask(in_obj);
        map_exists =kpds_query_map(in_obj);

        /* If map exists, pull all the data thru the map */
        if (map_exists)
          {
            KCALL(kpds_set_attribute(in_obj,KPDS_MAPPING_MODE,KMAPPED));
            KCALL(kpds_destroy_map(out_obj));
          }

        /* See how big the data is */
        KCALL(kpds_get_attributes(in_obj,
                                 KPDS_VALUE_SIZE,&wc,&hc,&dc,&tc,&ec,
                                 KPDS_VALUE_DATA_TYPE,&type,NULL));

        /* Make sure the data types are amenable to processing */
        if (type != KUBYTE && type != KBYTE && type  != KUSHORT &&
            type != KSHORT)
        {
           kerror(lib, rtn,"Histogram operations available for 8 and 16 bit data only");
	   (void)klist_free(objlist, (kfunc_void)lkcall_free);
           return(FALSE);
        }

        /* Make sure we don't hit a machine that has 32 bit bytes! */
        if (((type == KUBYTE || type== KBYTE) && sizeof(char) != 1) ||
            ((type == KUSHORT || type== KSHORT) && sizeof(short) != 2))
        {
           kerror(lib, rtn,"Histogram operations allowed on data occupying 16 bits or less");
	   (void)klist_free(objlist, (kfunc_void)lkcall_free);
           return(FALSE);
        }

        /* Create the output value segment if it's not there already and
           set it's size and data type attributes.  */
        if (!kpds_query_value(out_obj)) KCALL(kpds_create_value(out_obj));
        KCALL(kpds_set_attributes(out_obj,
                                 KPDS_VALUE_SIZE,wc,hc,dc,tc,ec,
                                 KPDS_VALUE_DATA_TYPE,type,NULL));

        /* Constrain the input data to long int type for reading */
        KCALL(kpds_set_attribute(in_obj, KPDS_VALUE_DATA_TYPE, KLONG, NULL));
        if (mask_exists)
          {
            KCALL(kpds_set_attribute(in_obj, KPDS_MASK_DATA_TYPE, KLONG, NULL));
          }

        /* Ref the output object to avoid side effects and proceed to
           constrain it to long int for writing */
        KCALL((out_obj = kpds_reference_object(out_obj)) != KOBJECT_INVALID);
        objlist = klist_add(objlist,out_obj,"KOBJECT");
        KCALL(kpds_set_attribute(out_obj, KPDS_VALUE_DATA_TYPE, KLONG, NULL));

	/* The task now is to figure out how to access the data so we can handle
	   large data sets on the processing unit spceified by the user. */

        /* If NO flags are set, set to WxH for default histogramming unit */
        if (w+h+d+t+e == 0) w=h=1;
 
        /* What we need to do is first figure out how many histogram units
           there are. This is determined only by the size of the data and the
           unit directions specified by the user. While we're at it, get
           the dimensions of each unit and the number of such units in
           each direction of the dataset. */
        numptsunit = 1;
        numptsunit *= (uw = w ? wc : 1);
        numptsunit *= (uh = h ? hc : 1);
        numptsunit *= (ud = d ? dc : 1);
        numptsunit *= (ue = t ? tc : 1);
        numptsunit *= (ut = e ? ec : 1);
        nuw = w ? 1 : wc;
        nuh = h ? 1 : hc;
        nud = d ? 1 : dc;
        nut = t ? 1 : tc;
        nue = e ? 1 : ec;
 
        /* Now we need to figure out the size and number of "optimal regions"
           that form each histogram unit. We'll use the constrained
           optimal region size attribute to do this.
 
           First, set up constraints such that the optimal region will be of
           size 1 on each axis. */
        wcon  = hcon  = dcon  = tcon  = econ  = 1;
        wflag = hflag = dflag = tflag = eflag = KOPTIMAL_FIXED_VALUE;
        /* Now remove the constraint for each axis that forms a
           histogram unit. */
        if (w) wflag = KOPTIMAL_FACTOR;
        if (h) hflag = KOPTIMAL_FACTOR;
        if (d) dflag = KOPTIMAL_FACTOR;
        if (t) tflag = KOPTIMAL_FACTOR;
        if (e) eflag = KOPTIMAL_FACTOR;
 
        /* Use the constraint values and flags to pull back the optimal region
           size for a subunit. */
        KCALL(kpds_set_attribute(in_obj,KPDS_VALUE_OPTIMAL_REGION_FLAGS,
             wflag,hflag,dflag,tflag,eflag));
        KCALL(kpds_set_attribute(in_obj,KPDS_VALUE_OPTIMAL_REGION_CONSTRAINT,
              wcon,hcon,dcon,tcon,econ));
        KCALL(kpds_get_attributes(in_obj,
              KPDS_VALUE_OPTIMAL_REGION_SIZE,&suw,&suh,&sud,&sut,&sue,&junk,
                       NULL));
 
        /* Compute number of points per subunit, number of subunits per
           unit, and the number of subunits per unit. */
        nsw = uw/suw;
        nsh = uh/suh;
        nsd = ud/sud;
        nst = ut/sut;
        nse = ue/sue;
        numptssub = suw*suh*sud*sut*sue;

        KCALL(kpds_set_attribute(in_obj,
                                  KPDS_VALUE_REGION_SIZE,suw,suh,sud,sut,sue));
        KCALL(kpds_set_attribute(out_obj,
                                  KPDS_VALUE_REGION_SIZE,suw,suh,sud,sut,sue));

        /* If a mask is in place set the region size attributes appropriately */
	if (mask_exists)
          {
            KCALL(kpds_set_attribute(in_obj,
                                  KPDS_MASK_REGION_SIZE,suw,suh,sud,sut,sue));
            KCALL(kpds_set_attribute(out_obj,
                                  KPDS_MASK_REGION_SIZE,suw,suh,sud,sut,sue));
          }

        /* Set up offset and range for the various data types */
	mostneg=0; maxcnt=256; /* Make lint shut up */
	if (type == KBYTE) { mostneg=-128; maxcnt=256; }
	else if (type == KUBYTE) { mostneg=0; maxcnt=256; }
	else if (type == KSHORT) { mostneg=-32768; maxcnt=65536; }
	else if (type == KUSHORT) { mostneg=0; maxcnt=65536; }
        else
          {
            kerror(lib, rtn, "Unsupported data type, %d.",type);
            (void)klist_free(objlist, (kfunc_void)lkcall_free);
            return(FALSE);
          }

        /* Allocate space for the various data buffers */
        KCALL(!((b = (long *)kmalloc((size_t)(suw*suh*sud*sut*sue*sizeof(long)))) == NULL));
        KCALL(!((bmask = (long *)kmalloc((size_t)(suw*suh*sud*sut*sue*sizeof(long)))) == NULL));
        objlist = klist_add(objlist,b,"KMALLOC");
        objlist = klist_add(objlist,bmask,"KMALLOC");

        /* Handle each unit in WHDTE block order */
        for (numu_e=0; numu_e<nue; numu_e++)
          {
            for (numu_t=0; numu_t<nut; numu_t++)
              {
                for (numu_d=0; numu_d<nud; numu_d++)
                  {
                    for (numu_h=0; numu_h<nuh; numu_h++)
                      {
                        for (numu_w=0; numu_w<nuw; numu_w++)
                          {

                            /* Zero the histo array */
	                    (void)kbzero((kaddr)hist,(size_t)maxcnt*sizeof(long));

                            /* Compute the offset into the data of the origin of each unit */
                            upos_w = numu_w*uw;
                            upos_h = numu_h*uh;
                            upos_d = numu_d*ud;
                            upos_t = numu_t*ut;
                            upos_e = numu_e*ue;
 
                            /* Step through each of the subunits in WHDTE block order */
                            for (nums_e=0; nums_e<nse; nums_e++)
                              {
                                for (nums_t=0; nums_t<nst; nums_t++)
                                  {
                                    for (nums_d=0; nums_d<nsd; nums_d++)
                                      {
                                        for (nums_h=0; nums_h<nsh; nums_h++)
                                          {
                                            for (nums_w=0; nums_w<nsw; nums_w++)
                                              {
                                                /* Compute the offset into the data of the origin of each subunit,
                                                   a relative computation that used the unit position as a reference. */
                                                spos_w = upos_w + nums_w*suw;
                                                spos_h = upos_h + nums_h*suh;
                                                spos_d = upos_d + nums_d*sud;
                                                spos_t = upos_t + nums_t*sut;
                                                spos_e = upos_e + nums_e*sue;
 
                                                /* Set the position on the input object to the location of the subunit
                                                   origin. This guarantees that we get the *right* subunit, not simply
                                                   the next one that the autoincrement code thinks we should get (which
                                                   is normally the next one along the lowest index order. */
                                                KCALL(kpds_set_attribute(in_obj,KPDS_VALUE_POSITION,spos_w,spos_h,spos_d,spos_t,spos_e));
                                                if (mask_exists)
                                                  KCALL(kpds_set_attribute(in_obj,KPDS_MASK_POSITION,spos_w,spos_h,spos_d,spos_t,spos_e));
 
                                                /* Grab a region of goodies from the input object */
                                                KCALL(kpds_get_data(in_obj,KPDS_VALUE_REGION, (kaddr)b));
                                                if (mask_exists) KCALL(kpds_get_data(in_obj,KPDS_MASK_REGION,(kaddr)bmask));

                                                /* Form the histogram */
                                                if (!mask_exists)
                                                  {
                                                    for (i=0; i<numptssub; i++)
                                                      {
                                                        n = b[i]-mostneg;
                                                        hist[n]++;
                                                      }
                                                  }
                                                else
                                                  {
                                                    for (i=0; i<numptssub; i++)
                                                      {
                                                        if (bmask[i] != 0)
                                                          {
                                                            n = b[i]-mostneg;
                                                            hist[n]++;
                                                          }
			                  	      }
			                          }

                                              }
                                          }
                                      }
                                  }
                              } /* End of subunits loops */
           

                            /* Have now obtained the histogram for the unit, so compute a map that 
                               accomplishes the desired histogram operation. */
                            switch (khistopfunc)
	                      {
		                case KHISTOPS_EQUALIZE:

           		          /* Integrate to form equalization map */
	   		          k=0;
	  	 	          for (i=0; i<maxcnt; i++)
             	    	            {
             		              k += hist[i];
               		              map[i] = k;
             		            }

           		          /* Scale the map back to 0..maxcnt-1 to make use of the whole
                                     range of the data type, then compute the new value in the
                                     input data type that corresponds to the bin number.  This
				     means that it may be offset by the most negative number
                                     representable in that data type. */
           		          if (map[maxcnt-1] == 0)
            		            {
            		              kerror(lib, rtn, "Cumulative distribution is zero.");
                                      (void)klist_free(objlist, (kfunc_void)lkcall_free);
            		              return(FALSE);
           		            }
           		          scale=(double)(maxcnt-1)/(double)map[maxcnt-1];
           		          for (i=0; i<maxcnt; i++)
                                    {
                                      x = map[i]*scale+mostneg;
                                      map1[i] = ktrunc(x+0.5);
                                    }
			          break;
		                case KHISTOPS_STRETCH:
			          /* Locate upper and lower bounds */
		  	          lower=-1; upper=-1;
			          for (i=0; i<maxcnt; i++)
			            {
			              if (hist[i] != 0 && lower==-1) lower=i;
                                      if (lower != -1) break;
                                    }
                                  for (i=maxcnt-1; i>=0; i--)
                                    {
                                      if (hist[i] != 0 && upper==-1) upper=i;
                                      if (upper != -1) break;
                                    }
                                  if (lower == upper) upper=lower+1;
			          for (i=lower; i<=upper; i++) 
                                  map1[i] = ktrunc((double)mostneg+(double)(maxcnt-1)*
                                      (double)(i-lower)/(double)(upper-lower));
		     	          break;
                                default:
                                  kerror(lib, rtn, "Unknown operation");
                                  (void)klist_free(objlist, (kfunc_void)lkcall_free);
                                  return(FALSE);
	                      }

                            /* Invert the map if invert after processing is requested */
	                    if (invertflag) for (i=0; i<maxcnt; i++) map1[i]=(maxcnt-1)-map1[i];

                            /* Remap the data */

                            /* Step through each of the subunits in WHDTE block order */
                            for (nums_e=0; nums_e<nse; nums_e++)
                              {
                                for (nums_t=0; nums_t<nst; nums_t++)
                                  {
                                    for (nums_d=0; nums_d<nsd; nums_d++)
                                      {
                                        for (nums_h=0; nums_h<nsh; nums_h++)
                                          {
                                            for (nums_w=0; nums_w<nsw; nums_w++)
                                              {
                                                /* Compute the offset into the data of the origin of each subunit,
                                                   a relative computation that used the unit position as a reference. */
                                                spos_w = upos_w + nums_w*suw;
                                                spos_h = upos_h + nums_h*suh;
                                                spos_d = upos_d + nums_d*sud;
                                                spos_t = upos_t + nums_t*sut;
                                                spos_e = upos_e + nums_e*sue;
 
                                                /* Set the position on the input object to the location of the subunit
                                                   origin. This guarantees that we get the *right* subunit, not simply
                                                   the next one that the autoincrement code thinks we should get (which
                                                   is normally the next one along the lowest index order. */
                                                KCALL(kpds_set_attribute(in_obj,KPDS_VALUE_POSITION,spos_w,spos_h,spos_d,spos_t,spos_e));
                                                if (mask_exists)
                                                  KCALL(kpds_set_attribute(in_obj,KPDS_MASK_POSITION,spos_w,spos_h,spos_d,spos_t,spos_e));
 
                                                /* Grab a region of goodies from the input object */
                                                KCALL(kpds_get_data(in_obj,KPDS_VALUE_REGION, (kaddr)b));
                                                if (mask_exists) KCALL(kpds_get_data(in_obj,KPDS_MASK_REGION,(kaddr)bmask));

                                                if (!mask_exists)
                                                  {
                                                    for (i=0; i<numptssub; i++)
                                                      {
                                                        n=b[i]-mostneg;
                                                        b[i]=map1[n];
                                                      }
                                                  }
                                                else
                                                  {
                                                    for (i=0; i<numptssub; i++)
                                                      {
                                                        if (bmask[i] != 0)
                                                          {
                                                            n=b[i]-mostneg;
                                                            b[i]=map1[n];
                                                          }
                                                      }
                                                  }

                                                /* Write the newly mapped data to the corresponding subunit in
                                                   the output object. */
                                                KCALL(kpds_set_attribute(out_obj,KPDS_VALUE_POSITION,spos_w,spos_h,spos_d,spos_t,spos_e));
                                                KCALL(kpds_put_data(out_obj, KPDS_VALUE_REGION, (kaddr) b));
                                              }
                                          }
                                      }
                                  }
                              } /* End of subunits loops */
                          }
                      }
                  }
              }
          } /* End of units loop */

        (void)klist_free(objlist, (kfunc_void)lkcall_free);

	return(TRUE);
}
/* -library_code_end */
