 /*
  * Khoros: $Id: lkimportasc.c,v 1.12 1996/07/05 02:12:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lkimportasc.c,v 1.12 1996/07/05 02:12:14 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for kimportasc
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lkimportasc
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lkimportasc - import ASCII data into specified segment of data object
* 
*       Purpose: This routine will import an ASCII data file,
*		 ascii_file, and insert it into a data
*		 object, source_obj, if it is specified.
*		 If there is no source object to insert
*		 the new segment into, pass NULL for source_obj and
*		 the data object specified by the parameter 
*		 destination_obj, will be created.
*		 The segment specified by segment_name will be
*		 inserted into this data object.  If a source_obj
*		 was specified, then all of its data segments and
*		 the global object attributes will be copied to 
*		 the destination.
*
*		 If the segment_name is one of the Polymorphic
*		 segments, the correct dimensionality must be provided :
*
*                .TS H
*                center tab(:) ;
*                lfB  lfB  
*                lf(CW) l .
*        	 Segment Name:Dimension
*                =
*                .TH
*        	 KDMS_SEGMENT_VALUE     :  5
*        	 KDMS_SEGMENT_MAP       :  5
*        	 KDMS_SEGMENT_MASK      :  5
*        	 KDMS_SEGMENT_LOCATION  :  4
*        	 KDMS_SEGMENT_TIME      :  1
*		 .TE
*
*		 There are four attributes of the new segment that
*		 must be passed in:  dimension, index order, size and 
*		 data type.  The number of indices in the index order 
*		 and the sizes specified must be equal to the dimension.
*
*		 The parameter dimension must be greater than 0 and less 
*		 than or equal to 5.
*
*       	 The parameter index_order is a string that contains the
*		 indices of the segment separated by spaces, "elements
*		 width height depth time" or "e w h d t".  The only allowed
*		 values are :
*
*                .TS
*                center tab(:) ;
*                l l .
*        	 width (w)      :  mapwidth (mw)
*	 	 height (h)     :  mapheight (mh)
*        	 depth (d)      :  mapdepth (md)
*	 	 time (t)       :  maptime (mt)
*        	 elements (e)   :  mapelements (me)
*	 	 dimension (n)  :
*		 .TE
*
*                where the characters in the ()'s are accepted
*                abbreviations.
*
*	         The parameter, index_order, is used to specify the 
*		 orientation of how data is stored and retrieved.  For 
*		 example, if the index_order is specified as "w h d t e", 
*		 then data will be accessed along width first, followed by 
*		 height, depth, time, and elements.
*
*		 The parameter size_vector contains the size of the
*		 segment to create.  All the values must be > 1.
*
*		 The parameter data_type_string must contain one of
*		 the following strings to specify the data type of
*		 the segment:
*
*                .TS H
*                center tab(:) ;
*                lfB  lfB  lfB
*                l    l    lf(CW) .
*                Data Type:Abbreviation:Data Type
*                =
*                .TH
*                bit              :bi     : KBIT
*                byte             :by     : KBYTE
*                unsigned byte    :un by  : KUBYTE
*                ubyte            :uby    : KUBYTE
*                short            :sh     : KSHORT
*                unsigned short   :un sh  : KUSHORT
*                ushort           :ush    : KUSHORT
*                integer          :in     : KINT
*                unsigned integer :un in  : KUINT
*                uint             :ui     : KUINT
*                long             :lon    : KLONG
*                unsigned long    :un lon : KULONG
*                ulong            :ul     : KULONG
*                float            :fl     : KFLOAT
*                double           :do     : KDOUBLE
*                complex          :co     : KCOMPLEX
*                double complex   :do co  : KDCOMPLEX
*                dcomplex         :dc     : KDCOMPLEX
*                .TE
*
*		 The parameter start_offset is the point in the ASCII
*		 source at which to start importing in the ASCII data.
*		 All points before this one are ignored.  When start_offset
*		 is something other than zero, the offset will be with
*		 respect to the index order specified.
*
*		 The parameter skip_factor is the number of points to
*		 skip in the ASCII data between reads.  
*
*		 The parameter num_reads specifies the number of data points
*		 to read before skipping.  For example, if num_reads is
*		 2 and skip factor is 3, two values will be read, three
*		 will be skipped, followed by two more reads, three skips,
*		 etc.
*
*         Input: ascii_file       - file name of ASCII file 
*                source_obj       - source data object to insert segment in
*                segment_name     - name of segment to create
*                dimension        - dimensionality of segment being created
*                index_order      - index order of segment being created
*                size_vector      - size of segment being created
*                data_type_string - data type of segment being created
*                start_offset     - starting point in ascii data to start from
*                skip_factor      - skip factor to use while importing ascii 
*				    data
*		 num_reads	  - number of points to read before skipping
*
*        Output: destination_obj - data object to create containing 
*			           new segment
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: John M. Salas, Donna Koechner, Jeremy Worley & Becky Bishop
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: Consider an ascii file which looks as follows:
*
*		 .begin code
*              ! 1 2 3 4
*	       ! 1 2 3 4	
*	       ! 1 2 3 4
*	       ! 1 2 3 4
*	       ! 1 2 3 4
*		 .end
*
*		 Suppose you want create (x,y) data from column 1 and
*		 column 3 such that the output object will look as follows:
*
*		 .begin code
*	       ! 1 3
*	       ! 1 3
*	       ! 1 3
*	       ! 1 3
*	       ! 1 3
*		 .end
*
*		 with size "2x5x1x1x1" with index order "w h d t e".  Also
*		 consider that the data will be stored in the value
*		 segment as integer data.
*
*		 In this case, you would call "lkimportasc" as follows:
*
*		 .begin code
*		! char *ascii_file_string = "ascii_input_data";
*		! char *prog = "main";
*		! char *rtn = "my_program";
*		! char *segment_string = "value";
*		! char *data_type_string = "integer";
*		! char *index_order_string = "w h d t e";
*		! int  dimension_int = 5;
*		! int  size_vector_int[5];
*		! int  offset_int = 0;
*		! int  skip_int = 1;
*		! int  n_reads_int = 1;
*
*		! size_vector_int[0] = 2;
*		! size_vector_int[1] = 5;
*		! size_vector_int[2] = 1;
*		! size_vector_int[3] = 1;
*		! size_vector_int[4] = 1;
*
*		! if (!lkimportasc(ascii_file_string,
*	        !	           segment_string,
*	        !	           dimension_int,
*	        !	           index_order_string,
*	        !	           size_vector_int,
*	        !	           data_type_string,
*	        !	           offset_int,
*	        !	           skip_int,
*	        !	           n_reads_int,
*	        !	           NULL))
*               !{
*	        !   kexit(KEXIT_FAILURE);
*               !}
*		 .end code
*
*		 Suppose you want import the above ascii data as a single
*		 column of data and specifically the second column.  In this
*		 case the data will be imported to the value segment as size 
*		 "1x5x1x1x1", with data type integer, and index order 
*		 "w h d t e".   In this case, the output data will look
*		 as follows:
*
*		 .begin code
*	       ! 2
*	       ! 2
*	       ! 2
*	       ! 2
*	       ! 2
*		 .end
*
*		 In this case, it is necessary to specify a start_offset of 1,
*		 a skip_factor of 3, and num_reads should be 1.  The following
*		 call of lkimportasc accomplishes this:
*
*		 .begin code
*		! int offset_int = 1;
*		! int skip_factor = 3;
*		! int n_reads_int = 1;
*
*		! if (!lkimportasc(ascii_file_string,
*	        !	           segment_string,
*	        !	           dimension_int,
*	        !	           index_order_string,
*	        !	           size_vector_int,
*	        !	           data_type_string,
*	        !	           offset_int,
*	        !	           skip_int,
*	        !	           n_reads_int,
*	        !	           NULL))
*               !{
*	        !   kexit(KEXIT_FAILURE);
*               !}
*		 .end
*
* Modifications: 
****************************************************************/
/* -library_def */
int 
lkimportasc(
   char   *ascii_file,
   kobject source_obj,
   char   *segment_name,
   int     dimension,
   char   *index_order,
   int    *size_vector,
   char   *data_type_string,
   int     start_offset,
   int     skip_factor,
   int     num_reads,
   kobject destination_obj)
/* -library_def_end */

/* -library_code */
{
   char   *lib = "ksegops";	/* String for use by KCALL & kerror */
   char   *rtn = "lkimportasc";	/* String for use by KCALL & kerror */
   klist  *objlist = NULL;	/* List of all kobjects & allocated pointers */
   char  **index_strings;	/* Strings for each index order axis */
   int    *index_order_tmp;	/* Variable for storing index order */
   int     i;			/* Looping variable */
   ssize_t  count;		/* Number of axes provided for index order */
   int     data_type;		/* Data type of output in integer form */
   int     success = FALSE;	/* Variable indicating success of routine */
   int    *begin = NULL;	/* Begin coordinate of ascii data */
   int    *end = NULL;		/* End coordinate of ascii data */
   int    *nbegin = NULL;	/* Begin coordinate of output object */
   int    *nend = NULL;		/* End coordinate of output object */
   int     size;		/* Represents total size of data */
   int    *orig_size_vector;	/* Used to contain original size of data */
   int     ascii_data_type;	/* Data type of ascii input */
   int    *ascii_size_vector;   /* Size of ascii input data */
   int     ascii_dimension;	/* Dimension of ascii input data */
   int    *ascii_index_order;   /* Index order of ascii input data */
   int     offset;		/* Variable used to track data offset */
   int     check_ascii_size;    /* Variable for checking valid ascii size */
   int     total_segment_size;  /* Variable for checking valid ascii size */
   int     reads_needed;        /* Variable for checking valid ascii size */
   kobject source_object1;	/* Input ascii data */
   kobject tmp_srcobject;	/* Used to hold temporary ascii data */
   kobject tmp_object;	        /* Used to hold temporary data */
   kobject reference_object;	/* Reference object to avoid side effects */
   kaddr   data = NULL;		/* Pointer to data */


   /*------------------------------------------------------------------------
   | Open the specified ASCII input file.  We will be using the ASCII 
   | transport available with data services to read this data, hence the 
   | kdms_open() call.
   --------------------------------------------------------------------------*/


   KCALL((source_object1 = kdms_open(ascii_file, KOBJ_READ)) != NULL);
   objlist = klist_add(objlist, source_object1, "KOBJECT");

   /* Check to see if an index order was provided */
   if (index_order == NULL)
   {
      kerror(lib, rtn, "No index order specified.");
      return (FALSE);
   }

   /* Check to see if a data type was provided */
   if (data_type_string == NULL)
   {
      kerror(lib, rtn, "No data type specified.");
      return (FALSE);
   }

   /* Check to see if a size was provided */
   if (size_vector == NULL)
   {
      kerror(lib, rtn, "No size specified.");
      return (FALSE);
   }

   /*------------------------------------------------------------------------
   | Check to make sure the dimensionality is reasonable for data services
   | i.e. 1 <= dimension <= 5.
   --------------------------------------------------------------------------*/
   if (dimension > 5 || dimension < 1)
   {
      kerror(lib, rtn, "Dimension (dimension) must be between 1 and 5.");
      return (FALSE);
   }

   /*------------------------------------------------------------------------
   | Check the start offset to make sure it isn't unreasonably small i.e
   | the offset must be greater than 0.
   --------------------------------------------------------------------------*/
   if (start_offset < 0)
   {
      kerror(lib, rtn, "Start offset (start_offset) must be > 0.");
      return (FALSE);
   }

   /*------------------------------------------------------------------------
   | Check the skip factor to make sure it isn't unreasonably small i.e.
   | the skip_factor must be greater than 0.
   --------------------------------------------------------------------------*/
   if (skip_factor < 0)
   {
      kerror(lib, rtn, "Skip factor (skip_factor) must be > 0.");
      return (FALSE);
   }


   /*------------------------------------------------------------------------
   | Check the given data segment name -- make sure that the specified
   | dimension is in-line with the expectcted dimension sizes for the
   | polymorphic segments.   If it's not a polymorphic segment, then
   | they are calling the shots and there's nothing we can check.
   --------------------------------------------------------------------------*/
   if (segment_name == NULL || kstrcmp(segment_name, "") == 0)
   {
      kerror(lib, rtn, "No segment specified (segment_name)");
      return (FALSE);
   }
   else
   {
      kstring_lower(segment_name, segment_name);
      if ((kstrcmp(KDMS_SEGMENT_VALUE, segment_name) == 0) && 
	   dimension != 5)
      {
	 kerror(lib, rtn, 
		"The value segment must be 5D, not '%d'D.", dimension);
	 return (FALSE);
      }
      else if ((kstrcmp(KDMS_SEGMENT_MASK, segment_name) == 0) && 
	        dimension != 5)
      {
	 kerror(lib, rtn, 
		"The mask segment must be 5D, not '%d'D.", dimension);
	 return (FALSE);
      }
      else if ((kstrcmp(KDMS_SEGMENT_MAP, segment_name) == 0) &&
	        dimension != 5)
      {
	 kerror(lib, rtn, "The map segment must be 5D, not '%d'D.", dimension);
	 return (FALSE);
      }
      else if ((kstrcmp(KDMS_SEGMENT_LOCATION, segment_name) == 0) &&
	        dimension != 4)
      {
	 kerror(lib, rtn, 
		"The location segment must be 4D, not '%d'D.", dimension);
	 return (FALSE);
      }
      else if ((kstrcmp(KDMS_SEGMENT_TIME, segment_name) == 0) &&
	        dimension != 1)
      {
	 kerror(lib, rtn, 
		"The time segment must be 1D, not '%d'D.", dimension);
	 return (FALSE);
      }


      /*---------------------------------------------------------------------
      | Break up the specified index order into separate index strings
      | using kparse_string_delimit().
      ----------------------------------------------------------------------*/
      index_strings = kparse_string_delimit(index_order, " ", 
					    KDELIM_CLEAN, &count);

      /*---------------------------------------------------------------------
      | Check to see if the correct number of indices in the index order
      | was specified i.e. that the number of indices equals the 
      | dimensionlity specified.
      ----------------------------------------------------------------------*/
      if (count != dimension)
      {
	 kerror(lib, rtn, 
		"You specified %d indices for the index order, but the "
		"specified dimensionality is %d.  You must specify %d "
		"indices in the index order", count, dimension, dimension);
	 return (FALSE);
      }

      KCALL(!((index_order_tmp = (int *)kcalloc((size_t)(kmax(dimension, 5)),
					    sizeof(int))) == NULL));
      objlist = klist_add(objlist, index_order_tmp, "KMALLOC");

      /*---------------------------------------------------------------------
      | From the string index_order provided, determine the corresponding
      | data services 'values' for each axis in the index order.
      ----------------------------------------------------------------------*/
      for (i = 0; i < count; i++)
      {
	 if ((kstrcmp(index_strings[i], "width") == 0) ||
	     (index_strings[i][0] == 'w' && 
	      kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KWIDTH;
	 }
	 else if (!kstrcmp(index_strings[i], "height") ||
		  (index_strings[i][0] == 'h'
		   && kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KHEIGHT;
	 }
	 else if (!kstrcmp(index_strings[i], "depth") ||
		  (index_strings[i][0] == 'd'
		   && kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KDEPTH;
	 }
	 else if ((!kstrcmp(index_strings[i], "elements")) ||
		 (!kstrcmp(index_strings[i], "element")) ||
		  (index_strings[i][0] == 'e'
		   && kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KELEMENTS;
	 }
	 else if (!kstrcmp(index_strings[i], "time") ||
		  (index_strings[i][0] == 't'
		   && kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KTIME;
	 }
	 else if (!kstrcmp(index_strings[i], "dimension") ||
		  (index_strings[i][0] == 'n'
		   && kstrlen(index_strings[i]) == 1))
	 {
	    index_order_tmp[i] = KDIMENSION;
	 }
	 else if (!kstrcmp(index_strings[i], "mapwidth") ||
		  !kstrcmp(index_strings[i], "mw"))
	 {
	    index_order_tmp[i] = KMAP_WIDTH;
	 }
	 else if (!kstrcmp(index_strings[i], "mapheight") ||
		  !kstrcmp(index_strings[i], "mh"))
	 {
	    index_order_tmp[i] = KMAP_HEIGHT;
	 }
	 else if (!kstrcmp(index_strings[i], "mapelements") ||
		  !kstrcmp(index_strings[i], "me"))
	 {
	    index_order_tmp[i] = KMAP_ELEMENTS;
	 }
	 else if (!kstrcmp(index_strings[i], "mapdepth") ||
		  !kstrcmp(index_strings[i], "md"))
	 {
	    index_order_tmp[i] = KMAP_DEPTH;
	 }
	 else if (!kstrcmp(index_strings[i], "maptime") ||
		  !kstrcmp(index_strings[i], "mt"))
	 {
	    index_order_tmp[i] = KMAP_TIME;
	 }
	 else
	 {
	    kerror(lib, rtn, 
		   "The only indices supported are : width (w), height (h), "
		   "depth (d), time (t), elements (e), dimension (n), "
		   "mapwidth (mw), mapheight (mh), mapelements (me), "
		   "mapdepth (md) and maptime (mt).");
	    return (FALSE);
	 }
      }

      /* null out any empty indices above the specified dimensionality */
      for (i = count; i < 5; i++)
	 index_order_tmp[i] = KNONE;
   }

   /*------------------------------------------------------------------------
   | All necessary arguments have been provided and they all seem to be okay.
   | Next, we want to check out the input ASCII file.  The ASCII transport 
   | places everything in the value segment, so we look there.
   --------------------------------------------------------------------------*/
   KCALL (kdms_get_attributes(source_object1, KDMS_SEGMENT_VALUE,
			    KDMS_DIMENSION,   &ascii_dimension,
			    KDMS_SIZE,        &ascii_size_vector,
			    KDMS_DATA_TYPE,   &ascii_data_type,
			    KDMS_INDEX_ORDER, &ascii_index_order,
			    NULL));

   /*------------------------------------------------------------------------
   | Make sure the size of the output file matches the input file or
   | at least is not greater than it.
   --------------------------------------------------------------------------*/
   for (check_ascii_size = ascii_size_vector[0], i = 1;
	i < ascii_dimension; i++)
   {
      check_ascii_size *= ascii_size_vector[i];
   }

   for (total_segment_size = size_vector[0], i = 1; i < dimension; i++)
   {
      total_segment_size *= size_vector[i];
   }

   reads_needed = total_segment_size/num_reads;
   if (total_segment_size % num_reads != 0)
      reads_needed++;

   reads_needed *= (num_reads + skip_factor);
   if (reads_needed <= check_ascii_size)
      reads_needed -= skip_factor;

   if (reads_needed + start_offset > check_ascii_size)
   {
      kerror(lib, rtn, 
	     "There are not enough data points in the ascii file to fill "
	     "out a segment of the specified size in the output file.");
      (void) klist_free(objlist, (kfunc_void) lkcall_free);
      return (FALSE);
   }

   /*------------------------------------------------------------------------
   | If input source viff exists then copy it to the output before 
   | inserting the new segment.
   --------------------------------------------------------------------------*/
   if (source_obj)
      KCALL(kdms_copy_object(source_obj, destination_obj, TRUE, TRUE));

   /*------------------------------------------------------------------------
   | Check to see if the destination object already has a segment of the 
   | given name.  If we're dealing with the location segment, set 
   | KPDS_LOCATION_GRID to KCURVILINEAR before the segment is actually
   | created.  By doing so, the location segment will have a grid type
   | imposed.
   --------------------------------------------------------------------------*/
   if (!kdms_query_segment(destination_obj, segment_name))
   {
      if (kstrcmp(KDMS_SEGMENT_LOCATION, segment_name) == 0)
         kpds_set_attribute(destination_obj, KPDS_LOCATION_GRID, KCURVILINEAR);
      KCALL(kdms_create_segment(destination_obj, segment_name));
   }

   /* Check to see that the data type specified is legal */
   if ((data_type = kdatatype_to_define(data_type_string)) == 0)
   {
      kerror(lib, rtn,
	     "Invalid data type '%s' was specified.", data_type_string);
      (void) klist_free(objlist, (kfunc_void) lkcall_free);
      return (FALSE);
   }

   /* Set up the new segment attributes in the destination object */
   KCALL(kdms_set_attributes(destination_obj, segment_name,
			    KDMS_DIMENSION,     dimension,
			    KDMS_INDEX_ORDER,   index_order_tmp,
			    KDMS_DATA_TYPE,     data_type,
			    KDMS_SIZE,          size_vector,
			    NULL));
   
   /*------------------------------------------------------------------------
   | Create a reference on the destination so we can deal with 
   | the data in a friendly data type, yet still store the data in 
   | the desired data type.
   --------------------------------------------------------------------------*/
   KCALL((reference_object = kdms_reference(destination_obj)) 
          != KOBJECT_INVALID);
   objlist = klist_add(objlist, reference_object, "KOBJECT");

   KCALL(kdms_set_attribute(reference_object, segment_name,
		      KDMS_DATA_TYPE, ascii_data_type));

   /*------------------------------------------------------------------------
   | Copy the data with kdms_get_data() and then put it with kdms_put_data().
   | First figure at the begin and end region first. 
   --------------------------------------------------------------------------*/
   if (start_offset == 0 && skip_factor == 0 &&
       (ascii_dimension != dimension ||
	ascii_data_type != data_type || !success))
   {

      KCALL(!((begin = (int *)
	    kmalloc(ascii_dimension * sizeof(int))) == NULL));
      objlist = klist_add(objlist, begin, "KMALLOC");

      KCALL(!((end = (int *)
            kmalloc(ascii_dimension * sizeof(int))) == NULL));
      objlist = klist_add(objlist, end, "KMALLOC");

      KCALL(!((nbegin = (int *)
            kmalloc(kmax(dimension, 5) * sizeof(int))) == NULL));
      objlist = klist_add(objlist, nbegin, "KMALLOC");

      KCALL(!((nend = (int *)
	    kmalloc(kmax(dimension, 5) * sizeof(int))) == NULL));
      objlist = klist_add(objlist, nend, "KMALLOC");

      for (i = 0; i < ascii_dimension; i++)
      {
	 begin[i] = 0;
	 end[i]   = ascii_size_vector[i] - 1;
      }

      for (i = 0; i < dimension; i++)
      {
	 nbegin[i] = 0;
	 nend[i]   = size_vector[i] - 1;
      }
      for (i = dimension; i < 5; i++)
      {
	 nbegin[i] = 0;
	 nend[i]   = 0;
      }

      if ((data = kdms_get_data(source_object1, KDMS_SEGMENT_VALUE,
			   begin, end, NULL)) == NULL)
      {
         kerror(lib, rtn, "Failed to get from source object, %s", ascii_file);
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
	 return(FALSE);
      }
      if (!kdms_put_data(reference_object, segment_name, nbegin, nend, data))
      {
         kerror(lib, rtn, "Failed to put data in %s segment", segment_name);
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
	 return(FALSE);
      }

   }
   else if (start_offset != 0 || skip_factor != 0 || !success)
   {

      KCALL(!((orig_size_vector = (int *)
	       kmalloc(kmax(dimension, 5) * sizeof(int))) == NULL));
      objlist = klist_add(objlist, orig_size_vector, "KMALLOC");

      KCALL(!((begin = (int *)
	       kmalloc(ascii_dimension * sizeof(int))) == NULL));
      objlist = klist_add(objlist, begin, "KMALLOC");

      KCALL(!((end = (int *)
	       kmalloc(ascii_dimension * sizeof(int))) == NULL));
      objlist = klist_add(objlist, end, "KMALLOC");

      KCALL(!((nbegin = (int *)
	       kmalloc(kmax(dimension, 5) * sizeof(int))) == NULL));
      objlist = klist_add(objlist, nbegin, "KMALLOC");

      KCALL(!((nend = (int *)
	       kmalloc(kmax(dimension, 5) * sizeof(int))) == NULL));
      objlist = klist_add(objlist, nend, "KMALLOC");

      for (i = 0, size = 1.0; i < ascii_dimension; i++)
      {
	 size *= ascii_size_vector[i];
	 begin[i] = 0.0;
	 end[i] = ascii_size_vector[i] - 1;

	 orig_size_vector[i] = 1.0;
	 nbegin[i] = 0;
	 nend[i] = 0;
      }

      nend[0] = size - 1;
      orig_size_vector[0] = size;

      KCALL((tmp_srcobject = kdms_create()) != KOBJECT_INVALID);
      objlist = klist_add(objlist, tmp_srcobject, "KOBJECT");

      KCALL(kdms_create_segment(tmp_srcobject, "tempsegment"));

      KCALL(kdms_set_attributes(tmp_srcobject, "tempsegment",
			  KDMS_DIMENSION, 5,
			  KDMS_INDEX_ORDER, ascii_index_order,
			  KDMS_DATA_TYPE, ascii_data_type,
			  KDMS_SIZE, orig_size_vector,
			  NULL));

      if ((data = kdms_get_data(source_object1, KDMS_SEGMENT_VALUE,
			   begin, end, data)) == NULL)
      {
         kerror(lib, rtn, 
	        "Failed to get data from source object, %s", ascii_file);
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
         return(FALSE);
      }

      if (!kdms_put_data(tmp_srcobject, "tempsegment", nbegin, nend, data))
      {
         kerror(lib, rtn, "Failed to put data in temporary object");
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
         return(FALSE);
      }

      kfree_and_NULL(data);
      data = NULL;

      for (i = 0; i < ascii_dimension; i++)
      {
	 begin[i] = 0.0;
	 end[i]   = 0.0;
      }

      for (i = 0, size = 1.0; i < dimension; i++)
      {
	 orig_size_vector[i] = size_vector[i];
	 size *= size_vector[i];
	 size_vector[i] = 1.0;
	 nbegin[i] = 0;
	 nend[i] = 0;
      }
      size_vector[0] = size;

      for (i = dimension; i < 5; i++)
      {
	 nbegin[i] = 0;
	 nend[i]   = 0;
      }


      KCALL((tmp_object = kdms_create()) != KOBJECT_INVALID);
      objlist = klist_add(objlist, tmp_object, "KOBJECT");

      KCALL(kdms_create_segment(tmp_object, segment_name));

      KCALL(kdms_copy_segment_attributes(destination_obj, segment_name,
				         tmp_object, segment_name));

      KCALL(kdms_set_attributes(tmp_object, segment_name,
			  KDMS_DATA_TYPE, ascii_data_type,
			  KDMS_SIZE, size_vector,
			  NULL));

      offset = 0;
      nbegin[0] = 0;

      while (size > nbegin[0])
      {
         begin[0] = offset + start_offset;
	 end[0] = begin[0] + num_reads - 1;
	 nend[0] = nbegin[0] + num_reads - 1;
         if (nend[0] >= size)
            nend[0] = size - 1;
     
	 if ((data = kdms_get_data(tmp_srcobject, "tempsegment", 
				   begin, end, data)) == NULL)
         {
	    kerror(lib, rtn, 
		   "Failed to get data from temporary source object");
	    (void) klist_free(objlist, (kfunc_void) lkcall_free);
	    return(FALSE);
         }
	 if (!kdms_put_data(tmp_object, segment_name, nbegin, nend, data))
	 {
	    kerror(lib, rtn, 
		   "Failed to put data into temporary object");
	    (void) klist_free(objlist, (kfunc_void) lkcall_free);
	    return(FALSE);
	 }

	 offset += skip_factor + num_reads;
	 nbegin[0] = nend[0] + 1;
      }

      nbegin[0] = 0.0;
      for (i = 0, size = 1.0; i < dimension; i++)
      {
	 begin[i] = 0;
	 end[i] = orig_size_vector[i] - 1;
      }

      for (i = dimension; i < 5; i++)
      {
	 begin[i] = 0;
	 end[i] = 0;
      }

      kfree_and_NULL(data);

      if ((data = kdms_get_data(tmp_object, segment_name, 
		  nbegin, nend, data)) == NULL)
      {
         kerror(lib, rtn, "Failed to get data from temporary object");
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
         return(FALSE);
      }
      if (!kdms_put_data(reference_object, segment_name, begin, end, data))
      {
         kerror(lib, rtn, "Failed to put data to output reference object");
	 (void) klist_free(objlist, (kfunc_void) lkcall_free);
         return(FALSE);
      }
   }
   for (i = 0; i < count; i++)
      kfree_and_NULL(index_strings[i]);

   /*------------------------------------------------------------------------
   | This should be legal to free but it breaks on Linux so have commented 
   | it out for now.
   kfree_and_NULL(index_order_tmp);
   ------------------------------------------------------------------------*/

   kfree_and_NULL(index_strings);

   kfree_and_NULL(data);
  
   klist_free(objlist, (kfunc_void) lkcall_free);

   return (TRUE);
}
/* -library_code_end */
