#! /bin/sh

echo "running test2.sh for kcompare ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
tolerance="$5"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tolerance" -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  IF unsigned byte = unsigned byte, THEN out = -1, ELSE out = 99
# ------------------------------------------------------------------------
test_num=1
descr="IF ubyte = ubyte THEN out = -1, ELSE out = 99"
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 7 8 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 3
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in1.$$ \
         -o $tmpdir/junk1.$$ -tval -1.0 -fval 99.0 -eq
cat > $tmpdir/asc.$$ <<EOF
-1 -1 -1 -1 -1 -1 -1 -1 -1 -1
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 4
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$

# ------------------------------------------------------------------------
# Test  2:  IF unsigned byte = unsigned byte, THEN out = 99, ELSE out = -1
# ------------------------------------------------------------------------
test_num=2
descr="IF ubyte = ubyte THEN out = 99, ELSE out = -1"
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 7 8 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 3
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 1 1 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in2.$$ -wsize 10 -hsize 1 -type 3
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
         -o $tmpdir/junk1.$$ -tval 99 -fval -1 -eq
cat > $tmpdir/asc.$$ <<EOF
99 99 99 99 99 99 -1 -1 99 99
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 4
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$
rm -rf $tmpdir/in2.$$

# ------------------------------------------------------------------------
# Test  3:  IF unsigned byte = unsigned byte, THEN out = 99, ELSE out = 1
# ------------------------------------------------------------------------
test_num=3
descr="IF ubyte = ubyte THEN out = 99, ELSE out = -1"
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 7 8 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 3
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in1.$$ \
         -o $tmpdir/junk1.$$ -tval 99 -fval 1 -eq
cat > $tmpdir/asc.$$ <<EOF
99 99 99 99 99 99 99 99 99 99
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 3
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$

# ------------------------------------------------------------------------
# Test  4:  IF unsigned byte = short, THEN out = 99, ELSE out = -1
# ------------------------------------------------------------------------
test_num=4
descr="IF ubyte = short THEN out = 99, ELSE out = -1"
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 7 8 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 3
cat > $tmpdir/asc.$$ <<EOF
1 2 3 4 5 6 1 1 9 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in2.$$ -wsize 10 -hsize 1 -type 4
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
         -o $tmpdir/junk1.$$ -tval 99 -fval -1 -eq
cat > $tmpdir/asc.$$ <<EOF
99 99 99 99 99 99 -1 -1 99 99
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 4
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$
rm -rf $tmpdir/in2.$$

# ------------------------------------------------------------------------
# Test  5:  IF bit = bit, THEN out = 0, ELSE out = 1
# ------------------------------------------------------------------------
test_num=5
descr="IF bit = bit THEN out = 0, ELSE out = 1"
cat > $tmpdir/asc.$$ <<EOF
1 1 1 1 1 1 0 0 0 0
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 1
cat > $tmpdir/asc.$$ <<EOF
0 1 1 1 1 1 0 0 0 1
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in2.$$ -wsize 10 -hsize 1 -type 1
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
         -o $tmpdir/junk1.$$ -tval 0 -fval 1 -eq
cat > $tmpdir/asc.$$ <<EOF
1 0 0 0 0 0 0 0 0 1
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 1
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$
rm -rf $tmpdir/in2.$$

# ------------------------------------------------------------------------
# Test  6:  IF bit = bit, THEN out = 0, ELSE out = 10
# ------------------------------------------------------------------------
test_num=6
descr="IF bit = bit THEN out = 0, ELSE out = 10"
cat > $tmpdir/asc.$$ <<EOF
1 1 1 1 1 1 0 0 0 0
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in1.$$ -wsize 10 -hsize 1 -type 1
cat > $tmpdir/asc.$$ <<EOF
0 1 1 1 1 1 0 0 0 1
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/in2.$$ -wsize 10 -hsize 1 -type 1
${BINARY_PATH}/kcompare -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
         -o $tmpdir/junk1.$$ -tval 0 -fval 10 -eq
cat > $tmpdir/asc.$$ <<EOF
10 0 0 0 0 0 0 0 0 10
EOF
kasc2val -i1 $tmpdir/asc.$$ -o $tmpdir/out.$$ -wsize 10 -hsize 1 -type 2
report $test_num "$tmpdir/junk1.$$" "$tmpdir/out.$$" "$descr" "1e-5"

rm -rf  $tmpdir/junk1.$$ $tmpdir/stats.$$ $tmpdir/asc.$$ $tmpdir/in1.$$
rm -rf $tmpdir/in2.$$

exit $status
