#! /bin/sh

echo "running test1.sh for kfft ... process ($$)"

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

kextract -i image:moon -o $tmpdir/ref.$$  -woff 200 -hoff 200 \
	-doff 0 -toff 0 -eoff 0 -wsize 64 -hsize 64 -tsize 1 -dsize 1 -esize 1 \
	 -subpos FALSE
${BINARY_PATH}/kfft -i $tmpdir/ref.$$ -o $tmpdir/junk1.$$ -w -h -scale 1 -type complex -dir 0 -center 0
${BINARY_PATH}/kfft -i $tmpdir/junk1.$$ -o $tmpdir/junk2.$$ -w -h -scale 2 -type complex -dir 1 -center 0
karith2 -i1 $tmpdir/junk2.$$ -i2 $tmpdir/ref.$$ -o $tmpdir/junk1.$$ -sub
kconvert -i $tmpdir/junk1.$$ -o $tmpdir/diff.$$ -val -type 8
kstats -i $tmpdir/diff.$$ -o $tmpdir/stats.$$ -all
ksegcmp -i1 $tmpdir/stats.$$ -i2 test1.results/stats1 -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 1 - (input-ifft(fft(moon)))->long(mag)->stats = zero (64x64)"
else
   echo "FAILED:  Test 1 - (input-ifft(fft(moon)))->long(mag)->stats = zero (64x64)"
   status=1
fi


#   BEGIN SECOND TEST

${BINARY_PATH}/kfft -i $tmpdir/ref.$$   -o $tmpdir/junk1.$$ -w -h -scale 1 -type "double complex" -dir 0 -center 1
${BINARY_PATH}/kfft -i $tmpdir/junk1.$$ -o $tmpdir/junk2.$$ -w -h -scale 2 -type 13 -dir 1 -center 1
karith2 -i1 $tmpdir/junk2.$$ -i2 $tmpdir/ref.$$ -o $tmpdir/junk1.$$ -sub
kconvert -i $tmpdir/junk1.$$ -o $tmpdir/diff.$$ -val -type 8
kstats -i $tmpdir/diff.$$ -o $tmpdir/stats.$$ -all
ksegcmp -i1 $tmpdir/stats.$$ -i2 test1.results/stats2 -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 2 - test for scaling and centering (64x64)"
else
   echo "FAILED:  Test 2 - test for scaling and centering (64x64)"
   status=1
fi

/bin/rm -f  $tmpdir/ref.$$ $tmpdir/junk1.$$ $tmpdir/junk2.$$ $tmpdir/diff.$$ $tmpdir/stats.$$

#	BEGIN THIRD TEST

kimpulse -o $tmpdir/imp.$$ -wsize 4 -wnum 1 -hsize 4 -hnum 1 -dsize 4 -dnum 1 -tsize 4 -tnum 1 -esize 4 -enum 1 -woff 0 -hoff 0 -doff 0 -toff 0 -eoff 0 -fgreal 1 -type 10
${BINARY_PATH}/kfft -i $tmpdir/imp.$$ -o $tmpdir/junk1.$$ -w -d -h -t -e -center 1 -scale 1 -dir 0
kcmplx2real -i $tmpdir/junk1.$$ -or $tmpdir/junk2.$$
kstats -i $tmpdir/junk2.$$ -o $tmpdir/stats.$$ -sum
cat << EOF > $tmpdir/junk.asc.$$
1024
EOF
kasc2val -i1 $tmpdir/junk.asc.$$ -o $tmpdir/junk.val.$$ -type 11 -wsize 1 -hsize 1
ksegcmp -i1 $tmpdir/stats.$$ -i2 $tmpdir/junk.val.$$ -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 3 - test for 5D impulse (4x4x4x4x4)"
else
   echo "FAILED:  Test 3 - test for 5D impulse (4x4x4x4x4)"
   status=1
fi

/bin/rm -f  $tmpdir/junk1.$$ $tmpdir/junk2.$$ $tmpdir/junk.asc.$$ $tmpdir/junk.val.$$ $tmpdir/stats.$$ $tmpdir/imp.$$

#       BEGIN FOURTH TEST

kgsin -o $tmpdir/vol.$$ -sin1 -ampl 1 -wsize 32 -hsize 32 -dsize 32 -wnum 2 -hnum 3 \
-dnum 4 -type 11
${BINARY_PATH}/kfft -i $tmpdir/vol.$$ -o $tmpdir/junk1.$$ -w -d -h -center 0 -scale 1 -dir 0
kcmplx2real -i $tmpdir/junk1.$$ -om $tmpdir/junk2.$$ -mag 
kstats -i $tmpdir/junk2.$$ -o $tmpdir/stats.$$ -maxval -wmax -hmax -dmax
cat << EOF > $tmpdir/junk.asc.$$
16384
2 3 4
EOF
kasc2val -i1 $tmpdir/junk.asc.$$ -o $tmpdir/junk.val.$$ -type 11 -wsize 1 -esize 4
ksegcmp -i1 $tmpdir/stats.$$ -i2 $tmpdir/junk.val.$$ -tol 1e-5 -sat 1 -s -rt 1 \
-all
if [ $? = 0 ]
then
   echo "Passed:  Test 4 - test for 3D sinusoid (32x32x32)"
else
   echo "FAILED:  Test 4 - test for 3D sinusoid (32x32x32)"
   status=1
fi

/bin/rm -f  $tmpdir/junk1.$$ $tmpdir/junk2.$$ $tmpdir/junk.asc.$$ $tmpdir/junk.val.$$ $tmpdir/stats.$$ $tmpdir/vol.$$

#       BEGIN FIFTH TEST
#	Added on behalf of Roberto Lotufo SRW 28-Jul-94

kgsin -o $tmpdir/vol.$$ -sin1 -ampl 1 -wsize 16 -hsize 1024 -wnum 2 -hnum 3 -type 11
${BINARY_PATH}/kfft -i $tmpdir/vol.$$ -o $tmpdir/junk1.$$ -w -h -center 0 -scale 3 -dir 0
${BINARY_PATH}/kfft -i $tmpdir/junk1.$$ -o $tmpdir/junk2.$$ -w -h -center 0 -scale 3 -dir 1
kcmplx2real -i $tmpdir/junk2.$$ -or $tmpdir/junk2.$$ 
karith2 -i1 $tmpdir/junk2.$$ -i2 $tmpdir/vol.$$ -o $tmpdir/junk2.$$ -sub
kstats -i $tmpdir/junk2.$$ -o $tmpdir/stats.$$ -maxval
cat << EOF > $tmpdir/junk.asc.$$
0
EOF
kasc2val -i1 $tmpdir/junk.asc.$$ -o $tmpdir/junk.val.$$ -type 11 -wsize 1 -hsize 1 
ksegcmp -i1 $tmpdir/stats.$$ -i2 $tmpdir/junk.val.$$ -tol 1e-5 -sat 1 -s -rt 1 \
-all
if [ $? = 0 ]
then
   echo "Passed:  Test 5 - test for sqrt scaling (16x1024)"
else
   echo "FAILED:  Test 5 - test for sqrt scaling (16x1024)"
   status=1
fi

/bin/rm -f  $tmpdir/junk1.$$ $tmpdir/junk2.$$ $tmpdir/junk.asc.$$ \
$tmpdir/junk.val.$$ $tmpdir/stats.$$ $tmpdir/vol.$$

#       BEGIN SIXTH TEST

kgconst -o $tmpdir/vol.$$ -wsize 32768 -hsize 1 -type 11 -real 1
${BINARY_PATH}/kfft -i $tmpdir/vol.$$ -o $tmpdir/junk1.$$ -w -center 0 -scale 3 -dir 0
${BINARY_PATH}/kfft -i $tmpdir/junk1.$$ -o $tmpdir/junk2.$$ -w -center 0 -scale 3 -dir 1
kcmplx2real -i $tmpdir/junk2.$$ -or $tmpdir/junk2.$$ 
karith2 -i1 $tmpdir/junk2.$$ -i2 $tmpdir/vol.$$ -o $tmpdir/junk2.$$ -sub
kstats -i $tmpdir/junk2.$$ -o $tmpdir/stats.$$ -maxval
cat << EOF > $tmpdir/junk.asc.$$
0
EOF
kasc2val -i1 $tmpdir/junk.asc.$$ -o $tmpdir/junk.val.$$ -type 11 -wsize 1 -hsize 1
ksegcmp -i1 $tmpdir/stats.$$ -i2 $tmpdir/junk.val.$$ -tol 1e-5 -sat 1 -s -rt 1 \
-all
if [ $? = 0 ]
then
   echo "Passed:  Test 6 - test for 1D max length vector (1x32768)"
else
   echo "FAILED:  Test 6 - test for 1D max length vector (1x32768)"
   status=1
fi

/bin/rm -f  $tmpdir/junk1.$$ $tmpdir/junk2.$$ $tmpdir/junk.asc.$$ \
$tmpdir/junk.val.$$ $tmpdir/stats.$$ $tmpdir/vol.$$

#       BEGIN SEVENTH TEST

cat > $tmpdir/in.$$ <<EOF
1 2 3 4
2 3 4 5
5 4 3 2
1 1 1 1
EOF

cat > $tmpdir/hcent.$$ <<EOF
1 -1 1 -1
1 -1 1 -1
1 -1 1 -1
1 -1 1 -1
EOF

kmul -i1 $tmpdir/in.$$ -i2 $tmpdir/hcent.$$ -o $tmpdir/inh.$$
${BINARY_PATH}/kfft -i $tmpdir/inh.$$ -o $tmpdir/inh.fft.$$ -w -dir 0 -center 0 -scale 1

${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/inH.fft.$$ -w -dir 0 -center 1 -scale 1

ksegcmp -i1 $tmpdir/inh.fft.$$ -i2 $tmpdir/inH.fft.$$ -tol 1e-5 -sat 1 -s \
-rt 1 -all

if [ $? = 0 ]
then
   echo "Passed:  Test 7a - test for Nyquist modulation k-vector - match"
else
   echo "FAILED:  Test 7a - test for Nyquist modulation k-vector - match"
   status=1
fi

kcmplx2real -i $tmpdir/inH.fft.$$ -or $tmpdir/inH.fft.r.$$ -oi $tmpdir/inH.fft.i.$$

cat > $tmpdir/in.r.$$ <<EOF
-2 -2 10 -2
-2 -2 14 -2
2 2 14 2
0 0 4 0
EOF

cat > $tmpdir/in.i.$$ << EOF
0 -2 0 2
0 -2 0 2
0 2 0 -2
0 0 0 0
EOF
kconvert -i $tmpdir/in.r.$$ -o $tmpdir/in.r.$$ -val -type 10
kconvert -i $tmpdir/in.i.$$ -o $tmpdir/in.i.$$ -val -type 10

ksegcmp -i1 $tmpdir/inH.fft.r.$$ -i2 $tmpdir/in.r.$$ -tol 1e-5 -sat 1 -s \
-rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 7b - test for Nyquist modulation k-vector - real part"
else
   echo "FAILED:  Test 7b - test for Nyquist modulation k-vector - real part"
   status=1
fi

ksegcmp -i1 $tmpdir/inH.fft.i.$$ -i2 $tmpdir/in.i.$$ -tol 1e-5 -sat 1 -s \
-rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 7c - test for Nyquist modulation k-vector - imag part"
else
   echo "FAILED:  Test 7c - test for Nyquist modulation k-vector - imag part"
   status=1
fi

/bin/rm -f $tmpdir/in.$$ $tmpdir/hcent.$$ $tmpdir/inh.$$
/bin/rm -f $tmpdir/inh.fft.$$ $tmpdir/inH.fft.$$
/bin/rm -f $tmpdir/inH.fft.r.$$ $tmpdir/inH.fft.i.$$
/bin/rm -f $tmpdir/in.r.$$ $tmpdir/in.i.$$

#       BEGIN EIGTH TEST
#
#	This test is taken directly from Gonzalez and Wintz, 2nd Ed,
#	pp 70 and 71, where there is a simple 1D example that should serve
#	as a nice reference.

#	Set up input files
cat > $tmpdir/in.$$ <<EOF
2 3 4 4
EOF

cat > $tmpdir/in.cmplx.$$ <<EOF
(2.0,0.0) (3.0,0.0) (4.0,0.0) (4.0,0.0)
EOF

# 	fwd, none, 1D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -w -dir 0 -center 0 \
-scale 1 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(13,0.0) (-2,1.0) (-1.0,0.0) (-2.0,-1.0)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8a - test for scaling - fwd, none, 1D"
else
   echo "FAILED:  Test 8a - test for scaling - fwd, none, 1D"
   status=1
fi

# 	rev, 1/N, 1D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -w -dir 1 \
	-center 0 -scale 2 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8b - test for scaling - rev, 1/N, 1D"
else
   echo "FAILED:  Test 8b - test for scaling - rev, 1/N, 1D"
   status=1
fi

#	fwd, 1/N 1D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -w -dir 0 -center 0 \
-scale 2 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(3.25,0.0) (-0.5,0.25) (-0.25,0.0) (-0.5,-0.25)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8c - test for scaling - fwd, 1/N, 1D"
else
   echo "FAILED:  Test 8c - test for scaling - fwd, 1/N, 1D"
   status=1
fi

#       rev, none, 1D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -w -dir 1 \
	-center 0 -scale 1 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8d - test for scaling - rev, none, 1D"
else
   echo "FAILED:  Test 8d - test for scaling - rev, none, 1D"
   status=1
fi

#	fwd, 1/sqrt(N), 1D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -w -dir 0 -center 0 \
-scale 3 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(6.5,0.0) (-1.0,0.5) (-0.5,0.0) (-1.0,-0.5)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8e - test for scaling - fwd, 1/sqrt(N), 1D"
else
   echo "FAILED:  Test 8e - test for scaling - fwd, 1/sqrt(N), 1D"
   status=1
fi

#       rev, 1/sqrt(N), 1D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -w -dir 1 \
	-center 0 -scale 3 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8f - test for scaling - rev, 1/sqrt(N), 1D"
else
   echo "FAILED:  Test 8f - test for scaling - rev, 1/sqrt(N), 1D"
   status=1
fi

#	test along W only, 2D data
cat > $tmpdir/in.$$ <<EOF
2 3 4 4
2 3 4 4
2 3 4 4
EOF

#       fwd, 1/N 1D, on 2D data
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -w -dir 0 -center 0 \
-scale 2 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(3.25,0.0) (-0.5,0.25) (-0.25,0.0) (-0.5,-0.25)
(3.25,0.0) (-0.5,0.25) (-0.25,0.0) (-0.5,-0.25)
(3.25,0.0) (-0.5,0.25) (-0.25,0.0) (-0.5,-0.25)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8g - test for scaling - fwd, 1/N, W only on non-square WxH data"
else
   echo "FAILED:  Test 8g - test for scaling - fwd, 1/N, W only on non-square WxH data"
   status=1
fi

#       test along H only, 2D data
cat > $tmpdir/in.$$ <<EOF
2 2 2
3 3 3
4 4 4
4 4 4
EOF

#       fwd, 1/N 1D, on 2D data
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -h -dir 0 -center 0 \
-scale 2 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(3.25,0.0) (3.25,0.0) (3.25,0.0)
(-0.5,0.25) (-0.5,0.25) (-0.5,0.25)
(-0.25,0.0) (-0.25,0.0) (-0.25,0.0)
(-0.5,-0.25) (-0.5,-0.25) (-0.5,-0.25)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 8h - test for scaling - fwd, 1/N, H only on non-square WxH data"
else
   echo "FAILED:  Test 8h - test for scaling - fwd, 1/N, H only on non-square WxH data"
   status=1
fi

#       BEGIN NINTH TEST
#
#	This is to check the scaling and centering and numerical values
#	for the 2D case.

#	Set up input files
cat > $tmpdir/in.$$ <<EOF
2 3 4 4
2 3 4 4
2 3 4 4
2 3 4 4
EOF

cat > $tmpdir/in.cmplx.$$ <<EOF
(2.0,0.0) (3.0,0.0) (4.0,0.0) (4.0,0.0)
(2.0,0.0) (3.0,0.0) (4.0,0.0) (4.0,0.0)
(2.0,0.0) (3.0,0.0) (4.0,0.0) (4.0,0.0)
(2.0,0.0) (3.0,0.0) (4.0,0.0) (4.0,0.0)
EOF

# 	fwd, none, 2D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -w -h -dir 0 \
	-center 1 -scale 1 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(-4.0,0.0) (-8.0,-4.0) (52.0,0.0) (-8.0,4.0)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9a - test for scaling - fwd, none, 2D"
else
   echo "FAILED:  Test 9a - test for scaling - fwd, none, 2D"
   status=1
fi

# 	rev, 1/N, 2D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -whole -dir 1 \
	-center 1 -scale 2 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9b - test for scaling - rev, 1/N, 2D"
else
   echo "FAILED:  Test 9b - test for scaling - rev, 1/N, 2D"
   status=1
fi

#	fwd, 1/N 2D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -whole -dir 0 \
	-center 1 -scale 2 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(-0.25,0.0) (-0.5,-0.25) (3.25,0.0) (-0.5,0.25)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9c - test for scaling - fwd, 1/N, 2D"
else
   echo "FAILED:  Test 9c - test for scaling - fwd, 1/N, 2D"
   status=1
fi

#       rev, none, 1D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -whole -dir 1 \
	-center 1 -scale 1 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9d - test for scaling - rev, none, 2D"
else
   echo "FAILED:  Test 9d - test for scaling - rev, none, 2D"
   status=1
fi

#	fwd, 1/sqrt(N), 2D
${BINARY_PATH}/kfft -i $tmpdir/in.$$ -o $tmpdir/out.fft.$$ -whole -dir 0 \
	-center 1 -scale 3 -type 13

cat > $tmpdir/out.fft.correct.$$ << EOF
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
(-1.0,0.0) (-2.0,-1.0) (13.0,0.0) (-2.0,1.0)
(0.0,0.0) (0.0,0.0) (0.0,0.0) (0.0,0.0)
EOF

ksegcmp -i1 $tmpdir/out.fft.$$ -i2 $tmpdir/out.fft.correct.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9e - test for scaling - fwd, 1/sqrt(N), 2D"
else
   echo "FAILED:  Test 9e - test for scaling - fwd, 1/sqrt(N), 2D"
   status=1
fi

#       rev, 1/sqrt(N), 2D
${BINARY_PATH}/kfft -i $tmpdir/out.fft.$$ -o $tmpdir/in2.$$ -whole -dir 1 \
	-center 1 -scale 3 -type 13

ksegcmp -i1 $tmpdir/in.cmplx.$$ -i2 $tmpdir/in2.$$ -tol 1e-5 \
-sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test 9f - test for scaling - rev, 1/sqrt(N), 2D"
else
   echo "FAILED:  Test 9f - test for scaling - rev, 1/sqrt(N), 2D"
   status=1
fi

/bin/rm -f $tmpdir/in.$$ $tmpdir/in.cmplx.$$ $tmpdir/out.fft.$$
/bin/rm -f $tmpdir/out.fft.correct.$$ $tmpdir/in2.$$

exit $status
