#! /bin/sh

echo "running test.sh for kflip ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set data path for image data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
dpath2="$tbpath/data/sequences"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  16:  int value data flipped along width and height with time data
# ------------------------------------------------------------------------
test_num=16
descr="int value data flipped along width and height with time data"
cat <<EOF > $tmpdir/val_ascii.$$
1 2 3 4 5 6 7 8
2 4 6 8 1 3 5 7
3 6 9 2 4 6 8 1
EOF
kasc2val -i1 $tmpdir/val_ascii.$$ -wsize 8 -hsize 3 -type 6 -o $tmpdir/val.$$

cat <<EOF > $tmpdir/time_ascii.$$
1
EOF
kasc2time -i1 $tmpdir/time_ascii.$$ -tsize 1 -o $tmpdir/time.$$

kinsertseg -i1 $tmpdir/time.$$ -i2 $tmpdir/val.$$ -o $tmpdir/combined.$$ -time

${BINARY_PATH}/kflip -i $tmpdir/combined.$$ -o $tmpdir/out.$$ -w -h

cat <<EOF > $tmpdir/ascii.$$
1 8 6 4 2 9 6 3 
7 5 3 1 8 6 4 2
8 7 6 5 4 3 2 1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 8 -hsize 3 -type 6 -o $tmpdir/result.$$
kinsertseg -i1 $tmpdir/time.$$ -i2 $tmpdir/result.$$ -o $tmpdir/result.$$ -time

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/val_ascii.$$ $tmpdir/val.$$ $tmpdir/time_ascii.$$ 
/bin/rm -f $tmpdir/time.$$ $tmpdir/combined.$$ $tmpdir/out.$$
/bin/rm -f $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  17:  int value data flipped along width with a map
# ------------------------------------------------------------------------
test_num=17
descr="int value data flipped along width with a map"
cat <<EOF > $tmpdir/val_ascii.$$
0 1 1
2 3 3
6 0 0
EOF
kasc2val -i1 $tmpdir/val_ascii.$$ -wsize 3 -hsize 3 -type 6 -o $tmpdir/val.$$

cat <<EOF > $tmpdir/map_ascii.$$
70 60 50 40 30 20 10
EOF
kasc2map -i1 $tmpdir/map_ascii.$$ -wsize 1 -hsize 7 -o $tmpdir/map.$$

kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/val.$$ -o $tmpdir/combined.$$ -map

${BINARY_PATH}/kflip -i $tmpdir/combined.$$ -o $tmpdir/out.$$ -w

cat <<EOF > $tmpdir/ascii.$$
1 1 0
3 3 2
0 0 6
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 3 -type 6 -o $tmpdir/result.$$
kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/result.$$ -o $tmpdir/result.$$ -map

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/val_ascii.$$ $tmpdir/val.$$ $tmpdir/map_ascii.$$
/bin/rm -f $tmpdir/map.$$ $tmpdir/combined.$$ $tmpdir/out.$$
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$


# ------------------------------------------------------------------------
# Test  18:  int value data flipped along width and depth with a
#           map that has map depth size equal to value depth size
# ------------------------------------------------------------------------
test_num=18
descr="int value data flipped along width and depth with a map that has map depth size equal to value depth size"
cat <<EOF > $tmpdir/val_ascii.$$
0 1 1
2 3 3
6 0 0
0 1 2
3 4 5
6 1 0
1 2 3
4 5 6
0 1 2
EOF
kasc2val -i1 $tmpdir/val_ascii.$$ -wsize 3 -hsize 3 -dsize 3 -type 6 -o $tmpdir/val.$$

cat <<EOF > $tmpdir/map_ascii.$$
70 60 50 40 30 20 10
10 20 30 40 50 60 70
10 30 50 70 20 40 60
EOF
kasc2map -i1 $tmpdir/map_ascii.$$ -wsize 1 -hsize 7 -dsize 3 -o $tmpdir/map.$$

kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/val.$$ -o $tmpdir/combined.$$ -map

${BINARY_PATH}/kflip -i $tmpdir/combined.$$ -o $tmpdir/out.$$ -w -d

cat <<EOF > $tmpdir/ascii.$$
3 2 1
6 5 4
2 1 0
2 1 0
5 4 3
0 1 6
1 1 0
3 3 2
0 0 6
EOF
kasc2val -i1  $tmpdir/ascii.$$ -wsize 3 -hsize 3 -dsize 3 -type 6 -o $tmpdir/result.$$
kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/result.$$ -o $tmpdir/result.$$ -map

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/val_ascii.$$ $tmpdir/val.$$ $tmpdir/map_ascii.$$
/bin/rm -f $tmpdir/map.$$ $tmpdir/combined.$$ $tmpdir/out.$$
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  19:  int value data flipped along width and elements with a
#            map that has map elements size equal to value elements size
# ------------------------------------------------------------------------
test_num=19
descr="int value data flipped along width and elements with a map that has map elements size equal to value elements size"
cat <<EOF > $tmpdir/val_ascii.$$
0 1 1
2 3 3
6 0 0
0 1 2
3 4 5
6 1 0
1 2 3
4 5 6
0 1 2
EOF
kasc2val -i1 $tmpdir/val_ascii.$$ -wsize 3 -hsize 3 -esize 3 -type 6 -o $tmpdir/val.$$

cat <<EOF > $tmpdir/map_ascii.$$
70 60 50 40 30 20 10
10 20 30 40 50 60 70
10 30 50 70 20 40 60
EOF
kasc2map -i1 $tmpdir/map_ascii.$$ -wsize 1 -hsize 7 -esize 3 -o $tmpdir/map.$$

kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/val.$$ -o $tmpdir/combined.$$ -map

${BINARY_PATH}/kflip -i $tmpdir/combined.$$ -o $tmpdir/out.$$ -w -e

cat <<EOF > $tmpdir/ascii.$$
3 2 1
6 5 4
2 1 0
2 1 0
5 4 3
0 1 6
1 1 0
3 3 2
0 0 6
EOF
kasc2val -i1  $tmpdir/ascii.$$ -wsize 3 -hsize 3 -esize 3 -type 6 -o $tmpdir/result.$$
kinsertseg -i1 $tmpdir/map.$$ -i2 $tmpdir/result.$$ -o $tmpdir/result.$$ -map

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/val_ascii.$$ $tmpdir/val.$$ $tmpdir/map_ascii.$$
/bin/rm -f $tmpdir/map.$$ $tmpdir/combined.$$ $tmpdir/out.$$
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$

exit $status
