#! /bin/sh

echo "running test.sh for kflip ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set data path for image data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/testsuite/kflip/test4.inputs"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
#  This testsuite tests all possible combinations of kflip and 
#  operates on integer data only since other testsuites test
#  the gamut of datatypes.
#
#  W   H   D   T   E
#  0   0   0   0   1     ---->  Flipped along e
#  0   0   0   1   0     ---->  Flipped along t
#  0   0   0   1   1     ---->  Flipped along t, e
#  0   0   1   0   0     ---->  Flipped along d
#  0   0   1   0   1     ---->  Flipped along d, e
#  0   0   1   1   0     ---->  Flipped along d, t
#  0   0   1   1   1     ---->  Flipped along d, t, e
#  0   1   0   0   0     ---->  Flipped along h
#  0   1   0   0   1     ---->  Flipped along h, e
#  0   1   0   1   0     ---->  Flipped along h, t
#  0   1   0   1   1     ---->  Flipped along h, t, e
#  0   1   1   0   0     ---->  Flipped along h, d
#  0   1   1   0   1     ---->  Flipped along h, d, e
#  0   1   1   1   0     ---->  Flipped along h, d, t
#  0   1   1   1   1     ---->  Flipped along h, d, t, e
#  1   0   0   0   0     ---->  Flipped along w
#  1   0   0   0   1     ---->  Flipped along w, e
#  1   0   0   1   0     ---->  Flipped along w, t
#  1   0   0   1   1     ---->  Flipped along w, t, e
#  1   0   1   0   0     ---->  Flipped along w, d
#  1   0   1   0   1     ---->  Flipped along w, d, e
#  1   0   1   1   0     ---->  Flipped along w, d, t
#  1   0   1   1   1     ---->  Flipped along w, d, t, e
#  1   1   0   0   0     ---->  Flipped along w, h
#  1   1   0   0   1     ---->  Flipped along w, h, e
#  1   1   0   1   0     ---->  Flipped along w, h, t
#  1   1   0   1   1     ---->  Flipped along w, h, t, e
#  1   1   1   0   0     ---->  Flipped along w, h, d
#  1   1   1   0   1     ---->  Flipped along w, h, d, e
#  1   1   1   1   0     ---->  Flipped along w, h, d, t
#  1   1   1   1   1     ---->  Flipped along w, h, d, t, e
# ------------------------------------------------------------------------

# ------------------------------------------------------------------------
# Test  20:  5D int value data flipped along elements only
# ------------------------------------------------------------------------
test_num=20
descr="5D int value data flipped along elements only"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -e

kimportasc -i1 $dpath/test_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_e.$$

# ------------------------------------------------------------------------
# Test  21:  5D int value data flipped along time only
# ------------------------------------------------------------------------
test_num=21
descr="5D int value data flipped along time only"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -t

kimportasc -i1 $dpath/test_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_t.$$


# ------------------------------------------------------------------------
# Test  22:  5D int value data flipped along time & elements
# ------------------------------------------------------------------------
test_num=22
descr="5D int value data flipped along time & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -t -e

kimportasc -i1 $dpath/test_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6  -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_t_e.$$

# ------------------------------------------------------------------------
# Test  23:  5D int value data flipped along depth only
# ------------------------------------------------------------------------
test_num=23
descr="5D int value data flipped along depth only"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -d

kimportasc -i1 $dpath/test_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_d.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_d.$$

# ------------------------------------------------------------------------
# Test  24:  5D int value data flipped along depth & elements
# ------------------------------------------------------------------------
test_num=24
descr="5D int value data flipped along depth & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -d -e

kimportasc -i1 $dpath/test_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_d_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_d_e.$$

# ------------------------------------------------------------------------
# Test  25:  5D int value data flipped along depth & time
# ------------------------------------------------------------------------
test_num=25
descr="5D int value data flipped along depth & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -d -t

kimportasc -i1 $dpath/test_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_d_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_d_t.$$

# ------------------------------------------------------------------------
# Test  26:  5D int value data flipped along depth, time, & elements
# ------------------------------------------------------------------------
test_num=26
descr="5D int value data flipped along depth, time & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$

${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -d -t -e

kimportasc -i1 $dpath/test_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_d_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_d_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_d_t_e.$$

# ------------------------------------------------------------------------
# Test  27:  5D int value data flipped along height only
# ------------------------------------------------------------------------
test_num=27
descr="5D int value data flipped along height only"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h

kimportasc -i1 $dpath/test_h.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h.$$

# ------------------------------------------------------------------------
# Test  28:  5D int value data flipped along height & elements
# ------------------------------------------------------------------------
test_num=28
descr="5D int value data flipped along height & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -e

kimportasc -i1 $dpath/test_h_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_e.$$

# ------------------------------------------------------------------------
# Test  29:  5D int value data flipped along height & time
# ------------------------------------------------------------------------
test_num=29
descr="5D int value data flipped along height & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -t

kimportasc -i1 $dpath/test_h_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_t.$$

# ------------------------------------------------------------------------
# Test  30:  5D int value data flipped along height, time, & elements
# ------------------------------------------------------------------------
test_num=30
descr="5D int value data flipped along height, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -t -e

kimportasc -i1 $dpath/test_h_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_t_e.$$

# ------------------------------------------------------------------------
# Test  31:  5D int value data flipped along height & depth
# ------------------------------------------------------------------------
test_num=31
descr="5D int value data flipped along height & depth"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -d

kimportasc -i1 $dpath/test_h_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_d.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_d.$$

# ------------------------------------------------------------------------
# Test  32:  5D int value data flipped along height, depth, & elements
# ------------------------------------------------------------------------
test_num=32
descr="5D int value data flipped along height, depth, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -d -e

kimportasc -i1 $dpath/test_h_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_d_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_d_e.$$

# ------------------------------------------------------------------------
# Test  33:  5D int value data flipped along height, depth, & time
# ------------------------------------------------------------------------
test_num=33
descr="5D int value data flipped along height, depth, & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -d -t

kimportasc -i1 $dpath/test_h_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_d_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_d_t.$$

# ------------------------------------------------------------------------
# Test  34:  5D int value data flipped along height, depth, time, & elements
# ------------------------------------------------------------------------
test_num=34
descr="5D int value data flipped along height, depth, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -h -d -t -e

kimportasc -i1 $dpath/test_h_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_h_d_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_h_d_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_h_d_t_e.$$

# ------------------------------------------------------------------------
# Test  35:  5D int value data flipped along width only
# ------------------------------------------------------------------------
test_num=35
descr="5D int value data flipped along width only"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w

kimportasc -i1 $dpath/test_w.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w.$$

# ------------------------------------------------------------------------
# Test  36:  5D int value data flipped along width & elements
# ------------------------------------------------------------------------
test_num=36
descr="5D int value data flipped along width & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -e

kimportasc -i1 $dpath/test_w_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_e.$$

# ------------------------------------------------------------------------
# Test  37:  5D int value data flipped along width & time
# ------------------------------------------------------------------------
test_num=37
descr="5D int value data flipped along width & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -t

kimportasc -i1 $dpath/test_w_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_t.$$

# ------------------------------------------------------------------------
# Test  38:  5D int value data flipped along width, time, & elements
# ------------------------------------------------------------------------
test_num=38
descr="5D int value data flipped along width, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -t -e

kimportasc -i1 $dpath/test_w_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_t_e.$$

# ------------------------------------------------------------------------
# Test  39:  5D int value data flipped along width & depth
# ------------------------------------------------------------------------
test_num=39
descr="5D int value data flipped along width & depth"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -d

kimportasc -i1 $dpath/test_w_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_d.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_d.$$

# ------------------------------------------------------------------------
# Test  40:  5D int value data flipped along width, depth, & elements
# ------------------------------------------------------------------------
test_num=40
descr="5D int value data flipped along width, depth, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -d -e

kimportasc -i1 $dpath/test_w_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_d_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_d_e.$$

# ------------------------------------------------------------------------
# Test  41:  5D int value data flipped along width, depth, & time
# ------------------------------------------------------------------------
test_num=41
descr="5D int value data flipped along width, depth, & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -d -t

kimportasc -i1 $dpath/test_w_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_d_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_d_t.$$

# ------------------------------------------------------------------------
# Test  42:  5D int value data flipped along width, depth, time, & elements
# ------------------------------------------------------------------------
test_num=42
descr="5D int value data flipped along width, depth, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -d -t -e

kimportasc -i1 $dpath/test_w_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_d_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_d_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_d_t_e.$$

# ------------------------------------------------------------------------
# Test  43:  5D int value data flipped along width & height
# ------------------------------------------------------------------------
test_num=43
descr="5D int value data flipped along width & height"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h

kimportasc -i1 $dpath/test_w_h.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h.$$

# ------------------------------------------------------------------------
# Test  44:  5D int value data flipped along width, height, & elements
# ------------------------------------------------------------------------
test_num=44
descr="5D int value data flipped along width, height, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -e

kimportasc -i1 $dpath/test_w_h_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_e.$$

# ------------------------------------------------------------------------
# Test  45:  5D int value data flipped along width, height, & time
# ------------------------------------------------------------------------
test_num=45
descr="5D int value data flipped along width, height, & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -t

kimportasc -i1 $dpath/test_w_h_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_t.$$

# ------------------------------------------------------------------------
# Test  46:  5D int value data flipped along width, height, time, & elements
# ------------------------------------------------------------------------
test_num=46
descr="5D int value data flipped along width, height, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -t -e

kimportasc -i1 $dpath/test_w_h_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_t_e.$$

# ------------------------------------------------------------------------
# Test  47:  5D int value data flipped along width, height, & depth
# ------------------------------------------------------------------------
test_num=47
descr="5D int value data flipped along width, height, & depth"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -d

kimportasc -i1 $dpath/test_w_h_d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_d.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_d.$$

# ------------------------------------------------------------------------
# Test  48:  5D int value data flipped along width, height, depth, & elements
# ------------------------------------------------------------------------
test_num=48
descr="5D int value data flipped along width, height, depth, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -d -e

kimportasc -i1 $dpath/test_w_h_d_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_d_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_d_e.$$

# ------------------------------------------------------------------------
# Test  49:  5D int value data flipped along width, height, depth, & time
# ------------------------------------------------------------------------
test_num=49
descr="5D int value data flipped along width, height, depth, & time"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -d -t

kimportasc -i1 $dpath/test_w_h_d_t.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_t.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_d_t.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_d_t.$$

# ------------------------------------------------------------------------
# Test  50:  5D int value data flipped along width, height, depth, time, 
#            & elements
# ------------------------------------------------------------------------
test_num=50
descr="5D int value data flipped along width, height, depth, time, & elements"

kimportasc -i1 $dpath/input_5d.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/input_5d.$$
${BINARY_PATH}/kflip -i $tmpdir/input_5d.$$ -o $tmpdir/kflip_out.$$ -w -h -d -t -e

kimportasc -i1 $dpath/test_w_h_d_t_e.ascii -so 0 -sf 0 -segment 'value' -dim 5 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -wsize 3 -hsize 3 -dsize 3 -tsize 3 -esize 3 -o $tmpdir/test_w_h_d_t_e.$$

report $test_num "$tmpdir/kflip_out.$$" "$tmpdir/test_w_h_d_t_e.$$" "$descr"
/bin/rm -f $tmpdir/input_5d.$$ $tmpdir/kflip_out.$$ $tmpdir/test_w_h_d_t_e.$$

exit $status
