#! /bin/sh

# Replace the oname in the following line with the name of the program
# you are testing.

object_name="kgate"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 59).
# ---------------------------------------------------------------

file_name=`echo $0 | sed -e 's/^.*\///'`
echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------
# need to test:
#	case where gate is a mask (should handle it, but 
#		kapu_upgrade_objects may pass back a fail state)
#	test for subobject position set on all 3 (verify that the 
#		attribute is actually getting set - ksegcmp doesn't
#		catch it)
#	test for subobject position disabled
#	map only case

# ------------------------------------------------------------------------
#  set path for test data, and temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  value data, different sized objects, logic = 1
# ------------------------------------------------------------------------
test_num=1
descr="value data, different sized objects, logic = 1"
${BINARY_PATH}/kgate -i1 $dpath/2x3.float.xv -i2 $dpath/3x4.int.viff \
      -igate $dpath/2x3gate.byte.viff -o $tmpdir/out.$$
report $test_num "$tmpdir/out.$$" "test1.results/2x3_3x4_2x3g_L1" "$descr" "1e-5"
# cp $tmpdir/out.$$ out.1
/bin/rm -f $tmpdir/out.$$

# ------------------------------------------------------------------------
# Test  2:  value data, different sized objects, logic = 0
# ------------------------------------------------------------------------
test_num=2
descr="value data, different sized objects, logic = 0"
${BINARY_PATH}/kgate -i1 $dpath/2x3.float.xv -i2 $dpath/3x4.int.viff \
      -igate $dpath/2x3gate.byte.viff -o $tmpdir/out.$$ -logic 0
report $test_num "$tmpdir/out.$$" "test1.results/2x3_3x4_2x3g_L0" "$descr" "1e-5"
# cp $tmpdir/out.$$ out.2
/bin/rm -f $tmpdir/out.$$

# ------------------------------------------------------------------------
# Test  3:  value data, different sized objects, logic = 0, gval = 1
# ------------------------------------------------------------------------
test_num=3         
descr="value data, different sized objects, logic = 0, gval = 1"
${BINARY_PATH}/kgate -i1 $dpath/2x3.float.xv -i2 $dpath/3x4.int.viff \
      -igate $dpath/2x3gate.byte.viff -o $tmpdir/out.$$ -logic 0 -gval 1
report $test_num "$tmpdir/out.$$" "test1.results/2x3_3x4_2x3g_L0-1" "$descr" "1e-5"
# cp $tmpdir/out.$$ out.3
/bin/rm -f $tmpdir/out.$$

# ------------------------------------------------------------------------
# Test  4:  value data, different sized objects, logic = 1, gval = -7
# ------------------------------------------------------------------------
test_num=4
descr="value data, different sized objects, logic = 1, gval = -7"
${BINARY_PATH}/kgate -i1 $dpath/2x3.float.xv -i2 $dpath/3x4.int.viff \
      -igate $dpath/2x3gate.byte.viff -o $tmpdir/out.$$ -logic 1 -gval -7
report $test_num "$tmpdir/out.$$" "test1.results/2x3_3x4_2x3g_L1-7" "$descr" "1e-5"
# cp $tmpdir/out.$$ out.4
/bin/rm -f $tmpdir/out.$$

# ------------------------------------------------------------------------
# Test  5: value, map, and mask data, different sized objects, logic = 0
# ------------------------------------------------------------------------
test_num=5
descr="value, map, and mask data, different size objects, logic=0"
${BINARY_PATH}/kgate -i1 $dpath/2x3ival_6x3fmap.viff \
	-i2 $dpath/3x4masked.byte.viff -igate $dpath/4x2gate.byte.xv \
	-o $tmpdir/out.$$ -logic 0
report $test_num "$tmpdir/out.$$" "test1.results/2x3vmp_3x4vmk_4x2g_L0" "$descr" "1e-5"
# cp $tmpdir/out.$$ out.5
/bin/rm -f $tmpdir/out.$$
# to run on command line:
# kgate -i1 ~/datamanip/data/testdata/2x3ival_6x3fmap.viff -i2 ~/datamanip/data/testdata/3x4masked.byte.viff -igate ~/datamanip/data/testdata/4x2gate.byte.xv -o junk -logic 0

# ------------------------------------------------------------------------
# Test  6:  test for subobject position set and disabled
# ------------------------------------------------------------------------

cat <<EOF > $tmpdir/ascii.$$
1 1 1 1 1 1 1 1
1 1 1 0 0 1 1 1
1 1 1 0 0 1 1 1
1 0 0 0 0 0 0 1
1 0 0 0 0 0 0 1
1 1 1 0 0 1 1 1
1 1 1 0 0 1 1 1
1 1 1 1 1 1 1 1
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 8 -hsize 8 -type "unsigned byte" \
	-o $tmpdir/gate.$$
ksetdattr -i $tmpdir/gate.$$ -o $tmpdir/gate.$$ -wpos 20 -hpos 30
kgconst -o $tmpdir/in1.$$ -wsize 64 -hsize 64 -real 255 -type "unsigned byte"
ksetdattr -i $tmpdir/in1.$$ -wpos 10 -hpos 5 -o $tmpdir/in1.$$

test_num=6a
descr="test for subobject position set"
${BINARY_PATH}/kgate -i1 $tmpdir/in1.$$ -i2 $tmpdir/gate.$$ \
	-igate $tmpdir/gate.$$ -o $tmpdir/result.$$ 
report $test_num "$tmpdir/result.$$" "test1.results/subpos-true.viff" \
	"$descr" "1e-5"
# cp $tmpdir/result.$$ out.4a

test_num=6b
descr="test for subobject position disabled"
${BINARY_PATH}/kgate -i1 $tmpdir/in1.$$ -i2 $tmpdir/gate.$$ \
	-igate $tmpdir/gate.$$ -o $tmpdir/result.$$ -subpos 0
report $test_num "$tmpdir/result.$$" "test1.results/subpos-false.viff" \
	"$descr" "1e-5"
# cp $tmpdir/result.$$ out.4b

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/gate.$$ $tmpdir/in1.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  7:  all objects have only maps 
# ------------------------------------------------------------------------
test_num=7
descr="all objects have only maps"

cat <<EOF > $tmpdir/ascii.$$
1 2 3
4 5 6
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 2 -type "byte" \
	-o $tmpdir/in1.$$

cat <<EOF > $tmpdir/ascii.$$
7 8
9 10
11 12
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type "integer" \
	-o $tmpdir/in2.$$

cat <<EOF > $tmpdir/ascii.$$
0 1
1 0
0 1
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type "byte" \
	-o $tmpdir/gate.$$

cat <<EOF > $tmpdir/ascii.$$
7 2 3
4 10 6
11 0 0
EOF
kasc2map -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 3 -type "integer" \
	-o $tmpdir/answer.$$

${BINARY_PATH}/kgate -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
	-igate $tmpdir/gate.$$ -o $tmpdir/result.$$  

report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
# cp $tmpdir/result.$$ result.7

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/in1.$$ $tmpdir/in2.$$ $tmpdir/gate.$$
/bin/rm -f $tmpdir/result.$$ $tmpdir/answer.$$

# ------------------------------------------------------------------------
# Test  8:  using mask instead of value for gate object
# ------------------------------------------------------------------------
test_num=8
descr="using mask instead of value for gate object"
cat <<EOF > $tmpdir/ascii.$$
1 2 3
4 5 6
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 2 -type "unsigned byte" \
	-o $tmpdir/in1.$$

cat <<EOF > $tmpdir/ascii.$$
7 8
9 10
11 12
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type "integer" \
	-o $tmpdir/in2.$$

cat <<EOF > $tmpdir/ascii.$$
0 1
1 0
0 1
EOF
kasc2mask -i1 $tmpdir/ascii.$$ -wsize 2 -hsize 3 -type "byte" \
	-o $tmpdir/gate.$$

cat <<EOF > $tmpdir/ascii.$$
7 2 3
4 10 6
11 0 0
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 3 -type "integer" \
	-o $tmpdir/answer.$$

${BINARY_PATH}/kgate -i1 $tmpdir/in1.$$ -i2 $tmpdir/in2.$$ \
	-igate $tmpdir/gate.$$ -o $tmpdir/result.$$

report $test_num "$tmpdir/result.$$" "$tmpdir/answer.$$" "$descr" "1e-5"
# cp $tmpdir/result.$$ result.8
/bin/rm -f $tmpdir/ascii.$$ $tmpdir/in1.$$ $tmpdir/in2.$$ $tmpdir/gate.$$
/bin/rm -f $tmpdir/result.$$ $tmpdir/answer.$$

# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
