#! /bin/sh

echo "running test1.sh for kimportasc ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set data path for image data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/testsuite/kimportasc/test1.inputs"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  data imported as bit w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=1
descr="data imported as bit w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
0 1 0 1
0 0 0 0
1 1 1 1
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 1 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 2 -sf 1 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
0 0 0 1 1
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 1 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  2:  data imported as byte w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=2
descr="data imported as byte w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 2 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  3:  data imported as unsigned byte w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=3
descr="data imported as ubyte w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 3 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 3 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  4:  data imported as short w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=4
descr="data imported as short w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 4 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 4 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  5:  data imported as ushort w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=5
descr="data imported as ushort w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 5 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  6:  data imported as integer w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=6
descr="data imported as integer w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 6 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 6 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  7:  data imported as uinteger w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=7
descr="data imported as uinteger w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 7 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 7 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  8:  data imported as long w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=8
descr="data imported as long w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 8 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 8 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  9:  data imported as ulong w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=9
descr="data imported as ulong w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 9 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 9 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  10:  data imported as float w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=10
descr="data imported as float w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 10 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 10 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  11:  data imported as double w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=11
descr="data imported as double w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 11 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 1 -sf 3 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
2
2
2
2
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 11 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  12:  data imported as complex w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=12
descr="data imported as complex w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2) (3,3) (4,4)
(5,5) (6,6) (7,7) (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 12 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 2 -sf 1 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
(3,3) (5,5) (7,7) (9,9) (11,11)
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 12 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  13:  data imported as double complex w/offset & skip factor (whdte)
# ------------------------------------------------------------------------
test_num=13
descr="data imported as double complex w/offset & skip factor (whdte)"
cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2) (3,3) (4,4)
(5,5) (6,6) (7,7) (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 13 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 2 -sf 1 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
(3,3) (5,5) (7,7) (9,9) (11,11)
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 5 -hsize 1 -dsize 1 -tsize 1 -esize 1 -type 13 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  14:  data imported as bit (whdte)
# ------------------------------------------------------------------------
test_num=14
descr="data imported as bit (whdte)"
cat <<EOF > $tmpdir/ascii.$$
0 1 0 1
0 0 1 1
1 1 0 0
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 1 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
0 1 0 1
0 0 1 1
1 1 0 0
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 1 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  15:  data imported as byte (whdte)
# ------------------------------------------------------------------------
test_num=15
descr="data imported as byte (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 2 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  16:  data imported as ubyte (whdte)
# ------------------------------------------------------------------------
test_num=16
descr="data imported as ubyte (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 3 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 3 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  17:  data imported as short (whdte)
# ------------------------------------------------------------------------
test_num=17
descr="data imported as short (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 4 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 4 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  18:  data imported as ushort (whdte)
# ------------------------------------------------------------------------
test_num=18
descr="data imported as ushort (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 5 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  19:  data imported as integer (whdte)
# ------------------------------------------------------------------------
test_num=19
descr="data imported as integer (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 6 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 6 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  20  data imported as uinteger (whdte)
# ------------------------------------------------------------------------
test_num=20
descr="data imported as uinteger (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 7 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 7 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  21  data imported as long (whdte)
# ------------------------------------------------------------------------
test_num=21
descr="data imported as long (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 8 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 8 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  22  data imported as ulong (whdte)
# ------------------------------------------------------------------------
test_num=22
descr="data imported as ulong (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 9 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 9 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  23  data imported as float (whdte)
# ------------------------------------------------------------------------
test_num=23
descr="data imported as float (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 10 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 10 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  24  data imported as double (whdte)
# ------------------------------------------------------------------------
test_num=24
descr="data imported as double (whdte)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 11 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 11 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  25  data imported as complex (whdte)
# ------------------------------------------------------------------------
test_num=25
descr="data imported as complex (whdte)"
cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2)   (3,3)   (4,4)
(5,5) (6,6)   (7,7)   (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 12 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2)   (3,3)   (4,4)
(5,5) (6,6)   (7,7)   (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 12 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  26  data imported as double complex (whdte)
# ------------------------------------------------------------------------
test_num=26
descr="data imported as double complex (whdte)"
cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2)   (3,3)   (4,4)
(5,5) (6,6)   (7,7)   (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 13 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
(1,1) (2,2)   (3,3)   (4,4)
(5,5) (6,6)   (7,7)   (8,8)
(9,9) (10,10) (11,11) (12,12)
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 13 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  27  3D (w,h,d) data imported as bit (dwhte) (3x3x4x1x1)
# ------------------------------------------------------------------------
test_num=27
descr="3D (w,h,d) data imported as bit (dwhte) (3x3x4x1x1)"
cat <<EOF > $tmpdir/ascii.$$
0 0 0 0
0 0 0 1
0 0 1 0

0 0 1 1
0 1 0 0
0 1 0 1

0 1 1 0
0 1 1 1
1 0 0 0
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -dsize 3 -wsize 3 -hsize 4 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 1 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
0 0 0 0
0 0 0 1
0 0 1 0

0 0 1 1
0 1 0 0
0 1 0 1

0 1 1 0
0 1 1 1
1 0 0 0
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 3 -tsize 1 -esize 1 -type 1 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  28  3D (w,h,d) data imported as byte (dwhte) (3x3x4x1x1)
# ------------------------------------------------------------------------
test_num=28
descr="3D (w,h,d) data imported as byte (dwhte) (3x3x4x1x1)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12

13 14 15 16
17 18 19 20
21 22 23 24

25 26 27 28
29 30 31 32
33 34 35 36
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -dsize 3 -wsize 3 -hsize 4 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12

13 14 15 16
17 18 19 20
21 22 23 24

25 26 27 28
29 30 31 32
33 34 35 36
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 3 -tsize 1 -esize 1 -type 2 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  29  4D (w,h,d,t) data imported as byte (dwhte) size = (3x3x4x2x1)
# ------------------------------------------------------------------------
test_num=29
descr="4D (w,h,d,t) data imported as byte (dwhte) size = (3x3x4x2x1)"
cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12

13 14 15 16
17 18 19 20
21 22 23 24

25 26 27 28
29 30 31 32
33 34 35 36

37 38 39 40
41 42 43 44
45 46 47 48

49 50 51 52
53 54 55 56
57 58 59 60

61 62 63 64
65 66 67 68
69 70 71 72
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -dsize 3 -wsize 3 -hsize 4 -tsize 2 -esize 1 -o $tmpdir/output.$$ -type 2 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -dim 5 -segment value

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12

13 14 15 16
17 18 19 20
21 22 23 24

25 26 27 28
29 30 31 32
33 34 35 36

37 38 39 40
41 42 43 44
45 46 47 48

49 50 51 52
53 54 55 56
57 58 59 60

61 62 63 64
65 66 67 68
69 70 71 72
EOF

${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 3 -tsize 2 -esize 1 -type 2 -dindex 1 -windex 2 -hindex 3 -tindex 4 -eindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  30  5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2)
# ------------------------------------------------------------------------
test_num=30
descr="5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2)"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 3 -wsize 3 -hsize 4 -dsize 3 -tsize 2 -o $tmpdir/output.$$ -type 2 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value

${BINARY_PATH}/kasc2val -i1 $dpath/test_e_w_h_d_t.ascii -wsize 3 -hsize 4 -dsize 3 -tsize 2 -esize 3 -type 2 -dindex 4 -windex 2 -hindex 3 -tindex 5 -eindex 1 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/output.$$ 

# ------------------------------------------------------------------------
# Test  31  5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2)
#           w/offset & skip factor
# ------------------------------------------------------------------------
test_num=31
descr="5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2) w/offset & skip factor"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 1 -wsize 3 -hsize 4 -dsize 3 -tsize 2 -o $tmpdir/output.$$ -type 2 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value -so 0 -sf 2

cat <<EOF > $tmpdir/ascii.$$
1 2  3  4
5 6  7  8
9 10 11 12
 
13 14 15 16
17 18 19 20
21 22 23 24
 
25 26 27 28
29 30 31 32
33 34 35 36
 
37 38 39 40
41 42 43 44
45 46 47 48
 
49 50 51 52
53 54 55 56
57 58 59 60
 
61 62 63 64
65 66 67 68
69 70 71 72
EOF
 
${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 3 -tsize 2 -esize 1 -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$

# ------------------------------------------------------------------------
# Test  32  5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2)
#           w/offset & skip factor
# ------------------------------------------------------------------------
test_num=32
descr="5D (w,h,d,t,e) data imported as byte (ewhdt) size = (3x3x4x3x2) w/offset & skip factor"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 1 -wsize 3 -hsize 4 -dsize 3 -tsize 2 -o $tmpdir/output.$$ -type 2 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value -so 1 -sf 2

cat <<EOF > $tmpdir/ascii.$$
73 75 77 79
81 83 85 87
89 91 93 95

97  99  101 103
105 107 109 111
113 115 117 119

121 123 125 127
129 131 133 135
137 139 141 143

145 147 149 151
153 155 157 159
161 163 165 167

169 171 173 175
177 179 181 183
185 187 189 191

193 195 197 199
201 203 205 207
209 211 213 215
EOF
 
${BINARY_PATH}/kasc2val -i1 $tmpdir/ascii.$$ -wsize 3 -hsize 4 -dsize 3 -tsize 2 -esize 1 -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/ascii.$$ $tmpdir/result.$$ $tmpdir/output.$$

# ------------------------------------------------------------------------
# Test  33  5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x17x4x1x1)
#           w/offset only
# ------------------------------------------------------------------------
test_num=33
descr="5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x17x4x1x1) w/offset only"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 3 -wsize 17 -hsize 4 -dsize 1 -tsize 1 -o $tmpdir/output.$$ -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value -so 12 -sf 0

${BINARY_PATH}/kasc2val -i1 $dpath/test_offset_no_skip.ascii -wsize 17 -hsize 4 -dsize 1 -tsize 1 -esize 3 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/output.$$


# ------------------------------------------------------------------------
# Test  34  5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x9x2x1x1)
#           w/number read & skip
# ------------------------------------------------------------------------
test_num=34
descr="5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x9x2x1x1) w/num read & skip"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 3 -wsize 9 -hsize 2 -dsize 1 -tsize 1 -o $tmpdir/output.$$ -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value -so 0 -sf 9 -nr 3

${BINARY_PATH}/kasc2val -i1 $dpath/test_num_read_skip.ascii -wsize 9 -hsize 2 -dsize 1 -tsize 1 -esize 3 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/output.$$

# ------------------------------------------------------------------------
# Test  35  5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x5x5x1x1)
#           w/number read & skip
# ------------------------------------------------------------------------
test_num=35
descr="5D (w,h,d,t,e) data imported as int (ewhdt) size = (3x5x5x1x1) w/num read & skip"

${BINARY_PATH}/kimportasc -i1 $dpath/test_e_w_h_d_t.ascii -esize 3 -wsize 5 -hsize 5 -dsize 1 -tsize 1 -o $tmpdir/output.$$ -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -dim 5 -segment value -so 3 -sf 9 -nr 5

${BINARY_PATH}/kasc2val -i1 $dpath/test_offset_num_read_skip.ascii -wsize 5 -hsize 5 -dsize 1 -tsize 1 -esize 3 -type 6 -eindex 1 -windex 2 -hindex 3 -dindex 4 -tindex 5 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/result.$$ $tmpdir/output.$$

# ------------------------------------------------------------------------
# Test  36:  data imported as byte (whdte) to map
# ------------------------------------------------------------------------
test_num=36
descr="data imported as byte (whdte) to map"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 0 -sf 0 -dim 5 -segment map

cat <<EOF > $tmpdir/ascii.$$
1 2 3 4
1 2 3 4
1 2 3 4
1 2 3 4
EOF

${BINARY_PATH}/kasc2map -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 2 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  37:  data imported as byte (whdte) to mask
# ------------------------------------------------------------------------
test_num=37
descr="data imported as byte (whdte) to mask"
cat <<EOF > $tmpdir/ascii.$$
1 0 1 0
0 0 0 0
0 1 0 1
1 1 1 1
EOF

${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 1 -tsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -windex 1 -hindex 2 -dindex 3 -tindex 4 -eindex 5 -so 0 -sf 0 -dim 5 -segment mask

cat <<EOF > $tmpdir/ascii.$$
1 0 1 0
0 0 0 0
0 1 0 1
1 1 1 1
EOF

${BINARY_PATH}/kasc2mask -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 1 -tsize 1 -esize 1 -type 2 -o $tmpdir/result.$$

report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"

/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  38:  data imported as byte (t) to time
# ------------------------------------------------------------------------
test_num=38
descr="data imported as byte (t) to time"
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4 5 6 7 8 9 10
EOF
 
${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -tsize 10 -o $tmpdir/output.$$ -type 2 -so 0 -sf 0 -dim 1 -segment time -tindex 1

cat <<EOF > $tmpdir/ascii.$$
1 2 3 4 5 6 7 8 9 10
EOF
 
${BINARY_PATH}/kasc2time -i1 $tmpdir/ascii.$$ -tsize 10 -type 2 -o $tmpdir/result.$$
 
report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"
 
/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  39:  data imported as byte (whdn) to loc
# ------------------------------------------------------------------------
test_num=39
descr="data imported as byte (whdn) to loc"
cat <<EOF > $tmpdir/ascii.$$
1  2  3  4
5  6  7  8
9  10 11 12
13 14 15 16
17 18 19 20
EOF
 
${BINARY_PATH}/kimportasc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 5 -dsize 1 -esize 1 -o $tmpdir/output.$$ -type 2 -so 0 -sf 0 -dim 4 -segment location -windex 1 -hindex 2 -dindex 3 -eindex 4

cat <<EOF > $tmpdir/ascii.$$
1  2  3  4
5  6  7  8
9  10 11 12
13 14 15 16
17 18 19 20
EOF
 
${BINARY_PATH}/kasc2loc -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 5 -dsize 1 -nsize 1 -type 2 -windex 1 -hindex 2 -dindex 3 -nindex 4 -o $tmpdir/result.$$
 
report $test_num "$tmpdir/output.$$" "$tmpdir/result.$$" "$descr"
 
/bin/rm -f $tmpdir/output.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

exit $status
