#! /bin/sh

echo "testing kimportraw  ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
datapath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  4x2 big endian IEEE float into value segment
# ------------------------------------------------------------------------
test_num=1
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -o $tmpdir/raw.$$ -wsize 4 -hsize 2 -type float -arch 3 -segment 1
descr="Big Endian IEEE:  4x2 float data set into value segment"
report $test_num "$tmpdir/raw.$$" "test1.results/4x2.float.value" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  2:  4x2 big endian IEEE float into mask segment
# ------------------------------------------------------------------------
test_num=2
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -o $tmpdir/raw.$$ -wsize 4 -hsize 2 -type float -arch 3 -segment 2
descr="Big Endian IEEE:  4x2 float data set into mask segment"
report $test_num "$tmpdir/raw.$$" "test1.results/4x2.float.mask" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  3:  4x2 big endian IEEE float into map segment
# ------------------------------------------------------------------------
test_num=3
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -o $tmpdir/raw.$$ -wsize 4 -hsize 2 -type float -arch 3 -segment 3
descr="Big Endian IEEE:  4x2 float data set into map segment"
report $test_num "$tmpdir/raw.$$" "test1.results/4x2.float.map" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  4:  4x2 big endian IEEE float into location segment
# ------------------------------------------------------------------------
test_num=4
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -o $tmpdir/raw.$$ -wsize 4 -hsize 2 -type float -arch 3 -segment 4
descr="Big Endian IEEE:  4x2 float data set into location segment"
report $test_num "$tmpdir/raw.$$" "test1.results/4x2.float.location" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  5:  8 big endian IEEE floats into time segment
# ------------------------------------------------------------------------
test_num=5
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -o $tmpdir/raw.$$ -tsize 8 -type float -arch 3 -segment 5
descr="Big Endian IEEE:  8 floats into time segment"
report $test_num "$tmpdir/raw.$$" "test1.results/8.float.time" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  6:  4x2 big endian IEEE floats into mask segment of another data set
# ------------------------------------------------------------------------
test_num=6
${BINARY_PATH}/kimportraw -i1 $datapath/4x2.ieeebe.float.raw -i2 $datapath/4x2.float.xv -o $tmpdir/raw.$$ -wsize 4 -hsize 2 -type float -arch 3 -segment 2
descr="Big Endian IEEE:  4x2 floats into mask of existing data set"
report $test_num "$tmpdir/raw.$$" "test1.results/4x2.float.insert" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  7:  128x128 SPARC 32-bit ints value segment
#           This data provided by tilton@chrpisis.gsfc.nasa.gov
#           (James C. Tilton). It tickled a problem in which the bit patterns
#           in the raw data caused the format checking code to claim that
#	    the data wasn't raw. This should no longer happen!
# ------------------------------------------------------------------------
test_num=7
${BINARY_PATH}/kimportraw -i1 $datapath/vdrf.npixmap.raw -o $tmpdir/raw.$$ -wsize 128 -hsize 128 -type integer -arch 3 -segment 1
descr="128x128 SPARC 32-bit ints value segment"
report $test_num "$tmpdir/raw.$$" "test1.results/vdrf.npixmap.import" "$descr"
/bin/rm -f $tmpdir/raw.$$

# ------------------------------------------------------------------------
# Test  8:  Read into comment field, also tests header skip
#	    This test is a little bogus 'cause we can't check the comment
#	    field without also checking the date; this would cause the
#	    test to always fail! I did check it by hand though, on a
#           Sparc5 under solaris. SRW
# ------------------------------------------------------------------------
test_num=8
descr="Read into comment field, also tests header skip"

cat > $tmpdir/in.$$ << EOF
abcdefghijkl
EOF

${BINARY_PATH}/kimportraw -i1 $tmpdir/in.$$ -o $tmpdir/out.$$ -wsize 4 -hsize 1 -segment 6 -skip 3
# kprdata -attr -i $tmpdir/out.$$
${BINARY_PATH}/kgconst -o $tmpdir/com.$$ 
${BINARY_PATH}/krmseg -i $tmpdir/com.$$ -o $tmpdir/com.$$ -val
${BINARY_PATH}/kcomment -i $tmpdir/com.$$ -o $tmpdir/com.$$ -s defg -re
# kprdata -attr -i $tmpdir/com.$$

report $test_num "$tmpdir/out.$$" "$tmpdir/com.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/in.$$ $tmpdir/com.$$

# ------------------------------------------------------------------------
# Test  9:  unsigned byte into value segment, test header skip
# ------------------------------------------------------------------------
test_num=9
descr="Read into value segment, test header skip"
 
cat > $tmpdir/in.$$ << EOF
123456789
EOF
 
${BINARY_PATH}/kimportraw -i1 $tmpdir/in.$$ -o $tmpdir/out.$$ -wsize 4 -hsize 1 -segment 1 -skip 3 -arch 1 -type 3 

cat > $tmpdir/in.$$ << EOF
52 53 54 55
EOF
${BINARY_PATH}/kconvert -i $tmpdir/in.$$ -o $tmpdir/in.$$ -val -type 3
 
report $test_num "$tmpdir/out.$$" "$tmpdir/in.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/in.$$

exit $status
