#! /bin/sh 
 
echo "running test4.sh for klinearop (subregion testing) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: correlation, Double 1D - W, (0) hotspot, subregion at start
# ------------------------------------------------------------------------
test_num=1
descr="correlation, Double 1D - W, (0) hotspot, subregion at start" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -wsize 4

cat > $tmpdir/out2.$$ << EOF
0 3 5 6 1 1 1 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: correlation, Double 1D - W, (1) hotspot, subregion at start
# ------------------------------------------------------------------------
test_num=2
descr="correlation, Double 1D - W, (1) hotspot, subregion at start" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -wc 1 -wsize 4 

cat > $tmpdir/out2.$$ << EOF
0 0 3 5 1 1 1 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: correlation, Double 1D - W, (2) hotspot, subregion at start
# ------------------------------------------------------------------------
test_num=3
descr="correlation, Double 1D - W, (2) hotspot, subregion at start" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 1 -l -real 0 -imag 0 -wc 2 -wsize 4 

cat > $tmpdir/out2.$$ << EOF
0 0 0 3 1 1 1 1
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  4: convolution, Double 1D - W, (0) hotspot, subregion at end
# ------------------------------------------------------------------------
test_num=4
descr="convolution, Double 1D - W, (0) hotspot, subregion at end" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wsize 4 -woff 4

cat > $tmpdir/out2.$$ << EOF
0 0 0 1 3 6 6 6
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  5: convolution, Double 1D - W, (1) hotspot, subregion at end
# ------------------------------------------------------------------------
test_num=5
descr="convolution, Double 1D - W, (1) hotspot, subregion at end" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 1 -wsize 4 -woff 4

cat > $tmpdir/out2.$$ << EOF
0 0 0 1 6 6 6 5
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  6: convolution, Double 1D - W, (2) hotspot, subregion at end
# ------------------------------------------------------------------------
test_num=6
descr="convolution, Double 1D - W, (2) hotspot, subregion at end" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 1 1 1 1
EOF

cat > $tmpdir/patt.$$ << EOF
1 2 3
EOF

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -wc 2 -wsize 4 -woff 4

cat > $tmpdir/out2.$$ << EOF
0 0 0 1 6 6 5 3
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

exit $status
