#! /bin/sh 
 
echo "running test7.sh for klinearop (data type testing) ... process ($$)"
echo " "
 
# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
 
ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -rt 1 -all -s
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}
 
#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0
 
# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
bpath=`kecho -tb DATAMANIP -echo binpath`
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/"

# ------------------------------------------------------------------------
# Test  1: convolution, double 1D
# ------------------------------------------------------------------------
test_num=1
descr="convolution, double 1D" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 11

cat > $tmpdir/patt.$$ << EOF
1e3 2e3 3e3
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 11

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 1e3 2e3 3e3 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 11

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  2: convolution, ubyte 1D
# ------------------------------------------------------------------------
test_num=2
descr="convolution, ubyte 1D" 

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 3

cat > $tmpdir/patt.$$ << EOF
10 20 30
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 3

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 10 20 30 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 3

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  3: convolution, int 1D
# ------------------------------------------------------------------------
test_num=3
descr="convolution, int 1D"

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 6

cat > $tmpdir/patt.$$ << EOF
-16384 32768 49152
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 6

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 -16384 32768 49152 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 6

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  4: convolution, uint 1D
# ------------------------------------------------------------------------
test_num=4
descr="convolution, uint 1D"

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 7

cat > $tmpdir/patt.$$ << EOF
16384 32768 49152
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 7

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 16384 32768 49152 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 7

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  5: convolution, float 1D
# ------------------------------------------------------------------------
test_num=5
descr="convolution, float 1D"

cat > $tmpdir/imp.$$ << EOF
0 0 0 1 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 10

cat > $tmpdir/patt.$$ << EOF
2.7 -7.3 1234.567
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 10

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
0 0 0 2.7 -7.3 1234.567 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 10

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  6: convolution, complex 1D
# ------------------------------------------------------------------------
test_num=6
descr="convolution, complex 1D"

cat > $tmpdir/imp.$$ << EOF
(0,0) (0,0) (0,0) (1,0) (0,0) (0,0) (0,0) (0,0)
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 12

cat > $tmpdir/patt.$$ << EOF
(2.7,0) (-7.3,0) (1234.567,0)
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 12

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
(0,0) (0,0) (0,0) (2.7,0) (-7.3,0) (1234.567,0) (0,0) (0,0)
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 12

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  7: convolution, dcomplex 1D
# ------------------------------------------------------------------------
test_num=7
descr="convolution, dcomplex 1D"

cat > $tmpdir/imp.$$ << EOF
(0,0) (0,0) (0,0) (1,0) (0,0) (0,0) (0,0) (0,0)
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 13

cat > $tmpdir/patt.$$ << EOF
(2.7,0) (-7.3,0) (1234.567,0)
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 13

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0

cat > $tmpdir/out2.$$ << EOF
(0,0) (0,0) (0,0) (2.7,0) (-7.3,0) (1234.567,0) (0,0) (0,0)
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 13

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

# ------------------------------------------------------------------------
# Test  8: convolution, ushort 1D, upcast
# ------------------------------------------------------------------------
test_num=8
descr="convolution, ushort 1D, upcast"

cat > $tmpdir/imp.$$ << EOF
0 0 0 10 0 0 0 0
EOF
kconvert -i $tmpdir/imp.$$ -o $tmpdir/imp.$$ -val -type 5

cat > $tmpdir/patt.$$ << EOF
16384 32768 49152
EOF
kconvert -i $tmpdir/patt.$$ -o $tmpdir/patt.$$ -val -type 5

$bpath/klinearop -i1 $tmpdir/imp.$$ -i2 $tmpdir/patt.$$ -o $tmpdir/out.$$ \
-corr 0 -l -real 0 -imag 0 -upcast

cat > $tmpdir/out2.$$ << EOF
0 0 0 163840 327680 491520 0 0
EOF
kconvert -i $tmpdir/out2.$$ -o $tmpdir/out2.$$ -val -type 11

report $test_num "$tmpdir/out.$$" "$tmpdir/out2.$$" "$descr"

/bin/rm -f $tmpdir/imp.$$ $tmpdir/patt.$$ $tmpdir/out.$$ $tmpdir/out2.$$

exit $status
