#! /bin/sh

echo "running test1.sh for klogexp ... process ($$)" 

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  log - double input, log10 no offset
# ------------------------------------------------------------------------
test_num=1
descr="log - double input, log10 no offset"

cat > $tmpdir/in.$$ << EOF
         .01  1  2  3  
        10  11  12  13  
        20  21  22  23  
EOF
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -b10 -offset 0

cat > $tmpdir/result.$$ << EOF
-2.0 0.0 0.30103 0.47712125
1.0 1.0413927 1.079182 1.1139434
1.30103 1.3222193 1.3424227 1.3617278
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  2:  log - double input, log10 no offset
# ------------------------------------------------------------------------
test_num=2
descr="log - double input, log10 no offset"

cat > $tmpdir/in.$$ << EOF
         0  1  
       -10  11  
        20 -21  
       -30  31  
EOF
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -b10 -offset 0

cat > $tmpdir/result.$$ << EOF
-1.0e38 0
-1.0e38 1.0413927
1.30103 -1.0e38
-1.0e38 1.4913617
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  3:  log - double input, ln 1 offset
# ------------------------------------------------------------------------
test_num=3
descr="log - double input, ln 1 offset"

cat > $tmpdir/in.$$ << EOF
         0  1  
       -10  11  
        20 -21  
       -30  31  
EOF
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -be -offset 1

cat > $tmpdir/result.$$ << EOF
0.0 0.6931471806  
-1.0e38 2.48490665  
3.044522438 -1.0e38  
-1.0e38 3.465735903  
EOF

report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  4:  log - double input, log base 3.21 no offset
# ------------------------------------------------------------------------
test_num=4
descr="log - double input, log base 3.21 no offset"
 
cat > $tmpdir/in.$$ << EOF
         0  1
        10  11
        20 21
        .007  31
EOF
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -offset 0 \
-base 3.21
 
cat > $tmpdir/result.$$ << EOF
        -1.0e38 0.0
        1.974314 2.0560362
        2.5686418 2.6104761
        -4.2544532 2.9444163
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  5:  exp - double input, natural base, 1 offset
# ------------------------------------------------------------------------
test_num=5
descr="exp - double input, natural base, 1 offset"
cat > $tmpdir/in.$$ << EOF
         0  -1
	.003 1.5e-7
	-.0001 3
EOF
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -offset 1 -inv \
-be
 
cat > $tmpdir/result.$$ << EOF
	0	-.6321206
	0.0030045	0.0000002
	-0.0001000	19.0855369
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  6:  exp - float input, natural base, 1 offset
# ------------------------------------------------------------------------
test_num=6
descr="exp - float input, natural base, 1 offset"
cat > $tmpdir/in.$$ << EOF
         0  -1
        .003 1.5e-7
        -.0001 3
EOF

kconvert -i $tmpdir/in.$$ -o $tmpdir/in.$$ -val -type float
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -offset 1 -inv \
-be
 
cat > $tmpdir/result.$$ << EOF
        0       -.6321206
        0.0030045       0.0000002
        -0.0001000      19.0855369
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  7:  exp - bit input, natural base, 1 offset
# ------------------------------------------------------------------------
test_num=7
descr="exp - bit input, natural base, 1 offset"
cat > $tmpdir/in.$$ << EOF
         0  1
         1  0
         1  1
EOF
 
kconvert -i $tmpdir/in.$$ -o $tmpdir/in.$$ -val -type bit
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -offset 1 -inv \
-be
 
cat > $tmpdir/result.$$ << EOF
	0	1.7182818
	1.7182818	0
	1.7182818	1.7182818
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  8:  exp - complex input, natural base
# ------------------------------------------------------------------------
test_num=8
descr="exp - complex input, natural base"
cat > $tmpdir/in.$$ << EOF
(1.0,0.0) (0.0,1.0)
(-1.0,0.0) (0.0,-1.0)
(-1.0,-1.0) (-1.0,1.0)
(1.0,-1.0) (1.0,1.0)
EOF
 
kasc2val -i1 $tmpdir/in.$$ -o $tmpdir/in.$$ -wsize 2 -hsize 4 -type 12
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$  -inv -be -offset 0
 
cat > $tmpdir/result.$$ << EOF
(2.71828,0.) (0.540302,0.841471)
(0.367879,0.) (0.540302,-0.841471)
(0.198766,-0.309560) (0.198766,0.309560)
(1.46869,-2.28736) (1.46869,2.28736)
EOF
kasc2val -i1 $tmpdir/result.$$ -o $tmpdir/result.$$ -wsize 2 -hsize 4 -type 13
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  9:  log - complex input, natural base
# ------------------------------------------------------------------------
test_num=9
descr="log - complex input, natural base"
cat > $tmpdir/in.$$ << EOF
(1.0,0.0) (0.0,1.0)
(-1.0,0.0) (0.0,-1.0)
(-1.0,-1.0) (-1.0,1.0)
(1.0,-1.0) (1.0,1.0)
EOF
 
kasc2val -i1 $tmpdir/in.$$ -o $tmpdir/in.$$ -wsize 2 -hsize 4 -type 12
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$  -be -offset 0
 
cat > $tmpdir/result.$$ << EOF
(0.0,0.0) (0.0,1.57080)
(0.0,3.14159) (0.0,-1.57080)
(0.346574,-2.35619) (0.346574,2.35619)
(0.346574,-0.785398) (0.346574,0.785398)
EOF
kasc2val -i1 $tmpdir/result.$$ -o $tmpdir/result.$$ -wsize 2 -hsize 4 -type 13
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$

# ------------------------------------------------------------------------
# Test  10:  log - int value data, double map, log10 no offset
# ------------------------------------------------------------------------
test_num=10
descr="log - int value data,  double map, log10 no offset"
 
cat > $tmpdir/inval.$$ << EOF
1 2 3 4
5 6 7 8
9 10 11 12
EOF

cat > $tmpdir/inmap.$$ << EOF
.01
1
2
3
10
11
12
13
20
21
22
23
EOF
kasc2map -i1 $tmpdir/inmap.$$ -i2 $tmpdir/inval.$$ -wsize 1 -hsize 12 \
-type double -o $tmpdir/in.$$

${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -b10 -offset 0
 
cat > $tmpdir/resultmap.$$ << EOF
-2.0
0.0
0.30103
0.47712125
1.0
1.0413927
1.079182
1.1139434
1.30103
1.3222193
1.3424227
1.3617278
EOF

kasc2map -i1 $tmpdir/resultmap.$$ -i2 $tmpdir/inval.$$ -wsize 1 -hsize 12 \
-type double -o $tmpdir/result.$$
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$
/bin/rm -f $tmpdir/inmap.$$ $tmpdir/inval.$$ 
/bin/rm -f $tmpdir/resultmap.$$ 

# ------------------------------------------------------------------------
# Test  11:  log - double input, log10 no offset, large data set test
# ------------------------------------------------------------------------
test_num=11
descr="log - double input, log10 no offset, large data set test"
 
cat > $tmpdir/in.$$ << EOF
         .01  1  2  3
        10  11  12  13
        20  21  22  23
EOF

set KHOROS_KDMS_BUFSIZE=20
export KHOROS_KDMS_BUFSIZE
${BINARY_PATH}/klogexp -i $tmpdir/in.$$ -o $tmpdir/out.$$ -b10 -offset 0
unset KHOROS_KDMS_BUFSIZE
 
cat > $tmpdir/result.$$ << EOF
-2.0 0.0 0.30103 0.47712125
1.0 1.0413927 1.079182 1.1139434
1.30103 1.3222193 1.3424227 1.3617278
EOF
 
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.$$ $tmpdir/result.$$
 

exit $status
