#! /bin/sh

echo "running test6.sh for knormal ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
tolerance="$5"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tolerance" -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0


# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  Check normalization of ulong [0..212] to 0..255. This is to make 
#           sure a specific bug is gone.
# ------------------------------------------------------------------------
test_num=1
descr="Check normalization of ulong [0..212] to 0..255"
cat > $tmpdir/asc.$$ <<EOF
0 212
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -val -type 9
${BINARY_PATH}/knormal -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -whole
cat > $tmpdir/asc.$$ <<EOF
0 255
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -val -type 9
 
report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

# ------------------------------------------------------------------------
# Test  2:  Check normalization of ulong [1..212] to 0..255. This is to make 
#           sure a specific bug is gone.
# ------------------------------------------------------------------------
test_num=2
descr="Check normalization of ulong [1..212] to 0..255"
cat > $tmpdir/asc.$$ <<EOF
1 212
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -val -type 9
${BINARY_PATH}/knormal -i $tmpdir/asc.$$ -o $tmpdir/out.$$ -whole
cat > $tmpdir/asc.$$ <<EOF
0 255
EOF
kconvert -i $tmpdir/asc.$$ -o $tmpdir/asc.$$ -val -type 9

report $test_num "$tmpdir/out.$$" "$tmpdir/asc.$$" "$descr" "1e-5"
/bin/rm -f $tmpdir/out.$$ $tmpdir/asc.$$

exit $status

