#! /bin/sh

echo "running test2.sh for ksampline ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb DATAMANIP -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  no mask, no map, 5D sampling (4x4x4x4x4), partially outside data set
# ------------------------------------------------------------------------
test_num=1
descr="no mask, no map, 5D sampling (4x4x4x4x4), partially outside data set"
kgconst -o $tmpdir/in.$$ -wsize 4 -hsize 4 -dsize 4 -tsize 4 -esize 4 \
        -real 1 -type 11
${BINARY_PATH}/ksampline -i $tmpdir/in.$$ -o $tmpdir/out.$$ \
    -wc1 -1 -hc1 -1 -dc1 -1 -ec1 -1 -tc1 -1 \
    -wc2 4 -hc2 4 -dc2 4 -tc2 4 -ec2 4 -n 11
cat <<EOF > $tmpdir/result.$$
0 0 1 1 1 1 1 1 1 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  2:  no mask, no map, 5D sampling (10x10x10x10x10), partially outside 
#           data set
# ------------------------------------------------------------------------
test_num=2
descr="no mask, no map, 5D sampling (10x10x10x10x10), partially outside data set"
kgconst -o $tmpdir/in.$$ -wsize 10 -hsize 10 -dsize 10 -tsize 10 -esize 10 \
        -real 1 -type 11
${BINARY_PATH}/ksampline -i $tmpdir/in.$$ -o $tmpdir/out.$$ \
    -wc1 -1 -hc1 -1 -dc1 -1 -ec1 -1 -tc1 -1 \
    -wc2 10 -hc2 10 -dc2 10 -tc2 10 -ec2 10 -n 11
cat <<EOF > $tmpdir/result.$$
0 1 1 1 1 1 1 1 1 1 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  3:  no mask, no map, 5D sampling (20x20x20x20x20), partially outside 
#           data set. This is a special test to make sure that a specific
#           bug is gone that has to do with initializing a mask when it's
#           on a transport.
# ------------------------------------------------------------------------
test_num=3
descr="no mask, no map, 5D sampling (20x20x20x20x20), partially outside data set (special test)"
kgconst -o $tmpdir/in.$$ -wsize 20 -hsize 20 -dsize 20 -tsize 20 -esize 20 \
        -real 1 -type 11
${BINARY_PATH}/ksampline -i $tmpdir/in.$$ -o $tmpdir/out.$$ \
    -wc1 -1 -hc1 -1 -dc1 -1 -ec1 -1 -tc1 -1 \
    -wc2 20 -hc2 20 -dc2 20 -tc2 20 -ec2 20 -n 11
cat <<EOF > $tmpdir/result.$$
0 1 1 1 1 1 1 1 1 1 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$


# ------------------------------------------------------------------------
# Test  4:  no mask, no map, 5D sampling (128x128x60x3x3), partially outside 
#           data set
# ------------------------------------------------------------------------
test_num=4
descr="no mask, no map, 5D sampling, partially outside data set"
${BINARY_PATH}/ksampline -i volume:5D_head -o $tmpdir/out.$$ \
    -wc1 -1 -hc1 -1 -dc1 -1 -ec1 -1 -tc1 -1 \
    -wc2 128 -hc2 128 -dc2 60 -tc2 3 -ec2 3 -n 10 
cat <<EOF > $tmpdir/result.$$
0 0 0 174.2610546 178.3151775 176.1267954 159.3191784 0 0 0  
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  5:  no mask, no map, 3D WxHxD, contained in data set
# ------------------------------------------------------------------------
test_num=5
descr="no mask, no map, 3D WxHxD, contained in data set"
cat <<EOF > $tmpdir/ascii.$$
1 1 1 1
1 1 1 1
1 1 1 1
1 1 1 1
2 2 2 2
2 2 2 2
2 2 2 2
2 2 2 2
3 3 3 3
3 3 3 3
3 3 3 3
3 3 3 3
4 4 4 4
4 4 4 4
4 4 4 4
4 4 4 4
EOF
kasc2val -i1 $tmpdir/ascii.$$ -o $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 4 \
   -type double
${BINARY_PATH}/ksampline -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ \
   -wc1 0 -hc1 0 -dc1 0 \
   -wc2 3 -hc2 3 -dc2 3 -n 7
cat <<EOF > $tmpdir/result.$$
1 1.5 2 2.5 3 3.5 4
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  6:  no mask, no map, 3D WxHxD, partially outside data set
# ------------------------------------------------------------------------
test_num=6
descr="no mask, no map, 3D WxHxD, partially outside data set"
cat <<EOF > $tmpdir/ascii.$$
1 1 1 1
1 1 1 1
1 1 1 1
1 1 1 1
2 2 2 2
2 2 2 2
2 2 2 2
2 2 2 2
3 3 3 3
3 3 3 3
3 3 3 3
3 3 3 3
4 4 4 4
4 4 4 4
4 4 4 4
4 4 4 4
EOF
kasc2val -i1 $tmpdir/ascii.$$ -o $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 4 \
   -type double
${BINARY_PATH}/ksampline -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ \
   -wc1 -1 -hc1 -1 -dc1 -1 \
   -wc2 4 -hc2 4 -dc2 4 -n 11
cat <<EOF > $tmpdir/result.$$
0 0 1 1.5 2 2.5 3 3.5 4 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$

# ------------------------------------------------------------------------
# Test  7:  same as test #6, large data set
# ------------------------------------------------------------------------
test_num=7
descr="same as test #6, large data set"
cat <<EOF > $tmpdir/ascii.$$
1 1 1 1
1 1 1 1
1 1 1 1
1 1 1 1
2 2 2 2
2 2 2 2
2 2 2 2
2 2 2 2
3 3 3 3
3 3 3 3
3 3 3 3
3 3 3 3
4 4 4 4
4 4 4 4
4 4 4 4
4 4 4 4
EOF
kasc2val -i1 $tmpdir/ascii.$$ -o $tmpdir/ascii.$$ -wsize 4 -hsize 4 -dsize 4 \
   -type double
set KHOROS_KDMS_BUFSIZE=1
export KHOROS_KDMS_BUFSIZE
${BINARY_PATH}/ksampline -i $tmpdir/ascii.$$ -o $tmpdir/out.$$ \
   -wc1 -1 -hc1 -1 -dc1 -1 \
   -wc2 4 -hc2 4 -dc2 4 -n 11
unset KHOROS_KDMS_BUFSIZE
cat <<EOF > $tmpdir/result.$$
0 0 1 1.5 2 2.5 3 3.5 4 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$


exit $status

