/*
 * Khoros: $Id: kdataccess.h,v 1.7.4.1 1996/12/06 20:28:26 fas Exp $
 */
 
 
/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions and prototypes for library kdataccess
   >>>>
   >>>>       Toolbox: DATASERV
   >>>>
   >>>>    Written By: Steve Kubica
   >>>>
   >>>>          Date: 
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _kdataccess_h_
#define _kdataccess_h_


	/*-------------------------------------*
	|       #include
	---------------------------------------*/

#include "attributes.h"
#include "buffer.h"
#include "transport.h"
#include "format.h"
#include "segment.h"
#include "pres.h"

	/*-------------------------------------*
	|       #defines
	---------------------------------------*/

/*-- file declaration definitions --*/
#define KOBJ_READ       0x1
#define KOBJ_WRITE      0x2
#define KOBJ_STREAM     0x8000
#define KOBJ_HEADERLESS 0x4000
#define KOBJ_RAW        KOBJ_HEADERLESS

/*-- convenient array definitions --*/
#define KDAS_MAX_DIM    5
#define KDAS_ZERO_ARRAY {0, 0, 0, 0, 0}
#define KDAS_ONE_ARRAY  {1, 1, 1, 1, 1}

/*-- defines for the clippers and interpolaters --*/
#define KDAS_READ   1
#define KDAS_WRITE  0

/*-- defines for the testing routines --*/
#define KDAS_TEST_SEGMENT      1
#define KDAS_TEST_PRESENTATION 2

	/*-------------------------------------*
	|       typedefs
	---------------------------------------*/

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

extern int
   KDAS_ESEG_UNKNOWN_DIMENSIONALITY,
   KDAS_ESEG_UNKNOWN_SIZE,
   KDAS_ESEG_UNKNOWN_ORDER,
   KDAS_ESEG_UNKNOWN_DATATYPE,
   KDAS_EATTR_UNKNOWN_DATATYPE,
   KDAS_EATTR_UNSET,
   KDAS_EATTR_SET_FAILED,
   KDAS_EATTR_GET_FAILED,
   KDAS_EATTR_COPY_FAILED,
   KDAS_EATTR_QUERY_FAILED,
   KDAS_EATTR_PRINT_FAILED,
   KDAS_EINVALID_NORMALIZATION;

	/*-------------------------------------*
	|       macros
	---------------------------------------*/

/*-- nifty macros to issue error messages in addition to setting errno --*/
#define kdas_set_error(num) \
   { \
     errno = num; \
     kinfo(KDEBUG,"%s:%d\t%s\n",__FILE__,__LINE__,kerrno_lookup(num)); \
   }

#define kdas_fail_error(num) \
   { \
     errno = num;  \
     kinfo(KDEBUG,"%s:%d\t%s\n",__FILE__,__LINE__,kerrno_lookup(num)); \
     return FALSE; \
   }

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*-- init.c --*/
void kdas_init PROTO((void));

/*-- attr.c --*/
kdas_attribute *kdas_attr_generic_construct PROTO((ktoken, int, int, int, 
						   int, kaddr));

kdas_attribute *kdas_attr_quasi_construct PROTO((char *, 
   int (*get)   PROTO ((kobject,ktoken,ktoken,ktoken,kva_list *)), 
   int (*set)   PROTO ((kobject,ktoken,ktoken,ktoken,kva_list *)), 
   int (*match) PROTO ((kobject,kobject,ktoken,ktoken,kaddr,kaddr)), 
   int (*copy)  PROTO ((kobject,kobject,ktoken,ktoken,kaddr,kaddr)), 
   int (*query) PROTO ((kobject,ktoken,ktoken,kaddr,int *,int *,int *,int *)), 
   int (*print) PROTO ((kobject,ktoken,ktoken,kaddr,kfile *)),
   kaddr));

void kdas_attr_destruct PROTO((kdas_attribute *));
int  kdas_attr_get      PROTO((kdas_attribute *, kobject, ktoken, kva_list *));
int  kdas_attr_set      PROTO((kdas_attribute *, kobject, ktoken, kva_list *));
int  kdas_attr_match    PROTO((kdas_attribute *, kdas_attribute *, 
			       kobject, kobject, ktoken));
int  kdas_attr_copy     PROTO((kdas_attribute *, kdas_attribute *, 
			       kobject, kobject, ktoken));
int  kdas_attr_query    PROTO((kdas_attribute *, kobject, ktoken, int *, 
			       int *, int *, int *));
int  kdas_attr_print    PROTO((kdas_attribute *, kobject, ktoken, kfile *));

void   kdas_attribute_list_free   PROTO((klist *));

klist *kdas_list_string_locate    PROTO((klist *, char *));

void kdas_attr_setmode    PROTO((int));

int _kdas_vargs_to_attribute_data PROTO((kaddr *, int, int, int, kva_list *));
int _kdas_attribute_data_to_vargs PROTO((kaddr, int, int, int, kva_list *));

/*-- buffer.c --*/
kdas_buffer * kdas_adjust_buffer_size PROTO((kdas_buffer *, int, int *, int));
void          kdas_free_all_buffers   PROTO((void));

/*-- format.c --*/
kdas_format *kdas_format_construct PROTO((char *, char *, 
			  int     (*) PROTO ((int)), 
			  int     (*) PROTO ((kobject, int, int)),
			  int     (*) PROTO ((kobject, int, int)),
			  int     (*) PROTO ((kobject)),
			  int     (*) PROTO ((kobject)),
			  int     (*) PROTO ((kobject, char *)),
			  ssize_t (*) PROTO ((int, kaddr, size_t, int)),
			  ssize_t (*) PROTO ((int, kaddr, size_t, int)),
			  off_t   (*) PROTO ((int, off_t, int)),
			 long int (*) PROTO ((int))));
kdas_format *kdas_format_dup      PROTO((kdas_format *));

void kdas_format_destruct PROTO((kdas_format *format));
int  kdas_to_kfile_flags  PROTO((int));


/*-- memory.c --*/
kaddr kdas_memory_construct  PROTO((int, int *, int, double, double));
void  kdas_memory_destruct   PROTO((kaddr));
kaddr kdas_memory_recast     PROTO((kaddr, int, int *, int, int));
kaddr kdas_memory_resize     PROTO((kaddr, int, int, int *, int *, 
				    double, double));
kaddr kdas_memory_reorder    PROTO((kaddr, int, int, int *, int *, 
				    int *, int *));
void  kdas_memory_clip       PROTO((kaddr, kaddr, int, int, int *, int *, 
				    int *, int *, int *, int *, int));
void  kdas_memory_interp     PROTO((kaddr, kaddr, int, int, int *, int *, 
				    int *, int *, int *, int *, int));

/*-- transport.c --*/
kdas_transport *kdas_temp_construct  PROTO((int, int *, int, double, double));
void            kdas_temp_destruct   PROTO((kdas_transport *));
kdas_transport *kdas_temp_recast     PROTO((kdas_transport *, int, int *, 
					    int, int));
kdas_transport *kdas_temp_resize     PROTO((kdas_transport *, int, int, 
					    int *, int *, double, double));
kdas_transport *kdas_temp_reorder    PROTO((kdas_transport *, int, int, 
					    int *, int *, int *, int *));
kdas_transport *kdas_temp_copy       PROTO((kdas_transport *, int, int, 
					    int *));
void kdas_transport_clip             PROTO((kdas_transport *, kaddr, int, int,
					    int *, int *, int *, int *, 
					    int *, int *, int));
void kdas_transport_interp           PROTO((kdas_transport *, kaddr, int, int,
					    int *, int *, int *, int *, 
					    int *, int *, int));

/*-- pres.c --*/
kdas_presentation *kdas_pres_slap_on_segment PROTO((kdas_segment *, int));
kdas_presentation *kdas_pres_construct       PROTO((char *, int));
kdas_presentation *kdas_pres_duplicate       PROTO((kdas_presentation *));

void kdas_pres_destruct      PROTO((kdas_presentation *));
int  kdas_pres_set_all       PROTO((kdas_presentation *, int, int *, int *, 
				    int, kaddr));
int  kdas_pres_set_dimension PROTO((kdas_presentation *, int));
int  kdas_pres_get_dimension PROTO((kdas_presentation *, int *));
int  kdas_pres_set_size      PROTO((kdas_presentation *, int *));
int  kdas_pres_get_size      PROTO((kdas_presentation *, int **));
int  kdas_pres_set_order     PROTO((kdas_presentation *, int *));
int  kdas_pres_get_order     PROTO((kdas_presentation *, int **));
int  kdas_pres_set_datatype  PROTO((kdas_presentation *, int));
int  kdas_pres_get_datatype  PROTO((kdas_presentation *, int *));

kaddr kdas_pres_get_data     PROTO((kdas_presentation *, int *, int *, kaddr));
int   kdas_pres_put_data     PROTO((kdas_presentation *, int *, int *, kaddr));


/*-- seg.c --*/
kdas_segment *kdas_segment_construct   PROTO((char *));

void  kdas_segment_destruct            PROTO((kdas_segment *));
int   kdas_segment_set_all             PROTO((kdas_segment *, int, int *, 
					      int *, int, kaddr));
int   kdas_segment_get_all             PROTO((kdas_segment *, int *, int **,
					      int **, int *, kaddr *));
int   kdas_segment_set_dimension       PROTO((kdas_segment *, int));
int   kdas_segment_set_size            PROTO((kdas_segment *, int *));
int   kdas_segment_set_order           PROTO((kdas_segment *, int *));
int   kdas_segment_set_datatype        PROTO((kdas_segment *, int));

kaddr kdas_segment_get_data  PROTO((kdas_segment *, int *, int *, int *, int, 
				    double, double, kaddr));
int   kdas_segment_put_data  PROTO((kdas_segment *, int *, int *, int *, int, 
				    kaddr));
int   kdas_segment_write     PROTO((kdas_segment *));

/*-- sampo.c --*/
int     kdas_test_mode         	 PROTO((int, int));
int     kdas_test_architecture 	 PROTO((unsigned char));
kobject kdas_object_open       	 PROTO((char *, int));
int     kdas_object_close      	 PROTO((kobject));
int     kdas_object_set_coupling PROTO((kobject, int));

int     kdas_object_put_data   PROTO((kobject, int *, int *, kaddr));
kaddr   kdas_object_get_data   PROTO((kobject, int *, int *, kaddr));
int 	kdas_object_set_all    PROTO((kobject, int, int *, int *, int, kaddr));
int 	kdas_object_get_all    PROTO((kobject, int *, int **, int **, 
				      int *, kaddr *));

int     kdas_object_set_presentation_size       PROTO((kobject, int *, int, 
						       double, double));
int     kdas_object_set_presentation_order      PROTO((kobject, int *));

int     kdas_object_set_presentation_datatype        PROTO((kobject, int));

int     kdas_object_set_presentation_scaling         PROTO((kobject, double, 
							    double, double));
int     kdas_object_set_presentation_normalization   PROTO((kobject, double, 
							    double));
int     kdas_object_set_presentation_complex_convert PROTO((kobject, int));

/*-- utilities.c --*/
int  kdas_advance_position    PROTO((int, int *, int *, int *));
int  kdas_cast_data           PROTO((kaddr, int, int, int, int *, int, kaddr));
void kdas_preset_data         PROTO((kaddr, int, int *, int, double, double));
void kdas_preset_file         PROTO((int, int, int *, int, double, double));
void kdas_optimal_region_size PROTO((int, int *, int, int *));
int  kdas_is_big              PROTO((int, int *, int));
void kdas_set_threshold       PROTO((int));
int  kdas_get_order           PROTO((int *, int *, int, int, int));
void kdas_array_reorder       PROTO((int *, int, int *, int *));
int  kdas_array_max           PROTO((int *, int));
int  kdas_initialized         PROTO((int *, int));
int  kdas_size_initialized    PROTO((int *, int));

#endif /* _kdataccess_h_ */
/* Don't add after this point */
