 /*
  * Khoros: $Id: Ascii.c,v 1.9 1996/06/26 14:45:54 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Ascii.c,v 1.9 1996/06/26 14:45:54 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros ASCII Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_ascii()
   >>>>                 init_ascii()
   >>>>   Static:
   >>>>                 ascii_check()
   >>>>                 ascii_input()
   >>>>                 ascii_output()
   >>>>                 ascii_prep()
   >>>>                 ascii_destroy()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KASCII_DEF)

void
init_ascii(void) /*-- NULL ascii initialization --*/
{
   return;
}

#else

#include "kdatafmt/ascii.h"

static int ascii_check   PROTO((int));
static int ascii_input   PROTO((kobject, int, int));
static int ascii_output  PROTO((kobject, int, int));
static int ascii_prep    PROTO((kobject));
static int ascii_destroy PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_ascii
|       Purpose: initialize the ascii file format
------------------------------------------------------------*/
void
init_ascii(void)
{
   kdms_add_file_format("ascii", 
			"Ascii File Format (ascii)",
			ascii_check, 
			ascii_input, 
			ascii_output, 
			ascii_destroy,
			ascii_prep,
			NULL, NULL, NULL, NULL, NULL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  ascii  -  Ascii File Format (ascii)
 */
typedef struct
{
   int    fid;
   ascii *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: ascii_check - Checks to see if the file is an ascii
|
|       Purpose: This function is used to check an fid to see if it is 
|                a file containing recognizable ascii data. 
|
|         Input: fid - file id of a file to check
|        Output: none
|       Returns: returns TRUE if the file can be read as  ascii
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
ascii_check(int fid)
{
   ascii *image;

   /*-- return whether this is an ascii file or not --*/
   image = ascii_fdread(fid);

   if (image == NULL)
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: ascii_input - Read an ascii image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is ascii image.  We can simply set up
|                the header and initialize the value segment.
|
|         Input: object - object to read ascii into
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
ascii_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   ascii *image;

   int size[KDMS_MAX_DIM];
   int val_order[5] = {KWIDTH, KHEIGHT, KELEMENTS, KDEPTH, KTIME};


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;
   
   if ((image = resources->image = ascii_fdread(fid)) == NULL)
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->width != 0 && image->height != 0)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      size[0] = image->width;
      size[1] = image->height;
      size[2] = size[3] = size[4] = 1;

      /*-- store all value information --*/
      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, val_order, 
		       image->type, image->data);

      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->data = NULL;
   }
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: ascii_output - Closes the ascii image
|
|       Purpose: This function is used to close the ascii image.  
|		 It writes out the value data in ascii form.
|
|         Input: object - the ascii object to be closed
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
ascii_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   ascii          *image;


   /*-- get the resource structure, should already be initialized --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return FALSE;

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if (resources->image == NULL)
   {
      if ((resources->image = ascii_create()) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return FALSE;
      }
   }
   image = resources->image;

   
   /*-----------------------------------------------------------------------
   |  value data : if present has been prepared
   \-----------------------------------------------------------------------*/
   if (!kpds_query_value(object))
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }

   /*-- get the size --*/
   kpds_get_attributes(object, 
		       KPDS_VALUE_DATA_TYPE, &image->type,
		       KPDS_VALUE_SIZE, &image->width, &image->height,
		       NULL, NULL, NULL,
		       NULL);
   
   /*-- get the value data --*/
   if ((image->data = kpds_get_data(object, KPDS_VALUE_ALL, NULL)) == NULL)
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }

   /*-- write the ascii structure out to the specified filename --*/
   return ascii_fdwrite(fid, image);
}

/*-----------------------------------------------------------
|
|  Routine Name: ascii_prep - prepare an object for output to ascii
|
|       Purpose: This function is used to prepare an object for
|		 output to ascii.  It will print out information 
|		 about what is not getting stored out to the ascii
|		 file due to the limitations of the ascii format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 20, 1996
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
ascii_prep(kobject object)
{
   char *warn = NULL;

   /*-----------------------------------------------------------------------
   |  location, mask, map, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_location(object))
      warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");
   
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_time(object))
       warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   if (kpds_query_map(object))
      warn = kstring_append(warn, "- no support for storing map "
			    ": map segment ignored.\n  "
			    "(to preserve map values, use kmapdata before "
			    "writing to ascii.)");

   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      int w, h, d, t, e;
      int ntype, otype;
      
      /*-- data type : can only print kdcomplex or double --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &otype);

      /*-- set a presentation data type which we can print --*/
      ntype = (otype == KCOMPLEX || otype == KDCOMPLEX) ? KDCOMPLEX : KDOUBLE;
      kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, ntype);
      /*-- not lossy, so don't warn --*/

      /*-- size issue : can only store wxh plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e);
      if (d > 1 || t > 1 || e > 1)
	 warn = kstring_append(warn, "- only able to write first "
			       "plane of value data to output ascii file.\n");
   }
   else
      warn = kstring_append(warn, "- object must contain value data!\n");

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "ASCII format output :\n%s", warn);
      kfree_and_NULL(warn);
   }
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: ascii_destroy - Frees an ascii image
|
|       Purpose: This function is used to create an ascii image.  Which
|                means that we use the current dimension values in order
|                to create a proper ascii.
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|        Output: none
|	Returns: returns TRUE or FALSE if an error occurs
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
ascii_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   /*-- free the ascii structure --*/
   ascii_free(resources->image);

   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

#endif /* KASCII_DEF */
