/*
  * Khoros: $Id: Pnm.c,v 1.10 1996/07/02 20:38:57 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Pnm.c,v 1.10 1996/07/02 20:38:57 mirage Exp $";

#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros PNM Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_pnm()
   >>>>                 init_pnm()
   >>>>   Static:
   >>>>                 pnm_check()
   >>>>                 pnm_input()
   >>>>                 pnm_output()
   >>>>                 pnm_destroy()
   >>>>                 pnm_prep()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KPNM_DEF)

void
init_pnm(void) /*-- NULL pnm initialization --*/
{
   return;
}

#else

#include "kdatafmt/pnm.h"

static int pnm_check   PROTO((int));
static int pnm_input   PROTO((kobject, int, int));
static int pnm_output  PROTO((kobject, int, int));
static int pnm_destroy PROTO((kobject));
static int pnm_prep    PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_pnm
|       Purpose: initialize the pnm file format
------------------------------------------------------------*/
void
init_pnm(void)
{
   kdms_add_file_format("pnm", 
			"PBMPLUS's Portable Anymap File Format (pnm)",
			pnm_check, 
			pnm_input, 
			pnm_output, 
			pnm_destroy, 
			pnm_prep,
			NULL, NULL, NULL, NULL, NULL);

   /*-- need some colorspace attributes --*/
   kcolor_init();

   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  pnm  -  PBMPLUS's Portable Anymap File Format (pnm)
 */
typedef struct
{
   int   fid;
   pnm  *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: pnm_check - Checks to see if the file is a pnm
|
|       Purpose: This function is used to check the first few bytes 
|                of a fid to see if it is a pnm file. 
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: returns TRUE if the file is a pnm
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
pnm_check(int fid)
{
   char line[KLENGTH];
   kfile *file;

   file = kfdopen(fid, "r");
   
   if (!kfgets(line, KLENGTH, file))
   {
      return FALSE;           /* not an error */
   }
   
   if (kstrncmp(line, "P1", (size_t) 2) == 0 ||
       kstrncmp(line, "P2", (size_t) 2) == 0 ||
       kstrncmp(line, "P3", (size_t) 2) == 0 ||
       kstrncmp(line, "P4", (size_t) 2) == 0 ||
       kstrncmp(line, "P5", (size_t) 2) == 0 ||
       kstrncmp(line, "P6", (size_t) 2) == 0)
   {
      return TRUE;
   }

   return FALSE;
}


/*-----------------------------------------------------------
|
|  Routine Name: pnm_input - Read a pnm image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is pnm image.  We can simply set up 
|                the header and initialize that data segments.
|
|         Input: object - the pnm object to be initialized
|        Output: none
|	Returns: returns TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
pnm_input(kobject object, int fid, int flags)
{
   int datatype;
   int num_bands;
   int size [KDMS_MAX_DIM];
   int order[KDMS_MAX_DIM];


   ResourceStruct *resources;
   pnm *image;

   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if ((image = resources->image = pnm_fdread(fid)) == NULL)
   {
      /* _kdms_set_error(KDMS_EFMT_FAILURE); */
      return FALSE;
   }

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->data != NULL)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      if (image->type == PBM)
      {
         datatype = KBIT;
         num_bands = 1;
      }
      else
      {
         datatype = (image->raw) ? KUBYTE : KINT;
         num_bands = (image->type == PGM) ? 1 : 3;
      }

      if (num_bands == 1)
      {
	 size[0] = image->width;
	 size[1] = image->height;
	 size[2] = size[3] = size[4] = 1;

	 order[0] = KWIDTH;
	 order[1] = KHEIGHT;
	 order[2] = KDEPTH;
	 order[3] = KTIME;
	 order[4] = KELEMENTS;

	 kcolor_set_attribute(object, KCOLOR_COLORSPACE, KGREY);
      }
      else
      {
	 size[0] = num_bands;
	 size[1] = image->width;
	 size[2] = image->height;
	 size[3] = size[4] = 1;

	 order[0] = KELEMENTS;
	 order[1] = KWIDTH;
	 order[2] = KHEIGHT;
	 order[3] = KDEPTH;
	 order[4] = KTIME;

	 kcolor_set_attribute(object, KCOLOR_COLORSPACE, KRGB);
      }

      /*-- store all value information --*/
      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, order,
		       datatype, image->data);

      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->data = NULL;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: pnm_output - Closes the pnm image
|
|       Purpose: This function is used to close the pnm image.  
|                It writes out the value data to the most appropriate
|		 PNM format.
|
|         Input: object - the pnm object to be closed
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
pnm_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;

   kaddr data = NULL;
   unsigned char *map_data = NULL;
   pnm  *image = NULL;
   int  *size;
   int 	 i;
   int 	 begin[5] = {0, 0, 0, 0, 0};
   int 	 end[5]   = {0, 0, 0, 0, 0};
   int 	 type;
   int 	 status;
   

   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if (resources->image == NULL)
   {
      if ((image = resources->image = pnm_create(0, 0, PBM, FALSE)) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return (FALSE);
      }
   }
   else
      image = resources->image;
 
   /*-----------------------------------------------------------------------
   |  map data -
   |  want the map data to be unsigned char, 
   |  and index order to be map_width by map_height
   \-----------------------------------------------------------------------*/
   if (kpds_query_map(object)) 
   {
      kdms_set_attributes(object, KDMS_SEGMENT_MAP, 
			  KDMS_COUPLING,    KUNCOUPLED,
			  KDMS_DATA_TYPE,   KUBYTE,
			  NULL);

      map_data = kpds_get_data(object, KPDS_MAP_ALL, NULL);
   }
      
   /*-----------------------------------------------------------------------
   |  value data -
   |  want it to be elements first 
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      int order[KDMS_MAX_DIM] = {KELEMENTS, KWIDTH, KHEIGHT, KDEPTH, KTIME};
      unsigned char *dta;

      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE, &type);

      kdms_set_attributes(object, KDMS_SEGMENT_VALUE,
			  KDMS_COUPLING,    KUNCOUPLED,
			  KDMS_INDEX_ORDER, order,
			  NULL);

      if (type != KUBYTE && map_data == NULL) 
      {
	 kdms_set_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE, KINT);
	 image->raw = FALSE;
      }
      else
      {
	 kdms_set_attribute(object, KDMS_SEGMENT_VALUE, KDMS_DATA_TYPE,KUBYTE);
	 image->raw = TRUE;
      }
      
      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_SIZE, &size);
      
      image->width  = size[1];
      image->height = size[2];

      for (i = 0; i < 3; i++)   /*-- only take width x height x elements --*/
	 end[i] = size[i] - 1;
      
      data = kdms_get_data(object, KDMS_SEGMENT_VALUE, begin, end, NULL);

      if (image->raw)
      {
	 if (map_data != NULL) 
	 {
            /* Changed d from char to unsigned char 3 May 1995 SRW */
	    unsigned char *d;
	    unsigned char *c = (unsigned char *)data;
	    
	    d = (unsigned char *)kmalloc((size_t)(size[1] * size[2] * 3 *
						  sizeof(unsigned char)));
            image->data = (char *)d;
	    size[0] = 3;
	    for (i = 0; i < size[1] * size[2]; i++)
	    {
	       d[3*i] = map_data[3 * c[i]];
	       d[3*i + 1] = map_data[3 * c[i] + 1];
	       d[3*i + 2] = map_data[3 * c[i] + 2];
	    }
	    image->maxval = 255;
	 }
	 else
	    image->data = (char *)data;

	 dta = (unsigned char *)image->data;
	 
	 image->type = (size[0] == 3) ? PPM : PGM;
	 
	 if (image->maxval == 0)
	    for (i = 0, image->maxval = 0;i < size[0] * size[1] * size[2]; i++)
	       image->maxval = (int)kmax(image->maxval,(int)dta[i]);
      } 
      else
      {
	 int *d = (int *)data;
	 image->data = (char *)data;
	 
	 image->type = (size[0] == 3) ? PPM : PGM;
	 
	 if (image->maxval == 0)
	    for (i = 0, image->maxval = 0;i < size[0] * size[1] * size[2]; i++)
	       image->maxval = (int)kmax(image->maxval,d[i]);
      }
   }

   /*-- write the pnm structure out to the specified filename --*/
   status = pnm_fdwrite(fid, image);

   /*-- the map data was allocated here, the image data belongs to the pnm --*/
   kfree_and_NULL(map_data);

   return status;
}

/*-----------------------------------------------------------
|
|  Routine Name: pnm_destroy - Frees an pnm image
|
|       Purpose: This function is used to destroy a pnm image.  
|
|         Input: object
|        Output: none
|	Returns: returns TRUE on success, FALSE otherwise
|
|    Written By: Mark Young
|          Date: July 13, 1992
| Modifications:
|
------------------------------------------------------------*/
static int
pnm_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   /*-- free the pnm structure --*/
   pnm_free(resources->image);

   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: pnm_prep - prepare an object for output to pnm
|
|       Purpose: This function is used to prepare an object for
|		 output to pnm.  It will print out information 
|		 about what is not getting stored out to the pnm
|		 file due to the limitations of the pnm format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
pnm_prep(kobject object)
{
   char *warn = NULL;
   int   d, t, e;
   int   type;
   int   has_map = FALSE;
   

   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
      warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
      warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   if (kpds_query_map(object))
   {
      has_map = TRUE;
      warn = kstring_append(warn, "- no support for map segment "
			    ": mapping data through the map.\n");

      kpds_get_attribute(object, KPDS_MAP_DATA_TYPE, &type);

      if (type != KUBYTE)
	 warn = kstring_append(warn, "  mapped map data will be cast to "
			       "unsigned byte.\n");
   }
   
   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- type issue : can only store int images --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &type);

      if (type > KINT && !has_map) 
	 warn = kstring_append(warn, "- casting value data to int "
			       "for storage.\n");

      else if (type > KUBYTE && has_map)
	 warn = kstring_append(warn, "- casting value data to unsigned byte "
			       "for mapping.\n");

      else if (type > KUBYTE)
	 warn = kstring_append(warn, "- casting value data to unsigned "
			       "byte.\n");
      

      /*-- size issue : can only store 2D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1)
      {
	 warn = kstring_append(warn, "- ignoring depth and time planes of "
			       "value segment.\n");
      }

      if (e > 1 && has_map)
      {
	 warn = kstring_append(warn, "- only storing first element plane of "
			       "value segment.\n");
      }


   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "PNM format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

#endif /* KPNM_DEF */

/* don`t add after the endif */
