/*
  * Khoros: $Id: Xbm.c,v 1.7 1996/06/26 14:46:02 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Xbm.c,v 1.7 1996/06/26 14:46:02 mirage Exp $";

#endif



/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros XBM Data Service Routines
   >>>>		
   >>>>  Private:
   >>>>                 init_xbm()
   >>>>                 init_xbm()
   >>>>   Static:
   >>>>                 xbm_check()
   >>>>                 xbm_input()
   >>>>                 xbm_output()
   >>>>                 xbm_destroy()
   >>>>                 xbm_prep()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

#if !defined(KXBM_DEF)

void
init_xbm(void) /*-- NULL xpm initialization --*/
{
   return;
}

#else

#include "kdatafmt/xbm.h"

static int xbm_check   PROTO((int));
static int xbm_input   PROTO((kobject, int, int));
static int xbm_output  PROTO((kobject, int, int));
static int xbm_destroy PROTO((kobject));
static int xbm_prep    PROTO((kobject));

/*=====================================================================*/

/*-----------------------------------------------------------
|  Routine Name: init_xbm
|       Purpose: initialize the xbm file format
------------------------------------------------------------*/
void
init_xbm(void)
{
   kdms_add_file_format("xbm", 
			"X11 Bitmap File Format (xbm)",
			xbm_check, 
			xbm_input, 
			xbm_output, 
			xbm_destroy, 
			xbm_prep,
			NULL, NULL, NULL, NULL, NULL);
   return;
}

/*=====================================================================*/

/*
 *  Internal Resource Structure 
 *  xbm  -  X11 Bitmap File Format (xbm)
 */
typedef struct
{
   int  fid;
   xbm *image;
}
ResourceStruct;

/****    =============================================================
 ****    Format Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: xbm_check - Checks to see if the file is an xbm
|
|       Purpose: This function is used to check the first few bytes 
|                of a fid to see if it is an xbm file. 
|
|         Input: fid - file id of a file to check
|        Output: none
|	Returns: TRUE if the file is an xbm
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
xbm_check(int fid)
{
   xbm *check;
   
   /*-- seems like the easiest way to do this is to check the header --*/
   if ((check = xbm_readheader(fid)) == NULL)
      return FALSE;

   xbm_free(check);
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xbm_input - Read an xbm image
|
|       Purpose: This function is used to read in the image if the
|                supplied data is an xbm image.  We can simply set up 
|                the header and initialize the data segments.
|
|         Input: object - the xbm object to be initialized
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
xbm_input(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   xbm  *image;
   int 	 total;
   int 	 i;
   int   size[KDMS_MAX_DIM];


   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   resources->fid = fid;

   if ((image = resources->image = xbm_fdread(fid)) == NULL)
   {
      kdas_set_error(KDMS_EFMT_FAILURE);
      return FALSE;
   }

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (image->width != 0 && image->height != 0)
   {
      /*-- create the value segment --*/
      kpds_create_value(object);

      size[0] = image->width;
      size[1] = image->height;
      size[2] = size[3] = size[4] = 1;

      total = ((size[0] + 7) >> 3) << 3;
      for (i = 1; i < KDMS_MAX_DIM; i++)
	 total *= size[i];

      kdata_cast(image->data, KBIT_LSB, KBIT, total, KNONE, 1.0, 1.0, 
		 image->data);

      /*-- store all value information --*/
      kdms_set_segment(object, KDMS_SEGMENT_VALUE, 5, size, NULL, 
		       KBIT, image->data);

      /*---------------------------------------------------------- 
      |  let data services handle freeing this, since set_segment 
      |  has assigned the data directly. 
      \----------------------------------------------------------*/
      image->data = NULL;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xbm_output - Closes a xbm image
|
|       Purpose: This function is used to close the xbm image.  
|
|         Input: object - the xbm object to be closed
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static int
xbm_output(kobject object, int fid, int flags)
{
   ResourceStruct *resources;
   int 	begin[KDMS_MAX_DIM] = {0, 0, 0, 0, 0};
   int 	end  [KDMS_MAX_DIM];
   int *size;
   int  num;
   
   xbm *image = NULL;
   

   /*-- get the resource structure --*/ 
   if (!kdms_initialized(object))
   {
      if ((resources = kcalloc((size_t)1, sizeof(ResourceStruct))) == NULL)
         return(FALSE);

      kdms_glue_set_resources(object, resources);
   }
   else
      resources = (ResourceStruct *) kdms_glue_get_resources(object);

   if (resources->image == NULL)
   {
      if ((image = resources->image = xbm_create()) == NULL)
      {
         kdas_set_error(KDMS_EFMT_FAILURE);
         return FALSE;
      }
   }
   else
      image = resources->image;


   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- must get as bit data --*/ 
      if (!kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, KBIT))
	 return FALSE;

      kdms_get_attribute(object, KDMS_SEGMENT_VALUE, KDMS_SIZE, &size);
   
      end[0] = size[0] - 1;
      end[1] = size[1] - 1;
      end[2] = end[3] = end[4] = 0;
   
      image->width  = (int)size[0];
      image->height = (int)size[1];

      image->data = kdms_get_data(object, KDMS_SEGMENT_VALUE, begin, end, 
				  NULL);

      num = ((size[0] + 7) >> 3) << 3;
      kdata_cast(image->data, KBIT_LSB, KBIT, 
		 (num * size[1] * size[2] * size[3] * size[4]), 
		 KNONE, 1.0, 1.0, image->data);
      
   }
   
   /*-- write the xbm structure out to the specified filename --*/
   return xbm_fdwrite(fid, image);
}

/*-----------------------------------------------------------
|
|  Routine Name: xbm_destroy - Frees an xbm image
|
|       Purpose: This function is used to create an xbm image.  
|
|         Input: data - initial sample of data
|                num  - number of bytes in the sample
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: March 14, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
xbm_destroy(kobject object)
{
   ResourceStruct *resources;

   resources = (ResourceStruct *) kdms_glue_get_resources(object);
   if (resources == NULL)
      return TRUE;

   /*-- free the xbm structure --*/
   xbm_free(resources->image);
   
   /*-- free the resources --*/
   kfree(resources);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: xbm_prep - prepare an object for output to xbm
|
|       Purpose: This function is used to prepare an object for
|		 output to xbm.  It will print out information 
|		 about what is not getting stored out to the xbm
|		 file due to the limitations of the xbm format.	
|
|         Input: object - the object to be prepared
|        Output: none
|	Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 18, 1996
|
------------------------------------------------------------*/
/*ARGSUSED2*/
static int
xbm_prep(kobject object)
{
   char *warn = NULL;
   int   d, t, e;
   int   type;
   

   /*-----------------------------------------------------------------------
   |  mask, location, and time data
   \-----------------------------------------------------------------------*/
   if (kpds_query_mask(object))
      warn = kstring_append(warn, "- no support for storing mask "
			    ": mask segment ignored.\n");

   if (kpds_query_location(object))
      warn = kstring_append(warn, "- no support for storing location "
			    ": location segment ignored.\n");

   if (kpds_query_time(object))
      warn = kstring_append(warn, "- no support for storing time "
			    ": time segment ignored.\n");

   if (kpds_query_map(object))
      warn = kstring_append(warn, "- no support for storing map "
			    ": map segment ignored.\n");
   /*-- support for checking extraneous attributes coming soon! --*/

   /*-----------------------------------------------------------------------
   |  value data
   \-----------------------------------------------------------------------*/
   if (kpds_query_value(object))
   {
      /*-- size issue : can only store 2D data plane --*/
      kpds_get_attribute(object, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

      if (d > 1 || t > 1 || e > 1)
      {
	 warn = kstring_append(warn, "- ignoring depth, time, and element "
			       "planes of value segment.\n");
      }

      /*-- type issue : can only store bit --*/
      kpds_get_attribute(object, KPDS_VALUE_DATA_TYPE, &type);

      if (type != KBIT)
	 warn = kstring_append(warn, "- casting value data to bit "
			       "for storage.\n");
   }

   /*-- if we have any news to report, report it now --*/
   if (warn)
   {
      kinfo(KSTANDARD, "XBM format output :\n%s", warn);
      kfree_and_NULL(warn);
   }

   return TRUE;
}

#endif /* KXBM_DEF */

/* don`t add after the endif */
