 /*
  * Khoros: $Id: pres.c,v 1.11 1996/06/30 23:33:18 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: pres.c,v 1.11 1996/06/30 23:33:18 mirage Exp $";
#endif


/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Presentation Code
   >>>>		
   >>>>  Private:
   >>>>                 kdas_pres_slap_on_segment()
   >>>>                 kdas_pres_construct()
   >>>>                 kdas_pres_duplicate()
   >>>>                 kdas_pres_destruct()
   >>>>                 kdas_pres_set_all()
   >>>>                 kdas_pres_set_dimension()
   >>>>                 kdas_pres_get_dimension()
   >>>>                 kdas_pres_set_size()
   >>>>                 kdas_pres_get_size()
   >>>>                 kdas_pres_set_order()
   >>>>                 kdas_pres_get_order()
   >>>>                 kdas_pres_set_datatype()
   >>>>                 kdas_pres_get_datatype()
   >>>>                 kdas_pres_get_data()
   >>>>                 kdas_pres_put_data()
   >>>>   Static:
   >>>>                 _need_to_transpose()
   >>>>                 _need_to_complex_convert()
   >>>>                 _wrap_region()
   >>>>                 _get_transposed_data()
   >>>>                 _put_transposed_data()
   >>>>                 _bigger_type()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static int   _need_to_transpose       PROTO((kdas_presentation *));
static int   _need_to_complex_convert PROTO((kdas_presentation *));
static void  _wrap_region             PROTO((kaddr, int *, int *, int *, int *,
					     int *, int *, int *, kaddr));
static kaddr _get_transposed_data     PROTO((kdas_presentation *, 
					     int *, int *, int *, int *, int, 
					     int, int, kaddr));
static int   _put_transposed_data     PROTO((kdas_presentation *, int *, 
					     int *, int *, int, kaddr));
static int   _bigger_type             PROTO((int, int));

/****    =============================================================
 ****    Temporary Buffers
 ****    =============================================================
 ****/

/*-- clip buffer --*/
static kdas_buffer *cbuffer = NULL;

/*-- transpose buffer --*/
static kdas_buffer *tbuffer = NULL;


/****    =============================================================
 ****    Presentation Manifestation Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_slap_on_segment
|
|       Purpose: This routine will slap a presentation layer on 
|		 an existing segment.
|
|         Input: segment  - the segment to wrap 
|		 coupling - the coupling between segment & presentation;
|                           can be either KCOUPLED, KDEMAND, KUNCOUPLED
|        Output: none
|
|       Returns: the constructed presentation or NULL if it can't create
|
|    Written By: Steve Kubica
|          Date: October 12, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_presentation *
kdas_pres_slap_on_segment(kdas_segment *segment, int coupling)
{
   kdas_presentation *pres;

   if (segment == NULL)
      return NULL;

   /*-- allocate the new presentation and initialze its fields --*/
   pres = kmalloc(sizeof(kdas_presentation));

   pres->id = segment->id;
   pres->coupling = coupling;

   pres->attributes = NULL;

   pres->segment   = segment;
   pres->dimension = segment->dimension;
   pres->datatype  = segment->datatype;

   pres->size  = (int *) kdupalloc((kaddr)segment->size, 
				   sizeof(int) * segment->dimension);

   pres->order = (int *) kdupalloc((kaddr)segment->order, 
				   sizeof(int) * segment->dimension);

   pres->size_interpolate   = KPAD;
   pres->access_interpolate = KPAD;

   pres->complex_convert = KREAL;
   pres->rpad            = 0.0;
   pres->ipad            = 0.0;
   pres->scaling         = KNONE;
   pres->scale_factor    = 1.0;
   pres->scale_roffset   = 0.0;
   pres->scale_ioffset   = 0.0;
   pres->norm_min        = 0.0;
   pres->norm_max        = 1.0;

   /*-- add this presentation to the segment's presentation list --*/
   pres->segment->references = klist_add(pres->segment->references, 
					 pres, NULL);
   return pres;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_construct
|
|       Purpose: This routine will construct a presentation 
|		 for a given segment name.
|
|         Input: segment_name - the name of the segment
|		 coupling     - the coupling between segment & presentation;
|                               can be either KCOUPLED, KDEMAND, KUNCOUPLED
|        Output: none
|
|       Returns: the constructed presentation or NULL if it can't create
|
|    Written By: Steve Kubica
|          Date: October 12, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_presentation *
kdas_pres_construct(char *segment_name, int coupling)
{
   kdas_segment *segment;

   /*-- allocate a new segment --*/
   segment = kdas_segment_construct(segment_name);

   /*-- slap the presentation on the new segment and return it --*/
   return kdas_pres_slap_on_segment(segment, coupling);
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_duplicate
|
|       Purpose: This routine will duplicate a given presentation.
|
|         Input: presentation - the presentation to duplicate
|
|        Output: none
|
|       Returns: an allocated, duplicated presentation
|
|    Written By: Steve Kubica
|          Date: October 31, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_presentation *
kdas_pres_duplicate(kdas_presentation *pres)
{
   kdas_presentation *presentation;

   /*-- allocate the new presentation and initialze its fields --*/
   presentation = kdupalloc(pres, sizeof(kdas_presentation));

   /*-- don't duplicate the attribute list --*/
   pres->attributes = NULL;

   return presentation;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_destruct
|
|       Purpose: This routine will destruct a given presentation.
|
|         Input: pres - the presentation to destruct
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: October 12, 1994
| Modifications:
|
------------------------------------------------------------*/
void kdas_pres_destruct(kdas_presentation *pres)
{
   /*-- trivial free --*/
   if (pres == NULL)
      return;

   /*-- remove this pres from the segment's list of references --*/
   pres->segment->references = klist_delete(pres->segment->references, pres);
   
   /*-- if this was the last reference, then delete the segment as well --*/
   if (pres->segment->references == NULL)
      kdas_segment_destruct(pres->segment);

   /*-- if there are any attributes, then delete those --*/
   if (pres->attributes)
      klist_free(pres->attributes, kdas_attribute_list_free);

   kfree(pres->size);
   kfree(pres->order);

   kfree(pres);
   return;
}


/****    =============================================================
 ****    Presentation Characteristic Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_set_all - assign everything for a presentation
|
|       Purpose: This routine will set the dimensionality, size, 
|		 index order and datatype of a presentation as well as 
|		 optionally allowing you to assign the actual segment 
|		 data directly.  
|
|		 The underlying segment will be updated with all this 
|		 new information as well. Anything that was previously 
|		 contained in the segment will be discarded.  Any other
|		 presentations which may reference this segment are not
|		 modified.   Typically, this routine should only be
|		 called when initializing a new segment.
|
|		 The data should only be assigned directly
|		 if it is all present.  The act of assigning this
|		 data with this routine will cause the large
|		 data set management to be bypassed.  If you
|		 have the capability of handling large data sets
|		 then NULL should be passed for the data as the
|		 data will be clipped directly from the file.
|
|         Input: pres      - presentation on which we will set everything
|		 dimension - new dimensionality to set
|		 size      - new size to set
|		 order     - new order to set
|		 datatype  - new datatype to set
|		 data      - data if all is present
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: May 11, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_set_all(
   kdas_presentation *pres,
   int                dimension,
   int               *size,
   int               *order,
   int                datatype,
   kaddr              data)
{
   if (pres == NULL)
      return FALSE;

   /*-- tidy up if we have to --*/
   if (pres->size)  kfree(pres->size);
   if (pres->order) kfree(pres->order);

   /*-- assign what we've got --*/
   pres->dimension = dimension;
   pres->datatype  = datatype;
   pres->size      = kmalloc(sizeof(int) * dimension);
   pres->order     = kmalloc(sizeof(int) * dimension);

   kmemcpy(pres->size,  size,  sizeof(int) * dimension);
   kmemcpy(pres->order, order, sizeof(int) * dimension);

   /*-- set up the underlying segment --*/
   return kdas_segment_set_all(pres->segment, dimension, size, order, 
			       datatype, data);
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_set_dimension - set the dimensionality
|
|       Purpose: Set the dimensionality of a presentation.  
|
|		 Setting the dimensionality is what manages the
|		 size of the presentation and segment's size arrays.  
|		 The size of the presenation's size array should 
|		 only be changed by this routine.   
|
|         Input: pres      - presentation on which dimensionality is set
|		 dimension - new dimensionality to set
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_set_dimension(
   kdas_presentation *pres,
   int                dimension)
{
   int odimension;
   int i;

   if (dimension == -1) /*-- nullop --*/
      return TRUE;

   if (pres == NULL)
      return FALSE;

   /*-- save the old dimension for comparison purposes --*/
   odimension = pres->segment->dimension;
   
   /*----------------------------------------------------------------
   |   propogate the dimensionality to the physical segment as there
   |   is no notion of dimensionality at the presentation layer.
   \-----------------------------------------------------------------*/
   kdas_segment_set_dimension(pres->segment, dimension);

   /*----------------------------------------------------------------
   |   modify the presentation size dimensionality
   \-----------------------------------------------------------------*/
   if (pres->size == NULL)
   {
      /*-- set the size array up to be of new dimension and initialize --*/
      pres->size = kmalloc(sizeof(int) * dimension );
      for (i = 0; i < dimension; i++)
	 pres->size[i] = -1;
   }
   else /*-- dimensionality changing --*/
   {
      if (dimension < odimension)  /*-- smaller --*/
      {
	 /*-- roll old sizes into the last dimension --*/
	 for (i = dimension; i < odimension; i++)
	    pres->size[dimension-1] *= pres->size[i];

	 pres->size = krealloc(pres->size, sizeof(int) * dimension);
      }
      else if (dimension > odimension)  /*-- larger --*/
      {
	 pres->size  = krealloc(pres->size, sizeof(int) * dimension);
	 
	 /*-- pad out new dimension sizes with 1 --*/ 
	 for (i = odimension; i < dimension; i++)
	    pres->size[i] = 1;
      }
   }

   /*----------------------------------------------------------------
   |   modify the presentation order dimensionality
   \-----------------------------------------------------------------*/
   if (pres->order == NULL)
   {
      /*-- set the order array up to be of new dimension and initialize --*/
      pres->order = kmalloc(sizeof(int) * dimension);
      for (i = 0; i < dimension; i++)
	 pres->order[i] = -1;
   }
   else /*-- dimensionality changing --*/
   {
      pres->order = krealloc(pres->order, sizeof(int) * dimension);

      /*-- needed for kimportasc : null out the index order --*/
      for (i = 0; i < dimension; i++)
	 pres->order[i] = -1;
   }

   /*-- keep our own copy for reference --*/
   pres->dimension = dimension;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_get_dimension - get the dimensionality
|
|       Purpose: Get the dimensionality of a presentation.  Since
|		 there is technically not a presentation dimensionality,
|		 we will return it from the physical layer.
|
|         Input: pres - presentation from which dimensionality is got
|
|        Output: dimension - variable in which to return dimensionality
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_get_dimension(
   kdas_presentation *pres,
   int               *dimension)
{
   if (pres == NULL)
      return FALSE;

   /*-- return the presentation dimensionality --*/
   *dimension = pres->segment->dimension;
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_set_size - set the presentation size
|
|       Purpose: Set the size of a presentation.  
|
|		 This routine assumes that the presentation's size
|		 array is already initialized to the proper size.
|		 Note that setting the dimensionality is what 
|		 allocates the presentation's size array.  
|
|         Input: pres - presentation on which we will set the size
|		 size - new size to set
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_set_size(
   kdas_presentation *pres,
   int               *size)
{
   int nsize[KDAS_MAX_DIM] = KDAS_ONE_ARRAY;
   int dimension;
   int i;


   if (pres == NULL)
      return FALSE;

   dimension = pres->segment->dimension;

   if (dimension == -1) /*-- dimension has not been set before --*/
      kdas_fail_error(KDAS_ESEG_UNKNOWN_DIMENSIONALITY);

   /*----------------------------------------------------------------
   |   sanity check - make sure our dimensionality is in sync with 
   |   the physical segment dimensionality.
   \-----------------------------------------------------------------*/
   if (dimension != pres->dimension)
      kdas_pres_set_dimension(pres, dimension);

   /*----------------------------------------------------------------
   |   propogate the size to the physical segment if :
   |   [1] we are coupled
   |   [2] the physical segment size is not initialized
   \-----------------------------------------------------------------*/
   if (pres->coupling == KCOUPLED  ||
       !kdas_size_initialized(pres->segment->size, dimension))
      kdas_segment_set_size(pres->segment, size);

   /*----------------------------------------------------------------
   |   at this point, we know we have a valid presentation dimension;
   |   construct a new size array, ignoring new sizes that are -1
   \-----------------------------------------------------------------*/
   for (i = 0; i < dimension; i++)
      nsize[i] = (size[i] == -1) ? pres->size[i] : size[i];

   /*-- assign the new size to the presentation's size array --*/
   kmemcpy(pres->size, nsize, sizeof(int) * dimension);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_get_size - get the size
|
|       Purpose: Get the size of a presentation.  
|
|		 This routine should be used to get the size of a
|		 presentation.  If the presentation size has not 
|		 yet been initialized, this routine will attempt
|		 to initialize the presentation size before returning
|		 it by using the size of the underlying data segment.
|
|         Input: presentation - presentation from which the size is gotten
|
|        Output: size - variable with which to return the size.
|		        this is a pointer to the actual internal
|		        size array in the presentation : do not
|		        alter its values!
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_get_size(
   kdas_presentation  *pres,
   int               **size)
{
   int dimension;


   if (pres == NULL)
      return FALSE;

   dimension = pres->segment->dimension;

   /*----------------------------------------------------------------
   |   sanity check - make sure our dimensionality is in sync with 
   |   the physical segment dimensionality.
   \-----------------------------------------------------------------*/
   if (dimension != pres->dimension)
      kdas_pres_set_dimension(pres, dimension);

   /*----------------------------------------------------------------
   |   if the presentation size is not initialized, take a look
   |   at the physical segment and use it to initialize
   \-----------------------------------------------------------------*/
   if (!kdas_size_initialized(pres->size, dimension))
   {
      /*-- if there's no physical size, there's nothing we can do --*/
      if (pres->segment->size == NULL)
      {
	 *size = NULL;
	 kdas_set_error(KDAS_ESEG_UNKNOWN_SIZE);
	 return FALSE;
      }
      else
	 kmemcpy(pres->size, pres->segment->size, sizeof(int) * dimension);
   }

   /*-- return the presentation size --*/
   *size = pres->size;
   
   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_set_order - set the presentation order
|
|       Purpose: Set the index order of a presentation.  
|
|		 This routine assumes that the presentation's order
|		 array is already initialized to the proper size.
|		 Note that setting the dimensionality is what 
|		 allocates the presentation's order array.  
|
|         Input: pres - presentation on which we will set the order
|		 order        - new order to set
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_set_order(
   kdas_presentation *pres,
   int               *order)
{
   int  norder[KDAS_MAX_DIM] = KDAS_ONE_ARRAY;
   int  order_changing = FALSE;
   int  dimension;
   int i;


   if (pres == NULL)
      return FALSE;

   dimension = pres->segment->dimension;

   if (dimension == -1) /*-- dimension has not been set before --*/
      kdas_fail_error(KDAS_ESEG_UNKNOWN_DIMENSIONALITY);

   /*----------------------------------------------------------------
   |   sanity check - make sure our dimensionality is in sync with 
   |   the physical segment dimensionality.
   \-----------------------------------------------------------------*/
   if (dimension != pres->dimension)
      kdas_pres_set_dimension(pres, dimension);

   /*----------------------------------------------------------------
   |   propogate the order to the physical segment if :
   |   [1] we are coupled
   |   [2] the physical segment order is not initialized
   \-----------------------------------------------------------------*/
   if (pres->coupling == KCOUPLED || 
       !kdas_initialized(pres->segment->order, dimension))
      kdas_segment_set_order(pres->segment, order);

   /*----------------------------------------------------------------
   |   at this point, we know we have a valid presentation dimension;
   |   construct a new order array, ignoring new directions that are -1
   \-----------------------------------------------------------------*/
   for (i = 0; i < dimension; i++)
   {
      norder[i] = (order[i] == -1) ? pres->order[i] : order[i];

      if ((order[i] != -1) && (norder[i] != pres->order[i])) 
	 order_changing = TRUE;
   }

   /*----------------------------------------------------------------
   |   if we have a size array, and our new order is all initialized,
   |   and different from what we had before, (and what we had before was
   |   all initialized) reorder the size array so it is in the correct 
   |   new index order.
   \-----------------------------------------------------------------*/
   if (pres->size && order_changing && kdas_initialized(norder, dimension) &&
                                       kdas_initialized(pres->order,dimension))
      kdas_array_reorder(pres->size, dimension, pres->order, norder);

   /*-- assign the new size to the presentation's order array --*/
   kmemcpy(pres->order, norder, sizeof(int) * dimension);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_get_order - get the order
|
|       Purpose: Get the index order of a presentation.  
|
|		 This routine should be used to get the order of a
|		 presentation.  If the presentation order has not 
|		 yet been initialized, this routine will attempt
|		 to initialize the presentation order before returning
|		 it by using the order of the underlying data segment.
|
|         Input: pres - presentation from which the order is gotten
|
|        Output: order - variable with which to return the order.
|		         this is a pointer to the actual internal
|		         order array in the presentation : do not
|		         alter its values!
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_get_order(
   kdas_presentation  *pres,
   int               **order)
{
   int dimension;


   if (pres == NULL)
      return FALSE;

   dimension = pres->segment->dimension;

   /*----------------------------------------------------------------
   |   sanity check - make sure our dimensionality is in sync with 
   |   the physical segment dimensionality.
   \-----------------------------------------------------------------*/
   if (dimension != pres->dimension)
      kdas_pres_set_dimension(pres, dimension);

   /*----------------------------------------------------------------
   |   if the presentation order is not initialized, take a look
   |   at the physical segment and use it to initialize
   \-----------------------------------------------------------------*/
   if (!kdas_initialized(pres->order, dimension))
   {
      /*-- if there's no physical order, there's nothing we can do --*/
      if (pres->segment->order == NULL)
      {
	 *order = NULL;
	 kdas_set_error(KDAS_ESEG_UNKNOWN_ORDER);
	 return FALSE;
      }
      else
	 kmemcpy(pres->order, pres->segment->order, 
		 sizeof(int) * dimension);
   }

   /*-- return the presentation order --*/
   *order = pres->order;
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_set_datatype - set the presentation datatype
|
|       Purpose: Set the datatype of a presentation.  
|
|         Input: pres - presentation on which we will set the datatype
|		 datatype     - new data type to set
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_set_datatype(
   kdas_presentation *pres,
   int                datatype)
{
   if (pres == NULL)
      return FALSE;

   /*----------------------------------------------------------------
   |   propogate the datatype to the physical segment if :
   |   [1] we are coupled
   |   [2] the physical segment datatype is not initialized
   \-----------------------------------------------------------------*/
   if (pres->coupling == KCOUPLED || pres->segment->datatype == -1)
      kdas_segment_set_datatype(pres->segment, datatype);

   /*-- assign the new data type to the presentation --*/
   pres->datatype = datatype;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_get_datatype - get the datatype
|
|       Purpose: Get the datatype of a presentation.  
|
|		 This routine should be used to get the datatype of a
|		 presentation.  If the presentation datatype has not 
|		 yet been initialized, this routine will attempt
|		 to initialize the presentation datatype before returning
|		 it by using the datatype of the underlying data segment.
|
|         Input: pres - presentation from which the datatype is gotten
|
|        Output: datatype - variable with which to return the datatype
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_get_datatype(
   kdas_presentation *pres,
   int               *datatype)
{
   if (pres == NULL)
      return FALSE;

   /*----------------------------------------------------------------
   |   if the presentation datatype is not initialized, take a look
   |   at the physical segment and use it to initialize
   \-----------------------------------------------------------------*/
   if (pres->datatype == -1)
   {
      /*-- if there's no physical datatype, there's nothing we can do --*/
      if (pres->segment->datatype == -1)
      {
	 *datatype = -1;
	 kdas_set_error(KDAS_ESEG_UNKNOWN_DATATYPE);
	 return FALSE;
      }
      else
	 pres->datatype = pres->segment->datatype;
   }

   /*-- return the presentation datatype --*/
   *datatype = pres->datatype;
   
   return TRUE;
}

/****    =============================================================
 ****    Presentation Data Access Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_get_data - retreive data block
|
|       Purpose: This routine will return a data block from a data space.
|		 The location of the data block is specified by a begin
|		 point and end point which are relative to the zero 
|		 origin of the data space.  The data block can be outside
|		 the bounds of the data space, in which case the 
|		 block will be padded using the presentation pad value.
|                         
|	          DATA BLOCK 
|	               +---------+
|	              /| | | | | |      __end
|	      begin__/ |-+-+-+-+-|     /
|	               | | | | | |    /    DATA SPACE
|	               |-+-+-+-+-|---/---------+
|	               | |/|*|*|*| |/| | | | | |
|	               |-/-+-+-+-|-/-+-+-+-+-+-|
|                      |/| |*|*|*|/| | | | | | |
|                      /---------+-+-+-+-+-+-+-|
|             (0,0)___/    | | | | | | | | | | |
|                          |-+-+-+-+/\-+-+-/\+-|
|                          | /\| | /  \/\|/  \ |
|                          |/  \/\/  :   V    \|
|                                    :
|                                    :
|
|		 The data block can be preallocated or can be allocated
|		 by this routine.  A pointer to the retrieved data block 
|		 will be returned by this routine.
|
|		 The data space will "appear" differently from how it
|		 is actually stored. Specifically, the data space
|		 can appear to be of a different size than it actually
|		 is.  This appearence is handled by the underlying
|		 segment abstraction by either padding or interpolating
|		 the data.
|
|		 Before the data is returned, it will be passed
|		 through the presentation pipeline with stages
|		 of the pipeline being invoked as necessary for each
|		 presentation characteristic.  
|
|		 Specifically, these characteristics are :
|
|	    	    !    o  datatype
|	    	    !    o  index order
|	    	    !    o  complex conversion
|	    	    !    o  interpolation or padding
|	    	    !    o  scaling and offset
|	    	    !    o  normalization
|		 
|         Input: pres - the presentation with the segment 
|		                to get the data from
|                begin   - the first corner bounding the region to retrieve
|                end     - the end point bounding the region to retrieve
|		 data    - the data block to use when returning the retrieved
|			   data; if NULL, then space will be allocated.
|        Output: none
|
|       Returns: a pointer to the data block being retrieved is returned
|
|    Written By: Steve Kubica
|          Date: January 6, 1996
| Modifications:
|
------------------------------------------------------------*/
kaddr
kdas_pres_get_data(
   kdas_presentation *pres,
   int      	     *begin, 
   int   	     *end, 
   kaddr 	      data)  
{
   int zero_array    [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clipped_begin [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clipped_end   [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clip_unit     [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int block_begin   [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int block_size    [KDAS_MAX_DIM];

   int upcast      = (pres->datatype > pres->segment->datatype);
   int downcast    = (pres->datatype < pres->segment->datatype);
   int convert     = _need_to_complex_convert(pres);
   int transpose   = _need_to_transpose(pres);
   int dimension   = pres->segment->dimension;
   int interpolate = pres->access_interpolate;
   int i;

   int clip    = FALSE;
   int outside = FALSE;

   kaddr cdata = NULL;  /*-- data clipped from the segment --*/

   /*-- sanity check --*/
   if (!begin || !end)
      return NULL;  /*-- kerror : invalid data access --*/

   /*-- figure out the overall size of the data being retrieved --*/
   for (i = 0; i < dimension; i++)
      block_size[i] = end[i] - begin[i] + 1;

   /*----------------------------------------------------------------
   |   determine if we are getting data outside our presentation size
   \-----------------------------------------------------------------*/
   kdas_clip_corners(dimension, pres->size, begin, end, &clip, &outside,
		     clipped_begin, clipped_end, block_begin, clip_unit);

   /*----------------------------------------------------------------
   |   clipping check : 
   |       if we are out of bounds and shouldn't be, leave now
   |       if we are out of bounds and wrapping, then our clipped
   |       size will be the whole size.
   \-----------------------------------------------------------------*/
   if (clip)
   { 
      if (interpolate == KNONE)
	 return NULL;

      /*-- this should never happen: its meaningless for data access --*/
      if (interpolate == KZERO_ORDER)  
	 return NULL;

      /*-------------------------------------------------------------
      |   if wrapping, our clipped size will be our entire block size 
      |   and our block begin will start at zero in the destination.
      \--------------------------------------------------------------*/
      if (interpolate == KWRAP)
      {
	 kmemcpy(clip_unit,   block_size, sizeof(int) * dimension);
	 kmemcpy(block_begin, zero_array, sizeof(int) * dimension);
      }
   }
   
   /*----------------------------------------------------------------
   |   'data' is the final destination, allocate it if necessary : 
   |       this must be done after above check for clipping with 
   |	   with KNONE interpolation.
   \-----------------------------------------------------------------*/
   if (data == NULL)
      data = kdas_memory_construct(dimension, block_size, pres->datatype,
				   0.0, 0.0);

   /*----------------------------------------------------------------
   |   set up the block buffer in which to do clipping : 
   |     if we are not doing any fancy presentation stuff, then we
   |     can just clip into the final data buffer.
   |
   |   we want to follow this, keeping data = cdata if possible :
   |    segment -> (transpose) cdata -> pipeline -> clip back into data
   \-----------------------------------------------------------------*/
   if (upcast || downcast || transpose || convert || clip)
   {
      cbuffer = kdas_adjust_buffer_size(cbuffer, dimension, clip_unit, 
					_bigger_type(pres->segment->datatype, 
					             pres->datatype));
      cdata = cbuffer->data;
   }
   else
      cdata = data;

   /*----------------------------------------------------------------
   |   if we've gone outside the apparent size and are padding, 
   |   pad the entire destination space with the pad value.
   \-----------------------------------------------------------------*/
   if (clip && (interpolate == KPAD))
      kdas_preset_data(data, dimension, block_size, pres->datatype, 
		       pres->rpad, pres->ipad);

   /*----------------------------------------------------------------
   | get the data from the segment : 
   |    we will either clip or wrap the data into a buffer after 
   |    which we will transpose it into the presentation index order.
   |    only do this if we are not totally outside the bounds of the
   |    actual data (or if we are wrapping).
   \-----------------------------------------------------------------*/
   if (!outside || interpolate == KWRAP)
      if ((cdata = _get_transposed_data(pres, clipped_begin, clipped_end, 
					clip_unit, begin, transpose, clip, 
					interpolate, cdata)) == NULL)
      return FALSE;  /*-- unable to get data from the segment --*/

   /*----------------------------------------------------------------
   |   cdata now contains       size     : "clip_unit" 
   |   the data we need         datatype : "segment->datatype" 
   |   to pipeline :	        order    : "segment->order"
   \-----------------------------------------------------------------*/

   /*------------------------------------------------------------------
   |   pipeline : this deals with the remainder of the presentation
   |      o  complex conversion
   |   	  o  upcasting
   |   	  o  scaling
   |   	  o  normalization
   |   	  o  downcasting
   |
   |   don't worry about losing cdata as a memory leak;
   |   the cdata pointer at this point is pointing to either :
   |      o  the data to return (which implies that only in-place 
   |         stages will be used in the pipeline)
   |      o  a temporary buffer, which we have another handle on (this
   |         will be freed when data services is no longer needed).
   |   so we can safely assign it to the return value.
   \-------------------------------------------------------------------*/
   cdata = kdas_pipeline_data(pres, upcast, downcast, convert, clip_unit, 
			      TRUE, cdata);

   /*------------------------------------------------------------------
   |   clip the data back into final data block if we need to 
   \-------------------------------------------------------------------*/
   if (data != cdata)
      kdas_memory_clip(data, cdata, dimension, pres->datatype, block_size,
		       clip_unit, NULL, block_begin, zero_array, clip_unit, 
		       KDAS_WRITE);
   return data;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_pres_put_data - store data block in data space
|
|       Purpose: This routine will clip a data block into a data space.
|		 The location of the data block is specified by a begin
|		 point and end point which are relative to the zero 
|		 origin of the data space.  The data block can be outside
|		 the bounds of the data space, in which case the data
|		 values outside of the data space will be ignored. 
|                         
|	          DATA BLOCK 
|	               +-+-+-+-+-+
|	              /| | | | | |      __end
|	      begin__/ +-+-+-+-+-+     /
|	               | | | | | |    /    DATA SPACE
|	               +-+-+-+-+-|---/---------+   of size "apparent_size"
|	               | |/|*|*|*| |/| | | | | |
|	               +-/-+-+-+-|-/-+-+-+-+-+-+
|                      |/| |*|*|*|/| | | | | | |
|                      /-+-------+-+-+-+-+-+-+-+
|             (0,0)___/    | | | | | | | | | | |
|                          +-+-+-+-+/\-+-+-/\+-+
|                          | /\| | /  \/\|/  \ |
|                          |/  \/\/  :   V    \|
|                                    :
|                                    :
|
|		 Before the data is stored, it will be passed
|		 through the presentation pipeline with stages
|		 of the pipeline being invoked as necessary for each
|		 presentation characteristic.  
|
|		 Specifically, these characteristics are :
|
|	    	    !    datatype
|	    	    !    index order
|	    	    !    complex conversion
|	    	    !    interpolation or padding
|	    	    !    scaling and offset
|	    	    !    normalization
|
|         Input: pres - the presentation with the segment 
|		                to put the data into
|                begin   - the first corner bounding the region being stored
|                end     - the end point bounding the region begin stored
|		 data    - the data block being put
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: January 6, 1996
| Modifications:
|
------------------------------------------------------------*/
int
kdas_pres_put_data(
   kdas_presentation *pres, 
   int   	     *begin, 
   int   	     *end, 
   kaddr 	      data)  
{
   int zero_array    [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clipped_begin [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clipped_end   [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int clip_unit     [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int block_begin   [KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int block_size    [KDAS_MAX_DIM];

   int upcast      = (pres->datatype < pres->segment->datatype);
   int downcast    = (pres->datatype > pres->segment->datatype);
   int transpose   = _need_to_transpose(pres);
   int convert     = _need_to_complex_convert(pres);
   int dimension   = pres->segment->dimension;
   int interpolate = pres->access_interpolate;
   int total;
   int i;

   int clip    = FALSE;
   int outside = FALSE;

   kaddr cdata = NULL;  /*-- data clipped from 'data' --*/

   /*-- sanity check --*/
   if (!begin || !end)
      return FALSE;  /*-- kerror : invalid data access --*/

   /*-- gotta have something to put! --*/
   if (data == NULL)
      return FALSE;

   /*-- figure out the overall size of the data being stored --*/
   for (i = 0, total = 1; i < dimension; total *= block_size[i], i++)
      block_size[i] = end[i] - begin[i] + 1;

   /*----------------------------------------------------------------
   |   determine if are putting data outside our presentation size
   \-----------------------------------------------------------------*/
   kdas_clip_corners(dimension, pres->size, begin, end, &clip, &outside, 
		     clipped_begin, clipped_end, block_begin, clip_unit);
   
   /*----------------------------------------------------------------
   |   clipping check : 
   |       if we are out of bounds and shouldn't be, leave now
   |       if we are totally outside, then also leave now
   \-----------------------------------------------------------------*/
   if (clip)
   { 
      if (pres->size_interpolate == KNONE)
	 return FALSE;

      /*-- this should never happen: its meaningless for data access --*/
      if (interpolate == KZERO_ORDER)  
	 return FALSE;

      if (outside)
	 return TRUE; /*-- trivial put : not an error --*/
   }


   /*----------------------------------------------------------------
   |   set up the block buffer in which to do clipping and pipeline: 
   |
   |     if we are doing any fancy presentation stuff, we'll need 
   |     our own copy of the data as we will be doing a lot of in-place
   |     casting and converting.
   |
   |     if we are clipping, then do that now.  otherwise, copy the 
   |     data into our buffer.
   |
   |     if we are not doing any fancy presentation stuff, we 
   |     can just put the initial data buffer.
   \-----------------------------------------------------------------*/
   if (upcast || downcast || transpose || convert || clip)
   {
      cbuffer = kdas_adjust_buffer_size(cbuffer, dimension, clip_unit,
					_bigger_type(pres->segment->datatype,
					     	     pres->datatype));
      cdata = cbuffer->data;
      
      /*-- if we are clipping, clip the data from we need to put --*/
      if (clip)
	 kdas_memory_clip(cdata, data, dimension, pres->datatype, clip_unit,
			  block_size, NULL, zero_array, block_begin, 
			  clip_unit, KDAS_WRITE);
      else
	 kmemcpy(cdata, data, (size_t) total * kdata_size(pres->datatype));
   }
   else
      cdata = data;
   /*----------------------------------------------------------------
   |   cdata now contains       size     : "clip_unit" 
   |   the data we need         datatype : "segment->datatype" 
   |   to pipeline :	        order    : "segment->order"
   \-----------------------------------------------------------------*/

   /*------------------------------------------------------------------
   |   pipeline the puppy :
   |   this deals with the remainder of the presentation
   |   	  o  complex conversion
   |   	  o  upcasting
   |   	  o  scaling
   |   	  o  normalization
   |   	  o  downcasting
   |
   |   don't worry about losing cdata as a memory leak;
   |   the cdata pointer at this point is pointing to either :
   |      o  the data being put
   |      o  a temporary buffer, which we don't want to free until
   |         data services is no longer needed
   |   so we can safely assign it to the return value.
   \-------------------------------------------------------------------*/
   cdata = kdas_pipeline_data(pres, upcast, downcast, convert, clip_unit, 
			      FALSE, cdata);

   /*----------------------------------------------------------------
   |   put the data into data segment, transposing it before we put
   \-----------------------------------------------------------------*/
   return _put_transposed_data(pres, clipped_begin, clipped_end, clip_unit, 
			       transpose, cdata);
}

/****    =============================================================
 ****    Static Utility Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _need_to_transpose
|
|       Purpose: This routine determines if it is necessary
|		 to transpose data in presentating data from 
|		 a given data presentation. 
|
|         Input: pres - presentation to examine
|
|        Output: none
|
|       Returns: TRUE if a transpose is necessary, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_need_to_transpose(kdas_presentation *pres)
{
   int *sorder = pres->segment->order;
   int *porder = pres->order;

   int dim = pres->segment->dimension;
   int i;
   int same = TRUE;

   if (sorder == NULL || porder == NULL)
      return FALSE;

   for (i = 0; i < dim; i++)
      same &= (sorder[i] == porder[i]);

   return (!same);
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _need_to_complex_convert
|
|       Purpose: This routine determines if it is necessary
|		 to complex convert data in presentating data from 
|		 a given data presentation. 
|
|         Input: pres - presentation to examine
|
|        Output: none
|
|       Returns: TRUE if complex conversion is necessary, 
|		 FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: October 16, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_need_to_complex_convert(kdas_presentation *pres)
{
   int sdatatype = pres->segment->datatype;
   int pdatatype = pres->datatype;
   
   if ((sdatatype == KCOMPLEX || sdatatype == KDCOMPLEX) &&
       (pdatatype != KCOMPLEX && pdatatype != KDCOMPLEX))
      return TRUE;

   if ((pdatatype == KCOMPLEX || pdatatype == KDCOMPLEX) &&
       (sdatatype != KCOMPLEX && sdatatype != KDCOMPLEX))
      return TRUE;

   return FALSE;
   
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _wrap_region
|
|       Purpose: This routine is used with kdas_wrap_data to 
|		 extract a region from the presentation and 
|		 insert it into a data block.
|
|    Written By: Steve Kubica
|          Date: January 6, 1996
| Modifications:
|
------------------------------------------------------------*/
static void
_wrap_region(
   kaddr source,
   int  *data_size,
   int  *block_size,
   int  *sb,
   int  *se,
   int  *db,
   int  *de,
   int  *unit,
   kaddr destination)
{
   kdas_presentation *pres = (kdas_presentation *) source;
   int   zero[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   kaddr region = NULL;
   

   /*-- read this piece of the block from the segment... --*/
   region = kdas_segment_get_data(pres->segment, sb, se, pres->size, 
				  pres->size_interpolate, 
				  pres->rpad, pres->ipad, NULL);

   /*-- ...and stuff it into the overall region --*/
   if (region)
      kdas_memory_clip(destination, region, pres->dimension, 
		       pres->segment->datatype, block_size, unit, block_size,
		       db, zero, unit, KDAS_WRITE);
   kfree(region);
			 
   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _get_transposed_data
|
|       Purpose: This routine will retrieve a block of data from 
|		 a segment, transposing it after it gets it, if needed.
|
|	  	 The purpose of this routine is primarily to hide the
|		 nasty array reordering required when switching index
|		 order.  It does all the reordering before going to 
|		 the segment level here.   Above this routine in the
|		 presentation layer and in the pipeline, arrays
|		 should all be in the presentation index order, below
|		 this routine in the segment layer, arrays should
|		 all be in the physical index order.
|		
|		 The data returned from this routine will be in
|		 the presentation index order.
|	  	 
|		 This routine also deals with any presentation 
|		 wrapping we might have to do during a get data 
|		 call.
|
|         Input: pres          - the presentation with the segment that 
|		                 we are getting data from
|		 clipped_begin - begin point in segment from which to get
|		 clipped_end   - end point in segment from which to get  
|		 clip_unit     - unit size to get
|		 begin         - "real" unclipped begin point in 
|				 presentation space, needed to calculate
|				 the wrap partitions
|		 transpose     - TRUE if transposing
|		 clip          - TRUE if clipping
|		 interpolate   - interpolate type to check for wrapping
|		 cdata         - array to retrieve the data into
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: February 5, 1996
| Modifications:
|
------------------------------------------------------------*/
static kaddr
_get_transposed_data(
   kdas_presentation *pres, 
   int               *clipped_begin, 
   int               *clipped_end,
   int               *clip_unit,
   int               *begin, 
   int                transpose,
   int                clip,
   int                interpolate,
   kaddr              cdata)
{
   int clipped_begin_s [KDAS_MAX_DIM], *clipped_begin_t = clipped_begin;
   int clipped_end_s   [KDAS_MAX_DIM], *clipped_end_t   = clipped_end;
   int clip_unit_s     [KDAS_MAX_DIM], *clip_unit_t     = clip_unit;
   int begin_s         [KDAS_MAX_DIM], *begin_t         = begin;
   int apparent_size_s [KDAS_MAX_DIM], *apparent_size_t = pres->size;
   
   int  dimension  = pres->dimension;
   int *pres_order = pres->order;
   int *seg_order  = pres->segment->order;
   
   kaddr tdata;

   /*----------------------------------------------------------------
   |   worry about the order of our arrays if we are transposing
   \-----------------------------------------------------------------*/
   if (transpose)
   {
      /*-- make our own reorderable copies of these arrays --*/ 
      kmemcpy(clipped_begin_s, clipped_begin, pres->dimension * sizeof(int));
      kmemcpy(clipped_end_s,   clipped_end,   pres->dimension * sizeof(int));
      kmemcpy(clip_unit_s,     clip_unit,     pres->dimension * sizeof(int));
      kmemcpy(begin_s,         begin,         pres->dimension * sizeof(int));
      kmemcpy(apparent_size_s, pres->size,    pres->dimension * sizeof(int));

      clipped_begin_t = &clipped_begin_s[0];
      clipped_end_t   = &clipped_end_s[0];
      clip_unit_t     = &clip_unit_s[0];
      begin_t         = &begin_s[0];
      apparent_size_t = &apparent_size_s[0];

      /*-- reorder our arrays to reflect the segment ordering --*/ 
      kdas_array_reorder(clipped_begin_t, dimension, pres_order, seg_order);
      kdas_array_reorder(clipped_end_t,   dimension, pres_order, seg_order);
      kdas_array_reorder(clip_unit_t,     dimension, pres_order, seg_order);
      kdas_array_reorder(begin_t,         dimension, pres_order, seg_order);
      kdas_array_reorder(apparent_size_t, dimension, pres_order, seg_order);

      /*-- set up a transposing buffer --*/ 
      tbuffer = kdas_adjust_buffer_size(tbuffer, dimension, clip_unit,
					_bigger_type(pres->segment->datatype, 
					             pres->datatype));
      tdata = tbuffer->data;
   }
   else
      tdata = cdata;

   /*----------------------------------------------------------------
   |   get the data from the segment : 
   |      we will either clip or wrap the data into the tdata buffer.
   |      if we are transposing, then we will transpose from tdata into
   |      cdata after we get this, otherwise tdata *is* cdata. 
   \-----------------------------------------------------------------*/
   if (clip && (interpolate == KWRAP))
      kdas_wrap_data(pres, dimension, apparent_size_t, clip_unit_t,
		     begin_t, _wrap_region, tdata);
   else
      tdata = kdas_segment_get_data(pres->segment,
				    clipped_begin_t, clipped_end_t,
				    apparent_size_t, pres->size_interpolate, 
				    pres->rpad, pres->ipad, tdata);

   /*-- make sure we got something back from the segment --*/
   if (tdata == NULL)
      return NULL;

   /*----------------------------------------------------------------
   |   transpose the segment data into our presentation order
   |
   |   NOTE : if a transpose isn't absolutely necessary, the tdata
   |          pointer will be returned by this function
   \-----------------------------------------------------------------*/
   if (transpose)
      return kdas_transpose_data(tdata, dimension, 
				 pres->segment->datatype,
				 clip_unit_t, clip_unit, seg_order, 
				 pres_order, cdata);
   else
      return tdata;
}   

/*-----------------------------------------------------------
|
|  Routine Name: (static) _put_transposed_data
|
|       Purpose: This routine will put a block of data into a segment,
|		 transposing it first, if needed.
|
|	  	 The purpose of this routine is primarily to hide the
|		 nasty array reordering required when switching index
|		 order.  It does all the reordering before going to 
|		 the segment level here.   Above this routine in the
|		 presentation layer and in the pipeline, arrays
|		 should all be in the presentation index order, below
|		 this routine in the segment layer, arrays should
|		 all be in the physical index order.
|
|		 The data passed into this routine is assumed to be 
|		 in the presentation index order.  It is transposed
|		 before it is put into the segment.
|	  	 
|         Input: pres          - the presentation with the segment that 
|		                 we are putting data into
|		 clipped_begin - begin point in segment from which to put
|		 clipped_end   - end point in segment from which to put
|		 clip_unit     - unit size to put
|		 transpose     - TRUE if transposing
|		 cdata         - data array to store 
|
|        Output: cdata is stored into pres->segment
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: February 5, 1996
| Modifications:
|
------------------------------------------------------------*/
static int
_put_transposed_data(
   kdas_presentation *pres, 
   int               *clipped_begin, 
   int               *clipped_end,
   int               *clip_unit,
   int                transpose,
   kaddr              cdata)
{
   int clipped_begin_s [KDAS_MAX_DIM], *clipped_begin_t = clipped_begin;
   int clipped_end_s   [KDAS_MAX_DIM], *clipped_end_t   = clipped_end;
   int clip_unit_s     [KDAS_MAX_DIM], *clip_unit_t     = clip_unit;
   int apparent_size_s [KDAS_MAX_DIM], *apparent_size_t = pres->size;

   int  dimension  = pres->dimension;
   int *pres_order = pres->order;
   int *seg_order  = pres->segment->order;
   
   kaddr tdata;


   /*----------------------------------------------------------------
   |   worry about the order of our arrays if we are transposing
   \-----------------------------------------------------------------*/
   if (transpose)
   {
      /*-- make our own reorderable copies of these arrays --*/ 
      kmemcpy(clipped_begin_s, clipped_begin, pres->dimension * sizeof(int));
      kmemcpy(clipped_end_s,   clipped_end,   pres->dimension * sizeof(int));
      kmemcpy(clip_unit_s,     clip_unit,     pres->dimension * sizeof(int));
      kmemcpy(apparent_size_s, pres->size,    pres->dimension * sizeof(int));

      clipped_begin_t = &clipped_begin_s[0];
      clipped_end_t   = &clipped_end_s[0];
      clip_unit_t     = &clip_unit_s[0];
      apparent_size_t = &apparent_size_s[0];

      /*-- reorder our arrays to reflect the segment ordering --*/ 
      kdas_array_reorder(clipped_begin_t, dimension, pres_order, seg_order);
      kdas_array_reorder(clipped_end_t,   dimension, pres_order, seg_order);
      kdas_array_reorder(clip_unit_t,     dimension, pres_order, seg_order);
      kdas_array_reorder(apparent_size_t, dimension, pres_order, seg_order);

      /*-- set up a transposing buffer --*/ 
      tbuffer = kdas_adjust_buffer_size(tbuffer, dimension, clip_unit,
					_bigger_type(pres->segment->datatype, 
					             pres->datatype));
      tdata = tbuffer->data;

      /*-------------------------------------------------------------
      | transpose the segment data into our presentation order
      |
      | NOTE : if a transpose isn't absolutely necessary, the cdata
      |        pointer will be returned and assigned to tdata
      \--------------------------------------------------------------*/
      tdata = kdas_transpose_data(cdata, dimension,
				  pres->segment->datatype,
				  clip_unit, clip_unit_t, pres_order, 
				  seg_order, tdata);
   }
   else
      tdata = cdata;

   /*----------------------------------------------------------------
   |   put the data into the segment : 
   |      we will put the tdata buffer into the segment.
   |      if we are transposing, then we tdata is the transposed
   |      cdata, otherwise tdata *is* cdata. 
   \-----------------------------------------------------------------*/
   return kdas_segment_put_data(pres->segment, clipped_begin_t, clipped_end_t,
				apparent_size_t, pres->size_interpolate, 
				tdata);
}   

/*-----------------------------------------------------------
|
|  Routine Name: (static) _bigger_type
|
|       Purpose: Given two data types, return the one which takes up
|                more storage space.  Keep in mind that, depending
|                on the architecture, a "larger" data type may
|                take up less space.  A good example is on the 
|                DEC Alpha, a KULONG is 8 bytes, while a float 
|                is 4 bytes. 
|
|         Input: type1 - first type
|                type2 - second type
|       Returns: larger type
|
|    Written By: Steve Kubica and John Salas
|          Date: June 30, 1996  
| Modifications:
|
------------------------------------------------------------*/
static int
_bigger_type(int type1, int type2)
{
   if (kdata_size(type1) > kdata_size(type2))
      return type1;
   else
      return type2;
}
