/*
  * Khoros: $Id: sampo.c,v 1.4 1996/06/08 18:34:29 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: sampo.c,v 1.4 1996/06/08 18:34:29 mirage Exp $";
#endif


/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Management Services Test Code
   >>>>		
   >>>>		"We have passed the tests, we have earned the sampo!!"
   >>>>		
   >>>>  Private:
   >>>>                 kdas_test_mode()
   >>>>                 kdas_test_architecture()
   >>>>                 kdas_object_open()
   >>>>                 kdas_object_close()
   >>>>                 kdas_object_get_fid()
   >>>>                 kdas_object_set_coupling()
   >>>>                 kdas_object_put_data()
   >>>>                 kdas_object_get_data()
   >>>>                 kdas_object_set_all()
   >>>>                 kdas_object_get_all()
   >>>>                 kdas_object_get_physical()
   >>>>                 kdas_object_set_presentation_size()
   >>>>                 kdas_object_set_presentation_order()
   >>>>                 kdas_object_set_presentation_datatype()
   >>>>                 kdas_object_set_presentation_scaling()
   >>>>                 kdas_object_set_presentation_normalization()
   >>>>                 kdas_object_set_presentation_complex_convert()
   >>>>   Static:
   >>>>                 initialize_object()
   >>>>                 test_check()
   >>>>                 test_input()
   >>>>                 test_output()
   >>>>                 test_destroy()
   >>>>                 test_locate_segment()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

/*-- this is an useful typedef to have in dbx --*/
typedef struct _kobject _test_object;

static int test_check 	       PROTO((int));
static int test_input 	       PROTO((kobject, int, int));
static int test_output         PROTO((kobject, int, int));
static int test_destroy        PROTO((kobject));
static int test_locate_segment PROTO((kobject, char *));

static kobject initialize_object PROTO((int));

/*-- format identifier for the test format --*/
unsigned char test_id[3] = {36, 22, 36};

#define KOBJ_TEST 69

   /*----------------------------------------------------------------
   |   kobject test structure
   |   . . . . . . . . . . . . . . 
   |
   |   This is a proto-object, to be used for purposes of testing
   |   the format code in conjunction with the segment and presentation.
   \-----------------------------------------------------------------*/
struct _kobject
{
   int type;  /*-- kobject identifier --*/
   
   int fid;
   int flags;

   int offset;
   kdas_format       *format;

   kdas_presentation *pres;     /*-- NULL if testing only segment --*/
   kdas_segment      *segment;  /*-- NULL if testing only presentation --*/

   /*-- presentation values to use if testing segment only --*/
   int          *apparent_size;
   double        rpad, ipad;
   int           interpolate;
};

/****    =============================================================
 ****    Testing Modes
 ****    =============================================================
 ****/

static int test_mode  = KDAS_TEST_SEGMENT;
static int test_large = FALSE;

static unsigned char test_mach  = KNONE;

/*-----------------------------------------------------------
|
|  Routine Name: kdas_test_mode
|
|       Purpose: This routine is used to set the mode and the functions
|
|         Input: mode  - the mode to test in set according to what we
|		      	 want to test.  it will either be                    
|		      	 KDAS_TEST_SEGMENT or KDAS_TEST_PRESENTATION
|	         large - TRUE if large data set handling functions
|		         should be used when testing.
|
|        Output: none
|
|    Written By: Steve Kubica
|          Date: November 1, 1995
| Modifications:
|
------------------------------------------------------------*/
int kdas_test_mode(int mode, int large)
{
   if (mode != KDAS_TEST_SEGMENT && mode != KDAS_TEST_PRESENTATION)
   {
      kerror("kdataccess", "kdas_test_mode", 
	     "Invalid test mode set. Must use either KDAS_TEST_SEGMENT "
	     "or KDAS_TEST_PRESENTATION");
      return FALSE;
   }
   else
      test_mode = mode;

   test_large = large;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_test_architecture
|
|       Purpose: This routine is used to set the test architecture
|		 to be used when opening up test output objects
|
|         Input: mach - the mach id to use, KNONE to use local
|
|        Output: none
|
|    Written By: Steve Kubica
|          Date: January 9, 1996
| Modifications:
|
------------------------------------------------------------*/
int kdas_test_architecture(unsigned char mach)
{
   test_mach = mach;

   return TRUE;
}

/****    =============================================================
 ****    Object Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|  Routine Name: kdas_object_open
|       Purpose: This routine will open a test object
|		 given a file name and a flag.
|    Written By: Steve Kubica
|          Date: November 1, 1995
------------------------------------------------------------*/
kobject 
kdas_object_open(char *filename, int flags)
{
   kobject object;
   int     fid;
   int     kflags   = kdas_to_kfile_flags(flags);
   int     coupling = (flags & KOBJ_READ) ? KUNCOUPLED : KCOUPLED;
   

   /*-- initialize the object depending on the mode --*/
   object = initialize_object(coupling);

   /*-- open the file with the specified flags --*/
   fid = kopen(filename, kflags | KOPEN_LOCK, 0666);

   /*-- set the test architecture, if needed --*/
   if (test_mach != KNONE && flags & KOBJ_WRITE)
      kfile_setmachtype(fid, test_mach); 

   /*-- is this an input file? --*/
   if (flags & KOBJ_READ)
   {
      if (test_check(fid))
	 object->format->input(object, fid, flags);
      else
      {
	 kerror("kdataccess", "kdas_open_object", 
		"File is not in the test format");
	 return FALSE;
      }
   }

   object->fid   = fid;
   object->flags = flags;
   object->format->flags = flags;

   return object;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_close
|       Purpose: This routine will close a test object.
|    Written By: Steve Kubica
|          Date: November 1, 1995
------------------------------------------------------------*/
int 
kdas_object_close(kobject object)
{
   int kflags;

   if (!object || object->type != KOBJ_TEST)
      return TRUE;

   /*-- call the output method if we are in fact the output format --*/
   if (object->flags & KOBJ_WRITE)
   {
      kflags = kdas_to_kfile_flags(object->flags);
      
      klseek(object->fid, (off_t) 0, KSEEK_SET);
      object->format->output(object, object->fid, kflags);

      if (object->format->locate_segment)
      {
	 if (object->pres)
	    kdas_segment_write(object->pres->segment);
	 else
	    kdas_segment_write(object->segment);
      }
   }

   if (object->segment)
      kdas_segment_destruct(object->segment);

   if (object->pres)
      kdas_pres_destruct(object->pres);

   kdas_format_destruct(object->format);

   /*-- clear this out for sanity --*/
   object->type = KNONE;

   kfree(object->apparent_size);
   kclose(object->fid);
   kfree(object);

   return TRUE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_get_fid
|       Purpose: This routine will return the fid associated 
|		 with the data object.  DO NOT CLOSE IT!
|    Written By: Steve Kubica
|          Date: November 19, 1995
------------------------------------------------------------*/
int 
kdas_object_get_fid(kobject object)
{
   if (!object || object->type != KOBJ_TEST)
      return -1;
   else
      return object->fid;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_coupling
|       Purpose: This routine will set the coupling on a
|		 data object to KUNCOUPLED, KDEMAND, or KCOUPLED.
|    Written By: Steve Kubica
|          Date: February 26, 1996
------------------------------------------------------------*/
int 
kdas_object_set_coupling(kobject object, int coupling)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   if (object->pres)
   {
      object->pres->coupling = coupling;
      return TRUE;
   }
   else
      return FALSE;
}


/*-----------------------------------------------------------
|  Routine Name: kdas_object_put_data
|       Purpose: This routine will insert data into a test object.
|    Written By: Steve Kubica
|          Date: November 19, 1995
------------------------------------------------------------*/
int 
kdas_object_put_data(kobject object, int *begin, int *end, kaddr data)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return kdas_segment_put_data(object->segment, begin, end, 
				   object->apparent_size, object->interpolate,
				   data);
   else if (object->pres)
      return kdas_pres_put_data(object->pres, begin, end, data);

   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_get_data
|       Purpose: This routine will retrieve data from a test object.
|    Written By: Steve Kubica
|          Date: November 19, 1995
------------------------------------------------------------*/
kaddr 
kdas_object_get_data(kobject object, int *begin, int *end, kaddr data)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return kdas_segment_get_data(object->segment, begin, end, 
				   object->apparent_size, object->interpolate,
				   object->rpad, object->ipad, data);
   else if (object->pres)
      return kdas_pres_get_data(object->pres, begin, end, data);

   else
      return NULL;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_all
|       Purpose: This routine will set all data characteristics 
|		 of the segment.
|    Written By: Steve Kubica
|          Date: November 1, 1995
------------------------------------------------------------*/
int 
kdas_object_set_all(
   kobject object,
   int     dimension,
   int    *size,
   int    *order,
   int     datatype,
   kaddr   data)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return kdas_segment_set_all(object->segment, dimension, size, order, 
				  datatype, data);

   else if (object->pres)
      return kdas_pres_set_all(object->pres, dimension, size, order, 
			       datatype, data);
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_get_all
|       Purpose: This routine will retrieve all data characteristics 
|		 of the segment.
|    Written By: Steve Kubica
|          Date: November 1, 1995
------------------------------------------------------------*/
int 
kdas_object_get_all(
   kobject object,
   int    *dimension,
   int   **size,
   int   **order,
   int    *datatype, 
   kaddr  *data)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;
   
   if (object->segment)
      return kdas_segment_get_all(object->segment, 
			       dimension, size, order, datatype, data);

   else if (object->pres)
   {
      if (dimension)
	 kdas_pres_get_dimension(object->pres, dimension);

      if (size)
	 kdas_pres_get_size(object->pres, size);

      if (order)
	 kdas_pres_get_order(object->pres, order);

      if (datatype)
	 kdas_pres_get_datatype(object->pres, datatype);

      if (data)
	 *data = object->pres->segment->data;
      
      return TRUE;
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_get_physical
|       Purpose: This routine will get all the physical characteristics
|		 of the test segment.
|    Written By: Steve Kubica
|          Date: February 26, 1996
------------------------------------------------------------*/
int 
kdas_object_get_physical(
   kobject object,
   int    *dimension,
   int   **size,
   int   **order,
   int    *datatype, 
   kaddr  *data)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;
   
   if (object->segment)
      return kdas_segment_get_all(object->segment, 
				  dimension, size, order, datatype, data);

   else if (object->pres)
      return kdas_segment_get_all(object->pres->segment, 
				  dimension, size, order, datatype, data);
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_size
|       Purpose: This routine will assign the presentation size
|    Written By: Steve Kubica
|          Date: November 19, 1995
------------------------------------------------------------*/
int 
kdas_object_set_presentation_size(
   kobject object, 
   int    *apparent_size, 
   int     interpolate, 
   double  rpad, 
   double  ipad)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object && object->segment)
   {
      object->apparent_size = (int *) kdupalloc(apparent_size, 
						sizeof(int) *
						object->segment->dimension);
      object->interpolate = interpolate;
      object->rpad = rpad;
      object->ipad = ipad;

      return TRUE;
   }
   else if (object->pres)
   {
      object->pres->size_interpolate = interpolate;

      /*-- can't zero order interpolate access, only size --*/ 
      if (interpolate == KZERO_ORDER) 
	 object->pres->access_interpolate = KPAD;
      else
	 object->pres->access_interpolate = interpolate;
      
      object->pres->rpad = rpad;
      object->pres->ipad = ipad;

      return kdas_pres_set_size(object->pres, apparent_size);
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_order
|       Purpose: This routine will assign the presentation order
|    Written By: Steve Kubica
|          Date: November 19, 1995
------------------------------------------------------------*/
int 
kdas_object_set_presentation_order(
   kobject object, 
   int    *order) 
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   else if (object->pres)
      return kdas_pres_set_order(object->pres, order);

   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_datatype
|       Purpose: This routine will assign the presentation datatype
|    Written By: Steve Kubica
|          Date: February 24, 1996
------------------------------------------------------------*/
int 
kdas_object_set_presentation_datatype(
   kobject object, 
   int     datatype) 
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   else if (object->pres)
   {
      object->pres->datatype = datatype;
      return TRUE;
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_scaling
|       Purpose: This routine will assign presentation scaling fields
|    Written By: Steve Kubica
|          Date: February 26, 1996
------------------------------------------------------------*/
int 
kdas_object_set_presentation_scaling(
   kobject object, 
   double  scale_factor,
   double  scale_roffset,
   double  scale_ioffset) 
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   else if (object->pres)
   {
      object->pres->scaling = KSCALE;
      object->pres->scale_factor  = scale_factor;
      object->pres->scale_roffset = scale_roffset;
      object->pres->scale_ioffset = scale_ioffset;
      
      return TRUE;
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_normalization
|       Purpose: This routine will assign normalization fields
|    Written By: Steve Kubica
|          Date: February 26, 1996
------------------------------------------------------------*/
int 
kdas_object_set_presentation_normalization(
   kobject object, 
   double  norm_min,
   double  norm_max) 
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   else if (object->pres)
   {
      object->pres->scaling = KNORMALIZE;
      object->pres->norm_min = norm_min;
      object->pres->norm_max = norm_max;
      
      return TRUE;
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: kdas_object_set_presentation_complex_convert
|       Purpose: This routine will assign the complex convert field
|    Written By: Steve Kubica
|          Date: February 26, 1996
------------------------------------------------------------*/
int 
kdas_object_set_presentation_complex_convert(
   kobject object, 
   int     convert)
{
   if (!object || object->type != KOBJ_TEST)
      return FALSE;

   if (object->segment)
      return FALSE;  /*-- can't even fake this one --*/
   
   else if (object->pres)
   {
      object->pres->complex_convert = convert;

      return TRUE;
   }
   else
      return FALSE;
}

/*-----------------------------------------------------------
|  Routine Name: (static) initialize_object
|       Purpose: This routine initializes an object according
|		 to the testing mode.
|    Written By: Steve Kubica
|          Date: November 1, 1995
------------------------------------------------------------*/
static kobject 
initialize_object(
   int coupling)
{
   kobject            object;
   kdas_format       *format;

   /*-- initialize the format depending on the mode --*/
   if (test_large)
      format = kdas_format_construct("test", "data access test format",
				     test_check, test_input, test_output, 
				     test_destroy, NULL, test_locate_segment,
				     NULL, NULL, NULL, NULL);

   else  /*-- .. use no large data set support functions --*/
      format = kdas_format_construct("test", "data access test format",
				     test_check, test_input, test_output, 
				     test_destroy, NULL, NULL, NULL, NULL, 
				     NULL, NULL);

   object = (kobject) kmalloc(sizeof(struct _kobject));

   object->type = KOBJ_TEST;

   object->format  = format;
   object->segment = NULL;
   object->pres    = NULL;

   /*-- set resonable defaults for these --*/
   object->apparent_size = NULL;
   object->interpolate   = KNONE;
   object->rpad          = 0.0;
   object->ipad          = 0.0;

   /*-- initialize a segment or a presentation depending on our mode --*/
   if (test_mode == KDAS_TEST_SEGMENT)
   {
      object->segment = kdas_segment_construct("value");
      object->segment->format = format;
      object->segment->object = object;
   }
   else  /*-- KDAS_TEST_PRESENTATION --*/
   {
      object->pres = kdas_pres_construct("value", coupling);
      object->pres->segment->format = format;
      object->pres->segment->object = object;
   }
   
   return object;
}


/****    =============================================================
 ****    Test Format Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) test_check
|
|       Purpose: This routine will check an open fid to see if
|		 it is stored in the test format.  
|	
|         Input: fid - the file to read from when checking
|        Output: none
|       Returns: TRUE if the file is in the test format, 
|		 FALSE otherwise
|
|    Written By: Steve Kubica & Mark Young
|          Date: November 19, 1995
| Modifications:
|
------------------------------------------------------------*/
static int 
test_check(int fid)
{
   unsigned char tmp[3];

   if ((kread_ubyte(fid, tmp, (size_t) 3) != 3) || 
       (kmemcmp(test_id, tmp, (size_t) 3) != 0))
      return FALSE;

   return TRUE;
}
   
/*-----------------------------------------------------------
|
|  Routine Name: (static) test_input
|
|       Purpose: This routine will input a file stored in the
|		 test file format.
|	
|		 This file format contains a single segment
|		 named "value".
|
|		 The format header is as follows
|	         ---------------------------------------------
|		    identifier {36, 22, 36}  : 3 bytes
|		    machine identifier       : 1 byte
|		    dimension                : 1 int
|		    size                     : dimension ints
|		    order                    : dimension ints
|		    datatype                 : 1 int
|	         ---------------------------------------------
|		 The actual data is then stored after that in 
|		 a continuous block in the machine native format.
|
|		 Depending on the testing mode, it will either 
|		 access the object through its presentation, 
|		 or will ignore the presentation and access
|		 the segment directly.   It also will deal
|		 with the data differently depending on whether
|		 or not we are testing large data set functionality.
|
|         Input: object - object to read data into
|		 fid    - fid opened to the beginning of the file to read
|		 flags  - data service flags used when opening
|        Output: none
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: November 19, 1995
| Modifications:
|
------------------------------------------------------------*/
static int
test_input(
   kobject object,
   int     fid,
   int     flags)
{
   unsigned char mach;
   int  dimension;
   int *size;
   int *order;
   int  total;
   int  datatype;
   int  st;
   int  i;

   kaddr data = NULL;

   
   /*-- determine the machine type for this --*/
   kread_ubyte(fid, &mach, (size_t) 1);
   kfile_setmachtype(fid, (int) mach);

   kread_int(fid, &dimension, (size_t) 1);

   size = (int *) kmalloc(sizeof(int) * dimension);
   order = (int *) kmalloc(sizeof(int) * dimension);

   kread_int(fid, size, (size_t) dimension);  
   kread_int(fid, order,  (size_t) dimension);  
   kread_int(fid, &datatype, (size_t) 1);

   /*-- we have read the header and are now at the segment --*/
   object->offset = (int) ktell(fid);

   for (i = 0, total = 1; i < dimension; i++)
      total *= size[i]; 

   /*-- if we can't handle large data sets, then read in the data now --*/
   if (!test_large)
   {
      if ((data = (kaddr) 
	   kmalloc((size_t)(kdata_size(datatype)*total))) == NULL)
	 return FALSE;
      kread_generic(fid, data, (size_t) total, datatype);
   }
   /*-- if we didn't read in the data yet, it will be deferred until later --*/
   
   /*-- assign the information, and data if we have it, to the object --*/
   st = kdas_object_set_all(object, dimension, size, order, datatype, data);

   kfree(size);
   kfree(order);

   return st;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) test_output
|
|       Purpose: This routine will output a data object to
|		 a file in a test file format.
|		 
|		 This file format contains a single segment
|		 named "value".   The header for the 
|		 test_input() function contains a description
|		 of the format.
|
|		 Depending on the testing mode, it will either 
|		 access the object through its presentation, 
|		 or will ignore the presentation and access
|		 the segment directly.   It also will deal
|		 with the data differently depending on whether
|		 or not we are testing large data set functionality.
|
|         Input: object - object to output
|        Output: none
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: November 19, 1995
| Modifications:
|
------------------------------------------------------------*/
static int 
test_output(kobject object, int fid, int flags)
{
   unsigned char mach;
   int  dimension;
   int *size;
   int *order;
   int  datatype;
   int  i;

   kaddr data = NULL;

   /*-- write out the format identifier --*/
   kwrite_ubyte(fid, test_id, (size_t) 3);
   
   /*-- write out the local machine type or our test_mach if defined --*/
   mach = (test_mach == KNONE) ? kmach_local() : test_mach;
   kwrite_ubyte(fid, &mach, (size_t) 1);

   /*-- get the physical segment information, if any --*/
   if (!kdas_object_get_physical(object, &dimension, &size, &order, 
				 &datatype, &data))
      return TRUE;

   kwrite_int(fid, &dimension, (size_t) 1);
   kwrite_int(fid, size, (size_t) dimension);  
   kwrite_int(fid, order, (size_t) dimension);  
   kwrite_int(fid, &datatype, (size_t) 1);

   /*-- we have written the header and are now at the segment --*/
   object->offset = (int) ktell(fid);

   /*-- if we can't handle large data sets, then write out the data now --*/
   if (!test_large && data)
   {
      int total;
      
      for (i = 0, total = 1; i < dimension; i++)
	 total *= size[i]; 

      kwrite_generic(fid, data, (size_t) total, datatype);
   }
   
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) test_destroy
|
|       Purpose: This routine will free any resources which 
|		 we may have used -- which means for this test
|		 format it does nothing.
|
|         Input: object - object from which to free resources
|        Output: none
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: November 19, 1995
| Modifications:
|
------------------------------------------------------------*/
static int 
test_destroy(kobject object)
{
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) test_locate_segment
|
|       Purpose: This routine will locate a segment in the 
|		 test format.  The only segment contained
|		 in the test format is the "value" segment.
|
|         Input: object - the object for which to locate the segment
|		 name   - the name of the segment to locate
|        Output: none
|       Returns: the offset on success, 0 on failure.
|
|    Written By: Steve Kubica
|          Date: November 19, 1995
| Modifications:
|
------------------------------------------------------------*/
static int 
test_locate_segment(kobject object, char *name)
{
   int fid;
   ktoken oid;
   
   if (object->pres)
      oid = object->pres->id;
   else if (object->segment)
      oid = object->segment->id;
   else
      return -1;
   
   /*-- make sure this is the one segment that we have --*/
   if (kstrcmp(name, ktoken_to_string(oid)) != 0)
      return -1;
   else 
   {
      fid = kdas_object_get_fid(object);
      klseek(fid, (off_t) object->offset, KSEEK_SET);
      return fid;
   }
}

