 /*
  * Khoros: $Id: transport.c,v 1.15 1996/06/08 18:34:39 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: transport.c,v 1.15 1996/06/08 18:34:39 mirage Exp $";
#endif


/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Transport Code
   >>>>		
   >>>>  Private:
   >>>>                 kdas_temp_construct()
   >>>>                 kdas_temp_destruct()
   >>>>                 kdas_temp_recast()
   >>>>                 kdas_temp_resize()
   >>>>                 kdas_temp_reorder()
   >>>>                 kdas_temp_copy()
   >>>>                 kdas_transport_clip()
   >>>>                 kdas_transport_interp()
   >>>>   Static:
   >>>>                 __transport_clip()
   >>>>                 __transport_bit_clip()
   >>>>                 __transport_interp()
   >>>>                 __transport_bit_interp()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <float.h>

static void __transport_clip       PROTO((kdas_transport *, kubyte *, int, 
					  int *, int *, int *, int *, int, 
					  int, int));

static void __transport_bit_clip   PROTO((kdas_transport *, kubyte *, kubyte *,
					  int, int *, int *, int *, int *, 
					  int *, int, int, int, int, int,
					  int));

static void __transport_interp     PROTO((kdas_transport *, kubyte *, 
					  kubyte *, int, int, int *, int *, 
					  int *, int *, int *, int *, int *, 
					  int *, int *, int *, int *, int *, 
					  double *, int, int, int, int));

static void __transport_bit_interp PROTO((kdas_transport *, kubyte *, 
					  kubyte *, int, int, int *, int *, 
					  int *, int *, int *, int *, int *, 
					  int *, int *, int *, int *, int *, 
					  double *, int, int, int, int, 
					  int, int));

/*-- useful debugging macro for printing bytes in binary --*/
#define BYTE_PRINT(a)	                                      \
   {                                                          \
      kubyte  m[] ={0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01}; \
      int i;                                                  \
      for (i = 0; i < 8; i++)                                 \
      {                                                       \
	 if (a & m[i])                                        \
	    kprintf("*");     				      \
	 else                 				      \
	    kprintf("-");     				      \
      }                       				      \
      kprintf(" ");           				      \
   }

/****    =============================================================
 ****    Tempfile Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_construct - construct a tempfile
|
|       Purpose: This routine will construct a tempfile 
|	 	 of the given size, dimension, and datatype.
|	
|		 The tempfile is initialized to this size using
|		 a simple klseek, and then is rewound using 
|		 klseek(fid, 0, KSEEK_SET).
|
|         Input: dimension - the dimensionality of the segment
|		 size      - the size array of the segment
|		 datatype  - the datatype of the segment
|		 rval      - real value to initialize to
|		 ival      - imaginary value to initialize to
|
|        Output: none
|
|       Returns: the constructed tempfile
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_transport *
kdas_temp_construct(
   int    dimension,
   int   *size,
   int    datatype,
   double rval,
   double ival)
{
   kdas_transport *ntransport;

   int ntemp;
   int overall_size;
   int i;


   /*-- create a new tempfile --*/
   ntemp = ktmpfid();

   if (ntemp < 0)
      return NULL;  /*-- error, could not open file --*/
   
   /*-- compute the overall size as a product of size and datatype --*/
   for (i = 0, overall_size = 1; i < dimension; i++)
      overall_size *= size[i];

   /*-- avoid preseting if we don't have to --*/
   if (rval == 0.0 && ival == 0.0)
   {
      overall_size *= kdata_size(datatype);

      /*-- seek to the end to make the file look appropriately BIG --*/
      klseek(ntemp, (off_t) overall_size, KSEEK_SET);
      klseek(ntemp, (off_t) 0, KSEEK_SET);
   }
   else
      kdas_preset_file(ntemp, dimension, size, datatype, rval, ival);

   /*-- create a new transport --*/
   ntransport = (kdas_transport *) kmalloc(sizeof(kdas_transport));
   ntransport->id = ntemp;

   /*-- tempfile will use standard reader, writer, and seeker functions --*/
   ntransport->reader = kread_generic;
   ntransport->writer = kwrite_generic;
   ntransport->seeker = klseek;
   ntransport->teller = ktell;
   
   return ntransport;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_destruct - destruct a tempfile
|
|       Purpose: This routine will destruct a given tempfile.
|	
|         Input: transport - the transport to destruct
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kdas_temp_destruct(kdas_transport *transport)
{
   if (transport == NULL)
      return;
   
   kclose(transport->id);
   kfree(transport);
   
   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_recast - recast data in a temp file
|
|       Purpose: This routine will recast the data in a given
|		 temp file and place it into a new temp file.
|
|		 The new temp file will be returned and the old
|		 one closed.
|
|         Input: temp      - tempfile to recast
|                dimension - the dimensionality of the segment
|		 size      - the size array of the segment
|		 odatatype - the original datatype of the segment
|		 ndatatype - the new datatype to cast to 
|		 
|        Output: none
|
|       Returns: A new tempfile with the recast data.
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_transport *
kdas_temp_recast(
   kdas_transport *temp,
   int             dimension,
   int            *size,
   int             odatatype,
   int             ndatatype)
{
   kdas_transport *ntemp;

   kaddr oblock;
   kaddr nblock;
   int   unit[KDAS_MAX_DIM];
   int   position[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int   zero[KDAS_MAX_DIM]     = KDAS_ZERO_ARRAY;
   int   num;
   int   i;


   /*-- get the optimal region size to process in --*/
   kdas_optimal_region_size(dimension, size, odatatype, unit);

   /*-- create a new tempfile to in which to return the recast data --*/
   ntemp = kdas_temp_construct(dimension, size, ndatatype, 0.0, 0.0);

   /*-- compute the number of elements in the the block --*/
   for (i = 0, num = 1; i < dimension; i++)
      num *= unit[i];

   /*-- allocate data blocks for casting --*/
   oblock = kmalloc((size_t)(num*kdata_size(odatatype)));
   nblock = kmalloc((size_t)(num*kdata_size(ndatatype)));

   do
   {
      /*-- read from the old transport --*/
      kdas_transport_clip(temp, oblock, dimension, odatatype, size, unit, NULL,
			  position, zero, unit, KDAS_READ);

      /*-- recast the data --*/
      kdas_cast_data(oblock, odatatype, ndatatype, dimension, unit, 
		     KREAL, nblock);

      /*-- write into the new transport --*/
      kdas_transport_clip(ntemp, nblock, dimension, ndatatype, size, unit, 
			  NULL, position, zero, unit, KDAS_WRITE);
   }
   while (kdas_advance_position(dimension, size, unit, position));

   /*-- cleanup --*/
   kfree(oblock);
   kfree(nblock);

   /*-- rewind the new temp file --*/
   ntemp->seeker(ntemp->id, (off_t) 0, KSEEK_SET);

   /*-- say adios to the old temp file --*/
   kdas_temp_destruct(temp);

   return ntemp;
}


/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_resize - resize data in a temp file
|
|       Purpose: This routine will resize the data in a given
|		 temp file and place it into a new temp file.
|
|		 The new temp file will be returned and the old
|		 one closed.
|
|         Input: temp      - tempfile to resize
|                dimension - the dimensionality of the segment
|		 datatype  - the datatype of the segment
|		 osize     - the original size array of the segment
|		 nsize     - the new size array of the segment
|		 rval      - real value to use when padding to 
|			     larger size
|		 ival      - imaginary value to use when padding 
|			     to larger size
|		 
|        Output: none
|
|       Returns: A new tempfile with the resized data.
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_transport *
kdas_temp_resize(
   kdas_transport *temp,
   int             dimension,
   int             datatype,
   int            *osize,
   int            *nsize,
   double          rval,
   double          ival)
{
   kdas_transport *ntemp;
   kaddr           block;

   int    msize[KDAS_MAX_DIM];
   int    unit[KDAS_MAX_DIM];
   int    position[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int    zero[KDAS_MAX_DIM]     = KDAS_ZERO_ARRAY;
   int    num;
   int    i;


   /*-- compute the minimumest size : move this much --*/
   for (i = 0, num = 1; i < dimension; i++)
      msize[i] = kmin(nsize[i], osize[i]);

   /*-- get the optimal region size for access --*/
   kdas_optimal_region_size(dimension, msize, datatype, unit);

   /*-- create a new tempfile to in which to return the resized data --*/
   ntemp = kdas_temp_construct(dimension, nsize, datatype, rval, ival);

   /*-- compute the number of elements in the the block --*/
   for (i = 0, num = 1; i < dimension; i++)
      num *= unit[i];

   /*-- allocate data block for resizing --*/
   block = kmalloc((size_t)(num*kdata_size(datatype)));

   do
   {
      /*-- read a unit from the old transport --*/
      kdas_transport_clip(temp, block, dimension, datatype, osize, unit,
			  NULL, position, zero, unit, KDAS_READ);

      /*-- write into the new transport --*/
      kdas_transport_clip(ntemp, block, dimension, datatype, nsize, unit,
			  NULL, position, zero, unit, KDAS_WRITE);
   }
   while (kdas_advance_position(dimension, msize, unit, position));

   /*-- cleanup --*/
   kfree(block);

   /*-- rewind the new temp file --*/
   ntemp->seeker(ntemp->id, (off_t) 0, KSEEK_SET);

   /*-- say adios to the old temp file --*/
   kdas_temp_destruct(temp);

   return ntemp;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_reorder - reorder data in a temp file
|
|       Purpose: This routine will reorder the data in a given
|		 temp file and place it into a new temp file.
|
|		 The new temp file will be returned and the old
|		 one closed.
|
|         Input: temp      - tempfile to reorder
|                dimension - the dimensionality of the segment
|		 datatype  - the datatype of the segment
|                osize     - size array in the original order
|                nsize     - size array in the new order
|		 oorder    - the original order array of the segment
|		 norder    - the order size array of the segment
|		 
|        Output: none
|
|       Returns: A new tempfile with the reordered data.
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_transport *
kdas_temp_reorder(
   kdas_transport *temp,
   int             dimension,
   int             datatype,
   int            *osize,
   int            *nsize,
   int            *oorder,
   int            *norder)
{
   kdas_transport *ntemp;

   kaddr oblock;
   kaddr nblock;
   kaddr tdata;  /*-- pointer to tdata : either nblock or oblock --*/

   int   ounit[KDAS_MAX_DIM];
   int   nunit[KDAS_MAX_DIM];
	 
   int   oposition[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int   nposition[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
	 
   int   zero[KDAS_MAX_DIM]     = KDAS_ZERO_ARRAY;
   int   num;
   int   i;


   /*-- get the optimal region size for access --*/
   kdas_optimal_region_size(dimension, osize, datatype, ounit);

   /*-- determine transposed unit size --*/
   kmemcpy(nunit, ounit, sizeof(int) * dimension);
   kdas_array_reorder(nunit, dimension, oorder, norder);

   /*-- create a new tempfile to in which to return the reordered data --*/
   ntemp = kdas_temp_construct(dimension, nsize, datatype, 0.0, 0.0);

   /*-- compute the number of elements in the the block --*/
   for (i = 0, num = 1; i < dimension; i++)
      num *= ounit[i];

   /*-- allocate data blocks for reordering --*/
   oblock = kmalloc((size_t)(num*kdata_size(datatype)));
   nblock = kmalloc((size_t)(num*kdata_size(datatype)));

   do
   {
      /*-- read a unit from the old transport --*/
      kdas_transport_clip(temp, oblock, dimension, datatype, osize, ounit,
			  NULL, oposition, zero, ounit, KDAS_READ);

      /*-- reorder the data into the new block --*/
      tdata = kdas_transpose_data(oblock, dimension, datatype, ounit, nunit, 
				  oorder, norder, nblock);

      /*-- determine transposed position --*/
      kmemcpy(nposition, oposition, sizeof(int) * dimension);
      kdas_array_reorder(nposition, dimension, oorder, norder);

      /*-- write into the new transport --*/
      kdas_transport_clip(ntemp, tdata, dimension, datatype, nsize, nunit,
			  NULL, nposition, zero, nunit, KDAS_WRITE);
   }
   while (kdas_advance_position(dimension, osize, ounit, oposition));

   /*-- cleanup --*/
   kfree(oblock);
   kfree(nblock);

   /*-- rewind the new temp file --*/
   ntemp->seeker(ntemp->id, (off_t) 0, KSEEK_SET);

   /*-- say adios to the old temp file --*/
   kdas_temp_destruct(temp);

   return ntemp;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdas_temp_copy - copy data from one file to another
|
|       Purpose: This routine will copy the data in a given
|		 temp file and into a new temp file.
|
|         Input: temp      - tempfile to resize
|                dimension - the dimensionality of the segment
|		 datatype  - the datatype of the segment
|		 size      - the original size array of the segment
|		 
|        Output: none
|
|       Returns: A new tempfile with the copy data.
|
|    Written By: Steve Kubica
|          Date: May 23, 1994
| Modifications:
|
------------------------------------------------------------*/
kdas_transport *
kdas_temp_copy(
   kdas_transport *temp,
   int             dimension,
   int             datatype,
   int            *size)
{
   kdas_transport *ntemp;
   kaddr           block;

   int    unit[KDAS_MAX_DIM];
   int    position[KDAS_MAX_DIM] = KDAS_ZERO_ARRAY;
   int    zero[KDAS_MAX_DIM]     = KDAS_ZERO_ARRAY;
   int    num;
   int    i;


   /*-- get the optimal region size for access --*/
   kdas_optimal_region_size(dimension, size, datatype, unit);

   /*-- create a new tempfile to in which to return the copied data --*/
   ntemp = kdas_temp_construct(dimension, size, datatype, 0.0, 0.0);

   /*-- compute the number of elements in the the block --*/
   for (i = 0, num = 1; i < dimension; i++)
      num *= unit[i];

   /*-- allocate data block for resizing --*/
   block = kmalloc((size_t)(num*kdata_size(datatype)));

   do
   {
      /*-- read a unit from the old transport --*/
      kdas_transport_clip(temp, block, dimension, datatype, size, unit,
			  NULL, position, zero, unit, KDAS_READ);
      
      /*-- write into the new transport --*/
      kdas_transport_clip(ntemp, block, dimension, datatype, size, unit,
			  NULL, position, zero, unit, KDAS_WRITE);
   }
   while (kdas_advance_position(dimension, size, unit, position));

   /*-- cleanup --*/
   kfree(block);

   /*-- rewind the new temp file --*/
   ntemp->seeker(ntemp->id, (off_t) 0, KSEEK_SET);

   return ntemp;
}

/****    =============================================================
 ****    Transport Clipper Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_tranport_clip
|
|       Purpose: This routine is an n-dimensional clipping routine
|		 which will clip an arbitrary block of memory either
|		 into or out of a data transport into memory.
|
|	        DATA "SPACE" ON TRANSPORT  DESTINATION "BLOCK" IN MEMORY
|
|                                                     ___dposition
|		 +-+-+-+-+-+-+-+-+-+             +-+-/-+-+-+-+-+       
|		 | | | | | | | | | | <- write    | |/| | | | | |
|	         |-O-+-+-+-+-+-+-+-+             +-O-+-+-+-+-+-+
|	         |/|*|*|*|*|*| | | |   /|___|\   | |*|*|*|*|*| |
|	      	 /-+-+-+-+-+-+-+-+-+  /       \  +-+-+-+-+-+-+-+
|    tpositon___/| |*|*|*|*|*| | | |  \  ___  /  | |*|*|*|*|*| |
|		 +-+-+-+-+-+-+-+-+-+   \|   |/   +-+-+-+-+-+-+-+
|		 | |*|*|*|*|*| | | |             | |*|*|*|*|*| |
|		 +-+-+-+-+-+-+-+-+-+     read -> +-+-+-+-+-+-+-+
|	
|	       [size indicated by tsize]       [size indicated by bsize]
|                                  
|		       [dimension and datatype apply to both]
|
|                        +-+-+-+-+-+
|		         |*|*|*|*|*|    
|                        +-+-+-+-+-+    the size of the actual data 
|		         |*|*|*|*|*|    unit being transfered is 
|                        +-+-+-+-+-+    indicated by unit size
|		         |*|*|*|*|*|
|                        +-+-+-+-+-+
|
|		 It is assumed that transport is pointing to the
|		 beginning of the dataspace and block is pointing
|		 to the beginning of the blockspace when this routine 
|		 is called.
|
|         Input: transport - transport to clip into and out of
|		 block     - the block of data
|                dimension - dimensionality of the data
|                datatype  - datatype of the data
|		 tsize     - the total size of the data transport
|		 bsize     - the total size of the block space
|	         asize     - the apparent size of the data transport (unused)
|                tposition - beginning corner of the unit in the tranport
|                bposition - beginning corner of the unit in the block space
|                unit      - size of the block 
|		 action    - KDAS_READ for reading, KDAS_WRITE for writing
|
|        Output: block of memory is clipped either to or from the transport
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: May 22, 1995
| Modifications:
|
------------------------------------------------------------*/
void
kdas_transport_clip(
   kdas_transport *transport,
   kaddr	   block,
   int  	   dimension,
   int  	   datatype,
   int  	  *tsize,
   int  	  *bsize,
   int            *asize,
   int  	  *tposition,
   int  	  *bposition,
   int  	  *unit,
   int  	   action)       
{
   int dim = dimension;
   int toffset;
   int boffset;
   
   int tskip[KDAS_MAX_DIM];
   int bskip[KDAS_MAX_DIM];
   int tstride[KDAS_MAX_DIM];
   int bstride[KDAS_MAX_DIM];

   int _tsize[KDAS_MAX_DIM];
   int _bsize[KDAS_MAX_DIM];
   int _unit[KDAS_MAX_DIM];
   int _tposition[KDAS_MAX_DIM];
   int _bposition[KDAS_MAX_DIM];

   kubyte *buffer = NULL;

   int position;
   int tdatasize = kmach_sizeof((unsigned)kfile_getmachtype(transport->id), 
				(unsigned)datatype);
   int bdatasize = kdata_size(datatype);


   /*-- save this so we can put the transport back when we are done --*/
   position = transport->teller(transport->id);
   
   /*-- make copies of these, as they can change if we collapse them --*/
   kmemcpy(_tsize,     tsize,     sizeof(int) * dim);
   kmemcpy(_bsize,     bsize, 	  sizeof(int) * dim);
   kmemcpy(_unit,      unit,  	  sizeof(int) * dim);
   kmemcpy(_tposition, tposition, sizeof(int) * dim);
   kmemcpy(_bposition, bposition, sizeof(int) * dim);

   /*-- collapse the space down if possible --*/
   if (datatype != KBIT)
      kdas_collapse_space(&dim, _tsize, _bsize, _unit, _tposition, _bposition);
   else
      _tposition[0] = 0;  /*-- bit data requires we work with entire lines --*/

   /*-- compute the initial offsets into the dataspace --*/
   kdas_compute_offsets(dim, datatype, tdatasize, bdatasize, _tsize, _bsize, 
			_tposition, _bposition, &toffset, &boffset);

   /*-- compute the skips needed to traverse the data space --*/
   kdas_compute_strides(dim, datatype, tdatasize, bdatasize, _tsize, 
			_bsize, tstride, bstride);
   kdas_compute_skips(dim, tstride, bstride, _unit, _unit, tskip, bskip);

   /*-- move the transport to the offset --*/
   transport->seeker(transport->id, (off_t) toffset, SEEK_CUR);

   /*-- clip the data from the source to the destination --*/
   if (datatype == KBIT)
   {
      int tshift   = tposition[0] % 8;
      int bshift   = bposition[0] % 8;
      int linesize = ((_tsize[0] + 7) >> 3); /*-- byte aligned line --*/
      int unitsize = ((_unit[0]  + 7) >> 3); /*-- byte aligned unit --*/

      buffer = (kubyte *) kcalloc((size_t) 1, (size_t) linesize);
      
      __transport_bit_clip(transport, (kubyte *) block, buffer, dim-1, 
			   _unit, &toffset, &boffset, tskip, bskip, linesize,
			   unitsize, tshift, bshift, tposition[0]>>3, action);
      kfree(buffer);
   }
   else 
      __transport_clip(transport, (kubyte *) block, dim-1, _unit, &boffset,
		       tskip, bskip, datatype, bdatasize, action);

   /*-- put the transport back where we found it --*/
   transport->seeker(transport->id, (off_t) position, KSEEK_SET);

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) __transport_clip
|
|       Purpose: This routine is a recursive orthogonal clipper which
|		 clips to and from a data transport.
|
|		 On each recursion, if we have recursed down to the 0th
|		 dimension, we copy a line of data.  Otherwise, we
|		 move skip a unit in our current dimension and recurse 
|		 down by one dimension.
|
|         Input: space    - data space to clip into and out of
|		 block    - the block of data
|		 rank     - current dimension being considered
|		 unit     - size of the block to clip
|		 offset   - current offset into the data space 
|		 tskip    - skips to use when traversing transport
|		 bskip    - skips to use when traversing data block
|		 datatype - the datatype of the tranport and data block
|		 datasize - the datasize of the datatype on the transport
|		 action   - KDAS_READ for reading, KDAS_WRITE for writing
|	
|        Output: block of memory is clipped either to or from the transport
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: May 22, 1994
| Modifications:
|
------------------------------------------------------------*/
static void 
__transport_clip(
   kdas_transport *transport,
   kubyte         *block,
   int    	   rank,
   int    	  *unit,
   int    	  *offset,
   int    	  *tskip,
   int    	  *bskip,
   int    	   datatype,
   int             datasize,
   int    	   action)  
{
   register int i;
   register int len = unit[rank];
   register int bskp = bskip[rank];

   if (rank > 0)
   {
      for (i = 0; i < len; i++, *offset += bskp)
      {
	 __transport_clip(transport, block, rank-1, unit, offset, tskip, 
			  bskip, datatype, datasize, action);
	 transport->seeker(transport->id, (off_t) tskip[rank], SEEK_CUR);
      }
   }
   else
   {
      if (action == KDAS_READ)
	 transport->reader(transport->id, block+*offset, (size_t) unit[0], 
			   datatype);
      else
	 transport->writer(transport->id, block+*offset, (size_t) unit[0], 
			   datatype);

      /*-- transport has been moved for us by the write... --*/

      /*--  ... but we must move the offset ourselves --*/
      *offset += unit[0] * datasize;
   }
   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) __transport_bit_clip
|
|       Purpose: This routine is a recursive orthogonal bit clipper
|		 which clips either to or from a data transport to another.
|
|		 On each recursion, if we have recursed down to the 0th
|		 dimension, we copy a line of data.  Otherwise, we
|		 move skip a unit in our current dimension and recurse 
|		 down by one dimension.    We read an entire line in
|		 order to guarantee that we get all the necessary bytes
|		 to correctly perform the read.  (Sometimes you have
|		 to get 9 bytes to fill out enough bits for 8 bytes
|		 if the bits cross byte boundaries).   We read if
|		 we can when writing, so that the data being written 
|		 can be or'd with the data already there.
|
|		 This routine deals with headache of byte aligned
|		 bit data.  The shifts relative to the unit boundary
|		 byte in both the data space and data block are 
|		 accounted for in this routine.
|
|         Input: transport - data space to clip into and out of
|		 block     - the block of data
|		 buffer    - allocated buffer to hold a line of data
|		 rank      - current dimension being considered
|		 unit      - size of the block to clip
|		 toffset   - current offset into the data transport
|		 boffset   - current offset into the data block
|		 tskip     - skips to use when traversing data space
|		 bskip     - skips to use when traversing data block
|		 linesize  - the base size of the 0th dimension * datatype
|		 tshift    - # of bits in data space that data is shifted by
|		 bshift    - # of bits in block that data is shifted by
|		 pos       - position along line relative to 0 
|		 action    - KDAS_READ for reading, KDAS_WRITE for writing
|	
|        Output: block of transport is clipped either to or from the data space
|
|       Returns: nothing
|	
|    Written By: Steve Kubica
|          Date: March 29, 1996
| Modifications:
|
------------------------------------------------------------*/
static void 
__transport_bit_clip(
   kdas_transport *transport,
   kubyte 	  *block,
   kubyte 	  *buffer,
   int    	   rank,
   int    	  *unit,
   int    	  *toffset,
   int    	  *boffset,
   int    	  *tskip,
   int    	  *bskip,
   int    	   linesize,
   int    	   unitsize,
   int    	   tshift,
   int    	   bshift,
   int    	   pos,
   int    	   action)
{
   register int i;
   register int len  = unit[rank];
   register int tskp = tskip[rank];
   register int bskp = bskip[rank];
   register int do_read = (action == KDAS_READ);

   if (rank > 0)
   {
      for (i = 0; i < len; i++, *boffset += bskp)
      {
	 __transport_bit_clip(transport, block, buffer, rank-1, unit, 
			      toffset, boffset, tskip, bskip, linesize, 
			      unitsize, tshift, bshift, pos, action);
	 transport->seeker(transport->id, (off_t) tskp, SEEK_CUR);
      }
   }
   else
   {      
      kubyte  himask[] = {0xff,0x7f,0x3f,0x1f,0x0f,0x07,0x03,0x01};
      kubyte  lomask[] = {0x00,0x80,0xc0,0xe0,0xf0,0xf8,0xfc,0xfe};
      kubyte *dst, *src;
      int     ds, ss, c;

      if (do_read)
	 dst = block + *boffset, src = buffer + pos, ds = bshift, ss = tshift;
      else
	 src = block + *boffset, dst = buffer + pos, ds = tshift, ss = bshift;

      /*-- if we can, get current data, either to read it or to OR with it --*/
      if (kfile_isread(kfidfile(transport->id)))
      {
	 transport->reader(transport->id, buffer, (size_t) linesize<<3, KBIT);
	 transport->seeker(transport->id, (off_t) -linesize, KSEEK_CUR);
      }

      if (ss == ds)
      {
	 for (i = 0; i < unitsize - 1; i++)
	    *dst++ = *src++;

	 if ((c = (unit[0] % 8)) != 0) /*-- last byte : only get left bits --*/
	    *dst &= himask[c], *dst |= *src & ~himask[c]; 
	 else
	    *dst = *src;
      }
      else if (ss > ds)
      {
	 for (i = 0; i < unitsize; i++)
	 {
	    *dst    = 0x00;
	    *dst   |= (*src++ << (ss - ds))     & lomask[8 - ss + ds];
	    *dst++ |= (*src   >> (8 - ss + ds)) & himask[8 - ss + ds];
	 }
      }
      else /* ss < ds */
      {
	 for (i = 0; i < unitsize - 1; i++)
	 {
	    *dst   &= ~himask[ds - ss];
	    *dst++ |= (*src   >> (ds - ss))     & himask[ds - ss];
	    *dst   &= ~lomask[ds - ss]; 
	    *dst   |= (*src++ << (8 - ds + ss)) & lomask[ds - ss];
	 }

	 /*-- last byte : zero out left bits and or in new bits --*/
	 /*
	 *dst &= ~himask[ds - ss]; 
	 *dst |= (*src >> (ds - ss)) & himask[ds - ss];
	 */
      }

      if (!do_read)
      {
	 transport->writer(transport->id, buffer, (size_t) linesize<<3, KBIT);
	 transport->seeker(transport->id, (off_t) -linesize, KSEEK_CUR);
      }
      /*-- we seek back since skips have linesize built in --*/
   }
      
   return;
}

/****    =============================================================
 ****    Transport Interpolater Routines
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: kdas_transport_interp
|
|       Purpose: This routine is an n-dimensional clipping routine
|		 which will clip an arbitrary block in memory either
|		 into or out of data space stored on a transport.  The 
|		 source space may "appear" to be a size which
|		 is larger or smaller than its actual size, in which
|		 case this routine will zero order interpolate the
|		 data line-by-line as it is being clipped to or from
|		 the destination to make it appear as if the the data
|		 was clipped from a space which was the apparent size.
|
|	   DATA "SPACE" ON TRANSPORT        DATA "BLOCK" IN MEMORY
|                                         shown at an apparent size
|
|						  ___bposition
|	     +-+-+-+-+-+-+-+-+-+             +-+-/-+-+-+-+-+-+
|	     | | | | | | | | | | <- write    | |/| | | | | | |
|	     |-O-+-+-+-+-+-+-+-+             +-O-+-+-+-+-+-+-+
|	     |/|1|2|3| | | | | |   /|___|\   | |1|1|2|2|3|3| |
|	     /-+-+-+-+-+-+-+-+-+  /       \  +-+-+-+-+-+-+-+-+
| tpositon__/| |4|5|6| | | | | |  \  ___  /  | |1|1|2|2|3|3| |
|            +-+-+-+-+-+-+-+-+-+   \|   |/   +-+-+-+-+-+-+-+-+
|...appears  | |7|8|9| | | | | |             | |4|4|5|5|6|6| |
|  to be at  +-+-+-+-+-+-+-+-+-+     read -> +-+-+-+-+-+-+-+-+
| aposition  | | | | | | | | | |             | |4|4|5|5|6|6| |
|	     +-+-+-+-+-+-+-+-+-+             +-+-+-+-+-+-+-+-+
|	     | | | | | | | | | |             | |7|7|8|8|9|9| |
|	     +-+-+-+-+-+-+-+-+-+             +-+-+-+-+-+-+-+-+
|	     | | | | | | | | | |             | |7|7|8|8|9|9| |
|	     +-+-+-+-+-+-+-+-+-+             +-+-+-+-+-+-+-+-+
|	  
|	 [size indicated by tsize]       [actual size indicated by bsize]
|    [apparent size indicated by asize]
|				    
|	        [dimension and datatype apply to both]
|
|                        +-+-+-+-+-+-+
|		         |1|1|2|2|3|3|    
|                        +-+-+-+-+-+-+    the size of the actual data 
|		         |1|1|2|2|3|3|    unit being transfered is 
|                        +-+-+-+-+-+-+    indicated by unit size in 
|		         |4|4|5|5|6|6|    apparent space
|                        +-+-+-+-+-+-+
|		         |4|4|5|5|6|6|    
|                        +-+-+-+-+-+-+
|		         |7|7|8|8|9|9|    
|                        +-+-+-+-+-+-+
|		         |7|7|8|8|9|9|    
|                        +-+-+-+-+-+-+
|
|		 It is assumed that space is pointing to the
|		 beginning of the dataspace and block is pointing
|		 to the beginning of the blockspace when this routine 
|		 is called.
|
|         Input: transport - transport to interpolate into and out of
|		 block     - the block of data
|                dimension - dimensionality of the data
|                datatype  - datatype 
|		 tsize     - the total size of the data transport
|		 bsize     - the total size of the block space
|	 	 asize     - the total apparent size of the data space
|                aposition - beginning corner of the unit in the apparent space
|                bposition - beginning corner of the unit in the block space
|                aunit     - size of the block in apparent space
|		 action    - KDAS_READ for reading, KDAS_WRITE for writing
|
|        Output: block of transport is clipped either to or from the data space
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: December 26, 1995
| Modifications:
|
------------------------------------------------------------*/
void
kdas_transport_interp(
   kdas_transport *transport,
   kaddr	   block,
   int  	   dimension,
   int  	   datatype,
   int  	  *tsize,
   int  	  *bsize,
   int  	  *asize,
   int  	  *aposition,
   int  	  *bposition,
   int  	  *aunit,
   int  	   action)
{
   int toffset;
   int boffset;
   int dim = dimension;

   int tskip[KDAS_MAX_DIM];
   int bskip[KDAS_MAX_DIM];
   int tstride[KDAS_MAX_DIM];
   int bstride[KDAS_MAX_DIM];
   int tposition[KDAS_MAX_DIM];
   int tunit[KDAS_MAX_DIM];

   double fskip[KDAS_MAX_DIM];

   kubyte *buffer = NULL;

   int i;
   int position;

   int tdatasize = kmach_sizeof((unsigned)kfile_getmachtype(transport->id), 
				(unsigned)datatype);
   int bdatasize = kdata_size(datatype);

   /*-- save this so we can put the transport back when we are done --*/
   position = transport->teller(transport->id);

   /*----------------------------------------------------------------
   |   determine the fractional skips needed to traverse the source :
   |   we will also determine our actual block size and position in 
   |   our actual data space from our apparent block size and position
   |   in the apparent space.    mathematically, I am not sure why
   |   this is such a mess, but it makes the testsuite pass ;-)
   \-----------------------------------------------------------------*/
   for (i = 0; i < dimension; i++)
   {
      fskip[i]     = (double)tsize[i] / (double)asize[i];
      tposition[i] = (int) kfloor((double) aposition[i] * fskip[i]);
      tunit[i]     = (int) kceil((double) aunit[i] * fskip[i]);

      /*-- backend clipping if we've rounded up too far for the unit --*/
      if (tposition[i] + tunit[i] > tsize[i])
	 tunit[i]--;
   }

   /*-- for bit data, we need to track line position outside of soffset --*/
   /* if (datatype == KBIT)
      tposition[0] = 0; */

   /*-- compute the initial offsets into the dataspace --*/
   kdas_compute_offsets(dim, datatype, tdatasize, bdatasize, tsize, bsize, 
			tposition, bposition, &toffset, &boffset);

   /*-- move the transport to this offset --*/
   transport->seeker(transport->id, (off_t) toffset, SEEK_CUR);

   /*-- compute the skips needed to traverse the data space --*/
   kdas_compute_strides(dim, datatype, tdatasize, bdatasize, tsize, bsize, 
			tstride, bstride);
   kdas_compute_skips(dim, tstride, bstride, tunit, aunit, tskip, bskip);
   
   /*-- clip the data from the source to the destination --*/
   if (datatype == KBIT)
   {
      int a = (tunit[0] + 7) >> 3;
      int b = (aunit[0] + 7) >> 3;

      int ashift = aposition[0] % 8;
      int bshift = bposition[0] % 8;

      buffer = (kubyte *) kmalloc((size_t)((a > b) ? a : b));

      __transport_bit_interp(transport, (kubyte *) block, buffer, dim-1, dim, 
			     aunit, tunit, tsize, bsize, aposition, bposition,
			     &toffset, &boffset, tstride, bstride, tskip,
			     bskip, fskip, datatype, tdatasize, bdatasize,
			     ashift, bshift, action);
      kfree(buffer);
      
      return;
   }
   else
   {
      int a = tunit[0] * tdatasize;
      int b = aunit[0] * bdatasize;

      buffer = (kubyte *) kmalloc((size_t)((a > b) ? a : b));
      
      __transport_interp(transport, (kubyte *) block, buffer, dim-1, dim, 
			 aunit, tunit, tsize, bsize, aposition, bposition, 
			 &toffset, &boffset, tstride, bstride, tskip, bskip, 
			 fskip, datatype, tdatasize, bdatasize, action);
      kfree(buffer);
   }
   
   /*-- put the transport back where we found it --*/
   transport->seeker(transport->id, (off_t) position, KSEEK_SET);
   
   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) __transport_interp
|
|       Purpose: This routine is a recursive orthogonal clipper which
|		 clips to and from a transport, interpolating as it clips.
|
|		 On each recursion, if we have recursed down to the 0th
|		 dimension, we interpolate a line of data.  After each
|		 line we may either back up a line, or skip some lines
|		 depending on the interpolation factor in that dimension.
|		 An entire line of the actual sized data is read into
|		 memory, and then interpolated to the new size for reads.
|
|         Input: transport - transport to interpolate into and out of
|		 block     - the block of data
|		 buffer    - allocated buffer to hold a line of data
|		 rank      - current dimension being considered
|		 dim       - the dimensionality of the data
|		 unit      - size of the block to clip in the apparent space
|		 tunit     - size of the block to clip in the actual space
|		 tsize     - actual size of the data source
|		 bsize     - actual size of the data block
|		 aposition - current position in the apparent data space
|		 bposition - current position in the data block
|		 toffset   - current offset into the data space 
|		 boffset   - current offset into the data block
|		 tstride   - strides to use when traversing the data space
|		 bstride   - strides to use when traversing the data block
|		 tskip     - skips to use when traversing data space
|		 bskip     - skips to use when traversing data block
|		 fskip     - floating point scale factors to transform
|		 	     between the apparent size and actual size of
|			     the data space
|		 datatype  - the datatype of the data
|		 tdatasize - the size of the datatype on the transport
|		 bdatasize - the size of the datatype in memory
|		 action    - KDAS_READ for reading, KDAS_WRITE for writing
|	
|        Output: block of transport is clipped either to or from the data space
|
|       Returns: nothing
|
|    Written By: Steve Kubica and Jeremy Worley
|          Date: May 22, 1994
| Modifications:
|
------------------------------------------------------------*/
static void 
__transport_interp(
   kdas_transport *transport,
   kubyte         *block,
   kubyte         *buffer,
   int    	   rank,
   int    	   dim,
   int    	  *unit,
   int    	  *tunit,
   int    	  *tsize,
   int    	  *bsize,
   int    	  *aposition,
   int    	  *bposition,
   int    	  *toffset,
   int    	  *boffset,
   int    	  *tstride,
   int    	  *bstride,
   int    	  *tskip,
   int    	  *bskip,
   double 	  *fskip,
   int    	   datatype,
   int    	   tdatasize,
   int             bdatasize,
   int    	   action)
{
   register int i;
   register int len;
   double   f, b, skp;
   int      pos;

   if (rank > 0)
   {
      /*----------------------------------------------------------------
      |   set up our backtracking variables : we will backtrack in the
      |   dataspace if reading, and in the blockspace if writing.
      \-----------------------------------------------------------------*/
      if (action == KDAS_READ)
         len = unit[rank],  pos = aposition[rank], skp = fskip[rank];
      else
         len = tunit[rank], pos = bposition[rank], skp = 1.0/fskip[rank];

      f = kabs(pos + 1) * skp;
      b = f - skp + FLT_EPSILON;

      for (i = 0; i < len; i++, b = f, f += skp + FLT_EPSILON)
      {
	 __transport_interp(transport, block, buffer, rank-1, dim, 
			    unit, tunit, tsize, bsize, 
			    aposition, bposition, toffset, boffset, 
			    tstride, bstride, tskip, bskip, fskip, 
			    datatype, tdatasize, bdatasize, action);

	 /*------------------------------------------------------------
         |   the rank-1 case has moved us to the next rank unit.
	 |   we may now want to back up by one unit, depending on how
	 |   far we are along this dimension.
	 \-------------------------------------------------------------*/
	 if (action == KDAS_READ)
	    transport->seeker(transport->id, 
			      (off_t) ((int)f - (int)b - 1) * tstride[rank], 
			      SEEK_CUR);
	 else
	    *boffset += ((int)f - (int)b - 1) * bstride[rank];
      }

      if (rank < dim - 1)
      {
	 transport->seeker(transport->id, (off_t) tskip[rank+1], SEEK_CUR);
	 *boffset += bskip[rank+1];
      }
   }
   else
   {
      if (action == KDAS_READ)
      {
	 transport->reader(transport->id, buffer, (size_t) tunit[0], datatype);
         kdata_resample(buffer, tunit[0], unit[0], datatype, 
                        KZERO_ORDER, KREAL, fskip[0], 0.0, 0.0, 
                        (double) kfraction(aposition[0] * fskip[0]), 0.0, 
			block+*boffset);
      }
      else
      {
         kdata_resample(block+*boffset, unit[0], tunit[0], datatype,
                        KZERO_ORDER, KREAL, (1.0/fskip[0]), 0.0, 0.0,
                        0.0, 0.0, buffer);
         transport->writer(transport->id, buffer, (size_t) tunit[0], datatype);
      }

      /*-- seek past the rest of the unit --*/
      transport->seeker(transport->id, 
			(off_t) ((tsize[0] - tunit[0]) * tdatasize), SEEK_CUR);
      *boffset += bsize[0] * bdatasize;
   }

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) __transport_bit_interp
|
|       Purpose: a recursive orthogonal clipinterp routine.
|         Input:
|        Output:
|       Returns:
|
|    Written By: Jeremy Worley
|          Date: June 4, 1994
| Modifications:
|
------------------------------------------------------------*/
static void 
__transport_bit_interp(
   kdas_transport *transport,
   kubyte         *block,
   kubyte         *buffer,
   int    	   rank,
   int    	   dim,
   int    	  *unit,
   int    	  *tunit,
   int    	  *tsize,
   int    	  *bsize,
   int    	  *aposition,
   int    	  *bposition,
   int    	  *toffset,
   int    	  *boffset,
   int    	  *tstride,
   int    	  *bstride,
   int    	  *tskip,
   int    	  *bskip,
   double 	  *fskip,
   int    	   datatype,
   int    	   tdatasize,
   int             bdatasize,
   int             tshift,
   int             bshift,
   int    	   action)
{
   register int i;
   register int backtrack;
   register int len;
   double   f, b, skp;
   int      pos, rest;

   if (rank > 0)
   {
      /*----------------------------------------------------------------
      |   set up our backtracking variables : we will backtrack in the
      |   dataspace if reading, and in the blockspace if writing.
      \-----------------------------------------------------------------*/
      if (action == KDAS_READ)
         len = unit[rank],  pos = aposition[rank], skp = fskip[rank];
      else
         len = tunit[rank], pos = bposition[rank], skp = 1.0/fskip[rank];

      f = kabs(pos + 1) * skp;
      b = f - skp + FLT_EPSILON;

      for (i = 0; i < len; i++, b = f, f += skp + FLT_EPSILON)
      {
	 __transport_bit_interp(transport, block, buffer, rank-1, dim, unit, 
				tunit, tsize, bsize, aposition, bposition, 
				toffset, boffset, tstride, bstride, tskip, 
				bskip, fskip, datatype, tdatasize, bdatasize, 
				tshift, bshift, action);

	 /*------------------------------------------------------------
         |   the rank-1 case has moved us to the next rank unit.
	 |   we may now want to back up by one unit, depending on how
	 |   far we are along this dimension.
	 \-------------------------------------------------------------*/
	 backtrack = ((int)f - (int)b - 1);

	 if (action == KDAS_READ)
	    transport->seeker(transport->id, 
			      (off_t) ((int)f - (int)b - 1) * tstride[rank],
			      SEEK_CUR);
	 else
	    *boffset += ((int)f - (int)b - 1) * bstride[rank];
      }

      if (rank < dim - 1)
      {
	 transport->seeker(transport->id, (off_t) tskip[rank+1], SEEK_CUR);
	 *boffset += bskip[rank+1];
      }
   }
   else
   {
      if (action == KDAS_READ)
      {
	 transport->reader(transport->id, buffer, (size_t) tunit[0], KBIT);
	 kdas_bit_resample(buffer, tunit[0], unit[0], bshift, 
			   (double)(aposition[0] % 8) * fskip[0],
			   block+*boffset);
      }
      else
      {
	 kdas_bit_resample(block+*boffset, unit[0], tunit[0], tshift,
			   (double)(aposition[0] % 8) / fskip[0], buffer);
         transport->writer(transport->id, buffer, (size_t) tunit[0], KBIT);
      }

      /*-- seek past the rest of the unit --*/
      rest = ((tsize[0]+7) >> 3) - ((tunit[0]+7) >> 3);
      transport->seeker(transport->id, (off_t)rest, SEEK_CUR);

      *boffset += (bsize[0] + 7) >> 3;
   }

   return;
}
