 /*
  * Khoros: $Id: attributes.c,v 1.18.2.1 1996/12/06 20:28:41 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: attributes.c,v 1.18.2.1 1996/12/06 20:28:41 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Management Attribute Code
   >>>>		
   >>>>  Private:
   >>>>                 kdms_free_attribute_defines()
   >>>>                 kdms_copy_attribute_list()
   >>>>                 kdms_get_Vattribute()
   >>>>                 kdms_set_Vattribute()
   >>>>                 kdms_get_stored_attribute_names()
   >>>>   Static:
   >>>>                 _kdms_construct_generic_defin()
   >>>>                 _kdms_construct_quasi_defin()
   >>>>                 _kdms_destruct_attribute_defin()
   >>>>                 _kdms_locate_definition()
   >>>>                 _kdms_add_definition()
   >>>>                 _kdms_locate_attribute_defin()
   >>>>                 _kdms_delete_attribute_defin()
   >>>>                 _free_defin()
   >>>>                 _kdms_free_list_attribute_defin()
   >>>>                 _kdms_establish_attribute()
   >>>>                 _kdms_locate_attribute_list()
   >>>>                 _kdms_retrieve_attribute()
   >>>>                 _kdms_copy_attribute()
   >>>>   Public:
   >>>>                 kdms_define_quasi_attribute()
   >>>>                 kdms_define_attribute()
   >>>>                 kdms_undefine_attribute()
   >>>>                 kdms_query_attribute_definition()
   >>>>                 kdms_create_attribute()
   >>>>                 kdms_destroy_attribute()
   >>>>                 kdms_vset_attribute()
   >>>>                 kdms_vset_attributes()
   >>>>                 kdms_set_attribute()
   >>>>                 kdms_set_attributes()
   >>>>                 kdms_vget_attribute()
   >>>>                 kdms_vget_attributes()
   >>>>                 kdms_get_attribute()
   >>>>                 kdms_get_attributes()
   >>>>                 kdms_match_attribute()
   >>>>                 kdms_vmatch_attributes()
   >>>>                 kdms_match_attributes()
   >>>>                 kdms_copy_attribute()
   >>>>                 kdms_vcopy_attributes()
   >>>>                 kdms_copy_attributes()
   >>>>                 kdms_query_attribute()
   >>>>                 kdms_print_attribute()
   >>>>                 kdms_get_attribute_names()
   >>>>                 kdms_copy_segment_attributes()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

   /*----------------------------------------------------------------
   |   kdms_attribute_defin structure
   |   . . . . . . . . . . . . . . 
   |
   |   This is the structure for attribute definitions.  It contains
   |   the attribute itself, as well as all auxiliary info on how,
   |   where, and when to instantiate the attribute.
   \-----------------------------------------------------------------*/
typedef struct {

   char *attribute; 
   char *association; 

   kdas_attribute *atr;
   int             shared;    /*-- TRUE if at physical : else presentation --*/

} kdms_attribute_defin;

/****    =============================================================
 ****    Attribute Definition Tables
 ****    =============================================================
 ****/

static klist *attribute_lists;

/****    =============================================================
 ****    Attribute Definition Methods
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_construct_generic_defin
|
|       Purpose: This routine will construct a generic-type
|                attribute definition according to the given arguments.
|
|		 The attribute definition is distinguished by its
|		 attribute identifier, which is simply a tokenized
|		 version of the attribute name.
|
|		 This attribute definition will be used as a template
|                for creating (establishing) instances of the attribute
|		 on an actual data object.  Attributes can be 
|		 established either on the main object (object-level)
|		 or on any of the object's data segments (segment-level).
|		 The definition's "association" determines where it is
| 	         legal to establish the attribute.
|
|                The association is simply a tokenized string.  The 
|                string itself indicates where it is legal to establish
|                an attribute.  A NULL association indicates that the
|		 attribute should established at the object-level.
|		 An association of KDMS_ALL_SEGMENTS indicates that the
|		 attribute can be established at the segment level 
|		 on any given segment name.  If a specific segment name,
| 		 such as "value" is given for the association, then
|		 the attribute can only established on the "value" segment.
|		
|		 In addition to being established either at the
|		 object or segment level, an attribute can also
|		 be established at the physical or presentation level.
|
|		 Changes to physical level attributes are visible
|		 to all references of the data object and are termed
|		 as "shared" attributes.  Changes to presentation
|		 level attributes are visible only to the reference
|		 object on which the change was made, and thus 
|		 the attribute is termed as "unshared". 
|
|		 A default data block may be provided for
|		 initializing the value of any attributes created
|                from this definition template. If a default value 
|		 is provided, a copy of that value will be made and 
|		 assigned to the definition. If no default is 
|		 provided (NULL), then the the default value will
|		 be initialized to be NULL.
|
|                Note that the constructors for each attribute
|                definition type are implemented as separate functions 
|                because of their wildly different function parameters.
|
|         Input: attribute   - attribute identifier 
|		 association - attribute association
|		 datatype    - data type of the attribute
|  		 argsize     - size of each attribute argument
|		 numargs     - number of arguments in attribute
| 		 permanent   - TRUE if attribute is permanent
|	         shared      - TRUE if attribute is shared
|		 def         - default values to use when
|                              initializing the attribute value.
|        Output: none
|
|       Returns: a pointer to the constructed attribute definition 
|		 on success, NULL otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
static kdms_attribute_defin *
_kdms_construct_generic_defin(
   char *attribute,
   char *association,
   int   datatype,
   int   argsize,
   int   numargs,
   int   permanent,
   int   shared,
   kaddr def)
{
   kdms_attribute_defin *defin;

   /*-- allocate a new attribute definition structure --*/
   if ((defin = (kdms_attribute_defin *)
	kcalloc((size_t) 1, sizeof(kdms_attribute_defin))) == NULL)
   {
      kdas_set_error(KMEMORY_ALLOCATION);
      return NULL;
   }

   /*-- fill out the attribute definition structure --*/
   defin->attribute   = kstrdup(attribute);
   defin->association = kstrdup(association);
   defin->shared      = shared;

   defin->atr = kdas_attr_generic_construct(kstring_to_token(attribute), 
					    datatype, argsize,
					    numargs, permanent, def);
   if (defin->atr == NULL)
   {
      kfree_and_NULL(defin);
      kinfo(KDEBUG, "_kdms_construct_generic_defin : unable to construct "
	    "a generic attribute for definition of '%s'.", attribute);
      return NULL;
   }

   return defin;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_construct_quasi_defin
|
|       Purpose: This routine will construct a quasi-type
|                attribute definition according to the given arguments.
|
|		 The attribute definition is distinguished by its
|		 attribute identifier, which is simply a tokenized
|		 version of the attribute name. 
|
|		 Quasi-attributes are essentially "action" attributes.
|	         They technically have no data associated with them.
|		 They instead consist of a number of attribute functions, 
|		 or methods.  These methods, provided when calling
|		 this function, provide specific actions which are invoked
|		 from higher level public functions.  
|
|		 Specifically, these functions are :
|
|		 !   get method   - kdms_get_attribute(s)
|		 !   set method   - kdms_set_attribute(s)
|		 !   match method - kdms_match_attribute(s)
|		 !   copy method  - kdms_copy_attribute(s)
|		 !   query method - kdms_query_attribute
|		 !   print method - kdms_print_attribute
|		 
|                Each of these higher-level functions uses a "segment"
|		 parameter to indicate whether the attribute is 
|		 considered to be associated with the main object
|		 (object-level), or with a specific segment (segment-level).
|		 While this notion is purely conceptual for a 
|		 quasi-attribute, it provides a means for defining
|	         identically named attributes at different levels
|		 or for different segments. 
|		 
|		 The "association" provided for the definition will be 
|		 used to check against the provided association from the 
|		 higher-level segment argument.  The association must 
|		 match before the attribute's method is called. 
|
|                The association provided is simply a tokenized string.
|                The string itself indicates what associations are legal
|		 for invoking the quasi-attribute methods.  A NULL
|                association indicates that the quasi-attribute methods
|		 can be evoked at the object level, but not at the 
|		 segment level.
|
|		 An association of KDMS_ALL_SEGMENTS indicates that the
|		 attribute can be evoked at the segment level 
|		 on any given segment name.  If a specific segment name,
| 		 such as "value" is given for the association, then
|		 the attribute can only established on the "value" segment.
|
|		 Functions are provided for each of the attribute's
|		 methods. If NULL is passed in for any of the
|		 functions, an undefined function will be used.
|
|		 Client data may be given to the attribute definition.
|		 This client data will be passed into each of the given 
|		 attribute methods.  Note that a copy of this client 
|		 data is _not_ made, so the address of a static 
|		 variable must be provided.
|
|                Note that the constructors for each attribute
|                definition type are implemented as separate functions 
|                because of their wildly different function parameters.
|
|         Input: attribute   - attribute identifier 
|                association - attribute association
|		 get()       - get function to use
|  		 set()       - set function to use
|		 match()     - match function to use
| 		 copy()      - copy function to use 
|		 query()     - query function to use 
|		 print()     - print function to use 
|		 client_data - client data to pass into the attribute's
|			       methods.
|        Output: none
|
|       Returns: a pointer to the constructed attribute definition
|		 on success, NULL otherwise
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
static kdms_attribute_defin *
_kdms_construct_quasi_defin(
   char *attribute,
   char *association,
   int (*get)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*set)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*match) PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*copy)  PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*query) PROTO((kobject, ktoken, ktoken,kaddr,int *,int *,int *,int *)),
   int (*print) PROTO((kobject, ktoken, ktoken, kaddr, kfile *)),
   kaddr client_data)
{
   kdms_attribute_defin *defin;

   /*-- allocate a new attribute definition structure --*/
   if ((defin = (kdms_attribute_defin *)
	kcalloc((size_t) 1, sizeof(kdms_attribute_defin))) == NULL)
   {
      kdas_set_error(KMEMORY_ALLOCATION);
      return NULL;
   }

   /*-- fill out the attribute definition structure --*/
   defin->attribute   = kstrdup(attribute);
   defin->association = kstrdup(association);
   defin->shared      = TRUE;

   defin->atr = kdas_attr_quasi_construct(attribute, get, set, match, copy,
					  query, print, client_data);
   if (defin->atr == NULL)
   {
      kinfo(KDEBUG, "_kdms_construct_quasi_defin : unable to construct "
	    "quasi attribute %s.", attribute);
      kfree(defin);
      return NULL;
   }

   return defin;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_destruct_attribute_defin
|
|       Purpose: Destroy an attribute definition.  This 
|		 function can destroy both generic and quasi 
|		 attribute definitions.  All internal attribute 
|		 data, as well as the attribute definition 
|		 itself, will be freed.
|
|         Input: defin - the attribute definition to free
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: March 17, 1994
| Modifications:
|
------------------------------------------------------------*/
static void
_kdms_destruct_attribute_defin(kdms_attribute_defin *defin)
{
   /*-- trivial free --*/
   if (defin == NULL)
      return;

   /*-- free the internals of the definition --*/
   kfree(defin->attribute);
   kfree(defin->association);

   kdas_attr_destruct(defin->atr);

   /*-- finally free the attribute definition --*/
   kfree(defin);

   return;
}

/****    =============================================================
 ****    Attribute Definition Hash Management
 ****    =============================================================
 ****/

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_locate_definition
|
|       Purpose: This routine locates the data services attribute
|	         definition structure for a specified atttribute and
|	         association.
|
|         Input: attribute - the name of the attribute to find
|                seg       - the tokenized segment name to match
|
|        Output: none
|
|       Returns: the definition or NULL if it doesn't exist
|
|    Written By: Steve Kubica
|          Date: March 12, 1996
| Modifications:
|
------------------------------------------------------------*/
static kdms_attribute_defin *_kdms_locate_definition(
   char   *attribute,
   ktoken  seg)
{
   kdms_attribute_defin *defin = NULL;
   static ktoken all_segments = 0;
   klist *l;

   if (all_segments == 0)
      all_segments = kstring_to_token(KDMS_ALL_SEGMENTS);

   /*-- checks for NULL, or specific segments --*/
   if ((l = klist_locate(attribute_lists, seg)) != NULL)
       khash_clientdata((khash *) l->client_data, attribute, &defin);

   /*------------------------------------------------------------------
   |  if it is not a NULL segment, and it wasn't on the specific list 
   |  for the named segment, try the all segments segment.
   \-----------------------------------------------------------------*/
   if (seg != NULL && (l == NULL || defin == NULL))
   {
      if ((l = klist_locate(attribute_lists, all_segments)) == NULL)
	 return NULL;
   }

   if (l != NULL)
      khash_clientdata((khash *) l->client_data, attribute, &defin);

   return defin;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_add_definition
|
|       Purpose: Add an attribute definition to the static
|		 attribute definition tables. 
|
|         Input: defin - attribute definition to add
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_kdms_add_definition(
   char  	       	*attribute,
   ktoken	     	 seg,       
   kdms_attribute_defin *defin)
{
   klist *l;
   khash *h;
   

   /*-- first, get the appropriate hash table --*/
   if ((l = klist_locate(attribute_lists, seg)) == NULL)
   {
      h = khash_init(KSTRING, (ssize_t) KLENGTH);
      attribute_lists = klist_add(attribute_lists, (kaddr) seg, (kaddr) h);
   }
   else
      h = (khash *) l->client_data;

   /*-- hash in the new definition --*/
   return khash_add(h, kstrdup(attribute), (kaddr) defin);
}

#if 0
/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_locate_attribute_defin
|
|       Purpose: Given an attribute identifier, locate the 
|                attribute definition on the static attribute
|		 hash table and return it.
|
|         Input: attribute   - identifier of attribute definition to search for
|		 association - association to use when searching
|
|        Output: none
|
|       Returns: the attribute definition if it is on the list, NULL otherwise
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
static kdms_attribute_defin *
_kdms_locate_attribute_defin(
   char *attribute,
   char *association)
{
   int indx;

   if (!attribute)
      return NULL;

   /*-- check for "most hit" attributes with NULL association --*/
   else if (strcmp(attribute, "pdsMappingMode") == 0)
   {
      static kdms_attribute_defin *mode = NULL;

      if (mode == NULL)
	 mode = _kdms_find_defin(attribute, association, &indx);

      return mode;
   }

   else if (association == NULL) 
      return _kdms_find_defin(attribute, association, &indx);
	      
   /*-- check for "most hit" attributes with some association --*/
   else if (strcmp(attribute,  "internalPosition") == 0)
   {
      static kdms_attribute_defin *position = NULL;

      if (position == NULL)
	 position = _kdms_find_defin(attribute, association, &indx);

      return position;
   }
   else if (strcmp(attribute, "internalIncrementStateMachine") == 0)
   {
      static kdms_attribute_defin *inc_state = NULL;

      if (inc_state == NULL)
	 inc_state = _kdms_find_defin(attribute, association, &indx);

      return inc_state;
   }
   else if (strcmp(attribute, "size") == 0)
   {
      static kdms_attribute_defin *size = NULL;

      if (size == NULL)
	 size = _kdms_find_defin(attribute, association, &indx);

      return size;
   }

   else return _kdms_find_defin(attribute, association, &indx);
}
#endif

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_delete_definition
|
|       Purpose: Given an attribute and association, locate the 
|                attribute definition on the static attribute
|		 hash table and delete it.
|
|         Input: attribute   - name of attribute definition to delete.
|                association - corresponding attribute association.
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_kdms_delete_attribute_defin(
   char  *attribute,
   ktoken seg)
{
   kdms_attribute_defin *defin = NULL;
   klist *l;
   khash *h;
   

   /*-- first, get the appropriate hash table --*/
   if ((l = klist_locate(attribute_lists, seg)) == NULL)
      return TRUE;  /*-- don't even have the association --*/
   
   h = (khash *) l->client_data;

   khash_clientdata(h, attribute, defin);
   
   if (defin == NULL)
      return TRUE;  /*-- don't have the attribute --*/
   
   /*-- delete the definition from the hash... --*/
   khash_delete(h, attribute);

   /*-- ... and free the definition structure */
   _kdms_destruct_attribute_defin(defin);

   return TRUE;
}


/*-----------------------------------------------------------
|
|  Routine Name: _kdms_free_list_attribute_defin
|
|       Purpose: Free up a hash entry from a khash structure
|		 under the assumption that it contains a 
|		 an attribute definition.
|
|         Input: d - hash entry to be freed
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: March 25, 1995
| Modifications:
|
------------------------------------------------------------*/
static void _free_defin(kaddr d)
{
   _kdms_destruct_attribute_defin((kdms_attribute_defin *) d);

   return;
}

static void _kdms_free_list_attribute_defin(klist *list)
{
   if (list == NULL)
      return;

   khash_free((khash *) list->client_data, _free_defin);

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_free_attribute_defines
|
|       Purpose: Free up the static attribute definition hash table.
|		 All the definitions are freed and then the table
|		 itself is freed.
|
|         Input: none
|
|        Output: none
|
|       Returns: nothing
|
|    Written By: Steve Kubica
|          Date: March 21, 1994
| Modifications:
|
------------------------------------------------------------*/
void
kdms_free_attribute_defines(void)
{
   klist_free(attribute_lists, _kdms_free_list_attribute_defin);
   return;
}



/*
 *    ================================================================== 
 *    Attribute Management at the kobject Level 
 *    ==================================================================
 */

/*-----------------------------------------------------------
|
|  Routine Name: _kdms_establish_attribute
|
|       Purpose: This routine will establish an attribute on
|		 a given attribute list which is a copy of
|		 a given attribute.  The established 
|	         attribute will be returned.
|
|		 Only generic attributes may be 
|		 physically instantiated on the list.  If a 
|		 quasi attribute is passed into this routine,
|		 and error will occur.
|			
|         Input: atr_list - the address of a pointer to the list
| 		            to establish the attribute on.
|		 atr      - the attribute to use as a template
|			    for establishing the new attribute
|		 	    to be established.
|
|        Output: none
|
|       Returns: the established attribute
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: October 4, 1993
| Modifications:
|
------------------------------------------------------------*/
static kdas_attribute *
_kdms_establish_attribute(
   klist         **atr_list,
   kdas_attribute *atr)
{
   kdas_attribute *new_atr = NULL;

   if (atr_list == NULL || atr == NULL)
   {
      kinfo(KDEBUG, "_kdms_establish_attribute : attribute or list is NULL");
      return NULL;
   }

   /*-- generic attributes need to be instantiated --*/
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
   {
      new_atr = kdas_attr_generic_construct(atr->generic.attribute,
					    atr->generic.datatype,
					    atr->generic.argsize,
					    atr->generic.numargs,
					    atr->generic.permanent,
					    atr->generic.value);

      /*-- append attribute to the given attribute list --*/
      *atr_list = klist_add(*atr_list, (char *) new_atr->base.attribute, 
			    new_atr);
   }

   /*-- quasi-attributes can not be established --*/
   else 
   {
      kerror("kdataman", "kdms_establish_attribute", 
	     "Bogus attempt to establish quasi attribute %s\n",
	     ktoken_to_string(atr->quasi.attribute));
      return NULL;
   }

   return new_atr;
}


/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_locate_attribute_list
|
|       Purpose: Locate the attribute list in an object for
|	         a given association.   Either the physical
|		 or presentation attribute list for that
|		 association will be returned depending on
|		 the shared parameter.
|
|		 Remember, attributes at the physical level
|		 are considered to be shared among all
|		 reference objects.  Attributes at the 
|		 presentation level are specific to the
|		 presentation of that reference object and
|		 are thus unshared.
|
|         Input: obj    - the object to retrieve the list from
|                assoc  - the association of the list to retrieve
|                shared - TRUE if the physical list is desired
|                         FALSE if the presentation list is desired
|
|        Output: none
|
|       Returns: the attribute list for the given association,
| 		 NULL otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: October 4, 1993
| Modifications:
|
------------------------------------------------------------*/
static
klist **
_kdms_locate_attribute_list(
   kobject obj,
   ktoken  assoc,
   int     shared)
{
   kdas_presentation *pres = NULL;
   klist  **at_list = NULL;

   /*-- object level attributes --*/
   if (assoc == 0)
      at_list = (shared) ? &(obj->phys->attributes) : &(obj->attributes);

   /*-- segment level attributes --*/
   else if ((pres = kdms_get_segtoken_info(obj, assoc)) != NULL)
      at_list = (shared) ? &(pres->segment->attributes) : &(pres->attributes);

   else
   {
      kdas_set_error(KDMS_ESEG_NONEXIST);
      kinfo(KDEBUG,
	    "_kdms_locate_attribute_list: The segment '%s' doesn't exist.\n",
	    ktoken_to_string(assoc));

      return NULL;
   }

   return at_list;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_retrieve_attribute
|
|       Purpose: Retrieve an attribute from an object for a given
|	         association.  The association determines whether
|	         the attribute should be on the object level 
|		 attribute list or on a segment level attribute list.
|		 
|		 The presentation attribute list will be searched first, 
|		 followed by the physical attribute list.  If the
|		 attribute is not found on either of those lists,
|		 it will be established on one of the lists according
|		 to its definition.  If the attribute is shared,
|		 it will be established on the physical list. If it
|		 is not shared, it will be established on the presentation
|		 list.
|		 
|		 The resulting attribute, whether it was found, or
|		 established, will be returned.
|
|         Input: obj         - the object to retrieve the attribute from
|                attribute   - the attribute to retrieve
|                association - the attribute association
|
|        Output: none
|
|       Returns: the attribute on success, NULL otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: October 4, 1993
| Modifications:
|
------------------------------------------------------------*/
static kdas_attribute *
_kdms_retrieve_attribute(
   kobject obj,
   char   *attribute,
   ktoken  assoc)
{
   kdms_attribute_defin *defin;
   kdas_attribute       *atr = NULL;

   klist **pres_list = NULL;
   klist **phys_list = NULL;
   klist  *l;

   /*-- try to get the attribute list from the presentation first --*/
   if ((pres_list = _kdms_locate_attribute_list(obj, assoc, FALSE)) != NULL)
      if ((l = kdas_list_string_locate(*pres_list, attribute)) != NULL)
	 atr = (kdas_attribute *) l->client_data;

   /*-- if that didn't do it, then try the physical next --*/
   if (atr == NULL)
   {
      if ((phys_list = _kdms_locate_attribute_list(obj, assoc, TRUE)) != NULL) 
	 if ((l = kdas_list_string_locate(*phys_list, attribute)) != NULL)
	    atr = (kdas_attribute *) l->client_data;
   }

   /*-- if we still don't have it, then create it --*/
   if (atr == NULL)
   {
      /*-- try the definition first --*/
      defin = _kdms_locate_definition(attribute, assoc);

      if (defin == NULL)
	 return NULL; /*-- not an error...query uses this for existence --*/

      /*-- if we have a quasi-attribute definition, use that --*/
      if (defin)
	 if (defin->atr->base.type == KDAS_QUASI_ATTRIBUTE)
	    return(defin->atr);

      /*-- estabish attribute from definition --*/
      if (defin->shared)
	 atr = _kdms_establish_attribute(phys_list, defin->atr);
      else
	 atr = _kdms_establish_attribute(pres_list, defin->atr);
   }

   return atr;
}

/*-----------------------------------------------------------
|
|  Routine Name: _kdms_copy_attribute
|
|       Purpose: This routine will copy an established attribute
|		 from one object to another. 
|  
|         Input: obj1        - the object containing the source list
|                obj2        - the object containing the destination list
|                association - the association of the list
|                attribute   - the attribute to copy
|
|        Output: none
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: April 6, 1994
| Modifications:
|
------------------------------------------------------------*/
static int
_kdms_copy_attribute(
   kobject obj1,
   kobject obj2,
   char   *association,
   char   *attribute)
{
   klist **pres_list = NULL;
   klist **phys_list = NULL;
   klist **obj2_list = NULL;
   klist  *l;

   kdms_attribute_defin *defin = NULL;
   kdas_attribute *atr1 = NULL, *atr2 = NULL;
   int status = TRUE;
   int shared = FALSE;

   /*-- get the attribute definition --*/
   defin = _kdms_locate_definition(attribute, association);

   if (defin != NULL && defin->atr->base.type == KDAS_QUASI_ATTRIBUTE) 
   {
      status &= defin->atr->quasi.copy(obj1, obj2, 
				       kstring_to_token(association),
				       defin->atr->quasi.attribute,
				       defin->atr->quasi.client_data,
				       defin->atr->quasi.client_data);
   }
   else if (defin != NULL)  /* simple define-based copy */
   {
      ktoken assoc  = kstring_to_token(association);

      /*-- retrieve the attribute for object1 --*/
      if ((atr1 = _kdms_retrieve_attribute(obj1, attribute, assoc)) == NULL)
      {
	 kdas_set_error(KDMS_EATTR_NODEF);
         return FALSE;
      }
      
      /*-- retrieve the attribute for object2 --*/
      if ((atr2 = _kdms_retrieve_attribute(obj2, attribute, assoc)) == NULL)
      {
	 kdas_set_error(KDMS_EATTR_NODEF);
         return FALSE;
      }
      
      kdas_attr_copy(atr1, atr2, obj1, obj2, assoc);
   } 
   else  /*-- this is an attribute for which we have no definition --*/
   {
      ktoken assoc  = kstring_to_token(association);
      
      /*-- try to get the attribute list from the presentation first --*/
      pres_list = _kdms_locate_attribute_list(obj1, assoc, FALSE);
   
      /*-- if attribute wasn't on presentation, then check physical --*/ 
      if ((l = kdas_list_string_locate(*pres_list, attribute)) == NULL) 
      {
         shared = TRUE;  /* since it isn't presentation, assume physical */
    
         phys_list = _kdms_locate_attribute_list(obj1, assoc, TRUE);
         if ((l = kdas_list_string_locate(*phys_list, attribute)) == NULL) 
	 {
            kdas_set_error(KDMS_EATTR_NODEF);
            return FALSE;
         }     
      } 
      atr1 = (kdas_attribute *) l->client_data;

      /*-- now have obj1 attribute, get corresponding list from obj2... --*/
      obj2_list = _kdms_locate_attribute_list(obj2, assoc, shared);

      /*-- ...and establish attribute on it --*/
      if ((atr1 = _kdms_establish_attribute(obj2_list, atr1)) == NULL) 
      {
         kdas_set_error(KDAS_EATTR_COPY_FAILED);
         kinfo(KDEBUG,"_kdms_copy_attribute : failed to establish "
	       "the attribute '%s' on the destination object.", attribute);
         return FALSE;
      } else
         return TRUE;
   }

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_copy_attribute_list
|
|       Purpose: This routine will visit each attribute in a
|		 a given attribute list, and copy them to a 
|		 second attribute list.  For each attribute on
|		 the first list, the second list is checked.
|		 If the attribute does not exist on the 
|		 second list, a duplicate is constructed
|		 and placed on the second list.
|   
|		 The copy method of the attribute is then
|		 used to actually perform the copy.  Since
|                the quasi-copy requires the objects and the
|	         association, those parameters need to be
|	         passed in as well. 
|
|         Input: obj1  - the object containing the source list
|                obj2  - the object containing the destination list
|                assoc - the association of the list
|                list1 - the source list to copy
|
|        Output: list2 - the destination list
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: April 6, 1994
| Modifications:
|
------------------------------------------------------------*/
int
kdms_copy_attribute_list(
   kobject obj1,
   kobject obj2,
   ktoken  assoc,
   klist  *list1,
   klist **list2)
{
   kdas_attribute *atr1 = NULL, *atr2 = NULL;
   int status = TRUE;
   klist *l, *l2;


   /*-- trivial copy --*/
   if (list1 == NULL)
      return TRUE;

   /*--------------------------------------------------------------------
   |   check for the existence of each attribute in list2,
   |   if it doesn't exist, then establish it (make an empty copy
   |   of the existing one).  once we are sure it is in the list2, 
   |   then copy it with the attribute copy method.
   \--------------------------------------------------------------------*/ 
   for (l = list1; l != NULL; l = klist_next(l)) 
   {
      atr1 = (kdas_attribute *) l->client_data;

      l2 = kdas_list_string_locate(*list2, (char *) atr1->base.attribute);

      /*-------------------------------------------------------------------- 
      |   if the attribute is not on the second list, establishing it will
      |   generate a copy.  if it is on the second list, then simply copy
      |   the attribute.
      \--------------------------------------------------------------------*/
      if (l2 == NULL)
	 _kdms_establish_attribute(list2, atr1);

      else /*-- copy it according to type --*/
      {
	 atr2 = (kdas_attribute *) l2->client_data;

	 if (!kdas_attr_copy(atr1, atr2, obj1, obj2, assoc))
	    status = FALSE;
      }
   }

   return status;
}

/*
 *    ================================================================== 
 *    Public Routines for Attributes 
 *    ==================================================================
 */

/************************************************************
*
*  Routine Name: kdms_define_quasi_attribute - define a quasi attribute
*
*       Purpose: This routine will define a data services quasi-attribute 
*		 for a session.  A quasi-attribute is one which does not
*		 technically have any data associated with it, but one for
*		 which a given action will be invoked.  From the higher
*		 level programmer's point of view, a quasi-attribute will 
*		 behave identically to a generic attribute. The higher-level
*		 function calls will be identical. That is, the same get, 
*		 set, match, query, and print attribute functions will work on
*		 both quasi-attributes and generic attributes.
*		 Quasi-attributes are provided simply to allow
*		 an application service programmer the ability to 
*		 provide their own custom functionality for an attribute.
*		 
* 		 This quasi-attribute definition will be used to determine
*		 what action should be taken on a get, set, match, copy
*		 or print attribute call.  On any kdms attribute call for
*	 	 this attribute, the relevant action handler provided 
*		 in this definition will be called or invoked. 
*
*	         This attribute definition is distinguished by the
*		 attribute name provided here.  The name must be a string,
*		 unique for the given association.
*
*		 In each attribute call, a segment argument will
*		 be provided to give scope to the attribute.
*		 The association given in this definition is used
*		 to determine the allowable scope of the attribute.
*		 Attributes can be either scoped to the main object 
*		 (object-level) or scoped to any of the 
*		 object's data segments (segment-level).  
*
*		 The provided association is simply a string.
*                A NULL association indicates that the attribute 
*		 can only be invoked when NULL is passed in as the
*		 segment argument.  An association of KDMS_ALL_SEGMENTS 
*		 indicates that the attribute can be invoked at the 
*		 segment level for any segment name. If a specific 
*		 segment name, such as "value" is given for the 
*		 association, then the attribute can only invoked 
*		 when "value" is provided as the segment name.
*
*		 This quasi-attribute can be undefined with the 
*		 kdms_undefine_attribute call.
*
*         Input: association    - string indicating where it is legal to
*				  to invoke the atttribute.  NULL implies
*				  the attribute can be invoked at
*				  the object level.  A segment name
*			          implies that the attribute can only
*			          be invoked on that segment.  The
*			          identifier KDMS_ALL_SEGMENTS implies
*				  that the attribute can be invoked
*				  for any segment, but not for the object.
*
*		 attribute      - attribute string identifier.
*
*		 client_data    - pointer to client data.  This client
*				  data will be passed in to all the 
*			          the routines for this attribute.
*
*   		 get            - get handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_get_attribute, kdms_get_attributes, 
*				  kdms_vget_attribute, or kdms_vget_attributes
*				  function is called and the given segment 
*				  matches the definition's association.
*
*       !The get handler declaration is of the form :
*       !
*       !   int get_handler(
*       !       kobject   object,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data,
*       !       kva_list *list)
*
*                                 The object is passed from the calling get
*                                 function. The association is the tokenized
*                                 representation of the segment argument
*                                 from the get function.  The attribute is
*                                 the tokenized representation of the
*                                 attribute name.  The client_data is the
*                                 client_data from this definition.  The
*                                 variable argument list is an already
*                                 opened varargs list which contains the
*                                 pointers with which to return the 
*                                 attribute data.  The variable arguments
*                                 can be pulled off with the kva_arg()
*                                 function.  The list will be closed after
*                                 this handler returns.  The return value
*				  of this handler will also be propogated
*			          up to be the return value of the calling
*				  get function.
*
*		 set            - set handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_set_attribute, kdms_set_attributes, 
*				  kdms_vset_attribute, or kdms_vset_attributes
*				  function is called and the given segment 
*				  matches the definition's association.
*
*	!The set handler declaration is of the form :
*       !
*	!   int set_handler(
*	!       kobject   object,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data,
*       !       kva_list *list)
*
*                                 The object is passed from the calling set
*                                 function. The association is the tokenized
*                                 representation of the segment argument
*                                 from the set function.  The attribute is
*                                 the tokenized representation of the
*                                 attribute name.  The client_data is the
*                                 client_data from this definition.  The
*                                 variable argument list is an already
*                                 opened varargs list which contains the
*                                 data with which to set the attribute
*                                 data.  The variable arguments
*                                 can be pulled off with the kva_arg()
*                                 function.  The list will be closed after
*                                 this handler returns.  The return value
*                                 of this handler will also be propogated
*                                 up to be the return value of the calling
*                                 set function.
*
*		 match          - match handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_match_attribute, 
*				  kdms_match_attributes, or 
*			          kdms_vmatch_attributes
*				  function is called and the given segment 
*				  matches the definition's association.
*
*	!The match handler declaration is of the form :
*       !
*	!   int match_handler(
*	!       kobject   object1,
*	!       kobject   object2,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data1,
*       !       kaddr     client_data2)
*
*                                 The object1 and object2 arguments are 
*                                 passed from the calling match function. 
*                                 The association is the tokenized
*                                 representation of the segment argument
*                                 from the match function.  The attribute is
*                                 the tokenized representation of the
*                                 attribute name.  The client_data1 and
*                                 client_data2 arguments are from 
*                                 from this definition. The return value
*                                 of this handler will also be propogated
*                                 up to be the return value of the calling
*                                 match function.
*
*		 copy           - copy handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_copy_attribute, 
*				  kdms_copy_attributes, or 
*			          kdms_vcopy_attributes
*				  function is called and the given segment 
*				  matches the definition's association.
*
*	!The copy handler declaration is of the form :
*       !
*	!   int copy_handler(
*	!       kobject   object1,
*	!       kobject   object2,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data1,
*       !       kaddr     client_data2)
*
*                                 The object1 and object2 arguments are 
*                                 passed from the calling copy function. 
*                                 The association is the tokenized
*                                 representation of the segment argument
*                                 from the copy function.  The attribute is
*                                 the tokenized representation of the
*                                 attribute name.  The client_data1 and
*                                 client_data2 arguments are from 
*                                 from this definition. The return value
*                                 of this handler will also be propogated
*                                 up to be the return value of the calling
*                                 copy function.
*
*		 query          - query handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_query_attribute function is 
*				  called and the given segment 
*				  matches the definition's association.
*
*	!The query handler declaration is of the form :
*       !
*	!   int query_handler(
*	!       kobject   object,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data,
*       !       int      *num_args,
*       !       int      *arg_size,
*       !       int      *data_type,
*       !       int      *permanent)
*
*                                 The object is passed in from the 
*                                 calling query function.  The association 
*                                 is the tokenized representation of the 
*                                 segment argument from the query function.  
*                                 The attribute is the tokenized 
*                                 representation of the attribute name.  
*                                 The client_data argument is from 
*                                 from this definition.  The num_args
*                                 argument can be used to return the
*                                 expected number of arguments for this
*                                 attribute.  The arg_size argument can
*                                 be used to return the expected argument
*                                 size for this attribute.  The data_type
*                                 argument can be used to return the 
*                                 expected data type for this attribute.
*			          The permanent argument can be used to
*				  indicate if this attribute is stored
*				  or not.  Note that it is up to the 
*				  programmer to ensure that the values which 
*				  the handler returns for any of those
*				  arguments matches what is going to be
*				  processed in the get and set handlers.
*				  The return value of this handler will 
*				  be propogated up to be the return value 
*				  of the calling query function.  In general,
*			          a value of TRUE is interpreted to mean 
*				  that the attribute 'exists'.
*
*		 print          - print handler function for this attribute.
*			          This function will be invoked whenever 
*				  a kdms_print_attribute function is 
*				  called and the given segment 
*				  matches the definition's association.
*
*	!The print handler declaration is of the form :
*       !
*	!   int print_handler(
*	!       kobject   object,
*       !       ktoken    association,
*       !       ktoken    attribute,
*       !       kaddr     client_data,
*       !       kfile    *outfile)
*
*                                 The object is passed in from the 
*                                 calling print function.  The association 
*                                 is the tokenized representation of the 
*                                 segment argument from the print function.  
*                                 The attribute is the tokenized 
*                                 representation of the attribute name.  
*                                 The client_data argument is from 
*                                 from this definition.  The outfile
*                                 argument is the (hopefully) open 
*                                 khoros transort which this handler
*                                 can use to print the attribute to.
*                                 The return value of this handler will 
*                                 be propogated up to be the return value 
*                                 of the calling print function.
*        Output: none
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley and Steve Kubica
*          Date: Mar 25, 1994 13:13
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_define_quasi_attribute(
*                !      char *association,
*   		 !      char *attribute,
*   		 !      kaddr client_data,
*   		 !      kfunc_int *get,
*   		 !      kfunc_int *set,
*   		 !      kfunc_int *match,
*   		 !      kfunc_int *copy,
*   		 !      kfunc_int *query,
*		 !      kfunc_int *print)
*
*************************************************************/
int
kdms_define_quasi_attribute(
   char *association,
   char *attribute,
   kaddr client_data,
   int (*get)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*set)   PROTO((kobject, ktoken, ktoken, kaddr, kva_list *)),
   int (*match) PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*copy)  PROTO((kobject, kobject, ktoken, ktoken, kaddr, kaddr)),
   int (*query) PROTO((kobject, ktoken, ktoken,kaddr,int *,int *,int *,int *)),
   int (*print) PROTO((kobject, ktoken, ktoken, kaddr, kfile *)))
{
   kdms_attribute_defin *defin = NULL;
   kdms_attribute_defin *old_defin = NULL;

   KDMS_INIT;

   old_defin = _kdms_locate_definition(attribute, association);
   
   if (old_defin != NULL)
   {
      kdas_set_error(KDMS_EATTR_DEF_EXISTS);
      kinfo(KDEBUG, "kdms_define_quasi_attribute: attempt to redefine the " 
	            "attribute '%s'.", attribute);
      return FALSE;
   }

   defin = _kdms_construct_quasi_defin(attribute, association, get, set, 
				       match, copy, query, print, client_data);

   return _kdms_add_definition(attribute, kstring_to_token(association),
			       defin);
}

/************************************************************
*
*  Routine Name: kdms_define_attribute - define an attribute for
*					 for a session          
*
*	Purpose: This routine will define a data services attribute 
*		 for a session.  This attribute definition will be used
*		 as a template for creating instances of the attribute
*                on an actual data object.  This creation will be
*                implicit when the attribute is accessed by any other 
*		 kdms attribute routine.
*
*	         The attribute definition is distinguished by the
*		 given attribute name.  The name must be a string,
*		 unique for the given association.
*		 The attribute acts as storage for multiple argument 
*		 values.  Each attribute argument can be of any size 
*		 or data type.  The number of arguments, the argument
*	         size, and the data type are all specified in this
*		 definition.   
*
*		 Attributes can be permanent, implying that they are 
*		 stored to the disk when the object is closed.  
*		 Technically, a permanent attribute will only be
*		 stored if the underlying file format has the
*		 capacity for storing it.   See the man page for
*		 kdatafmt for more information of file formats.
*		 
*		 Attributes can be either created on the main object 
*		 (object-level) or created on any of the 
*		 object's data segments (segment-level).  
*		 Whether an attribute is created at the object-level
*		 or at the segment-level is determined by the 
*		 attribute definition's association.  
*
*		 The provided association is simply a string.
*                A NULL association indicates that the attribute 
*		 can only be created at the object-level.  An 
*		 association of KDMS_ALL_SEGMENTS indicates that the 
*		 attribute can be created at the segment level on 
*		 any segment name. If a specific segment name, such 
*		 as "value" is given for the association, then the 
*		 attribute can only created on the "value" segment.  
*
*		 The attribute can be either permanent or transient.
*		 Permanent impiles that when the object is closed,
*		 a representation of the attribute will be written
*		 out to the disk.  Note that this representation completely
*		 describes the attribute, and it will be possible to
*		 read this attribute later in another session even if it
*		 has not been defined there.
*
*		 In addition to being created either at the
*		 object or segment level, an attribute can also
*		 be created at the physical or presentation level.
*
*		 Changes to physical level attributes are visible
*		 to all references of the data object and are termed
*		 as "shared" attributes.  Changes to presentation
*		 level attributes are visible only to the reference
*		 object on which the change was made, and thus 
*		 the attribute is termed as "unshared".  Since all
*		 references may have potentially different values,
*		 unshared attributes may not be permanent.
*
*		 A default must be provided.  This default must consist
*		 of the proper number of arguments where each argument is of
*	         the same size and data type specified in the definition.
*		 This default will be used when initializing any attributes 
*		 created from this definition. 
*
*		 This attribute can be undefined with the 
*		 kdms_undefine_attribute call.
*
*         Input: association    - string indicating where it is legal to
*				  to create the atttribute.  NULL implies
*				  the attribute can be instantiated at
*				  the object level.  A segment name
*			          implies that the attribute can only
*			          be established on that segment.  The
*			          identifier KDMS_ALL_SEGMENTS implies
*				  that the attribute can be established
*				  on any segment, but not on the object.
*
*		 attribute      - attribute string identifier
*
*		 numargs        - number of arguments in the attribute
*
*  		 argsize        - number of units of the data type for  
* 			   	  each attribute argument 
*
*		 datatype       - data type of the attribute can be
*		 	           KBIT, KUBYTE, KBYTE,
*		 	           KUSHORT, KSHORT, KUINT, 
*			           KINT, KULONG, KLONG, KFLOAT
*			           KDOUBLE, KCOMPLEX, 
*			           KDCOMPLEX, or KSTRING  
*
* 		 permanent      - TRUE if attribute is permanent
*
*	         shared         - TRUE if attribute is shared
*
*                kvalist        - the default value in a variable 
*				  argument list containing data in the form :
*
*		 	      value1 [, value2, ...]
*                	
*                	      The list should consist of "numarg" arguments,
*                	      where each argument is of "argsize" size of
*                	      "datatype" data type.
*
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley and Steve Kubica
*          Date: September 23, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_define_attribute(
*                !      char *association,
*                !      char *attribute,
*                !      int   numargs,
*                !      int   argsize,
*                !      int   datatype,
*                !      int   permanent,
*                !      int   shared,
*                !      kvalist)
*
*************************************************************/
int
kdms_define_attribute(
   char *association,
   char *attribute,
   int   numargs,
   int   argsize,
   int   datatype,
   int   permanent,
   int   shared,
   kvalist)
{
   kdms_attribute_defin *defin = NULL;
   kdms_attribute_defin *old_defin = NULL;
   kva_list list;
   kaddr  def = NULL;
   int    status = TRUE;
   int    i;
   size_t size;

   KDMS_INIT;

   /*-------------------------------------------------------------
   |   do not allow stored attributes to exist only at the
   |   presentation layer or they will not be written out
   \--------------------------------------------------------------*/
   if (permanent && !shared)
   {
      kerror("kdataman", "kdms_define_attribute",
	     "Illegal Definition for attribute '%s' : Unshared "
	     "attributes can not be permanent.", attribute);
      return FALSE;
   }

   /*-- has this been defined previously? --*/
   old_defin = _kdms_locate_definition(attribute, association);

   if (old_defin != NULL)
   {
      /*-- if previous defin was quasi, it's okay to redefine as generic --*/
      if (old_defin->atr->base.type != KDAS_QUASI_ATTRIBUTE)
      {
	 kdas_set_error(KDMS_EATTR_DEF_EXISTS);
	 kinfo(KDEBUG, "kdms_define_attribute: attempt to redefine the "
	               "attribute '%s'.", attribute);
	 return FALSE;
      }
   }

   /*-- start the variable arguments list after the permanent argument --*/
   kva_start(list, shared);

   /*-- pull the default off of the varargs list --*/
   status &= _kdas_vargs_to_attribute_data(&def, datatype, argsize,
					   numargs, &list);

   /*-- construct the definition --*/
   defin = _kdms_construct_generic_defin(attribute, association, 
					 datatype, argsize, numargs, 
					 permanent, shared, def);

   /*-- free our copy of the default --*/
   if (def != NULL)
   {
      if (datatype == KSTRING)
      {
	 kstring *cdata = (kstring *) def;
	 for (i = 0; i < argsize * numargs; i++)
	    kfree(cdata[i]);
	 kfree_and_NULL(def);
      }
      else if (kstruct_getinfo(datatype, NULL, &size, NULL, NULL, NULL, NULL))
      {
	 char *c1  = (char *) def;
	 
	 for (i = 0; i < numargs; i++, c1 += ((int)size)*argsize) 
	    kstruct_free(c1, datatype, FALSE);

	 kfree_and_NULL(def);
      }
      else
	 kfree_and_NULL(def);
   }

   /*-- add the definition to the definition list --*/
   status &= _kdms_add_definition(attribute, kstring_to_token(association), 
				  defin);

   /*-- terminate the variable arguments list --*/
   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kdms_undefine_attribute - undefine a defined attribute
*
*       Purpose: This function will remove an attribute definition 
*		 from the definition list.  The attribute definition
*		 corresponding to the given association and attribute
*		 name will be removed.  This has no effect on any
*		 attribute which may have already been instantiated
*		 on an object.  This function can be used to undefine
*		 both generic and quasi attributes.
*
*         Input: association    - string indicating the scope of the
*				  attribute.  The same association
*			          that was used to define the attribute
*			          must be used here.
*
*		 attribute      - attribute string identifier
*
*        Output: none
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_undefine_attribute(
*                !      char *association,
*                !      char *attribute)
*
*************************************************************/
int
kdms_undefine_attribute(
   char *association,
   char *attribute)
{
   KDMS_INIT;

   return _kdms_delete_attribute_defin(attribute, association);
}

/************************************************************
*
*  Routine Name: kdms_query_attribute_definition - determines if an attribute
*						   is defined.
*
*       Purpose: This function will check to see if an attribute is
*		 defined on the definition list.  The attribute definition
*		 corresponding to the given association and attribute
*		 name will be searched for. 
*
*         Input: association    - string indicating the scope of the
*				  attribute.  The same association
*			          that was used to define the attribute
*			          must be used here.
*
*		 attribute      - attribute string identifier
*
*        Output: none
*
*       Returns: TRUE if attribute is defined, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*   Declaration: int kdms_query_attribute_definition(
*                !      char *association,
*                !      char *attribute)
*
*************************************************************/
int
kdms_query_attribute_definition(
   char *association,
   char *attribute)
{
   KDMS_INIT;

   if (_kdms_locate_definition(attribute, association) == NULL)
      return FALSE;
   else
      return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_create_attribute - instantiate an attribute
*
*       Purpose: This routine provides the programmer with a 
*		 mechanism for creating attributes specific
*		 to a program being written.
*
*		 The attribute acts as storage for multiple argument 
*		 values.  Each attribute argument can be of any size 
*		 or data type.  The number of arguments, the argument
*	         size, and the data type are all specified here.
*
*		 The attributes can be either created on the 
*		 main object (object-level) or created on any of the 
*		 object's data segments (segment-level).  
*		 Whether the attribute is created at the object-level
*		 or at the segment-level is determined by the 
*		 segment argument.  If the segment argument is
*		 NULL, then the attribute will be instantiated
*		 at the object-level.  Otherwise, the attribute
*		 will be instantiated on the specified segment, if
*		 it exists.  If an attribute with the same name
*		 and scope already exists, then this function will fail.
*
*		 The attribute can be either permanent or transient.
*		 Permanent impiles that when the object is closed,
*		 a representation of the attribute will be written
*		 out to the disk.  
*
*		 In addition to being created either at the
*		 object or segment level, the attribute can also
*		 be created at the physical or presentation level.
*
*		 Changes to physical level attributes are visible
*		 to all references of the data object and are termed
*		 as "shared" attributes.  Changes to presentation
*		 level attributes are visible only to the reference
*		 object on which the change was made, and thus 
*		 the attribute is termed as "unshared". Since all
*		 references may have potentially different values,
*		 unshared attributes may not be permanent.
*
*		 An initial value must be provided.  This initial
*		 value  must consist of the proper number of arguments 
*		 where each argument is of the same size and data type 
*		 specified in this call.
*
*		 The attribute can be destroyed with the
*		 kdms_destroy_attribute function.
*
*        Input:  object         - the object on which to instantiate
*				  the new attribute.
*
*		 segment        - segment identifier string specifying
*				  which segment to create the attribute in.  
*				  If NULL, then the attribute will be
*				  created at the object level.
*
*		 attribute      - attribute identifier string.  This
*				  identifier must be unique for the
*			 	  given segment.
*
*		 numargs        - number of arguments in the attribute
*
*  		 argsize        - number of units of the data type for  
* 			   	  each attribute argument 
*
*		 datatype        - data type of the attribute can be
*		 	           KBIT, KUBYTE, KBYTE,
*		 	           KUSHORT, KSHORT, KUINT, 
*			           KINT, KULONG, KLONG, KFLOAT
*			           KDOUBLE, KCOMPLEX, 
*			           KDCOMPLEX, or KSTRING  
*
* 		 permanent      - TRUE if attribute is permanent.
*
*	         shared         - TRUE if attribute is shared
*
*        Output: none
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: This attribute will override any defined attributes with
*		 the same name and scope.  
*    Written By: Jeremy Worley and Steve Kubica
*          Date: December 14, 1992
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: int kdms_create_attribute(
*                !      kobject  object,
*		 !      char    *segment,
*                !      char    *attribute,
*                !      int      numargs,
*                !      int      argsize,
*                !      int      datatype,
*                !      int      permanent,
*                !      int      shared)
*
*************************************************************/
int
kdms_create_attribute(
   kobject object,
   char *segment,
   char *attribute,
   int   numargs,
   int   argsize,
   int   datatype,
   int   permanent,
   int   shared)
{
   ktoken assoc  = kstring_to_token(segment);

   kdas_attribute *atr = NULL;
   klist   	 **atr_list = NULL;
   klist          *l;
   kaddr 	   def = NULL;

   KDMS_INIT;


   /*-------------------------------------------------------------
   |   do not allow stored attributes to exist only at the
   |   presentation layer or they will not be written out
   \--------------------------------------------------------------*/
   if (permanent && !shared)
   {
      kerror("kdataman", "kdms_define_attribute",
	     "Can not create attribute '%s' : Unshared "
	     "attributes can not be permanent.", attribute);
      return FALSE;
   }

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- get the attribute list for the given association --*/
   atr_list = _kdms_locate_attribute_list(object, assoc, shared);

   if (atr_list == NULL)
      return FALSE;		/* errno set already */

   /*-- make sure this attribute does not already exist --*/
   if ((l = kdas_list_string_locate(*atr_list, attribute)) != NULL)
   {
      kdas_set_error(KDMS_EATTR_ALREADY_EXISTS);
      kinfo(KDEBUG, "kdms_create_attribute: attempt to create attribute '%s' "
	    "which already exists for the association '%s'.",
	    attribute, segment == NULL ? "null" : segment);
      return FALSE;
   }

   /*-- create a default value to assign --*/
   def = kcalloc((size_t)(argsize * numargs), (size_t)kdata_size(datatype));

   /*-- construct a generic attribute according to what they wanted --*/
   atr = kdas_attr_generic_construct(kstring_to_token(attribute), datatype, 
				     argsize, numargs, permanent, def);
   
   /*-- free our copy of the default --*/
   kfree(def);

   /*-- add the attribute to the attribute list --*/
   *atr_list = klist_add(*atr_list, (char *) atr->base.attribute, atr);
         /*-- add using the tokenized string, so we can do string compares --*/

   /*-- if the list is not NULL, then it worked! --*/
   return (*atr_list != NULL);
}

/************************************************************
*
*  Routine Name: kdms_destroy_attribute - destroy an attribute
*
*       Purpose: This routine provides the programmer with a
*		 mechanism for destroying an instantiation
*	         of an attribute.
*
*		 The segment argument is used to indicate which
*		 segment the attribute exists in. If the segment 
*		 argument is NULL, then	the attribute is assumed
*		 to exist at the object level.
*
*		 If the attribute is not physically instantiated
*		 on the object, then it can not be destroyed.
*		 In general, this routine should only be used
*		 to destroy attributes that have been explicitly
*		 created with a kdms_create_attribute call.
*
*        Input:  object    - the object with the attribute to destroy
*
*		 segment   - the segment which contains the
*			     attribute.  If this is NULL, then
*			     the attribute is assumed to exist
*			     at the object level.
*
*		 attribute - string identifying the name of the 
*			     attribute to destroy.
*
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley and Steve Kubica
*          Date: February 15, 1993
*      Verified:
*  Side Effects:
* Modifications:
*   Declaration: int kdms_destroy_attribute(
*                !      kobject  object,
*		 !      char    *segment,
*                !      char    *attribute)
*
*************************************************************/
int
kdms_destroy_attribute(
   kobject object,
   char   *segment,
   char   *attribute)
{
   ktoken assoc = kstring_to_token(segment);

   klist         **atr_list = NULL;
   klist          *l;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- try to get the attribute list from the presentation first --*/
   atr_list = _kdms_locate_attribute_list(object, assoc, FALSE);

   if (atr_list == NULL)
      return FALSE;		/* errno set already */

   /*------------------------------------------------------------------- 
   |   if the attribute was on the presentation list, then delete it.
   |   otherwise, get the physical list so we can check there.  
   \-------------------------------------------------------------------*/
   l = kdas_list_string_locate(*atr_list, attribute);
   if (l == NULL)
      atr_list = _kdms_locate_attribute_list(object, assoc, TRUE);
   else
   {
      /*-- delete using the tokenized version of the string... it'll work --*/
      *atr_list = klist_delete(*atr_list, kstring_to_token(attribute));
      return TRUE;
   }

   /*------------------------------------------------------------------- 
   |   if the attribute was on the physical list, then delete it.
   |   otherwise, attribute does not exist anyway.
   \-------------------------------------------------------------------*/
   if ((l = kdas_list_string_locate(*atr_list, attribute)) != NULL)
   {
      /*-- delete using the tokenized version of the string... it'll work --*/
      *atr_list = klist_delete(*atr_list, kstring_to_token(attribute));
      return TRUE;
   }
   else
   {
      kinfo(KDEBUG, "kdms_destroy_attribute: deleting the non-existent "
	            "attribute '%s'.", attribute);
      return FALSE;
   }
}


/************************************************************
*
*  Routine Name: kdms_vset_attribute - open varargs set attribute
*
*       Purpose: This routine allows the programmer to
*		 set the value of an attribute associated
*		 with a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*                The effect of setting the attribute is
*                immediate.  For example, if this routine
*                is used to set the KDMS_DATATYPE attribute,
*                then any future access of the data in this
*                image will involve data of the specified
*                data type.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attribute to set
*		 a global attribute, if available.  In either
*		 case, if the attribute does not exist, then
*		 this routine returns FALSE, indicating a
*		 failure.
*
*         Input: object  - object to set the attribute on
*		 segment - name of segment to set the attribute on.
*		 attribute - name of the attribute to set
*		 va_list - a valist containing the values of the
*			   attribute.
*
*         Input: 
*
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremey Worley and Steve Kubica
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_vset_attribute(
   kobject   object,
   char     *segment,
   char     *attribute,
   kva_list *list)
{
   kdms_quasi_segment *segd   = NULL;
   kdas_attribute     *atr    = NULL;
   ktoken              stoken = kstring_to_token(segment);
   int                 status = TRUE;
   int                 dupdee;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- if there is a defined segment set_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->set_atr)
      {
	 segd->lock = TRUE;
	 status = segd->set_atr(object, segment, attribute, list);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal set attribute --*/   

   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL)
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   /*-- now we can set it--*/
   kdms_get_attribute(object, NULL, "DuplicateStructuresOnSet",
                      &dupdee);
   kdas_attr_setmode(dupdee);
   return kdas_attr_set(atr, object, stoken, list);
}


/************************************************************
*
*  Routine Name: kdms_vset_attributes - set attributes on a kvalist
*
*       Purpose: This routine allows the programmer to
*		 set the values of multiple attributes associated
*		 with a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*                The effect of setting the attribute is
*                immediate.  For example, if this routine
*                is used to set the KDMS_DATATYPE attribute,
*                then any future access of the data in this
*                image will involve data of the specified
*                data type.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attribute to set
*		 a global attribute, if available.  In either
*		 case, if the attribute does not exist, then
*		 this routine returns FALSE, indicating a
*		 failure.
*
*         Input: object  - object to set the attribute on
*		 segment - name of segment to set the attribute on.
*		 va_list - a valist containing attributes and their
*			   values.
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremey Worley
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_vset_attributes(
   kobject   object,
   char     *segment,
   kva_list *list)
{
   char *attribute;

   while ((attribute = kva_arg(*list, char *)) != NULL)
   {
      if (!kdms_vset_attribute(object, segment, attribute, list))
	 return FALSE;	/* errno set already */
   }

   return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_set_attribute - set the value of an attribute
*
*       Purpose: This routine allows the programmer to
*		 set the value of an attribute associated
*		 with a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*                The effect of setting the attribute is
*                immediate.  For example, if this routine
*                is used to set the KDMS_DATATYPE attribute,
*                then any future access of the data in this
*                image will involve data of the specified
*                data type.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attribute to set
*		 a global attribute, if available.  In either
*		 case, if the attribute does not exist, then
*		 this routine returns FALSE, indicating a
*		 failure.
*
*
*        INPUT:  object    - the object that is involved in
*			     the set attribute operation.
*		 segment   - the data segment whose attribute
*			     is being set.
*		  va_alist - variable argument list, that
*			     contains an attribute followed
*			     by any values associated with
*			     that attribute.  It takes the
*			     form:
*
*			     ATTRIBUTE_NAME, value1 [, value2, ...]
*
*			     The number of value arguments in
*			     the variable argument list depends
*			     on the specific attribute.  For
*			     example, KDMS_DATATYPE takes only
*			     one value, but KDMS_SIZE takes
*			     five values.
*
*        Output: none
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Mark Young and Jeremy Worley
*          Date: August 11, 1992
*      Verified:
*  Side Effects: 
* Modifications:
*
*************************************************************/
int
kdms_set_attribute(
   kobject object,
   char   *segment,
   char   *attribute,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- start the argument list to begin after the attribute argument --*/
   kva_start(list, attribute);

   /*-- call the variable argument version of kdms_set_attribute --*/
   status = kdms_vset_attribute(object, segment, attribute, &list);

   /*-- terminate the argument list --*/
   kva_end(list);

   return status;
}


/************************************************************
*
*  Routine Name: kdms_set_attributes - sets the values of multiple attributes
*
*       Purpose: The purpose of this routine is to set the values of 
*		 multiple attributes of a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*                The effect of setting an attribute is
*                immediate.  For example, if this routine
*                is used to set the KDMS_DATA_TYPE attribute,
*                then any future access of the data in this
*                image will involve data of the specified
*                data type.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attributes to get
*		 a series of global attributes, if available.
*		 In either case, if an attribute does not exist,
*		 then this routine returns FALSE, indicating a
*		 failure.
*
*         Input: object    - the object that is involved in
*			     the set attribute operation.
*		 segment   - the data segment whose attribute
*			     is being set.
*		 va_alist  - variable argument list, that
*			     contains a set of attributes, each
*			     followed by any values associated
*			     with that attribute.  It takes the
*			     form:
*
* 			     ATTRIBUTE_NAME1, value1 [, value2, ...],
*			     ATTRIBUTE_NAME2, value1,[, value2, ...],
*			     ..., NULL.
*
*			     The number of value arguments in
*			     variable argument list for each
*			     attribute depends on the specific
*			     attribute.  For
*			     example, KDMS_DATATYPE takes only
*			     one value, but KDMS_SIZE takes
*			     five values.  The NULL at the
*			     end of the variable argument
*			     list serves as a flag indicating
*			     the end of the list to kdms_set_attributes.
*
*        Output:
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley and Mark Young
*          Date: August 11, 1992
*      Verified:
*  Side Effects: 
* Modifications:
*
*************************************************************/
int
kdms_set_attributes(
   kobject object,
   char   *segment,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- set the argument list to start after the segment argument --*/
   kva_start(list, segment);

   /*-- call the variable arguments version of this routine --*/
   status = kdms_vset_attributes(object, segment, &list);

   /*-- terminate the argument list --*/
   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kdms_vget_attribute - get a single attribute on a kvalist
*
*       Purpose: The purpose of this routine is to allow the programmer to
*		 get the value of an attribute associated
*		 with a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_get_attribute to get
*		 a global attribute, if available.  In either
*		 case, if the attribute does not exist, then
*		 this routine returns FALSE, indicating a
*		 failure.
*
*         Input: 
*
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremey Worley and Steve Kubica
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*
*************************************************************/
int
kdms_vget_attribute(
   kobject   object,
   char     *segment,
   char     *attribute,
   kva_list *list)
{
   kdms_quasi_segment *segd   = NULL;
   kdas_attribute     *atr    = NULL;
   ktoken              stoken = kstring_to_token(segment);
   int                 status = TRUE;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- if there is a defined segment get_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->get_atr)
      {
	 segd->lock = TRUE;
	 status = segd->get_atr(object, segment, attribute, list);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal get attribute --*/   


   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL)
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   /*-- now we can get it--*/
   return kdas_attr_get(atr, object, stoken, list);
}

/************************************************************
*
*  Routine Name: kdms_vget_attributes - get attributes on a kvalist
*
*       Purpose: The purpose of this routine is to allow the programmer to
*		 get the values of multiple attributes associated
*		 with a data object
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attributes to get
*		 a series of global attributes, if available.
*		 In either case, if an attribute does not exist,
*		 then this routine returns FALSE, indicating a
*		 failure.
*
*         Input: 
*
*        Output: 
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley
*          Date: March 25, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_vget_attributes(
   kobject   object,
   char     *segment,
   kva_list *list)
{
   char *attribute;

   while ((attribute = kva_arg(*list, char *)) != NULL)
   {
      if (!kdms_vget_attribute(object, segment, attribute, list))
	 return FALSE;	/* errno set already */
   }

   return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_get_attribute - get the value of an
*		 attribute within a segment of an abstract
*		 object.
*
*       Purpose: The purpose of this routine is to allow the programmer to
*		 get the value of an attribute associated
*		 with a data object.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_get_attribute to get
*		 a global attribute, if available.  In either
*		 case, if the attribute does not exist, then
*		 this routine returns FALSE, indicating a
*		 failure.
*
*
*        Input:  object    - the object that is involved in
*			     the get attribute operation.
*		 segment   - the data segment whose attribute
*			     is being retrieved.
*		 va_alist  - variable argument list, that
*			     contains an attribute followed
*			     by any addresses that will be
*			     filled out with the values
*			     associated with that attribute.
*			     The variable argument list takes
*			     the form:
*
*			     ATTRIBUTE_NAME, &value1 [, &value2, ...]
*
*			     The number of value arguments in
*			     variable argument list depends
*			     on the specific attribute.  For
*			     example, KDMS_DATATYPE requires
*			     one value, but KDMS_SIZE requires
*			     five values.
*
*        Output: See the va_list description in Input.
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions:
*    Written By: Mark Young and Jeremy Worley
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_get_attribute(
   kobject object,
   char   *segment,
   char   *attribute,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- start the argument list after the attribute argument --*/
   kva_start(list, attribute);

   /*-- call the variable argument version of this routine --*/
   status = kdms_vget_attribute(object, segment, attribute, &list);

   /*-- terminate the argument list --*/
   kva_end(list);
   return status;
}


/************************************************************
*
*  Routine Name: kdms_get_attributes - gets the values of a
*		 variable number of attributes within a
*		 single segment of an object.
*
*       Purpose: The purpose of this routine is to allow the programmer to
*		 get the values of multiple attributes associated
*		 with a data object
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 this instructs kdms_set_attributes to get
*		 a series of global attributes, if available.
*		 In either case, if an attribute does not exist,
*		 then this routine returns FALSE, indicating a
*		 failure.
*
*         Input: object    - the object that is involved in
*			     the set attribute operation.
*		 segment   - the data segment whose attribute
*			     is being retrieved.
*		 va_alist  - a variable argument list that
*			     contains a set attributes, each
*			     followed by addresses to variables
*			     that will be assigned with values
*			     associated with that attribute.
*			     The variable argument list takes the
*			     form:
*
*			     ATTRIBUTE_NAME1, &value1 [, &value2, ...],
*			     ATTRIBUTE_NAME2, &value1,[, &value2, ...],
*			     ..., NULL.
*
*			     The number of value arguments in
*			     the variable argument list for each
*			     attribute depends on the specific
*			     attribute.  For
*			     example, KDMS_DATATYPE takes only
*			     one value, but KDMS_SIZE takes
*			     multiple values.  The NULL at the
*			     end of the variable argument
*			     list serves as a flag indicating
*			     the end of the list to kdms_get_attributes.
*
*        Output: See the va_alist description under Input.
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: Use only as directed...
*
*    Written By: Mark Young and Jeremy Worley
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_get_attributes(
   kobject object,
   char   *segment,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- set the argument list to start after the segment argument --*/
   kva_start(list, segment);

   /*-- call the variable argument version of this routine --*/
   status = kdms_vget_attributes(object, segment, &list);

   /*-- terminate the argument list --*/
   kva_end(list);

   return status;
}


/************************************************************
*
*  Routine Name: kdms_match_attribute - returns TRUE if the
*		 same segment attribute in two abstract
*		 data objects match.
*
*       Purpose: The purpose of this routine is to allow the programmer 
*		 to compare a single attribute between two data objects.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 This routine will return TRUE if the
*		 specified attribute has the same value
*		 in both of the abstract data objects.  This
*		 routine will return FALSE if the attribute
*		 does not have the same value in both of
*		 of the objects kdms_match_attribute will
*		 also return FALSE if the attribute does
*		 not exist in either or both of the objects.
*
*		 If the segment argument is NULL, then
*		 that implies that the attribute is a
*		 a global attribute in each of the abstract
*		 data objects.
*
*        INPUT:   object1 - the first abstract data object on
*			    which to match the specified attribute
*		  object2 - the second abstract data object on
*			    which to match the specified attribute
*		  segment  - the data segment in each abstract
*			     data object in which we will be
*			     matching the attribute.
*		  attribute - the attribute that will be
*			      compared in the two objects.
*
*        Output: none
*
*       Returns: There are three ways for this routine to
*		 return a FALSE:  (1) if the attribute
*		 in the two objects does not match; (2) if
*		 either object does not contain the specified 
*		 attribute; (3) an error condition
*		 resulting from an invalid object or segment.
*		 If none of these three conditions exist, then
*		 this function will return TRUE.
*
*  Restrictions: 
*
*    Written By: Mark Young and Jeremy Worley
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_match_attribute(
   kobject object1,
   kobject object2,
   char   *segment,
   char   *attribute)
{
   kdms_quasi_segment *segd   = NULL;
   kdas_attribute     *atr1   = NULL;
   kdas_attribute     *atr2   = NULL;
   ktoken              stoken = kstring_to_token(segment);
   int                 status = TRUE;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object1 == NULL || object1->type != KOBJ_DATA ||
       object2 == NULL || object2->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- if there is a defined segment match_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->match_atr)
      {
	 segd->lock = TRUE;
	 status = segd->match_atr(object1, object2, segment, attribute);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal match attribute --*/   


   /*-- retrieve the attribute definitions --*/
   if ((atr1 = _kdms_retrieve_attribute(object1, attribute, stoken)) == NULL) 
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   if ((atr2 = _kdms_retrieve_attribute(object2, attribute, stoken)) == NULL)
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   /*-- now we can match it --*/
   return kdas_attr_match(atr1, atr2, object1, object2, stoken);
}

/************************************************************
*
*  Routine Name: kdms_vmatch_attributes - returns true if the
*		 vararg list of segment attributes in two
*		 abstract data objects match.
*
*       Purpose: The purpose of this routine
*		 is to allow the programmer to compare multiple
*		 attributes in two object.
*
*		 This routine will return TRUE if all of the
*		 specified attributes have the same value
*		 in the objects.  This routine will return
*		 FALSE if any of the attributes do not
*		 match  kdms_match_attributes will also
*		 return FALSE if any of the attributes do
*		 not exist in either or both of the two objects.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 that implies that the attributes are
*		 global attributes in each of the abstract
*		 data objects.
*
*        INPUT:   object1  - the first abstract data object on
*			     which to match the specified attributes
*		  object2  - the second abstract data object on
*			     which to match the specified attributes
*		  segment  - the data segment in each abstract
*			     data object in which we will be
*			     matching the attributes.
*		  list     - variable argument list, that
*			     contains an arbitrarily long
*			     list of attributes followed
*			     a NULL.  It takes the form:
*
*			     ATTRIBUTE_NAME1, ATTRIBUTE_NAME2, ..., NULL
*
*        Output: none
*       Returns: There are three ways for this routine to
*		 return a FALSE:  (1) if any of the attributes
*		 between the two objects do not match; (2) if
*		 either object does not contain one or more of
*		 the specified attributes; (3) an error condition
*		 resulting from an invalid object or segment.
*		 If none of these three conditions exist, then
*		 this function will return TRUE.
*
*  Restrictions: 
*
*    Written By: Jeremy Worley & John Salas
*          Date: October 6, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_vmatch_attributes(
   kobject   object1,
   kobject   object2,
   char     *segment,
   kva_list *list)
{
   char *attribute;

   KDMS_INIT;

   while ((attribute = kva_arg(*list, char *)) != NULL)
   {
      if (!kdms_match_attribute(object1, object2, segment, attribute))
      {
	 kva_end(*list);
	 return FALSE;	/* not an error */
      }
   }

   return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_match_attributes - returns true if the
*		 list of segment attributes in two
*		 abstract data objects match.
*
*       Purpose: The purpose of this routine
*		 is to allow the programmer to compare multiple
*		 attributes in two object.
*
*		 This routine will return TRUE if all of the
*		 specified attributes have the same value
*		 in the objects.  This routine will return
*		 FALSE if any of the attributes do not
*		 match  kdms_match_attributes will also
*		 return FALSE if any of the attributes do
*		 not exist in either or both of the two objects.
*
*		 Data services manages two sets of attributes
*		 with each object.  Internally, they are referred
*		 to as the physical attributes and the presentation
*		 attributes.  The presentation version of an attribute
*		 is the value that is desired by the programmer.  The
*		 corresponding physical attribute is what is physically
*		 stored on the file or transport associated with the
*		 object.  If the two versions of an attribute result
*		 in different presentations of data, then data services
*		 automatically translates the data from the physical
*		 interpretation to the presentation interpretation, 
*		 or visa-versa, as appropriate.  For example, if the
*		 data type attribute (KDMS_DATA_TYPE) is physically
*		 unsigned int (KULONG), but the presentation value
*		 of that attribute is float (KFLOAT), then data services
*		 will retrieve the data from the object and and cast it to
*		 float before returning it to the user.  Similar behavior
*		 will occur for attributes such as index order 
*		 (KDMS_INDEX_ORDER) and size (KDMS_SIZE).
*
*		 The presentation attributes are settable
*		 at any time throughout a object's lifetime.  The
*		 physical attributes, however, at some point are
*		 locked and thereafter cannot change.  An object
*		 that was opened as an input object (via kdms_input
*		 or kdms_open) has its physical attributes locked
*		 immediately.
*
*		 If the segment argument is NULL, then
*		 that implies that the attributes are
*		 global attributes in each of the abstract
*		 data objects.
*
*        INPUT:   object1 - the first abstract data object on
*			    which to match the specified attributes
*		  object2 - the second abstract data object on
*			    which to match the specified attributes
*		  segment  - the data segment in each abstract
*			     data object in which we will be
*			     matching the attributes.
*		  va_alist - variable argument list, that
*			     contains an arbitrarily long
*			     list of attributes followed
*			     a NULL.  It takes the form:
*
*			     ATTRIBUTE_NAME1, ATTRIBUTE_NAME2, ..., NULL
*
*        Output: none
*       Returns: There are three ways for this routine to
*		 return a FALSE:  (1) if any of the attributes
*		 between the two objects do not match; (2) if
*		 either object does not contain one or more of
*		 the specified attributes; (3) an error condition
*		 resulting from an invalid object or segment.
*		 If none of these three conditions exist, then
*		 this function will return TRUE.
*
*  Restrictions: 
*
*    Written By: Mark Young and Jeremy Worley
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_match_attributes(
   kobject object1,
   kobject object2,
   char   *segment,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- set the argument list to start after the segment argument --*/
   kva_start(list, segment);

   /*-- call the varargs version of the routine --*/
   status = kdms_vmatch_attributes(object1, object2, segment, &list);

   /*-- terminate the argument list --*/
   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kdms_copy_attribute - copy an attribute from a 
*			source object to a destination object.
*
*       Purpose: This function is used to copy a single 
*		 attribute from one object to another object.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_VALUE_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*
*		 This function accepts a source object, a
*		 destination object, and an attribute name.
*		 If the attribute exists in the source object,
*		 then it will be copied to the destination object.
*		 If the attribute does not exist in the source
*		 object, then an error condition is returned.
*		
*         Input: object1 - the source for the copy operation
*		 object2 - the destination for the copy operation
*		 segment - the segment which contains the object to
*		 	   be copied.
*		 attribute - the attribute to be copied.
*
*        Output: 
*       Returns: TRUE on success, FALSE otherwise.
*  Restrictions: 
*
*    Written By: Jeremy Worley and Steve Kubica
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_copy_attribute(
   kobject object1,
   kobject object2,
   char   *segment,
   char   *attribute)
{
   kdms_quasi_segment *segd = NULL;
   int status = TRUE;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object1 == NULL || object1->type != KOBJ_DATA ||
       object2 == NULL || object2->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- if there is a defined segment copy_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->copy_atr)
      {
	 segd->lock = TRUE;
	 status = segd->copy_atr(object1, object2, segment, attribute);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal copy attribute --*/   

   return _kdms_copy_attribute(object1, object2, segment, attribute);
}

/************************************************************
*
*  Routine Name: kdms_vcopy_attributes - copy attributes given in a kvalist
*
*       Purpose: This function is used to copy multiple attributes
*		 given a variable argument list containing a list
*		 attributes to copy.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_VALUE_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*
*		 This function accepts a source object, a
*		 destination object, and an attribute name.
*		 If the attribute exists in the source object,
*		 then it will be copied to the destination object.
*		 If the attribute does not exist in the source
*		 object, then an error condition is returned.
*		
*         Input: object1 - the source for the copy operation
*		 object2 - the destination for the copy operation
*		 segment - the segment which contains the object to
*		 	   be copied.
*		 kvalist - A NULL terminated variable argument 
*		 	   list of attributes to be copied.
*
*        Output: 
*       Returns: TRUE on success, FALSE otherwise.
*
*  Restrictions: 
*
*    Written By: Jeremy Worley & Steve Kubica
*          Date: October 6, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_vcopy_attributes(
   kobject   object1,
   kobject   object2,
   char     *segment,
   kva_list *list)
{
   char *attribute;

   KDMS_INIT;

   while ((attribute = kva_arg(*list, char *)) != NULL)
   {
      if (!kdms_copy_attribute(object1, object2, segment, attribute))
      {
	 kva_end(*list);
	 return FALSE;	/* errno set already */
      }
   }

   return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_copy_attributes - copy attributes from a 
*		 		        source object to a destination object.
*
*       Purpose: This function is used to copy multiple attributes
*		 given a variable argument list containing a list
*		 attributes to copy.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KPDS_VALUE_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object (see
*	         kpds_query_attribute for information on how to
*	         determine whether an attribute is shared or
*	         unshared).
*
*		 This function accepts a source object, a
*		 destination object, and an attribute name.
*		 If the attribute exists in the source object,
*		 then it will be copied to the destination object.
*		 If the attribute does not exist in the source
*		 object, then an error condition is returned.
*		
*         Input: object1 - the source for the copy operation
*		 object2 - the destination for the copy operation
*		 segment - the segment which contains the object to
*		 	   be copied.
*		 kvalist - A NULL terminated variable argument 
*		 	   list of attributes to be copied.
*
*        Output:
*       Returns: TRUE on success, FALSE otherwise.
*		
*  Restrictions: 
*
*    Written By: Jeremy Worley and Steve Kubica
*          Date: August 11, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_copy_attributes(
   kobject object1,
   kobject object2,
   char   *segment,
   kvalist)
{
   kva_list list;
   int status;

   KDMS_INIT;

   /*-- set the argument list to start after the segment argument --*/
   kva_start(list, segment);

   /*-- call the varargs version of the routine --*/
   status = kdms_vcopy_attributes(object1, object2, segment, &list);

   /*-- terminate the argument list --*/
   kva_end(list);

   return status;
}

/************************************************************
*
*  Routine Name: kdms_query_attribute - get information about an attribute
*
*       Purpose: This function is used for two purposes:  
*		 (1) to determine the existence of an attribute; and
*		 (2) to obtain the characteristics of the attribute.
*
*		 Data Services manages two versions of some of the
*		 attributes associated with each object.  These
*		 attributes are the size and data type.  Internally,
*		 the two versions of these attributes are referred to
*		 as the physical attribute and the presentation
*		 attribute.  Typically, the programmer has access
*		 to only the presentation versions of these attributes.
*		 The physical attributes are set indirectly depending
*		 on the setting of KDMS_COUPLING.  See kpds_get_data
*		 for a description of how the presentation and physical
*		 attributes affect interaction with the data object.
*
*	         Other attributes are classified as either shared or
*	         unshared.  Shared attributes are stored at the
*	         physical layer of the attribute, and thus can be
*	         shared by multiple references of the data object (see
*	         kpds_reference_object for more information about
*	         references).  Unshared attributes, on the other hand,
*	         can only be used by the local object.
*
*		 The difference between shared and unshared attributes
*		 is abstracted from the user at the PDS level.  The
*		 permanent attributes are generally shared, and the
*		 non-permanent attributes are generally non-shared.
*		 Permanent attributes are attributes that will be stored
*		 as part of an output object when it is written.  Any
*		 attributes that are retrieved when an object is opened
*		 are also permanent attributes.  Non-permanent attributes
*		 exist only while the program that is operating on the
*		 object is executing.
*
*		 The datatype argument indicates what kind of
*		 information is stored in the attribute. Attributes
*		 can be one of the following data types: KBYTE,
*		 KUBYTE, KSHORT, KUSHORT, KINT, KUINT, KLONG, KULONG,
*		 KFLOAT, KDOUBLE, KCOMPLEX, or KDCOMPLEX.  
*
*		 The numargs argument indicates how many arguments
*		 must be passed in an argument list to one of the
*		 attribute functions.
*
*		 The argsize arguments indicates the number of
*		 units of the data type there are in each argument.
*		 This argument allows arrays of information to be
*		 stored as attributes.
*
*         Input: object      - the object with the attribute being
*			       queried
*				
*		 segment     - the segment that the attribute
*			       is stored in.  If this argument
*			       is NULL, then the attribute is
*			       global to the object.
*		 name        - name of the attribute to be
*			       queried.
*		 datatype    - datatype of the attribute
*		 numargs     - number of arguments in this attribute
*		 argsize     - size of each argument in this
*		 	       attribute.
*		 permanent   - is the attribute stored or
*			       transient?  The return value
*			       will be either TRUE or FALSE
*
*       Returns: TRUE if attribute exists, FALSE otherwise
*
*  Restrictions: 
*    Written By: Jeremy Worley and Steve Kubica
*          Date: September 23, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_query_attribute(
   kobject  object,
   char    *segment,
   char    *attribute,
   int     *numargs,
   int     *argsize,
   int     *datatype,
   int     *permanent)
{ 
   kdms_quasi_segment *segd   = NULL;
   kdas_attribute     *atr    = NULL;
   ktoken              stoken = kstring_to_token(segment);
   int                 status = TRUE;


   KDMS_INIT;

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- if there is a defined segment query_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->query_atr)
      {
	 segd->lock = TRUE;
	 status = segd->query_atr(object, segment, attribute, numargs,
				  argsize, datatype, permanent);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal query attribute --*/   

   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL)
      return FALSE; /* this is not an error...false == non-exist */

   /*-- now we can query it - query it according to type --*/
   return kdas_attr_query(atr, object, stoken, numargs, argsize,
			  datatype, permanent);
}

/************************************************************
*
*  Routine Name: kdms_print_attribute - print the value of an attribute
*
*       Purpose: This function is used to print a single
*		 attribute to an open kfile.
*
*		 This function is typically used by such programs as
*		 kprdata to print out the values of attributes in
*		 an object.
*
*         Input: object     - the object containing the attribute
*		 segment    - the segment in the object which contains
*		 	      the attribute.
*		 attribute  - the attribute to print
*		 printfile  - the open kfile to print to
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: September 23, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_print_attribute(
   kobject object,
   char   *segment,
   char   *attribute,
   kfile  *printfile)
{
   kdms_quasi_segment *segd   = NULL;
   kdas_attribute     *atr    = NULL;
   ktoken              stoken = kstring_to_token(segment);
   int                 status = TRUE;

   
   KDMS_INIT;

   /*-- sanity check --*/
   if (object == NULL || object->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   if (printfile == NULL)
   {
      kdas_set_error(KINVALID_FILE);
      return FALSE;
   }

   /*-- if there is a defined segment print_atr routine, then use that --*/
   if ((segd = kdms_locate_segment_define(segment)) != NULL)
      if (segd->print_atr)
      {
	 segd->lock = TRUE;
	 status = segd->print_atr(object, segment, attribute, printfile);
	 segd->lock = FALSE;
	 return status;
      }
   /*-- otherwise, do a normal print attribute --*/   

   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL)
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   /*-- now we can print --*/
   return kdas_attr_print(atr, object, stoken, printfile);
}

/*
 *    ================================================================== 
 *    Attribute Utility Functions
 *    ==================================================================
 */

/************************************************************
*
*  Routine Name: kdms_get_attribute_names - get a list of attributes 
*	                                    from an object.
*
*       Purpose: This function returns a list of attributes
*		 associated with the specified segment.  If
*		 the segment is NULL, then it returns a list
*		 of the attributes for the object.
*
*		 The allowable regular expression syntax is :
*
* !   .             Match any single character except newline
* !
* !   *             Match the preceding character or range 
* !                 of characters 0 or more times.  The 
* !                 matching includes items within a [...].
* !
* ! [...] or [^..]  Matches any one character contained within 
* !                 the brackets. If the first character after 
* !                 the '[' is the ']', then it is included in 
* !                 the characters to match.  If the first 
* !                 character after the '[' is a '^', then it 
* !                 will match all characters NOT included in 
* !                 the [].  The '-' will indicate a range of 
* !                 characters.  For example, [a-z] specifies 
* !                 all characters between and including the 
* !                 ascii values 'a' and 'z'.  If the '-' 
* !                 follows the '[' or is right before the ']' 
* !                 then it is interpreted literally.
* !             
* !   ^             If this is the first character of the 
* !                 regular expression, it matches the beginning
* !                 of the line.
* !  
* !   $             If this is the last character of the 
* !                 regular expression, it matches the end of
* !                 the line. 
* !
* !   \\            This escapes the meaning of a special character.
* ! 
*
*		 The array that is returned must be freed by
*		 the user using the call karray_free.
*
*         Input: object      - the object to get the attribute names from. 
*
*	         segment     - the segment to get the attribute names from.
*			       If NULL, then get the object-level names.
*
*                filter      - a regular expression filter.  Only
*			       attribute names passing the filter
*			       will be returned in the list.
*
*                permanent   - TRUE if only permanent attributes 
*                              should be included in the list
*
*        Output: number - the number of attribute names that
*		          are being returned
*
*       Returns: an array of attribute names
*
*  Restrictions: 
*    Written By: Steve Kubica and Jeremy Worley
*          Date: April 30, 1994
*      Verified:
*  Side Effects:
*    Modifications:
*
*************************************************************/
char **
kdms_get_attribute_names(
   kobject object,
   char   *segment,
   char   *filter,
   int     permanent,
   int    *number)
{
   ktoken assoc = kstring_to_token(segment);
   kdas_attribute *atr = NULL;
   char **attributes   = NULL;
   char *a = NULL;
   int 	 nattributes = 0;
   int 	 pass;
   int 	 have_filter = FALSE;

   klist **atr_list = NULL;
   klist  *l;


   /*-- compile the filter if it is present --*/
   if (filter != NULL)
   {
      if (kre_comp(filter) != NULL)
      {
	 kinfo(KDEBUG, "kdms_get_attribute_list: the filter '%s' does not "
	               "compile into a valid regular expression\n", filter);
	 kinfo(KDEBUG, "Filter not being used");
      }
      else
	 have_filter = TRUE;
   }

   /*-- permenent attributes are never at the presentation --*/
   if (!permanent)
   {
      /*-- get the presentation attribute list for the given association --*/
      atr_list = _kdms_locate_attribute_list(object, assoc, FALSE);

      if (atr_list != NULL)
	 for (l = *atr_list; l != NULL; l = klist_next(l))
	 {
	    atr = (kdas_attribute *) l->client_data;
	    
	    a = ktoken_to_string(atr->base.attribute);

	    if (have_filter)
	       pass = kre_exec(a);
	    else
	       pass = TRUE;

	    if (pass)
	    {
	       attributes = karray_add(attributes, KSTRING, kstrdup(a),
				(size_t) nattributes);
	       nattributes++;
	    }
	 }
   }

   /*-- get the physical attribute list for the given association --*/
   atr_list = _kdms_locate_attribute_list(object, assoc, TRUE);

   if (atr_list != NULL)
      for (l = *atr_list; l != NULL; l = klist_next(l))
      {
	 atr = (kdas_attribute *) l->client_data;

	 /*-- only get permanent generic attributes with data --*/
	 if (permanent)
	 {
	    if ((atr->base.type == KDAS_GENERIC_ATTRIBUTE) &&
		atr->generic.permanent == TRUE && atr->generic.value != NULL)
	    {
	       a = ktoken_to_string(atr->base.attribute);

	       if (have_filter)
		  pass = kre_exec(a);
	       else
		  pass = TRUE;

	       if (pass)
	       {
		  attributes = karray_add(attributes, KSTRING, kstrdup(a),
				   (size_t) nattributes);
		  nattributes++;
	       }
	    }
	 }
	 else
	 {
	    a = ktoken_to_string(atr->base.attribute);

	    if (have_filter)
	       pass = kre_exec(a);
	    else
	       pass = TRUE;

	    if (pass)
	    {
	       attributes = karray_add(attributes, KSTRING, kstrdup(a),
				(size_t) nattributes);
	       nattributes++;
	    }
	 }
      }

   if (number != NULL)
      *number = nattributes;

   return attributes;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_get_Vattribute
|
|       Purpose: 
|
|         Input: 
|
|        Output: 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: November 17, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kdms_get_Vattribute(
   kobject object,
   char   *segment,
   char   *attribute,
   kaddr  *dude)
{
   kdas_attribute *atr    = NULL;
   ktoken          stoken = kstring_to_token(segment);

   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL) 
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;		/* errno set already */
   }

   /*-- this only works for generic attributes --*/
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
      *dude = (kaddr) atr->generic.value;
   else
      return FALSE;

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_set_Vattribute
|
|       Purpose: 
|
|         Input: 
|
|        Output: 
|
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Jeremy Worley
|          Date: November 17, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kdms_set_Vattribute(
   kobject object,
   char   *segment,
   char   *attribute,
   kaddr   dude)
{
   kdas_attribute *atr    = NULL;
   ktoken          stoken = kstring_to_token(segment);

   /*-- retrieve the attribute definition --*/
   if ((atr = _kdms_retrieve_attribute(object, attribute, stoken)) == NULL)
   {
      kdas_set_error(KDMS_EATTR_NODEF);
      return FALSE;
   }

   /*-- this only works for generic attributes --*/
   if (atr->base.type == KDAS_GENERIC_ATTRIBUTE)
   {
      int    i;
      size_t size;

      /*-- free the data that was there --*/
      if (atr->generic.value != NULL)
      {
	 if (atr->generic.datatype == KSTRING)
	 {
	    kstring *cdata = (kstring *) atr->generic.value;
	    for (i = 0; i < atr->generic.argsize * atr->generic.numargs; i++)
	       kfree(cdata[i]);
	    kfree_and_NULL(atr->generic.value);
	 }
	 else if (kstruct_getinfo(atr->generic.datatype, NULL, &size, 
				  NULL, NULL, NULL, NULL))
	 {
	    char *c1  = (char *) atr->generic.value;

	    for (i = 0; i < atr->generic.numargs; i++) 
	    {
	       kstruct_free(c1, atr->generic.datatype, FALSE);
	       c1 += ((int)size) * atr->generic.argsize;
	    }

	    kfree_and_NULL(atr->generic.value);
	 }
	 else if (atr->generic.value != NULL)
	    kfree_and_NULL(atr->generic.value);
      }

      /*-- assign the attribute data --*/
      atr->generic.value = dude;
   }
   return TRUE;
}

/************************************************************
*
*  Routine Name: kdms_copy_segment_attributes - copy attributes 
*                from a segment in a source object to the segment 
*	  	 of a destination object.
*
*       Purpose: This routine will copy the list of instantiated
*		 attributes from one segment in one object to
*		 one segment in another.  This routine can also
*		 copy the object level attributes, if NULL is
*		 passed in for the source segment name.
*
*		 The presentation of the given object's segment
*		 size, index order, and data type are set as 
*		 the size, index order, and data type of the 
*		 destination segment.
*
*		 Currently, only the physically instantiated 
*		 user-attributes are copied.
*
*         Input: sobject   - the object to copy from
*
*		 segment   - the segment to copy from.
*			     If this is NULL, then the
*			     object level attributes will 
*			     be copied.  
*
*                dobject   - the object to copy into
*
*		 dsegment  - the segment to copy to, if different
*			     than the source segment. If NULL, then
*			     the source segments will be used.
*        Output: none
*
*       Returns: TRUE on success, FALSE otherwise
*
*  Restrictions: 
*    Written By: Steve Kubica
*          Date: March 30, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
int
kdms_copy_segment_attributes(
   kobject sobject,
   char   *segment,
   kobject dobject,
   char   *dsegment)
{
   kdas_presentation *spres = NULL;
   kdas_presentation *dpres = NULL;
   int transferable;


   KDMS_INIT;
   
   if (dsegment == NULL)
      dsegment = segment;
   
   /*-- sanity check --*/
   if (sobject == NULL || sobject->type != KOBJ_DATA ||
       dobject == NULL || dobject->type != KOBJ_DATA)
      kdas_fail_error(KDMS_ENULL_KOBJECT);

   /*-- copy only the object level attributes --*/
   if (segment == NULL)
   {
      /*-- copy all of the physical object-level attributes --*/
      kdms_copy_attribute_list(sobject, dobject, NULL, 
			       sobject->phys->attributes, 
			       &dobject->phys->attributes);

      /*-- copy all of the presentation object-level attributes --*/
      kdms_copy_attribute_list(sobject, dobject, NULL,
			       sobject->attributes, &dobject->attributes);

      return TRUE;
   }

   if (!kdms_get_attribute(sobject, segment, KDMS_TRANSFERABLE, &transferable))
      return FALSE;

   /*-- if the segment is not transferable, then drop out now --*/
   if (!transferable)
      return TRUE;
   
   /*-- get the presentation associated with the first segment --*/
   if ((spres = kdms_get_segment_info(sobject, segment)) == NULL)
      return FALSE;

   /*------------------------------------------------------------------------
   | try to get the presentation associated with the second segment. if it
   | does not exist, then create one.  Otherwise, copy the attributes.
   \------------------------------------------------------------------------*/
   if ((dpres = kdms_get_segment_info(dobject, dsegment)) == NULL)
      dpres = kdms_segment_create(dobject, dsegment);

   /*-- copy all the physical attributes --*/
   kdms_segment_copy_physical_attributes(spres, dpres, TRUE);

   /*-- copy over all presentation attributes --*/
   dpres->complex_convert    = spres->complex_convert;
   dpres->access_interpolate = spres->access_interpolate;
   dpres->rpad 		     = spres->rpad;
   dpres->ipad 		     = spres->ipad;
   dpres->scaling      	     = spres->scaling;
   dpres->scale_factor 	     = spres->scale_factor;
   dpres->scale_roffset      = spres->scale_roffset;
   dpres->scale_ioffset      = spres->scale_ioffset;
   dpres->norm_min           = spres->norm_min;
   dpres->norm_max           = spres->norm_max;
   
   /*-- copy all of the user defined attributes --*/
   kdms_copy_attribute_list(sobject, dobject, NULL, 
			    spres->segment->attributes,
			    &dpres->segment->attributes);

   kdms_copy_attribute_list(sobject, dobject, NULL,
			    spres->attributes, &dpres->attributes);
   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: kdms_get_stored_attribute_names
|
|       Purpose: Gets a list of stored attributes
|		 from an object or segment.
|
|		 THIS IS A PRIVATE FUNCTION THAT SHOULD NOT
|		 BE USED.  THIS FUNCTIONALITY WILL BE ROLLLED
|		 INTO A MORE GENERAL MECHANISM IN THE NEAR 
|		 FUTURE.
|         Input:
|        Output:
|       Returns:
|    Written By: Jeremy Worley
|          Date: August 29, 1994
| Modifications:
|
------------------------------------------------------------*/
char **
kdms_get_stored_attribute_names(
   kobject obj,
   char   *segment,
   int    *number)
{
   kdas_presentation *pres;
   kdas_attribute    *atr;
   char **attributes  = NULL;
   int    nattributes = 0;
   klist *l;

   char *a;
   
   if (segment == NULL)
   {
      for (l = obj->phys->attributes; l != NULL; l = klist_next(l))
      {
	 atr = (kdas_attribute *) l->client_data;
	 
	 if (atr->generic.permanent)
	 {
	    a = ktoken_to_string(atr->base.attribute);
	    attributes = karray_add(attributes, KSTRING, kstrdup(a),
				(size_t) nattributes);
	    nattributes++;
	 }
      }
   }
   else if ((pres = kdms_get_segment_info(obj, segment)) == NULL)
   {
      if (pres->segment->attributes)
      {
	 for (l = pres->segment->attributes; l != NULL; l = klist_next(l))
	 {
	    atr = (kdas_attribute *) l->client_data;

	    if (atr->generic.permanent)
	    {
	       a = ktoken_to_string(atr->base.attribute);
	    
	       attributes = karray_add(attributes, KSTRING, kstrdup(a),
				       (size_t) nattributes);
	       nattributes++;
	    }
	 }
      }
   }
   
   *number = nattributes;
   
   return attributes;
}


