/*
  * Khoros: $Id: init.c,v 1.10 1996/04/04 23:14:33 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: init.c,v 1.10 1996/04/04 23:14:33 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Data Services Initialization 
   >>>>		
   >>>>  Private:
   >>>>                 kdms_init()
   >>>>   Static:
   >>>>                 _kdms_init_formats()
   >>>>                 _kdms_exit_data_services()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

static void _kdms_exit_data_services PROTO((kexit_status, kaddr));
static void _kdms_init_formats       PROTO((void));

int _dms_initialized = FALSE;

/* ################################################################## */

/*-----------------------------------------------------------
|
|  Routine Name: kdms_init
|
|       Purpose: Initializes all attributes, errors, formats,
|		 etc. needed by Data Management Services.
|
|         Input: none
|
|        Output: none
|
|       Returns: TRUE all the time
|
|    Written By: Jeremy Worley and Steve Kubica
|          Date: October 4, 1993
| Modifications:
|
------------------------------------------------------------*/
int
kdms_init(void)
{
   if (_dms_initialized == TRUE)
      return TRUE;

   /*-- prevent this routine from being called again --*/
   _dms_initialized = TRUE;

   /*-- initialize low level data service errnos --*/
   kdas_init();

   /*-- initialize the formats --*/
   _kdms_init_formats();
   
   /*-- initialize the errors --*/
   kdms_init_errors();

   /*-- define all the dms attributes --*/
   kdms_init_attributes();

   /*-- add exit handler to cleanup data services upon exit --*/
   kexit_handler(_kdms_exit_data_services, NULL);

   return TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_init_formats
|
|       Purpose: This routine initializes the raw and kdf
|		 formats for kdms.
|
|         Input: none
|
|        Output: none
|
|       Returns: none
|
|    Written By: Steve Kubica
|          Date: March 7, 1996
| Modifications:
|
------------------------------------------------------------*/
static void
_kdms_init_formats(void)
{
   init_kdf();
   init_raw();

   return;
}

/*-----------------------------------------------------------
|
|  Routine Name: (static) _kdms_exit_data_services
|
|       Purpose: This routine is used to cleanup data services upon
|                exiting of the application.  Currently the only
|                thing it does is to make sure that all the data
|                services are closed.
|
|         Input: status      - the exit status
|                client_data - the data we installed with the 
|                              kexit_handler handler
|
|        Output: nothing
|       Returns: TRUE on success, FALSE otherwise
|
|    Written By: Mark Young and Jeremy Worley
|          Date: May 17, 1993
| Modifications:
|
------------------------------------------------------------*/
/*ARGSUSED */
static void
_kdms_exit_data_services(kexit_status status, kaddr client_data)
{
   /*-- close any open objects --*/
   kdms_close_all_objects(status);

   /*-- free all attribtue definitions --*/
   kdms_free_attribute_defines();

   /*-- free all segment definitions --*/ 
   kdms_free_segment_defines();

   /*-- free up the data used by the data pipelines --*/
   kdas_free_all_buffers();

   /*-- free up all formats which have been defined --*/
   kdms_free_formats();
   
   return;
}

