#include <dataserv.h>

/*--*  This test checks :
 *--*      
 *--*   o  basic segment presentation functionality with large data set support
 *--*   o  presentation wrapping on reads for out of bounds data access
 *--*
 *--*   o  write an entire data set to an output object 
 *--*
 *--*   o  set larger presentation size with WRAPPING
 *--*
 *--*  A command-line parameter varies the threshold size 
 *--*     ktest 30  : will use a threshold of 30 bytes when testing 
 *--*/

#define FAIL(error)               \
        {                         \
          kinfo(KFORCE,error);    \
          kexit(KEXIT_FAILURE);   \
        }

#define O_DIM    3
#define O_SIZE  {5, 3, 2}
#define O_ORDER {1, 2, 3}

/*-- the original 5x3x2 data array to consider --*/
#define DATA_ARRAY \
    {  1, 1, 1, 1, 1, \
       1, 2, 3, 9, 3, \
       8, 7, 6, 5, 4, \
                      \
       2, 2, 2, 2, 2, \
       2, 9, 1, 7, 3, \
       4, 8, 7, 5, 6  }

#define PIECE1_LOC { 0, 0, 0 }   /*-- in-bounds piece --*/
#define PIECE1_SIZ { 2, 3, 2 }
#define PIECE1 \
   { 1, 1,     \
     1, 2,     \
     8, 7,     \
     2, 2,     \
     2, 9,     \
     4, 8   }

#define PIECE2_LOC { -2, -1,  0 }
#define PIECE2_SIZ {  3,  6,  2 }
#define PIECE2   \
   {  5,  4,  8, \
      1,  1,  1, \
      9,  3,  1, \
      5,  4,  8, \
      1,  1,  1, \
      9,  3,  1, \
      5,  6,  4, \
      2,  2,  2, \
      7,  3,  2, \
      5,  6,  4, \
      2,  2,  2, \
      7,  3,  2  }

#define PIECE3_LOC { -1, -2, -1 }
#define PIECE3_SIZ {  3,  6,  2 }
#define PIECE3   \
   {  3,  2,  9, \
      6,  4,  8, \
      2,  2,  2, \
      3,  2,  9, \
      6,  4,  8, \
      2,  2,  2, \
      3,  1,  2, \
      4,  8,  7, \
      1,  1,  1, \
      3,  1,  2, \
      4,  8,  7, \
      1,  1,  1  }

#define PIECE4_LOC { -1, -2,  1 }
#define PIECE4_SIZ {  3,  6,  2 }
#define PIECE4   \
   {  3,  2,  9, \
      6,  4,  8, \
      2,  2,  2, \
      3,  2,  9, \
      6,  4,  8, \
      2,  2,  2, \
      3,  1,  2, \
      4,  8,  7, \
      1,  1,  1, \
      3,  1,  2, \
      4,  8,  7, \
      1,  1,  1  }

#define PIECE5_LOC { -1, -1, -1 }
#define PIECE5_SIZ {  7,  5,  5 }
#define PIECE5   \
   {  6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2,  \
      3,  2,  9,  1,  7,  3,  2,  \
      6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2,  \
\
      4,  8,  7,  6,  5,  4,  8,  \
      1,  1,  1,  1,  1,  1,  1,  \
      3,  1,  2,  3,  9,  3,  1,  \
      4,  8,  7,  6,  5,  4,  8,  \
      1,  1,  1,  1,  1,  1,  1,  \
\
      6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2,  \
      3,  2,  9,  1,  7,  3,  2,  \
      6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2,  \
\
      4,  8,  7,  6,  5,  4,  8,  \
      1,  1,  1,  1,  1,  1,  1,  \
      3,  1,  2,  3,  9,  3,  1,  \
      4,  8,  7,  6,  5,  4,  8,  \
      1,  1,  1,  1,  1,  1,  1,  \
\
      6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2,  \
      3,  2,  9,  1,  7,  3,  2,  \
      6,  4,  8,  7,  5,  6,  4,  \
      2,  2,  2,  2,  2,  2,  2   }

#define PIECE6_LOC { -2, -2, -2 }   /*-- totally out of bounds piece --*/
#define PIECE6_SIZ {  2,  2,  2 }
#define PIECE6   \
   {  9,  3, \
      5,  4, \
      7,  3, \
      5,  6,  }
   
/*-- compare two sets of physical characteristics --*/
int 
compare_attributes(
   int 	dim1,
   int 	dim2,
   int *size1,
   int *size2,
   int *order1,
   int *order2,
   int 	datatype1,
   int 	datatype2)
{
   if (dim1 != dim2)
      FAIL("dimension mismatch\n");

   if (size1 && size2)
   {
      if (kmemcmp(size1, size2, dim1*sizeof(int)) != 0)
	 FAIL("size mismatch");
   }
   else if (!size1 || !size2)
      FAIL("size missing");

   if (order1 && order2)
   {
      if (kmemcmp(order1, order2, dim1*sizeof(int)) != 0)
	 FAIL("order mismatch");
   }
   else if (!order1 || !order2)
      FAIL("order missing");
      
   if (datatype1 != datatype2)
      FAIL("datatype mismatch");

   return TRUE;
}

/*-- initialize a position and unit in some direction --*/
void
initialize_unit(
   int  dimension,
   int  direction,
   int *size,
   int *position,
   int *unit)
{
   int i;

   for (i = 0; i < dimension; i++)
   {
      position[i] = 0;
      unit[i]     = (i == direction) ? size[i] : 1;
   }
   return;
}

/*-- compare two 3D blocks of data --*/
void
compare_data(int *data, int *cdata, int *size)
{
   int j;

#if 0
   int k;
   kprintf("\n");
   for (k = 0; k < size[1]*size[2]; k++)
   {
      for (j = 0; j < size[0]; j++)
	 kprintf(" %3d ", data[j + k*size[0]]);
      kprintf("\n");
   }
#endif

   /*-- compare the data --*/
   for (j = 0; j < size[0]*size[1]*size[2]; j++)
      if (data[j] != cdata[j])
	 FAIL("incorrect data retrieved");

   return;
}

void
main(
   int  argc,
   char *argv[])
{
   kobject i, o;

   int  odim          = O_DIM;
   int  osize[O_DIM]  = O_SIZE;
   int  oorder[O_DIM] = O_ORDER;
   int  odatatype     = KINT;

   int  piece1[12]  = PIECE1;
   int  piece2[36]  = PIECE2;
   int  piece3[36]  = PIECE3;
   int  piece4[36]  = PIECE4;
   int  piece5[175] = PIECE5;
   int  piece6[8]   = PIECE6;

   int  p1[O_DIM] = PIECE1_LOC;
   int  p2[O_DIM] = PIECE2_LOC;
   int  p3[O_DIM] = PIECE3_LOC;
   int  p4[O_DIM] = PIECE4_LOC;
   int  p5[O_DIM] = PIECE5_LOC;
   int  p6[O_DIM] = PIECE6_LOC;

   int  s1[O_DIM] = PIECE1_SIZ;
   int  s2[O_DIM] = PIECE2_SIZ;
   int  s3[O_DIM] = PIECE3_SIZ;
   int  s4[O_DIM] = PIECE4_SIZ;
   int  s5[O_DIM] = PIECE5_SIZ;
   int  s6[O_DIM] = PIECE6_SIZ;

   int  array[30]  = DATA_ARRAY;

   int  idim;
   int *isize;
   int *iorder;
   int  idatatype;

   int  interpolate = KWRAP;

   int *data = NULL;

   int  b[O_DIM] = {0, 0, 0};
   int  e[O_DIM];
   int  j;
   int  threshold = 1000;
   int  total;


   /*-- test segment with large data set support --*/
   kdas_test_mode(KDAS_TEST_PRESENTATION, TRUE);

   /*-- handle different thresholds according to command line argument --*/
   if (argc > 1)
      threshold = atoi(argv[1]);
   kdas_set_threshold(threshold);

   kprintf("basic presentation WITH large data set support\n\n");

   for (j = 0, total = 1; j < odim; j++)
      total *= osize[j];
   total *= kdata_size(odatatype);

   kprintf("\tdata size is ");
   if (total < threshold)
      kprintf("BELOW");
   else
      kprintf("ABOVE");
   kprintf(" the threshold size of %d\n\n", threshold);

   /*-- =========================================================== --*/
   /*-- phase I : output testing                                    --*/
   /*-- =========================================================== --*/
   kprintf("\tphase  I\n");
   kprintf("\t   writing all data to output...");

   /*-- write out an output object --*/
   if ((o = kdas_object_open("./test.data", KOBJ_WRITE)) == NULL)
      FAIL("unable to open output object\n");

   /*-- set physical characteristics --*/
   if (!kdas_object_set_all(o, odim, osize, oorder, odatatype, NULL))
      FAIL("unable to set output object characteristics\n");

   for (j = 0; j < odim; j++)
      e[j] = osize[j] - 1;

   if (!kdas_object_put_data(o, b, e, array))
      FAIL("unable to put the data\n");

   /*-- close the object --*/
   if (!kdas_object_close(o))
      FAIL("unable to close output object\n");

   kprintf("successful\n");


   /*-- =========================================================== --*/
   /*-- phase II : input regions                                    --*/
   /*-- =========================================================== --*/
   kprintf("\tphase II\n");
   kprintf("\t   reading out of bounds regions from data set with KWRAP ...");

   /*-- read the object back in --*/
   if ((i = kdas_object_open("./test.data", KOBJ_READ)) == NULL)
      FAIL("unable to open input object\n");

   if (!kdas_object_get_all(i, &idim, &isize, &iorder, &idatatype, NULL))
      FAIL("unable to retrieve input object's characteristics\n");

   if (!compare_attributes(odim, idim, osize, isize, oorder, iorder,
			   odatatype, idatatype))
      FAIL("physical characteristics not the same\n");

   /*-- set the physical characteristics we are interested in --*/
   if (!kdas_object_set_presentation_size(i, isize, interpolate, 99.0, 0.0))
      FAIL("unable to set presentation characteristics\n");


   /*-- first piece : totally in bounds --*/
   for (j = 0; j < odim; j++)
      e[j] = p1[j] + s1[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p1, e, NULL)) == NULL)
      FAIL("unable to get the first piece\n");

   compare_data(data, piece1, s1);
   kfree(data);


   /*-- second piece --*/
   for (j = 0; j < odim; j++)
      e[j] = p2[j] + s2[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p2, e, NULL)) == NULL)
      FAIL("unable to get the second piece\n");

   compare_data(data, piece2, s2);
   kfree(data);


   /*-- third piece --*/
   for (j = 0; j < odim; j++)
      e[j] = p3[j] + s3[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p3, e, NULL)) == NULL)
      FAIL("unable to get the third piece\n");

   compare_data(data, piece3, s3);
   kfree(data);


   /*-- fourth piece --*/
   for (j = 0; j < odim; j++)
      e[j] = p4[j] + s4[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p4, e, NULL)) == NULL)
      FAIL("unable to get the fourth piece\n");

   compare_data(data, piece4, s4);
   kfree(data);


   /*-- fifth piece --*/
   for (j = 0; j < odim; j++)
      e[j] = p5[j] + s5[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p5, e, NULL)) == NULL)
      FAIL("unable to get the fifth piece\n");

   compare_data(data, piece5, s5);
   kfree(data);


   /*-- sixth piece : totally out of bounds --*/
   for (j = 0; j < odim; j++)
      e[j] = p6[j] + s6[j] - 1;

   if ((data = (int *) kdas_object_get_data(i, p6, e, NULL)) == NULL)
      FAIL("unable to get the sixth piece\n");

   compare_data(data, piece6, s6);
   kfree(data);
   kprintf("successful\n");


   if (!kdas_object_close(i))
      FAIL("unable to close input object\n");

   kunlink("test.data");

   kexit(KEXIT_SUCCESS);
}
