#include <xvinclude.h>

/*
 *  This example creates two buttons, a "Manhatten" and a "Spline", and
 *  connects them with a connection object.  a callback is installed 
 *  that change the  connection Spline to Manhatten, and Manhatten to Spline.
 */

void ConnectionCallback PROTO((xvobject, kaddr, kaddr));
xvobject connection, spline, manhatten;

main(
   int  argc,
   char *argv[])
{
	xvobject manager;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* Initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror("example", "main", "unable to open display");
	   kexit(1);
	}

	/* create the manager backplane, (200x200) pixels */
	manager = xvw_create_manager(NULL, "manager");
	xvw_set_attributes(manager,
		XVW_WIDTH,  200,
		XVW_HEIGHT, 200,
		NULL);

	/* create manhatten button in the middle of the left side */
	manhatten = xvw_create_button(manager, "manhatten");
	xvw_set_attributes(manhatten,
		XVW_LABEL,    "Manhattan",
	        XVW_CHAR_XPOS, 2.0,
	        XVW_CHAR_YPOS, 2.0,
		NULL);

	/* create spline button in the middle of the right side */
	spline = xvw_create_button(manager, "spline");
	xvw_set_attributes(spline,
		XVW_LABEL,   "Spline",
                XVW_CHAR_XPOS, 9.0,
                XVW_CHAR_YPOS, 4.0,
		NULL);

	/* create a connection between the two buttons.  */
	connection = xvw_create_connection(manager, "connection");
	xvw_set_attributes(connection,
		XVW_CONNECTION_BEGIN, manhatten,
		XVW_CONNECTION_END,   spline,
		NULL);

	/* add callback on manhatten button */
	xvw_add_callback(manhatten, XVW_BUTTON_SELECT, ConnectionCallback, NULL);

	/* add callback on spline button */
	xvw_add_callback(spline,  XVW_BUTTON_SELECT, ConnectionCallback, NULL);

	/* display & run program */
	xvf_run_form();
}

/*
 *  here's the connection callback.  if the object passed in is the manhatten
 *  button, it changes the connection type to manhatten;  if the object passed
 *  in is the spline button, it changes the connection type to spline.
 */
void ConnectionCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	if (object == manhatten)
	   xvw_set_attribute(connection, XVW_CONNECTION_TYPE, KCONNECTION_TYPE_MANHATTAN);
	else
	   xvw_set_attribute(connection, XVW_CONNECTION_TYPE, KCONNECTION_TYPE_SPLINE);
}
