#include <xvinclude.h>

/*
 *   This example demonstrates the use of the toolbox menu, a set of
 *   menu buttons and walking menus, which list the available operators 
 *   in the Khoros system by category / subcategory / icon name.
 *
 *   An event handler is installed for creation of an operator 
 *   that is selected from the toolboxmenu.
 *
 *   Use the menubuttons labeled with the category to display menus 
 *   with subcategory listings.  Follow subcategory listings to display
 *   walking menus with the icon names of operators in that subcategory.
 *   Selecting an icon name from the walking menu will cause the cursor
 *   will change to the zen sign at which time you can move the
 *   pointer into the workspace and place a newly created glyph.
 */
static void toolboxmenu_create_cb PROTO((xvobject, kaddr, kaddr));

xvobject workspace;

main(
   int  argc,
   char *argv[])
{
	xvobject parent, toolboxmenu;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror("example", "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 * create the toolbox menu object, and install event
	 * handlers to print information on selection of an operator, and
	 * to create a glyph in the workspace.
	 */
	toolboxmenu = xvw_create_toolboxmenu(NULL, "toolboxmenu");

	xvw_add_callback(toolboxmenu, XVW_TOOLBOXMENU_CREATE, 
			 toolboxmenu_create_cb, NULL);

	workspace = xvw_create_workspace(NULL, "workspace");
	xvw_set_attributes(workspace,
			   XVW_WIDTH,  500,
			   XVW_HEIGHT, 600,
			   NULL);

	/* display & run the program */
	xvf_run_form();
}

void toolboxmenu_create_cb(
    xvobject object,
    kaddr client_data,
    kaddr call_data)

{
	xvobject glyph;
        xvw_program_info *program_info = (xvw_program_info *) call_data;

	if (program_info->type != XVL_OPERATOR)
	{
	    kinfo(KSTANDARD, "Sorry, but we are not dealing with procedures, loops, or conditionals in this example");
	    return;
	}
	glyph = xvw_create_glyph(workspace, "glyph");
        xvw_set_attributes(glyph,
                XVW_NODE_NAME,      program_info->icon_name,
                XVW_GLYPH_FORMFILE, program_info->panefile,
                XVW_GLYPH_TBNAME,   program_info->tbname,
                XVW_GLYPH_ONAME,    program_info->oname,
                NULL);

}
