#include <xvinclude.h>

/*
 *  This example creates a parent object with a button and a canvas object; 
 *  then creates a button label on the canvas to act as a sample object
 *  that can be selected and moved about.
 *
 *  The button above the canvas may be used to change
 *  the background pixmap of the canvas. (Mostly to show off all the
 *  cool background pixmaps available).
 */
static void change_background PROTO((xvobject, kaddr, kaddr));

main(
   int  argc,
   char *argv[])
{
	int gridsize = 20;
	xvobject parent, canvas, button;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "DESIGN");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/* create parent to act as backplane */
	parent = xvw_create_manager(NULL, "parent");

	/*
	 *  put a button at the top;
         *  install callback so user can change background
	 */
	button = xvw_create_button(parent, "button");
	xvw_set_attributes(button,
			   XVW_BELOW,      NULL,
			   XVW_RIGHT_OF,   NULL,
			   XVW_LEFT_OF,    NULL,
			   XVW_LABEL,     "Change background",
			   NULL);

	canvas = xvw_create_canvas(parent, "canvas");
	xvw_set_attributes(canvas,
		XVW_CANVAS_GRIDSIZE, gridsize,
		XVW_BELOW,           button,
		XVW_WIDTH,           512,
		XVW_HEIGHT,          512,
		NULL);

	xvw_add_callback(button, XVW_BUTTON_SELECT, change_background, canvas);

	/* put a button near the top of the canvas to be an object
	   that can be moved around */
	button = xvw_create_button(canvas, "button");
	xvw_set_attributes(button,
		           XVW_LABEL,       "sample object",
		           XVW_CHAR_XPOS,   10.0,
		           XVW_CHAR_YPOS,   10.0,
		           XVW_CHAR_HEIGHT, 2.0,
		           NULL);
	

	/* and we're off! */
	xvf_run_form();
}

static void change_background(
    xvobject object,
    kaddr    client_data,
    kaddr    call_data)
{
	int      indx;
 static int      count = 0;
	xvobject canvas = (xvobject) client_data;

 static char *filenames[] = {"backgrounds:hex_grid",
			     "backgrounds:brown_marble",
			     "backgrounds:black_marble",
			     "backgrounds:esher_grid",
			     "backgrounds:purple_marble",
			     "backgrounds:gravel_marble",
			     "backgrounds:purple_maze",
			     "backgrounds:wood_pulp",
			     "backgrounds:bw_marble"};
	
	indx = count % knumber(filenames);
	xvw_set_attribute(canvas, XVW_BACKGROUND_PIXMAPFILE, filenames[indx]);

	count++;
}
