#include <xvinclude.h>

/*
 *  This example tests the query dialog box produced
 *  by a call to the xvutils utility xvu_query_wait().
 *
 *  First, the query object is used to prompt the user for a float 
 *  constant (no default).  
 *
 *  Next, the query object is used to prompt 
 *  the user for a set of labels.
 *
 *  This is repeated three times.
 */

void main(
   int  argc,
   char *argv[])
{
	char  *label1       = "Please enter a floating point constant";
        char  *label2       = "Please personalize these labels";
	char  *prompts1[1]  = {"value:"};
        char  *prompts2[]   = {"label1:",
			       "label2:",
			       "label3:",
			       "label4:",
			       "label5:"};
	char  **answers;
	int   i, status, count = 0; 
	size_t num_prompts;
	float f;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "CANTATA");

        /* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "Cannot open display");
           kexit(KEXIT_FAILURE);
        }


	while (count < 3)
	{
	    /* allocate the first answers array, no default */
	    num_prompts = knumber(prompts1);
	    answers = (char **) kcalloc(num_prompts, sizeof(char *));
	    answers[0] = (char *) kcalloc(num_prompts, KLENGTH*sizeof(char));

	    status = xvu_query_wait(label1, prompts1, "ok", 
	                            answers, num_prompts, 30);
	    if (status == 0)
	       kfprintf(kstderr, "User cancelled query\n");
	    else 
	    {
	        if (answers[0] != NULL)
	        {
	            f = atof(answers[0]);
	            kfprintf(kstderr, "User entered constant %g\n", f);
	        }
	        else kfprintf(kstderr, "You left query blank but hit 'Ok'\n");
	    }
	    karray_free(answers, KSTRING, num_prompts, NULL);

            /* allocate the second answers array */
            num_prompts = knumber(prompts2);
            answers = (char **) kcalloc(num_prompts, sizeof(char *));
	
            /* initialize the answers array */
	    for (i = 0; i < num_prompts; i++)
	    {
	        answers[i] = kcalloc((size_t)1, KLENGTH*sizeof(char));
	        ksprintf(answers[i], "%s", "test");
	    }

            status = xvu_query_wait(label2, prompts2, NULL,
                                    answers, num_prompts, 50);
            if (status == 0)
               kfprintf(kstderr, "User cancelled query\n");
            else
            {
	        for (i = 0; i < num_prompts; i++)
	        {
                    if (answers[i] != NULL)
                        kfprintf(kstderr, "%s: value %s\n", 
			         prompts2[i], answers[i]);
		    else kfprintf(kstderr, "%s: value NULL\n", 
			          prompts2[i]); 
	        }
            }
	    karray_free(answers, KSTRING, num_prompts, NULL);
	    kfprintf(kstderr, "\n\n");
	    count++;
	}
}
