 /*
  * Khoros: $Id: Button.h,v 1.1 1996/05/06 21:59:50 mirage Exp $
  */
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */
 
 
/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Defines for the Button Widget
   >>>>
   >>>>     Written By: Mark Young
   >>>>
   >>>>          Date: Apr 23, 1996
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */
 
#ifndef _kwidgets_Button_h_
#define _kwidgets_Button_h_
 
        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/
 
/*------------------------------------------------------------------------
| Attribute:    XVW_BUTTON_SELECTED
|
| Description:  Whether the button is selected or not, this will give the
|		appearance that the button is depressed or not.  Which can
|		be used by certain applications to indicate 
|		be used to show optional selected buttons.
|
| Type:		int
| Default:	FALSE
| Legal Values:	TRUE/FALSE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_BUTTON_SELECTED       "buttonSelected"

/*------------------------------------------------------------------------
| Attribute:    XVW_BUTTON_SELECT
|
| Description:  A button is not much fun without a callback installed on
|		it;  it has no functionality otherwise. Use 
|		\fIxvw_add_callback()\fP to install a callback on the button
|		object which will be fired when the user clicks on the 
|		button.  When calling \fIxvw_add_callback()\fP, pass this 
|		attribute directly, as in
|               .begin code
|               xvw_add_callback(button, XVW_BUTTON_SELECT,
|                                button_cb, client_data);
|               .end code
|
| Type:		kfunc_void
| Default:	NULL
| Legal Values:	callback function, in the form:
|               .begin code
|               void callback_function
|                  xvobject object,
|                  kaddr client_data,
|                  kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_BUTTON_SELECT       "buttonSelect"

/*------------------------------------------------------------------------
| Attribute:    XVW_BUTTON_ARM
|
| Description:  The arm callback is called when the button is about to be
|		selected or deselected.  When the user performs a button
|		press the button is said to be "armed", when they release
|		the button will generate a button select callback, which
|		will be followed by a dis-arm callback.
|               xvw_add_callback(button, XVW_BUTTON_ARM,
|                                button_cb, client_data);
|               .end code
|
| Type:		kfunc_void
| Default:	NULL
| Legal Values:	callback function, in the form:
|               .begin code
|               void callback_function
|                  xvobject object,
|                  kaddr client_data,
|                  kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_BUTTON_ARM       "buttonArm"

/*------------------------------------------------------------------------
| Attribute:    XVW_LABEL
|
| Description:  The label used for the button object.  Note that this
|		is only used when the label is not NULL, otherwise if
|		the label is NULL, then no text will appear in the
|		button.
|
| Type:         char *
| Default:      NULL
| Legal Values: any printable text
| Resource:     label
------------------------------------------------------------------------*/
/* attribute defined to "label" in Label.h */

/*------------------------------------------------------------------------
| Attribute:    XVW_LABEL_JUSTIFY
|
| Description:  Specifies whether the label on the button
|               is to be left justified, middle justified, or right justified.
|		Note that this is only used when the
|               .symbol index XVW_LABEL
|               is set not set to NULL.
|
| Type:         int
| Default:      .symbol index KLABEL_JUSTIFY_CENTER
| Legal Values: .symbol index KLABEL_JUSTIFY_LEFT
|               .symbol index KLABEL_JUSTIFY_CENTER
|               .symbol index KLABEL_JUSTIFY_RIGHT
| Resource:     labelJustify
------------------------------------------------------------------------*/
/* attribute defined to "labelJustify" in Label.h */

/*------------------------------------------------------------------------
| Attribute:	XVW_PIXMAP
|
| Description:  This is the pixmap (or bitmap) that appears on the button when
|               the
|               .symbol index XVW_PIXMAP .
|		is not set to NULL.  If NULL (the default) then no pixmap
|		will appear within the button.
|		Candidates for the value of this attribute may be created 
|		with the use of \fIXCreatePixmap()\fP;  see \fIThe Xlib 
|		Reference Manual\fP by O'Reilly and Associates.  Note that 
|		this attribute is mutually exclusive with
|               .symbol index XVW_PIXMAP_FILENAME ;
|               specify one or the other, not both.
|
| Type:		Pixmap
| Default:	NULL
| Legal Values:	Valid Pixmap structure
| Resource:     pixmap
------------------------------------------------------------------------*/
/* attribute defined to "pixmap" in Pixmap.h */

/*------------------------------------------------------------------------
| Attribute:    XVW_PIXMAP_FILENAME
|
| Description:  This is the name of the file defining the pixmap (or bitmap)
|		to be displayed.
|
| Type:		char *
| Default:	NULL
| Legal Values: The full path to a valid xpm or xbm file, dfining the 
|		desired pixmap (or bitmap).  Note that the path may contain
|               $TOOLBOX.
| Resource:     N/A
------------------------------------------------------------------------*/
/* attribute defined to "pixmapFilename" in Pixmap.h */

/*------------------------------------------------------------------------
| Attribute:    XVW_PIXMAP_JUSTIFY
|
| Description:  Specifies how the pixmap is justified within the pixmap
|               object.  The pixmap can be set to be
|               left justified, middle justified, or right justified.
|               Note that the same values are used as for
|               .symbol index XVW_LABEL_JUSTIFY .
| Type:         int
| Default:      .symbol index KPIXMAP_JUSTIFY_CENTER
| Legal Values: .symbol index KPIXMAP_JUSTIFY_LEFT
|               .symbol index KPIXMAP_JUSTIFY_CENTER
|               .symbol index KPIXMAP_JUSTIFY_RIGHT
| Resource:     pixmapJustify
------------------------------------------------------------------------*/
/* attribute defined to "pixmapJustify" in Pixmap.h */

/*------------------------------------------------------------------------
| Attribute:    XVW_ATEXT
|
| Description:  The accelerator text used for the button object.  This is
|		used when there is an accelerator/action associated with a
|		button.
|
| Type:         char *
| Default:      NULL
| Legal Values: any printable text
| Resource:     atext
------------------------------------------------------------------------*/
#define XVW_ATEXT "atext"

#define XVW_BUTTON_SELECT_TYPE       "buttonSelectType"
#define XVW_BUTTON_UNSELECT_TYPE     "buttonUnselectType"
#define XVW_BUTTON_FORCE_ARMED       "buttonForceArmed"

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Button Object by.
 */
#define ButtonWidgetClass	xvwButtonWidgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration ButtonWidgetClass
 */
extern   WidgetClass     ButtonWidgetClass;


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for Button Widget
 */
xvobject xvw_create_button            PROTO((xvobject, char *));
void xvw_reverse_colors               PROTO((xvobject, int));


#endif /* _kwidgets_Button_h_ */
/* Do not add after this line */
