 /*
  * Khoros: $Id: Animate.h,v 1.5 1996/04/19 09:34:00 mirage Exp $
  */

 /*
  * $Log: Animate.h,v $
  * Revision 1.5  1996/04/19 09:34:00  mirage
  * update for cantata
  *
  * Revision 1.3  1996/02/17 20:48:57  fas
  * Copyright updated
  *
  * Revision 1.2  1995/12/06 01:30:21  mirage
  * Placing mirage under cvs
  *
  */

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>                animate image display widget (used in
   >>>>		       such programs as animate).
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Oct 05, 1992 15:00
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Animate_h_
#define _xvisual_Animate_h_


	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*------------------------------------------------------------------------
| Attribute:    XVW_ANIMATE_UPDATETIME
|
| Description:  The time interval, in seconds, which will elapse before the 
|               animation object is updated with the next image in the sequence.
|
| Type:         double 
| Default:      1.0
| Legal Values: value > 0.0 
| Resource:     animateUpdatetime
------------------------------------------------------------------------*/
#define  XVW_ANIMATE_UPDATETIME	     "animateUpdatetime"

/*------------------------------------------------------------------------
| Attribute:    XVW_ANIMATE_CONTROL
|
| Description: 	This attribute allows you to control how the sequencing is 
|              	performed when the animation is in motion.  Animation is 
|              	considered "in motion" when the animation direction is 
|	       	.symbol index KANIMATE_DIRECTION_FORWARD ,
|		or
|		.symbol index KANIMATE_DIRECTION_REVERSE .
|		The animation may be performed once completely, repeated 
|		indefinitely,  or performed in one direction then reversed and 
|		repeated in the other direction.
|		.sp
|		.symbol index KANIMATE_CONTROL_LOOP 
|		("Loop" on the "Control" selection of the menuform)
|		does a full sequence through the animation, in the current 
|		animation direction.  As soon as the sequence is finished, it 
|		is started again, so that the animation is put into a loop.  
|		This procedure will repeat until the animation is stopped.
|		.sp
|		.symbol index KANIMATE_CONTROL_SINGLE
|		("Single" on the "Control" selection of the menuform)
|		does a single complete sequence of the animation, in the 
|		current animation direction,  and then stops.
|		.sp
|		.symbol index ANIMATE_CONTROL_AUTOREVERSE
|		("Reverse" on the "Control" selection of the menuform)
|		does a single complete sequence of the animation, in the current
|		animation direction, then reverses the direction and sequences 
|		back.  This procedure will repeat until the animation is 
|		stopped.
|
| Type:         int
|
| Default:	.symbol index KANIMATE_CONTROL_LOOP
|
| Legal Values:	.symbol index KANIMATE_CONTROL_LOOP
|		.symbol index KANIMATE_CONTROL_SINGLE
|		.symbol index KANIMATE_CONTROL_AUTOREVERSE
|
| Resource:	 animateControl
------------------------------------------------------------------------*/
#define  XVW_ANIMATE_CONTROL	     "animateControl"

/*------------------------------------------------------------------------
| Attribute:    XVW_ANIMATE_DIRECTION
|
| Description: 	This attribute provides control over the direction in which 
|		the images are sequenced.  Values include:
|		.sp
|		.symbol index KANIMATE_DIRECTION_STOP
|		("Stop" on the "Direction" toggle of the menuform)
|		causes the animation to stop.
|		.sp
|		.symbol index KANIMATE_DIRECTION_PREVIOUS , 
|		("<" on the "Direction" toggle of the menuform)
|		advances a single frame in a backwards direction 
|		(frame N to N-1).
|		.sp
|		.symbol index KANIMATE_DIRECTION_NEXT ,
|		(">" on the "Direction" toggle of the menuform)
|		advances a single frame in a forward direction (from 
|		frame N to N+1).
|		.sp
|		.symbol index KANIMATE_DIRECTION_REVERSE ,
|		("<<" on the "Direction" toggle of the menuform)
|		causes the animation to sequence in a backward direction. 
|		.sp
|		.symbol index KANIMATE_DIRECTION_FORWARD ,
|		(">>" on the "Direction" toggle of the menuform)
|		causes the animation to sequence in a forward direction. 
|
| Type:         int
|
| Default:      .symbol index KANIMATE_DIRECTION_STOP
|
| Legal Values: .symbol index KANIMATE_DIRECTION_STOP
|		.symbol index KANIMATE_DIRECTION_PREVIOUS
|		.symbol index KANIMATE_DIRECTION_NEXT
|		.symbol index KANIMATE_DIRECTION_REVERSE
|		.symbol index KANIMATE_DIRECTION_FORWARD
|
| Resource:	animateDirection
------------------------------------------------------------------------*/
#define  XVW_ANIMATE_DIRECTION	     "animateDirection"

/*------------------------------------------------------------------------
| Attribute:    XVW_ANIMATE_CALLBACK
|
| Description:  If desired, a callback may be installed on the animate object 
|               that will be fired each time a new image in the sequence is 
|               displayed.  The frame number will be passed in as the 
|               call_data;  cast this parameter to an integer 
|		before using, as in:
|               .br
|               \f(CWint frame_number = (int) call_data;\fP
|
| Type:         kfunc_void
|
| Default:      NULL
|
| Legal Values:	.begin code
|		void callback_function
|		  xvobject object,
|		  kaddr client_data,
|		  kaddr call_data)
|		.end code
|
| Resource:	N/A
------------------------------------------------------------------------*/
#define  XVW_ANIMATE_CALLBACK	     "animateCallback"

/*
 *  Values for the controlling how the animation will playback its
 *  images.  We use this when we come to the either end of the image
 *  list.
 */
#define KANIMATE_CONTROL_LOOP         1
#define KANIMATE_CONTROL_SINGLE       2
#define KANIMATE_CONTROL_AUTOREVERSE  3

/*
 *  Values for the direction in which the animation will playback its
 *  images.  We use this when we come to the either end of the image
    list.
 */
#define KANIMATE_DIRECTION_STOP	     1
#define KANIMATE_DIRECTION_PREVIOUS  2
#define KANIMATE_DIRECTION_NEXT	     3
#define KANIMATE_DIRECTION_REVERSE   4
#define KANIMATE_DIRECTION_FORWARD   5

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Animate Object by.
 */
#define AnimateWidgetClass	xvwAnimateWidgetClass

	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Function definitions for Animate Visual Object
 */
xvobject xvw_create_animate  PROTO((xvobject, char *));


    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration AnimateWidgetClass
 */
extern   WidgetClass	AnimateWidgetClass;


#endif /* _xvisual_Animate_h_ */
/* Don't add after this point */
