/*
  * Khoros: $Id: Palette.h,v 1.4 1996/04/04 23:22:10 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       Palette widget.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 16, 1992 16:20
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Palette_h_
#define _xvisual_Palette_h_

    /*-------------------------------------*
    |       #defines 
    ---------------------------------------*/

/*
 *  Attribute definitions for Palette Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_CALLBACK
|
| Description:  If desired, a callback may be installed on the palette visual
|               object, which will be fired when a color cell on the palette
|               has been selected by the user.
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|               .begin code
|               void callback_function
|                   xvobject object,
|                   kaddr client_data,
|                   kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_CALLBACK	"paletteCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_TYPE
| Description:  This attribute specifies how the palette will be displayed; 
|		it may be one of:
|		.symbol index PALETTE_TYPE_COLORBAR
|		("Color Bar" on the menuform)
|		for a linearly increasing color bar.
|		.br
|		.symbol index KPALETTE_TYPE_COLORCELL
|		("Color Cell" on the menuform)
|		for a palette of rectangular color cells.
|		.br
|		or
|		.br
|		.symbol index PALETTE_TYPE_COLORWHEEL
|		("Color Wheel" on the menuform)
|		for a pie chart of colors.
|
| Type:         int
| Default:      .symbol index KPALETTE_TYPE_COLORBAR
| Legal Values: .symbol index KPALETTE_TYPE_COLORBAR
|               .symbol index KPALETTE_TYPE_COLORCELL
|               .symbol index KPALETTE_TYPE_COLORWHEEL
| Resource:     paletteType
------------------------------------------------------------------------*/
#define XVW_PALETTE_TYPE	"paletteType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_INDEXLIST
|
| Description:  This \fIread-only\fP attribute allows you to obtain the
|               palette list (ie, those data values (pixels) in the palette
|		that are currently highlighted).  By default, there is no
|		palette list (ie, no pixels in the palette are highlighted).
|
| Type:         int *
| Default:      N/A
| Legal Values: integer array of image pixel values
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_INDEXLIST         "paletteIndexlist"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_INDEXNUM
|
| Description:  This is \fIread-only\fP attribute allows you to obtain the
|               number of image data values (pixels) currently highlighted
|               in the palette object. Note that this is the size
|               of the array returned by
|               .symbol noindex XVW_PALETTE_INDEXLIST .
|
| Type:         int
| Default:      N/A
| Legal Values: size of the array specified by
|               .symbol index XVW_PALETTE_INDEXLIST
|
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_INDEXNUM          "paletteIndexnum"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_ADD
|
| Description:  Use of this \fIwrite-only\fP attribute adds the specified
|               image data value (pixel) to \fIpalette list\fP (the list
|               of pixels highlighted in the palette).
|
| Type:         int
| Default:      N/A
| Legal Values: valid pixel value of the displayed data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_ADD               "paletteAdd"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_DELETE
|
| Description:  Use of this \fIwrite-only\fP attribute deletes
|               the image data value (pixel) from the \fIpalette list\fP
|               (the list of pixels highlighted in the palette).
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: valid pixel value of the displayed data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_DELETE            "paletteDelete"

/*------------------------------------------------------------------------
| Attribute:    XVW_PALETTE_CLEAR
|
| Description:  This \fIaction\fP attribute clears the \fIpalette list\fP
|               so that no pixels are highlighted in the palette.
|
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PALETTE_CLEAR             "paletteClear"


/*
 *  Palette type for the controlling how the palette should be displayed.
 *  The choices are rectangular, gridded, spherical...
 */
#define KPALETTE_TYPE_COLORBAR	 1
#define KPALETTE_TYPE_COLORCELL	 2
#define KPALETTE_TYPE_COLORWHEEL 3

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Palette Object by.
 */
#define PaletteWidgetClass	xvwPaletteWidgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration for PaletteWidgetClass
 */
extern   WidgetClass     PaletteWidgetClass;


    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Function definitions for Palette Visual Object
 */
xvobject xvw_create_palette  PROTO((xvobject, char *));


#endif /* _xvisual_Palette_h_ */
/* Don't add after this point */
