/*
  * Khoros: $Id: Plot2D.h,v 1.9 1996/04/04 23:22:15 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       graphical plot2d object.
   >>>>
   >>>>    Written By: John Salas and Mark Young
   >>>>
   >>>>          Date: Aug 05, 1992 14:27
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Plot2D_h_
#define _xvisual_Plot2D_h_

    /*-------------------------------------*
    |       #defines 
    ---------------------------------------*/

/*
 *  Attribute definitions for Plot 2D Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_PLOTOBJ
|
| Description:  The data object containing the 2D plot data to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PLOT2D_PLOTFILE ;
|               use one or the other, not both.
|
| Type:         kobject
| Default:      NULL
| Legal Values: any valid data object
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_PLOTOBJ		"plot2DPlotobj"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_PLOTFILE
|
| Description:  The file containing the 2D plot data to be displayed.
|               Note that this attribute is mutually exclusive with
|               .symbol index XVW_PLOT2D_PLOTOBJ ;
|               use one or the other, not both.
|
| Type:         char *
| Default:      NULL
| Legal Values: any valid input file
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_PLOTFILE		"plot2DPlotfile"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_COLOR_ORIGINATION
|
| Description:  This attribute is only used when the plot data is provided 
|		via a kobject specified using the
|               .symbol index XVW_PLOT2D_PLOTOBJ 
|		or
|		.symbol index XVW_PLOT2D_PLOTFILE
|		attributes.  It dictates whether the color of the plot is to 
|		be specified by the plot foreground color or by the plot data 
|		itself.  
|	
|		If the entire plot is to appear in a particular
|		color, set this attribute to 
|		.symbol index KPLOT2D_COLOR_FROM_FOREGROUND;
|		then, specify the desired color using the 
|		.symbol index XVW_FOREGROUND
|		or
|		.symbol index XVW_FOREGROUND_COLOR 
|		attributes.
|
|		If each point in the data set is to have its color specified
|		individually, set this attribute to
|		.symbol index KPLOT2D_COLOR_FROM_DATA .
|		The color of each point in the plot will be dictated
|		by the values stored in the kobject; see ??? for details
|		on how the color is interpreted from the data of the kobject.
|
| Type:         int
| Default:      .symbol index KPLOT2D_COLOR_FROM_FOREGROUND
| Legal Values: .symbol index KPLOT2D_COLOR_FROM_FOREGROUND
|		or
|		symbol index KPLOT2D_COLOR_FROM_DATA
| Resource:     plot2DColorOrigination
------------------------------------------------------------------------*/
#define XVW_PLOT2D_COLOR_ORIGINATION         "plot2DColorOrigination"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_DATA_MAX_X
|
| Description:  This \fIread-only\fP attribute returns the maximum of the 
|               X coordinates in the 2D plot data currently in the Coord array 
|               specified with 
|               .symbol index XVW_PLOT2D_POINTS .
|
| Type:         double
| Default:      1.0 
| Legal Values: the maximum X world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DATA_MAX_X		"plot2DDataMaxX"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_DATA_MAX_Y
|
| Description:  This \fIread-only\fP attribute returns the maximum of the
|               Y coordinates in the 2D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT2D_POINTS .
|
| Type:         double
| Default:      1.0
| Legal Values: the maximum Y world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DATA_MAX_Y		"plot2DDataMaxY"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_DATA_MIN_X
|
| Description:  This \fIread-only\fP attribute returns the minimum of the
|               X coordinates in the 2D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT2D_POINTS .
|
| Type:         double
| Default:      0.0
| Legal Values: the minimum X world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DATA_MIN_X		"plot2DDataMinX"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_DATA_MIN_Y
|
| Description:  This \fIread-only\fP attribute returns the minimum of the
|               Y coordinates in the 2D plot data currently in the Coord array
|               specified with
|               .symbol index XVW_PLOT2D_POINTS .
|
| Type:         double
| Default:      0.0
| Legal Values: the minimum Y world coordinate value (read only)
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DATA_MIN_Y		"plot2DDataMinY"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_PLOTTYPE
| Description:  This attribute indicates how the plot is to be displayed. 
|               Plot types include line plots, discrete plots, bar graphs,
|		polymarker plots (also known as scatter plots), and linemarker
|		plots (a combination of the line plot and the scatter plot).
| Type:         int
| Default:      .symbol index KPLOT2D_LINEPLOT
| Legal Values  .symbol index KPLOT2D_LINEPLOT
|               .symbol index KPLOT2D_BARGRAPH
|               .symbol index KPLOT2D_DISCRETE
|               .symbol index KPLOT2D_POLYMARKER
|               .symbol index KPLOT2D_LINEMARKER
| Resource:     plot2DPlotType
------------------------------------------------------------------------*/
#define XVW_PLOT2D_PLOTTYPE		"plot2DPlotType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_PLOTSIZE
|
| Description:  This is the number of data points contained in the Coord 
|               array specified by the attribute
|               .symbol noindex XVW_PLOT2D_POINTS .
|               Note that you \fImust\fP use this attribute to specify the 
|               number of points prior to specifying the data points with
|               .symbol noindex XVW_PLOT2D_POINTS .
|
| Type:         int
| Default:      0
| Legal Values: values > 0
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_PLOTSIZE		"plot2DPlotsize"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_POINTS
|
| Description:  This is the array of coordinates defining the 2D data points 
|               in the plot. It is an array of type Coord, where the Coord 
|               structure is defined as:
|               .begin code
|               typedef  struct
|               {
|                   Real  x, y, z;
|                   Index d;
|               }  Coord;
|               .end code
|               Note that the
|               .symbol index XVW_PLOT2D_PLOTSIZE
|               attribute \fImust\fP be set to the number of points in the
|               Coord array \fIprior\fP to setting the
|               .symbol index XVW_PLOT2D_POINTS
|               attribute.
|               .br
|               The \fIz\fP value is ignored for all 2D plot types.  
|               .br
|		If the color of a plot point is to be dictated by either the
|               .symbol index XVW_FOREGROUND
|               or the
|               .symbol index XVW_FOREGROUND_COLOR
|		attributes, 
|               set the \fId\fP value to
|		.symbol index KGRAPHICS_UNINITIALIZED .
|		
|		Alternatively, you may set the \fId\fP value to the pixel
|		value which specifies the desired color of the plot point.
|		See ??? for more details on specifying plot colors.
|
| Type:         Coord *
| Default:      NULL
| Legal Values: array of Coords defining data points for 2D plot
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_POINTS		"plot2DPoints"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_X_ORIENTATION
|
| Description:  This attribute is only used when the plot data is provided
|               via a kobject specified using the
|               .symbol index XVW_PLOT2D_PLOTOBJ
|               or
|               .symbol index XVW_PLOT2D_PLOTFILE
|               attributes;  furthermore, it only applies when the plot data
|		is stored in the value segment of the data object.
|
|		It dictates how the 2D plot data is to be extracted from the 
|		value segment of the data object.  The 2D plot data is always
|		extracted as a single line, but that line may be oriented
|		down width, height, depth, time, or elements.  
| Type:         int
| Default:      .symbol index KWIDTH
| Legal Values: .symbol index KWIDTH
|		.symbol index KHEIGHT
|		.symbol index KDEPTH
|		.symbol index KTIME
|		.symbol index KELEMENTS
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_X_ORIENTATION            "plot2DXOrientation"

/*
 * this is a private attribute for now; name might change, values might change
 */
#define XVW_PLOT2D_X_EXTRACT_SIZE            "plot2DXExtractType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_WIDTH_OFFSET
|
| Description:  This attribute in conjunction with the
|		.symbol index XVW_PLOT2D_X_ORIENTATION
|		attribute;  when 
|		.symbol index XVW_PLOT2D_X_ORIENTATION
|		is \fInot\fP set to 
|		.symbol index KWIDTH ,
|		it is used to specify the width offset at which the
|		line of plot data is to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < width of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_WIDTH_OFFSET            "plot2DWidthOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_HEIGHT_OFFSET
|
| Description:  This attribute in conjunction with the
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               attribute;  when 
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               is \fInot\fP set to 
|               .symbol index KHEIGHT ,
|               it is used to specify the height offset at which the
|               line of plot data is to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < height of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_HEIGHT_OFFSET            "plot2DHeightOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_DEPTH_OFFSET
|
| Description:  This attribute in conjunction with the
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               attribute;  when 
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               is \fInot\fP set to 
|               .symbol index KDEPTH ,
|               it is used to specify the depth offset at which the
|               line of plot data is to be extracted.
| Type:         int
| Default:      0
| Legal Values: 0 < value < depth of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DEPTH_OFFSET            "plot2DDepthOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_TIME_OFFSET
|
| Description:  This attribute in conjunction with the
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               attribute;  when 
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               is \fInot\fP set to 
|               .symbol index KTIME ,
|               it is used to specify the time offset at which the
|               line of plot data is to be extracted.
| Type:         int
| Default:      0
| Legal Values: value < time size of data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_TIME_OFFSET            "plot2DTimeOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_ELEMENTS_OFFSET
|
| Description:  This attribute in conjunction with the
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               attribute;  when 
|               .symbol index XVW_PLOT2D_X_ORIENTATION
|               is \fInot\fP set to 
|               .symbol index KELEMENTS ,
|               it is used to specify the elements offset at which the
|               line of plot data is to be extracted.
| Type:         int
| Default:      0
| Legal Values: value < number of elements in data object value segment
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_ELEMENTS_OFFSET            "plot2DElementsOffset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_ROI
|
| Description:  This is a read-only action attribute; that is, it can only 
|               be used with \fIxvw_get_attribute(s)()\fP.
|               It allows the user to interactively specify a desired region
|               of interest in the 2D plot object. The plot points that 
|		are inside the specified region of interest will be returned
|		in a coordinate array.
|
| Type:         Coord *
| Default:      NULL
| Legal Values: Those points that fall inside the specified ROI
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_ROI                   "plot2DRoi"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_ROI_SHAPE
|
| Description:  When the
|               .symbol index XVW_PLOT2D_ROI
|               attribute is used to extract the plot points that fall 
|		within a particular region of interest, this attribute 
|		specifies the shape of the ROI that will be interactively
|		drawn by the user.  Supported ROI shapes include rectangle, 
|		polygon, circle, ellipse, line, and freehand.
|
| Type:         int
| Default:      .symbol index KPLOT2D_ROI_RECTANGLE
| Legal Values: .symbol index KPLOT2D_ROI_RECTANGLE
|               .symbol index KPLOT2D_ROI_POLYLINE
|               .symbol index KPLOT2D_ROI_CIRCLE
|               .symbol index KPLOT2D_ROI_ELLIPSE
|               .symbol index KPLOT2D_ROI_LINE
|               .symbol index KPLOT2D_ROI_FREEHAND
|
| Resource:     plot2DRoiShape
------------------------------------------------------------------------*/
#define XVW_PLOT2D_ROI_SHAPE                   "plot2DRoiShape"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_ROI_POLICY
|
| Description:  When the
|               .symbol index XVW_PLOT2D_ROI
|               attribute is used to extract a region of interest,
|               this attribute specifies whether the ROI is defined by the
|               region inside the shape, by the region outside the shape,
|               or by the outline of the shape itself.
|
| Type:         int
| Default:      KPLOT2D_ROI_INSIDE
| Legal Values: .symbol index KPLOT2D_ROI_INSIDE
|               .symbol index KPLOT2D_ROI_OUTLINE
|               .symbol index KPLOT2D_ROI_OUTSIDE
| Resource:     imageRoiPolicy
------------------------------------------------------------------------*/
#define XVW_PLOT2D_ROI_POLICY                  "plot2DRoiPolicy"

/*------------------------------------------------------------------------
| Attribute:    PLOT2D_DELETE_PTS_INTERACTIVELY
|
| Description:  This \fIaction attribute\fP allows the user to rubberband
|		about desired points in the plot and deletes them automatically.
|		As an action attribute, this attribute can only be used with
|		\fIxvw_set_attribute(s)()\fP, not with 
|		\fIxvw_get_attribute(s)()\fP.
|
|		When this attribute is set, the user is expected to move
|		the pointer into the 2D plot object; the cursor will change
|		to the hand, indicating that the user should rubber-band about
|		the points that they wish to be deleted from the plot.
|		The ROI shape that is used for rubberbanding about the points
|		is specified using the
|               .symbol index XVW_PLOT2D_ROI_SHAPE
|		attribute.  
|
|		After the user has completed the rubber-banding
|		procedure, the points inside the ROI specified are automatically
|		deleted from the plot data.  Thus, setting this action attribute
|		has the side effect of modifying the data being used by the
|		2D plot object;  if the data has been specified using the
|		.symbol index XVW_PLOT2D_POINTS
|		attribute, the pointer to the Coord array should be 
|		re-obtained with \fIxvw_get_attribute(s)()\fP before it is
|		used again by the application.
|
| Type:         int
| Default:      N/A 
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_DELETE_PTS_INTERACTIVELY	"plot2DDeletePtsInteractively"

/*------------------------------------------------------------------------
| Attribute:    XVW_PLOT2D_HIGHLIGHTCOLOR
|
| Description:  When the
|               .symbol index PLOT2D_COLOR_PTS_INTERACTIVELY
|               attribute is used to interactively change the color of a 
|		region in the plot within a particular region of interest, 
|		this attribute specifies the color in which that portion of
|		the plot will be displayed.  The color is specified by name. 
|
| Type:         char *
| Default:      "red"
| Legal Values: any valid color name: see /usr/lib/X11/rgb.txt or Section
|               7.1.1 of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:     plot2DHighlightColor
------------------------------------------------------------------------*/
#define XVW_PLOT2D_HIGHLIGHTCOLOR    "plot2DHighlightColor"


/*------------------------------------------------------------------------
| Attribute:    PLOT2D_COLOR_PTS_INTERACTIVELY
|
| Description:  This \fIaction attribute\fP allows the user to rubberband
|               about desired points in the plot and colors them automatically,
|		in the color specified by 
|		.symbol index XVW_PLOT2D_HIGHLIGHTCOLOR .
|               As an action attribute, this attribute can only be used with
|               \fIxvw_set_attribute(s)()\fP, not with
|               \fIxvw_get_attribute(s)()\fP.
|
|               When this attribute is set, the user is expected to move
|               the pointer into the 2D plot object; the cursor will change
|               to the hand, indicating that the user should rubber-band about
|               the points that they wish to be colored differently in the plot.
|               The ROI shape that is used for rubberbanding about the points
|               is specified using the
|               .symbol index XVW_PLOT2D_ROI_SHAPE
|               attribute.
|
|               After the user has completed the rubber-banding
|               procedure, the points inside the ROI specified are automatically
|               colored in the highlight color.  The points in the plot that
|		are ouside the rubberbanded area will remain in the original 
|		color. 
|
| Type:         int
| Default:      N/A
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PLOT2D_COLOR_PTS_INTERACTIVELY     "plot2DColorPtsInteractively"

/*
 * Allowed values for XVW_PLOT2D_ROI_SHAPE
 */
#define KPLOT2D_ROI_RECTANGLE  KROI_RECTANGLE
#define KPLOT2D_ROI_POLYLINE   KROI_POLYLINE
#define KPLOT2D_ROI_CIRCLE     KROI_CIRCLE
#define KPLOT2D_ROI_ELLIPSE    KROI_ELLIPSE
#define KPLOT2D_ROI_LINE       KROI_LINE
#define KPLOT2D_ROI_FREEHAND   KROI_FREEHAND

/*
 * Allowed values for XVW_PLOT2D_ROI_POLICY
 */
#define KPLOT2D_ROI_INSIDE   KROI_INSIDE
#define KPLOT2D_ROI_OUTLINE  KROI_OUTLINE
#define KPLOT2D_ROI_OUTSIDE  KROI_OUTSIDE

/*
 *  Value definitions for Plot 2D Visual Object
 */

/* Values allowed for XVW_PLOT2D_COLOR_ORIGINATION */
#define KPLOT2D_COLOR_FROM_FOREGROUND 1
#define KPLOT2D_COLOR_FROM_DATA       2

/* 2D Plot types used by graphics library */
#define KPLOT2D_LINEPLOT           1
#define KPLOT2D_DISCRETE           2
#define KPLOT2D_BARGRAPH           3
#define KPLOT2D_POLYMARKER         4
#define KPLOT2D_LINEMARKER         5
#define KPLOT2D_COLORLINE          7

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Plot2D Object by.
 */
#define Plot2DGadgetClass	xvwPlot2DGadgetClass

    /*-------------------------------------*
    |       global variable declarations
    ---------------------------------------*/

/*
 *  XToolkit Widget Declaration for Plot2DGadgetClass
 */
extern   WidgetClass     Plot2DGadgetClass;


    /*-------------------------------------*
    |       routine definitions
    ---------------------------------------*/

/*
 *  Function Definitions Plot 2D Visual Object
 */
xvobject xvw_create_plot2d  PROTO((xvobject, char *));


#endif /* _xvisual_Plot2D_h_ */
/* Don't add after this point */
