/*
  * Khoros: $Id: Threshold.h,v 1.4 1996/04/04 23:22:35 fas Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: #defines and #includes for creating a
   >>>>		       Threshold widget.
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 16, 1992 16:20
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvisual_Threshold_h_
#define _xvisual_Threshold_h_

	/*-------------------------------------*
	|       #defines 
	---------------------------------------*/

/*
 *  Attribute definitions for Threshold Visual Object
 */

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_SHOW_PALETTE
| Description:  This attribute causes the palette of linear colormap values
|               to be displayed above the lower/upper/range scrollbars.  
|               Set to FALSE if the palette is not to be shown.
| Type:         int
| Default:      TRUE
| Legal Values: TRUE/FALSE
| Resource:     thresholdShowPalette
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_SHOW_PALETTE  "thresholdShowPalette"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_PALETTE_OBJECT
| Description:  This is the palette component of the threshold object.
|               The palette component is where the actual pixels are
|               displayed and reflect the thresholding process.
| Type:         xvobject
| Default:      NULL
| Legal Values: valid xvobject
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_PALETTE_OBJECT "thresholdPaletteObject"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_POLICY
| Description:  A threshold visual object provides a mechanism with which you 
|               may perform various types of thresholding on an image.   When 
|		.symbol index XVW_THRESHOLD_POLICY
|		is set to
|		.symbol index KTHRESHOLD_POLICY_THRESH ,
|		the threshold visual object will perform pixel thresholding 
|		on the image.  When 
|		.symbol index XVW_THRESHOLD_POLICY
|		is set to
|		.symbol index KTHRESHOLD_POLICY_CLIP ,
|		the threshold visual object will perform pixel clipping on 
|		the image. When
|               .symbol index XVW_THRESHOLD_POLICY
|               is set to
|               .symbol index KTHRESHOLD_POLICY_WINDOWED,
|		a windowed thresholding algorithm is used to increase the
|		contrast of the image.
|		.begin tagged
|		.item "Pixel Clipping"
|		Pixel clipping is used to mask out certain parts of the image 
|		so that only a desired range of pixels is observed.  Pixel 
|	 	clipping only allows a specified region of the original pixel 
|		values to be displayed; all values outside the specified region
|		will be set to a predetermined pixel value.  The user 
|		interactively specifies the region of pixel values that will 
|		retain their original values. The pixel value to be used for 
|		the values outside the specified region may be set using the
|		.symbol index XVW_THRESHOLD_CLIP_PIXELVAL
|		attribute.
|		.item "Pixel Thresholding"
|		Pixel thresholding is used to set all values in the image to
|		one of two values; all values inside the specified range will
|		be displayed using a non-zero pixel value, while all values
|		outside the range will be displayed in black.The user
|               interactively specifies the region of pixel values that will
|               be displayed in the non-zero pixel value.  The non-zero pixel
|		value to be used may be set using the
|		.symbol index XVW_THRESHOLD_PIXELVAL
|		attribute.
|		.item "Window Contrast Enhancement"
|		Windowed thresholding is used to increase the contrast of
|		the image in a particular pixel range, while masking out
|		the other pixels in the image.  The user interactively 
|		specifies the region of pixel values that will be displayed;
|		all pixels outside the range will appear in black.  The pixel
|		values inside the specified range will have their pixel values
|		normalized to cover the entire range of pixels in the image;
|		thus, this operation can be thought of as a histogram stretch.
|		.end tagged
| Type:         int
| Default:      .symbol index KTHRESHOLD_POLICY_CLIP
| Legal Values: .symbol index KTHRESHOLD_POLICY_THRESH
|               .symbol index KTHRESHOLD_POLICY_CLIP
|               .symbol index KTHRESHOLD_POLICY_WINDOWED
| Resource:     thresholdPolicy
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_POLICY  	    "thresholdPolicy"


/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_LOWERVAL
| Description:  The lower value of the thresholding (or clipping) region.
| Type:         double
| Default:      minval
| Legal Values: minval - maxval
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_LOWERVAL      "thresholdLowerval"

/*
 * range integer object only exists for the user's ease in changing
 * the upper and lower values of the threshold region; its own integer
 * value is not actually used.
 */
#define XVW_THRESHOLD_RANGEVAL      "thresholdRangeval"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_UPPERVAL
| Description:  The upper value of the thresholding (or clipping) region
| Type:         double 
| Default:      maxval 
| Legal Values: minval - maxval 
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_UPPERVAL      "thresholdUpperval"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_RESET
| Description:  This \fIaction attribute\fP resets the lower and upper values
|		of the threshold range to the lower and upper values of the
|		data object.  This has the effect of undoing any prior 
|		thresholding operations, and setting the data object back
|		to it's "un-thresholded" state. 
| Type:         int
| Default:      N/A (action attribute)
| Legal Values: TRUE
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_RESET      "thresholdReset"


/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_PIXELVAL
| Description:  Used only when the 
|		.symbol index XVW_THRESHOLD_POLICY
|		attribute is set to
|		.symbol index KTHRESHOLD_POLICY_THRESH ,
|		this is the pixel value assigned to all the pixels 
|		within the range defined by the scroll bars of the threshold
|		object. All pixels outside the specified range will appear 
|		black.  The opposite happens when 
|		.symbol index XVW_THRESHOLD_INVERT
|		is set to TRUE (ie, the pixel value is used for the pixels
|		outside the range, while all pixels inside the range are black).
| Type:         int
| Default:      black
| Legal Values: any legal Pixel value
| Resource:     thresholdPixelVal
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_PIXELVAL      "thresholdPixelVal"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_CLIP_PIXELVAL
| Description:  Used only when the
|               .symbol index XVW_THRESHOLD_POLICY
|               attribute is set to
|		.symbol index KTHRESHOLD_POLICY_CLIP ,
|		this is the pixel value used in the clipped, or masked-out,
|		regions of the image.  All pixels inside the specified range
|		will appear in their original pixel values.  The opposite
|		happens when
|               .symbol index XVW_THRESHOLD_INVERT
|               is set to TRUE (ie, the pixel value is used for the pixels
|               insie the range, while all pixels outside the range appear
|		in their original pixel values).
| Type:         Pixel
| Default:      white
| Legal Values: any legal Pixel value
| Resource:     thresholdClipPixelVal
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_CLIP_PIXELVAL      "thresholdClipPixelVal"


/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_INVERT
| Description:  When the threshold object is set to "non-inverted" (ie, if
|		.symbol index XVW_THRESHOLD_THRES_INVERT
|		is set to FALSE), the pixel thresholding, pixel clipping,
|		and windowed thresholding operations will work normally.
|		When the threshold object is set to "inverted" (ie, if
|		.symbol index XVW_THRESHOLD_THRES_INVERT
|		is set to TRUE),  the pixel thresholding, pixel clipping,
|		and windowed thresholding operations will have the opposite
|		effect from what they do normally.  For pixel thresholding,
|		the values outside the range (rather than inside) will be
|		displayed in the pixel value specified; for pixel clipping, 
|		the values outside the range (rather than inside) will appear 
|		in their original values; for windowed thresholding, the 
|		values outside the range (rather than inside) will have their 
|		values normalized.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:     thresholdInvert
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_INVERT  "thresholdInvert"

/*------------------------------------------------------------------------
| Attribute:    XVW_THRESHOLD_CALLBACK
|
| Description:  If desired, a callback may be installed on the threshold
|               object that will be fired each time the user employs one of
|               the scrollbars to change the lower or upper bound of the
|               thresholding (or clipping) region.  An 
|               \fIxvw_threshold_struct\fP will be passed in as the call_data; 
|               cast this parameter accordingly before using, as in:
|               .begin code
|               xvw_threshold_struct *thresh_struct;
|               thresh_struct = (xvw_threshold_struct *) call_data;\fP
|               .end code
|               The \fIxvw_threshold_struct\fP is defined in
|               $ENVISION/include/xvisual/Threshold.h as follows:
|               .begin code
|               typedef struct
|               {
|                   xvobject intobj;
|                   int      type;
|                   int      value;
|               } xvw_threshold_struct;
|               .end code
|
| Type:         kfunc_void
| Default:      NULL
| Legal Values: callback function, in the form:
|               .begin code
|               void callback_function
|                   xvobject object,
|                   kaddr client_data,
|                   kaddr call_data)
|               .end code
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_THRESHOLD_CALLBACK	    "thresholdCallback"

/*
 *  Value definitions for Threshold Visual Object
 */
#define KTHRESHOLD_POLICY_CLIP	   1
#define KTHRESHOLD_POLICY_THRESH   2
#define KTHRESHOLD_POLICY_WINDOWED 3

/*
 *  Value definitions for Threshold Visual Object
 */
#define KTHRESHOLD_LOWER	0
#define KTHRESHOLD_UPPER	1
#define KTHRESHOLD_RANGE	2

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Threshold Object by.
 */
#define ThresholdWidgetClass	xvwThresholdWidgetClass
 
        /*-------------------------------------*
        |       typedefs
        ---------------------------------------*/

/*
 *  Callback structure for the XVW_THRESHOLD_CALLBACK.  The calldata
 *  consists of:
 *  1)  the integer object that allows the user to change the threshold value, 
 *  2)  which value the integer object is changing: 
 *      the upper value, the lower value, or the range, 
 *  3)  the current value of the integer object.
 */
typedef struct
{
	xvobject intobj;
	int	 type;
	int      value;
} xvw_threshold_struct;

	/*-------------------------------------*
	|       global variable declarations
	---------------------------------------*/

/*
 *  XToolkit Widget Declaration for ThresholdWidgetClass
 */
extern   WidgetClass     ThresholdWidgetClass;


	/*-------------------------------------*
	|       routine definitions
	---------------------------------------*/

/*
 *  Function definitions for Threshold Visual Object
 */
xvobject xvw_create_threshold  PROTO((xvobject, char *));


#endif /* _xvisual_Threshold_h_ */
/* Don't add after this point */
