 /*
  * Khoros: $Id: Port.h,v 1.11 1996/07/02 04:39:55 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>       Purpose: Definitions for Khoros Port Object
   >>>>
   >>>>    Written By: Mark Young
   >>>>
   >>>>          Date: Nov 22, 1993
   >>>>
   >>>> Modifications:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#ifndef _xvlang_Port_h_
#define _xvlang_Port_h_

        /*-------------------------------------*
        |       #defines
        ---------------------------------------*/


/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_DAV
| Description:  Whether the port currently has data available or not.  Data
|		available is used to indicate whether the port is referencing
|		valid data or not.  This attribute is used in conjunction with
|		the GlyphClass to coordinate the scheduling and execution of
|		glyphs.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_DAV		   "portDav"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_TYPE
| Description:  The type of port.  The port type is used to indicate how
|		the information of data available should be propagated.
|		If the port is of type 
|		.symbol index KPORT_DATA_OUTPUT ,
|		then when data
|		available, 
|		.symbol index XVW_PORT_DAV ,
|		is set then the value will propagated
|		to the other connections.
| Type:         int
| Default:      .symbol index KPORT_UNKNOWN
| Legal Values: .symbol index KPORT_UNKNOWN
|		.symbol index KPORT_DATAFLOW
|		.symbol index KPORT_CONTROLFLOW
|		.symbol index KPORT_PARAMETER
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_TYPE		   "portType"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_DIRECTION
| Description:  The direction of the port.  The port direction is used to
|		indicate how the information of data available should be
|		propagated.
|		If the port is of type 
|		.symbol index KPORT_OUTPUT ,
|		then when data
|		available, 
|		.symbol index XVW_PORT_DAV ,
|		is set then the value will propagated
|		to the other connections.
| Type:         int
| Default:      .symbol index KPORT_UNKNOWN
| Legal Values: .symbol index KPORT_UNKNOWN
|		.symbol index KPORT_INPUT
|		.symbol index KPORT_OUTPUT
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_DIRECTION	   "portDirection"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_SELECTION
| Description:  The UIS selection that is used to represent the port.  The
|		selection needs to be either of type
|		.symbol index KUIS_INPUTFILE
|		or
|		.symbol index KUIS_OUTPUTFILE .
| Type:         kformstruct *
| Default:      NULL
| Legal Values: a legal
|		.symbol index KUIS_INPUTFILE
|		or
|		.symbol index KUIS_OUTPUTFILE
|		selection
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_SELECTION	   "portSelection"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_SELECTED
| Description:  Whether the port is currently optionally selected or not.
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_SELECTED	   "portSelected"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_RESET
| Description:  Reset the Port
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_RESET	   "portReset"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_MODIFIED
| Description:  Reset the Port
| Type:         int
| Default:      FALSE
| Legal Values: TRUE/FALSE
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_MODIFIED	   "portModified"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_CONNECTION_PARENT
| Description:  The parent in which connections between ports should be
|               created.  If NULL, then use the port's parent object.
| Type:         xvobject
| Default:      NULL
| Legal Values: any xvobject that is subclassed off the ManagerClass
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PORT_CONNECTION_PARENT "portConnectionParent"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_FILENAME
| Description:  Filename associated with the port.  Depending on whether
|	        the filename is an input or output port, the filename
|		will be propagated to other input ports.
| Type:         char *
| Default:      NULL
| Legal Values: any legal path and filename
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_FILENAME		"portFilename"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_REMOTE_LOCATION
| Description:  Where the port is suppose to be executed if remote enable
|               is set to TRUE.
| Type:         char *
| Default:      localhost
| Legal Values: any legal host
| Resource:     portRemoteLocation
------------------------------------------------------------------------*/
#define XVW_PORT_REMOTE_LOCATION "portRemoteLocation"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_DAV_CALLBACK
| Description:  Callback routines which will be notified when a port's data
|		available state changes.
| Type:         kfunc_void
| Default:      NULL
| Legal Values: any legal callback routine
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_DAV_CALLBACK	  "portDavCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_SELECTED_CALLBACK
| Description:  Callback routines which will be notified when a port changes
|               whether it is selected or not.
| Type:         kfunc_void
| Default:      NULL
| Legal Values: any legal callback routine
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_SELECTED_CALLBACK "portSelectedCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_MODIFIED_CALLBACK
| Description:  Callback routines which will be notified when a port's modified
|		state changes.
| Type:         kfunc_void
| Default:      NULL
| Legal Values: any legal callback routine
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_MODIFIED_CALLBACK	  "portModifiedCallback"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_FILENAME_CALLBACK
| Description:  Callback routines which will be notified when a port's filename
|               changes.
| Type:         kfunc_void
| Default:      NULL
| Legal Values: any legal callback routine
| Resource:	N/A
------------------------------------------------------------------------*/
#define XVW_PORT_FILENAME_CALLBACK "portFilenameCallback"


/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_DAV_PIXEL
| Description:	The pixmap that used to visually indicate that a port's
|		data available is present.
| Type:         unsigned long
| Default:      pixel associated with "green" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portDavPixel
------------------------------------------------------------------------*/
#define XVW_PORT_DAV_PIXEL        "portDavPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_OPTIONAL_DATA_PIXEL
| Description:  The pixmap that is used to visually indicate that a data port
|		is optionally not selected.
| Type:         unsigned long
| Default:      pixel associated with "blue" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portOptionalDataPixel
------------------------------------------------------------------------*/
#define XVW_PORT_OPTIONAL_DATA_PIXEL   "portOptionalDataPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_REQUIRED_DATA_PIXEL
| Description:  The pixmap that is used to visually indicate that a data port
|               is selected.
| Type:         unsigned long
| Default:      pixel associated with "yellow" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portRequiredDataPixel
------------------------------------------------------------------------*/
#define XVW_PORT_REQUIRED_DATA_PIXEL   "portRequiredDataPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_CONTROL_PIXEL
| Description:  The pixmap that is used to visually indicate a port
|		is control port type.
| Type:         unsigned long
| Default:      pixel associated with "pink" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portControlPixel
------------------------------------------------------------------------*/
#define XVW_PORT_CONTROL_PIXEL   "portControlPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_PARAMETER_PIXEL
| Description:  The pixmap that is used to visually indicate a port
|		is parameter port type.
| Type:         unsigned long
| Default:      pixel associated with "white" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portParameterPixel
------------------------------------------------------------------------*/
#define XVW_PORT_PARAMETER_PIXEL   "portParameterPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_SELECTED_PIXEL
| Description:  The pixmap that is used to visually indicate that a port
|		is selected for connecting with other ports.
| Type:         unsigned long
| Default:      pixel associated with "black" 
| Legal Values: any valid pixel value: see Section 7.3 and 7.4
|               of \fIThe XLib Programming Manual\fP by Adrian Nye
| Resource:	portSelectedPixel
------------------------------------------------------------------------*/
#define XVW_PORT_SELECTED_PIXEL   "portSelectedPixel"

/*------------------------------------------------------------------------
| Attribute:    XVW_PORT_DELETE_CONNECTION
| Description:  This \fIaction attribute\fP deletes the connection between
|               the output port for which it is set, and the input port which
|               is specified as the resource value.
| Type:         xvobject
| Default:      N/A
| Legal Values: only input port objects to which the output port object
|               is currently connected.
| Resource:     N/A
------------------------------------------------------------------------*/
#define XVW_PORT_DELETE_CONNECTION        "portDeleteConnection"

/* Port Type */
#define KPORT_UNKNOWN		0
#define KPORT_DATAFLOW		1
#define KPORT_CONTROLFLOW	2
#define KPORT_PARAMETER		3

/* Port Direction */
#define KPORT_INPUT		1
#define KPORT_OUTPUT		2

/*
 *  Define the Class name, by which people should be externally referencing
 *  the Port Object by.
 */
#define PortGadgetClass	xvwPortGadgetClass

        /*-------------------------------------*
        |       global variable declarations
        ---------------------------------------*/

/*
 *  XToolkit WidgetClass Declaration PortGadgetClass
 */
extern   WidgetClass     PortGadgetClass;


        /*-------------------------------------*
        |       routine definitions
        ---------------------------------------*/

/*
 *  Khoros Function Definition for Port Object
 */
xvobject xvw_create_port PROTO((xvobject, char *));


#endif /* _xvlang_Port_h_ */
/* Do not add after this line */
