/*
  * Khoros: $Id: Motif.c,v 1.8 1996/05/04 14:26:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Motif.c,v 1.8 1996/05/04 14:26:27 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		<<<<
   >>>>		Motif xvobject Creation		      <<<<
   >>>>  Private:
   >>>>                 xvw_create_scrollbar()
   >>>>                 xvw_create_list()
   >>>>                 xvw_create_text()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

#ifdef KMOTIF_DEF

klist *xvw_textwidget_list = NULL;
klist *xvw_listwidget_list = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_scrollbar
|
|       Purpose: Creates a scrollbar using the Motif Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference scrollbar
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro & John Salas
|          Date: April 14, 1993
| Modifications:
|
------------------------------------------------------------*/

xvobject xvw_create_scrollbar(
   xvobject parent,
   char   *name)
{
	int     i;
	Arg     arg[MAX_ARGS];
	Widget  scroll;

	static initialized = FALSE;

	i = 0;
	XtSetArg(arg[i], XmNorientation, XmHORIZONTAL);		i++;
	XtSetArg(arg[i], XmNtraversalOn, FALSE);		i++;
	XtSetArg(arg[i], XmNnavigationType, XmNONE);		i++;
	scroll = XtCreateManagedWidget(name, xmScrollBarWidgetClass,
			 xvw_widget(parent), arg, (unsigned) i);

	if (!initialized)
	{
	   initialized = TRUE;

	   xvw_init_attributes(XtClass(scroll), NULL, 0, NULL, 0, NULL);
	   xvw_define_attributes(XtClass(scroll),
	      XVW_SCROLL_MIN,  XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	      XVW_SCROLL_MAX,  XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	      XVW_SCROLL_VALUE,XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	      XVW_SCROLL_INCR, XtRDouble, xvw_set_scrollbar, xvw_get_scrollbar,
	      XVW_ORIENTATION, XtRInt, xvw_set_orientation, NULL,
	      NULL);
           xvw_define_callback(XVW_SCROLL_CONT_MOTION, XmNdragCallback,
              xvw_update_scroll_value);
           xvw_define_callback(XVW_SCROLL_INCR_MOTION, XmNvalueChangedCallback,
	      xvw_update_scroll_value);
	}
	return(xvw_object(scroll));
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_list
|
|       Purpose: Creates a compound list object using the Motif Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference list
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro & John Salas
|          Date: April 16, 1993
| Modifications:
|
------------------------------------------------------------*/

static xvattribute list_attributes[] = {
{XVW_LIST_SIZE,  XmNitemCount,  XtRInt,  NULL},
};

xvobject xvw_create_list(
   xvobject parent,
   char   *name)
{
	int    i;
	Arg    arg[MAX_ARGS];
	Widget list_widget;
        klist  *listwidget_list;

	static int initialized = FALSE;

	i = 0;
        XtSetArg(arg[i], XmNlistSpacing,            5);			i++;
        XtSetArg(arg[i], XmNscrollBarDisplayPolicy, XmAS_NEEDED);	i++;
        XtSetArg(arg[i], XmNlistSizePolicy,         XmCONSTANT);	i++;
        XtSetArg(arg[i], XmNmappedWhenManaged,      TRUE);   		i++;
        XtSetArg(arg[i], XmNscrollBarPlacement,     XmBOTTOM_RIGHT);	i++;
	XtSetArg(arg[i], XmNitemCount,              0);			i++;
	XtSetArg(arg[i], XmNwidth,		    10);		i++;
	XtSetArg(arg[i], XmNheight,		    10);		i++;
	list_widget = XmCreateScrolledList(xvw_widget(parent), name, 
					   arg, (unsigned) i);


        listwidget_list = klist_locate(xvw_listwidget_list, 
				       xvw_parent(xvw_object(list_widget)));
        if (listwidget_list != NULL)
           listwidget_list->client_data = (kaddr) list_widget;
        else xvw_listwidget_list = klist_add(xvw_listwidget_list, 
					    xvw_parent(xvw_object(list_widget)),
                                            list_widget);

	if (!initialized)
	{
	   initialized = TRUE;

	   xvw_init_attributes(XtClass(list_widget), list_attributes, 
			    (int) knumber(list_attributes), NULL, 0, NULL);
	   xvw_define_attributes(XtClass(list_widget),
             XVW_LIST_HIGHLT_ELEM,   XtRInt, xvw_set_list_highlight,  
					     xvw_get_list_highlight,
             XVW_LIST_UNHIGHLT_ELEM, XtRInt, xvw_set_list_unhighlight, NULL,
             NULL);
	   xvw_define_callback(XVW_LIST_ITEM_SELECT, XmNbrowseSelectionCallback,
              xvw_update_list_struct);
	   xvw_define_callback(XVW_LIST_ITEM_ACTION, XmNdefaultActionCallback,
              xvw_update_list_struct);
	}
	XtManageChild(list_widget);
	return(xvw_parent(xvw_object(list_widget)));
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_create_text
|
|       Purpose: Creates a text object using the Motif Widget set.
|
|         Input: parent - the parent xvobject
|                name   - name with which to reference text object
|        Output: none
|       Returns: none
|    Written By: Danielle Argiro & John Salas
|          Date: April 14, 1993
| Modifications:
|
------------------------------------------------------------*/

static xvattribute text_attributes[] = {
{XVW_TEXT_INSERT_POS,   XmNcursorPosition,   XtRInt,              NULL},
{XVW_TEXT_LENGTH,       XmNmaxLength,        XtRInt,              NULL},
{XVW_TEXT_LEFT_MARGIN,  XmNmarginWidth,      XtRDimension,        NULL},
{XVW_BORDER_WIDTH,	XmNshadowThickness,  XtRInt,		  XtRDimension},
};

xvobject xvw_create_text(
   xvobject parent,
   char   *name)
{
	int     i;
	Arg     arg[MAX_ARGS];
	Widget  text;
        klist  *textwidget_list;
	kaddr   type;

	static int initialized = FALSE;

        i = 0;
	XtSetArg(arg[i], XmNborderWidth,        0);			i++;
	XtSetArg(arg[i], XmNhighlightThickness, 0);			i++;
	XtSetArg(arg[i], XmNmaxLength, KLENGTH);			i++;
	XtSetArg(arg[i], XmNwordWrap, TRUE);				i++;
	XtSetArg(arg[i], XmNeditable, TRUE);                            i++;
#if XmVERSION == 1 && XmREVISION == 2
	XtSetArg(arg[i], XmNtraversalOn, FALSE);			   i++;
#endif
	XtSetArg(arg[i], XmNnavigationType, XmNONE);			   i++;
	text = XtCreateManagedWidget(name, xmTextWidgetClass,
		       xvw_widget(parent), arg, (unsigned) i);

	type = (kaddr) KTEXT_TYPE_STRING;
        textwidget_list = klist_locate(xvw_textwidget_list, xvw_object(text));
        if (textwidget_list != NULL)
           textwidget_list->client_data = type;
        else xvw_textwidget_list = klist_add(xvw_textwidget_list, 
					     xvw_object(text), type);

	if (!initialized)
	{
	   initialized = TRUE;

	   xvw_init_attributes(XtClass(text), text_attributes,
	                    (int) knumber(text_attributes), NULL, 0, NULL);
	   xvw_define_attributes(XtClass(text),
	     XVW_TEXT_MULTILINE, XtRInt, xvw_set_textmulti, xvw_get_textmulti,
             XVW_TEXT_TYPE,      XtRInt, xvw_set_texttype,  xvw_get_texttype,
             XVW_TEXT_STRING,    XtRString, xvw_set_text,   xvw_get_text,
             XVW_TEXT_EDIT_TYPE, XtRInt,    xvw_set_misc,   xvw_get_misc,
             XVW_TEXT_WRAP,      XtRInt,    xvw_set_misc,   xvw_get_misc,
             XVW_TEXT_SCROLL_VERT, XtRInt,    xvw_set_misc,   xvw_get_misc,
             XVW_TEXT_SCROLL_HORIZ,XtRInt,    xvw_set_misc,   xvw_get_misc,
	     NULL);
	}
#if !defined(KOPSYS_AIX)
	xvw_add_event(xvw_object(text), EnterWindowMask | LeaveWindowMask,
		      xvw_set_textfocus, NULL);
#endif
	xvw_set_attribute(xvw_object(text), XVW_TEXT_MULTILINE, FALSE);
	return(xvw_object(text));
}

#else

/*
 * These are dummy function calls so that if the widget set is not supported
 * on the machine a message is printed out.
 */

xvobject xvw_create_scrollbar(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Motif widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}

xvobject xvw_create_list(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Motif widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}

xvobject xvw_create_text(
   xvobject parent,
   char   *name)
{
   kfprintf(kstderr,"\nThe Motif widget set is either not supported for this \
machine or was not\ncompiled in by the installer of Khoros.  Please contact \
the installer.\n\n");
   return(NULL);
}
#endif /* KMOTIF_DEF */
/* Don't add after the endif */
