/*
 * Khoros: $Id: draw_util.c,v 1.12 1996/06/25 16:33:31 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: draw_util.c,v 1.12 1996/06/25 16:33:31 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		3D Drawing Utilities
   >>>>		
   >>>>  Private:
   >>>>                 _X3D_find_color()
   >>>>                 _X3D_find_corner()
   >>>>                 _X3D_find_level()
   >>>>                 _X2D_draw_markers()
   >>>>                 _X3D_set_fgcolor()
   >>>>                 _X3D_set_bgcolor()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "graphics.h"


/*-----------------------------------------------------------
|
|  Routine Name: _X3D_find_color
|
|       Purpose: 
|
|         Input: indx	      -
|		 colors	      -
|		 color_offset -
|		 num	      -
|
|        Output: none
|
|       Returns: indx into the XColor array
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/
int _X3D_find_color(
   register int    indx,
   register XColor *colors,
   int             color_offset,
   int             num)
{
	register int  cmax, cmin;

	cmin = color_offset;
	cmax = color_offset+num-1;

	if (indx > cmax)
	   indx = cmax-1;

	while ((char)colors[indx].flags == 0 && indx > cmin)
	   indx--;

	if ((char)colors[indx].flags == 0)
	{
	   while ((char)colors[indx].flags == 0 && indx < cmax)
	      indx++;

	   if ((char)colors[indx].flags == 0)
	      return(-1);
	}
	return(indx);
}

/*-----------------------------------------------------------
|
|  Routine Name: _X3D_find_corner
|
|       Purpose: This routine finds the rear most corner to start
|		 drawing a plot.  Horizon & Mesh plots use painter's
|		 algorithm in order to draw the plot, thus it is
|		 important to find the opposite corner in which to
|		 start drawing from.
|
|         Input: graphics -
|		 pt0	  -
|		 pt1	  -
|		 pt2	  -
|		 pt3	  -
|
|        Output: none
|
|       Returns: the corner the point
|		 BL - Back Left  BR - Back Right  
|		 FL - Front Left  FR - Front Right
|
|                         t  h  e  t  a
|
|                      | 0 - 90 |  90 - 180  180 - 270  270 - 360
|          -------------------------------------------------------
|       a   0 - 90     |  BL    |    FL     |    FR   |    BR
|          -------------------------------------------------------
|       l   90 - 180   |  BL    |    FL     |    FR   |    BR
|          -------------------------------------------------------
|       p   180 - 270  |  FL    |    FR     |    BR   |    BL
|          -------------------------------------------------------
|       h   270 - 360  |  FL    |    FR     |    BR   |    BL
|          -------------------------------------------------------
|       a
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
int _X3D_find_corner(
   X3DGraphics *graphics,
   Coord       *pt0,
   Coord       *pt1,
   Coord       *pt2,
   Coord       *pt3)
{
        Real   alpha = graphics->alpha, theta = graphics->theta;
 
        if ((theta >= 0) && (theta <= 90))
        {
             if ((alpha >= 0) && (alpha <= 180))
                 return (BACKLEFT);
             else return (FRONTLEFT);
        }
        else if ((theta >= 90) && (theta <= 180))
        {
             if ((alpha >= 0) && (alpha <= 180))
                 return (FRONTLEFT);
             else return (FRONTRIGHT);
        }
        else if ((theta >= 180) && (theta <= 270))
        {
             if ((alpha >= 0) && (alpha <= 180))
                 return (FRONTRIGHT);
             else return (BACKRIGHT);
        }
        else if ((theta >= 270) && (theta <= 360))
        {
             if ((alpha >= 0) && (alpha <= 180))
                 return (BACKRIGHT);
             else return (BACKLEFT);
        }
	return(graphics->corner);
}

/*-----------------------------------------------------------
|
|  Routine Name: _X3D_find_level
|
|       Purpose: 
|
|         Input: value	- the value to check
|		 levels - the array of levels
|		 num	- the number of levels
|
|        Output: none
|
|       Returns: the indx of the level the value is above
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

int _X3D_find_level(
   Real value,
   Real *levels,
   int  num)
{
	int	indx = num -1;

	while (value < levels[indx] && indx > 0)
	   indx--;

	return(indx);
}

/*-----------------------------------------------------------
|
|  Routine Name: _X2D_draw_markers
|
|       Purpose: Draws a single marker at the world coordinate point specified
|
|         Input: id	     - graphics id
|		 points	     - array of points to draw
|		 size	     - number of points to draw 
|		 scale       - amount to scale marker with (2 is twice 
|			       as big, 3 is three times as big, etc)
|		 marker_type - marker type to draw with
|
|    Written By: Mark Young
|          Date: 
| Modifications:
|
------------------------------------------------------------*/

void _X2D_draw_markers(
   XVGraphicsID   id,
   Coord          *coords,
   XPoint         *points,
   int            size,
   int            scale,
   int            marker_type,
   XColor         *fg)
{
	register int	i;
	X3DGraphics	*graphics;
	int		old_line_type;
	Colormap        colormap;
	XPoint          diamond[6];

	if ((graphics = _X3D_get_graphics(id)) == NULL)
	{
	   kerror (NULL, "_X2D_draw_markers", "Unknown graphics id %d\n", id);
	   return;
	}

	old_line_type = graphics->line_type;
	X3D_set_line_type(id, KLINE_SOLID);
        colormap = X3D_get_colormap(graphics->id);

	switch (marker_type)
	{
	   case KMARKER_POINT:	/* marker star or astrix */
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x, points[i].y +2*scale,
				 points[i].x, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x +2*scale, points[i].y +2*scale,
				 points[i].x -2*scale, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x +2*scale, points[i].y -2*scale,
				 points[i].x -2*scale, points[i].y +2*scale);
		}
		break;

	   case KMARKER_SQUARE:
		for (i = 0; i < size; i++)
		{
		       _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		       X3D_draw[graphics->device].rectangle(graphics,
			      points[i].x-2*scale, points[i].y -2*scale,
			      (unsigned int) 4*scale, (unsigned int) 4*scale);
		}
		break;

	   case KMARKER_TRIANGLE:

		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);

                    X3D_draw[graphics->device].line(graphics,
                             points[i].x -2*scale, points[i].y -2*scale,
                             points[i].x +2*scale, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
                             points[i].x +2*scale, points[i].y -2*scale,
                             points[i].x, points[i].y+2*scale);
		    X3D_draw[graphics->device].line(graphics,
                             points[i].x, points[i].y +2*scale,
                             points[i].x - 2*scale, points[i].y-2*scale);
		}
		break;

	   case KMARKER_DIAMOND:
		diamond[0].x = diamond[0].y = diamond[1].x = 0;
		diamond[2].y = diamond[3].x = diamond[3].y =
		diamond[4].x =  2*scale;
		diamond[1].y = diamond[2].x = diamond[4].y = diamond[5].x =
		diamond[5].y = -2*scale;
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    diamond[0] = points[i];
		    X3D_draw[graphics->device].lines(graphics, diamond, 6,
			          CoordModePrevious);
		}
		break;

	   case KMARKER_DAGGER:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);

		    X3D_draw[graphics->device].line(graphics,
				 points[i].x -1*scale, points[i].y + 1*scale,
				 points[i].x +1*scale,points[i].y + 1*scale);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x, points[i].y +2*scale,
				 points[i].x, points[i].y -2*scale);
		}
		break;

	   case KMARKER_CROSS:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);

		    X3D_draw[graphics->device].line(graphics,
				 points[i].x -2*scale, points[i].y,
				 points[i].x +2*scale, points[i].y);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x, points[i].y+2*scale,
				 points[i].x, points[i].y-2*scale);
		}
		break;

	   case KMARKER_X:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x -2*scale, points[i].y -2*scale,
				 points[i].x +2*scale, points[i].y +2*scale);
		    X3D_draw[graphics->device].line(graphics,
				 points[i].x -2*scale, points[i].y +2*scale,
				 points[i].x +2*scale, points[i].y -2*scale);
		}
		break;

	   case KMARKER_V:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y +2*scale,
			      points[i].x -2*scale, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y +2*scale,
			      points[i].x +2*scale, points[i].y -2*scale);
		}
		break;

	   case KMARKER_CARET:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y -2*scale,
			      points[i].x -2*scale, points[i].y +2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y -2*scale,
			      points[i].x +2*scale, points[i].y +2*scale);
		}
		break;

	   case KMARKER_BOW_TIE:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y -2*scale,
			      points[i].x +2*scale, points[i].y +2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y +2*scale,
			      points[i].x +2*scale, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y +2*scale,
			      points[i].x -2*scale, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x +2*scale, points[i].y +2*scale,
			      points[i].x +2*scale, points[i].y -2*scale);
		}
		break;

	   case KMARKER_BOX:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].rectangle(graphics,
			      points[i].x-2*scale, points[i].y -2*scale,
			      (unsigned int) 4*scale, (unsigned int) 4*scale);

		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y -2*scale,
			      points[i].x +2*scale, points[i].y +2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y +2*scale,
			      points[i].x +2*scale, points[i].y -2*scale);
		}
		break;

	   case KMARKER_CIRCLE:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);

                    X3D_draw[graphics->device].arc(graphics,
                             points[i].x, points[i].y,
                             (unsigned int) 4*scale, (unsigned int) 4*scale,
			     0, 360);

		}
		break;

/*
	   case KMARKER_HOTSPOT:
		for (i = 0; i < size; i++)
		{
		       _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		       X3D_draw[graphics->device].arc(graphics,
			     points[i].x-2*scale-1, points[i].y -2*scale-1,
			     4*scale+2, 4*scale+2, 0, 360);
		}
		break;
*/
	   case KMARKER_ARC:
		for (i = 0; i < size; i++)
		{
		       _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
                       X3D_draw[graphics->device].arc(graphics,
                             points[i].x, points[i].y + 1*scale,
                             (unsigned int) 4*scale, (unsigned int) 4*scale,
			     0, 180);

		}
		break;

	   case KMARKER_HEXAGON:
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y +2*scale,
			      points[i].x +2*scale, points[i].y +1*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x +2*scale, points[i].y +1*scale,
			      points[i].x +2*scale, points[i].y -1*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x +2*scale, points[i].y -1*scale,
			      points[i].x, points[i].y -2*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x, points[i].y -2*scale,
			      points[i].x -2*scale, points[i].y -1*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y -1*scale,
			      points[i].x -2*scale, points[i].y +1*scale);
		    X3D_draw[graphics->device].line(graphics,
			      points[i].x -2*scale, points[i].y +1*scale,
			      points[i].x, points[i].y +2*scale);
		}
		break;

           case KMARKER_DOT:
		scale = 1;
		for (i = 0; i < size; i++)
		{
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
		    X3D_draw[graphics->device].arc(graphics,
			     points[i].x, points[i].y,
			     (unsigned int) 2*scale, (unsigned int) 2*scale,
			     0, 360);
		}
		break;

           case KMARKER_PIXEL:
                for (i = 0; i < size; i++)
                {
		    _X3D_set_fgcolor(graphics, colormap, coords[i].d, fg);
                    X3D_draw[graphics->device].line(graphics,
                                 points[i].x, points[i].y,
                                 points[i].x, points[i].y);
                }
	        break;
	}
	X3D_set_line_type(id, old_line_type);

}

void _X3D_set_fgcolor(
    X3DGraphics *graphics,
    Colormap     colormap,
    Index        d,
    XColor       *fg)
{
	XColor          xcolor;

	if (d != KGRAPHICS_UNINITIALIZED)
        {
	    xcolor.pixel = d;
            X11_set_draw(graphics, &xcolor, TRUE);
        }
	else X3D_draw[graphics->device].draw_color(graphics, fg, FALSE);
}

void _X3D_set_bgcolor(
    X3DGraphics *graphics,
    Colormap     colormap,
    Index        d,
    XColor       *bg)
{
        XColor          xcolor;

        if (d != KGRAPHICS_UNINITIALIZED)
        {
            xcolor.pixel = d;
            X11_set_fill(graphics, &xcolor, TRUE);
        }
        else X3D_draw[graphics->device].fill_color(graphics, bg, FALSE);
}

