/*
  * Khoros: $Id: Circle.c,v 1.14 1996/05/15 22:21:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Circle.c,v 1.14 1996/05/15 22:21:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Circle Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 PickGadget()
   >>>>                 EraseSelection()
   >>>>                 RefreshSelection()
   >>>>   Public:
   >>>>                 xvw_create_circle()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvisual/CircleP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ChangeGeometry	 PROTO((Widget));
static int  PickGadget		 PROTO((Widget, int, int));

static void EraseSelection       PROTO((Widget));
static void RefreshSelection     PROTO((Widget));



/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_CIRCLE_XCENTER,   NULL,    XtRDouble,    NULL},
{XVW_CIRCLE_YCENTER,   NULL,    XtRDouble,    NULL},
{XVW_CIRCLE_RADIUS,    NULL,    XtRDouble,     NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwCircleGadgetRec, circle.field)

static XtResource resources[] = { 
{XVW_CIRCLE_XCENTER, NULL, XtRDouble, sizeof(double), 
      offset(center.x), XtRString, (XtPointer) "0.0"},
{XVW_CIRCLE_YCENTER, NULL, XtRDouble, sizeof(double), 
      offset(center.y), XtRString, (XtPointer) "0.0"},
{XVW_CIRCLE_RADIUS, NULL, XtRDouble, sizeof(double), 
      offset(radius), XtRString, (XtPointer) "1.0"},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwCircleGadgetClassRec xvwCircleGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Circle",			        /* class_name		  */
    sizeof(XvwCircleGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    EraseSelection,   			/* erase selected proc    */
    RefreshSelection,			/* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwCircleGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwCircleGadgetClass for public consumption
 */
WidgetClass xvwCircleGadgetClass = (WidgetClass) &xvwCircleGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of circle object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young & John Salas
|          Date: Aug 15, 1992
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwCircleGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Circle.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Circle");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for all the fields for a circle gadget instance.
|
|         Input: request - widget instance with requested public settings
|                new     - widget instance after initialization, with
|                          private settings initialized and public
|                          settings verified
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 4, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwCircleGadget xobj = (XvwCircleGadget) new;

	xobj->graphics.old_x      = new->core.x;
	xobj->graphics.old_y      = new->core.y;
	xobj->graphics.old_width  = new->core.width;
	xobj->graphics.old_height = new->core.height;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the circle object in response
|                to an expose event. 
|
|         Input: widget - the circle object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|        Output: None
|
|    Written By: Mark Young
|          Date: Jun 29, 1992
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwCircleGadget xobj = (XvwCircleGadget) widget;

	XVGraphicsID id;
	XColor       *fg, *bg;

	id = xobj->graphics.id;
	fg = &xobj->graphics.fg;
	bg = &xobj->graphics.bg;

	if (xobj->graphics.filled == FALSE)
	   X2D_draw_circle(id, xobj->circle.center,
			   (Real) xobj->circle.radius, fg);
	else
	   X2D_fill_circle(id, xobj->circle.center,
			   (Real) xobj->circle.radius, fg, bg);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the radius or the postion of the circle 
|		 has changed, recalculate the size and position
|		 of the circle.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1992
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues (
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwCircleGadget cobj = (XvwCircleGadget) current;
	XvwCircleGadget nobj = (XvwCircleGadget) new;


	if (cobj->circle.center.x != nobj->circle.center.x ||
	    cobj->circle.center.y != nobj->circle.center.y ||
	    cobj->circle.radius   != nobj->circle.radius)
	{
	   if (nobj->circle.radius <= 0.0)
	   {
	      kinfo(KFORCE, "Circle Object: The radius must be > 0.0, setting \
back to %g.\n", cobj->circle.radius);
	      nobj->circle.radius = cobj->circle.radius;
	   }
	   GraphicsRecomputePosition(new, FALSE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new center and radius, recompute
|		 the new device coordinate position of 
|		 the circle. 
|
|         Input: widget - the circle object  
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Jan 8, 1996 
| Modifications: 
|
------------------------------------------------------------*/

static void RecomputePosition(
   Widget widget)
{
	XvwCircleGadget xobj = (XvwCircleGadget) widget;

	Coord  coord;
	Real   xpos1, xpos2, ypos1, ypos2;
	XVGraphicsID id = xobj->graphics.id;

	coord = xobj->circle.center;
	coord.x -= xobj->circle.radius;
	coord.y -= xobj->circle.radius;
	X2D_convert_point_wc_to_dc(id, coord, &xpos1, &ypos1);

	coord = xobj->circle.center;
	coord.x += xobj->circle.radius;
	coord.y += xobj->circle.radius;
	X2D_convert_point_wc_to_dc(id, coord, &xpos2, &ypos2);

	xobj->graphics.old_x      = widget->core.x;
        xobj->graphics.old_y      = widget->core.y;
        xobj->graphics.old_width  = widget->core.width;
        xobj->graphics.old_height = widget->core.height;

	widget->core.x = (Position) kmin(xpos1, xpos2);
        widget->core.y = (Position) kmin(ypos1, ypos2);
        widget->core.width  = (Dimension) kabs(xpos2 - xpos1);
        widget->core.height = (Dimension) kabs(ypos2 - ypos1);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a new device coordinate description of the
|		 circle, recompute the world coordinate center
|		 and radius.
|
|         Input: widget - the circle object
|
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Jan 5, 1996
| Modifications: 
|
------------------------------------------------------------*/

static void ChangeGeometry(
   Widget widget)
{
	XvwCircleGadget xobj = (XvwCircleGadget) widget;
	Coord coord1, coord2;
	Real  xpos, ypos; 
        int   radius;

	XVGraphicsID id = xobj->graphics.id;

	/*
         * Resize procedure: compute the new world coordinate radius. 
	 * RecomputePosition is called to update the device coordinates, 
	 * since in this case DC change => WC change => DC change.  This is not
	 * typical, (usally in ChangeGeometry, DC change => WC change, period)
	 * but is necessary to keep center of circle at the same point 
	 * during the resize procedure.
         */
	if (xobj->graphics.pick_status == PICK_RESIZE)
	{
	    /* 
	     * width changes more than height? change to width "wins" and
	     * dictates change to both width and height (or vice versa).
	     */
	    if (kabs((Real) widget->core.width -
                     (Real) xobj->graphics.old_width) >
                kabs((Real) widget->core.height -
                     (Real) xobj->graphics.old_height))
		radius = widget->core.width/2;
            else radius = widget->core.height/2;

	    /*-- compute the new world coordinate radius --*/
	    X2D_convert_point_dc_to_wc(id, 0.0, 0.0, &coord1);
            X2D_convert_point_dc_to_wc(id, (Real)radius, 0.0, &coord2);
            xobj->circle.radius = kabs(coord2.x - coord1.x);

	    /*-- update core.x, core.y DC's --*/
	    RecomputePosition(widget); 
	}

	/*
	 * Move procedure: 
	 * maintain size of circle, update center point
	 */
	else if (xobj->graphics.pick_status == PICK_MOVE)
	{
	    /*-- compute world coordinate center point --*/
	    xpos = (Real) (widget->core.x + (Real)widget->core.width/2.0);
            ypos = (Real) (widget->core.y + (Real)widget->core.height/2.0);
            X2D_convert_point_dc_to_wc(id, xpos, ypos,
                                       &xobj->circle.center);

	    RecomputePosition(widget);  
	}
	
	/*
	 * Display procedure:
	 * get placement & radius from DC's
	 */
	else
	{
            /*-- compute world coordinate center point --*/
            xpos = (Real) (widget->core.x + (Real)widget->core.width/2.0);
            ypos = (Real) (widget->core.y + (Real)widget->core.height/2.0);
            X2D_convert_point_dc_to_wc(id, xpos, ypos,
                                       &xobj->circle.center);

            /*-- compute the new world coordinate radius --*/
            X2D_convert_point_dc_to_wc(id, 0.0, 0.0, &coord1);
            X2D_convert_point_dc_to_wc(id, (Real)widget->core.width/2.0,
				        0.0, &coord2);
            xobj->circle.radius = kabs(coord2.x - coord1.x);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a circle object and an attempted picking location,
|		 see if the circle has been picked.  Note whether the 
|		 circle was picked on a corner marker (implying a resize)
|		 or on the circle radius (implying a move).
|
|         Input: widget - the circle object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Jan 5, 1995
| Modifications: 
|
------------------------------------------------------------*/

static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
        XvwCircleGadget xobj = (XvwCircleGadget) widget;

	/*
         *  circle was picked on one of its corners markers;
         */
	if (xvw_pick_cornermarkers(widget, x, y))
	{
	    xobj->graphics.pick_status = PICK_RESIZE;
	    return(TRUE);
	}

	/*
	 *  circle was picked on one of its side markers  
 	 */
	else if (xvw_pick_sidemarkers(widget, x, y))
	{
	    xobj->graphics.pick_status = PICK_RESIZE;
	    return(TRUE);
	}

	/*
	 * consider it picked for moving
	 */
	else
        {
           xobj->graphics.pick_status = PICK_MOVE;
           return(TRUE);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: EraseSelection
|
|       Purpose: Erase the side and corner markers on the circle
|		 which had indicated that the circle had been selected
|
|         Input: widget - the circle whose corner markers we are erasing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
static void EraseSelection(
    Widget widget)
{
	XvwGraphicsGadgetClassRec *graphics_class;

        /*
         * call utility routine to refresh the side markers around circle
         */
        xvw_erase_sidemarkers(widget);

	/* 
	 * call the EraseSelection method of the Graphics Class
	 * to erase the corner markers around the circle
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwCircleGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.erase_selection)(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelection
|
|       Purpose: Draws little side and corner markers on the circle
|                to indicate that the circle has been selected
|
|         Input: widget - circle whose side & corner markers we are refreshing
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/

/* ARGSUSED */
static void RefreshSelection(
    Widget widget)
{
        XvwCircleGadget xwid = (XvwCircleGadget) widget;
	XvwGraphicsGadgetClassRec *graphics_class;
	GC gc = xwid->manager.foreground_gc;

	/*
	 * call utility routine to refresh the side markers around circle
	 */
	xvw_draw_sidemarkers(widget, gc);

	/* 
	 * call the RefreshSelection method of the Graphics Class
	 * to refresh the corner markers around the circle
	 */
	graphics_class = (XvwGraphicsGadgetClassRec *) 
		         (xvwCircleGadgetClass->core_class.superclass);
	(*graphics_class->manager_class.refresh_selection)(widget);
}


/************************************************************
*
*  Routine Name: xvw_create_circle - create a circle object.
*
*       Purpose: A circle visual object supports the display of a circle.  
*
*                The (x, y) location of the circle center and the radius of 
*                the circle may be specified in world coordinates, where the 
*		 world coordinate range is dictated by the "controlling" visual
*		 object. By default, the parent of the circle object is its
*		 controlling visual object; this may be changed using the 
*		 XVW_GRAPHICS_ATTACH attribute.
*
*		 Alternatively, the size and location of the circle may be
*		 specified using device coordinates. When device coordinates
*		 are used, they are with respect to the bounding box surrounding
*		 the circle; specify the (x,y) location of the upper left hand 
*		 corner of the bounding box, as well as the width and height
*		 of the bounding box.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically.
*                name   - a name for this particular instance of the circle 
*                         object (for use in app-defaults files, etc)
*
*        Output: 
*	Returns: The circle xvobject on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young 
*          Date: Jun 29, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_circle(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Create the circle object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, CircleGadgetClass);
	return(object);
}
