/*
  * Khoros: $Id: Color.c,v 1.25 1996/06/20 22:50:09 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Color.c,v 1.25 1996/06/20 22:50:09 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Graphics Widget/Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassPartInitializeWidget()
   >>>>                 ClassPartInitializeGadget()
   >>>>                 ClassInitializeGadget()
   >>>>                 ClassInitializeWidget()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 SetValues()
   >>>>                 ColorSetVisual()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/ColorP.h>

static void ClassInitializeGadget	PROTO((void));
static void ClassInitializeWidget	PROTO((void));
static void ClassPartInitializeGadget	PROTO((WidgetClass));
static void ClassPartInitializeWidget	PROTO((WidgetClass));

static void Initialize	   PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy	   PROTO((Widget));
static Boolean SetValues   PROTO((Widget, Widget, Widget, ArgList, Cardinal *));
static void ColorSetVisual PROTO((Widget));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_COLOR_BEGIN,          NULL,     XtRInt,         XtRPosition},
{XVW_COLOR_END,            NULL,     XtRInt,         XtRPosition},
{XVW_COLOR_COLOROBJ,       NULL,     XtRPointer,     NULL},
{XVW_COLOR_CALLBACK,	   NULL,     XtRCallback,    NULL},
{XVW_COLOR_NORM_TYPE,	   NULL,     XtRInt,         NULL},
{XVW_COLOR_NORM_METHOD,	   NULL,     XtRInt,         NULL},
{XVW_COLOR_RED_FUNCTION,   NULL,     XtRString,      NULL},
{XVW_COLOR_GREEN_FUNCTION, NULL,     XtRString,      NULL},
{XVW_COLOR_BLUE_FUNCTION,  NULL,     XtRString,      NULL},
{XVW_COLOR_ALLOC_POLICY,   NULL,     XtRInt,         NULL},
{XVW_COLOR_NORM_UBYTE,     NULL,     XtRInt,         XtRBoolean},
{XVW_COLOR_PRIVATE_CMAP,   NULL,     XtRInt,         XtRBoolean},

{XVW_VISUAL,   NULL,     XtRVisual,         NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwColorPart, field)

static XtResource color_resources[] = {
{XVW_COLOR_BEGIN, NULL, XtRPosition, sizeof(Position),
      offset(begin), XtRImmediate, (XtPointer) 0},
{XVW_COLOR_END, NULL, XtRPosition, sizeof(Position),
      offset(end), XtRImmediate, (XtPointer) 0},
{XVW_COLOR_COLOROBJ, NULL, XtRPointer, sizeof(kobject),
      offset(object), XtRImmediate, (XtPointer) NULL},
{XVW_COLOR_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(color_callbacks), XtRCallback, (XtPointer) NULL},
{XVW_COLOR_NORM_TYPE, NULL, XtRInt, sizeof(int),
      offset(norm_type), XtRImmediate, (XtPointer) KCOLOR_NORM_GLOBAL},
{XVW_COLOR_NORM_METHOD, NULL, XtRInt, sizeof(int),
      offset(norm_method), XtRImmediate, (XtPointer) KCOLOR_NORM_MAXCOLORS},
{XVW_COLOR_RED_FUNCTION, NULL, XtRString, sizeof(String),
      offset(red_function), XtRString, (XtPointer) "M0"},
{XVW_COLOR_GREEN_FUNCTION, NULL, XtRString, sizeof(String),
      offset(green_function), XtRString, (XtPointer) "M1"},
{XVW_COLOR_BLUE_FUNCTION, NULL, XtRString, sizeof(char *),
      offset(blue_function), XtRString, (XtPointer) "M2"},
{XVW_COLOR_ALLOC_POLICY, NULL, XtRInt, sizeof(int),
      offset(alloc_policy), XtRImmediate, (XtPointer) KCOLOR_ALLOC_READONLY},
{XVW_COLOR_NORM_UBYTE, NULL, XtRBoolean, sizeof(Boolean),
      offset(norm_ubyte), XtRImmediate, (XtPointer) TRUE},
{XVW_COLOR_PRIVATE_CMAP, NULL, XtRBoolean, sizeof(Boolean),
      offset(private_cmap), XtRImmediate, (XtPointer) FALSE},

{NULL, NULL, XtRInt, sizeof(int),
      offset(autocolor), XtRImmediate, (XtPointer) KGREYSCALE},

{NULL, NULL, XtRBoolean, sizeof(Boolean),
      offset(initialize_visual), XtRImmediate, (XtPointer) TRUE},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwColorGadgetClassRec xvwColorGadgetClassRec =
{
  {
	 (WidgetClass) SUPERCLASS,	/* superclass		  */	
	 "Color",			/* class_name		  */
	 sizeof(XvwColorGadgetRec),	/* size			  */
	 ClassInitializeGadget,		/* class_initialize	  */
	 ClassPartInitializeGadget,	/* class_part_initialize  */
	 FALSE,				/* class_inited		  */
	 Initialize,			/* initialize		  */
	 NULL,		                /* initialize_hooks       */
	 NULL,				/* realize		  */
	 NULL,				/* actions		  */
	 0,				/* num_actions		  */
	 NULL,				/* resources		  */
	 0,				/* resource_count	  */
	 NULLQUARK,			/* xrm_class		  */
	 FALSE,				/* compress_motion	  */
	 FALSE,				/* compress_exposure	  */
	 FALSE,				/* compress_enterleave    */
	 FALSE,				/* visible_interest	  */
	 Destroy,			/* destroy		  */
	 XtInheritResize,		/* resize		  */
	 NULL,				/* expose		  */
	 SetValues,			/* set_values		  */
	 NULL,				/* set_values_hook	  */
	 XtInheritSetValuesAlmost,	/* set_values_almost	  */
	 NULL,				/* get_values_hook	  */
	 NULL,				/* accept_focus		  */
	 XtVersion,			/* version		  */
	 NULL,				/* callback_private	  */
	 NULL,				/* tm_table		  */
	 NULL,				/* query_geometry	  */
	 NULL,				/* display_accelerator	  */
	 NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
	 XtInheritPicking,              /* pick object proc       */
	 XtInheritEraseSel,             /* erase selected proc    */
	 XtInheritRefreshSel,           /* refresh selection proc */
         NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
	 XtInheritRecomputePosition,	/* recompute position proc       */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
	 NULL,				/* reload color proc		 */
  },  /* XvwColorGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwColorGadgetClass for public consumption
 */
WidgetClass xvwColorGadgetClass = (WidgetClass) &xvwColorGadgetClassRec;



#define SUPERCLASS (&xvwGraphicsWidgetClassRec)

XvwColorWidgetClassRec xvwColorWidgetClassRec =
{
  {
	 (WidgetClass) SUPERCLASS,	/* superclass		  */	
	 "Color",			/* class_name		  */
	 sizeof(XvwColorWidgetRec),	/* size			  */
	 ClassInitializeWidget,		/* class_initialize	  */
	 ClassPartInitializeWidget,	/* class_part_initialize  */
	 FALSE,				/* class_inited		  */
	 Initialize,			/* initialize		  */
	 NULL,				/* initialize_hook	  */
	 XtInheritRealize,		/* realize		  */
	 NULL,				/* actions		  */
	 0,				/* num_actions		  */
	 NULL,				/* resources		  */
	 0,				/* resource_count	  */
	 NULLQUARK,			/* xrm_class		  */
	 TRUE,				/* compress_motion	  */
	 XtExposeCompressMaximal,	/* compress_exposure	  */
	 TRUE,				/* compress_enterleave    */
	 FALSE,				/* visible_interest	  */
	 Destroy,			/* destroy		  */
         NULL,				/* resize		  */
	 XtInheritExpose,		/* expose		  */
	 SetValues,			/* set_values		  */
	 NULL,				/* set_values_hook	  */
	 XtInheritSetValuesAlmost,	/* set_values_almost	  */
	 NULL,				/* get_values_hook	  */
	 NULL,				/* accept_focus		  */
	 XtVersion,			/* version		  */
	 NULL,				/* callback_private	  */
	 XtInheritTranslations,		/* tm_table		  */
	 NULL,				/* query_geometry	  */
	 NULL,				/* display_accelerator	  */
	 NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
         NULL,                    	/* geometry_manager	  */
	 XtInheritChangeManaged,        /* change_managed         */
	 XtInheritInsertChild,          /* insert_child           */
	 XtInheritDeleteChild,          /* delete_child           */
	 NULL,                          /* extension              */
  },  /* CompositeClass fields initialization */
  {
	 NULL,		                /* subresources           */
	 0,				/* subresources_count     */
	 sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size        */
	 NULL,			        /* initialize             */
	 NULL,                          /* destroy                */
	 NULL,                          /* set_values             */
	 NULL,                          /* extension              */
  },  /* ConstraintClass fields initialization */
  {
         XtInheritLayout,		/* pre-layout routine     */
         XtInheritLayout,		/* post-layout routine    */
         XtInheritChangeSel,	        /* change selected proc   */
         XtInheritEraseSel,		/* erase selected proc    */
         XtInheritRefreshSel,		/* refresh selection proc */
         XtInheritResize,		/* resize		  */
         XtInheritGeometryManager,  	/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
	 NULL,                          /* field not used         */

  },  /* XvwGraphicsWidgetClass fields initialization */
  {
	 NULL,
  },  /* XvwColorWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwColorWidgetClass for public consumption
 */
WidgetClass xvwColorWidgetClass = (WidgetClass) &xvwColorWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitializeWidget
|
|       Purpose: This method is called when the first time an
|                instance of XvwColorWidgetClass or a subclass of
|                the XvwColorWidgetClass is created.  It handles the
|                inheritence of special methods from the superclass
|                to the subclass.
|
|         Input: wclass - the object class being initialized
|        Output:
|    Written By: Mark Young
|          Date: May 24, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitializeWidget(
   WidgetClass wclass)
{
	XvwColorWidgetClass cclass = (XvwColorWidgetClass) wclass;
	XvwColorWidgetClass sclass = (XvwColorWidgetClass)
					wclass->core_class.superclass;

	/*
	 *  make sure we aren't trying to inherit from the color widget's
	 *  superclass.
	 */
	if (wclass != xvwColorWidgetClass)
	{
	   if (cclass->color_class.color_reload == XtInheritColorReload)
	   {
	      cclass->color_class.color_reload =
			sclass->color_class.color_reload;
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitializeGadget
|
|       Purpose: This method is called when the first time an 
|                instance of XvwColorGadgetClass or a subclass of 
|                the XvwColorGadgetClass is created.  It handles the
|                inheritence of special methods from the superclass 
|                to the subclass.
|
|         Input: wclass - the object class being initialized
|        Output:
|    Written By: Mark Young
|          Date: May 24, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitializeGadget(
   WidgetClass wclass)
{
	XvwColorGadgetClass cclass = (XvwColorGadgetClass) wclass;
	XvwColorGadgetClass sclass = (XvwColorGadgetClass)
					wclass->core_class.superclass;

	/*
	 *  make sure we aren't trying to inherit from the color object's
	 *  superclass.
	 */
	if (wclass != xvwColorGadgetClass)
	{
	   if (cclass->color_class.color_reload == XtInheritColorReload)
	   {
	      cclass->color_class.color_reload =
			sclass->color_class.color_reload;
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitializeGadget
|
|       Purpose: This method is called the first time an 
|                instance of color object class has been created. 
|		 It makes a copy of the color resources so that
|		 the color object class has them available.
|
|         Input: None
|
|        Output: None
|
|    Written By: Mark Young
|          Date: May 15, 1993 14:45
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitializeGadget(void)
{
	int	   i;
	XtResource *resources;
	size_t num_bytes     = sizeof(color_resources);
	size_t num_resources = knumber(color_resources);
	unsigned int res_offset    = XtOffsetOf(XvwColorGadgetRec, color);


	/*
	 *  Redo the resources so that we can share them between both
	 *  the xvwColorGadgetClass and xvwColorWidgetClass
	 */
	resources = (XtResource *) kmalloc(num_bytes);
	kmemcpy(resources, (char *) color_resources, num_bytes);

	for (i = 0; i < num_resources; i++)
	    resources[i].resource_offset += res_offset;

	xvwColorGadgetClass->core_class.resources     = resources;
	xvwColorGadgetClass->core_class.num_resources = num_resources;
	xvw_init_attributes(xvwColorGadgetClass, attributes,
		            (int) knumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Color");

	xvw_define_attributes(xvwColorGadgetClass,
	  XVW_COLOR_RELOAD,       XtRInt,     ColorSetUtil, NULL,
	  XVW_COLOR_COLORFILE,    XtRString,  ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_RED_MAPCOL,   XtRInt,     ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_GREEN_MAPCOL, XtRInt,     ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_BLUE_MAPCOL,  XtRInt,     ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_SAVEMAP,	  XtRString,  ColorSetUtil, NULL,
	  XVW_COLOR_CURRENTMAP,   XtRPointer, ColorSetUtil, NULL,
	  XVW_COLOR_COLORS,       XtRPointer, NULL,         ColorGetUtil,
	  XVW_COLOR_NUMCOLORS,    XtRInt,     NULL,         ColorGetUtil,

	  XVW_COLOR_CHANGE_RED_MAPCOL,   XtRInt, ColorChangeMapCol, NULL,
	  XVW_COLOR_CHANGE_GREEN_MAPCOL, XtRInt, ColorChangeMapCol, NULL,
	  XVW_COLOR_CHANGE_BLUE_MAPCOL,  XtRInt, ColorChangeMapCol, NULL,
	  NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an 
|                instance of color widget class has been created. 
|		 It makes a copy of the color resources so that
|		 the color widget class has them available.
|
|         Input: None
|
|        Output: None
|
|    Written By: Mark Young
|          Date: May 15, 1993 14:45
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitializeWidget(void)
{
	int	   i;
	XtResource *resources;
	size_t num_bytes     = sizeof(color_resources);
	size_t num_resources = knumber(color_resources);
	unsigned int res_offset    = XtOffsetOf(XvwColorWidgetRec, color);


	resources = (XtResource *) kmalloc(num_bytes);
	kmemcpy(resources, (char *) color_resources, num_bytes);

	for (i = 0; i < num_resources; i++)
	    resources[i].resource_offset += res_offset;

	xvwColorWidgetClass->core_class.resources     = resources;
	xvwColorWidgetClass->core_class.num_resources = num_resources;
	xvw_init_attributes(xvwColorWidgetClass, attributes,
		            (int) knumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Color");

	xvw_define_attributes(xvwColorWidgetClass,

	  XVW_COLOR_RELOAD,       XtRInt,    ColorSetUtil, NULL,
	  XVW_COLOR_COLORFILE,    XtRString, ColorSetUtil, ColorGetUtil,

	  XVW_COLOR_RED_MAPCOL,   XtRInt,    ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_GREEN_MAPCOL, XtRInt,    ColorSetUtil, ColorGetUtil,
	  XVW_COLOR_BLUE_MAPCOL,  XtRInt,    ColorSetUtil, ColorGetUtil,

	  XVW_COLOR_SAVEMAP,	  XtRString, ColorSetUtil, NULL,
	  XVW_COLOR_CURRENTMAP,   XtRPointer,ColorSetUtil, NULL,

	  XVW_COLOR_COLORS,       XtRPointer, NULL,         ColorGetUtil,
	  XVW_COLOR_NUMCOLORS,    XtRInt,     NULL,         ColorGetUtil,

	  XVW_COLOR_CHANGE_RED_MAPCOL,   XtRInt, ColorChangeMapCol, NULL,
	  XVW_COLOR_CHANGE_GREEN_MAPCOL, XtRInt, ColorChangeMapCol, NULL,
	  XVW_COLOR_CHANGE_BLUE_MAPCOL,  XtRInt, ColorChangeMapCol, NULL,

	  NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial image
|                for an image widget instance.
|
|	  Input: request - not used
|                new     - widget instance after initialization, with
|                          initial image
|
|	 Output: None
|
|    Written By: Mark Young
|          Date: May 15, 1993 14:49
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
	Widget   request,
	Widget   new,
	ArgList  args,
	Cardinal *num_args)
{
	XvwColorPart *color   = GetColorPart(new);
	Display      *display = XtDisplay(new);
        int            screen = XScreenNumberOfScreen(XtScreen(new));

	color->info = NULL;
	color->allocated = FALSE;
	color->red_function   = kstrdup(color->red_function);
	color->green_function = kstrdup(color->green_function);
	color->blue_function  = kstrdup(color->blue_function);
	color->visual         = DefaultVisual(display, screen);
	if (color->initialize_visual == TRUE)
	   ColorSetVisual(new);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Destroy the color object/widget
|
|         Input: widget - the color object/widget being destroyed
|
|        Output: None
|
|    Written By: Mark Young
|          Date: Nov 17, 1992 19:24
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
	Widget widget)
{
	XvwColorPart *color = GetColorPart(widget);
	xvobject object;

	/*
	 *  Now that the object has been closed, see if we should clean up
	 *  the object by calling ColorInit() to remove the callback, etc.
	 */
	if (color->info != NULL && color->info->widget != NULL)
	{
	   if (color->info->widget != widget)
	   {
              /*
               * need to check if object is NULL before removing callback,
               * as it will be NULL if a new colormap was set using
               * XVW_COLOR_COLOROBJ (DA)
               */
              object = xvw_object(color->info->widget);
              if (object != NULL)
                  xvw_remove_callback(object, XVW_COLOR_CALLBACK, 
                                      ColorUtilCallback, (XtPointer) widget);
	   }
	   else
	      color->info->widget = NULL;
	}
	kfree_and_NULL(color->red_function);
	kfree_and_NULL(color->green_function);
	kfree_and_NULL(color->blue_function);
	if (color->info != NULL)
	   color->info->references--;

        if (color->info != NULL && color->info->references == 0)
        {
           if (color->info->id != 0)
              XtRemoveTimeOut(color->info->id);

           ColorFree(widget, 0, -1);
        }

	/*
	 *  First close the object, so we are no longer referencing it.
	 */
	kpds_close_object(color->object);
	color->object = NULL;
}


void ColorInfoFree(
	kobject object,
	char   *type,
	kaddr   client_data)
{
        ColorObjInfo *color_info = (ColorObjInfo *) client_data;

	kfree_and_NULL(color_info->pixels);
	kfree_and_NULL(color_info->xcolors);
	kfree_and_NULL(color_info->mapping);
        kfree_and_NULL(color_info->maps);
        kfree_and_NULL(color_info->red);
        kfree_and_NULL(color_info->green);
        kfree_and_NULL(color_info->blue);
        kfree_and_NULL(color_info->filename);
        kfree_and_NULL(color_info->histogram);
	kfree_and_NULL(color_info);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a color object/widget instance.  The actual changes
|		 to the color part are made throught a specific set
|		 values call to a routine which operates only on the
|		 color part.
|                 
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|
|        Output: None 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: May 15, 1993 14:45
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
	Widget   current,
	Widget   request,
	Widget   new,
	ArgList  args,
	Cardinal *num_args)
{
	XvwColorPart *ncolor = GetColorPart(new);
	XvwColorPart *ccolor = GetColorPart(current);


	if (ccolor->object != ncolor->object)
	{
	   /*
	    *  Make a reference of the data object, so we can twiddle
	    *  attributes and then add a map callback to the object.
	    *  But before adding a map callback we need to call
	    *  ColorInit(), since this will create a map segment
	    *  if needed.
	    */
	   if (ccolor->object != NULL)
	       ColorFree(current, 0, -1);
	   if (ncolor->object != NULL)
	   {
	       ncolor->object = kpds_reference_object(ncolor->object);
	       ColorInit(new);
	       if (ncolor->info != NULL)
	           ncolor->info->references++;
	       if (ccolor->info != NULL)
	           ccolor->info->references--;
	       kpds_close_object(ccolor->object);
	   }
	}

	if (ccolor->norm_method    != ncolor->norm_method    ||
	    ccolor->norm_type      != ncolor->norm_type      ||
	    ccolor->red_function   != ncolor->red_function   ||
	    ccolor->green_function != ncolor->green_function ||
	    ccolor->blue_function  != ncolor->blue_function)
	{
	   Boolean update = FALSE;

	   /*
	    *  According to the RGB mapping functions, load the red, green, blue
	    *  map arrays.
	    */
	   if (ccolor->red_function != ncolor->red_function)
	   {
	      ncolor->red_function = kstrdup(ncolor->red_function);
	      if (kstrcmp(ccolor->red_function, ncolor->red_function) != 0)
	      {
	         ApplyMapcolFunction(new, RED_COLOR);
	         update = TRUE;
	      }
	      kfree(ccolor->red_function);
	   }

	   if (ccolor->green_function != ncolor->green_function)
	   {
	      ncolor->green_function = kstrdup(ncolor->green_function);
              if (kstrcmp(ccolor->green_function, ncolor->green_function) != 0)
              {
	          ApplyMapcolFunction(new, GREEN_COLOR);
	          update = TRUE;
	      }
	      kfree(ccolor->green_function);
	   }

	   if (ccolor->blue_function != ncolor->blue_function)
	   {
	      ncolor->blue_function = kstrdup(ncolor->blue_function);
              if (kstrcmp(ccolor->blue_function, ncolor->blue_function) != 0)
              {
	          ApplyMapcolFunction(new, BLUE_COLOR);
	          update = TRUE;
	      }
              kfree(ccolor->blue_function);
	   }

	   if (ccolor->norm_method != ncolor->norm_method ||
	       ccolor->norm_type   != ncolor->norm_type)
	   {
	     update = TRUE;
	   }

	   if (update == TRUE)
	      ColorReloadXData(new);
	}

	if (ccolor->alloc_policy != ncolor->alloc_policy)
	{
	   XVisualInfo vinfo;
	   char *class_str;

	   xvw_get_attribute(xvw_object(new), XVW_VISUAL, &vinfo);
           if (ncolor->alloc_policy == KCOLOR_ALLOC_READONLY && 
	       (vinfo.class == TrueColor || vinfo.class == StaticColor ||
	        vinfo.class == StaticGray))
	   {
		switch (vinfo.class) 
		{
		   case TrueColor:   class_str = "TrueColor"; break;
		   case StaticColor: class_str = "StaticColor"; break;
		   case StaticGray:  class_str = "StaticGray"; break;
		}
		kerror(XVISUAL, "SetValues", "Cannot use a Color Allocation Policy of Read/Write when the visual of the screen being used for display is a (read-only) %s", class_str);
		ncolor->alloc_policy = KCOLOR_ALLOC_READONLY;
	   }
	   else
	   {
              ColorFree(current, 0, -1);
              ColorStore(new, -1, NULL);
	   }
	}
	if (ccolor->private_cmap != ncolor->private_cmap)
	{
	   ColorFree(current, 0, -1);
	   ColorStore(new, -1, NULL);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorSetVisual - routine for getting the best visual
|
|       Purpose: This routine sets the visual given an object's data
|		 object.
|
|         Input: 
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Jun 20, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorSetVisual(
	Widget   widget)
{
	XvwColorPart *color = GetColorPart(widget);

	Display	*display  = XtDisplay(widget);
	xvobject object   = xvw_object(widget);
	int     screen    = XScreenNumberOfScreen(XtScreen(widget));

	int         depth;
	XVisualInfo vinfo;
	Visual      *visual;


	/*
	 *  Set the current visual to the default visual if it is null
	 */
	visual = NULL;
	xvw_get_attribute(object, XVW_VISUAL, &visual);
	if (!visual)
	   visual = DefaultVisual(display, screen);

	/*
	 *   Find a useable visual
	 */
	depth = widget->core.depth;
	if (XMatchVisualInfo(display, screen, 24, TrueColor, &vinfo)      ||
	    XMatchVisualInfo(display, screen, depth, PseudoColor, &vinfo) ||
	    XMatchVisualInfo(display, screen, depth, TrueColor,   &vinfo) ||
	    XMatchVisualInfo(display, screen, depth, DirectColor, &vinfo))
	{
	   if (visual != vinfo.visual)
	      xvw_set_attribute(object, XVW_VISUAL, vinfo.visual);

	   visual = vinfo.visual;
	   color->visual_depth = vinfo.depth;
	   color->visual = visual;
	}
	else
	   color->visual_depth = DefaultDepth(display, screen);

	/*
	 *  If we are sporting a TrueColor or DirectColor display, then we
	 *  want to figure out the RGB mask and shift factors.
	 */
	if (vinfo.class == TrueColor || vinfo.class == DirectColor)
	{
	   color->rshift = 0; color->rmask = visual->red_mask;
	   color->gshift = 0; color->gmask = visual->green_mask;
	   color->bshift = 0; color->bmask = visual->blue_mask;
	   for (; !(color->rmask & 0x01) && color->rmask; color->rshift++)
	       color->rmask >>= 1;
           for( ; !(color->rmask & 0x80) && color->rmask; color->rshift--)
             color->rmask <<= 1;
	   for (; !(color->gmask & 0x01) && color->gmask; color->gshift++)
	       color->gmask >>= 1;
           for( ; !(color->gmask & 0x80) && color->gmask; color->gshift--)
             color->gmask <<= 1;
	   for (; !(color->bmask & 0x01) && color->bmask; color->bshift++)
	       color->bmask >>= 1;
           for( ; !(color->bmask & 0x80) && color->bmask; color->bshift--)
             color->bmask <<= 1;
	}
	else
	{
	   color->rshift = color->gshift = color->bshift = 0;
	   color->rmask  = color->gmask  = color->bmask  = 0;
	}
}
