/*
 * Khoros: $Id: Graphics.c,v 1.17 1996/06/20 22:50:14 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODecemberENTER__)
static char rcsid[] = "Khoros: $Id: Graphics.c,v 1.17 1996/06/20 22:50:14 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Graphics Widget/Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassPartInitializeWidget()
   >>>>                 ClassPartInitializeGadget()
   >>>>                 ClassInitializeGadget()
   >>>>                 ClassInitializeWidget()
   >>>>                 Initialize()
   >>>>                 RealizeWidget()
   >>>>                 RealizeGadget()
   >>>>                 ResizeWidget()
   >>>>                 ResizeGadget()
   >>>>                 Destroy()
   >>>>                 SetValuesGadget()
   >>>>                 SetValuesWidget()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 EraseSelection()
   >>>>                 RefreshSelectionWidget()
   >>>>                 RefreshSelectionGadget()
   >>>>                 RefreshSelection()
   >>>>                 DefaultAreaAttach()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/GraphicsP.h>

static void ClassPartInitializeWidget	PROTO((WidgetClass));
static void ClassPartInitializeGadget	PROTO((WidgetClass));
static void ClassInitializeGadget	PROTO((void));
static void ClassInitializeWidget	PROTO((void));
static void Initialize			PROTO((Widget, Widget, ArgList,
					       Cardinal *));
static void RealizeWidget		PROTO((Widget,XtValueMask *,
					       XSetWindowAttributes *));
static void RealizeGadget		PROTO((Widget,XtValueMask *,
					       XSetWindowAttributes *));
static void ResizeGadget		PROTO((Widget));
static void ResizeWidget		PROTO((Widget));
static void Destroy			PROTO((Widget));
static Boolean SetValuesGadget		PROTO((Widget, Widget, Widget,
					       ArgList, Cardinal *));
static Boolean SetValuesWidget		PROTO((Widget, Widget, Widget,
				      	       ArgList, Cardinal *));
static Boolean SetValues		PROTO((XvwGraphicsPart *, 
					       XvwGraphicsPart *));
static void RecomputePosition           PROTO((Widget));
static void EraseSelection         	PROTO((Widget));
static void RefreshSelectionWidget 	PROTO((Widget));
static void RefreshSelectionGadget 	PROTO((Widget));
static void RefreshSelection       	PROTO((Widget, GC));
static void DefaultAreaAttach           PROTO((Widget, int, XrmValue *));


/*-- GraphicsUtil.c functions --*/
extern XvwGraphicsPart *GetGraphicsPart PROTO((Widget));
extern void SetAttachedWorldView      PROTO((XvwGraphicsPart *, 
					       XvwGraphicsPart *));
extern void SetAttachedWorldAttributes     PROTO((Widget, XvwGraphicsPart *));
extern void UpdateAttachAttributes      PROTO((Widget, XvwGraphicsPart *,
					       XvwGraphicsPart *));
extern void CheckAxisModeValidity       PROTO((XvwGraphicsPart *, 
					       XvwGraphicsPart *));




/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_GRAPHICS_WCMIN_X,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_WCMIN_Y,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_WCMIN_Z,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_WCMAX_X,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_WCMAX_Y,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_WCMAX_Z,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_FILLED,	          NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_LINETYPE,	          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_MARKERTYPE,         NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_MARKERSCALE,        NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_LINEWIDTH,          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_PROPORTIONAL,       NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_CLIPPING,	          NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_RESET_WORLD,        NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_DEPTH_CLIPPING,     NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_PROJECTION,         NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_ALPHA,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_THETA,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_GAMMA,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_EYE,	          NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWDISTANCE,       NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MIN_X,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MIN_Y,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MIN_Z,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MAX_X,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MAX_Y,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_VIEWPORT_MAX_Z,     NULL,    XtRDouble,	  NULL},
{XVW_GRAPHICS_MODE_X,	          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_MODE_Y,	          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_MODE_Z,	          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_SYMMETRIC_X,        NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_SYMMETRIC_Y,        NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_SYMMETRIC_Z,        NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_TYPE, 	          NULL,    XtRInt,	  NULL},
{XVW_GRAPHICS_UPDATEDC, 	  NULL,    XtRInt,	  XtRBoolean},
{XVW_GRAPHICS_ATTACH,	          NULL,	   XtRObject,	  XtRWidget},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwGraphicsPart, field)

static double graphics_alpha   = 100.0,
	      graphics_theta   = 200.0,
	      graphics_gamma   = 0.0,
	      graphics_eye     = 0.0,
	      graphics_view    = 6.0;

static double graphics_min     = DEFAULT_WCMIN,
	      graphics_max     = DEFAULT_WCMAX;

static XtResource graphics_resources[] = { 
{XVW_BACKGROUND, NULL, XtRPixel, sizeof(Pixel),
      offset(bg.pixel), XtRString, XtDefaultBackground},
{XVW_FOREGROUND, NULL, XtRPixel, sizeof(Pixel),
      offset(fg.pixel), XtRString, XtDefaultForeground},
{XVW_GRAPHICS_VIEWDISTANCE, NULL, XtRDouble, sizeof(double), 
      offset(view_distance), XtRDouble, (XtPointer) &graphics_view},
{XVW_GRAPHICS_VIEWPORT_MIN_X, NULL, XtRDouble, sizeof(double), 
      offset(viewmin.x), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_VIEWPORT_MIN_Y, NULL, XtRDouble, sizeof(double), 
      offset(viewmin.y), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_VIEWPORT_MIN_Z, NULL, XtRDouble, sizeof(double), 
      offset(viewmin.z), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_VIEWPORT_MAX_X, NULL, XtRDouble, sizeof(double), 
      offset(viewmax.x), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_VIEWPORT_MAX_Y, NULL, XtRDouble, sizeof(double), 
      offset(viewmax.y), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_VIEWPORT_MAX_Z, NULL, XtRDouble, sizeof(double), 
      offset(viewmax.z), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_LINETYPE,  NULL, XtRInt, sizeof(int),
      offset(line_type), XtRImmediate, (XtPointer) KLINE_SOLID},
{XVW_GRAPHICS_MARKERTYPE,  NULL, XtRInt, sizeof(int),
      offset(marker_type), XtRImmediate, (XtPointer) KMARKER_SQUARE},
{XVW_GRAPHICS_MARKERSCALE,  NULL, XtRInt, sizeof(int),
      offset(marker_scale), XtRImmediate, (XtPointer) 1},
{XVW_GRAPHICS_FILLED,  NULL, XtRBoolean, sizeof(Boolean),
      offset(filled), XtRImmediate, (XtPointer) FALSE},
{XVW_GRAPHICS_LINEWIDTH,  NULL, XtRInt, sizeof(int),
      offset(line_width), XtRImmediate, (XtPointer) KLINE_EXTRA_FINE},
{XVW_GRAPHICS_WCMIN_X, NULL, XtRDouble, sizeof(double), 
      offset(wcmin.x), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_WCMIN_Y, NULL, XtRDouble, sizeof(double), 
      offset(wcmin.y), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_WCMIN_Z, NULL, XtRDouble, sizeof(double), 
      offset(wcmin.z), XtRDouble, (XtPointer) &graphics_min},
{XVW_GRAPHICS_WCMAX_X, NULL, XtRDouble, sizeof(double), 
      offset(wcmax.x), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_WCMAX_Y, NULL, XtRDouble, sizeof(double), 
      offset(wcmax.y), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_WCMAX_Z, NULL, XtRDouble, sizeof(double), 
      offset(wcmax.z), XtRDouble, (XtPointer) &graphics_max},
{XVW_GRAPHICS_PROPORTIONAL, NULL, XtRInt, sizeof(int), 
      offset(proportional), XtRImmediate, (XtPointer) KGRAPHICS_NONPROP},
{XVW_GRAPHICS_CLIPPING, NULL, XtRBoolean, sizeof(Boolean), 
      offset(clipping), XtRImmediate, (XtPointer) TRUE},
{XVW_GRAPHICS_DEPTH_CLIPPING, NULL, XtRBoolean, sizeof(Boolean), 
      offset(depth_clipping), XtRImmediate, (XtPointer) TRUE},
{XVW_GRAPHICS_RESET_WORLD, NULL, XtRBoolean, sizeof(Boolean), 
      offset(reset_world), XtRImmediate, (XtPointer) TRUE},
{XVW_GRAPHICS_PROJECTION, NULL, XtRInt, sizeof(int), 
      offset(projection), XtRImmediate,(XtPointer) KGRAPHICS_PERSPECTIVE},
{XVW_GRAPHICS_ALPHA, NULL, XtRDouble, sizeof(double), 
      offset(alpha), XtRDouble, (XtPointer) &graphics_alpha},
{XVW_GRAPHICS_THETA, NULL, XtRDouble, sizeof(double), 
      offset(theta), XtRDouble, (XtPointer) &graphics_theta},
{XVW_GRAPHICS_GAMMA, NULL, XtRDouble, sizeof(double), 
      offset(gamma), XtRDouble, (XtPointer) &graphics_gamma},
{XVW_GRAPHICS_EYE, NULL, XtRDouble, sizeof(double), 
      offset(eye_distance), XtRDouble, (XtPointer) &graphics_eye},
{XVW_GRAPHICS_MODE_X, NULL, XtRInt, sizeof(int),
      offset(axis_mode_x), XtRImmediate, (XtPointer) KGRAPHICS_LINEAR},
{XVW_GRAPHICS_MODE_Y, NULL, XtRInt, sizeof(int),
      offset(axis_mode_y), XtRImmediate, (XtPointer) KGRAPHICS_LINEAR},
{XVW_GRAPHICS_MODE_Z, NULL, XtRInt, sizeof(int),
      offset(axis_mode_z), XtRImmediate, (XtPointer) KGRAPHICS_LINEAR},
{XVW_GRAPHICS_SYMMETRIC_X, NULL, XtRBoolean, sizeof(Boolean), 
      offset(symmetric_x), XtRImmediate, (XtPointer) FALSE},
{XVW_GRAPHICS_SYMMETRIC_Y, NULL, XtRBoolean, sizeof(Boolean), 
      offset(symmetric_y), XtRImmediate, (XtPointer) FALSE},
{XVW_GRAPHICS_SYMMETRIC_Z, NULL, XtRBoolean, sizeof(Boolean), 
      offset(symmetric_z), XtRImmediate, (XtPointer) FALSE},
{XVW_GRAPHICS_TYPE, NULL, XtRInt, sizeof(int), offset(graphics_type),
      XtRImmediate, (XtPointer) KGRAPHICS_2D},
{XVW_GRAPHICS_UPDATEDC, NULL, XtRBoolean, sizeof(Boolean), offset(updatedc),
      XtRImmediate, (XtPointer) TRUE},
{XVW_GRAPHICS_ATTACH, NULL, XtRWidget, sizeof(Widget),
	offset(attach), XtRCallProc, (XtPointer) DefaultAreaAttach},
};
#undef offset

/*--------------------------------------------------------
|
|   Graphics Gadget Class
|
--------------------------------------------------------*/

#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwGraphicsGadgetClassRec xvwGraphicsGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Graphics",				/* class_name		  */
    sizeof(XvwGraphicsGadgetRec),	/* size			  */
    ClassInitializeGadget,		/* class_initialize	  */
    ClassPartInitializeGadget,		/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,		                /* initialize_hooks       */
    (XtProc) RealizeGadget,		/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    NULL,				/* resources		  */
    0,					/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    ResizeGadget,			/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValuesGadget,			/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,                   /* pick object proc       */
    EraseSelection,                     /* erase selected proc    */
    RefreshSelectionGadget,             /* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc       */
  },  /* XvwGraphicsGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwGraphicsGadgetClass for public consumption
 */
WidgetClass xvwGraphicsGadgetClass = (WidgetClass) &xvwGraphicsGadgetClassRec;


/*--------------------------------------------------------
|
|   Graphics Widget Class
|
--------------------------------------------------------*/


#define SUPERCLASS (&xvwManagerWidgetClassRec)

XvwGraphicsWidgetClassRec xvwGraphicsWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Graphics",				/* class_name		  */
    sizeof(XvwGraphicsWidgetRec),	/* size			  */
    ClassInitializeWidget,		/* class_initialize	  */
    ClassPartInitializeWidget,		/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    RealizeWidget,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    NULL,				/* resources		  */
    0,					/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValuesWidget,			/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,             /* change_managed         */
    XtInheritInsertChild,               /* insert_child           */
    XtInheritDeleteChild,               /* delete_child           */
    NULL,                               /* extension              */
  },  /* CompositeClass fields initialization */
  {
    NULL,	    			/* subresources       */
    0,   			        /* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec), /* constraint_size    */
    NULL,		                /* initialize         */
    NULL,                               /* destroy            */
    NULL,		                /* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    EraseSelection,                     /* erase selected proc    */
    RefreshSelectionWidget,             /* refresh selection proc */
    ResizeWidget,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwGraphicsWidgetClass for public consumption
 */
WidgetClass xvwGraphicsWidgetClass = (WidgetClass) &xvwGraphicsWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitializeWidget
|
|       Purpose: This method would be called when the first time an 
|                instance of graphics widget class or a subclass of 
|                the graphics widget class is created.  
|
|    Written By: Mark Young & John Salas
|          Date: August 15, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitializeWidget(
   WidgetClass wclass)
{

}

/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitializeGadget
|
|       Purpose: This method is called when the first time an 
|                instance of XvwGraphicsGadgetClass or a subclass of 
|                the XvwGraphicsGadgetClass is created.  It handles the
|                inheritence of special methods from the superclass 
|                to the subclass.
|
|         Input: wclass - the object class being initialized
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitializeGadget(
   WidgetClass wclass)
{
	XvwGraphicsGadgetClass gclass = (XvwGraphicsGadgetClass) wclass;
	XvwGraphicsGadgetClass sclass = (XvwGraphicsGadgetClass)
					wclass->core_class.superclass;

	/*
	 *  make sure we aren't trying to inherit from the graphics object's
	 *  superclass.
	 */
	if (wclass != xvwGraphicsGadgetClass)
	{
	   if (gclass->graphics_class.recompute_position ==
	       XtInheritRecomputePosition)
	   {
	      gclass->graphics_class.recompute_position =
			sclass->graphics_class.recompute_position;
	   }
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitializeGadget
|
|       Purpose: This method is called the first time an 
|                instance of graphics gadget class has been created. 
|		 It makes a copy of the graphics resources so that
|		 the graphics gadget class has them available.
|
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitializeGadget(void)
{
	int	   i;
	XtResource *resources;
	int num_bytes     = sizeof(graphics_resources);
	int num_resources = knumber(graphics_resources);
	unsigned int res_offset    = XtOffsetOf(XvwGraphicsGadgetRec, graphics);

	/*
	 *  Redo the resources so that we can share them between both
	 *  the xvwGraphicsGadgetClass and xvwGraphicsWidgetClass
	 */
	resources = (XtResource *) XtMalloc((unsigned) num_bytes);
	kmemcpy(resources, (char *) graphics_resources, (size_t) num_bytes);

	for (i = 0; i < num_resources; i++)
	    resources[i].resource_offset += res_offset;

	xvwGraphicsGadgetClass->core_class.resources     = resources;
	xvwGraphicsGadgetClass->core_class.num_resources = num_resources;
        xvw_init_attributes(xvwGraphicsGadgetClass, attributes,
		            (int) knumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Graphics");
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitializeWidget
|
|       Purpose: This method is called the first time an 
|                instance of graphics widget class has been created. 
|		 It makes a copy of the graphics resources so that
|		 the graphics widget class has them available.
|
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitializeWidget(void)
{
	int	   i;
	XtResource *resources;
	int  num_bytes     = sizeof(graphics_resources);
	int  num_resources = knumber(graphics_resources);
	unsigned int res_offset    = XtOffsetOf(XvwGraphicsWidgetRec, graphics);

	resources = (XtResource *) XtMalloc((unsigned) num_bytes);
	kmemcpy(resources, (char *) graphics_resources, (size_t) num_bytes);

	for (i = 0; i < num_resources; i++)
	    resources[i].resource_offset += res_offset;

	xvwGraphicsWidgetClass->core_class.resources     = resources;
	xvwGraphicsWidgetClass->core_class.num_resources = num_resources;
        xvw_init_attributes(xvwGraphicsWidgetClass, 
			    attributes, (int) knumber(attributes), 
			    NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Graphics");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values 
|                for all the fields for a graphics widget or
|		 gadget instance.   This will also set up a 
|		 unique graphics id in the xvgraphics library
|		 with default world view settings and initialize
|		 the graphics structure.
|
|         Input: request - widget instance with requested public settings
|                new     - widget instance after initialization, with
|                          private settings initialized and public 
|                          settings verified 
|
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwGraphicsPart *graphics = GetGraphicsPart(new);
	XvwGraphicsPart *parent_graphics = GetGraphicsPart(XtParent(new));
	Colormap colormap;

	/*
	 *  Graphical objects border width is really it's line width.  If
	 *  the line width is 0, this means it uses a hardware line width
	 *  of 1, so make sure that the border width reflects this.
	 */
	new->core.border_width = graphics->line_width;
	if (new->core.border_width == 0) new->core.border_width = 1;

	/* 
	 *  By default, assume that we will be picking the object to move it.
	 */
	graphics->pick_status = 0;

	/*
	 *  Get the xcolors for the desired foreground & background
	 */
	if (XtIsSubclass(new, xvwGraphicsWidgetClass))
	   colormap = new->core.colormap;
        else
	   colormap = new->core.parent->core.colormap;

	XQueryColor(XtDisplay(new), colormap, &graphics->fg);
	XQueryColor(XtDisplay(new), colormap, &graphics->bg);

	/*
	 *  Compute the position of the object in device coordinates
	 */
	graphics->id = (XVGraphicsID) new;
	graphics->gc = NULL;
	X3D_init_graphics(graphics->id, graphics->graphics_type);
	X3D_set_X11(graphics->id, XtDisplay(new), None, XtParent(new));
  	X3D_set_colormap(graphics->id, new->core.colormap);

	/*
	 * This is done so that all the memory pointed to by the Coord is
	 * set to zero and purify does not report UMR.  The Coord structure
	 * is padded with 4 bytes on some systems.  Since, we pass structures
	 * to xvgraphics routines a UMR is indicated by purify.
	 */
	kmemset((kaddr) &graphics->wcmin, 0, sizeof(Coord));
	kmemset((kaddr) &graphics->wcmax, 0, sizeof(Coord));
	kmemset((kaddr) &graphics->viewmin, 0, sizeof(Coord));
	kmemset((kaddr) &graphics->viewmax, 0, sizeof(Coord));

	/*
	 *  Get the viewport and world coordinate min and max values.
	 *  Have to do the memsets to pacify purify, and the casts to
	 *  Index on d, because it's an unsigned long.
	 */

        /*
	 * Set world view from parent's world view by default
         */
	if (parent_graphics != NULL)
	{
             SetAttachedWorldView(graphics, parent_graphics);
	     SetWorldView(new);
	}
	else
	{
	    X3D_get_wc_min_max(graphics->id, 
			       &graphics->wcmin, &graphics->wcmax);
	    X3D_get_viewport(graphics->id, 
	                     &graphics->viewmin, &graphics->viewmax);
	    SetWorldView(new);
	}

}

/*-----------------------------------------------------------
|
|  Routine Name: RealizeWidget
|
|       Purpose: This method will set up the GC for the graphics
|                widget.  It then calls the Manager Widget's realize
|		 method to create the window for the graphics widget.
|		 Finally it initialized the xvgraphics library with 
|		 a call to X3D_set_X11. The desired initial window 
|		 attributes are passed in via a mask and list of 
|		 specification attributes. 
|
|         Input: widget     - the widget to realize
|                valuemask  - determines the window attributes being passed in 
|                attributes - the list of window attributes to be set 
|
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void RealizeWidget(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
        XvwGraphicsWidget xwid = (XvwGraphicsWidget) widget;
	XGCValues values;
	unsigned  long mask;


	values.foreground = xwid->graphics.fg.pixel;
	values.background = xwid->graphics.bg.pixel;
	mask = GCForeground | GCBackground;
	xwid->graphics.gc = XtGetGC(widget, mask, &values);

	(*xvwGraphicsWidgetClass->core_class.superclass->core_class.realize)
                (widget, valuemask, winattrib);
	X3D_set_X11(xwid->graphics.id, XtDisplay(widget), None,  widget);

}

/*-----------------------------------------------------------
|
|  Routine Name: RealizeGadget
|
|       Purpose: This method will set up the GC for the graphics
|                gadget, as well as initializing the xvgraphics
|		 library with a call to X3D_set_X11.  By now, we
|		 have determined who we are attached to and we have
|		 a correct world view.  Thus, this method 
|		 will also convert our WC representation for our 
|		 specific subclass into DC's by calling 
|		 GraphicsRecomputePosition so that the gadget
|		 DC's are consistent with it's own newly realized 
|		 world view.
|
|         Input: widget     - the object to realize
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RealizeGadget(
   Widget		widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
	XvwGraphicsGadget xobj = (XvwGraphicsGadget) widget;
	XGCValues values;
	unsigned  long mask;


	values.foreground = xobj->graphics.fg.pixel;
	values.background = xobj->graphics.bg.pixel;
	mask = GCForeground | GCBackground;
	xobj->graphics.gc = XtGetGC(widget, mask, &values);

	X3D_set_X11(xobj->graphics.id,XtDisplay(widget),None,XtParent(widget));
	GraphicsRecomputePosition(widget, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ResizeWidget
|
|       Purpose: This method is used to set the graphics widget
|		 to a new size.  It will also redraw the widget
|		 and all its children at the new size. 
|
|         Input: widget - the widget containing the new position 
|                         and size information
|    Written By: Mark Young
|          Date: June 29, 1992

------------------------------------------------------------*/
/* ARGSUSED */
static void ResizeWidget(
   Widget widget)
{
	XvwGraphicsWidget xwid = (XvwGraphicsWidget) widget;

	int	i;
	Widget	current;


	if (!XtWindow(widget))
	   return;

	for (i = xwid->composite.num_children-1; i >=0; i--)
	{
	    current = xwid->composite.children[i];
	    if (XtIsSubclass(current, xvwGraphicsGadgetClass))
	    {
		/*
		 *  Kludge:  Apr 14, 1995  (MY & DA)
		 *
		 *  The AxisGadget is special, since it had two graphics ids.
		 *  If the Axis is not managed, but realized, then the Resize()
		 *  method will not be called, and the second graphics id not
		 *  updated.  So by managing the child before calling it's
		 *  recompute position, we avoid this problem.
		 */
		if (!XtIsManaged(current) &&
		    XtIsSubclass(current, AxisGadgetClass))
		{
		   XtManageChild(current);
		}

		X3D_set_X11((XVGraphicsID) current, XtDisplay(widget), 
			     None, widget);
	        GraphicsRecomputePosition(current, TRUE);
	    }
	}
	ManagerRequestLayout(widget, FALSE);
	X3D_set_X11((XVGraphicsID) widget, XtDisplay(widget), None, widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: ResizeGadget
|
|       Purpose: This will reset the xvgraphics library for some
|		 undoubtedly very good reason which is not readily
|		 apparent at this time. ;-)
|
|         Input: widget - the object to resize
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ResizeGadget(
   Widget widget)
{
	XvwGraphicsGadget xobj = (XvwGraphicsGadget) widget;

	X3D_set_X11(xobj->graphics.id,XtDisplay(widget),None,XtParent(widget));
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will close the xvgraphics id
|		 and releases the GC, freeing all associated 
|		 memory.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwGraphicsPart *graphics = GetGraphicsPart(widget);

	XtReleaseGC(widget, graphics->gc); 
	X3D_close_graphics(graphics->id);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValuesGadget
|
|       Purpose: This method is used to set the public values
|                of a graphics gadget instance.
|                 
|         Input: current - the gadget containing current settings
|                request - the gadget containing requested settings
|                new     - the gadget processed through all set values methods
|
|       Returns: TRUE if redisplay is required, FALSE otherwise
|    Written By: Mark Young & John Salas & Danielle Argiro & Steve Kubica
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValuesGadget(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwGraphicsGadget      cobj   = (XvwGraphicsGadget) current;
	XvwGraphicsGadget      nobj   = (XvwGraphicsGadget) new;
	XvwGraphicsGadgetClass gclass = (XvwGraphicsGadgetClass)
                                         new->core.widget_class;

	Boolean redisplay = FALSE;


	/*
	 *  Supposing we set our DC's and set our attached widget to
	 *  ourself, then the DC->WC conversion will have been done 
	 *  by now by the Manager, but with our world view inherited
	 *  from whichever widget we were previously attached to.   
	 *
	 *  So, if our DC's have changed, reset our world view to 
	 *  that of our new attached widget, reset our world view 
	 *  based on its world view, and call our change_geometry to
	 *  redo the DC->WC conversion based on our now-correct world 
	 *  view.  Voila!  (Pay no attention to the man behind the curtain.)
	 */
	 if ((new->core.x      != current->core.x)     ||
	     (new->core.y      != current->core.y)     ||
	     (new->core.width  != current->core.width) ||
	     (new->core.height != current->core.height))
	 {
             SetAttachedWidget(current, new, FALSE);
	     SetWorldView(new);
	     if (gclass->manager_class.change_geometry != NULL)
	         gclass->manager_class.change_geometry(new);
	 }


	/*
	 *  If SetValues() returns TRUE, then we need to reset the world
	 *  view, propogate changes to attached siblings, recompute our
	 *  DC's from our updated WC's.  And while we're at it, indicate
	 *  that we should be redisplayed.
	 */
	if (SetValues(&cobj->graphics, &nobj->graphics))
	{
	    SetWorldView(new);
	    PropagateAttachedWorldView(XtParent(new), new, current);
	    GraphicsRecomputePosition(new, FALSE);
	    redisplay = TRUE;
	}

	/*
	 *  If the XVW_FOREGROUND or XVW_BACKGROUND values have 
	 *  changed, then get the xcolors for the new foreground 
	 *  and background.
	 */
	if (cobj->manager.foreground != nobj->manager.foreground ||
	    cobj->manager.background != nobj->manager.background)
	{
	   Colormap colormap = new->core.parent->core.colormap;
	   nobj->graphics.fg.pixel = nobj->manager.foreground;
	   nobj->graphics.bg.pixel = nobj->manager.background;

	   XQueryColor(XtDisplay(new), colormap, &nobj->graphics.fg);
	   XQueryColor(XtDisplay(new), colormap, &nobj->graphics.bg);

	   if (nobj->graphics.gc)
	   {
	      XGCValues values;
              unsigned  long mask;

	      XtReleaseGC(new, nobj->graphics.gc);
              values.foreground = nobj->graphics.fg.pixel;
              values.background = nobj->graphics.bg.pixel;
              mask = GCForeground | GCBackground;
	      nobj->graphics.gc = XtGetGC(new, mask, &values);
	   }
	   redisplay = TRUE;
	}

	/*
	 * check if attached widget changed
	 */
	if (cobj->graphics.attach != nobj->graphics.attach)
	{
	    SetAttachedWidget(current, new, TRUE);
	    redisplay = TRUE;
	}

	/*
	 *  Only redisplay if we are actually mapped.
	 */
	return(redisplay && nobj->manager.mapped);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValuesWidget
|
|       Purpose: This method is used to set the public values
|                of a graphics widget instance.  
|                 
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|
|       Returns: TRUE if redisplay is required, FALSE otherwise
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValuesWidget (
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwGraphicsWidget  nxwid  = (XvwGraphicsWidget) new;
	XvwGraphicsWidget  cxwid  = (XvwGraphicsWidget) current;

	Boolean redisplay = FALSE;


	/* 
	 *  If SetValues() returns TRUE, then we need to reset the
	 *  world view, propogate changes to attached siblings.  And
	 *  while we're at it, indicate that we should be redisplayed.
	 */
        if (SetValues(&cxwid->graphics, &nxwid->graphics))
        {
           SetWorldView(new); 
	   PropagateAttachedWorldView(new, new, current);
        }

	/*
	 *  If the XVW_FOREGROUND or XVW_BACKGROUND values have 
	 *  changed, then get the xcolors for the new foreground 
	 *  and background.
	 */
	if (cxwid->core.background_pixel    != nxwid->core.background_pixel ||
	    cxwid->manager.foreground_pixel != nxwid->manager.foreground_pixel)
	{
	   Colormap colormap = new->core.colormap;
	   nxwid->graphics.bg.pixel = nxwid->core.background_pixel;
	   nxwid->graphics.fg.pixel = nxwid->manager.foreground_pixel;

	   XQueryColor(XtDisplay(new), colormap, &nxwid->graphics.fg);
	   XQueryColor(XtDisplay(new), colormap, &nxwid->graphics.bg);
	   redisplay = TRUE;
	}

	/*
	 * check if attached widget changed
	 */
	if (cxwid->graphics.attach != nxwid->graphics.attach)
	{
	    SetAttachedWidget(current, new, TRUE);
	    redisplay = TRUE;
	}

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the values of public
|		 attributes, which are stored in the graphics part for
|		 both graphics widgets and graphics gadgets.  This 
|		 method is used by both the graphics widget and 
|		 graphics gadget SetValues code.
|                 
|         Input: cpart - the graphicsPart containing current settings
|                npart - the graphicsPart containing new settings
|
|       Returns: TRUE if redisplay is required, FALSE otherwise
|    Written By: Mark Young & John Salas
|          Date: October 23, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   XvwGraphicsPart *cpart,
   XvwGraphicsPart *npart)
{
	Boolean	   redisplay = FALSE;

	/*
	 * If anything has changed which required redrawing, 
	 * indicate that we need to redray.
	 */
	if (cpart->projection   != npart->projection    ||
	    cpart->line_type    != npart->line_type     ||
	    cpart->marker_type  != npart->marker_type   ||
	    cpart->marker_scale != npart->marker_scale  ||
	    cpart->line_width   != npart->line_width    ||
	    cpart->filled       != npart->filled        ||
	    cpart->clipping     != npart->clipping      ||
            cpart->depth_clipping != npart->depth_clipping)
	{
	   redisplay = TRUE;
	}

        /*
         * See if our world view has changed.  Verify that we are setting
	 * world coordinates that are valid with the axis mode and
	 * indicate that we want to redraw.  
         */
        if (!GraphicsCheckCoord(cpart->wcmin, npart->wcmin)     ||
            !GraphicsCheckCoord(cpart->wcmax, npart->wcmax)     ||
            !GraphicsCheckCoord(cpart->viewmin, npart->viewmin) ||
            !GraphicsCheckCoord(cpart->viewmax, npart->viewmax) ||
	    cpart->proportional != npart->proportional          ||
	    cpart->axis_mode_x 	!= npart->axis_mode_x           ||
	    cpart->axis_mode_y 	!= npart->axis_mode_y           ||
	    cpart->axis_mode_z 	!= npart->axis_mode_z )
        {
	   CheckAxisModeValidity(npart, cpart);
	   redisplay = TRUE;
	}

        /*
         * If we have a 3D world view, and we've changed our viewing
	 * position, indicate that we need to redraw.
         */
	if (npart->graphics_type == KGRAPHICS_3D &&
	    (cpart->alpha	   != npart->alpha ||
	     cpart->theta	   != npart->theta ||
	     cpart->gamma	   != npart->gamma ||
	     cpart->eye_distance  != npart->eye_distance ||
	     cpart->view_distance != npart->view_distance))
	{
	   redisplay = TRUE;
	}

	if (npart->attach != cpart->attach)
	{
	   redisplay = TRUE;
	}

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Recalculate the device coordinates of the 
|		 graphics object according to the new world 
|		 coordinate maxes and mins.
|
|         Input: widget - the object to calculate the new position for
|    Written By: Mark Young
|          Date: April 14, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputePosition(
   Widget widget)
{
	XvwGraphicsGadget xobj = (XvwGraphicsGadget) widget;

	Coord pt;
	Real xpos1, xpos2, ypos1, ypos2, x, y;
	XVGraphicsID id = xobj->graphics.id;


	if (xobj->manager.realized == FALSE)
	   return;

	if (xobj->graphics.graphics_type == KGRAPHICS_2D)
	{
	   X2D_convert_point_wc_to_dc(id,xobj->graphics.wcmin, &xpos1, &ypos1);
	   X2D_convert_point_wc_to_dc(id,xobj->graphics.wcmax, &xpos2, &ypos2);
	}
	else
	{
	   pt.x = xobj->graphics.wcmin.x;
	   pt.y = xobj->graphics.wcmin.y;
	   pt.z = xobj->graphics.wcmin.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = xpos2 = x; ypos1 = ypos2 = y;

	   pt.x = xobj->graphics.wcmax.x;
	   pt.y = xobj->graphics.wcmin.y;
	   pt.z = xobj->graphics.wcmin.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);

	   pt.x = xobj->graphics.wcmin.x;
	   pt.y = xobj->graphics.wcmax.y;
	   pt.z = xobj->graphics.wcmin.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);

	   pt.x = xobj->graphics.wcmin.x;
	   pt.y = xobj->graphics.wcmin.y;
	   pt.z = xobj->graphics.wcmax.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);

	   pt.x = xobj->graphics.wcmin.x;
	   pt.y = xobj->graphics.wcmax.y;
	   pt.z = xobj->graphics.wcmax.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);

	   pt.x = xobj->graphics.wcmax.x;
	   pt.y = xobj->graphics.wcmax.y;
	   pt.z = xobj->graphics.wcmin.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);

	   pt.x = xobj->graphics.wcmax.x;
	   pt.y = xobj->graphics.wcmax.y;
	   pt.z = xobj->graphics.wcmax.z;
	   X3D_convert_point_wc_to_dc(id, pt, &x, &y);
	   xpos1 = kmin(xpos1, x); xpos2 = kmax(xpos2, x);
	   ypos1 = kmin(ypos1, y); ypos2 = kmax(ypos2, y);
	}

	widget->core.x 	    = (Position) kmin(xpos1, xpos2);
	widget->core.y 	    = (Position) kmin(ypos1, ypos2);
	widget->core.width  = (Dimension) kabs(xpos2 - xpos1) + 1;
	widget->core.height = (Dimension) kabs(ypos2 - ypos1) + 1;
}

/*-----------------------------------------------------------
|
|  Routine Name: EraseSelection
|
|       Purpose: Erase the corner markers which had indicated that
|                the gadget had been selected.
|
|         Input: widget - the object whose corner markers are being erased
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: December 28, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void EraseSelection(
    Widget widget)
{

        Window  window;
        Display *display;
	int buffer;

	Position corner1_x, corner1_y, corner2_x, corner2_y;
	Position corner3_x, corner3_y, corner4_x, corner4_y;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

	/* 
	 *   buffer of one pixel between the visual object and the
	 *   corner markers.
	 */
	buffer = 1;

        corner1_x = widget->core.x - buffer - FAT;
        corner1_y = widget->core.y - buffer - FAT;
        corner2_x = widget->core.x + widget->core.width + 2*buffer;
        corner2_y = widget->core.y - buffer - FAT;
        corner3_x = widget->core.x + widget->core.width + 2*buffer;
        corner3_y = widget->core.y + widget->core.height + 2*buffer;
        corner4_x = widget->core.x - buffer - FAT;
        corner4_y = widget->core.y + widget->core.height + 2*buffer;

	/*   __
	 *  |    upper left
	 */
        XClearArea(display, window, corner1_x, corner1_y, 
		   FAT, LENGTH, TRUE);
        XClearArea(display, window, corner1_x, corner1_y, 
		   LENGTH, FAT, TRUE);

	/*   __
	 *     | upper right
	 */
        XClearArea(display, window, corner2_x - LENGTH + FAT, corner2_y, 
		   LENGTH, FAT, TRUE);
        XClearArea(display, window, corner2_x, corner2_y, 
		   FAT, LENGTH, TRUE);

	/* 
	 *   __| lower right
	 */
        XClearArea(display, window, corner3_x, corner3_y - LENGTH + FAT, 
		   FAT, LENGTH, TRUE);
        XClearArea(display, window, corner3_x - LENGTH + FAT, corner3_y, 
		   LENGTH, FAT, TRUE);

	/* 
	 *  |__  lower 
	 */
        XClearArea(display, window, corner4_x, corner4_y -LENGTH + FAT, 
		   FAT, LENGTH, TRUE);
        XClearArea(display, window, corner4_x, corner4_y, 
		   LENGTH, FAT, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelectionWidget
|
|       Purpose: Front end to RefreshSelection for widgets.
|
|         Input: widget - the widget whose corner markers we are refreshing
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: December 28, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RefreshSelectionWidget(
    Widget widget)
{
	XvwGraphicsWidget xwid = (XvwGraphicsWidget) widget;

	RefreshSelection(widget, xwid->manager.foreground_gc);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelectionGadget
|
|       Purpose: Front end to RefreshSelection for gadgets.
|
|         Input: widget - the gadgets whose corner markers we are refreshing
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: December 28, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RefreshSelectionGadget(
    Widget gadget)
{
	XvwGraphicsGadget xobj = (XvwGraphicsGadget) gadget;

	RefreshSelection(gadget, xobj->manager.foreground_gc);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelection
|
|       Purpose: Draws little corner markers at the corners of 
|		 the object to indicate that it has been selected.
|
|         Input: widget - the visual object whose selection we are refreshing
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: December 28, 1995 
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void RefreshSelection(
    Widget widget,
    GC     gc)
{
        Window    window;
        Display   *display;
	int       buffer; 

	Position corner1_x, corner1_y, corner2_x, corner2_y;
	Position corner3_x, corner3_y, corner4_x, corner4_y;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

	/* 
	 *   buffer of one pixel between the visual object and the
	 *   corner markers.
	 */
	buffer = 1;

	corner1_x = widget->core.x - buffer - FAT;
	corner1_y = widget->core.y - buffer - FAT;
	corner2_x = widget->core.x + widget->core.width + 2*buffer;
	corner2_y = widget->core.y - buffer - FAT;
	corner3_x = widget->core.x + widget->core.width + 2*buffer;
        corner3_y = widget->core.y + widget->core.height + 2*buffer;
	corner4_x = widget->core.x - buffer - FAT;
        corner4_y = widget->core.y + widget->core.height + 2*buffer;

	/*   __
	 *  |    upper left
	 */
        XFillRectangle(display, window, gc, 
		       corner1_x, corner1_y, FAT, LENGTH); 
        XFillRectangle(display, window, gc,
		       corner1_x, corner1_y, LENGTH, FAT ); 

	/*   __
	 *     | upper right
	 */
        XFillRectangle(display, window, gc,
		       corner2_x - LENGTH + FAT, corner2_y, LENGTH, FAT); 
        XFillRectangle(display, window, gc,
		       corner2_x, corner2_y, FAT, LENGTH); 

	/* 
	 *   __| lower right
	 */
        XFillRectangle(display, window, gc,
                       corner3_x, corner3_y - LENGTH + FAT, FAT, LENGTH);
        XFillRectangle(display, window, gc,
                       corner3_x - LENGTH + FAT, corner3_y, LENGTH, FAT);

	/* 
	 *  |__  lower 
	 */
        XFillRectangle(display, window, gc,
                       corner4_x, corner4_y -LENGTH + FAT, FAT, LENGTH);
        XFillRectangle(display, window, gc,
                       corner4_x, corner4_y, LENGTH, FAT);
}


/*-----------------------------------------------------------
|
|  Routine Name: DefaultAreaAttach
|		 
|       Purpose: This utility routine will initialize the 
|		 attached widget for a graphics widget to 
|		 its parent.  This will not work for graphics
|		 gadgets, hence we need to initialize it 
|		 explicitly in the Initialize method as well.
|
|    Written By: Mark Young, John Salas, & Steve Kubica
|          Date: December 21, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void DefaultAreaAttach(
    Widget      widget,
    int         offset,
    XrmValue    *value)
{
        value->addr = (kaddr) &widget->core.parent;
}

