/*
 * Khoros: $Id: ImageRoi.c,v 1.16 1996/05/18 14:54:17 mirage Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ImageRoi.c,v 1.16 1996/05/18 14:54:17 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Image Roi Routines
   >>>>		
   >>>>  Private:
   >>>>                 ImageGetRoi()
   >>>>                 ImageSetRoi()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/ImageP.h>

/*-----------------------------------------------------------
|
|  Routine Name: ImageGetRoi
|
|       Purpose: ImageGetRoi is used to get a region of interest according
|		 to the displayed image.
|
|         Input: object    - the object in which we will be getting the roi
|		 attribute - the attribute name XVW_IMAGE_ROI
|        Output: calldata  - pointer to retrieve the roi
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Dec 18, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
int ImageGetRoi(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwImageWidget xwid = (XvwImageWidget) xvw_widget(object);
	kobject *roi  = (kobject *) calldata;

	kaddr   data;
	int     erase, shape, fill, policy, multiple;
	kobject iobject = xwid->image.iobject;
	int     x, y, w, h, d, t, e, xoffset, yoffset, type, num = 1;
	int     reload, update, scale, initialized = FALSE;

	XEvent   event;
        Window   window;

	policy   = xwid->image.roi_policy;
	multiple = xwid->image.multiple_roi;

	/*
	 *  Get the ROI extent that we need in which to extract the data set
	 */
	kpds_get_attribute(iobject, KPDS_VALUE_SIZE, &w, &h, &d, &t, &e);
	*roi = kpds_create_object();
	if ((multiple == TRUE) || (policy == KIMAGE_ROI_OUTSIDE))
	{
	   if (!kpds_query_mask(iobject))
	   {
	      kpds_create_mask(*roi);
	      kpds_set_attributes(*roi,
			KPDS_MASK_DATA_TYPE, KUBYTE,
	      		KPDS_MASK_SIZE, w, h, d, t, e,
			NULL);
	      if (policy == KIMAGE_ROI_OUTSIDE)
		 kpds_initialize_mask(*roi, 1.0);
	      else kpds_initialize_mask(*roi, 0.0);
	   }
	   else
	      kpds_copy_mask_data(iobject, *roi, FALSE);
	}

	/*
	 *  Initialize ROI values
	 */
	window = xvw_window(object);
	do
	{
	   erase = (multiple == FALSE);
	   fill = (policy != KIMAGE_ROI_OUTLINE); 
	   shape = xwid->image.roi_shape;
	   if (!xvw_extract_roi(object, *roi, num, fill, shape, 
				erase, multiple, policy, d, t, e))
		break;
	   num++;
           initialized = TRUE;   
	   if (!xwid->image.multiple_roi) break;

	   /* 
	    * The following is a kludge that delays calling the xvw_extract_roi
            * routine until the user starts the actual extraction process by
            * doing a button press.  Without this code, if the user changes
	    * the ROI shape during a multiple ROI extraction, the extraction
	    * will be "one behind" -- that is, if they change from rectangle
	    * to circle, it'll extract one more rectangle before changing to
	    * circle, since without this code, we will already be in the process
	    * of extraction by the time the shape is changed on the menuform
	    */
           while (1)
           {
              XtAppNextEvent(xvw_appcontext(NULL), &event);
              if (event.type == ButtonPress && event.xany.window == window)
                 break;

              XtDispatchEvent(&event);
           }

	} while (event.xbutton.button != Button3);

	xvw_refresh(object);
	if (!initialized)
	{
           kpds_close_object(*roi); *roi = NULL;
           return(FALSE);
	}

	xoffset = xwid->image.xoffset;
	yoffset = xwid->image.yoffset;
	kpds_get_attribute(iobject, KPDS_VALUE_DATA_TYPE, &type);
	kpds_create_value(*roi);


	if ((multiple == FALSE) && (policy != KIMAGE_ROI_OUTSIDE))
	{
	    kpds_get_attribute(*roi, KPDS_SUBOBJECT_POSITION, 
			       &x, &y, &d, &t, &e);

	    /* copy over all global attributes */
	    kpds_copy_global_attr(iobject, *roi); 
	    kpds_set_attribute(*roi, KPDS_SUBOBJECT_POSITION,
			       x + xoffset, y + yoffset, d, t, e);
	}
	else
	{
	    x = 0; y = 0;
	    /* copy over all global attributes */
	    kpds_copy_global_attr(iobject, *roi); 
	}

	/* 
	 * want the resulting ROI to have width & height set from mask,
         * but need to preserve the depth, time, and elements dimensions
	 * from the value segment of the original kobject
         */
	  
	kpds_get_attribute(*roi, KPDS_MASK_SIZE, &w, &h, NULL, NULL, NULL);
	kpds_get_attribute(iobject, KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e);

	kpds_set_attributes(*roi,
		KPDS_VALUE_DATA_TYPE, type,
		KPDS_VALUE_SIZE, w, h, d, t, e,
		NULL);

	kpds_set_attributes(iobject,
		KPDS_VALUE_POSITION, x+xoffset, y+yoffset, 0, 0, 0,
		KPDS_VALUE_REGION_SIZE, w, h, d, t, e,
		NULL);
		
	/*
	 *  Get the data from the original image object, and then put it
	 *  into the newly create image object.
	 */
	xvw_get_type(iobject, &reload, &update);
	xvw_set_type(iobject, reload, IMAGE_ROI_UPDATE);
	
	kpds_get_attribute(iobject, KPDS_VALUE_SCALING, &scale);
	kpds_set_attribute(iobject, KPDS_VALUE_SCALING, KNONE);
	kpds_set_attribute(*roi, KPDS_VALUE_SCALING, KNONE);
        if ((data = kpds_get_data(iobject, KPDS_VALUE_REGION, NULL)) == NULL)
	{
	   kpds_close_object(*roi); *roi = NULL;
	   kpds_set_attribute(iobject, KPDS_VALUE_SCALING, scale);
           return(FALSE);
	}
	xvw_set_type(iobject, reload, update);
	kpds_put_data(*roi, KPDS_VALUE_ALL, data); kfree_and_NULL(data);

	kpds_copy_remaining_data(iobject, *roi);

	if (kpds_query_map(iobject)  && !(xvw_is_rgb(iobject)))
	{
           xvw_set_attribute(object, XVW_COLOR_CURRENTMAP, *roi);
	}
	kpds_set_attribute(iobject, KPDS_VALUE_SCALING, scale);
	kpds_set_attribute(*roi, KPDS_VALUE_SCALING, scale);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ImageSetRoi
|
|       Purpose: ImageSetRoi is used to set the region of interest according
|		 to the displayed image.
|
|         Input: object    - the object in which we will be setting the roi
|		 attribute - the attribute name XVW_IMAGE_ROI
|        Output: calldata  - pointer to set the roi
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Dec 31, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
int ImageSetRoi(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	return(TRUE);
}
