/*
  * Khoros: $Id: Line.c,v 1.14 1996/06/25 16:33:38 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Line.c,v 1.14 1996/06/25 16:33:38 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Line Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 ExtentToEndPoints()
   >>>>                 PickGadget()
   >>>>                 EraseSelection()
   >>>>                 RefreshSelection()
   >>>>   Public:
   >>>>                 xvw_create_line()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/LineP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ExtentToEndPoints    PROTO((Widget, Position *, Position *, 
					Position *, Position *));
static void ChangeGeometry	 PROTO((Widget));
static int  PickGadget		 PROTO((Widget, int, int));
static void EraseSelection	 PROTO((Widget));
static void RefreshSelection	 PROTO((Widget));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_LINE_XBEGIN,    NULL,    XtRDouble,    NULL},
{XVW_LINE_YBEGIN,    NULL,    XtRDouble,    NULL},
{XVW_LINE_XEND,      NULL,    XtRDouble,    NULL},
{XVW_LINE_YEND,      NULL,    XtRDouble,    NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwLineGadgetRec, line.field)

static XtResource resources[] = { 
{XVW_LINE_XBEGIN, NULL, XtRDouble, sizeof(double), 
      offset(begin.x), XtRString, (XtPointer) "0.0"},
{XVW_LINE_YBEGIN, NULL, XtRDouble, sizeof(double), 
      offset(begin.y), XtRString, (XtPointer) "0.0"},
{XVW_LINE_XEND, NULL, XtRDouble, sizeof(double), 
      offset(end.x), XtRString, (XtPointer) "1.0"},
{XVW_LINE_YEND, NULL, XtRDouble, sizeof(double), 
      offset(end.y), XtRString, (XtPointer) "1.0"},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwLineGadgetClassRec xvwLineGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Line",				/* class_name		  */
    sizeof(XvwLineGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    EraseSelection,                     /* erase selected proc    */
    RefreshSelection,	                /* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwLineGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwLineGadgetClass for public consumption
 */
WidgetClass xvwLineGadgetClass = (WidgetClass) &xvwLineGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of line object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
	xvw_init_attributes(xvwLineGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Line.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Line");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values
|                for all the fields for a line gadget instance.
|
|         Input: request - widget instance with requested public settings
|                new     - widget instance after initialization, with
|                          private settings initialized and public
|                          settings verified
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 23, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwLineGadget xobj = (XvwLineGadget) new;

        xobj->graphics.old_x      = new->core.x;
        xobj->graphics.old_y      = new->core.y;
        xobj->graphics.old_width  = new->core.width;
        xobj->graphics.old_height = new->core.height;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the line object in response
|                to an expose event. 
|
|         Input: widget - the line object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;


	X2D_draw_line(xobj->graphics.id, xobj->line.begin, xobj->line.end,
			&xobj->graphics.fg);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or postion of the line has changed, 
|		 recalculate the size and position of the line.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwLineGadget cobj = (XvwLineGadget) current;
	XvwLineGadget nobj = (XvwLineGadget) new;


	if (cobj->line.begin.x != nobj->line.begin.x ||
	    cobj->line.begin.y != nobj->line.begin.y ||
	    cobj->line.end.x   != nobj->line.end.x   ||
	    cobj->line.end.y   != nobj->line.end.y)
	{
	   GraphicsRecomputePosition(new, FALSE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new world coordinate line begin 
|		 and end point, recompute the new device 
|		 coordinate position of the line. 
|
|         Input: widget - the line object 
|
|        Output: None directly 
|
|    Written By: Mark Young
|          Date: Jun 29, 1992 10:35
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputePosition(
   Widget widget)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;

	XVGraphicsID id = xobj->graphics.id;
	Real xpos1, xpos2, ypos1, ypos2;
        int  linewidth = (int) xobj->graphics.line_width; 
	
	/*-- make linewidth significant dammit! --*/
	if (linewidth == 0)
	   linewidth = 1;

	X2D_convert_point_wc_to_dc(id, xobj->line.begin, &xpos1, &ypos1);
	X2D_convert_point_wc_to_dc(id, xobj->line.end, &xpos2, &ypos2);

        xobj->graphics.old_x      = widget->core.x;
        xobj->graphics.old_y      = widget->core.y;
        xobj->graphics.old_width  = widget->core.width;
        xobj->graphics.old_height = widget->core.height;

	widget->core.x = (Position) kmin(xpos1, xpos2);
	widget->core.y = (Position) kmin(ypos1, ypos2);

	if (kabs(xpos2 - xpos1) <= 0)
	    widget->core.width = linewidth;
	else widget->core.width  = (Dimension) kabs(xpos2 - xpos1);

	if (kabs(ypos2 - ypos1) <= 0)
	    widget->core.height = linewidth;
	else widget->core.height = (Dimension) kabs(ypos2 - ypos1);

}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a new device coordinate description of the
|                line, recompute the world coordinate begin and end 
|                point.
|
|         Input: widget - the line object
|
|        Output: None directly 
|
|    Written By: Mark Young
|          Date: Jun 29, 1992 10:35
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeGeometry(
   Widget widget)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;

	XVGraphicsID id = xobj->graphics.id;
	Position xpos1, xpos2, ypos1, ypos2;

	
	ExtentToEndPoints(widget, &xpos1, &xpos2, &ypos1, &ypos2);

	X2D_convert_point_dc_to_wc(id, (Real) xpos1, (Real) ypos1, 
				   &xobj->line.begin);
	X2D_convert_point_dc_to_wc(id, (Real) xpos2, (Real) ypos2, 
				   &xobj->line.end);
}

/*-----------------------------------------------------------
|
|  Routine Name: ExtentToEndPoints
|
|       Purpose: Given the location and extent of a line, determine  
|                the device coordinates of the begin and end points
|                of the lines.  Complexity simply because we have to
|		 preserve the positions of begin and end points.
|
|         Input: widget - the line object
|
|	 Output: xpos1  - the x coordinate to be used for begin.x 
|	         xpos2  - the x coordinate to be used for end.x
|	         ypos1  - the y coordinate to be used for begin.y
|	         ypos2  - the y coordinate to be used for end.y
|
|    Written By: Steve Kubica & Danielle Argiro 
|          Date: Dec 29, 1995 
| Modifications:
|
------------------------------------------------------------*/
static void ExtentToEndPoints(
   Widget    widget, 
   Position *xpos1,
   Position *xpos2,
   Position *ypos1,
   Position *ypos2)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;
        int  linewidth = (int) xobj->graphics.line_width; 
	int  upper_left_x, upper_right_x, lower_left_x, lower_right_x;
	int  upper_left_y, upper_right_y, lower_left_y, lower_right_y;

	/*-- make linewidth significant dammit! --*/
	if (linewidth == 0)
	   linewidth = 1;

	/* no case currently where min > max, but just being safe for later */
	if (xobj->graphics.wcmin.x < xobj->graphics.wcmax.x)
	{
	   upper_left_x  = widget->core.x;

           if ( widget->core.width == 1 )
	      upper_right_x = widget->core.x;
	   else
	      upper_right_x = widget->core.x + widget->core.width;

	   lower_left_x  = widget->core.x;

           if ( widget->core.width == 1 )
	      lower_right_x = widget->core.x;
	   else
	      lower_right_x = widget->core.x + widget->core.width;
	}
	else
	{
           if ( widget->core.width == 1 )
              upper_left_x  = widget->core.x;
           else
              upper_left_x  = widget->core.x + widget->core.width;

	   upper_right_x = widget->core.x;

           if ( widget->core.width == 1 )
              lower_left_x  = widget->core.x;
           else
              lower_left_x  = widget->core.x + widget->core.width;

           lower_right_x = widget->core.x;
	}

	/* area objects, managers have min < max; images have min > max */
	if (xobj->graphics.wcmin.y < xobj->graphics.wcmax.y)
        {
	   upper_left_y  = widget->core.y;

           if ( widget->core.height == 1 )
	      lower_left_y  = widget->core.y;
	   else
	      lower_left_y  = widget->core.y + widget->core.height;

	   upper_right_y = widget->core.y;

           if ( widget->core.height == 1 )
	      lower_right_y = widget->core.y;
	   else
	      lower_right_y = widget->core.y + widget->core.height;
        }
        else
        {
           if ( widget->core.height == 1 )
	      upper_left_y  = widget->core.y;
	   else
	      upper_left_y  = widget->core.y + widget->core.height;

	   lower_left_y  = widget->core.y;

           if ( widget->core.height == 1 )
	      upper_right_y = widget->core.y;
	   else
	      upper_right_y = widget->core.y + widget->core.height;

	   lower_right_y = widget->core.y;
        }

        /*--  begin "/" end --*/
        if (xobj->line.begin.x <= xobj->line.end.x &&
            xobj->line.begin.y <= xobj->line.end.y)
        {
               *xpos1 = lower_left_x;
               *ypos1 = lower_left_y;
               *xpos2 = upper_right_x;
               *ypos2 = upper_right_y;
	}

        /*--   begin "\" end  -- */
        else if (xobj->line.begin.x <= xobj->line.end.x &&
                 xobj->line.begin.y >= xobj->line.end.y)
        {
               *xpos1 = upper_left_x;
               *ypos1 = upper_left_y;
               *xpos2 = lower_right_x;
               *ypos2 = lower_right_y;
        }

        /*--  end "\" begin --*/
        else if (xobj->line.begin.x >= xobj->line.end.x &&
                 xobj->line.begin.y <= xobj->line.end.y)
        {
               *xpos1 = lower_right_x;
               *ypos1 = lower_right_y;
               *xpos2 = upper_left_x;
               *ypos2 = upper_left_y;
        }

        /*--  end "/" begin --*/
        else if (xobj->line.begin.x >= xobj->line.end.x &&
                 xobj->line.begin.y >= xobj->line.end.y)
        {
               *xpos1 = upper_right_x;
               *ypos1 = upper_right_y;
               *xpos2 = lower_left_x;
               *ypos2 = lower_left_y;
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a line object and an attempted picking location,
|                see if the line has been picked. 
|
|         Input: widget - the line object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|
|        Output: None 
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise|
|    Written By: Mark Young
|          Date: Jun 29, 1992 10:35
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;

	Position boundy, deltax, deltay, rise, run, tolerance;
	Position xpos1, xpos2, ypos1, ypos2;

	ExtentToEndPoints(widget, &xpos1, &xpos2, &ypos1, &ypos2);

	tolerance = PICK_TOLERANCE*2;

	/*
	 *  line was picked on one of the end points 
	 */
	if (((kabs(x - xpos1) <= tolerance) && (kabs(y - ypos1) <= tolerance))||
	    ((kabs(x - xpos2) <= tolerance) && (kabs(y - ypos2) <= tolerance)))
	{
	   xobj->graphics.pick_status = PICK_RESIZE;
	   return(TRUE);
	}

	/*
	 *  otherwise, see if we're on the line within tolerance 
	 */

	xobj->graphics.pick_status = PICK_MOVE;

	rise = ypos2 - ypos1;
	run  = xpos2 - xpos1;

	/*-- vertical line --*/
	if (run == 0)
	{
           if (kabs(x - xpos1) <= tolerance) 
	      return(TRUE);
	   else
	      return(FALSE);
	}

        /*-- horizontal line --*/
        if (rise == 0)
        {
           if (kabs(y - ypos1) <= tolerance) 
              return(TRUE);
           else
              return(FALSE);
	}

	deltax = x - xpos1;
	deltay = y - ypos1;

	if (kabs(run) <= tolerance)
	{
	   if (kabs(deltax) <= tolerance)
	      return(TRUE);
	}
	else
	{
	   boundy = (rise * deltax) / run;
	   if (kabs(deltay - boundy) <= tolerance)
	      return(TRUE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: EraseSelection
|
|       Purpose: Erase the boxes which had indicated that 
|                the line had been selected 
|
|         Input: widget - the line object whose selection we are erasing 
|
|    Written By: Mark Young & Steve Kubica & Danielle Argiro
|          Date: Dec 29, 1995
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void EraseSelection(
   Widget widget)
{
        XvwLineGadget xobj = (XvwLineGadget) widget;

        Window  window;
        Display *display;

        Position tolerance, xpos1, xpos2, ypos1, ypos2;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

        tolerance = kmax(PICK_TOLERANCE, xobj->graphics.line_width);

        ExtentToEndPoints(widget, &xpos1, &xpos2, &ypos1, &ypos2);

        XClearArea(display, window,
                       (int) (xpos1 - tolerance/2),
                       (int) (ypos1 - tolerance/2),
                       (unsigned int) tolerance,
                       (unsigned int) tolerance, TRUE);
        XClearArea(display, window,
                       (int) (xpos2 - tolerance/2),
                       (int) (ypos2 - tolerance/2),
                       (unsigned int) tolerance,
                       (unsigned int) tolerance, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshSelection
|
|       Purpose: Draw little boxes at the ends of the line to
|                indicate that the line had been selected 
|
|         Input: widget - the line object whose selection we are refreshing
|
|    Written By: Mark Young & Steve Kubica & Danielle Argiro
|          Date: Dec 29, 1995 
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void RefreshSelection(
   Widget widget)
{
	XvwLineGadget xobj = (XvwLineGadget) widget;

        Window  window;
        Display *display;

        Position tolerance, xpos1, xpos2, ypos1, ypos2;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

	tolerance = kmax(PICK_TOLERANCE, xobj->graphics.line_width);

	ExtentToEndPoints(widget, &xpos1, &xpos2, &ypos1, &ypos2);

        XFillRectangle(display, window, xobj->manager.foreground_gc,
                       (int) (xpos1 - tolerance/2),
                       (int) (ypos1 - tolerance/2),
                       (unsigned int) tolerance,
                       (unsigned int) tolerance);
        XFillRectangle(display, window, xobj->manager.foreground_gc,
                       (int) (xpos2 - tolerance/2),
                       (int) (ypos2 - tolerance/2),
                       (unsigned int) tolerance,
                       (unsigned int) tolerance);
}


/************************************************************
*
*  Routine Name: xvw_create_line - create a line object.
*
*       Purpose: A line visual object supports the display of a line.
*
*                The (x, y) location of the two end points of the
*                line may be specified in world coordinates, where the
*                world coordinate range is dictated by the "controlling" visual
*                object. By default, the parent of the line object is its
*                controlling visual object; this may be changed using the
*                XVW_GRAPHICS_ATTACH attribute.
*
*                While not recommended, the size and location of the line may be
*                specified using device coordinates. When device coordinates
*                are used, they are with respect to the bounding box surrounding
*                the line; specify the (x,y) location of the upper left hand
*                corner of the bounding box, as well as the width and height
*                of the bounding box.  Due to lack of directional information, the
*		 line in this case will be drawn from the upper left hand corner
*		 of the bounding box to the lower right of the bounding box.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the line object
*                         (for use in app-defaults files, etc)
*
*        Output:
*	Returns: The line object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young 
*          Date: Jun 29, 1992 10:31
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_line(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the line object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, LineGadgetClass);
	return(object);
}
