/*
  * Khoros: $Id: Marker.c,v 1.12 1996/04/04 23:29:43 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Marker.c,v 1.12 1996/04/04 23:29:43 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Marker Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RecomputePosition()
   >>>>                 ChangeGeometry()
   >>>>                 PickGadget()
   >>>>   Public:
   >>>>                 xvw_create_marker()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"
#include <xvisual/MarkerP.h>

static void ClassInitialize	 PROTO((void));
static void Initialize           PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static void RecomputePosition	 PROTO((Widget));
static void ChangeGeometry	 PROTO((Widget));
static int PickGadget            PROTO((Widget, int, int));


/*------------------------------------------------------- 
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_MARKER_XPLACEMENT,    NULL,   XtRDouble,   NULL},
{XVW_MARKER_YPLACEMENT,    NULL,   XtRDouble,   NULL},
};

/*------------------------------------------------------- 
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwMarkerGadgetRec, marker.field)

static XtResource resources[] = { 
{XVW_MARKER_XPLACEMENT, NULL, XtRDouble, sizeof(double),
     offset(placement.x), XtRString, (XtPointer) "0.0"},
{XVW_MARKER_YPLACEMENT, NULL, XtRDouble, sizeof(double),
     offset(placement.y), XtRString, (XtPointer) "0.0"},

{XVW_WIDTH, NULL, XtRDimension, sizeof(Dimension),
     XtOffsetOf(XvwMarkerGadgetRec, rectobj.width), XtRImmediate,
     (XtPointer) 15},
{XVW_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
     XtOffsetOf(XvwMarkerGadgetRec, rectobj.height), XtRImmediate,
     (XtPointer) 15},
};
#undef offset


#define SUPERCLASS (&xvwGraphicsGadgetClassRec)

XvwMarkerGadgetClassRec xvwMarkerGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Marker",			        /* class_name		  */
    sizeof(XvwMarkerGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    ChangeGeometry,			/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    RecomputePosition,			/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwMarkerGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwMarkerGadgetClass for public consumption
 */
WidgetClass xvwMarkerGadgetClass = (WidgetClass) &xvwMarkerGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of marker object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwMarkerGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Marker.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Marker");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial values for the marker
|
|         Input: request - widget instance with requested public settings
|                new     - widget instance after initialization, with
|                          private settings initialized and public
|                          settings verified
|
|    Written By: Mark Young and John M. Salas
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwMarkerGadget xobj = (XvwMarkerGadget) new;

        /*
         * This is done so that all the memory pointed to by the Coord is
         * set to zero and purify does not report UMR.  The Coord structure
         * is padded with 4 bytes on some systems.  Since, we pass structures
         * to xvgraphics routines a UMR is indicated by purify.
         */
        kmemset((kaddr) &xobj->marker.placement, 0, sizeof(Coord));
	xobj->marker.placement.d = (unsigned long) KGRAPHICS_UNINITIALIZED;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the marker object in response
|                to an expose event. 
|
|         Input: widget - the marker object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwMarkerGadget xobj = (XvwMarkerGadget) widget;


	X2D_draw_marker(xobj->graphics.id, xobj->marker.placement,
	    	        xobj->graphics.marker_scale, xobj->graphics.marker_type,
		        &xobj->graphics.fg);
}


/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Update if the the postion of the marker or if the
|		 marker type has changed, 
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwMarkerGadget cobj = (XvwMarkerGadget) current;
	XvwMarkerGadget robj = (XvwMarkerGadget) request;
	XvwMarkerGadget nobj = (XvwMarkerGadget) new;


	if (!GraphicsCheckCoord(robj->marker.placement,cobj->marker.placement))
	   GraphicsRecomputePosition(new, FALSE);

	if (nobj->graphics.marker_scale != cobj->graphics.marker_scale)
	   GraphicsRecomputePosition(new, FALSE);

	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputePosition
|
|       Purpose: Given a new world coordinate marker location 
|                recompute the new device coordinate position 
|		 of the marker. 
|
|         Input: widget - the marker object 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputePosition(
   Widget widget)
{
	XvwMarkerGadget xobj = (XvwMarkerGadget) widget;

	Real x, y;
	int  scale     = (int) xobj->graphics.marker_scale;
	int  linewidth = (int) xobj->graphics.line_width;

	X2D_convert_point_wc_to_dc(xobj->graphics.id,
 		                   xobj->marker.placement, &x, &y);

	widget->core.x      = (Position) x - 2*scale - linewidth;
        widget->core.y      = (Position) y - 2*scale - linewidth;
        widget->core.width  = 4*scale + 2*linewidth;
        widget->core.height = 4*scale + 2*linewidth;
	xvw_set_attributes(xvw_object(widget),
			   XVW_PREFERRED_WIDTH,  widget->core.width,
			   XVW_PREFERRED_HEIGHT, widget->core.height,
			   NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeGeometry
|
|       Purpose: Given a new device coordinate location of the
|                marker, recompute the world coordinate location.
|
|         Input: widget - the marker object
|
|        Output: None directly 
|
|    Written By: Mark Young, Danielle Argiro, Steve Kubica
|          Date: Dec 28, 1995
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeGeometry(
   Widget widget)
{
	XvwMarkerGadget xobj = (XvwMarkerGadget) widget;
	Real x, y;

	x = (Real) widget->core.x + (Real) widget->core.width/2.0;
	y = (Real) widget->core.y + (Real) widget->core.height/2.0;

	X2D_convert_point_dc_to_wc(xobj->graphics.id, x, y,
			&xobj->marker.placement);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a marker object and an attempted picking location,
|                see if the marker has been picked. 
|
|         Input: widget - the marker object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise|
|    Written By: John Salas
|          Date: Mar 13, 1993 
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
	return(TRUE);
}


/************************************************************
*
*  Routine Name: xvw_create_marker - create a marker object
*
*       Purpose: A marker visual object supports the display of a marker;
*		 A wide variety of marker types are supported. 
*
*                The (x, y) location of the marker center 
*                may be specified in world coordinates, where the
*                world coordinate range is dictated by the "controlling" visual
*                object. By default, the parent of the marker object is its
*                controlling visual object; this may be changed using the
*                XVW_GRAPHICS_ATTACH attribute.
*
*                Alternatively, the location of the marker may be
*                specified using device coordinates. When device coordinates
*                are used, they are with respect to the bounding box surrounding
*                the marker; specify the (x,y) location of the upper left hand
*                corner of the bounding box, as well as the width and height
*                of the bounding box.
*
*		 Because the size of a marker is automatically determined
*		 according to the scale, width and height should not be 
*		 specified;  instead, to increase the size of the marker,
*		 provide the desired scale value as an integer multiple of 
*		 the original (default) size.
*		 
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the marker
*                         object (for use in app-defaults files, etc)
*
*        Output: 
*	Returns: The marker object on success, NULL on failure
*  Restrictions:
*    Written By: Mark Young 
*          Date: Jun 29, 1992 10:31
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_marker(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the marker object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, MarkerGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,     /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
