/*
  * Khoros: $Id: Palette.c,v 1.22 1996/06/20 22:50:18 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Palette.c,v 1.22 1996/06/20 22:50:18 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Palette Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Realize()
   >>>>                 Destroy()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 GetInfo()
   >>>>                 UpdatePalette()
   >>>>                 ColorReloadCallback()
   >>>>                 RefreshElement()
   >>>>                 ChangePaletteIndex()
   >>>>   Public:
   >>>>                 xvw_create_palette()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/PaletteP.h>


static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static void Realize	    PROTO((Widget, XtValueMask *,
				   XSetWindowAttributes *));
static void Destroy         PROTO((Widget));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));

static void GetInfo PROTO((xvobject, int, int, int, int *, int *, double *));
static void UpdatePalette       PROTO((xvobject, kaddr, XEvent *, int *));
static void ColorReloadCallback PROTO((xvobject, kaddr));
static void RefreshElement      PROTO((xvobject, int, int, int));
static int ChangePaletteIndex   PROTO((xvobject, char *, kaddr));

#undef kwidget
#undef kwidgetclass
#undef kconstraint

#define kwidget(widget)      (XvwPaletteWidget) (widget)
#define kwidgetclass(widget) (XvwPaletteWidgetClass) (widget)->core.widget_class
#define kconstraint(widget)  (XvwPaletteWidgetConstraints) (widget)->core.constraints

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PALETTE_TYPE,      NULL,    XtRInt,      NULL},
{XVW_PALETTE_CALLBACK,  NULL,	   XtRCallback, NULL},
{XVW_PALETTE_INDEXLIST, NULL,    XtRPointer,  NULL},
{XVW_PALETTE_INDEXNUM,  NULL,    XtRInt,      NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPaletteWidgetRec, palette.field)

static XtResource resources[] = { 

{XVW_PALETTE_TYPE, NULL, XtRInt, sizeof(int),
      offset(type), XtRImmediate, (XtPointer) KPALETTE_TYPE_COLORBAR},
{XVW_PALETTE_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
      offset(update_callback), XtRCallback, (XtPointer) NULL},
{XVW_PALETTE_INDEXLIST, NULL, XtRPointer, sizeof(XtPointer),
      offset(indexes), XtRImmediate, (XtPointer) NULL},
{XVW_PALETTE_INDEXNUM, NULL, XtRInt, sizeof(int),
      offset(indexnum), XtRImmediate, (XtPointer) 0},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwPaletteWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_IN},
{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(CoreRec, core.width),
        XtRImmediate, (XtPointer) 300},
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(CoreRec, core.height),
        XtRImmediate, (XtPointer) 150},
};
#undef offset


#define SUPERCLASS (&xvwColorWidgetClassRec)

XvwPaletteWidgetClassRec xvwPaletteWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Palette",				/* class_name		  */
    sizeof(XvwPaletteWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    XtInheritResize,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwPaletteWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPaletteWidgetClass for public consumption
 */
WidgetClass xvwPaletteWidgetClass = (WidgetClass) &xvwPaletteWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of palette widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young 
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPaletteWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Palette.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Palette");

        xvw_define_attributes(xvwPaletteWidgetClass,
          XVW_PALETTE_ADD,     XtRInt, ChangePaletteIndex, NULL,
          XVW_PALETTE_DELETE,  XtRInt, ChangePaletteIndex, NULL,
          XVW_PALETTE_CLEAR,   XtRInt, ChangePaletteIndex, NULL,
          NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial image
|		 for a palette widget instance. 
|
|         Input: request - not used
|		 new	 - widget instance after initialization
|    Written By: Mark Young
|          Date: Oct 23, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPaletteWidget xwid = kwidget(new);

	xwid->palette.select = FALSE;
	xwid->palette.old_indexnum = 0;
	xwid->palette.old_indexes  = NULL;
	xvw_add_event(xvw_object(new), ButtonPressMask|ButtonMotionMask,
			UpdatePalette, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|       Purpose: This method will set up the window attributes 
|                and create a window for the palette widget.
|
|         Input: widget	    - the widget to realize 
|		 valuemask  - determines the window attributes being passed in
|		 attributes - the list of window attributes to be set 
|    Written By: Mark Young
|          Date: Nov 28, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static Pixel white, black;
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
	XvwPaletteWidget xwid = kwidget(widget);

	XColor	     color;
	XtGCMask     mask;
	XGCValues    values;
	Pixmap       stipple;
	static unsigned char pixmap_bits[] = { 0x0f, 0x0b, 0x0e, 0x07 };


	/*
	 *  Create the gc to be used with the refresh mechanism
	 */
        values.line_width = 0;
	mask = GCForeground | GCLineWidth;
	color.flags = DoRed | DoGreen | DoBlue;

        color.red = color.green = color.blue = 0;
        if (XAllocColor(XtDisplay(widget), widget->core.colormap, &color) != 0)
           black = values.foreground = color.pixel;
        xwid->palette.gc1 = XtGetGC(widget, mask, &values);
 
        color.red = color.green = color.blue = 65535;
        if (XAllocColor(XtDisplay(widget), widget->core.colormap, &color) != 0)
           white = values.foreground = color.pixel;
        xwid->palette.gc2 = XtGetGC(widget, mask, &values);

	(*xvwPaletteWidgetClass->core_class.superclass->core_class.realize)
                (widget, valuemask, winattrib);

	stipple = XCreateBitmapFromData(XtDisplay(widget), XtWindow(widget),
		(char *) pixmap_bits, 4, 4);
	XSetStipple(XtDisplay(widget), xwid->graphics.gc, stipple);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Frees memory associated with Palette object 
|
|         Input: widget - the palette widget being destroyed
|    Written By: Danielle Argiro
|          Date: Sept 21, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPaletteWidget xobj = (XvwPaletteWidget) widget;

        karray_free(xobj->palette.indexes, KINT, 
		    (size_t) xobj->palette.indexnum, NULL);
}



/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the area widget in response
|                to an expose event 
|
|         Input: widget - widget data structure for palette
|		 event	- the event that caused the redraw
|		 region	- the region that was exposed
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	xvobject object = xvw_object(widget);
	XvwPaletteWidget xwid = kwidget(widget);

	int      i, l, num;
	unsigned long *histogram;


	/*
	 *  Sanity check to make sure that we have legal values to render
	 *  the color palette.
	 */
	if (!xwid->color.info || xwid->color.info->ncolors <= 0 ||
	    !xvw_check_visible(object))
	{
	   return;
	}

	/*
	 *  Refresh the color palette display
	 */
	histogram = ColorGetHistogram(widget);
	if (histogram != NULL)
	{
	   for (i = 0, num = 0; i < xwid->color.info->ncolors; i++)
	        if (histogram[i] > 0) num++;

	    /*
	     *  Just do it...  Render the range by gapping the slots between
	     *  valid and invalid (non-allocated) pixels.  Eventually we want
	     *  this to be some kind of logrithmic or radial pattern, but for
	     *  now just do it simple and clean.
	     */
	    for (i = l = 0; i < xwid->color.info->ncolors; i++)
	    {
	       if (histogram[i] > 0)
	          RefreshElement(object, i, l++, num);
	    }
	}

	if (event != NULL && region != NULL)
	{
	   (*xvwPaletteWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new	 - the widget processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPaletteWidget cxwid = kwidget(current);
	XvwPaletteWidget nxwid = kwidget(new);

	Boolean redisplay = FALSE;


	if (cxwid->palette.type != nxwid->palette.type)
	   redisplay = TRUE;

        if (cxwid->color.object != nxwid->color.object)
           redisplay = TRUE;

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: GetInfo
|
|       Purpose: This routine gets various info depending in the information
|		 passed.  Such as computing the indx or pos for a given x,y.
|
|         Input: object - the object to be refreshed (the Palette)
|		 xpos   - the x device position
|		 ypos   - the y device position
|		 num    - the number of total "valid" elements
|	 Output: indx   - returns the index
|		 pos    - returns the element position
|		 factor - returns the factor according to palette type
|    Written By: Mark Young
|          Date: May 07, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void GetInfo(
   xvobject object,
   int	    xpos,
   int	    ypos,
   int      num,
   int      *indx,
   int      *pos,
   double   *factor)
{
	Widget widget = xvw_widget(object);
	XvwPaletteWidget xwid = (XvwPaletteWidget) kwidget(widget);

	double tmpfactor = 1.0;
	int width   = xwid->core.width-4;
	int height  = xwid->core.height-4;
	int ncolors = xwid->color.info->ncolors;
	int x, y, w, h, i, size, cnt, tmppos = 0;
	unsigned long *histogram = ColorGetHistogram(widget);


	if (width < 1) width = 1; if (height < 1) height = 1;

        if (num == 0)
        {
           for (i = num = 0; i < xwid->color.info->ncolors; i++)
              if (histogram[i] > 0) num++;
        }
 
	if (factor != NULL || indx != NULL || pos != NULL)
	{
	   if (xwid->palette.type == KPALETTE_TYPE_COLORBAR)
	      tmpfactor = width/((double) num);
	   else if (xwid->palette.type == KPALETTE_TYPE_COLORWHEEL)
	      tmpfactor = 360.0/num;
	   else if (xwid->palette.type == KPALETTE_TYPE_COLORCELL)
	      tmpfactor = ksqrt((double) ((width*height)/num)) - 4;

	   if (factor) *factor = tmpfactor;
	}
	tmpfactor = kmax(tmpfactor, 1.0);

	if (indx != NULL || pos != NULL)
	{
	   if (xwid->palette.type == KPALETTE_TYPE_COLORWHEEL)
	   {
              double angle, degrees;

	      x = xpos - width/2; y = ypos - height/2;
              if (x == 0 && y == 0)
                 degrees = 0.0;
	      else
	      {
                 angle = katan2((double) y, (double) x);
                 degrees = kradians_degrees(angle);
                 if (degrees < 0.0)
	            degrees = kabs(degrees);
	         else
	            degrees = 360.0 - degrees;
	      }
	      tmppos = degrees/tmpfactor; 
	   }
	   else if (xwid->palette.type == KPALETTE_TYPE_COLORCELL)
	   {
              /*
               *  Compute the palette width & height to hold the color boxes.
               */
	      size = (int) width/tmpfactor;
	      w = width/size; h = height/((double) num/size+0.5);
	      x = (xpos-2)/w; y = (ypos-2)/h;
	      tmppos = size*y + x;
	   }
	   else if (xwid->palette.type == KPALETTE_TYPE_COLORBAR)
	      tmppos = kmax((xpos-2)/tmpfactor, 0);

	   if (indx != NULL)
	   {
              for (i = cnt = 0; cnt < tmppos && i < ncolors; i++)
	         if (histogram[i] > 0) cnt++;

	      if (i < ncolors)
	         *indx = i;
	      else *indx = -1;
	   }
	   if (pos != NULL) *pos = tmppos;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: UpdatePalette
|
|       Purpose: updates the position of the cursor in the palette object
|		 so that if there is other devices that have update callbacks
|		 we will know it.
|
|         Input: widget      - image widget
|                client_data - client data (unused)
|                event       - type of event that triggered us
|        Output: dispatch    - whether to continue dispatching this event
|    Written By: Mark Young
|          Date: Oct 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void UpdatePalette(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	Widget widget = xvw_widget(object);
	XvwPaletteWidget xwid = kwidget(widget);

        unsigned long *histogram;
	size_t onum, num = xwid->palette.indexnum;
	int *oindexes, *indexes = xwid->palette.indexes;
	int i, found, restore, boundary, indx, xpos, ypos,
	    index1, index2, begin, end;


	if (!xwid->palette.update_callback || !xwid->color.info ||
	    xwid->color.info->ncolors <= 0)
	{
	   return;
	}

	if (event->type == ButtonPress)
	   xpos = event->xbutton.x, ypos = event->xbutton.y;
	else
	   xpos = event->xmotion.x, ypos = event->xmotion.y;

	/*
	 *  Find the index at the x & y position.  Also, see if the index
	 *  is on our palette list.  This information is used by the palette
	 *  to indicate whether we are adding or deleting elements from the
	 *  palette selection mechanism.
	 */
	GetInfo(object, xpos, ypos, 0, &indx, NULL, NULL);
	if (indx == -1)
	   return;


	/*
	 *  If this is a ButtonPress event then whether the index is currently
	 *  selected or not will dictate whether we will be adding or deleting
	 *  items from the list.  If the event is a ButtonMotion then if
	 *  the item is already selected and we are adding elements, or if
	 *  item is not selected and we are erasing then return.
	 */
	if (event->type == ButtonPress)
	{
	   xwid->palette.index1 = xwid->palette.index2 = indx;

	   kfree_and_NULL(xwid->palette.old_indexes);
	   xwid->palette.old_indexnum = num;
	   xwid->palette.old_indexes  = kdupalloc(indexes, num*sizeof(int));

	   if (karray_locate(indexes, KINT, indx, num) == -1)
	      xwid->palette.select = TRUE;
	   else
	      xwid->palette.select = FALSE;

	   /* Call the callbacks with the indx.  */
	   XtCallCallbacks(widget, XVW_PALETTE_CALLBACK, &indx);
	}
	else
	{
	   histogram = ColorGetHistogram(widget);

	   restore = boundary = FALSE;
	   index1 = xwid->palette.index1;
	   index2 = xwid->palette.index2;
	   if (indx <= index1 && indx > index2)
	      restore = TRUE;
	   else if (indx >= index1 && indx < index2)
	      restore = TRUE;
	   else if (indx <= index1 && index2 > index1)
	      boundary = TRUE;
	   else if (indx >= index1 && index2 < index1)
	      boundary = TRUE;

	   if (restore == TRUE || boundary == TRUE)
	   { 
	      if (boundary == TRUE)
		 begin = kmin(index1, index2), end = kmax(index1, index2);
	      else
		 begin = kmin(indx, index2), end = kmax(indx, index2);

	      for (i = begin; i < end; i++)
	      {
	         onum     = xwid->palette.old_indexnum;
	         oindexes = xwid->palette.old_indexes;
	         if (histogram[i] == 0 || i == index1)
		    continue;

	         /* Call the callbacks with the indx. */
	         if (karray_locate(oindexes, KINT, i, onum) == -1)
		    found = FALSE;
	         else
		    found = TRUE;

		 if (found == xwid->palette.select)
		    continue;

	         XtCallCallbacks(widget, XVW_PALETTE_CALLBACK, &i);
	      }
	   }

	   if (restore == FALSE)
	   {
	      if (boundary == TRUE)
		 begin = kmin(indx, index1), end = kmax(indx, index1);
	      else
		 begin = kmin(indx, index2), end = kmax(indx, index2);

	      for (i = begin; i < end; i++)
	      {
                 num     = xwid->palette.indexnum;
		 indexes = xwid->palette.indexes;

	         if (histogram[i] == 0 || i == index1)
		    continue;

	         if (karray_locate(indexes, KINT, i, num) == -1)
		    found = FALSE;
	         else
		    found = TRUE;

	         if (found == xwid->palette.select)
		    continue;

	         /* Call the callbacks with the indx.  */
	         XtCallCallbacks(widget, XVW_PALETTE_CALLBACK, &i);
	      }
	   }
	   xwid->palette.index2 = indx;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Refresh the palette display since our ColorClass
|		 has had a change to its map data, and we are required
|		 to change the colors in the palette.
|
|         Input: object - the object to be refreshed (the Palette)
|		 call_data   - (not used)
|    Written By: Mark Young & Danielle Argiro
|          Date: May 24, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	XvwPaletteWidget xwid = (XvwPaletteWidget) xvw_widget(object);
	XEvent event;
	Region  region   = XCreateRegion();

	event.type = Expose;
	event.xexpose.x = event.xexpose.y = 0;
	event.xexpose.width  = xwid->core.width;
	event.xexpose.height = xwid->core.height;
	XtAddExposureToRegion(&event, region);
	Redisplay((Widget) xwid, &event, region);
	XDestroyRegion(region);
}

/*-----------------------------------------------------------
|
|  Routine Name: RefreshElement
|
|       Purpose: Refresh a single element within the palette display.
|         Input: object - the object to be refreshed (the Palette)
|                indx   - the index of the element to be refreshed
|                pos    - it's element position within the palette
|                num    - total number of elements in the palette
|    Written By: Mark Young
|          Date: May 11, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RefreshElement(
   xvobject object,
   int indx,
   int pos,
   int num)
{
	Widget  widget = xvw_widget(object);
        XvwPaletteWidget xwid = (XvwPaletteWidget) widget;
	unsigned long *histogram = ColorGetHistogram(widget);
        Display *display = XtDisplay(widget);
        Window  window   = XtWindowOfObject(widget);

        Pixel   pixel;
        XPoint  pts[4];
        double  factor, degree;
        GC      gc1, gc2, gc = xwid->graphics.gc;
	int     i, ncolors = xwid->color.info->ncolors;
        int     size, width, height, w, h, x, y, *indexes, highlight;
	size_t  indexnum;


	if (num <= 0)
           for (i = num = 0; i < ncolors; i++) if (histogram[i] > 0) num++;

	if (pos <= -1)
           for (i = pos = 0; i < indx; i++) if (histogram[i] > 0) pos++;

	indexes  = xwid->palette.indexes;
	indexnum = xwid->palette.indexnum;

	width = xwid->core.width-4; height = xwid->core.height-4;

        /* error check for elements so small that there's no point
           in refreshing them (and avoid floating pt exception to boot) */
        if ((width < 1) || (height < 1)) return;

	GetInfo(xvw_object(widget), 0, 0, num, NULL, NULL, &factor);

	pixel = ColorAllocate(widget, (unsigned) indx);

	if (gc == NULL || !xvw_check_visible(object)) return;

	XSetForeground(display, gc, pixel);

	if (karray_locate(indexes, KINT, indx, indexnum) != -1)
	{
	   highlight = TRUE;
	   XSetFillStyle(display, gc, FillOpaqueStippled);
	}
	else
	{
	   highlight = FALSE;
	   XSetFillStyle(display, gc, FillSolid);
	}
        gc1 = xwid->palette.gc1;
	gc2 = xwid->palette.gc2;

	if (xwid->palette.type == KPALETTE_TYPE_COLORBAR)
	{
	   x = pos*factor + 4;
	   size = ((pos+1)*factor + 4) - x;
	   XFillRectangle(display, window, gc, x, 2, 
			  (unsigned) size, (unsigned) height);
	}
	else if (xwid->palette.type == KPALETTE_TYPE_COLORCELL)
	{
	   size = (int) width/factor;
	   w = width/size; h = height/((double) num/size+0.5);
	   x = (pos % size) * w + 2;
	   y = (pos / size) * h + 2;
	   x += 1; y += 1; w -= 2; h -= 2;
	   XFillRectangle(display, window, gc, x, y, 
			  (unsigned) w, (unsigned) h);

	   if (highlight == TRUE)
	      gc1 = xwid->palette.gc1, gc2 = xwid->palette.gc2;
	   else
	      gc2 = xwid->palette.gc1, gc1 = xwid->palette.gc2;

	   pts[0].x = x;         pts[0].y = y + h-1;
	   pts[1].x = x;         pts[1].y = y;
	   pts[2].x = x + w-1;   pts[2].y = y;
	   XDrawLines(display, window, gc1, pts, 3, CoordModeOrigin);

	   pts[1].x = x + w-1;   pts[1].y = y + h-1;
	   XDrawLines(display, window, gc2, pts, 3, CoordModeOrigin);
	}
	else if (xwid->palette.type == KPALETTE_TYPE_COLORWHEEL)
	{
	   degree = pos*factor; x = degree*64; w = factor*64+0.5;
	   XFillArc(display, window, gc, 2, 2, (unsigned) width, 
		    (unsigned) height, x, w);
	   if (degree >= 30.0 && degree <= 210.0)
	      XDrawArc(display, window, gc2, 2, 2, 
		       (unsigned) width, (unsigned) height, x, w);
	   else
	      XDrawArc(display, window, gc1, 2, 2, 
		       (unsigned) width, (unsigned) height, x, w);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangePaletteIndex
|
|       Purpose: ChangePaletteIndex is used to set the indexes associated
|                with the colorcell widget.
|
|         Input: object    - the object in which we will be getting the
|                            filename
|                attribute - the old object to be closed
|                calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young & Danielle Argiro
|          Date: May 02, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ChangePaletteIndex(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
        XvwPaletteWidget xwid = (XvwPaletteWidget) xvw_widget(object);
        int indx = *((int *) calldata);

        int     i;
        size_t  indexnum = xwid->palette.indexnum;
        int     *indexes = xwid->palette.indexes;


        if (kstrcmp(attribute, XVW_PALETTE_ADD) == 0)
        {
           if (karray_locate(indexes, KINT, indx, indexnum) == -1)
           {
              indexes = karray_insert(indexes, KINT, indx, indexnum, 
				      KLIST_TAIL, TRUE);
              indexnum++;
           }
	   else return(TRUE);
        }
        else if (kstrcmp(attribute, XVW_PALETTE_DELETE) == 0)
        {
           i = karray_locate(indexes, KINT, indx, indexnum);
           if (i != -1)
           {
              indexes = karray_delete(indexes, KINT, indx, indexnum);
              indexnum--;
           }
	   else return(TRUE);
        }
        else if (kstrcmp(attribute, XVW_PALETTE_CLEAR) == 0)
        {
           kfree_and_NULL(indexes);
           indexes = NULL; 
	   indexnum = 0;
        }
        xwid->palette.indexnum = indexnum;
        xwid->palette.indexes  = indexes;

	if (kstrcmp(attribute, XVW_PALETTE_CLEAR) == 0)
	{
	   Redisplay(xvw_widget(object), NULL, NULL);
	   return(TRUE);
	}

	/*
	 *  Only need to refresh when adding or deleting...
	 */
	if (kstrcmp(attribute, XVW_PALETTE_CLEAR) != 0)
           RefreshElement(object, indx, -1, 0);

        return(TRUE);
}


/************************************************************
*
*  Routine Name: xvw_create_palette - create a palette object
*
*       Purpose: The palette visual object provides a visual display 
*                of the colors defined by a colormap. The colors may be 
*                displayed as a grid of color cells, a color palette, or 
*                a color wheel.
*
*		 Note that when an RGB image is displayed on an
*                8-bit screen, a 3-3-2 transformation is used in
*                order to convert the RGB image into an 8-bit representation.
*                Because of this, the colors in the palette object that
*                are displayed in this situation will reflect the 3-3-2
*                map, and may not seem to correspond to the displayed image.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the palette
*                         object (for use in app-defaults files, etc)
*
*        Output: 
*	Returns: The palette xvobject on success, NULL otherwise
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 17, 1992
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_palette(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, PaletteWidgetClass);
	return(object);
}
