/*
  * Khoros: $Id: Plot2D.c,v 1.22 1996/04/16 05:27:30 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Plot2D.c,v 1.22 1996/04/16 05:27:30 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		2D Plot Gadget Routines
   >>>>		
   >>>>  Private:
   >>>>                 Plot2DRefresh()
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 GetObjectName()
   >>>>                 SetObjectName()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 ReloadCallback()
   >>>>                 ColorReloadCallback()
   >>>>                 ReloadPlot()
   >>>>                 PickGadget()
   >>>>   Public:
   >>>>                 xvw_create_plot2d()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/Plot2DP.h>


static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static int  GetObjectName   PROTO((xvobject, char *, kaddr));
static int  SetObjectName   PROTO((xvobject, char *, kaddr));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void ReloadCallback  PROTO((kobject, char *, Widget, kdms_callback *));
static void ColorReloadCallback PROTO((xvobject, kaddr));

static void Destroy	    PROTO((Widget));
static void ReloadPlot	    PROTO((Widget));
static int  PickGadget	    PROTO((Widget, int, int));


/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PLOT2D_PLOTOBJ,           NULL,    XtRInt,        NULL}, 
{XVW_PLOT2D_DATA_MAX_X,        NULL,    XtRDouble,     NULL}, 
{XVW_PLOT2D_DATA_MAX_Y,        NULL,    XtRDouble,     NULL}, 
{XVW_PLOT2D_DATA_MIN_X,        NULL,    XtRDouble,     NULL}, 
{XVW_PLOT2D_DATA_MIN_Y,        NULL,    XtRDouble,     NULL}, 
{XVW_PLOT2D_PLOTTYPE,          NULL,    XtRInt,        NULL}, 
{XVW_PLOT2D_PLOTSIZE,          NULL,    XtRInt,        NULL}, 
{XVW_PLOT2D_POINTS,            NULL,    XtRPointer,    NULL}, 
{XVW_PLOT2D_ROI,               NULL,    XtRPointer,    NULL},
{XVW_PLOT2D_ROI_SHAPE,         NULL,    XtRInt,        NULL},
{XVW_PLOT2D_HIGHLIGHTCOLOR,    NULL,    XtRString,     NULL},
{XVW_PLOT2D_COLOR_ORIGINATION, NULL,    XtRInt,        NULL},
{XVW_PLOT2D_X_ORIENTATION,     NULL,    XtRInt,        NULL},
{XVW_PLOT2D_WIDTH_OFFSET,      NULL,    XtRInt,        NULL},
{XVW_PLOT2D_HEIGHT_OFFSET,     NULL,    XtRInt,        NULL},
{XVW_PLOT2D_DEPTH_OFFSET,      NULL,    XtRInt,        NULL},
{XVW_PLOT2D_TIME_OFFSET,       NULL,    XtRInt,        NULL},
{XVW_PLOT2D_ELEMENTS_OFFSET,   NULL,    XtRInt,        NULL},
{XVW_PLOT2D_X_EXTRACT_SIZE,    NULL,    XtRInt,        NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPlot2DGadgetRec, plot2d.field)

static double data_min     = 0.0,
              data_max     = 1.0;

static XtResource resources[] = {
{XVW_PLOT2D_DATA_MAX_X,  NULL, XtRDouble, sizeof(double),
      offset(data_max_x), XtRDouble, (XtPointer) &data_max},
{XVW_PLOT2D_DATA_MAX_Y,  NULL, XtRDouble, sizeof(double),
      offset(data_max_y), XtRDouble, (XtPointer) &data_max},
{XVW_PLOT2D_DATA_MIN_X,  NULL, XtRDouble, sizeof(double),
      offset(data_min_x), XtRDouble, (XtPointer) &data_min},
{XVW_PLOT2D_DATA_MIN_Y,  NULL, XtRDouble, sizeof(double),
      offset(data_min_y), XtRDouble, (XtPointer) &data_min},
{XVW_PLOT2D_PLOTTYPE,  NULL, XtRInt, sizeof(int),
      offset(plot_type), XtRImmediate, (XtPointer) KPLOT2D_LINEPLOT},
{XVW_PLOT2D_PLOTSIZE, NULL, XtRInt, sizeof(int),
      offset(plot_size), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_POINTS, NULL, XtRPointer, sizeof(Coord *),
      offset(points), XtRPointer, (XtPointer) NULL},
{XVW_PLOT2D_PLOTOBJ, NULL, XtRPointer, sizeof(kobject),
      offset(pobject), XtRPointer, (XtPointer) NULL},
{XVW_PLOT2D_ROI_SHAPE, NULL, XtRInt, sizeof(int),
      offset(roi_shape), XtRImmediate, (XtPointer) KPLOT2D_ROI_RECTANGLE},
{XVW_PLOT2D_HIGHLIGHTCOLOR, NULL, XtRString, sizeof(String),
      offset(highlight_color), XtRImmediate, (XtPointer) "red"},
{XVW_PLOT2D_COLOR_ORIGINATION, NULL, XtRInt, sizeof(int),
      offset(color_origination), XtRImmediate, 
      (XtPointer) KPLOT2D_COLOR_FROM_FOREGROUND},
{XVW_PLOT2D_X_ORIENTATION, NULL, XtRInt, sizeof(int),
      offset(x_orientation), XtRImmediate, (XtPointer) KWIDTH},
{XVW_PLOT2D_WIDTH_OFFSET, NULL, XtRInt, sizeof(int),
      offset(width_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_HEIGHT_OFFSET, NULL, XtRInt, sizeof(int),
      offset(height_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_DEPTH_OFFSET, NULL, XtRInt, sizeof(int),
      offset(depth_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_TIME_OFFSET, NULL, XtRInt, sizeof(int),
      offset(time_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_ELEMENTS_OFFSET, NULL, XtRInt, sizeof(int),
      offset(elements_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT2D_X_EXTRACT_SIZE, NULL, XtRInt, sizeof(int),
      offset(extract_size), XtRImmediate, (XtPointer) 1},


{XVW_GRAPHICS_UPDATEDC, NULL, XtRBoolean, sizeof(Boolean),
      XtOffsetOf(XvwPlot2DGadgetRec, graphics.updatedc),
      XtRImmediate, (XtPointer) FALSE},

{NULL, NULL, XtRInt, sizeof(int),
      XtOffsetOf(XvwPlot2DGadgetRec, color.autocolor), 
      XtRImmediate, (XtPointer) KRAINBOW},

};
#undef offset


#define SUPERCLASS (&xvwColorGadgetClassRec)

XvwPlot2DGadgetClassRec xvwPlot2DGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Plot2D",				/* class_name		  */
    sizeof(XvwPlot2DGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    XtInheritRecomputePosition,		/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwPlot2DGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPlot2DGadgetClass for public consumption
 */
WidgetClass xvwPlot2DGadgetClass = (WidgetClass) &xvwPlot2DGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of plot2d object class has been created.
|                It will initialize all the class attributes.
|
|         Input: None 
|
|        Output: None
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPlot2DGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Plot2D.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Plot2D");

	xvw_define_attributes(xvwPlot2DGadgetClass, 
	    XVW_PLOT2D_PLOTFILE, XtRString,  SetObjectName, GetObjectName,
	    XVW_PLOT2D_ROI,      XtRPointer, NULL,          Plot2DGetRoi,
	    XVW_PLOT2D_DELETE_PTS_INTERACTIVELY, 
				 XtRInt, Plot2DDeletePtsInteractively, NULL,
	    XVW_PLOT2D_COLOR_PTS_INTERACTIVELY, 
				 XtRInt, Plot2DColorPtsInteractively, NULL,
				 NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial settings 
|                for a 2D plot object instance. 
|
|         Input: request - not used
|                new     - object instance after initialization, 
|                          with attributes set
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) new;

	/*
	 *  Find the world coordinate minimum and maximum values (some of
	 *  the 3D plotting routines need them.
	 */
	new->core.border_width = 0;
	ReloadPlot(new);
	Plot2DFindMinMax(xobj);
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay 
|
|       Purpose: This method is used to initially draw the 
|                the 2D plot object and for redrawing the object 
|                whenever part of it becomes exposed.
|
|         Input: widget - the object being exposed
|                event  - the expose event with information
|                         about what is being exposed
|                region - the region being exposed
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
-----------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) widget;

	register Coord *points;
	register int  size, marker_type, marker_scale;
        XVGraphicsID  id; 
	XColor        *plotcolor;

	/*
	 *  Check to see if we are currently visible and we have points
	 *  to be displayed
	 */
	if (!xobj->plot2d.points || !xvw_check_visible(xvw_object(widget)))
	   return;

	id	     = xobj->graphics.id;
	size	     = xobj->plot2d.plot_size;
	points	     = xobj->plot2d.points;
	plotcolor    = &xobj->graphics.fg;
	marker_type  = xobj->graphics.marker_type;
	marker_scale = xobj->graphics.marker_scale;

	switch (xobj->plot2d.plot_type)
	{
	   case KPLOT2D_LINEPLOT:         
		X2D_draw_polyline(id, points, size, plotcolor);
		break;

	   case KPLOT2D_DISCRETE:   
		X2D_draw_discrete(id, points, size, plotcolor);
		break;

	   case KPLOT2D_BARGRAPH:  
		X2D_draw_bargraph(id, points, size, plotcolor);
		break;

	   case KPLOT2D_POLYMARKER: 
		X2D_draw_polymarker(id, points, size, marker_scale,
				    marker_type, plotcolor);
		break;

	   case KPLOT2D_LINEMARKER: 
		X2D_draw_linemarker(id, points, size, marker_scale,
				    marker_type, plotcolor);
		break;
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: GetObjectName
|
|       Purpose: GetObjectName is used to get the filename associated
|		 with the attribute.
|
|         Input: object    - the object in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|
|        Output: calldata  - pointer to retrieve the object name
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int GetObjectName(
   xvobject object,
   char   *attribute,
   kaddr  calldata)
{
	kobject plot;
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) xvw_widget(object);
	String *filename = (String *) calldata;


	if (kstrcmp(attribute, XVW_PLOT2D_PLOTFILE) == 0)
	   plot = xobj->plot2d.pobject;
	else
	{
	   kerror(XVISUAL, "GetObjectName", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	*filename = NULL;
	return(kpds_get_attribute(plot, KPDS_NAME, filename));
}

/*-----------------------------------------------------------
|
|  Routine Name: SetObjectName
|
|       Purpose: SetObjectName is used to set the filename associated
|		 with the attribute.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int SetObjectName(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	kobject plot = NULL;
	int	locate = TRUE;
	String *filename = (String *) calldata;


	if (*filename != NULL && (plot = kdms_locate(*filename)) == NULL)
	{
	   locate = FALSE;
	   if ((plot = kpds_open_object(*filename, KOBJ_READ)) == NULL)
	   {
	      kerror(XVISUAL, "SetObjectName", "Failed to load 2D Plot '%s'",
			*filename);
	      return(TRUE);
	   }
	}

	if (kstrcmp(attribute, XVW_PLOT2D_PLOTFILE) == 0)
	   xvw_set_attribute(object, XVW_PLOT2D_PLOTOBJ, plot);
	else
	{
	   kerror(XVISUAL, "SetObjectName", "invalid attribute %s",
			attribute);
	   if (!locate) kpds_close_object(plot);
	   return(FALSE);
	}
	if (!locate) kpds_close_object(plot);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a 2D plot object instance.  These values
|		 include the name and the plot object or points. 
|
|         Input: current - the object containing current settings
|                request - the object containing requested settings
|                new     - the object processed through all set values methods
|
|       Returns: FALSE (0) - XPlotRedraw will handle the redraw in any case  
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPlot2DGadget cobj = (XvwPlot2DGadget) current;
	XvwPlot2DGadget robj = (XvwPlot2DGadget) request;
	XvwPlot2DGadget nobj = (XvwPlot2DGadget) new;

	xvobject object;
	Boolean  redisplay = FALSE;
	int      i;


	if (cobj->plot2d.pobject         != nobj->plot2d.pobject       ||
	    robj->graphics.reset_world   != cobj->graphics.reset_world ||
	    cobj->plot2d.plot_size       != nobj->plot2d.plot_size     ||
	    cobj->plot2d.points	         != nobj->plot2d.points)
	{
	   /* New Coord Array */
	   if (cobj->plot2d.points != nobj->plot2d.points)
	   {
	      Coord *tmp;
	      size_t size = (size_t) (nobj->plot2d.plot_size * sizeof(Coord));
	      /* 
	       * we will be changing "d" values, don't want to over-write those
	       * of the caller, so make a copy of the coord array that we can
	       * modify with impunity
	       */
	      tmp = (Coord *) kmalloc(size);
	      kfree_and_NULL(cobj->plot2d.points);
	      kmemcpy(tmp, nobj->plot2d.points, size);
	      nobj->plot2d.points = tmp;  
	      for (i = 0; i < nobj->plot2d.plot_size; i++)
	      {
	        if (tmp[i].d != KGRAPHICS_UNINITIALIZED)
		    tmp[i].d = ColorAllocate(new, (unsigned) tmp[i].d);
	      }
	   }

	   /* New Data Object */
	   if (cobj->plot2d.pobject != nobj->plot2d.pobject)
	   {
	      nobj->plot2d.pobject =kpds_reference_object(nobj->plot2d.pobject);
	      kpds_close_object(cobj->plot2d.pobject);
	      kpds_add_value_callback(nobj->plot2d.pobject,KPDS_CALLBACK_CHANGE,
			ReloadCallback, (kaddr) new);
	      object = xvw_object(new);
	      if (cobj->color.object == nobj->color.object)
	           xvw_set_attribute(object,XVW_COLOR_COLOROBJ,
				     nobj->plot2d.pobject);

	      /* if explicit settings are not done, reapply defaults */
              if (cobj->plot2d.x_orientation == nobj->plot2d.x_orientation)
                  nobj->plot2d.x_orientation = KWIDTH;
              if (cobj->plot2d.width_offset == nobj->plot2d.width_offset)
                  nobj->plot2d.width_offset = 0;
              if (cobj->plot2d.height_offset == nobj->plot2d.height_offset)
                  nobj->plot2d.height_offset = 0;
              if (cobj->plot2d.depth_offset == nobj->plot2d.depth_offset)
                  nobj->plot2d.depth_offset = 0;
              if (cobj->plot2d.time_offset == nobj->plot2d.time_offset)
                  nobj->plot2d.time_offset = 0;
              if (cobj->plot2d.elements_offset == nobj->plot2d.elements_offset)
                  nobj->plot2d.elements_offset = 0;


	   }
	   ReloadPlot(new);
	   Plot2DFindMinMax(nobj);
	   redisplay = TRUE;
	}

	if (cobj->plot2d.x_orientation   != nobj->plot2d.x_orientation ||
	    cobj->plot2d.width_offset    != nobj->plot2d.width_offset  ||
	    cobj->plot2d.height_offset   != nobj->plot2d.height_offset || 
	    cobj->plot2d.depth_offset    != nobj->plot2d.depth_offset  || 
	    cobj->plot2d.time_offset     != nobj->plot2d.time_offset   || 
	    cobj->plot2d.elements_offset != nobj->plot2d.elements_offset)
	{
	    if (Plot2DCheckOffset(current, new))
	    {
                nobj->graphics.reset_world = TRUE;
                ReloadPlot(new);
                Plot2DFindMinMax(nobj);
                redisplay = TRUE;
	    }
	}


	if (cobj->plot2d.color_origination != nobj->plot2d.color_origination)
	{
	    /*
	     *  Color origination from data only applies if we have
	     *  a data object, not if we have a coord array.
	     */
            if ((nobj->plot2d.pobject == NULL) &&
                (nobj->plot2d.points != NULL))
	    {
		kerror(XVISUAL, "SetValues", "Color origination must be set to foreground color unless input is specified as a data object.");
		nobj->plot2d.color_origination = KPLOT2D_COLOR_FROM_FOREGROUND;
	    }
            /*
             *  If we are changing color origination from data to foreground,
             *  and we have a data object (not a coord array) then set d values
             *  back to KGRAPHICS_UNINITIALIZED so fg color can "take".
             */
            if ((nobj->plot2d.color_origination == KPLOT2D_COLOR_FROM_FOREGROUND
) &&
                (nobj->plot2d.pobject != NULL) &&
                (nobj->plot2d.points != NULL))
            {
                for (i = 0; i < nobj->plot2d.plot_size; i++)
                {
                     if (nobj->plot2d.points[i].d != KGRAPHICS_UNINITIALIZED)
                        nobj->plot2d.points[i].d = 
				(Index) KGRAPHICS_UNINITIALIZED;
                }
            }
	    ReloadPlot(new);
	    redisplay = TRUE;
	}

	if (cobj->plot2d.plot_type != nobj->plot2d.plot_type)
	{
	   redisplay = TRUE;
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Close the image, shape, colormap, and overlay
|		 objects before the image widget is destroyed.
|
|         Input: widget - the image widget being destroyed 
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) widget;

	/*
	 *  Close plot object
	 */
	kpds_close_object(xobj->plot2d.pobject);
	kfree_and_NULL(xobj->plot2d.points);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReloadCallback
|
|       Purpose: Reload the plot2d data structure from the pobject
|		 data object.
|
|         Input:
|        Output: none
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReloadCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	int x, y, w, h;

	ReloadPlot(widget);
	Plot2DFindMinMax((XvwPlot2DGadget) widget);
	x = widget->core.x + 1; y = widget->core.y + 1;
	w = widget->core.width; h = widget->core.height;
	XClearArea(XtDisplay(widget), XtWindowOfObject(widget),
		   x, y, (unsigned) w, (unsigned) h, FALSE);
	Redisplay(widget, NULL, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Refresh the palette display since our ColorClass
|		 has had a change to its map data, and we are required
|		 to change the colors in the palette.
|
|         Input: object - the object to be refreshed (the Plot2D)
|		 call_data   - (not used)
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: June 28, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	XEvent  event;
	Widget  widget = xvw_widget(object);
	Region  region   = XCreateRegion();

	event.type = Expose;
	event.xexpose.x = event.xexpose.y = 0;
	event.xexpose.width  = widget->core.width;
	event.xexpose.height = widget->core.height;
	XtAddExposureToRegion(&event, region);
	ReloadPlot(widget);
	Redisplay(widget, &event, region);
	XDestroyRegion(region);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReloadPlot
|
|       Purpose: This routine will subsample the data from
|		 a given 2D plot object.
|
|         Input: xobj - the object to subsample 
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Feb 23, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReloadPlot(
   Widget widget)
{
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) widget;

	kobject  object;
	char     *format;
	int      i, w, h, d, t, e, count, size, datatype;
 	double   *plotdata = NULL;
	Coord    *points = NULL, *tmp_points;

	unsigned char *pixels = NULL;
	unsigned char *mask   = NULL; 

	int color_orig = xobj->plot2d.color_origination;


	/*
	 *  Sanity check to make sure that the image object exists
	 */
	if ((object = xobj->plot2d.pobject) == NULL)
	   return;

	/*
	 *  Get the value data dimensionality, file format, data type
	 */
	kpds_get_attributes(object, 
			    KPDS_VALUE_SIZE,      &w, &h, &d, &t, &e,
			    KPDS_FORMAT,          &format,
			    KPDS_VALUE_DATA_TYPE, &datatype,
			    NULL);

	/*	
	 * courtesy for the user: smart enough to know to plot height if
	 * width is size one, width if height is size one
	 */
	if ((w == 1) && (h > 1))
	    xobj->plot2d.x_orientation = KHEIGHT;
	else if ((h == 1) && (w > 1))
	    xobj->plot2d.x_orientation = KWIDTH;

	color_orig = xobj->plot2d.color_origination;

	/* specify region size based on the orientation specified */
	if (xobj->plot2d.x_orientation == KWIDTH)
	{
	    /* kludge: to cont support for end-to-end scanline extraction
	       for the sake of signal ROI extraction only at this point */
	    if (xobj->plot2d.extract_size == 1)
	    {
	        kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                                   w, 1, 1, 1, 1);
	        kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                                   w, 1, 1, 1, 1);
	        size = w;
	    }
	    else
	    {
		kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                                   w, h, 1, 1, 1);
                kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                                   w, h, 1, 1, 1);
                size = w * h;
	    }
	}
	else if (xobj->plot2d.x_orientation == KHEIGHT)
	{
            kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                               1, h, 1, 1, 1);
	    kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                               1, h, 1, 1, 1);
	    size = h;
	}
	else if (xobj->plot2d.x_orientation == KDEPTH)
	{
            kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                               1, 1, d, 1, 1);
	    kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                               1, 1, d, 1, 1);
	    size = d;
	}
	else if (xobj->plot2d.x_orientation == KTIME)
	{
            kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                               1, 1, 1, t, 1);
	    kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                               1, 1, 1, t, 1);
	    size = t;
	}
	else if (xobj->plot2d.x_orientation == KELEMENTS)
	{
            kpds_set_attribute(object, KPDS_VALUE_REGION_SIZE,
                               1, 1, 1, 1, e);
	    kpds_set_attribute(object, KPDS_MASK_REGION_SIZE,
                               1, 1, 1, 1, e);
	    size = e;
	}


	/* 
	 * case for ascii data points 
	 */
	if (kstrcmp(format, "ascii") == 0)
	{
	    size = h;
	    points = (Coord *) kmalloc(size*sizeof(Coord));
	    for (i = 0; i < h; i++)
            {
		plotdata = (double *)
			    kpds_get_data(object, KPDS_VALUE_REGION, plotdata);
                points[i].x = plotdata[0];
                points[i].y = plotdata[1];
                if (color_orig == KPLOT2D_COLOR_FROM_DATA)
                    points[i].d = (Index) ColorAllocate(widget, 
							(unsigned) plotdata[1]);
                else points[i].d = (unsigned long) KGRAPHICS_UNINITIALIZED;
            }
            kfree_and_NULL(plotdata);
	    kfree_and_NULL(xobj->plot2d.points);
            xobj->plot2d.points    = points;
            xobj->plot2d.plot_size = size;
	}

	/*
	 *  General Case: X is implicit, Y and D are in value data
         *          -OR-  X is in location data; Y and D in value data
	 */
	else
	{
	    /* allocate points array */
	    points = (Coord *) kmalloc(size*sizeof(Coord));

	    /*
             *  Get the color data from the value segment
             */
	    kpds_set_attribute(object, KPDS_VALUE_POSITION, 
			       xobj->plot2d.width_offset,
			       xobj->plot2d.height_offset,
			       xobj->plot2d.depth_offset,
			       xobj->plot2d.time_offset,
			       xobj->plot2d.elements_offset,
			       NULL);

	    /*
	     *  need to do color normalization for float, double,
	     *  complex data types the same way that the image object does.
	     *  NOTE: same code in Plot3D.c, ColorUtil.c
	     */
	    if (datatype == KFLOAT   || datatype == KDOUBLE ||
                datatype == KCOMPLEX || datatype == KDCOMPLEX)
            {
		double dmin, dmax, offset, factor;
                kpds_set_attributes(object,
                                    KPDS_VALUE_SCALING, KNONE,
                                    KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
                                    NULL);
                kpds_get_attribute(object, KPDS_VALUE_MINMAX, &dmin, &dmax);
                factor = (255.0/(dmax-dmin));
                offset = 0.0-dmin*factor;

                kpds_set_attributes(object,
                                    KPDS_VALUE_SCALING,      KSCALE,
                                    KPDS_VALUE_SCALE_FACTOR, factor,
                                    KPDS_VALUE_SCALE_OFFSET, offset, 0.0,
                                    KPDS_VALUE_DATA_TYPE,    KUBYTE,
                                    NULL);
	    }

            if ((pixels = (unsigned char *) 
			   kpds_get_data(object, KPDS_VALUE_REGION, pixels)))
            {
                for (i = 0; i < size; i++)
                {
                    if (color_orig == KPLOT2D_COLOR_FROM_DATA)
                        points[i].d = (Index) ColorAllocate(widget, 
					      (unsigned int) pixels[i]);
                    else points[i].d = (Index) KGRAPHICS_UNINITIALIZED;
                }
		kfree_and_NULL(pixels);
            }

	    /*
	     *  Get the plot data from the value segment
	     */
	    kpds_set_attributes(object, 
			        KPDS_VALUE_DATA_TYPE, KDOUBLE,
				KPDS_VALUE_SCALING,   KNONE,
				KPDS_VALUE_SCALE_OFFSET, 0.0, 0.0,
				NULL);
	    kpds_set_attribute(object, KPDS_VALUE_POSITION, 
			       xobj->plot2d.width_offset,
			       xobj->plot2d.height_offset,
			       xobj->plot2d.depth_offset,
			       xobj->plot2d.time_offset,
			       xobj->plot2d.elements_offset,
			       NULL);

	    plotdata = (double *) 
			kpds_get_data(object, KPDS_VALUE_REGION, NULL);

            /*
	     * X is in location data; Y and D in value data
	     */
	    if (kpds_query_location(object))
	    {
	        double *locdata = NULL;
		kpds_set_attributes(object,
				    KPDS_LOCATION_DATA_TYPE, KDOUBLE,
				    KPDS_LOCATION_POSITION,  0, 0, 0, 0, 0,
				    NULL);
		locdata = kpds_get_data(object, KPDS_LOCATION_PLANE, NULL);
		for (i = 0; i < size; i++)
                {
                    points[i].x = locdata[i];
                    points[i].y = plotdata[i];
                }
	        kfree(locdata);
	    }

	    /*
	     * X is implicit, Y and D are in value data
	     */
	    else
	    {
	        for (i = 0; i < size; i++)
	        {
	            points[i].x = i;
	            points[i].y = plotdata[i];
	        }
	    }
	    kfree_and_NULL(xobj->plot2d.points);
	    xobj->plot2d.points    = points;
	    xobj->plot2d.plot_size = size;
	    kfree_and_NULL(plotdata);
	}

	/*
	 *  Data object contains a mask: need to use only those points 
	 *  with mask values of 1.  Set tmp ptr to original points array; 
	 *  transfer only the valid points back into original points array.
	 */
	if (kpds_query_mask(object))
	{
	    kpds_set_attribute(object, KPDS_MASK_DATA_TYPE, 
			       KUBYTE);
	    kpds_set_attribute(object, KPDS_MASK_POSITION,
                           xobj->plot2d.width_offset,
                           xobj->plot2d.height_offset,
                           xobj->plot2d.depth_offset,
                           xobj->plot2d.time_offset,
                           xobj->plot2d.elements_offset,
                           NULL);
	    mask = (unsigned char *) 
		    kpds_get_data(object, KPDS_MASK_REGION, NULL);

	    tmp_points = xobj->plot2d.points;
	    points = NULL;
	    count = 0;
            for (i = 0; i < size; i++)
            {
		if (mask[i] > 0)
		{
                     points = (Coord *) krealloc(points, (count+1) *
						 sizeof(Coord));
                     points[count].x = tmp_points[i].x;
                     points[count].y = tmp_points[i].y;
                     points[count].d = tmp_points[i].d;
	             count++;
	        }
            }
	    kfree_and_NULL(tmp_points);
	    kfree_and_NULL(mask);
            xobj->plot2d.points    = points;
            xobj->plot2d.plot_size = count;
	    xobj->graphics.reset_world = TRUE;
	}

	kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, datatype);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a plot object and an attempted picking location,
|                see if the plot has been picked. 
|
|         Input: widget - the line object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications: 
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) widget;

	Real  xpos1, xpos2, ypos1, ypos2;
	Coord c1, c2, c, *points;
	int i; 
        XVGraphicsID id = xobj->graphics.id;


	if ((points = xobj->plot2d.points) == NULL)
	   return(FALSE);

	xpos1 = x - PICK_TOLERANCE; ypos1 = y - PICK_TOLERANCE;
	xpos2 = x + PICK_TOLERANCE; ypos2 = y + PICK_TOLERANCE;
	X2D_convert_point_dc_to_wc(id, xpos1, ypos1, &c1);
	X2D_convert_point_dc_to_wc(id, xpos2, ypos2, &c2);
	X2D_convert_point_dc_to_wc(id, (Real) x, (Real) y, &c);

	xpos1 = kabs(c1.x - c2.x);
	ypos1 = kabs(c1.y - c2.y);
	for (i = 0; i < xobj->plot2d.plot_size; i++)
	{
	   if (kabs(c.x-points[i].x) < xpos1 && kabs(c.y - points[i].y) < ypos1)
	      return(TRUE);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Plot2DRefresh
|
|       Purpose: This routine is a kludge; by having this routine
|		 only private, not static, Plot2DColorPointsInteractively()
|		 in Plot2Dutil.c can call this routine, which in turn
|		 just calls the Redisplay() method to redisplay the plot,
|		 which needs to be redisplayed because colors of some points
|		 have changed.
|
|         Input: widget - the plot2D object which needs to be refreshed
|    Written By: Danielle Argiro
|          Date: Nov 08, 1995
| Modifications:
|
-----------------------------------------------------------*/
/* ARGSUSED */
void Plot2DRefresh(
   Widget widget)
{
	Redisplay(widget, NULL, NULL);
}

/************************************************************
*
*  Routine Name: xvw_create_plot2d - create a 2D plot object 
*
*       Purpose: A 2D plot object is used to display a 2D plot.
*
*                A variety of plot types are supported, including
*                line plots, discrete plots, bar graphs, polymarker
*                plots, linemarker plots, and colormarker plots.
*                The 2D plot object is generally used in conjunction 
*                with a 2D axis object, where both are created as children
*                of an area object so that a coordinated world coordinate
*                view is automatically supported.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the 2D plot
*                         object (for use in app-defaults files, etc)
*        Output:
*       Returns: Returns the created 2D plot xvobject, or NULL upon failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Aug 03, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_plot2d(
   xvobject parent,
   char     *name)
{
	xvobject  object;


	/*
	 *  Create the main plot workspace
	 */
	object = xvw_create(parent, FALSE, TRUE, name, Plot2DGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,    /* menuable       */
                XVW_RESIZABLE,    FALSE,   /* resizable      */
                XVW_SELECTABLE,   TRUE,   /* selectable     */
		NULL);
	}
	return(object);
}
