/*
 * Khoros: $Id: Plot2Dutil.c,v 1.7 1996/04/04 23:29:49 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Plot2Dutil.c,v 1.7 1996/04/04 23:29:49 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Plot2D Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 Plot2DDeletePtsInteractively()
   >>>>                 Plot2DColorPtsInteractively()
   >>>>                 Plot2DFindMinMax()
   >>>>                 Plot2DCheckOffset()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/Plot2DP.h>

/*-----------------------------------------------------------
|
|  Routine Name: Plot2DDeletePtsInteractively
|
|       Purpose: This routine supports the action attribute
|		 PLOT2D_DELETE_PTS_INTERACTIVELY, which allows
|		 the user to rubberband about desired points in 
|		 the plot and have them deleted automatically.
|
|         Input: object    - the plot2D object
|		 attribute - the attribute name PLOT2D_DELETE_PTS_INTERACTIVELY
|        Output: calldata  - not used
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Danielle Argiro
|          Date: Sept 20, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
int Plot2DDeletePtsInteractively(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
        kobject      roi;
	Coord        *points = NULL;
        Coord        *new_points = NULL;
        int          i, ptnum, new_ptnum = 0;

        /*
         *  Let user interactively specify ROI in which pts are to be deleted
         */
        xvw_get_attribute(object, XVW_PLOT2D_ROI, &roi);

	/*
	 *  Get Coord array of plot points: note this routine doesn't
	 *  work if XVW_PLOT2D_PLOTOBJ was used instead of XVW_PLOT2D_POINTS.
	 */
	xvw_get_attribute(object, XVW_PLOT2D_PLOTSIZE, &ptnum);
	xvw_get_attribute(object, XVW_PLOT2D_POINTS,   &points);

        /*
         * Go through each point in the plot array, using the utility routine
         * xvw_plot2d_point_inside_roi() to determine whether or not the point
         * was inside the ROI.  If it was not inside the ROI, copy it to the
         * new array; if it was inside the ROI, leave it out (ie, delete it)
         */
        for (i = 0; i < ptnum; i++)
        {
           /* point outside ROI shape: copy point to new array */
            if (!(xvw_plot2d_point_inside_roi(object, roi, points[i])))
	    {
               new_ptnum++;
               new_points = (Coord *) krealloc(new_points,
                                               new_ptnum*sizeof(Coord));
               new_points[new_ptnum-1].x = points[i].x;
               new_points[new_ptnum-1].y = points[i].y;
               new_points[new_ptnum-1].d = points[i].d;
            }
        }

        /*
         *  Tell plot object to use new array, with new number of points.
         *  Setting XVW_GRAPHICS_RESET_WORLD to FALSE prevents the axes
         *  from resetting themselves to fit the bounds of the new data.
         */
        xvw_set_attributes(object,
                XVW_GRAPHICS_RESET_WORLD, FALSE,
                XVW_PLOT2D_POINTS,        new_points,
                XVW_PLOT2D_PLOTSIZE,      new_ptnum,
                NULL);

        kfree_and_NULL(points);
        ptnum  = new_ptnum;
        points = new_points;

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Plot2DColorPtsInteractively
|
|       Purpose: This routine supports the action attribute
|		 PLOT2D_COLOR_PTS_INTERACTIVELY, which allows
|		 the user to rubberband about desired points in 
|		 the plot and have them colored in a highlight color
|		 automatically.
|
|         Input: object    - the plot2D object
|		 attribute - the attribute name PLOT2D_COLOR_PTS_INTERACTIVELY
|        Output: calldata  - not used
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Danielle Argiro
|          Date: Nov 8, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
int Plot2DColorPtsInteractively(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
        kobject         roi;
	Coord           *points = NULL;
        int             i, ptnum;
	Pixel           color;
	XvwPlot2DGadget xobj = (XvwPlot2DGadget) xvw_widget(object);

        /*
         *  Let user interactively specify ROI in which pts are to be deleted
         */
        xvw_get_attribute(object, XVW_PLOT2D_ROI, &roi);

	/*
	 *  Get Coord array of plot points: note this routine doesn't
	 *  work if XVW_PLOT2D_PLOTOBJ was used instead of XVW_PLOT2D_POINTS.
	 */
	xvw_get_attribute(object, XVW_PLOT2D_PLOTSIZE, &ptnum);
	xvw_get_attribute(object, XVW_PLOT2D_POINTS,   &points);

        /*
         * Go through each point in the plot array, using the utility routine
         * xvw_plot2d_point_inside_roi() to determine whether or not the point
         * was inside the ROI.  If it was not inside the ROI, leave it alone;
         * if it was inside the ROI, give it the highlighted color
         */
	color = xvw_allocate_color(object, xobj->plot2d.highlight_color);
        for (i = 0; i < ptnum; i++)
        {
           /* point outside ROI shape: copy point to new array */
            if (xvw_plot2d_point_inside_roi(object, roi, points[i]))
		points[i].d = color;
        }
	

	Plot2DRefresh(xvw_widget(object));
	return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: Plot2DFindMinMax
|
|       Purpose: This routine will find the min and max values
|		 from the data in a 2D plot object
|
|         Input: xobj - the object for which to find the min and max 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Feb 23, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void Plot2DFindMinMax(
   XvwPlot2DGadget xobj)
{
	register int i;
	register Coord wcmin, wcmax, *points;


	if ((points = xobj->plot2d.points) == NULL)
	   return;

	wcmin.x = wcmax.x = points[0].x;
	wcmin.y = wcmax.y = points[0].y;
	wcmin.d = wcmax.d = points[0].d;

	for (i = xobj->plot2d.plot_size -1; i >= 0; i--)
	{
	   if (points[i].x > wcmax.x) wcmax.x = points[i].x;
	   else if (points[i].x < wcmin.x) wcmin.x = points[i].x;

	   if (points[i].y > wcmax.y) wcmax.y = points[i].y;
	   else if (points[i].y < wcmin.y) wcmin.y = points[i].y;

	   if (points[i].d > wcmax.d) wcmax.d = points[i].d;
	   else if (points[i].d < wcmin.d) wcmin.d = points[i].d;
	}

	xobj->plot2d.data_max_x = wcmax.x;
	xobj->plot2d.data_max_y = wcmax.y;
	xobj->plot2d.data_min_x = wcmin.x;
	xobj->plot2d.data_min_y = wcmin.y;

	if (xobj->graphics.reset_world)
	{
	   if ( wcmax.x < xobj->graphics.wcmin.x )
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMIN_X, wcmin.x,
			         XVW_GRAPHICS_WCMAX_X, wcmax.x,
				 NULL);
	   }
	   else
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMAX_X, wcmax.x,
			         XVW_GRAPHICS_WCMIN_X, wcmin.x,
				 NULL);
	   }

	   if ( wcmax.y < xobj->graphics.wcmin.y )
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMIN_Y, wcmin.y,
			         XVW_GRAPHICS_WCMAX_Y, wcmax.y,
				 NULL);
	   }
	   else
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMAX_Y, wcmax.y,
			         XVW_GRAPHICS_WCMIN_Y, wcmin.y,
				 NULL);
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Plot2DCheckOffset
|
|       Purpose: Error checks a change to the width offset, the height 
|		 offset, the depth offset, the time offset, or the 
|		 elements offset.  
|
|         Input: current - Widget with current settings
|		 new     - Widget with new settings
|
|    Written By: Danielle Argiro
|          Date: Feb 23, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int Plot2DCheckOffset(
   Widget current,
   Widget new)
{
	XvwPlot2DGadget cobj = (XvwPlot2DGadget) current;
        XvwPlot2DGadget nobj = (XvwPlot2DGadget) new;

        /*
         *  Offset into data only applies if we have
         *  a data object, not if we have a coord array.
         */
        if ((nobj->plot2d.pobject == NULL) &&
            (nobj->plot2d.points != NULL))
        {
            kerror(XVISUAL, "SetValues", "X orientation / offset settings do not apply unless input is specified with a data object");
            nobj->plot2d.color_origination = KPLOT2D_COLOR_FROM_FOREGROUND;
        }

        if (nobj->plot2d.x_orientation == KWIDTH)
        {
            if (cobj->plot2d.width_offset  != nobj->plot2d.width_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Width offset must stay 0 when X Orientation is set to 'Width'");
                nobj->plot2d.width_offset = 0;
                return(FALSE);
            }
            nobj->plot2d.width_offset = 0;
        }
        if (nobj->plot2d.x_orientation == KHEIGHT)
        {
            if (cobj->plot2d.height_offset  != nobj->plot2d.height_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                     "Height offset must stay 0 when X Orientation is set to 'Height'");
                nobj->plot2d.height_offset = 0;
                return(FALSE);
            }
            nobj->plot2d.height_offset = 0;
        }
        if (nobj->plot2d.x_orientation == KDEPTH)
        {
            if (cobj->plot2d.depth_offset  != nobj->plot2d.depth_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Depth offset must stay 0 when X Orientation is set to 'Depth'");
                nobj->plot2d.depth_offset = 0;
                return(FALSE);
            }
            nobj->plot2d.depth_offset = 0;
        }
        if (nobj->plot2d.x_orientation == KTIME)
        {
            if (cobj->plot2d.time_offset  != nobj->plot2d.time_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Time offset must stay 0 when X Orientation is set to 'Time'");
                nobj->plot2d.time_offset = 0;
                return(FALSE);
            }
            nobj->plot2d.time_offset = 0;
        }
        if (nobj->plot2d.x_orientation == KELEMENTS)
        {
            if (cobj->plot2d.elements_offset  != nobj->plot2d.elements_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Elements offset must stay 0 when X Orientation is set to 'Elements'");
                nobj->plot2d.elements_offset = 0;
                return(FALSE);
            }
            nobj->plot2d.elements_offset = 0;
        }
	return(TRUE);
}

