/*
  * Khoros: $Id: Plot3D.c,v 1.17.4.1 1996/12/06 20:29:36 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Plot3D.c,v 1.17.4.1 1996/12/06 20:29:36 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		3D Plot Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Redisplay()
   >>>>                 GetObjectName()
   >>>>                 SetObjectName()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 ColorReloadCallback()
   >>>>                 ReloadPlot()
   >>>>   Public:
   >>>>                 xvw_create_plot3d()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/Plot3DP.h>


static void ClassInitialize PROTO((void));
static void Initialize	    PROTO((Widget, Widget, ArgList, Cardinal *));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static int  GetObjectName   PROTO((xvobject, char *, kaddr));
static int  SetObjectName   PROTO((xvobject, char *, kaddr));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void ColorReloadCallback   PROTO((xvobject, kaddr));
static void Destroy	    PROTO((Widget));
static void ReloadPlot	    PROTO((Widget));


/*--------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PLOT3D_PLOTOBJ,           NULL,    XtRInt,        NULL},
{XVW_PLOT3D_DATA_MAX_X,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_DATA_MAX_Y,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_DATA_MAX_Z,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_DATA_MIN_X,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_DATA_MIN_Y,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_DATA_MIN_Z,        NULL,    XtRDouble,     NULL},
{XVW_PLOT3D_PLOTTYPE,          NULL,    XtRInt,        NULL},
{XVW_PLOT3D_SHADETYPE,         NULL,    XtRInt,        NULL},
{XVW_PLOT3D_POINTS,            NULL,    XtRPointer,    NULL},
{XVW_PLOT3D_PLOTSIZE,          NULL,    XtRInt,        NULL},
{XVW_PLOT3D_PLOTWIDTH,         NULL,    XtRInt,        NULL},
{XVW_PLOT3D_COLOR_ORIGINATION, NULL,    XtRInt,        NULL},
{XVW_PLOT3D_X_ORIENTATION,     NULL,    XtRInt,        NULL},
{XVW_PLOT3D_Y_ORIENTATION,     NULL,    XtRInt,        NULL},
{XVW_PLOT3D_WIDTH_OFFSET,      NULL,    XtRInt,        NULL},
{XVW_PLOT3D_HEIGHT_OFFSET,     NULL,    XtRInt,        NULL},
{XVW_PLOT3D_DEPTH_OFFSET,      NULL,    XtRInt,        NULL},
{XVW_PLOT3D_TIME_OFFSET,       NULL,    XtRInt,        NULL},
{XVW_PLOT3D_ELEMENTS_OFFSET,   NULL,    XtRInt,        NULL},
{XVW_PLOT3D_NUM_CONTOUR_LEVELS,  NULL,    XtRInt,        NULL},
{XVW_PLOT3D_CONTOUR_LEVELS,    NULL,    XtRPointer,    NULL},
};

/*--------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPlot3DGadgetRec, plot3d.field)

static double data_min     = 0.0,
              data_max     = 1.0;

static XtResource resources[] = {
{XVW_PLOT3D_DATA_MAX_X,  NULL, XtRDouble, sizeof(double),
      offset(data_max_x), XtRDouble, (XtPointer) &data_max},
{XVW_PLOT3D_DATA_MAX_Y,  NULL, XtRDouble, sizeof(double),
      offset(data_max_y), XtRDouble, (XtPointer) &data_max},
{XVW_PLOT3D_DATA_MAX_Z,  NULL, XtRDouble, sizeof(double),
      offset(data_max_z), XtRDouble, (XtPointer) &data_max},
{XVW_PLOT3D_DATA_MIN_X,  NULL, XtRDouble, sizeof(double),
      offset(data_min_x), XtRDouble, (XtPointer) &data_min},
{XVW_PLOT3D_DATA_MIN_Y,  NULL, XtRDouble, sizeof(double),
      offset(data_min_y), XtRDouble, (XtPointer) &data_min},
{XVW_PLOT3D_DATA_MIN_Z,  NULL, XtRDouble, sizeof(double),
      offset(data_min_z), XtRDouble, (XtPointer) &data_min},
{XVW_PLOT3D_PLOTTYPE,  NULL, XtRInt, sizeof(int),
      offset(plot_type), XtRImmediate, (XtPointer) KPLOT3D_LINEPLOT},
{XVW_PLOT3D_SHADETYPE,  NULL, XtRInt, sizeof(int),
      offset(shade_type), XtRImmediate, (XtPointer) KPLOT3D_SHADE_IMAGERY},
{XVW_PLOT3D_POINTS, NULL, XtRPointer, sizeof(Coord *),
      offset(points), XtRPointer, (XtPointer) NULL},
{XVW_PLOT3D_PLOTSIZE, NULL, XtRInt, sizeof(int),
      offset(plot_size), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_PLOTWIDTH, NULL, XtRInt, sizeof(int),
      offset(plot_width), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_PLOTOBJ, NULL, XtRPointer, sizeof(kobject),
      offset(pobject), XtRPointer, (XtPointer) NULL},
{XVW_PLOT3D_COLOR_ORIGINATION, NULL, XtRInt, sizeof(int),
      offset(color_origination), XtRImmediate,
      (XtPointer) KPLOT3D_COLOR_FROM_FOREGROUND},
{XVW_PLOT3D_X_ORIENTATION, NULL, XtRInt, sizeof(int),
      offset(x_orientation), XtRImmediate, (XtPointer) KWIDTH},
{XVW_PLOT3D_Y_ORIENTATION, NULL, XtRInt, sizeof(int),
      offset(y_orientation), XtRImmediate, (XtPointer) KHEIGHT},
{XVW_PLOT3D_WIDTH_OFFSET, NULL, XtRInt, sizeof(int),
      offset(width_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_HEIGHT_OFFSET, NULL, XtRInt, sizeof(int),
      offset(height_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_DEPTH_OFFSET, NULL, XtRInt, sizeof(int),
      offset(depth_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_TIME_OFFSET, NULL, XtRInt, sizeof(int),
      offset(time_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_ELEMENTS_OFFSET, NULL, XtRInt, sizeof(int),
      offset(elements_offset), XtRImmediate, (XtPointer) 0},
{XVW_PLOT3D_NUM_CONTOUR_LEVELS, NULL, XtRInt, sizeof(int),
      offset(num_contour_levels), XtRImmediate, (XtPointer) -1},
{XVW_PLOT3D_CONTOUR_LEVELS, NULL, XtRPointer, sizeof(Real *),
      offset(contour_levels), XtRPointer, (XtPointer) NULL},

{XVW_GRAPHICS_CLIPPING, NULL, XtRBoolean, sizeof(Boolean), 
      XtOffsetOf(XvwPlot3DGadgetRec, graphics.clipping), XtRImmediate,
      (XtPointer) FALSE},
{NULL, NULL, XtRInt, sizeof(int), 
      XtOffsetOf(XvwPlot3DGadgetRec, graphics.graphics_type), XtRImmediate,
      (XtPointer) KGRAPHICS_3D},
{NULL, NULL, XtRInt, sizeof(int),
      XtOffsetOf(XvwPlot3DGadgetRec, color.autocolor), 
      XtRImmediate, (XtPointer) KRAINBOW},
};
#undef offset


#define SUPERCLASS (&xvwColorGadgetClassRec)

XvwPlot3DGadgetClassRec xvwPlot3DGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Plot3D",				/* class_name		  */
    sizeof(XvwPlot3DGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),			/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    FALSE,				/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    XtInheritRecomputePosition,		/* recompute position proc    */
  },  /* XvwGraphicsGadgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwPlot3DGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPlot3DGadgetClass for public consumption
 */
WidgetClass xvwPlot3DGadgetClass = (WidgetClass) &xvwPlot3DGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of plot3d object class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPlot3DGadgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/Plot3D.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/Plot3D");

        xvw_define_attribute(xvwPlot3DGadgetClass, XVW_PLOT3D_PLOTFILE,
		XtRString, SetObjectName, GetObjectName);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial settings 
|                for a 3D plot object instance. 
|
|         Input: request - not used
|                new     - object instance after initialization, 
|                          with attributes set
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPlot3DGadget xobj = (XvwPlot3DGadget) new;


	/*
	 *  Find the world coordinate minimum and maximum values (some of
	 *  the 3D plotting routines need them.
	 */
	ReloadPlot(new);
	Plot3DFindMinMax(xobj);
	xobj->plot3d.contour_levels = NULL;
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay 
|
|       Purpose: This method is used to initially draw the 
|                the 3D plot object and for redrawing the object 
|                whenever part of it becomes exposed.
|
|         Input: widget - the object being exposed
|                event  - the expose event with information
|                         about what is being exposed
|                region - the region being exposed
|
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
-----------------------------------------------------------*/
/* ARGSUSED */
#define FAKE_NUM 15
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwPlot3DGadget xobj = (XvwPlot3DGadget) widget;

	register Coord *points;
	XColor   *bg, *plotcolor, *compact_colors = NULL, *xcolors = NULL;
	register int i, width, size, colornum, num_contour_levels;
	XVGraphicsID id;

	/*
	 *  Check to see if we are currently visible and we have points
	 *  to be displayed
	 */
        if ( !xobj->plot3d.points )
	   return;

        if (!xobj->plot3d.plot_size)
        {
           kerror(XVISUAL, "Redisplay",
                  "The plot size is 0 can not display plot");
           return;
           
        }

        if (!xobj->plot3d.plot_width)
        {
           kerror(XVISUAL, "Redisplay",
                  "The plot width is 0 can not display plot");
           return;
        }

	id	  = xobj->graphics.id;
	size	  = xobj->plot3d.plot_size;
	width	  = xobj->plot3d.plot_width;
	points	  = xobj->plot3d.points;
	bg	  = &xobj->graphics.bg;
	plotcolor = &xobj->graphics.fg;

	/*
	 *  If we are doing a shaded plot, need to get the colors
	 *  from the colormap in which to draw it.  Need to compact
	 *  the colormap so that there are no "unused entries" within
	 *  it - this prevents "dropout" colors from appearing.
	 */
        if (xobj->plot3d.plot_type == KPLOT3D_CONTOUR_2D        ||
            xobj->plot3d.plot_type == KPLOT3D_CONTOUR_3D        ||
            xobj->plot3d.plot_type == KPLOT3D_CONSTANT_SHADING  ||
            xobj->plot3d.plot_type == KPLOT3D_GHOURAUD_SHADING  ||
            xobj->plot3d.plot_type == KPLOT3D_PHONG_SHADING)
        {
	    /* 
	     * there was no colormap set: all we have is the fg color 
	     */
            if ((xcolors = ColorGetXColors(widget)) == NULL ||
		xobj->plot3d.color_origination == KPLOT3D_COLOR_FROM_FOREGROUND ) 
	    {
	        compact_colors = (XColor *) kmalloc(sizeof(XColor) * FAKE_NUM);
		for (i = 0; i < FAKE_NUM; i++)
		    compact_colors[i] = *plotcolor;
		colornum = FAKE_NUM;
	    }
	    else
	    {
	        compact_colors = (XColor *) 
	    	                kmalloc((size_t) xobj->color.info->ncolors * 
			 		         sizeof(XColor));
	        for (colornum = 0, i = 0;  i < xobj->color.info->ncolors; i++)
	        {
	             if (xcolors[i].flags != 0)
		        compact_colors[colornum++] = xcolors[i];
	        }
	        /* nothing useful in colormap - use fg color */
	        if (colornum == 0)
	        {
		    for (i = 0; i < xobj->color.info->ncolors; i++)
	  	        compact_colors[i] = *plotcolor;
		    colornum = xobj->color.info->ncolors;
	        }
	    }
	}

	switch (xobj->plot3d.plot_type)
	{
	   case KPLOT3D_LINEPLOT:
		X3D_draw_lineplot(id, points, width, size, plotcolor);
		break;

	   case KPLOT3D_MESH:
		X3D_draw_mesh(id, points, width, size, plotcolor, bg);
		break;

	   case KPLOT3D_IMPULSE:
		X3D_draw_impulse(id, points, size, plotcolor);
		break;

	   case KPLOT3D_SCATTER:
		X3D_draw_polymarker(id, points, size, 
				    xobj->graphics.marker_scale, 
				    xobj->graphics.marker_type, 
				    plotcolor);
		break;

	   case KPLOT3D_HORIZON:
		X3D_draw_horizon(id, points, width, size, plotcolor, bg,
			xobj->graphics.wcmin, xobj->graphics.wcmax);
		break;

	   case KPLOT3D_WIREFRAME:
		X3D_draw_wireframe(id, points, width, size, plotcolor);
		break;

	   case KPLOT3D_CONTOUR_2D:
	        num_contour_levels = (xobj->plot3d.num_contour_levels == -1? 
				colornum:xobj->plot3d.num_contour_levels);
		X3D_draw_contour(id, points, width, size, xobj->plot3d.contour_levels,
			compact_colors, xobj->color.begin, num_contour_levels, TRUE);
		break;

	   case KPLOT3D_CONTOUR_3D:
	        num_contour_levels = (xobj->plot3d.num_contour_levels == -1? 
				colornum:xobj->plot3d.num_contour_levels);
		X3D_draw_contour(id, points, width, size, xobj->plot3d.contour_levels,
			compact_colors, xobj->color.begin, num_contour_levels, FALSE);
		break;  

	   case KPLOT3D_CONSTANT_SHADING:
		X3D_shade_constant(id, points, width, size, NULL,
			compact_colors, xobj->color.begin, colornum,
		        xobj->graphics.wcmin, xobj->graphics.wcmax,
			xobj->plot3d.shade_type);
		break;

	   case KPLOT3D_GHOURAUD_SHADING:
		X3D_shade_ghouraud(id, points, width, size, NULL,
			compact_colors, xobj->color.begin, colornum,
			xobj->graphics.wcmin, xobj->graphics.wcmax,
			xobj->plot3d.shade_type);
		break;

	   case KPLOT3D_PHONG_SHADING:
		X3D_shade_phong(id, points, width, size, NULL,
			compact_colors, xobj->color.begin, colornum,
			xobj->graphics.wcmin, xobj->graphics.wcmax,
			xobj->plot3d.shade_type);
		break;

	}
	kfree(compact_colors);
}

/*-----------------------------------------------------------
|
|  Routine Name: GetObjectName
|
|       Purpose: GetObjectName is used to get the filename associated
|		 with the attribute.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|
|        Output: calldata  - pointer to retrieve the object name
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int GetObjectName(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	kobject plot;
	XvwPlot3DGadget xobj = (XvwPlot3DGadget) xvw_widget(object);
	String *filename = (String *) calldata;


	if (kstrcmp(attribute, XVW_PLOT3D_PLOTFILE) == 0)
	   plot = xobj->plot3d.pobject;
	else
	{
	   kerror(XVISUAL, "GetObjectName", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	*filename = NULL;
	return(kpds_get_attribute(plot, KPDS_NAME, filename));
}

/*-----------------------------------------------------------
|
|  Routine Name: SetObjectName
|
|       Purpose: SetObjectName is used to set the filename associated
|		 with the attribute.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int SetObjectName(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	kobject plot = NULL;
	int	locate = TRUE;
	String *filename = (String *) calldata;


	if (*filename != NULL && (plot = kdms_locate(*filename)) == NULL)
	{
	   locate = FALSE;
	   if ((plot = kpds_open_object(*filename, KOBJ_READ)) == NULL)
	   {
	      kerror(XVISUAL, "SetObjectName", "Failed to load 3D Plot '%s'",
			*filename);
	      return(TRUE);
	   }
	}

	if (kstrcmp(attribute, XVW_PLOT3D_PLOTFILE) == 0)
	   xvw_set_attribute(object, XVW_PLOT3D_PLOTOBJ, plot);
	else
	{
	   kerror(XVISUAL, "SetObjectName", "invalid attribute %s",
			attribute);
	   if (!locate) kpds_close_object(plot);
	   return(FALSE);
	}
	if (!locate) kpds_close_object(plot);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a 3D plot object instance.  These values
|                include the name, width, and height of the plot as
|                well as the plot image or object. 
|
|         Input: current - the object containing current settings
|                request - the object containing requested settings
|                new     - the object processed through all set values methods
|
|       Returns: FALSE (0) - XPlotRedraw will handle the redraw in any case  
|    Written By: Mark Young
|          Date: Aug 03, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPlot3DGadget cobj = (XvwPlot3DGadget) current;
	XvwPlot3DGadget robj = (XvwPlot3DGadget) request;
	XvwPlot3DGadget nobj = (XvwPlot3DGadget) new;

	xvobject object;
	Boolean  redisplay = FALSE;
	size_t   size;
	int      i;
	Coord    *tmp;

	if (cobj->plot3d.pobject       != nobj->plot3d.pobject ||
            cobj->plot3d.points        != nobj->plot3d.points  ||
            robj->graphics.reset_world != nobj->graphics.reset_world)
	{
	   if (cobj->plot3d.points != nobj->plot3d.points)
	   {
	      size = (size_t) (nobj->plot3d.plot_size * sizeof(Coord));
	      tmp = (Coord *) kmalloc(size);
              if (tmp == NULL)
              {
                 kerror(XVISUAL, "SetValues",
                        "Could not allocate memory for plot data");
              }
              else
              {
                 if (cobj->plot3d.plot_width == nobj->plot3d.plot_width &&
                     nobj->plot3d.plot_size != 0)
                 {
                    nobj->plot3d.plot_width = 
				ksqrt((float) nobj->plot3d.plot_size);
                 }
                 kfree_and_NULL(cobj->plot3d.points);
                 kmemcpy(tmp, nobj->plot3d.points, (size_t) size);
                 nobj->plot3d.points = tmp;
	         for (i = 0; i < nobj->plot3d.plot_size; i++)
	         {
	             if (tmp[i].d != KGRAPHICS_UNINITIALIZED)
	                tmp[i].d = ColorAllocate(new, (unsigned) tmp[i].d);
	         }
              }
	   }

	   if (cobj->plot3d.pobject != nobj->plot3d.pobject)
	   {
	      object = xvw_object(new);
	      nobj->plot3d.pobject =kpds_reference_object(nobj->plot3d.pobject);
	      kpds_close_object(cobj->plot3d.pobject);
	      if (cobj->color.object == nobj->color.object)
	          xvw_set_attribute(xvw_object(new), XVW_COLOR_COLOROBJ,
                        nobj->plot3d.pobject);
	
	      /* if explicit settings are not done, reapply defaults */
	      if (cobj->plot3d.x_orientation == nobj->plot3d.x_orientation)
		  nobj->plot3d.x_orientation = KWIDTH;
	      if (cobj->plot3d.y_orientation == nobj->plot3d.y_orientation)
		  nobj->plot3d.y_orientation = KHEIGHT;
	      if (cobj->plot3d.width_offset == nobj->plot3d.width_offset)
		  nobj->plot3d.width_offset = 0;
	      if (cobj->plot3d.height_offset == nobj->plot3d.height_offset)
		  nobj->plot3d.height_offset = 0;
	      if (cobj->plot3d.depth_offset == nobj->plot3d.depth_offset)
		  nobj->plot3d.depth_offset = 0;
	      if (cobj->plot3d.time_offset == nobj->plot3d.time_offset)
		  nobj->plot3d.time_offset = 0;
	      if (cobj->plot3d.elements_offset == nobj->plot3d.elements_offset)
		  nobj->plot3d.elements_offset = 0;
	   }
	   ReloadPlot(new);
	   Plot3DFindMinMax(nobj);
	   redisplay = TRUE;
	}

        if (cobj->plot3d.x_orientation   != nobj->plot3d.x_orientation ||
            cobj->plot3d.y_orientation   != nobj->plot3d.y_orientation ||
            cobj->plot3d.width_offset    != nobj->plot3d.width_offset  ||
            cobj->plot3d.height_offset   != nobj->plot3d.height_offset ||
            cobj->plot3d.depth_offset    != nobj->plot3d.depth_offset  ||
            cobj->plot3d.time_offset     != nobj->plot3d.time_offset   ||
            cobj->plot3d.elements_offset != nobj->plot3d.elements_offset)
        {
            if (Plot3DCheckOffset(current, new))
            {
                nobj->graphics.reset_world = TRUE;
                ReloadPlot(new);
                Plot3DFindMinMax(nobj);
                redisplay = TRUE;
            }
        }

        if (cobj->plot3d.color_origination != nobj->plot3d.color_origination)
        {
            /*
             *  Color origination from data only applies if we have
             *  a data object, not if we have a coord array.
             */
            if ((nobj->plot3d.pobject == NULL) &&
	        (nobj->plot3d.points != NULL))
            {
                kerror(XVISUAL, "SetValues", "Color origination must be set to foreground color unless input is specified as a data object.");
                nobj->plot3d.color_origination = KPLOT3D_COLOR_FROM_FOREGROUND;
            }

	    /*
	     *  If we are changing color origination from data to foreground,
	     *  and we have a data object (not a coord array) then set d values
	     *  back to KGRAPHICS_UNINITIALIZED so fg color can "take".
             */
            if ((nobj->plot3d.color_origination == KPLOT3D_COLOR_FROM_FOREGROUND) &&
		(nobj->plot3d.pobject != NULL) && 
		(nobj->plot3d.points != NULL))
            {
                for (i = 0; i < nobj->plot3d.plot_size; i++)
                {
                     if (nobj->plot3d.points[i].d != KGRAPHICS_UNINITIALIZED)
                        nobj->plot3d.points[i].d = 
				(Index) KGRAPHICS_UNINITIALIZED;
                }
                ColorFree(current, 0, -1);
                ColorStore(new, -1, NULL);
            }
            ReloadPlot(new);
            redisplay = TRUE;
        }

	if ((cobj->plot3d.num_contour_levels != nobj->plot3d.num_contour_levels) &&
	    (nobj->plot3d.num_contour_levels == 0 || nobj->plot3d.num_contour_levels < -1))
	{
	    kerror(XVISUAL, "Plot3D SetValues",
                  "Invalid contour num_contour_levels '%d' specified; resetting contour "
		  "level number to -1 (levels automatically calculated)",
		  nobj->plot3d.num_contour_levels);
	    nobj->plot3d.num_contour_levels = -1;
	}

	if (cobj->plot3d.plot_type  != nobj->plot3d.plot_type ||
	    cobj->plot3d.shade_type != nobj->plot3d.shade_type)
	{
	   if ( (cobj->plot3d.plot_type  != nobj->plot3d.plot_type) &&
	        ((nobj->plot3d.plot_width==0)||(nobj->plot3d.plot_width==1)) &&
	         (nobj->plot3d.plot_type == KPLOT3D_MESH       ||
                  nobj->plot3d.plot_type == KPLOT3D_CONTOUR_2D ||
                  nobj->plot3d.plot_type == KPLOT3D_CONTOUR_3D ||
                  nobj->plot3d.plot_type == KPLOT3D_CONSTANT_SHADING))
	   {
	            kerror(XVISUAL, "Plot3D SetValues",
		           "Cannot display current data as mesh, contour 2D, contour 3D or constant shade plot;  these plot types need each row of data to be greater than size 1.");

	   }
	   else redisplay = TRUE;
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Close the image, shape, colormap, and overlay
|                objects before the image widget is destroyed.
|
|         Input: widget - the image widget being destroyed
|
|    Written By: Mark Young
|          Date: Nov 21, 1992 12:52
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPlot3DGadget xobj = (XvwPlot3DGadget) widget;

        /*
         *  Close plot object
         */
        kpds_close_object(xobj->plot3d.pobject);
        kfree_and_NULL(xobj->plot3d.points);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Refresh the palette display since our ColorClass
|		 has had a change to its map data, and we are required
|		 to change the colors in the palette.
|
|         Input: object - the object to be refreshed (the Plot3D)
|		 call_data   - (not used)
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: June 28, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	Widget widget = xvw_widget(object);
	Region region = XCreateRegion();
	XEvent event;

	event.type = Expose;
	event.xexpose.x = event.xexpose.y = 0;
	event.xexpose.width  = widget->core.width;
	event.xexpose.height = widget->core.height;
	XtAddExposureToRegion(&event, region);
	ReloadPlot(widget);
	Redisplay(widget, &event, region);
	XDestroyRegion(region);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReloadPlot
|
|       Purpose: This routine loads in the data and stores the
|                data points in the plot3d coord structure.
|
|         Input: xobj - the object to subsample 
|
|        Output: None
|
|       Returns: None 
|
|    Written By: Mark Young
|          Date: Oct 24, 1992 23:59
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReloadPlot(
   Widget widget)
{
	XvwPlot3DGadget xobj = (XvwPlot3DGadget) widget;

	kobject  object;
        char     *format;
	int      i, w, h, d, t, e;
	int      width = 1, height = 1, depth = 1, ttime = 1, elements = 1; 
	int      size, x_size, datatype;
	double   *plotdata = NULL; 
	double   min_z; 
	Coord    *points = NULL;

	unsigned char *pixels = NULL;
	unsigned char *mask   = NULL;

	int color_orig = xobj->plot3d.color_origination;

	/*
	 *  Sanity check to make sure that the image object exists
	 */ 
	if ((object = xobj->plot3d.pobject) == NULL)
	   return;

	/*
	 *  Get the value data dimensionality
	 */
	kpds_get_attributes(xobj->plot3d.pobject,
		            KPDS_VALUE_SIZE,      &w, &h, &d, &t, &e,
			    KPDS_FORMAT,          &format,
		            KPDS_VALUE_DATA_TYPE, &datatype,
		            NULL);

        /* get sizes based on the X & Y orientations specified */
        if (xobj->plot3d.x_orientation == KWIDTH)         width    = x_size = w;
        else if (xobj->plot3d.x_orientation == KHEIGHT)   height   = x_size = h;
        else if (xobj->plot3d.x_orientation == KDEPTH)    depth    = x_size = d;
        else if (xobj->plot3d.x_orientation == KTIME)     ttime    = x_size = t;
        else if (xobj->plot3d.x_orientation == KELEMENTS) elements = x_size = e;

	if (xobj->plot3d.y_orientation == KWIDTH)         width    = w; 
        else if (xobj->plot3d.y_orientation == KHEIGHT)   height   = h;
        else if (xobj->plot3d.y_orientation == KDEPTH)    depth    = d;
        else if (xobj->plot3d.y_orientation == KTIME)     ttime    = t;
        else if (xobj->plot3d.y_orientation == KELEMENTS) elements = e;

	size = width*height*depth*ttime*elements;

	/*
         * Special case: triples of ascii data points
         */
        if (w == 3)
        {
            /*
             * Go ahead and malloc room for the new data...
             */
            if (xobj->plot3d.plot_size != h)
            {
                xobj->plot3d.plot_size = h;
                xobj->plot3d.points = (Coord *) krealloc(xobj->plot3d.points,
                                      xobj->plot3d.plot_size*sizeof(Coord));
                if (xobj->plot3d.points == NULL)
                {
                    kerror(XVISUAL, "ReloadPlot",
                           "Unable to allocate plot coordinate data");
                    return;
                }
            }

            /*
             * get data points in triples
             */

            points = xobj->plot3d.points;
	    xobj->plot3d.plot_width = 1;
            kpds_set_attribute(xobj->plot3d.pobject, KPDS_VALUE_REGION_SIZE,
                               w, 1, 1, 1, 1);
            for (i = 0; i < h; i++)
            {
                plotdata = (double *) kpds_get_data(object, KPDS_VALUE_REGION, 
						    plotdata);
                points[i].x = plotdata[0];
                points[i].y = plotdata[1];
                points[i].z = plotdata[2];
		if (color_orig == KPLOT3D_COLOR_FROM_DATA)
                    points[i].d = (Index) ColorAllocate(widget,
                                                       (unsigned) plotdata[2]);
	        else points[i].d = (Index) KGRAPHICS_UNINITIALIZED;
            }
            kfree_and_NULL(plotdata);
        }

	/*
         *  General case: X and Y are implicit, Z and D are in value data
         */
	else
	{
	    /* allocate points array */
            points = (Coord *) kmalloc(size*sizeof(Coord));

            kpds_set_attribute(xobj->plot3d.pobject, KPDS_VALUE_REGION_SIZE,
                               width, height, depth, ttime, elements);
            kpds_set_attribute(xobj->plot3d.pobject, KPDS_MASK_REGION_SIZE,
                               width, height, depth, ttime, elements);
            kpds_set_attribute(object, KPDS_VALUE_POSITION,
                               xobj->plot3d.width_offset,
                               xobj->plot3d.height_offset,
                               xobj->plot3d.depth_offset,
                               xobj->plot3d.time_offset,
                               xobj->plot3d.elements_offset,
                               NULL);
            kpds_set_attribute(object, KPDS_MASK_POSITION,
                               xobj->plot3d.width_offset,
                               xobj->plot3d.height_offset,
                               xobj->plot3d.depth_offset,
                               xobj->plot3d.time_offset,
                               xobj->plot3d.elements_offset,
                               NULL);

            /*
             *  need to do color normalization for float, double,
             *  complex data types the same way that the image object does.
             *  NOTE: same code in Plot3D.c, ColorUtil.c
             */
            if (datatype == KFLOAT   || datatype == KDOUBLE ||
                datatype == KCOMPLEX || datatype == KDCOMPLEX)
            {
                double dmin, dmax, offset, factor;
                kpds_set_attributes(object,
                                    KPDS_VALUE_SCALING, KNONE,
                                    KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
                                    NULL);
                kpds_get_attribute(object, KPDS_VALUE_MINMAX, &dmin, &dmax);
                factor = (255.0/(dmax-dmin));
                offset = 0.0-dmin*factor;

                kpds_set_attributes(object,
                                    KPDS_VALUE_SCALING,      KSCALE,
                                    KPDS_VALUE_SCALE_FACTOR, factor,
                                    KPDS_VALUE_SCALE_OFFSET, offset, 0.0,
                                    KPDS_VALUE_DATA_TYPE,    KUBYTE,
                                    NULL);
            }

            if ((pixels = (unsigned char *)
                           kpds_get_data(object, KPDS_VALUE_REGION, pixels)))
            {
                for (i = 0; i < size; i++)
                {
                    if (color_orig == KPLOT2D_COLOR_FROM_DATA)
                        points[i].d = (Index) ColorAllocate(widget,
                                              (unsigned int) pixels[i]);
                    else points[i].d = (Index) KGRAPHICS_UNINITIALIZED;
                }
	        kfree_and_NULL(pixels);
            }


            /*
             *  Get the plot data from the value segment
             */
            kpds_set_attributes(object,
                                KPDS_VALUE_DATA_TYPE,    KDOUBLE,
                                KPDS_VALUE_SCALING,      KNONE,
                                KPDS_VALUE_SCALE_OFFSET, 0.0, 0.0,
                                NULL);
            kpds_set_attribute(object, KPDS_VALUE_POSITION,
                               xobj->plot3d.width_offset,
                               xobj->plot3d.height_offset,
                               xobj->plot3d.depth_offset,
                               xobj->plot3d.time_offset,
                               xobj->plot3d.elements_offset,
                               NULL);

            plotdata = (double *)
                        kpds_get_data(object, KPDS_VALUE_REGION, NULL);

	    min_z = plotdata[0];
            for (i = 0; i < size; i++)
            {
                points[i].x = i % x_size;
		points[i].y = i / x_size;
                points[i].z = plotdata[i];
	        if (points[i].z < min_z) 
		    min_z = points[i].z;
            }
            kfree_and_NULL(xobj->plot3d.points);
            xobj->plot3d.points     = points;
            xobj->plot3d.plot_size  = size;
            xobj->plot3d.plot_width = x_size;
	    kfree_and_NULL(plotdata);
        }

	/*
         *  Data object contains a mask: need to use only those points
         *  with mask values of 1.  Set tmp ptr to original points array;
         *  transfer only the valid points back into original points array.
         */
        if (kpds_query_mask(object))
        {
            kpds_set_attribute(object, KPDS_MASK_DATA_TYPE, KUBYTE);
            kpds_set_attribute(object, KPDS_MASK_POSITION,
                           xobj->plot3d.width_offset,
                           xobj->plot3d.height_offset,
                           xobj->plot3d.depth_offset,
                           xobj->plot3d.time_offset,
                           xobj->plot3d.elements_offset,
                           NULL);
            mask = (unsigned char *)
                    kpds_get_data(object, KPDS_MASK_REGION, NULL);

            for (i = 0; i < size; i++)
            {
                if (mask[i] == 0)
                {
                     points[i].z = min_z;
                     points[i].d = (Index) KGRAPHICS_UNINITIALIZED;
                }
            }
            xobj->graphics.reset_world = TRUE;
	    kfree_and_NULL(mask);
        }

        kpds_set_attribute(object, KPDS_VALUE_DATA_TYPE, datatype);
}


/************************************************************
*
*  Routine Name: xvw_create_plot3d - create a 3D plot object
*
*       Purpose: A 3D plot object is used to display a 3D plot.
*                A variety of plot types are supported, including
*                line plots, mesh plots, scatter plots, 2D and 3D
*                contour plots, impulse plots, horizon plots, wireframe
*                plots, and shaded plots.
*
*                Ideally, the 3D plot object would be used in conjunction 
*                with a 3D axis object, where both were created as children
*                of an area object so that a coordinated world coordinate
*                view is automatically supported.  Unfortunately, the 3D
*		 axis object has not yet been developed (sorry, folks).
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the 3D plot
*                         object (for use in app-defaults files, etc)
*        Output: 
*       Returns: Returns the created 3D plot xvobject, or NULL upon failure
*
*  Restrictions:
*    Written By: Mark Young and John M. Salas
*          Date: Aug 03, 1994
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_plot3d(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the main plot workspace
	 */
	object = xvw_create(parent, FALSE, TRUE, name, Plot3DGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,     /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
