/*
 * Khoros: $Id: Plot3Dutil.c,v 1.3 1996/04/04 23:29:51 fas Exp $
 */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Plot3Dutil.c,v 1.3 1996/04/04 23:29:51 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Plot3D Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 Plot3DFindMinMax()
   >>>>                 Plot3DCheckOffset()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/Plot3DP.h>

/*-----------------------------------------------------------
|
|  Routine Name: Plot3DFindMinMax
|
|       Purpose: This routine will find the min and max values
|		 from the data in a 3D plot object
|
|         Input: xobj - the object for which to find the min and max 
|
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: Feb 23, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void Plot3DFindMinMax(
   XvwPlot3DGadget xobj)
{
	register int i;
	register Coord wcmin, wcmax, *points;


	if ((points = xobj->plot3d.points) == NULL)
	   return;

	wcmin.x = wcmax.x = points[0].x;
	wcmin.y = wcmax.y = points[0].y;
	wcmin.z = wcmax.z = points[0].z;
	wcmin.d = wcmax.d = points[0].d;

	for (i = xobj->plot3d.plot_size-1; i >= 0; i--)
	{
	   if (points[i].x > wcmax.x) wcmax.x = points[i].x;
	   else if (points[i].x < wcmin.x) wcmin.x = points[i].x;

	   if (points[i].y > wcmax.y) wcmax.y = points[i].y;
	   else if (points[i].y < wcmin.y) wcmin.y = points[i].y;

	   if (points[i].z > wcmax.z) wcmax.z = points[i].z;
	   else if (points[i].z < wcmin.z) wcmin.z = points[i].z;

	   if (points[i].d > wcmax.d) wcmax.d = points[i].d;
	   else if (points[i].d < wcmin.d) wcmin.d = points[i].d;
	}

	xobj->plot3d.data_max_x = wcmax.x;
	xobj->plot3d.data_max_y = wcmax.y;
	xobj->plot3d.data_max_z = wcmax.z;
	xobj->plot3d.data_min_x = wcmin.x;
	xobj->plot3d.data_min_y = wcmin.y;
	xobj->plot3d.data_min_z = wcmin.z;

	if (xobj->graphics.reset_world)
	{
	   if ( wcmax.x < xobj->graphics.wcmin.x )
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMIN_X, wcmin.x,
			         XVW_GRAPHICS_WCMAX_X, wcmax.x,
				 NULL);
	   }
	   else
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMAX_X, wcmax.x,
			         XVW_GRAPHICS_WCMIN_X, wcmin.x,
				 NULL);
	   }

	   if ( wcmax.y < xobj->graphics.wcmin.y )
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMIN_Y, wcmin.y,
			         XVW_GRAPHICS_WCMAX_Y, wcmax.y,
				 NULL);
	   }
	   else
	   {
	      xvw_set_attributes(xvw_object((Widget) xobj),
			         XVW_GRAPHICS_WCMAX_Y, wcmax.y,
			         XVW_GRAPHICS_WCMIN_Y, wcmin.y,
				 NULL);
	   }

	   if ( wcmax.z < xobj->graphics.wcmin.z )
           {
              xvw_set_attributes(xvw_object((Widget) xobj),
                                 XVW_GRAPHICS_WCMIN_Z, wcmin.z,
                                 XVW_GRAPHICS_WCMAX_Z, wcmax.z,
                                 NULL);
           }
           else
           {
              xvw_set_attributes(xvw_object((Widget) xobj),
                                 XVW_GRAPHICS_WCMAX_Z, wcmax.z,
                                 XVW_GRAPHICS_WCMIN_Z, wcmin.z,
                                 NULL);
           }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: Plot3DCheckOffset
|
|       Purpose: Error checks a change to the width offset, the height 
|		 offset, the depth offset, the time offset, or the 
|		 elements offset.  
|
|         Input: current - Widget with current settings
|		 new     - Widget with new settings
|
|    Written By: Danielle Argiro
|          Date: Feb 23, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int Plot3DCheckOffset(
   Widget current,
   Widget new)
{
	XvwPlot3DGadget cobj = (XvwPlot3DGadget) current;
        XvwPlot3DGadget nobj = (XvwPlot3DGadget) new;

        /*
         *  Offset into data only applies if we have
         *  a data object, not if we have a coord array.
         */
        if ((nobj->plot3d.pobject == NULL) &&
            (nobj->plot3d.points != NULL))
        {
            kerror(XVISUAL, "SetValues", "X orientation / offset settings do not apply unless input is specified with a data object");
            nobj->plot3d.color_origination = KPLOT3D_COLOR_FROM_FOREGROUND;
        }

	/*
	 *  Check X offsets wrt X Orientation
	 */
        if ((nobj->plot3d.x_orientation == KWIDTH) ||
            (nobj->plot3d.y_orientation == KWIDTH))
        {
            if (cobj->plot3d.width_offset  != nobj->plot3d.width_offset) 
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Width offset must stay 0 when X or Y Orientation is set to 'Width'");
                nobj->plot3d.width_offset = 0;
                return(FALSE);
            }
            nobj->plot3d.width_offset = 0;
        }
        if ((nobj->plot3d.x_orientation == KHEIGHT) ||
	    (nobj->plot3d.y_orientation == KHEIGHT))
        {
            if (cobj->plot3d.height_offset  != nobj->plot3d.height_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                     "Height offset must stay 0 when X or Y Orientation is set to 'Height'");
                nobj->plot3d.height_offset = 0;
                return(FALSE);
            }
            nobj->plot3d.height_offset = 0;
        }
        if ((nobj->plot3d.x_orientation == KDEPTH) ||
	    (nobj->plot3d.y_orientation == KDEPTH))
        {
            if (cobj->plot3d.depth_offset  != nobj->plot3d.depth_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Depth offset must stay 0 when X or Y Orientation is set to 'Depth'");
                nobj->plot3d.depth_offset = 0;
                return(FALSE);
            }
            nobj->plot3d.depth_offset = 0;
        }
        if ((nobj->plot3d.x_orientation == KTIME) ||
	    (nobj->plot3d.y_orientation == KTIME))
        {
            if (cobj->plot3d.time_offset  != nobj->plot3d.time_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Time offset must stay 0 when X or Y Orientation is set to 'Time'");
                nobj->plot3d.time_offset = 0;
                return(FALSE);
            }
            nobj->plot3d.time_offset = 0;
        }
        if ((nobj->plot3d.x_orientation == KELEMENTS) ||
	    (nobj->plot3d.y_orientation == KELEMENTS))
        {
            if (cobj->plot3d.elements_offset  != nobj->plot3d.elements_offset)
            {
                kerror(XVISUAL, "ChangeOffset",
                      "Elements offset must stay 0 when X or Y Orientation is set to 'Elements'");
                nobj->plot3d.elements_offset = 0;
                return(FALSE);
            }
            nobj->plot3d.elements_offset = 0;
        }
	return(TRUE);
}

