/*
  * Khoros: $Id: PrintMapVal.c,v 1.24 1996/06/25 16:33:39 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: PrintMapVal.c,v 1.24 1996/06/25 16:33:39 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		PrintMapVal Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 InitLabels()
   >>>>                 CreateLabelGrid()
   >>>>                 ReInitLabelGrid()
   >>>>                 SetLabelFormat()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 PrintMapValGetName()
   >>>>                 PrintMapValSetName()
   >>>>                 ReloadCallback()
   >>>>                 PositionCallback()
   >>>>                 ColorReloadCallback()
   >>>>                 RedisplayMapVals()
   >>>>                 RedisplayActualValues()
   >>>>                 RedisplayDisplayedValues()
   >>>>                 RedisplayDisplayedValuesRGB()
   >>>>   Public:
   >>>>                 xvw_create_printmapval()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/PrintMapValP.h>


static void ClassInitialize PROTO((void));
static void Destroy	    PROTO((Widget));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static void InitLabels        PROTO((Widget, int, int));
static xvobject *CreateLabelGrid   PROTO((xvobject, int, int, int));
static void ReInitLabelGrid   PROTO((Widget, int));

static void Redisplay	    PROTO((Widget, XEvent *, Region));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,
				   Cardinal *));
static void SetLabelFormat  PROTO((Widget));


static int  PrintMapValGetName    PROTO((xvobject, char *, kaddr));
static int  PrintMapValSetName    PROTO((xvobject, char *, kaddr));
static void ReloadCallback   PROTO((kobject, char *, Widget, kdms_callback *));
static void PositionCallback PROTO((kobject, char *, Widget, kdms_callback *));
static void ColorReloadCallback   PROTO((xvobject, kaddr));
static void RedisplayMapVals      PROTO((Widget));
static void RedisplayActualValues PROTO((Widget, double *, unsigned char *));
static void RedisplayDisplayedValues  PROTO((Widget, double *, unsigned char *));
static void RedisplayDisplayedValuesRGB PROTO((Widget, unsigned char *));

#undef  kwidget
#define kwidget(widget)      (XvwPrintMapValWidget) (widget)

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PRINTMAPVAL_WIDTH,     NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_HEIGHT,    NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_EDITABLE,  NULL,    XtRInt,        XtRBoolean},
{XVW_PRINTMAPVAL_OBJECT,    NULL,    XtRPointer,    NULL},
{XVW_PRINTMAPVAL_SHOWCOLOR, NULL,    XtRInt,        XtRBoolean},
{XVW_PRINTMAPVAL_UPDATEMODE,NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_XPOSITION, NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_YPOSITION, NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_POLICY,    NULL,    XtRInt,        NULL},
{XVW_PRINTMAPVAL_CLIPOBJ,   NULL,    XtRPointer,    NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPrintMapValWidgetRec, printmapval.field)

static XtResource resources[] = { 

{XVW_PRINTMAPVAL_WIDTH, NULL, XtRInt, sizeof(int),
      offset(width), XtRImmediate, (XtPointer) 7},
{XVW_PRINTMAPVAL_HEIGHT, NULL, XtRInt, sizeof(int),
      offset(height), XtRImmediate, (XtPointer) 9},
{XVW_PRINTMAPVAL_EDITABLE, NULL, XtRBoolean, sizeof(Boolean),
      offset(editable), XtRImmediate, (XtPointer) FALSE},
{XVW_PRINTMAPVAL_SHOWCOLOR, NULL, XtRBoolean, sizeof(Boolean),
      offset(showcolor), XtRImmediate, (XtPointer) FALSE},
{XVW_PRINTMAPVAL_OBJECT, NULL, XtRPointer, sizeof(kobject),
      offset(object), XtRImmediate, (XtPointer) NULL},
{XVW_PRINTMAPVAL_UPDATEMODE, NULL, XtRInt, sizeof(int),
      offset(update_mode), XtRImmediate,(XtPointer) KPRINTMAPVAL_UM_CONTINUOUS},
{XVW_PRINTMAPVAL_XPOSITION, NULL, XtRInt, sizeof(int),
      offset(xposition), XtRImmediate, (XtPointer) 0},
{XVW_PRINTMAPVAL_YPOSITION, NULL, XtRInt, sizeof(int),
      offset(yposition), XtRImmediate, (XtPointer) 0},
{XVW_PRINTMAPVAL_POLICY, NULL, XtRInt, sizeof(int),
      offset(policy), XtRImmediate,(XtPointer) KPRINTMAPVAL_DISPLAYEDVALUES},
{XVW_PRINTMAPVAL_CLIPOBJ, NULL, XtRPointer, sizeof(kobject),
      offset(clipmask), XtRImmediate, (XtPointer) NULL},

{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(black), XtRString, (XtPointer) "#000000"},
{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(white), XtRString, (XtPointer) "#ffffff"},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwPrintMapValWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
};
#undef offset


#define SUPERCLASS (&xvwColorWidgetClassRec)

XvwPrintMapValWidgetClassRec xvwPrintMapValWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "PrintMapVal",			/* class_name		  */
    sizeof(XvwPrintMapValWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    XtInheritResize,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwPrintMapValWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPrintMapValWidgetClass for public consumption
 */
WidgetClass xvwPrintMapValWidgetClass = (WidgetClass) &xvwPrintMapValWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of printmapval widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young 
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPrintMapValWidgetClass, attributes,
	 	      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/PrintMapVal.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/PrintMapVal");

	xvw_define_attributes(xvwPrintMapValWidgetClass,
	           XVW_PRINTMAPVAL_FILENAME, XtRString, 
			      	PrintMapValSetName, PrintMapValGetName,
	           XVW_PRINTMAPVAL_CLIPFILE, XtRString, 
				PrintMapValSetName, PrintMapValGetName,
	  	   NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial image
|                for a printmapval widget instance.
|
|         Input: request - not used
|                new     - printmapval widget instance after initialization
|    Written By: Mark Young
|          Date: Mon Jun 29 10:35:11 MDT 1992
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwPrintMapValWidget xwid = kwidget(new);

	XColor color;

        /* not assuming RGB by default */
        xwid->printmapval.rgb = FALSE;

	/* allocate black specially */
	color.flags = DoRed | DoGreen | DoBlue;
	color.red = color.green = color.blue = 0;
	if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
	   xwid->printmapval.black = color.pixel;

	/* allocate white specially */
	color.red = color.green = color.blue = 65535;
	if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
	   xwid->printmapval.white = color.pixel;

	/* general initializations */
        xwid->printmapval.title        = NULL;
        xwid->printmapval.r_title      = NULL;
        xwid->printmapval.r_parent     = NULL;
        xwid->printmapval.r_labels     = NULL;
        xwid->printmapval.r_none_label = NULL;
        xwid->printmapval.g_title      = NULL;
        xwid->printmapval.g_parent     = NULL;
        xwid->printmapval.g_labels     = NULL;
        xwid->printmapval.g_none_label = NULL;
        xwid->printmapval.b_title      = NULL;
        xwid->printmapval.b_parent     = NULL;
        xwid->printmapval.b_labels     = NULL;
        xwid->printmapval.b_none_label = NULL;
        xwid->printmapval.mapdata      = NULL;
        xwid->printmapval.labelsize    = 4;

        /* create printmapval display */
        InitLabels(new, xwid->printmapval.width, xwid->printmapval.height);

}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Frees memory before the printmapval object is destroyed.
|
|         Input: widget - the image widget being destroyed
|
|    Written By: Danielle Argiro
|          Date: Aug 11, 1995 
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPrintMapValWidget xwid = kwidget(widget);

        kfree_and_NULL(xwid->printmapval.mapdata);
	kfree_and_NULL(xwid->printmapval.r_labels);
	kfree_and_NULL(xwid->printmapval.g_labels);
	kfree_and_NULL(xwid->printmapval.b_labels);
}

/*-----------------------------------------------------------
|
|  Routine Name: InitLabels
|
|       Purpose: This utility routine creates and initializes
|                the PrintMapVal display.  If non-RGB, creates
|                one grid of labels to display pixel values.
|                If RGB, creates three grids of labels
|                to display red, green, and blue values.
|
|         Input: widget - The PrintMapVal widget
|                parent - parent of
|    Written By: Danielle Argiro
|          Date: May 23, 1995
|
------------------------------------------------------------*/
static void InitLabels(
    Widget    widget,
    int       old_width,
    int       old_height)
{
        XvwPrintMapValWidget xwid = kwidget(widget);
        int width  = xwid->printmapval.width;
        int height = xwid->printmapval.height;
        xvobject back = xvw_object(widget);
	char title[KLENGTH];
        int  i;

	if (xwid->printmapval.policy == KPRINTMAPVAL_MAPDATAVALUES)
            ksprintf(title, "Actual Map Values Defining R,G,B:");
        else ksprintf(title, "RGB Components of Displayed Colors:");

	/* create overall title */
	if (xwid->printmapval.title == NULL)
	{
	    xwid->printmapval.title = xvw_create_label(back, "title");
	    xvw_set_attributes(xwid->printmapval.title,
			       XVW_BELOW, NULL,
			       XVW_LABEL, title,
			       NULL);
	}

	/* 
	 * RED 
	 */
	if (xwid->printmapval.r_title == NULL)
	{
	    xwid->printmapval.r_title = xvw_create_label(back, "rtitle");
	    xvw_set_attributes(xwid->printmapval.r_title,
			       XVW_LABEL,   "Red",
			       XVW_BELOW,    xwid->printmapval.title,
                               XVW_RIGHT_OF, xwid->printmapval.title,
			       XVW_LEFT_OF,  xwid->printmapval.title,
			       NULL);
	}
        if (xwid->printmapval.r_parent == NULL)
	{
	    xwid->printmapval.r_parent = xvw_create_manager(back,"rparent");
	    xvw_set_attributes(xwid->printmapval.r_parent,
			       XVW_BELOW,    xwid->printmapval.r_title,
                               XVW_RIGHT_OF, NULL,
			       NULL);
	}
	if (xwid->printmapval.r_none_label == NULL)
	{
	    xvobject label;
	    xwid->printmapval.r_none_label = 
		xvw_create_manager(xwid->printmapval.r_parent, "r_none");
	    xvw_set_attributes(xwid->printmapval.r_none_label, 
			       XVW_ABOVE,        NULL,
			       XVW_BELOW,        NULL,
			       XVW_RIGHT_OF,     NULL,
			       XVW_LEFT_OF,      NULL,
			       XVW_BORDER_WIDTH, 2,
                               XVW_CHAR_HEIGHT,  1.25,
                               XVW_CHAR_WIDTH,   20.0,
                               XVW_MAPPED,	 FALSE,
			       NULL);
	    label = xvw_create_label(xwid->printmapval.r_none_label, "r_nonel");
	    xvw_set_attributes(label, 
			       XVW_LABEL,            "No Map Data Available",
			       XVW_FOREGROUND_COLOR, "black",
			       NULL);
			       
	}
        if ((xwid->printmapval.r_labels == NULL) ||
            (old_width != width || old_height != height))
        {
	    if (xwid->printmapval.r_labels != NULL)
	    {
                for (i = 0; i < old_width * old_height; i++)
                    xvw_destroy(xwid->printmapval.r_labels[i]);
                kfree_and_NULL(xwid->printmapval.r_labels);
	    }
            xwid->printmapval.r_labels =
		CreateLabelGrid(xwid->printmapval.r_parent,
                            width, height, xwid->printmapval.labelsize);

        }

        /* 
	 * GREEN 
	 */
	if (xwid->printmapval.g_title == NULL)
	{
            xwid->printmapval.g_title = xvw_create_label(back, "gtitle");
            xvw_set_attributes(xwid->printmapval.g_title,
                               XVW_LABEL,   "Green",
                               XVW_BELOW,    xwid->printmapval.r_parent,
                               XVW_RIGHT_OF, xwid->printmapval.title,
			       XVW_LEFT_OF,  xwid->printmapval.title,
                               NULL);
        }
        if (xwid->printmapval.g_parent == NULL)
        {
	    xwid->printmapval.g_parent = xvw_create_manager(back,"gparent");
            xvw_set_attributes(xwid->printmapval.g_parent,
                               XVW_BELOW, xwid->printmapval.g_title,
                               XVW_RIGHT_OF, NULL,
			       NULL);
	}
        if (xwid->printmapval.g_none_label == NULL)
        {
	    xvobject label;
            xwid->printmapval.g_none_label =
                xvw_create_manager(xwid->printmapval.g_parent, "g_none");
            xvw_set_attributes(xwid->printmapval.g_none_label,
                               XVW_ABOVE,        NULL,
                               XVW_BELOW,        NULL,
                               XVW_RIGHT_OF,     NULL,
                               XVW_LEFT_OF,      NULL,
                               XVW_BORDER_WIDTH, 2,
                               XVW_CHAR_HEIGHT,  1.25,
                               XVW_CHAR_WIDTH,   20.0,
                               XVW_MAPPED,	 FALSE,
                               NULL);
            label = xvw_create_label(xwid->printmapval.g_none_label, "g_nonel");
            xvw_set_attributes(label, 
			       XVW_LABEL,            "No Map Data Available",
			       XVW_FOREGROUND_COLOR, "black",
			       NULL);

        }
        if ((xwid->printmapval.g_labels == NULL) ||
	    (old_width != width || old_height != height))
        {
            if (xwid->printmapval.g_labels != NULL)
            {
	        for (i = 0; i < old_width * old_height; i++)
                    xvw_destroy(xwid->printmapval.g_labels[i]);
                kfree_and_NULL(xwid->printmapval.g_labels);
	    }
            xwid->printmapval.g_labels = 
			CreateLabelGrid(xwid->printmapval.g_parent, 
			    width, height, xwid->printmapval.labelsize);

        }


        /* 
	 * BLUE 	
	 */
	if (xwid->printmapval.b_title == NULL)
        {
            xwid->printmapval.b_title = xvw_create_label(back, "btitle");
            xvw_set_attributes(xwid->printmapval.b_title,
                               XVW_LABEL,   "Blue",
                               XVW_BELOW,    xwid->printmapval.g_parent,
                               XVW_RIGHT_OF, xwid->printmapval.title,
			       XVW_LEFT_OF,  xwid->printmapval.title,
                               NULL);
        }
        if (xwid->printmapval.b_parent == NULL)
        {
	    xwid->printmapval.b_parent = xvw_create_manager(back,"bparent");
            xvw_set_attributes(xwid->printmapval.b_parent,
                               XVW_BELOW,    xwid->printmapval.b_title,
                               XVW_RIGHT_OF, NULL,
			       NULL);
    
        }
        if (xwid->printmapval.b_none_label == NULL)
        {
	    xvobject label;

            xwid->printmapval.b_none_label =
                xvw_create_manager(xwid->printmapval.b_parent, "b_none");
            xvw_set_attributes(xwid->printmapval.b_none_label,
                               XVW_ABOVE,        NULL,
                               XVW_BELOW,        NULL,
                               XVW_RIGHT_OF,     NULL,
                               XVW_LEFT_OF,      NULL,
                               XVW_BORDER_WIDTH, 2,
                               XVW_CHAR_HEIGHT,  1.25,
                               XVW_CHAR_WIDTH,   20.0,
                               XVW_MAPPED,	 FALSE,
                               NULL);
            label = xvw_create_label(xwid->printmapval.b_none_label, "b_nonel");
            xvw_set_attributes(label, 
			       XVW_LABEL,            "No Map Data Available",
			       XVW_FOREGROUND_COLOR, "black",
			       NULL);

        }
        if ((xwid->printmapval.b_labels == NULL) ||
            (old_width != width || old_height != height))
        {
	    if (xwid->printmapval.b_labels != NULL)
	    {
                for (i = 0; i < old_width * old_height; i++)
                    xvw_destroy(xwid->printmapval.b_labels[i]);
                kfree_and_NULL(xwid->printmapval.b_labels);
	    }
            xwid->printmapval.b_labels =
			CreateLabelGrid(xwid->printmapval.b_parent,
                            width, height, xwid->printmapval.labelsize);

        }



}

/*-----------------------------------------------------------
|
|  Routine Name: CreateLabelGrid
|
|       Purpose: This utility routine creates a single grid of labels
|                to display values.  Grid may be used for display of pixel
|                values (non-RGB images) or for RGB values (RGB images).
|
|         Input: parent  - parent on which to create label objects
|		 offset  - label object, under which to create grid
|                width   - width of printmapval display
|                height  - width of printmapval display
|		 labelsize - number of characters to make min width
|    Written By: Danielle Argiro
|          Date: May 23, 1995
|
------------------------------------------------------------*/

static xvobject *CreateLabelGrid(
    xvobject parent,
    int      width,
    int      height,
    int      labelsize)
{
        int       i, j;
        xvobject  vert_offset = NULL; 
	xvobject  horiz_offset = NULL;
	xvobject  *labels = NULL;

	labels = (xvobject *) kcalloc((size_t) width*height, 
					sizeof(xvobject));
        /*
         * create display using array of label objects
         */
        for (i = 0; i < height; i++)
        {
	    for (j = 0; j < width; j++)
            {
                labels[i*width+j]= xvw_create_label(parent, "label");
                xvw_set_attributes(labels[i*width+j],
                            XVW_LABEL,                "0",
                            XVW_FORCE_REDISPLAY,      TRUE, 
                            XVW_LABEL_JUSTIFY,        KLABEL_JUSTIFY_CENTER,
                            XVW_LABEL_EMPHASIZE,      TRUE,
                            XVW_CHAR_MIN_WIDTH,       (float) labelsize,
                            XVW_RIGHT_OF,             horiz_offset,
                            XVW_BELOW,                vert_offset,
                            XVW_FOREGROUND_COLOR,     "white",
                            XVW_LABEL_EMPHASIZECOLOR, "black",
                            NULL);
                if ((i == (int) height/2) && (j == (int) width/2))
                     xvw_set_attribute(labels[i*width+j], XVW_BORDER_WIDTH, 1);
                horiz_offset = labels[i*width+j];
            }
            vert_offset  = labels[i*width];
            horiz_offset = NULL;
        }
	return(labels);

}

/*-----------------------------------------------------------
|
|  Routine Name: ReInitLabelGrid
|
|       Purpose: Since the label grid is not re-created on a new
|		 incoming image, this utility routine updates the 
|		 size of the labels according to the new value
|		 of labelsize, and re-initializes values to 0.
|
|         Input: widget     - printmapval widget
|                size_only  - TRUE to reset labelsize only
|    Written By: Danielle Argiro
|          Date: May 25, 1995
|
------------------------------------------------------------*/

static void ReInitLabelGrid(
    Widget   widget,
    int      size_only)
{
        int       i, j;
	XvwPrintMapValWidget xwid = kwidget(widget);
	int width        = xwid->printmapval.width;
	int height       = xwid->printmapval.height;
	int labelsize    = xwid->printmapval.labelsize;

        /*
         * re-initialize display of label objects
         */
        for (i = 0; i < height; i++)
        {
            for (j = 0; j < width; j++)
            {
		/* re-initialize label, size, and background */
		if (!size_only)
		{
                    xvw_set_attributes(xwid->printmapval.r_labels[i*width+j],
                                   XVW_LABEL,          "0",
                                   XVW_CHAR_MIN_WIDTH, (float) labelsize,
                                   XVW_BACKGROUND,     xwid->printmapval.black,
                                   NULL);
                    xvw_set_attributes(xwid->printmapval.g_labels[i*width+j],
                                   XVW_LABEL,          "0",
                                   XVW_CHAR_MIN_WIDTH, (float) labelsize,
                                   XVW_BACKGROUND,     xwid->printmapval.black,
                                   NULL);
                    xvw_set_attributes(xwid->printmapval.b_labels[i*width+j],
                                   XVW_LABEL,          "0",
                                   XVW_CHAR_MIN_WIDTH, (float) labelsize,
                                   XVW_BACKGROUND,     xwid->printmapval.black,
                                   NULL);
		}
		/* re-initialize size only */
		else
		{
                    xvw_set_attribute(xwid->printmapval.r_labels[i*width+j],
                                      XVW_CHAR_MIN_WIDTH, (float) labelsize);
                    xvw_set_attribute(xwid->printmapval.g_labels[i*width+j],
                                      XVW_CHAR_MIN_WIDTH, (float) labelsize);
                    xvw_set_attribute(xwid->printmapval.b_labels[i*width+j],
                                      XVW_CHAR_MIN_WIDTH, (float) labelsize);
		}
            }
        }
}


/*-----------------------------------------------------------
|
|  Routine Name: SetLabelFormat
|
|       Purpose: This utility routine looks at the data object
|                and decides on a good size for the labels.
|
|         Input: widget  - the PrintMapVal widget
|    Written By: Danielle Argiro
|          Date: May 23, 1995
|
------------------------------------------------------------*/

static void SetLabelFormat(
     Widget widget)
{
        XvwPrintMapValWidget xwid = kwidget(widget);
	int    labelsize;
	int    num;
        double magval, logval, minval, maxval;

	/* displaying normalized values that define displayed color => BYTE */
	if (xwid->printmapval.policy == KPRINTMAPVAL_DISPLAYEDVALUES ||
	    !xwid->color.info->initialized)
	{
	   xwid->printmapval.labelsize = 4;
	   return;
	}

	/* displaying values actually in the map segment: size need varies */
        switch (xwid->printmapval.map_datatype) {
            case KBIT:
                 xwid->printmapval.labelsize = 2;
                 break;

            case KUBYTE:
                 xwid->printmapval.labelsize = 4;
                 break;

            case KBYTE:
                 if (xwid->color.info->minval < 0)
                     xwid->printmapval.labelsize = 5;
                 else xwid->printmapval.labelsize = 4;

            case KSHORT:
            case KUSHORT:
            case KINT:
            case KUINT:
            case KLONG:
            case KULONG:
                 if (xwid->color.info->histogram != NULL)
                 {
                     magval = kmax(kabs(xwid->color.info->minval),
                                kabs(xwid->color.info->maxval));
                     logval = klog10(magval);
                     xwid->printmapval.labelsize = (int) (logval + 1.0);
		     if (xwid->color.info->minval < 0)
                         xwid->printmapval.labelsize++;
                 }
                 else xwid->printmapval.labelsize = 7;
                 break;

            case KFLOAT:
            case KDOUBLE:
		 num = xwid->printmapval.mapwidth*xwid->printmapval.mapheight;
		 kdata_minmax(xwid->printmapval.mapdata, num,
			      KDOUBLE, &minval, &maxval);
		 magval = kmax(kabs(minval), kabs(maxval));
                 logval = klog10(magval);
                 labelsize = (int) (logval + 1.0);
		 labelsize = kmax(labelsize, 5);
		 if (minval < 0) labelsize++;
	         xwid->printmapval.labelsize = labelsize;
                 break;

            case KCOMPLEX:
            case KDCOMPLEX:
                 xwid->printmapval.labelsize = 20;
                 break;
        }

}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine calls the RedisplayMapVal procedure when an
|                expose event occurs.
|
|         Input: widget - widget data structure for palette
|		 event	- the event that caused the redraw
|		 region	- the region that was exposed
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	RedisplayMapVals(widget);
	(*xvwPrintMapValWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: Determine what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new	 - the widget processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/

/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	int e;
	XvwPrintMapValWidget cxwid = kwidget(current);
	XvwPrintMapValWidget nxwid = kwidget(new);

	Boolean redisplay = FALSE;

        /*
         *  Change of width and/or height: note that we have
         *  to check this FIRST so InitLabels will be called before
         *  ReInitLabelGrid (if the object is also being set)
         */
	if (cxwid->printmapval.width  != nxwid->printmapval.width  ||
	    cxwid->printmapval.height != nxwid->printmapval.height)
	{
           kpds_set_attribute(nxwid->printmapval.object, KPDS_MAP_REGION_SIZE, 
		  nxwid->printmapval.width, nxwid->printmapval.height, 1, 1, 1);
	   if (nxwid->printmapval.clipmask != NULL)
               kpds_set_attribute(nxwid->printmapval.clipmask,
                        KPDS_VALUE_REGION_SIZE,  nxwid->printmapval.width, 
				         nxwid->printmapval.height, 1, 1, 1);

           /* re-create printmapval display */
           InitLabels(new, cxwid->printmapval.width, cxwid->printmapval.height);
	   redisplay = TRUE;
	}

	/*
	 *  Setting New Data Object 
	 */
	if (cxwid->printmapval.object != nxwid->printmapval.object)
	{
	   /* see what type of map data we have */
	   kpds_get_attribute(nxwid->printmapval.object, KPDS_MAP_DATA_TYPE, 
			      &nxwid->printmapval.map_datatype);

	   /* not supporting complex maps */
	   if (nxwid->printmapval.map_datatype == KCOMPLEX || 
	       nxwid->printmapval.map_datatype == KDCOMPLEX)
	   {
		kerror("xvisual", "PrintMapVal SetValues",
		       "Printmapval object cannot handle data objects with complex or double complex maps");
	        nxwid->printmapval.object = cxwid->printmapval.object;
		return(FALSE);
	   }
	
	   /* 
	    * create reference object, add value callbacks, set color object 
	    */
	   nxwid->printmapval.object =
		kpds_reference_object(nxwid->printmapval.object);
	   kpds_close_object(cxwid->printmapval.object);
	   kpds_add_value_callback(nxwid->printmapval.object,
		KPDS_CALLBACK_CHANGE, ReloadCallback, (kaddr) new);
	   kpds_add_value_callback(nxwid->printmapval.object,
                KPDS_CALLBACK_ACCESS, PositionCallback, (kaddr) new);

	   xvw_set_attribute(xvw_object(new), XVW_COLOR_COLOROBJ,
                        nxwid->printmapval.object);

           /*
	    * get attributes for book-keeping
            */
           kpds_get_attributes(nxwid->printmapval.object, 
			KPDS_VALUE_SIZE,      NULL, NULL, NULL, NULL, &e,
			KPDS_VALUE_DATA_TYPE, &nxwid->printmapval.val_datatype,
                        KPDS_MAP_SIZE,        &nxwid->printmapval.mapwidth, 
		 		              &nxwid->printmapval.mapheight, 
				              NULL, NULL, NULL,
                        KPDS_MAP_DATA_TYPE,   &nxwid->printmapval.map_datatype,
			NULL);
           nxwid->printmapval.rgb = xvw_is_rgb(nxwid->printmapval.object);

	   /*
	    * set attribute for later processing
	    */
           kpds_set_attributes(nxwid->printmapval.object, 
			KPDS_VALUE_DATA_TYPE, KDOUBLE,
                        KPDS_MAP_POSITION,    0, 0, 0, 0, 0,
                        KPDS_MAP_DATA_TYPE,   KDOUBLE,
                        KPDS_MAP_REGION_SIZE, nxwid->printmapval.mapwidth,
			                      nxwid->printmapval.mapheight, 
                                              1, 1, 1,
                        NULL);

           /* read in the whole map now */
	   kfree_and_NULL(nxwid->printmapval.mapdata);
           nxwid->printmapval.mapdata = kpds_get_data(nxwid->printmapval.object,
                                                      KPDS_MAP_REGION, NULL);
           SetLabelFormat(new);
           ReInitLabelGrid(new, FALSE);
	   redisplay = TRUE;
	}

        /*
         *  Change of clip mask
         */
        if (cxwid->printmapval.clipmask != nxwid->printmapval.clipmask)
        {
            nxwid->printmapval.clipmask =
                 kpds_reference_object(nxwid->printmapval.clipmask);
            kpds_close_object(cxwid->printmapval.clipmask);
            kpds_add_value_callback(nxwid->printmapval.clipmask,
                KPDS_CALLBACK_CHANGE, ReloadCallback, (kaddr) new);

            kpds_set_attributes(nxwid->printmapval.clipmask,
                    KPDS_VALUE_REGION_SIZE,  nxwid->printmapval.width, 
					     nxwid->printmapval.height, 1, 1, 1,
                    KPDS_VALUE_DATA_TYPE,    KUBYTE,
                    NULL);
            redisplay = TRUE;
        }


	/*
	 *  Setting New Policy
	 */
	if (cxwid->printmapval.policy != nxwid->printmapval.policy)
        {
	    char title[KLENGTH];
	    if (nxwid->printmapval.policy == KPRINTMAPVAL_MAPDATAVALUES)
                ksprintf(title, "Actual Map Values Defining R,G,B:");
            else ksprintf(title, "RGB Components of Displayed Colors:");

	    if (nxwid->printmapval.title != NULL)
		xvw_set_attribute(nxwid->printmapval.title, XVW_LABEL, title);
            SetLabelFormat(new);
            ReInitLabelGrid(new, FALSE);
	    redisplay = TRUE;
	}

	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: PrintMapValGetName
|
|       Purpose: PrintMapValGetName is used to get the filename associated
|		 with the attribute.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|        Output: calldata  - pointer to retrieve the object name
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PrintMapValGetName(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwPrintMapValWidget xwid = (XvwPrintMapValWidget) xvw_widget(object);
	String *filename = (String *) calldata;

	if (kstrcmp(attribute, XVW_PRINTMAPVAL_FILENAME) == 0)
	{
	    *filename = NULL;
            return(kpds_get_attribute(xwid->printmapval.object, KPDS_NAME,
				      filename));
	}
        else if (kstrcmp(attribute, XVW_PRINTMAPVAL_CLIPFILE) == 0)
        {
            *filename = NULL;
             return(kpds_get_attribute(xwid->printmapval.clipmask,
                                      KPDS_NAME, filename));
        }
        else
	{
	   kerror(XVISUAL, "PrintMapValGetName", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: PrintMapValSetName
|
|       Purpose: PrintMapValSetName is used to set the filename associated
|		 with the attribute.
|
|         Input: object    - the object in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PrintMapValSetName(
   xvobject printmapval,
   char     *attribute,
   kaddr    calldata)
{
	int	locate = TRUE;
	kobject object = NULL;
	String  *filename = (String *) calldata;

	if (*filename != NULL && !(object = kdms_locate(*filename)))
	{
	   locate = FALSE;
	   if (!(object = kpds_open_object(*filename, KOBJ_READ)))
	   {
	      kerror(XVISUAL, "PrintMapValSetName", "Failed to load image '%s'",
			*filename);
	      return(TRUE);
	   }
	}

	if (kstrcmp(attribute, XVW_PRINTMAPVAL_FILENAME) == 0)
	   xvw_set_attribute(printmapval, XVW_PRINTMAPVAL_OBJECT, object);
        else if (kstrcmp(attribute, XVW_PRINTMAPVAL_CLIPFILE) == 0)
            xvw_set_attribute(printmapval, XVW_PRINTMAPVAL_CLIPOBJ, object);
	else
	{
	   kerror(XVISUAL, "PrintMapValSetName", "invalid attribute %s",
			attribute);
	   if (!locate) kpds_close_object(object);
	   return(FALSE);
	}
	if (!locate) kpds_close_object(object);
	
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReloadCallback
|
|       Purpose: Reload the mapvals since the data has changed
|
|         Input: xwid - the printmapval widget to load the value data
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReloadCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	/* perform check to see if we are in the area that's changed */
	RedisplayMapVals(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: PositionCallback
|
|       Purpose: This re-prints the mapvals of the object being tracked
|
|         Input: xwid - the printmapval widget to load the value data
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PositionCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	int      w, h, update;
	XvwPrintMapValWidget xwid = (XvwPrintMapValWidget) widget;


	/*
	 *  Make sure we aren't here because of ourself
	 */
	if (object == cb->object)
	   return;

	xvw_get_type(object, NULL, &update);
	if ((xwid->printmapval.update_mode == KPRINTMAPVAL_UM_CONTINUOUS &&
	                            update != MOTION_EVENT_UPDATE) ||
            (xwid->printmapval.update_mode == KPRINTMAPVAL_UM_BUTTON_PRESS &&
	                            update != BUTTONPRESS_EVENT_UPDATE))
	{
	   return;
	}

	w = (cb->end.w - cb->begin.w)/2 + cb->begin.w;
	h = (cb->end.h - cb->begin.h)/2 + cb->begin.h;
	xwid->printmapval.xposition = w - xwid->printmapval.width/2;
	xwid->printmapval.yposition = h - xwid->printmapval.height/2;
	RedisplayMapVals(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Update the printmapval since our pixels have changed
|
|         Input: object - the object to be refreshed (the PrintMapVal)
|		 call_data   - (not used)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 11, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	Widget widget = xvw_widget(object);
        XvwPrintMapValWidget xwid = kwidget(widget);


	/* save width, height, and data type of maps */
	kpds_get_attributes(xwid->printmapval.object,
		KPDS_MAP_SIZE,      &xwid->printmapval.mapwidth,
				    &xwid->printmapval.mapheight,
				    NULL, NULL, NULL,
		KPDS_MAP_DATA_TYPE, &xwid->printmapval.map_datatype,
		NULL);

	/* set position, region size & data type in prep. for reading map */
	kpds_set_attributes(xwid->printmapval.object,
		KPDS_MAP_POSITION,    0, 0, 0, 0, 0,
		KPDS_MAP_DATA_TYPE,   KDOUBLE,
		KPDS_MAP_REGION_SIZE, xwid->printmapval.mapwidth,
				      xwid->printmapval.mapheight, 1, 1, 1,
		NULL);

	/* read in the whole map now */
	kfree_and_NULL(xwid->printmapval.mapdata);
	xwid->printmapval.mapdata = kpds_get_data(xwid->printmapval.object,
						      KPDS_MAP_REGION, NULL);

	RedisplayMapVals(xvw_widget(object));
}
 
/*-----------------------------------------------------------
|
|  Routine Name: RedisplayMapVals - create and relayout the mapvals
|
|       Purpose: create and relayout the mapvals being displayed.
|
|         Input: widget - the printmapval widget to create the mapvals for
|    Written By: Mark Young & Danielle Argiro
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RedisplayMapVals(
   Widget widget)
{
	XvwPrintMapValWidget xwid = kwidget(widget);

	unsigned char *maskdata = NULL;
        unsigned char *maskptr;
        unsigned char *clipdata = NULL;
	int	      i, map_available; 
	double        *data; 

	kobject  data_object  = xwid->printmapval.object;
	int      xposition    = xwid->printmapval.xposition;
	int      yposition    = xwid->printmapval.yposition;
	int      width        = xwid->printmapval.width;
	int      height       = xwid->printmapval.height;
	int      policy       = xwid->printmapval.policy;

	/*
	 *  if it's not visible, no sense in redisplaying it.
	 */
	if (!xvw_check_visible(xvw_object(widget)) || data_object == NULL)
	   return;

	
	map_available = xwid->color.info->initialized;
	if (!map_available && policy == KPRINTMAPVAL_MAPDATAVALUES)
	{
	    xvw_map(xwid->printmapval.r_none_label);
	    xvw_map(xwid->printmapval.g_none_label);
	    xvw_map(xwid->printmapval.b_none_label);
	    xvw_raise(xwid->printmapval.r_none_label);
	    xvw_raise(xwid->printmapval.g_none_label);
	    xvw_raise(xwid->printmapval.b_none_label);
	    return;
	}
	else
	{
            xvw_unmap(xwid->printmapval.r_none_label);
            xvw_unmap(xwid->printmapval.g_none_label);
            xvw_unmap(xwid->printmapval.b_none_label);
	}

	/*
	 *  set the printmapval object to reflect the map values indexed
         *  by the pixel at the location of the pointer in the image
	 */
	kpds_set_attributes(data_object,
		KPDS_VALUE_POSITION,    xposition, yposition, 0, 0, 0,
		KPDS_VALUE_DATA_TYPE,   KDOUBLE,
                KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
                KPDS_VALUE_SCALING,     KNONE, 
		KPDS_VALUE_REGION_SIZE, width, height, 1, 1, 1,
		NULL);

	/*
	 *  get the value data at that location, for use in indexing into maps
	 */
	if (!(data = kpds_get_data(data_object, KPDS_VALUE_REGION, NULL)))
        {
           kerror(XVISUAL, "RedisplayMapVals", "Unable to get image mapvals");
           return;
        }

	/*
	 *  if there is a mask, will want to reflect which values are
	 *  valid in the printmapval display
	 */
        if (kpds_query_mask(data_object) == TRUE)
	{
	   kpds_set_attributes(data_object,
		KPDS_MASK_POSITION,    xposition, yposition, 0, 0, 0,
		KPDS_MASK_DATA_TYPE,   KUBYTE,
		KPDS_MASK_REGION_SIZE, width, height, 1, 1, 1,
		NULL);
	   maskdata = kpds_get_data(data_object, KPDS_MASK_REGION, NULL);
	}

        /*
         *  If we have a clip object, we better get the clip data and combine
         *  it with the mask data.
         */
        if (xwid->printmapval.clipmask)
        {
           kpds_set_attribute(xwid->printmapval.clipmask,
                   KPDS_VALUE_POSITION, xposition, yposition, 0, 0, 0);

           clipdata = kpds_get_data(xwid->printmapval.clipmask,
                                    KPDS_VALUE_REGION, NULL);
           if (maskdata != NULL)
           {
               maskptr = maskdata;
               for (i = 0; i < width*height; i++)
                  *maskptr++ = (*clipdata++ && *maskptr) ? *maskptr : 0;
           }
           else maskdata = clipdata;
        }

	/* print actual values stored in map data */
	if (policy == KPRINTMAPVAL_MAPDATAVALUES)
	    RedisplayActualValues(widget, data, maskdata);
	
	/* print values comprising RGB components */
	else
	{
	     /* of non-RGB images */
	     if (!xwid->printmapval.rgb)
		 RedisplayDisplayedValues(widget, data, maskdata);

	     /* of RGB images */
	     else RedisplayDisplayedValuesRGB(widget, maskdata);
	}
	kfree(data);
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayActualValues
|
|       Purpose: Updates the display of actual map values.
|
|         Input: widget   - the printmapval widget to update display for
|                data     - region of value data (for indexing into maps)
|                maskdata - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayActualValues(
    Widget   widget,
    double   *valuedata,
    unsigned char *maskdata)
{
        XvwPrintMapValWidget xwid = kwidget(widget);

        int     i, indx, reinit = FALSE; 
        char    temp_r[KLENGTH];
        char    temp_g[KLENGTH];
        char    temp_b[KLENGTH];
	double  factor = 1.0;
        Pixel   pixel;

        double minval = xwid->color.info->minval;
        double maxval = xwid->color.info->maxval;
        int indx_min  = xwid->color.info->indx_min;
        int indx_max  = xwid->color.info->indx_max;

	int      mapwidth     = xwid->printmapval.mapwidth;
        int      width        = xwid->printmapval.width;
        int      height       = xwid->printmapval.height;
	double  *mapdata      = xwid->printmapval.mapdata;

        factor = (indx_max - indx_min)/(maxval - minval);
        for (i = 0; i < width*height; i++)
        {
	    indx = (int) ((valuedata[i] - minval) * factor);

            /*
             * print value into label, unless value is masked out,
             * in which case we leave the value blank.
             */
            if ((maskdata != NULL) && (maskdata[i] == 0))
            {
                ksprintf(temp_r, "  ");
                ksprintf(temp_g, "  ");
                ksprintf(temp_b, "  ");
            }
            else
            {
                ksprintf(temp_r, "%g", mapdata[indx*mapwidth+0]);
                ksprintf(temp_g, "%g", mapdata[indx*mapwidth+1]);
                ksprintf(temp_b, "%g", mapdata[indx*mapwidth+2]);
            }

            /*
             * set background of label to color of pixel,
             * unless value is masked out in which case use bg color
             */
            if (xwid->printmapval.showcolor)
            {
                if ((maskdata != NULL) && (maskdata[i] == 0))
                    pixel = xwid->core.background_pixel;
                else pixel =  ColorAllocate(widget, (unsigned) indx);
            }
            else pixel = xwid->printmapval.black;

	    if (kstrlen(temp_r) > xwid->printmapval.labelsize)
	    {
		xwid->printmapval.labelsize = kstrlen(temp_r);
		reinit = TRUE;
	    }
            if (kstrlen(temp_g) > xwid->printmapval.labelsize)
	    {
		xwid->printmapval.labelsize = kstrlen(temp_g);
		reinit = TRUE;
	    }
            if (kstrlen(temp_b) > xwid->printmapval.labelsize)
	    {
		xwid->printmapval.labelsize = kstrlen(temp_b);
		reinit = TRUE;
	    }
	    if (reinit)
                ReInitLabelGrid(widget, TRUE);

            xvw_set_attributes(xwid->printmapval.r_labels[i],
                               XVW_LABEL,      temp_r,
                               XVW_BACKGROUND, pixel,
                               NULL);
            xvw_set_attributes(xwid->printmapval.g_labels[i],
                               XVW_LABEL,      temp_g,
                               XVW_BACKGROUND, pixel,
                               NULL);
            xvw_set_attributes(xwid->printmapval.b_labels[i],
                               XVW_LABEL,      temp_b,
                               XVW_BACKGROUND, pixel,
                               NULL);

        }
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayDisplayedValues
|
|       Purpose: Updates the display of displayed values for
|		 non-RGB images; supports data objects of all
|		 data types, including complex
|
|         Input: widget    - the printmapval widget to update display for
|                valuedata - region of value data (for indexing into maps)
|                maskdata  - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayDisplayedValues(
    Widget   widget,
    double   *valuedata,
    unsigned char *maskdata)
{
        XvwPrintMapValWidget xwid = kwidget(widget);
	XColor   *xcolors = ColorGetXColors(widget);

        int     i, indx;
        char    temp_r[KLENGTH];
        char    temp_g[KLENGTH];
        char    temp_b[KLENGTH];
	double  factor = 1.0;
        Pixel   pixel;

	double minval, maxval;
        int indx_min  = xwid->color.info->indx_min;
        int indx_max  = xwid->color.info->indx_max;

        int      width        = xwid->printmapval.width;
        int      height       = xwid->printmapval.height;

	/* NOTE: for unknown reasons, these settings prevent ABR's for
	   complex images */
        kpds_set_attributes(xwid->printmapval.object,
                   KPDS_VALUE_SCALING,         KNONE,
                   KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
                   NULL);
	kpds_get_attribute(xwid->printmapval.object, KPDS_VALUE_MINMAX, 
			   &minval, &maxval);

        factor = (indx_max - indx_min)/(maxval - minval);
        for (i = 0; i < width*height; i++)
        {
	    indx = (int) ((valuedata[i] - minval) * factor);

            /*
             * print value into label, unless value is masked out,
             * in which case we leave the value blank.
             */
            if ((maskdata != NULL) && (maskdata[i] == 0))
	    {
                ksprintf(temp_r, "  ");
                ksprintf(temp_g, "  ");
                ksprintf(temp_b, "  ");
	    }
            else 
	    {
		ksprintf(temp_r, "%3d", xcolors[indx].red   >> 8);
		ksprintf(temp_g, "%3d", xcolors[indx].green >> 8);
		ksprintf(temp_b, "%3d", xcolors[indx].blue  >> 8);
	    }

            /*
             * set background of label to color of pixel,
             * unless value is masked out in which case use bg color
             */
            if (xwid->printmapval.showcolor)
            {
                if ((maskdata != NULL) && (maskdata[i] == 0))
                    pixel = xwid->core.background_pixel;
	        else pixel =  ColorAllocate(widget, (unsigned) indx);
            }
            else pixel = xwid->printmapval.black;

            xvw_set_attributes(xwid->printmapval.r_labels[i],
                               XVW_LABEL,      temp_r,
                               XVW_BACKGROUND, pixel,
                               NULL);
	    xvw_set_attributes(xwid->printmapval.g_labels[i],
                               XVW_LABEL,      temp_g,
                               XVW_BACKGROUND, pixel,
                               NULL);
	    xvw_set_attributes(xwid->printmapval.b_labels[i],
                               XVW_LABEL,      temp_b,
                               XVW_BACKGROUND, pixel,
                               NULL);

        }
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayDisplayedValuesRGB
|
|       Purpose: Updates the display of displayed values for
|		 RGB images.
|
|         Input: widget    - the printmapval widget to update display for
|                valuedata - region of value data (for indexing into maps)
|                maskdata  - region of mask data (if applicable)
|    Written By: Danielle Argiro
|          Date: March 5, 1996
|
------------------------------------------------------------*/

static void RedisplayDisplayedValuesRGB(
    Widget   widget,
    unsigned char *maskdata)
{
        XvwPrintMapValWidget xwid = kwidget(widget);

        int     i, j;
        Pixel   pixel;
        double  dvect[4];
        char    temp_r[KLENGTH];
        char    temp_g[KLENGTH];
        char    temp_b[KLENGTH];

        int      xpos        = xwid->printmapval.xposition;
        int      ypos        = xwid->printmapval.yposition;
        int      width       = xwid->printmapval.width;
        int      height      = xwid->printmapval.height;
        kobject  data_object = xwid->printmapval.object;

        kpds_set_attribute(data_object, KPDS_VALUE_DATA_TYPE, KDOUBLE);

        for (i = 0; i < height; i++)
        {
            for (j = 0; j < width; j++)
            {
                kpds_set_attribute(data_object, KPDS_VALUE_POSITION,
                                   xpos+j, ypos+i, 0, 0, 0);

                /*
                 * print value into label, unless value is masked out,
                 * in which case we leave the value blank.
                 */
                if ((maskdata != NULL) && (maskdata[i] == 0))
	        {
                    ksprintf(temp_r, "  ");
                    ksprintf(temp_g, "  ");
                    ksprintf(temp_b, "  ");
	        }
                else if (kpds_get_data(data_object, KPDS_VALUE_VECTOR,
                                       dvect)!= NULL)
	        {
		    ksprintf(temp_r, "%g", dvect[0]);
		    ksprintf(temp_g, "%g", dvect[1]);
		    ksprintf(temp_b, "%g", dvect[2]);
	        }
    
                /*
                 * set background of label to color of pixel,
                 * unless value is masked out in which case use bg color
                 */
                if (xwid->printmapval.showcolor)
                {
                    if ((maskdata != NULL) && (maskdata[i] == 0))
                        pixel = xwid->core.background_pixel;
	            else pixel =  ColorRGBAllocate(widget, (int) dvect[0],
				           (int) dvect[1], (int) dvect[2]);
                }
                else pixel = xwid->printmapval.black;
    
                xvw_set_attributes(xwid->printmapval.r_labels[i*width+j],
                                   XVW_LABEL,      temp_r,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
	        xvw_set_attributes(xwid->printmapval.g_labels[i*width+j],
                                   XVW_LABEL,      temp_g,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
	        xvw_set_attributes(xwid->printmapval.b_labels[i*width+j],
                                   XVW_LABEL,      temp_b,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
	    }
        }
}

/************************************************************
*
*  Routine Name: xvw_create_printmapval - create a printmapval xvobject
*
*       Purpose: A printmapval object provides a mechanism which will track 
*                pointer movement in an image object and print the values of 
*                the map data indexed by the pixel under the pointer.
*                It was designed for use in conjunction with an image object,
*                and is only useful with data objects having a map segment.  
*
*                The printmapval object is made up of one or more 
*                (width x height) sets of label objects;  typically, there 
*                will be three such sets, one each for the Red, Green, and 
*                Blue columns of the colormap.  The background of each label 
*                is the color defined by the values of the map columns at that 
*                pixel, and the value displayed is the value of the 
*                corresponding map column.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the printmapval
*                         object (for use in app-defaults files, etc)
*        Output: 
*	Returns: The printmapval xvobject on success, NULL otherwise
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 17, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_printmapval(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, PrintMapValWidgetClass);
	return(object);
}


