/*
  * Khoros: $Id: PrintPixel.c,v 1.20.4.1 1996/12/06 20:29:39 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: PrintPixel.c,v 1.20.4.1 1996/12/06 20:29:39 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		PrintPixel Widget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 InitLabels()
   >>>>                 CreateLabelGrid()
   >>>>                 ReInitLabelGrid()
   >>>>                 SetLabelFormat()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 PrintPixelGetName()
   >>>>                 PrintPixelSetName()
   >>>>                 ReloadCallback()
   >>>>                 PositionCallback()
   >>>>                 ColorReloadCallback()
   >>>>                 RedisplayPixels()
   >>>>                 RedisplayPixelsReal()
   >>>>                 RedisplayPixelsRealRGB()
   >>>>                 RedisplayPixelsComplex()
   >>>>                 RedisplayPixelsComplexRGB()
   >>>>   Public:
   >>>>                 xvw_create_printpixel()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/PrintPixelP.h>


static void ClassInitialize           PROTO((void));
static void Destroy                   PROTO((Widget));
static void Initialize                PROTO((Widget, Widget, ArgList, 
					     Cardinal *));
static void Redisplay	              PROTO((Widget, XEvent *, Region));
static Boolean SetValues              PROTO((Widget, Widget, Widget, ArgList,
				             Cardinal *));

static int  PrintPixelGetName             PROTO((xvobject, char *, kaddr));
static int  PrintPixelSetName             PROTO((xvobject, char *, kaddr));
static void ReloadCallback            PROTO((kobject, char *, Widget, 
				             kdms_callback *));
static void PositionCallback          PROTO((kobject, char *, Widget,
				             kdms_callback *));
static void ColorReloadCallback       PROTO((xvobject, kaddr));
static void RedisplayPixels           PROTO((Widget));
static void RedisplayPixelsReal       PROTO((Widget, unsigned char *));
static void RedisplayPixelsRealRGB    PROTO((Widget, unsigned char *));
static void RedisplayPixelsComplex    PROTO((Widget, unsigned char *));
static void RedisplayPixelsComplexRGB PROTO((Widget, unsigned char *));
static void InitLabels                PROTO((Widget, int, int));
static void CreateLabelGrid           PROTO((xvobject *, xvobject, 
					     int, int, int));
static void ReInitLabelGrid           PROTO((Widget, int));

#undef  kwidget
#define kwidget(widget)      (XvwPrintPixelWidget) (widget)

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PRINTPIXEL_WIDTH,            NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_HEIGHT,           NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_EDITABLE,         NULL,    XtRInt,        XtRBoolean},
{XVW_PRINTPIXEL_OBJECT,           NULL,    XtRPointer,    NULL},
{XVW_PRINTPIXEL_SHOWCOLOR,        NULL,    XtRInt,        XtRBoolean},
{XVW_PRINTPIXEL_UPDATEMODE,       NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_XPOSITION,        NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_YPOSITION,        NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_CLIPOBJ,          NULL,    XtRPointer,    NULL},
{XVW_PRINTPIXEL_DEPTH_OFFSET,     NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_TIME_OFFSET,      NULL,    XtRInt,        NULL},
{XVW_PRINTPIXEL_ELEMENTS_OFFSET,  NULL,    XtRInt,        NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwPrintPixelWidgetRec, printpixel.field)

static XtResource resources[] = { 

{XVW_PRINTPIXEL_WIDTH, NULL, XtRInt, sizeof(int),
      offset(width), XtRImmediate, (XtPointer) 7},
{XVW_PRINTPIXEL_HEIGHT, NULL, XtRInt, sizeof(int),
      offset(height), XtRImmediate, (XtPointer) 9},
{XVW_PRINTPIXEL_EDITABLE, NULL, XtRBoolean, sizeof(Boolean),
      offset(editable), XtRImmediate, (XtPointer) FALSE},
{XVW_PRINTPIXEL_SHOWCOLOR, NULL, XtRBoolean, sizeof(Boolean),
      offset(showcolor), XtRImmediate, (XtPointer) TRUE},
{XVW_PRINTPIXEL_OBJECT, NULL, XtRPointer, sizeof(kobject),
      offset(object), XtRImmediate, (XtPointer) NULL},
{XVW_PRINTPIXEL_UPDATEMODE, NULL, XtRInt, sizeof(int),
      offset(update_mode), XtRImmediate, (XtPointer) KPRINTPIXEL_UM_CONTINUOUS},
{XVW_PRINTPIXEL_XPOSITION, NULL, XtRInt, sizeof(int),
      offset(xposition), XtRImmediate, (XtPointer) 0},
{XVW_PRINTPIXEL_YPOSITION, NULL, XtRInt, sizeof(int),
      offset(yposition), XtRImmediate, (XtPointer) 0},
{XVW_PRINTPIXEL_CLIPOBJ, NULL, XtRPointer, sizeof(kobject),
      offset(clipmask), XtRImmediate, (XtPointer) NULL},
{XVW_PRINTPIXEL_DEPTH_OFFSET, NULL, XtRInt, sizeof(int),
      offset(depth_offset), XtRImmediate, (XtPointer) 0},
{XVW_PRINTPIXEL_TIME_OFFSET, NULL, XtRInt, sizeof(int),
      offset(time_offset), XtRImmediate, (XtPointer) 0},
{XVW_PRINTPIXEL_ELEMENTS_OFFSET, NULL, XtRInt, sizeof(int),
      offset(elements_offset), XtRImmediate, (XtPointer) 0},

{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(black), XtRString, (XtPointer) "#000000"},
{NULL, NULL, XtRPixel, sizeof(Pixel),
      offset(white), XtRString, (XtPointer) "#ffffff"},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwPrintPixelWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
};
#undef offset


#define SUPERCLASS (&xvwColorWidgetClassRec)

XvwPrintPixelWidgetClassRec xvwPrintPixelWidgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "PrintPixel",			/* class_name		  */
    sizeof(XvwPrintPixelWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    knumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    XtInheritTranslations,		/* tm_table		  */
    NULL,				/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,	            		/* subresources       */
    0,					/* subresources_count */
    sizeof(XvwManagerWidgetConstraintsRec),/* constraint_size    */
    NULL,				/* initialize         */
    NULL,                               /* destroy            */
    NULL,				/* set_values         */
    NULL,                               /* extension          */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			/* pre-layout routine     */
    XtInheritLayout,			/* post-layout routine    */
    XtInheritChangeSel,		        /* change selected proc   */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    XtInheritResize,			/* resize		  */
    XtInheritGeometryManager,  		/* geometry_manager	  */
  }, /* XvwManagerWidgetClass fields initialization */
  {
    NULL,                                     /* field not used    */
  },  /* XvwGraphicsWidgetClass fields initialization */
  {
    ColorReloadCallback,                /* reload color proc    */
  },  /* XvwColorWidgetClass fields initialization */
  {
    NULL,                               /* field not used    */
  },  /* XvwPrintPixelWidgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwPrintPixelWidgetClass for public consumption
 */
WidgetClass xvwPrintPixelWidgetClass = (WidgetClass) &xvwPrintPixelWidgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of printpixel widget class has been created.
|                It will initialize all the class attributes.
|
|    Written By: Mark Young 
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
        xvw_init_attributes(xvwPrintPixelWidgetClass, attributes,
		      (int) knumber(attributes), NULL, 0, 
		      "$DESIGN/objects/library/xvisual/uis/PrintPixel.pane");
	xvw_load_resources("$DESIGN/objects/library/xvisual/app-defaults/PrintPixel");

	xvw_define_attributes(xvwPrintPixelWidgetClass,
	  XVW_PRINTPIXEL_FILENAME, XtRString, PrintPixelSetName, PrintPixelGetName,
	  XVW_PRINTPIXEL_CLIPFILE, XtRString, PrintPixelSetName, PrintPixelGetName,
	  NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial image
|                for an image widget instance.
|
|         Input: request - not used
|                new     - widget instance after initialization, with
|                          initial image
|    Written By: Mark Young & Danielle Argiro
|          Date: May 16, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwPrintPixelWidget xwid = kwidget(new);

	XColor    color;

	/* not assuming RGB by default */
	xwid->printpixel.rgb = FALSE;

	/* allocate black specially */
	color.flags = DoRed | DoGreen | DoBlue;
	color.red = color.green = color.blue = 0;
	if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
	   xwid->printpixel.black = color.pixel;

	/* allocate white specially */
	color.flags = DoRed | DoGreen | DoBlue;
	color.red = color.green = color.blue = 65535;
	if (XAllocColor(XtDisplay(new), new->core.colormap, &color) != 0)
	   xwid->printpixel.white = color.pixel;

	xwid->printpixel.parent    = NULL;
	xwid->printpixel.labels    = NULL;
	xwid->printpixel.r_parent  = NULL;
	xwid->printpixel.r_labels  = NULL;
	xwid->printpixel.g_parent  = NULL;
	xwid->printpixel.g_labels  = NULL;
	xwid->printpixel.b_parent  = NULL;
	xwid->printpixel.b_labels  = NULL;
	xwid->printpixel.labelsize = 4;

	/* create printpixel display */
	InitLabels(new, xwid->printpixel.width, xwid->printpixel.height);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Frees memory before the printpixel object is destroyed.
|
|         Input: widget - the image widget being destroyed
|
|    Written By: Danielle Argiro
|          Date: Aug 11, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwPrintPixelWidget xwid = kwidget(widget);

        kfree_and_NULL(xwid->printpixel.labels);
        kfree_and_NULL(xwid->printpixel.r_labels);
        kfree_and_NULL(xwid->printpixel.g_labels);
        kfree_and_NULL(xwid->printpixel.b_labels);
}


/*-----------------------------------------------------------
|
|  Routine Name: InitLabels
|
|       Purpose: This utility routine creates and initializes
|		 the PrintPixel display.  If non-RGB, creates
|		 one grid of labels to display pixel values.
|		 If RGB, creates three grids of labels
|		 to display red, green, and blue values.
|
|         Input: widget - The PrintPixel widget
|		 parent - parent of 
|    Written By: Danielle Argiro
|          Date: May 16, 1995
|
------------------------------------------------------------*/
static void InitLabels(
    Widget    widget,
    int       old_width,
    int       old_height)
{
        XvwPrintPixelWidget xwid = kwidget(widget);
	int width  = xwid->printpixel.width;
	int height = xwid->printpixel.height;
	xvobject back = xvw_object(widget);
	int i;

	/*
	 *  non-RGB: use single manager with single grid
	 */
	if (!xwid->printpixel.rgb) 
	{
	    xvw_unmanage(xwid->printpixel.r_parent);
	    xvw_unmanage(xwid->printpixel.g_parent);
	    xvw_unmanage(xwid->printpixel.b_parent);
	    xvw_manage(xwid->printpixel.parent);

	    if (xwid->printpixel.labels != NULL)
	    {
	        for (i = 0; i < old_width * old_height; i++)
	    	    xvw_destroy(xwid->printpixel.labels[i]);
		kfree_and_NULL(xwid->printpixel.labels);
	    }
            xwid->printpixel.labels = 
		(xvobject *) kcalloc((size_t) width*height,sizeof(xvobject));

	    if (xwid->printpixel.parent == NULL)
	        xwid->printpixel.parent = xvw_create_manager(back, "parent");
	    CreateLabelGrid(xwid->printpixel.labels, xwid->printpixel.parent, 
			    width, height, xwid->printpixel.labelsize);
	}

	/*
	 * RGB: use three managers each with their own grid
	 */
	else
	{
	    xvw_unmanage(xwid->printpixel.parent);
	    xvw_manage(xwid->printpixel.r_parent);
	    xvw_manage(xwid->printpixel.g_parent);
	    xvw_manage(xwid->printpixel.b_parent);

	    /* RED */
	    if (xwid->printpixel.r_labels != NULL)
	    {
	        for (i = 0; i < old_width * old_height; i++)
	    	    xvw_destroy(xwid->printpixel.r_labels[i]);
		kfree_and_NULL(xwid->printpixel.r_labels);
	    }
            xwid->printpixel.r_labels = 
                (xvobject *) kcalloc((size_t) width*height,sizeof(xvobject));

	    if (xwid->printpixel.r_parent == NULL)
                xwid->printpixel.r_parent = xvw_create_manager(back, "rparent");
            CreateLabelGrid(xwid->printpixel.r_labels, 
			    xwid->printpixel.r_parent, width, height, 
			    xwid->printpixel.labelsize);
	    /* GREEN */
	    if (xwid->printpixel.g_labels != NULL)
	    {
	        for (i = 0; i < old_width * old_height; i++)
	    	    xvw_destroy(xwid->printpixel.g_labels[i]);
		kfree_and_NULL(xwid->printpixel.g_labels);
	    }
            xwid->printpixel.g_labels = 
                (xvobject *) kcalloc((size_t) width*height,sizeof(xvobject));

	    if (xwid->printpixel.g_parent == NULL)
	    {
                xwid->printpixel.g_parent = xvw_create_manager(back, "gparent");
		xvw_set_attribute(xwid->printpixel.g_parent, XVW_BELOW,
				  xwid->printpixel.r_parent);
	    }
            CreateLabelGrid(xwid->printpixel.g_labels, 
			    xwid->printpixel.g_parent, width, height,
			    xwid->printpixel.labelsize);
	    /* BLUE */
            if (xwid->printpixel.b_labels != NULL)
            {
                for (i = 0; i < old_width * old_height; i++)
                    xvw_destroy(xwid->printpixel.b_labels[i]);
                kfree_and_NULL(xwid->printpixel.b_labels);
            }
            xwid->printpixel.b_labels = 
                (xvobject *) kcalloc((size_t) width*height,sizeof(xvobject));

	    if (xwid->printpixel.b_parent == NULL)
	    {
                xwid->printpixel.b_parent = xvw_create_manager(back, "bparent");
                xvw_set_attribute(xwid->printpixel.b_parent, XVW_BELOW,
                                  xwid->printpixel.g_parent);
	    }
            CreateLabelGrid(xwid->printpixel.b_labels, 
			    xwid->printpixel.b_parent, width, height,
			    xwid->printpixel.labelsize);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: CreateLabelGrid
|
|       Purpose: This utility routine creates a single grid of labels 
|		 to display values.  Grid may be used for display of pixel 
|		 values (non-RGB images) or for RGB values (RGB images).
|
|         Input: labels  - previously-allocated array of label objects
|		 parent  - parent on which to create label objects
|		 width   - width of printpixel display
|		 height  - width of printpixel display
|    Written By: Danielle Argiro
|          Date: May 17, 1995
|
------------------------------------------------------------*/

static void CreateLabelGrid(
    xvobject *labels,
    xvobject parent,
    int      width,
    int      height,
    int      labelsize)
{
	int       i, j;
	xvobject  vert_offset = NULL, horiz_offset = NULL;

	/*
         * create display using array of label objects
         */
         for (i = 0; i < height; i++)
         {
             for (j = 0; j < width; j++)
             {
                 labels[i*width+j]= xvw_create_label(parent, "label");
                 xvw_set_attributes(labels[i*width+j],
                            XVW_LABEL,                "0",
                            XVW_FORCE_REDISPLAY,      TRUE,
                            XVW_LABEL_JUSTIFY,        KLABEL_JUSTIFY_CENTER,
                            XVW_LABEL_EMPHASIZE,      TRUE,
                            XVW_CHAR_MIN_WIDTH,       (float) labelsize,
                            XVW_RIGHT_OF,             horiz_offset,
                            XVW_BELOW,                vert_offset,
                            XVW_FOREGROUND_COLOR,     "white",
                            XVW_LABEL_EMPHASIZECOLOR, "black",
                            NULL);
                 if ((i == (int) height/2) && (j == (int) width/2))
                     xvw_set_attribute(labels[i*width+j], XVW_BORDER_WIDTH, 1);
                 horiz_offset = labels[i*width+j];
            }
            vert_offset  = labels[i*width];
            horiz_offset = NULL;
	}

}

/*-----------------------------------------------------------
|
|  Routine Name: ReInitLabelGrid
|
|       Purpose: Since the label grid is not re-created on a new
|                incoming image, this utility routine updates the
|                size of the labels according to the new value
|                of labelsize, and re-initializes values to 0.
|
|         Input: widget     - printpixel widget
|		 size_only  - TRUE to reset labelsize only
|    Written By: Danielle Argiro
|          Date: May 25, 1995
|
------------------------------------------------------------*/

static void ReInitLabelGrid(
    Widget   widget,
    int      size_only)
{
        int       i, j;
        XvwPrintPixelWidget xwid = kwidget(widget);
        int width        = xwid->printpixel.width;
        int height       = xwid->printpixel.height;
        int labelsize    = xwid->printpixel.labelsize;

        /*
         * re-initialize display of label objects
         */
        for (i = 0; i < height; i++)
        {
            for (j = 0; j < width; j++)
            {
		if (!size_only)
                    xvw_set_attributes(xwid->printpixel.labels[i*width+j],
                                   XVW_LABEL,          "0",
                                   XVW_CHAR_MIN_WIDTH, (float) labelsize,
                                   XVW_BACKGROUND,     xwid->printpixel.black,
                                   NULL);
		else xvw_set_attributes(xwid->printpixel.labels[i*width+j],
                                   XVW_CHAR_MIN_WIDTH, (float) labelsize,
                                   NULL);

            }
        }
}


/*-----------------------------------------------------------
|
|  Routine Name: SetLabelFormat
|
|       Purpose: This utility routine looks at the data object
|		 and decides on a good size for the labels.
|
|         Input: widget  - the PrintPixel widget
|    Written By: Danielle Argiro
|          Date: May 17, 1995
|
------------------------------------------------------------*/

static void SetLabelFormat(
     Widget widget)
{
	int    type, labelsize;
	double magval, logval;
	XvwPrintPixelWidget xwid = kwidget(widget);
	kobject  data_object = xwid->printpixel.object;

	kpds_get_attribute(data_object, KPDS_VALUE_DATA_TYPE, &type);

	switch (type) {
	    case KBIT:
		 xwid->printpixel.labelsize = 2;
		 break;

	    case KUBYTE:
		 xwid->printpixel.labelsize = 4;
		 break;

	    case KBYTE:
		 if (xwid->color.info->minval < 0)
		     xwid->printpixel.labelsize = 5;
	         else xwid->printpixel.labelsize = 4;

	    case KSHORT:
	    case KUSHORT:
	    case KINT:
	    case KUINT:
	    case KLONG:
	    case KULONG:
	         if (xwid->color.info->histogram != NULL)
	         {
		     magval = kmax(kabs(xwid->color.info->minval), 
			        kabs(xwid->color.info->maxval));
		     if (magval < 1.0)
			xwid->printpixel.labelsize = 1;
		     else
		     {
		        logval = klog10(magval);
		        xwid->printpixel.labelsize = (int) (logval + 1.0);
		        if (xwid->color.info->minval < 0) 
			    xwid->printpixel.labelsize++;
		     }
	         }
		 else xwid->printpixel.labelsize = 7;
		 break;

	    case KFLOAT:
	    case KDOUBLE:
		 magval = kmax(kabs(xwid->color.info->minval),
			       kabs(xwid->color.info->maxval));
		 if (magval < 1.0)
		    xwid->printpixel.labelsize = 1;
		 else
		 {
                    logval = klog10(magval);
	            labelsize = (int) (logval + 1.0);
                    labelsize = kmax(labelsize, 5);
                    if (xwid->color.info->minval < 0) labelsize++;
                    xwid->printpixel.labelsize = labelsize;
		 }
		 break;

	    case KCOMPLEX:
	    case KDCOMPLEX:
		 xwid->printpixel.labelsize = 20;
		 break;
	}

}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This method calls the RedisplayPixel procedure when an
|                expose event occurs.
|
|         Input: widget - widget data structure for palette
|		 event	- the event that caused the redraw
|		 region	- the region that was exposed
|    Written By: Mark Young
|          Date: Nov 17, 1992
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	RedisplayPixels(widget);
	(*xvwPrintPixelWidgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method determines what the widget should do by
|		 comparing the current values verse the
|		 new settings.
|
|         Input: current - the widget containing current settings 
|		 request - the widget containing requested settings
|		 new	 - the widget processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: May 16, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwPrintPixelWidget cxwid = kwidget(current);
	XvwPrintPixelWidget nxwid = kwidget(new);

	Boolean redisplay = FALSE;

	int nwidth  = nxwid->printpixel.width;
	int nheight = nxwid->printpixel.height;


	/*
	 *  Change of width and/or height: note that we have
	 *  to check this FIRST so InitLabels will be called before
	 *  ReInitLabelGrid (if the object is also being set)
	 */
	if (cxwid->printpixel.width  != nxwid->printpixel.width  ||
	    cxwid->printpixel.height != nxwid->printpixel.height)
	{
	   kpds_set_attribute(nxwid->printpixel.object,
                        KPDS_VALUE_REGION_SIZE, nwidth, nheight, 1, 1, 1);
	   kpds_set_attribute(nxwid->printpixel.object,
                        KPDS_MASK_REGION_SIZE,  nwidth, nheight, 1, 1, 1);

	   if (nxwid->printpixel.clipmask != NULL)
	       kpds_set_attribute(nxwid->printpixel.clipmask,
                        KPDS_VALUE_REGION_SIZE,  nwidth, nheight, 1, 1, 1);

	   /* re-create printpixel display */
	   InitLabels(new, cxwid->printpixel.width, cxwid->printpixel.height);
	   redisplay = TRUE;
	}

	/*
	 *  Change of data object
	 */
	if (cxwid->printpixel.object != nxwid->printpixel.object)
	{
	   nxwid->printpixel.object =
		kpds_reference_object(nxwid->printpixel.object);
	   kpds_close_object(cxwid->printpixel.object);
	   kpds_add_value_callback(nxwid->printpixel.object,
		KPDS_CALLBACK_CHANGE, ReloadCallback, (kaddr) new);
	   kpds_add_value_callback(nxwid->printpixel.object,
                     KPDS_CALLBACK_ACCESS, PositionCallback, (kaddr) new);

	   xvw_set_attribute(xvw_object(new), XVW_COLOR_COLOROBJ,
                        nxwid->printpixel.object);

	   /*
            *  see if we have an RGB image
	    */
           kpds_get_attribute(nxwid->printpixel.object, KPDS_VALUE_DATA_TYPE,
			      &nxwid->printpixel.val_datatype);

           nxwid->printpixel.rgb = xvw_is_rgb(nxwid->printpixel.object);
	
	   /*
	    * set region size, mask data type for later
	    */
	   kpds_set_attribute(nxwid->printpixel.object,
			KPDS_VALUE_REGION_SIZE, nwidth, nheight, 1, 1, 1);

	   kpds_set_attributes(nxwid->printpixel.object,
                        KPDS_MASK_REGION_SIZE,  nwidth, nheight, 1, 1, 1,
                        KPDS_MASK_DATA_TYPE,    KUBYTE,
                        NULL);


	   SetLabelFormat(new);
	   if (nxwid->printpixel.rgb != cxwid->printpixel.rgb)
		InitLabels(new, cxwid->printpixel.width, 
			   cxwid->printpixel.height);
           else ReInitLabelGrid(new, FALSE);
	   redisplay = TRUE;
	}

	if (cxwid->printpixel.depth_offset != nxwid->printpixel.depth_offset ||
	    cxwid->printpixel.time_offset != nxwid->printpixel.time_offset ||
	    cxwid->printpixel.elements_offset != nxwid->printpixel.elements_offset)
	{
	    int d, t, e;

	    kpds_get_attribute(nxwid->printpixel.object,
	                       KPDS_VALUE_SIZE, NULL, NULL, &d, &t, &e,
			       NULL);
	    if ((cxwid->printpixel.depth_offset != 
	         nxwid->printpixel.depth_offset) && 
		 (nxwid->printpixel.depth_offset > d-1))
	    {
	 	kerror(XVISUAL, "PrintPixel SetValues",
		       "Data object is only of depth %d; can't set depth offset to %d\n", d, nxwid->printpixel.depth_offset);
	        nxwid->printpixel.depth_offset = cxwid->printpixel.depth_offset;
	    }
	    if ((cxwid->printpixel.time_offset != 
	         nxwid->printpixel.time_offset) && 
		 (nxwid->printpixel.time_offset > t-1))
	    {
	 	kerror(XVISUAL, "PrintPixel SetValues",
		       "Data object is only of time %d; can't set time offset to %d\n", t, nxwid->printpixel.time_offset);
	        nxwid->printpixel.time_offset = cxwid->printpixel.time_offset;
	    }
	    if ((cxwid->printpixel.elements_offset != 
	         nxwid->printpixel.elements_offset) && 
		 (nxwid->printpixel.elements_offset > e-1))
	    {
	 	kerror(XVISUAL, "PrintPixel SetValues",
		       "Data object only has %d elements; can't set elements offset to %d\n", e, nxwid->printpixel.elements_offset);
	        nxwid->printpixel.elements_offset = cxwid->printpixel.elements_offset;
	    }
	}

	/*
         *  Change of clip mask
         */
        if (cxwid->printpixel.clipmask != nxwid->printpixel.clipmask)
        {
	    nxwid->printpixel.clipmask =
                 kpds_reference_object(nxwid->printpixel.clipmask);
            kpds_close_object(cxwid->printpixel.clipmask);
            kpds_add_value_callback(nxwid->printpixel.clipmask,
                KPDS_CALLBACK_CHANGE, ReloadCallback, (kaddr) new);

	    kpds_set_attributes(nxwid->printpixel.clipmask,
                        KPDS_VALUE_REGION_SIZE,  nwidth, nheight, 1, 1, 1,
                        KPDS_VALUE_DATA_TYPE,    KUBYTE,
                        NULL);
	    redisplay = TRUE;
	}

	/*
	 * Change of showcolor status
	 */
	if (cxwid->printpixel.showcolor  != nxwid->printpixel.showcolor)
	    redisplay = TRUE;
	    
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: PrintPixelGetName
|
|       Purpose: PrintPixelGetName is used to get the values of the
|		 XVW_PRINTPIXEL_FILENAME and XVW_PRINTPIXEL_CLIPFILE
|		 attributes.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|        Output: calldata  - pointer to retrieve the object name
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: August 3, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PrintPixelGetName(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwPrintPixelWidget xwid = (XvwPrintPixelWidget) xvw_widget(object);
	String *filename = (String *) calldata;


	if (kstrcmp(attribute, XVW_PRINTPIXEL_FILENAME) == 0)
	{
	    *filename = NULL;
	    return(kpds_get_attribute(xwid->printpixel.object, 
				      KPDS_NAME, filename));
	}
	else if (kstrcmp(attribute, XVW_PRINTPIXEL_CLIPFILE) == 0)
	{
	    *filename = NULL;
	     return(kpds_get_attribute(xwid->printpixel.clipmask,
                                      KPDS_NAME, filename));
	}
	else
	{
	   kerror(XVISUAL, "PrintPixelGetName", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
}


/*-----------------------------------------------------------
|
|  Routine Name: PrintPixelSetName
|
|       Purpose: PrintPixelSetName is used to set the filename associated
|		 with the XVW_PRINTPIXEL_FILENAME and XVW_PRINTPIXEL_CLIPFILE
|                attributes.
|
|         Input: object    - the object in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young & Danielle Argiro
|          Date: August 3, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PrintPixelSetName(
   xvobject printpixel,
   char     *attribute,
   kaddr    calldata)
{
	int	locate = TRUE;
	kobject object = NULL;
	String *filename = (String *) calldata;

	if (*filename != NULL && (object = kdms_locate(*filename)) == NULL)
	{
	    locate = FALSE;
	    if ((object = kpds_open_object(*filename, KOBJ_READ)) == NULL)
	    {
	        kerror(XVISUAL, "PrintPixelSetName", 
		       "Failed to load image '%s'", *filename);
	        return(TRUE);
	    }
	}
	if (kstrcmp(attribute, XVW_PRINTPIXEL_FILENAME) == 0)
	    xvw_set_attribute(printpixel, XVW_PRINTPIXEL_OBJECT, object);
	else if (kstrcmp(attribute, XVW_PRINTPIXEL_CLIPFILE) == 0)
	    xvw_set_attribute(printpixel, XVW_PRINTPIXEL_CLIPOBJ, object);
	else
	{
	    kerror(XVISUAL, "PrintPixelSetName", "invalid attribute %s",
	    	   attribute);
	    if (!locate) kpds_close_object(object);
	    return(FALSE);
	}
	if (!locate) kpds_close_object(object);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ReloadCallback
|
|       Purpose: Reload the pixels since the data has changed
|
|         Input: xwid - the printpixel widget to load the value data
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ReloadCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	/* perform check to see if we are in the area that's changed */
	RedisplayPixels(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: PositionCallback
|
|       Purpose: This re-prints the pixels of the object being tracked
|
|         Input: xwid - the printpixel widget to load the value data
|    Written By: Mark Young
|          Date: Nov 17, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PositionCallback(
   kobject object,
   char    *segment,
   Widget  widget,
   kdms_callback *cb)
{
	int      w, h, update;
	XvwPrintPixelWidget xwid = (XvwPrintPixelWidget) widget;


	/*
	 *  Make sure we aren't here because of ourself
	 */
	if (object == cb->object)
	   return;

	xvw_get_type(object, NULL, &update);
	if ((xwid->printpixel.update_mode == KPRINTPIXEL_UM_CONTINUOUS &&
	                           update != MOTION_EVENT_UPDATE) ||
	    (xwid->printpixel.update_mode == KPRINTPIXEL_UM_BUTTON_PRESS &&
	                           update != BUTTONPRESS_EVENT_UPDATE))
	{
	   return;
	}

	w = (cb->end.w - cb->begin.w)/2 + cb->begin.w;
	h = (cb->end.h - cb->begin.h)/2 + cb->begin.h;
	xwid->printpixel.xposition = w - xwid->printpixel.width/2;
	xwid->printpixel.yposition = h - xwid->printpixel.height/2;
	xwid->printpixel.depth_offset    = cb->begin.d;
        xwid->printpixel.time_offset     = cb->begin.t;
        xwid->printpixel.elements_offset = cb->begin.e;
#if 0
kfprintf(kstderr, "d, t, e = %d %d %d\n", 
         cb->begin.d, cb->begin.t, cb->begin.e);
#endif
	RedisplayPixels(widget);
}

/*-----------------------------------------------------------
|
|  Routine Name: ColorReloadCallback
|
|       Purpose: Update the printpixel since our pixels have changed
|
|         Input: object - the object to be refreshed (the PrintPixel)
|		 call_data   - (not used)
|    Written By: Mark Young
|          Date: Jun 02, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ColorReloadCallback(
   xvobject object,
   kaddr    call_data)
{
	RedisplayPixels(xvw_widget(object));
}
 
/*-----------------------------------------------------------
|
|  Routine Name: RedisplayPixels  
|
|       Purpose: Updates the display of pixels
|
|         Input: widget - the printpixel widget to update display for
|    Written By: Mark Young & Danielle Argiro
|          Date: May 16, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RedisplayPixels(
   Widget widget)
{
	XvwPrintPixelWidget xwid = kwidget(widget);
	xvobject printpixel = xvw_object(widget);

	int i;
        unsigned char *maskdata = NULL;
	unsigned char *maskptr;
        unsigned char *clipdata = NULL;

        int val_datatype    = xwid->printpixel.val_datatype;
        int xpos            = xwid->printpixel.xposition;
	int ypos            = xwid->printpixel.yposition;
        int width           = xwid->printpixel.width;
        int height          = xwid->printpixel.height;
	kobject data_object =  xwid->printpixel.object;

	if (!xvw_check_visible(printpixel) || data_object == NULL)
           return;

        /*
         *  see if there is any masked-out data
         */
        if (kpds_query_mask(data_object) == TRUE)
        {
           kpds_set_attribute(data_object, KPDS_MASK_POSITION,
                              xpos, ypos, 0, 0, 0);
           maskdata = kpds_get_data(data_object, KPDS_MASK_REGION, NULL);
        }

	/*
         *  If we have a clip object, we better get the clip data and combine
         *  it with the mask data.
         */
        if (xwid->printpixel.clipmask)
        {
           kpds_set_attribute(xwid->printpixel.clipmask,
                   KPDS_VALUE_POSITION, xpos, ypos, 0, 0, 0);

	   clipdata = kpds_get_data(xwid->printpixel.clipmask, 
				    KPDS_VALUE_REGION, NULL);
           if (maskdata != NULL)
           {
	       maskptr = maskdata;
               for (i = 0; i < width*height; i++)
		  *maskptr++ = (*clipdata++ && *maskptr) ? *maskptr : 0;
           }
	   else maskdata = clipdata;
	}

	/* 
	 * for non-complex data
	 */
        if (val_datatype != KCOMPLEX && val_datatype != KDCOMPLEX)
	{
	    /* non-RGB images */
	    if (!xwid->printpixel.rgb)
	        RedisplayPixelsReal(widget, maskdata);

	    /* RGB images */
	    else RedisplayPixelsRealRGB(widget, maskdata);
	}
	/*
	 * for complex data
	 */
	else 
	{
	    /* non-RGB images */
            if (!xwid->printpixel.rgb)
                RedisplayPixelsComplex(widget, maskdata);

            /* RGB images */
            else RedisplayPixelsComplexRGB(widget, maskdata);
	}
        kfree_and_NULL(maskdata);
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayPixelsReal
|
|       Purpose: Updates the display of pixels when the data object:
|		  -> IS NOT of complex or double complex data type
|		  -> IS NOT an an RGB image.  
|		 Assumes that the KPDS_VALUE_REGION_SIZE has already been set!
|
|         Input: widget   - the printpixel widget to update display for
|	         maskdata - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayPixelsReal(
    Widget   widget,
    unsigned char *maskdata)
{
	XvwPrintPixelWidget xwid = kwidget(widget);

	int     i, indx; 
	char    temp[KLENGTH];
        double  *valuedata = NULL;
	double  factor = 1.0;
	Pixel   pixel;

 	double minval = xwid->color.info->minval;
        double maxval = xwid->color.info->maxval;
        int indx_min  = xwid->color.info->indx_min;
        int indx_max  = xwid->color.info->indx_max;
	

	int      width        = xwid->printpixel.width;
        int      height       = xwid->printpixel.height;
        int      depth        = xwid->printpixel.depth_offset;
        int      ttime        = xwid->printpixel.time_offset;
        int      elements     = xwid->printpixel.elements_offset;
	int      val_datatype = xwid->printpixel.val_datatype;
	kobject  data_object  = xwid->printpixel.object;
	int      xpos         = xwid->printpixel.xposition;
	int      ypos         = xwid->printpixel.yposition;
	xvobject *labels      = xwid->printpixel.labels;

        kpds_set_attribute(data_object, KPDS_VALUE_POSITION,
                           xpos, ypos, depth, ttime, elements);

	kpds_set_attribute(data_object, KPDS_VALUE_DATA_TYPE, KDOUBLE);
        valuedata = kpds_get_data(data_object, KPDS_VALUE_REGION, NULL);

        if (val_datatype == KFLOAT   || val_datatype == KDOUBLE ||
            val_datatype == KCOMPLEX || val_datatype == KDCOMPLEX)
	{
	    factor = (indx_max - indx_min)/(maxval - minval);
	}

        for (i = 0; i < width*height; i++)
        {
            /* 
	     * print value into label, unless value is masked out,
	     * in which case we leave the value blank.
	     */
            if ((maskdata != NULL) && (maskdata[i] == 0))
                    ksprintf(temp, "  ");
            else ksprintf(temp, "%g", valuedata[i]);

	    indx = (int) ((valuedata[i] - minval) * factor);

            /* 	
	     * set background of label to color of pixel,
	     * unless value is masked out in which case use bg color
	     */
	    if (xwid->printpixel.showcolor)
	    {
	        if ((maskdata != NULL) && (maskdata[i] == 0))
	            pixel = xwid->core.background_pixel;
    	        else pixel = ColorAllocate(widget, (unsigned) indx);
	    }
	    else pixel = xwid->printpixel.black;

	    if (kstrlen(temp) > xwid->printpixel.labelsize)
	    {
		xwid->printpixel.labelsize = kstrlen(temp);
	        ReInitLabelGrid(widget, TRUE);
	    }
            xvw_set_attributes(labels[i], 
			       XVW_LABEL,      temp,
            		       XVW_BACKGROUND, pixel,
			       NULL);
	}
	kfree_and_NULL(valuedata);
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayPixelsRealRGB
|
|       Purpose: Updates the display of pixels when the data object:
|                 -> IS NOT of complex or double complex data type
|                 -> IS an RGB image.
|                Assumes that the KPDS_VALUE_REGION_SIZE
|                and KPDS_VALUE_POSITION have already been set!
|
|         Input: widget   - the printpixel widget to update display for
|                maskdata - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayPixelsRealRGB(
    Widget   widget,
    unsigned char *maskdata)
{
	XvwPrintPixelWidget xwid = kwidget(widget);

        int     i, j;
	Pixel   pixel;
        double  dvect[4];
        char    tempR[KLENGTH];
        char    tempG[KLENGTH];
        char    tempB[KLENGTH];

        int      xpos        = xwid->printpixel.xposition;
        int      ypos        = xwid->printpixel.yposition;
        int      width       = xwid->printpixel.width;
        int      height      = xwid->printpixel.height;
        int      depth       = xwid->printpixel.depth_offset;
        int      ttime       = xwid->printpixel.time_offset;
        int      elements    = xwid->printpixel.elements_offset;
	kobject  data_object = xwid->printpixel.object;
	xvobject *r_labels   = xwid->printpixel.r_labels;
	xvobject *g_labels   = xwid->printpixel.g_labels;
	xvobject *b_labels   = xwid->printpixel.b_labels;

        kpds_set_attribute(data_object, KPDS_VALUE_DATA_TYPE, KDOUBLE);

        for (i = 0; i < height; i++)
        {
            for (j = 0; j < width; j++)
	    {
		kpds_set_attribute(data_object, KPDS_VALUE_POSITION,
                                   xpos+j, ypos+i, depth, ttime, elements);

		/*
                 * print value into label, unless value is masked out,
                 * in which case we leave the value blank.
                 */
            	if ((maskdata != NULL) && (maskdata[i] == 0))
	        {
                    ksprintf(tempR, "  ");
                    ksprintf(tempG, "  ");
                    ksprintf(tempB, "  ");
		}
            	else if (kpds_get_data(data_object, KPDS_VALUE_VECTOR, 
				       dvect)!= NULL)
                {
                    ksprintf(tempR, "%g", dvect[0]);
                    ksprintf(tempG, "%g", dvect[1]);
                    ksprintf(tempB, "%g", dvect[2]);
		}

	       /*
                * set background of labels to color of pixel,
                * unless value is masked out in which case use bg color
		*/

		if (xwid->printpixel.showcolor)
                {
                    if ((maskdata != NULL) && (maskdata[i] == 0))
                        pixel = xwid->core.background_pixel;
                    else pixel = ColorRGBAllocate(widget, (int) dvect[0],
					        (int) dvect[1], (int) dvect[2]);
		}
		else pixel = xwid->printpixel.black;

		xvw_set_attributes(r_labels[i*width+j],
                                   XVW_LABEL,      tempR,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
                xvw_set_attributes(g_labels[i*width+j],
                                   XVW_LABEL,      tempG,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
                xvw_set_attributes(b_labels[i*width+j],
                                   XVW_LABEL,      tempB,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
	    }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayPixelsComplex
|
|       Purpose: Updates the display of pixels when the data object:
|                 -> IS of complex or double complex data type
|                 -> IS NOT an an RGB image.
|                Assumes that the KPDS_VALUE_REGION_SIZE has already been set!
|
|         Input: widget   - the printpixel widget to update display for
|                maskdata - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayPixelsComplex(
    Widget   widget,
    unsigned char *maskdata)
{
        XvwPrintPixelWidget xwid = kwidget(widget);

        int         i, indx;
        char        temp[KLENGTH];
        kdcomplex  *valuedata = NULL;
	double      *dbl_valuedata = NULL;
	double      factor; 
        Pixel       pixel;

        double minval = xwid->color.info->minval;
        double maxval = xwid->color.info->maxval;
        int indx_min  = xwid->color.info->indx_min;
        int indx_max  = xwid->color.info->indx_max;

        int      width       = xwid->printpixel.width;
        int      height      = xwid->printpixel.height;
        int      depth       = xwid->printpixel.depth_offset;
        int      ttime       = xwid->printpixel.time_offset;
        int      elements    = xwid->printpixel.elements_offset;
        kobject  data_object = xwid->printpixel.object;
        int      xpos        = xwid->printpixel.xposition;
        int      ypos        = xwid->printpixel.yposition;
        xvobject *labels     = xwid->printpixel.labels;

	/* need a version of the data in type double to set bg color */
        kpds_set_attributes(data_object, 
		KPDS_VALUE_POSITION,        xpos, ypos, depth, ttime, elements,
		KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
		KPDS_VALUE_DATA_TYPE,       KDOUBLE,
		NULL);
        dbl_valuedata = kpds_get_data(data_object, KPDS_VALUE_REGION, NULL);

	/* need a version of the data in type complex to display pixel val*/
        kpds_set_attributes(data_object, 
		KPDS_VALUE_POSITION,        xpos, ypos, depth, ttime, elements,
		KPDS_VALUE_DATA_TYPE,       KDCOMPLEX,
		NULL);
        valuedata = kpds_get_data(data_object, KPDS_VALUE_REGION, NULL);

	factor = (indx_max - indx_min)/(maxval - minval);

        for (i = 0; i < width*height; i++)
        {
	    indx = (int) ((dbl_valuedata[i] - minval) * factor);

            /*
             * print value into label, unless value is masked out,
             * in which case we leave the value blank.
             */
            if ((maskdata != NULL) && (maskdata[i] == 0))
                    ksprintf(temp, "  ");
            else ksprintf(temp, "%.4e %cj%.4e", valuedata[i].r, 
			  valuedata[i].i < 0.0 ? '-' : '+', 
			  kabs(valuedata[i].i));

            /*
             * set background of label to color of pixel,
             * unless value is masked out in which case use bg color
             */
	    if (xwid->printpixel.showcolor)
            {
                if ((maskdata != NULL) && (maskdata[i] == 0))
                    pixel = xwid->core.background_pixel;
                else pixel = ColorAllocate(widget, (unsigned) indx);
            }
            else pixel = xwid->printpixel.black;

            xvw_set_attributes(labels[i],
                               XVW_LABEL,      temp,
                               XVW_BACKGROUND, pixel,
                               NULL);
        }
	kfree_and_NULL(dbl_valuedata);
	kfree_and_NULL(valuedata);
}

/*-----------------------------------------------------------
|
|  Routine Name: RedisplayPixelsComplexRGB
|
|       Purpose: Updates the display of pixels when the data object:
|                 -> IS of complex or double complex data type
|                 -> IS an RGB image.
|                Assumes that the KPDS_VALUE_REGION_SIZE
|                and KPDS_VALUE_POSITION have already been set!
|
|         Input: widget   - the printpixel widget to update display for
|                maskdata - region of mask data (if applicable)
|    Written By: Danielle Argiro & Mark Young
|          Date: May 16, 1995
|
------------------------------------------------------------*/

static void RedisplayPixelsComplexRGB(
    Widget   widget,
    unsigned char *maskdata)
{
        XvwPrintPixelWidget xwid = kwidget(widget);

        int        i, j, indx;
	Pixel      pixel;
        kdcomplex  dcvect[4];
        char       tempR[KLENGTH];
        char       tempG[KLENGTH];
        char       tempB[KLENGTH];
	double     *dbl_valuedata = NULL;
	double     factor;

        double minval = xwid->color.info->minval;
        double maxval = xwid->color.info->maxval;
        int indx_min  = xwid->color.info->indx_min;
        int indx_max  = xwid->color.info->indx_max;

        int      xpos        = xwid->printpixel.xposition;
        int      ypos        = xwid->printpixel.yposition;
        int      width       = xwid->printpixel.width;
        int      height      = xwid->printpixel.height;
        int      depth       = xwid->printpixel.depth_offset;
        int      ttime       = xwid->printpixel.time_offset;
        int      elements    = xwid->printpixel.elements_offset;
	kobject  data_object = xwid->printpixel.object;
	xvobject *r_labels   = xwid->printpixel.r_labels;
	xvobject *g_labels   = xwid->printpixel.g_labels;
	xvobject *b_labels   = xwid->printpixel.b_labels;

	/* need a version of the data in type double to set bg color */
        kpds_set_attributes(data_object,
                KPDS_VALUE_POSITION,        xpos, ypos, depth, ttime, elements,
                KPDS_VALUE_COMPLEX_CONVERT, KLOGMAGP1,
                KPDS_VALUE_DATA_TYPE,       KDOUBLE,
                NULL);
        dbl_valuedata = kpds_get_data(data_object, KPDS_VALUE_REGION, NULL);

        /* need a version of the data in type complex to display pixel val*/
        kpds_set_attributes(data_object,
                KPDS_VALUE_POSITION,        xpos, ypos, depth, ttime, elements,
                KPDS_VALUE_DATA_TYPE,       KDCOMPLEX,
                NULL);

        factor = (indx_max - indx_min)/(maxval - minval);

        for (i = 0; i < height; i++)
        {
            for (j = 0; j < width; j++)
	    {
		indx = (int) ((dbl_valuedata[i*width+j] - minval) * factor);

		kpds_set_attribute(data_object, KPDS_VALUE_POSITION,
                                   xpos+j, ypos+i, depth, ttime, elements);
		/*
                 * print value into label, unless value is masked out,
                 * in which case we leave the value blank.
                 */
            	if ((maskdata != NULL) && (maskdata[i] == 0))
	        {
                    ksprintf(tempR, "  ");
                    ksprintf(tempG, "  ");
                    ksprintf(tempB, "  ");
		}
            	else if (kpds_get_data(data_object, KPDS_VALUE_VECTOR, 
				       dcvect)!= NULL)
                {
                    ksprintf(tempR, "%.4e %cj%.4e", dcvect[0].r,
                             dcvect[0].i < 0.0 ? '-' : '+', kabs(dcvect[0].i));

                    ksprintf(tempG, "%.4e %cj%.4e", dcvect[1].r,
                             dcvect[1].i < 0.0 ? '-' : '+', kabs(dcvect[1].i));

                    ksprintf(tempB, "%.4e %cj%.4e", dcvect[2].r,
                             dcvect[2].i < 0.0 ? '-' : '+', kabs(dcvect[2].i));
		}


	       /*
                * set background of labels to color of pixel,
                * unless value is masked out in which case use bg color
		*/

                if (xwid->printpixel.showcolor)
                {
                    if ((maskdata != NULL) && (maskdata[i] == 0))
                        pixel = xwid->core.background_pixel;
 		    else pixel = ColorAllocate(widget, (unsigned) indx);
                }
		else pixel = xwid->printpixel.black;

		xvw_set_attributes(r_labels[i*width+j],
                                   XVW_LABEL,      tempR,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
                xvw_set_attributes(g_labels[i*width+j],
                                   XVW_LABEL,      tempG,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
                xvw_set_attributes(b_labels[i*width+j],
                                   XVW_LABEL,      tempB,
                                   XVW_BACKGROUND, pixel,
                                   NULL);
	    }
	}
	kfree(dbl_valuedata);
}

/************************************************************
*
*  Routine Name: xvw_create_printpixel - create a printpixel xvobject
*
*       Purpose: A printpixel object provides a mechanism which will track 
*                pointer movement in an image object and print the values of 
*                the pixels in the area under the pointer.
*                It was designed for use in conjunction with an image object.
*
*                The printpixel object is made up of one or more 
*                (width x height) sets of label objects.  The background of 
*                each label is the color in which the pixel appears.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the printpixel
*                         object (for use in app-defaults files, etc)
*        Output: 
*	Returns: The printpixel xvobject on success, NULL otherwise
*  Restrictions:
*    Written By: Mark Young
*          Date: Nov 17, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_printpixel(
   xvobject parent,
   char     *name)
{
	xvobject object;


	/*
	 *  Call the xvw_create() routine to do the actual creation.
	 */
	object = xvw_create(parent, FALSE, TRUE, name, PrintPixelWidgetClass);
	return(object);
}
