/*
  * Khoros: $Id: utilities.c,v 1.8 1996/04/16 05:27:37 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.8 1996/04/16 05:27:37 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		General Utility Routines
   >>>>		
   >>>>  Private:
   >>>>                 xvw_draw_sidemarkers()
   >>>>                 xvw_erase_sidemarkers()
   >>>>                 xvw_pick_sidemarkers()
   >>>>                 xvw_pick_cornermarkers()
   >>>>                 xvw_set_type()
   >>>>                 xvw_get_type()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvisual/GraphicsP.h>

/*-----------------------------------------------------------
|
|  Routine Name: xvw_draw_sidemarkers
|
|       Purpose: Draws side markers on an annotation;
|		 utility routine used by RefreshSelection()
|		 method of various annotations (circle, rectangle..)
|         Input: widget - widget on which to draw side markers
|		 gc     - GC of manager class of widget
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/
void xvw_draw_sidemarkers(
    Widget widget,
    GC     gc)
{
        Window    window;
        Display   *display;
        Position  xpos, ypos;
        int       buffer;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

	/* 
	 *   buffer of one pixel between the visual object and the
	 *   side markers.
	 */
	buffer = 1;

        /*
         *  Draw the side markers
         */

        /*- top side marker -*/
        xpos = widget->core.x + widget->core.width/2 - LENGTH/2;
        ypos = widget->core.y - FAT - buffer;
        XFillRectangle(display, window, gc,
                       xpos, ypos, LENGTH, FAT);

        /*- bottom side marker -*/
        xpos = widget->core.x + widget->core.width/2  - LENGTH/2;
        ypos = widget->core.y + widget->core.height + 2*buffer;
        XFillRectangle(display, window, gc,
                       xpos, ypos, LENGTH, FAT);

        /*- left side marker -*/
        xpos = widget->core.x - FAT - buffer;
        ypos = widget->core.y + widget->core.height/2 - LENGTH/2;
        XFillRectangle(display, window, gc,
                       xpos, ypos, FAT, LENGTH);

        /*- right side marker -*/
        xpos = widget->core.x + widget->core.width + 2*buffer;
        ypos = widget->core.y + widget->core.height/2 - LENGTH/2;
        XFillRectangle(display, window, gc,
                       xpos, ypos, FAT, LENGTH);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_erase_sidemarkers
|
|       Purpose: Erases side markers on an annotation;
|                utility routine used by EraseSelection()
|                method of various annotations (circle, rectangle..)
|         Input: widget - widget on which to draw side markers
|		 gc     - GC of manager class of widget
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/
void xvw_erase_sidemarkers(
    Widget widget)
{
        Window    window;
        Display   *display;
        Position  xpos, ypos;
        int       buffer;

        display = XtDisplay(widget);
        window  = XtWindowOfObject(widget);

	/* 
	 *   buffer of one pixel between the visual object and the
	 *   corner markers.
	 */
	buffer = 1;

        /*
         *  Erase the side markers
         */

        /*- top side marker -*/
        xpos = widget->core.x + widget->core.width/2 - LENGTH/2;
        ypos = widget->core.y - FAT - buffer;
        XClearArea(display, window, xpos, ypos, LENGTH, FAT, TRUE);

        /*- bottom side marker -*/
        xpos = widget->core.x + widget->core.width/2  - LENGTH/2;
        ypos = widget->core.y + widget->core.height + 2*buffer;
        XClearArea(display, window, xpos, ypos, LENGTH, FAT, TRUE);

        /*- left side marker -*/
        xpos = widget->core.x - FAT - buffer;
        ypos = widget->core.y + widget->core.height/2 - LENGTH/2;
        XClearArea(display, window, xpos, ypos, FAT, LENGTH, TRUE);

        /*- right side marker -*/
        xpos = widget->core.x + widget->core.width + 2*buffer;
        ypos = widget->core.y + widget->core.height/2 - LENGTH/2;
        XClearArea(display, window, xpos, ypos, FAT, LENGTH, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_pick_sidemarkers
|
|       Purpose: Determines whether or not a visual object was picked
|		 on one of its sidemarkers.  Utility routine used by 
|		 PickGadget() method of various annotations (circle, 
|		 rectangle..)
|
|         Input: widget - widget on which to see if sidemarker was picked
|		 x, y   - point at which widget was picked
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/
int xvw_pick_sidemarkers(
    Widget widget,
    int    x,
    int    y)
{
	Position xpos, ypos;

        /*-- point on top --*/
        xpos = widget->core.x + widget->core.width/2 - x;
        ypos = widget->core.y - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on bottom --*/
        xpos = widget->core.x + widget->core.width/2  - x;
        ypos = widget->core.y + widget->core.height - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on left --*/
        xpos = widget->core.x - x;
        ypos = widget->core.y + widget->core.height/2 - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on right --*/
        xpos = widget->core.x + widget->core.width  - x;
        ypos = widget->core.y + widget->core.height/2 - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

	return(FALSE);
}


/*-----------------------------------------------------------
|
|  Routine Name: xvw_pick_cornermarkers
|
|       Purpose: Determines whether or not a gadget was picked
|                on one of its corner markers.  Utility routine used by 
|                PickGadget() method of various annotations (circle, 
|                rectangle..)
|
|         Input: widget - widget on which to see if corner marker was picked
|		 x, y   - point at which widget was picked
|
|    Written By: Danielle Argiro & Steve Kubica
|          Date: Jan 5, 1996
| Modifications:
|
------------------------------------------------------------*/
int xvw_pick_cornermarkers(
    Widget widget,
    int    x,
    int    y)
{
	Position xpos, ypos;

	/*-- point on upper left corner --*/
        xpos = widget->core.x - x;
        ypos = widget->core.y - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on upper right corner --*/
        xpos = widget->core.x + widget->core.width  - x;
        ypos = widget->core.y - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on lower right corner --*/
        xpos = widget->core.x + widget->core.width  - x;
        ypos = widget->core.y + widget->core.height - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);

        /*-- point on lower left corner --*/
        xpos = widget->core.x - x;
        ypos = widget->core.y + widget->core.height - y;
        if (kabs(xpos) <= PICK_TOLERANCE && kabs(ypos) <= PICK_TOLERANCE)
           return(TRUE);
	
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_set_type - set the update and reload type
|
|       Purpose: 
|         Input: 
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Nov 03, 1993
| Modifications:
|
------------------------------------------------------------*/

int xvw_set_type(
  kobject object,
  int     reload_type,
  int     update_type)
{
	if (!object)
	   return(FALSE);

	/*
	 *  Set the reload type...
	 */
	if (!kdms_query_attribute(object, NULL, KRELOAD_TYPE, NULL, NULL,
				    NULL, NULL))
	{
	   kdms_create_attribute(object, NULL, KRELOAD_TYPE, 1, 1, 
				 (int) KINT, FALSE, FALSE);
	   kpds_set_attribute(object, KRELOAD_TYPE, KNONE);
	}

	if (reload_type != -1)
	   kpds_set_attribute(object, KRELOAD_TYPE, reload_type);


	/*
	 *  Get the reload type...
	 */
	if (!kdms_query_attribute(object, NULL, KUPDATE_TYPE, NULL, NULL,
				    NULL, NULL))
	{
	   kdms_create_attribute(object, NULL, KUPDATE_TYPE, 1, 1, 
				 (int) KINT, FALSE, FALSE);
	   kpds_set_attribute(object, KUPDATE_TYPE, KNONE);
	}

	if (update_type != -1)
	   kpds_set_attribute(object, KUPDATE_TYPE, update_type);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_get_type - get the update and reload type
|
|       Purpose: 
|         Input: 
|        Output: 
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|
|    Written By: Mark Young
|          Date: Nov 03, 1993
| Modifications:
|
------------------------------------------------------------*/

int xvw_get_type(
  kobject object,
  int     *reload_type,
  int     *update_type)
{
	if (!object)
	   return(FALSE);

	/*
	 *  Get the reload type...
	 */
	if  (!kdms_query_attribute(object, NULL, KRELOAD_TYPE, NULL, NULL,
				    NULL, NULL))
	{
	   kdms_create_attribute(object, NULL, KRELOAD_TYPE, 1, 1, 
				 (int) KINT, FALSE, FALSE);
	   kpds_set_attribute(object, KRELOAD_TYPE, KNONE);
	}

	if (reload_type)
	{
	   *reload_type = KNONE;
	   kpds_get_attribute(object, KRELOAD_TYPE, reload_type);
	}


	/*
	 *  Get the update type...
	 */
	if (!kdms_query_attribute(object, NULL, KUPDATE_TYPE, NULL, NULL,
				    NULL, NULL))
	{
	   kdms_create_attribute(object, NULL, KUPDATE_TYPE, 1, 1, 
				 (int) KINT, FALSE, FALSE);
	   kpds_set_attribute(object, KUPDATE_TYPE, KNONE);
	}

	if (update_type)
	{
	   *update_type = KNONE;
	   kpds_get_attribute(object, KUPDATE_TYPE, update_type);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvw_is_rgb() - determine if data object is RGB
|
|       Purpose: Determines if input data object is RGB
|         Input: object - data object in question
|        Output:
|       Returns: TRUE (1) if data object is RGB, FALSE (0) if not
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Mar 26, 1996
| Modifications:
|
------------------------------------------------------------*/
int xvw_is_rgb(
    kobject object)
{
	int e, has_alpha, colorspace;

	if (!(kcolor_get_attributes(object,
                KCOLOR_HAS_ALPHA, &has_alpha,
                KCOLOR_COLORSPACE, &colorspace,
                NULL))) return(FALSE);

	if (!(kpds_get_attributes(object,
                KPDS_VALUE_SIZE, NULL, NULL, NULL, NULL, &e,
                NULL))) return(FALSE);

	if (colorspace == KRGB && (e == 3 || (e == 4 && has_alpha)))
	    return(TRUE);
	else return(FALSE);
}
