 /*
  * Khoros: $Id: ExtPort.c,v 1.30 1996/07/04 18:33:04 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: ExtPort.c,v 1.30 1996/07/04 18:33:04 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros External Port Widget
   >>>>  Private:
   >>>>   Static:
   >>>> 		ClassInitialize()
   >>>> 		Initialize()
   >>>> 		SetValues()
   >>>> 		Destroy()
   >>>> 		ExtPortCheck()
   >>>> 		ExtPortExec()
   >>>> 		ExtPortSave()
   >>>> 		ExtPortRestore()
   >>>> 		InputDavCallback()
   >>>>
   >>>>   Public:
   >>>>			xvw_create_extport()
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/ExtPortP.h>

static void ClassInitialize PROTO((void));
static void Initialize	   PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean  SetValues  PROTO((Widget, Widget, Widget, ArgList, Cardinal *));
static void Destroy	   PROTO((Widget));
static int  ExtPortCheck   PROTO((Widget));
static void ExtPortExec    PROTO((Widget, int));
static void ExtPortSave    PROTO((xvobject, kfile *, int));
static int  ExtPortRestore PROTO((xvobject, xvobject, kfile *, char *, int,
				  int, int *));

static void InputDavCallback PROTO((xvobject, kaddr, kaddr));

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/


static xvattribute attributes[] = {
{XVW_EXTPORT_SELECTION,	  NULL,	  XtRPointer,  NULL},
{XVW_EXTPORT_PORT_OBJECT, NULL,	  XtRObject,  NULL},
};

/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwExtPortWidgetRec, extport.field)

static XtResource resources[] = { 
{XVW_EXTPORT_SELECTION, NULL, XtRPointer, sizeof(XtPointer),
        offset(selection), XtRImmediate, (XtPointer) NULL},
{XVW_EXTPORT_PORT_OBJECT, NULL, XtRObject, sizeof(xvobject),
        offset(port), XtRImmediate, (XtPointer) NULL},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for External Port Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwNodeWidgetClassRec)

XvwExtPortWidgetClassRec xvwExtPortWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "ExtPort",				/* class_name		  */
    sizeof(XvwExtPortWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwExtPortWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    ExtPortCheck,	                    /* firing rule check  */
    ExtPortExec, 	                    /* execute node       */
    ExtPortSave, 	                    /* save node          */
    ExtPortRestore, 	                    /* restore node       */
    NULL, 		                    /* build command      */
    NULL,  		                    /* extension	  */
  },  /* XvwNodeWidgetClass fields initialization */
  {
    NULL,  		                    /* extension	  */
  },  /* XvwExtPortWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwExtPortWidgetClass = (WidgetClass) &xvwExtPortWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)	     (XvwExtPortWidget) (widget)
#define kwidgetclass(widget) (XvwExtPortWidgetClass) (widget)->core.widget_class


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwExtPortWidget class has been created.
|                This is where the attributes for the class are initialized. 
|
|    Written By: Mark Young
|          Date: July 31, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwExtPortWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$GUIVIZ/objects/library/xvlang/app-defaults/ExtPort");
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial node
|                widget instance.
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: July 31, 1995 
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwExtPortWidget xwid = kwidget(new);
	xvobject object = xvw_object(new);

        xwid->extport.ifeed = xvw_create_port(object, "IfeedPort");
	xvw_set_attributes(xwid->extport.ifeed,
		XVW_PORT_TYPE,		    KPORT_DATAFLOW,
		XVW_PORT_DIRECTION,	    KPORT_INPUT,
		XVW_ABOVE,		    NULL,
		XVW_BELOW,		    NULL,
		XVW_RIGHT_OF,		    NULL,
		XVW_PORT_CONNECTION_PARENT, xvw_parent(object),
		NULL);
	xvw_unmap(xwid->extport.ifeed);

        xwid->extport.port = xvw_create_port(object, "ExtPort");
	xvw_set_attributes(xwid->extport.port,
		XVW_PORT_TYPE,		    KPORT_DATAFLOW,
		XVW_ABOVE,		    NULL,
		XVW_BELOW,		    NULL,
		XVW_RIGHT_OF,		    xwid->extport.ifeed,
		XVW_PORT_CONNECTION_PARENT, xvw_parent(object),
		NULL);
        xvw_set_attributes(xwid->node.icon,
                XVW_TACK_EDGE, KMANAGER_TACK_ALL,
		XVW_BELOW,    NULL,
		XVW_ABOVE,    NULL,
		XVW_LEFT_OF,  NULL,
		XVW_RIGHT_OF, NULL,
                NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwExtPortWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the node.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: July 31, 1995 
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num)
{
	XvwExtPortWidget cxwid = kwidget(current);
	XvwExtPortWidget nxwid = kwidget(new);

	int direction;

	if (cxwid->node.modified != nxwid->node.modified)
	{
	   xvw_get_attribute(nxwid->extport.port,XVW_PORT_DIRECTION,&direction);
	   if (direction == KPORT_OUTPUT && nxwid->node.modified == TRUE)
	      xvw_set_attribute(nxwid->extport.port, XVW_PORT_DAV, FALSE);
	   else if (direction == KPORT_INPUT && nxwid->node.modified == FALSE)
	      xvw_set_attribute(nxwid->extport.port, XVW_PORT_MODIFIED, FALSE);
	}

	if (cxwid->extport.selection != nxwid->extport.selection)
	{
	   xvobject right = NULL, left = NULL;

	   if (nxwid->extport.selection == NULL)
	      direction = KPORT_UNKNOWN;
	   else if (nxwid->extport.selection->Selptr->type == KUIS_OUTPUTFILE)
	      direction = KPORT_INPUT,  right  = (xvobject) KMANAGER_UNDEFINED;
	   else if (nxwid->extport.selection->Selptr->type == KUIS_INPUTFILE)
	      direction = KPORT_OUTPUT, left  = (xvobject) KMANAGER_UNDEFINED;
	   else
	      direction = KPORT_UNKNOWN;

	   if (direction == KPORT_INPUT)
	   {
	      xvw_map(nxwid->extport.ifeed);
	      xvw_insert_callback(nxwid->extport.port, XVW_PORT_DAV_CALLBACK,
			FALSE, InputDavCallback, xvw_object(new));
	      nxwid->node.iports = karray_add(nxwid->node.iports, KSTRUCT,
			nxwid->extport.port, nxwid->node.isize++);
	      xvw_set_attributes(nxwid->extport.port,
		XVW_PORT_DIRECTION, direction,
		XVW_LEFT_OF, right,
		XVW_RIGHT_OF, left,
		NULL);
	   }
	   else if (direction == KPORT_OUTPUT)
	   {
	      nxwid->node.oports = karray_add(nxwid->node.oports, KSTRUCT,
			nxwid->extport.port, nxwid->node.osize++);
	      xvw_set_attributes(nxwid->extport.port,
		XVW_PORT_DIRECTION, direction,
		XVW_LEFT_OF, right,
		XVW_RIGHT_OF, left,
		NULL);
	   }
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the extport...
|
|         Input: widget - the extport is being destroyed 
|    Written By: Mark Young
|          Date: Aug 12, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwExtPortWidget xwid = kwidget(widget);
	Widget parent = XtParent(widget);

	if (!parent->core.being_destroyed && xwid->extport.selection)
	   xvf_set_attribute(xwid->extport.selection, XVF_DELETE, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ExtPortCheck
|
|       Purpose: This routine will check to see if a extport is ready to run.
|
|         Input: widget      - the extport widget to be checked
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int ExtPortCheck(
   Widget   widget)
{
        XvwExtPortWidget xwid = (XvwExtPortWidget) widget;

	char *filename;
	int direction, test, selected;

	if (xwid->extport.port == NULL)
	   return(FALSE);

	xvf_get_attribute(xwid->extport.selection, XVF_FILE_NAME, &filename);
	if (kstring_cleanup(filename, filename) == NULL)
	{
	   kfree(filename);
	   return(FALSE);
	}
	kfree(filename);

	xvw_get_attributes(xwid->extport.port,
			XVW_PORT_DAV,       &test,
	      		XVW_PORT_SELECTED,  &selected,
			XVW_PORT_DIRECTION, &direction,
			XVW_PORT_FILENAME,  &filename,
			NULL);

	if (filename == NULL)
	   return(FALSE);
	else if (direction == KPORT_OUTPUT)
	   return(TRUE);
	else if (direction == KPORT_INPUT && test == TRUE && selected == TRUE)
	   return(TRUE);
	else
	   return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ExtPortExec
|
|       Purpose: This routine will run or stop the execution of an extport .
|
|         Input: widget      - the extport widget to be run
|                exec        - whether we wish to run or stop the extport
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ExtPortExec(
   Widget   widget,
   int      exec)
{
	int  direction;
	char *filename, *portfile;
        XvwExtPortWidget xwid = (XvwExtPortWidget) widget;

	if (exec == FALSE)
	{
	   xvl_stopcopy(xvw_object(widget));
	   return;
	}

	xvw_get_attributes(xwid->extport.port,
		XVW_PORT_DIRECTION, &direction,
		XVW_PORT_FILENAME,  &portfile,
		NULL);
	xvf_get_attribute(xwid->extport.selection, XVF_FILE_NAME, &filename);
	if (filename == NULL || portfile == NULL)
	{
	   kfree(filename);
	   return;
	}

	if (direction == KPORT_OUTPUT)
	{
	   xvl_copydata(xvw_object(widget), filename, portfile,
		xwid->extport.port);
	}
	else if (direction == KPORT_INPUT)
	   xvl_copydata(xvw_object(widget), portfile, filename, NULL);

	kfree(filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: ExtPortSave - Save an external port object
|
|       Purpose: This routine does the actually saving of an external port
|		 object to the workspace file.
|
|         Input: extport - the external port object to be saved
|                file  - the file the information is suppose to be written to
|		 ident   - the extport identifier
|    Written By: Mark Young
|          Date: Aug 8, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ExtPortSave(
    xvobject extport,
    kfile    *file,
    int      ident)
{
	int   xpos, ypos;
	xvobject procedure;
	kform_struct *selection = NULL;
	char  *name, *direction = NULL, *varname = NULL;

        /*
         *  create I/O ports on procedure glyph corresponding to
         *  the external connection that we just created.
         */
        procedure = ProcedureLocate(XtParent(xvw_widget(extport)));
        if (procedure != NULL)
            GlyphUpdateForm(xvw_widget(procedure));

	xvw_get_attributes(extport,
		XVW_EXTPORT_SELECTION, &selection,
		XVW_NODE_NAME, &name,
		XVW_XPOSITION, &xpos,
		XVW_YPOSITION, &ypos,
		NULL);

	if (selection != NULL)
	{
	   xvf_get_attribute(selection, XVF_VARIABLE, &varname);
	   if (selection->Selptr->type == KUIS_INPUTFILE)
	      direction = "in";
	   else if (selection->Selptr->type == KUIS_OUTPUTFILE)
	      direction = "out";
	}
	name      = !name ? "(unknown)" : name;
	varname   = !varname ? "" : varname;
	direction = !direction ? "(unknown)" : direction;

	/* save the external port information */
	kfprintf(file, "\n#  ExtPort '%s'\nExtPort:%s:%s:%s:%d:%d:%d:\n",
		name, direction, name, varname, ident, xpos, ypos);
}

/*-----------------------------------------------------------
|
|  Routine Name: ExtPortRestore - Restore the external port
|
|       Purpose: This routine does the actually restoring of a external port
|		 from a workspace file.
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|        Output: ident - the node indentifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ExtPortRestore(
    xvobject extport,
    xvobject parent,
    kfile    *file,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    int      *ident)
{
	kform *form = NULL;
	kform_struct *selection = NULL;
        xvobject procedure;

	int      xpos, ypos;
	char     name[KLENGTH], varname[KLENGTH], type[KLENGTH], *var;

	if (ksscanf(line, "ExtPort:%[^:]:%[^:]:%[^:]:%d:%d:%d:", type,
                    name, varname, ident, &xpos, &ypos) != 6)
        {
           return(FALSE);
        }
	var = kstring_cleanup(varname, varname);

	xvw_get_attribute(parent, XVW_WORKSPACE_WKSPGUI, &form);
	if (kstrcmp(type, "in") == 0)
	   selection = xvl_create_selection(form, KUIS_INPUTFILE, var);
	else if (kstrcmp(type, "out") == 0)
	   selection = xvl_create_selection(form, KUIS_OUTPUTFILE, var);

	xvw_set_attributes(extport,
		XVW_XPOSITION, xpos,
		XVW_YPOSITION, ypos,
		XVW_NODE_NAME, name,
		XVW_EXTPORT_SELECTION, selection,
		NULL);

        /*
         *  create I/O ports on procedure glyph corresponding to
         *  the external connection that we just created.
         */
        procedure = ProcedureLocate(xvw_widget(parent));
        if (procedure != NULL)
            GlyphUpdateForm(xvw_widget(procedure));

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: InputDavCallback - dav change on our input detected
|
|       Purpose: This routine is used to detect when a change to a extport's
|		 input data has ocurred. 
|
|         Input: extport - extport to be used to change to be sensitive
|    Written By: Mark Young
|          Date: Mar 27, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void InputDavCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvobject extport = (xvobject) client_data;
	XvwExtPortWidget xwid = (XvwExtPortWidget) xvw_widget(extport);

	xwid->node.modified = FALSE;
	xvw_set_attribute(extport, XVW_NODE_MODIFIED, TRUE);
}

/************************************************************
*
*  Routine Name: xvw_create_extport - create an external port object
*
*       Purpose:
*		
*         Input: parent - parent of the external port object; NULL will cause
*                         a default toplevel to be created automatically
*                name   - name with which to reference external port object
*        Output: 
*       Returns: The external port object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: July 31, 1995
*      Verified: 
*  Side Effects: 
* Modifications:
*
*******************************************************************/

xvobject xvw_create_extport(
   xvobject parent,
   char     *name)
{
	xvobject extport;


	extport = xvw_create(parent, FALSE, TRUE, name, ExtPortWidgetClass);
	xvw_set_attributes(extport,
		XVW_MENUABLE,  FALSE,
		XVW_RESIZABLE, FALSE,
		NULL);
	return(extport);
}
