 /*
  * Khoros: $Id: Glyph.c,v 1.54.2.1 1996/12/06 20:29:51 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Glyph.c,v 1.54.2.1 1996/12/06 20:29:51 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Glyph Widget
   >>>>  Private:
   >>>> 		MessageCallback()
   >>>>   Static:
   >>>> 		ClassInitialize()
   >>>> 		Initialize()
   >>>> 		SetValues()
   >>>> 		Destroy()
   >>>> 		GlyphCheck()
   >>>> 		GlyphBuild()
   >>>> 		GlyphExec()
   >>>> 		GlyphSave()
   >>>> 		GlyphRestore()
   >>>> 		DispatchCallback()
   >>>>   Public:
   >>>>			xvw_create_glyph()
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/GlyphP.h>
#include <xvlang/PortP.h>
#include <kutils/ksignal.h>
#include <sys/times.h>

static void ClassInitialize PROTO((void));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues    PROTO((Widget, Widget, Widget,ArgList,Cardinal *));
static void    Destroy	    PROTO((Widget));
static int     GlyphCheck   PROTO((Widget));
static void    GlyphExec    PROTO((Widget, int));
static void    GlyphSave    PROTO((xvobject, kfile *, int));
static int     GlyphBuild   PROTO((Widget, char *));
static int GlyphRestore PROTO((xvobject,xvobject,kfile *,char *,int,int,int *));

static void DispatchCallback PROTO((int, kaddr));
static void NodeRunCallback  PROTO((xvobject, kaddr, kaddr));

static void GlyphStartTime PROTO((Widget));
static void GlyphEndTime   PROTO((Widget));
static void GlyphPrintTime PROTO((Widget));


/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/

#define ERROR_PIXMAP	  "$DESIGN/objects/library/xvlang/misc/glyph/error.xpm"
#define INFO_PIXMAP	  "$DESIGN/objects/library/xvlang/misc/glyph/info.xpm"
#define OPENPANE_PIXMAP	  "$DESIGN/objects/library/xvlang/misc/glyph/pane_open.xpm"
#define CLOSEPANE_PIXMAP  "$DESIGN/objects/library/xvlang/misc/glyph/pane_close.xpm"

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_GLYPH_FORM,         NULL, XtRPointer,  NULL},
{XVW_GLYPH_WKSPGUI,      NULL, XtRPointer,  NULL},
{XVW_GLYPH_TBNAME,       NULL, XtRString,   NULL},
{XVW_GLYPH_ONAME,        NULL, XtRString,   NULL},
{XVW_GLYPH_ECHO,	 NULL, XtRInt,      NULL},
{XVW_GLYPH_REPORTING,	 NULL, XtRInt,      XtRBoolean},
{XVW_GLYPH_CONNECTION_PARENT, NULL, XtRObject, XtRWidget},
{XVW_GLYPH_EXPRESSION_ID, NULL, XtRPointer, NULL},

{XVW_GLYPH_INFO_PIXMAP, NULL, XtRPixmap, NULL},
{XVW_GLYPH_ERROR_PIXMAP, NULL, XtRPixmap, NULL},
{XVW_GLYPH_OPENPANE_PIXMAP, NULL, XtRPixmap, NULL},
{XVW_GLYPH_CLOSEPANE_PIXMAP, NULL, XtRPixmap, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwGlyphWidgetRec, glyph.field)

static XtResource resources[] = { 
{XVW_GLYPH_FORM, NULL, XtRPointer, sizeof(XtPointer),
        offset(form), XtRImmediate, (XtPointer) NULL},
{XVW_GLYPH_TBNAME, NULL, XtRString, sizeof(String),
        offset(tbname), XtRImmediate, (XtPointer) NULL},
{XVW_GLYPH_ONAME, NULL, XtRString, sizeof(String),
        offset(oname), XtRImmediate, (XtPointer) NULL},
{XVW_GLYPH_ECHO, NULL, XtRInt, sizeof(int),
        offset(echo), XtRImmediate, (XtPointer) KGLYPH_ECHO_TO_CONSOLE},
{XVW_GLYPH_REPORTING, NULL, XtRBoolean, sizeof(Boolean),
        offset(reporting), XtRImmediate, (XtPointer) TRUE},
{XVW_GLYPH_CONNECTION_PARENT, NULL, XtRWidget, sizeof(Widget),
        offset(connection_parent), XtRImmediate, (XtPointer) NULL},
{XVW_GLYPH_EXPRESSION_ID, NULL, XtRPointer, sizeof(XtPointer),
        offset(expression_id), XtRImmediate, (XtPointer) 0},

{XVW_GLYPH_OPENPANE_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(openpane_pixmap), XtRString, (XtPointer) OPENPANE_PIXMAP},
{XVW_GLYPH_CLOSEPANE_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(closepane_pixmap), XtRString, (XtPointer) CLOSEPANE_PIXMAP},
{XVW_GLYPH_ERROR_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(error_pixmap), XtRString, (XtPointer) ERROR_PIXMAP},
{XVW_GLYPH_INFO_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(info_pixmap), XtRString, (XtPointer) INFO_PIXMAP},

{XVW_CURSOR, XVW_CURSORNAME, XtRCursor, sizeof(Cursor),
      XtOffsetOf(XvwGlyphWidgetRec, manager.cursor), XtRString,
      (XtPointer) "hand1"},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Glyph Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwNodeWidgetClassRec)

XvwGlyphWidgetClassRec xvwGlyphWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Glyph",				/* class_name		  */
    sizeof(XvwGlyphWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwGlyphWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selection proc */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    GlyphCheck,		                    /* firing rule check  */
    GlyphExec, 		                    /* execute node       */
    GlyphSave, 		                    /* save node          */
    GlyphRestore, 	                    /* restore node       */
    GlyphBuild, 	                    /* build command      */
    NULL, 		                    /* extension          */
  },  /* XvwNodeWidgetClass fields initialization */
  {
    NULL,                   	    	    /* extension  */
  },  /* XvwGlyphWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwGlyphWidgetClass = (WidgetClass) &xvwGlyphWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)	    (XvwGlyphWidget) (widget)
#define kwidgetclass(widget)(XvwGlyphWidgetClass) (widget)->core.widget_class


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwGlyphWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwGlyphWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvlang/uis/Glyph.pane");
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/Glyph");

	xvw_define_attributes(xvwGlyphWidgetClass,
	    XVW_GLYPH_FORMFILE, XtRString, GlyphSetFormfile, GlyphGetFormfile,
	    NULL);

	kvf_set_expr_handler(GlyphFormExpression);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial glyph
|                widget instance.
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: May 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	unsigned long mask;
        XvwGlyphWidget xwid = kwidget(new);

	xvobject connection_parent;
	xvobject glyph  = xvw_object(new);

	xwid->glyph.num_runs      = 0;
	xwid->glyph.start_time    = 0;
	xwid->glyph.end_time      = 0;
	xwid->glyph.dbxpid        = 0;
	xwid->glyph.rsize         = 0;
	xwid->glyph.rsels         = NULL;
	xwid->glyph.isels         = NULL;
	xwid->glyph.osels         = NULL;
	xwid->glyph.help          = NULL;
	xwid->glyph.cli           = NULL;
	xwid->glyph.panefile      = NULL;
	xwid->glyph.error         = NULL;
	xwid->glyph.export_button = NULL;
	xwid->glyph.last_iport    = NULL;
	xwid->glyph.debug         = FALSE;
	xwid->glyph.runtime       = FALSE;
	xwid->glyph.created_form  = FALSE;
	xwid->glyph.command	  = NULL;
	xwid->glyph.options_initialized = FALSE;
	xwid->glyph.tbname        = NULL;
	xwid->glyph.oname         = NULL;
	xwid->glyph.panefile      = NULL;
	xwid->glyph.command       = NULL;

	xwid->glyph.menuform  = xvw_create_pixmap(glyph, "menuform");
	xwid->glyph.cflow_in  = xvw_create_port(glyph, "Control");
	xwid->glyph.cflow_out = xvw_create_port(glyph, "Control");

	if (xwid->glyph.connection_parent == NULL)
	   xwid->glyph.connection_parent = XtParent(new);

	connection_parent = xvw_object(xwid->glyph.connection_parent);
	xvw_set_attributes(xwid->glyph.menuform,
		XVW_PIXMAP,	  xwid->glyph.openpane_pixmap,
		XVW_BELOW,	  NULL,
		XVW_RIGHT_OF,     NULL,
		NULL);
	xvw_set_attributes(xwid->glyph.cflow_in,
		XVW_WIDTH,	      8,
		XVW_BORDER_WIDTH,     1,
		XVW_SHADOW_THICKNESS, 1,
		XVW_SHADOW_TYPE,    KSHADOW_OUT,
		XVW_RIGHT_OF,	    NULL,
		XVW_BELOW,	    xwid->glyph.menuform,
		XVW_PORT_DIRECTION, KPORT_INPUT,
		XVW_PORT_TYPE,      KPORT_CONTROLFLOW,
		XVW_PORT_CONNECTION_PARENT, connection_parent,
		NULL);
	xvw_set_attributes(xwid->glyph.cflow_out,
		XVW_WIDTH,	      8,
		XVW_BORDER_WIDTH,     1,
		XVW_SHADOW_THICKNESS, 1,
		XVW_SHADOW_TYPE,    KSHADOW_OUT,
		XVW_RIGHT_OF,	    xwid->node.icon,
		XVW_BELOW,	    xwid->glyph.menuform,
		XVW_PORT_DIRECTION, KPORT_OUTPUT,
		XVW_PORT_TYPE,      KPORT_CONTROLFLOW,
		XVW_PORT_CONNECTION_PARENT, connection_parent,
		NULL);
	xvw_set_attributes(xwid->node.icon,
		XVW_BELOW,	  xwid->glyph.menuform,
		XVW_RIGHT_OF,     xwid->glyph.cflow_in,
		XVW_ABOVE,	  KMANAGER_UNDEFINED,
		XVW_LEFT_OF,      KMANAGER_UNDEFINED,
		XVW_SHADOW_TYPE,  KSHADOW_IN,
		XVW_FORCE_REDISPLAY, TRUE,
		XVW_BORDER_WIDTH, 1,
		XVW_WIDTH,        18,
		XVW_HEIGHT,       18,
		NULL);
	xvw_insert_callback(glyph, XVW_NODE_RUN_CALLBACK, FALSE,
		NodeRunCallback, xwid);
	xvw_insert_callback(xwid->glyph.cflow_in, XVW_PORT_DAV_CALLBACK,
		FALSE, GlyphInputDavCallback, glyph);

	mask = ButtonPressMask | ButtonReleaseMask |
	       EnterWindowMask | LeaveWindowMask;
	xvw_insert_event(xwid->glyph.menuform, mask, GlyphHandler,
		glyph, KLIST_HEAD);
	xvw_insert_event(xwid->node.icon, mask, GlyphHandler,
		glyph, KLIST_HEAD);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwGlyphWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the glyph.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num)
{
	XvwGlyphWidget cxwid = kwidget(current);
	XvwGlyphWidget nxwid = kwidget(new);

	kguide   *guide;

	/*
	 *  Set the form associated with the glyph:
         *  1) create the I/O ports on the glyph according to selected pane
         *  2) set the name of the glyph according to the selected pane
         *  3) if there is > 1 pane, install callbacks to update the glyph
         *     name and I/O ports when the pane is changed.
	 */
	if (cxwid->glyph.form != nxwid->glyph.form)
	{
	   /* 
	    * update the glyph label and the I/O ports according
            * to the currently selected pane
	    */
	   GlyphInitForm(new);

	   /* 
	    * more than one pane? add callbacks to each guide button that
            * will update the glyph label & I/O ports when pane is changed
            */
	   if ((guide=kvf_search_sel_guide(nxwid->glyph.form->subform)) != NULL)
	   {
	      while (guide != NULL)
	      {
		 xvf_add_extra_call(guide->back_kformstruct, GlyphInitForm,
				       (kaddr) nxwid, XVF_CALL_LAST);
		 guide = guide->next;
	      }
	   }
	   nxwid->glyph.created_form = FALSE;

	   if (cxwid->glyph.form != NULL && cxwid->glyph.created_form)
	      xvf_destroy_form(cxwid->glyph.form);
	}

	if (cxwid->glyph.tbname != nxwid->glyph.tbname)
	{
	   kfree_and_NULL(cxwid->glyph.tbname);
	   nxwid->glyph.tbname = kstrdup(nxwid->glyph.tbname);
	}

	if (cxwid->glyph.oname != nxwid->glyph.oname)
	{
	   kfree_and_NULL(cxwid->glyph.oname);
	   nxwid->glyph.oname = kstrdup(nxwid->glyph.oname);
	}

	if (cxwid->node.modified != nxwid->node.modified)
	{
	   if (nxwid->node.modified == TRUE)
	      xvw_set_attribute(nxwid->glyph.cflow_out, XVW_PORT_DAV, FALSE);
	   else
	      xvw_set_attribute(nxwid->glyph.cflow_out, XVW_PORT_DAV, TRUE);
	}

	if (cxwid->node.nodesize != nxwid->node.nodesize)
	{
	   int size;

	   if (nxwid->node.nodesize == KNODE_SIZE_LARGE)
	      size = 18;
	   else if (nxwid->node.nodesize == KNODE_SIZE_MEDIUM)
	      size = 16;
	   else if (nxwid->node.nodesize == KNODE_SIZE_SMALL)
	      size = 14;

	   xvw_set_attributes(nxwid->node.icon,
		XVW_WIDTH,  size,
		XVW_HEIGHT, size,
		NULL);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the glyph...
|
|         Input: widget - the glyph widget being destroyed 
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwGlyphWidget xwid   = kwidget(widget);
	Widget         parent =  XtParent(widget);

	int        i;
	kselection *selection;

	/*
	 *  Now destroy any associated objects...
	 */
	if (xwid->glyph.help)
	   xvw_destroy(xwid->glyph.help);

	if (xwid->node.pid > 0)
	   xvw_remove_handler(xwid->node.pid);

	if (!parent->core.being_destroyed && xwid->glyph.error)
	   xvw_destroy(xwid->glyph.error);

	/*
	 *  Delete any extensions associated with the glyph's gui...
	 */
	if (xwid->glyph.form != NULL)
	{
	   selection = xwid->glyph.form->subform->guide->pane->sel_list;
	   while (selection != NULL)
	   {
	      if (selection->extensions != NULL)
              {
                 for (i = 0; i < selection->extension_num; i++)
                 {
	           xvf_set_attribute(selection->extensions[i]->back_kformstruct,
				      XVF_DELETE, TRUE);
		 }
	      }
	      selection = selection->next;
	   }
	   if (xwid->glyph.created_form) xvf_destroy_form(xwid->glyph.form);
	}

	/*
	 *  Free memory...  We do what we can to conserve
	 */
	kfree_and_NULL(xwid->glyph.isels);
	kfree_and_NULL(xwid->glyph.osels);
	kfree_and_NULL(xwid->glyph.rsels);
	kfree_and_NULL(xwid->glyph.panefile);
	kfree_and_NULL(xwid->glyph.tbname);
	kfree_and_NULL(xwid->glyph.oname);
	kfree_and_NULL(xwid->glyph.command);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphCheck
|
|       Purpose: This routine will check to see if a glyph is ready to run.
|         Input: widget      - the glyph widget to be checked
|
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int GlyphCheck(
   Widget   widget)
{
        XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	int i, selected = FALSE, test = FALSE;

	for (i = 0; i < xwid->node.isize; i++)
	{
	   xvw_get_attributes(xwid->node.iports[i],
			XVW_PORT_DAV, &test,
			XVW_PORT_SELECTED, &selected,
			NULL);
	   if (test == FALSE && selected == TRUE)
	      return(FALSE);
	}
	xvw_get_attributes(xwid->glyph.cflow_in,
		XVW_PORT_DAV, &test,
		XVW_PORT_SELECTED, &selected,
		NULL);

	if (test == FALSE && selected == TRUE)
	   return(FALSE);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphExec
|
|       Purpose: This routine will run or stop the execution of an glyph .
|         Input: widget      - the glyph widget to be run
|                exec        - whether we wish to run or stop the glyph
|
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void GlyphExec(
   Widget   widget,
   int      exec)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	int   i;
	pid_t pid;
	kfile *file;
	char temp[10*KLENGTH], command[10*KLENGTH];
	XvwGlyphWidgetClass class = kwidgetclass(widget);

	if (exec == TRUE && xwid->node.contrun == TRUE)
	{
	   pid = xwid->node.pid;

	   if (pid != -1 &&
	       xvw_getinfo_handler(pid, NULL, NULL, NULL, NULL, &file) &&
	       class->node_class.build_command != NULL &&
	       class->node_class.build_command(widget, command) == TRUE)
	   {
	      ksprintf(temp, "%s\n", command);
	      kfputs(temp, file);
	   }
	}
	else if (exec == TRUE)
	{
	   /* Make sure that our glyph data available is set to false.  */
	   for (i = 0; i < xwid->node.osize; i++)
	      xvw_set_attribute(xwid->node.oports[i], XVW_PORT_DAV, FALSE);

	   xwid->glyph.num_runs = 0;
           xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND_COLOR, "red");
	   if (xwid->glyph.cli)
	   {
	      xvw_unmap(xwid->glyph.error);
	      xvw_set_attribute(xwid->glyph.cli,XVW_TEXTDISPLAY_CLEARTEXT,TRUE);
	   }
 
	   if (class->node_class.build_command != NULL &&
	       class->node_class.build_command(widget, command) == TRUE)
	   {
	      /* recording start time */
	      kfree(xwid->glyph.command);
	      xwid->glyph.command = kstrdup(command);
	      if (xwid->glyph.debug) kputenv("KHOROS_NOTIFY=SYSLIB");
              xwid->node.pid = xvw_fork(command, DispatchCallback,
			MessageCallback, NodeIpcCallback, widget);
	      if (xwid->glyph.debug)
	      {
		 kremenv("KHOROS_NOTIFY");
		 GlyphStartDebugger(widget, TRUE);
	      }
	   }
	}
	else if (exec == FALSE)
	{
	   /*  Update the execution time. */
	   GlyphEndTime(widget);
	   GlyphPrintTime(widget);

	   kfree_and_NULL(xwid->glyph.command);
	   if (xwid->node.pid > 0)
	   {
	      xvw_remove_handler(xwid->node.pid);
	      xwid->node.pid = 0;
	      GlyphStartDebugger(widget, FALSE);
              for (i = 0; i < xwid->node.osize; i++)
                 xvw_set_attribute(xwid->node.oports[i], XVW_PORT_DAV, FALSE);
	   }
	   xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND,
			xwid->core.background_pixel);

	   if (!xwid->node.modified)
	   {
	      int  selected, outputs = FALSE;
	      xvobject *oports = xwid->node.oports;

	      for (i = 0; i < xwid->node.osize; i++)
	      {
                 xvw_get_attribute(oports[i], XVW_PORT_SELECTED, &selected);
	         if (selected == TRUE && xwid->node.khoros_routine == FALSE)
		    xvw_set_attribute(oports[i], XVW_PORT_DAV, TRUE);

	         outputs |= selected;
	      }
	      if (!outputs) xwid->node.modified = TRUE;
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphSave - Save a glyph/conditional object
|
|       Purpose: This routine does the actually saving of a glyph/control to a
|		 workspace file.
|
|         Input: object - the glyph/control to be saved
|                file  - the file the information is suppose to be written to
|		 ident  - the glyph/control identifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void GlyphSave(
    xvobject glyph,
    kfile    *file,
    int	     ident)
{
	XvwGlyphWidget xglyph = (XvwGlyphWidget) xvw_widget(glyph);

	kform        *form;
	kselection   *selection;
        kform_struct **extensions;

	klist    *list, *tblist = NULL;
	ktoken   tbname_token, oname_token;
        int      i, num, xpos, ypos, temporary, selected; 
	char     **info, *tbname, *oname, *title, *name, *var, *filename,
		 *tmp, *type, *machine, temp[KLENGTH], command[10*KLENGTH],
		 variables[KLENGTH];

        if (xvw_check_subclass(glyph, ConditionalWidgetClass))
	   type = "Conditional";
	else
	   type = "Glyph";

	xvw_get_attributes(glyph,
		XVW_GLYPH_TBNAME, &tbname,
		XVW_GLYPH_ONAME,  &oname,
		XVW_GLYPH_FORM,   &form,
		XVW_NODE_NAME,    &title,
		XVW_XPOSITION,    &xpos,
		XVW_YPOSITION,    &ypos,
		XVW_NODE_REMOTE_LOCATION, &machine,
		NULL);

	if (form != NULL)
	   tmp = kvf_construct_partial_cmd(form, form->subform, NULL, FALSE);
	else
	   tmp = kstrdup("(unknown)");

	kstrcpy(command, tmp); kfree(tmp);
	for (i = 0; i < xglyph->node.isize; i++)
	{
	   PortGetInfo(xglyph->node.iports[i], &var, &filename, &temporary,
		       &selected, NULL, NULL, &num, NULL);

	   if (temporary == FALSE && selected && num == 0 && filename != NULL)
	   {
	      ksprintf(temp, " -%s %s", var, filename);
	      kstrcat(command, temp);
	   }
	   kfree(var);
	}

	for (i = 0; i < xglyph->node.osize; i++)
	{
	   PortGetInfo(xglyph->node.oports[i], &var, &filename, &temporary,
		       NULL, NULL, NULL, &num, NULL);

	   if (temporary == FALSE && filename != NULL)
	   {
	      ksprintf(temp, " -%s %s", var, filename);
	      kstrcat(command, temp);
	   }
	   kfree(var);
	}

	variables[0] = '\0';
	extensions = kvf_get_extensions(form, &num);
	for (i = 0; i < num; i++)
	{
	    selection = extensions[i]->Selptr->extensions[0];
	    if ((selection->type != KUIS_MUTEXCL) &&
		(selection->type != KUIS_MUTINCL) &&
		(selection->type != KUIS_GROUP)) 
	    {
	        xvf_get_attribute(selection->back_kformstruct, 
				  XVF_VARIABLE, &var);
	        kstrcat(variables, " ");
	        kstrcat(variables, var);
		kfree(var);
    
	        xvf_get_attribute(extensions[i], XVF_VARIABLE, &var);
	        kstrcat(variables, "!");
	        kstrcat(variables, var);
		kfree(var);
	    }
	}
	kfree(extensions);

	/*
	 *  Wanna see if the title needs to be explicitly saved.  If the
	 *  user overrides the default name, then we have to save it.
	 */
	xvl_get_contents(NULL, &tblist);
	tbname_token = kstring_to_token(tbname);
	oname_token  = kstring_to_token(oname);
	if ((list = klist_locate(tblist, (kaddr) tbname_token)) != NULL &&
	    (list = (klist *) klist_clientdata(list))           != NULL &&
	    (list = klist_locate(list, (kaddr) oname_token))    != NULL)
	{
	   info = (char **) klist_clientdata(list);
	   if (title != NULL && kstrcmp(title, info[0]) != 0)
	      name = title;
	   else name = "";
	}
	else name = "";

	tbname = !tbname ? "(unknown)" : tbname;
	oname  = !oname  ? "(unknown)" : oname;
	title  = !title  ? "(unknown)" : title;

	/*
	 *  Need to make sure that the command doesn't contain new lines.
	 *  If they do then we will want to substitute these with \\n newline
	 *  tokens.
	 */
	kstring_replace(command, "\n", "\\n", command);
	if (machine != NULL)
	   machine = kstring_replace(machine, ":", "\\:", temp);
	else
	   machine = "";

	/* save the glyph/conditional information */
	kfprintf(file, "\n#  %s '%s'\n%s:%s:%s:%s:%s:%d:%d:%d:%s:%s\n", type,
		title, type, tbname, oname, name, machine, ident, xpos, ypos,
		variables, command);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphRestore - Restore the glyph/conditional
|
|       Purpose: This routine does the actually restoring of a glyph or
|		 conditional from a workspace file.
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|        Output: ident - the glyph indentifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int GlyphRestore(
    xvobject glyph,
    xvobject parent,
    kfile    *file,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    int      *ident)
{
	XvwGlyphWidget xglyph = (XvwGlyphWidget) xvw_widget(glyph);

	kguide     *guide;
	kselection *selection;
	kform      *form = NULL;
	kobject    toolbox = NULL, program = NULL, paneobj = NULL;

	int      i, xpos, ypos; 
	klist    *list, *tblist = NULL;
	ktoken   tbname_token, oname_token;
	char     *name = NULL, *panefile = NULL, **info, *tmp, *var, *varname;
	char	 command[10*KLENGTH], variables[KLENGTH], newname[KLENGTH],
		 tbname[KLENGTH],  oname[KLENGTH], identifier[KLENGTH],
		 machine[KLENGTH];


	machine[0] = newname[0] = '\0'; kstrcpy(command, " ");
	if (ksscanf(line,"%[^:]:%[^#:]:%[^:]:%d:%d:%d:%[^:]:%[^\n]",identifier,
	        tbname, oname, ident, &xpos, &ypos, variables,
		&command[1]) != 8 &&
	    ksscanf(line,"%[^:]:%[^#:]:%[^:]:%[^:]:%d:%d:%d:%[^:]:%[^\n]",
		identifier, tbname, oname, newname, ident, &xpos, &ypos,
		variables, &command[1]) != 9 &&
	    ksscanf(line,"%[^:]:%[^#:]:%[^:]:%[^:]:%[^:]:%d:%d:%d:%[^:]:%[^\n]",
		identifier, tbname, oname, newname, machine, ident,
		&xpos, &ypos, variables, &command[1]) != 10)
	{
	   return(FALSE);
	}
	xvw_set_attributes(glyph,
		   XVW_XPOSITION, xpos + xoffset,
		   XVW_YPOSITION, ypos + yoffset,
		   XVW_NODE_NAME, oname,
		   NULL);

	xvl_get_contents(NULL, &tblist);
	tbname_token = kstring_to_token(tbname);
	oname_token  = kstring_to_token(oname);
	if ((list = klist_locate(tblist, (kaddr) tbname_token)) != NULL &&
	    (list = (klist *) klist_clientdata(list))           != NULL &&
	    (list = klist_locate(list, (kaddr) oname_token))    != NULL)
	{
	   info = (char **) klist_clientdata(list);
	   name = info[0];
	   panefile = info[5];
	}
	else
	{
	   kinfo(KXVLIB, "Swing and a miss... for toolbox '%s' oname '%s\n",
			tbname, oname);
	   if ((toolbox = kcms_open_toolbox(tbname)) == NULL ||
	       (program = kcms_open_cmobj(toolbox, oname)) == NULL)
	   {
	      if (toolbox == NULL)
	      {
	         kerror(XVLANG, "WorkspaceRestore", "Failed to open "
			"toolbox '%s'.  Failed to restore the object '%s' "
			"within toolbox '%s'.  Please check to see that "
			"toolbox '%s' exists and is accessible to '%s' before "
			"restoring this workspace.", tbname, oname, tbname,
			tbname, kprog_get_program());
	      }
	      else
	      {
	         kerror(XVLANG, "WorkspaceRestore", "Failed to open object "
			"'%s'.  Failed to restore the object '%s' within "
			"toolbox '%s'.  Please check to see that object '%s' "
			"exists within this toolbox and is accessible to '%s' "
			"before restoring this workspace.", oname, oname,
			tbname, oname, kprog_get_program());
	       }
	       return(FALSE);
	   }
	   kcms_get_attribute(program, KCMS_CMOBJ_ICON_NAME, &name);
	   kcms_get_attribute(program, KCMS_CMOBJ_UIS_PANE, &paneobj);
	   kcms_get_attribute(paneobj, KCMS_PATH, &panefile);
	}

	kstring_replace(machine, "\\:", ":", machine);
	if (kstrlen(newname) > 0)
	   name = newname;

	xvw_set_attributes(glyph,
		   XVW_NODE_NAME,      name,
		   XVW_NODE_REMOTE_LOCATION, machine,
		   XVW_GLYPH_FORMFILE, panefile,
		   XVW_GLYPH_TBNAME,   tbname,
		   XVW_GLYPH_ONAME,    oname,
		   NULL);

	if (xvw_get_attribute(glyph, XVW_GLYPH_FORM, &form) && form != NULL)
	{
	   /*
	    *  Need to make sure that the command doesn't contain new line
	    *  tokens.  If they do then we will want to substitute these with
	    *  real new line characters.
	    */
	   kstring_replace(command, "\\n", "\n", command);
	   xvf_modify_form_from_cmd(form, command);

           /* find the selected pane */
           guide = kvf_search_sel_guide(form->subform);
           xvf_clear_modified(guide->pane->sel_list);
	}

	/*
	 *  Clear the temporary output connections
	 */
	for (i = 0; i < xglyph->node.osize; i++)
	{
	   tmp = NULL;
	   xvf_get_attribute(xglyph->glyph.osels[i], XVF_FILE_NAME, &tmp);
	   if (kstrstr(tmp, "/io") != NULL)
	      xvf_set_attribute(xglyph->glyph.osels[i], XVF_LITERAL, NULL);
	   xvw_set_attribute(xglyph->node.oports[i], XVW_PORT_RESET, TRUE);
	   kfree(tmp);
	}

	/*
	 *  Clear the temporary input connections
	 */
	for (i = 0; i < xglyph->node.isize; i++)
	{
	   tmp = NULL;
	   xvf_get_attribute(xglyph->glyph.isels[i], XVF_FILE_NAME, &tmp);
	   if (kstrstr(tmp, "/io") != NULL)
	      xvf_set_attribute(xglyph->glyph.isels[i], XVF_LITERAL, NULL);
	   xvw_set_attribute(xglyph->node.iports[i], XVW_PORT_RESET, TRUE);
	   kfree(tmp);
	}

	/*
	 *  Export the desired selections, specified in the variables
	 *  array, to the workspace gui.
	 */
	if ((var = kstring_cleanup(variables, variables)) != NULL)
	{
	   while (var != NULL)
	   {
	      if ((tmp = kstrchr(var, ' ')) != NULL)
	         *tmp++ = '\0';

	      if ((varname = kstrchr(var, '!')) != NULL)
		 *varname++ = '\0';

	      if ((selection = kvf_variable_sel_search(form, var)) != NULL)
	      {
		 if (selection->back_group != NULL)
		    xvl_copy_selection(parent,glyph,selection->back_group,NULL);

	         xvl_copy_selection(parent, glyph, selection, varname);
	      }
	      var = tmp;
	   }
	}
	if (program) kcms_close(program);
	if (toolbox) kcms_close(toolbox);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphBuild
|
|       Purpose: This routine will run build the glyph command to be run.
|         Input: widget      - the glyph widget to be run
|                command     - the command to build.
|
|    Written By: Mark Young
|          Date: Apr 13, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int GlyphBuild(
   Widget   widget,
   char     *command)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;
	xvobject glyph = xvw_object(widget);
	int expand;

	char *machine, *temp;

#if 1
	if (xwid->glyph.command != NULL)
	{
	   kstrcpy(command, xwid->glyph.command);
	   return(TRUE);
	}
#else
	kfree_and_NULL(xwid->glyph.command);
#endif
	GlyphFormCallback(xwid->glyph.form, xwid->glyph.form->subform, glyph);
	machine = (!xwid->node.remote_enable) ? NULL : xwid->node.machine;
	expand = (!xwid->node.remote_enable) ? TRUE : FALSE;
        if ((temp = kvf_form_to_cmd(xwid->glyph.form, xwid->glyph.form->subform,
				    machine, TRUE)) == NULL)
        {
           kerror(XVLANG, "GlyphBuild", "Unable to build command line");
           return(FALSE);
        }
        kstrcpy(command, temp);
 
	/*
         *  Check to see if we should echo the command, before executing
         */
        if (xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE ||
            xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
        {
           kprintf("\n# (%s) %s %s %s\n%s\n", !xwid->node.contrun ? "new" :
		    "contrun", xwid->glyph.tbname, xwid->glyph.oname,
		    xwid->node.name, command);
        }
	if (xwid->glyph.echo == KGLYPH_ECHO_TO_LOG ||
            xwid->glyph.echo == KGLYPH_ECHO_TO_CONSOLE_AND_LOG)
        {
	   kfile *file;

	   file = xvl_get_logfile();
	   if (file != NULL)
	   {
               kfprintf(file, "\n# (%s) %s %s %s\n%s\n", 
			!xwid->node.contrun ? "new" :
                        "contrun", xwid->glyph.tbname, xwid->glyph.oname,
                        xwid->node.name, command);
	       kfclose(file);
	   }
	}
	xwid->glyph.command = kstrdup(command);
	kfree(temp);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: DispatchCallback - execute callback routine
|
|       Purpose: This routine is used to process when a glyph finishes
|		 executing.
|         Input: object - object to be used to build the form
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void DispatchCallback(
   int     status,
   kaddr   client_data)
{
	Widget widget = (Widget) client_data;
	XvwGlyphWidget xwid = kwidget(client_data);

	char temp[KLENGTH];
	
	if (xwid->core.being_destroyed == TRUE)
	   return;

	if (WIFEXITED(status) == FALSE || WEXITSTATUS(status) != 0)
	{
	   /* Report that we did not exit normally */
	   if (WIFSIGNALED(status) == FALSE)
	      ksprintf(temp, "\nProcess terminated with an exit status of %d", 
		     WEXITSTATUS(status));
	   else
	      ksignal_format(WTERMSIG(status), temp);

	   MessageCallback(NULL, client_data, temp);
	   if (xwid->glyph.error != NULL)
	   {
	      xvw_set_attribute(xwid->glyph.error, XVW_PIXMAP,
	     	     xwid->glyph.error_pixmap);
	   }

	   /*
	    *  Make sure that the glyph realizes that we aren't running
	    *  anymore. 
	    */
	   xwid->node.pid = 0;
           xwid->node.status = -1;
	   xvw_set_attributes(xvw_object(widget),
		   XVW_NODE_EXECUTE, FALSE,
		   XVW_NODE_MODIFIED, TRUE,
		   NULL);
	}
	else if (xwid->node.khoros_routine == FALSE || xwid->node.pid != 0)
	{
           /*
            *  Make sure that the glyph realizes that we aren't running
            *  anymore.
            */
           xwid->node.pid = 0;
           xwid->node.status = 0;
	   xvw_set_attributes(xvw_object(widget),
	    	   XVW_NODE_MODIFIED, FALSE,
	    	   XVW_NODE_EXECUTE, FALSE,
		   NULL);
	}
	else
           xwid->node.status = 0;
}

/*-----------------------------------------------------------
|
|  Routine Name: NodeRunCallback - node run callback routine
|
|       Purpose: This routine is the node run callback used to track the number
|		 excutions.
|         Input: object - object to be used to build the form
|    Written By: Mark Young
|          Date: Jun 16, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void NodeRunCallback(
   xvobject node,
   kaddr    client_data,
   kaddr    call_data)
{
        int running = *((int *) call_data);
	XvwGlyphWidget xwid = kwidget(client_data);


	if (running == TRUE)
	{
	   if (xwid->glyph.num_runs == 0)
	      GlyphStartTime((Widget) xwid);
	}
	else if (running == FALSE)
	{
	   if (((xwid->glyph.num_runs+1) % 50) == 0)
	   {
	      GlyphEndTime((Widget) xwid);
	      GlyphPrintTime((Widget) xwid);
	   }
	   else
	      xwid->glyph.num_runs++;
	}
}

static void GlyphStartTime(
    Widget widget)
{
        XvwGlyphWidget xwid = (XvwGlyphWidget) widget;
	static struct tms start_buffer;

	xwid->glyph.num_runs   = 0;
	if (xwid->glyph.runtime == FALSE)
	   return;

	xwid->glyph.end_time   = (time_t) 0;
	xwid->glyph.start_time = times(&start_buffer);
}

static void GlyphEndTime(
    Widget widget)
{
        XvwGlyphWidget xwid = (XvwGlyphWidget) widget;
	static struct tms end_buffer;

	xwid->glyph.num_runs++;
	if (xwid->glyph.runtime == FALSE)
	   return;

	xwid->glyph.end_time = times(&end_buffer);
}

static void GlyphPrintTime(
   Widget widget)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	size_t num;
	time_t diff;
	double elapsed;
	char   temp[KLENGTH];

        static long clktck = 0;
  
	/*
	 *  Report number of runs.
	 */
	if ((num = xwid->glyph.num_runs) == 0)
	   return;

        xvl_report(TRUE, "Glyph %s has run %d times\n", xwid->node.name,
			xwid->glyph.num_runs);

	xwid->glyph.num_runs = 0;
	if (xwid->glyph.runtime == FALSE)
	   return;

	diff = (xwid->glyph.end_time - xwid->glyph.start_time);
	if (clktck == 0)
	   clktck = sysconf(_SC_CLK_TCK);

	/*
	 *  Report elaped time.
	 */
	if ((elapsed = diff/(double) clktck) > 0.0)
	{
           ksprintf(temp, "%s: %d runs in %g seconds - %g runs/sec.\n",
		xwid->node.name, num, elapsed, num/elapsed);
	   MessageCallback(NULL, widget, temp);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: MessageCallback - message callback routine
|
|       Purpose: This routine is used to process when a glyph sends
|		 a message.
|         Input: object - object to be used to build the form
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void MessageCallback(
   kfile *file,
   kaddr client_data,
   char  *buffer)
{
        Widget widget = (Widget) client_data;
	XvwGlyphWidget xwid = kwidget(client_data);

	long id;
	char error[KLENGTH], *tmp, *ptr, *new = NULL;

	/*
	 *  But first lets see if any of the output is for the parser.
	 *  What we need to do is send in the message line by line to the
	 *  parser.  If it parses then great, but if not we want to put 
	 *  it into a string "new" so that we can post it to the glyph cli.
	 */
	if (xwid->core.being_destroyed == TRUE)
	   return;

	if (xwid->node.exectype == KNODE_EXECTYPE_PARSER)
	{
	   id = (long) xwid->glyph.expression_id;
	   new = kstring_cleanup(buffer, NULL);

	   tmp = new;
	   while (tmp != NULL)
	   {
	      if ((ptr = kstrchr(tmp, '\n')) != NULL)
		 *ptr = '\0';

	      if (!kexpr_evaluate_generic(id, tmp, KSTRING, NULL, error))
	      {
	         if (new != tmp)
		    kstring_3cat(new, tmp, "\n", new);
	      }
	      else if (new == tmp)
	         *new = '\0';

	      tmp = ptr;
	   }
	   buffer = new;
	}

	if (kstrlen(buffer) > 0 && xwid->glyph.reporting == TRUE)
	{
	   GlyphCreateCli(widget);
	   xvw_set_attribute(xwid->glyph.cli, XVW_TEXTDISPLAY_ADDTEXT, buffer);
	   xvw_set_attribute(xwid->glyph.error, XVW_PIXMAP,
			     xwid->glyph.info_pixmap);
	   xvw_map(xwid->glyph.error);
	}
	else if (kstrlen(buffer) > 0)
	{
	   while (buffer != NULL)
	   {
	      if ((tmp = kstrchr(buffer, '\n')) != NULL)
		 *tmp++ = '\0';
	      kfprintf(kstderr, "%d: %s\n", xwid->node.pid, buffer);
	      buffer = tmp;
	   }
	}
	kfree(new);
}


/************************************************************
*
*  Routine Name: xvw_create_glyph - create a glyph object
*
*       Purpose: The purpose of the Glyph GUI object is to used to represent
*		 an operator within Khoros.  Typically these are data processing
*		 routines that have a set of inputs and outputs that can be
*		 used to connect to other glyph's in which to form a visual
*		 program.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The glyph object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 14, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_glyph(
   xvobject parent,
   char     *name)
{
	xvobject glyph;


	glyph = xvw_create(parent, FALSE, TRUE, name, GlyphWidgetClass);
	xvw_set_attributes(glyph,
		XVW_SELECTABLE, TRUE,
		XVW_RESIZABLE, FALSE,
		XVW_MENUABLE, FALSE,
		NULL);
	return(glyph);
}
