 /*
  * Khoros: $Id: GlyphUtil.c,v 1.47 1996/08/10 03:25:12 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: GlyphUtil.c,v 1.47 1996/08/10 03:25:12 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Glyph Widget Utilities
   >>>>  Private:
   >>>> 		GlyphInitForm()
   >>>> 		GlyphUpdateForm()
   >>>>			GlyphSetFormfile()
   >>>>			GlyphGetFormfile()
   >>>> 		GlyphHandler()
   >>>> 		GlyphFormExpression()
   >>>> 		GlyphFormCallback()
   >>>> 		GlyphInputFilenameCallback()
   >>>> 		GlyphInputDavCallback()
   >>>> 		GlyphRunCallback()
   >>>> 		GlyphCreateCli()
   >>>> 		GlyphStartDebugger()
   >>>>   Static:
   >>>> 		OptionsInitialize()
   >>>> 		OptionsCallback()
   >>>> 		OptionsActions()
   >>>> 		ComposerCallback()
   >>>> 		DebuggerCallback()
   >>>>   Public:
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <kutils/ksignal.h>
#include <xvlang/GlyphP.h>

static void OptionsInitialize PROTO((xvobject));
static void OptionsCallback   PROTO((xvobject, kaddr, kaddr));
static void OptionsActions    PROTO((xvobject, kaddr, kaddr));
static void ComposerCallback  PROTO((int, kaddr));

static char  *options[] = {
  "Bugs/Todo...",
  "Composer...",
  "Object Info...",
  "",
  "Runtime Logging <Off>",
  "Process Debugging <Off>",
  "GUI Editing <Off>",
  "",
  "Export to Workspace GUI",
};


/*-----------------------------------------------------------
|
|  Routine Name: OptionsInitialize
|
|       Purpose: Initialize the options to be used with the object's option
|		 menu.
|
|         Input: object - the object in which to initialize the options for
|    Written By: Mark Young
|          Date: Apr 25, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void OptionsInitialize(
   xvobject glyph)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	int      i;
	kguide   *guide;
	xvobject button, menu;
	kform_struct **kformstructs;


	/*
	 *  Check to see if the options have already been initialized...
	 */
	if (xwid->glyph.options_initialized == TRUE)
	   return;

	/* find the selected pane */
	guide = kvf_search_sel_guide(xwid->glyph.form->subform);

	/* get the options button on the backplane for the selections */
	if ((kformstructs = kvf_get_selection_kformstructs(guide->pane,
			KUIS_OPTIONS, &i)) != NULL && i == 1)
	{
	   menu = kformstructs[0]->Selptr->back;
	   for (i = 0; i < knumber(options); i++)
	   {
	      if (kstrlen(options[i]) == 0)
	      {
	         xvw_create_separator(menu, "Separator");
		 continue;
	      }

	      button = xvw_create_button(menu, options[i]);
	
	      /* by default, "Export to Workspace GUI" is disabled */
	      if (kstrcmp(options[i], "Export to Workspace GUI") == 0)
	      {
		 xvw_sensitive(button, FALSE);
		 xwid->glyph.export_button = button;
	      }

	      /* 
	       * procedures, loops and conditionals have "Bugs/Todo" disabled 
	       */
	      else if (((xwid->glyph.tbname == NULL) ||
		        (kstrcmp(xwid->glyph.tbname, "CONDITIONAL") == 0)) &&
		       ((kstrcmp(options[i], "Bugs/Todo...") == 0) ||
		        (kstrcmp(options[i], "Composer...") == 0)  ||
		        (kstrcmp(options[i], "Object Info...") == 0)))
                 xvw_sensitive(button, FALSE);

	      xvw_set_attribute(button, XVW_LABEL, options[i]);
	      xvw_insert_callback(button, XVW_BUTTON_SELECT, TRUE,
			OptionsActions, glyph);
	   }
	   xwid->glyph.options_button = xvf_get_xvobject(kformstructs[0],
			XVF_BUTTON_OBJ, TRUE);
	   xvw_insert_callback(guide->pane->back, XVW_SELECT_CALLBACK, FALSE,
			OptionsCallback, xwid);
	   kfree(kformstructs);
	}
	xwid->glyph.options_initialized = TRUE;
}

/*-----------------------------------------------------------
|
|  Routine Name: OptionsCallback
|
|       Purpose: This routine will be called when the pane is put into
|		 edit mode.
|
|         Input: object      - which button being manipulated
|                client_data - the glyph
|                call_data   - not used
|    Written By: Mark Young
|          Date: Aug 28, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void OptionsCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	int      editing = *((int *) call_data);
	kselection *sel_list, *quit_sel;
	XvwGlyphWidget xwid = (XvwGlyphWidget) client_data;


	if (xwid->glyph.options_button == NULL)
	   return;

	if (editing)
	{
	   xvw_raise(xwid->glyph.options_button);
	   if (xwid->glyph.export_button != NULL)
              xvw_sensitive(xwid->glyph.export_button, TRUE);

	   sel_list = xwid->glyph.form->subform->guide->pane->sel_list;
           quit_sel = kvf_find_specified_selection(sel_list, KUIS_QUIT,
			                          (ktoken) NULL);
	   if (quit_sel != NULL) xvw_raise(quit_sel->value_object);
	}
	else
	{
	   if (xwid->glyph.export_button != NULL)
	      xvw_sensitive(xwid->glyph.export_button, FALSE);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: OptionsActions
|
|       Purpose: This routine will be called when the help quit is
|                activated.
|
|         Input: object      - which button being manipulated
|                client_data - the glyph
|                call_data   - not used
|    Written By: Mark Young
|          Date: Dec 1, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void OptionsActions(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject glyph = (xvobject) client_data;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	kguide   *guide;
	xvobject connection_parent;
	kobject  toolbox = NULL, cmobj = NULL;
	char     temp[KLENGTH], filename[KLENGTH], *name = NULL;


	xvw_get_attribute(object, XVW_LABEL, &name);
	if (kstrcmp(name, "Bugs/Todo...") == 0)
	{
	   GlyphCreateCli((Widget) xwid);
	   kdirname(xwid->glyph.panefile, temp), kdirname(temp, temp);
	   ksprintf(filename, "%s/info", temp);
	   xvw_set_attributes(xwid->glyph.help,
		XVW_HELP_MORE_FILES, TRUE,
		XVW_HELP_FILENAME, filename,
		NULL);
	}
	else if (kstrcmp(name, "Object Info...") == 0)
	{
	   unsigned long  otype;
	   char *author, *email, *category, *subcategory, *iconname, *type;


           if ((toolbox = kcms_open_toolbox(xwid->glyph.tbname)) == NULL ||
               (cmobj   = kcms_open_cmobj(toolbox, xwid->glyph.oname)) == NULL)
           {
              (void) kerror(XVLANG, "OptionsActions", "Cannot open %s '%s'.",
                        (!toolbox) ? "toolbox" : "object",
			(!toolbox) ? xwid->glyph.tbname :
				     xwid->glyph.oname);
	      if (cmobj)  kcms_close(cmobj);
	      if (toolbox) kcms_close(toolbox);
	      return;
	   }
	   author = email = category = subcategory = iconname = type = NULL;
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_AUTHOR,       &author);
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_AUTHOR_EMAIL, &email);
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_CATEGORY,     &category);
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_SUBCATEGORY,  &subcategory);
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_ICON_NAME,    &iconname);
	   kcms_get_attribute(cmobj, KCMS_CMOBJ_PROGTYPE,     &otype);
	   if      (otype == KCMS_XVROUTINE) type = "xvroutine";
	   else if (otype == KCMS_KROUTINE)  type = "kroutine";
	   else if (otype == KCMS_PANE)      type = "pane";
	   else if (otype == KCMS_SCRIPT)    type = "pane";
	   else type = "(unknown)";

	   if (!author) author = "(unknown)";
	   if (!email)  email  = "(unknown)";
	   if (!iconname)    iconname     = "(unknown)";
	   if (!category)    category     = "(unknown)";
	   if (!subcategory) subcategory  = "(unknown)";

	   kinfo(KFORCE,
		"\t\tObject Attributes\n\n"
		"Toolbox      : %s\n"
		"Object       : %s\n"
		"Object Type  : %s\n\n"
		"Category     : %s\n"
		"Subcategory  : %s\n"
		"Icon Name    : %s\n\n"
		"Author Name  : %s\n"
		"Author Email : %s", xwid->glyph.tbname, xwid->glyph.oname,
		    type, category, subcategory, iconname, author, email);
	   kcms_close(cmobj); kcms_close(toolbox);
	}
	else if (kstrcmp(name, "Composer...") == 0)
	{
	   ksprintf(temp,"$DESIGNBIN/composer -tb %s -oname %s",
		xwid->glyph.tbname, xwid->glyph.oname);
	   kspawn(temp);
#if 0
	   xvw_set_attribute(object, XVW_LABEL, "Composer <running>");
	   xvw_sensitive(object, FALSE);
#endif
	}
	else if (kstrcmp(name, "GUI Editing <Off>") == 0)
	{
           if ((guide=kvf_search_sel_guide(xwid->glyph.form->subform)) == NULL)
              return;

	   /*
	    *  Turn edit mode on for the pane, and also make sure that
	    *  the options button rides on top of the editing window...
	    */
	   xvw_set_attribute(guide->pane->back, XVW_EDIT_MODE_ON, TRUE);
	   xvw_set_attribute(object, XVW_LABEL, "GUI Editing <On>");
	}
	else if (kstrcmp(name, "GUI Editing <On>") == 0)
	{
           if ((guide=kvf_search_sel_guide(xwid->glyph.form->subform)) == NULL)
              return;

	   /*
	    *  Turn editing off...
	    */
	   xvw_set_attribute(guide->pane->back, XVW_EDIT_MODE_ON, FALSE);
	   xvw_set_attribute(object, XVW_LABEL, "GUI Editing <Off>");
	}
	else if (kstrcmp(name, "Process Debugging <Off>") == 0)
	{
	   xwid->glyph.debug = TRUE;
	   GlyphStartDebugger((Widget) xwid, TRUE);
	   xvw_set_attribute(object, XVW_LABEL, "Process Debugging <On>");
	}
	else if (kstrcmp(name, "Process Debugging <On>") == 0)
	{
	   xwid->glyph.debug = FALSE;
	   GlyphStartDebugger((Widget) xwid, FALSE);
	   xvw_set_attribute(object, XVW_LABEL, "Process Debugging <Off>");
	}
	else if (kstrcmp(name, "Runtime Logging <Off>") == 0)
	{
	   xwid->glyph.runtime = TRUE;
	   xvw_set_attribute(object, XVW_LABEL, "Runtime Logging <On>");
	}
	else if (kstrcmp(name, "Runtime Logging <On>") == 0)
	{
	   xwid->glyph.runtime = FALSE;
	   xvw_set_attribute(object, XVW_LABEL, "Runtime Logging <Off>");
	}
	else if (kstrcmp(name, "Export to Workspace GUI") == 0)
	{
	   connection_parent = xvw_object(xwid->glyph.connection_parent);
	   xvl_export_selections(connection_parent, glyph);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ComposerCallback - composer callback routine
|
|       Purpose: This routine is used to change the fact that
|		 a running composer is terminated.
|
|         Input: object - object to be used to change to be sensitive
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ComposerCallback(
   int    status,
   kaddr  client_data)
{
        xvobject object = (xvobject) client_data;
	xvw_sensitive(object, TRUE);
	xvw_set_attribute(object, XVW_LABEL, "Composer...");
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphInputFilenameCallback - input filename change callback
|
|       Purpose: This routine is used to detect when a change to a glyph's
|		 input filename has ocurred. 
|		 
|         Input: glyph - glyph to be used to change to be sensitive
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphInputFilenameCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvobject glyph = (xvobject) client_data;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	kfree_and_NULL(xwid->glyph.command);
	xvw_set_attribute(glyph, XVW_NODE_RESET, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphInputDavCallback - dav change on our input detected
|
|       Purpose: This routine is used to detect when a change to a glyph's
|		 input data has ocurred. 
|
|         Input: glyph - glyph to be used to change to be sensitive
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphInputDavCallback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	int i, modified, dav = *((int *) call_data);
	xvobject glyph = (xvobject) client_data;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	if (xwid->node.running == TRUE)
	   return;

	if (dav == FALSE)
	{
	   xwid->glyph.last_iport = NULL;
	   for (i = 0;  i < xwid->node.isize; i++)
	   {
	      xvw_get_attributes(xwid->node.iports[i],
			XVW_PORT_DAV, &dav,
			XVW_PORT_MODIFIED, &modified,
			NULL);
	      if (dav == TRUE && modified == TRUE)
		 xwid->glyph.last_iport = xwid->node.iports[i];
	   }
	}
	else if (dav == TRUE && object != xwid->glyph.cflow_in)
	   xwid->glyph.last_iport = object;

	xwid->node.modified = FALSE;
	xvw_set_attribute(glyph, XVW_NODE_MODIFIED, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphRunCallback - the run button on the glyph executed
|
|       Purpose: This performs a run of the glyph via the run button
|		 on the panefile.
|
|         Input: client_data - the glyph widget
|    Written By: Mark Young
|          Date: Jul 23, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphRunCallback(
   kaddr    client_data)
{
	xvobject glyph = (xvobject) client_data;

	int i, selected;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);


	for (i = 0; i < xwid->node.isize; i++)
	{
	   xvw_get_attribute(xwid->node.iports[i], XVW_PORT_SELECTED,
			&selected);
	   if (selected == TRUE)
	      xvw_set_attribute(xwid->node.iports[i], XVW_PORT_DAV, TRUE);
	}
	xvw_set_attribute(glyph, XVW_NODE_RESET, TRUE);
	xvw_set_attribute(glyph, XVW_NODE_EXECUTE, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphUpdateForm
|
|       Purpose: Updates the I/O ports & label of a glyph depending 
|                on the currently selected pane of the form associated
|                with a glyph.  This routine is called directly from 
|                the glyph widget's SetValues routine to initially create 
|                I/O ports and label when the form is first associated 
|                with the glyph using the XVW_GLYPH_FORM attribute; it
|                is also used as a callback installed with xvf_add_extra_call()
|                for when the pane on a form associated with a glyph is
|                changed.
|
|         Input: xwid  - the glyph widget
|    Written By: Danielle Argiro and Mark Young
|          Date: Oct 13, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
void GlyphUpdateForm(
    Widget widget)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	kguide   *guide;
	kform_struct **sel;
        int      i, indx, num;
        xvobject offset, *port, connection_parent;
	xvobject glyph  = xvw_object((Widget) xwid);

	/* find the selected pane */
        guide = kvf_search_sel_guide(xwid->glyph.form->subform);
	xvf_clear_modified(guide->pane->sel_list);
	connection_parent = xvw_object(xwid->glyph.connection_parent);

        /*
	 *  get the array of backplanes for all output selections on new
	 *  pane and allocate new array of output ports
	 */
        sel = kvf_get_selection_kformstructs(guide->pane, KUIS_INPUTFILE, &num);
	port = (xvobject *) kmalloc((size_t) (num * sizeof(xvobject)));

	/* destroy all the old input ports */
        for (i = 0; i < xwid->node.isize; i++)
	{
	    if ((indx = karray_locate(sel, KSTRUCT, xwid->glyph.isels[i],
				(size_t) num)) == -1)
	    {
	       xvw_destroy(xwid->node.iports[i]);
	    }
	}

        offset = xwid->glyph.cflow_in;
        for (i = 0; i < num; offset = port[i++])
        {
	    if ((indx = karray_locate(xwid->glyph.isels, KSTRUCT, sel[i],
				(size_t) xwid->node.isize)) == -1)
	    {
               port[i] = xvw_create_port(glyph, "InputPort");
	       xvw_set_attributes(port[i],
		   XVW_PORT_SELECTION, sel[i],
		   XVW_PORT_DIRECTION, KPORT_INPUT,
		   XVW_PORT_TYPE,      KPORT_DATAFLOW,
		   XVW_PORT_RESET,     TRUE,
		   XVW_BELOW,	       offset,
		   XVW_RIGHT_OF,       NULL,
		   XVW_PORT_CONNECTION_PARENT, connection_parent,
		   NULL);
	       xvw_insert_callback(port[i], XVW_PORT_DAV_CALLBACK,
			FALSE, GlyphInputDavCallback, glyph);
	       xvw_insert_callback(port[i], XVW_PORT_FILENAME_CALLBACK,
			FALSE, GlyphInputFilenameCallback, glyph);
	       xvw_insert_callback(port[i], XVW_PORT_SELECTED_CALLBACK,
			FALSE, GlyphInputFilenameCallback, glyph);
	    }
	    else
	    {
	       port[i] = xwid->node.iports[indx];
	       xvw_set_attribute(port[i], XVW_BELOW, offset);
	    }
	}
	xwid->node.isize = num;
	kfree(xwid->glyph.isels); xwid->glyph.isels = sel;
	kfree(xwid->node.iports); xwid->node.iports = port;


        /*
	 *  get the array of backplanes for all output selections on new
	 *  pane and allocate new array of output ports
	 */
        sel = kvf_get_selection_kformstructs(guide->pane,KUIS_OUTPUTFILE,&num);
	port = (xvobject *) kmalloc((size_t) (num * sizeof(xvobject)));

	/* destroy all the old input ports */
        for (i = 0; i < xwid->node.osize; i++)
	{
	    if ((indx = karray_locate(sel, KSTRUCT, xwid->glyph.osels[i],
				      (size_t) num)) == -1)
	    {
	       xvw_destroy(xwid->node.oports[i]);
	    }
	}

        /* create all the new output ports according to output selections */
        offset = xwid->glyph.cflow_out;
        for (i = 0; i < num; offset = port[i++])
        {
	    if ((indx = karray_locate(xwid->glyph.osels, KSTRUCT, sel[i],
				(size_t) xwid->node.osize)) == -1)
	    {
               port[i] = xvw_create_port(glyph, "OutputPort");
	       xvw_set_attributes(port[i],
		   XVW_PORT_SELECTION, sel[i],
		   XVW_PORT_DIRECTION, KPORT_OUTPUT,
		   XVW_PORT_TYPE,      KPORT_DATAFLOW,
		   XVW_PORT_RESET,     TRUE,
		   XVW_BELOW,	       offset,
		   XVW_LEFT_OF,        NULL,
		   XVW_PORT_CONNECTION_PARENT, connection_parent,
		   NULL);
	    }
	    else
	    {
	       port[i] = xwid->node.oports[indx];
	       xvw_set_attribute(port[i], XVW_BELOW, offset);
	    }
        }
	xwid->node.osize = num;

	kfree(xwid->glyph.osels); xwid->glyph.osels = sel;
	kfree(xwid->node.oports); xwid->node.oports = port;
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphInitForm
|
|       Purpose: This routine is called directly from the glyph widget's
|		 SetValues routine to initially create I/O ports and label
|		 when the form is first associated with the glyph using the
|		 XVW_GLYPH_FORM attribute.
|
|         Input: xwid  - the glyph widget
|    Written By: Danielle Argiro and Mark Young
|          Date: Oct 13, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
void GlyphInitForm(
    kaddr client_data)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) client_data;

	kguide   *guide;
        int      i, type;
	char	 temp[KLENGTH], path[KLENGTH];
	xvobject glyph = xvw_object((Widget) xwid);


	/* find the selected pane */
        guide = kvf_search_sel_guide(xwid->glyph.form->subform);
	GlyphUpdateForm((kaddr) xwid);

        /* get the array of backplanes for all routine selections on new pane */
	kfree(xwid->glyph.rsels);
        if ((xwid->glyph.rsels = kvf_get_selection_kformstructs(guide->pane,
			KUIS_ROUTINE, &xwid->glyph.rsize)) != NULL)
	{
	   xvf_get_attribute(xwid->glyph.rsels[0], XVF_EXECTYPE, &type);
	   xwid->node.exectype = type;
	}
	else
	{
	   xwid->node.exectype = 0;
	   xwid->node.runnable = FALSE;
	   xvw_set_attribute(xwid->glyph.cflow_in,  XVW_MAPPED, FALSE);
	   xvw_set_attribute(xwid->glyph.cflow_out, XVW_MAPPED, FALSE);
	}

	if (xwid->node.runnable == FALSE)
	{
	   xvw_unmap(xwid->node.icon);
	   for (i = 0; i < xwid->node.osize; i++)
	      xvw_set_attribute(xwid->node.oports[i], XVW_PORT_DAV, TRUE);
	}
	else
	{
	   xvw_map(xwid->node.icon);
	   xvw_set_attribute(glyph, XVW_NODE_MODIFIED, TRUE);
	}

	/* IconNode support */
	if (kdirname(xwid->glyph.panefile, temp) != NULL)
	{
	   ksprintf(path, "%s/ObjectIcon", temp);
	   if (kaccess(path, R_OK) == 0)
	      xvw_set_attribute(glyph, XVW_NODE_PIXMAPFILE, path);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphGetFormfile
|
|       Purpose: GlyphGetFormfile is used to get the filename associated
|		 with the current form.
|
|         Input: widget    - the widget in which we will be getting the
|			     filename
|		 attribute - the old object to be closed
|        Output: calldata  - pointer to retrieve the object name
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
int GlyphGetFormfile(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(object);
	String *filename = (String *) calldata;


	if (kstrcmp(attribute, XVW_GLYPH_FORMFILE) != 0)
	{
	   kerror(XVLANG, "GlyphGetFormfile", "invalid attribute %s",
			attribute);
	   return(FALSE);
	}
	*filename = xwid->glyph.panefile;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphSetFormfile
|
|       Purpose: GlyphSetFormfile is used to set the filename associated
|		 with the attribute.
|
|         Input: object    - the object in which we will be getting the filename
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
int GlyphSetFormfile(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(object);
	String *filename = (String *) calldata;

	kform  *form;
	kselection *sel_list, *selection;

	if (kstrcmp(attribute, XVW_GLYPH_FORMFILE) != 0)
	{
	   kerror(XVLANG, "GlyphSetFormfile", "invalid attribute %s",attribute);
	   return(FALSE);
	}

	/*
	 *  So create the form...
	 */
	if ((form = kvf_create_form(*filename, KNONE, NULL, NULL)) == NULL)
	   return(FALSE);

	/*
	 *  Go ahead and override the run button on the pane file (if one
	 *  exists).  We do this so that the run button will use our scheduler.
	 */
	kvf_change_entry(form, FALSE, GlyphFormCallback, object);
	if (kvf_check_for_single_pane(form, "GlyphSetFormfile"))
	{
	   sel_list = form->subform->guide->pane->sel_list;
	   selection = kvf_gui_named_item_exists(sel_list, KUIS_ROUTINE, NULL);
	   if (selection != NULL)
	   {
	      xvf_add_extra_call(selection->back_kformstruct,
		    GlyphRunCallback, (kaddr) object, XVF_CALL_SUBSTITUTE);
	   }
	}
	kfree(xwid->glyph.panefile);
	xwid->glyph.panefile = kstrdup(*filename);
	xvw_set_attribute(object, XVW_GLYPH_FORM, form);
	xwid->glyph.created_form = TRUE;
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphHandler
|
|       Purpose: This routine will perform a variety of glyph callbacks,
|		 such as destroy, menu, execute, etc.
|
|         Input: object      - the button being activated
|                client_data - the glyph widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Nov 23, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	xvobject glyph = (xvobject) client_data;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	Widget widget;
	int    execute;
	static xvobject selected_object = NULL;


	*dispatch = FALSE;
	widget = xvw_widget(object);
	object = xvw_object(widget);
	if (event->type == LeaveNotify)
	{
	   kannounce(XVLANG, "GlyphHandler", " ");
	   return;
	}
	else if (event->type == EnterNotify)
	{
	   char *tmp = NULL;

	   if (object == xwid->glyph.menuform)
	      tmp = "Pane Access";
	   else if (object == xwid->glyph.error)
	      tmp = "Display Error Message";
	   else if (object == xwid->node.icon)
	      tmp = "Run/Stop The Glyph";
	   else if (object == xwid->node.label)
	      tmp = "Glyph's Icon Name";
	   else
	      return;

	   kannounce(XVLANG, "GlyphHandler", "%s:  %s", xwid->node.name, tmp);
	   return;
	}
	else if (event->type == ButtonPress)
	{
	   selected_object = object;
	   return;
	}
	else if (event->type != ButtonRelease || object != selected_object)
	{
	   return;
	}

	if (object == xwid->glyph.error)
	{
	   xvw_unmap(xwid->glyph.error);
	   xvw_manage(xwid->glyph.help);
	   xvw_map(xwid->glyph.help);
	   xvw_raise(xwid->glyph.help);
	}
	else if (object == xwid->glyph.menuform)
	{
	   if (xwid->glyph.form != NULL)
	   {
	      if (!xvw_check_mapped(xwid->glyph.form->subform->toplevel))
	      {
		 xvw_set_attribute(object, XVW_PIXMAP,
			xwid->glyph.closepane_pixmap);
	         xvf_map_subform(xwid->glyph.form->subform);
		 if (xwid->node.leave == FALSE) xvw_unmap(glyph);
	      }
	      else
	      {
		 xvw_set_attribute(object, XVW_PIXMAP,
			xwid->glyph.openpane_pixmap);
	         xvf_unmap_subform(xwid->glyph.form->subform);
		 xvw_map(glyph);
	      }
	      OptionsInitialize(glyph);
	   }
	   else
	      kerror(XVLANG, "GlyphHandler",
		     "No form associated with glyph");
	}
	else if (object == xwid->node.icon)
	{
	   int runnable = FALSE;

	   xvw_get_attributes(glyph,
		XVW_NODE_EXECUTE, &execute,
		XVW_NODE_RUNNABLE, &runnable,
		NULL);

	   if (execute || xwid->node.contrun)
	   {
	      /*  Stop running the Glyph by setting execute to FALSE */
	      xvw_set_attribute(glyph, XVW_NODE_EXECUTE, FALSE);
	   }
	   else if (runnable == TRUE)
	   {
	      /*
	       *  Force the glyph to execute.  We do this by clearing it's
	       *  data availables and then running the glyph.
	       */
	      xvw_set_attribute(glyph, XVW_NODE_RESET, TRUE);
	      xvw_set_attribute(glyph, XVW_NODE_EXECUTE, TRUE);
	   }
	   else
	   {
	      /*
	       *  Can't run so we better inform the dofus to go click
	       *  somewhere else.
	       */
	      kwarn("xvlang", "Glyph Actions",
		     "Warning: This glyph isn't ready to be run.  Typically "
		     "this is because the input data to the glyph is not "
		     "available.  If you wish to override and forceable "
		     "run the glyph, then use the 'Run' button found on the "
		     "glyph's menu pane.");
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphFormExpression
|
|       Purpose: This routine will evaluate an expression associated with
|		 a form.
|
|         Input: form - the form that the expression should be evaluated for
|                client-data - the client data associated with the form
|                string - the expression to be evaluated
|                type   - the type of expression to be evaluated
|	 Output: return_value - the return value to be evaluated
|		 error        - the error string to be returned
|    Written By: Mark Young
|          Date: Aug 14, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
int GlyphFormExpression(
   kform *form,
   kaddr client_data,
   char  *string,
   int   type,
   kaddr return_value,
   char  *error)
{
	xvobject glyph = (xvobject) client_data;
        XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	long id = (long) xwid->glyph.expression_id;

	return(kexpr_evaluate_generic(id, string, type, return_value, error));
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphFormCallback - form callback routine
|
|       Purpose: This routine is used to process the user's form
|                changes.   Any changes in the form are noted
|                and the object/gadget/KHOROS-item is updated.  Update is
|                accomplished by using the xvw_set_attributes() to modify
|                the attributes of the object/gadget/KHOROS-item.
|
|         Input: object - object to be used to build the form
|    Written By: Mark Young
|          Date: Nov 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphFormCallback(
   kform    *form,
   ksubform *subform,
   kaddr    client_data)
{
        xvobject glyph = (xvobject) client_data;
	XvwGlyphWidget xwid = (XvwGlyphWidget) xvw_widget(glyph);

	kguide *guide;
	int    i, modified;
	kselection *selection;


	if (form->quit)
	{
	   xvw_map(glyph);
	   form->quit = subform->quit = FALSE;
	   xvw_set_attribute(xwid->glyph.menuform, XVW_PIXMAP,
		xwid->glyph.openpane_pixmap);
	}

	for (i = 0, modified = FALSE; i < xwid->node.isize; i++)
	{
	   xvf_get_attribute(xwid->glyph.isels[i], XVF_MODIFIED, &modified);
	   if (modified)
	   {
	      xvf_set_attribute(xwid->glyph.isels[i], XVF_MODIFIED, FALSE);
	      xvw_set_attribute(xwid->node.iports[i], XVW_PORT_RESET, TRUE);
	   }
	}
	if (modified) kfree_and_NULL(xwid->glyph.command);

	for (i = 0, modified = FALSE; i < xwid->node.osize; i++)
	{
	   xvf_get_attribute(xwid->glyph.osels[i], XVF_MODIFIED, &modified);
	   if (modified)
	   {
	      xvf_set_attribute(xwid->glyph.osels[i], XVF_MODIFIED, FALSE);
	      xvw_set_attribute(xwid->node.oports[i], XVW_PORT_RESET, TRUE);
	   }
	}
	if (modified) kfree_and_NULL(xwid->glyph.command);

	if ((guide = kvf_search_sel_guide(subform)) != NULL)
	{
	   selection = kvf_gui_named_item_exists(guide->pane->sel_list,
				                 KUIS_QUIT, NULL);
	   if (selection != NULL)
	      selection->modified = FALSE;

	   if (xwid->node.runnable == TRUE &&
	       xvf_check_modified(guide->pane->sel_list))
	   {
	      kfree_and_NULL(xwid->glyph.command);
	      xvf_collect_check_data(form);
	      xvf_clear_modified(guide->pane->sel_list);
	      xvw_set_attribute(glyph, XVW_NODE_RESET, TRUE);
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphCreateCli
|
|       Purpose: GlyphCreateCli is used to create the command line interface
|		 widget for a given object.
|
|         Input: glyph - 
|    Written By: Mark Young
|          Date: May 31, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphCreateCli(
   Widget widget)
{
	xvobject glyph;
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	char temp[KLENGTH];

	if (xwid->glyph.help)
	   return;

	glyph = xvw_object(widget);
	xwid->glyph.error = xvw_create_pixmap(xvw_parent(glyph), "error/info");
        xvw_set_attributes(xwid->glyph.error,
                XVW_PIXMAP,        xwid->glyph.error_pixmap,
                XVW_MAPPED,        FALSE,
                XVW_BELOW,         glyph,
                XVW_LEFT_OF,       glyph,
                XVW_RIGHT_OF,      glyph,
                XVW_SELECTABLE,    FALSE,
                XVW_RESIZABLE,     FALSE,
                XVW_MENUABLE,      FALSE,
                NULL);
	xvw_insert_event(xwid->glyph.error, EnterWindowMask | LeaveWindowMask |
		         ButtonPressMask | ButtonReleaseMask, GlyphHandler,
			 glyph, KLIST_HEAD);

	ksprintf(temp, "Output For '%s'", xwid->node.name);
	xwid->glyph.help = xvw_create_help(NULL, "Glyph Help Output");
	xvw_set_attributes(xwid->glyph.help,
			XVW_HELP_TITLE, temp,
			XVW_PREFERRED_HEIGHT, 350,
			XVW_HELP_QUITLABEL,   "Close",
			XVW_HELP_DESTROY_ON_QUIT, FALSE,
			NULL);
	xvw_unmanage(xwid->glyph.help);
	xvw_get_attribute(xwid->glyph.help, XVW_HELP_TEXTDISPLAY_OBJECT,
			&xwid->glyph.cli);
}
