 /*
  * Khoros: $Id: Node.c,v 1.52 1996/08/10 03:25:16 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Node.c,v 1.52 1996/08/10 03:25:16 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Node Widget
   >>>>  Private:
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 ClassPartInitialize()
   >>>>                 Initialize()
   >>>>                 Realize()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 ChangeSelection()
   >>>>                 ResetNode()
   >>>>                 GetExecute()
   >>>>                 SetExecute()
   >>>>                 NodeHandler()
   >>>>                 NodeSave()
   >>>>                 NodeRestore()
   >>>>                 NodeIpcCallback()
   >>>>   Public:
   >>>>                 xvw_create_node()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/NodeP.h>
#include <xvlang/PortP.h>

static void ClassInitialize     PROTO((void));
static void ClassPartInitialize PROTO((WidgetClass));
static void Initialize	        PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy             PROTO((Widget));
static void Realize      PROTO((Widget, XtValueMask *, XSetWindowAttributes *));
static Boolean SetValues PROTO((Widget, Widget, Widget, ArgList, Cardinal *));

static void ChangeSelection     PROTO((Widget, int));
static int  ResetNode	        PROTO((xvobject, char *, kaddr));
static int  SetExecute	        PROTO((xvobject, char *, kaddr));
static int  GetExecute	        PROTO((xvobject, char *, kaddr));
static void NodeHandler	        PROTO((xvobject, kaddr, XEvent *, int *));


/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

#define REMOTE_PIXMAP "$DESIGN/objects/library/xvlang/misc/glyph/remote.xpm"

static xvattribute attributes[] = {
{XVW_NODE_NAME,              NULL,     XtRString,   NULL},
{XVW_NODE_PLACEMENT,         NULL,     XtRInt,      XtRBoolean},
{XVW_NODE_LEAVE,             NULL,     XtRInt,      XtRBoolean},
{XVW_NODE_MODIFIED,          NULL,     XtRInt,      XtRBoolean},
{XVW_NODE_REMOTE_ENABLE,     NULL,     XtRInt,      XtRBoolean},
{XVW_NODE_REMOTE_LOCATION,   NULL,     XtRString,   NULL},
{XVW_NODE_EXECTYPE,          NULL,     XtRInt,      NULL},
{XVW_NODE_SIZE,              NULL,     XtRInt,      NULL},
{XVW_NODE_RUN_CALLBACK,      NULL,     XtRCallback, NULL},
{XVW_NODE_MODIFIED_CALLBACK, NULL,     XtRCallback, NULL},
{XVW_NODE_STATUS,            NULL,     XtRInt,      XtRBoolean},

{XVW_NODE_PIXMAP,     NULL,   XtRPixmap, NULL},
{XVW_NODE_PIXMAPFILE, XVW_NODE_PIXMAP, XtRString, XtRPixmap},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwNodeWidgetRec, node.field)

static XtResource resources[] = { 
{XVW_NODE_NAME, NULL, XtRString, sizeof(String),
        offset(name), XtRImmediate, (XtPointer) NULL},
{XVW_NODE_EXECTYPE, NULL, XtRInt, sizeof(int),
        offset(exectype), XtRImmediate, (XtPointer) KNODE_EXECTYPE_NONE},
{XVW_NODE_PLACEMENT, NULL, XtRBoolean, sizeof(Boolean),
        offset(placement), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_LEAVE, NULL, XtRBoolean, sizeof(Boolean),
        offset(leave), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_MODIFIED, NULL, XtRBoolean, sizeof(Boolean),
        offset(modified), XtRImmediate, (XtPointer) TRUE},
{XVW_NODE_REMOTE_ENABLE, NULL, XtRBoolean, sizeof(Boolean),
        offset(remote_enable), XtRImmediate, (XtPointer) FALSE},
{XVW_NODE_RUN_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
        offset(run_callback), XtRImmediate, (XtPointer) NULL},
{XVW_NODE_MODIFIED_CALLBACK, NULL, XtRCallback, sizeof(XtPointer),
        offset(modified_callback), XtRImmediate, (XtPointer) NULL},
{XVW_NODE_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(pixmap), XtRImmediate, (XtPointer) None},
{XVW_NODE_SIZE, NULL, XtRInt, sizeof(int),
        offset(nodesize), XtRImmediate, (XtPointer) KNODE_SIZE_LARGE},
{XVW_NODE_REMOTE_LOCATION, NULL, XtRString, sizeof(String),
        offset(machine), XtRImmediate, (XtPointer) "localhost"},
{XVW_NODE_STATUS, NULL, XtRBoolean, sizeof(Boolean),
        offset(status), XtRImmediate, (XtPointer) 0},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwNodeWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwNodeWidgetRec, core.border_width),
        XtRImmediate, (XtPointer) 1},
{XVW_BUFFER_DIST, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwNodeWidgetRec, manager.buffer_dist),
        XtRImmediate, (XtPointer) 0},
{XVW_DEF_HORIZ_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwNodeWidgetRec, manager.horiz_offset),
	XtRImmediate, (XtPointer) 2},
{XVW_DEF_VERT_DIST, NULL, XtRPosition, sizeof(Position),
        XtOffsetOf(XvwNodeWidgetRec, manager.vert_offset),
	XtRImmediate, (XtPointer) 2},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Node Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwManagerWidgetClassRec)

XvwNodeWidgetClassRec xvwNodeWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Node",				/* class_name		  */
    sizeof(XvwNodeWidgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    ClassPartInitialize,		/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwNodeWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    ChangeSelection,			    /* change selected proc   */
    NULL,			   	    /* erase selected proc    */
    NULL,	   	    		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* firing rule check  */
    NULL,  		                    /* execute node       */
    NULL, 		                    /* save node          */
    NULL, 		                    /* restore node       */
    NULL, 		                    /* build command      */
    NULL,  		                    /* extension	  */
  },  /* XvwNodeWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwNodeWidgetClass = (WidgetClass) &xvwNodeWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)	     (XvwNodeWidget) (widget)
#define kwidgetclass(widget) (XvwNodeWidgetClass) (widget)->core.widget_class

/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwNodeWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwNodeWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/Node");

	xvw_define_attributes(xvwNodeWidgetClass,
		XVW_NODE_RESET,    XtRInt,    ResetNode,   NULL,
		XVW_NODE_EXECUTE,  XtRInt,    SetExecute,  GetExecute,
		XVW_NODE_RUNNABLE, XtRInt,    NULL,        GetExecute,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitialize
|
|       Purpose: This method is called when the first time an
|                instance of XvwNodeWidgetClass or a subclass of
|                the XvwNodeWidgetClass is created.  It handles the
|                inheritence of the run_operator method from the superclass
|                to the subclass.
|
|         Input: wclass - the widget class
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitialize(
   WidgetClass wclass)
{
        XvwNodeWidgetClass mclass = (XvwNodeWidgetClass) wclass;
        XvwNodeWidgetClass sclass = (XvwNodeWidgetClass)
                                                wclass->core_class.superclass;
 
        if (wclass != xvwNodeWidgetClass)
        {
           if (mclass->node_class.exec_node == XtInheritExecNode)
              mclass->node_class.exec_node = sclass->node_class.exec_node;

           if (mclass->node_class.check_node == XtInheritCheckNode)
              mclass->node_class.check_node = sclass->node_class.check_node;

           if (mclass->node_class.save_node == XtInheritSaveNode)
              mclass->node_class.save_node = sclass->node_class.save_node;

           if (mclass->node_class.restore_node == XtInheritRestoreNode)
              mclass->node_class.restore_node = sclass->node_class.restore_node;

           if (mclass->node_class.build_command == XtInheritBuildCommand)
              mclass->node_class.build_command=sclass->node_class.build_command;
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial node
|                widget instance.
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: May 26, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwNodeWidget xwid = kwidget(new);
	xvobject object = xvw_object(new);
	xvobject parent = xvw_parent(object);
	XvwNodeWidgetClass class = kwidgetclass(new);

	xwid->core.x = -1;
	xwid->core.y = -1;
	xwid->node.pid = 0;
	xwid->node.isize   = 0;
	xwid->node.osize   = 0;
	xwid->node.iports  = NULL;
	xwid->node.oports  = NULL;
	xwid->node.contrun = FALSE;
	xwid->node.running = FALSE;
	xwid->node.khoros_routine = FALSE;
	xwid->node.modified_pixmap = XtUnspecifiedPixmap;
	xwid->node.machine = kstrdup(xwid->node.machine);
	xwid->node.name = kstrdup("none");
	xwid->node.runnable = (class->node_class.exec_node != NULL);

	xwid->node.remote = xvw_create_pixmap(object, "NodePixmap");
	xvw_set_attributes(xwid->node.remote,
		XVW_BELOW,           NULL,
	        XVW_LEFT_OF,         NULL,
                XVW_RIGHT_OF,        NULL,
	        XVW_MAPPED,          FALSE,
                XVW_PIXMAP_FILENAME, REMOTE_PIXMAP,
                NULL);

	xwid->node.icon = xvw_create_pixmap(object, "NodeIcon");
        xvw_set_attributes(xwid->node.icon,
                XVW_BELOW,      xwid->node.remote,
                XVW_ABOVE,      NULL,
                XVW_LEFT_OF,    NULL,
                XVW_RIGHT_OF,   NULL,
		NULL);

        xwid->node.label = xvw_create_label(parent, "NodeLabel");
        xvw_set_attributes(xwid->node.label,
                XVW_BELOW,      object,
                XVW_LEFT_OF,    object,
                XVW_RIGHT_OF,   object,
                XVW_SELECTABLE, FALSE,
                XVW_RESIZABLE,  FALSE,
                XVW_MENUABLE,   FALSE,
                XVW_VERT_DIST,  new->core.border_width,
                NULL);
	
        xvw_insert_event(xwid->node.label,
		ButtonPressMask | ButtonReleaseMask |
	        EnterWindowMask | LeaveWindowMask, 
                NodeHandler, object, KLIST_HEAD);
        xvw_insert_event(xwid->node.remote, 
		ButtonPressMask | ButtonReleaseMask |
	        EnterWindowMask | LeaveWindowMask, 
		NodeHandler, xvw_object(new), KLIST_HEAD);
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This method will set up the window attributes
|                and create a window for the XvwNodeWidget.
|                The desired initial window attributes are passed
|                in via a mask and list of specification attributes.
|                The gc is also initialized here.
|
|         Input: widget     - the widget to realize
|                valuemask  - determines the window attributes being passed in
|                attributes - the list of window attributes to be set
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *attrib)
{
        XvwNodeWidget xwid = kwidget(widget);

	Window   rw, cw;
	unsigned int mask;
	int      x, y, wx, wy, rx, ry, snap, xsnap = 10, ysnap = 10, dist = 0;
 
 
	if (xwid->core.x == -1 && xwid->core.y == -1)
	{
	   if (xwid->node.placement == FALSE ||
	       xvw_check_visible(xvw_object(XtParent(widget))) == FALSE)
	   {
	      XQueryPointer(XtDisplay(widget), XtWindow(XtParent(widget)), &rw,
		&cw, &rx, &ry, &wx, &wy, &mask);

	      x = wx - ((int) xwid->core.width/2);
	      if (x < 0) x = 0;

	      y = wy - ((int) xwid->core.height/2);
	      if (y < 0) y = 0;
	   }
	   else
	   {
	      xvw_getposition(xvw_object(widget), &x, &y);
	      if (widget->core.being_destroyed == TRUE)
	         return;
	   }
/*
 *  temporary kludge.  really need to make this part of the layout method...
 */
	   xvw_get_attributes(xvw_parent(xvw_object(widget)),
		  XVW_XSNAP, &xsnap, XVW_YSNAP, &ysnap, XVW_BUFFER_DIST, &dist,
		  NULL);
	   snap = (x-dist) % xsnap;
	   x += (snap > xsnap/2) ? xsnap - snap : -snap;
	   snap = (y-dist) % ysnap;
	   y += (snap > ysnap/2) ? ysnap - snap : -snap;
	   xwid->core.x = x+xwid->core.border_width;
	   xwid->core.y = y+xwid->core.border_width;
	}

	/*
	 *  Now do the important thing.  Realize the Node so that we can
	 *  get on with it.
	 */
        (*xvwNodeWidgetClass->core_class.superclass->core_class.realize)
                (widget, valuemask, attrib);

	xvw_set_attribute(xwid->node.label, XVW_LABEL, xwid->node.name);
	xwid->node.modified_pixmap = XmuCreateStippledPixmap(XtScreen(widget),
		xwid->core.background_pixel, xwid->manager.bottom_shadow,
		xwid->core.depth);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwNodeWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the node.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num)
{
	XvwNodeWidget cxwid = kwidget(current);
	XvwNodeWidget nxwid = kwidget(new);

	int i, modified;

	/*
	 *  set the name of the node
	 */
	if (cxwid->node.name != nxwid->node.name)
	{
	   kfree_and_NULL(cxwid->node.name);
	   nxwid->node.name = kstrdup(nxwid->node.name);
	   if (XtWindowOfObject(new))
	      xvw_set_attribute(nxwid->node.label, XVW_LABEL, nxwid->node.name);
	}

        if (cxwid->node.machine != nxwid->node.machine)
        {
           kfree(cxwid->node.machine);
           nxwid->node.machine = kstring_cleanup(nxwid->node.machine, NULL);
           if (nxwid->node.machine == NULL)
              nxwid->node.machine = kstrdup("localhost");

	   for (i = 0; i < nxwid->node.isize; i++)
	   {
	      xvw_set_attribute(nxwid->node.iports[i],
			XVW_NODE_REMOTE_LOCATION, nxwid->node.machine);
	   }

	   for (i = 0; i < nxwid->node.osize; i++)
	   {
	      xvw_set_attribute(nxwid->node.oports[i],
			XVW_NODE_REMOTE_LOCATION, nxwid->node.machine);
	   }
        }

	if (cxwid->node.pixmap != nxwid->node.pixmap)
	   xvw_set_attribute(nxwid->node.icon, XVW_PIXMAP, nxwid->node.pixmap);

	if (cxwid->node.modified != nxwid->node.modified)
	{
	   modified = nxwid->node.modified;
	   for (i = 0; !modified && i < nxwid->node.isize; i++)
	      xvw_set_attribute(nxwid->node.iports[i], XVW_PORT_MODIFIED,FALSE);

	   XtCallCallbacks(new, XVW_NODE_MODIFIED_CALLBACK, &modified);
	}

	if (cxwid->node.remote_enable != nxwid->node.remote_enable)
        {
	   if (nxwid->node.remote_enable)
	   {
	      xvobject object = xvw_object(new);
	      if (!xvw_check_subclass(object, ConditionalWidgetClass) && 
	          !xvw_check_subclass(object, ExtPortWidgetClass))
	          xvw_map(nxwid->node.remote);
	   }
	   else
	      xvw_unmap(nxwid->node.remote);
        }

	if (cxwid->node.nodesize != nxwid->node.nodesize)
	{
	   int dist;

	   if (nxwid->node.nodesize == KNODE_SIZE_LARGE)
	      dist = 2;
	   else if (nxwid->node.nodesize == KNODE_SIZE_MEDIUM)
	      dist = 1;
	   else if (nxwid->node.nodesize == KNODE_SIZE_SMALL)
	      dist = 1;

	   xvw_set_attributes(xvw_object(new),
		XVW_DEF_HORIZ_DIST, dist,
		XVW_DEF_VERT_DIST, dist,
		NULL);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the node...
|         Input: widget - the node widget being destroyed
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwNodeWidget xwid = kwidget(widget);
	Widget parent =  XtParent(widget);


        if (!parent->core.being_destroyed && xwid->node.label != NULL)
           xvw_destroy(xwid->node.label);

	xwid->node.label = NULL;
        kfree_and_NULL(xwid->node.name);
        kfree_and_NULL(xwid->node.machine);
	kfree_and_NULL(xwid->node.iports);
	kfree_and_NULL(xwid->node.oports);
}

/*-----------------------------------------------------------
|
|  Routine Name: ChangeSelection
|
|       Purpose: Reduce or increase the border width depending on whether
|		 we are selected or not.
|         Input: widget - the node object which is having it's selection
|			  changed
|    Written By: Mark Young
|          Date: Dec 08, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ChangeSelection(
   Widget widget,
   int	  selected)
{
        XvwNodeWidget xwid = (XvwNodeWidget) widget;

	if (selected == TRUE)
	{
	   xvw_raise(xvw_object(widget));
	   xvw_set_attributes(xvw_object(widget),
		XVW_BACKGROUND_PIXMAP, xwid->node.modified_pixmap,
		XVW_SHADOW_THICKNESS, 3,
		XVW_SHADOW_TYPE, KSHADOW_IN,
		NULL);
	}
	else
	   xvw_set_attributes(xvw_object(widget),
		XVW_BACKGROUND_PIXMAP, XtUnspecifiedPixmap,
		XVW_SHADOW_THICKNESS, 2,
		XVW_SHADOW_TYPE, KSHADOW_OUT,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ResetNode
|
|       Purpose: ResetNode is used to reset the node.
|         Input: node  - the object in which we will be resetting
|		 attribute - the XVW_NODE_RESET attribute
|		 calldata  - not used
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int ResetNode(
   xvobject node,
   char     *attribute,
   kaddr    calldata)
{
	Widget widget = xvw_widget(node);
	XvwNodeWidget xwid = (XvwNodeWidget) widget;

	int i;

	for (i = 0; i < xwid->node.isize; i++)
	{
	   XvwPortGadget xobj = (XvwPortGadget)xvw_widget(xwid->node.iports[i]);
	   xobj->port.kludge = FALSE;
	   xvw_set_attribute(xwid->node.iports[i], XVW_PORT_MODIFIED, FALSE);
	}

	for (i = 0; xwid->node.runnable && i < xwid->node.osize; i++)
	   xvw_set_attribute(xwid->node.oports[i], XVW_PORT_DAV, FALSE);

	xwid->node.modified = FALSE;
	xvw_set_attribute(node, XVW_NODE_MODIFIED, TRUE);
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: GetExecute
|
|       Purpose: GetExecute is used to get whether the node is currently
|		 running or not.
|
|         Input: node      - the object in which we will be getting the
|			     status for
|		 attribute - the execute attribute
|        Output: calldata  - pointer to retrieve the status for
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int GetExecute(
   xvobject node,
   char     *attribute,
   kaddr    calldata)
{
	int *execute = (int *) calldata;
	XvwNodeWidget xwid = (XvwNodeWidget) xvw_widget(node);


	if (kstrcmp(attribute, XVW_NODE_EXECUTE) != 0 &&
	    kstrcmp(attribute, XVW_NODE_RUNNABLE) != 0)
	{
	   kerror(XVLANG, "GetExecute", "invalid attribute %s", attribute);
	   return(FALSE);
	}

	if (kstrcmp(attribute, XVW_NODE_RUNNABLE) == 0)
	{
	   XvwNodeWidgetClass class = kwidgetclass(xwid);

	   if (xwid->node.runnable == FALSE)
	      *execute = FALSE;
	   else if (xwid->node.running == TRUE)
	      *execute = FALSE;
	   else if (class->node_class.check_node != NULL)
	      *execute = class->node_class.check_node((Widget) xwid);
	   else
	      *execute = TRUE;
	}
	else
	   *execute = xwid->node.running;

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetExecute
|
|       Purpose: GetExecute is used to whether we should execute the node
|		 or not
|
|         Input: node  - the object in which we will be setting whether
|			     to execute or not
|		 attribute - the old object to be closed
|		 calldata  - the filename to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static int SetExecute(
   xvobject node,
   char     *attribute,
   kaddr    calldata)
{
	Widget widget = xvw_widget(node);
	int    execute = *((int *) calldata);
	XvwNodeWidget xwid = (XvwNodeWidget) widget;

	int  i, running, status, type;

	if (kstrcmp(attribute, XVW_NODE_EXECUTE) != 0)
	{
	   kerror(XVLANG, "SetExecute", "invalid attribute %s", attribute);
	   return(FALSE);
	}

	/*
	 *  Execute the sucker if ain't modified and it ain't currently
	 *  being executed or waiting to be executed...
	 */
	if (xwid->node.runnable == FALSE)
	   return(TRUE);
	else if (execute == TRUE && xwid->node.modified == FALSE)
	   return(TRUE);
	 
	type = xwid->node.exectype;
	xvw_get_attribute(node, XVW_NODE_EXECUTE, &running);
	if (execute && running == execute)
	   return(TRUE);

	if (execute == TRUE)
	{
	   XvwNodeWidgetClass class;

	   xvw_get_attribute(node, XVW_NODE_RUNNABLE, &status);
	   if (status == FALSE)
	      return(TRUE);

	   /*
	    *  Start the sucker...
	    */
	   xwid->node.status = 0;
	   xwid->node.running = running = TRUE;
	   xwid->node.khoros_routine = FALSE;
	   class = (XvwNodeWidgetClass) widget->core.widget_class;
	   class->node_class.exec_node(widget, TRUE);
	   XtCallCallbacks(widget, XVW_NODE_RUN_CALLBACK, &running);
	}

	if (execute == FALSE)
	{
	   XvwNodeWidgetClass class;

	   for (i = 0; i < xwid->node.isize; i++)
	      xvw_set_attribute(xwid->node.iports[i], XVW_PORT_MODIFIED, FALSE);

	   /*
	    *  Stop the sucker...
	    */
	   running = xwid->node.contrun = xwid->node.running = FALSE;
	   class = (XvwNodeWidgetClass) widget->core.widget_class;
	   class->node_class.exec_node(widget, FALSE);
	   XtCallCallbacks(widget, XVW_NODE_RUN_CALLBACK, &running);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: NodeHandler
|
|       Purpose: This routine will perform the changing of the node name.
|
|         Input: object      - the button being activated
|                client_data - the node widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young & Danielle Argiro
|          Date: Apr 18, 1995
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void NodeHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	xvobject node = (xvobject) client_data;
	XvwNodeWidget xwid = (XvwNodeWidget) xvw_widget(node);

	static int working = FALSE;
	static xvobject selected_object = NULL;


	*dispatch = FALSE;
	if (working) return;

	working = TRUE;
	if (event->type == ButtonPress)
	{
	   selected_object = object;
	   working = FALSE;
	   return;
	}
	else if (event->type != ButtonRelease || object != selected_object)
	{
	   working = FALSE;
	   if (event->type == EnterNotify)
	   {
	      kannounce(XVLANG, "NodeHandler", "Current host for execution: %s",
		  xwid->node.machine);
	   }
	   else if (event->type == LeaveNotify)
	      kannounce(XVLANG, "NodeHandler", " ");
	   return;
	}

	if (object == xwid->node.label)
	{
	   char *prompt = "Node Name: ", temp[KLENGTH], *answer = temp;

	   kstrcpy(answer, xwid->node.name);
	   if (xvu_query_wait("Please enter new Icon Name", &prompt, "OK",
                              &answer, 1, 35) && kstrlen(answer) > 0)
           {
	      xvw_set_attribute(node, XVW_NODE_NAME, answer);
           }
	}
	else if (object == xwid->node.remote)
	{
	   size_t num;
	   int    choice;
	   char   *new_machine = NULL, **machines = NULL;

           *dispatch = FALSE;
	   kannounce(XVLANG, "NodeHandler", "Current host for execution: %s",
		  xwid->node.machine);

	   /*
            * get list of remote machines for distributed computing
            */
           machines = kremote_list(&num);
           if (num == 0 || machines == NULL)
	   {
	       kerror(XVLANG, "NodeHandler", "No machines available for "
		      "remote execution");
	       working = FALSE;
	       return;
	   }

	   /*
            *  Prompt the user for which one they want.
            */
           choice = xvu_choose_wait("Machine to execute process on:",
			machines, (int) num, -1, FALSE, &new_machine);
           if (new_machine == NULL) 
	   {
	       working = FALSE;
	       return;
	   }

           /*
            *  Set the machine in which to execute the glyph on
            */
	   xvw_set_attribute(node, XVW_NODE_REMOTE_LOCATION, new_machine);

           /*
            * If the choice was not on the list, then add it.  We know
            * if wasn't part of the list if the index was -1.
            */
           if (choice == -1)
              khost_add(new_machine);

           kfree(new_machine);
	}
	working = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: NodeSave - Save a node object
|
|       Purpose: This routine calls the node's method which will do the
|		 actually saving of the node object to the workspace file.
|
|         Input: file  - the file the information is suppose to be written to
|		 node  - the node object to be saved
|		 ident - the node identifier
|        Output:
|    Written By: Mark Young
|          Date: Jun 6, 1996
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void NodeSave(
    xvobject node,
    kfile    *file,
    int      ident)
{
	XvwNodeWidgetClass class = (XvwNodeWidgetClass) xvw_class(node);

	if (class && class->node_class.save_node != NULL)
	   class->node_class.save_node(node, file, ident);
}

/*-----------------------------------------------------------
|
|  Routine Name: NodeRestore - Restore a node object
|
|       Purpose: This routine calls the node's method which will do the
|                actually restoring of the node object from the workspace file.
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|        Output: ident - the node indentifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
xvobject NodeRestore(
    xvobject parent,
    kfile    *file,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    int      *ident)
{
	xvclass  class = NULL;
	xvobject node  = NULL;
	char     *name = NULL;
	XvwNodeWidgetClass nodeclass;

	if (kstring_subcmp(line, "Glyph:") == 0)
	   class = GlyphWidgetClass, name = "Glyph";
	else if (kstring_subcmp(line, "Conditional:") == 0)
	   class = ConditionalWidgetClass, name = "ConditionalGlyph";
	else if (kstring_subcmp(line, "ProcedureBegin:") == 0)
	   class = ProcedureWidgetClass, name = "Procedure";
	else if (kstring_subcmp(line, "LoopBegin:") == 0)
	   class = LoopWidgetClass, name = "Loop";
	else if (kstring_subcmp(line, "ExtPort:") == 0)
	   class = ExtPortWidgetClass, name = "ExtPort";
	else
	{
	   return(NULL);
	}

        if ((node = xvw_create(parent, FALSE, TRUE, name, class)) == NULL)
	   return(NULL);

	xvw_set_attributes(node,
		XVW_SELECTABLE, TRUE,
		XVW_RESIZABLE, FALSE,
		XVW_MENUABLE, FALSE,
		NULL);

	nodeclass = (XvwNodeWidgetClass) class;
	if (nodeclass && nodeclass->node_class.restore_node != NULL)
	{
	   if (!(nodeclass->node_class.restore_node(node, parent, file, line,
				xoffset, yoffset, ident)))
	   {
	      xvw_destroy(node);
	      return(NULL);
	   }
	}
	return(node);
}

/*-----------------------------------------------------------
|
|  Routine Name: PortModifiedCallback - modified change on our output
|
|       Purpose: This routine is used to detect when a change to a glyph's
|		 input data has ocurred. 
|
|         Input: oport - the output port that changed it's modified status
|    Written By: Mark Young
|          Date: Jun 14, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void PortModifiedCallback(
   Widget   widget,
   kaddr    client_data,
   kaddr    call_data)
{
	pid_t pid;
	char  *name;
	kfile *file;
	int modified = *((int *) call_data);
	XvwNodeWidget xwid = (XvwNodeWidget) client_data;

	if (xwid->node.running == FALSE || modified == FALSE)
	{
	   XtRemoveCallback(widget, XVW_PORT_MODIFIED_CALLBACK,
		PortModifiedCallback, client_data);
	}

	if (xwid->node.running && !modified && (pid = xwid->node.pid) > 0 &&
	    xvw_getinfo_handler(pid, NULL, NULL, NULL, NULL, &file) && file)
	{
	   xvobject oport = xvw_object(widget);

	   xvw_get_attribute(oport, XVW_PORT_FILENAME, &name);
	   if (kfile_getpermanence(name) == TRUE)
	      xvw_set_attribute(oport, XVW_PORT_DAV, FALSE);
	   else
	      xvw_set_attribute(oport, XVW_PORT_DAV, TRUE);

	   kfputs("yes\n", file);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: NodeIpcCallback - routine for handling ipc messages
|
|       Purpose: This routine is called when one of the sub-process wants to
|                inform of some process update
|
|         Input: id  - the callback id
|                fid - the input file descriptor
|                client_data - not used
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Dec 12, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
void NodeIpcCallback(
   kfile *file,
   char  *type,
   char  *request,
   kaddr client_data)
{
	Widget widget = (Widget) client_data;
	XvwNodeWidget xwid = kwidget(client_data);

	XvwPortGadget xobj;
	int i, size, running, dav;
	xvobject *ports, object = NULL;
	char command[KLENGTH], name[KLENGTH], flags[KLENGTH], temp[KLENGTH];

	if (!xwid || xwid->core.being_destroyed == TRUE)
	   return;

	xwid->node.khoros_routine = TRUE;
	if (kstrcmp(type, KIPC_TRANSPORT) == 0)
	{
           if (ksscanf(request, "%*[ ]%[^' ]%*[ ]%*[']%[^']%*[']%*[ ]%*[']"
		       "%[^']%*[']", command, name, flags) != 3 ||
               (kstrcmp("open",  command) != 0 &&
	        kstrcmp("close", command) != 0 &&
	        kstrcmp("check", command) != 0))
           {
              kfputs("munged\n", file);
	      kfprintf(kstderr, "MUNGED %s: '%s'\n", type, request);
              return;
           }

	   kfullpath(name, NULL, name);
	   if (kstrcmp("check", command) == 0)
	      size = xwid->node.isize, ports = xwid->node.iports;
	   else
	      size = xwid->node.osize, ports = xwid->node.oports;

	   for (i = 0; i < size; i++)
	   {
	      xobj = (XvwPortGadget) xvw_widget(ports[i]);
	      if (!kstrcmp(xobj->port.filename, name) ||
		  !kstrcmp(kfullpath(xobj->port.filename, NULL, temp), name))
	      {
		 object = ports[i];
		 break;
	      }
           }

	   if (object == NULL)
	   {
	      xvl_report(FALSE, "NodeIpcCallback: no port for '%s'\n", name);
	      kfputs("unknown\n", file);  /* ack unknown response */
	      return;
	   }
	   else if (kstrcmp("open", command) == 0)
	   {
	      if (xobj->port.modified == TRUE)
	      {
	         XtAddCallback((Widget) xobj, XVW_PORT_MODIFIED_CALLBACK,
			PortModifiedCallback, xwid);
		 return;
	      }

	      /* if a permanent transport then data available is false */
	      dav = kfile_getpermanence(name) == FALSE;
	      xvw_set_attribute(object, XVW_PORT_DAV, dav);
	      kfputs("yes\n", file);  /* ack a positive response */
	   }
	   else if (kstrcmp("close", command) == 0)
	   {
	      /* if a permanent transport then data available is true */
	      dav = kfile_getpermanence(name) == TRUE;
	      xvw_set_attribute(object, XVW_PORT_DAV, dav);
	      kfputs("yes\n", file);  /* ack a positive response */
	   }
	   else if (!kstrcmp("check", command))
	   {
	      if (xobj->port.modified == TRUE)
	         kfputs("yes\n", file);  /* ack a positive response */
	      else
	         kfputs("no\n", file);  /* ack a negative response */
	   }
	}
	else if (kstrcmp(type, KIPC_PROCESS) == 0)
	{
	   request = kstring_cleanup(request, name);
	   if (kstrcmp(request, "initialize contrun") == 0)
	       xwid->node.contrun = TRUE;
	   else if (kstrcmp(request, "initialize process") == 0)
	       xwid->node.khoros_routine = TRUE;
	   else if (kstrcmp(request, "next contrun") == 0)
	   {
	      xwid->node.running = running = FALSE;
	      xvw_set_attribute(xvw_object(widget), XVW_NODE_MODIFIED, FALSE);
	      XtCallCallbacks(widget, XVW_NODE_RUN_CALLBACK, &running);
	   }
	   else if (kstrcmp(request, "exit process") == 0)
	   {
	      if (xwid->node.pid != 0)
	      {
	         xwid->node.pid = 0;
	         xwid->node.status = 0;
	         xvw_set_attributes(xvw_object(widget),
	    	         XVW_NODE_MODIFIED, FALSE,
	    	         XVW_NODE_EXECUTE, FALSE,
		         NULL);
	      }
              kfputs("exit\n", file);
	   }
	   else
	      kfputs("exit\n", file);

	   xvl_report(FALSE, "%s: %d '%s'\n", type, xwid->node.pid, request);
	}
}


/************************************************************
*
*  Routine Name: xvw_create_node - create an node object
*
*       Purpose: The node object provides the base class which is 
*		 used to create iconic representations of textual
*		 language routines.  The node object is the object from 
*		 which the \fIglyph\fP objects are subclassed, and should
*		 be used as the base class for any new visual programming
*		 objects that might be created for textual code 
*		 representation.
*		
*		 The node object consists of a small window
*		 with an optional pixmap in the middle, and a 
*		 name that appears below the window.
*		 The node object supports interactive placement;
*		 the user can place the node using a shadow outline,
*		 or the (x, y) position may be specified by the
*		 application.
*
*		 A node object allows (1) specification of the 
*		 corresponding routine to be executed, and (2) rules 
*		 for determining when that routine may be executed.
*
*		 When the routine to be executed is specified as an
*		 external process, the Node object does "large grain"
*		 execution, or distributed execution of operators 
*		 representing individual programs, as the Glyph object does.
*		 On the other hand, when the routine to be executed is
*		 specified as an internally defined function, the node object 
*		 does ""fine grain" execution, or the calling of a subroutine 
*		 or function internal to the visual programming language itself.
*		 Thus, the Node object can support both large grain and
*		 fine grain visual programming.
*
*		 The node object can be used to create new iconic 
*		 representations; thus, it allows the \fIxvlang\fP library 
*		 to be extended to support visual programming constructs 
*		 other than the ones currently used in \fHcantata\fP.
*
*		 When the Node object is used as the base class for iconic 
*		 representation of modules and the Workspace object is utilized 
*		 as the visual editor, any new visual object that is written
*		 to be subclassed off the Node will automatically be supported 
*		 by the existing system.
*
*		 In addition, any such new visual programming objects will be 
*		 capable of coexisting with existing visual programming 
*		 objects. They can be combined in a visual network as desired; 
*		 allowed combinations are only restricted by the visual 
*		 programming model, as appropriate, but not by the visual 
*		 programming objects themselves.
*
*         Input: parent - parent of the node object; NULL will cause
*                         a default toplevel to be created automatically
*                name   - name with which to reference node object
*
*        Output: 
*       Returns: The node widget on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 14, 1993
*      Verified: 
*  Side Effects: 
* Modifications:
*
*******************************************************************/

xvobject xvw_create_node(
   xvobject parent,
   char     *name)
{
	xvobject node;

	node = xvw_create(parent, FALSE, TRUE, name, NodeWidgetClass);
	xvw_set_attributes(node,
		XVW_SELECTABLE, TRUE,
		XVW_RESIZABLE, FALSE,
		XVW_MENUABLE, FALSE,
		NULL);
	return(node);
}
