 /*
  * Khoros: $Id: Procedure.c,v 1.29.2.1 1996/12/06 20:29:59 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: Procedure.c,v 1.29.2.1 1996/12/06 20:29:59 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Procedure Widget
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 ClassPartInitialize()
   >>>>                 Initialize()
   >>>>                 SetValues()
   >>>>                 Destroy()
   >>>>                 ProcedureExec()
   >>>>                 ProcedureSave()
   >>>>                 ProcedureRestore()
   >>>>                 ProcedureRunCallback()
   >>>>                 ProcedureHandler()
   >>>>   Private:
   >>>>                 ProcedureLocate()
   >>>>   Public:
   >>>>                 xvw_create_procedure()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/ProcedureP.h>
#include <kwidgets/ViewportP.h>

static void ClassInitialize PROTO((void));
static void ClassPartInitialize PROTO((WidgetClass));
static void Initialize       PROTO((Widget, Widget, ArgList, Cardinal *));
static Boolean SetValues     PROTO((Widget, Widget, Widget,ArgList,Cardinal *));
static void    Destroy	     PROTO((Widget));
static void ProcedureExec    PROTO((Widget, int));
static void ProcedureSave    PROTO((xvobject, kfile *, int));
static int  ProcedureRestore PROTO((xvobject, xvobject, kfile *, char *, int,
				    int, int *));

static void ProcedureHandler	 PROTO((xvobject, kaddr, XEvent *, int *));
static void ProcedureRunCallback PROTO((xvobject, kaddr, kaddr));
static int  ProcedureActions     PROTO((xvobject, char *, kaddr));

klist *procedurelist = NULL;

/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/

#define OPEN_PIXMAP "$DESIGN/objects/library/xvlang/misc/control/procedure_open.xpm"
#define CLOSE_PIXMAP "$DESIGN/objects/library/xvlang/misc/control/procedure_close.xpm"
#define PROCEDURE_PIXMAP "$DESIGN/objects/library/xvlang/misc/control/procedure.xpm"
/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_PROCEDURE_PIXMAP,       NULL, XtRPixmap, NULL},
{XVW_PROCEDURE_OPEN_PIXMAP,  NULL, XtRPixmap, NULL},
{XVW_PROCEDURE_CLOSE_PIXMAP, NULL, XtRPixmap, NULL},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwProcedureWidgetRec, procedure.field)

static XtResource resources[] = { 
{XVW_PROCEDURE_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(procedure_pixmap), XtRString, (XtPointer) PROCEDURE_PIXMAP},
{XVW_PROCEDURE_OPEN_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(open_pixmap), XtRString, (XtPointer) OPEN_PIXMAP},
{XVW_PROCEDURE_CLOSE_PIXMAP, NULL, XtRPixmap, sizeof(Pixmap),
        offset(close_pixmap), XtRString, (XtPointer) CLOSE_PIXMAP},
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for Procedure Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwGlyphWidgetClassRec)

XvwProcedureWidgetClassRec xvwProcedureWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "Procedure",			/* class_name		  */
    sizeof(XvwProcedureWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    ClassPartInitialize,		/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    XtInheritRealize,			/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,			/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwProcedureWidgetConstraintsRec),/* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,			    /* pre-layout routine     */
    XtInheritLayout,			    /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    XtInheritCheckNode,	                    /* firing rule check  */
    ProcedureExec,  	                    /* execute node       */
    ProcedureSave, 	                    /* save node          */
    ProcedureRestore, 	                    /* restore node       */
    NULL, 		                    /* build command      */
    NULL,  		                    /* extension          */
  },  /* XvwNodeWidgetClass fields initialization */
  {
    NULL,  		                    /* extension           */
  },  /* XvwGlyphWidgetClass fields initialization */
  {
    NULL,  		                    /* run callback       */
    NULL,  		                    /* extension          */
  },  /* XvwProcedureWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwProcedureWidgetClass = (WidgetClass) &xvwProcedureWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for Class and Constraint Declarations
|
--------------------------------------------------------------------*/

#define kwidget(widget)	    (XvwProcedureWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwProcedureWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Mark Young
|          Date: Oct 19, 1993
|
------------------------------------------------------------*/

static void ClassInitialize(void)
{
	xvw_init_attributes(xvwProcedureWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvlang/uis/Procedure.pane");
	xvw_load_resources("$DESIGN/objects/library/xvlang/app-defaults/Procedure");
	xvw_define_attributes(xvwProcedureWidgetClass,
		XVW_PROCEDURE_RESTORE, XtRString, ProcedureActions, NULL,
		NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: ClassPartInitialize
|
|       Purpose: This method is called when the first time an
|                instance of XvwProcedureWidgetClass or a subclass of
|                the XvwProcedureWidgetClass is created.  It handles the
|                inheritence of the run_callback method from the superclass
|                to the subclass.
|
|         Input: wclass - the widget class
|    Written By: Mark Young
|          Date: Dec 11, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassPartInitialize(
   WidgetClass wclass)
{
        XvwProcedureWidgetClass mclass = (XvwProcedureWidgetClass) wclass;
        XvwProcedureWidgetClass sclass = (XvwProcedureWidgetClass)
                                                wclass->core_class.superclass;
 
        if (wclass != xvwProcedureWidgetClass)
        {
           if (mclass->procedure_class.run_callback == XtInheritRunCallback)
              mclass->procedure_class.run_callback =
				sclass->procedure_class.run_callback;
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|       Purpose: This method will set up the initial procedure
|                widget instance.
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: May 26, 1993 13:28
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
        XvwProcedureWidget xwid = kwidget(new);
	xvobject object = xvw_object(new);

	xvobject workspace;
	unsigned long mask;
	kform *form = NULL;
	Widget parent = XtParent(new);


	xwid->procedure.workspace = NULL;

	/*
	 *  Add a workspace pixmap button to the center of the procedure
	 *  so that we associate the pixmap with the procedure's workspace.
	 */
	xvw_set_attributes(object,
		XVW_NODE_PIXMAP, xwid->procedure.procedure_pixmap,
		NULL);

	xwid->procedure.open = xvw_create_pixmap(object, "open");
	xvw_set_attributes(xwid->procedure.open,
		XVW_PIXMAP,       xwid->procedure.open_pixmap,
		XVW_BELOW,	  NULL,
		XVW_LEFT_OF,      NULL,
		XVW_VERT_DIST,    2,
                NULL);
	mask = ButtonPressMask | ButtonReleaseMask |
	       EnterWindowMask | LeaveWindowMask;
	xvw_insert_event(xwid->procedure.open, mask, ProcedureHandler,
			object, KLIST_HEAD);

	/*
	 *  Now create the workspace for the procedure
	 */
	if (XtIsSubclass(XtParent(new), xvwViewportWidgetClass) == TRUE)
	{
	   XvwViewportWidget xpar = (XvwViewportWidget) parent;

	   xpar->viewport.override = TRUE;
	   workspace = xvw_create_workspace(xvw_object(parent),
				"ProcedureWorkspace");
	   xpar->viewport.override = FALSE;
	}
	else
	{
	   workspace = xvw_create_workspace(xvw_object(parent),
				"ProcedureWorkspace");
	}

	if (XtClass(parent) == xvwWorkspaceWidgetClass)
	    xvl_propagate_wksp_attributes(xvw_object(parent), workspace);

	xvw_unmanage(workspace);
	xvw_set_attributes(workspace,
		XVW_TACK_EDGE,  KMANAGER_TACK_ALL,
		XVW_VERT_DIST,  1,
		XVW_HORIZ_DIST, 1,
		NULL);

	/*
	 *  Get workspace gui for our procedure's user interface.  Also set
	 *  the expression id parent for the workspace to be our glyph
	 *  expression id.
	 */
        xvw_get_attribute(workspace, XVW_WORKSPACE_WKSPGUI, &form);
        kexpr_parent_set((long) workspace, (long) xwid->glyph.expression_id);
        xwid->glyph.form = form;
	xwid->procedure.workspace = xvw_widget(workspace);
        xwid->glyph.expression_id = workspace;

	/*
	 *  Add run callback for the procedure in order to know when we are
	 *  being run or finished running.  We use this for maintaining our
	 *  external inputs and outputs.
	 */
        xvw_insert_callback(workspace, XVW_WORKSPACE_RUN_CALLBACK, FALSE,
		ProcedureRunCallback, object);

	procedurelist = klist_add(procedurelist, 
		(kaddr) xwid->procedure.workspace, object);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: This method is used to set the public values
|                of a XvwProcedureWidget instance.  The public values
|                which can be changed are all related to the display
|		 of the procedure.
|
|         Input: current - the widget containing current settings
|                request - the widget containing requested settings
|                new     - the widget processed through all set values methods
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Aug 19, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num)
{
	XvwProcedureWidget cxwid = kwidget(current);
	XvwProcedureWidget nxwid = kwidget(new);
	xvobject workspace;

	if (nxwid->procedure.workspace != NULL)
	    workspace = xvw_object(nxwid->procedure.workspace);

	if (cxwid->node.modified != nxwid->node.modified &&
	    nxwid->node.modified && workspace != NULL)
	{
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RESET, TRUE);
	}

	if (cxwid->node.name != nxwid->node.name && workspace != NULL)
	{
	   xvw_set_attribute(workspace, XVW_WORKSPACE_NAME,
				nxwid->node.name);
	}

        if (cxwid->node.machine != nxwid->node.machine && 
	    workspace != NULL)
        {
           xvw_set_attribute(workspace, XVW_NODE_REMOTE_LOCATION,
                             nxwid->node.machine);
        }

        if (cxwid->node.remote_enable != nxwid->node.remote_enable && 
	    workspace != NULL)
        {
           xvw_set_attribute(workspace, XVW_NODE_REMOTE_ENABLE,
                             nxwid->node.remote_enable);
        }


        if (cxwid->glyph.expression_id != nxwid->glyph.expression_id && 
	    workspace != NULL)
	{
	   long id = (long) nxwid->glyph.expression_id;

           nxwid->glyph.expression_id = workspace;
	   kexpr_parent_set((long) workspace, id);
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: destroy the procedure...
|         Input: widget - the procedure widget being destroyed 
|    Written By: Mark Young
|          Date: Nov 21, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
	XvwProcedureWidget xwid = kwidget(widget);
	klist *entry = NULL;

	entry = klist_locate(procedurelist, xwid->procedure.workspace);
	if (entry != NULL)
	    procedurelist = klist_delete(procedurelist, 
				         xwid->procedure.workspace);
	xvw_destroy(xvw_object(xwid->procedure.workspace));
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureExec
|
|       Purpose: This routine will run or stop the execution of a procedure.
|         Input: widget      - the procedure widget to be run
|                exec        - whether we wish to run or stop the procedure
|    Written By: Mark Young
|          Date: Jul 30, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ProcedureExec(
   Widget   widget,
   int      exec)
{
	XvwProcedureWidget xwid = kwidget(widget);
	xvobject workspace = xvw_object(xwid->procedure.workspace);

	int i, running;

	if (exec == TRUE)
	{
	   /*
	    *  Make sure that our glyph data available is set to
	    *  false.
	    */
	   for (i = 0; i < xwid->node.osize; i++)
	      xvw_set_attribute(xwid->node.oports[i], XVW_PORT_DAV, FALSE);

	   xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND_COLOR, "red");
           xvw_set_attribute(workspace, XVW_WORKSPACE_RESET, TRUE);

           if (xwid->glyph.cli)
           {
              xvw_unmap(xwid->glyph.error);
              xvw_set_attribute(xwid->glyph.cli,XVW_TEXTDISPLAY_CLEARTEXT,TRUE);
           }
	   xvw_set_attribute(workspace, XVW_WORKSPACE_RUN, TRUE);
	}
	else
	{
	   /* Set run to FALSE so that the procedure stops running.  */
	   xvw_set_attribute(xwid->node.icon, XVW_BACKGROUND,
			xwid->core.background_pixel);

	   xvw_get_attribute(workspace, XVW_WORKSPACE_RUN, &running);

	   if (running)
	       xvw_set_attribute(workspace, XVW_WORKSPACE_STOP, TRUE);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureSave - Saves a procedure
|
|       Purpose: This routine does the actually saving of a procedure to a
|		 workspace file.
|
|         Input: procedure  - the glyph to be saved
|        	 file  - the file the information is suppose to be written to
|		 ident      - the procedure identifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ProcedureSave(
    xvobject procedure,
    kfile    *file,
    int      ident)
{
        XvwProcedureWidget xwid = (XvwProcedureWidget) xvw_widget(procedure);

	kform	 *form;
	int      xpos, ypos;
	char     *command, *name, *type, *oname, *machine, temp[KLENGTH];

        if (xvw_check_subclass(procedure, LoopWidgetClass))
	   type = "Loop";
	else
	   type = "Procedure";

	xvw_get_attributes(procedure,
		XVW_GLYPH_FORM,   &form,
		XVW_GLYPH_ONAME,  &oname,
		XVW_NODE_NAME,    &name,
		XVW_XPOSITION,    &xpos,
		XVW_YPOSITION,    &ypos,
		XVW_NODE_REMOTE_LOCATION, &machine,
		NULL);

	name  = !name  ? "(unknown)" : name;
	oname = !oname ? "(unknown)" : oname;
	machine = kstring_replace(machine, ":", "\\:", temp);

	if (form != NULL)
           command = kvf_construct_partial_cmd(form, form->subform, NULL,FALSE);
	else
	   command = "(unknown)";

	/* save the procedure information */
	kfprintf(file,"\n#  %s '%s'\n%sBegin:%s:%s:%s:%d:%d:%d:%s\n", type,name,
			type, name, oname, machine, ident, xpos, ypos, command);

	/*
	 *  Oh No!  It's a procedure, so we need to do the nasty.
	 */
	if (xwid->procedure.workspace != NULL)
	{
	   WorkspaceSave(xwid->procedure.workspace, file,
			FALSE, FALSE, FALSE);
	}
	kfprintf(file,"\n%sEnd:%s\n", type, name);
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureRestore - Restore the procedure
|
|       Purpose: This routine does the actually restoring of a glyph from a
|		 workspace file.
|
|         Input: file  - the file the information is suppose to be read from
|		 line  - the information just read
|		 xoffset,yoffset - initial x & y offset
|        Output: ident - the glyph indentifier
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ProcedureRestore(
    xvobject procedure,
    xvobject parent,
    kfile    *file,
    char     *line,
    int	     xoffset,
    int	     yoffset,
    int      *ident)
{
	XvwProcedureWidget xwid = (XvwProcedureWidget) xvw_widget(procedure);

	int      xpos, ypos;
	char     name[KLENGTH], oname[KLENGTH], identifier[KLENGTH],
	         command[10*KLENGTH], machine[KLENGTH];

	kstrcpy(oname, " "); kstrcpy(command, " "); kstrcpy(machine, " ");
	if (ksscanf(line, "%[^B]Begin:%[^:]:%[^:]:%[^:]:%d:%d:%d:%[^\n]",
		identifier, name, oname, machine, ident, &xpos, &ypos,
		&command[1]) != 8 &&
	   ksscanf(line, "%[^B]Begin:%[^:]:%[^:]:%d:%d:%d:%[^\n]", identifier,
		name, oname, ident, &xpos, &ypos, &command[1]) != 7 &&
	   ksscanf(line, "%[^B]Begin:%[^:]:%d:%d:%d:%[^\n]",
		identifier, name, ident, &xpos, &ypos, &command[1]) != 6)
	{
	   return(FALSE);
	}

	kstring_replace(machine, "\\:", ":", machine);
	xvw_set_attributes(procedure,
		XVW_NODE_NAME, name,
		XVW_NODE_REMOTE_LOCATION, machine,
		XVW_GLYPH_ONAME, oname,
		XVW_XPOSITION, xpos + xoffset,
		XVW_YPOSITION, ypos + yoffset,
		NULL);

	if (xwid->procedure.workspace != NULL)
	   WorkspaceRestore(xwid->procedure.workspace, file, 0, 0);

	if (xwid->glyph.form != NULL)
	{
	   GlyphUpdateForm((Widget) xwid);
	   xvf_modify_form_from_cmd(xwid->glyph.form, command);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureRunCallback
|
|       Purpose: This routine will be called on the workspace run callback.
|         Input: workspace   - the workspace for the run callback
|                client_data - the procedure widget
|                call_data   - whether we wish to run or stop
|    Written By: Mark Young
|          Date: Sep 06, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ProcedureRunCallback(
   xvobject workspace,
   kaddr    client_data,
   kaddr    call_data)
{
	int running    = *((int *) call_data);
	xvobject glyph = (xvobject) client_data;
	Widget widget  = xvw_widget(glyph);

	int status;
	char temp[KLENGTH];
	XvwProcedureWidgetClass class;
	XvwProcedureWidget xwid = (XvwProcedureWidget) widget;

	if (xwid->node.running == FALSE)
	   return;

	xvw_get_attribute(workspace, XVW_WORKSPACE_STATUS, &status);
	xwid->node.status = status;
	if (status != 0)
	{
	   xvw_set_attributes(glyph,
		  XVW_NODE_MODIFIED, TRUE,
		  XVW_NODE_EXECUTE, FALSE,
		  NULL);
	   ksprintf(temp, "\nThe workspace has completed running, but errors\n"
			  "were encountered.  Aborting scheduling any down-\n"
			  "stream operators.\n\n");
	   MessageCallback(NULL, (kaddr) widget, temp);
	   xvw_set_attribute(xwid->glyph.error, XVW_PIXMAP,
	     	     xwid->glyph.error_pixmap);
	   return;
	}

	class = (XvwProcedureWidgetClass) xwid->core.widget_class;
	if (class->procedure_class.run_callback == NULL)
	{
	   if (running == FALSE)
	   {
	      /*
	       *  Make sure that our glyph data available is set to true.  This
	       *  will also start any down stream glyphs, before we stop the
	       *  procedure.  Note:  this means that you must set DAV to true
	       *  before stopping the procedure.
	       */
	      xvl_set_extport_davs(workspace, xwid->node.oports, 
				   xwid->node.osize);

	      xvw_set_attributes(glyph,
			XVW_NODE_EXECUTE, FALSE,
			XVW_NODE_MODIFIED, FALSE,
			NULL);
	   }
	}
	else
	   class->procedure_class.run_callback(widget, running);
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureActions
|
|       Purpose: ProcedureActions is used to set the certain actions associated
|		 with the current procedure.
|
|         Input: object    - the object in which we will be setting the
|			     attribute value for
|		 attribute - the desired attribute
|		 calldata  - the value to be set
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 27, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int ProcedureActions(
   xvobject procedure,
   char     *attribute,
   kaddr    calldata)
{
	XvwProcedureWidget xwid = (XvwProcedureWidget) xvw_widget(procedure);

	char *filename;

	if (kstrcmp(attribute, XVW_PROCEDURE_RESTORE) == 0)
	{
	   filename = *((String *) calldata);
	   xvw_set_attribute(xvw_object(xwid->procedure.workspace), 
			     XVW_WORKSPACE_RESTORE, filename);
	   GlyphUpdateForm((Widget) xwid);
	}
	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureHandler
|
|       Purpose: This routine will perform a variety of procedure callbacks,
|		 such as opening the workspace.
|         Input: object      - the button being activated
|                client_data - the procedure widget
|                event       - event the activated us
|		 dispatch    - whether to continue dispatch the event
|    Written By: Mark Young
|          Date: Jul 27, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void ProcedureHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event,
   int      *dispatch)
{
	xvobject procedure = (xvobject) client_data;
	XvwProcedureWidget xwid = (XvwProcedureWidget) xvw_widget(procedure);
	xvobject workspace = xvw_object(xwid->procedure.workspace);

	Widget widget;

	*dispatch = FALSE;
	widget = xvw_widget(object);
	object = xvw_object(widget);
	if (event->type == LeaveNotify)
	{
	   kannounce(XVLANG, "GlyphHandler", " ");
	   return;
	}
	else if (event->type == EnterNotify)
	{
	   char     *tmp = NULL;

	   if (object == xwid->procedure.open)
	      tmp = "Procedure's Workspace";

	   kannounce(XVLANG,"ProcedureHandler","%s:  %s", xwid->node.name, tmp);
	   return;
	}
	else if (event->type != ButtonRelease)
	   return;


	if (object == xwid->procedure.open)
	{
	   if (!xvw_check_mapped(workspace))
	   {
	      if (xvw_check_managed(workspace) == FALSE)
	         xvw_manage(workspace);

	      xvw_map(workspace);
	      WorkspaceActive(workspace, TRUE);
	      xvw_set_attribute(object,XVW_PIXMAP,xwid->procedure.close_pixmap);
	   }
	   else
	   {
	      xvw_unmap(workspace);
	      WorkspaceActive(workspace, FALSE);
	      xvw_set_attribute(object, XVW_PIXMAP,xwid->procedure.open_pixmap);
	   }
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: ProcedureLocate
|
|       Purpose: Given a workspace, returns the corresponding
|                procedure glyph if there is one.
|
|         Input: workspace - workspace for which we are looking for the
|			     procedure glyph
|                            attribute value for
|       Returns: the procedure glyph on success, NULL on failure
|    Written By: Mark Young & Danielle Argiro
|          Date: June 24, 1996
|
------------------------------------------------------------*/
xvobject ProcedureLocate(
    Widget wksp)
{

        klist *entry = NULL;

        entry = klist_locate(procedurelist, wksp);

	return(klist_clientdata(entry));

}

/************************************************************
*
*  Routine Name: xvw_create_procedure - create a procedure object
*
*       Purpose: The purpose of the procedure GUI object is to provide a
*		 visual heirarchy for visual programming.  The procedure
*		 object consists of glyph object, which is the node used
*		 to represent the iconified visual program.  When the Procedure
*		 is opened a workspace object will be mapped which will contain
*		 the visual program.  The workspace can be used to construct
*		 visual programs, where the exported input/output parameters
*		 will appear on the Procedure object as input & output
*		 ports.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - the name with which to reference the object
*        Output: None
*       Returns: The procedure object on success, NULL on failure
*
*  Restrictions:
*    Written By: Mark Young
*          Date: Jan 14, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_procedure(
   xvobject parent,
   char     *name)
{
	xvobject procedure;

	procedure = xvw_create(parent, FALSE, TRUE, name, ProcedureWidgetClass);
	xvw_set_attributes(procedure,
		XVW_SELECTABLE, TRUE,
		XVW_RESIZABLE, FALSE,
		XVW_MENUABLE, FALSE,
		NULL);
	return(procedure);
}
