 /*
  * Khoros: $Id: WorkSave.c,v 1.16 1996/06/26 00:59:42 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: WorkSave.c,v 1.16 1996/06/26 00:59:42 mirage Exp $";
#endif


/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Khoros Workspace Widget Utilities
   >>>>  Private:
   >>>>                 WorkspaceSave()
   >>>>   Static:
   >>>>                 LocateExported()
   >>>>                 SaveNodeConnection()
   >>>>                 SaveConnections()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/WorkspaceP.h>
#include <xvlang/GlyphP.h>
#include <xvlang/ExtPortP.h>

static klist *extglyphs;
static klist *extports;

/*-----------------------------------------------------------
|
|  Routine Name: LocateExported - located the exported connection
|
|       Purpose: This routine actually returns the exported connection number
|
|         Input: port  - the port in which to check if we should export
|		 direction - the port direction
|		 sels  - the other selections that are being saved
|		 num_sels - the number of selections in the selection array
|        Output: file  - the file the information is suppose to be written to
|		
|    Written By: Mark Young
|          Date: Nov 12, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static int LocateExported(
   xvobject object,
   int	    direction,
   xvobject *sels,
   size_t   num_sels,
   xvobject port,
   kfile    *file)
{
	klist	 *entry;
	char	 name[KLENGTH], *label;
	int      type, num, xpos, ypos, x, y;
	xvobject parent = xvw_parent(object);


	/* if we ain't an external connection then return -1 */
	if (karray_locate(sels, KSTRUCT, parent, num_sels) != -1)
	   return(-1);

	if ((entry = klist_locate(extglyphs, (kaddr) object)) != NULL)
	   return((int) klist_clientdata(entry));

	if ((entry = klist_locate(extports, (kaddr) port)) != NULL)
	   return((int) klist_clientdata(entry));

	/* compute the locateion */
	xvw_translate_coords(xvw_parent(parent), 0, 0, &x, &y);
	xvw_translate_coords(port, -x, -y, &xpos, &ypos);

	num = num_sels + klist_size(extglyphs);
	if (direction == KPORT_INPUT)
	{
	      xpos -= 100; label = "in";
	      ksprintf(name, "External Input (%d)", num - (num_sels-1));
	}
	else
	{
	      xpos += 100; label = "out";
	      ksprintf(name, "External Output (%d)", num - (num_sels-1));
	}

	/* write out the external port entry and add it to our list */
	kfprintf(file,"ExtPort:%s:%s::%d:%d:%d:\n",label,name,num,xpos,ypos);

	xvw_get_attribute(port, XVW_PORT_TYPE, &type);
	kfprintf(file,"ExtConnection:%s:%x:%d:%d:\n", 
		 label, object, num, type);

	extglyphs = klist_add(extglyphs, (kaddr) object, (kaddr) num);
	extports  = klist_add(extports, (kaddr) port, (kaddr) num);
	return(num);
}

/*-----------------------------------------------------------
|
|  Routine Name: SaveNodeConnection - Save the connections
|
|       Purpose: This routine does the actually saving of a single port
|		 connections whether internal or external
|
|         Input: workspace - the workspace associated with the node
|		 port  - the port to save the connections
|		 ident - the node identifier
|                export - whether we should export external links or not
|		 sels  - the other selections that are being saved
|		 num_sels - the number of selections in the selection array
|        Output: file  - the file the information is suppose to be written to
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void SaveNodeConnection(
    xvobject workspace,
    xvobject port,
    int      ident,
    int      export,
    xvobject *sels,
    size_t   num_sels,
    kfile    *file)
{
	xvobject tmp, parent, *ports;
	int      i, indx, num, dir, val;
	char	 *var1 = NULL, *var2 = NULL, *type;

	if (port == NULL)
	   return;

	PortGetInfo(port, &var1, NULL, NULL, NULL, &dir, &val, &num, &ports);
	if (val == KPORT_DATAFLOW)
	   type = "data";
	else if (val == KPORT_CONTROLFLOW)
	   type = "control";
	else if (val == KPORT_PARAMETER)
	   type = "parameter";
	else
	   type = "";

	for (i = 0; i < num; i++)
	{
	   tmp = NULL;
	   if (dir == KPORT_INPUT)
	      xvw_get_attribute(ports[i], XVW_CONNECTION_BEGIN, &tmp);
	   else
	      xvw_get_attribute(ports[i], XVW_CONNECTION_END, &tmp);

	   parent = xvw_parent(tmp);
	   indx = karray_locate(sels, KSTRUCT, parent, num_sels);
	   if (indx != -1)
	   {
	      if (dir == KPORT_INPUT)
	         continue;

	      PortGetInfo(tmp, &var2, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
	   }
	   else if (export == TRUE)
	      indx = LocateExported(tmp, dir, sels, num_sels, port, file);
	   else
	      continue;

	   if (dir  == KPORT_INPUT)
	     kfprintf(file,"NodeConnection:%s:%d:%s:%d:%s\n",
		    type, indx, !var2 ? "" : var2, ident, !var1 ? "" : var1);
	   else
	     kfprintf(file,"NodeConnection:%s:%d:%s:%d:%s\n",
		    type, ident, !var1 ? "" : var1, indx, !var2 ? "" : var2);
	}
	kfree(var1); kfree(var2);
}

/*-----------------------------------------------------------
|
|  Routine Name: SaveConnections - Save the connections
|
|       Purpose: This routine does the actually saving of a glyph's connections
|		 to a workspace file.
|
|         Input: workspace - the workspace associated with the node
|		 node  - the node to save the connections
|		 ident - the node identifier
|                export - whether we should export external links or not
|		 selections - the other selections that are being saved
|		 num_sels    - the number of selections in the selection array
|        Output: file  - the file the information is suppose to be written to
|    Written By: Mark Young
|          Date: Aug 26, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
static void SaveConnections(
    xvobject workspace,
    xvobject node,
    int      ident,
    int	     export,
    xvobject *sels,
    size_t   num_sels,
    kfile    *file)
{
	XvwGlyphWidget   xglyph;
	XvwExtPortWidget xextport;

	int      i;
	xvobject port;
	char     *name = NULL;


	xvw_get_attribute(node, XVW_NODE_NAME, &name);
	kfprintf(file,"\n#  Connections '%s'\n", name);

	if (xvw_check_subclass(node, GlyphWidgetClass) == TRUE)
	{
	   xglyph = (XvwGlyphWidget) xvw_widget(node);

	   for (i = 0; i < xglyph->node.osize; i++)
	   {
	      port = xglyph->node.oports[i];
	      SaveNodeConnection(workspace, port, ident, export, sels,
				 num_sels, file);
	   }

	   for (i = 0; i < xglyph->node.isize; i++)
	   {
	      port = xglyph->node.iports[i];
	      SaveNodeConnection(workspace, port, ident, export, sels,
				 num_sels, file);
	   }

	   port = xglyph->glyph.cflow_out;
	   SaveNodeConnection(workspace, port, ident, export, sels,
                                 num_sels, file);

	   port = xglyph->glyph.cflow_in;
	   SaveNodeConnection(workspace, port, ident, export, sels,
                                 num_sels, file);

	}
	else if (xvw_check_subclass(node, ExtPortWidgetClass) == TRUE)
	{
	   xextport = (XvwExtPortWidget) xvw_widget(node);

	   port = xextport->extport.port;
	   SaveNodeConnection(workspace, port, ident, export, sels,
			      num_sels, file);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: WorkspaceSave
|
|       Purpose: WorkspaceSave is used to save the glyph's within a
|		 workspace.
|
|         Input: widget - the workspace widget
|		 file   - the file pointer in which to write the workspace to
|		 header - whether we should write the workspace header
|		 export - whether we should export external links or not
|		 selected - save only the currently selected glyphs
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Nov 30, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int WorkspaceSave(
   Widget widget,
   kfile  *file,
   int    header,
   int    export,
   int    selected)
{
	XvwWorkspaceWidget xwid = (XvwWorkspaceWidget) widget;

	size_t   num;
	time_t   tval;
        int      i, saved;
	char	 *filename, *username;
	xvobject workspace = xvw_object(widget);
	xvobject *sels, object = xwid->viewport.plane;

	static int count = 0;

	if (header == TRUE)
	{
	   tval = time(NULL);
	   if ((username = kgetenv("USER")) == NULL)
	      username = "(unknown)";

	   filename = kfile_filename(kfileno(file));
	   kfprintf(file, "# Khoros Visual Programming Workspace\n#\n# %s "
			  "workspace file (%s) was created \n# on %s# by "
			  "user %s\n#\n", kprog_get_program(), filename,
			  ctime(&tval), username);
	}
	extglyphs = NULL;
	extports = NULL;

	/* Save the start of the workspace to the file */
	if (count == 0)
	   kfprintf(file,"\nWorkspaceBegin: %g\n", WORKSPACE_VERSION);

	count++;

	/* get the selections to be saved */
	sels = WorkspaceChildren(xvw_object(widget), selected, &num);

	/* Save the variables to the file */
	kfprintf(file,"\nVariablesBegin:\n");
	kexpr_variables_print((long) xvw_object(widget), file);
	kfprintf(file,"VariablesEnd:\n");

	/*
	 *  Save the Node information
	 */
	for (i = 0, saved = FALSE; i < num; i++)
	{
	   if (xvw_check_subclass(sels[i], NodeWidgetClass) == FALSE)
	      continue;

	   NodeSave(sels[i], file, i); saved = TRUE;
 	}
	if (saved) kfprintf(file, "\n");

	/*
	 *  Save the Local and External Connections associated with the above
	 *  Glyph/Procedure/Conditional/ExtPort objects
	 */
	for (i = 0, saved = FALSE; i < num; i++)
	{
	   if (xvw_check_subclass(sels[i], NodeWidgetClass) == FALSE)
	      continue;

	   SaveConnections(workspace, sels[i], i, export, sels, num, file);
	   saved = TRUE;
 	}
	if (saved) kfprintf(file, "\n");

	/* Save the annotations to the file */
	xvw_save_annotations(object, file);

	count--;

	/* Save the end of the workspace to the file */
	if (count == 0)
	   kfprintf(file,"\nWorkspaceEnd: %g\n", WORKSPACE_VERSION);

	/* Cleanup, since we are done */
	kfree(sels);
	klist_free(extglyphs, NULL);
	klist_free(extports, NULL);
	return(TRUE);
}
