 /*
  * Khoros: $Id: callback.c,v 1.6 1996/04/16 05:28:31 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: callback.c,v 1.6 1996/04/16 05:28:31 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Various Private Utility Routines For the kexpr Library
   >>>>		
   >>>>  Private:
   >>>>                 xvl_expr_call_callback()
   >>>>                 xvl_expr_add_callback()
   >>>>                 xvl_expr_remove_callback()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

klist *callbacks = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: xvl_expr_call_callback - calls the list of callbacks
|
|       Purpose: Call the callbacks for the expression parser.
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Tue Jul 25 1995
|
------------------------------------------------------------*/
void xvl_expr_call_callback(
    long id,
    int  operation)
{
	kfunc_void routine;
	kaddr  client_data;
	klist *tmp = callbacks;
	xvobject workspace = (xvobject) id;

	while (tmp != NULL)
	{
	   routine = (kfunc_void) klist_identifier(tmp);
	   client_data = klist_clientdata(tmp);
	   (*routine)(workspace, operation, client_data);
	   tmp = klist_next(tmp);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_expr_add_callback - adds a callback that reports id changes
|
|       Purpose: Add a callback for the expression parser.  When an id is added
|		 to the expression parser, deleted from, or updated, the
|		 specified callback will be called (fired).
|
|                A callback must be declared in the following form:\f(CW
|                !void expression_callback(
|                !   xvobject workspace,
|                !   int    operation,
|                !   kaddr  client_data)\fP
|
|                !\fIworkspace -\fP
|                !    The workspace for which the callback is being
|                !    invoked.
|
|                !\fIoperation -\fP
|                !    Indicates the operation that caused the callback to
|                !    be invoked.  One of: KEXPR_ID_ADD, KEXPR_ID_DELETE,
|		 !    KEXPR_ID_UPDATE.
|
|                !\fIclient_data -\fP
|                !    The pointer to the client data, used to pass parameters
|                !    from the application to the event handler.
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Tue Jul 25 1995
|
------------------------------------------------------------*/
void xvl_expr_add_callback(
      kfunc_void callback_routine,
      kaddr	 client_data)
{
	callbacks = klist_add(callbacks, (kaddr) callback_routine, client_data);
}

/*-----------------------------------------------------------
|
|  Routine Name: xvl_expr_add_callback - adds a callback that reports id changes
|
|       Purpose: Removes a callback from the expression parser.
|
|    Written By: Mark Young & Danielle Argiro
|          Date: Tue Jul 25 1995
|
------------------------------------------------------------*/
/*ARGSUSED*/
void xvl_expr_remove_callback(
      kfunc_void callback_routine,
      kaddr	 client_data)
{
	callbacks = klist_delete(callbacks, (kaddr) callback_routine);
}
