 /*
  * Khoros: $Id: debugger.c,v 1.2 1996/08/17 22:14:32 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: debugger.c,v 1.2 1996/08/17 22:14:32 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros Glyph Debug Utilities
   >>>>  Private:
   >>>> 		GlyphStartDebugger()
   >>>>   Static:
   >>>> 		DebuggerCallback()
   >>>>   Public:
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvlang/GlyphP.h>

static void DebuggerCallback  PROTO((int, kaddr));

#ifdef KOPSYS_SOLARIS
#define DefaultDebugCommand "xterm -e dbx %prog %pid"
#endif

#ifdef KOPSYS_IRIX
#define DefaultDebugCommand "xterm -e dbx -p %pid"
#endif

#ifdef KOPSYS_OSF
#define DefaultDebugCommand "xterm -e dbx -pid %pid %prog"
#endif

#ifdef __GNUC__
#undef  DefaultDebugCommand
#define DefaultDebugCommand "xterm -e gdb %prog %pid"
#endif

#ifndef DefaultDebugCommand
#define DefaultDebugCommand "xterm -e dbx %prog %pid"
#endif

/*-----------------------------------------------------------
|
|  Routine Name: DebuggerCallback - debugger callback routine
|
|       Purpose: This routine is used to change the fact that
|		 the debugger has terminated.
|
|         Input: 
|    Written By: Mark Young
|          Date: Jun 21, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
static void DebuggerCallback(
   int    status,
   kaddr  client_data)
{
        XvwGlyphWidget xwid = (XvwGlyphWidget) client_data;

	xwid->glyph.dbxpid = 0;
	if (xwid->node.pid > 0)
           kill(xwid->node.pid, SIGCONT);
}

/*-----------------------------------------------------------
|
|  Routine Name: GlyphStartDebugger
|
|       Purpose: GlyphStartDebugger starts a debugger for a given glyph.
|
|         Input: widget - the widget to start the debugger for
|    Written By: Mark Young
|          Date: Jun 19, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */ 
void GlyphStartDebugger(
   Widget widget,
   int    start)
{
	XvwGlyphWidget xwid = (XvwGlyphWidget) widget;

	pid_t dbxpid = xwid->glyph.dbxpid;
	char  temp[KLENGTH], command[KLENGTH], *tmp;

	if (start == TRUE)
	{
	   if (xwid->node.pid <= 0 || (dbxpid > 0 &&
	       xvw_check_process(dbxpid, NULL)))
	   {
	      return;
	   }
	   xwid->glyph.dbxpid = 0;
	}
	else
	{
	   if (dbxpid > 0 && xvw_check_process(dbxpid, NULL))
	      kill(dbxpid, SIGTERM);

	   if (xwid->node.pid > 0)
	      kill(xwid->node.pid, SIGCONT);

	   xwid->glyph.dbxpid = 0;
	   return;
	}

	if ((tmp = kgetenv("KHOROS_DEBUGGER")) == NULL)
	   tmp = DefaultDebugCommand;

	kstrcpy(command, tmp);
	kstring_replace(command, "%g", "80x48", command);
	kstring_replace(command, "%t", xwid->node.name, command);
	kstring_replace(command, "%fg", "black", command);
	kstring_replace(command, "%bg", "white", command);
        tmp = kstrchr(xwid->glyph.command, ' ');
        kstring_ncopy(xwid->glyph.command, tmp - xwid->glyph.command, temp);
	kstring_replace(command, "%prog", temp, command);
	ksprintf(temp, "%d", xwid->node.pid);
	kstring_replace(command, "%pid", temp, command);

	dbxpid = xvw_fork(command, DebuggerCallback, NULL, NULL, widget);
	ksprintf(temp, "... Starting debug process for process %d ...\n",
			xwid->node.pid);
	MessageCallback(NULL, (kaddr) widget, temp);
	sleep(5);
}
