/*
  * Khoros: $Id: Connection.c,v 1.14.4.1 1996/12/06 20:30:21 fas Exp $
  */
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Connection Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 RedrawConnection()
   >>>>                 PickGadget()
   >>>>                 RecomputeSize()
   >>>>   Public:
   >>>>                 xvw_create_connection()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvobjects/ConnectionP.h>

static void ClassInitialize	 PROTO((void));
static void Redisplay		 PROTO((Widget, XEvent *, Region));
static Boolean SetValues	 PROTO((Widget, Widget, Widget, ArgList,
					Cardinal *));
static int  PickGadget		 PROTO((Widget, int, int));
static void RecomputeSize	 PROTO((Widget, int));


/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_CONNECTION_BEGIN,      NULL,   XtRObject,   NULL},
{XVW_CONNECTION_END,        NULL,   XtRObject,   NULL},
{XVW_CONNECTION_TYPE,       NULL,   XtRInt,	 NULL},
{XVW_CONNECTION_LINEWIDTH,  NULL,   XtRInt,	 NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwConnectionGadgetRec, connection.field)

static XtResource resources[] = { 
{XVW_CONNECTION_BEGIN, NULL, XtRObject, sizeof(xvobject), 
      offset(begin), XtRObject, (XtPointer) NULL},
{XVW_CONNECTION_END, NULL, XtRObject, sizeof(xvobject), 
      offset(end), XtRObject, (XtPointer) NULL},
{XVW_CONNECTION_TYPE, NULL, XtRInt, sizeof(int), 
      offset(type), XtRImmediate, (XtPointer) KCONNECTION_TYPE_MANHATTAN},
{XVW_CONNECTION_LINEWIDTH, NULL, XtRInt, sizeof(int), 
      offset(linewidth), XtRImmediate, (XtPointer) 0},
};
#undef offset


#define SUPERCLASS (&xvwManagerGadgetClassRec)

XvwConnectionGadgetClassRec xvwConnectionGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Connection",			/* class_name		  */
    sizeof(XvwConnectionGadgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    NULL,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL, 	                        /* not used - obj         */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    NULL,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    PickGadget,				/* pick object proc       */
    XtInheritEraseSel,			/* erase selected proc    */
    XtInheritRefreshSel,		/* refresh selection proc */
    NULL,				/* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwConnectionGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwConnectionGadgetClass for public consumption
 */
WidgetClass xvwConnectionGadgetClass = (WidgetClass) &xvwConnectionGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of connection object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Nov 23, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwConnectionGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvobjects/uis/Connection.pane");
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/Connection");
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the connection object in response
|                to an expose event. 
|
|         Input: widget - the connection object that was exposed 
|		 draw_foreground - draw the foreground connecion
|		 draw_background - draw the background connecion
|		 recompute_size  - whether to recompute the size/position
|    Written By: Mark Young
|          Date: Jun 29, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RedrawConnection(
   Widget widget,
   int    draw_foreground,
   int    draw_background,
   int    recompute_size)
{
	XvwConnectionGadget xobj = (XvwConnectionGadget) widget;

	XPoint   pt;
	xvobject end   = xobj->connection.end;
	xvobject begin = xobj->connection.begin;
	GC gc1 = xobj->manager.foreground_gc;
	GC gc2 = xobj->manager.background_gc;

	Window  window   = XtWindow(widget);
	Display *display = XtDisplay(widget);

        XGCValues  values;
	unsigned long mask = GCLineStyle | GCLineWidth | 
                             GCCapStyle | GCJoinStyle;
	unsigned int save_linewidth;

	/*
	 *  If the begin and end points aren't set then don't draw anything
	 */
	if (!window || !xvw_check_realized(begin) || !xvw_check_realized(end))
	   return;

	if (!gc1 || !gc2)
	   return;

	if (recompute_size == TRUE)
	   RecomputeSize(widget, FALSE);

	/*
	 *  Actually draw the line
	 */
	pt = xobj->connection.pt[0];

	XGetGCValues(display, gc2, mask, &values);
	save_linewidth = values.line_width;

	if (draw_background == TRUE)
	{

	   XSetLineAttributes(display, gc2, 
		       (unsigned) xobj->connection.linewidth,
 		       values.line_style, values.cap_style, values.join_style);

	   xobj->connection.pt[0].y--;
	   if (xobj->connection.type == KCONNECTION_TYPE_MANHATTAN)
	      xobj->connection.pt[0].x--;
	   XDrawLines(display, window, gc2, xobj->connection.pt,
		      xobj->connection.num, CoordModePrevious);

	   xobj->connection.pt[0].y += 2;
	   if (xobj->connection.type == KCONNECTION_TYPE_MANHATTAN)
	      xobj->connection.pt[0].x += 2;
	   XDrawLines(display, window, gc2, xobj->connection.pt,
		      xobj->connection.num, CoordModePrevious);

	   xobj->connection.pt[0].y++;
	   if (xobj->connection.type == KCONNECTION_TYPE_MANHATTAN)
	      xobj->connection.pt[0].x++;
	   XDrawLines(display, window, gc2, xobj->connection.pt,
		      xobj->connection.num, CoordModePrevious);

	   XSetLineAttributes(display, gc2, save_linewidth, values.line_style,
			      values.cap_style, values.join_style);

	}

	if (draw_foreground == TRUE)
	{
	   XSetLineAttributes(display, gc1, 
		       (unsigned) xobj->connection.linewidth,
		       values.line_style,values.cap_style, values.join_style);

	   xobj->connection.pt[0] = pt;
	   XDrawLines(display, window, gc1, xobj->connection.pt,
		      xobj->connection.num, CoordModePrevious);

	   XSetLineAttributes(display, gc1, save_linewidth, values.line_style,
			      values.cap_style, values.join_style);
	}
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or postion of the connection has changed, 
|		 recalculate the size and position of the connection.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwConnectionGadget cobj = (XvwConnectionGadget) current;
	XvwConnectionGadget nobj = (XvwConnectionGadget) new;

	Boolean  redisplay = FALSE;


	if (cobj->connection.begin     != nobj->connection.begin     ||
	    cobj->connection.end       != nobj->connection.end	     ||
	    cobj->connection.type      != nobj->connection.type)
	{
	   RecomputeSize(new, TRUE);
	   redisplay = TRUE;
	}

	if (cobj->manager.foreground   != nobj->manager.foreground  ||
	    cobj->manager.background   != nobj->manager.background)
	{
	   int draw_foreground = FALSE, draw_background = FALSE;

	   if (cobj->manager.foreground != nobj->manager.foreground)
	      draw_foreground = TRUE;
	   if (cobj->manager.background != nobj->manager.background)
	      draw_background = TRUE;

	   RedrawConnection(new, draw_foreground, draw_background, FALSE);
	}
	return(redisplay);
}

/*-----------------------------------------------------------
|
|  Routine Name: RedrawConnection
|
|       Purpose: This routine will redraw the connection object in response
|                to an expose or update event.
|
|         Input: widget - the connection object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Jun 29, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	RedrawConnection(widget, TRUE, TRUE, TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: PickGadget
|
|       Purpose: Given a connection object and an attempted picking location,
|                see if the connection has been picked. 
|
|         Input: widget - the connection object we are checking 
|                x      - the x device coordinate that the pick attempt was at 
|                y      - the y device coordinate that the pick attempt was at
|       Returns: TRUE (1) if object was successfully picked, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Jun 29, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static int PickGadget(
   Widget widget,
   int    x,
   int    y)
{
	XvwConnectionGadget xobj = (XvwConnectionGadget) widget;

	int	 i;
	XPoint	 *pt = xobj->connection.pt;
	Position run, rise, deltax, deltay, temp, xtmp, ytmp;


	if (!xobj->connection.begin || !xobj->connection.end)
	   return(FALSE);

	xtmp = pt[0].x;
	ytmp = pt[0].y;
	for (i = 1; i < xobj->connection.num; i++)
	{
	   run  = pt[i].x;
	   rise = pt[i].y;
	   deltax = x - xtmp;
	   deltay = y - ytmp;
	   if (kabs(run) <= PICK_TOLERANCE)
	   {
	      if (kabs(deltax) <= PICK_TOLERANCE && 
		  y+PICK_TOLERANCE > kmin(ytmp,ytmp+rise) &&
		  y-PICK_TOLERANCE < kmax(ytmp,ytmp+rise))
		 return(TRUE);
	   }
	   else if (kabs(rise) <= PICK_TOLERANCE)
	   {
	      if (kabs(deltay) <= PICK_TOLERANCE && 
		  x+PICK_TOLERANCE > kmin(xtmp,xtmp+run) &&
		  x-PICK_TOLERANCE < kmax(xtmp,xtmp+run))
		 return(TRUE);
	   }
	   else
	   {
	      temp = (rise * deltax) / run;
	      if (kabs(deltay - temp) <= PICK_TOLERANCE)
		 return(TRUE);
	   }
	   xtmp += pt[i].x;
	   ytmp += pt[i].y;
	}
	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: RecomputeSize
|
|       Purpose: recomputes the size
|
|         Input: widget   - the line object whose selection we are refreshing
|		 override - whether to override the initial position check
|    Written By: Mark Young
|          Date: Nov 18, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void RecomputeSize(
   Widget  widget,
   int     override)
{
	XvwConnectionGadget xobj = (XvwConnectionGadget) widget;

	XtWidgetGeometry temp;
	XPoint   *pt, c[4];
	unsigned int w1 = 0, w2 = 0, h1 = 0, h2 = 0;
	xvobject object = xvw_object(widget);
	xvobject parent = xvw_parent(object);
	int i, x, y, xpos1 = 0, xpos2 = 0, ypos1 = 0, ypos2 = 0, tmpx, tmpy;

	static int num_steps = 32;
	double step, unit, t, t0, t1, t2, t3;

	/*
	 *  Initialize the points to the begin and end points
	 */
	xvw_translate_coords(parent, 0, 0, &x, &y);
	xvw_translate_coords(xobj->connection.begin, -x, -y, &xpos1, &ypos1);
	xvw_translate_coords(xobj->connection.end,   -x, -y, &xpos2, &ypos2);
	xvw_geometry(xobj->connection.begin, NULL, NULL, &w1, &h1, NULL);
	xvw_geometry(xobj->connection.end, NULL, NULL, &w2, &h2, NULL);

	xpos1 = xpos1+w1; ypos1 = ypos1+h1/2;
	xpos2 = xpos2; ypos2 = ypos2+h2/2;
	if (!override &&
	    xobj->connection.x1 == xpos1 && xobj->connection.y1 == ypos1 &&
	    xobj->connection.x2 == xpos2 && xobj->connection.y2 == ypos2)
	{
	   return;
	}
	xobj->connection.x1 = xpos1; xobj->connection.y1 = ypos1;
	xobj->connection.x2 = xpos2; xobj->connection.y2 = ypos2;


	pt = xobj->connection.pt;
	if (xobj->connection.type == KCONNECTION_TYPE_LINEAR)
	{
	   xobj->connection.num = 4;
	   pt[0].x = xpos1;      pt[0].y = ypos1;
	   pt[1].x = xpos1 + 10; pt[1].y = ypos1;
	   pt[2].x = xpos2 - 10; pt[2].y = ypos2;
	   pt[3].x = xpos2;      pt[3].y = ypos2;
	}
	else if (xobj->connection.type == KCONNECTION_TYPE_MANHATTAN)
	{
	   if (xpos1+20 < xpos2)
	   {
	      xobj->connection.num = 4;
	      pt[0].x = xpos1;   	        pt[0].y = ypos1;
	      pt[1].x = xpos1 + (xpos2-xpos1)/2; pt[1].y = ypos1;
	      pt[2].x = xpos1 + (xpos2-xpos1)/2; pt[2].y = ypos2;
	      pt[3].x = xpos2;   	        pt[3].y = ypos2;
	   }
	   else
	   {
	      xobj->connection.num = 6;
	      pt[0].x = xpos1;      pt[0].y = ypos1;
	      pt[1].x = xpos1 + 10; pt[1].y = ypos1;
	      pt[2].x = xpos1 + 10; pt[2].y = ypos1 + (ypos2-ypos1)/2;
	      pt[3].x = xpos2 - 10; pt[3].y = ypos1 + (ypos2-ypos1)/2;
	      pt[4].x = xpos2 - 10; pt[4].y = ypos2;
	      pt[5].x = xpos2;      pt[5].y = ypos2;
	   }
	}
	else if (xobj->connection.type == KCONNECTION_TYPE_HEXAGON)
	{
	   if (xpos1+20 < xpos2)
	   {
	      xobj->connection.num = 4;
	      pt[0].x = xpos1;   	        pt[0].y = ypos1;
	      pt[1].x = xpos1+(xpos2-xpos1)/2; pt[1].y = ypos1+(ypos2-ypos1)/4;
	      pt[2].x = xpos1+(xpos2-xpos1)/2; pt[2].y = ypos2-(ypos2-ypos1)/4;
	      pt[3].x = xpos2;   	        pt[3].y = ypos2;
	   }
	   else
	   {
	      xobj->connection.num = 6;
	      pt[0].x = xpos1;      pt[0].y = ypos1;
	      pt[1].x = xpos1 + 10; pt[1].y = ypos1;
	      pt[2].x = xpos1 + 10; pt[2].y = ypos1 + (ypos2-ypos1)/2;
	      pt[3].x = xpos2 - 10; pt[3].y = ypos1 + (ypos2-ypos1)/2;
	      pt[4].x = xpos2 - 10; pt[4].y = ypos2;
	      pt[5].x = xpos2;      pt[5].y = ypos2;
	   }
	}
	else if (xobj->connection.type == KCONNECTION_TYPE_DIAMOND)
	{
	   if (xpos1+20 < xpos2)
	   {
	      xobj->connection.num = 4;
	      pt[0].x = xpos1;   	        pt[0].y = ypos1;
	      pt[1].x = xpos2-(xpos2-xpos1)/4; pt[1].y = ypos1+(ypos2-ypos1)/4;
	      pt[2].x = xpos1+(xpos2-xpos1)/4; pt[2].y = ypos2-(ypos2-ypos1)/4;
	      pt[3].x = xpos2;   	        pt[3].y = ypos2;
	   }
	   else
	   {
	      xobj->connection.num = 6;
	      pt[0].x = xpos1;      pt[0].y = ypos1;
	      pt[1].x = xpos1 + 10; pt[1].y = ypos1;
	      pt[2].x = xpos1 + 10; pt[2].y = ypos1 + (ypos2-ypos1)/2;
	      pt[3].x = xpos2 - 10; pt[3].y = ypos1 + (ypos2-ypos1)/2;
	      pt[4].x = xpos2 - 10; pt[4].y = ypos2;
	      pt[5].x = xpos2;      pt[5].y = ypos2;
	   }
	}
	else if (xobj->connection.type == KCONNECTION_TYPE_SPLINE)
	{

	   if (xpos1+20 < xpos2)
	   {
	      x = kmax(kabs((xpos2-xpos1)/2),20);
	      y = kmax(kabs((ypos2-ypos1)/4),10);
	      c[0].x = xpos1;       c[0].y = ypos1;
	      c[1].x = xpos1 + x;   c[1].y = ypos1;
	      c[2].x = xpos2 - x;   c[2].y = ypos2;
	      c[3].x = xpos2;       c[3].y = ypos2;
	   }
	   else
	   {
	      x = kmax(kabs((xpos2-xpos1)/2),20);
	      y = kmax(kabs((ypos2-ypos1)/2),20);
	      c[0].x = xpos1;     c[0].y = ypos1;
	      c[1].x = xpos1 + x; c[1].y = (ypos1 > ypos2) ? ypos1-y : ypos1+y;
	      c[2].x = xpos2 - x; c[2].y = (ypos1 > ypos2) ? ypos2+y : ypos2-y;
	      c[3].x = xpos2;     c[3].y = ypos2;
	   }

	   unit = 1.0/(num_steps-1);
	   xobj->connection.num = num_steps;
	   for (i = 0, step = 0.0; i < num_steps; i++, step += unit)
	   {
	      t  = (1.0 - step);
	      t0 = t*t*t;
	      t1 = 3*step*t*t;
	      t2 = 3*step*step*t;
	      t3 = step*step*step;
	      pt[i].x = c[0].x*t0 + c[1].x*t1 + c[2].x*t2 + c[3].x*t3 + 0.5;
	      pt[i].y = c[0].y*t0 + c[1].y*t1 + c[2].y*t2 + c[3].y*t3 + 0.5;
	   }
	}

	x = xpos1 = xpos2 = pt[0].x;
	y = ypos1 = ypos2 = pt[0].y;
	for (i = 1; i < xobj->connection.num; i++)
	{
	   xpos1 = kmin(xpos1, pt[i].x);
	   xpos2 = kmax(xpos2, pt[i].x);
	   ypos1 = kmin(ypos1, pt[i].y);
	   ypos2 = kmax(ypos2, pt[i].y);

	   tmpx = pt[i].x;
	   tmpy = pt[i].y;
	   pt[i].x = pt[i].x - x; x = tmpx;
	   pt[i].y = pt[i].y - y; y = tmpy;
	}
	temp.request_mode = CWWidth | CWHeight | CWX | CWY;
	temp.x = (Position) xpos1 - 3;
	temp.y = (Position) ypos1 - 3;
	temp.width  = (Dimension) (xpos2-xpos1) + 6 +xobj->connection.linewidth;
	temp.height = (Dimension) (ypos2-ypos1) + 6 +xobj->connection.linewidth;
	(void) XtMakeGeometryRequest(widget, &temp, &temp);
}


/************************************************************
*
*  Routine Name: xvw_create_connection - create a connection object 
*
*       Purpose: The connection object provides a mechanism for visually
*		 linking two objects.  It is used in \fHcantata\fP to connect 
*                glyph objects together, but can be used to connect
*                any two objects.  A line will be drawn between the two
*		 objects; the line will resize and redraw if either of the
*		 two objects are moved.  Several styles of connections
*		 are provided, which can be used to change the appearance of
*		 the connection.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The connection object on success, NULL on failure.
*  Restrictions:
*    Written By: Mark Young 
*          Date: Jun 29, 1993
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_connection(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the line object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, ConnectionGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     FALSE,    /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   FALSE,    /* selectable     */
		NULL);
	}
	return(object);
}
