/*
  * Khoros: $Id: NotifyWindow.c,v 1.20 1996/06/25 16:34:01 mirage Exp $
  */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: NotifyWindow.c,v 1.20 1996/06/25 16:34:01 mirage Exp $";
#endif
 
 
/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>	              Khoros NotifyWindow Widget
   >>>>  Private:
   >>>>   Static:
   >>>> 		ClassInitialize()
   >>>>			Initialize()
   >>>>                 Destroy()
   >>>> 		Realize()
   >>>>
   >>>> 		NotifySetString()
   >>>> 		NotifyGetString()
   >>>>   Public:
   >>>>			xvw_create_notifywindow()
   >>>>	
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvobjects/NotifyWindowP.h>

static void     ClassInitialize PROTO((void));
static void	Initialize	PROTO((Widget, Widget, ArgList, Cardinal *));
static void     Destroy		PROTO((Widget));
static void	Realize         PROTO((Widget, XtValueMask *, 
                                       XSetWindowAttributes *));
static int NotifySetString	PROTO((xvobject, char *, kaddr));
static int NotifyGetString	PROTO((xvobject, char *, kaddr));


/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/
 
#define NOTIFY_PIXMAP "$DESIGN/objects/library/xvobjects/misc/pixmaps/notify.xpm"

/*-------------------------------------------------------------------*
|
|   Full class attributes
|
--------------------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_NOTIFYWINDOW_LABEL_OBJECT,   NULL,   XtRPointer,  NULL},
{XVW_NOTIFYWINDOW_ICON_OBJECT,    NULL,   XtRPointer,  NULL},
{XVW_NOTIFYWINDOW_TEXT_OBJECT,    NULL,   XtRPointer,  NULL},
{XVW_NOTIFYWINDOW_LABEL,	  NULL,   XtRString,   NULL},
{XVW_NOTIFYWINDOW_MESSAGE,	  NULL,   XtRString,   NULL},
{XVW_NOTIFYWINDOW_TITLE,	  NULL,   XtRString,   NULL},
{XVW_NOTIFYWINDOW_VISIBLE,	  NULL,   XtRInt,      NULL},
{XVW_NOTIFYWINDOW_NOTIFYFOR,	  NULL,   XtRPointer,  NULL},

{XVW_NOTIFYWINDOW_PIXMAP,	  NULL,   XtRPixmap,   NULL},
{XVW_NOTIFYWINDOW_PIXMAPFILE,	  XVW_NOTIFYWINDOW_PIXMAP,   XtRString, XtRPixmap},
};


/*-------------------------------------------------------------------*
|
|   Full class record constant
|
--------------------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwNotifyWindowWidgetRec, notifywindow.field)

static XtResource resources[] = { 
{XVW_NOTIFYWINDOW_LABEL_OBJECT, NULL, XtRPointer, sizeof(XtPointer),
        offset(label), XtRImmediate, (XtPointer) NULL},
{XVW_NOTIFYWINDOW_ICON_OBJECT, NULL, XtRPointer, sizeof(XtPointer),
        offset(icon), XtRImmediate, (XtPointer) NULL},
{XVW_NOTIFYWINDOW_TEXT_OBJECT, NULL, XtRPointer, sizeof(XtPointer),
        offset(text), XtRImmediate, (XtPointer) NULL},
{XVW_NOTIFYWINDOW_NOTIFYFOR, NULL, XtRPointer, sizeof(XtPointer),
        offset(notifyfor), XtRImmediate, (XtPointer) NULL},
{XVW_NOTIFYWINDOW_PIXMAP, XVW_NOTIFYWINDOW_PIXMAPFILE,  XtRPixmap, 
	sizeof(Pixmap),
        offset(notify_pixmap), XtRString, (XtPointer) NOTIFY_PIXMAP},

{XVW_SHADOW_TYPE, NULL, XtRInt, sizeof(int),
        XtOffsetOf(XvwNotifyWindowWidgetRec, manager.shadow_type),
        XtRImmediate, (XtPointer) KSHADOW_OUT},
{XVW_PREFERRED_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwNotifyWindowWidgetRec, manager.preferred_width),
        XtRImmediate, (XtPointer) 350},
{XVW_PREFERRED_HEIGHT, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwNotifyWindowWidgetRec, manager.preferred_height),
        XtRImmediate, (XtPointer) 150},
{XVW_MANAGER_DELAYED_LAYOUT, NULL, XtRBoolean, sizeof(Boolean),
        XtOffsetOf(XvwNotifyWindowWidgetRec, manager.delayed_layout),
	XtRImmediate, (XtPointer) FALSE },
};
#undef offset


/*-------------------------------------------------------------------*
|
|   Class Declaration for NotifyWindow Widget
|
--------------------------------------------------------------------*/

#define superclass (&xvwManagerWidgetClassRec)

XvwNotifyWindowWidgetClassRec xvwNotifyWindowWidgetClassRec =
{
  {
    (WidgetClass) superclass,		/* superclass		  */	
    "NotifyWindow",			/* class_name		  */
    sizeof(XvwNotifyWindowWidgetRec),	/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    Realize,				/* realize		  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    TRUE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    TRUE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    NULL,				/* resize		  */
    XtInheritExpose,				/* expose		  */
    NULL,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    XtInheritQueryGeometry,		/* query_geometry	  */
    XtInheritDisplayAccelerator,	/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* CoreClass fields initialization */
  {
    NULL,                    		/* geometry_manager	  */
    XtInheritChangeManaged,		/* change_managed	  */
    XtInheritInsertChild,		/* insert_child	  	  */
    XtInheritDeleteChild,		/* delete_child	  	  */
    NULL,				/* extension	 	  */
  },  /* CompositeClass fields initialization */
  {
    NULL,				    /* subresources	  */
    0,					    /* subresources_count */
    sizeof(XvwNotifyWindowWidgetConstraintsRec),  /* constraint_size	  */
    NULL,				    /* initialize	  */
    NULL,				    /* destroy		  */
    NULL,				    /* set_values	  */
    NULL,				    /* extension	  */
  },  /* ConstraintClass fields initialization */
  {
    XtInheritLayout,                        /* pre-layout routine     */
    XtInheritLayout,                        /* post-layout routine    */
    XtInheritChangeSel,			    /* change selected proc   */
    XtInheritEraseSel,			    /* erase selected proc    */
    XtInheritRefreshSel,		    /* refresh selection proc */
    XtInheritResize,			    /* resize		      */
    XtInheritGeometryManager,		    /* geometry_manager	      */
  },  /* XvwManagerWidgetClass fields initialization */
  {
    NULL,  		                    /* extension          */
  },  /* XvwNotifyWindowWidgetClass fields initialization */
};
#undef superclass

  /* for public consumption */
WidgetClass xvwNotifyWindowWidgetClass = (WidgetClass) &xvwNotifyWindowWidgetClassRec;


/*-------------------------------------------------------------------*
|
|   Miscelleanous defines for widget definition
|
--------------------------------------------------------------------*/

#undef  kwidget
#define kwidget(widget)	     (XvwNotifyWindowWidget) (widget)


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an
|                instance of a XvwNotifyWindowWidget class has been created.
|                This is where the attributes for the class are
|		 initialized. 
|
|    Written By: Neil Bowers and Mark Young
|          Date: Nov 24, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwNotifyWindowWidgetClass, attributes,
		XtNumber(attributes), NULL, 0, NULL);
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/NotifyWindow");
        xvw_define_attributes(xvwNotifyWindowWidgetClass,
           XVW_NOTIFYWINDOW_LABEL,   XtRString,NotifySetString, NotifyGetString,
           XVW_NOTIFYWINDOW_MESSAGE, XtRString,NotifySetString, NotifyGetString,
           XVW_NOTIFYWINDOW_TITLE,   XtRString,NotifySetString, NotifyGetString,
           XVW_NOTIFYWINDOW_VISIBLE, XtRInt,   NotifySetString, NotifyGetString,
           NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: Initialize
|
|	Purpose: This method will set up the initial notifywindow widget
|		 instance.
|
|	  Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwNotifyWindowWidget xwid = kwidget(new);
	xvobject object = xvw_object(new);


	xwid->notifywindow.visible = FALSE;
	xwid->notifywindow.label_string = NULL;
	xwid->notifywindow.text_string  = NULL;
	xwid->notifywindow.title_string = NULL;

	xwid->notifywindow.icon = xvw_create_pixmap(object, "pixmap");
	xvw_set_attributes(xwid->notifywindow.icon,
		XVW_PIXMAP,	      xwid->notifywindow.notify_pixmap,
		XVW_FORCE_REDISPLAY,  TRUE, 
		XVW_BELOW,            NULL,
		XVW_RIGHT_OF,         NULL,
		XVW_VERT_DIST,        8,
		XVW_HORIZ_DIST,       8,
		NULL);

	xwid->notifywindow.label = xvw_create_label(object, "label");
	xvw_set_attributes(xwid->notifywindow.label,
	   XVW_LABEL,		"Please Wait",
	   XVW_FORCE_REDISPLAY, TRUE, 
	   XVW_BELOW,		xwid->notifywindow.icon,
	   XVW_ABOVE,		xwid->notifywindow.icon,
	   XVW_RIGHT_OF,	xwid->notifywindow.icon,
	   XVW_VERT_DIST,       8,
	   XVW_HORIZ_DIST,      8,
	   NULL);

	xwid->notifywindow.text = xvw_create_label(object, "text");
	xvw_set_attributes(xwid->notifywindow.text,
	   XVW_LABEL,		"   Initializing...   ",
	   XVW_FORCE_REDISPLAY, TRUE, 
	   XVW_BELOW,		xwid->notifywindow.icon,
	   XVW_ABOVE,		NULL,
	   XVW_LEFT_OF,		NULL,
	   XVW_RIGHT_OF,	NULL,
	   XVW_LABEL_JUSTIFY,	KLABEL_JUSTIFY_CENTER,
	   NULL);
}


/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: Free associated strings...
|
|         Input: widget - the notifywindow widget being destroyed
|    Written By: Mark Young
|          Date: Nov 01, 1993
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwNotifyWindowWidget xwid = kwidget(widget);
 
	kfree(xwid->notifywindow.label_string);
	kfree(xwid->notifywindow.text_string);
	kfree(xwid->notifywindow.title_string);
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This method will set up the window attributes
|                and create a window for the XvwNotifyWindowWidget. 
|                The desired initial window attributes are passed 
|                in via a mask and list of specification attributes.
|
|         Input: widget     - the widget to realize
|                valuemask  - determines the window attributes being passed in
|                attributes - the list of window attributes to be set
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Realize(
   Widget               widget,
   XtValueMask          *valuemask,
   XSetWindowAttributes *winattrib)
{
	XvwNotifyWindowWidget xwid = kwidget(widget);

	int	 mode;
	xvobject child;

        if ((mode = XDoesBackingStore(XtScreen(widget))) != NotUseful)
        {
           *valuemask |= CWBackingStore;
           winattrib->backing_store = mode;
        }
	(*xvwNotifyWindowWidgetClass->core_class.superclass->core_class.realize)
		(widget, valuemask, winattrib);

	child = xwid->notifywindow.icon;
	ManagerObjRealize(xvw_class(child), xvw_widget(child));
	child = xwid->notifywindow.label;
	ManagerObjRealize(xvw_class(child), xvw_widget(child));
	child = xwid->notifywindow.text;
	ManagerObjRealize(xvw_class(child), xvw_widget(child));
}

/*-----------------------------------------------------------
|
|  Routine Name: NotifySetString - select an item
|
|       Purpose: Resource attribute for setting  an item.  The
|
|         Input: xvobject - the parent object in which we are to select the item
|                attribute - the 
|                calldata - the item to be selected
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/

static int stop_it = FALSE;

void all_done(xvobject object, kaddr clientdata, int *stop_timer)
{
	stop_it = TRUE;

	*stop_timer = TRUE;
	
	return;
}

void sleep_and_process_events(xvobject object)
{
	xvw_add_timeout(object, 0.1, all_done, NULL);

	while (!stop_it) xvw_sync(FALSE);

	stop_it = FALSE;

	return;
}

/* ARGSUSED */
static int NotifySetString(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	xvobject toplevel;
        Widget   widget = xvw_widget(object);
        XvwNotifyWindowWidget xwid = kwidget(widget);


	if (kstrcmp(attribute, XVW_NOTIFYWINDOW_TITLE) == 0)
	{
	   String   string = *(String *) calldata;

	   toplevel = xvw_toplevel(object);
	   kfree_and_NULL(xwid->notifywindow.title_string);
	   xwid->notifywindow.title_string = kstrdup(string);
	   xvw_set_attribute(toplevel, XVW_SHELL_TITLE, 
			xwid->notifywindow.title_string);

	   xvw_sync(FALSE);
	}
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_LABEL) == 0)
	{
	   String   string = *(String *) calldata;

	   kfree_and_NULL(xwid->notifywindow.label_string);
	   xwid->notifywindow.label_string = kstrdup(string);
	   xvw_set_attribute(xwid->notifywindow.label, XVW_LABEL,
			xwid->notifywindow.label_string);

	   xvw_sync(FALSE);
	}
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_MESSAGE) == 0)
	{
	   String   string = *(String *) calldata;

	   kfree_and_NULL(xwid->notifywindow.text_string);
	   xwid->notifywindow.text_string = kstrdup(string);
	   xvw_set_attribute(xwid->notifywindow.text, XVW_LABEL,
			xwid->notifywindow.text_string);

	   xvw_sync(FALSE);
	}
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_VISIBLE) == 0)
	{
	   int visible = *(int *) calldata;

	   xwid->notifywindow.visible = visible;
	   if (xwid->notifywindow.visible)
	   {
	      if (xvw_check_toplevel(object) == FALSE)
	      {
	         xvw_map(object);
	         xvw_raise(object);
	      }
	      else
	         xvw_place(object, xwid->notifywindow.notifyfor, 0, 0);
	   }
	   else
	   {
	      xvw_unmap(object);
	      xvw_sync(FALSE);
	      return(TRUE);
	   }

	   xvw_sync(FALSE);

	   /* XSync(xvw_display(object), FALSE); */
	}
        else
        {
           kinfo(KSTANDARD,"NotifySetString:  Unknown xvattribute '%s' ",
			attribute);
           return(FALSE);
        }

	sleep_and_process_events(object);

	return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: NotifyGetString - select an item
|
|       Purpose: Resource attribute for setting  an item.  The
|
|         Input: xvobject - the parent object in which we are to select the item
|                attribute - the 
|                calldata - the item to be selected
|    Written By: Mark Young
|          Date: Apr 10, 1994
|
------------------------------------------------------------*/
/* ARGSUSED */
static int NotifyGetString(
   xvobject object,
   char     *attribute,
   kaddr    calldata)
{
	String   *string  = (String *) calldata;
	int      *visible = (int *) calldata;
        Widget   widget = xvw_widget(object);
        XvwNotifyWindowWidget xwid = kwidget(widget);


	if (kstrcmp(attribute, XVW_NOTIFYWINDOW_TITLE) == 0)
	   *string = xwid->notifywindow.title_string;
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_LABEL) == 0)
	   *string = xwid->notifywindow.label_string;
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_MESSAGE) == 0)
	   *string = xwid->notifywindow.text_string;
	else if (kstrcmp(attribute, XVW_NOTIFYWINDOW_VISIBLE) == 0)
	   *visible = xwid->notifywindow.visible;
        else
        {
           kinfo(KSTANDARD,"NotifyGetString:  Unknown xvattribute '%s' ",
			attribute);
           return(FALSE);
        }
	return(TRUE);
}


/************************************************************
*
*  Routine Name: xvw_create_notifywindow - create a notifywindow object
*
*       Purpose: The notifywindow object provides a convenient mechanism
*		 for the application to notify the user when it is performing
*		 an operation that will take some time, and that the user
*		 is expected to wait patiently until it is done.  
*
*		 The notifywindow object pops up without placement;
*		 it contains an icon (by default, an hourglass), a
*		 label, and a message so that the application may inform
*		 the user exactly why they are waiting. 
*
*		 With applications that will use the notifywindow periodically,
*		 it is recommended that a single notifywindow be created for the
*                application on startup, with \f(CWXVW_NOTIFYWINDOW_VISIBLE\fP
*		 set to FALSE.  Then, whenever necessary, the message displayed 
*		 by the notifywindow can be updated using the 
*		 \f(CWXVW_NOTIFYWINDOW_MESSAGE\fP attribute, and the 
*                notifywindow object popped up using the 
*                \f(CWXVW_NOTIFYWINDOW_VISIBLE\fP attribute.
*		 When the application is done, \f(CWXVW_NOTIFYWINDOW_VISIBLE\fP
*                can be set to FALSE again, until the next time the notifywindow
*		 object is to be popped up.
*
*         Input: parent - parent of the notifywindow object; NULL will cause
*                         a default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The notifywindow object on success, NULL on failure
*
*  Restrictions:
*    Written By: Neil Bowers and Mark Young
*          Date: Nov 24, 1993
*      Verified: 
*  Side Effects:
* Modifications:
*
*******************************************************************/

xvobject xvw_create_notifywindow(
   xvobject parent,
   char     *name)
{
	xvobject object;

	object = xvw_create(parent, TRUE, TRUE, name, NotifyWindowWidgetClass);
	xvw_unmap(object);
	return(object);
}
