/*
  * Khoros: $Id: Sash.c,v 1.3 1996/05/04 16:43:06 mirage Exp $
  */


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Sash Gadget Routines
   >>>>		
   >>>>   Static:
   >>>>                 ClassInitialize()
   >>>>                 Initialize()
   >>>>                 Destroy()
   >>>>                 Redisplay()
   >>>>                 SetValues()
   >>>>                 PositionGrip()
   >>>>   Public:
   >>>>                 xvw_create_sash()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"
#include <xvobjects/SashP.h>
#include <kwidgets/ManagerP.h>

static void ClassInitialize PROTO((void));
static void Initialize      PROTO((Widget, Widget, ArgList, Cardinal *));
static void Destroy  	    PROTO((Widget));
static void Redisplay	    PROTO((Widget, XEvent *, Region));
static Boolean SetValues    PROTO((Widget, Widget, Widget, ArgList,Cardinal *));
static void SashHandler     PROTO((xvobject, kaddr, XEvent *));
static void PositionGrip    PROTO((Widget));

/*-------------------------------------------------------------------*
|
|   Define Default Pixmap Paths
|
--------------------------------------------------------------------*/
 
#define GRIP_PIXMAP "$DESIGN/objects/library/xvobjects/misc/pixmaps/sash_grip.xpm"

/*-------------------------------------------------------
|
|   Full class attributes
|
--------------------------------------------------------*/

static xvattribute attributes[] = {
{XVW_SASH_GRIP_OBJECT,     NULL,     XtRObject,     NULL},
{XVW_SASH_GRIP_PIXMAP,     NULL,     XtRPixmap,     NULL},
};

/*-------------------------------------------------------
|
|   Full class record constant
|
--------------------------------------------------------*/

#define offset(field) XtOffsetOf(XvwSashGadgetRec, sash.field)

static XtResource resources[] = { 
{XVW_SASH_GRIP_OBJECT, NULL,  XtRObject, sizeof(xvobject),
        offset(grip), XtRImmediate, (XtPointer) NULL},
{XVW_SASH_GRIP_PIXMAP, NULL,  XtRPixmap, sizeof(Pixmap),
        offset(grip_pixmap), XtRString, (XtPointer) GRIP_PIXMAP},

{XVW_BORDER_WIDTH, NULL, XtRDimension, sizeof(Dimension),
        XtOffsetOf(XvwSashGadgetRec, rectobj.border_width),
        XtRImmediate, (XtPointer) 0},
};
#undef offset


#define SUPERCLASS (&xvwSeparatorGadgetClassRec)

XvwSashGadgetClassRec xvwSashGadgetClassRec =
{
  {
    (WidgetClass) SUPERCLASS,		/* superclass		  */	
    "Sash",				/* class_name		  */
    sizeof(XvwSashGadgetRec),		/* size			  */
    ClassInitialize,			/* class_initialize	  */
    NULL,				/* class_part_initialize  */
    FALSE,				/* class_inited		  */
    Initialize,				/* initialize		  */
    NULL,				/* initialize_hook	  */
    NULL,				/* not used - obj	  */
    NULL,				/* actions		  */
    0,					/* num_actions		  */
    resources,				/* resources		  */
    XtNumber(resources),		/* resource_count	  */
    NULLQUARK,				/* xrm_class		  */
    FALSE,				/* compress_motion	  */
    XtExposeCompressMaximal,		/* compress_exposure	  */
    FALSE,				/* compress_enterleave    */
    FALSE,				/* visible_interest	  */
    Destroy,				/* destroy		  */
    XtInheritResize,			/* resize		  */
    Redisplay,				/* expose		  */
    SetValues,				/* set_values		  */
    NULL,				/* set_values_hook	  */
    XtInheritSetValuesAlmost,		/* set_values_almost	  */
    NULL,				/* get_values_hook	  */
    NULL,				/* accept_focus		  */
    XtVersion,				/* version		  */
    NULL,				/* callback_private	  */
    NULL,				/* tm_table		  */
    NULL,				/* query_geometry	  */
    NULL,				/* display_accelerator	  */
    NULL				/* extension		  */
  },  /* RectObjClass fields initialization */
  {
    XtInheritPicking,			/* pick object proc       */
    XtInheritEraseSel,                  /* erase selected proc    */
    XtInheritRefreshSel,	        /* refresh selection proc */
    NULL,			        /* change geometry proc   */
  },  /* XvwManagerGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwSeparatorGadgetClass fields initialization */
  {
    NULL,				/* extension - not used   */
  },  /* XvwSashGadgetClass fields initialization */
};
#undef SUPERCLASS

/*
 * xvwSashGadgetClass for public consumption
 */
WidgetClass xvwSashGadgetClass = (WidgetClass) &xvwSashGadgetClassRec;


/*-----------------------------------------------------------
|
|  Routine Name: ClassInitialize
|
|       Purpose: This method is called the first time an  
|                instance of sash object class has been created.
|                It will initialize all the class attributes. 
|
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void ClassInitialize(void)
{
	xvw_init_attributes(xvwSashGadgetClass, attributes,
		XtNumber(attributes), NULL, 0, 
		"$DESIGN/objects/library/xvobjects/uis/Sash.pane");
	xvw_load_resources("$DESIGN/objects/library/xvobjects/app-defaults/Sash");
}

/*-----------------------------------------------------------
|
|  Routine Name: Realize
|
|       Purpose: This routine initializes the Sash by creating the grip.
|		 You ignorant fool!
|
|         Input: request - not used
|                new     - widget instance after initialization
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Initialize(
   Widget request,
   Widget new,
   ArgList args,
   Cardinal *num_args)
{
	XvwSashGadget xobj = (XvwSashGadget) new;

	xvobject object = xvw_object(new);
	xvobject parent = xvw_parent(object);

	xobj->sash.grip = xvw_create_pixmap(parent, "grip");
	xvw_set_attributes(xobj->sash.grip,
		XVW_PIXMAP, xobj->sash.grip_pixmap,
		NULL);
	xvw_add_event(xobj->sash.grip, ButtonPressMask | ButtonReleaseMask |
		ButtonMotionMask, SashHandler, (kaddr) XtParent(new));
	PositionGrip(new);
}

/*-----------------------------------------------------------
|
|  Routine Name: Destroy
|
|       Purpose: This method will free the label string.
|
|         Input: widget - the widget being destroyed
|    Written By: Mark Young
|          Date: Apr 17, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Destroy(
   Widget widget)
{
        XvwSashGadget xobj = (XvwSashGadget) widget;

	Widget parent = XtParent(widget);

	if (parent->core.being_destroyed)
	   return;

	xvw_destroy(xobj->sash.grip);
}

/*-----------------------------------------------------------
|
|  Routine Name: Redisplay
|
|       Purpose: This routine will redraw the separator object and then the
|                grip in order to make sure that the grip is not visually
|		 obscurred.
|
|         Input: widget - the separator object that was exposed 
|                event  - the event that caused the redraw
|                region - the region that was exposed 
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void Redisplay(
   Widget widget,
   XEvent *event,
   Region region)
{
	XvwSashGadget xobj = (XvwSashGadget) widget;

	Widget grip;

	(*xvwSashGadgetClass->core_class.superclass->core_class.expose)
                (widget, event, region);

	grip = xvw_widget(xobj->sash.grip);
	ManagerObjForceRedisplay(grip, grip, FALSE, NULL);
}

/*-----------------------------------------------------------
|
|  Routine Name: SetValues
|
|       Purpose: If the the size or postion of the sash has changed, 
|		 recalculate the size and position of the sash.
|
|         Input: current - the object containing current settings 
|                request - the object containing requested settings
|                new     - the object processed through all set values methods 
|       Returns: TRUE (1) if redisplay is required, FALSE (0) otherwise
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static Boolean SetValues(
   Widget   current,
   Widget   request,
   Widget   new,
   ArgList  args,
   Cardinal *num_args)
{
	XvwSashGadget cobj = (XvwSashGadget) current;
	XvwSashGadget nobj = (XvwSashGadget) new;

	if (current->core.y != new->core.y &&
	    nobj->separator.type == KSEPARATOR_ORIENTATION_HORIZ)
	{
	   PositionGrip(new);
	}
	else if (cobj->rectobj.x != nobj->rectobj.x &&
	         nobj->separator.type == KSEPARATOR_ORIENTATION_VERT)
	{
	   PositionGrip(new);
	}
	else if (cobj->separator.type != nobj->separator.type)
	   PositionGrip(new);

	return(FALSE);
}

/*-----------------------------------------------------------
|
|  Routine Name: SashHandler
|
|       Purpose: Sash handler for making the grip do it's thang!
|
|         Input: object - the grip objet
|		 client_data - the parent (Manager widget)
|		 event  - the event that got us here
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void SashHandler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
	Widget parent = (Widget) client_data;

	ManagerWidgetHandler(object, client_data, event);
	if (event->type == ButtonRelease)
	   xvw_set_attribute(xvw_object(parent), XVW_SELECT_DELETE, object);
}

/*-----------------------------------------------------------
|
|  Routine Name: PositionGrip
|
|       Purpose: Position the sash and grip according to orientation, etc.
|
|         Input: widget - the sash widget
|    Written By: Mark Young
|          Date: Mar 03, 1996
|
------------------------------------------------------------*/
/* ARGSUSED */
static void PositionGrip(
   Widget   widget)
{
	XvwSashGadget xobj = (XvwSashGadget) widget;

	xvobject object;

	if (xobj->sash.grip == NULL)
	   return;

	object = xvw_object(widget);
	if (xobj->separator.type == KSEPARATOR_ORIENTATION_HORIZ)
	{
	   xvw_set_attributes(xobj->sash.grip,
		XVW_LEFT_OF,  NULL,
		XVW_ABOVE,    KMANAGER_UNDEFINED,
		XVW_VERT_DIST,   0,
		XVW_HORIZ_DIST,  30,
		XVW_YPOSITION,   xobj->rectobj.y,
		NULL);
	   xvw_set_attributes(object,
		XVW_LEFT_OF,  NULL,
		XVW_RIGHT_OF, NULL,
		XVW_ABOVE,    xobj->sash.grip,
		XVW_BELOW,    xobj->sash.grip,
		XVW_TACK_EDGE, KMANAGER_TACK_ALL,
		NULL);
	}
	else
	{
	   xvw_set_attributes(xobj->sash.grip,
		XVW_LEFT_OF,  KMANAGER_UNDEFINED,
		XVW_BELOW,    NULL,
		XVW_VERT_DIST,   30,
		XVW_HORIZ_DIST,  0,
		XVW_XPOSITION,   xobj->rectobj.x,
		NULL);
	   xvw_set_attributes(object,
		XVW_LEFT_OF,  xobj->sash.grip,
		XVW_RIGHT_OF, xobj->sash.grip,
		XVW_TACK_EDGE, KMANAGER_TACK_ALL,
		XVW_ABOVE,    NULL,
		XVW_BELOW,    NULL,
		NULL);
	}
}

/************************************************************
*
*  Routine Name: xvw_create_sash - create a sash object
*
*       Purpose: A sash object simply supports display of a 
*		 bitmap or sash, as defined by an xbm or xpm file.
*
*         Input: parent - the parent object; NULL will cause a
*                         default toplevel to be created automatically
*                name   - a name for this particular instance of the object
*                         (for use in app-defaults files, etc)
*
*        Output:
*       Returns: The sash object on success, NULL on failure
*  Restrictions: 
*    Written By: Mark Young 
*          Date: Mar 03, 1996
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/

xvobject xvw_create_sash(
   xvobject parent,
   char   *name)
{
	xvobject  object;


	/*
	 *  Create the line object
	 */
	object = xvw_create(parent, FALSE, TRUE, name, SashGadgetClass);
	if (object != NULL)
	{
	   xvw_set_attributes(object,
                XVW_MENUABLE,     TRUE,     /* menuable       */
                XVW_RESIZABLE,    FALSE,    /* resizable      */
                XVW_SELECTABLE,   TRUE,     /* selectable     */
		NULL);
	}
	return(object);
}
