 /*
  * Khoros: $Id: help.c,v 1.11 1996/06/11 17:25:17 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: help.c,v 1.11 1996/06/11 17:25:17 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		        Online Help 
   >>>>
   >>>>   Static:
   >>>>                 xvu_quit_help()
   >>>>   Public:
   >>>>                 xvu_help_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int  xvu_help_done  = FALSE;


/*-----------------------------------------------------------
|
|  Routine Name: xvu_quit_help()
|
|       Purpose: Quits the help object.
|
|         Input: object      - the quit button
|                client_data - stores the Help_Data structure
|                call_data   - unused
|
|        Output: none
|          Date: Jul 9, 1992
|    Written By: Danielle Argiro
| Modifications: Converted from Khoros 1.0 (DA)
|
------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_quit_help(
   xvobject object,
   kaddr  client_data,
   kaddr  call_data)
{
	/*
	 * set the toplevel to designate the we are done
	 */
	xvu_help_done = TRUE;
}

/************************************************************
*
*  Routine Name: xvu_help_wait - display help file or help directory
*
*       Purpose: Create a large read-only text object in which one or
*		 more ascii files may be displayed.
*
*		 If the path specified is to a single file, that
*		 file is displayed. If the path specified is a directory, 
*		 the text object will have one button at the top labeled
*		 with the name of each file in the directory.  Clicking
*		 on such a button will cause the contents of the file labeled
*		 to appear in the text object.  In either case, the on-line
*		 help object will be displayed until the user clicks on the
*		 "Quit" button.  This utility will not grab events from the 
*		 application program.
*
*         Input: path   - the path of the (single) file to be displayed, 
*                         or the directory containing the (multiple) files 
*                         to be displayed in the help object.  
*			  The path may include a '~' if desired, or may contain
*			  a reference to $TOOLBOX.
*
*                label  - short label for top of help object;
*			  NULL gives default of "Help".
*
*        Output:
*	Returns: TRUE on success, FALSE on failure
*  Restrictions:
*    Written By: Danielle Argiro & Mark Young
*          Date: Jul 9, 1992
*      Verified:
*  Side Effects:
* Modifications: Converted from Khoros 1.0 (DA)
*
*************************************************************/

int xvu_help_wait(
   char *path,
   char *label)
{
	xvobject help;
	char     buffer[KLENGTH];
	char     name[KLENGTH];

	xvu_initialize();

	xvu_help_done = FALSE;
	if (xvw_display(NULL) == NULL)
	{
	    kinfo(KSTANDARD, "see '%s'\n", path);
	    return(TRUE);
	}

	if (label != NULL)
	    ksprintf(buffer, "%s", label);
	else ksprintf(buffer, "Online Help");

	help = xvw_create_help(NULL, "online_help");
	xvw_set_attributes(help,
		XVW_HELP_TITLE,	    buffer,
		XVW_HELP_FILENAME,  path,
		NULL);

	(void) ksprintf(name, "Online Help for %s", kprog_get_program());
	xvw_set_attribute(xvw_toplevel(help), XVW_SHELL_TITLE, name);

	xvw_add_callback(help, XVW_HELP_CALLBACK, xvu_quit_help, NULL);

	while(!xvu_help_done)
	    xvw_process_event();  

	return(TRUE);
}
