 /*
  * Khoros: $Id: quit.c,v 1.7 1996/06/25 16:34:03 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: quit.c,v 1.7 1996/06/25 16:34:03 mirage Exp $";
#endif



/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>                 Quit Dialog Box
   >>>>  Private:
   >>>>                 xvu_create_quit()
   >>>>   Static:
   >>>>                 xvu_quit_yes()
   >>>>                 xvu_quit_cancel()
   >>>>   Public:
   >>>>                 xvu_quit_wait()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static int xvu_quit_done  = FALSE;
static int xvu_quit_signal;

static void xvu_quit_accept   PROTO((xvobject, kaddr, kaddr));
static void xvu_quit_cancel   PROTO((xvobject, kaddr, kaddr));

#define QUIT_PIXMAP "$DESIGN/objects/library/xvutils/misc/pixmaps/quit.xpm"

/*-----------------------------------------------------------------
|
|  Routine Name: xvu_quit
|
|       Purpose: xvu_quit produces standardized prompting for asking the
|		 user if they wish to quit an application.  It will be set
|		 up as the quit handler for kquit in all X-based Khoros
|		 routines by xvf_initialize().
|
|         Input: message   - grammatically correct, clear prompt asking
|                            the user if they wish to quit the application.
|        Output:
|    Written By: Mark Young
|          Date: Apr 16, 1996
| Modifications:
|
--------------------------------------------------------------*/

int xvu_quit(
   char *message)
{
	knotify handler;
	int     status;

	handler = kset_quithandler(NULL);
	status = xvu_quit_wait(message, NULL, NULL);

	(void) kset_quithandler(handler);
        return(status);
}

/************************************************************
*
*  Routine Name: xvu_quit_wait - pop up a quit dialog box; wait for response
*
*       Purpose: Creates and maps a pop-up quit dialog box which looks like:
*
*                !
*		 ! 	  -------------------------------
*                !        |                             |
*                !        |     Please Confirm Exit     |
*                !        |       Exit program?         |
*                !        |                             |      
*		 !  	  - - - - - - - - - - - - - - - -
*                !        |  --------      ----------   |      
*                !        |  | Exit |      | Cancel |   |      
*                !        |  --------      ----------   |      
*		 !  	  -------------------------------
*                !
*
*                The quit dialog box will block input to the application until
*                it is acknowledged by the user.  The user must click on 
*                "Exit" or "Cancel" before control will be returned to 
*                the application program.
*
*        Input:   string       - Optional string for first line of
*                                label; if non-NULL, will replace
*                                "Please Confirm Exit\nExit program?"
*
*		  quit_label   - Optional label for button which returns 
*                                value of 1; if non-NULL, will replace label
*                                of "Exit" on first button 
*
*		  cancel_label - Optional label for button which returns 
*                                value of 0; if non-NULL, will replace label
*                                of "Cancel" on first button
*
*        Output:
*	Returns:  1 if user selected "Exit" (first) button
*		  0 if user selected "Cancel" (second) button
*
*  Restrictions: 
*    Written By: Danielle Argiro  
*          Date: April 16, 1996
*      Verified:
*  Side Effects:
* Modifications: 
*
*************************************************************/

int xvu_quit_wait(
   char *string,
   char *quit_label,
   char *cancel_label)
{
	xvu_quit_done = FALSE;

	if (!(xvu_create_quit(string, quit_label, cancel_label)))
	    return(FALSE);

	while(!xvu_quit_done)
	    xvw_process_event(); 

	return(xvu_quit_signal);
}


/*------------------------------------------------------------
|
|  Routine Name: xvu_create_quit()
|
|       Purpose: This utility routine actually creates the quit dialog box:
|
|                       -------------------------------
|                       |                             |
|                       |     Please Confirm Exit     |
|                       |       Exit program?         |
|                       |                             |
|                       - - - - - - - - - - - - - - - -
|                       |  --------      ----------   |
|                       |  | Exit |      | Cancel |   |
|                       |  --------      ----------   |
|                       -------------------------------
|
|        Input:   string       - Optional string for second line of
|                                label; if non-NULL, will replace
|                                "Please Confirm Exit\nExit Program"?
|
|                 quit_label   - Optional label for button which returns
|                                value of 1; if non-NULL, will replace label
|                                of "Exit" on first button
|
|                 cancel_label - Optional label for button which returns
|                                value of 0; if non-NULL, will replace label
|                                of "Cancel" on first button
|
|        Output: Returns TRUE on success, FALSE on failure
|          Date: April 16, 1996
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/

int xvu_create_quit(
   char *string,
   char *quit_label,
   char *cancel_label)
{
	xvobject back;        /* backplane for entire quit dialog box       */
	xvobject label;       /* label "Please Confirm Exit\nExit Program?" */
	xvobject quit;        /* "Exit" button                              */
	xvobject separator;   /* separator to create visual seperation      */
	xvobject cancel;      /* "Cancel" button                            */
	xvobject pixmap;      
	int      height;

	char     temp[KLENGTH];
	char     name[KLENGTH];

	xvu_initialize();

        /* create the backplane object */
	back = xvw_create(NULL, TRUE, TRUE, "quit_dialog", ManagerWidgetClass);

	(void) ksprintf(name, "Quit Dialog for %s", kprog_get_program());
	xvw_set_attribute(xvw_toplevel(back), XVW_SHELL_TITLE, name);

        /* set strings to appear as labels on buttons */
        if (quit_label == NULL)
           ksprintf(temp, "Exit");
        else ksprintf(temp, "%s", quit_label);

        /* create the "Exit" button (returns 1) */
        quit = xvw_create_button(back, "quit_button");
        cancel = xvw_create_button(back, "cancel_button");

        xvw_set_attributes(quit,
                           XVW_LABEL,       temp,
                           XVW_CHAR_WIDTH,  7.0,
                           XVW_CHAR_HEIGHT, 1.25,
                           XVW_RIGHT_OF,    NULL,
                           XVW_LEFT_OF,     cancel,
                           XVW_ABOVE,       NULL,
                           XVW_VERT_DIST,   8,
                           NULL);
        xvw_add_callback(quit, XVW_BUTTON_SELECT, xvu_quit_accept, back);

        if (cancel_label == NULL)
           ksprintf(temp, "Cancel");
        else ksprintf(temp, "%s", cancel_label);

        /* create the "Cancel" button (returns 0) */
        xvw_set_attributes(cancel,
                           XVW_LABEL,       temp,
                           XVW_CHAR_WIDTH,  7.0,
                           XVW_CHAR_HEIGHT, 1.25,
                           XVW_LEFT_OF,     NULL,
                           XVW_RIGHT_OF,    quit,
                           XVW_ABOVE,       NULL,
                           XVW_VERT_DIST,   8,
                           NULL);
        xvw_add_callback(cancel, XVW_BUTTON_SELECT, xvu_quit_cancel, back);

	separator = xvw_create_separator(back, "separator");
	xvw_set_attributes(separator, 
			   XVW_ABOVE,     cancel,
			   XVW_VERT_DIST, 8,
			   NULL);

	/* set strings to appear as labels on the quit object */
	if (string == NULL)
	    ksprintf(temp, "Please Confirm Exit\nExit %s?",
		     kprog_get_program());
	else ksprintf(temp, "%s", string);

	height = xvu_figure_height(temp, 32);
	xvw_set_attributes(back,
			   XVW_CHAR_WIDTH,  32.0,
			   XVW_CHAR_HEIGHT, 5.0,
			   NULL);

	pixmap = xvw_create_pixmap(back, "pixmap");
        xvw_set_attributes(pixmap,
                XVW_PIXMAP_FILENAME, QUIT_PIXMAP,
                XVW_BELOW,           NULL,
                XVW_RIGHT_OF,        NULL,
                XVW_ABOVE,           separator,
                XVW_VERT_DIST,       8,
                XVW_HORIZ_DIST,      8,
                NULL);


	/* create the first label */
        label = xvw_create_label(back, "label");
        xvw_set_attributes(label,
		           XVW_LABEL,                 temp,  
                           XVW_BELOW,                 NULL,   
                           XVW_ABOVE,                 separator,   
                           XVW_LEFT_OF,               NULL,   
                           XVW_RIGHT_OF,              NULL,   
                           XVW_LABEL_JUSTIFY,         KLABEL_JUSTIFY_CENTER,   
			   XVW_TACK_EDGE,	      KMANAGER_TACK_ALL,
			   XVW_VERT_DIST,	      8,
                           XVW_HORIZ_DIST,            8,
                           NULL);

        /* Selecting "delete" with window manager will equal hitting "Cancel" */
	xvw_add_protocol(back, "WM_DELETE_WINDOW", xvu_quit_cancel, back);

	/* Map and place the quit dialog box */
	xvw_place(back, quit, 0, 0);

	/* good job, eh? */
	return(TRUE);
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_quit_accept()
|
|       Purpose: Destroys quit dialog box when the user clicks on
|                the "Exit" button; returns status of 1.
|
|         Input: object      - the "Exit" acknowledgement button
|                client_data - passes in the backplane
|                call_data   - unused
|        Output: 
|          Date: April 16, 1996         
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_quit_accept(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject back = (xvobject) client_data;

	xvw_unmap(back);
	xvw_destroy(back);
	xvu_quit_done = TRUE;
	xvu_quit_signal = 1;
}

/*------------------------------------------------------------
|
|  Routine Name: xvu_quit_cancel()
|
|       Purpose: Destroys quit dialog box when the user clicks on
|                the "Cancel" button; returns status of 0.
|
|         Input: object      - the "Cancel" acknowledgement button
|                client_data - passes in the backplane
|                call_data   - unused
|        Output: 
|          Date: April 16, 1996 
|    Written By: Danielle Argiro
| Modifications: 
|
-------------------------------------------------------------*/
/* ARGSUSED */
static void xvu_quit_cancel(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	xvobject back = (xvobject) client_data;

	xvw_unmap(back);
	xvw_destroy(back);
	xvu_quit_done = TRUE;
	xvu_quit_signal = 0;
}

