 /*
  * Khoros: $Id: do_pref.c,v 1.6 1996/07/04 02:55:14 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_pref.c,v 1.6 1996/07/04 02:55:14 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for guidepane pref
   >>>> 
   >>>>  Private: 
   >>>> 	pref_apply_changes
   >>>> 	pref_restore_defaults
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

/*-----------------------------------------------------------
| 
|  Routine Name: pref_apply_changes
| 
|       Purpose: Do routine which is called when
|                  pane action button ok is used
| 
|         Input: pref_info - ptr to SubformInfo struct for pref subform
| 
|        Output: None
|    Written By: Danielle Argiro & Becky Bishop
|          Date: June 18, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void pref_apply_changes(
     cantata_pref *pref_info)
{
	char *background_color = NULL;

        pref_grid       *canvas_info = pref_info->grid;
        pref_glyph      *glyph_info  = pref_info->glyph;
        pref_workspace  *wksp_info   = pref_info->workspace;
        pref_commandbar *cmdbar_info = pref_info->commandbar;
	
	/*
	 *  Translate values on Canvas Grid pane to workspace attributes
	 */
	xvw_set_attributes(main_workspace, 
		XVW_CANVAS_GRID,      canvas_info->griddisplay_val,
	        XVW_CANVAS_GRIDSIZE,  canvas_info->gridsize,
		XVW_FOREGROUND_COLOR, canvas_info->gridcolor,
		NULL);
  
	/*
         *  Translate values on Glyph pane to workspace attributes
         */
        xvw_set_attributes(main_workspace, 
	    	XVW_CONNECTION_TYPE,          glyph_info->connecttype_num,
            	XVW_REFRESH_CHILDREN_ON_EDIT, glyph_info->connectredraw_val,
	   	XVW_NODE_PLACEMENT,           glyph_info->glyphplacement_val,
                XVW_NODE_LEAVE,               glyph_info->glyphdisplay_val,
                XVW_GLYPH_REPORTING,          glyph_info->glyphreport_val,
		NULL);
  
	/*
         *  Translate values on Workspace pane to workspace attributes
         */
        xvw_set_attribute(main_workspace, XVW_GLYPH_ECHO, 
		          pref_info->workspace->echoexec_val);
  
        if (wksp_info->backgroundcolor_optsel)
        {
           xvw_set_attribute(main_workspace, 
  			   XVW_BACKGROUND_PIXMAPFILE, "Unspecified");
  
           if (kstrcmp(wksp_info->backgroundcolor, "default") == 0)
              background_color = kstrdup("#979697");
           else background_color = kstrdup(wksp_info->backgroundcolor);
           xvw_set_attribute(main_workspace, 
  	  		     XVW_BACKGROUND_COLOR, background_color);
           kfree(background_color);
        }
        else if (wksp_info->backpixmaps_optsel &&
		 wksp_info->backpixmaps != NULL)
           xvw_set_attribute(main_workspace,
  	         XVW_BACKGROUND_PIXMAPFILE, pref_info->workspace->backpixmaps);
  
	/*
         *  Translate values on Command Bar pane to workspace attributes
         */
	xvw_set_attributes(commandbar, 
		  XVW_COMMANDBAR_SHOW_RUN,         cmdbar_info->runwksp,
		  XVW_COMMANDBAR_SHOW_STOP,        cmdbar_info->stopwksp,
                  XVW_COMMANDBAR_SHOW_RUNMODE,     cmdbar_info->runmode,
                  XVW_COMMANDBAR_SHOW_RESET,       cmdbar_info->resetwksp,
                  XVW_COMMANDBAR_SHOW_CLEAR,       cmdbar_info->clearwksp,
                  XVW_COMMANDBAR_SHOW_CHECK,       cmdbar_info->checkwksp,
                  XVW_COMMANDBAR_SHOW_REDRAW,      cmdbar_info->redrawwksp,
                  XVW_COMMANDBAR_SHOW_INFO,        cmdbar_info->wkspinfo,
                  XVW_COMMANDBAR_SHOW_CUT,         cmdbar_info->cut,
                  XVW_COMMANDBAR_SHOW_COPY,        cmdbar_info->copy,
                  XVW_COMMANDBAR_SHOW_PASTE,       cmdbar_info->paste,
                  XVW_COMMANDBAR_SHOW_CLIPBOARD,   cmdbar_info->showclipboard,
                  XVW_COMMANDBAR_SHOW_DUPLICATE,   cmdbar_info->duplicate,
                  XVW_COMMANDBAR_SHOW_SELECTALL,   cmdbar_info->selectall,
                  XVW_COMMANDBAR_SHOW_UNSELECTALL, cmdbar_info->unselectall,
                  XVW_COMMANDBAR_SHOW_DELETE,      cmdbar_info->delete,
                  XVW_COMMANDBAR_SHOW_UNDO,        cmdbar_info->undodelete,
                  XVW_COMMANDBAR_SHOW_PROCEDURE,   cmdbar_info->createprocedure,
                  XVW_COMMANDBAR_SHOW_COUNTLOOP,   cmdbar_info->createcount,
                  XVW_COMMANDBAR_SHOW_WHILELOOP,   cmdbar_info->createwhile,
                  XVW_COMMANDBAR_SHOW_OPEN,        cmdbar_info->openwksp,
                  XVW_COMMANDBAR_SHOW_SAVE,        cmdbar_info->savewksp,
                  XVW_COMMANDBAR_SHOW_SAVEAS,      cmdbar_info->savewkspas,
		  NULL);
}


/*-----------------------------------------------------------
| 
|  Routine Name: pref_restore_defaults
| 
|       Purpose: Do routine which is called when
|                  pane action button defaults is used
| 
|         Input: pref_info - ptr to SubformInfo struct for pref subform
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 19, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void pref_restore_defaults(
     cantata_pref *pref_info)
{
	pref_grid       *canvas_info = pref_info->grid;
	pref_glyph      *glyph_info  = pref_info->glyph;
	pref_workspace  *wksp_info   = pref_info->workspace;
	pref_commandbar *cmdbar_info = pref_info->commandbar;
	char            *background_color = NULL;

	/*
	 *  Restore values on Canvas Grid subform according to defaults;
	 *  Restore canvas grid workspace attributes according to defaults
	 */
	canvas_info->griddisplay_val = pref_def_griddisplay;
        canvas_info->griddisplay_num = pref_def_griddisplay+1;
        xvf_set_attribute(canvas_info->griddisplay_struct, 
			  XVF_TOGGLE_NUM, canvas_info->griddisplay_num);

        kfree(canvas_info->gridcolor);
        canvas_info->gridcolor = kstrdup(pref_def_gridcolor);
        xvf_set_attribute(canvas_info->gridcolor_struct,
                          XVF_STRING_VAL, pref_def_gridcolor);

        canvas_info->gridsize = pref_def_gridsize;
        xvf_set_attribute(canvas_info->gridsize_struct,
                          XVF_INT_VAL, pref_def_gridsize);

	if (!pref_def_griddisplay)
	{
            xvf_set_attribute(canvas_info->gridsize_struct,
                              XVF_ACTIVATE, FALSE);
            xvf_set_attribute(canvas_info->gridcolor_struct,
                              XVF_ACTIVATE, FALSE);
            xvf_set_attribute(canvas_info->gridcolor_struct,
                              XVF_ACTIVATE, FALSE);
	}

	xvw_set_attributes(main_workspace,
                   XVW_CANVAS_GRID,      pref_def_griddisplay,
                   XVW_FOREGROUND_COLOR, pref_def_gridcolor,
                   XVW_CANVAS_GRIDSIZE,  pref_def_gridsize,
                   NULL);

	/*
         *  Restore values on Glyph subform according to defaults;
         *  Restore glyph workspace attributes according to defaults
         */
        glyph_info->connecttype_val = pref_def_connecttype; 
	glyph_info->connecttype_num = pref_def_connecttype;
        xvf_set_attribute(glyph_info->connecttype_struct, 
			  XVF_TOGGLE_NUM, glyph_info->connecttype_num);

        glyph_info->connectredraw_val = pref_def_connectredraw; 
	glyph_info->connectredraw_num = pref_def_connectredraw+1;
        xvf_set_attribute(glyph_info->connectredraw_struct, 
			  XVF_TOGGLE_NUM, glyph_info->connectredraw_num);

        glyph_info->glyphplacement_val = pref_def_glyphplacement; 
	glyph_info->glyphplacement_num = pref_def_glyphplacement+1;
        xvf_set_attribute(glyph_info->glyphplacement_struct, 
			  XVF_TOGGLE_NUM, glyph_info->glyphplacement_num);

        glyph_info->glyphdisplay_val = pref_def_glyphdisplay; 
	glyph_info->glyphdisplay_num = pref_def_glyphdisplay+1;
	xvf_set_attribute(glyph_info->glyphdisplay_struct, 
			  XVF_TOGGLE_NUM, glyph_info->glyphdisplay_num);

        glyph_info->glyphreport_val = pref_def_glyphreport; 
	glyph_info->glyphreport_num = pref_def_glyphreport+1;
        xvf_set_attribute(glyph_info->glyphreport_struct, 
			  XVF_TOGGLE_NUM, glyph_info->glyphreport_num);

        xvw_set_attributes(main_workspace,
                   XVW_CONNECTION_TYPE,          pref_def_connecttype,
                   XVW_REFRESH_CHILDREN_ON_EDIT, pref_def_connectredraw,
                   XVW_NODE_PLACEMENT,           pref_def_glyphplacement,
                   XVW_NODE_LEAVE,               pref_def_glyphdisplay,
                   XVW_GLYPH_REPORTING,          pref_def_glyphreport,
                   NULL);

        /*
         *  Restore values on Workspace subform according to defaults;
         *  Restore glyph workspace attributes according to defaults
         */

	wksp_info->echoexec_val = pref_def_echoexec;
	wksp_info->echoexec_num = pref_def_echoexec+1;
        xvf_set_attribute(glyph_info->glyphplacement_struct,
                          XVF_TOGGLE_NUM, wksp_info->echoexec_num);

	xvf_set_attribute(wksp_info->backgroundcolor_struct, XVF_OPTSEL, 1);
        xvw_set_attribute(main_workspace, XVW_BACKGROUND_PIXMAPFILE, 
			  "Unspecified");
 
        if (kstrcmp(pref_def_backgroundcolor, "default") == 0)
              background_color = kstrdup("#979697");
        else background_color = kstrdup(pref_def_backgroundcolor);
        xvw_set_attribute(main_workspace, XVW_BACKGROUND_COLOR,
		          background_color);
	kfree(background_color);

        kfree(wksp_info->backgroundcolor);
        wksp_info->backgroundcolor = kstrdup(pref_def_backgroundcolor);
        xvf_set_attribute(wksp_info->backgroundcolor_struct,
                          XVF_STRING_VAL, pref_def_backgroundcolor);

	kfree(wksp_info->backpixmaps);
        wksp_info->backpixmaps = kstrdup(pref_def_backpixmaps);
	xvf_set_attribute(wksp_info->backpixmaps_struct,
                          XVF_STRING_VAL, pref_def_backpixmaps);

	xvw_set_attributes(main_workspace,
                   XVW_GLYPH_ECHO,            pref_def_echoexec,
                   XVW_BACKGROUND_COLOR,      pref_def_backgroundcolor,
                   NULL);

	if (pref_def_backpixmaps != NULL &&
	    kstrcmp(pref_def_backpixmaps, "Unspecified") != 0)
            xvw_set_attribute(main_workspace, XVW_BACKGROUND_PIXMAPFILE, 
  			      pref_def_backpixmaps);

        /*
         *  Restore values on Command bar subform according to defaults;
         *  Restore command bar attributes according to defaults
         */
	xvw_set_attributes(commandbar,
                  XVW_COMMANDBAR_SHOW_RUN,         pref_def_runwksp,
                  XVW_COMMANDBAR_SHOW_STOP,        pref_def_stopwksp,
                  XVW_COMMANDBAR_SHOW_RUNMODE,     pref_def_runmode,
                  XVW_COMMANDBAR_SHOW_RESET,       pref_def_resetwksp,
                  XVW_COMMANDBAR_SHOW_CLEAR,       pref_def_clearwksp,
                  XVW_COMMANDBAR_SHOW_CHECK,       pref_def_checkwksp,
                  XVW_COMMANDBAR_SHOW_REDRAW,      pref_def_redrawwksp,
                  XVW_COMMANDBAR_SHOW_INFO,        pref_def_wkspinfo,
                  XVW_COMMANDBAR_SHOW_CUT,         pref_def_cut,
                  XVW_COMMANDBAR_SHOW_COPY,        pref_def_copy,
                  XVW_COMMANDBAR_SHOW_PASTE,       pref_def_paste,
                  XVW_COMMANDBAR_SHOW_CLIPBOARD,   pref_def_showclipboard,
                  XVW_COMMANDBAR_SHOW_DUPLICATE,   pref_def_duplicate,
                  XVW_COMMANDBAR_SHOW_SELECTALL,   pref_def_selectall,
                  XVW_COMMANDBAR_SHOW_UNSELECTALL, pref_def_unselectall,
                  XVW_COMMANDBAR_SHOW_DELETE,      pref_def_delete,
                  XVW_COMMANDBAR_SHOW_UNDO,        pref_def_undodelete,
                  XVW_COMMANDBAR_SHOW_PROCEDURE,   pref_def_createprocedure,
                  XVW_COMMANDBAR_SHOW_COUNTLOOP,   pref_def_createcount,
                  XVW_COMMANDBAR_SHOW_WHILELOOP,   pref_def_createwhile,
                  XVW_COMMANDBAR_SHOW_OPEN,        pref_def_openwksp,
                  XVW_COMMANDBAR_SHOW_SAVE,        pref_def_savewksp,
                  XVW_COMMANDBAR_SHOW_SAVEAS,      pref_def_savewkspas,
                  NULL);

        cmdbar_info->runwksp         = pref_def_runwksp,
        cmdbar_info->stopwksp        = pref_def_stopwksp,
        cmdbar_info->runmode         = pref_def_runmode,
        cmdbar_info->resetwksp       = pref_def_resetwksp,
        cmdbar_info->clearwksp       = pref_def_clearwksp,
        cmdbar_info->checkwksp       = pref_def_checkwksp,
        cmdbar_info->redrawwksp      = pref_def_redrawwksp,
        cmdbar_info->wkspinfo        = pref_def_wkspinfo,
        cmdbar_info->cut             = pref_def_cut,
        cmdbar_info->copy            = pref_def_copy,
        cmdbar_info->paste           = pref_def_paste,
        cmdbar_info->showclipboard   = pref_def_showclipboard,
        cmdbar_info->duplicate       = pref_def_duplicate,
        cmdbar_info->selectall       = pref_def_selectall,
        cmdbar_info->unselectall     = pref_def_unselectall,
        cmdbar_info->delete          = pref_def_delete,
        cmdbar_info->undodelete      = pref_def_undodelete,
        cmdbar_info->createprocedure = pref_def_createprocedure,
        cmdbar_info->createcount     = pref_def_createcount,
        cmdbar_info->createwhile     = pref_def_createwhile,
        cmdbar_info->openwksp        = pref_def_openwksp,
        cmdbar_info->savewksp        = pref_def_savewksp,
        cmdbar_info->savewkspas      = pref_def_savewkspas,

	xvf_set_attribute(cmdbar_info->runwksp_struct, 
			  XVF_OPTSEL, pref_def_runwksp);
        xvf_set_attribute(cmdbar_info->stopwksp_struct, 
			  XVF_OPTSEL, pref_def_stopwksp);
        xvf_set_attribute(cmdbar_info->runmode_struct, 
			  XVF_OPTSEL, pref_def_runmode);
        xvf_set_attribute(cmdbar_info->resetwksp_struct, 
			  XVF_OPTSEL, pref_def_resetwksp);
        xvf_set_attribute(cmdbar_info->clearwksp_struct, 
			  XVF_OPTSEL, pref_def_clearwksp);
        xvf_set_attribute(cmdbar_info->checkwksp_struct, 
			  XVF_OPTSEL, pref_def_checkwksp);
        xvf_set_attribute(cmdbar_info->redrawwksp_struct, 
			  XVF_OPTSEL, pref_def_redrawwksp);
        xvf_set_attribute(cmdbar_info->wkspinfo_struct, 
			  XVF_OPTSEL, pref_def_wkspinfo);
        xvf_set_attribute(cmdbar_info->cut_struct, 
			  XVF_OPTSEL, pref_def_cut);
        xvf_set_attribute(cmdbar_info->copy_struct, 
			  XVF_OPTSEL, pref_def_copy);
        xvf_set_attribute(cmdbar_info->paste_struct, 
			  XVF_OPTSEL, pref_def_paste);
        xvf_set_attribute(cmdbar_info->showclipboard_struct, 
			  XVF_OPTSEL, pref_def_showclipboard);
        xvf_set_attribute(cmdbar_info->duplicate_struct, 
			  XVF_OPTSEL, pref_def_duplicate);
        xvf_set_attribute(cmdbar_info->selectall_struct, 
			  XVF_OPTSEL, pref_def_selectall);
        xvf_set_attribute(cmdbar_info->unselectall_struct, 
			  XVF_OPTSEL, pref_def_unselectall);
        xvf_set_attribute(cmdbar_info->delete_struct, 
			  XVF_OPTSEL, pref_def_delete);
        xvf_set_attribute(cmdbar_info->undodelete_struct, 
			  XVF_OPTSEL, pref_def_undodelete);
        xvf_set_attribute(cmdbar_info->createprocedure_struct, 
			  XVF_OPTSEL, pref_def_createprocedure);
        xvf_set_attribute(cmdbar_info->createcount_struct, 
			  XVF_OPTSEL, pref_def_createcount);
        xvf_set_attribute(cmdbar_info->createwhile_struct, 
			  XVF_OPTSEL, pref_def_createwhile);
        xvf_set_attribute(cmdbar_info->openwksp_struct, 
			  XVF_OPTSEL, pref_def_openwksp);
        xvf_set_attribute(cmdbar_info->savewksp_struct, 
			  XVF_OPTSEL, pref_def_savewksp);
        xvf_set_attribute(cmdbar_info->savewkspas_struct, 
			  XVF_OPTSEL, pref_def_savewkspas);
}

/*-----------------------------------------------------------
|
|  Routine Name: init_pref_subform
|
|       Purpose: This callback is added to the Preferences subform
|                to initialize values correctly the first time it
|                is opened.
|
|         Input: none
|        Output: none
|    Written By: Danielle Argiro
|          Date: June 19, 1996
| Modifications:
|
------------------------------------------------------------*/
void init_pref_subform(void)
{
	static int initialized = FALSE;

	if (initialized) return;

	pref_restore_defaults(gui_info->pref);
	
	initialized = TRUE;
}
