 /*
  * Khoros: $Id: form_init.c,v 1.24 1996/07/04 02:55:22 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: form_init.c,v 1.24 1996/07/04 02:55:22 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	FormInfo Initialization Routines
   >>>> 
   >>>>  Private: 
   >>>> 	_xvf_init_cantata
   >>>> 	_xvf_free_cantata
   >>>> 
   >>>>   Static: 
   >>>> 	_xvf_init_cantata_warning
   >>>> 	_xvf_init_cantata_error
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "cantata.h"

static void _xvf_init_cantata_warning PROTO((char *));
static void _xvf_init_cantata_error   PROTO((char *, char *));

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_cantata
| 
|       Purpose: Initializes the GUI Information (gui_info_struct) structure
| 
|         Input: forminfo - ptr to _xvf_init_cantata_gui_struct
|                form     - pointer to the form tree
| 
|        Output: none
|    Written By: conductor
|          Date: Jul 03, 1996
| Modifications:  
| 
------------------------------------------------------------*/
int _xvf_init_cantata(

    kform *form, 
    gui_info_struct *forminfo)
{
	ksubform   *subform;
	kcontrol   *pane;
	kselection *selection;
	kcontrol   *master;
	kcontrol   *guidepane;

	master = form->master;
	/*
	 * initializing 'cantata' Form Info structure
	 */

	forminfo->workspace_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.workspace");
	if (forminfo->workspace_struct == NULL)
	    _xvf_init_cantata_warning("workspace 'workspace' on master form");
	else
	{
	    kvf_get_attribute(forminfo->workspace_struct, KVF_WORKSPACE,
	                    &(forminfo->workspace));

	}
	forminfo->workspace_file_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.workspace_file");
	if (forminfo->workspace_file_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'workspace_file' on master form");

	forminfo->new_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.new");
	if (forminfo->new_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'new' on master form");
	forminfo->new = FALSE;
	selection = forminfo->new_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->open_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.open");
	if (forminfo->open_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'open' on master form");
	forminfo->open = FALSE;
	selection = forminfo->open_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->resave_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.resave");
	if (forminfo->resave_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'resave' on master form");
	forminfo->resave = FALSE;
	selection = forminfo->resave_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->save_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.save");
	if (forminfo->save_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'save' on master form");
	forminfo->save = FALSE;
	selection = forminfo->save_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->close_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.close");
	if (forminfo->close_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'close' on master form");
	forminfo->close = FALSE;
	selection = forminfo->close_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->quit_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.quit");
	if (forminfo->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on master form");
	forminfo->quit = FALSE;

	forminfo->edit_menu_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.edit_menu");
	if (forminfo->edit_menu_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'edit_menu' on master form");

	forminfo->cut_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.cut");
	if (forminfo->cut_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'cut' on master form");
	forminfo->cut = FALSE;
	selection = forminfo->cut_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->copy_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.copy");
	if (forminfo->copy_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'copy' on master form");
	forminfo->copy = FALSE;
	selection = forminfo->copy_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->paste_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.paste");
	if (forminfo->paste_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'paste' on master form");
	forminfo->paste = FALSE;
	selection = forminfo->paste_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->clipboard_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.clipboard");
	if (forminfo->clipboard_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'clipboard' on master form");
	forminfo->clipboard = FALSE;
	selection = forminfo->clipboard_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b1_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b1");
	if (forminfo->b1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b1' on master form");
	forminfo->duplicate_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.duplicate");
	if (forminfo->duplicate_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'duplicate' on master form");
	forminfo->duplicate = FALSE;
	selection = forminfo->duplicate_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b2_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b2");
	if (forminfo->b2_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b2' on master form");
	forminfo->delete_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.delete");
	if (forminfo->delete_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'delete' on master form");
	forminfo->delete = FALSE;
	selection = forminfo->delete_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->undo_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.undo");
	if (forminfo->undo_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'undo' on master form");
	forminfo->undo = FALSE;
	selection = forminfo->undo_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b3_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b3");
	if (forminfo->b3_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b3' on master form");
	forminfo->select_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.select");
	if (forminfo->select_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'select' on master form");
	forminfo->select = FALSE;
	selection = forminfo->select_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->unselect_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.unselect");
	if (forminfo->unselect_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'unselect' on master form");
	forminfo->unselect = FALSE;
	selection = forminfo->unselect_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b4_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b4");
	if (forminfo->b4_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b4' on master form");
	forminfo->workspace_menu_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.workspace_menu");
	if (forminfo->workspace_menu_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'workspace_menu' on master form");

	forminfo->run_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.run");
	if (forminfo->run_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'run' on master form");
	forminfo->run = FALSE;
	selection = forminfo->run_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->stop_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.stop");
	if (forminfo->stop_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'stop' on master form");
	forminfo->stop = FALSE;
	selection = forminfo->stop_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b5_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b5");
	if (forminfo->b5_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b5' on master form");
	forminfo->runonce_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.runonce");
	if (forminfo->runonce_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'runonce' on master form");
	forminfo->runonce = FALSE;
	selection = forminfo->runonce_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b7_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b7");
	if (forminfo->b7_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b7' on master form");
	forminfo->reset_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.reset");
	if (forminfo->reset_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'reset' on master form");
	forminfo->reset = FALSE;
	selection = forminfo->reset_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->clear_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.clear");
	if (forminfo->clear_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'clear' on master form");
	forminfo->clear = FALSE;
	selection = forminfo->clear_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->check_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.check");
	if (forminfo->check_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'check' on master form");
	forminfo->check = FALSE;
	selection = forminfo->check_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->redraw_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.redraw");
	if (forminfo->redraw_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'redraw' on master form");
	forminfo->redraw = FALSE;
	selection = forminfo->redraw_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->info_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.info");
	if (forminfo->info_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'info' on master form");
	forminfo->info = FALSE;
	selection = forminfo->info_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b8_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b8");
	if (forminfo->b8_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b8' on master form");
	forminfo->options_menu_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.options_menu");
	if (forminfo->options_menu_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'options_menu' on master form");

	forminfo->b9_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b9");
	if (forminfo->b9_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b9' on master form");
	forminfo->commandbar_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.commandbar");
	if (forminfo->commandbar_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'commandbar' on master form");
	forminfo->commandbar = FALSE;
	selection = forminfo->commandbar_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->console_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.console");
	if (forminfo->console_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'console' on master form");
	forminfo->console = FALSE;
	selection = forminfo->console_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->size_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.size");
	if (forminfo->size_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'size' on master form");
	forminfo->size = FALSE;
	selection = forminfo->size_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b20_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b20");
	if (forminfo->b20_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b20' on master form");
	forminfo->clearconsole_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.clearconsole");
	if (forminfo->clearconsole_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'clearconsole' on master form");
	forminfo->clearconsole = FALSE;
	selection = forminfo->clearconsole_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->control_menu_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.control_menu");
	if (forminfo->control_menu_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'control_menu' on master form");

	forminfo->create_procedure_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_procedure");
	if (forminfo->create_procedure_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_procedure' on master form");
	forminfo->create_procedure = FALSE;
	selection = forminfo->create_procedure_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_countloop_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_countloop");
	if (forminfo->create_countloop_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_countloop' on master form");
	forminfo->create_countloop = FALSE;
	selection = forminfo->create_countloop_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_whileloop_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_whileloop");
	if (forminfo->create_whileloop_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_whileloop' on master form");
	forminfo->create_whileloop = FALSE;
	selection = forminfo->create_whileloop_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b10_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b10");
	if (forminfo->b10_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b10' on master form");
	forminfo->create_expression_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_expression");
	if (forminfo->create_expression_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_expression' on master form");
	forminfo->create_expression = FALSE;
	selection = forminfo->create_expression_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_ifelse_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_ifelse");
	if (forminfo->create_ifelse_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_ifelse' on master form");
	forminfo->create_ifelse = FALSE;
	selection = forminfo->create_ifelse_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_merge_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_merge");
	if (forminfo->create_merge_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_merge' on master form");
	forminfo->create_merge = FALSE;
	selection = forminfo->create_merge_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_switch_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_switch");
	if (forminfo->create_switch_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_switch' on master form");
	forminfo->create_switch = FALSE;
	selection = forminfo->create_switch_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->create_trigger_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.create_trigger");
	if (forminfo->create_trigger_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'create_trigger' on master form");
	forminfo->create_trigger = FALSE;
	selection = forminfo->create_trigger_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b11_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b11");
	if (forminfo->b11_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b11' on master form");
	forminfo->panel_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.panel");
	if (forminfo->panel_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'panel' on master form");
	forminfo->panel = FALSE;
	selection = forminfo->panel_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->export_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.export");
	if (forminfo->export_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'export' on master form");
	forminfo->export = FALSE;
	selection = forminfo->export_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b12_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b12");
	if (forminfo->b12_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b12' on master form");
	forminfo->help_menu_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.help_menu");
	if (forminfo->help_menu_struct == NULL)
	    _xvf_init_cantata_warning("unknown selection 'help_menu' on master form");

	forminfo->help1_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.help1");
	if (forminfo->help1_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help1' on master form");

	forminfo->tooltips_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.tooltips");
	if (forminfo->tooltips_struct == NULL)
	    _xvf_init_cantata_warning("master action button 'tooltips' on master form");
	forminfo->tooltips = FALSE;
	selection = forminfo->tooltips_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);

	forminfo->b16_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.b16");
	if (forminfo->b16_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'b16' on master form");
	forminfo->license_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.license");
	if (forminfo->license_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'license' on master form");

	forminfo->bugs_struct =
	    kvf_create_struct_from_selname(master->sel_list, "cantata.bugs");
	if (forminfo->bugs_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'bugs' on master form");

	forminfo->find_struct =
	    kvf_create_struct_from_subformname(form, "cantata.find");
	if (forminfo->find_struct == NULL)
	{
	    _xvf_init_cantata_error("subform 'find' on form 'cantata'", 
			    "cantata_find");
	    return(FALSE);
	}

	/*
	 * initializing 'cantata->find' Subform Info structure
	 */

	forminfo->find = (cantata_find *)
				kcalloc((size_t) 1, sizeof(cantata_find));
	subform = forminfo->find_struct->Subformptr;
	guidepane = subform->guidepane;
	forminfo->find->openglyph_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.find.openglyph");
	if (forminfo->find->openglyph_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'openglyph' on subform 'find'");
	forminfo->find->openglyph = FALSE;
	selection = forminfo->find->openglyph_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->find->quit_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.find.quit");
	if (forminfo->find->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on subform 'find'");
	forminfo->find->quit = FALSE;

	forminfo->find->help_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.find.help");
	if (forminfo->find->help_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help' on subform 'find'");


	/*
	 * initializing 'cantata->find->routines' Pane Info structure
	 */

	forminfo->find->routines = (find_routines *)
				kcalloc((size_t)1, sizeof(find_routines));
	forminfo->find->routines_struct =
	    kvf_create_struct_from_panename(subform, "cantata.find.routines");
	if (forminfo->find->routines_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'routines' on subform 'find'", "find_routines");
	    return(FALSE);
	}
	pane = forminfo->find->routines_struct->Controlptr;
	if (!(xvf_create_subform(subform, -1, -1, FALSE)))
	   return(FALSE);
	_xvf_get_find(forminfo->find);
	if (!(xvf_create_pane(pane, FALSE)))
	   return(FALSE);
	_xvf_get_routines(forminfo->find->routines);
	forminfo->find->routines->rlists_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.find.routines.rlists");
	if (forminfo->find->routines->rlists_struct == NULL)
	    _xvf_init_cantata_warning("workspace 'rlists' on pane 'routines' of subform 'find'");
	else
	{
	    kvf_get_attribute(forminfo->find->routines->rlists_struct, KVF_WORKSPACE,
	                    &(forminfo->find->routines->rlists));

	}

	/*
	 * initializing 'cantata->find->finder' Pane Info structure
	 */

	forminfo->find->finder = (find_finder *)
				kcalloc((size_t)1, sizeof(find_finder));
	forminfo->find->finder_struct =
	    kvf_create_struct_from_panename(subform, "cantata.find.finder");
	if (forminfo->find->finder_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'finder' on subform 'find'", "find_finder");
	    return(FALSE);
	}
	pane = forminfo->find->finder_struct->Controlptr;
	if (!(xvf_create_pane(pane, FALSE)))
	   return(FALSE);
	_xvf_get_finder(forminfo->find->finder);
	forminfo->find->finder->flist_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.find.finder.flist");
	if (forminfo->find->finder->flist_struct == NULL)
	    _xvf_init_cantata_warning("workspace 'flist' on pane 'finder' of subform 'find'");
	else
	{
	    kvf_get_attribute(forminfo->find->finder->flist_struct, KVF_WORKSPACE,
	                    &(forminfo->find->finder->flist));

	}
	forminfo->find->finder->expr_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.find.finder.expr");
	if (forminfo->find->finder->expr_struct != NULL)
	{
	    selection = forminfo->find->finder->expr_struct->Selptr;
	    kvf_get_attribute(forminfo->find->finder->expr_struct, KVF_STRING_DEF,
	                    &(forminfo->find->finder->expr));
	}
	else
	    _xvf_init_cantata_warning("string selection 'expr' on pane 'finder' of subform 'find'");

	forminfo->find->finder->expr_selected = FALSE;

	forminfo->var_struct =
	    kvf_create_struct_from_subformname(form, "cantata.var");
	if (forminfo->var_struct == NULL)
	{
	    _xvf_init_cantata_error("subform 'var' on form 'cantata'", 
			    "cantata_var");
	    return(FALSE);
	}

	/*
	 * initializing 'cantata->var' Subform Info structure
	 */

	forminfo->var = (cantata_var *)
				kcalloc((size_t) 1, sizeof(cantata_var));
	subform = forminfo->var_struct->Subformptr;

	/*
	 * initializing 'cantata->var->variables' Pane Info structure
	 */

	forminfo->var->variables = (var_variables *)
				kcalloc((size_t)1, sizeof(var_variables));
	forminfo->var->variables_struct =
	    kvf_create_struct_from_panename(subform, "cantata.var.variables");
	if (forminfo->var->variables_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'variables' on subform 'var'", "var_variables");
	    return(FALSE);
	}
	pane = forminfo->var->variables_struct->Controlptr;
	forminfo->var->variables->expression_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.expression");
	if (forminfo->var->variables->expression_struct != NULL)
	{
	    selection = forminfo->var->variables->expression_struct->Selptr;
	    kvf_get_attribute(forminfo->var->variables->expression_struct, KVF_STRING_DEF,
	                    &(forminfo->var->variables->expression));
	}
	else
	    _xvf_init_cantata_warning("string selection 'expression' on pane 'variables' of subform 'var'");

	forminfo->var->variables->expression_selected = FALSE;

	forminfo->var->variables->scope_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.scope");
	if (forminfo->var->variables->scope_struct != NULL)
	{
	    selection = forminfo->var->variables->scope_struct->Selptr;
	    kvf_get_attribute(forminfo->var->variables->scope_struct, KVF_LIST_VAL,
	                    &(forminfo->var->variables->scope));
	    kvf_get_attribute(forminfo->var->variables->scope_struct, KVF_LIST_LABEL,
	                    &(forminfo->var->variables->scope_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'scope' on pane 'variables' of subform 'var'");

	forminfo->var->variables->scope_selected = FALSE;

	forminfo->var->variables->varorig_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.varorig");
	if (forminfo->var->variables->varorig_struct != NULL)
	{
	    selection = forminfo->var->variables->varorig_struct->Selptr;
	    kvf_get_attribute(forminfo->var->variables->varorig_struct, KVF_LIST_VAL,
	                    &(forminfo->var->variables->varorig));
	    kvf_get_attribute(forminfo->var->variables->varorig_struct, KVF_LIST_LABEL,
	                    &(forminfo->var->variables->varorig_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'varorig' on pane 'variables' of subform 'var'");

	forminfo->var->variables->varorig_selected = FALSE;

	forminfo->var->variables->varval_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.varval");
	if (forminfo->var->variables->varval_struct != NULL)
	{
	    selection = forminfo->var->variables->varval_struct->Selptr;
	    kvf_get_attribute(forminfo->var->variables->varval_struct, KVF_LIST_VAL,
	                    &(forminfo->var->variables->varval));
	    kvf_get_attribute(forminfo->var->variables->varval_struct, KVF_LIST_LABEL,
	                    &(forminfo->var->variables->varval_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'varval' on pane 'variables' of subform 'var'");

	forminfo->var->variables->varval_selected = FALSE;

	forminfo->var->variables->delete_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.delete");
	if (forminfo->var->variables->delete_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'delete' on pane 'variables' of subform 'var'");
	forminfo->var->variables->delete = FALSE;
	selection = forminfo->var->variables->delete_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->var->variables->quit_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.quit");
	if (forminfo->var->variables->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on pane 'variables' of subform 'var'");
	forminfo->var->variables->quit = FALSE;

	forminfo->var->variables->help_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.help");
	if (forminfo->var->variables->help_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help' on pane 'variables' of subform 'var'");

	forminfo->var->variables->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.blank1");
	if (forminfo->var->variables->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on pane 'variables' of subform 'var'");
	forminfo->var->variables->blank2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.var.variables.blank2");
	if (forminfo->var->variables->blank2_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank2' on pane 'variables' of subform 'var'");
	forminfo->distcomp_struct =
	    kvf_create_struct_from_subformname(form, "cantata.distcomp");
	if (forminfo->distcomp_struct == NULL)
	{
	    _xvf_init_cantata_error("subform 'distcomp' on form 'cantata'", 
			    "cantata_distcomp");
	    return(FALSE);
	}

	/*
	 * initializing 'cantata->distcomp' Subform Info structure
	 */

	forminfo->distcomp = (cantata_distcomp *)
				kcalloc((size_t) 1, sizeof(cantata_distcomp));
	subform = forminfo->distcomp_struct->Subformptr;

	/*
	 * initializing 'cantata->distcomp->dist' Pane Info structure
	 */

	forminfo->distcomp->dist = (distcomp_dist *)
				kcalloc((size_t)1, sizeof(distcomp_dist));
	forminfo->distcomp->dist_struct =
	    kvf_create_struct_from_panename(subform, "cantata.distcomp.dist");
	if (forminfo->distcomp->dist_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'dist' on subform 'distcomp'", "distcomp_dist");
	    return(FALSE);
	}
	pane = forminfo->distcomp->dist_struct->Controlptr;
	forminfo->distcomp->dist->remote_exec_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.remote_exec");
	if (forminfo->distcomp->dist->remote_exec_struct != NULL)
	{
	    selection = forminfo->distcomp->dist->remote_exec_struct->Selptr;
	    kvf_get_attribute(forminfo->distcomp->dist->remote_exec_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->distcomp->dist->remote_exec_num));
	    forminfo->distcomp->dist->remote_exec_val = 2;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'remote_exec' on pane 'dist' of subform 'distcomp'");

	forminfo->distcomp->dist->remote_exec_selected = FALSE;

	forminfo->distcomp->dist->remote_hosts_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.remote_hosts");
	if (forminfo->distcomp->dist->remote_hosts_struct != NULL)
	{
	    selection = forminfo->distcomp->dist->remote_hosts_struct->Selptr;
	    kvf_get_attribute(forminfo->distcomp->dist->remote_hosts_struct, KVF_LIST_VAL,
	                    &(forminfo->distcomp->dist->remote_hosts));
	    kvf_get_attribute(forminfo->distcomp->dist->remote_hosts_struct, KVF_LIST_LABEL,
	                    &(forminfo->distcomp->dist->remote_hosts_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'remote_hosts' on pane 'dist' of subform 'distcomp'");

	forminfo->distcomp->dist->start_daemon_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.start_daemon");
	if (forminfo->distcomp->dist->start_daemon_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'start_daemon' on pane 'dist' of subform 'distcomp'");
	forminfo->distcomp->dist->start_daemon = FALSE;
	selection = forminfo->distcomp->dist->start_daemon_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->distcomp->dist->stop_daemon_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.stop_daemon");
	if (forminfo->distcomp->dist->stop_daemon_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'stop_daemon' on pane 'dist' of subform 'distcomp'");
	forminfo->distcomp->dist->stop_daemon = FALSE;
	selection = forminfo->distcomp->dist->stop_daemon_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->distcomp->dist->save_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.save");
	if (forminfo->distcomp->dist->save_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'save' on pane 'dist' of subform 'distcomp'");
	forminfo->distcomp->dist->save = FALSE;
	selection = forminfo->distcomp->dist->save_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->distcomp->dist->delete_remote_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.delete_remote");
	if (forminfo->distcomp->dist->delete_remote_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'delete_remote' on pane 'dist' of subform 'distcomp'");
	forminfo->distcomp->dist->delete_remote = FALSE;
	selection = forminfo->distcomp->dist->delete_remote_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->distcomp->dist->add_name_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.add_name");
	if (forminfo->distcomp->dist->add_name_struct != NULL)
	{
	    selection = forminfo->distcomp->dist->add_name_struct->Selptr;
	    kvf_get_attribute(forminfo->distcomp->dist->add_name_struct, KVF_STRING_DEF,
	                    &(forminfo->distcomp->dist->add_name));
	}
	else
	    _xvf_init_cantata_warning("string selection 'add_name' on pane 'dist' of subform 'distcomp'");

	forminfo->distcomp->dist->add_name_selected = FALSE;

	forminfo->distcomp->dist->quit_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.quit");
	if (forminfo->distcomp->dist->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on pane 'dist' of subform 'distcomp'");
	forminfo->distcomp->dist->quit = FALSE;

	forminfo->distcomp->dist->help_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.distcomp.dist.help");
	if (forminfo->distcomp->dist->help_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help' on pane 'dist' of subform 'distcomp'");

	forminfo->pref_struct =
	    kvf_create_struct_from_subformname(form, "cantata.pref");
	if (forminfo->pref_struct == NULL)
	{
	    _xvf_init_cantata_error("subform 'pref' on form 'cantata'", 
			    "cantata_pref");
	    return(FALSE);
	}

	/*
	 * initializing 'cantata->pref' Subform Info structure
	 */

	forminfo->pref = (cantata_pref *)
				kcalloc((size_t) 1, sizeof(cantata_pref));
	subform = forminfo->pref_struct->Subformptr;
	guidepane = subform->guidepane;
	forminfo->pref->apply_changes_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.pref.apply_changes");
	if (forminfo->pref->apply_changes_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'apply_changes' on subform 'pref'");
	forminfo->pref->apply_changes = FALSE;
	selection = forminfo->pref->apply_changes_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->pref->restore_defaults_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.pref.restore_defaults");
	if (forminfo->pref->restore_defaults_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'restore_defaults' on subform 'pref'");
	forminfo->pref->restore_defaults = FALSE;
	selection = forminfo->pref->restore_defaults_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->pref->help_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.pref.help");
	if (forminfo->pref->help_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help' on subform 'pref'");

	forminfo->pref->blank1_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.pref.blank1");
	if (forminfo->pref->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on subform 'pref'");
	forminfo->pref->quit_struct =
	    kvf_create_struct_from_selname(guidepane->sel_list, "cantata.pref.quit");
	if (forminfo->pref->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on subform 'pref'");
	forminfo->pref->quit = FALSE;


	/*
	 * initializing 'cantata->pref->grid' Pane Info structure
	 */

	forminfo->pref->grid = (pref_grid *)
				kcalloc((size_t)1, sizeof(pref_grid));
	forminfo->pref->grid_struct =
	    kvf_create_struct_from_panename(subform, "cantata.pref.grid");
	if (forminfo->pref->grid_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'grid' on subform 'pref'", "pref_grid");
	    return(FALSE);
	}
	pane = forminfo->pref->grid_struct->Controlptr;
	forminfo->pref->grid->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.grid.blank1");
	if (forminfo->pref->grid->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on pane 'grid' of subform 'pref'");
	forminfo->pref->grid->griddisplay_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.grid.griddisplay");
	if (forminfo->pref->grid->griddisplay_struct != NULL)
	{
	    selection = forminfo->pref->grid->griddisplay_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->grid->griddisplay_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->grid->griddisplay_num));
	    forminfo->pref->grid->griddisplay_val = 0;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'griddisplay' on pane 'grid' of subform 'pref'");

	forminfo->pref->grid->griddisplay_selected = FALSE;

	forminfo->pref->grid->gridsize_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.grid.gridsize");
	if (forminfo->pref->grid->gridsize_struct != NULL)
	{
	    selection = forminfo->pref->grid->gridsize_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->grid->gridsize_struct, KVF_INT_DEF,
	                    &(forminfo->pref->grid->gridsize));
	}
	else
	    _xvf_init_cantata_warning("integer selection 'gridsize' on pane 'grid' of subform 'pref'");


	forminfo->pref->grid->gridcolor_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.grid.gridcolor");
	if (forminfo->pref->grid->gridcolor_struct != NULL)
	{
	    selection = forminfo->pref->grid->gridcolor_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->grid->gridcolor_struct, KVF_STRING_DEF,
	                    &(forminfo->pref->grid->gridcolor));
	}
	else
	    _xvf_init_cantata_warning("string list selection 'gridcolor' on pane 'grid' of subform 'pref'");



	/*
	 * initializing 'cantata->pref->glyph' Pane Info structure
	 */

	forminfo->pref->glyph = (pref_glyph *)
				kcalloc((size_t)1, sizeof(pref_glyph));
	forminfo->pref->glyph_struct =
	    kvf_create_struct_from_panename(subform, "cantata.pref.glyph");
	if (forminfo->pref->glyph_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'glyph' on subform 'pref'", "pref_glyph");
	    return(FALSE);
	}
	pane = forminfo->pref->glyph_struct->Controlptr;
	forminfo->pref->glyph->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.blank1");
	if (forminfo->pref->glyph->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on pane 'glyph' of subform 'pref'");
	forminfo->pref->glyph->connecttype_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.connecttype");
	if (forminfo->pref->glyph->connecttype_struct != NULL)
	{
	    selection = forminfo->pref->glyph->connecttype_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->glyph->connecttype_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->glyph->connecttype_num));
	    forminfo->pref->glyph->connecttype_val = 2;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'connecttype' on pane 'glyph' of subform 'pref'");


	forminfo->pref->glyph->connectredraw_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.connectredraw");
	if (forminfo->pref->glyph->connectredraw_struct != NULL)
	{
	    selection = forminfo->pref->glyph->connectredraw_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->glyph->connectredraw_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->glyph->connectredraw_num));
	    forminfo->pref->glyph->connectredraw_val = 1;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'connectredraw' on pane 'glyph' of subform 'pref'");


	forminfo->pref->glyph->glyphplacement_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.glyphplacement");
	if (forminfo->pref->glyph->glyphplacement_struct != NULL)
	{
	    selection = forminfo->pref->glyph->glyphplacement_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->glyph->glyphplacement_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->glyph->glyphplacement_num));
	    forminfo->pref->glyph->glyphplacement_val = 1;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'glyphplacement' on pane 'glyph' of subform 'pref'");


	forminfo->pref->glyph->glyphdisplay_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.glyphdisplay");
	if (forminfo->pref->glyph->glyphdisplay_struct != NULL)
	{
	    selection = forminfo->pref->glyph->glyphdisplay_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->glyph->glyphdisplay_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->glyph->glyphdisplay_num));
	    forminfo->pref->glyph->glyphdisplay_val = 1;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'glyphdisplay' on pane 'glyph' of subform 'pref'");


	forminfo->pref->glyph->glyphreport_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.glyph.glyphreport");
	if (forminfo->pref->glyph->glyphreport_struct != NULL)
	{
	    selection = forminfo->pref->glyph->glyphreport_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->glyph->glyphreport_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->glyph->glyphreport_num));
	    forminfo->pref->glyph->glyphreport_val = 1;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'glyphreport' on pane 'glyph' of subform 'pref'");



	/*
	 * initializing 'cantata->pref->workspace' Pane Info structure
	 */

	forminfo->pref->workspace = (pref_workspace *)
				kcalloc((size_t)1, sizeof(pref_workspace));
	forminfo->pref->workspace_struct =
	    kvf_create_struct_from_panename(subform, "cantata.pref.workspace");
	if (forminfo->pref->workspace_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'workspace' on subform 'pref'", "pref_workspace");
	    return(FALSE);
	}
	pane = forminfo->pref->workspace_struct->Controlptr;
	forminfo->pref->workspace->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.blank1");
	if (forminfo->pref->workspace->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on pane 'workspace' of subform 'pref'");
	forminfo->pref->workspace->echoexec_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.echoexec");
	if (forminfo->pref->workspace->echoexec_struct != NULL)
	{
	    selection = forminfo->pref->workspace->echoexec_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->workspace->echoexec_struct, KVF_TOGGLE_NUM,
	                    &(forminfo->pref->workspace->echoexec_num));
	    forminfo->pref->workspace->echoexec_val = 1;
	}
	else
	    _xvf_init_cantata_warning("toggle selection 'echoexec' on pane 'workspace' of subform 'pref'");


	forminfo->pref->workspace->blank2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.blank2");
	if (forminfo->pref->workspace->blank2_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank2' on pane 'workspace' of subform 'pref'");
	forminfo->pref->workspace->backgroundpixmap_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.backgroundpixmap");
	if (forminfo->pref->workspace->backgroundpixmap_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'backgroundpixmap' on pane 'workspace' of subform 'pref'");
	forminfo->pref->workspace->backgroundcolor_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.backgroundcolor");
	if (forminfo->pref->workspace->backgroundcolor_struct != NULL)
	{
	    selection = forminfo->pref->workspace->backgroundcolor_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->workspace->backgroundcolor_struct, KVF_STRING_DEF,
	                    &(forminfo->pref->workspace->backgroundcolor));
	}
	else
	    _xvf_init_cantata_warning("string list selection 'backgroundcolor' on pane 'workspace' of subform 'pref'");

	forminfo->pref->workspace->backgroundcolor_optsel = TRUE;

	forminfo->pref->workspace->backpixmaps_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.workspace.backpixmaps");
	if (forminfo->pref->workspace->backpixmaps_struct != NULL)
	{
	    selection = forminfo->pref->workspace->backpixmaps_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->workspace->backpixmaps_struct, KVF_STRING_DEF,
	                    &(forminfo->pref->workspace->backpixmaps));
	}
	else
	    _xvf_init_cantata_warning("string list selection 'backpixmaps' on pane 'workspace' of subform 'pref'");

	forminfo->pref->workspace->backpixmaps_optsel = FALSE;


	/*
	 * initializing 'cantata->pref->commandbar' Pane Info structure
	 */

	forminfo->pref->commandbar = (pref_commandbar *)
				kcalloc((size_t)1, sizeof(pref_commandbar));
	forminfo->pref->commandbar_struct =
	    kvf_create_struct_from_panename(subform, "cantata.pref.commandbar");
	if (forminfo->pref->commandbar_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'commandbar' on subform 'pref'", "pref_commandbar");
	    return(FALSE);
	}
	pane = forminfo->pref->commandbar_struct->Controlptr;
	forminfo->pref->commandbar->runwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.runwksp");
	if (forminfo->pref->commandbar->runwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->runwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->runwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->runwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'runwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->cut_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.cut");
	if (forminfo->pref->commandbar->cut_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->cut_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->cut_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->cut));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'cut' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->undodelete_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.undodelete");
	if (forminfo->pref->commandbar->undodelete_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->undodelete_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->undodelete_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->undodelete));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'undodelete' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->stopwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.stopwksp");
	if (forminfo->pref->commandbar->stopwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->stopwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->stopwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->stopwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'stopwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->copy_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.copy");
	if (forminfo->pref->commandbar->copy_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->copy_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->copy_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->copy));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'copy' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->createprocedure_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.createprocedure");
	if (forminfo->pref->commandbar->createprocedure_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->createprocedure_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->createprocedure_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->createprocedure));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'createprocedure' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->runmode_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.runmode");
	if (forminfo->pref->commandbar->runmode_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->runmode_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->runmode_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->runmode));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'runmode' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->paste_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.paste");
	if (forminfo->pref->commandbar->paste_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->paste_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->paste_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->paste));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'paste' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->createcount_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.createcount");
	if (forminfo->pref->commandbar->createcount_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->createcount_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->createcount_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->createcount));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'createcount' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->resetwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.resetwksp");
	if (forminfo->pref->commandbar->resetwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->resetwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->resetwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->resetwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'resetwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->showclipboard_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.showclipboard");
	if (forminfo->pref->commandbar->showclipboard_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->showclipboard_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->showclipboard_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->showclipboard));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'showclipboard' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->createwhile_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.createwhile");
	if (forminfo->pref->commandbar->createwhile_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->createwhile_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->createwhile_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->createwhile));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'createwhile' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->clearwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.clearwksp");
	if (forminfo->pref->commandbar->clearwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->clearwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->clearwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->clearwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'clearwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->duplicate_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.duplicate");
	if (forminfo->pref->commandbar->duplicate_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->duplicate_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->duplicate_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->duplicate));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'duplicate' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->openwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.openwksp");
	if (forminfo->pref->commandbar->openwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->openwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->openwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->openwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'openwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->checkwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.checkwksp");
	if (forminfo->pref->commandbar->checkwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->checkwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->checkwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->checkwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'checkwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->selectall_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.selectall");
	if (forminfo->pref->commandbar->selectall_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->selectall_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->selectall_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->selectall));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'selectall' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->savewksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.savewksp");
	if (forminfo->pref->commandbar->savewksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->savewksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->savewksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->savewksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'savewksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->redrawwksp_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.redrawwksp");
	if (forminfo->pref->commandbar->redrawwksp_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->redrawwksp_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->redrawwksp_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->redrawwksp));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'redrawwksp' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->unselectall_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.unselectall");
	if (forminfo->pref->commandbar->unselectall_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->unselectall_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->unselectall_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->unselectall));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'unselectall' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->savewkspas_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.savewkspas");
	if (forminfo->pref->commandbar->savewkspas_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->savewkspas_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->savewkspas_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->savewkspas));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'savewkspas' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->wkspinfo_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.wkspinfo");
	if (forminfo->pref->commandbar->wkspinfo_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->wkspinfo_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->wkspinfo_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->wkspinfo));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'wkspinfo' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->delete_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.delete");
	if (forminfo->pref->commandbar->delete_struct != NULL)
	{
	    selection = forminfo->pref->commandbar->delete_struct->Selptr;
	    kvf_get_attribute(forminfo->pref->commandbar->delete_struct, KVF_OPTSEL,
	                    &(forminfo->pref->commandbar->delete));
	}
	else
	    _xvf_init_cantata_warning("flag selection 'delete' on pane 'commandbar' of subform 'pref'");

	forminfo->pref->commandbar->blank1_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank1");
	if (forminfo->pref->commandbar->blank1_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank1' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank3_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank3");
	if (forminfo->pref->commandbar->blank3_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank3' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank4_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank4");
	if (forminfo->pref->commandbar->blank4_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank4' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank5_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank5");
	if (forminfo->pref->commandbar->blank5_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank5' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank6_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank6");
	if (forminfo->pref->commandbar->blank6_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank6' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank8_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank8");
	if (forminfo->pref->commandbar->blank8_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank8' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank18_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank18");
	if (forminfo->pref->commandbar->blank18_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank18' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank19_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank19");
	if (forminfo->pref->commandbar->blank19_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank19' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank20_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank20");
	if (forminfo->pref->commandbar->blank20_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank20' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank12_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank12");
	if (forminfo->pref->commandbar->blank12_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank12' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank9_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank9");
	if (forminfo->pref->commandbar->blank9_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank9' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank11_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank11");
	if (forminfo->pref->commandbar->blank11_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank11' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank13_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank13");
	if (forminfo->pref->commandbar->blank13_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank13' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank16_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank16");
	if (forminfo->pref->commandbar->blank16_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank16' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank17_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank17");
	if (forminfo->pref->commandbar->blank17_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank17' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank14_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank14");
	if (forminfo->pref->commandbar->blank14_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank14' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank15_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank15");
	if (forminfo->pref->commandbar->blank15_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank15' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank21_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank21");
	if (forminfo->pref->commandbar->blank21_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank21' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank22_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank22");
	if (forminfo->pref->commandbar->blank22_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank22' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank2_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank2");
	if (forminfo->pref->commandbar->blank2_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank2' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank10_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank10");
	if (forminfo->pref->commandbar->blank10_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank10' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank7_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank7");
	if (forminfo->pref->commandbar->blank7_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank7' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank23_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank23");
	if (forminfo->pref->commandbar->blank23_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank23' on pane 'commandbar' of subform 'pref'");
	forminfo->pref->commandbar->blank24_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.pref.commandbar.blank24");
	if (forminfo->pref->commandbar->blank24_struct == NULL)
	    _xvf_init_cantata_warning("blank selection 'blank24' on pane 'commandbar' of subform 'pref'");
	forminfo->create_struct =
	    kvf_create_struct_from_subformname(form, "cantata.create");
	if (forminfo->create_struct == NULL)
	{
	    _xvf_init_cantata_error("subform 'create' on form 'cantata'", 
			    "cantata_create");
	    return(FALSE);
	}

	/*
	 * initializing 'cantata->create' Subform Info structure
	 */

	forminfo->create = (cantata_create *)
				kcalloc((size_t) 1, sizeof(cantata_create));
	subform = forminfo->create_struct->Subformptr;

	/*
	 * initializing 'cantata->create->program' Pane Info structure
	 */

	forminfo->create->program = (create_program *)
				kcalloc((size_t)1, sizeof(create_program));
	forminfo->create->program_struct =
	    kvf_create_struct_from_panename(subform, "cantata.create.program");
	if (forminfo->create->program_struct == NULL)
	{
	    _xvf_init_cantata_error("pane 'program' on subform 'create'", "create_program");
	    return(FALSE);
	}
	pane = forminfo->create->program_struct->Controlptr;
	forminfo->create->program->create_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.create");
	if (forminfo->create->program->create_struct == NULL)
	    _xvf_init_cantata_warning("pane action button 'create' on pane 'program' of subform 'create'");
	forminfo->create->program->create = FALSE;
	selection = forminfo->create->program->create_struct->Selptr;
	kvf_set_attribute(selection->back_kformstruct,
	                  KVF_SELECTED, FALSE);
	forminfo->create->program->help_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.help");
	if (forminfo->create->program->help_struct == NULL)
		    _xvf_init_cantata_warning("unknown selection 'help' on pane 'program' of subform 'create'");

	forminfo->create->program->quit_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.quit");
	if (forminfo->create->program->quit_struct == NULL)
		    _xvf_init_cantata_warning("quit button 'quit' on pane 'program' of subform 'create'");
	forminfo->create->program->quit = FALSE;

	forminfo->create->program->toolboxes_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.toolboxes");
	if (forminfo->create->program->toolboxes_struct != NULL)
	{
	    selection = forminfo->create->program->toolboxes_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->toolboxes_struct, KVF_LIST_VAL,
	                    &(forminfo->create->program->toolboxes));
	    kvf_get_attribute(forminfo->create->program->toolboxes_struct, KVF_LIST_LABEL,
	                    &(forminfo->create->program->toolboxes_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'toolboxes' on pane 'program' of subform 'create'");

	forminfo->create->program->toolboxes_selected = FALSE;

	forminfo->create->program->objects_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.objects");
	if (forminfo->create->program->objects_struct != NULL)
	{
	    selection = forminfo->create->program->objects_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->objects_struct, KVF_LIST_VAL,
	                    &(forminfo->create->program->objects));
	    kvf_get_attribute(forminfo->create->program->objects_struct, KVF_LIST_LABEL,
	                    &(forminfo->create->program->objects_label));
	}
	else
	    _xvf_init_cantata_warning("displaylist selection 'objects' on pane 'program' of subform 'create'");

	forminfo->create->program->objects_selected = FALSE;

	forminfo->create->program->tbname_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.tbname");
	if (forminfo->create->program->tbname_struct != NULL)
	{
	    selection = forminfo->create->program->tbname_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->tbname_struct, KVF_STRING_DEF,
	                    &(forminfo->create->program->tbname));
	}
	else
	    _xvf_init_cantata_warning("string selection 'tbname' on pane 'program' of subform 'create'");


	forminfo->create->program->oname_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.oname");
	if (forminfo->create->program->oname_struct != NULL)
	{
	    selection = forminfo->create->program->oname_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->oname_struct, KVF_STRING_DEF,
	                    &(forminfo->create->program->oname));
	}
	else
	    _xvf_init_cantata_warning("string selection 'oname' on pane 'program' of subform 'create'");


	forminfo->create->program->category_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.category");
	if (forminfo->create->program->category_struct != NULL)
	{
	    selection = forminfo->create->program->category_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->category_struct, KVF_STRING_DEF,
	                    &(forminfo->create->program->category));
	}
	else
	    _xvf_init_cantata_warning("string list selection 'category' on pane 'program' of subform 'create'");

	forminfo->create->program->category_selected = FALSE;

	forminfo->create->program->subcategory_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.subcategory");
	if (forminfo->create->program->subcategory_struct != NULL)
	{
	    selection = forminfo->create->program->subcategory_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->subcategory_struct, KVF_STRING_DEF,
	                    &(forminfo->create->program->subcategory));
	}
	else
	    _xvf_init_cantata_warning("string list selection 'subcategory' on pane 'program' of subform 'create'");


	forminfo->create->program->icon_name_struct =
	    kvf_create_struct_from_selname(pane->sel_list, "cantata.create.program.icon_name");
	if (forminfo->create->program->icon_name_struct != NULL)
	{
	    selection = forminfo->create->program->icon_name_struct->Selptr;
	    kvf_get_attribute(forminfo->create->program->icon_name_struct, KVF_STRING_DEF,
	                    &(forminfo->create->program->icon_name));
	}
	else
	    _xvf_init_cantata_warning("string selection 'icon_name' on pane 'program' of subform 'create'");


	forminfo->cantata_struct =
	    kvf_create_struct_from_mastername(form, "cantata");
	if (forminfo->cantata_struct == NULL)
	{
	    _xvf_init_cantata_error("form 'cantata'",
			    "gui_info");
	    return(FALSE);
	}

	return(TRUE);

}

/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_cantata_warning
| 
|       Purpose: Prints warning when code is out of sync with UIS file
| 
|         Input: string - item that cannot be found
| 
|        Output: none
|    Written By: conductor
|          Date: Jul 03, 1996
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_cantata_warning(
    char *string)
{
	kfprintf(kstderr, "Warning: cannot find %s.\n", string);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_init_cantata_error
| 
|       Purpose: Prints error when code is irreconcilable with UIS file
| 
|         Input: string1 - item that cannot be found
|                string2 - name of FormInfo structure
| 
|        Output: none
|    Written By: conductor
|          Date: Jul 03, 1996
| Modifications:  
| 
------------------------------------------------------------*/
static void _xvf_init_cantata_error(
    char *string1,
    char *string2)
{
	kfprintf(kstderr, "ERROR: cannot find %s.\n", string1);
	kfprintf(kstderr, "         UIS file out of sync with code\n");
	kfprintf(kstderr, "         generated by conductor in 'form_init.c'.\n");
	kfprintf(kstderr, "         Unable to initialize '%s' structure as defined in form_info.h; Aborting.\n", string2);
}


/*-----------------------------------------------------------
| 
|  Routine Name: _xvf_free_cantata
| 
|       Purpose: Frees the gui_info_struct
| 
|         Input: forminfo - ptr to _xvf_free_cantata_gui_struct
| 
|        Output: none
|    Written By: conductor
|          Date: Jul 03, 1996
| Modifications:  
| 
------------------------------------------------------------*/
void _xvf_free_cantata(
    gui_info_struct *forminfo)
{
	if (forminfo->workspace_struct != NULL)
	    kfree_and_NULL(forminfo->workspace_struct);
	if (forminfo->workspace_file_struct != NULL)
	    kfree_and_NULL(forminfo->workspace_file_struct);
	if (forminfo->new_struct != NULL)
	    kfree_and_NULL(forminfo->new_struct);
	if (forminfo->open_struct != NULL)
	    kfree_and_NULL(forminfo->open_struct);
	if (forminfo->resave_struct != NULL)
	    kfree_and_NULL(forminfo->resave_struct);
	if (forminfo->save_struct != NULL)
	    kfree_and_NULL(forminfo->save_struct);
	if (forminfo->close_struct != NULL)
	    kfree_and_NULL(forminfo->close_struct);
	if (forminfo->quit_struct != NULL)
	    kfree_and_NULL(forminfo->quit_struct);
	if (forminfo->edit_menu_struct != NULL)
	    kfree_and_NULL(forminfo->edit_menu_struct);
	if (forminfo->cut_struct != NULL)
	    kfree_and_NULL(forminfo->cut_struct);
	if (forminfo->copy_struct != NULL)
	    kfree_and_NULL(forminfo->copy_struct);
	if (forminfo->paste_struct != NULL)
	    kfree_and_NULL(forminfo->paste_struct);
	if (forminfo->clipboard_struct != NULL)
	    kfree_and_NULL(forminfo->clipboard_struct);
	if (forminfo->b1_struct != NULL)
	    kfree_and_NULL(forminfo->b1_struct);
	if (forminfo->duplicate_struct != NULL)
	    kfree_and_NULL(forminfo->duplicate_struct);
	if (forminfo->b2_struct != NULL)
	    kfree_and_NULL(forminfo->b2_struct);
	if (forminfo->delete_struct != NULL)
	    kfree_and_NULL(forminfo->delete_struct);
	if (forminfo->undo_struct != NULL)
	    kfree_and_NULL(forminfo->undo_struct);
	if (forminfo->b3_struct != NULL)
	    kfree_and_NULL(forminfo->b3_struct);
	if (forminfo->select_struct != NULL)
	    kfree_and_NULL(forminfo->select_struct);
	if (forminfo->unselect_struct != NULL)
	    kfree_and_NULL(forminfo->unselect_struct);
	if (forminfo->b4_struct != NULL)
	    kfree_and_NULL(forminfo->b4_struct);
	if (forminfo->workspace_menu_struct != NULL)
	    kfree_and_NULL(forminfo->workspace_menu_struct);
	if (forminfo->run_struct != NULL)
	    kfree_and_NULL(forminfo->run_struct);
	if (forminfo->stop_struct != NULL)
	    kfree_and_NULL(forminfo->stop_struct);
	if (forminfo->b5_struct != NULL)
	    kfree_and_NULL(forminfo->b5_struct);
	if (forminfo->runonce_struct != NULL)
	    kfree_and_NULL(forminfo->runonce_struct);
	if (forminfo->b7_struct != NULL)
	    kfree_and_NULL(forminfo->b7_struct);
	if (forminfo->reset_struct != NULL)
	    kfree_and_NULL(forminfo->reset_struct);
	if (forminfo->clear_struct != NULL)
	    kfree_and_NULL(forminfo->clear_struct);
	if (forminfo->check_struct != NULL)
	    kfree_and_NULL(forminfo->check_struct);
	if (forminfo->redraw_struct != NULL)
	    kfree_and_NULL(forminfo->redraw_struct);
	if (forminfo->info_struct != NULL)
	    kfree_and_NULL(forminfo->info_struct);
	if (forminfo->b8_struct != NULL)
	    kfree_and_NULL(forminfo->b8_struct);
	if (forminfo->options_menu_struct != NULL)
	    kfree_and_NULL(forminfo->options_menu_struct);
	if (forminfo->b9_struct != NULL)
	    kfree_and_NULL(forminfo->b9_struct);
	if (forminfo->commandbar_struct != NULL)
	    kfree_and_NULL(forminfo->commandbar_struct);
	if (forminfo->console_struct != NULL)
	    kfree_and_NULL(forminfo->console_struct);
	if (forminfo->size_struct != NULL)
	    kfree_and_NULL(forminfo->size_struct);
	if (forminfo->b20_struct != NULL)
	    kfree_and_NULL(forminfo->b20_struct);
	if (forminfo->clearconsole_struct != NULL)
	    kfree_and_NULL(forminfo->clearconsole_struct);
	if (forminfo->control_menu_struct != NULL)
	    kfree_and_NULL(forminfo->control_menu_struct);
	if (forminfo->create_procedure_struct != NULL)
	    kfree_and_NULL(forminfo->create_procedure_struct);
	if (forminfo->create_countloop_struct != NULL)
	    kfree_and_NULL(forminfo->create_countloop_struct);
	if (forminfo->create_whileloop_struct != NULL)
	    kfree_and_NULL(forminfo->create_whileloop_struct);
	if (forminfo->b10_struct != NULL)
	    kfree_and_NULL(forminfo->b10_struct);
	if (forminfo->create_expression_struct != NULL)
	    kfree_and_NULL(forminfo->create_expression_struct);
	if (forminfo->create_ifelse_struct != NULL)
	    kfree_and_NULL(forminfo->create_ifelse_struct);
	if (forminfo->create_merge_struct != NULL)
	    kfree_and_NULL(forminfo->create_merge_struct);
	if (forminfo->create_switch_struct != NULL)
	    kfree_and_NULL(forminfo->create_switch_struct);
	if (forminfo->create_trigger_struct != NULL)
	    kfree_and_NULL(forminfo->create_trigger_struct);
	if (forminfo->b11_struct != NULL)
	    kfree_and_NULL(forminfo->b11_struct);
	if (forminfo->panel_struct != NULL)
	    kfree_and_NULL(forminfo->panel_struct);
	if (forminfo->export_struct != NULL)
	    kfree_and_NULL(forminfo->export_struct);
	if (forminfo->b12_struct != NULL)
	    kfree_and_NULL(forminfo->b12_struct);
	if (forminfo->help_menu_struct != NULL)
	    kfree_and_NULL(forminfo->help_menu_struct);
	if (forminfo->help1_struct != NULL)
	    kfree_and_NULL(forminfo->help1_struct);
	if (forminfo->tooltips_struct != NULL)
	    kfree_and_NULL(forminfo->tooltips_struct);
	if (forminfo->b16_struct != NULL)
	    kfree_and_NULL(forminfo->b16_struct);
	if (forminfo->license_struct != NULL)
	    kfree_and_NULL(forminfo->license_struct);
	if (forminfo->bugs_struct != NULL)
	    kfree_and_NULL(forminfo->bugs_struct);
	if (forminfo->find->openglyph_struct != NULL)
	    kfree_and_NULL(forminfo->find->openglyph_struct);
	if (forminfo->find->quit_struct != NULL)
	    kfree_and_NULL(forminfo->find->quit_struct);
	if (forminfo->find->help_struct != NULL)
	    kfree_and_NULL(forminfo->find->help_struct);
	if (forminfo->find->routines->rlists_struct != NULL)
	    kfree_and_NULL(forminfo->find->routines->rlists_struct);
	kfree_and_NULL(forminfo->find->routines);
	if (forminfo->find->routines_struct != NULL)
	    kfree_and_NULL(forminfo->find->routines_struct);

	if (forminfo->find->finder->flist_struct != NULL)
	    kfree_and_NULL(forminfo->find->finder->flist_struct);
	kfree_and_NULL(forminfo->find->finder->expr);
	if (forminfo->find->finder->expr_struct != NULL)
	    kfree_and_NULL(forminfo->find->finder->expr_struct);
	kfree_and_NULL(forminfo->find->finder);
	if (forminfo->find->finder_struct != NULL)
	    kfree_and_NULL(forminfo->find->finder_struct);


	if (forminfo->find_struct != NULL)
	    kfree_and_NULL(forminfo->find_struct);
	kfree_and_NULL(forminfo->find);
	kfree_and_NULL(forminfo->var->variables->expression);
	if (forminfo->var->variables->expression_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->expression_struct);
	if (forminfo->var->variables->scope_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->scope_struct);
	kfree_and_NULL(forminfo->var->variables->scope_label);
	if (forminfo->var->variables->varorig_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->varorig_struct);
	kfree_and_NULL(forminfo->var->variables->varorig_label);
	if (forminfo->var->variables->varval_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->varval_struct);
	kfree_and_NULL(forminfo->var->variables->varval_label);
	if (forminfo->var->variables->delete_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->delete_struct);
	if (forminfo->var->variables->quit_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->quit_struct);
	if (forminfo->var->variables->help_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->help_struct);
	if (forminfo->var->variables->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->blank1_struct);
	if (forminfo->var->variables->blank2_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables->blank2_struct);
	kfree_and_NULL(forminfo->var->variables);
	if (forminfo->var->variables_struct != NULL)
	    kfree_and_NULL(forminfo->var->variables_struct);


	if (forminfo->var_struct != NULL)
	    kfree_and_NULL(forminfo->var_struct);
	kfree_and_NULL(forminfo->var);
	if (forminfo->distcomp->dist->remote_exec_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->remote_exec_struct);
	if (forminfo->distcomp->dist->remote_hosts_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->remote_hosts_struct);
	kfree_and_NULL(forminfo->distcomp->dist->remote_hosts_label);
	if (forminfo->distcomp->dist->start_daemon_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->start_daemon_struct);
	if (forminfo->distcomp->dist->stop_daemon_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->stop_daemon_struct);
	if (forminfo->distcomp->dist->save_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->save_struct);
	if (forminfo->distcomp->dist->delete_remote_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->delete_remote_struct);
	kfree_and_NULL(forminfo->distcomp->dist->add_name);
	if (forminfo->distcomp->dist->add_name_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->add_name_struct);
	if (forminfo->distcomp->dist->quit_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->quit_struct);
	if (forminfo->distcomp->dist->help_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist->help_struct);
	kfree_and_NULL(forminfo->distcomp->dist);
	if (forminfo->distcomp->dist_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp->dist_struct);


	if (forminfo->distcomp_struct != NULL)
	    kfree_and_NULL(forminfo->distcomp_struct);
	kfree_and_NULL(forminfo->distcomp);
	if (forminfo->pref->apply_changes_struct != NULL)
	    kfree_and_NULL(forminfo->pref->apply_changes_struct);
	if (forminfo->pref->restore_defaults_struct != NULL)
	    kfree_and_NULL(forminfo->pref->restore_defaults_struct);
	if (forminfo->pref->help_struct != NULL)
	    kfree_and_NULL(forminfo->pref->help_struct);
	if (forminfo->pref->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->pref->blank1_struct);
	if (forminfo->pref->quit_struct != NULL)
	    kfree_and_NULL(forminfo->pref->quit_struct);
	if (forminfo->pref->grid->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->pref->grid->blank1_struct);
	if (forminfo->pref->grid->griddisplay_struct != NULL)
	    kfree_and_NULL(forminfo->pref->grid->griddisplay_struct);
	if (forminfo->pref->grid->gridsize_struct != NULL)
	     kfree_and_NULL(forminfo->pref->grid->gridsize_struct);
	kfree_and_NULL(forminfo->pref->grid->gridcolor);
	if (forminfo->pref->grid->gridcolor_struct != NULL)
	    kfree_and_NULL(forminfo->pref->grid->gridcolor_struct);
	kfree_and_NULL(forminfo->pref->grid);
	if (forminfo->pref->grid_struct != NULL)
	    kfree_and_NULL(forminfo->pref->grid_struct);

	if (forminfo->pref->glyph->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->blank1_struct);
	if (forminfo->pref->glyph->connecttype_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->connecttype_struct);
	if (forminfo->pref->glyph->connectredraw_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->connectredraw_struct);
	if (forminfo->pref->glyph->glyphplacement_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->glyphplacement_struct);
	if (forminfo->pref->glyph->glyphdisplay_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->glyphdisplay_struct);
	if (forminfo->pref->glyph->glyphreport_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph->glyphreport_struct);
	kfree_and_NULL(forminfo->pref->glyph);
	if (forminfo->pref->glyph_struct != NULL)
	    kfree_and_NULL(forminfo->pref->glyph_struct);

	if (forminfo->pref->workspace->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->blank1_struct);
	if (forminfo->pref->workspace->echoexec_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->echoexec_struct);
	if (forminfo->pref->workspace->blank2_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->blank2_struct);
	if (forminfo->pref->workspace->backgroundpixmap_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->backgroundpixmap_struct);
	kfree_and_NULL(forminfo->pref->workspace->backgroundcolor);
	if (forminfo->pref->workspace->backgroundcolor_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->backgroundcolor_struct);
	kfree_and_NULL(forminfo->pref->workspace->backpixmaps);
	if (forminfo->pref->workspace->backpixmaps_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace->backpixmaps_struct);
	kfree_and_NULL(forminfo->pref->workspace);
	if (forminfo->pref->workspace_struct != NULL)
	    kfree_and_NULL(forminfo->pref->workspace_struct);

	if (forminfo->pref->commandbar->runwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->runwksp_struct);
	if (forminfo->pref->commandbar->cut_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->cut_struct);
	if (forminfo->pref->commandbar->undodelete_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->undodelete_struct);
	if (forminfo->pref->commandbar->stopwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->stopwksp_struct);
	if (forminfo->pref->commandbar->copy_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->copy_struct);
	if (forminfo->pref->commandbar->createprocedure_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->createprocedure_struct);
	if (forminfo->pref->commandbar->runmode_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->runmode_struct);
	if (forminfo->pref->commandbar->paste_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->paste_struct);
	if (forminfo->pref->commandbar->createcount_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->createcount_struct);
	if (forminfo->pref->commandbar->resetwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->resetwksp_struct);
	if (forminfo->pref->commandbar->showclipboard_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->showclipboard_struct);
	if (forminfo->pref->commandbar->createwhile_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->createwhile_struct);
	if (forminfo->pref->commandbar->clearwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->clearwksp_struct);
	if (forminfo->pref->commandbar->duplicate_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->duplicate_struct);
	if (forminfo->pref->commandbar->openwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->openwksp_struct);
	if (forminfo->pref->commandbar->checkwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->checkwksp_struct);
	if (forminfo->pref->commandbar->selectall_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->selectall_struct);
	if (forminfo->pref->commandbar->savewksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->savewksp_struct);
	if (forminfo->pref->commandbar->redrawwksp_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->redrawwksp_struct);
	if (forminfo->pref->commandbar->unselectall_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->unselectall_struct);
	if (forminfo->pref->commandbar->savewkspas_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->savewkspas_struct);
	if (forminfo->pref->commandbar->wkspinfo_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->wkspinfo_struct);
	if (forminfo->pref->commandbar->delete_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->delete_struct);
	if (forminfo->pref->commandbar->blank1_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank1_struct);
	if (forminfo->pref->commandbar->blank3_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank3_struct);
	if (forminfo->pref->commandbar->blank4_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank4_struct);
	if (forminfo->pref->commandbar->blank5_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank5_struct);
	if (forminfo->pref->commandbar->blank6_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank6_struct);
	if (forminfo->pref->commandbar->blank8_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank8_struct);
	if (forminfo->pref->commandbar->blank18_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank18_struct);
	if (forminfo->pref->commandbar->blank19_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank19_struct);
	if (forminfo->pref->commandbar->blank20_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank20_struct);
	if (forminfo->pref->commandbar->blank12_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank12_struct);
	if (forminfo->pref->commandbar->blank9_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank9_struct);
	if (forminfo->pref->commandbar->blank11_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank11_struct);
	if (forminfo->pref->commandbar->blank13_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank13_struct);
	if (forminfo->pref->commandbar->blank16_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank16_struct);
	if (forminfo->pref->commandbar->blank17_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank17_struct);
	if (forminfo->pref->commandbar->blank14_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank14_struct);
	if (forminfo->pref->commandbar->blank15_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank15_struct);
	if (forminfo->pref->commandbar->blank21_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank21_struct);
	if (forminfo->pref->commandbar->blank22_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank22_struct);
	if (forminfo->pref->commandbar->blank2_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank2_struct);
	if (forminfo->pref->commandbar->blank10_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank10_struct);
	if (forminfo->pref->commandbar->blank7_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank7_struct);
	if (forminfo->pref->commandbar->blank23_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank23_struct);
	if (forminfo->pref->commandbar->blank24_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar->blank24_struct);
	kfree_and_NULL(forminfo->pref->commandbar);
	if (forminfo->pref->commandbar_struct != NULL)
	    kfree_and_NULL(forminfo->pref->commandbar_struct);


	if (forminfo->pref_struct != NULL)
	    kfree_and_NULL(forminfo->pref_struct);
	kfree_and_NULL(forminfo->pref);
	if (forminfo->create->program->create_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->create_struct);
	if (forminfo->create->program->help_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->help_struct);
	if (forminfo->create->program->quit_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->quit_struct);
	if (forminfo->create->program->toolboxes_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->toolboxes_struct);
	kfree_and_NULL(forminfo->create->program->toolboxes_label);
	if (forminfo->create->program->objects_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->objects_struct);
	kfree_and_NULL(forminfo->create->program->objects_label);
	kfree_and_NULL(forminfo->create->program->tbname);
	if (forminfo->create->program->tbname_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->tbname_struct);
	kfree_and_NULL(forminfo->create->program->oname);
	if (forminfo->create->program->oname_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->oname_struct);
	kfree_and_NULL(forminfo->create->program->category);
	if (forminfo->create->program->category_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->category_struct);
	kfree_and_NULL(forminfo->create->program->subcategory);
	if (forminfo->create->program->subcategory_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->subcategory_struct);
	kfree_and_NULL(forminfo->create->program->icon_name);
	if (forminfo->create->program->icon_name_struct != NULL)
	    kfree_and_NULL(forminfo->create->program->icon_name_struct);
	kfree_and_NULL(forminfo->create->program);
	if (forminfo->create->program_struct != NULL)
	    kfree_and_NULL(forminfo->create->program_struct);


	if (forminfo->create_struct != NULL)
	    kfree_and_NULL(forminfo->create_struct);
	kfree_and_NULL(forminfo->create);
	kfree_and_NULL(forminfo);
}
