 /*
  * Khoros: $Id: do_copy_pane.c,v 1.3 1996/07/11 19:57:31 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_copy_pane.c,v 1.3 1996/07/11 19:57:31 mirage Exp $";
#endif

/*
 * Copyright (C) 1993, 1994, 1995, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane copy_pane
   >>>> 
   >>>>  Private: 
   >>>> 	copy_pane_copy_to
   >>>> 	copy_pane_copy
   >>>> 
   >>>>   Static: 
   >>>>         copy_file_object
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "composer.h"

static void copy_file_object(kobject  fobj, kstring  newname);

/*-----------------------------------------------------------
| 
|  Routine Name: copy_pane_copy_to
| 
|       Purpose: Do routine which is called when
|                  string selection copy_to is used
| 
|         Input: copy_pane_info - ptr to PaneInfo struct for copy_pane pane
| 
|        Output: None
|    Written By: George Ruban
|          Date: Oct 06, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void copy_pane_copy_to(
     copyfile_copy_pane *copy_pane_info)
{
  copy_file_object(cur_fileobj, copy_pane_info->copy_to);
}


/*-----------------------------------------------------------
| 
|  Routine Name: copy_pane_copy
| 
|       Purpose: Do routine which is called when
|                  pane action button copy is used
| 
|         Input: copy_pane_info - ptr to PaneInfo struct for copy_pane pane
| 
|        Output: None
|    Written By: George Ruban
|          Date: Oct 06, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void copy_pane_copy(
     copyfile_copy_pane *copy_pane_info)
{
  copy_file_object(cur_fileobj, copy_pane_info->copy_to);
}



/*-----------------------------------------------------------
| Routine Name:	copy_file_object - copy file object
|
| Purpose:	This function is used to copy a file within an object.
|
| Input:	fileobj	-	the file object
|		newname	-	new name (full path) of file object
|
| Written By:	George Ruban
| Date:		Oct 06, 1995
------------------------------------------------------------*/
static void
copy_file_object(
   kobject  fobj,
   kstring  newname)
{
   kstring  routine           = "Copy File Object";
   kstring  new = NULL;
   kstring  oldname = NULL;
   kstring  oldpath = NULL;
   kobject  copy_obj = NULL;
   char     *dir;
   char     newpath[KLENGTH];
   char     temp[KLENGTH];
   long     ftype, fsubtype;


   if (newname == NULL || (new = kstring_cleanup(newname, NULL)) == NULL)
   {
      kerror(NULL, routine, "How about you specify a new filename?");
      return;
   }

   if (!kcms_get_attributes(fobj, KCMS_FOBJ_FULLNAME, &oldpath, 
			    KCMS_FOBJ_BASENAME, &oldname,
			    KCMS_END)){
     kerror(NULL, routine, "Could not get old file's attributes.");
     kfree_and_NULL(new); /* you guys keep forgetting to do this - 
		    free your dynam allocs on error returns too!! */
     return;
   }

#ifdef UNDEFINED  /* GR: darned if I know why this is here, 
		     but left it in from rename_file_object() */
   xvf_set_attribute(gui_info->utilities_struct,
		     XVF_DISPLAY_SUBFORM, 0);
#endif


  xvw_format_attribute(xvoNotifier, XVW_NOTIFYWINDOW_MESSAGE,
			"Copying file \"%s\" to \"%s\".", oldname, newname);
   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, TRUE);

   if(!guess_type(new, &ftype, &fsubtype, &dir, NULL, NULL)){
     xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
     return;
   }
   ksprintf(newpath, "%s/%s/%s", cmobj_opath, dir, new);

   /*-- Does a file exist with the same path? -------------------------*/
   if (kaccess(newpath, R_OK) == 0)
   {
     /*-- prompt the user for confirmation --*/
     if (!kprompt(KSTANDARD, "Yes", "No", FALSE,
		  "Operation: copy file\n\n"
		  "\tToolbox:   %s\n"
		  "\tObject:    %s\n"
		  "\tCopy from: %s\n\n"
		  "A file %s already exists.\n"
		  "Do you want to continue? "
		  "(The old file will be renamed with .bak added).",
		  strToolbox, strObject, oldname, newpath)){
       xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
       kfree_and_NULL(new);
       return;
     }
     ksprintf(temp, "%s.bak", newpath);
     if (krename(newpath, temp) != 0){
       kerror(NULL, routine, "Rename failed - aborting.");
       xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
       kfree_and_NULL(new);
       return;
     }
   }
   
   if(kcopyfile(oldpath, newpath) > 0){
     copy_obj = kcms_create_fileobj(cmsObject, newpath,
				  NULL, ftype, fsubtype,
				  KCMS_FOBJ_GEN_NONE,
				  KCMS_FOBJ_ACCESS_RDWR);
     if (copy_obj == NULL){
       kerror(NULL, routine, "Unable to add the file %s", newpath);
     } else {
       reload_object();
     }
   } else {
     kerror(NULL, routine, "Could not kcopyfile(%s, %s).", oldpath, newpath);
   }
   
   xvw_set_attribute(xvoNotifier, XVW_NOTIFYWINDOW_VISIBLE, FALSE);
   kfree_and_NULL(new);
}


