/*
  * Khoros: $Id: create_util.c,v 1.7 1996/04/04 23:38:17 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: create_util.c,v 1.7 1996/04/04 23:38:17 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         GUI Item Creation Utility Routines 
   >>>>
   >>>>  Private:
   >>>>            create_new_selection
   >>>>            add_item_to_submenu
   >>>>            calculate_guidebutton_position
   >>>>            compose_pane_name
   >>>>            compose_subform_name
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "guise.h"


/*-----------------------------------------------------------
|
|  Routine Name: create_new_selection
|
|       Purpose: creates a new selection based on the UIS line passed in.
|         Input: display_control - master, guidepane, or pane to create 
|                                  selection on.
|                line            - UIS line describing selection
|                name            - name for GUI item
|        Output: 
|       Returns: 
|    Written By: Danielle Argiro
|          Date: Nov 12, 1993
| Modifications:
|
------------------------------------------------------------*/
kselection *create_new_selection(
    kcontrol *display_control,
    char     *line)
{
	kselection *selection;

	/*
	 *  create the new selection in the form tree
	 */
	selection = kvf_create_new_selection(display_control, line);

        /*
         * create the new selection on the GUI
         */
	if ((selection->type != KUIS_MUTEXCL) &&
	    (selection->type != KUIS_MUTINCL) &&
	    (selection->type != KUIS_GROUP))
            xvf_create_selection(selection, display_control->back,
                                 display_control->type);

	/*
	 *  update the Save button to say, 'Save (Needed)'
	 */
	set_save_needed(gui_info->options->create->save_struct, NULL, NULL,
                        gui_info->options->create->save_struct);

	/*
	 *  make sure changes to selection will change 
         *  the Save button to say 'Save (Needed)'
	 */
	xvf_add_gui_callback(selection->back_kformstruct, "xvf_all_attributes",
                             set_save_needed, 
			     (kaddr) gui_info->options->create->save_struct);

	/*
	 *  go ahead and put the selection in "selected" mode
	 */
	if (selection->type != KUIS_STARTSUBMENU)
	{
	    xvw_set_attributes(display_control->back, 
				XVW_SELECT_REPLACE, selection->back,
				XVW_EDIT_MODE_ON,   TRUE,
				NULL);
	    
	}

	return(selection);
}

/*-----------------------------------------------------------
|
|  Routine Name: add_item_to_submenu
|
|       Purpose: Adds a GUI item into the submenu list.
|         Input: submenu     - pointer to submenu selection
|                kformstruct - kformstruct of selection, subform button,
|                              or guide button to be linked in
|        Output: none
|       Returns: TRUE on success, FALSE on failure
|    Written By: Danielle Argiro
|          Date: Mon Jan 3, 1994
| Modifications:
|
------------------------------------------------------------*/
int add_item_to_submenu(
   kselection   *submenu,
   kform_struct *kformstruct)
{

	kform_struct *tmpstruct;

        if (submenu->submenu_next == NULL)
        {
	    kformstruct->next = NULL;
            submenu->submenu_next = kformstruct;
        }
        else
        {
            tmpstruct = submenu->submenu_next;
            while (tmpstruct->next != NULL)
                tmpstruct = tmpstruct->next;
	    kformstruct->next = NULL;
            tmpstruct->next = kformstruct;
        }
	/* Note: DON'T link the kformstruct - because you don't want
	   it's "next" pointer to be used to point to the global list
	   of kformstructs;  the "next" pointer has to be used to end
	   the list! - it won't get freed, but oh well. */

	switch (kformstruct->type)
	{
	    case KSELECTION:
                 kformstruct->Selptr->back_submenu = submenu;
                 kformstruct->Selptr->back_kformstruct = kformstruct;
		 break;

	    case KSUBFORM:
                 kformstruct->Subformptr->back_submenu = submenu;
                 kformstruct->Subformptr->back_kformstruct = kformstruct;
		 break;

	    case KGUIDE:
                 kformstruct->Guideptr->back_submenu = submenu;
                 kformstruct->Guideptr->back_kformstruct = kformstruct;
		 break;
	}
        return(TRUE);
}


/*-----------------------------------------------------------
|
|  Routine Name: calculate_gb_position
|
|       Purpose: Finds the location for the next guide button
|         Input: guidepane - guidepane
|        Output: None
|       Returns: The number of the guide/pane pair to be added
|    Written By: Danielle Argiro
|          Date: Aug 31, 1993
| Modifications:
|
------------------------------------------------------------*/
float calculate_gb_position(
      kcontrol *guidepane)
{
	float   height = 0.0, ypos, max_ypos = 0.0;
	kguide *guide, *save_guide = NULL;

	/*
	 *  guidepane NULL implies that this will be the second
	 *  guide button to be created (first will be hardwired to
	 *  y location of 1.5)
	 */
	if (guidepane == NULL)
	    return(3.0);

	/*
	 *  otherwise, go through guide buttons and find lowest one.
	 */
	guide = guidepane->guide_list;
	while (guide != NULL)
        {
	    kvf_get_attribute(guide->back_kformstruct, KVF_BUTTONY, &ypos);
	    if (ypos > max_ypos) 
	    {
	 	max_ypos = ypos;
		save_guide = guide;
	    }
	    guide = guide->next;
        }
	if (save_guide != NULL)
	    kvf_get_attribute(save_guide->back_kformstruct, 
			      KVF_BUTTONHEIGHT, &height);

	/*
	 * have to add in height of last button, plus spacing of 0.5
	 */
	max_ypos = max_ypos + height + 0.5;
	return(max_ypos);
}

/*-----------------------------------------------------------
|
|  Routine Name: compose_sel_name
|
|       Purpose: Finds a unique name for the selection
|         Input: control - master form, guidepane, or pane on which
|			   selection will be appearing.
|                base    - base variable name, from selection,
|			   for example "int" for an integer, "flt" for
|			   a float, etc.
|        Output: None
|       Returns: An allocated variable name for the selection
|    Written By: Danielle Argiro
|          Date: May 15, 1995
| Modifications:
|
------------------------------------------------------------*/

char *compose_sel_name(
    kcontrol *control,
    char     *base)
{
        kselection *selection, *group, *subgroup;
        char        name[KLENGTH];
        ktoken     *seltokens = NULL;
        size_t      seltokennum = 0;
        int         ok, count = 1;

        /*
	 * passing in base as NULL internal error
         */
	if (base == NULL)
	{
	    kerror(NULL, "compose_sel_name", "Error: NULL base passed in");
	    return(kstrdup("tmp"));
	}

	/*
	 * first selection in sel list is easy; return predefined variable name
	 */
        if (control->sel_list == NULL)
	{
	    ksprintf(name, "%s1", base, count);
	    return(kstrdup(name));
	}

	/*
	 * go thru sel list building up list of tokens assoc w/ used variables
	 * this should probably be recursive but i don't feel like it now
	 */
	selection = control->sel_list;
        while (selection != NULL)
        {
            seltokens = karray_add(seltokens, KSTRUCT, selection->var_token, 
		                   seltokennum);
            seltokennum++;
	    if (selection->group_next != NULL)
	    {
		group = selection->group_next;
		while (group != NULL)
                {
                    seltokens = karray_add(seltokens, KSTRUCT, 
			                   group->var_token, seltokennum);
                    seltokennum++;
                    if (group->group_next != NULL)
                    {
                        subgroup = group->group_next;
		        while (subgroup != NULL)
                        {
                            seltokens = karray_add(seltokens, KSTRUCT, 
					   subgroup->var_token, seltokennum);
                            seltokennum++;
		            subgroup = subgroup->next;
		        }
                    }
		    group = group->next;
	       }
            }
	    selection = selection->next;
        }

        /*
         * now, find variable for selection that isn't already used
         */
        ok = FALSE;
        while (!ok)
        {
            ksprintf(name, "%s%d", base, count++);
            if (karray_locate(seltokens, KSTRUCT, kstring_to_token(name),
                              seltokennum) == -1)
                ok = TRUE;
        }
        kfree_and_NULL(seltokens);
        return(kstrdup(name));
}

/*-----------------------------------------------------------
|
|  Routine Name: compose_pane_name
|
|       Purpose: Finds a unique name for the guide button/pane pair
|         Input: None - uses global display_form for reference
|        Output: None
|       Returns: An allocated variable name for the guide/pane pair
|    Written By: Danielle Argiro
|          Date: May 15, 1995
| Modifications:
|
------------------------------------------------------------*/

char *compose_pane_name(void)
{
        ksubform  *subform;
        kguide    *guide;
        char      name[KLENGTH];
        ktoken   *panetokens = NULL;
        size_t    panetokennum = 0;
        int       ok, count;

        /*
         * go thru entire form, collecting pane name tokens into array
         */
        if (display_form->master != NULL)
            subform = display_form->master->subform_list;
        else subform = display_form->subform;
        while (subform != NULL)
        {
            if (subform->guidepane != NULL)
                guide = subform->guidepane->guide_list;
            else guide = subform->guide;
            while (guide != NULL)
            {
                panetokens = karray_add(panetokens, KSTRUCT,
                                        guide->pane->var_token, panetokennum);
                panetokennum++;
                guide = guide->next;
            }
            subform = subform->next;
        }

	/*
	 * now, find name for pane that isn't already used
	 */
        ok = FALSE;
        count = 2;
        while (!ok)
        {
            ksprintf(name, "pane%d", count++);
            if (karray_locate(panetokens, KSTRUCT, kstring_to_token(name),
                                panetokennum) == -1)
                ok = TRUE;
        }
        kfree_and_NULL(panetokens);
        return(kstrdup(name));
}
   
/*-----------------------------------------------------------
|
|  Routine Name: compose_subform_name
|
|       Purpose: Finds a unique name for the subform button/subform pair
|         Input: None - uses global display_form for reference
|        Output: None
|       Returns: An allocated variable name for the subform
|    Written By: Danielle Argiro
|          Date: May 15, 1995
| Modifications:
|
------------------------------------------------------------*/

char *compose_subform_name(void)
{
        ksubform  *subform;
        char      name[KLENGTH];
        ktoken   *subformtokens = NULL;
        int       subformtokennum = 0;
        int       ok, count;

        /*
         * go thru entire form, collecting subform name tokens into array
         */
        if (display_form->master != NULL)
            subform = display_form->master->subform_list;
        else subform = display_form->subform;
        while (subform != NULL)
        {
            subformtokens = karray_add(subformtokens, KSTRUCT,
                                       subform->var_token, subformtokennum);
            subformtokennum++;
            subform = subform->next;
        }

	/*
	 * now, find name for subform that isn't already used
	 */
        ok = FALSE;
        count = 2;
        while (!ok)
        {
            ksprintf(name, "subform%d", count++);
            if (karray_locate(subformtokens, KSTRUCT, kstring_to_token(name),
                                subformtokennum) == -1)
                ok = TRUE;
        }
        kfree_and_NULL(subformtokens);
        return(kstrdup(name));
}
   
