/*
 * Khoros: $Id: utilities.c,v 1.14.2.1 1996/12/06 20:31:06 fas Exp $
 */
 
#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: utilities.c,v 1.14.2.1 1996/12/06 20:31:06 fas Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>            utilities.c
   >>>>
   >>>>  Static:
   >>>>             print_fields_to_file()
   >>>>             extract_gui()
   >>>>             cleanup_temp_file()
   >>>>             prompt_submit_reg()
   >>>>             email_registration()
   >>>>             save_registration()
   >>>>             prompt_for_filename()
   >>>>             xvreg_return_cr()
   >>>>  Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "xvregister.h"


/*-----------------------------------------------------------
|
|  Routine Name: print_fields_to_file - Print registration fields
|
|       Purpose: Print fields specified in the registration process
|                to a temporary file first.  Later on, the user will
|                be queried as to whether the file should be emailed
|		 to Khoral Research or saved to a file in the user's
|		 home directory.
|
|         Input: print_fields_struct  *fields - Fields to print
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications: Tom Konerth, added the version, major, and minor info
|                to the file created.  Jul 09, 1996
|
------------------------------------------------------------*/
char *print_fields_to_file(print_fields_struct *fields)
{
   kfile	*file_fd;
   char		*unique_file;
   char 	**address_contents;
   ssize_t	 	address_lines;
   int	 	i;

   unique_file = ktempnam(NULL, "file=xvregister");
   
   if ((file_fd = kfopen(unique_file, "w")) == NULL)
   {
      kerror("xvregister", "print_fileds_to_file",
	     "Unable to open temporary file '%s', unique_file");
      return NULL;
   }
   
   kfprintf(file_fd, "\n\n\t********************************************************\n");
   kfprintf(file_fd, "\t\t\t%s %d.%d.%d User Registration\n",
	    KHOROS_RELEASE_NAME, KHOROS_RELEASE_VERSION, KHOROS_RELEASE_MAJOR,
            KHOROS_RELEASE_MINOR);
   kfprintf(file_fd, "\t********************************************************\n\n");
   kfprintf(file_fd, "               Name:  ");
   kfprintf(file_fd, "%s\n", fields->name_string);

   kfprintf(file_fd, "            Company:  ");
   if (fields->company_string != NULL && kstrlen(fields->company_string) > 0)
      kfprintf(file_fd, "%s\n", fields->company_string);

   kfprintf(file_fd, "     Street Address:  ");
   address_contents = kparse_string_delimit(fields->address_string, "\n",
					    KDELIM_CLEAN, &address_lines);
   if (address_lines == 1)
      kfprintf(file_fd, "%s\n", fields->address_string);
   else
   {
      for (i = 0; i < address_lines; i++)
      {
         if (i == 0)
            kfprintf(file_fd, "%s\n", address_contents[i]);
         else if (address_contents[i] != NULL && 
                  kstrlen(address_contents[i]) > 0)
            kfprintf(file_fd, "                      %s\n", 
		     address_contents[i]);
      }
   }

   kfprintf(file_fd, "               City:  ");
   kfprintf(file_fd, "%-26s", fields->city_string);

   kfprintf(file_fd, "State/Province:  ");
   kfprintf(file_fd, "%s\n", fields->state_string);

   kfprintf(file_fd, "    Zip/Postal Code:  ");
   kfprintf(file_fd, "%-26s", fields->zip_string);

   kfprintf(file_fd, "       Country:  ");
   kfprintf(file_fd, "%s\n", fields->country_string);

   
   if (fields->phone_string != NULL && kstrlen(fields->phone_string) > 0)
   {
      kfprintf(file_fd, "              Phone:  ");
      kfprintf(file_fd, "%-26s", fields->phone_string);
   }
   else
      kfprintf(file_fd, "              Phone:                            ");
   

   kfprintf(file_fd, "           Fax:  ");
   if (fields->fax_string != NULL && kstrlen(fields->fax_string) > 0)
      kfprintf(file_fd, "%s\n", fields->fax_string);
   else
      kfprintf(file_fd, "\n");

   kfprintf(file_fd, "   Internet Address:  ");
   if (fields->internet_string != NULL && kstrlen(fields->internet_string) > 0)
      kfprintf(file_fd, "%s\n          ", fields->internet_string);

   kfprintf(file_fd, "\n\n");
   kfprintf(file_fd, 
    "   Send me a free subscription to the Khoros Knews quarterly newsletter?");
   if (fields->newsletter_logical == TRUE)
      kfprintf(file_fd, "  YES\n");
   else
      kfprintf(file_fd, "  NO\n\n");
   kfprintf(file_fd, 
      "   Notify me of new releases, patches, and upgrades                     ");
   if (fields->notify_upgrades_logical == TRUE)
      kfprintf(file_fd, "  YES\n");
   else
      kfprintf(file_fd, "  NO\n\n");
   kfprintf(file_fd, 
   "   You may publish my name on other mailing lists                       ");
   if (fields->publish_name_logical == TRUE)
      kfprintf(file_fd, "  YES\n");
   else
      kfprintf(file_fd, "  NO\n\n");

   kfprintf(file_fd, 
      "\n\n   What is your organization's primary business?\n\n");

   if (fields->org_aerospace_logical == TRUE)
      kfprintf(file_fd, "\tX  Aerospace\n");
   else
      kfprintf(file_fd, "\t   Aerospace\n");
   if (fields->org_engineering_logical == TRUE)
      kfprintf(file_fd, "\tX  Architecture/Engineering\n");
   else
      kfprintf(file_fd, "\t   Architecture/Engineering\n");
   if (fields->org_computer_logical == TRUE)
      kfprintf(file_fd, "\tX  Computer Hardware/Software\n");
   else
      kfprintf(file_fd, "\t   Computer Hardware/Software\n");
   if (fields->org_education_logical == TRUE)
      kfprintf(file_fd, "\tX  Education\n");
   else
      kfprintf(file_fd, "\t   Education\n");
   if (fields->org_medical_logical == TRUE)
      kfprintf(file_fd, "\tX  Health/Medical Services\n");
   else
      kfprintf(file_fd, "\t   Health/Medical Services\n");
   if (fields->org_manufacturing_logical == TRUE)
      kfprintf(file_fd, "\tX  Manufacturing\n");
   else
      kfprintf(file_fd, "\t   Manufacturing\n");
   if (fields->org_military_logical == TRUE)
      kfprintf(file_fd, "\tX  Military\n");
   else
      kfprintf(file_fd, "\t   Military\n");
   if (fields->org_transportation_logical == TRUE)
      kfprintf(file_fd, "\tX  Transportation\n");
   else
      kfprintf(file_fd, "\t   Transportation\n");
   if (fields->org_other_logical == TRUE)
   {
      kfprintf(file_fd, "\tX  Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  %s\n", 
	       fields->org_other_specify_string);
   }
   else
   {
      kfprintf(file_fd, "\t   Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  \n");
   }

   kfprintf(file_fd, "\n   What is your prime function?\n\n");
   if (fields->func_educator_logical == TRUE)
      kfprintf(file_fd, "\tX  Educator\n");
   else
      kfprintf(file_fd, "\t   Educator\n");
   if (fields->func_engineer_logical == TRUE)
      kfprintf(file_fd, "\tX  Engineer\n");
   else
      kfprintf(file_fd, "\t   Engineer\n");
   if (fields->func_mis_logical == TRUE)
      kfprintf(file_fd, "\tX  MIS\n");
   else
      kfprintf(file_fd, "\t   MIS\n");
   if (fields->func_rnd_logical == TRUE)
      kfprintf(file_fd, "\tX  R&D\n");
   else
      kfprintf(file_fd, "\t   R&D\n");
   if (fields->func_other_logical == TRUE)
   {
      kfprintf(file_fd, "\tX  Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  %s\n", 
	       fields->func_other_specify_string);
   }
   else
   {
      kfprintf(file_fd, "\t   Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  \n");
   }

   kfprintf(file_fd, "\n   What do you use Khoros for?\n\n");
   if (fields->use_visualization_logical == TRUE)
      kfprintf(file_fd, "\tX  Data Analysis/Visualization\n");
   else
      kfprintf(file_fd, "\t   Data Analysis/Visualization\n");
   if (fields->use_prototyping_logical == TRUE)
      kfprintf(file_fd, "\tX  Prototyping\n");
   else
      kfprintf(file_fd, "\t   Prototyping\n");
   if (fields->use_swd_logical == TRUE)
      kfprintf(file_fd, "\tX  Software Development\n");
   else
      kfprintf(file_fd, "\t   Software Development\n");
   if (fields->use_rnd_logical == TRUE)
      kfprintf(file_fd, "\tX  Research & Development\n");
   else
      kfprintf(file_fd, "\t   Research & Development\n");
   if (fields->use_other_logical == TRUE)
   {
      kfprintf(file_fd, "\tX  Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  %s\n", 
	       fields->use_other_specify_string);
   }
   else
   {
      kfprintf(file_fd, "\t   Other\n");
      kfprintf(file_fd, "\t      Please  Specify:  \n");
   }
   kfprintf(file_fd, "\n\n\t--------------------------------------------------------\n");
   kfprintf(file_fd, "\t\t\tKhoral Research Inc.\n");
   kfprintf(file_fd, "\t\t\t6001 Indian School Rd. NE\n");
   kfprintf(file_fd, "\t\t\tSuite 200\n");
   kfprintf(file_fd, "\t\t\tAlbuquerque, NM  87110\n");
   kfprintf(file_fd, "\t\t\tPhone:  (505) 837-6500\n");
   kfprintf(file_fd, "\t\t\tFax:    (505) 881-3842\n");
   kfprintf(file_fd, "\t\t\tEmail:  kregister@khoral.com\n");
   kfprintf(file_fd, "\t\t\thttp://www.khoral.com\n");
   kfprintf(file_fd, "\t--------------------------------------------------------\n\n");
   kfclose(file_fd);
   kfree(address_contents);
   return unique_file;
}

/*-----------------------------------------------------------
|
|  Routine Name: extract_gui - Extract all fields from the GUI
|
|       Purpose: Extract all entries from the GUI and store them in
|                a data structure which will be used to indicate
|                what is printed to the output registration file.
|
|         Input: print_fields_struct  *fields - A structure to store GUI fields
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
print_fields_struct *extract_gui()
{
   print_fields_struct	*fields_to_print;

   char         *name_string      = NULL;
   char         *company_string   = NULL;
   char         *address_string   = NULL;
   char         *city_string      = NULL;
   char         *state_string     = NULL;
   char         *zip_string       = NULL;
   char         *country_string   = NULL;
 
   char         *phone_string     = NULL;
   char         *fax_string       = NULL;
   char         *internet_string  = NULL;
 
   char         *newsletter_string   = NULL;
   int          subscribe_newsletter;
 
   char         *upgrade_string   = NULL;
   int          notify_upgrades;
 
   char         *publish_string   = NULL;
   int          publish_name;
 
   int          org_aerospace;
   int          org_engineering;
   int          org_computer;
   int          org_education;
   int          org_medical;
   int          org_manufacturing;
   int          org_military;
   int          org_transportation;
   int          org_other;
   char         *org_other_specify   = NULL;
 
   int          func_educator;
   int          func_engineer;
   int          func_mis;
   int          func_rnd;
   int          func_other;
   char         *func_other_specify   = NULL;
 
   int          use_visualization;
   int          use_prototyping;
   int          use_swd;
   int          use_rnd;
   int          use_other;
   char         *use_other_specify   = NULL;

   fields_to_print = (print_fields_struct *)
                     kmalloc(sizeof(print_fields_struct));

   xvw_get_attribute(xvreg_gui_info->name, XVW_TEXT_STRING, &name_string);
   xvw_get_attribute(xvreg_gui_info->company, XVW_TEXT_STRING, &company_string);
   xvw_get_attribute(xvreg_gui_info->address, XVW_TEXT_STRING, &address_string);
   xvw_get_attribute(xvreg_gui_info->city, XVW_TEXT_STRING, &city_string);
   xvw_get_attribute(xvreg_gui_info->state, XVW_TEXT_STRING, &state_string);
   xvw_get_attribute(xvreg_gui_info->zip, XVW_TEXT_STRING, &zip_string);
   xvw_get_attribute(xvreg_gui_info->country, XVW_TEXT_STRING, &country_string);
   xvw_get_attribute(xvreg_gui_info->phone, XVW_TEXT_STRING, &phone_string);
   xvw_get_attribute(xvreg_gui_info->fax, XVW_TEXT_STRING, &fax_string);
   xvw_get_attribute(xvreg_gui_info->email, XVW_TEXT_STRING, &internet_string);

   if (name_string    == NULL || kstrlen(name_string)    == 0)
   {
      fields_to_print->status = NO_NAME_STRING;
      return(fields_to_print);
   }
   if (address_string    == NULL || kstrlen(address_string)    == 0)
   {
      fields_to_print->status = NO_ADDRESS_STRING;
      return(fields_to_print);
   }
   if (city_string    == NULL || kstrlen(city_string)    == 0)
   {
      fields_to_print->status = NO_CITY_STRING;
      return(fields_to_print);
   }
   if (state_string    == NULL || kstrlen(state_string)    == 0)
   {
      fields_to_print->status = NO_STATE_STRING;
      return(fields_to_print);
   }
   if (zip_string    == NULL || kstrlen(zip_string)    == 0)
   {
      fields_to_print->status = NO_ZIP_STRING;
      return(fields_to_print);
   }
   if (country_string    == NULL || kstrlen(country_string)    == 0)
   {
      fields_to_print->status = NO_COUNTRY_STRING;
      return(fields_to_print);
   }
   fields_to_print->name_string = name_string;
   fields_to_print->company_string = company_string;
   fields_to_print->address_string = address_string;
   fields_to_print->city_string = city_string;
   fields_to_print->state_string = state_string;
   fields_to_print->zip_string = zip_string;
   fields_to_print->country_string = country_string;
   fields_to_print->phone_string = phone_string;
   fields_to_print->fax_string = fax_string;
   fields_to_print->internet_string = internet_string;

   xvw_get_attribute(xvreg_gui_info->newsletter, XVW_LABEL, &newsletter_string);
 
   if (kstrcmp(newsletter_string, "Yes") == 0)
      subscribe_newsletter = 1;
   else
      subscribe_newsletter = 0;
 
   fields_to_print->newsletter_logical = subscribe_newsletter;
 
   xvw_get_attribute(xvreg_gui_info->upgrade, XVW_LABEL, &upgrade_string);
 
   if (kstrcmp(upgrade_string, "Yes") == 0)
      notify_upgrades = 1;
   else
      notify_upgrades = 0;
 
   fields_to_print->notify_upgrades_logical = notify_upgrades;
 
   xvw_get_attribute(xvreg_gui_info->mailinglist, XVW_LABEL, &publish_string);
 
   if (kstrcmp(publish_string, "Yes") == 0)
      publish_name = 1;
   else
      publish_name = 0;
 
   fields_to_print->publish_name_logical = publish_name;
 
   if (xvreg_gui_info->org_aerospace.toggle_button_highlighted == TRUE)
      org_aerospace = 1;
   else
      org_aerospace = 0;
   fields_to_print->org_aerospace_logical = org_aerospace;
 
   if (xvreg_gui_info->org_engineering.toggle_button_highlighted == TRUE)
      org_engineering = 1;
   else
      org_engineering = 0;
   fields_to_print->org_engineering_logical = org_engineering;
 
   if (xvreg_gui_info->org_computer.toggle_button_highlighted == TRUE)
      org_computer = 1;
   else
      org_computer = 0;
   fields_to_print->org_computer_logical = org_computer;
 
   if (xvreg_gui_info->org_education.toggle_button_highlighted == TRUE)
      org_education = 1;
   else
      org_education = 0;
   fields_to_print->org_education_logical = org_education;
 
   if (xvreg_gui_info->org_medical.toggle_button_highlighted == TRUE)
      org_medical = 1;
   else
      org_medical = 0;
   fields_to_print->org_medical_logical = org_medical;

   if (xvreg_gui_info->org_manufacturing.toggle_button_highlighted == TRUE)
      org_manufacturing = 1;
   else
      org_manufacturing = 0;
   fields_to_print->org_manufacturing_logical = org_manufacturing;
 
   if (xvreg_gui_info->org_military.toggle_button_highlighted == TRUE)
      org_military = 1;
   else
      org_military = 0;
   fields_to_print->org_military_logical = org_military;
 
   if (xvreg_gui_info->org_transportation.toggle_button_highlighted == TRUE)
      org_transportation = 1;
   else
      org_transportation = 0;
   fields_to_print->org_transportation_logical = org_transportation;
 
   if (xvreg_gui_info->org_other.toggle_button_highlighted == TRUE)
   {
      xvw_get_attribute(xvreg_gui_info->org_other_specify,
                        XVW_TEXT_STRING, &org_other_specify);
      if (org_other_specify == NULL)
      {
	 fields_to_print->status = NO_ORG_PLEASE_SPECIFY;
         return(fields_to_print);
      }
      org_other = 1;
   }
   else
      org_other = 0;
   fields_to_print->org_other_logical = org_other;
   fields_to_print->org_other_specify_string = org_other_specify;
 
   if (xvreg_gui_info->func_educator.toggle_button_highlighted == TRUE)
      func_educator = 1;
   else
      func_educator = 0;
   fields_to_print->func_educator_logical = func_educator;
 
   if (xvreg_gui_info->func_engineer.toggle_button_highlighted == TRUE)
      func_engineer = 1;
   else
      func_engineer = 0;
   fields_to_print->func_engineer_logical = func_engineer;
 
   if (xvreg_gui_info->func_mis.toggle_button_highlighted == TRUE)
      func_mis = 1;
   else
      func_mis = 0;
   fields_to_print->func_mis_logical = func_mis;
 
   if (xvreg_gui_info->func_rnd.toggle_button_highlighted == TRUE)
      func_rnd = 1;
   else
      func_rnd = 0;
   fields_to_print->func_rnd_logical = func_rnd;

   if (xvreg_gui_info->func_other.toggle_button_highlighted == TRUE)
   {
      xvw_get_attribute(xvreg_gui_info->func_other_specify,
                        XVW_TEXT_STRING, &func_other_specify);
      if (func_other_specify == NULL)
      {
	 fields_to_print->status = NO_FUNC_PLEASE_SPECIFY;
         return(fields_to_print);
      }
      func_other = 1;
   }
   else
      func_other = 0;
   fields_to_print->func_other_logical = func_other;
   fields_to_print->func_other_specify_string = func_other_specify;

   if (xvreg_gui_info->use_visualization.toggle_button_highlighted == TRUE)
      use_visualization = 1;
   else
      use_visualization = 0;
   fields_to_print->use_visualization_logical = use_visualization;
 
   if (xvreg_gui_info->use_prototyping.toggle_button_highlighted == TRUE)
      use_prototyping = 1;
   else
      use_prototyping = 0;
   fields_to_print->use_prototyping_logical = use_prototyping;
 
   if (xvreg_gui_info->use_swd.toggle_button_highlighted == TRUE)
      use_swd = 1;
   else
      use_swd = 0;
   fields_to_print->use_swd_logical = use_swd;
 
   if (xvreg_gui_info->use_rnd.toggle_button_highlighted == TRUE)
      use_rnd = 1;
   else
      use_rnd = 0;
   fields_to_print->use_rnd_logical = use_rnd;
 
   if (xvreg_gui_info->use_other.toggle_button_highlighted == TRUE)
   {
      xvw_get_attribute(xvreg_gui_info->use_other_specify,
                        XVW_TEXT_STRING, &use_other_specify);
      if (use_other_specify == NULL)
      {
	 fields_to_print->status = NO_USE_PLEASE_SPECIFY;
         return(fields_to_print);
      }
      use_other = 1;
   }
   else
      use_other = 0;
   fields_to_print->use_other_logical = use_other;
   fields_to_print->use_other_specify_string = use_other_specify;

   fields_to_print->status = ALL_FIELDS_OK;
   return(fields_to_print);
}

/*-----------------------------------------------------------
|
|  Routine Name: cleanup_temp_file - Cleanup temp file string
|
|       Purpose: Any file names created by "ktempnam" prepend the transport
|		 plus and = sign to the file name.  This needs to be 
|		 stripped off.
|
|         Input: char  *string - String to be cleaned up
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications: kevins made it _clean_.  Caller should free original pointer.
|
------------------------------------------------------------*/
char *cleanup_temp_file(char *string)
{
   char 	*tmp;

   if ((tmp = kstrchr(string, '=')) == NULL)
   {
      tmp = kstrdup(string);
      return(tmp);
   }
   else
   {
      tmp = kstrdup(++tmp);
      return(tmp);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: prompt_submit_reg - Prompt the user for submit method
|
|       Purpose: The user may want to email the registration or save it
|		 to a file.  This routine prompts them for the method
|		 and returns the method selected.
|
|         Input: None
|
|        Output: None
|
|       Returns: 0, 1 or 2 depending on what method was selected:
|                   0 - If the user selected "Cancel"
|		    1 - If submit method is via email
|		    2 - If submit method is via writing to a file
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
int prompt_submit_reg()
{
   char	*register_actions[2];
   int  action_chosen;
   int  i;
   char *dummy = NULL;

   register_actions[0] = kstrdup("Email registration to Khoral Research");
   register_actions[1] = kstrdup("Save registration to file");
   (void) kchoose(KFORCE, register_actions, 2, 1, dummy, &action_chosen,
                  "Select registration method");
   for (i = 0; i < 2; i++)
      kfree(register_actions[i]);
   return(action_chosen);
}

/*-----------------------------------------------------------
|
|  Routine Name: email_registration - Email the registration form to KRI
|
|       Purpose: Attempt to email the registration form to Khoral Research.
|		 If unable to email, then the file should be saved in the
|		 user's home directory.
|
|         Input: None
|
|        Output: None
|
|       Returns: TRUE or FALSE depending on whether emailing was successful
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
int email_registration(char *tmp_file)
{
   char	**path_contents;
   char	*path_env_var;
   char	*mail_header_file;
   char	*save_file_in_mail_error;
   char	cat_command[KLENGTH];
   char	mail_command[KLENGTH];
   int  num_path_entries;
   int  send_mail_in_path = FALSE;
   int  mailer_in_path    = FALSE;

   path_env_var = kgetenv("PATH");
   path_contents = 
     kparse_string_delimit(
	path_env_var, ":", KDELIM_CLEAN, (ssize_t *) &num_path_entries);
   if ((send_mail_in_path = 
	check_path(num_path_entries, path_contents, PATH_SENDMAIL)) == FALSE)
      mailer_in_path = check_path(num_path_entries, path_contents, PATH_MAILER);
   else
   {
      if ((mail_header_file = create_mail_header(tmp_file)) != NULL)
      {
         if (check_path(num_path_entries, path_contents, PATH_CAT) == TRUE)
         {
            ksprintf(cat_command, "cat %s %s | sendmail kregister@khoral.com",
                     mail_header_file, tmp_file);
            if (ksystem(cat_command) != 0)
	    {
	       kerror("xvregister", "email_registration",
		  "Failed to system sendmail command.  Your registration will be saved to a file:  ~/khoros_registration");
               save_file_in_mail_error = kstrdup("~/khoros_registration");
               save_file_in_mail_error = 
			kfullpath(save_file_in_mail_error, NULL, NULL);
               if (kcopyfile(tmp_file, save_file_in_mail_error) == -1)
                  print_error_not_saved(save_file_in_mail_error, tmp_file);
               else
               {
                  remove_temp_file(tmp_file, save_file_in_mail_error);
                  kinfo(KFORCE, "Your file has been saved to '%s'.  Please print and send your registration to Khoral Research", save_file_in_mail_error);
               }
               kfree(path_contents);
               return(FALSE);
	    }
            else
               kinfo(KFORCE, 
		     "Registration has been mailed to 'kregister@khoral.com'");
         }
      }
      else
      {
         kfree(path_contents);
         return(FALSE);
      }
   }
   if (mailer_in_path == TRUE && send_mail_in_path == FALSE)
   {
      ksprintf(mail_command, "mail kregister@khoral.com < %s", tmp_file);
      if (ksystem(mail_command) != 0)
      {
         kerror("xvregister", "email_registration",
	        "Failed to system mail command.  Your registration will be saved to a file:  ~/khoros_registration");
         save_file_in_mail_error = kstrdup("~/khoros_registration");
         save_file_in_mail_error = 
			kfullpath(save_file_in_mail_error, NULL, NULL);
         if (kcopyfile(tmp_file, save_file_in_mail_error) == -1)
            print_error_not_saved(save_file_in_mail_error, tmp_file);
         else
         {
            remove_temp_file(tmp_file, save_file_in_mail_error);
            kinfo(KFORCE, "Your file has been saved to '%s'.  Please print and send your registration to Khoral Research", save_file_in_mail_error);
         }
         kfree(path_contents);
         return(FALSE);
      }
      else
         kinfo(KFORCE, 
		"Registration has been mailed to 'kregister@khoral.com'");
   }
   if (mailer_in_path == FALSE && send_mail_in_path == FALSE)
   {
      kerror("xvregister", "email_registration",
             "Unable to mail your registration because 'sendmail' or 'mail' is not in your path.  The file will be saved in:  '~/khoros_registration'");
      save_file_in_mail_error = kstrdup("~/khoros_registration");
      save_file_in_mail_error = kfullpath(save_file_in_mail_error, NULL, NULL);
      if (kcopyfile(tmp_file, save_file_in_mail_error) == -1)
      {
         print_error_not_saved(save_file_in_mail_error, tmp_file);
      }
      else
      {
         remove_temp_file(tmp_file, save_file_in_mail_error);
         kinfo(KFORCE, "Your file has been saved to '%s'.  Please print and send your registration to Khoral Research", save_file_in_mail_error);
      }
      kfree(path_contents);
      return(FALSE);
   }
   kfree(path_contents);
   return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: save_registration - Save the registration to a file.
|
|       Purpose: Attempt to save the registration to a file where the
|		 filename is specified by the user.  If unable to save
|		 the file, a copy will remain in $TMPDIR.
|
|         Input: None
|
|        Output: None
|
|       Returns: TRUE or FALSE depending on whether file saving was successful
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
int save_registration(char *tmp_file)
{
   char	*filename = NULL;
   int	process_file_name;
   int	overwrite_file;

   filename = prompt_for_filename();
   if (filename == NULL)
   {
      print_error_not_saved(filename, tmp_file);
      return(FALSE);
   }
   else
   {
      process_file_name = TRUE;
      while(process_file_name)
      { 
         if (kaccess(filename, W_OK) == 0)
	 {
            overwrite_file = prompt_overwrite_file(filename);
            if (overwrite_file == TRUE)
            {
               copy_and_remove_file(tmp_file, filename);
               kinfo(KFORCE,
		     "Your file has been saved to '%s'.  Please print and send "
		     "your registration to Khoral Research", filename);
               process_file_name = FALSE;
            }
            else
            {
               kfree(filename);
               filename = prompt_for_filename();
               if (filename == NULL)
               {
                  print_error_not_saved(filename, tmp_file);
                  process_file_name = FALSE;
                  return(FALSE);
               }
            }
	 }
         else
	 {
            if (kcopyfile(tmp_file, filename) == -1)
            {
               print_error_save_failed(filename);
               if (filename != NULL)
                  kfree(filename);
               filename = prompt_for_filename();
               if (filename == NULL)
               {
                  print_error_not_saved(filename, tmp_file);
                  process_file_name = FALSE;
                  return(FALSE);
               }
            }
            else
            {
               remove_temp_file(tmp_file, filename);
               process_file_name = FALSE;
               kinfo(KFORCE,
		     "Your file has been saved to '%s'.  Please print and send "
		     "your registration to Khoral Research", filename);
            }
	 }
      } 
   }
   if (filename != NULL)
     kfree(filename);
   return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: prompt_for_filename - Prompt the user to enter a filename
|
|       Purpose: Prompt the user to enter a filename which indicates where
|		 to save the registration form.
|
|         Input: None
|
|        Output: None
|
|       Returns: char *filename - filename specified
|		 or NULL on failure
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
char *prompt_for_filename()
{
   char	*file_prompt[1];
   char	*file_answer[1];
   char	*save_button;

   file_prompt[0] = kstrdup("Filename:  ");
   file_answer[0] = kstrdup("./khoros_registration");

   save_button = kstrdup("Save");

   if ((xvu_query_wait("Please specify file name",
		        file_prompt, save_button, file_answer, 1, 50)) == 0)
   {
      kfree(file_prompt[0]);
      kfree(file_answer[0]);
      kfree(save_button);
      return(NULL);
   }
   else
   {
      kfree(file_prompt[0]);
      kfree(save_button);
      return(file_answer[0]);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: print_error_not_saved - Print error message
|
|       Purpose: Print an error message indicating that the registration 
|		 file was not saved.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
void print_error_not_saved(char *filename, char *tmp_file)
{
   if (filename == NULL)
      kinfo(KFORCE, "Your Khoros Registration was not saved.  A copy of your "
	            "registration still remains in '%s'",
                    tmp_file);
   else
      kinfo(KFORCE, "File '%s' was not saved.  A copy still remains in '%s'",
                    tmp_file, filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: print_error_save_failed - Print error message
|
|       Purpose: Print an error message indicating that the registration 
|		 file was not saved because of an error.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
void print_error_save_failed(char *filename)
{
   kerror("xvregister", "save_registration",
          "Unable to write file '%s'", filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: prompt_overwrite_file - Prompt to overwrite an existing file
|
|       Purpose: Prompt the user that the file exists and ask them if they
|		 want to overwrite.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
int prompt_overwrite_file(char *filename)
{
   int 	overwrite_status;

   overwrite_status = kprompt(KFORCE, NULL, NULL, 2,
		"File '%s' exists, do you want to overwrite it?",
	        filename);
   return(overwrite_status);
}

/*-----------------------------------------------------------
|
|  Routine Name: copy_and_remove_file - Copy the temp file to the specified
|					destination path and remove the
|					temporary file.
|
|       Purpose: A valid file name was specified by the user so copy the
|		 file in $TMPDIR to the destination file and remove the
|		 temporary file.  Inform the user that the file was
|		 copied.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
void copy_and_remove_file(char *tmp_file, char *filename)
{
   kcopyfile(tmp_file, filename);
   kunlink(tmp_file);
}

/*-----------------------------------------------------------
|
|  Routine Name: remove_temp_file - Remove the temporary registration
|                                   file from $TMP because it has been
|                                   successfully copied somewhere else.
|                                       temporary file.
|
|       Purpose: A valid file name was specified by the user so copy the
|                file in $TMPDIR to the destination file and remove the
|                temporary file.  Inform the user that the file was
|                copied.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
void remove_temp_file(char *tmp_file, char *filename)
{
   kunlink(tmp_file);
   kinfo(KFORCE, "File '%s' has been saved.", filename);
}

/*-----------------------------------------------------------
|
|  Routine Name: check_path - Check to see if an executable is in the
|			      user's path.
|
|       Purpose: This routine checks to see if an executable is in
|                the user's path.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
int check_path(int num_path_entries, char **path, int exec_type)
{
   int	search_path  = TRUE;
   int  exec_in_path = FALSE;
   int	i = 0;
   char *exec_path_entry = NULL;

   while(search_path == TRUE && i < num_path_entries)
   {
      if (exec_type == PATH_SENDMAIL)
      {
         if (exec_path_entry != NULL)
            kfree(exec_path_entry);
         exec_path_entry = kstring_cat(path[i], "/sendmail", NULL);
      }
      else if (exec_type == PATH_MAILER)
      {
         if (exec_path_entry != NULL)
            kfree(exec_path_entry);
         exec_path_entry = kstring_cat(path[i], "/mail", NULL);
      }
      else if (exec_type == PATH_CAT)
      {
         if (exec_path_entry != NULL)
            kfree(exec_path_entry);
         exec_path_entry = kstring_cat(path[i], "/cat", NULL);
      }
 
      if (kaccess(exec_path_entry, X_OK) == 0)
      {
         exec_in_path = TRUE;
         search_path = FALSE;
      }
      i++;
   }
   if (exec_path_entry != NULL)
      kfree(exec_path_entry);
   return(exec_in_path);
}
/*-----------------------------------------------------------
|
|  Routine Name: create_mail_header - Create a mail header file for sendmail
|
|       Purpose: This routine creates a file containing the mail header
|                which will later be concatenated with the registration
|		 file and mailed to KRI via sendmail.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: March 14, 1996
| Modifications:
|
------------------------------------------------------------*/
char *create_mail_header(char *tmp_file)
{
   char	  *mail_header_filename;
   char	  *save_file_in_mail_error;
   kfile  *mail_header_fp;

   mail_header_filename = ktempnam(NULL, "file=xvregister_sendmail"); 
   mail_header_filename = cleanup_temp_file(mail_header_filename);

   if ((mail_header_fp = kfopen(mail_header_filename, "w")) == NULL)
   {
      kerror("xvregister", "create_mail_header",
	     "Unable to open mail file '%s'", mail_header_filename);
      save_file_in_mail_error = kstrdup("~/khoros_registration");
      save_file_in_mail_error = kfullpath(save_file_in_mail_error, NULL, NULL);
      if (kcopyfile(tmp_file, save_file_in_mail_error) == -1)
         print_error_not_saved(save_file_in_mail_error, tmp_file);
      else
      {
         remove_temp_file(tmp_file, save_file_in_mail_error);
         kinfo(KFORCE, "Your file has been saved to '%s'.  Please print and send your registration to Khoral Research.", save_file_in_mail_error);
      }
      return (NULL);
   }
   else
   {
      kfprintf(mail_header_fp, "To:  kregister@khoral.com\n");
      kfprintf(mail_header_fp, "Subject:  Khoros Registration\n");
      kfclose(mail_header_fp);
      return (mail_header_filename);
   }
}

/*-----------------------------------------------------------
|
|  Routine Name: xvreg_return_cr - Null action handler to overcome Athena
|
|       Purpose: In Athena with text objects, even though the text object
|                is NOT multiline, when a cr is given after entering text,
|		 the object scrolls thus appearing to erase the text.  This
|		 action handler is basically a no-op to keep that from 
|		 happening.
|
|         Input: None
|
|        Output: None
|
|       Returns: Nothing
|
|    Written By: Becky Bishop
|          Date: April 3, 1996
| Modifications:
|
------------------------------------------------------------*/
void xvreg_return_cr(xvobject text_obj, kaddr client_data, XEvent *event)
{
   return;
}
