#include <xvinclude.h>

/*
 * This program demonstrates the use of the marker visual object;
 * it creates a manager object containing several markers using 
 * different foreground colors.
 *
 * The marker object may be selected and moved by holding down the shift key and
 * using the left mouse button. The menuform for the marker annotation may be 
 * brought up by holding down the meta key and clicking the middle mouse button;
 * the menuform may be used to change attributes of the marker such as type,
 * scale, color, and world coordinate placement.
 */

void main(int  argc,
   char *argv[])
{
	int    i;
        xvobject marker;
        xvobject parent;
	static char *list[] = {"yellow", "orange", "red", "violet", 
			       "purple", "blue", "green", "yellowgreen"}; 

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets library */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }

	/* 
	 * create a manager object to be the parent of all the markers;
         * make the background black
         */
	parent = xvw_create_manager(NULL, "parent");
	xvw_set_attributes(parent, 
			   XVW_BACKGROUND_COLOR, "black",
			   XVW_WIDTH,            450,
			   XVW_HEIGHT,           450,
			   NULL);
	
	/* 
	 * create one marker for each color in the list; 
	 * set its placement, color, type, and scale
	 */
	for (i = 0; i < knumber(list); i++)
	{
            marker = xvw_create_marker(parent, "marker");
            xvw_set_attributes(marker,
                               XVW_MARKER_XPLACEMENT,   (i+1) * 0.1,
                               XVW_MARKER_YPLACEMENT,   (i+1) * 0.1,
                               XVW_FOREGROUND_COLOR,     list[i],
                               XVW_GRAPHICS_MARKERTYPE,  KMARKER_PIXEL,
                               XVW_GRAPHICS_MARKERSCALE, 3,
                               NULL);
	}

	/* display & run the program */
        xvf_run_form();
}

