#include <xvinclude.h>

void marker_event_handler PROTO((xvobject, kaddr, XEvent *, int *));

/*
 * This is an example of a marker object using an event handler.
 * It creates a single marker annotation, and adds an event handler, so that
 * button press on the marker turns it blue!  Button release turns it red again!
 */

void main(
   int  argc,
   char *argv[])
{
	xvobject marker;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidget library */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
	}

	/* create the bowtie marker in red at (0.5, 0.5) with scale of 4 */
	marker = xvw_create_marker(NULL, "marker");
	xvw_set_attributes(marker,
		XVW_MARKER_XPLACEMENT,    0.5,
		XVW_MARKER_YPLACEMENT,    0.5,
		XVW_FOREGROUND_COLOR,     "red",
		XVW_GRAPHICS_MARKERTYPE,  KMARKER_BOW_TIE,
		XVW_GRAPHICS_MARKERSCALE, 4,
		NULL);

	/* 
	 * Add the event handler "marker_event_handler" on the marker, 
         * which will be invoked on both button press & button release.
	 */
	xvw_add_event(marker, ButtonPressMask | ButtonReleaseMask,
		      marker_event_handler, NULL);

	/* display and run */
	xvf_run_form();
}

/*
 *  here is the event handler invoked by button press & 
 *  button release events that happen on the marker.
 */
void marker_event_handler(
    xvobject  object,
    kaddr     client_data,
    XEvent    *event,
    int       *dispatch)
{
	if (event->type == ButtonPress)
	{
	   xvw_set_attribute(object, XVW_FOREGROUND_COLOR, "blue");
	   kfprintf(kstderr, "Turning marker blue\n");
	}
	else if (event->type == ButtonRelease)
	{
	   xvw_set_attribute(object, XVW_FOREGROUND_COLOR, "red");
	   kfprintf(kstderr, "Turning marker red\n");
	}
}
