#include <xvinclude.h>

/*
 * This program creates a window with an area object, containing a
 * 2D axis object and an indicator object.  The indicator object displays 
 * its world coordinate position within the area object. The internal menuform 
 * for the indicator is displayed automatically, and may be used to set 
 * attributes of the indicator.
 *
 * The indicator allows you to interactively it by "grabbing" it with the 
 * left mouse button and moving it around inside the area object; it will 
 * update the world coordinate value that it displays as it is moved.  
 * The 2D axis object serves to provide a reference as to the world 
 * coordinate system.
 */

void main(
   int  argc,
   char *argv[])
{
        xvobject parent, area, axis, indicator;

	/* initialize khoros program */
	khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }
	
	/*
	 *  create a manager backplane
	 */
	parent = xvw_create_manager(NULL, "parent");

        /*
         * create an area object to provide a coordinated WC view
	 * for the 2D axis object and the indicator object
         */
        area = xvw_create_area(parent, "area");
        xvw_set_attributes(area,
                     XVW_AREA_DISPLAY_DATE,     FALSE,
                     XVW_AREA_DISPLAY_TITLE,    TRUE,
		     XVW_MAXIMUM_WIDTH,                 300,  
		     XVW_MAXIMUM_HEIGHT,                300, 
                     NULL );

        /*
         *  the 2D axis system will display the world coordinates
         */
        axis = xvw_create_axis2d(area, "axis coordinated");
        xvw_set_attributes(axis,
                     XVW_GRAPHICS_ATTACH,         axis,
                     XVW_GRAPHICS_VIEWPORT_MIN_X, 0.1,
                     XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.1,
                     XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                     XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
                     NULL);

	/*
	 *  create the indicator object;  attach it to the axis so
	 *  that the axis will control the world view.  set the X & Y 
	 *  position values, and set the foreground color. 
	 */
        indicator = xvw_create_indicator(area, "indicator");
        xvw_set_attributes(indicator,
                XVW_GRAPHICS_ATTACH,      axis,
		XVW_INDICATOR_XPOS_VALUE, 0.5, 
		XVW_INDICATOR_YPOS_VALUE, 0.5, 
	        XVW_FOREGROUND_COLOR,     "pink",
		NULL);

	/* display internal menuform for indicator */
	xvw_activate_menu(indicator);

	/* display and run the program */
        xvf_run_form();
}
