#include <xvinclude.h>

/*
 *
 * This example creates a window with a single string annotation in it.
 *
 * After the manager object parent is put into edit mode (hold down
 * shift key and click the left mouse button), the string can be
 * interactively moved by "grabbing" it in the middle with the left mouse
 * button; grabbing it near an end will cause the string to resize.
 *
 * In this example, the internal menuform for the string is displayed
 * automatically.
 */

void main( 
   int  argc,
   char *argv[])
{
        xvobject parent;
        xvobject string;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

        /* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }

        /*
         *  create a manager backplane, width & height of 300.
         */
        parent = xvw_create_manager(NULL, "parent");
        xvw_set_attributes(parent,
                           XVW_WIDTH,      300,
                           XVW_HEIGHT,     300,
                           NULL);

        /*
         *  create the string object, specifying placement in world coordinates.
         *  an overly large character width & height is given so that 
	 *  justification can be tested.  specify string to be displayed and
	 *  style to display it in.
         */
        string = xvw_create_string(parent, "string");
        xvw_set_attributes(string,
		           XVW_STRING_XPLACEMENT,    0.5,
		           XVW_STRING_YPLACEMENT,    0.5,
			   XVW_CHAR_WIDTH,           25.0,
			   XVW_CHAR_HEIGHT,          10.0,
		           XVW_STRING_STRING,	    "Test\n String",
		           XVW_STRING_JUSTIFICATION, KSTRING_JUSTIFY_CENTER,
		           XVW_STRING_STYLE,         KSTRING_STYLE_EMBOSSED_OUT,
		           NULL);
        /*
         *  activate the menuform so user doesn't have to bring it up.
         */
        xvw_activate_menu(string);

        /* display & run the program */
        xvf_run_form();
}
