#include <xvinclude.h>

/*
 *  This program creates an image object, a position object to reflect the
 *  location of the mouse pointer in the image, and a printpixel object to
 *  print the values of the pixels surrounding that location.
 *
 *  Note that you do NOT have to write an event handler to make the 
 *  printpixel object update; the fact that we have created an image object
 *  using the same file for it's data does the trick.  The image object will
 *  recognise pointer motion within it, and cause the printpixel object (and
 *  the position object as well) to be updated automatically.
 */
xvobject image, position, printpixel;
static void infile_cb PROTO((xvobject, kaddr, kaddr));

void main(
     int  argc,
     char **argv, 
     char **envp)
{
	kobject  object;
/*	char     *filename = "./ankle_RGB.viff"; */
  	char     *filename = "image:mandril";
	xvobject parent, infile;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* create a data object from the information in the image file */
	object = kpds_open_input_object(filename);

        /* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/* create a manager to contain printpixel & position objects */
	parent = xvw_create_manager(NULL, "parent");

	/* 
	 * create the image object below the inputfile object,
	 * and associate it with the desired filen
	 */
	image = xvw_create_image(parent, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEOBJ, object);

	/* 
	 * create the position object in the center below the image; 
	 * associate it with the same data using XVW_POSITION_OBJECT.
	 */
	position = xvw_create_position(parent, "position");
	xvw_set_attributes(position,
			   XVW_BELOW,           image,
			   XVW_LEFT_OF,         image,
			   XVW_RIGHT_OF,        image,
			   XVW_POSITION_OBJECT, object,
			   NULL);

	/*
	 * create an inputfile object so we can enter a new filename
	 */
	infile = xvw_create_inputfile(parent, "infile");
	xvw_set_attributes(infile,
                           XVW_BELOW,              position,
                           XVW_TACK_EDGE,          KMANAGER_TACK_HORIZ,
                           XVW_INPUTFILE_FILENAME, filename,
                           NULL);
	xvw_add_callback(infile, XVW_INPUTFILE_CALLBACK, infile_cb, NULL);

	/*
	 * create the printpixel object in an independent window,
         * associate it with the same data using XVW_PRINTPIXEL_OBJECT.
	 * want 9 data pixel values in each row, 11 in each column.
	 */
	parent = xvw_create_manager(NULL, "printpixel_parent");
	printpixel = xvw_create_printpixel(parent, "printpixel");
	xvw_set_attributes(printpixel,
			   XVW_PRINTPIXEL_OBJECT,    object,
			   XVW_PRINTPIXEL_WIDTH,     9,
                           XVW_PRINTPIXEL_HEIGHT,    11, 
			   NULL);

	/* display & run the program */
	xvf_run_form();
}

/*
 *  callback to allow user to enter new filename
 */
static void infile_cb(
    xvobject object,
    kaddr    client_data,
    kaddr    call_data)
{
	char *filename;
	kobject data_object;

	xvw_get_attribute(object,     XVW_INPUTFILE_FILENAME, &filename);
	data_object = kpds_open_input_object(filename);

	xvw_set_attribute(image,      XVW_IMAGE_IMAGEFILE,     filename);
	xvw_set_attribute(position,   XVW_POSITION_FILENAME,   filename);
	xvw_set_attribute(printpixel, XVW_PRINTPIXEL_FILENAME, filename);
/*
        xvw_set_attribute(image,      XVW_IMAGE_IMAGEOBJ,    data_object);
        xvw_set_attribute(position,   XVW_POSITION_OBJECT,   data_object);
        xvw_set_attribute(printpixel, XVW_PRINTPIXEL_OBJECT, data_object);
*/
	
}
