#include <xvinclude.h>

/*
 *
 * This example demonstrates how to put an input handler on a visual object.
 * It creates an image object and puts an input handler on it that will update
 * the image if the file is over-written with a new image.  To see the example
 * in action you actually have to clobber the input file ("test_image.xv") 
 * located in this directory with another image.
 *
 * A string annotation is placed on top of the image simply for testing 
 * purposes; it should always appear, regardless of whether the image changes.
 *
 */
int detectfile_handler PROTO((xvobject, char *, kaddr));

void main(
   int  argc,
   char *argv[])
{
        xvobject string;
        xvobject image;
	char     *filename = "$ENVISION/examples/image/14.updateimage/test_image.xv";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }

	/*
	 *  Create an image that displays a picture of the moon.  This
         *  is done by creating an image widget, and then using the
	 *  XVW_IMAGE_IMAGEFILE attribute to indicate which data set to
	 *  display as the image.
	 */
	image = xvw_create_image(NULL, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEFILE, filename);

	/*
	 *  Create the string and set it's placement using device coordinates.
	 *  Or we can also use XVW_STRING_XPLACEMENT & XVW_STRING_YPLACEMENT
	 *  to place the string using world coordinates.
	 */
        string = xvw_create_string(image, "string");
        xvw_set_attributes(string,
                           XVW_XPOSITION, 	  256,
                           XVW_YPOSITION,	  256,
			   XVW_MENUABLE,          TRUE,
			   XVW_STRING_STRING,     "This is sample text",
			   XVW_FOREGROUND_COLOR,  "red",
                           NULL);

	/*
	 *  If the file changes then we will need to update it.  Currently
	 *  we have to write our own function to do so.  In the future this
	 *  won't be necessary.
	 */
	xvw_add_detectfile(image, filename, 0.5, detectfile_handler, NULL);

	/*
	 *  Display and run the program.  Note that we won't return until
	 *  the image widget is destroyed.
	 */
        xvf_run_form();
}

int detectfile_handler(
   xvobject image,
   char     *filename,
   kaddr    client_data)
{
	kobject data_object;


	/*
	 *  Make sure that the file has not been deleted.
	 */
	if (kaccess(filename, R_OK) == -1)
	   return(TRUE);

	/*
	 * Get data object from the modified file.  If we fail to open the
	 * data file then just return.  Otherwise, tell the image widget
	 * to update the image and then close the data object, since the
	 * image widget will make it's own reference copy.
	 */
	if ((data_object = kpds_open_object(filename, KOBJ_READ)) != NULL)
	{
	   xvw_set_attribute(image, XVW_IMAGE_IMAGEOBJ, data_object);
	   kpds_close_object(data_object);
	}
	return(TRUE);
}
