 #include <xvinclude.h>

/*
 *  This example program, like the other examples in this directory,
 *  creates an image object to display a large image, along with a pan icon
 *  that can be used to pan about the image, a position object that will
 *  track pointer motion, and a quit button in the upper right hand corner
 *  of the image.  Then, a number of different types of annotations are created
 *  at various locations about the image.
 *
 *  This example differs from the others in that it uses the
 *  WORLD COORDINATE placement to place annotations.
 *  Annotations are explicitly ATTACHED to THEMSELVES.
 *  As you use the pan icon to pan about the image, the annotations should
 *  stay exactly where they are in the image window, and not "move" with
 *  the image.
 */

static void quit_cb PROTO((xvobject, kaddr, kaddr));
void main(
   int  argc,
   char *argv[])
{
        xvobject parent;
        xvobject image;
        xvobject marker;
        xvobject circle;
        xvobject rectangle;
        xvobject ellipse;
        xvobject string;
        xvobject date;
        xvobject panicon;
        xvobject position;
        xvobject quit;
	char     *filename = "image:usa";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(KEXIT_FAILURE);
        }

        /* create a manager backplane to contain image, panicon, & position */
	parent = xvw_create_manager(NULL, "parent");

        /* create image object, set image file */
	image = xvw_create_image(parent, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEFILE, filename);

        /* create pan icon, set same image file */
	panicon = xvw_create_panicon(parent, "panicon");
	xvw_set_attribute(panicon, XVW_IMAGE_IMAGEFILE, filename);

        /* create position object to track pointer */
	position = xvw_create_position(parent, "position");
	xvw_set_attributes(position, 
			   XVW_POSITION_FILENAME,  filename,
			   XVW_BELOW,              image,
			   XVW_LEFT_OF,            NULL,
			   XVW_RIGHT_OF,           NULL,
			   NULL);

        /* create quit button to exit example program */
	quit = xvw_create_button(image, "quit");
	xvw_set_attributes(quit,
			   XVW_LABEL, "quit",
			   XVW_LEFT_OF, NULL,
			   XVW_BELOW,   NULL,
			   NULL);
	xvw_add_callback(quit, XVW_BUTTON_SELECT, quit_cb, NULL);

        /* create marker */
        marker = xvw_create_marker(image, "marker");
        xvw_set_attributes(marker,
			   XVW_GRAPHICS_ATTACH,      marker, 
                           XVW_MARKER_XPLACEMENT,    0.1,
                           XVW_MARKER_YPLACEMENT,    0.2,
                           XVW_GRAPHICS_MARKERSCALE, 9,
			   XVW_FOREGROUND_COLOR,     "red",
                           NULL);

        /* create circle */
	circle = xvw_create_circle(image, "circle");
        xvw_set_attributes(circle,
                           XVW_GRAPHICS_ATTACH,   circle,
                           XVW_CIRCLE_XCENTER,    0.76,
                           XVW_CIRCLE_YCENTER,    0.52,
                           XVW_CIRCLE_RADIUS,     0.1,
                           XVW_FOREGROUND_COLOR,  "magenta",
                           NULL);

        /* create ellipse */
        ellipse = xvw_create_ellipse(image, "ellipse");
        xvw_set_attributes(ellipse,
                           XVW_GRAPHICS_ATTACH,  ellipse,
                           XVW_ELLIPSE_XCENTER,  0.3,
                           XVW_ELLIPSE_YCENTER,  0.3,
                           XVW_ELLIPSE_WIDTH,    0.1,
                           XVW_ELLIPSE_HEIGHT,   0.3,
                           XVW_FOREGROUND_COLOR, "purple",
                           NULL);

        /* create rectangle */
	rectangle = xvw_create_rectangle(image, "rectangle");
        xvw_set_attributes(rectangle,
                           XVW_GRAPHICS_ATTACH,        rectangle,
                           XVW_RECTANGLE_X,            0.2,
                           XVW_RECTANGLE_Y,            0.45,
                           XVW_RECTANGLE_WIDTH,        0.3,
                           XVW_RECTANGLE_HEIGHT,       0.05,
                           XVW_FOREGROUND_COLOR,       "green",
                           NULL);

        /* create string */
        string = xvw_create_string(image, "string");
        xvw_set_attributes(string,
                           XVW_GRAPHICS_ATTACH,    string,
                           XVW_STRING_XPLACEMENT,  0.25,
                           XVW_STRING_YPLACEMENT,  0.5,
                           XVW_STRING_STRING,      "Test String", 
                           XVW_FOREGROUND_COLOR,   "orange",
                           XVW_STRING_STYLE,       KSTRING_STYLE_EMBOSSED_OUT,
                           NULL);

        /* create date */
	date = xvw_create_date(image, "date");
        xvw_set_attributes(date,
                           XVW_GRAPHICS_ATTACH,   string,
                           XVW_STRING_XPLACEMENT, 0.7,
                           XVW_STRING_YPLACEMENT, 0.9,
                           XVW_FOREGROUND_COLOR,  "blue",
                           XVW_STRING_STYLE,      KSTRING_STYLE_EMBOSSED_OUT,
                           NULL);

	/* display & run the program */
        xvf_run_form();
}

/*
 *  callback on quit button to exit program
 */
static void quit_cb(
    xvobject object,
    kaddr    client_data,
    kaddr    call_data)
{
	kexit(KEXIT_SUCCESS);
}
