#include <xvinclude.h>

/* 
 *  This introductory program creates an area object, which serves as a
 *  backplane for a 3D plot object.  The 3D plot data is stored in a KDF
 *  file;  since this data object has no map segment and the 
 *  XVW_PLOT3D_COLOR_ORIGINATION attribute is set to KPLOT3D_COLOR_FROM_DATA, 
 *  the 3D plot object by default will use a rainbow autocolor procedure to 
 *  color the mesh plot.
 */
void main(
   int  argc,
   char *argv[])
{
	xvobject area, plot3d;
	kobject  data_object;
	char     *filename = "image:flow";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/* open the data object defined by file */
	data_object = kpds_open_input_object(filename);

	/* create a 3D color mesh plot from the VIFF file */
	area = xvw_create_area(NULL, "area");
	xvw_set_attributes(area, 
		XVW_BACKGROUND_COLOR,        "black",
		XVW_AREA_TITLE_STRING,       "3D Mesh Plot",
                XVW_GRAPHICS_VIEWPORT_MIN_X,  0.15,
                XVW_GRAPHICS_VIEWPORT_MIN_Y,  0.15,
                XVW_GRAPHICS_VIEWPORT_MIN_Z,  0.15,
                XVW_GRAPHICS_VIEWPORT_MAX_X,  0.85,
                XVW_GRAPHICS_VIEWPORT_MAX_Y,  0.85,
                XVW_GRAPHICS_VIEWPORT_MAX_Z,  0.85,
		NULL);

	plot3d = xvw_create_plot3d(area, "Plot3D");
	xvw_set_attributes(plot3d, 
		XVW_PLOT3D_PLOTOBJ,            data_object,
                XVW_PLOT3D_COLOR_ORIGINATION,  KPLOT3D_COLOR_FROM_DATA,
		XVW_PLOT3D_PLOTTYPE,           KPLOT3D_MESH,
		NULL);

	/* display & run; there is no way to exit the program but ^C */
	xvf_run_form();
}
