#include <xvinclude.h>

/*
 *  This example displays the plot of the data in a KDF file using
 *  XVW_PLOT2D_PLOTFILE, and sets the color of the entire plot to
 *  orange using XVW_FOREGROUND.
 */

void main(
   int  argc,
   char *argv[])
{
	xvobject area, axis, plot;
	char     *filename = "image:featheye";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }

	/* create a black 500x300 area */
        area = xvw_create_area(NULL, "area");
        xvw_set_attributes(area,
                           XVW_WIDTH,           500,
                           XVW_HEIGHT,          300,
                           XVW_BACKGROUND_COLOR, "black",
                           NULL);

	/* create axis, which will dictate world view */
        axis = xvw_create_axis2d(area, "axis");
        xvw_set_attributes(axis, 
                           XVW_GRAPHICS_ATTACH,         axis,
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
                           NULL);

	/* 
	 * create plot, and:
	 * 1 - attach plot to axis so that it gets its world view from the axis
	 * 2 - set plot file to sample KDF file
	 * 3 - set plot type
	 * 4 - set foreground color to orange.
	 */
	plot = xvw_create_plot2d(area, "plot");
	xvw_set_attributes(plot,
                           XVW_GRAPHICS_ATTACH,    axis,
		           XVW_PLOT2D_PLOTFILE,    filename,
		           XVW_PLOT2D_PLOTTYPE,    KPLOT2D_POLYMARKER,
		           XVW_FOREGROUND_COLOR,   "orange",
		           NULL);

	/* bring up menuform automatically */
	xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
