#include <xvinclude.h>

/*
 *  This example creates a 3D plot from the "image:featheye" dataset;
 *  it will be drawn in the foreground color of orange.  
 *
 *  Since the "image:featheye" data set does not have a colormap, and
 *  since XVW_COLOR_COLOROBJ is NOT used to specify another data object
 *  containing a colormap, using the menuform to set "Color Origination" to 
 *  "Use Data Values" will result in the plot being displayed in the 
 *  Rainbow colormap, which is the default colormap for plots.
 */

void main(
   int  argc,
   char *argv[])
{
	xvobject area, plot;
	char     *filename = "image:featheye";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }

	/* create a black 450x450 area; set viewport */
        area = xvw_create_area(NULL, "area");
        xvw_set_attributes(area,
                           XVW_AREA_DISPLAY_TITLE,      FALSE,
                           XVW_WIDTH,                   450,
                           XVW_HEIGHT,                  450,
                           XVW_BACKGROUND_COLOR,        "black",
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.9,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.9,
                           NULL);
	/* 
	 * create plot, and:
	 * 1 - set plot file
	 * 2 - set plot type
	 * 3 - set foreground color to orange.
	 */
	plot = xvw_create_plot3d(area, "plot");
	xvw_set_attributes(plot,
		 XVW_PLOT3D_PLOTFILE,          filename,
		 XVW_PLOT3D_PLOTTYPE,          KPLOT3D_MESH,
		 XVW_FOREGROUND_COLOR,         "orange",
		 NULL);

	/* bring up menuform automatically */
	xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
