#include <xvinclude.h>

/*
 *  This example creates a 3D plot from the "plot3d:kitten" dataset, which
 *  has a colormap.  The plot will be drawn using the colors from the colormap,
 *  since XVW_PLOT3D_COLOR_ORIGINATION is set to KPLOT3D_COLOR_FROM_DATA.
 *
 *  Since XVW_FOREGROUND_COLOR is also used to specify a foreground color
 *  of yellow, using the menuform to set "Color Origination" to
 *  "Use Foreground Color" will result in the plot being displayed in yellow.
 */

void main(
   int  argc,
   char *argv[])
{
	xvobject area, plot;
	char     *filename = "plot3d:kitten";

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }

	/* create a black 450x450 area; set viewport */
        area = xvw_create_area(NULL, "area");
        xvw_set_attributes(area,
                           XVW_AREA_DISPLAY_TITLE,        FALSE,
                           XVW_WIDTH,                     450,
                           XVW_HEIGHT,                    450,
			   XVW_BACKGROUND_COLOR,         "black",
                           XVW_GRAPHICS_VIEWPORT_MIN_X,   0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y,   0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X,   0.8,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y,   0.8,
                           NULL);

	/* 
	 * create plot, and:
	 * 1 - set plot file
	 * 2 - set plot type
	 * 3 - set color origination flag to indicate that color is to be 
         *     obtained from the value data of the input file
         * 4 - set foreground color to yellow in case menuform is used
         *     to set "Color Origination" to "Use Foreground Color"
	 */
	plot = xvw_create_plot3d(area, "plot");
	xvw_set_attributes(plot,
		     XVW_PLOT3D_PLOTFILE,           filename,
		     XVW_PLOT3D_PLOTTYPE,           KPLOT3D_CONSTANT_SHADING,
		     XVW_PLOT3D_COLOR_ORIGINATION,  KPLOT3D_COLOR_FROM_DATA, 
		     XVW_FOREGROUND_COLOR,         "yellow",
		     NULL);

        /* bring up menuform automatically */
        xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
