#include <xvinclude.h>

/*
 *  This example creates a 3D plot from a generated dataset stored in a
 *  Coord array.  It will be drawn using the foreground color, which is set 
 *  to magenta.  Note that the "d" values of each Coord must be set to
 *  KGRAPHICS_UNINITIALIZED so that the foreground color will take effect;
 *  failing to do so will mean that the color of the plot will be undefined.
 *
 *  Since the XVW_PLOT3D_COLOR_ORIGINATION attribute does not apply when
 *  the plot input is provided as a Coord array, attempting to use the menuform
 *  to set "Color Origination" to "Use Data Values" will result in an error
 *  message.
 */

#define POINTNUM 50
#define CURVENUM 50 
void main(
   int  argc,
   char *argv[])
{
	xvobject area, plot;
	Coord    coords[POINTNUM*CURVENUM];
	int      i, j;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }

	/* 
	 * generate data to plot 
	 * NOTE: we MUST set coord.d values to KGRAPHICS_UNINITIALIZED so we 
	 *       can later use XVW_FOREGROUND_COLOR to set the color of the plot
	 */
	for (i = 0; i < POINTNUM; i++)
	   for (j = 0; j < POINTNUM; j++)
	   {
	      coords[i+j*POINTNUM].x = i;
	      coords[i+j*POINTNUM].y = j;
	      coords[i+j*POINTNUM].z = 0.1 * sin((double) 
					    kdegrees_radians(i * 360/POINTNUM));
	      coords[i+j*POINTNUM].d = KGRAPHICS_UNINITIALIZED;
	   }

	/* create a black 500x300 area */
        area = xvw_create_area(NULL, "area");
        xvw_set_attributes(area,
                           XVW_AREA_DISPLAY_TITLE,      FALSE,
                           XVW_WIDTH,                   450,
                           XVW_HEIGHT,                  450,
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Z, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.8,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.8,
                           XVW_GRAPHICS_VIEWPORT_MAX_Z, 0.8,
                           XVW_BACKGROUND_COLOR,        "black",
                           NULL);

	/* 
	 * create plot, and:
	 * 1 - set plot size; set plot points to generated data in coord array;
         *     set the number of points in each row to be plotted
	 * 2 - set plot type
	 * 3 - set foreground color to magenta.
	 */
	plot = xvw_create_plot3d(area, "plot");
	xvw_set_attributes(plot,
		           XVW_PLOT3D_PLOTSIZE,    POINTNUM*CURVENUM,
		           XVW_PLOT3D_POINTS,      coords,
		           XVW_PLOT3D_PLOTWIDTH,   POINTNUM,
		           XVW_PLOT3D_PLOTTYPE,    KPLOT3D_MESH,
		           XVW_FOREGROUND_COLOR,   "magenta",
		           NULL);

	/* bring up menuform automatically 
	xvw_activate_menu(plot); */

	/* display & run */
	xvf_run_form();
}
