#include <xvinclude.h>

/*
 *  This example creates a 3D plot from a generated dataset stored in a
 *  Coord array.  To provide color for the data set, the colormap is borrowed
 *  from the "image:mandril" image, and set using the XVW_COLOR_COLOROBJ
 *  attribute.  Note that the "d" value of each Coord must be set to the
 *  pixel value that will be used to index into the colormap.
 *
 *  Since the XVW_PLOT3D_COLOR_ORIGINATION attribute does not apply when
 *  the plot input is provided as a Coord array, attempting to use the menuform
 *  to set "Color Origination" to "Use Data Values" will result in an error
 *  message.
 */

#define POINTNUM 50

void main(
   int  argc,
   char *argv[])
{
	xvobject plot, area;
	kobject  color_object;
	char     *colorfile = "image:mandril";
        Coord    coords[POINTNUM*POINTNUM];
        int      i, j;


        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* initialize the xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
	   kerror(NULL, "main", "Cannot open display");
	   kexit(KEXIT_FAILURE);
        }
	
	/* open the image:mandril file, from which we'll use the colormap */
	color_object = kpds_open_input_object(colorfile);

	/*
	 * Generate a data set to put in the Coord array 
	 * Note that "d" values must be set to index into the colormap
	 * provided by the "image:mandril" file.
	 */
        for (i = 0; i < POINTNUM; i++)
           for (j = 0; j < POINTNUM; j++)
           {
              coords[i+j*POINTNUM].x = i;
              coords[i+j*POINTNUM].y = j;
              coords[i+j*POINTNUM].z = 0.1 * sin((double)
                                            kdegrees_radians(i * 360/POINTNUM));
              coords[i+j*POINTNUM].d = i;
           }

	/* create a black 500x300 area */
        area = xvw_create_area(NULL, "area");
	xvw_set_attributes(area,
                           XVW_AREA_DISPLAY_TITLE,      FALSE,
                           XVW_WIDTH,                   450,
                           XVW_HEIGHT,                  450,
                           XVW_GRAPHICS_VIEWPORT_MIN_X, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Y, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MIN_Z, 0.2,
                           XVW_GRAPHICS_VIEWPORT_MAX_X, 0.8,
                           XVW_GRAPHICS_VIEWPORT_MAX_Y, 0.8,
                           XVW_GRAPHICS_VIEWPORT_MAX_Z, 0.8,
			   XVW_BACKGROUND_COLOR, "black", 
			   NULL);

	/* 
	 * create plot, and:
	 * 1 - set plot size, plot coordinate array, and # points in each row
	 * 2 - set color object so that the mandril file specifies the colormap;
	 *     note that the colormap will be undefined unless this is done.
	 */
	plot = xvw_create_plot3d(area, "plot");
	xvw_set_attributes(plot,
                           XVW_PLOT3D_PLOTSIZE,    POINTNUM*POINTNUM,
                           XVW_PLOT3D_POINTS,      coords,
                           XVW_PLOT3D_PLOTWIDTH,   POINTNUM,
		           XVW_COLOR_COLOROBJ,     color_object,
		           NULL);

	/* automatically display menuform */
	xvw_activate_menu(plot);

	/* display & run */
	xvf_run_form();
}
