#include "xvinclude.h"

/*
 * This program puts up an image object displaying the entire image; it
 * also creates a smaller image object to the lower right in which a region 
 * of interest can be displayed.
 *
 * A button on the lower left allows you to set the shape of the ROI to
 * be extracted.  Set the desired shape and move the cursor into the image
 * to extract the specified type of ROI.  The extracted ROI will be displayed
 * in the image object to the lower right.
 */
static void change_shape PROTO((xvobject, kaddr, kaddr));
static void  get_region  PROTO((xvobject, kaddr, XEvent *));

xvobject image;
void main(
   int  argc,
   char *argv[])
{
        xvobject manager, roi, label, button;
        kobject image_object;


        /* initialize xvwidgets lib */
        if (!xvw_initialize(XVW_MENUS_XVFORMS))
        {
           kerror(NULL, "main", "unable to open display");
           kexit(1);
        }

        /* create the xvobject to display the image */
        image_object = kpds_open_object("image:ball", KOBJ_READ);
        kcolor_set_attribute(image_object, KCOLOR_MAP_AUTOCOLOR, KRGB_SPIRAL);

	/* create parent */
	manager = xvw_create_manager(NULL, "parent");

	/* create image object to display image */
        image = xvw_create_image(manager, "image");
        xvw_set_attribute(image, XVW_IMAGE_IMAGEOBJ, image_object);

	/* create image object to display extracted ROI */
	roi = xvw_create_image(manager, "roi");
	xvw_set_attributes(roi,
			   XVW_BELOW,   image,
			   XVW_LEFT_OF, NULL,
			   NULL);

	/* create label and button to change shape of extracted ROI */
	label  = xvw_create_label(manager, "label");
	xvw_set_attributes(label,
			   XVW_BELOW,    image,
			   XVW_RIGHT_OF, NULL,
			   XVW_LABEL,   "ROI Shape:",
			   NULL);
	button = xvw_create_button(manager, "button");
	xvw_set_attributes(button,
			   XVW_BELOW,    label,
			   XVW_RIGHT_OF, NULL,
			   XVW_LABEL,   "Rectangle",
			   NULL);
	xvw_add_callback(button, XVW_BUTTON_SELECT,
			 change_shape, image);

        /* add an event handler to start the region of interest extraction */
        xvw_add_event(image, ButtonPressMask, get_region, (kaddr) roi);

	/* display and run */
        xvf_run_form();


}

/*
 *  event handler to get region of interest and display the
 *  region of interest in the smaller image visual object.
 */
static void  get_region(
    xvobject object,
    kaddr    client_data,
    XEvent   *event)
{
        kobject  iobject = NULL;
        xvobject roi = (xvobject) client_data;

        /*
         * simply getting the XVW_IMAGE_ROI attribute causes all
         * the work to be done automatically;  the user is prompted to
         * rubberband the area of interest, the rubberbanding is performed,
         * and a data object containing the extracted region of interest
         * is returned.
         */
        xvw_get_attribute(image, XVW_IMAGE_ROI, &iobject);
        if (iobject == NULL) return;

        /*
         * region of interest successfully extracted; update image object
         */
        xvw_set_attribute(roi, XVW_IMAGE_IMAGEOBJ, iobject);
        kpds_close_object(iobject);
}

static void change_shape(
   xvobject button,
   kaddr    client_data,
   kaddr    call_data)
{
	char temp[KLENGTH];
        static int shape = KIMAGE_ROI_RECTANGLE;
	xvobject image_object = (xvobject) client_data;

	shape++;
	if (shape > KIMAGE_ROI_FREEHAND) 
	    shape = KIMAGE_ROI_RECTANGLE;

	if (shape == KIMAGE_ROI_RECTANGLE)
	    ksprintf(temp, "Rectangle");
	else if (shape == KIMAGE_ROI_POLYLINE)
	    ksprintf(temp, "Polyline");
	else if (shape == KIMAGE_ROI_CIRCLE)
	    ksprintf(temp, "Circle");
	else if (shape == KIMAGE_ROI_ELLIPSE)
	    ksprintf(temp, "Ellipse");
	else if (shape == KIMAGE_ROI_LINE)
	    ksprintf(temp, "Line");
	else if (shape == KIMAGE_ROI_FREEHAND)
	    ksprintf(temp, "Freehand");

	xvw_set_attribute(image_object,  XVW_IMAGE_ROI_SHAPE, shape);
	xvw_set_attribute(button, XVW_LABEL, temp);
}
