#include <xvinclude.h>

/*
 * This program creates an image object which displays the ball image.
 * Then, a zoom object is created, and is associated with the same information
 * as the image object.  Without installing any event handlers, the zoom 
 * object will automatically zoom in on the image displayed in the image object
 * when the pointer is moved across the image, since they are both using the 
 * same data object.
 */

void main(
   int  argc,
   char *argv[])
{
	kobject  object;
	char     *filename = "image:ball";
	xvobject image, zoom, position, parent;

        /* initialize Khoros program */
        khoros_initialize(argc, argv, "ENVISION");

	/* allow different images to be used, as in "% example image:lizard" */
	if (argc > 1)
	   filename = argv[1];

	/* open the data object associated with the input image */
	object = kpds_open_input_object(filename);

	/* initialize the xvwidgets lib */
	if (!xvw_initialize(XVW_MENUS_XVFORMS))
	{
	   kerror(NULL, "main", "unable to open display");
	   kexit(KEXIT_FAILURE);
	}

	/*
	 * a manager will be the backplane for the image and position objects
	 */
	parent = xvw_create_manager(NULL, "parent");

	/*
	 *  create the image object; associate it with the data object
         *  representing the input file
	 */
	image  = xvw_create_image(parent, "image");
	xvw_set_attribute(image, XVW_IMAGE_IMAGEOBJ, object);

	/*
	 *  create the position object under the image object; have it
         *  use the same data object.
	 */
	position = xvw_create_position(parent, "position");
	xvw_set_attributes(position,
			   XVW_BELOW,           image,
			   XVW_POSITION_OBJECT, object,
			   XVW_LEFT_OF,         NULL,
			   XVW_RIGHT_OF,        NULL,
			   NULL);

	/*
	 *  create the zoom object in an independant window. 
         *  specify the associated image object to be the same data.
	 */
	zoom = xvw_create_zoom(NULL, "zoom");
	xvw_set_attributes(zoom, 
			   XVW_IMAGE_IMAGEOBJ,      object,
			   XVW_ZOOM_FACTOR,         5.0,
			   XVW_ZOOM_LOCATIONMARKER, KZOOM_LM_CROSS,
			   XVW_MINIMUM_WIDTH,       200,
			   XVW_MINIMUM_HEIGHT,      200,
			   NULL);


	/* display and run */
	xvf_run_form();
}
