/*
  * Khoros: $Id: callbacks.c,v 1.3 1996/06/12 23:55:29 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: callbacks.c,v 1.3 1996/06/12 23:55:29 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>         Callbacks & handler routines for putdata
   >>>>
   >>>>  Private:
   >>>>          quit_handler
   >>>>          quit_callback
   >>>>          help_callback
   >>>>          menu_callback
   >>>>          detectfile_handler
   >>>>          image_update_callback
   >>>>          update_putdata_displays
   >>>>   Static:
   >>>>   Public:
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "putdata.h"

/*-----------------------------------------------------------
|
|  Routine Name: quit_handler()
|
|       Purpose: Action handler quits when user hits 'Q' or 'q',
|                or doubleclicks in window
|
|         Input: object      - the image, plot, or other object
|                client_data - not used
|                event       - the Keypress event
|        Output: None
|       Returns: None
|    Written By: Danielle Argiro
|          Date: Nov 3, 1993
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void quit_handler(
   xvobject object,
   kaddr    client_data,
   XEvent   *event)
{
        xvw_destroy(object);
        kexit(KEXIT_SUCCESS);
}

/*-----------------------------------------------------------
|
|  Routine Name: quit_callback()
|
|       Purpose: Callback quits when user hits quit button
|
|         Input: object      - the image, plot, or other object
|                client_data - not used
|                call_data   - not used
|        Output: None
|       Returns: None
|    Written By: Danielle Argiro
|          Date: March 6, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void quit_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
        xvw_destroy(xvw_parent(object));
        kexit(KEXIT_SUCCESS);
}



/*-----------------------------------------------------------
|
|  Routine Name: help_callback()
|
|       Purpose: Callback displays appropriate help page
|		 according to the type of visual display being used
|
|         Input: object      - the quit button
|                client_data - not used
|                call_data   - not used
|        Output: None
|       Returns: None
|    Written By: Danielle Argiro
|          Date: March 7, 1995
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void help_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	static xvobject help = NULL;
	char *filename = "$ENVISION/objects/xvroutine/putdata/help/putdata.hlp";

        if (help == NULL)
        {
            help = xvw_create_help(NULL, "Online Help");
            xvw_set_attributes(help,
                               XVW_HELP_TITLE,          "Online Help",
                               XVW_HELP_FILENAME,        filename,
                               XVW_HELP_DESTROY_ON_QUIT, FALSE,
                               NULL);
        }
        else if (!xvw_check_mapped(help))
             xvw_map(help);
        else xvw_raise(help);
}


/*-----------------------------------------------------------
|
|  Routine Name: menu_callback()
|
|       Purpose: Menu callback handler to invoke the current object's
|		 internal menus.
|
|         Input: object      - the menu object
|                client_data - not used
|                call_data   - the image, plot, or other object
|        Output: None
|       Returns: None
|    Written By: Mark Young
|          Date: Aug 19, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
void menu_callback(
    xvobject menu,
    kaddr    client_data,
    kaddr    call_data)
{
	xvobject object = (xvobject) client_data;

	xvw_activate_menu(object);
}

/*-----------------------------------------------------------
|
|  Routine Name: detectfile_handler
|
|       Purpose: This is the input handler that is called when
|                the input file containing the data is modified
|		 while putdata is running.  The update_putdata_displays()
|		 utility routine does the actual work of updating
|		 the appropriate displays correctly.
|
|         Input: dummy       - not used
|		 filename    - name of file w/ data object being displayed
|		 client_data - the visual object being used
|        Output: none
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 22, 1994
| Modifications:
|
------------------------------------------------------------*/
/* ARGSUSED */
int detectfile_handler(
   xvobject dummy,
   char     *filename,
   kaddr    client_data)
{
	xvobject object = (xvobject) client_data;
        kobject data_object;
 
        if (kaccess(filename, R_OK) == -1)
           return(TRUE);

	/*
	 * get data object from the modified file
	 */
        if ((data_object = kpds_open_object(filename, KOBJ_READ)) != NULL)
        {
	    update_putdata_displays(object, data_object);
	    kpds_close_object(data_object);
	}
        return(TRUE);
}

/*-----------------------------------------------------------
|
|  Routine Name: image_update_callback
|
|       Purpose: This is the callback that is called when
|                the image object's menuform is used to input
|		 the name of a new image.  The update_putdata_displays()
|                utility routine does the actual work of updating
|                the appropriate displays correctly.  Note that
|		 this is of course only useful when an image is
|		 being displayed (putimage).
|
|         Input: object      - the image object
|                client_data - 
|                client_data - passes in new data object
|        Output: 
|    Written By: Danielle Argiro
|          Date: April 11, 1996
| Modifications:
|
------------------------------------------------------------*/
void image_update_callback(
   xvobject object,
   kaddr    client_data,
   kaddr    call_data)
{
	char *filename = (char *) call_data;
	kobject data_object;

	/*
         * get data object from the modified file
         */
        if ((data_object = kpds_open_object(filename, KOBJ_READ)) != NULL)
        {
	    update_putdata_displays(object, data_object);
            kpds_close_object(data_object);
        }
}

/*-----------------------------------------------------------
|
|  Routine Name: update_putdata_displays
|
|       Purpose: This utility routine updates the various putdata
|		 displays when the input data object is updated.
|
|		 It is used both by the detectfile_handler() input handler
|		 and the image_update_callback() callback.
|
|                Depending on the type of visual object being
|                used (image, imageicon, zoom, animate, 2D plot,
|                3D plot, palette, printpixel, or printmapval),
|                the relevant visual object is updated using the
|                appropriate attribute.
|
|         Input: visual_object - the visual object being used by putdata
|		 data_object   - the new data object
|		 update_image  - TRUE to update image object (you want to
|	                         for the detectfile_handler(), but not for
|		                 the image_update_callback().
|        Output:
|    Written By: Mark Young & Danielle Argiro
|          Date: Aug 22, 1994
| Modifications:
|
------------------------------------------------------------*/

void update_putdata_displays(
	xvobject visual_object,
	kobject  data_object)
{
	/* update displayed imageicon object*/
	if (clui_info->imageicon_flag)
           xvw_set_attribute(visual_object,  XVW_IMAGE_IMAGEOBJ, data_object);

	/* update displayed zoom or animate object  */
	else if ((clui_info->zoom_flag) || (clui_info->animate_flag))
	{
           xvw_set_attribute(visual_object,  XVW_IMAGE_IMAGEOBJ, data_object);
	   if (position != NULL)
               xvw_set_attribute(position, XVW_POSITION_OBJECT, data_object);
	}

	/* update displayed image */
        else if (clui_info->image_flag) 
	{
           xvw_set_attribute(visual_object,   XVW_IMAGE_IMAGEOBJ, data_object);
	   if (position != NULL)
               xvw_set_attribute(position, XVW_POSITION_OBJECT, data_object);
	
	   if (panicon != NULL)
               xvw_set_attribute(panicon,  XVW_IMAGE_IMAGEOBJ, data_object);
	}

	/* update displayed 2D plot */
	else if (clui_info->plot2d_flag)
           xvw_set_attribute(visual_object, XVW_PLOT2D_PLOTOBJ, data_object);

	/* update displayed 3D plot */
	else if (clui_info->plot3d_flag)
           xvw_set_attribute(visual_object, XVW_PLOT3D_PLOTOBJ, data_object);

	/* update displayed palette */
	else if (clui_info->palette_flag)
           xvw_set_attribute(visual_object, XVW_COLOR_COLOROBJ, data_object);

	/* update displayed printpixel */
	else if (clui_info->pixel_flag)
           xvw_set_attribute(visual_object, XVW_PRINTPIXEL_OBJECT, data_object);

	/* update displayed printmapval */
	else if (clui_info->mapval_flag)
           xvw_set_attribute(visual_object, XVW_PRINTMAPVAL_OBJECT,data_object);

}

