/*
  * Khoros: $Id: do_disp.c,v 1.11 1996/07/04 19:21:48 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: do_disp.c,v 1.11 1996/07/04 19:21:48 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Functionality routines for pane disp
   >>>> 
   >>>>  Private: 
   >>>> 	disp_red
   >>>> 	disp_green
   >>>> 	disp_blue
   >>>> 	disp_red_func
   >>>> 	disp_green_func
   >>>> 	disp_blue_func
   >>>> 	disp_file_view
   >>>> 	disp_change_mapcol_names
   >>>> 	disp_norm_type
   >>>> 	disp_norm_method
   >>>> 	disp_norm_policy
   >>>> 	disp_t
   >>>> 	disp_alloc
   >>>>   Static: 
   >>>> 	change_mapcol
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */



#include "spectrum.h"

#define RED_MAPCOL   0
#define GREEN_MAPCOL 1
#define BLUE_MAPCOL  2

static int change_mapcol PROTO((int));


/*-----------------------------------------------------------
| 
|  Routine Name: disp_red
| 
|       Purpose: Do routine which is called when
|                  pane action button red is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_red(
     Disp_disp *disp_info)
{
	int          new_col;
	char         temp[KLENGTH];
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	if (!(change_mapcol(RED_MAPCOL)))
	{
	    spc_disp_working = FALSE;
	    return;
	}

	xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_RED_MAPCOL, &new_col);

	if (new_col != -1)
	{
	    ksprintf(temp, "%s", spc_mapcol_names[new_col]);
	    xvf_set_attribute(disp_info->redcol_struct, XVF_TITLE, temp);
	    ksprintf(temp, "M%d", new_col);
            xvf_set_attribute(disp_info->red_func_struct, XVF_STRING_VAL, temp);
	    kfree_and_NULL(disp_info->red_func);
            disp_info->red_func = kstrdup(temp);
	}
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");

	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	

	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_green
| 
|       Purpose: Do routine which is called when
|                  pane action button green is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_green(
     Disp_disp *disp_info)
{
	int          new_col;
	char         temp[KLENGTH];
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	if (!(change_mapcol(GREEN_MAPCOL)))
	{
	    spc_disp_working = FALSE;
	    return;
	}

	xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_GREEN_MAPCOL, &new_col);

	if (new_col != -1)
	{
	    ksprintf(temp, "%s", spc_mapcol_names[new_col]);
	    xvf_set_attribute(disp_info->grcol_struct, XVF_TITLE, temp);
	    ksprintf(temp, "M%d", new_col);
            xvf_set_attribute(disp_info->green_func_struct, XVF_STRING_VAL, temp);
	    kfree_and_NULL(disp_info->green_func);
            disp_info->green_func = kstrdup(temp);
	}
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");

	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	
	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_blue
| 
|       Purpose: Do routine which is called when
|                  pane action button blue is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_blue(
     Disp_disp *disp_info)
{
	int          new_col;
	char         temp[KLENGTH];
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	if (!(change_mapcol(BLUE_MAPCOL)))
	{
	    spc_disp_working = FALSE;
            return;
	}

	xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_BLUE_MAPCOL, &new_col);

        if (new_col != -1)
        {
	    ksprintf(temp, "%s", spc_mapcol_names[new_col]);
	    xvf_set_attribute(disp_info->blcol_struct, XVF_TITLE, temp);
	    ksprintf(temp, "M%d", new_col);
            xvf_set_attribute(disp_info->blue_func_struct, XVF_STRING_VAL, temp);
            kfree_and_NULL(disp_info->blue_func);
            disp_info->blue_func = kstrdup(temp);
	}
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");

	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	
	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_red_func
| 
|       Purpose: Do routine which is called when
|                  string selection red_func is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_red_func(
     Disp_disp *disp_info)
{
	int          mapcol = -1;
	char        *equation = NULL;
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	/* sanity check */
	if (disp_info->red_func == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "disp_red_func", "Cannot apply NULL equation");
	    spc_disp_working = FALSE;
	    return;
	}

	/*
	 *  see if the name of a file containing a function was provided.
         *  if so, read the function out of the file; otherwise, take
         *  the function directly from the parameter box.  Update the
         *  map column displayed as Red according to that function. 
	 */
	equation = spc_read_eqn_string_from_file(disp_info->red_func);
	if (equation == NULL)
	    equation = disp_info->red_func;
	
	/*
	 *  save red function as spc_display->red_func
	 */
	kfree_and_NULL(spc_display->red_func);
	spc_display->red_func = kstring_copy(disp_info->red_func, NULL);

	/*
	 *  the image object, by virtue of the fact that it is subclassed
         *  off the Color object, handles the application of the function
	 *  to the Red column.
	 */
	xvw_set_attribute(spc_display->workspace,
			  XVW_COLOR_RED_FUNCTION, spc_display->red_func);
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");
	
	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	
	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();

	/*
	 *  if the red function was simply a map column (not an equation)
	 *  update title of red column using map column names; else use eqn
	 */
	xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_RED_MAPCOL, &mapcol);

	/* function was an equation */
	if (mapcol == -1)
            xvf_set_attribute(disp_info->redcol_struct, XVF_TITLE,
                              disp_info->red_func);

	/* function was specification of a single map column */
        else if (mapcol < spc_map_colnum)
                xvf_set_attribute(disp_info->redcol_struct, XVF_TITLE,
                                  spc_mapcol_names[mapcol]);

	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_green_func
| 
|       Purpose: Do routine which is called when
|                  string selection green_func is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_green_func(
     Disp_disp *disp_info)
{
	int          mapcol = -1;
        char        *equation = NULL;
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

        /* sanity check */
        if (disp_info->green_func == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "disp_green_func", "Cannot apply NULL equation");
	    spc_disp_working = FALSE;
            return;
        }

        /*
         *  see if the name of a file containing a function was provided.
         *  if so, read the function out of the file; otherwise, take
         *  the function directly from the parameter box.  Update the
         *  map column displayed as Green according to that function.
         */
        equation = spc_read_eqn_string_from_file(disp_info->green_func);
        if (equation == NULL)
            equation = disp_info->green_func;
        
        /*
         *  save green function as spc_display->green_func
         */
        kfree_and_NULL(spc_display->green_func);
        spc_display->green_func = kstring_copy(disp_info->green_func, NULL);

        /*
         *  the image object, by virtue of the fact that it is subclassed
         *  off the Color object, handles the application of the function
         *  to the Green column.
         */
        xvw_set_attribute(spc_display->workspace,
                          XVW_COLOR_GREEN_FUNCTION, spc_display->green_func);
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");
        
	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}

	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();

        /*
         *  if the green function was simply a map column (not an equation)
         *  update title of green column using map column names; else use eqn
         */
        xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_GREEN_MAPCOL, &mapcol);

        /* function was an equation */
        if (mapcol == -1)
            xvf_set_attribute(disp_info->grcol_struct, XVF_TITLE,
                              disp_info->green_func);

        /* function was specification of a single map column */
        else if (mapcol < spc_map_colnum)
		xvf_set_attribute(disp_info->grcol_struct, XVF_TITLE,
                                  spc_mapcol_names[mapcol]);
	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_blue_func
| 
|       Purpose: Do routine which is called when
|                  string selection blue_func is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
|        Output: None
|    Written By: Danielle Argiro
|          Date: June 23, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_blue_func(
     Disp_disp *disp_info)
{
	int          mapcol = -1;
        char        *equation = NULL;
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

        /* sanity check */
        if (disp_info->blue_func == NULL)
        {
	    errno = SPC_USER_ERROR;
            kerror(NULL, "disp_blue_func", "Cannot apply NULL equation");
	    spc_disp_working = FALSE;
            return;
        }

        /*
         *  see if the name of a file containing a function was provided.
         *  if so, read the function out of the file; otherwise, take
         *  the function directly from the parameter box.  Update the
         *  map column displayed as Blue according to that function.
         */
        equation = spc_read_eqn_string_from_file(disp_info->blue_func);
        if (equation == NULL)
            equation = disp_info->blue_func;
        
        /*
         *  save blue function as spc_display->blue_func
         */
        kfree_and_NULL(spc_display->blue_func);
        spc_display->blue_func = kstring_copy(disp_info->blue_func, NULL);

        /*
         *  the image object, by virtue of the fact that it is subclassed
         *  off the Color object, handles the application of the function
         *  to the Blue column.
         */
        xvw_set_attribute(spc_display->workspace,
                          XVW_COLOR_BLUE_FUNCTION, spc_display->blue_func);
	xvf_set_attribute(disp_info->curr_trans_struct, XVF_TITLE, "none");


	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}

	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
       
        /*
         *  if the blue function was simply a map column (not an equation)
         *  update title of blue column using map column names; else use eqn
         */
        xvw_get_attribute(spc_display->workspace,
                          XVW_COLOR_BLUE_MAPCOL, &mapcol);

        /* function was an equation */
        if (mapcol == -1)
            xvf_set_attribute(disp_info->blcol_struct, XVF_TITLE,
                              disp_info->blue_func);

        /* function was specification of a single map column */
        else if (mapcol < spc_map_colnum)
             xvf_set_attribute(disp_info->blcol_struct, XVF_TITLE,
                                  spc_mapcol_names[mapcol]);

	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_change_mapcol_names
| 
|       Purpose: Do routine which is called when
|                  pane action button change_mapcol_names is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: Danielle Argiro
|          Date: Sep 30, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_change_mapcol_names(
     Disp_disp *disp_info)
{
	char temp[KLENGTH];
	char **prompts, **answers;
	int  i, ok = FALSE;

	/* sanity check */
	if (spc_image == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "disp_change_mapcol_names",
	         "You must display an image before changing band names!");
	    kinfo(KHOSTILE, "Use your head, doorknob.");
	    return;
	}

	prompts = (char **) kcalloc((size_t) 1, spc_map_colnum * sizeof(char *));
        answers = (char **) kcalloc((size_t) 1, spc_map_colnum * sizeof(char *));

	while (!ok)
	{
	    /*
	     *  set up prompts w/ old map column names entered
	     */
	    for (i = 0; i < spc_map_colnum; i++)
	    {
	        ksprintf(temp, "Band %d", i+1);
	        prompts[i] = kstring_copy(temp, NULL);
	        answers[i] = (char *) kcalloc((size_t) 1, KLENGTH * sizeof(char));
	        ksprintf(answers[i], "%s", spc_mapcol_names[i]); 
	    }

	    /*
             *  prompt for new names
             */
	    (void) xvu_query_wait("Specify New Names for Bands",
                                  prompts, "ok", answers, spc_map_colnum, 25);


           /*
            *  check for invalid blank entries
            */
	    for (i = 0; i < spc_map_colnum; i++)
    	    {
	        kstring_cleanup(answers[i], temp);
	        if ((kstrlen(temp) > 0) && ((ok) || (i == 0)))
		    ok = TRUE;
	        else 
	        {
	            errno = SPC_USER_ERROR;
		    kerror(NULL, "disp_change_mapcol_names",
			   "NO Band Name may be left blank. Please try again.");
	            kinfo(KHOSTILE, "Jeez - you ask to change the names of bands, then leave them blank.  What a doofus.");
		    ok = FALSE;
		    break;
	        }
	    }
	}

	/*
         *  change names stored in spc_mapcol_names array
         */
	for (i = 0; i < spc_map_colnum; i++)
        {
	    kfree_and_NULL(spc_mapcol_names[i]);
	    spc_mapcol_names[i] = kstrdup(answers[i]);
	}

	spc_update_GUI_from_mapcol_names();

	karray_free(prompts, KSTRING, (size_t) spc_map_colnum, NULL);
	karray_free(answers, KSTRING, (size_t) spc_map_colnum, NULL);
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_list_mapcol_names
| 
|       Purpose: Do routine which is called when
|                  pane action button list_mapcol_names is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: 
|          Date: Sep 30, 1993
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_list_mapcol_names(
     Disp_disp *disp_info)
{
	int  i;
	char temp[KLENGTH];
	char namelist[3*KLENGTH];

	/* sanity check */
	if (spc_image == NULL)
	{
	    errno = SPC_USER_ERROR;
	    kerror(NULL, "disp_list_mapcol_names",
		"You must display an image before listing band names!");
	    kinfo(KHOSTILE, "Have you seen a doctor lately?");
	    return;
	}
	ksprintf(namelist, "Band 1 = '%s'\n", spc_mapcol_names[0]);
	
	for (i = 1; i < spc_map_colnum; i++)
	{
	    ksprintf(temp, "Band %d = '%s'\n", i+1, spc_mapcol_names[i]);
	    kstring_cat(namelist, temp, namelist);
	}
	kinfo(KFORCE, namelist);

}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_norm_type
| 
|       Purpose: Do routine which is called when
|                  cycle selection norm_type is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: 
|          Date: Jun 22, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_norm_type(
     Disp_disp *disp_info)
{
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	/* set image workspace to use specified normalization type */
	xvw_set_attribute(spc_display->workspace, XVW_COLOR_NORM_TYPE,
			  disp_info->norm_type);

	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	
	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
	spc_disp_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_norm_method
| 
|       Purpose: Do routine which is called when
|                  logical selection norm_method is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: 
|          Date: Jun 22, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_norm_method(
     Disp_disp *disp_info)
{
	LegendEntry *legend_ptr;

	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	/* set image workspace to use specified normalization method */
	xvw_set_attribute(spc_display->workspace, XVW_COLOR_NORM_METHOD,
			  disp_info->norm_method_val);

	/* save color "under" class color in case it needs to be restored */
	legend_ptr = spc_legend_list;
	while (legend_ptr != NULL)
	{
	    xvw_set_attribute(legend_ptr->colorbox,XVW_COLORCELL_SAVE_COLOR,1);
	    legend_ptr = legend_ptr->next;
	}
	
	/* refresh assigned clusters to appear in appropriate class color */
	spc_refresh_class_colors();
	spc_disp_working = FALSE;
}

/*-----------------------------------------------------------
|
|  Routine Name: change_mapcol
|
|       Purpose: This routine pops up a list of the available map columns,
|                and allows the user to choose one to display as red, 
|                green, or blue.  It then calls xvw_set_attribute
|                on the corresponding red, green, blue mapcol attribute.
|
|         Input: color - RED_MAPCOL, GREEN_MAPCOL, or BLUE_MAPCOL
|        Output:
|       Returns: TRUE (1) on success, FALSE (0) otherwise
|    Written By: Danielle Argiro
|          Date: Jun 24, 1994
| Modifications:
|
------------------------------------------------------------*/
static int change_mapcol(
    int color)
{
        char **list;
        char *prompt;
	char *mapcol_attribute;
        int  i, indx = -1;
        char temp[KLENGTH];

        if (color == RED_MAPCOL)
        {
            mapcol_attribute = XVW_COLOR_RED_MAPCOL;
            prompt = "Select band to use as RED";
        }
        else if (color == GREEN_MAPCOL)
        {
            mapcol_attribute = XVW_COLOR_GREEN_MAPCOL;
            prompt = "Select band to use as GREEN";
        }
        else
        {
            mapcol_attribute = XVW_COLOR_BLUE_MAPCOL;
            prompt = "Select band to use as BLUE";
        }

	/*
         *  Create the list of map column selections
         */
        list = (char **) kcalloc((size_t) 1, spc_map_colnum * sizeof(char *));
        for (i = 0; i < spc_map_colnum; i++)
        {
            ksprintf(temp, "%s", spc_mapcol_names[i]);
            list[i] = kstring_copy(temp, NULL);
        }

	/*
         *  Prompt the user for which one they want.
         */
        if (kchoose(KFORCE, list, spc_map_colnum, -1, NULL, &indx, prompt) 
		    == NULL)
            indx = -1;
        else indx--;

        karray_free(list, KSTRING, (size_t) spc_map_colnum, NULL);

	/*
         *  If the indx is not -1 (cancel), change the appropriate attribute
         */
        if (indx != -1)
            xvw_set_attribute(spc_display->workspace, mapcol_attribute, indx);
	else return(FALSE);

        return(TRUE);
}

/*-----------------------------------------------------------
| 
|  Routine Name: disp_t
| 
|       Purpose: Do routine which is called when the user clicks
|                on the Transformations action button
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: 
|          Date: Aug 08, 1994
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_t(
     Disp_disp *disp_info)
{
	int  i; 
	int  transform_num;
	char **transform_names;
	char  *red_func, *green_func, *blue_func;
	char  *return_string = NULL;

	static int transforms_working = FALSE;

	if (transforms_working)
	    return;

	transforms_working = TRUE;

	/*
	 * the transforms were stored in the spc_trans_info array earlier, 
         * when the display environment ascii file was read in;  get the
	 * list of transform names from the spc_trans_info array, and
	 * pop up a list from which the user can select a transform.
	 */
	transform_names = (char **) kcalloc((size_t) spc_trans_num, 
					     sizeof(char *));
	for (i = 0; i < spc_trans_num; i++)
	    transform_names[i] = kstrdup(spc_trans_info[i]->name);
 	transform_num = xvu_choose_wait("Choose the desired transform",
				     transform_names, (size_t) spc_trans_num,
				      -1, FALSE, &return_string);

	karray_free(transform_names, KSTRING, (size_t) spc_trans_num, NULL);
	if (return_string == NULL) return;

	/*
	 * set the red function according to the selected transform.
	 * all we need to do is fill in the value in the GUI, and assign the
	 * gui_info value, before calling the do routine for red functions.
	 */
	red_func = spc_trans_info[transform_num-1]->red_equation;
	xvf_set_attribute(disp_info->red_func_struct, XVF_STRING_VAL,
			  red_func);
	kfree_and_NULL(disp_info->red_func);
	disp_info->red_func = kstrdup(red_func);
	disp_red_func(disp_info);

	/*
	 * set the green function according to the selected transform.
	 * all we need to do is fill in the value in the GUI, and assign the
	 * gui_info value, before calling the do routine for green functions.
	 */
	green_func = spc_trans_info[transform_num-1]->green_equation;
	xvf_set_attribute(disp_info->green_func_struct, XVF_STRING_VAL, 
			  green_func);
	kfree_and_NULL(disp_info->green_func);
	disp_info->green_func = kstrdup(green_func);
	disp_green_func(disp_info);
	
	/*
	 * set the blue function according to the selected transform.
	 * all we need to do is fill in the value in the GUI, and assign the
	 * gui_info value, before calling the do routine for blue functions.
	 */
	blue_func = spc_trans_info[transform_num-1]->blue_equation;
	xvf_set_attribute(disp_info->blue_func_struct, XVF_STRING_VAL, 
			  blue_func);
	kfree_and_NULL(disp_info->blue_func);
	disp_info->blue_func = kstrdup(blue_func);
	disp_blue_func(disp_info);
	
	xvf_set_attribute(disp_info->curr_trans_struct, 
			  XVF_TITLE, return_string);

	transforms_working = FALSE;
}


/*-----------------------------------------------------------
| 
|  Routine Name: disp_alloc
| 
|       Purpose: Do routine which is called when
|                  cycle selection alloc is used
| 
|         Input: disp_info - ptr to PaneInfo struct for disp pane
| 
|        Output: None
|    Written By: 
|          Date: Oct 17, 1995
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void disp_alloc(
     Disp_disp *disp_info)
{
	if (spc_disp_working)
	    return;

	spc_disp_working = TRUE;

	xvw_set_attribute(spc_display->workspace, XVW_COLOR_ALLOC_POLICY,
                          disp_info->alloc);

	spc_disp_working = FALSE;
}


