
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 *
 * Fri Apr 26 21:06:34 PDT 1996 - copyright update.
 */
 


#include "internals.h"

int
lunion_boxes(vertex_3d *s1_min,
	     vertex_3d *s1_max,
	     vertex_3d *s2_min,
	     vertex_3d *s2_max,
	     vertex_3d *dst_min,
	     vertex_3d *dst_max)
{
    /**
      * do a "union" of the bounding boxes s1 and s2.  place
      * results in dst.  written so that dst can be the same as
      * either of s1 or s2.
    **/
    int i;
    for (i=0;i<3;i++)
    {
	dst_min->v[i] = MIN(s1_min->v[i],s2_min->v[i]);
	dst_max->v[i] = MAX(s1_max->v[i],s2_max->v[i]);
    }
    return(CHILL);
}

int
lcompute_bbox(vertex_3d *v,
	      int n,
	      vertex_3d *bmin,
	      vertex_3d *bmax)
{
    register KGEOM_VERTEX_TYPE *tv;
    register int i;
    vertex_3d min,max;

    for (i=0;i<3;i++)
    {
	min.v[i] = KGEOM_BIGNUM;
	max.v[i] = KGEOM_SMALLNUM;
    }

    /**
      * the following loop has been optimized for speed, not
      * readability.  sorry.
      *
      * the general idea is that we'll loop/stride through all
      * the x coords first, all the y coords next and finally
      * all the z coords.
    **/
    
    tv = &(v->v[0]);
    /* do x coords. */
    for (i=0;i<n;i++,tv+=3)  /* assume that a vertex_3d is composed of
				3 contiguous KGEOM_VERTEX_TYPE's, and just
				stride over them. */
    {
	if (*tv < min.v[0])
	    min.v[0] = *tv;
	if (*tv > max.v[0])
	    max.v[0] = *tv;
    }
    
    /* do y coords. */
    
    tv = &(v->v[1]);
    for (i=0;i<n;i++,tv+=3)
    {
	if (*tv < min.v[1])
	    min.v[1] = *tv;
	if (*tv > max.v[1])
	    max.v[1] = *tv;
    }
    
    /* do z coords. */
    tv = &(v->v[2]);
    for (i=0;i<n;i++,tv+=3)
    {
	if (*tv < min.v[2])
	    min.v[2] = *tv;
	if (*tv > max.v[2])
	    max.v[2] = *tv;
    }

    VCOPY(bmin,&min);
    VCOPY(bmax,&max);

    return(CHILL);
}

int
lcompute_bbox_ptrs(vertex_3d **v,
		   int n,
		   vertex_3d *bmin,
		   vertex_3d *bmax)
{
    register vertex_3d **ptv;
    register KGEOM_VERTEX_TYPE tv;
    register int i;
    vertex_3d min,max;

    for (i=0;i<3;i++)
    {
	min.v[i] = KGEOM_BIGNUM;
	max.v[i] = KGEOM_SMALLNUM;
    }

    /**
      * the following loop has been optimized for speed, not
      * readability.  sorry.
      *
      * the general idea is that we'll loop/stride through all
      * the x coords first, all the y coords next and finally
      * all the z coords.
    **/
    
    ptv = v;
    /* do x coords. */
    for (i=0;i<n;i++,ptv++)  /* assume that a vertex_3d is composed of
				3 contiguous KGEOM_VERTEX_TYPE's, and just
				stride over them. */
    {
	tv = (*ptv)->v[0];
	if (tv < min.v[0])
	    min.v[0] = tv;
	if (tv > max.v[0])
	    max.v[0] = tv;
    }
    
    /* do y coords. */
    
    ptv = v;
    for (i=0;i<n;i++,ptv++)
    {
	tv = (*ptv)->v[1];
	if (tv < min.v[1])
	    min.v[1] = tv;
	if (tv > max.v[1])
	    max.v[1] = tv;
    }
    
    /* do z coords. */
    ptv = v;
    for (i=0;i<n;i++,ptv++)
    {
	tv = (*ptv)->v[2];
	if (tv < min.v[2])
	    min.v[2] = tv;
	if (tv > max.v[2])
	    max.v[2] = tv;
    }

    VCOPY(bmin,&min);
    VCOPY(bmax,&max);

    return(CHILL);
}

int
lget_min_max_ptrs(vertex_3d **v,
		  int n,
		  vertex_3d **xmin,
		  vertex_3d **xmax,
		  vertex_3d **ymin,
		  vertex_3d **ymax)
{
    /**
      * this takes as input a list of ptrs to vertex_3d's.  it will
      * scan through the list, returning pointers to the vertex_3d
      * within the list that has minx, max, .etc.
    **/
    vertex_3d *vminx,*vmaxx,*vminy,*vmaxy,*p;
    int i,tsize;

    tsize = sizeof(vertex_3d);

    vminx = vmaxx = vminy = vmaxy = *v;

    for (i=1,p=*v;i<n;i++,p++)
    {
	if (p->v[0] < vminx->v[0])
	    vminx = p;
	if (p->v[0] > vmaxx->v[0])
	    vmaxx = p;
	if (p->v[1] < vminy->v[1])
	    vminy = p;
	if (p->v[1] > vmaxy->v[1])
	    vmaxy = p;
    }
    return(TRUE);
}

