 /*
  * Khoros: $Id: lgtransform.c,v 1.4 1996/07/05 02:49:46 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lgtransform.c,v 1.4 1996/07/05 02:49:46 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for gtransform
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lgtransform
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lgtransform - perform geometric transformation upon the vertex (and possibly normals) data in a geom object.
* 
*       Purpose: To facilitate arbitrary transformations of geometry.
*
*         Input: input - the input geometry object
*                scale - float[3] used for scaling factors for x,y, z
*                    axes, respectively
*                do_normals - permit or inhibit transformation of surface
*                    normals, if present.
*
*        Output: output - the transformed geometry object
*
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*
*  Restrictions: Restrictions on data or input as applicable
*                radii of spheres are not altered in this version of
*                code - only the vertex data.
*    Written By: wes
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 
****************************************************************/
/* -library_def */
int lgtransform(kobject input,
		kobject output,
		float *scale,
		int do_normals)
/* -library_def_end */

/* -library_code */
{
    int status, num_prims;
    int *prim_list=NULL;
    int has_alpha;
    int i;
    vertex_3d bmin,bmax,center,lbmin,lbmax;
    
    status = kgeom_query_primitive_list(input);
    if (status)
    {
        status = kgeom_get_attribute(input,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,&num_prims);
    }
    else
    {
	kerror("GEOMETRY","lktubeness", "Can't get KGEOM_NUMBER_PRIMITVES attributes from the input object.  Maybe it's not a GEOM file?");
	return(FALSE);
    }

    /* grab the list of primitives */
    status = kgeom_get_attribute(input,KGEOM_OBJECT,KGEOM_PRIMITIVE_LIST,&prim_list);
    if (status == FALSE)
    {
	kfprintf(output,"\tCan't get PRIM_LIST attribute.  Maybe not a GEOM file?\n");
	return (FALSE);
    }
    
    kgeom_copy_attributes(input,output);
    kgeom_create_primitive_list(output);

    for (i=0;i<3;i++)
    {
	bmin.v[i] = KGEOM_BIGNUM;
	bmax.v[i] = -1. * KGEOM_BIGNUM;
    }

    status = kgeom_get_attribute(input,KGEOM_OBJECT,KGEOM_HAS_ALPHA,&has_alpha);
    if (!status)
	kerror("GEOMETRY","lktubeness","Can't get the has_alpha attribute from the input object.");
    status = kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_HAS_ALPHA,has_alpha);
    /* this might not be right.. */
    status = kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_LAYOUT,KPER_VERTEX);
    
    for (i=0;i<num_prims;i++)
    {
        kgeom_set_attribute(input,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,i);
	kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_NUMBER_PRIMITIVES,i+1);
        kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_PRIMITIVE_POSITION,i);

	switch (prim_list[i])
	{

	case KGEOM_POLYLINE_DISJOINT:
	case KGEOM_POLYLINE_CONNECTED:
	{
	    float *verts=NULL,*colors=NULL;
	    vertex_3d *outverts;
	    int nverts;
	    matrix4x4 m;

	    status = kgeom_get_attribute(input,KGEOM_POLYLINE_DISJOINT,KGEOM_NUMBER_VERTICES,&nverts);
	    if (nverts == 0)
		break;
	    
	    kgeom_get_data(input,prim_list[i],&verts,&colors);
	    outverts = (vertex_3d *)kmalloc(sizeof(vertex_3d)*nverts);
	    
	    identity_4x4(&m);
	    m.m[0][0] = scale[0];
	    m.m[1][1] = scale[1];
	    m.m[2][2] = scale[2];

	    npoint_xfrm(verts,&m,(float *)outverts,nverts);
	    kgeom_set_attribute(output,prim_list[i],KGEOM_NUMBER_VERTICES,nverts);
	    kgeom_put_data(output,prim_list[i],(float *)outverts,colors);

	    lcompute_bbox(outverts,nverts,&lbmin,&lbmax);
	    lunion_boxes(&lbmin,&lbmax,&bmin,&bmax,&bmin,&bmax);

	    kfree(outverts);
#if 0
	    kfree(verts);
	    kfree(colors);
#endif
	}
	break;
	
        case KGEOM_TRIANGLES_CONNECTED:
        case KGEOM_TRIANGLES_DISJOINT:
	{
	    int nverts;
	    float *v=NULL,*c=NULL,*norm=NULL,*tc=NULL;
	    vertex_3d *outverts;
	    matrix4x4 m;
	    
	    kgeom_get_attribute(input,prim_list[i],KGEOM_NUMBER_VERTICES,&nverts);
	    if (nverts == 0)
		break;
	    
	    kgeom_get_data(input,prim_list[i],&v,&c,&norm,&tc);
	    outverts = (vertex_3d *)kmalloc(sizeof(vertex_3d)*nverts);

	    identity_4x4(&m);
	    m.m[0][0] = scale[0];
	    m.m[1][1] = scale[1];
	    m.m[2][2] = scale[2];

	    npoint_xfrm(v,&m,(float *)outverts,nverts);

	    if (do_normals)
		kerror("GEOMETRY","lgtransform","transformation of normals is not ready yet.");
	    kgeom_set_attribute(output,prim_list[i],KGEOM_NUMBER_VERTICES,nverts);
	    kgeom_put_data(output,prim_list[i],(float *)outverts,c,norm,tc);
	    
	    lcompute_bbox(outverts,nverts,&lbmin,&lbmax);
	    lunion_boxes(&lbmin,&lbmax,&bmin,&bmax,&bmin,&bmax);

	    kfree(outverts);

	    kfree(v);
	    kfree(c);
	    kfree(norm);
	    kfree(tc);

	}
	break;
	default:
	    lprim_copy(input,output,prim_list[i],&bmin,&bmax);
	    break;
	}
	
    }

    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_BOUNDING_BOX,&(bmin.v[0]),&(bmax.v[0]));

    for (i=0;i<3;i++)
	center.v[i] = bmin.v[i] + 0.5 * (bmax.v[i] - bmin.v[i]);
    kgeom_set_attribute(output,KGEOM_OBJECT,KGEOM_CENTER,&(center.v[0]));
    
    return TRUE;
}
/* -library_code_end */
