
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 
/*
Generic Convex Polygon Scan Conversion and Clipping
by Paul Heckbert
from "Graphics Gems", Academic Press, 1990
*/

/* poly.h: definitions for polygon package */
#define VTYPE float 
/* #define VTYPE double */

#ifndef POLY_HDR
#define POLY_HDR

#define POLY_NMAX 4	/* max #sides to a polygon; change if needed */

typedef struct {		/* A POLYGON VERTEX */
    VTYPE sx, sy, sz, sw; 	/* screen space position  */
							/* (sometimes homo.) */
    VTYPE x, y, z;		/* world space position */
    VTYPE u, v, w;		/* texture position */
						/* (sometimes homogeneous) */
    VTYPE r, g, b, a;		/* (red,green,blue,alpha) color */
    VTYPE nx, ny, nz;		/* world space normal vector */
} Poly_vert;
/* update poly.c if you change this structure */

typedef struct {		/* A POLYGON */
    int n;			/* number of sides */
    int mask;			/* interpolation mask for vertex elems */
    Poly_vert vert[POLY_NMAX];	/* vertices */
} Poly;
/*
 * mask is an interpolation mask whose kth bit indicates whether the kth
 * VTYPE in a Poly_vert is relevant.
 * For example, if the valid attributes are sx, sy, and sz, then set
 *	mask = POLY_MASK(sx) | POLY_MASK(sy) | POLY_MASK(sz);
 */

typedef struct {		/* A BOX (TYPICALLY IN SCREEN SPACE) */
    VTYPE x0, x1;		/* left and right */
    VTYPE y0, y1;		/* top and bottom */
    VTYPE z0, z1;		/* near and far */
} Poly_box;


typedef struct {		/* WINDOW: A DISCRETE 2-D RECTANGLE */
    int x0, y0;			/* xmin and ymin */
    int x1, y1;			/* xmax and ymax (inclusive) */
} PWindow;

#define POLY_MASK(elem) (1 << (&poly_dummy->elem - \
							 (VTYPE *)poly_dummy))

#define POLY_CLIP_OUT 0		/* polygon entirely outside box */
#define POLY_CLIP_PARTIAL 1	/* polygon partially inside */
#define POLY_CLIP_IN 2		/* polygon entirely inside box */

extern Poly_vert *poly_dummy;	/* used superficially by */
							/* POLY_MASK macro */

void	poly_print(kstring, Poly *);
void	poly_vert_label(kstring, int);
void	poly_vert_print(kstring, Poly_vert *, int);
int		poly_clip_to_box(/* p1, box */);
void	poly_clip_to_halfspace(/* p, q, index, sign, k, name */);
void	poly_scan PROTO((Poly *, PWindow *, void (*pixelproc)(int,int,Poly_vert *)));

/******* FIREWALL *******/

#include "shader.h"

#define NO_PRIM 0
#define POLYTRI_DIS 1
#define POLYTRI_CON 2
#define POLYLIN_DIS 3
#define POLYLIN_CON 4
#define QMESH 5
#define OMESH 6
#define PHEDRON 7
#define SPHERE 8
#define DIRPT 9
#define TEXT 10

int sphere_tesselator PROTO((int, float *, Poly *));

#endif


