
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#include <design.h>
#include <geometry.h>
#include "rmonster.h"
#include "poly.h"
#include "geometry/rmatrix.h"
#include "camera.h"
#include "imaging.h"
#include "sort_trans.h"
#include "zbuffer.h"
#include "rmonster_mouse.h"
#include "vrxfrm.h"

extern void (*software_primfuncs[])(rmonster_prim *, int, int, vertex_3d *, vertex_3d *, vertex_3d *, float *, int, int, rmonsterobj *, int, int);

static void render_2d_stuff PROTO((rmonsterobj *, matrix4x4 *, matrix4x4 *, int, int, int, vertex_3d *, int, int));
static void render_opaque_subtree PROTO((rmonsterobj *, matrix4x4 *, matrix4x4 *, int, int, int, vertex_3d *, int, int));
static void render_transparent_subtree PROTO((rmonsterobj *, matrix4x4 *, matrix4x4 *, int, int, int, vertex_3d *, int, int));
static int is_oqaque PROTO((rmonsterobj *));

int map_to_renderprims PROTO((int));

int
software_render_image(void)
{
    /* see also: gl_render_func in gl.c */
    rmonsterobj *r;
    matrix4x4 xfrm,view;
    vertex_3d trans_vec;
    int have_transparent_objs;
    int w,h;
    int projection;
    int hither_flag, yon_flag;
    int image_display_choice;
    int do_named_transport;

    kobject output_soft_kimage;

    hither_flag = 1;		/* front clipping always on */
    get_yon_clip_flag(&yon_flag);

    get_image_display_choice(&image_display_choice);

    if (image_display_choice != sNO_DISPLAY) 
       show_render_cursor();
    
    clear_framebuffer();

    get_image_dims(&w,&h);

    identity_4x4(&xfrm);
    get_view_xform(&view);
    reset_zbuffer();
    reset_abuffer();
    init_bucket_zbuffer(w,h);

    get_projection(&projection);
    r = get_root_object();

    have_transparent_objs = have_any_transparent_objs(r);

    trans_vec.v[0] = trans_vec.v[1] = trans_vec.v[2] = 0.;

    /* render the opaque objects first */
    render_opaque_subtree(r,&xfrm,&view,w,h,projection,&trans_vec,
			  hither_flag,yon_flag); 
    
    if (have_transparent_objs == TRUE)
    {
	render_transparent_subtree(r,&xfrm,&view,w,h,projection,&trans_vec,
				   hither_flag,yon_flag);
	composite_buckets();
    }

    /* note that the 2d stuff gets pasted on top of everything. */
    render_2d_stuff(r,&xfrm,&view,w,h,projection,&trans_vec,
		    hither_flag,yon_flag);

    if (image_display_choice != sNO_DISPLAY) 
       clear_render_cursor();
    flush_output();

    /* write out a file if the user asked for it. */
    get_do_named_output(&do_named_transport);
    if (do_named_transport)
    {
	char tname[512];

	get_output_transport_name(tname);
	output_soft_kimage = kpds_open_object(tname,KOBJ_WRITE);
	
	kcolor_set_attribute(output_soft_kimage,KCOLOR_COLORSPACE,KRGB);
	kcolor_set_attribute(output_soft_kimage,KCOLOR_HAS_ALPHA,FALSE);
	kpds_create_value(output_soft_kimage);
	kpds_set_attribute(output_soft_kimage,KPDS_VALUE_SIZE,w,h,1,1,3);
	kpds_set_attribute(output_soft_kimage,KPDS_VALUE_DATA_TYPE,KFLOAT);
	kpds_set_attribute(output_soft_kimage,KPDS_ELEMENTS_FIRST,TRUE);

	software_renderer_copyfb(output_soft_kimage);

	kpds_close_object(output_soft_kimage);
    }
    
    dump_xform_output();
    
    return(CHILL);
}

static void
render_opaque_subtree(rmonsterobj *r,
		      matrix4x4 *xfrm,
		      matrix4x4 *view,
		      int w,int h,
		      int projection,
		      vertex_3d *tv,
		      int hither_flag,
		      int yon_flag)
{
    int display_status,opaque_status;
    int nprims,i;
    matrix4x4 m;
    vertex_3d trans_vec;
    rmonster_prim *p;
    vertex_3d *v,*n2, *norm;
    vertex_3d *obj_space,*v2,*view_space;
    float *radii;
    int prim_type;
    float radius_xy_scale,radius_depth_scale;

    /* premultiply object transformation with the current xform matrix. */
/*    mmul_4x4(&(r->xfrm),xfrm,&m); */
    /* postmultiply object transformation with the current xform matrix. */
    /** transformation on the current level is composed of:
      
      1. -center
      2. scale*rotate
      3 +center+translate.
      
    **/
    mmul_4x4(xfrm,&(r->xfrm),&m);

    if ((r->nchildren == 0) && (r->verts_nspace == 3))
	/* a leaf node && 3d vertices. */
    {
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if ((display_status == TRUE) && (opaque_status == TRUE))
	{
	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;
		n2 = NULL;
		radii = NULL;

		/* create space for the vertices: one copy for the
		   object space (transformed) vertices, one for the
		   view space vertices. */
		
		v = p->verts;
		obj_space = v2 = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nverts);
		view_space = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nverts);

		if (p->type != KGEOM_SPHERES)
		{
		    norm = p->prim_specifics.normals;
		    if (norm != NULL)
			n2 = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nthings);
		}
		else
		{
		    radii = p->prim_specifics.radii;
		    if (radii)
		    {
			radii = (float *)kmalloc(sizeof(float)*p->nthings);
			memcpy((char *)radii,(char *)(p->prim_specifics.radii),
			       sizeof(float)*p->nthings);
		    }
		}

		/**
		  * apply the geometric transformations in object space
		**/
		if (r->verts_nspace != 2)
		{
		    geom_xform_points(&(r->xfrm),&(r->translate_vect),&(r->center),
				      v,obj_space,xfrm,tv,p->nverts);
		}
		else
		    kmemcpy(obj_space,v,(p->nverts * sizeof(vertex_3d)));
		
		/* transform the normals, if there are any. note: the
		   transformation of the normals does not involve any
		   translational component, so there is a specific
		   routine to deal with this transformation. */
		if (p->type != KGEOM_SPHERES)
		{
		    nnormal_xfrm(norm,&m,n2,p->nthings);
		    /* the normals are not run thru the view transformation
		       matrix since 1. this coordinate system is not
		       necessarily orthonormal to the world coord system,
		       and 2. lighting calculations are performed in the
		       world coord system. */
		    nnormal_unit(n2,n2,p->nthings);
		}
		else
		{
#if 0
		    /* the radii values must be scaled by an amount which
		       is the maximum of the scale components of the
		       object transformation matrix. */
		    work.v[0] = m.m[0][0] * view->m[0][0];
		    work.v[1] = m.m[1][1] * view->m[1][1];
		    work.v[2] = m.m[2][2] * view->m[2][2];
		    vertex_mag(&work,&mag);
		    fmag = mag;
#endif
		    get_world_to_view_scale(&radius_xy_scale,&radius_depth_scale);
		    scale_vector(radii,radius_xy_scale,radii,p->nthings,1);
		}

		/* now, do the viewing transformation */
		if (r->verts_nspace != 2)
		    
		    npoint_xfrm(&(obj_space->v[0]),view,&(view_space->v[0]),p->nverts);
		else
		{
		    /* for now, set all z's to 1.0.  this should be done
		     somewhere else.  */
		    int i;
			
		    kmemcpy(view_space,obj_space,(p->nverts * sizeof(vertex_3d)));
		    for (i=0;i<p->nverts;i++)
			view_space[i].v[2] = 1.0;
		}

		/* at this point, we have polygons with vertices
		   in the a normalized space. */

		prim_type = map_to_renderprims(p->type);
		(*software_primfuncs[prim_type])(p,w,h,obj_space,view_space,
						 n2,radii,projection,
						 opaque_status,r,
						 hither_flag,yon_flag);
		kfree(obj_space);
		kfree(view_space);
		if (radii)
		    kfree(radii);
		if (n2)
		    kfree(n2);
		    
	    }
	}
    }
    else
    {
	matrix4x4 minusC,SR,C,composite;
	
	for (i=0;i<3;i++)
	    trans_vec.v[i] = tv->v[i] + r->translate_vect.v[i];

	/**
	  * build composite transformation matrix:
	  * M = -CSRCT, where
	  *    C = object's center point,
	  *    SR = scale and rotate components.
	  *    T = translate vector.
	  *
	**/
	identity_4x4(&minusC);
	identity_4x4(&SR);
	identity_4x4(&C);
	
	identity_4x4(&composite);
	
	minusC.m[3][0] = -1. * r->center.v[0];
	minusC.m[3][1] = -1. * r->center.v[1];
	minusC.m[3][2] = -1. * r->center.v[2];
	
	matrix_4x4copy(&SR,&(r->xfrm));
	
	C.m[3][0] = r->center.v[0];
	C.m[3][1] = r->center.v[1];
	C.m[3][2] = r->center.v[2];
	
	mmul_4x4(&minusC,&SR,&composite);
	mmul_4x4(&composite,&C,&composite);

	mmul_4x4(xfrm,&composite,&m);

	for (i=0;i<r->nchildren;i++)
	{
	    render_opaque_subtree(r->children[i],&m,view,w,h,projection,
				  &trans_vec,hither_flag,yon_flag);
	}
    }
}

static void
render_2d_stuff(rmonsterobj *r,
		matrix4x4 *xfrm,
		matrix4x4 *view,
		int w,int h,
		int projection,
		vertex_3d *tv,
		int hither_flag,
		int yon_flag)
{
    int display_status,opaque_status;
    int nprims,i;
    matrix4x4 m;
    vertex_3d trans_vec;
    rmonster_prim *p;
    vertex_3d *v,*n2, *norm;
    vertex_3d *obj_space,*v2,*view_space;
    float *radii;
    int prim_type;
    float radius_xy_scale,radius_depth_scale;

    /* this routine is much like render_opaque_subtree, except that
       we ignore transformations.
       NOTE:  it is assumed that if verts_nspace == 2, that ALL primitives
       in that object have 2d vertices.  further, note that all primitives
       in all objects have, by definition (see object.c) 3d vertices.  the
       verts_nspace flag simply tells us to ignore the Z value. 
       */

    if ((r->nchildren == 0) && (r->verts_nspace == 2)) /* a leaf node */
    {
	
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if ((display_status == TRUE) && (opaque_status == TRUE))
	{
	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;
		n2 = NULL;
		radii = NULL;

		/* create space for the vertices: one copy for the
		   object space (transformed) vertices, one for the
		   view space vertices. */
		
		v = p->verts;

		/* do normals really matter for 2d objects?  leave this
		 in for now..it's used by the shaders */
		
		if (p->type != KGEOM_SPHERES)
		{
		    norm = p->prim_specifics.normals;
		}
#if 0
		/* 2D spheres??!? i don't think so... */
		
		else
		{
		    radii = p->prim_specifics.radii;
		    if (radii)
		    {
			radii = (float *)kmalloc(sizeof(float)*p->nthings);
			memcpy((char *)radii,(char *)(p->prim_specifics.radii),
			       sizeof(float)*p->nthings);
		    }
		}
#endif

		/* assume that we set all z's to 0.0, and tell the
		 scan converter to use KPROJECTION_PARALLEL (to avoid
		 the divide-by-zero. */
		
		prim_type = map_to_renderprims(p->type);
		obj_space = view_space = p->verts;
		{
		    /* for now, set all z's to 1.0.  this should be done
		     somewhere else.  */
		    int i;
			
		    kmemcpy(view_space,obj_space,(p->nverts * sizeof(vertex_3d)));
		    for (i=0;i<p->nverts;i++)
			view_space[i].v[2] = 0.0;
		}

		/* at this point, we have polygons with vertices
		   in the a normalized space. */

		(*software_primfuncs[prim_type])(p,w,h,obj_space,view_space,
						 norm,radii,
						 KPROJECTION_PARALLEL,
						 opaque_status,r,
						 hither_flag,yon_flag);
		if (radii)
		    kfree(radii);
	    }
	}
    }
    else
    {
	matrix4x4 minusC,SR,C,composite;
	
	for (i=0;i<3;i++)
	    trans_vec.v[i] = tv->v[i] + r->translate_vect.v[i];

	/**
	  * build composite transformation matrix:
	  * M = -CSRCT, where
	  *    C = object's center point,
	  *    SR = scale and rotate components.
	  *    T = translate vector.
	  *
	**/
	identity_4x4(&minusC);
	identity_4x4(&SR);
	identity_4x4(&C);
	
	identity_4x4(&composite);
#if 0	
	minusC.m[3][0] = -1. * r->center.v[0];
	minusC.m[3][1] = -1. * r->center.v[1];
	minusC.m[3][2] = -1. * r->center.v[2];
	
	matrix_4x4copy(&SR,&(r->xfrm));
	
	C.m[3][0] = r->center.v[0];
	C.m[3][1] = r->center.v[1];
	C.m[3][2] = r->center.v[2];
	
	mmul_4x4(&minusC,&SR,&composite);
	mmul_4x4(&composite,&C,&composite);
#endif
	mmul_4x4(xfrm,&composite,&m);

	for (i=0;i<r->nchildren;i++)
	{
	    render_2d_stuff(r->children[i],&m,view,w,h,projection,
			    &trans_vec,hither_flag,yon_flag);
	}
    }
}

static void
render_transparent_subtree(rmonsterobj *r,
			   matrix4x4 *xfrm,
			   matrix4x4 *view,
			   int w,
			   int h,
			   int projection,
			   vertex_3d *tv,
			   int hither_flag,
			   int yon_flag)
{
    int display_status,opaque_status;
    int nprims,i;
    matrix4x4 m;
    vertex_3d trans_vec;
    rmonster_prim *p;
    vertex_3d *v,*n2, *norm;
    vertex_3d *obj_space,*v2,*view_space;
    vertex_3d work;
    float *radii;
    double mag;
    float fmag;
    int prim_type;

    /* premultiply object transformation with the current xform matrix. */
/*    mmul_4x4(&(r->xfrm),xfrm,&m); */
    /* postmultiply object transformation with the current xform matrix. */
    mmul_4x4(xfrm,&(r->xfrm),&m);

    if ((r->nchildren == 0) && (r->verts_nspace == 3))
	/* a leaf node && 3d coordinates */
    {
	get_object_display_status(r,&display_status);
	opaque_status = is_opaque(r);
	if ((display_status == TRUE) && (opaque_status == FALSE))
	{
	    nprims = r->nprims;
	    for (i=0;i<nprims;i++)
	    {
		p = r->prims+i;
		n2 = NULL;
		radii = NULL;

		/* create space for the vertices: one copy for the
		   object space (transformed) vertices, one for the
		   view space vertices. */
		
		v = p->verts;
		obj_space = v2 = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nverts);
		view_space = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nverts);

		if (p->type != KGEOM_SPHERES)
		{
		    norm = p->prim_specifics.normals;
		    if (norm)
			n2 = (vertex_3d *)kmalloc(sizeof(vertex_3d)*p->nthings);
		}
		else
		{
		    radii = p->prim_specifics.radii;
		    if (radii)
		    {
			radii = (float *)kmalloc(sizeof(float)*p->nthings);
			memcpy((char *)radii,(char *)(p->prim_specifics.radii),
			       sizeof(float)*p->nthings);
		    }
		}

		/**
		  * apply the geometric transformations in object space
		**/
		geom_xform_points(&(r->xfrm),&(r->translate_vect),&(r->center),
				  v,obj_space,xfrm,tv,p->nverts);
		
		/* transform the normals, if there are any. note: the
		   transformation of the normals does not involve any
		   translational component, so there is a specific
		   routine to deal with this transformation. */
		if (p->type != KGEOM_SPHERES)
		{
		    nnormal_xfrm(norm,&m,n2,p->nthings);
		    /* the normals are not run thru the view transformation
		       matrix since 1. this coordinate system is not
		       necessarily orthonormal to the world coord system,
		       and 2. lighting calculations are performed in the
		       world coord system. */
		    nnormal_unit(n2,n2,p->nthings);
		}
		else
		{
		    /* the radii values must be scaled by an amount which
		       is the maximum of the scale components of the
		       object transformation matrix. */
		    work.v[0] = m.m[0][0] * view->m[0][0];
		    work.v[1] = m.m[0][1] * view->m[1][1];
		    work.v[2] = m.m[0][2] * view->m[2][2];
		    vertex_mag(&work,&mag);
		    fmag = mag;
		    scale_vector(radii,fmag,radii,p->nthings,1);
		}

		/* now, do the viewing transformation */
		npoint_xfrm(&(obj_space->v[0]),view,&(view_space->v[0]),p->nverts);

		/* at this point, we have polygons with vertices
		   in the a normalized space. */

		prim_type = map_to_renderprims(p->type);
		(*software_primfuncs[prim_type])(p,w,h,obj_space,view_space,
						 n2,radii,projection,
						 opaque_status,r,
						 hither_flag,yon_flag);
		kfree(obj_space);
		kfree(view_space);
		if (radii)
		    kfree(radii);
		if (n2)
		    kfree(n2);
		    
	    }
	}
    }
    else
    {
	matrix4x4 minusC,SR,C,composite;
	
	for (i=0;i<3;i++)
	    trans_vec.v[i] = tv->v[i] + r->translate_vect.v[i];

	/**
	  * build composite transformation matrix:
	  * M = -CSRCT, where
	  *    C = object's center point,
	  *    SR = scale and rotate components.
	  *    T = translate vector.
	  *
	**/
	identity_4x4(&minusC);
	identity_4x4(&SR);
	identity_4x4(&C);
	identity_4x4(&composite);

	minusC.m[3][0] = -1. * r->center.v[0];
	minusC.m[3][1] = -1. * r->center.v[1];
	minusC.m[3][2] = -1. * r->center.v[2];

	matrix_4x4copy(&SR,&(r->xfrm));
	
	C.m[3][0] = r->center.v[0];
	C.m[3][1] = r->center.v[1];
	C.m[3][2] = r->center.v[2];

	mmul_4x4(&minusC,&SR,&composite);
	mmul_4x4(&composite,&C,&composite);

	mmul_4x4(xfrm,&composite,&m);

	for (i=0;i<r->nchildren;i++)
	    render_transparent_subtree(r->children[i],&m,view,w,h,projection,
				       &trans_vec,hither_flag,yon_flag);
    }
}

int map_to_renderprims(int ktype)
{
    /* this routine is really a waste of time - we should get this
     stuff all set up when the object is read in, otherwise this
     routine just gets executed over and over again each time we
     invoke the renderer. */
    
    /* convert from khoros primitve type to one of the rmonster
       supported types. */
    int val;
    
    switch (ktype)
    {
    case KGEOM_POLYLINE_DISJOINT:
	val = POLYLIN_DIS;
	break;
    case KGEOM_POLYLINE_CONNECTED:
	val = POLYLIN_CON;
	break;
    case KGEOM_TRIANGLES_DISJOINT:
	val = POLYTRI_DIS;
	break;
    case KGEOM_TRIANGLES_CONNECTED:
	val = POLYTRI_CON;
	break;
    case KGEOM_SPHERES:
	val = SPHERE;
	break;
    case KGEOM_QUADMESH:
	val = QMESH;
	break;
    case KGEOM_OCTMESH:
	val = OMESH;
	break;
    case KGEOM_TEXT:
	val = TEXT;
	break;
	/* everything else isn't supported yet. */
    default:
	val = NO_PRIM;
	break;
    }
    return(val);
}

int
is_opaque(rmonsterobj *r)
{
    /* this is computed in the conversion from kobject to rmonsterobj
       routine.  it is based on a function of the object's rgba
       value (the alpha component), and the list of colors, which
       if present, is searched for non-unity opacity values. */
    
    if (r->is_transparent)
	return(FALSE);
    else
	return(TRUE);
}

int
have_any_transparent_objs (rmonsterobj *r)
{
    int i,rval;
    if (r->is_transparent)
	return(TRUE);
    else
    {
	for (i=0;i<r->nchildren;i++)
	{
	    rval = have_any_transparent_objs(r->children[i]);
	    if (rval == TRUE)
		return(rval);
	}
    }
    return(FALSE);
}
