
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 


#ifndef __shader_h
#define __shader_h

/* surface rendering modes */
#define NSHADERS 8
#define VOLUME_SHADER_BASE 5

#define KSHADER_NO_LIGHT 0
#define KSHADER_FLAT 1
#define KSHADER_GOUROUD 2
#define KSHADER_PHONG 3
#define KSHADER_INHERIT 4


/**
  * volume rendering modes:
  * these correspond to the list of functions in volrender.c
  *
  * VOXEL_DOT will add a zbucket entry at each voxel vertex.  this
  * is the quickest and dirtiest method, and may be useful for
  * composing a useful colormap.
  *
  * the SPLAT_HOMOGENOUS method will compute the 2d bounding box of
  * the voxel footprint on the screen, then for every pixel within this
  * bounding box, add a zbucket entry (for buffered transparency) of
  * constant color and opacity.  this method is slower than VOXEL_DOT,
  * and produces "blocky" but better images.
  *
  * SPLAT_HOMOGENOUS_GAUSS does the same thing as SPLAT_HOMOGENOUS, but
  * uses a 2d gaussian filter to attenuate the voxel opaticy across the
  * footprint using a spherical (assuming a square footprint) filter.
  * this method is slightly slower than SPLAT_HOMOGENOUS but produces
  * smoother images.  there are two parameters in the volrender.c file
  * which control this renderer which are currently compiled in.  one
  * parameter scales the width of the computed voxel footprint (making it
  * larger or smaller), while the other controls the spread of voxel
  * opacity within the footprint.  refer to the code for more information.
  * 
**/
#define KSHADER_VOXEL_DOT 5
#define KSHADER_SPLAT_HOMOGENEOUS 6
#define KSHADER_SPLAT_HOMOGENEOUS_GAUSS 7

/**
  * the following strings reflect the english version of the
  * shader used for reporting something intelligible to the user
  * via the GUI.
**/

static char *shader_strings[NSHADERS] =
{
    "No light",
    "Flat",
    "Gouroud",
    "Phong",
    "Inherit",
    "Voxel-Dot",
    "Voxel-Splat-Homogeneous",
    "Voxel-Splat-Gaussian"
};

/**
  *
**/

#include "rmonster_objs.h"

int vertex_shader PROTO((vertex_3d *verts, vertex_3d *normals, int nverts, \
			 surface_colors_rgba *in_fcolors, \
			 surface_colors_rgba **out_fcolors, \
			 surface_attribs *sa,\
			 float *global_alpha, \
			 int shader));

int tstrip_facet_shader PROTO((vertex_3d *verts, vertex_3d *normals, int nverts, \
			       surface_colors_rgba *in_fcolors, \
			       surface_colors_rgba **out_fcolors, \
			       surface_attribs *sa,\
			       float *global_alpha, \
			       int shader));

int vertex_per_face_normal_shader PROTO((vertex_3d *verts, vertex_3d *normals, int nverts, \
					 surface_colors_rgba *in_fcolors, \
					 surface_colors_rgba **out_fcolors, \
					 surface_attribs *sa,\
					 float *global_alpha, \
					 int shader));

int volume_vertex_shader PROTO((vertex_3d *verts, vertex_3d *normals, \
				int nverts, \
				surface_colors_rgba *in_fcolors, \
				surface_colors_rgba **out_fcolors, \
				surface_attribs *sa,\
				float *global_alpha, \
				int shader));

void nvertex_scaled_shade PROTO((surface_colors_rgba *out, \
				 float *scale, \
				 surface_colors_rgba *in, \
				 int nverts));

void nvertex_scaled_shade_per_face_norms PROTO((surface_colors_rgba *out, \
				 float *scale, \
				 surface_colors_rgba *in, \
				 int nverts));

void nvertex_scaled_shade_spec PROTO((surface_colors_rgba *out,float *scalar, \
				      surface_colors_rgba *in, \
				      int n,float *blend_coeff, \
				      vertex_3d *blend_color ));

void blend_vertex_shade PROTO((surface_colors_rgba *d,float *blend_coeff, \
			       vertex_3d *blend_color,int nverts));

#endif
