
/*
 * Copyright (c) 1996,  The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Visualization
 *      Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University of California, Berkeley nor of the 
 *    Lawrence Berkeley National Laboratory may be used to endorse or 
 *    promote products derived from this software without specific prior 
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This work is supported by the U. S. Department of Energy under contract 
 * number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
 * University of California.
 *
 *	Author: Wes Bethel
 *		Lawrence Berkeley National Laboratory
 *              Berkeley, California 
 *
 *  "this software is 100% hand-crafted by a human being in the USA"
 */
 

#include <design.h>

#include <geometry.h>
#include "sort_trans.h"
#include "zbuffer.h"

static float *zbuffer=NULL;
static int xsize=0,ysize=0;

#define ZINIT 1e10

int
init_zbuffer(int x,int y)
{
    register float *f;
    register int i;
    
    deinit_zbuffer();
    
    f = zbuffer = (float *)kmalloc(sizeof(float) * x * y);
    if (f == NULL)
    {
	kfprintf(kstderr,"ERROR: can't malloc space for zbuffer.\n");
	return(WHACKED);
    }

    xsize = x;
    ysize = y;
    for (i=0;i<x*y;i++)
	*f++ = ZINIT;
    return(CHILL);
}

int
deinit_zbuffer(void)
{
    if (zbuffer != NULL)
	kfree(zbuffer);
    zbuffer=NULL;
    return(CHILL);
}

int
reset_zbuffer(void)
{
    register float *z;
    register int i;

    z = zbuffer;
    for (i=0;i<xsize*ysize;i++)
	*z++ = ZINIT;
    return(CHILL);
}

int
zbuf_read(int x,int y,float *val)
{
    /* range check on x/y ? */
    
    *val = *(zbuffer+x+y*xsize);
    return(CHILL);
}

int
zbuf_write(int x,int y,float val)
{
    *(zbuffer+x+y*xsize) = val;
    return(CHILL);
}


#if 0
static char zbuf_fname[] = {"zbuf.ras"};
int
dump_zbuffer(void)
{
    float zmin,zmax,zscale;
    float *z;
    int i,j;
    FILE *f;
    unsigned char *linebuf;
    
    struct rasterfile head;
    head.ras_magic = RAS_MAGIC;
    head.ras_width = xsize;
    head.ras_height = ysize;
    head.ras_depth = 8;
    head.ras_length = xsize*ysize;
    head.ras_type = RT_STANDARD;
    head.ras_maptype = RMT_NONE;
    head.ras_maplength = 0;
    
    f = fopen(zbuf_fname,"w");
    fwrite((char *)&head,sizeof(struct rasterfile),1,f);

    zmin = ZINIT;
    zmax = -ZINIT;
    
    z = zbuffer;
    for (j=0;j<ysize;j++)
    {
	for (i=0;i<xsize;i++,z++)
	{
	    if (*z != ZINIT)
	    {
		if (*z < zmin)
		    zmin = *z;
		if (*z > zmax)
		    zmax = *z;
	    }
	}
    }

    z = zbuffer;
    zscale = (254.)/(zmax - zmin);
    
    linebuf = (unsigned char *)malloc(xsize*sizeof(unsigned char));
    for (j=0;j<ysize;j++)
    {
	for (i=0;i<xsize;i++,z++)
	{
	    if (*z == ZINIT)
		linebuf[i] = 0;
	    else
		linebuf[i] = (unsigned char)((*z - zmin) * zscale);
	}
	fwrite((char *)linebuf,sizeof(unsigned char),xsize,f);
    }

    free((char *)linebuf);
    fclose(f);
}


#endif
