#! /bin/sh 

# Replace oname and file.sh in the two lines below with the name 
# of the program you are testing and the file name of this test 
# (for example, test1.sh).

object_name="ifiltdesign"
file_name="test1.sh"

# ---------------------------------------------------------------
# ADD TESTS BETWEEN the  "Add tests below this" and "End of test 
# suite" comment lines (line 60).
# ---------------------------------------------------------------

echo "running "$file_name" for "$object_name" ... process ($$)"

# report utility
# ---------------------------------------------------------------
# Usage:  report  test_num test_obj cntrl_obj test_descr tol
#
# The report function uses ksegcmp to compare the test output 
# data object ($2) against the correct result ($3), and reports 
# success or failure of the test.  If the result of ksegcmp is 
# false, status is set to 1.
#
# Input Arguments: test_num - test number
#                  test_obj - the object generated by the test
#                  cntrl_obj - known result to compare against
#                  test_descr - a string describing the test
#                  tol - tolerance allowed when comparing object 
#                        data values. Recommended tolerance 1e-5
# ---------------------------------------------------------------
report()
{
   test_num="$1"
   test_obj="$2"
   cntrl_obj="$3"
   test_descr="$4"
   tol="$5"

   ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tol" -sat 1 -s -rt 1 -all
   if [ $? = 0 ]
   then
      echo "Passed:  Test  $test_num --> "$test_descr""
   else
      echo "FAILED:  Test  $test_num --> "$test_descr""
      status=1
   fi
}

# Initialize status to be success (ie. 0).  Upon failure, status 
# is initialized to 1.  Set the tmpdir variable.  Script will 
# use /tmp if $TMPDIR is not defined.
status=0
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb IMAGE -echo binpath`}

# ---------------------------------------------------------------
#  Add tests below this
# ---------------------------------------------------------------

# Do the automated comparison tests
test_num=0
for type in lp hp bp bs
  do
  for brand in ideal but gauss ch1 ch2 
    do
    test_num=`expr "$test_num" + 1`
    ifiltdesign -${type} -${brand} -wsize 32 -hsize 32 -pos 1 -order 2 \
                -lcutoff 0.25 -ucutoff 0.5 -eps 0.5 -type 11 \
                -o $tmpdir/out.$$
    descr="$type $brand 32x32" 
    report $test_num "$tmpdir/out.$$" "test1.results/test_${type}_${brand}" "$descr" "1e-5"
    /bin/rm -f $tmpdir/out.$$
  done
done

# ---------------------------------------------------------------
#  End of test suite
# ---------------------------------------------------------------

exit $status
