#! /bin/sh

echo "running test1.sh for irotate ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-5 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for test data, and path for temporary directory
# ------------------------------------------------------------------------
tbpath=`kecho -tb DATAMANIP -echo path`
dpath="$tbpath/data/testdata"
tmpdir=${TMPDIR:-/tmp}
BINARY_PATH=${BINARY_PATH:-`kecho -tb IMAGE -echo binpath`}

# ------------------------------------------------------------------------
# Test  1:  non-square, no map, no mask, double, planes, no rotation
# Note: This test will show some image degradation because there is no way
#       to access the required 2x2 area and still remain completely inside
#       the data set.
# ------------------------------------------------------------------------
test_num=1
descr="non-square, no map, no mask, double, planes, no rotation"
cat <<EOF > $tmpdir/in.$$
1 2 3 4 5
6 7 8 9 10
11 12 13 14 15
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
 -angle 0 -planes
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4 0
6 7 8 9 0
0 0 0 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  2:  non-square, no map, no mask, double, prisms, no rotation
# Note: This test will show some image degradation because there is no way
#       to access the required 2x2 area and still remain completely inside
#       the data set.
# ------------------------------------------------------------------------
test_num=2
descr="non-square, no map, no mask, double, prisms, no rotation"
cat <<EOF > $tmpdir/in.$$
1 2 3 4 5
6 7 8 9 10
11 12 13 14 15
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
 -angle 0 
cat <<EOF > $tmpdir/ascii.$$
1 2 3 4 0
6 7 8 9 0
0 0 0 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  3:  non-square, no map, no mask, ubyte, planes
# ------------------------------------------------------------------------
test_num=3
descr="non-square, no map, no mask, ubyte, planes"
kshrink -i image:moon -o $tmpdir/in.$$ -wmag 0.03125 -hmag 0.0625
cp $tmpdir/in.$$ in
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
  -angle 28 -planes
cp $tmpdir/out.$$ out
cat <<EOF > $tmpdir/ascii.$$
0  0  0  0  0  0  0  0  147  127  128  137  0  0  0  0  0  0  0  0  0  0  0  
146  148  141  139  132  127  140  0  0  0  0  0  0  0  0  0  154  145  137  
134  136  129  143  140  146  0  0  0  0  0  0  149  151  142  155  144  123  
133  140  142  141  0  0  0  0  0  0  156  142  111  158  180  123  138  139  
135  125  0  0  0  0  0  157  158  139  59  193  147  128  139  137  134  140  
0  0  0  0  0  161  155  163  153  160  137  133  138  134  140  147  0  0  
0  0  130  154  153  158  147  143  138  140  142  136  129  137  0  0  0  
137  154  146  149  154  147  142  133  131  132  145  120  107  0  0  0  146  
152  134  140  147  143  135  132  135  137  145  119  59  0  0  142  143  135  
136  143  139  134  125  125  133  148  181  78  86  0  0  140  131  123  132  
144  125  127  136  136  111  149  168  54  139  0  145  143  130  135  134  
124  126  129  136  137  77  156  93  97  136  0  141  140  133  137  141  
120  124  119  122  100  96  125  101  145  160  136  128  133  121  131  
130  125  104  80  115  116  123  138  142  151  157  130  125  123  115  
119  122  117  74  95  109  115  123  129  155  156  160  133  130  117  105  
110  111  96  56  97  116  122  121  140  155  160  162  139  125  106  110  
111  111  114  91  103  118  121  126  148  156  162  156  114  116  110  
114  114  110  102  79  96  117  125  128  146  149  148  0  123  124  117  
112  109  85  90  99  99  114  129  141  152  151  154  0  132  127  110  
109  112  102  102  102  107  123  136  148  157  158  0  0  122  117  118  
119  97  107  113  106  120  129  134  145  153  158  0  0  119  112  132  100  
89  106  107  102  112  124  140  142  145  0  0  0  114  121  130  67  71  
102  105  104  98  116  136  129  151  0  0  0  115  115  98  66  73  107  
110  110  111  118  110  132  0  0  0  0  113  105  81  79  91  103  114  112  
117  133  139  0  0  0  0  0  117  118  52  51  84  95  125  139  128  141  
156  0  0  0  0  0  107  104  15  36  88  108  119  120  126  147  0  0  0  
0  0  0  156  93  56  65  102  118  105  113  117  138  0  0  0  0  0  0  127  
75  83  106  115  113  111  119  118  0  0  0  0  0  0  0  0  0  105  116  123  
119  105  133  145  0  0  0  0  0  0  0  0  0  0  0  118  121  122  138  0  
0  0  0  0  0  0  0  
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 16 -hsize 32 -type 3 -o $tmpdir/result.$$
cp $tmpdir/result.$$ ref
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$
 
# ------------------------------------------------------------------------
# Test  4:  non-square, no map, no mask, ubyte, prisms
# ------------------------------------------------------------------------
test_num=4
descr="non-square, no map, no mask, ubyte, prisms"
kshrink -i image:moon -o $tmpdir/in.$$ -wmag 0.03125 -hmag 0.0625
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
  -angle 28
cat <<EOF > $tmpdir/ascii.$$
0  0  0  0  0  0  0  0  147  127  128  137  0  0  0  0  0  0  0  0  0  0  0
146  148  141  139  132  127  140  0  0  0  0  0  0  0  0  0  154  145  137
134  136  129  143  140  146  0  0  0  0  0  0  149  151  142  155  144  123
133  140  142  141  0  0  0  0  0  0  156  142  111  158  180  123  138  139
135  125  0  0  0  0  0  157  158  139  59  193  147  128  139  137  134  140
0  0  0  0  0  161  155  163  153  160  137  133  138  134  140  147  0  0
0  0  130  154  153  158  147  143  138  140  142  136  129  137  0  0  0
137  154  146  149  154  147  142  133  131  132  145  120  107  0  0  0  146
152  134  140  147  143  135  132  135  137  145  119  59  0  0  142  143  135
136  143  139  134  125  125  133  148  181  78  86  0  0  140  131  123  132
144  125  127  136  136  111  149  168  54  139  0  145  143  130  135  134
124  126  129  136  137  77  156  93  97  136  0  141  140  133  137  141
120  124  119  122  100  96  125  101  145  160  136  128  133  121  131
130  125  104  80  115  116  123  138  142  151  157  130  125  123  115
119  122  117  74  95  109  115  123  129  155  156  160  133  130  117  105
110  111  96  56  97  116  122  121  140  155  160  162  139  125  106  110
111  111  114  91  103  118  121  126  148  156  162  156  114  116  110
114  114  110  102  79  96  117  125  128  146  149  148  0  123  124  117
112  109  85  90  99  99  114  129  141  152  151  154  0  132  127  110
109  112  102  102  102  107  123  136  148  157  158  0  0  122  117  118
119  97  107  113  106  120  129  134  145  153  158  0  0  119  112  132  100
89  106  107  102  112  124  140  142  145  0  0  0  114  121  130  67  71
102  105  104  98  116  136  129  151  0  0  0  115  115  98  66  73  107
110  110  111  118  110  132  0  0  0  0  113  105  81  79  91  103  114  112
117  133  139  0  0  0  0  0  117  118  52  51  84  95  125  139  128  141
156  0  0  0  0  0  107  104  15  36  88  108  119  120  126  147  0  0  0
0  0  0  156  93  56  65  102  118  105  113  117  138  0  0  0  0  0  0  127
75  83  106  115  113  111  119  118  0  0  0  0  0  0  0  0  0  105  116  123
119  105  133  145  0  0  0  0  0  0  0  0  0  0  0  118  121  122  138  0
0  0  0  0  0  0  0
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 16 -hsize 32 -type 3 -o $tmpdir/result.$$
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  5:  non-square, no map, no mask, double, planes
# ------------------------------------------------------------------------
test_num=5
descr="non-square, no map, no mask, double, planes"
kshrink -i image:moon -o $tmpdir/in.$$ -wmag 0.03125 -hmag 0.0625
kconvert -i $tmpdir/in.$$ -o $tmpdir/in.$$ -type double -val
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
 -angle 28 -planes
cat <<EOF > $tmpdir/ascii.$$
         0  0  0  0  0  0  0  0  147.7231182  127.8488198  128.6749973  137.8065123  0  0  0  0  
         0  0  0  0  0  0  0  146.1653758  148.2105084  141.7243258  139.5966242  132.8020597  127.0003759  140.563375  0  0  
         0  0  0  0  0  0  0  154.3895727  145.2039497  137.7916681  134.7788479  136.1938559  129.7377156  143.977913  140.020634  146.2253267  
         0  0  0  0  0  0  149.0039004  151.022209  142.6497776  155.0969745  144.7849619  123.0704332  133.7732168  140.7680429  142.3933521  141.7175691  
         0  0  0  0  0  0  156.8264576  142.5884725  111.1422067  158.0444708  180.4859967  123.4636366  138.0305333  139.6124709  135.0850446  125.010151  
         0  0  0  0  0  157.8073744  158.9132677  139.3677154  59.80560985  193.9706294  147.1341424  128.1488147  139.4899497  137.5881168  134.8930706  140.0544981  
         0  0  0  0  0  161.9990933  155.9928689  163.4450403  153.4057241  160.2336511  137.173088  133.4365924  138.8943993  134.8281292  140.5053373  147.6817019  
         0  0  0  0  130.3442366  154.7557462  153.2217777  158.8337796  147.1450415  143.6266296  138.0864384  140.9075685  142.2269701  136.0037021  129.4178769  137.5096606  
         0  0  0  137.7662246  154.7263176  146.9304541  149.2066584  154.1679514  147.7004932  142.5866801  133.874221  131.9714962  132.7552389  145.8925441  120.9958975  107.5241264  
         0  0  0  146.2798411  152.4313934  134.5294281  140.6444233  147.0122806  143.4245964  135.8498831  132.7904907  135.5524577  137.5678516  145.3148935  119.6096978  59.82895012  
         0  0  142.6845521  143.8692298  135.9888591  136.5513692  143.7675162  139.3502149  134.4854081  125.3609676  125.9123767  133.193743  148.2694734  181.7815794  78.83197363  86.67013396  
         0  0  140.7465569  131.5009566  123.6024556  132.5321363  144.8355929  125.8992045  127.8712965  136.0532155  136.4490898  111.8018542  149.4360626  168.1319297  54.84317805  139.0481262  
         0  145.6137595  143.9556176  130.8623419  135.100898  134.734957  124.2515806  126.4152592  129.6208589  136.5404872  137.797221  77.07896934  156.5528789  93.7000157  97.7185009  136.1792955  
         0  141.0021169  140.0568807  133.9207536  137.7131968  141.8522202  120.9988085  124.2607922  119.6798361  122.9535872  100.5924861  96.34110156  125.8170114  101.8929975  145.1838419  160.7665701  
 136.1755832  128.2793664  133.2202698  121.722294  131.3871527  130.5195863  125.2847279  104.9132399  80.25474402  115.4333455  116.0063681  123.1784469  138.0278503  142.3849721  151.6343184  157.8163106  
 130.214107  125.9200464  123.8051518  115.5242963  119.1272398  122.0059722  117.3867024  74.76860549  95.50287761  109.8028426  115.9510912  123.2005471  129.3982378  155.4158064  156.0266338  160.0408795  
 133.7299298  130.9475929  117.0313885  105.9099476  110.2661298  111.2792183  96.55576207  56.25021449  97.56082603  116.2649139  122.3487874  121.7526296  140.4173212  155.1037129  160.1085409  162.4179147  
 139.0984794  125.0936694  106.1883087  110.5497059  111.0148661  111.986132  114.9164392  91.86303834  103.7378931  118.256466  121.4913456  126.8863267  148.10254  156.0262142  162.2592704  156.8014994  
 114.7794633  116.1391283  110.3823466  114.7746793  114.6443775  110.2607575  102.3597041  79.27190327  96.32195818  117.9410735  125.1038517  128.0186175  146.0134194  149.419073  148.5467814  0  
 123.4456468  124.1736189  117.0556673  112.5193894  109.2016636  85.98917131  90.73596212  99.07279967  99.15527606  114.8361418  129.9785977  141.9075349  152.3538252  151.0961142  154.8720298  0  
 132.0708094  127.1940104  110.0534154  109.7294717  112.5932506  102.5424963  102.3093392  102.6460031  107.628579  123.9464055  136.039384  148.2575797  157.9908223  158.495001  0  0  
 122.9997937  117.7671957  118.0160876  119.7838963  97.10060989  107.257629  113.7074938  106.224908  120.0884753  129.0927732  134.9395002  145.7971539  153.9212468  158.5790945  0  0  
 119.8608764  112.0915926  132.039349  100.1179661  89.14615063  106.916674  107.9390307  102.9409322  112.8118089  124.1178382  140.0623174  142.2935475  145.0173626  0  0  0  
 114.7602581  121.0479617  130.2816651  67.55843946  71.46107147  102.4072074  105.4596567  104.1972862  98.54791892  116.3919097  136.4142989  129.2213922  151.8962135  0  0  0  
 115.2163936  115.5907538  98.00359999  66.33508124  73.9166182  107.5489433  110.3935826  110.3913416  111.7731525  118.1191419  110.8136391  132.96479  0  0  0  0  
 113.5641157  105.4252391  81.26004598  79.98070109  91.17904748  103.3943189  114.7714067  112.0336383  117.2483801  133.5621272  139.5927648  0  0  0  0  0  
 117.5228241  118.4700637  52.0881106  51.7339388  84.48789384  95.86230732  125.3181163  139.7052987  128.2615353  141.2249225  156.3923188  0  0  0  0  0  
 107.0075069  104.4701833  15.6423976  36.60605756  88.08911574  108.0045838  119.255702  120.8502331  126.1779209  147.8022183  0  0  0  0  0  0  
 156.4598237  93.75633208  56.49174599  65.9402592  102.1227991  118.2621303  105.1196451  113.6600749  117.9950775  138.4829363  0  0  0  0  0  0  
 127.8638357  75.77037694  83.60308806  106.1512915  115.0292505  113.9477998  111.1516632  119.1897271  118.0809293  0  0  0  0  0  0  0  
         0  0  105.6873803  116.4713292  123.4335091  119.9711645  105.1969575  133.9400411  145.0715078  0  0  0  0  0  0  0  
         0  0  0  0  118.2950324  121.8244759  122.7742029  138.1822223  0  0  0  0  0  0  0  0  
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$ $tmpdir/in.$$
 
# ------------------------------------------------------------------------
# Test  6:  non-square, no map, no mask, double, prisms
# ------------------------------------------------------------------------
test_num=6
descr="non-square, no map, no mask, double, prisms"
kshrink -i image:moon -o $tmpdir/in.$$ -wmag 0.03125 -hmag 0.0625
kconvert -i $tmpdir/in.$$ -o $tmpdir/in.$$ -type double -val
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
 -angle 28
cat <<EOF > $tmpdir/ascii.$$
         0  0  0  0  0  0  0  0  147.7231182  127.8488198  128.6749973  137.8065123  0  0  0  0  
         0  0  0  0  0  0  0  146.1653758  148.2105084  141.7243258  139.5966242  132.8020597  127.0003759  140.563375  0  0  
         0  0  0  0  0  0  0  154.3895727  145.2039497  137.7916681  134.7788479  136.1938559  129.7377156  143.977913  140.020634  146.2253267  
         0  0  0  0  0  0  149.0039004  151.022209  142.6497776  155.0969745  144.7849619  123.0704332  133.7732168  140.7680429  142.3933521  141.7175691  
         0  0  0  0  0  0  156.8264576  142.5884725  111.1422067  158.0444708  180.4859967  123.4636366  138.0305333  139.6124709  135.0850446  125.010151  
         0  0  0  0  0  157.8073744  158.9132677  139.3677154  59.80560985  193.9706294  147.1341424  128.1488147  139.4899497  137.5881168  134.8930706  140.0544981  
         0  0  0  0  0  161.9990933  155.9928689  163.4450403  153.4057241  160.2336511  137.173088  133.4365924  138.8943993  134.8281292  140.5053373  147.6817019  
         0  0  0  0  130.3442366  154.7557462  153.2217777  158.8337796  147.1450415  143.6266296  138.0864384  140.9075685  142.2269701  136.0037021  129.4178769  137.5096606  
         0  0  0  137.7662246  154.7263176  146.9304541  149.2066584  154.1679514  147.7004932  142.5866801  133.874221  131.9714962  132.7552389  145.8925441  120.9958975  107.5241264  
         0  0  0  146.2798411  152.4313934  134.5294281  140.6444233  147.0122806  143.4245964  135.8498831  132.7904907  135.5524577  137.5678516  145.3148935  119.6096978  59.82895012  
         0  0  142.6845521  143.8692298  135.9888591  136.5513692  143.7675162  139.3502149  134.4854081  125.3609676  125.9123767  133.193743  148.2694734  181.7815794  78.83197363  86.67013396  
         0  0  140.7465569  131.5009566  123.6024556  132.5321363  144.8355929  125.8992045  127.8712965  136.0532155  136.4490898  111.8018542  149.4360626  168.1319297  54.84317805  139.0481262  
         0  145.6137595  143.9556176  130.8623419  135.100898  134.734957  124.2515806  126.4152592  129.6208589  136.5404872  137.797221  77.07896934  156.5528789  93.7000157  97.7185009  136.1792955  
         0  141.0021169  140.0568807  133.9207536  137.7131968  141.8522202  120.9988085  124.2607922  119.6798361  122.9535872  100.5924861  96.34110156  125.8170114  101.8929975  145.1838419  160.7665701  
 136.1755832  128.2793664  133.2202698  121.722294  131.3871527  130.5195863  125.2847279  104.9132399  80.25474402  115.4333455  116.0063681  123.1784469  138.0278503  142.3849721  151.6343184  157.8163106  
 130.214107  125.9200464  123.8051518  115.5242963  119.1272398  122.0059722  117.3867024  74.76860549  95.50287761  109.8028426  115.9510912  123.2005471  129.3982378  155.4158064  156.0266338  160.0408795  
 133.7299298  130.9475929  117.0313885  105.9099476  110.2661298  111.2792183  96.55576207  56.25021449  97.56082603  116.2649139  122.3487874  121.7526296  140.4173212  155.1037129  160.1085409  162.4179147  
 139.0984794  125.0936694  106.1883087  110.5497059  111.0148661  111.986132  114.9164392  91.86303834  103.7378931  118.256466  121.4913456  126.8863267  148.10254  156.0262142  162.2592704  156.8014994  
 114.7794633  116.1391283  110.3823466  114.7746793  114.6443775  110.2607575  102.3597041  79.27190327  96.32195818  117.9410735  125.1038517  128.0186175  146.0134194  149.419073  148.5467814  0  
 123.4456468  124.1736189  117.0556673  112.5193894  109.2016636  85.98917131  90.73596212  99.07279967  99.15527606  114.8361418  129.9785977  141.9075349  152.3538252  151.0961142  154.8720298  0  
 132.0708094  127.1940104  110.0534154  109.7294717  112.5932506  102.5424963  102.3093392  102.6460031  107.628579  123.9464055  136.039384  148.2575797  157.9908223  158.495001  0  0  
 122.9997937  117.7671957  118.0160876  119.7838963  97.10060989  107.257629  113.7074938  106.224908  120.0884753  129.0927732  134.9395002  145.7971539  153.9212468  158.5790945  0  0  
 119.8608764  112.0915926  132.039349  100.1179661  89.14615063  106.916674  107.9390307  102.9409322  112.8118089  124.1178382  140.0623174  142.2935475  145.0173626  0  0  0  
 114.7602581  121.0479617  130.2816651  67.55843946  71.46107147  102.4072074  105.4596567  104.1972862  98.54791892  116.3919097  136.4142989  129.2213922  151.8962135  0  0  0  
 115.2163936  115.5907538  98.00359999  66.33508124  73.9166182  107.5489433  110.3935826  110.3913416  111.7731525  118.1191419  110.8136391  132.96479  0  0  0  0  
 113.5641157  105.4252391  81.26004598  79.98070109  91.17904748  103.3943189  114.7714067  112.0336383  117.2483801  133.5621272  139.5927648  0  0  0  0  0  
 117.5228241  118.4700637  52.0881106  51.7339388  84.48789384  95.86230732  125.3181163  139.7052987  128.2615353  141.2249225  156.3923188  0  0  0  0  0  
 107.0075069  104.4701833  15.6423976  36.60605756  88.08911574  108.0045838  119.255702  120.8502331  126.1779209  147.8022183  0  0  0  0  0  0  
 156.4598237  93.75633208  56.49174599  65.9402592  102.1227991  118.2621303  105.1196451  113.6600749  117.9950775  138.4829363  0  0  0  0  0  0  
 127.8638357  75.77037694  83.60308806  106.1512915  115.0292505  113.9477998  111.1516632  119.1897271  118.0809293  0  0  0  0  0  0  0  
         0  0  105.6873803  116.4713292  123.4335091  119.9711645  105.1969575  133.9400411  145.0715078  0  0  0  0  0  0  0  
         0  0  0  0  118.2950324  121.8244759  122.7742029  138.1822223  0  0  0  0  0  0  0  0  
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  7:  square, no map, no mask, double, planes, COA testing
# ------------------------------------------------------------------------
test_num=7
descr="square, no map, no mask, double, planes, COA testing"
cat <<EOF > $tmpdir/in.$$
0 0 0 0 0
0 0 1 0 0
0 0 1 0 0
0 0 0 0 0
0 0 0 0 0
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wc 2 -hc 1 \
 -angle 0 -planes
cat <<EOF > $tmpdir/ascii.$$
0 0 0 0 0
0 0 0 0 0
0 0 1 0 0
0 0 1 0 0
0 0 0 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  8:  square, no map, no mask, double, prisms, COA testing
# ------------------------------------------------------------------------
test_num=8
descr="square, no map, no mask, double, prisms, COA testing"
cat <<EOF > $tmpdir/in.$$
0 0 0 0 0
0 0 1 0 0
0 0 1 0 0
0 0 0 0 0
0 0 0 0 0
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wc 1 -hc 1 \
 -angle 0 
cat <<EOF > $tmpdir/ascii.$$
0 0 0 0 0
0 0 0 0 0
0 0 0 1 0
0 0 0 1 0
0 0 0 0 0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  9:  non-square, mask, double, planes
# ------------------------------------------------------------------------
test_num=9
descr="non-square, mask, double, planes"
cat <<EOF > $tmpdir/val.$$
1 1 1 1 1 1
1 1 1 1 1 1
1 1 1 1 1 1
1 1 1 1 1 1
1 1 1 1 1 1
EOF
cat <<EOF > $tmpdir/mask.$$
1 1 1 1 1 1
1 1 1 1 1 1
1 1 0 1 1 1
1 1 1 1 1 1
1 1 1 1 1 1
EOF
kasc2mask -i1 $tmpdir/mask.$$ -i2 $tmpdir/val.$$ -o $tmpdir/in.$$ \
  -wsize 6 -hsize 5 -type double
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -wctr -hctr \
 -angle 0 -planes
cat <<EOF > $tmpdir/val.$$
1 1 1 1 1 0
1 1 1 1 1 0
1 1 1 1 1 0
1 1 1 1 1 0
0 0 0 0 0 0
EOF
cat <<EOF > $tmpdir/mask.$$
1 1 1 1 1 1
1 0 0 1 1 1
1 0 0 1 1 1
1 1 1 1 1 1
1 1 1 1 1 1
EOF
kasc2mask -i1 $tmpdir/mask.$$ -i2 $tmpdir/val.$$ -o $tmpdir/result.$$ \
  -wsize 6 -hsize 5 -type double
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$
/bin/rm -f $tmpdir/val.$$ $tmpdir/mask.$$

# ------------------------------------------------------------------------
# Test  10:  square, no map, no mask, double, planes, resize 15 deg
# ------------------------------------------------------------------------
test_num=10
descr="square, no map, no mask, double, planes, resize 15 deg"
cat <<EOF > $tmpdir/in.$$
1 1 1 1 1
1 2 2 2 1
1 2 3 2 1
1 2 2 2 1
1 1 1 1 1
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -resize \
 -angle 15 -planes -wctr -hctr
cat <<EOF > $tmpdir/ascii.$$
         0  0  0  0  0  0  0
         0  0  1.16885776  1.068148347  0  0  0
         0  0  1.775255129  2.025255129  1.775255129  1.16885776  0
         0  1.068148347  2.025255129  3  2.025255129  1.068148347  0
         0  1.16885776  1.775255129  2.025255129  1.775255129  0  0
         0  0  0  1.068148347  1.16885776  0  0
         0  0  0  0  0  0  0
EOF
report $test_num "$tmpdir/out.$$" "$tmpdir/ascii.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$

# ------------------------------------------------------------------------
# Test  11:  square, double map, byte val, no mask, planes, resize 15 deg
# ------------------------------------------------------------------------
test_num=11
descr="square, double map, byte val, no mask, planes, resize 15 deg"
cat <<EOF > $tmpdir/map.$$
5 21 64
1 1 1
2 2 2
3 3 3
EOF
cat <<EOF > $tmpdir/val.$$
1 1 1 1 1
1 2 2 2 1
1 2 3 2 1
1 2 2 2 1
1 1 1 1 1
EOF
kasc2val -i1 $tmpdir/val.$$ -o $tmpdir/val.$$ -wsize 5 -hsize 5 -type byte
kasc2map -i1 $tmpdir/map.$$ -i2 $tmpdir/val.$$ -wsize 3 -hsize 4 -type double \
  -o $tmpdir/in.$$
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.$$ -resize \
 -angle 15 -planes -wctr -hctr
cat <<EOF > $tmpdir/ascii.$$
         0  0  0  0  0  0  0
         0  0  1.16885776  1.068148347  0  0  0
         0  0  1.775255129  2.025255129  1.775255129  1.16885776  0
         0  1.068148347  2.025255129  3  2.025255129  1.068148347  0
         0  1.16885776  1.775255129  2.025255129  1.775255129  0  0
         0  0  0  1.068148347  1.16885776  0  0
         0  0  0  0  0  0  0
         0  0  0  0  0  0  0
         0  0  1.16885776  1.068148347  0  0  0
         0  0  1.775255129  2.025255129  1.775255129  1.16885776  0
         0  1.068148347  2.025255129  3  2.025255129  1.068148347  0
         0  1.16885776  1.775255129  2.025255129  1.775255129  0  0
         0  0  0  1.068148347  1.16885776  0  0
         0  0  0  0  0  0  0
         0  0  0  0  0  0  0
         0  0  1.16885776  1.068148347  0  0  0
         0  0  1.775255129  2.025255129  1.775255129  1.16885776  0
         0  1.068148347  2.025255129  3  2.025255129  1.068148347  0
         0  1.16885776  1.775255129  2.025255129  1.775255129  0  0
         0  0  0  1.068148347  1.16885776  0  0
         0  0  0  0  0  0  0
EOF
kasc2val -i1 $tmpdir/ascii.$$ -o $tmpdir/result.$$ -wsize 7 -hsize 7 \
 -esize 3 -type double
report $test_num "$tmpdir/out.$$" "$tmpdir/result.$$" "$descr"
/bin/rm -f $tmpdir/out.$$ $tmpdir/result.$$ $tmpdir/ascii.$$ $tmpdir/in.$$
/bin/rm -f $tmpdir/map.$$ $tmpdir/val.$$

# ------------------------------------------------------------------------
# Test  12:  square, no map, no mask, dcomplex, planes
# ------------------------------------------------------------------------
test_num=12
descr="square, no map, no mask, dcomplex, planes"
cat <<EOF > $tmpdir/in.$$
1 2 3 4
5 6 7 8
9 10 11 12
13 14 15 16
17 18 19 20
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.real.$$ -wctr -hctr \
 -angle 30 -planes
cat <<EOF > $tmpdir/in.$$
1.5 2.5 3.5 4.5
5.5 6.5 7.5 8.5
9.5 10.5 11.5 12.5
13.5 14.5 15.5 16.5
17.5 18.5 19.5 20.5
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.imag.$$ -wctr -hctr \
 -angle 30 -planes
cat <<EOF > $tmpdir/in.$$
(1,1.5) (2,2.5) (3,3.5) (4,4.5)
(5,5.5) (6,6.5) (7,7.5) (8,8.5)
(9,9.5) (10,10.5) (11,11.5) (12,12.5)
(13,13.5) (14,14.5) (15,15.5) (16,16.5)
(17,17.5) (18,18.5) (19,19.5) (20,20.5)
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.dc.$$ -wctr -hctr \
 -angle 30 -planes
kreal2cmplx -i1 $tmpdir/out.real.$$ -i2 $tmpdir/out.imag.$$ \
 -o $tmpdir/out.dc.combined.$$

report $test_num "$tmpdir/out.dc.$$" "$tmpdir/out.dc.combined.$$" "$descr"
/bin/rm -f $tmpdir/out.dc.$$ $tmpdir/in.$$ $tmpdir/out.real.$$
/bin/rm -f $tmpdir/out.imag.$$ 

# ------------------------------------------------------------------------
# Test  13:  square, no map, no mask, dcomplex, prisms
# ------------------------------------------------------------------------
test_num=13
descr="square, no map, no mask, dcomplex, prisms"
cat <<EOF > $tmpdir/in.$$
1 2 3 4
5 6 7 8
9 10 11 12
13 14 15 16
17 18 19 20
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.real.$$ -wctr -hctr \
 -angle 30
cat <<EOF > $tmpdir/in.$$
1.5 2.5 3.5 4.5
5.5 6.5 7.5 8.5
9.5 10.5 11.5 12.5
13.5 14.5 15.5 16.5
17.5 18.5 19.5 20.5
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.imag.$$ -wctr -hctr \
 -angle 30
cat <<EOF > $tmpdir/in.$$
(1,1.5) (2,2.5) (3,3.5) (4,4.5)
(5,5.5) (6,6.5) (7,7.5) (8,8.5)
(9,9.5) (10,10.5) (11,11.5) (12,12.5)
(13,13.5) (14,14.5) (15,15.5) (16,16.5)
(17,17.5) (18,18.5) (19,19.5) (20,20.5)
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.dc.$$ -wctr -hctr \
 -angle 30
kreal2cmplx -i1 $tmpdir/out.real.$$ -i2 $tmpdir/out.imag.$$ \
 -o $tmpdir/out.dc.combined.$$

report $test_num "$tmpdir/out.dc.$$" "$tmpdir/out.dc.combined.$$" "$descr"
/bin/rm -f $tmpdir/out.dc.$$ $tmpdir/in.$$ $tmpdir/out.real.$$
/bin/rm -f $tmpdir/out.imag.$$

# ------------------------------------------------------------------------
# Test  14:  square, no map, no mask, compare prisms to planes
# ------------------------------------------------------------------------
test_num=14
descr="square, no map, no mask, compare prisms to planes"
cat <<EOF > $tmpdir/in.$$
1 2 3 4
5 6 7 8
9 10 11 12
13 14 15 16
17 18 19 20
EOF
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.prisms.$$ -wctr -hctr \
 -angle 30
${BINARY_PATH}/irotate -i $tmpdir/in.$$ -o $tmpdir/out.planes.$$ -wctr -hctr \
 -angle 30
report $test_num "$tmpdir/out.prisms.$$" "$tmpdir/out.planes.$$" "$descr"
/bin/rm -f $tmpdir/in.$$ $tmpdir/out.prisms.$$ $tmpdir/out.planes.$$

exit $status

