 /*
  * Khoros: $Id: mtransform.c,v 1.4 1996/07/05 03:08:11 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: mtransform.c,v 1.4 1996/07/05 03:08:11 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for mtransform
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "mtransform.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Transform all vectors in an object using a single matrix
|
|       Purpose: main program for mtransform
|
|         Input:
|		char *clui_info->i1_file; {matrix to be used to transform the vectors}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {object of vectors (down E) to be transformed}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->o_file; {Object of transformed E vectors}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int   clui_info->type_list; {Data type of output}
|		char *clui_info->type_label;
|		int   clui_info->type_flag; {TRUE if -type specified}
|
|        Output:
|       Returns:
|
|    Written By: 
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
kobject mat_obj = NULL;
kobject in_obj = NULL;
kobject out_obj = NULL;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "MATRIX");
	kexit_handler(mtransform_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "MATRIX", "mtransform",
		mtransform_usage_additions, mtransform_get_args,
		mtransform_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        if ((mat_obj = kpds_open_input_object(clui_info->i1_file))
                == KOBJECT_INVALID)
        {
           kerror("mtransform","main", "Cannot open matrix object.");
           kexit(KEXIT_FAILURE);
        }
        if ((in_obj = kpds_open_input_object(clui_info->i2_file))
                == KOBJECT_INVALID)
        {
           kerror("mtransform","main", "Cannot open vector input object.");
           kexit(KEXIT_FAILURE);
        }
 
        if ((out_obj = kpds_open_output_object(clui_info->o_file))
                == KOBJECT_INVALID)
          {
            kerror("mtransform","main", "Cannot open output object.");
            kexit(KEXIT_FAILURE);
          }
        if (!kpds_copy_object_attr(in_obj, out_obj))
          {
            kerror("mtransform","main",
              "Unable to copy attributes from input to output object.");
            kexit(KEXIT_FAILURE);
          }
/* -main_before_lib_call_end */

/* -main_library_call */
	if (!lmtransform(mat_obj,in_obj,clui_info->type_label,
                         out_obj)) kexit(KEXIT_FAILURE);
/* -main_library_call_end */

/* -main_after_lib_call */
        /* add history to the output objects and write */
        if (!kpds_set_attribute(out_obj,KPDS_HISTORY,kpds_history_string()))
          {
            kerror("mtransform", "main","Unable to set history on output object");
            kexit(KEXIT_FAILURE);
          }
        kpds_close_object(mat_obj);
        kpds_close_object(out_obj);
        kpds_close_object(in_obj);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: mtransform_usage_additions
| 
|       Purpose: Prints usage additions in mtransform_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mtransform
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void mtransform_usage_additions(void)
{
	kfprintf(kstderr, "\tTransform all vectors in an object using a single matrix\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: mtransform_free_args
| 
|       Purpose: Frees CLUI struct allocated in mtransform_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname mtransform
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
mtransform_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->type_label);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
