 /*
  * Khoros: $Id: lrfclass.c,v 1.11 1996/07/05 03:19:11 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lrfclass.c,v 1.11 1996/07/05 03:19:11 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for lrfclass
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "lrfclass.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Classify Image Using the Localized Receptive Field Classifier  (K1)
|
|       Purpose: main program for lrfclass
|
|         Input:
|		char *clui_info->i1_file; {input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {cluster center image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {cluster variance image}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->i4_file; {weight image}
|		int   clui_info->i4_flag; {TRUE if -i4 specified}
|
|		char *clui_info->o_file; {classified image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->b_int; {Border Width}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer and Charlie Gage
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage  *image,         /* input image */
                *cc_img,        /* cluster center image */
                *var_img,       /* cluster variance image */
                *wt_img,        /* weight image */
                *class_img;     /* classified image */
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(lrfclass_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "lrfclass",
		lrfclass_usage_additions, lrfclass_get_args,
		lrfclass_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* Read in INPUT image for processing */
image = readimage(clui_info->i1_file);
if (image == NULL) {
   (void) kfprintf(kstderr,"lrfclass: Can not read input image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPE(program, image);
CHECKINPUT(program, image);

/* Read in CLUSTER CENTER image as input for initial LRF nodes */
cc_img = readimage(clui_info->i2_file);
if (cc_img == NULL) {
   (void) kfprintf(kstderr,"lrfclass: Can not read cluster center image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPE(program, cc_img);
CHECKINPUT(program, cc_img);

/* Read in VARIANCE image to get trace for widths of LRF centers */
var_img = readimage(clui_info->i3_file);
if (var_img == NULL) {
   (void) kfprintf(kstderr,"lrfclass: Can not read variance image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPE(program, var_img);
CHECKINPUT(program, var_img);

/* Read in WEIGHT image to initialize the Single Layer Perceptron */
wt_img = readimage(clui_info->i4_file);
if (wt_img == NULL) {
   (void) kfprintf(kstderr,"lrfclass: Can not read weight image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPE(program, wt_img);
CHECKINPUT(program, wt_img);

/* Check for proper number of bands in the cluster center image. */
if ( image->num_data_bands != (cc_img->num_data_bands - 1)) {
   kfprintf(kstderr,"lrfclass: The center/class image does not contain the correct\n");
   kfprintf(kstderr, "number of data_bands\n");
   kfprintf(kstderr, "Should have %d bands, but contains %d bands\n", (image->num_data_bands + 1), cc_img->num_data_bands);
   kexit(KEXIT_FAILURE);
}
/* Check for proper number of bands in the variance image. */
if ( image->num_data_bands != var_img->num_data_bands) {
    kfprintf(kstderr, "lrfclass: The variance image does not contain the correct\n");
    kfprintf(kstderr, "number of data_bands\n");
    kfprintf(kstderr, "Should have %d bands, but contains %d bands\n", image->num_data_bands , var_img->num_data_bands);
    kexit(KEXIT_FAILURE);
}
/* -main_before_lib_call_end */

/* -main_library_call */
if(! llrfclass (image, cc_img, var_img, wt_img, &class_img, clui_info->b_int) )
{
  (void) kfprintf(kstderr, "llrfclass Failed\n");
  kexit(KEXIT_FAILURE);
}
/* -main_library_call_end */

/* -main_after_lib_call */
/* Write out the classified image */
writeimage(clui_info->o_file, class_img);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: lrfclass_usage_additions
| 
|       Purpose: Prints usage additions in lrfclass_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname lrfclass
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void lrfclass_usage_additions(void)
{
	kfprintf(kstderr, "\tClassify Image Using the Localized Receptive Field Classifier\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: lrfclass_free_args
| 
|       Purpose: Frees CLUI struct allocated in lrfclass_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname lrfclass
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
lrfclass_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->i4_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
