 /*
  * Khoros: $Id: lrftrain.c,v 1.11 1996/07/05 03:19:19 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lrftrain.c,v 1.11 1996/07/05 03:19:19 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for lrftrain
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "lrftrain.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Calculate Weights for Localized Receptive Field Classifier  (K1)
|
|       Purpose: main program for lrftrain
|
|         Input:
|		char *clui_info->i1_file; {input image}
|		int   clui_info->i1_flag; {TRUE if -i1 specified}
|
|		char *clui_info->i2_file; {cluster center image}
|		int   clui_info->i2_flag; {TRUE if -i2 specified}
|
|		char *clui_info->i3_file; {cluster variance image}
|		int   clui_info->i3_flag; {TRUE if -i3 specified}
|
|		char *clui_info->i4_file; {cluster number image}
|		int   clui_info->i4_flag; {TRUE if -i4 specified}
|
|		char *clui_info->o_file; {weight image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		char *clui_info->f_file; {output file for training statistics}
|		int   clui_info->f_flag; {TRUE if -f specified}
|
|		int clui_info->b_int; {Border Width}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|		float clui_info->cv_float; {convergence value}
|		int   clui_info->cv_flag; {TRUE if -cv specified}
|
|		float clui_info->meu_float; {weight update value}
|		int   clui_info->meu_flag; {TRUE if -meu specified}
|
|		int clui_info->n_int; {Maximum number of iterations}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|		float clui_info->delta_float; {Min delta MSE value}
|		int   clui_info->delta_flag; {TRUE if -delta specified}
|
|		int clui_info->d_int; {MSE display interval}
|		int clui_info->d_flag; {TRUE if -d specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer and Charlie Gage
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage  *image,         /* input image */
                *cc_img,        /* cluster center image */
                *var_img,       /* cluster variance image */
                *cn_img,        /* cluster number image */
                *wt_img;        /* output weight image */
kfile *printdev;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(lrftrain_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "lrftrain",
		lrftrain_usage_additions, lrftrain_get_args,
		lrftrain_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* Read in INPUT image for processing */
image = readimage(clui_info->i1_file);
if (image == NULL) {
   (void) kfprintf(kstderr,"lrftrain: Can not read input image\n");
   kexit(KEXIT_FAILURE);
}
CHECKTYPE(program, image);
CHECKINPUT(program, image);

/* Read in CLUSTER CENTER image as input for initial LRF nodes */
cc_img = readimage(clui_info->i2_file);
if (cc_img == NULL) {
   (void) kfprintf(kstderr,"lrftrain: Can not read cluster center image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPE(program, cc_img);
CHECKINPUT(program, cc_img);

/* Read in VARIANCE image to get trace for widths of LRF centers */
var_img = readimage(clui_info->i3_file);
if (var_img == NULL) {
   (void) kfprintf(kstderr,"lrftrain: Can not read variance image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}

CHECKTYPE(program, var_img);
CHECKINPUT(program, var_img);

/* Read in CLUSTER NUMBER image */
cn_img = readimage(clui_info->i4_file);
if (cn_img == NULL) {
   (void) kfprintf(kstderr,"lrftrain: Can not read cluster number image\n");
   kexit(KEXIT_FAILURE);  /* Quit if bad image */
}
CHECKTYPEINT(program, cn_img);
CHECKINPUT(program, cn_img);

/* Check for proper number of bands in images. */
if ( image->num_data_bands != (cc_img->num_data_bands - 1)) {
   kfprintf(kstderr,"lrfclass: The center/class image does not contain the correct\n");
   kfprintf(kstderr, "number of data_bands\n");
   kfprintf(kstderr, "Should have %d bands, but contains %d bands\n", (image->num_data_bands + 1), cc_img->num_data_bands);
   kfprintf(kstderr, "Please ensure that the classified (ie. supervised) image\n");
   kfprintf(kstderr, " has been appended to the cluster center image.\n");
   kexit(KEXIT_FAILURE);
}
if ( image->num_data_bands != var_img->num_data_bands) {
    kfprintf(kstderr, "lrfclass: The variance image does not contain the correct\n");
    kfprintf(kstderr, "number of data_bands\n");
    kfprintf(kstderr, "Should have %d bands, but contains %d bands\n", image->num_data_bands , var_img->num_data_bands);
    kexit(KEXIT_FAILURE);
}

if (clui_info->f_flag) {
   if ((printdev = kfoutput(clui_info->f_file)) == NULL)
   {
      (void) kfprintf(kstderr, "%s:  Can't open ascii output file: \n",program);
      kexit(KEXIT_FAILURE);
   }
}
else
  printdev = kstdout;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! llrftrain (image, cc_img, var_img, cn_img, &wt_img, clui_info->cv_float, 
                clui_info->meu_float, clui_info->b_int, clui_info->n_int, 
                clui_info->d_int, clui_info->delta_float, printdev) )
{
  (void) kfprintf(kstderr, "llrftrain Failed\n");
  kexit(KEXIT_FAILURE);
}
/* -main_library_call_end */

/* -main_after_lib_call */
/* Write out the weight image */
writeimage(clui_info->o_file, wt_img);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: lrftrain_usage_additions
| 
|       Purpose: Prints usage additions in lrftrain_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname lrftrain
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void lrftrain_usage_additions(void)
{
	kfprintf(kstderr, "\tCalculate Weights for Localized Receptive Field Classifier\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: lrftrain_free_args
| 
|       Purpose: Frees CLUI struct allocated in lrftrain_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname lrftrain
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
lrftrain_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i1_file);
		kfree_and_NULL(clui_info->i2_file);
		kfree_and_NULL(clui_info->i3_file);
		kfree_and_NULL(clui_info->i4_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->f_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
