 /*
  * Khoros: $Id: vbandsp1.c,v 1.11 1996/07/05 03:19:27 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vbandsp1.c,v 1.11 1996/07/05 03:19:27 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vbandsp1
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vbandsp1.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Extract a Specified Data Band from a Multiband Image  (K1)
|
|       Purpose: main program for vbandsp1
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->b_int; {Data band number to extract}
|		int clui_info->b_flag; {TRUE if -b specified}
|
|        Output:
|       Returns:
|
|    Written By: Tom Sauer
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;

xvimage *image, **images_list;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vbandsp1_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vbandsp1",
		vbandsp1_usage_additions, vbandsp1_get_args,
		vbandsp1_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
        kerror(NULL, "vbandsp1", "Program NOT executed!\n"
                "  vbandsp1 functionality is replaced by the "
                "DATAMANIP::kextact operator.\n"
                "  To get the same result using kextract, set the elements "
		"size to 1 \n  (-esize 1), and the elements offset (-eoff) "
		"to the old band number (-b).");
        kexit(KEXIT_FAILURE);

image = readimage(clui_info->i_file);
if (image == NULL) kexit(KEXIT_FAILURE);                   /* Quit if bad image */

                                  /* If only 1 band, dont continue */
if (image->num_data_bands == 1)
{
  (void) kfprintf(kstderr,"Input image only contains ONE data band!\n");
  kexit(KEXIT_FAILURE);
}

proper_num_images(program,image,(unsigned long) 1,TRUE);

if ( clui_info->b_int > image->num_data_bands) {
   kfprintf(kstderr, "%s: Cannot extract out data band number %d,", program,clui_info->b_int);
   kfprintf(kstderr," input image\nonly contains %d data bands.\n", image->num_data_bands );
   kexit(KEXIT_FAILURE);
}

 /* create a list of pointers to xvimage structures, the
    number in the list will be the number of data bands */

images_list = create_images_list(image->num_data_bands);
if ( images_list == NULL)
{
   (void) kfprintf(kstderr, "create_images_list Failed\n");
   kexit(KEXIT_FAILURE);
}
/* -main_before_lib_call_end */

/* -main_library_call */
/* separate out all bands into separate images */

if(! lvbandsp1(image, images_list, clui_info->i_file, TRUE))
{
   (void) kfprintf(kstderr, "lvbandsprt Failed\n");
   kexit(KEXIT_FAILURE);
}

/* since we can only extract one band, extract the band number
   specified by the -b option */

image = images_list[clui_info->b_int];
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file, image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vbandsp1_usage_additions
| 
|       Purpose: Prints usage additions in vbandsp1_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vbandsp1
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vbandsp1_usage_additions(void)
{
	kfprintf(kstderr, "\tExtract a Specified Data Band from a Multiband Image  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vbandsp1_free_args
| 
|       Purpose: Frees CLUI struct allocated in vbandsp1_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vbandsp1
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vbandsp1_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
