 /*
  * Khoros: $Id: vdrf.c,v 1.11 1996/07/05 03:20:58 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vdrf.c,v 1.11 1996/07/05 03:20:58 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vdrf
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vdrf.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Optimal Difference Recursive Filter for Edge Detection  (K1)
|
|       Purpose: main program for vdrf
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->a1_float; {a1 Exponential Filter Parameter for DRF}
|		int   clui_info->a1_flag; {TRUE if -a1 specified}
|
|		float clui_info->a2_float; {a2 Exponential Filter Parameter for DRF}
|		int   clui_info->a2_flag; {TRUE if -a2 specified}
|
|		int clui_info->w_int; {Window size for calculating the adaptive gradient}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->t1_int; {1st Hysteresis Threshold}
|		int clui_info->t1_flag; {TRUE if -t1 specified}
|
|		int clui_info->t2_int; {2nd Hysteresis Threshold}
|		int clui_info->t2_flag; {TRUE if -t2 specified}
|
|		int clui_info->l_int; {Minimum Pixel Number in a Segment}
|		int clui_info->l_flag; {TRUE if -l specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
    char *program = NULL;
    xvimage *image;
    int t1,t2,length,window;
    float a1,a2;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vdrf_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vdrf",
		vdrf_usage_additions, vdrf_get_args,
		vdrf_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    program = "vdrf";

    READINPUT (image);
    CHECKINPUT (program,image);

    t1     = clui_info->t1_int;
    t2     = clui_info->t2_int;
    length = clui_info->l_int;
    window = clui_info->w_int;
    if (((window/2)*2) == window) window--; /* the window size must be odd */
    a1     = clui_info->a1_float;
    a2     = clui_info->a2_float;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvdrf(image,a1,a2,t1,t2,length,window))
    {
        (void) kerror(NULL, NULL, "lvdrf Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vdrf_usage_additions
| 
|       Purpose: Prints usage additions in vdrf_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdrf
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vdrf_usage_additions(void)
{
	kfprintf(kstderr, "\tOptimal Difference Recursive Filter for Edge Detection  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vdrf_free_args
| 
|       Purpose: Frees CLUI struct allocated in vdrf_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vdrf
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vdrf_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
