 /*
  * Khoros: $Id: venhance.c,v 1.11 1996/07/05 03:21:06 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: venhance.c,v 1.11 1996/07/05 03:21:06 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for venhance
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "venhance.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Enhance Image Using the Local Standard Deviation & Mean  (K1)
|
|       Purpose: main program for venhance
|
|         Input:
|		char *clui_info->i_file; {input image filename}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image filename}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->w_int; {window width of local area}
|		int clui_info->w_flag; {TRUE if -w specified}
|
|		int clui_info->h_int; {window height of local area}
|		int clui_info->h_flag; {TRUE if -h specified}
|
|		float clui_info->k_float; {specifies scale factor}
|		int   clui_info->k_flag; {TRUE if -k specified}
|
|		float clui_info->v_float; {specifies minimum standard deviation}
|		int   clui_info->v_flag; {TRUE if -v specified}
|
|        Output:
|       Returns:
|
|    Written By: Marcelo Teran
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(venhance_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "venhance",
		venhance_usage_additions, venhance_get_args,
		venhance_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
image = readimage(clui_info->i_file);
    if (image == NULL){
        (void)kerror(NULL, NULL,"Image could not be read");
         kexit(1);               /* Quit if bad image */
    }

    CHECKINPUT(program,image);  /* Verify input image */
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvenhance(image,clui_info->w_int,clui_info->h_int,clui_info->k_float,
                clui_info->v_float)){
        (void) kerror(NULL, NULL, "lvhmed Failed");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: venhance_usage_additions
| 
|       Purpose: Prints usage additions in venhance_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname venhance
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void venhance_usage_additions(void)
{
	kfprintf(kstderr, "\tEnhance Image Using the Local Standard Deviation & Mean  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: venhance_free_args
| 
|       Purpose: Frees CLUI struct allocated in venhance_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname venhance
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
venhance_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
