 /*
  * Khoros: $Id: vgef.c,v 1.11 1996/07/05 03:21:53 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgef.c,v 1.11 1996/07/05 03:21:53 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgef
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgef.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - First Derivative Operator for Symmetric Exponential Filter  (K1)
|
|       Purpose: main program for vgef
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		float clui_info->a0_float; {Exponential Filter Parameter for GEF}
|		int   clui_info->a0_flag; {TRUE if -a0 specified}
|
|		int clui_info->t1_int; {1st Hysteresis Threshold}
|		int clui_info->t1_flag; {TRUE if -t1 specified}
|
|		int clui_info->t2_int; {2nd Hysteresis Threshold}
|		int clui_info->t2_flag; {TRUE if -t2 specified}
|
|		int clui_info->l_int; {Minimum Number of Pixels in a Segment}
|		int clui_info->l_flag; {TRUE if -l specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal ADAM
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
    char *program = NULL;
    xvimage *image;
    int t1,t2,length;
    float a0;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgef_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgef",
		vgef_usage_additions, vgef_get_args,
		vgef_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
    program = "vgef";

    READINPUT (image);
    CHECKINPUT (program,image);

    t1     = clui_info->t1_int;
    t2     = clui_info->t2_int;
    length = clui_info->l_int;
    a0     = clui_info->a0_float;
/* -main_before_lib_call_end */

/* -main_library_call */
if(! lvgef(image,a0,t1,t2,length))
    {
        (void) kerror(NULL, NULL, "lvgef Failed\n");
        kexit(1);
    }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgef_usage_additions
| 
|       Purpose: Prints usage additions in vgef_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgef
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgef_usage_additions(void)
{
	kfprintf(kstderr, "\tFirst Derivative Operator for Symmetric Exponential Filter\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgef_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgef_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgef
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgef_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
