 /*
  * Khoros: $Id: vgettilt.c,v 1.11 1996/07/05 03:22:01 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgettilt.c,v 1.11 1996/07/05 03:22:01 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgettilt
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgettilt.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute the Best-Fit Plane Parameters for Input File  (K1)
|
|       Purpose: main program for vgettilt
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->f_file; {output ASCII file}
|		int   clui_info->f_flag; {TRUE if -f specified}
|
|		char *clui_info->m_file; {statistics operation masking image}
|		int   clui_info->m_flag; {TRUE if -m specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson, Donna Koechner
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	char    *program = NULL;
	xvimage *img1,*img2;
	kfile   *printdev;
	double  xslope, yslope, piston;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgettilt_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgettilt",
		vgettilt_usage_additions, vgettilt_get_args,
		vgettilt_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	READINPUT(img1) ;
	READMASK(img2) ;
	CHECKINPUT(program, img1) ;
	MATCHINPUT(program, img1, img2) ;

	if (clui_info->f_flag)
	{
	  if ((printdev = kfoutput(clui_info->f_file)) == NULL)
	  {
	    kerror("vgettilt", "main", "Can't open ascii output file: %s\n",
		   clui_info->f_file);
	    kexit(KEXIT_FAILURE);
	  }
	}
	else
	  printdev = kstdout;
/* -main_before_lib_call_end */

/* -main_library_call */
	if (! lvgettilt(img1,img2,clui_info->m_flag,&xslope,&yslope,&piston) )
	{
	  kexit(KEXIT_FAILURE);
	}
/* -main_library_call_end */

/* -main_after_lib_call */
	(void) kfprintf(printdev,"\nImage Tilt Information for File Name: %s\n",
			clui_info->i_file);

	(void) kfprintf(printdev,"    X Slope: %g per pixel\n",xslope);
	(void) kfprintf(printdev,"    Y Slope: %g per pixel\n",yslope);
	(void) kfprintf(printdev,"    Piston : %g\n",piston);

	kfclose(printdev);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgettilt_usage_additions
| 
|       Purpose: Prints usage additions in vgettilt_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgettilt
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgettilt_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute the Best-Fit Plane Parameters for Input File  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgettilt_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgettilt_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgettilt
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgettilt_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->f_file);
		kfree_and_NULL(clui_info->m_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
