 /*
  * Khoros: $Id: vgrow.c,v 1.11 1996/07/05 03:22:26 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vgrow.c,v 1.11 1996/07/05 03:22:26 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vgrow
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vgrow.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Compute the Median Axis Inverse Transform  (K1)
|
|       Purpose: main program for vgrow
|
|         Input:
|		char *clui_info->i_file; {input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {output image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|        Output:
|       Returns:
|
|    Written By: Pascal Adam
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
	char    *program = NULL;
	xvimage *image;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vgrow_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vgrow",
		vgrow_usage_additions, vgrow_get_args,
		vgrow_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
	image = readimage(clui_info->i_file);
	if (image == NULL) {
	   kerror("vgrow", "main", "Can not read input image\n");
	   kexit(KEXIT_FAILURE);  /* Quit if bad image */
	}
        CHECKINPUT(program, image) ;
/* -main_before_lib_call_end */

/* -main_library_call */
	if (! lvgrow(image))
        {
           kexit(KEXIT_FAILURE);
        }
/* -main_library_call_end */

/* -main_after_lib_call */
	writeimage(clui_info->o_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vgrow_usage_additions
| 
|       Purpose: Prints usage additions in vgrow_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgrow
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vgrow_usage_additions(void)
{
	kfprintf(kstderr, "\tCompute the Median Axis Inverse Transform  (K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vgrow_free_args
| 
|       Purpose: Frees CLUI struct allocated in vgrow_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vgrow
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vgrow_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
