 /*
  * Khoros: $Id: vquant.c,v 1.11 1996/07/05 03:23:54 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: vquant.c,v 1.11 1996/07/05 03:23:54 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Main program for vquant
   >>>> 
   >>>>  Private: 
   >>>> 	main
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "vquant.h"

clui_info_struct *clui_info = NULL;

/*-----------------------------------------------------------
|
|  Routine Name: main() - Perform N-Dimensional Vector Quantization or Classification (K1)
|
|       Purpose: main program for vquant
|
|         Input:
|		char *clui_info->i_file; {Input image}
|		int   clui_info->i_flag; {TRUE if -i specified}
|
|		char *clui_info->o_file; {Output cluster number image}
|		int   clui_info->o_flag; {TRUE if -o specified}
|
|		int clui_info->n_int; {Number of vectors}
|		int clui_info->n_flag; {TRUE if -n specified}
|
|		int clui_info->m_logic; {Output Image Map Enable}
|		int clui_info->m_flag; {TRUE if -m specified}
|
|		int clui_info->s_logic; {Select split point:  false = mid-span, true = mean}
|		int clui_info->s_flag; {TRUE if -s specified}
|
|		int clui_info->axis_toggle; {Select splitting axis}
|		int clui_info->axis_flag; {TRUE if -axis specified}
|
|		char *clui_info->o1_file; {Output cluster center image}
|		int   clui_info->o1_flag; {TRUE if -o1 specified}
|
|		char *clui_info->o2_file; {Output cluster variance image}
|		int   clui_info->o2_flag; {TRUE if -o2 specified}
|
|        Output:
|       Returns:
|
|    Written By: Scott Wilson
|          Date: Jul 04, 1996
| Modifications:
|
------------------------------------------------------------*/

void main(
   int  argc,
   char **argv)
{

	kform  *pane;         /* form tree representing *.pane file */
/* -main_variable_list */
char *program = NULL;
xvimage *image,*ccimage,*cvimage;
/* -main_variable_list_end */

	khoros_initialize(argc, argv, "RETRO");
	kexit_handler(vquant_free_args, NULL);

/* -main_get_args_call */
	kclui_initialize(PANEPATH, KGEN_KROUTINE, "RETRO", "vquant",
		vquant_usage_additions, vquant_get_args,
		vquant_free_args);
/* -main_get_args_call_end */

/* -main_before_lib_call */
/* -main_before_lib_call_end */

/* -main_library_call */
READINPUT(image);
CHECKINPUT(program,image);

if (! lvquant(image,clui_info->n_int,clui_info->m_logic,&ccimage,
                    clui_info->o1_flag, &cvimage,
                    clui_info->o2_flag,
                    clui_info->s_logic,
                    clui_info->axis_toggle))
 {
   (void) kerror(NULL, NULL, "lvquant Failed\n");
   kexit(1);
 }
/* -main_library_call_end */

/* -main_after_lib_call */
writeimage(clui_info->o_file,image);
if (clui_info->o1_flag) writeimage(clui_info->o1_file,image);
if (clui_info->o2_flag) writeimage(clui_info->o2_file,image);
/* -main_after_lib_call_end */


	kexit(KEXIT_SUCCESS);
}


/*-----------------------------------------------------------
| 
|  Routine Name: vquant_usage_additions
| 
|       Purpose: Prints usage additions in vquant_usage routine
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vquant
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
void vquant_usage_additions(void)
{
	kfprintf(kstderr, "\tPerform N-Dimensional Vector Quantization or Classification\n");
	kfprintf(kstderr, "\t(K1)\n");

/* -usage_additions */
/* -usage_additions_end */

}
/*-----------------------------------------------------------
| 
|  Routine Name: vquant_free_args
| 
|       Purpose: Frees CLUI struct allocated in vquant_get_args()
| 
|         Input: None
| 
|        Output: None
|    Written By: ghostwriter -oname vquant
|          Date: Jul 04, 1996
| Modifications: 
| 
------------------------------------------------------------*/
/* ARGSUSED */
void
vquant_free_args(
    kexit_status status,
    kaddr        client_data)
{

	/* do the wild and free thing */
	if (clui_info != NULL)
	{
		kfree_and_NULL(clui_info->i_file);
		kfree_and_NULL(clui_info->o_file);
		kfree_and_NULL(clui_info->o1_file);
		kfree_and_NULL(clui_info->o2_file);
		kfree_and_NULL(clui_info);
	}

/* -free_handler_additions */
/* -free_handler_additions_end */
}
