 /*
  * Khoros: $Id: lvdyth.c,v 1.9 1996/04/05 00:55:07 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvdyth.c,v 1.9 1996/04/05 00:55:07 fas Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vdyth
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvdyth
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvdyth - Dynamic thresholding on a median or mean value
* 
*       Purpose: lvdyth performs dynamic thresholding on an image by either
*		 using the median value or mean value of a window as the
*		 threshold value.  The window is specified by its width
*		 argument, w, and its height argument, h.  Thresholding is
*		 computed by moving the window over the image, and at each new
*		 window location over the image a new threshold value is
*		 determined for the pixel currently at the center of the window.
*         Input: image - xvimage structure
*		 wsize - window width argument
*		 hsize - window height argument
*		 type  - type of thresholding  (0=median, 1=mean)
*		 value - non zero output pixel value
*        Output: image - holds the result of the thresholded image
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: lvdyth supports only BYTE data storage type images.
*    Written By: Marcelo Teran, Ramiro Jordan
*          Date: Apr 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: John Rasure on Fri Feb 28 18:41:18 MST 1992 fixed several
*		 indexing bugs and array bound problems.
****************************************************************/
/* -library_def */
int lvdyth(
	xvimage *img,
	int wsize,
	int hsize,
	int type,
	int value)
/* -library_def_end */

/* -library_code */
{

    int row,                            /* points image row */ 
        col,                            /* points image col */ 
        th,                             /* gives the mediun of the wind. */
        tmdn,                           /* numb of pixels <= median */ 
        g1,                             /* auxiliar variable */
        xoff,                           /* horiz. offset in the img */
        yoff,                           /* vertic. offset in the img */
        xend,                           /* step in img up to here */
        yend,                           /* step in img up to here */
        mdn,                            /* this is the median value */
        nc,                             /* column size of image */
        nr,                             /* row size of image */
        index,                          /* window counter */
        i, j;                           /* general counters */

    unsigned int hist[256];             /* holds histogram of window */

    unsigned char *ptr,                 /* pointer to the image data char */
                  *result;              /* pointer to resulting image char */

    char  *program;                     /* contains the name of the library */

    program = "lvdyth";

    propertype(program,img,(unsigned long) VFF_TYP_1_BYTE,1);         /* check img. type */     

    th = wsize * hsize / 2;

    nr = img->col_size;
    nc = img->row_size;

    xend = nc / wsize;
    yend = nr / hsize;
    tmdn = 0; mdn = 0;

   /* Zero histogram */
    kmemset((kaddr) hist, 0, (size_t) (256 * sizeof(unsigned int)));

   /* Allocate space for resulting image */ 
    result = (unsigned char *)kmalloc((size_t)nc*nr*sizeof(unsigned char));
    if(result == NULL) {
        kerror("vipl","lvhmed", "insufficient memory available\n");
        return(FALSE);
    }

   /* Assign image data address to ptr */
    ptr = (unsigned char *)img->imagedata;

   /* perform dynamic thresholding on an image */
    for (row = 0 ; row < yend; row++){

       /* step along the image rows */
        for(col = 0; col < xend; col++){ 

            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    index = row*hsize*nc + col*wsize + i*nc + j;
                    if(type == 0){
                        g1 = ptr[index];
/*                        hist[g1]++;*/  hist[g1] = hist[g1] + 1;
                    } else {
/*                        g1 += ptr[index]; */ g1 = g1 + ptr[index];
                    }
                }
            }
            if(type == 0){
               /* get the median value */
                if(tmdn > th){
                    do {
                        mdn = mdn - 1;
                        tmdn = tmdn - hist[mdn];
                    } while(tmdn > th && mdn > 0);
                } else {
                    while( mdn < 256 && (tmdn + hist[mdn] <= th) ){
                        tmdn = tmdn + hist[mdn];
                        mdn = mdn + 1;
                    }
                }
               /* Reset histogram and variables */
		kmemset((kaddr) hist, 0, (size_t)(256 * sizeof(unsigned int)));
                tmdn = 0;
            } else {
                mdn = g1 / (wsize * hsize);
            }

           /* write the new values for the image */
            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    index = row*hsize*nc + col*wsize + i*nc + j;
                    if(ptr[index] > (unsigned) mdn)
                        result[index] = value;
                }
            }
            mdn = 0; g1 = 0;
        }
    }

    if(nc != wsize*xend){
       /* step thru the lateral remaining side */
        yoff = nc - wsize;
        for(row = 0; row < yend; row++){
            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    index = yoff + row*hsize*nc + i*nc + j;
                    if(type == 0){
                        g1 = ptr[index];
/*                        hist[g1]++;*/ hist[g1] = hist[g1] +1;
                    } else {
/*                        g1 += ptr[index];*/ g1 = ptr[index] + g1;
                    }
                }
            }

            if(type == 0){
           /* get the median value */
                if(tmdn > th){
                    do {
                        mdn = mdn - 1;
                        tmdn = tmdn - hist[mdn];
                    } while(tmdn > th && mdn > 0);
                } else {
                    while( mdn < 256 && (tmdn + hist[mdn] <= th) ){
                        tmdn = tmdn + hist[mdn];
                        mdn = mdn + 1;
                    }
                }
               /* Reset histogram and variables */
		kmemset((kaddr) hist, 0, (size_t)(256 * sizeof(unsigned int)));
                tmdn = 0;
            } else {
           /* get the mean value */
                mdn = g1 / (wsize * hsize);
            }

           /* write the new values for the image */
            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    index = yoff + row*hsize*nc + i*nc + j;
                    if(ptr[index] > (unsigned) mdn)
                        result[index] = value;
                }
            }
        }
    }

    if(nr != yend*hsize){
       /* step thru the bottom remaining part */
        xoff = (nr - hsize) * nc;
        for(col = 0; col < xend; col++){
            /* xoff = (nr - hsize) * nr;  Rasure */
            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    index = xoff + col*wsize + i*nc + j;
                    if(type == 0){
                        g1 = ptr[index];
/*                        hist[g1]++;*/ hist[g1] = hist[g1] +1;
                    } else {
/*                        g1 += ptr[index];*/ g1 = ptr[index] + g1;
                    }
                }
            }

            if(type == 0){
               /* get the median value */
                if(tmdn > th){
                    do {
                        mdn = mdn - 1;
                        tmdn = tmdn - hist[mdn];
                    } while(tmdn > th && mdn >0);
                } else {
                    while( mdn < 256 && (tmdn + hist[mdn] <= th) ){
                        tmdn = tmdn + hist[mdn];
                        mdn = mdn + 1;
                    }
                }
               /* Reset histogram and variables */
		kmemset((kaddr) hist, 0, (size_t)(256 * sizeof(unsigned int)));
                tmdn = 0;
            } else {
               /* get the mean value */
                mdn = g1 / (wsize * hsize);
            }

       /* write the new values for the image */
            for(i = 0; i < hsize; i++){
                for(j = 0; j < wsize; j++){
                    /* index = xoff + row*hsize*nc + i*nc + j; 
                        seemed like Marcelo yanked the
                        wrong piece of code - John Rasure */
                    index = xoff + col*wsize + i*nc + j;
                    if(ptr[index] > (unsigned) mdn)
                        result[index] = value;
                }
            }
        }
    }

    kfree_and_NULL(img->imagedata);
    img->imagedata = (char *) result;

    return(TRUE);
}
/* -library_code_end */
