 /*
  * Khoros: $Id: lvsurf.c,v 1.12 1996/07/05 03:28:01 mirage Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: lvsurf.c,v 1.12 1996/07/05 03:28:01 mirage Exp $";
#endif

/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */

/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>> 
   >>>> 	Library Routine for vsurf
   >>>> 
   >>>>  Private: 
   >>>> 
   >>>>   Static: 
   >>>>   Public: 
   >>>> 	lvsurf
   >>>> 
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/* -library_includes */
#define MAG(x0,x1,x2) sqrt(x0*x0+x1*x1+x2*x2) 

static void crossp(
	float *v1,
	float *v2,
	float *norm)
{
    float m;
    /* Compute cross product in a LEFT HANDED COORDINATE SYSTEM */
    /* Form cross product norm = v1 X v2 */
    norm[0] =  -v1[1]*v2[2]+v1[2]*v2[1];
    norm[1] =  -v1[0]*v2[2]+v2[0]*v1[2];
    norm[2] =  -v1[0]*v2[1]+v2[0]*v1[1];
    m = sqrt((double)(norm[0]*norm[0]+norm[1]*norm[1]+norm[2]*norm[2]));
    norm[0] /= m;
    norm[1] /= m;
    norm[2] /= m;

    return;
}

static void find_norm_ul(
	double a,
	double b,
	double c,
	double size,
	float *norm)
{
    /* Compute normal for upper left corner geometry */
    float v1[3],v2[3];

    /* Load the vectors */
    v1[0] = 0;      /* X component */
    v1[1] = size;   /* Y component */
    v1[2] = a-b;    /* Z component */
    v2[0] = size;   /* X component */
    v2[1] = 0;      /* Y component */
    v2[2] = c-b;    /* Z component */

    /* Obtain cross-product */
    crossp(v1,v2,norm);

    return;
} 

static void find_norm_ur(
	double a,
	double b,
	double c,
	double size,
	float *norm)
{
    /* Compute normal for upper right corner geometry */
    float v1[3],v2[3];

    /* Load the vectors */
    v1[0] = -size;   /* X component */
    v1[1] = 0;      /* Y component */
    v1[2] = a-b;    /* Z component */
    v2[0] = 0;      /* X component */
    v2[1] = size;   /* Y component */
    v2[2] = c-b;    /* Z component */

    /* Obtain cross-product */
    crossp(v1,v2,norm);

    return;
} 

static void find_norm_lr(
	double a,
	double b,
	double c,
	double size,
	float *norm)
{
    /* Compute normal for lower right corner geometry */
    float v1[3],v2[3];

    /* Load the vectors */
    v1[0] = 0;      /* X component */
    v1[1] = -size;   /* Y component */
    v1[2] = a-b;    /* Z component */
    v2[0] = -size;   /* X component */
    v2[1] = 0;      /* Y component */
    v2[2] = c-b;    /* Z component */

    /* Obtain cross-product */
    crossp(v1,v2,norm);

    return;
}

static void find_norm_ll(
	double a,
	double b,
	double c,
	double size,
	float *norm)
{
    /* Compute normal for lower left corner geometry */
    float v1[3],v2[3];

    /* Load the vectors */
    v1[0] = size;   /* X component */
    v1[1] = 0;      /* Y component */
    v1[2] = a-b;    /* Z component */
    v2[0] = 0;      /* X component */
    v2[1] = -size;  /* Y component */
    v2[2] = c-b;    /* Z component */

    /* Obtain cross-product */
    crossp(v1,v2,norm);

    return;
}

static void compute_normal(
	float *surface,
	int x,
	int y,
	int rows,
	int cols,
	double pixsiz,
	float *normal)
{
    /* Compute the surface normal at each point in SURFACE and store
       this vector in the vector image NORMAL at the corresponding
       location. 
    */
    float norm1[3],norm2[3],norm3[3],norm4[3];

    if (x == 0 && y == 0)
      {
        /* Upper left corner */
        find_norm_ul((double)surface[PIXEL(x,y+1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x+1,y,rows,cols)],pixsiz,norm1);
        normal[BPIXEL(0,x,y,rows,cols)] = norm1[0];
        normal[BPIXEL(1,x,y,rows,cols)] = norm1[1];
        normal[BPIXEL(2,x,y,rows,cols)] = norm1[2];
      }
    else if (x == cols-1 && y == 0)
      {
        /* Upper right corner */
        find_norm_ur((double)surface[PIXEL(x-1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y+1,rows,cols)],pixsiz,norm1);
        normal[BPIXEL(0,x,y,rows,cols)] = norm1[0];
        normal[BPIXEL(1,x,y,rows,cols)] = norm1[1];
        normal[BPIXEL(2,x,y,rows,cols)] = norm1[2];
      }
    else if (x == cols-1 && y == rows-1)
      {
        /* Lower right corner */
        find_norm_lr((double)surface[PIXEL(x,y-1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x-1,y,rows,cols)],pixsiz,norm1);
        normal[BPIXEL(0,x,y,rows,cols)] = norm1[0];
        normal[BPIXEL(1,x,y,rows,cols)] = norm1[1];
        normal[BPIXEL(2,x,y,rows,cols)] = norm1[2];
      }
    else if (x == 0 && y == rows-1)
      {
        /* Lower left corner */
        find_norm_ll((double)surface[PIXEL(x+1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y-1,rows,cols)],pixsiz,norm1);
        normal[BPIXEL(0,x,y,rows,cols)] = norm1[0];
        normal[BPIXEL(1,x,y,rows,cols)] = norm1[1];
        normal[BPIXEL(2,x,y,rows,cols)] = norm1[2];
      }
    else if (y == 0)
      {
        /* Top edge */
        find_norm_ul((double)surface[PIXEL(x,y+1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x+1,y,rows,cols)],pixsiz,norm1);
        find_norm_ur((double)surface[PIXEL(x-1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y+1,rows,cols)],pixsiz,norm2);
        normal[BPIXEL(0,x,y,rows,cols)] = 0.5*(norm1[0]+norm2[0]);
        normal[BPIXEL(1,x,y,rows,cols)] = 0.5*(norm1[1]+norm2[1]);
        normal[BPIXEL(2,x,y,rows,cols)] = 0.5*(norm1[2]+norm2[2]);
      }
    else if (y == rows-1)
      {
        /* Bottom edge */
        find_norm_lr((double)surface[PIXEL(x,y-1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x-1,y,rows,cols)],pixsiz,norm1);
        find_norm_ll((double)surface[PIXEL(x+1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y-1,rows,cols)],pixsiz,norm2);
        normal[BPIXEL(0,x,y,rows,cols)] = 0.5*(norm1[0]+norm2[0]);
        normal[BPIXEL(1,x,y,rows,cols)] = 0.5*(norm1[1]+norm2[1]);
        normal[BPIXEL(2,x,y,rows,cols)] = 0.5*(norm1[2]+norm2[2]);
      }
    else if (x == 0)
      {
        /* Left edge */
        find_norm_ul((double)surface[PIXEL(x,y+1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x+1,y,rows,cols)],pixsiz,norm1);
        find_norm_ll((double)surface[PIXEL(x+1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y-1,rows,cols)],pixsiz,norm2);
        normal[BPIXEL(0,x,y,rows,cols)] = 0.5*(norm1[0]+norm2[0]);
        normal[BPIXEL(1,x,y,rows,cols)] = 0.5*(norm1[1]+norm2[1]);
        normal[BPIXEL(2,x,y,rows,cols)] = 0.5*(norm1[2]+norm2[2]);
      }
    else if (x == cols-1)
      {
        /* Right edge */
        find_norm_ur((double)surface[PIXEL(x-1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y+1,rows,cols)],pixsiz,norm1);
        find_norm_lr((double)surface[PIXEL(x,y-1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x-1,y,rows,cols)],pixsiz,norm2);
        normal[BPIXEL(0,x,y,rows,cols)] = 0.5*(norm1[0]+norm2[0]);
        normal[BPIXEL(1,x,y,rows,cols)] = 0.5*(norm1[1]+norm2[1]);
        normal[BPIXEL(2,x,y,rows,cols)] = 0.5*(norm1[2]+norm2[2]);
      }
    else
      {
        /* In the middle somewhere */
        find_norm_ul((double)surface[PIXEL(x,y+1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x+1,y,rows,cols)],pixsiz,norm1);
        find_norm_ur((double)surface[PIXEL(x-1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y+1,rows,cols)],pixsiz,norm2);
        find_norm_lr((double)surface[PIXEL(x,y-1,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x-1,y,rows,cols)],pixsiz,norm3);
        find_norm_ll((double)surface[PIXEL(x+1,y,rows,cols)],
                  (double)surface[PIXEL(x,y,rows,cols)],
                  (double)surface[PIXEL(x,y-1,rows,cols)],pixsiz,norm4);
        normal[BPIXEL(0,x,y,rows,cols)] = 0.25*(norm1[0]+norm2[0]+norm3[0]+norm4[0]);
        normal[BPIXEL(1,x,y,rows,cols)] = 0.25*(norm1[1]+norm2[1]+norm3[1]+norm4[1]);
        normal[BPIXEL(2,x,y,rows,cols)] = 0.25*(norm1[2]+norm2[2]+norm3[2]+norm4[2]);
      }

    return;
}

static void compute_angle(
	float *surface,
	float *normal,
	int x,
	int y,
	int rows,
	int cols,
	float *angle)
{
    /* Compute angle between surface normal and the surface reference plane */
    double ions[3],vec[3],dangle,dp;

    /* Load the ion source vector */
    ions[0] = 0.0;  /* X component */
    ions[1] = 0.0;  /* Y component */
    ions[2] = 1.0;  /* Z component */

    /* Load the surface normal vector */
    vec[0] = normal[BPIXEL(0,x,y,rows,cols)];
    vec[1] = normal[BPIXEL(1,x,y,rows,cols)];
    vec[2] = normal[BPIXEL(2,x,y,rows,cols)];

    dp = kdmatrix_inner_prod(ions,vec,3);
    dp = dp / (MAG(vec[0],vec[1],vec[2]) * MAG(ions[0],ions[1],ions[2]));
    dangle = acos(dp);
    angle[PIXEL(x,y,rows,cols)] = dangle*180.0/3.1415926; /* Degrees! */

    return;
}
/* -library_includes_end */


/****************************************************************
* 
*  Routine Name: lvsurf - Compute surface parameters (normal and angle)
* 
*       Purpose: lvsurf will take a surface image and produce two
*		 output images: one with the normals for each point
*		 on the surface and one with the angle between the
*		 surface normal and the normal to the zero plane of
*		 the image.
*
*		 The angle output image is in degrees.
*
*		 The output images are allocated inside lvsurf; the
*		 input image is not modified.
*         Input: img1 - Pointer to FLOAT image used to represent a surface.
*        Output: norm - Pointer to image pointer that will receive the
*			image containing the normal vectors to each pixel
*			on the surface.
*		 ang  - Pointer to image pointer that will receive the
*			image containing the angle between the surface
*			normal and the reference plane normal.
*       Returns: TRUE (1) on success, FALSE (0) on failure
*  Restrictions: Works only on FLOAT images.
*    Written By: Scott Wilson
*          Date: Jul 04, 1996
*      Verified: 
*  Side Effects: 
*      Examples: 
* Modifications: 7-Mar-91 Scott Wilson - Added transfer of pixel sizes
*					 from input to output
****************************************************************/
/* -library_def */
int lvsurf(
	xvimage *img1,
	xvimage **img2,
	xvimage **img3)
/* -library_def_end */

/* -library_code */
{
    /* GEOMETRY ASSUMPTIONS:

       1. X axis increases from left to right.
       2. Y axis increases from back to front.
       3. Z axis increases from bottom to top.

       Result: left handed coordinate system.
    */

    float *surface,*normal,*angle;
    xvimage *n,*a;
    int i,j;
    int rows,cols;
    double pixsiz;

    /* Generate the desired surface profile */
    cols = img1->row_size;
    rows = img1->col_size;
    pixsiz = (double) img1->pixsizx;

    surface = (float *)(img1->imagedata);

    /* Now compute surface normal */
    normal = (float *)kmalloc(rows*cols*3*sizeof(float));
    for (i=0; i<rows; i++)
      for (j=0; j<cols; j++)
        compute_normal(surface,j,i,rows,cols,pixsiz,normal);

    /* Now compute the angle between the entrance plane normal
       and the surface normal */
    angle = (float *)kmalloc((size_t)(rows*cols*sizeof(float)));
    for (i=0; i<rows; i++)
      for (j=0; j<cols; j++)
        compute_angle(surface,normal,j,i,rows,cols,angle);

    /* Fix up KHOROS headers for the images */
    n = createimage((unsigned long) rows,
		    (unsigned long) cols,
		    (unsigned long) VFF_TYP_FLOAT,
		    (unsigned long) 1,
		    (unsigned long) 3,"Optic surface normals",
                    (unsigned long) 0,
		    (unsigned long) 0,
		    (unsigned long) VFF_MS_NONE,
		    (unsigned long) VFF_MAPTYP_NONE,
		    (unsigned long) VFF_LOC_IMPLICIT,
		    (unsigned long) 0);
    n->imagedata = (char *)normal;
    n->pixsizx = img1->pixsizx;
    n->pixsizy = img1->pixsizy;
    *img2 = n;
    a = createimage((unsigned long) rows,
		    (unsigned long) cols,
		    (unsigned long) VFF_TYP_FLOAT,
                    (unsigned long) 1,
		    (unsigned long) 1,"Optic surface normal angles",
                    (unsigned long) 0,
		    (unsigned long) 0,
		    (unsigned long) VFF_MS_NONE,
		    (unsigned long) VFF_MAPTYP_NONE,
		    (unsigned long) VFF_LOC_IMPLICIT,
		    (unsigned long) 0);
    a->imagedata = (char *)angle;
    a->pixsizx = img1->pixsizx;
    a->pixsizy = img1->pixsizy;
    *img3 = a;

    return(TRUE);
}
/* -library_code_end */
