/*
  * Khoros: $Id: cp_imgdata.c,v 1.6 1996/04/05 00:56:12 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: cp_imgdata.c,v 1.6 1996/04/05 00:56:12 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		File Contents Title
   >>>>		
   >>>>   Public:
   >>>>                 copyimagedata()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */


#include "internals.h"

/************************************************************
*
*  Routine Name: copyimagedata - Copy image data
*
*       Purpose: Copy image data
*
*         Input: img1 - original image
*		 img2 - pointer to new copy of img1
*		 malloc_flg - if set (=1) malloc room
*        Output:
*       Returns: TRUE (1) on success, FALSE (0) otherwise
*  Restrictions:
*    Written By: Ron E. Neher
*          Date: 3/23/89 
*      Verified:
*  Side Effects:
* Modifications:
*************************************************************/
int copyimagedata(xvimage *img1, xvimage **img2, int malloc_flg)
{
long	lsize;

/* room for the image data */

    switch (img1->data_storage_type)
    {
	case VFF_TYP_BIT:
	    lsize = ((img1->row_size+7)/8)*img1->col_size;
	    break;
	case VFF_TYP_1_BYTE:
	    lsize = img1->col_size*img1->row_size;
	    break;
	case VFF_TYP_2_BYTE:
	    lsize = img1->col_size*img1->row_size*sizeof(short);
	    break;
	case VFF_TYP_4_BYTE:
	    lsize = img1->col_size*img1->row_size*sizeof(int);
	    break;
	case VFF_TYP_FLOAT:
	    lsize = img1->col_size*img1->row_size*sizeof(float);
	    break;
	case VFF_TYP_DOUBLE:
	    lsize = img1->col_size*img1->row_size*sizeof(double);
	    break;
	case VFF_TYP_COMPLEX:
	    lsize = img1->col_size*img1->row_size*sizeof(float)*2;
	    break;
	case VFF_TYP_DCOMPLEX:
	    lsize = img1->col_size*img1->row_size*sizeof(double)*2;
	    break;
	default:
	    (void) kfprintf(kstderr,"copyimagedata: Not valid data \
storage type\n");
	    return(0);
    }
    lsize = lsize*img1->num_of_images*img1->num_data_bands;
    if (malloc_flg)
        if (((*img2)->imagedata = kmalloc((size_t) lsize)) == NULL)
        {
	    (void) kfprintf(kstderr,"copyimagedata: Not enough memory for\
 image data!\n");
	    return(0);
        }

    kmemcpy((kaddr) (*img2)->imagedata, (kaddr) img1->imagedata,(size_t)lsize);
    (*img2)->data_storage_type = img1->data_storage_type;
    (*img2)->row_size = img1->row_size;
    (*img2)->col_size = img1->col_size;
    (*img2)->num_of_images = img1->num_of_images;
    (*img2)->num_data_bands = img1->num_data_bands;

    return(1);
}
