/*
  * Khoros: $Id: unloadvector.c,v 1.5 1996/04/05 00:56:24 fas Exp $
  */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: unloadvector.c,v 1.5 1996/04/05 00:56:24 fas Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		
   >>>>   Static:
   >>>>                 unload_char_vector()
   >>>>                 unload_short_vector()
   >>>>                 unload_int_vector()
   >>>>                 unload_float_vector()
   >>>>                 unload_complex_vector()
   >>>>                 unload_double_vector()
   >>>>                 unload_dcomplex_vector()
   >>>>   Public:
   >>>>                 unload_vector()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"	

static char *unload_char_vector PROTO((char **, int, int, int, int, int));
static short *unload_short_vector PROTO((short **, int, int, int, int, int));
static int *unload_int_vector PROTO((int **, int, int, int, int, int));
static float *unload_float_vector PROTO((float **, int, int, int, int, int));
static float *unload_complex_vector PROTO((float **, int, int, int, int, int));
static double *unload_double_vector PROTO((double **, int, int, int, int, int));
static double *unload_dcomplex_vector PROTO((double **, int, int, int, int, int));
/************************************************************
*
*  Routine Name:  unload_vector()
*
*       Purpose:  
*              Takes a 2-D array of vectorized data and rearanges the data so
*	       that the data is stored in an image format.
*
*	       Each vector represents a pixel and the dimension of the
*              vector represents the number of data bands for the image.
*              So the mth element of the vector represents t pixel in
*              the mth data band.
*              
*              The image is built by filling row by row from the
*	       linear vector data.
*
*              The border will be equal on all sides
*
*         Input: input_vectors - a pointer to the 2-D vectorized data
*		 border - the border width to skip.
*		 num_vects - total number of vectors
*		 dimension - vector dimension
*		 type - data type
*		 nr - number of rows in the image (col_size)
*		 nc - number of columns in the image (row_size)
*
*        Output: argument4 - explanation
*                argument5 - explanation
*
*       Returns: a pointer to a 1-d array containing the image data
*
*  Restrictions:
*    Written By:  Tom Sauer
*          Date:  Thu Apr  5 11:46:03 MDT 1990
*      Verified:
*  Side Effects:
* Modifications:
*
*************************************************************/
char *unload_vector(
	char	**input_vector,
	int	border,
	int	type,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{
    char  *image_data;
    float *float_image_data;
    double *double_image_data;
    int   *int_image_data;
    short *short_image_data;
    char  *program = "load_vector";


    if (type == VFF_TYP_BIT)
    {
          kfprintf(kstderr, "%s: Can not work on BIT data storage type\n",
			program);
          kfprintf(kstderr, "Can only convert type Byte, Short, Integer, ");
          kfprintf(kstderr, "Float\n and Complex\n");
          return NULL;
    }

    switch(type) 
    {

       case VFF_TYP_1_BYTE:
          image_data = unload_char_vector((char **) input_vector, border, 
				  	   num_vects, dimension, nr, nc);
          if (image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_char_vector failed\n", program);
             return(NULL);
          }
          break;

       case VFF_TYP_2_BYTE:
          short_image_data = unload_short_vector((short **) input_vector, border,
					          num_vects, dimension, nr, nc);
          if (short_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_short_vector failed\n", program);
             return(NULL);
          }
          image_data = (char *) short_image_data;
          break;

       case VFF_TYP_4_BYTE:
          int_image_data = unload_int_vector((int **) input_vector, border,
					         num_vects, dimension, nr, nc);
          if (int_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_int_vector failed\n", program);
             return(NULL);
          }
          image_data = (char *) int_image_data;
          break;

       case VFF_TYP_FLOAT:
          float_image_data = unload_float_vector((float **)input_vector, border,
 						  num_vects, dimension, nr, nc);
          if (float_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_float_vector failed\n", program);
             return(NULL);
          }
          image_data = (char *) float_image_data;
          break;

       case VFF_TYP_COMPLEX:
          float_image_data = unload_complex_vector((float **)input_vector,
				border, num_vects, dimension, nr, nc);
          if (float_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_complex_vector failed\n", program);
             return(NULL);
          }

          image_data = (char *) float_image_data;
          break;

       case VFF_TYP_DOUBLE:
          double_image_data = unload_double_vector((double **)input_vector, 
			        border, num_vects, dimension, nr, nc);
          if (double_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_double_vector failed\n", program);
             return(NULL);
          }
          image_data = (char *) double_image_data;
          break;

       case VFF_TYP_DCOMPLEX:
          double_image_data = unload_dcomplex_vector((double **)input_vector,
				border, num_vects, dimension, nr, nc);
          if (double_image_data == NULL)
          {
             kfprintf(kstderr, "%s: unload_dcomplex_vector failed\n", program);
             return(NULL);
          }

          image_data = (char *) double_image_data;
          break;

       default:
          kfprintf(kstderr, "%s: Invalid image Data Storage type\n", program);
          kfprintf(kstderr, "Can only convert type Byte, Short, Integer, ");
          kfprintf(kstderr, "Float\nand Complex\n");
          return(NULL);
      }

       return(image_data);
}

/*-----------------------------------------------------------
|
|  Routine Name: unload_char_vector
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|	 	 border - the border width to skip.
|		 num_vects - total number of vectors
|		 dimension - vector dimension
|		 nr - number of rows in the image (col_size)
|		 nc - number of columns in the image (row_size)
|
|        Output:
|
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By: Tom Sauer
|          Date:  Thu Apr  5 11:46:03 MDT 1990
| Modifications:
|
------------------------------------------------------------*/
static char *unload_char_vector(
	char	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{
    char *imagedata, *cptr;
    int m, i, j, k;
    char *program = "unload_float_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (char *)kmalloc((unsigned int) nr * nc * dimension * sizeof (char));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (char));

          /* Assign vectors to the image data */

    cptr = (char *) imagedata;

          /* Assign ptr to input vectors minus the border */

    m = 0;
    for (i = border; i < nr - border; i++) 
    {
       for (j = border; j < nc - border; j++) 
       {
	  for (k = 0; k < dimension; k++) 
          {
	      cptr[(i * nc + j) + (k * nc * nr)] = input_vectors[m][k];
	  }
          m++;
       }
    }

       /* return the pointer to the image data */
    return(cptr);
}

/*-----------------------------------------------------------
|
|  Routine Name:  unload_short_vector()
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|		border - the border width to skip.
|		num_vects - total number of vectors
|		dimension - vector dimension
|		nr - number of rows in the image (col_size)
|		nc - number of columns in the image (row_size)
|
|        Output:
|
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Thu Apr  5 11:46:03 MDT 1990
| Modifications:
|
------------------------------------------------------------*/
static short *unload_short_vector(
	short	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    short *imagedata, *sptr;
    int m, i, j, k;
    char *program = "unload_float_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (short *)kmalloc((unsigned int) nr * nc * dimension * sizeof (short));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (short));

          /* Assign vectors to the image data */

    sptr = (short *) imagedata;

          /* Assign ptr to input vectors minus the border */

    m = 0;
    for (i = border; i < nr - border; i++) 
    {
       for (j = border; j < nc - border; j++) 
       {
	  for (k = 0; k < dimension; k++) 
          {
	      sptr[(i * nc + j) + (k * nc * nr)] = input_vectors[m][k];
	  }
          m++;
       }
    }

       /* return the pointer to the image data */
    return(sptr);
}


/*-----------------------------------------------------------
|
|  Routine Name:  unload_int_vector()
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|		 border - the border width to skip.
|		 num_vects - total number of vectors
|		 dimension - vector dimension
|		 nr - number of rows in the image (col_size)
|		 nc - number of columns in the image (row_size)
|        Output: 
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Thu Apr  5 11:46:03 MDT 1990
| Modifications:
|
------------------------------------------------------------*/
static int *unload_int_vector(
	int	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    int *imagedata, *iptr;
    int m, i, j, k;
    char *program = "unload_int_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (int *)kmalloc((unsigned int) nr * nc * dimension * sizeof (int));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (int));

          /* Assign vectors to the image data */

    iptr = (int *) imagedata;

          /* Assign ptr to input vectors minus the border */

    m = 0;
    for (i = border; i < nr - border; i++) 
    {
       for (j = border; j < nc - border; j++) 
       {
	  for (k = 0; k < dimension; k++) 
          {
	      iptr[(i * nc + j) + (k * nc * nr)] = input_vectors[m][k];
	  }
          m++;
       }
    }

       /* return the pointer to the image data */
    return(iptr);
}

/*-----------------------------------------------------------
|
|  Routine Name:  unload_float_vector()
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|		 border - the border width to skip.
|		 num_vects - total number of vectors
|		 dimension - vector dimension
|		 nr - number of rows in the image (col_size)
|		 nc - number of columns in the image (row_size)
|        Output:
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Thu Apr  5 11:46:03 MDT 1990
| Modifications:
|
------------------------------------------------------------*/
static float *unload_float_vector(
	float	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    float *imagedata, *fptr;
    int m, i, j, k;
    char *program = "unload_float_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (float *)kmalloc((unsigned int) nr * nc * dimension * sizeof (float));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (float));

          /* Assign vectors to the image data */

    fptr = (float *) imagedata;

          /* Assign ptr to input vectors minus the border */

    m = 0;
    for (i = border; i < nr - border; i++) 
    {
       for (j = border; j < nc - border; j++) 
       {
	  for (k = 0; k < dimension; k++) 
          {
	      fptr[(i * nc + j) + (k * nc * nr)] = input_vectors[m][k];
	  }
          m++;
       }
    }

       /* return the pointer to the image data */
    return(fptr);
}


/*-----------------------------------------------------------
|
|  Routine Name: unload_complex_vector
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|		 border - the border width to skip.
|		 num_vects - total number of vectors
|		 dimension - vector dimension
|		 nr - number of rows in the image (col_size)
|		 nc - number of columns in the image (row_size)
|        Output:
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Thu Apr  5 11:46:03 MDT 1990
| Modifications:
|
------------------------------------------------------------*/
static float *unload_complex_vector(
	float	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    float *imagedata, *fptr;
    int m, i, j, k, p, q;
    char *program = "unload_complex_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (float *)kmalloc((unsigned int) nr * nc * dimension * 2  * sizeof (float));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (float));

    fptr = (float *) imagedata;

          /* Assign ptr to input vectors minus the border */
          /* Assign vectors to the image data */

    m = 0;
    for (i = border; i < nr - border; i++)  
    {
        q = border*2;
        for (j = border; j < nc - border; j++ ) 
        {
            p = 0;
            for (k = 0; k < dimension; k++) 
            {
     	        fptr[(i * nc*2 + q)+(k * nc * nr*2)] = input_vectors[m][p];
                p++;
		fptr[(i * nc*2 + q+1)+(k * nc *nr*2)] = input_vectors[m][p];
                p++;
	    }
                 q +=2;
	         m++;
	 }
     }

       /* return the pointer to the image data */
    return(fptr);
}

/*-----------------------------------------------------------
|
|  Routine Name:  unload_double_vector()
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input: input_vectors - a pointer to the 2-D vectorized data
|		 border - the border width to skip.
|		 num_vects - total number of vectors
|		 dimension - vector dimension
|		 nr - number of rows in the image (col_size)
|		 nc - number of columns in the image (row_size)
|        Output:
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Mon Feb 25 15:15:57 MST 1991
| Modifications:
|
------------------------------------------------------------*/
static double *unload_double_vector(
	double	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    double *imagedata, *dptr;
    int m, i, j, k;
    char *program = "unload_float_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (double *)kmalloc((unsigned int) nr * nc * dimension * sizeof (double));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (double));

          /* Assign vectors to the image data */

    dptr = (double *) imagedata;

          /* Assign ptr to input vectors minus the border */

    m = 0;
    for (i = border; i < nr - border; i++) 
    {
       for (j = border; j < nc - border; j++) 
       {
	  for (k = 0; k < dimension; k++) 
          {
	      dptr[(i * nc + j) + (k * nc * nr)] = input_vectors[m][k];
	  }
          m++;
       }
    }

       /* return the pointer to the image data */
    return(dptr);
}

/*-----------------------------------------------------------
|
|  Routine Name:  unload_dcomplex_vector()
|
|       Purpose: Takes a 2-D array of vectorized data and rearanges the data so
|	         that the data is stored in an image format.
|
|	         Each vector represents a pixel and the dimension of the
|                vector represents the number of data bands for the image.
|                So the mth element of the vector represents t pixel in
|                the mth data band.
|              
|                The image is built by filling row by row from the
|	         linear vector data.
|
|                The border will be equal on all sides
|
|         Input:
|               input_vectors - a pointer to the 2-D vectorized data
|		border - the border width to skip.
|		num_vects - total number of vectors
|		dimension - vector dimension
|		nr - number of rows in the image (col_size)
|		nc - number of columns in the image (row_size)
|        Output:
|       Returns: a pointer to a 1-d array containing the image data
|
|    Written By:  Tom Sauer
|          Date:  Mon Feb 25 15:18:41 MST 1991
| Modifications:
|
------------------------------------------------------------*/
static double *unload_dcomplex_vector(
	double	**input_vectors,
	int	border,
	int	num_vects,
	int	dimension,
	int	nr,
	int	nc)
{

    double *imagedata, *dptr;
    int m, i, j, k, p, q;
    char *program = "unload_dcomplex_vector";

       /* check to make sure there is enough vector data to fill
        * the image less the border
        */

    if ((nr-(border*2)) * (nc-(border*2)) != num_vects)
    {
       (void)kfprintf(kstderr,
                      "%s: the number of vectors is not equal to\n",program);
       (void)kfprintf(kstderr,"the image rows + border * image cols + border\n");
       return(NULL);
    }

            /* Allocate space for image data */

    imagedata = (double *)kmalloc((unsigned int) nr * nc * dimension * 2  * sizeof (double));
    if (imagedata == NULL) 
    {
        (void) kfprintf (kstderr, "%s: insufficient memory available\n",program);
	return (NULL);
    }

          /* zero the image data */
    (void) kmemset((kaddr) imagedata, 0, (unsigned) nr * nc * dimension * sizeof (double));

    dptr = (double *) imagedata;

          /* Assign ptr to input vectors minus the border */
          /* Assign vectors to the image data */

    m = 0;
    for (i = border; i < nr - border; i++)  
    {
        q = border*2;
        for (j = border; j < nc - border; j++ ) 
        {
            p = 0;
            for (k = 0; k < dimension; k++) 
            {
     	        dptr[(i * nc*2 + q)+(k * nc * nr*2)] = input_vectors[m][p];
                p++;
		dptr[(i * nc*2 + q+1)+(k * nc *nr*2)] = input_vectors[m][p];
                p++;
	    }
                 q +=2;
	         m++;
	 }
     }

       /* return the pointer to the image data */
    return(dptr);
}
