/* Khoros: $Id: viff.c,v 1.6 1996/05/14 17:26:09 mirage Exp $ */

#if !defined(__lint) && !defined(__CODECENTER__)
static char rcsid[] = "Khoros: $Id: viff.c,v 1.6 1996/05/14 17:26:09 mirage Exp $";
#endif


/*
 * Copyright (C) 1993 - 1996, Khoral Research, Inc., ("KRI").
 * All rights reserved.  See $BOOTSTRAP/repos/license/License or run klicense.
 */


/* >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<<
   >>>>
   >>>>		Library Routine for vutils
   >>>>		
   >>>>  Private:
   >>>>                 imagesize()
   >>>>                 writeimage()
   >>>>                 readheader()
   >>>>                 readimage()
   >>>>                 freeimage()
   >>>>                 createimage()
   >>>>
   >>>>>>>>>>>>>>>>>>>>>>>>>>>>> <<<<<<<<<<<<<<<<<<<<<<<<<< */

#include "internals.h"

int imagesize (
	xvimage *image,
	int *dsize,
	int *dcount,
	int *msize,
	int *mcount,
	int *lsize,
	int *lcount)
{
	return(viff_size(image,dsize,dcount,msize,mcount,lsize,lcount));
}

int writeimage (
	char *filename,
	xvimage *image)
{
	return(viff_write(filename, image));
}

xvimage *readheader (
	int fid)
{
	return(viff_readheader(fid));
}

xvimage *readimage (
	char *filename)
{
	kobject object;
	int col_size = 0;
        int row_size = 0;
        int data_storage_type = KUBYTE;
        unsigned long dtype = VFF_TYP_1_BYTE;
        unsigned long mtype = VFF_MAPTYP_NONE;
        int num_of_images = 0;
        int num_data_bands = 0;
        char *comment = NULL;
        int map_row_size = 0;
        int map_col_size = 0;
        int map_scheme = VFF_MS_NONE;
        int map_enable = KOPTIONAL;
        int map_storage_type;
        unsigned long location_type = VFF_LOC_IMPLICIT;
        int location_dim = 0;
        int i = 0;
	kaddr valdata = NULL, mapdata = NULL;
	float *locdata = NULL;
	xvimage *image;
	unsigned long ispare1 = 0;
	unsigned long ispare2 = 0;
	float fspare1 = 0.0;
	float fspare2 = 0.0;


	object = kpds_open_input_object(filename);
        /* kevins patch */
        if (object == NULL) return(NULL);
       /* kevins patch end */

	if (kpds_query_value(object))
	{
		kpds_get_attributes(object, KPDS_VALUE_SIZE, &col_size,
				    &row_size, &i, &num_of_images,
				    &num_data_bands,
				    KPDS_VALUE_DATA_TYPE, &data_storage_type,
				    NULL);
		khoros2viff_datatype(data_storage_type, &dtype);
		num_data_bands = num_data_bands * i * num_of_images;
		num_of_images = 1;
		valdata = kpds_get_data(object, KPDS_VALUE_ALL, NULL);
		if (data_storage_type == KBIT)
		{
		   /* if its bit data, it likes it to be LSB. */
		   kdata_cast(valdata, KBIT_LSB, KBIT, 
			      col_size * row_size * num_data_bands,
			      KNONE, 1.0, 1.0, valdata);
		}
		
	}
	kpds_get_attribute(object, KPDS_COMMENT, &comment);
	if (kpds_query_map(object))
	{
		kpds_get_attributes(object, KPDS_MAP_SIZE, &map_row_size,
				    &map_col_size, NULL, NULL, NULL,
				    KPDS_MAP_DATA_TYPE, &map_storage_type,
				    NULL);
		khoros2viff_maptype(map_storage_type, &mtype);
		mapdata = kpds_get_data(object, KPDS_MAP_ALL, NULL);
	}
	   
	if (kpds_query_location(object))
	{
		kpds_get_attribute(object, KPDS_LOCATION_SIZE, NULL,
				    NULL, NULL, &location_dim);
		kpds_set_attribute(object, KPDS_LOCATION_DATA_TYPE, KFLOAT);
		locdata = (float *) kpds_get_data(object, KPDS_LOCATION_ALL,
						  NULL);
	}
	if (kpds_query_attribute(object, "xvimageMapEnable", NULL, NULL, 
				 NULL, NULL))
	   kpds_get_attribute(object, "xvimageMapEnable", &map_enable);

	if (kpds_query_attribute(object, "xvimageMapScheme", NULL, NULL, 
				 NULL, NULL))
	   kpds_get_attribute(object, "xvimageMapScheme", &map_scheme);

	image = createimage((unsigned long) row_size, (unsigned long) col_size,
			    dtype, (unsigned long) num_of_images,
			    (unsigned long) num_data_bands, comment,
			    (unsigned long) map_row_size,
			    (unsigned long) map_col_size,
			    (unsigned long) map_scheme, mtype, location_type,
			    (unsigned long) location_dim);

	image->map_enable = (map_enable == KFORCED) ? VFF_MAP_FORCE 
	                                            : VFF_MAP_OPTIONAL;

	/* -- handle ispare and fspare cases -- */
	if (kdms_get_attribute(object, NULL, "ispare1", &ispare1))
	   image->ispare1 = ispare1;

	if (kdms_get_attribute(object, NULL, "ispare2", &ispare2))
	   image->ispare2 = ispare2;

	/* luxury exception for spectrum */
	if (kdms_get_attribute(object, NULL, "colorMapContents", &ispare1))
	   image->ispare1 = ispare1;

	/* luxury exception for spectrum */
	if (kdms_get_attribute(object, NULL, "colorMapColnum", &ispare2))
	   image->ispare2 = ispare2;
   
	if (kdms_get_attribute(object, NULL, "fspare1", &fspare1))
	   image->fspare1 = fspare1;

	if (kdms_get_attribute(object, NULL, "fspare2", &fspare2))
	   image->fspare2 = fspare2;


	if (image == NULL)
		return(NULL);
	kfree_and_NULL(image->maps);
	image->maps = (char *) mapdata;
	kfree_and_NULL(image->location);
	image->location = locdata;
	kfree_and_NULL(image->imagedata);
	image->imagedata = (char *) valdata;
	return(image);
}

int freeimage (
	xvimage *image)
{
	return(viff_free(image));
}

xvimage *createimage (
	unsigned long col_size,
	unsigned long row_size,
	unsigned long data_storage_type,
	unsigned long num_of_images,
	unsigned long num_data_bands,
	char         *comment,
	unsigned long map_row_size,
	unsigned long map_col_size,
	unsigned long map_scheme,
	unsigned long map_storage_type,
	unsigned long location_type,
	unsigned long location_dim)
{
	return(viff_create(col_size, row_size, data_storage_type,
			num_of_images, num_data_bands, comment, map_row_size,
			map_col_size, map_scheme, map_storage_type,
			location_type, location_dim));
}
