#! /bin/sh

echo "running test.sh for vbandsp3 ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="three band separate, extract starting at band 1, byte"

KHOROS_KDMS_FORMAT=viff
export KHOROS_KDMS_FORMAT

cat <<EOF > $tmpdir/ascii.$$
1 11 12 13
1 14 15 16
1 17 18 19
2 21 22 23
2 24 25 26
2 27 28 29
3 31 32 33
3 34 35 36
3 37 38 39
4 40 41 42
4 43 44 45
4 46 47 48
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 4 -type byte -o $tmpdir/out1.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
2 21 22 23
2 24 25 26
2 27 28 29
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type byte -o $tmpdir/result1.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
3 31 32 33
3 34 35 36
3 37 38 39
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type byte -o $tmpdir/result2.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
4 40 41 42
4 43 44 45
4 46 47 48
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type byte -o $tmpdir/result3.$$

${BINARY_PATH}/vbandsp3 -i $tmpdir/out1.$$ -b 1 -o1 out1.$$.1 -o2 out1.$$.2 -o3 out1.$$.3

test_num="1a"
descr="three band separate, extract starting at band 1, extract band 2, byte"
report $test_num "out1.$$.1" "$tmpdir/result1.$$" "$descr"
test_num="1b"
descr="three band separate, extract starting at band 1, extract band 3, byte"
report $test_num "out1.$$.2" "$tmpdir/result2.$$" "$descr"
test_num="1c"
descr="three band separate, extract starting at band 1, extract band 4, byte"
report $test_num "out1.$$.3" "$tmpdir/result3.$$" "$descr"

/bin/rm -f out*.$$* $tmpdir/out*.$$*  $tmpdir/result*.$$ $tmpdir/ascii.$$

# ---------- Test 2 --> test description

test_num=2
descr="three band separate, extract starting at band 0, float"

KHOROS_KDMS_FORMAT=viff
export KHOROS_KDMS_FORMAT

cat <<EOF > $tmpdir/ascii.$$
1.5 11.6 12.7 13.8
1.5 14.6 15.7 16.8
1.5 17.6 18.7 19.8
2.5 21.6 22.7 23.8
2.5 24.6 25.7 26.8
2.5 27.6 28.7 29.8
3.5 31.6 32.7 33.8
3.5 34.6 35.7 36.8
3.5 37.6 38.7 39.8
4.5 40.6 41.7 42.8
4.5 43.6 44.7 45.8
4.5 46.6 47.7 48.8
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 4 -type float -o $tmpdir/out2.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
1.5 11.6 12.7 13.8
1.5 14.6 15.7 16.8
1.5 17.6 18.7 19.8
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type float -o $tmpdir/result1.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
2.5 21.6 22.7 23.8
2.5 24.6 25.7 26.8
2.5 27.6 28.7 29.8
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type float -o $tmpdir/result2.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
3.5 31.6 32.7 33.8
3.5 34.6 35.7 36.8
3.5 37.6 38.7 39.8
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type float -o $tmpdir/result3.$$

${BINARY_PATH}/vbandsp3 -i $tmpdir/out2.$$ -o1 out2.$$.1 -o2 out2.$$.2 -o3 out2.$$.3 -b 0

test_num=2a
descr="three band separate, extract starting at band 0, extract band 0, float"
report $test_num "out2.$$.1" "$tmpdir/result1.$$" "$descr"
test_num=2b
descr="three band separate, extract starting at band 0, extract band 1, float"
report $test_num "out2.$$.2" "$tmpdir/result2.$$" "$descr"
test_num=2c
descr="three band separate, extract starting at band 0, extract band 2, float"
report $test_num "out2.$$.3" "$tmpdir/result3.$$" "$descr"

/bin/rm -f out*.$$* $tmpdir/out*.$$*  $tmpdir/result*.$$ $tmpdir/ascii.$$


# ---------- Test 3 --> test description
test_num=3
descr="single band separate, extract band 2, long"

KHOROS_KDMS_FORMAT=viff
export KHOROS_KDMS_FORMAT

cat <<EOF > $tmpdir/ascii.$$
1 11 12 13
1 14 15 16
1 17 18 19
2 21 22 23
2 24 25 26
2 27 28 29
3 31 32 33
3 34 35 36
3 37 38 39
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 3 -type long -o $tmpdir/out3.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
1 11 12 13
1 14 15 16
1 17 18 19
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type long -o $tmpdir/result1.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
2 21 22 23
2 24 25 26
2 27 28 29
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type long -o $tmpdir/result2.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
3 31 32 33
3 34 35 36
3 37 38 39
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type long -o $tmpdir/result3.$$

${BINARY_PATH}/vbandsp3 -i $tmpdir/out3.$$ -b 0

test_num=3a
descr="three band separate, extract starting at band 0, extract band 0, long"
report $test_num "out3.$$.1" "$tmpdir/result1.$$" "$descr"
test_num=3b
descr="three band separate, extract starting at band 0, extract band 1, long"
report $test_num "out3.$$.2" "$tmpdir/result2.$$" "$descr"
test_num=3c
descr="three band separate, extract starting at band 0, extract band 2, long"
report $test_num "out3.$$.3" "$tmpdir/result3.$$" "$descr"

/bin/rm -f out*.$$* $tmpdir/out*.$$*  $tmpdir/result*.$$ $tmpdir/ascii.$$

# ---------- Test 4 --> test description
test_num=4
descr="single band separate, extract band 2, complex"

KHOROS_KDMS_FORMAT=viff
export KHOROS_KDMS_FORMAT

cat <<EOF > $tmpdir/ascii.$$
(1.5,4.0) (11.6,4.0) (12.7,4.0) (13.8,4.0)
(1.5,4.0) (14.6,4.0) (15.7,4.0) (16.8,4.0)
(1.5,4.0) (17.6,4.0) (18.7,4.0) (19.8,4.0)
(2.5,4.0) (21.6,4.0) (22.7,4.0) (23.8,4.0)
(2.5,4.0) (24.6,4.0) (25.7,4.0) (26.8,4.0)
(2.5,4.0) (27.6,4.0) (28.7,4.0) (29.8,4.0)
(3.5,4.0) (31.6,4.0) (32.7,4.0) (33.8,4.0)
(3.5,4.0) (34.6,4.0) (35.7,4.0) (36.8,4.0)
(3.5,4.0) (37.6,4.0) (38.7,4.0) (39.8,4.0)
(4.5,4.0) (40.6,4.0) (41.7,4.0) (42.8,4.0)
(4.5,4.0) (43.6,4.0) (44.7,4.0) (45.8,4.0)
(4.5,4.0) (46.6,4.0) (47.7,4.0) (48.8,4.0)
EOF
kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 4 -type complex -o $tmpdir/out.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
(1.5,4.0) (11.6,4.0) (12.7,4.0) (13.8,4.0)
(1.5,4.0) (14.6,4.0) (15.7,4.0) (16.8,4.0)
(1.5,4.0) (17.6,4.0) (18.7,4.0) (19.8,4.0)
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type complex -o $tmpdir/result1.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
(2.5,4.0) (21.6,4.0) (22.7,4.0) (23.8,4.0)
(2.5,4.0) (24.6,4.0) (25.7,4.0) (26.8,4.0)
(2.5,4.0) (27.6,4.0) (28.7,4.0) (29.8,4.0)
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type complex -o $tmpdir/result2.$$

/bin/rm -f $tmpdir/ascii.$$ 
cat <<EOF > $tmpdir/ascii.$$
(3.5,4.0) (31.6,4.0) (32.7,4.0) (33.8,4.0)
(3.5,4.0) (34.6,4.0) (35.7,4.0) (36.8,4.0)
(3.5,4.0) (37.6,4.0) (38.7,4.0) (39.8,4.0)
EOF

kasc2val -i1 $tmpdir/ascii.$$ -wsize 4 -hsize 3 -esize 1 -type complex -o $tmpdir/result3.$$

${BINARY_PATH}/vbandsp3 -i $tmpdir/out.$$ -o1 out.$$.1 -o2 out.$$.2 -o3 out.$$.3 -b 0

test_num=4a
descr="three band separate, extract starting at band 0, extract band 0, complex"
report $test_num "out.$$.1" "$tmpdir/result1.$$" "$descr"
test_num=4b
descr="three band separate, extract starting at band 0, extract band 1, complex"
report $test_num "out.$$.2" "$tmpdir/result2.$$" "$descr"
test_num=4c
descr="three band separate, extract starting at band 0, extract band 2, complex"
report $test_num "out.$$.3" "$tmpdir/result3.$$" "$descr"

/bin/rm -f out*.$$* $tmpdir/out*.$$* $tmpdir/result*.$$ $tmpdir/ascii.$$
 

exit $status
