#! /bin/sh

echo "running test.sh for vcast ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="vcast converting to float"

${BINARY_PATH}/vcast -i1 results/test_input.xv -i2 results/test2_input.xv -o1 $tmpdir/out1.$$ -o2 $tmpdir/out2.$$ 

report $test_num "$tmpdir/out1.$$" "./results/test1_results.xv" "$descr"
report $test_num "$tmpdir/out2.$$" "./results/test2_results.xv" "$descr"
/bin/rm -f $tmpdir/out*.$$ 

exit $status
