#! /bin/sh

echo "running test.sh for vdetilt ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="Remove tilt from NGL image, Mean not removed from image"

${BINARY_PATH}/vdetilt -i ./results/test_input.xv -o $tmpdir/out.$$ -p 0

report $test_num "$tmpdir/out.$$" "./results/test1_result.xv" "$descr"
/bin/rm -f $tmpdir/out.$$ 

# ---------- Test 2 --> test description
 
test_num=2
descr="Remove tilt from NGL image, Mean removed from image"

${BINARY_PATH}/vdetilt -i ./results/test_input.xv -o $tmpdir/out2.$$ -p 1
report $test_num "$tmpdir/out2.$$" "./results/test2_result.xv" "$descr"
/bin/rm -f $tmpdir/out2.$$ 


# ---------- Test 3 --> test description
 
test_num=3
descr="Remove tilt from NGL image, Mean not removed from image, image masked with a circle in the center"

vgcirc -r 332 -c 443 -d 200 -x 166 -y 221 -b 0 -f 255 -t "byte" -o $tmpdir/gate.$$
${BINARY_PATH}/vdetilt -i ./results/test_input.xv -o $tmpdir/out3.$$ -m $tmpdir/gate.$$  -p 0

report $test_num "$tmpdir/out3.$$" "./results/test3_result.xv" "$descr"
/bin/rm -f $tmpdir/out3.$$ $tmpdir/gate.$$

exit $status
