#! /bin/sh

echo "running test.sh for vgbox ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="default parameters tests"

${BINARY_PATH}/vgbox -o $tmpdir/out2.$$

${BINARY_PATH}/vgbox -r 512 -c 512 -px_h 1 -px_w 1 -px_y_off 0 -px_x_off 0 -h 40.3 -w 20.6 -im_y_off 0 -im_x_off 0 -angle 15 -corner_angl 90 -g_o 10 -g_i 200 -o $tmpdir/out.$$

report $test_num "$tmpdir/out.$$" "./results/default_results.xv" "$descr"
/bin/rm -f $tmpdir/out.$$ 

# ---------- Test 2 --> test description
 
test_num=2
descr="modified size, angle, corner_angl, pixel height and pixel width"

${BINARY_PATH}/vgbox -o $tmpdir/out2.$$ -r 64 -c 64 -px_h 0.413793 -px_w 0.482759 -px_y_off 0 -px_x_off 0 -h 40.3 -w 20.6 -im_y_off 0 -im_x_off 0 -angle 25 -corner_angl 179 -g_o 0 -g_i 255 

report $test_num "$tmpdir/out2.$$" "./results/test2_results.xv" "$descr"
/bin/rm -f $tmpdir/out2.$$ 

# ---------- Test 3 --> test description
 
test_num=3
descr="sum of two different box generations"


${BINARY_PATH}/vgbox -o  $tmpdir/out3a.$$ -r 64 -c 64 -px_h 0.413793 -px_w 0.482759 -px_y_off 0.482759 -px_x_off 0.37931 -h 20 -w 10 -im_y_off 10 -im_x_off 10 -angle 110 -corner_angl 179 -g_o 0 -g_i 255

${BINARY_PATH}/vgbox -o $tmpdir/out3b.$$ -r 64 -c 64 -px_h 0.5 -px_w 0.5 -px_y_off 0 -px_x_off 0 -h 20 -w 10 -im_y_off 5 -im_x_off 5 -angle 25 -corner_angl 1 -g_o 0 -g_i 255

kadd -i1 $tmpdir/out3a.$$ -i2 $tmpdir/out3b.$$ -o $tmpdir/out3.$$

report $test_num "$tmpdir/out3.$$" "./results/test3_results.xv" "$descr"
/bin/rm -f $tmpdir/out3*.$$ 

exit $status
