#! /bin/sh

echo "running test.sh for vgcirc ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="default test, except reduced size by half"


${BINARY_PATH}/vgcirc -o $tmpdir/out.$$ -r 128 -c 128 -d 64 -x 64 -y 64 -b 0 -f 255 -t "byte"

report $test_num "$tmpdir/out.$$" "./results/default_results.xv" "$descr"
/bin/rm -f $tmpdir/out.$$ 

# ---------- Test 2 --> test description
 
test_num=2
descr="modified size, X coord, Y cood, forground and background level, float result"

${BINARY_PATH}/vgcirc -o $tmpdir/out2.$$ -r 64 -c 64 -d 25 -x 6 -y 20 -b 10.5 -f 128.5 -t "float"

report $test_num "$tmpdir/out2.$$" "./results/test2_results.xv" "$descr"
/bin/rm -f $tmpdir/out2.$$ 

# ---------- Test 3 --> test description
 
test_num=3
descr="Compare the stats of the absolute difference of creating 2 partially overlapping circles"

${BINARY_PATH}/vgcirc -o $tmpdir/out3a.xv -r 64 -c 64 -d 25 -x 15 -y 20 -b 10.5 -f 128.5 -t "float" 
${BINARY_PATH}/vgcirc -o $tmpdir/out3b.xv -r 64 -c 64 -d 25 -x 6 -y 20 -b 10.5 -f 128.5 -t "float" 
kabsdiff -i1 $tmpdir/out3a.xv -i2 $tmpdir/out3b.xv  -o $tmpdir/out3c.$$
kstats -i $tmpdir/out3c.$$ -o $tmpdir/out3.$$ -mean -var -maxval -minval

report $test_num "$tmpdir/out3.$$" "./results/test3_results.viff" "$descr"
/bin/rm -f $tmpdir/out3*.$$ 

exit $status
