#! /bin/sh

echo "running test.sh for vgettilt ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol 1e-4 -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="Compute tilt on NGL image"

${BINARY_PATH}/vgettilt -i ../vdetilt/results/test_input.xv -f $tmpdir/out.$$

if cmp -s $tmpdir/out.$$ results/test1_result; then \
   echo "Passed:  Test  $test_num --> "$descr""
   status=0; \
else \
   echo "FAILED:  Test  $test_num --> "$descr""
   status=1; \
fi

/bin/rm -f $tmpdir/out.$$ 


# ---------- Test 2 --> test description
 
test_num=2
descr="Compute tilt on NGL image after removing the illumination gradient"

vdetilt -i ../vdetilt/results/test_input.xv -o /tmp/out2.vdetilt -p 0 
${BINARY_PATH}/vgettilt -i /tmp/out2.vdetilt -f $tmpdir/out2a.$$

if cmp -s $tmpdir/out2a.$$ results/test2_result; then \
   echo "Passed:  Test  $test_num --> "$descr""
   status=0; \
else \
   echo "FAILED:  Test  $test_num --> "$descr""
   status=1; \
fi

/bin/rm -f $tmpdir/out2*.$$ /tmp/out2.vdetilt

exit $status
