#! /bin/sh 

echo "running test.sh for vslope ... process ($$)"

# ------------------------------------------------------------------------
# report test_num test_obj cntrl_obj test_descr
#
# The report function uses ksegcmp to compare the test output data object
# ($2) against the correct result ($3), and reports success or failure of
# the test.  If the result of ksegcmp is false, status is set to 1.
# recommended toloerance 1e-4
# ------------------------------------------------------------------------
report()
{
test_num=$1
test_obj="$2"
cntrl_obj="$3"
test_descr="$4"
tolerance="$5"

ksegcmp -i1 "$test_obj" -i2 "$cntrl_obj" -tol "$tolerance" -sat 1 -s -rt 1 -all
if [ $? = 0 ]
then
   echo "Passed:  Test  $test_num --> "$test_descr""
else
   echo "FAILED:  Test  $test_num --> "$test_descr""
   status=1
fi
}

#
#  Initialize status to be success (ie. 0).  Upon failure, status is
#  initialized to 1.
#
status=0

# ------------------------------------------------------------------------
#  set path for temporary directory
# ------------------------------------------------------------------------
tmpdir=${TMPDIR:-/tmp}
tbpath=`kecho -tb SAMPLEDATA -echo path`
dpath="$tbpath/data/images"
BINARY_PATH=${BINARY_PATH:-`kecho -tb RETRO -echo binpath`}

# ---------- Test 1 --> test description
 
test_num=1
descr="vslope: generate slope and aspect images"

${BINARY_PATH}/vslope -i results/head.xv -s $tmpdir/out1.$$ -t $tmpdir/out2.$$ -so 2 -to 2 -z 2 
report $test_num "$tmpdir/out1.$$" "./results/slope.xv" "$descr" "1e-2"
report $test_num "$tmpdir/out2.$$" "./results/aspect.xv" "$descr" "1e-4"

/bin/rm -f $tmpdir/out*.$$ 

exit $status
